<?php

/**
 * @file api/v1/users/UserHandler.inc.php
 *
 * Copyright (c) 2014-2021 Simon Fraser University
 * Copyright (c) 2003-2021 John Willinsky
 * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
 *
 * @class UserHandler
 * @ingroup api_v1_users
 *
 * @brief Handle API requests for user operations.
 *
 */

import('lib.pkp.api.v1.users.PKPUserHandler');

class UserHandler extends PKPUserHandler { }
