<?php

/**
 * @defgroup plugins_generic_driver Driver Plugin
 */
 
/**
 * @file plugins/generic/driver/index.php
 *
 * Copyright (c) 2014-2021 Simon Fraser University
 * Copyright (c) 2003-2021 John Willinsky
 * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
 *
 * @ingroup plugins_generic_driver
 * @brief Wrapper for driver plugin.
 *
 */
require_once('DRIVERPlugin.inc.php');

return new DRIVERPlugin();


