#connect-orcid-button {
	margin-top: 2em;
	margin-bottom: 2em;
}

#orcid-id-logo {
	display: inline-block;
	width: 1.4em;
	height: 1.4em;
	margin-right: 0.5em;
	vertical-align: middle;
}

.orcid_icon {
	display: inline-block;
	margin-right: 0.25em;
	width: 24px;
	height: 24px;
	vertical-align: middle;
}

.orcid-failure {
	width:100%;
	background:#F6D8CE;
	border:1px solid #FF0000;
	border-radius:5px;
	padding:15px;
	margin-bottom:15px;
}

.orcid-success {
	width:100%;
	background:#E6F8E0;
	border:1px solid #3ADF00;
	border-radius:5px;
	padding:15px;
	margin-bottom:15px;
}
