/**
 * @file plugins/themes/default/notoSans.less
 *
 * Copyright (c) 2014-2021 Simon Fraser University
 * Copyright (c) 2003-2021 John Willinsky
 * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
 *
 * @brief Import the NotoSans font files
 *
 */
@import "../../../../../lib/pkp/styles/font.less";