//
// Common variables
//
// These variables set common standards to be used throughout pkp-lib and its
// associated projects. Unless a particular project is deliberately supposed
// to be distinguished in style, such as a custom theme, it should make use of
// these variables to define common sizing, colors and typography.
//

//
// Colors
//

// Background
@bg: #eaedee;
@bg-dark: #b7c5ca;
@bg-light: #eaedee;
@shade:			 #888;
@lift:			 #fff;
@bg-anchor:      #002C40; // Dark color background
@bg-anchor-lift: lighten(@bg-anchor, 10%);

// Primary color for links and buttons
@primary:        #006798;
@primary-lift:   lighten(@primary, 10%);
@primary-shade:  darken(@primary, 10%);

// Offset "highlight" color
@offset:         #D00A6C;
@offset-lift:    lighten(@offset, 20%);
@offset-shade:   #B50E61;

// Yes/no, success/error, on/off, go/stop, etc
@yes:			#00B24E;
@yes-lift:      desaturate(lighten(@yes, 10%), 20%);
@no:			@offset;
@no-lift:       desaturate(lighten(@no, 10%), 20%);
@warn:			@offset;
@help:          @yes;
@help-lift:     @yes-lift;
@star-on:  #DAA03C;
@star-off: #ddd;

// Submission stages
@submission: #d00a0a;
@review: #e08914;
@copyediting: #006798;
@production: #00b28d;


// Text
@text:			#222;
@text-rgba:		rgba(0, 0, 0, 0.84);
@text-light:		#777;
@text-light-rgba:	rgba(0, 0, 0, 0.54);
@text-disabled: #aaa;

// Border colors
@bg-border-color:       #bbb;
@bg-border-color-light: #ddd;
@grid-border-color:     #eee;

//
// Typography
//
@font:				"Noto Sans", "Noto Kufi Arabic", -apple-system, BlinkMacSystemFont, "Segoe UI", "Roboto", "Oxygen-Sans", "Ubuntu", "Cantarell", "Helvetica Neue", sans-serif;
@font-tiny:		0.75rem;
@line-tiny:		1rem;
@font-sml:		0.875rem;
@line-sml:		1.5rem;
@font-base:		1rem;
@line-base:		1.5rem;
@font-lead:		1.5rem;
@line-lead:		2rem;
@font-header:	2.25rem;
@line-header:	2.5rem;

// Font weights
@normal:		400;
@bold:			700;

//
// Borders
//
@bg-border:       1px solid @bg-border-color;
@bg-border-light: 1px solid @bg-border-color-light;
@grid-border:     1px solid @grid-border-color;
@radius:          2px;

// Base URL
// The base URL is set appropriately before the LESS stylesheet is compiled and
// will point to the site's base directory. The variable can be used to point
// to image and font assets.
@baseUrl:                   ''; // Leave empty
