/**
 * @file styles/index.less
 *
 * Copyright (c) 2014-2021 Simon Fraser University
 * Copyright (c) 2003-2021 John Willinsky
 * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
 *
 * @brief Main OJS stylesheet wrapper
 *
 */

/**
 * PKP Library
 *
 * OJS application styles should inherit all variables and reset styles from
 * the PKP Library. The library styles will also define many of the base
 * structural elements, as well as common controllers, UI components and
 * helpers.
 *
 * This @import should represent the single access point for all PKP Library
 * styles. Individual LESS stylesheets should never be loaded directly from OJS.
 */
 @import "lib/pkp/styles/index.less";