/**
 * @file plugins/themes/default/styles/sidebar.less
 *
 * Copyright (c) 2014-2021 Simon Fraser University
 * Copyright (c) 2003-2021 John Willinsky
 * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
 *
 * @brief Styles for the sidebar blocks
 *
 */

.pkp_block {
	padding: @triple @double;
	font-size: @font-base;
	line-height: @line-base;

	.title {
		display: block;
		margin-bottom: @base;
		margin-top: 0;
		font-family: @font-heading;
		font-size: @font-bump;
		font-weight: @bold;
		line-height: @line-bump;
		color: @text-light;
	}

	.content {

		 ul {
			&:extend(.pkp_unstyled_list);

			li {
				line-height: @line-base;
				padding: @half 0;
			}
		}

		p {
			line-height: @line-base-large;

			&:first-child {
				margin-top: 0;
			}

			&:last-child {
				margin-bottom: 0;
			}
		}
	}

	a {
		text-decoration: none;
	}

	@media(min-width: @screen-desktop) {
		padding: @triple;
	}
}

// Browse sidebar block
.block_browse {
	font-size: @font-sml;

	.has_submenu {
		margin-top: @double;
		font-size: @font-sml;
		font-weight: @bold;
		color: @text-light;

		ul {
			margin-top: calc(~"@{half} - 1px"); // account for border
			padding-top: @half;
			font-weight: @normal;
		}
	}

	.is_sub {
		margin-left: @base;
	}

	.current a {
		padding-left: 0.5em;
		border-left: 4px solid @bg-border-color;
		color: @text-light;
		cursor: text;
	}
}

.block_information,
.block_language_toggle {

	a {
		font-size: @font-sml;
	}
}

.block_make_submission {

	a {
		&:extend(.cmp_button);
	}
}

.block_subscription {

	.subscription_name {
		margin-bottom: 0;
		font-weight: @bold;
	}

	.subscription_membership {
		margin-top: 0;
	}
}
