/**
 * @file plugins/generic/tinymce/styles/content.css
 *
 * Copyright (c) 2014-2020 Simon Fraser University
 * Copyright (c) 2003-2020 John Willinsky
 * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
 *
 * @brief Additional CSS for TinyMCE content editor
 */

/* noto-sans-regular - vietnamese_latin-ext_latin_greek-ext_greek_devanagari_cyrillic-ext_cyrillic */
@font-face {
  font-family: 'Noto Sans';
  font-style: normal;
  font-weight: 400;
  src: url('../../../../lib/pkp/styles/fonts/noto-sans-v11-vietnamese_latin-ext_latin_greek-ext_greek_devanagari_cyrillic-ext_cyrillic-regular.eot'); /* IE9 Compat Modes */
  src: local(''),
       url('../../../../lib/pkp/styles/fonts/noto-sans-v11-vietnamese_latin-ext_latin_greek-ext_greek_devanagari_cyrillic-ext_cyrillic-regular.eot?#iefix') format('embedded-opentype'), /* IE6-IE8 */
       url('../../../../lib/pkp/styles/fonts/noto-sans-v11-vietnamese_latin-ext_latin_greek-ext_greek_devanagari_cyrillic-ext_cyrillic-regular.woff2') format('woff2'), /* Super Modern Browsers */
       url('../../../../lib/pkp/styles/fonts/noto-sans-v11-vietnamese_latin-ext_latin_greek-ext_greek_devanagari_cyrillic-ext_cyrillic-regular.woff') format('woff'), /* Modern Browsers */
       url('../../../../lib/pkp/styles/fonts/noto-sans-v11-vietnamese_latin-ext_latin_greek-ext_greek_devanagari_cyrillic-ext_cyrillic-regular.ttf') format('truetype'), /* Safari, Android, iOS */
       url('../../../../lib/pkp/styles/fonts/noto-sans-v11-vietnamese_latin-ext_latin_greek-ext_greek_devanagari_cyrillic-ext_cyrillic-regular.svg#NotoSans') format('svg'); /* Legacy iOS */
}

/* noto-sans-italic - vietnamese_latin-ext_latin_greek-ext_greek_devanagari_cyrillic-ext_cyrillic */
@font-face {
  font-family: 'Noto Sans';
  font-style: italic;
  font-weight: 400;
  src: url('../../../../lib/pkp/styles/fonts/noto-sans-v11-vietnamese_latin-ext_latin_greek-ext_greek_devanagari_cyrillic-ext_cyrillic-italic.eot'); /* IE9 Compat Modes */
  src: local(''),
       url('../../../../lib/pkp/styles/fonts/noto-sans-v11-vietnamese_latin-ext_latin_greek-ext_greek_devanagari_cyrillic-ext_cyrillic-italic.eot?#iefix') format('embedded-opentype'), /* IE6-IE8 */
       url('../../../../lib/pkp/styles/fonts/noto-sans-v11-vietnamese_latin-ext_latin_greek-ext_greek_devanagari_cyrillic-ext_cyrillic-italic.woff2') format('woff2'), /* Super Modern Browsers */
       url('../../../../lib/pkp/styles/fonts/noto-sans-v11-vietnamese_latin-ext_latin_greek-ext_greek_devanagari_cyrillic-ext_cyrillic-italic.woff') format('woff'), /* Modern Browsers */
       url('../../../../lib/pkp/styles/fonts/noto-sans-v11-vietnamese_latin-ext_latin_greek-ext_greek_devanagari_cyrillic-ext_cyrillic-italic.ttf') format('truetype'), /* Safari, Android, iOS */
       url('../../../../lib/pkp/styles/fonts/noto-sans-v11-vietnamese_latin-ext_latin_greek-ext_greek_devanagari_cyrillic-ext_cyrillic-italic.svg#NotoSans') format('svg'); /* Legacy iOS */
}

/* noto-sans-700 - vietnamese_latin-ext_latin_greek-ext_greek_devanagari_cyrillic-ext_cyrillic */
@font-face {
  font-family: 'Noto Sans';
  font-style: normal;
  font-weight: 700;
  src: url('../../../../lib/pkp/styles/fonts/noto-sans-v11-vietnamese_latin-ext_latin_greek-ext_greek_devanagari_cyrillic-ext_cyrillic-700.eot'); /* IE9 Compat Modes */
  src: local(''),
       url('../../../../lib/pkp/styles/fonts/noto-sans-v11-vietnamese_latin-ext_latin_greek-ext_greek_devanagari_cyrillic-ext_cyrillic-700.eot?#iefix') format('embedded-opentype'), /* IE6-IE8 */
       url('../../../../lib/pkp/styles/fonts/noto-sans-v11-vietnamese_latin-ext_latin_greek-ext_greek_devanagari_cyrillic-ext_cyrillic-700.woff2') format('woff2'), /* Super Modern Browsers */
       url('../../../../lib/pkp/styles/fonts/noto-sans-v11-vietnamese_latin-ext_latin_greek-ext_greek_devanagari_cyrillic-ext_cyrillic-700.woff') format('woff'), /* Modern Browsers */
       url('../../../../lib/pkp/styles/fonts/noto-sans-v11-vietnamese_latin-ext_latin_greek-ext_greek_devanagari_cyrillic-ext_cyrillic-700.ttf') format('truetype'), /* Safari, Android, iOS */
       url('../../../../lib/pkp/styles/fonts/noto-sans-v11-vietnamese_latin-ext_latin_greek-ext_greek_devanagari_cyrillic-ext_cyrillic-700.svg#NotoSans') format('svg'); /* Legacy iOS */
}

/* noto-sans-700italic - vietnamese_latin-ext_latin_greek-ext_greek_devanagari_cyrillic-ext_cyrillic */
@font-face {
  font-family: 'Noto Sans';
  font-style: italic;
  font-weight: 700;
  src: url('../../../../lib/pkp/styles/fonts/noto-sans-v11-vietnamese_latin-ext_latin_greek-ext_greek_devanagari_cyrillic-ext_cyrillic-700italic.eot'); /* IE9 Compat Modes */
  src: local(''),
       url('../../../../lib/pkp/styles/fonts/noto-sans-v11-vietnamese_latin-ext_latin_greek-ext_greek_devanagari_cyrillic-ext_cyrillic-700italic.eot?#iefix') format('embedded-opentype'), /* IE6-IE8 */
       url('../../../../lib/pkp/styles/fonts/noto-sans-v11-vietnamese_latin-ext_latin_greek-ext_greek_devanagari_cyrillic-ext_cyrillic-700italic.woff2') format('woff2'), /* Super Modern Browsers */
       url('../../../../lib/pkp/styles/fonts/noto-sans-v11-vietnamese_latin-ext_latin_greek-ext_greek_devanagari_cyrillic-ext_cyrillic-700italic.woff') format('woff'), /* Modern Browsers */
       url('../../../../lib/pkp/styles/fonts/noto-sans-v11-vietnamese_latin-ext_latin_greek-ext_greek_devanagari_cyrillic-ext_cyrillic-700italic.ttf') format('truetype'), /* Safari, Android, iOS */
       url('../../../../lib/pkp/styles/fonts/noto-sans-v11-vietnamese_latin-ext_latin_greek-ext_greek_devanagari_cyrillic-ext_cyrillic-700italic.svg#NotoSans') format('svg'); /* Legacy iOS */
}

html,
body {
	font-family: "Noto Sans", "Noto Kufi Arabic", Arial, Helvetica, sans-serif;
	font-size: 14px;
	line-height: 24px;
	background: #fff;
	padding: 14px;
	margin: 0;
}

p:first-child {
	margin-top: 0;
}

span.pkpTag {
	padding: 0.5em;
	font-size: 10px;
	font-weight: 700;
	text-transform: uppercase;
	color: #777;
}

a {
	color: #006798;
}

a:hover,
a:focus {
	color: #113241;
}
