/**
 * @file  plugins/themes/default/styles/pages/contact.less
 *
 * Copyright (c) 2014-2021 Simon Fraser University
 * Copyright (c) 2003-2021 John Willinsky
 * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
 *
 * @brief Styles applying to the contact page
 * @link templates/frontend/pages/contact.tpl
 */
.page_contact {

    .address,
    .phone,
    .email {
        margin-top: @base;
        margin-bottom: @base;
        font-size: @font-sml;
    }

    .address {
        margin-top: 0;

        p {
            margin: 0;
        }
    }

    .label {
        display: block;
        font-weight: @bold;
    }

    .contact.support {
        margin-top: @quadruple;
    }

    @media(min-width: @screen-tablet) {

        .contact_section {
            .pkp_helpers_clear;
        }

        .contact {
            float: left;
            width: 50%;

            &.primary {
                padding-right: @double;
            }

            &.support {
                margin-top: 0;
            }
        }
    }
}
