/**
 * @defgroup pkp_controllers_grid
 */

/**
 * @file styles/controllers/grid/index.less
 *
 * Copyright (c) 2014-2021 Simon Fraser University
 * Copyright (c) 2003-2021 John Willinsky
 * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
 *
 * @ingroup pkp_controllers_grid
 *
 * @brief Wrapper for pkp_controllers_grid
 */

/* Grid Styles */
@import "lib/pkp/styles/controllers/grid/grid.less";

/* Plugin grid styles */
@import "lib/pkp/styles/controllers/grid/plugins.less";

/* Query grid styles */
@import "lib/pkp/styles/controllers/grid/queries.less";

/* Notifications grid styles */
@import "lib/pkp/styles/controllers/grid/notifications.less";
