{**
 * loadingContainer.tpl
 *
 * Copyright (c) 2014-2021 Simon Fraser University
 * Copyright (c) 2000-2021 John Willinsky
 * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
 *
 * The default HTML to display in a grid before it is loaded.
 *
 *}

<div class="pkp_loading">
	<span class="pkp_spinner"></span>
	<span class="message">{translate key="common.loading"}</span>
</div>
