<?php

/**
 * @defgroup plugins_generic_usageEvent Usage Event Plugin
 */

/**
 * @file plugins/generic/usageEvent/index.php
 *
 * Copyright (c) 2014-2021 Simon Fraser University
 * Copyright (c) 2003-2021 John Willinsky
 * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
 *
 * @ingroup plugins_generic_usageEvent
 * @brief Wrapper for usage event plugin.
 *
 */
require_once('UsageEventPlugin.inc.php');

return new UsageEventPlugin();


