/**
 * @file styles/objects/article_summary.less
 *
 * Copyright (c) 2014-2021 Simon Fraser University
 * Copyright (c) 2003-2021 John Willinsky
 * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
 *
 * @brief Styles applying to a summary view of an article for lists
 * @link templates/frontend/objects/article_details.tpl
 */
.obj_article_summary {
	.pkp_helpers_clear();

	.cover {
		display: block;
		margin-bottom: @double;

		img {
			display: block;
			max-height: 250px;
			width: auto;
		}
	}

	> .title {
		font-family: @font-heading;
		font-size: @font-base;
		line-height: @line-base;
		font-weight: @bold;
		border-bottom: none;
		margin: 0;

		a {
			text-decoration: none;
		}
	}

	.subtitle {
		display: block;
		margin-top: 0.25em;
		margin-bottom: 0.5em;
		font-weight: @normal;
		color: @text-light;
	}

	.meta {
		position: relative;
		padding-top: @half;
		font-size: @font-sml;
		line-height: @double;
	}

	.pages,
	.published {
		color: @text-light;
	}

	.galleys_links {
		&:extend(.pkp_unstyled_list);
		margin-top: @base;

		li {
			display: inline-block;
			margin-right: 1em;

			&:last-child {
				margin-right: 0;
			}
		}
	}

	@media(min-width: @screen-tablet) {

		.authors {
			padding-right: 5em;
		}

		.pages {
			position: absolute;
			top: 0;
			right: 0;
			line-height: @line-lead;
		}

		.cover {
			float: left;
			width: 25%;
			height: auto;
			max-height: none;
			margin-right: @double;
		}
	}

	@media(min-width: @screen-desktop) {

		.cover {
			margin-right: @triple;
			margin-bottom: @triple;

			img {
				max-height: none;
			}
		}
	}
}
