/**
 * @file styles/pages/index.less
 *
 * Copyright (c) 2014-2021 Simon Fraser University
 * Copyright (c) 2003-2021 John Willinsky
 * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
 *
 * @brief Pages stylesheet wrapper
 *
 */
@import "dashboard.less";
@import "settings.less";
@import "submission.less";
@import "tools.less";
@import "workflow.less";
