/**
 * styles/rtl.less
 *
 * Copyright (c) 2014-2021 Simon Fraser University
 * Copyright (c) 2000-2021 John Willinsky
 * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
 *
 * Stylesheet for flipping content to RTL (e.g. for Arabic, Persian and Hebrew)
 *
 */

body[dir="rtl"] {
	direction: rtl;
	unicode-bidi: embed;

	// Help panel
	.pkp_help_tab,
	.pkp_help_modal {
		right: auto;
		left: 3rem;
	}

	.pkp_help_panel {

		.panel {
			left: 0;
			right: auto;
			.transform(translate3d(-100%,0,0));
		}

		&.is_visible .panel {
			.transform(translate3d(0,0,0));
		}

		.header {

			.home {
				float: right;
			}

			.close {
				float: left;
			}
		}

		.footer {

			.previous {
				float: right;

				&:before {
					content: @fa-var-long-arrow-right;
					margin: 0;
					margin-left: 0.25em;
				}
			}

			.next {
				float: left;

				&:after {
					content: @fa-var-long-arrow-left;
					margin: 0;
					margin-right: 0.25em;
				}
			}
		}
	}

	// Helpers
	.pkp_helpers_text_left {
		text-align: right;
	}

	.pkp_helpers_text_right {
		text-align: left;
	}

	.pkp_caret_right,
	.pkp_controllers_grid .show_extras:before {
		border-left: 4px solid transparent;
		border-right: 6px solid;
	}

	.pkp_caret_left {
		border-left: 4px solid transparent;
		border-right: 6px solid;
	}

	.pkp_screen_reader,
	.pkp_form label.locale,
	.ui-datepicker-prev > span,
	.ui-datepicker-next > span,
	.pkp_controllers_grid thead th:first-child:last-child,
	[id^="reviewersGrid"] thead {
		left: auto;
		right: -2000px;
	}

	// Forms
	.pkp_form .inline {
		float: right;
		padding-right: inherit;
		padding-left: 1rem;
	}

	// Grids
	.pkp_controllers_grid {

		th[style^="text-align: left"],
		td[style^="text-align: left"] {
			// This is necessary due to the inline styles added by grid components
			text-align: right !important;
		}

		.actions {
			float: left;
		}

		.pkp_linkaction_search:before {
			margin-right: 0;
			margin-left: 0.5em;
		}

		.has_extras td:first-child,
		.row_controls td:first-child {
			padding-left: 1rem;
			padding-right: 2rem;
		}

		.show_extras,
		.hide_extras {
			left: auto;
			right: 0;
		}
	}

	// Submission workflow
	.pkp_workflow_decisions {
		padding-right: 0;
	}
}
