<?php
/**
 * @file classes/components/form/context/ReviewGuidanceForm.inc.php
 *
 * Copyright (c) 2014-2021 Simon Fraser University
 * Copyright (c) 2000-2021 John Willinsky
 * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
 *
 * @class ReviewGuidanceForm
 * @ingroup classes_controllers_form
 *
 * @brief A preset form for configuring the guidance a reviewer should receive.
 */
namespace APP\components\forms\context;
use \PKP\components\forms\context\PKPReviewGuidanceForm;

class ReviewGuidanceForm extends PKPReviewGuidanceForm {}
