/**
 * @file plugins/themes/default/lato.less
 *
 * Copyright (c) 2014-2021 Simon Fraser University
 * Copyright (c) 2003-2021 John Willinsky
 * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
 *
 * @brief Import the Lato font files
 *
 */
/* lato-regular - latin-ext_latin */
@font-face {
  font-family: 'Lato';
  font-style: normal;
  font-weight: 400;
  src: url('@{baseUrl}/plugins/themes/default/fonts/lato-v17-latin-ext_latin-regular.eot'); /* IE9 Compat Modes */
  src: local(''),
       url('@{baseUrl}/plugins/themes/default/fonts/lato-v17-latin-ext_latin-regular.eot?#iefix') format('embedded-opentype'), /* IE6-IE8 */
       url('@{baseUrl}/plugins/themes/default/fonts/lato-v17-latin-ext_latin-regular.woff2') format('woff2'), /* Super Modern Browsers */
       url('@{baseUrl}/plugins/themes/default/fonts/lato-v17-latin-ext_latin-regular.woff') format('woff'), /* Modern Browsers */
       url('@{baseUrl}/plugins/themes/default/fonts/lato-v17-latin-ext_latin-regular.ttf') format('truetype'), /* Safari, Android, iOS */
       url('@{baseUrl}/plugins/themes/default/fonts/lato-v17-latin-ext_latin-regular.svg#Lato') format('svg'); /* Legacy iOS */
}

/* lato-italic - latin-ext_latin */
@font-face {
  font-family: 'Lato';
  font-style: italic;
  font-weight: 400;
  src: url('@{baseUrl}/plugins/themes/default/fonts/lato-v17-latin-ext_latin-italic.eot'); /* IE9 Compat Modes */
  src: local(''),
       url('@{baseUrl}/plugins/themes/default/fonts/lato-v17-latin-ext_latin-italic.eot?#iefix') format('embedded-opentype'), /* IE6-IE8 */
       url('@{baseUrl}/plugins/themes/default/fonts/lato-v17-latin-ext_latin-italic.woff2') format('woff2'), /* Super Modern Browsers */
       url('@{baseUrl}/plugins/themes/default/fonts/lato-v17-latin-ext_latin-italic.woff') format('woff'), /* Modern Browsers */
       url('@{baseUrl}/plugins/themes/default/fonts/lato-v17-latin-ext_latin-italic.ttf') format('truetype'), /* Safari, Android, iOS */
       url('@{baseUrl}/plugins/themes/default/fonts/lato-v17-latin-ext_latin-italic.svg#Lato') format('svg'); /* Legacy iOS */
}

/* lato-900 - latin-ext_latin */
@font-face {
  font-family: 'Lato';
  font-style: normal;
  font-weight: 900;
  src: url('@{baseUrl}/plugins/themes/default/fonts/lato-v17-latin-ext_latin-900.eot'); /* IE9 Compat Modes */
  src: local(''),
       url('@{baseUrl}/plugins/themes/default/fonts/lato-v17-latin-ext_latin-900.eot?#iefix') format('embedded-opentype'), /* IE6-IE8 */
       url('@{baseUrl}/plugins/themes/default/fonts/lato-v17-latin-ext_latin-900.woff2') format('woff2'), /* Super Modern Browsers */
       url('@{baseUrl}/plugins/themes/default/fonts/lato-v17-latin-ext_latin-900.woff') format('woff'), /* Modern Browsers */
       url('@{baseUrl}/plugins/themes/default/fonts/lato-v17-latin-ext_latin-900.ttf') format('truetype'), /* Safari, Android, iOS */
       url('@{baseUrl}/plugins/themes/default/fonts/lato-v17-latin-ext_latin-900.svg#Lato') format('svg'); /* Legacy iOS */
}

/* lato-900italic - latin-ext_latin */
@font-face {
  font-family: 'Lato';
  font-style: italic;
  font-weight: 900;
  src: url('@{baseUrl}/plugins/themes/default/fonts/lato-v17-latin-ext_latin-900italic.eot'); /* IE9 Compat Modes */
  src: local(''),
       url('@{baseUrl}/plugins/themes/default/fonts/lato-v17-latin-ext_latin-900italic.eot?#iefix') format('embedded-opentype'), /* IE6-IE8 */
       url('@{baseUrl}/plugins/themes/default/fonts/lato-v17-latin-ext_latin-900italic.woff2') format('woff2'), /* Super Modern Browsers */
       url('@{baseUrl}/plugins/themes/default/fonts/lato-v17-latin-ext_latin-900italic.woff') format('woff'), /* Modern Browsers */
       url('@{baseUrl}/plugins/themes/default/fonts/lato-v17-latin-ext_latin-900italic.ttf') format('truetype'), /* Safari, Android, iOS */
       url('@{baseUrl}/plugins/themes/default/fonts/lato-v17-latin-ext_latin-900italic.svg#Lato') format('svg'); /* Legacy iOS */
}
