{**
 * templates/controllers/grid/feature/gridOrderNonOrderableMessage.tpl
 *
 * Copyright (c) 2014-2021 Simon Fraser University
 * Copyright (c) 2000-2021 John Willinsky
 * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
 *
 * Message to be presented while hovering for non orderable grid rows in ordering mode.
 *}

<div class="order_message">{$orderMessage}</div>
