/**
 * @file plugins/themes/default/styles/pages/submissions.less
 *
 * Copyright (c) 2014-2021 Simon Fraser University
 * Copyright (c) 2003-2021 John Willinsky
 * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
 *
 * @brief Styles applying to submissions page
 * @link templates/frontend/pages/submissions.tpl
 */
.page_submissions {

	.submission_sections, .submission_checklist {

		ul {
			margin: @double 0 0;
			padding: 0;
			list-style: none;
			font-size: @font-sml;
			border: none;
		}

		li {
			position: relative;
			border-bottom: none;
		}
	}

	.submission_sections {
		h3 {
			margin-top: 0;
		}

		.cmp_notification {
			margin-bottom: 0;
		}
	}

	.submission_checklist {

		li {
			padding: @double;

			.fa {
				top: 50%;
				left: @half;
				-webkit-transform: translate(-50%, -50%);
				-moz-transform: translate(-50%, -50%);
				-ms-transform: translate(-50%, -50%);
				-o-transform: translate(-50%, -50%);
				transform: translate(-50%, -50%);
				position: absolute;
				font-size: @font-lead;
				color: @yes;
			}
		}
	}

	@media(min-width: @screen-phone) {

		.submission_sections {
			li {
				.fa {
					top: 22px;
					left: calc(~"@{double} - 3px");
				}
			}
		}

		.submission_sections, .submission_checklist {
			ul {
				border: @bg-border;
				border-bottom: none;
			}

			li {
				padding: @double @double @double @quadruple;
				border-bottom: @bg-border;

				.fa {
					left: @double;
				}
			}
		}
	}
}
