/**
 * @file styles/controllers/grid/notifications.less
 *
 * Copyright (c) 2014-2021 Simon Fraser University
 * Copyright (c) 2003-2021 John Willinsky
 * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
 *
 * @ingroup pkp_controllers_grid_notifications
 *
 * @brief Classes for notification grid structure
 */

#notificationsGrid {

	table {
		// Prevents table from overflowing parent div
		table-layout: fixed;
	}

	a {
		font-weight: @normal;
		text-decoration: none;
	}

	// Leave the header for screen readers but don't show it
	thead {
		position: absolute;
		top: 0;
		left: -9999px;
	}

	// No top border on the first task
	.gridRow:first-child {
		border-top: none;
	}

	.gridRow td:first-child {
		line-height: 2rem;
	}

	.gridRow td:nth-child(2) {
		transform: translateX(1rem); // prevent column from overlapping clickable area of checkbox
		padding-left: 0;
		padding-right: 0;
	}

	.footer {
		background: transparent;
		padding: 0;
	}

	// Override <ul> styles attached to navigation menu
	.actions.btm {
		float: none;
		padding-left: 38px; // aligns with text of each task
		text-align: left;

		a {
			font-weight: @bold;
		}

		.pkp_linkaction_deleteNotifications {
			color: @offset;
		}
	}

	.pkp_grid_no_title {
		border-bottom: none;
	}

	.task {

		.message {
			display: block;
			line-height: 20px;
			max-height: 3em;
			overflow: hidden;
		}

		.details {
			position: relative;
			display: block;
			margin-right: 1.5rem;
			font-size: @font-tiny;
			line-height: 24px;
			color: @text;
		}

		.acronym {
			background: @lift;
			padding: 0 0.5em;
			border: @bg-border;
			border-radius: @radius;
			box-shadow: -10px 0 10px @lift;
			line-height: 22px;
			color: @text-light;
		}

		.submission {
			display: block;
			white-space: nowrap;
			overflow: hidden;
		}

		&.unread {

			.message {
				font-weight: @bold;
			}
		}
	}
}
