{**
 * plugins/importexport/crossref/templates/statusMessage.tpl
 *
 * Copyright (c) 2014-2021 Simon Fraser University
 * Copyright (c) 2003-2021 John Willinsky
 * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
 *
 * Crossref failure messages
 *
 *}

<pre style="white-space: pre-wrap;">{$statusMessage}</pre>

