<?php

// autoload_namespaces.php @generated by Composer

$vendorDir = dirname(dirname(__FILE__));
$baseDir = dirname(dirname($vendorDir));

return array(
    'Pimple' => array($vendorDir . '/pimple/pimple/src'),
    'Less' => array($vendorDir . '/wikimedia/less.php/lib'),
    'HTMLPurifier' => array($vendorDir . '/ezyang/htmlpurifier/library'),
);
