<?php
/*
 * citeproc-php: InvalidStylesheetException.php
 * User: Sebastian Böttger <sebastian.boettger@thomascook.de>
 * created at 2019-03-01, 16:08
 */

namespace Seboettg\CiteProc\Exception;

class InvalidStylesheetException extends CiteProcException
{

}
