<?php

/**
 * @defgroup plugins_generic_customBlockManager Custom Block Manager Plugin
 */
 
/**
 * @file plugins/generic/customBlockManager/index.php
 *
 * Copyright (c) 2014-2020 Simon Fraser University
 * Copyright (c) 2003-2020 John Willinsky
 * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
 *
 * @brief Wrapper for Custom Block Manager plugin.
 *
 */

require_once('CustomBlockManagerPlugin.inc.php');
return new CustomBlockManagerPlugin();

