<?php
/**
 * @file api/v1/_payments/BackendPaymentsSettingsHandler.inc.php
 *
 * Copyright (c) 2014-2021 Simon Fraser University
 * Copyright (c) 2003-2021 John Willinsky
 * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
 *
 * @class BackendPaymentsSettingsHandler
 * @ingroup api_v1_backend
 */
import('lib.pkp.api.v1._payments.PKPBackendPaymentsSettingsHandler');

class BackendPaymentsSettingsHandler extends PKPBackendPaymentsSettingsHandler {}
