/**
 * @file  plugins/themes/default/styles/objects/announcement_summary.less
 *
 * Copyright (c) 2014-2021 Simon Fraser University
 * Copyright (c) 2003-2021 John Willinsky
 * Distributed under the GNU GPL v3. For full terms see the file docs/COPYING.
 *
 * @brief Styles applying to the summary view of an announcement
 * @link templates/frontend/objects/announcement_summary.tpl
 */
.obj_announcement_summary {

	h2,
	h3,
	h4 {
		margin: 0;
		font-size: @font-base;
		line-height: @line-base;

		a {
			text-decoration: none;
		}
	}

	.date {
		font-size: @font-sml;
		line-height: @line-base-large;
		color: @text-light;

		&:before {
			.fa();
			content: @fa-var-calendar;
			margin-right: 0.5em;
			font-size: @font-base;
			color: @bg-shade;
		}
	}

	.summary {
		font-size: @font-sml;
		line-height: @line-sml;
		margin-top: @base;

		p:first-child {
			margin-top: 0;
		}

		p:last-child {
			margin-bottom: 0;
		}
	}

	.read_more {
		.cmp_button_text(@fa-var-chevron-right);
	}
}
