(function(e){var t={};function n(r){if(t[r])return t[r].exports;var i=t[r]={i:r,l:!1,exports:{}};return e[r].call(i.exports,i,i.exports,n),i.l=!0,i.exports}n.m=e,n.c=t,n.d=function(e,t,r){n.o(e,t)||Object.defineProperty(e,t,{enumerable:!0,get:r})},n.r=function(e){"undefined"!==typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})},n.t=function(e,t){if(1&t&&(e=n(e)),8&t)return e;if(4&t&&"object"===typeof e&&e&&e.__esModule)return e;var r=Object.create(null);if(n.r(r),Object.defineProperty(r,"default",{enumerable:!0,value:e}),2&t&&"string"!=typeof e)for(var i in e)n.d(r,i,function(t){return e[t]}.bind(null,i));return r},n.n=function(e){var t=e&&e.__esModule?function(){return e["default"]}:function(){return e};return n.d(t,"a",t),t},n.o=function(e,t){return Object.prototype.hasOwnProperty.call(e,t)},n.p="/",n(n.s=0)})({0:function(e,t,n){n("86b0"),e.exports=n("746a")},"00ee":function(e,t,n){var r=n("b622"),i=r("toStringTag"),a={};a[i]="z",e.exports="[object z]"===String(a)},"010e":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("uz-latn",{months:"Yanvar_Fevral_Mart_Aprel_May_Iyun_Iyul_Avgust_Sentabr_Oktabr_Noyabr_Dekabr".split("_"),monthsShort:"Yan_Fev_Mar_Apr_May_Iyun_Iyul_Avg_Sen_Okt_Noy_Dek".split("_"),weekdays:"Yakshanba_Dushanba_Seshanba_Chorshanba_Payshanba_Juma_Shanba".split("_"),weekdaysShort:"Yak_Dush_Sesh_Chor_Pay_Jum_Shan".split("_"),weekdaysMin:"Ya_Du_Se_Cho_Pa_Ju_Sha".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"D MMMM YYYY, dddd HH:mm"},calendar:{sameDay:"[Bugun soat] LT [da]",nextDay:"[Ertaga] LT [da]",nextWeek:"dddd [kuni soat] LT [da]",lastDay:"[Kecha soat] LT [da]",lastWeek:"[O'tgan] dddd [kuni soat] LT [da]",sameElse:"L"},relativeTime:{future:"Yaqin %s ichida",past:"Bir necha %s oldin",s:"soniya",ss:"%d soniya",m:"bir daqiqa",mm:"%d daqiqa",h:"bir soat",hh:"%d soat",d:"bir kun",dd:"%d kun",M:"bir oy",MM:"%d oy",y:"bir yil",yy:"%d yil"},week:{dow:1,doy:7}});return t}))},"02fb":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("ml",{months:"ജനുവരി_ഫെബ്രുവരി_മാർച്ച്_ഏപ്രിൽ_മേയ്_ജൂൺ_ജൂലൈ_ഓഗസ്റ്റ്_സെപ്റ്റംബർ_ഒക്ടോബർ_നവംബർ_ഡിസംബർ".split("_"),monthsShort:"ജനു._ഫെബ്രു._മാർ._ഏപ്രി._മേയ്_ജൂൺ_ജൂലൈ._ഓഗ._സെപ്റ്റ._ഒക്ടോ._നവം._ഡിസം.".split("_"),monthsParseExact:!0,weekdays:"ഞായറാഴ്ച_തിങ്കളാഴ്ച_ചൊവ്വാഴ്ച_ബുധനാഴ്ച_വ്യാഴാഴ്ച_വെള്ളിയാഴ്ച_ശനിയാഴ്ച".split("_"),weekdaysShort:"ഞായർ_തിങ്കൾ_ചൊവ്വ_ബുധൻ_വ്യാഴം_വെള്ളി_ശനി".split("_"),weekdaysMin:"ഞാ_തി_ചൊ_ബു_വ്യാ_വെ_ശ".split("_"),longDateFormat:{LT:"A h:mm -നു",LTS:"A h:mm:ss -നു",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm -നു",LLLL:"dddd, D MMMM YYYY, A h:mm -നു"},calendar:{sameDay:"[ഇന്ന്] LT",nextDay:"[നാളെ] LT",nextWeek:"dddd, LT",lastDay:"[ഇന്നലെ] LT",lastWeek:"[കഴിഞ്ഞ] dddd, LT",sameElse:"L"},relativeTime:{future:"%s കഴിഞ്ഞ്",past:"%s മുൻപ്",s:"അൽപ നിമിഷങ്ങൾ",ss:"%d സെക്കൻഡ്",m:"ഒരു മിനിറ്റ്",mm:"%d മിനിറ്റ്",h:"ഒരു മണിക്കൂർ",hh:"%d മണിക്കൂർ",d:"ഒരു ദിവസം",dd:"%d ദിവസം",M:"ഒരു മാസം",MM:"%d മാസം",y:"ഒരു വർഷം",yy:"%d വർഷം"},meridiemParse:/രാത്രി|രാവിലെ|ഉച്ച കഴിഞ്ഞ്|വൈകുന്നേരം|രാത്രി/i,meridiemHour:function(e,t){return 12===e&&(e=0),"രാത്രി"===t&&e>=4||"ഉച്ച കഴിഞ്ഞ്"===t||"വൈകുന്നേരം"===t?e+12:e},meridiem:function(e,t,n){return e<4?"രാത്രി":e<12?"രാവിലെ":e<17?"ഉച്ച കഴിഞ്ഞ്":e<20?"വൈകുന്നേരം":"രാത്രി"}});return t}))},"0366":function(e,t,n){var r=n("1c0b");e.exports=function(e,t,n){if(r(e),void 0===t)return e;switch(n){case 0:return function(){return e.call(t)};case 1:return function(n){return e.call(t,n)};case 2:return function(n,r){return e.call(t,n,r)};case 3:return function(n,r,i){return e.call(t,n,r,i)}}return function(){return e.apply(t,arguments)}}},"03ec":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("cv",{months:"кӑрлач_нарӑс_пуш_ака_май_ҫӗртме_утӑ_ҫурла_авӑн_юпа_чӳк_раштав".split("_"),monthsShort:"кӑр_нар_пуш_ака_май_ҫӗр_утӑ_ҫур_авн_юпа_чӳк_раш".split("_"),weekdays:"вырсарникун_тунтикун_ытларикун_юнкун_кӗҫнерникун_эрнекун_шӑматкун".split("_"),weekdaysShort:"выр_тун_ытл_юн_кӗҫ_эрн_шӑм".split("_"),weekdaysMin:"вр_тн_ыт_юн_кҫ_эр_шм".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD-MM-YYYY",LL:"YYYY [ҫулхи] MMMM [уйӑхӗн] D[-мӗшӗ]",LLL:"YYYY [ҫулхи] MMMM [уйӑхӗн] D[-мӗшӗ], HH:mm",LLLL:"dddd, YYYY [ҫулхи] MMMM [уйӑхӗн] D[-мӗшӗ], HH:mm"},calendar:{sameDay:"[Паян] LT [сехетре]",nextDay:"[Ыран] LT [сехетре]",lastDay:"[Ӗнер] LT [сехетре]",nextWeek:"[Ҫитес] dddd LT [сехетре]",lastWeek:"[Иртнӗ] dddd LT [сехетре]",sameElse:"L"},relativeTime:{future:function(e){var t=/сехет$/i.exec(e)?"рен":/ҫул$/i.exec(e)?"тан":"ран";return e+t},past:"%s каялла",s:"пӗр-ик ҫеккунт",ss:"%d ҫеккунт",m:"пӗр минут",mm:"%d минут",h:"пӗр сехет",hh:"%d сехет",d:"пӗр кун",dd:"%d кун",M:"пӗр уйӑх",MM:"%d уйӑх",y:"пӗр ҫул",yy:"%d ҫул"},dayOfMonthOrdinalParse:/\d{1,2}-мӗш/,ordinal:"%d-мӗш",week:{dow:1,doy:7}});return t}))},"0450":function(e,t,n){},"0558":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e){return e%100===11||e%10!==1}function n(e,n,r,i){var a=e+" ";switch(r){case"s":return n||i?"nokkrar sekúndur":"nokkrum sekúndum";case"ss":return t(e)?a+(n||i?"sekúndur":"sekúndum"):a+"sekúnda";case"m":return n?"mínúta":"mínútu";case"mm":return t(e)?a+(n||i?"mínútur":"mínútum"):n?a+"mínúta":a+"mínútu";case"hh":return t(e)?a+(n||i?"klukkustundir":"klukkustundum"):a+"klukkustund";case"d":return n?"dagur":i?"dag":"degi";case"dd":return t(e)?n?a+"dagar":a+(i?"daga":"dögum"):n?a+"dagur":a+(i?"dag":"degi");case"M":return n?"mánuður":i?"mánuð":"mánuði";case"MM":return t(e)?n?a+"mánuðir":a+(i?"mánuði":"mánuðum"):n?a+"mánuður":a+(i?"mánuð":"mánuði");case"y":return n||i?"ár":"ári";case"yy":return t(e)?a+(n||i?"ár":"árum"):a+(n||i?"ár":"ári")}}var r=e.defineLocale("is",{months:"janúar_febrúar_mars_apríl_maí_júní_júlí_ágúst_september_október_nóvember_desember".split("_"),monthsShort:"jan_feb_mar_apr_maí_jún_júl_ágú_sep_okt_nóv_des".split("_"),weekdays:"sunnudagur_mánudagur_þriðjudagur_miðvikudagur_fimmtudagur_föstudagur_laugardagur".split("_"),weekdaysShort:"sun_mán_þri_mið_fim_fös_lau".split("_"),weekdaysMin:"Su_Má_Þr_Mi_Fi_Fö_La".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY [kl.] H:mm",LLLL:"dddd, D. MMMM YYYY [kl.] H:mm"},calendar:{sameDay:"[í dag kl.] LT",nextDay:"[á morgun kl.] LT",nextWeek:"dddd [kl.] LT",lastDay:"[í gær kl.] LT",lastWeek:"[síðasta] dddd [kl.] LT",sameElse:"L"},relativeTime:{future:"eftir %s",past:"fyrir %s síðan",s:n,ss:n,m:n,mm:n,h:"klukkustund",hh:n,d:n,dd:n,M:n,MM:n,y:n,yy:n},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return r}))},"057f":function(e,t,n){var r=n("fc6a"),i=n("241c").f,a={}.toString,o="object"==typeof window&&window&&Object.getOwnPropertyNames?Object.getOwnPropertyNames(window):[],s=function(e){try{return i(e)}catch(t){return o.slice()}};e.exports.f=function(e){return o&&"[object Window]"==a.call(e)?s(e):i(r(e))}},"06cf":function(e,t,n){var r=n("83ab"),i=n("d1e7"),a=n("5c6c"),o=n("fc6a"),s=n("c04e"),l=n("5135"),u=n("0cfb"),c=Object.getOwnPropertyDescriptor;t.f=r?c:function(e,t){if(e=o(e),t=s(t,!0),u)try{return c(e,t)}catch(n){}if(l(e,t))return a(!i.f.call(e,t),e[t])}},"0721":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("fo",{months:"januar_februar_mars_apríl_mai_juni_juli_august_september_oktober_november_desember".split("_"),monthsShort:"jan_feb_mar_apr_mai_jun_jul_aug_sep_okt_nov_des".split("_"),weekdays:"sunnudagur_mánadagur_týsdagur_mikudagur_hósdagur_fríggjadagur_leygardagur".split("_"),weekdaysShort:"sun_mán_týs_mik_hós_frí_ley".split("_"),weekdaysMin:"su_má_tý_mi_hó_fr_le".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D. MMMM, YYYY HH:mm"},calendar:{sameDay:"[Í dag kl.] LT",nextDay:"[Í morgin kl.] LT",nextWeek:"dddd [kl.] LT",lastDay:"[Í gjár kl.] LT",lastWeek:"[síðstu] dddd [kl] LT",sameElse:"L"},relativeTime:{future:"um %s",past:"%s síðani",s:"fá sekund",ss:"%d sekundir",m:"ein minuttur",mm:"%d minuttir",h:"ein tími",hh:"%d tímar",d:"ein dagur",dd:"%d dagar",M:"ein mánaður",MM:"%d mánaðir",y:"eitt ár",yy:"%d ár"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return t}))},"079e":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("ja",{eras:[{since:"2019-05-01",offset:1,name:"令和",narrow:"㋿",abbr:"R"},{since:"1989-01-08",until:"2019-04-30",offset:1,name:"平成",narrow:"㍻",abbr:"H"},{since:"1926-12-25",until:"1989-01-07",offset:1,name:"昭和",narrow:"㍼",abbr:"S"},{since:"1912-07-30",until:"1926-12-24",offset:1,name:"大正",narrow:"㍽",abbr:"T"},{since:"1873-01-01",until:"1912-07-29",offset:6,name:"明治",narrow:"㍾",abbr:"M"},{since:"0001-01-01",until:"1873-12-31",offset:1,name:"西暦",narrow:"AD",abbr:"AD"},{since:"0000-12-31",until:-1/0,offset:1,name:"紀元前",narrow:"BC",abbr:"BC"}],eraYearOrdinalRegex:/(元|\d+)年/,eraYearOrdinalParse:function(e,t){return"元"===t[1]?1:parseInt(t[1]||e,10)},months:"1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月".split("_"),monthsShort:"1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月".split("_"),weekdays:"日曜日_月曜日_火曜日_水曜日_木曜日_金曜日_土曜日".split("_"),weekdaysShort:"日_月_火_水_木_金_土".split("_"),weekdaysMin:"日_月_火_水_木_金_土".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY/MM/DD",LL:"YYYY年M月D日",LLL:"YYYY年M月D日 HH:mm",LLLL:"YYYY年M月D日 dddd HH:mm",l:"YYYY/MM/DD",ll:"YYYY年M月D日",lll:"YYYY年M月D日 HH:mm",llll:"YYYY年M月D日(ddd) HH:mm"},meridiemParse:/午前|午後/i,isPM:function(e){return"午後"===e},meridiem:function(e,t,n){return e<12?"午前":"午後"},calendar:{sameDay:"[今日] LT",nextDay:"[明日] LT",nextWeek:function(e){return e.week()!==this.week()?"[来週]dddd LT":"dddd LT"},lastDay:"[昨日] LT",lastWeek:function(e){return this.week()!==e.week()?"[先週]dddd LT":"dddd LT"},sameElse:"L"},dayOfMonthOrdinalParse:/\d{1,2}日/,ordinal:function(e,t){switch(t){case"y":return 1===e?"元年":e+"年";case"d":case"D":case"DDD":return e+"日";default:return e}},relativeTime:{future:"%s後",past:"%s前",s:"数秒",ss:"%d秒",m:"1分",mm:"%d分",h:"1時間",hh:"%d時間",d:"1日",dd:"%d日",M:"1ヶ月",MM:"%dヶ月",y:"1年",yy:"%d年"}});return t}))},"07ac":function(e,t,n){var r=n("23e7"),i=n("6f53").values;r({target:"Object",stat:!0},{values:function(e){return i(e)}})},"0a3c":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t="ene._feb._mar._abr._may._jun._jul._ago._sep._oct._nov._dic.".split("_"),n="ene_feb_mar_abr_may_jun_jul_ago_sep_oct_nov_dic".split("_"),r=[/^ene/i,/^feb/i,/^mar/i,/^abr/i,/^may/i,/^jun/i,/^jul/i,/^ago/i,/^sep/i,/^oct/i,/^nov/i,/^dic/i],i=/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre|ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,a=e.defineLocale("es-do",{months:"enero_febrero_marzo_abril_mayo_junio_julio_agosto_septiembre_octubre_noviembre_diciembre".split("_"),monthsShort:function(e,r){return e?/-MMM-/.test(r)?n[e.month()]:t[e.month()]:t},monthsRegex:i,monthsShortRegex:i,monthsStrictRegex:/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre)/i,monthsShortStrictRegex:/^(ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,monthsParse:r,longMonthsParse:r,shortMonthsParse:r,weekdays:"domingo_lunes_martes_miércoles_jueves_viernes_sábado".split("_"),weekdaysShort:"dom._lun._mar._mié._jue._vie._sáb.".split("_"),weekdaysMin:"do_lu_ma_mi_ju_vi_sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY h:mm A",LLLL:"dddd, D [de] MMMM [de] YYYY h:mm A"},calendar:{sameDay:function(){return"[hoy a la"+(1!==this.hours()?"s":"")+"] LT"},nextDay:function(){return"[mañana a la"+(1!==this.hours()?"s":"")+"] LT"},nextWeek:function(){return"dddd [a la"+(1!==this.hours()?"s":"")+"] LT"},lastDay:function(){return"[ayer a la"+(1!==this.hours()?"s":"")+"] LT"},lastWeek:function(){return"[el] dddd [pasado a la"+(1!==this.hours()?"s":"")+"] LT"},sameElse:"L"},relativeTime:{future:"en %s",past:"hace %s",s:"unos segundos",ss:"%d segundos",m:"un minuto",mm:"%d minutos",h:"una hora",hh:"%d horas",d:"un día",dd:"%d días",w:"una semana",ww:"%d semanas",M:"un mes",MM:"%d meses",y:"un año",yy:"%d años"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,doy:4}});return a}))},"0a84":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("ar-ma",{months:"يناير_فبراير_مارس_أبريل_ماي_يونيو_يوليوز_غشت_شتنبر_أكتوبر_نونبر_دجنبر".split("_"),monthsShort:"يناير_فبراير_مارس_أبريل_ماي_يونيو_يوليوز_غشت_شتنبر_أكتوبر_نونبر_دجنبر".split("_"),weekdays:"الأحد_الإثنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت".split("_"),weekdaysShort:"احد_اثنين_ثلاثاء_اربعاء_خميس_جمعة_سبت".split("_"),weekdaysMin:"ح_ن_ث_ر_خ_ج_س".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[اليوم على الساعة] LT",nextDay:"[غدا على الساعة] LT",nextWeek:"dddd [على الساعة] LT",lastDay:"[أمس على الساعة] LT",lastWeek:"dddd [على الساعة] LT",sameElse:"L"},relativeTime:{future:"في %s",past:"منذ %s",s:"ثوان",ss:"%d ثانية",m:"دقيقة",mm:"%d دقائق",h:"ساعة",hh:"%d ساعات",d:"يوم",dd:"%d أيام",M:"شهر",MM:"%d أشهر",y:"سنة",yy:"%d سنوات"},week:{dow:1,doy:4}});return t}))},"0aa5":function(e,t,n){},"0b25":function(e,t,n){var r=n("a691"),i=n("50c4");e.exports=function(e){if(void 0===e)return 0;var t=r(e),n=i(t);if(t!==n)throw RangeError("Wrong length or index");return n}},"0caa":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e,t,n,r){var i={s:["thoddea sekondamni","thodde sekond"],ss:[e+" sekondamni",e+" sekond"],m:["eka mintan","ek minut"],mm:[e+" mintamni",e+" mintam"],h:["eka voran","ek vor"],hh:[e+" voramni",e+" voram"],d:["eka disan","ek dis"],dd:[e+" disamni",e+" dis"],M:["eka mhoinean","ek mhoino"],MM:[e+" mhoineamni",e+" mhoine"],y:["eka vorsan","ek voros"],yy:[e+" vorsamni",e+" vorsam"]};return r?i[n][0]:i[n][1]}var n=e.defineLocale("gom-latn",{months:{standalone:"Janer_Febrer_Mars_Abril_Mai_Jun_Julai_Agost_Setembr_Otubr_Novembr_Dezembr".split("_"),format:"Janerachea_Febrerachea_Marsachea_Abrilachea_Maiachea_Junachea_Julaiachea_Agostachea_Setembrachea_Otubrachea_Novembrachea_Dezembrachea".split("_"),isFormat:/MMMM(\s)+D[oD]?/},monthsShort:"Jan._Feb._Mars_Abr._Mai_Jun_Jul._Ago._Set._Otu._Nov._Dez.".split("_"),monthsParseExact:!0,weekdays:"Aitar_Somar_Mongllar_Budhvar_Birestar_Sukrar_Son'var".split("_"),weekdaysShort:"Ait._Som._Mon._Bud._Bre._Suk._Son.".split("_"),weekdaysMin:"Ai_Sm_Mo_Bu_Br_Su_Sn".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"A h:mm [vazta]",LTS:"A h:mm:ss [vazta]",L:"DD-MM-YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY A h:mm [vazta]",LLLL:"dddd, MMMM Do, YYYY, A h:mm [vazta]",llll:"ddd, D MMM YYYY, A h:mm [vazta]"},calendar:{sameDay:"[Aiz] LT",nextDay:"[Faleam] LT",nextWeek:"[Fuddlo] dddd[,] LT",lastDay:"[Kal] LT",lastWeek:"[Fattlo] dddd[,] LT",sameElse:"L"},relativeTime:{future:"%s",past:"%s adim",s:t,ss:t,m:t,mm:t,h:t,hh:t,d:t,dd:t,M:t,MM:t,y:t,yy:t},dayOfMonthOrdinalParse:/\d{1,2}(er)/,ordinal:function(e,t){switch(t){case"D":return e+"er";default:case"M":case"Q":case"DDD":case"d":case"w":case"W":return e}},week:{dow:0,doy:3},meridiemParse:/rati|sokallim|donparam|sanje/,meridiemHour:function(e,t){return 12===e&&(e=0),"rati"===t?e<4?e:e+12:"sokallim"===t?e:"donparam"===t?e>12?e:e+12:"sanje"===t?e+12:void 0},meridiem:function(e,t,n){return e<4?"rati":e<12?"sokallim":e<16?"donparam":e<20?"sanje":"rati"}});return n}))},"0cfb":function(e,t,n){var r=n("83ab"),i=n("d039"),a=n("cc12");e.exports=!r&&!i((function(){return 7!=Object.defineProperty(a("div"),"a",{get:function(){return 7}}).a}))},"0d3b":function(e,t,n){var r=n("d039"),i=n("b622"),a=n("c430"),o=i("iterator");e.exports=!r((function(){var e=new URL("b?a=1&b=2&c=3","http://a"),t=e.searchParams,n="";return e.pathname="c%20d",t.forEach((function(e,r){t["delete"]("b"),n+=r+e})),a&&!e.toJSON||!t.sort||"http://a/c%20d?a=1&c=3"!==e.href||"3"!==t.get("c")||"a=1"!==String(new URLSearchParams("?a=1"))||!t[o]||"a"!==new URL("https://a@b").username||"b"!==new URLSearchParams(new URLSearchParams("a=b")).get("a")||"xn--e1aybc"!==new URL("http://тест").host||"#%D0%B1"!==new URL("http://a#б").hash||"a1c3"!==n||"x"!==new URL("http://x",void 0).host}))},"0d8e":function(e,t,n){},"0da8":function(e,t,n){"use strict";n("356a")},"0e37":function(e,t,n){"use strict";n("3169")},"0e49":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("fr-ch",{months:"janvier_février_mars_avril_mai_juin_juillet_août_septembre_octobre_novembre_décembre".split("_"),monthsShort:"janv._févr._mars_avr._mai_juin_juil._août_sept._oct._nov._déc.".split("_"),monthsParseExact:!0,weekdays:"dimanche_lundi_mardi_mercredi_jeudi_vendredi_samedi".split("_"),weekdaysShort:"dim._lun._mar._mer._jeu._ven._sam.".split("_"),weekdaysMin:"di_lu_ma_me_je_ve_sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[Aujourd’hui à] LT",nextDay:"[Demain à] LT",nextWeek:"dddd [à] LT",lastDay:"[Hier à] LT",lastWeek:"dddd [dernier à] LT",sameElse:"L"},relativeTime:{future:"dans %s",past:"il y a %s",s:"quelques secondes",ss:"%d secondes",m:"une minute",mm:"%d minutes",h:"une heure",hh:"%d heures",d:"un jour",dd:"%d jours",M:"un mois",MM:"%d mois",y:"un an",yy:"%d ans"},dayOfMonthOrdinalParse:/\d{1,2}(er|e)/,ordinal:function(e,t){switch(t){default:case"M":case"Q":case"D":case"DDD":case"d":return e+(1===e?"er":"e");case"w":case"W":return e+(1===e?"re":"e")}},week:{dow:1,doy:4}});return t}))},"0e6b":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("en-au",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(e){var t=e%10,n=1===~~(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th";return e+n},week:{dow:0,doy:4}});return t}))},"0e81":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"'inci",5:"'inci",8:"'inci",70:"'inci",80:"'inci",2:"'nci",7:"'nci",20:"'nci",50:"'nci",3:"'üncü",4:"'üncü",100:"'üncü",6:"'ncı",9:"'uncu",10:"'uncu",30:"'uncu",60:"'ıncı",90:"'ıncı"},n=e.defineLocale("tr",{months:"Ocak_Şubat_Mart_Nisan_Mayıs_Haziran_Temmuz_Ağustos_Eylül_Ekim_Kasım_Aralık".split("_"),monthsShort:"Oca_Şub_Mar_Nis_May_Haz_Tem_Ağu_Eyl_Eki_Kas_Ara".split("_"),weekdays:"Pazar_Pazartesi_Salı_Çarşamba_Perşembe_Cuma_Cumartesi".split("_"),weekdaysShort:"Paz_Pts_Sal_Çar_Per_Cum_Cts".split("_"),weekdaysMin:"Pz_Pt_Sa_Ça_Pe_Cu_Ct".split("_"),meridiem:function(e,t,n){return e<12?n?"öö":"ÖÖ":n?"ös":"ÖS"},meridiemParse:/öö|ÖÖ|ös|ÖS/,isPM:function(e){return"ös"===e||"ÖS"===e},longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[bugün saat] LT",nextDay:"[yarın saat] LT",nextWeek:"[gelecek] dddd [saat] LT",lastDay:"[dün] LT",lastWeek:"[geçen] dddd [saat] LT",sameElse:"L"},relativeTime:{future:"%s sonra",past:"%s önce",s:"birkaç saniye",ss:"%d saniye",m:"bir dakika",mm:"%d dakika",h:"bir saat",hh:"%d saat",d:"bir gün",dd:"%d gün",w:"bir hafta",ww:"%d hafta",M:"bir ay",MM:"%d ay",y:"bir yıl",yy:"%d yıl"},ordinal:function(e,n){switch(n){case"d":case"D":case"Do":case"DD":return e;default:if(0===e)return e+"'ıncı";var r=e%10,i=e%100-r,a=e>=100?100:null;return e+(t[r]||t[i]||t[a])}},week:{dow:1,doy:7}});return n}))},"0f14":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("da",{months:"januar_februar_marts_april_maj_juni_juli_august_september_oktober_november_december".split("_"),monthsShort:"jan_feb_mar_apr_maj_jun_jul_aug_sep_okt_nov_dec".split("_"),weekdays:"søndag_mandag_tirsdag_onsdag_torsdag_fredag_lørdag".split("_"),weekdaysShort:"søn_man_tir_ons_tor_fre_lør".split("_"),weekdaysMin:"sø_ma_ti_on_to_fr_lø".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY HH:mm",LLLL:"dddd [d.] D. MMMM YYYY [kl.] HH:mm"},calendar:{sameDay:"[i dag kl.] LT",nextDay:"[i morgen kl.] LT",nextWeek:"på dddd [kl.] LT",lastDay:"[i går kl.] LT",lastWeek:"[i] dddd[s kl.] LT",sameElse:"L"},relativeTime:{future:"om %s",past:"%s siden",s:"få sekunder",ss:"%d sekunder",m:"et minut",mm:"%d minutter",h:"en time",hh:"%d timer",d:"en dag",dd:"%d dage",M:"en måned",MM:"%d måneder",y:"et år",yy:"%d år"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return t}))},"0f38":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("tl-ph",{months:"Enero_Pebrero_Marso_Abril_Mayo_Hunyo_Hulyo_Agosto_Setyembre_Oktubre_Nobyembre_Disyembre".split("_"),monthsShort:"Ene_Peb_Mar_Abr_May_Hun_Hul_Ago_Set_Okt_Nob_Dis".split("_"),weekdays:"Linggo_Lunes_Martes_Miyerkules_Huwebes_Biyernes_Sabado".split("_"),weekdaysShort:"Lin_Lun_Mar_Miy_Huw_Biy_Sab".split("_"),weekdaysMin:"Li_Lu_Ma_Mi_Hu_Bi_Sab".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"MM/D/YYYY",LL:"MMMM D, YYYY",LLL:"MMMM D, YYYY HH:mm",LLLL:"dddd, MMMM DD, YYYY HH:mm"},calendar:{sameDay:"LT [ngayong araw]",nextDay:"[Bukas ng] LT",nextWeek:"LT [sa susunod na] dddd",lastDay:"LT [kahapon]",lastWeek:"LT [noong nakaraang] dddd",sameElse:"L"},relativeTime:{future:"sa loob ng %s",past:"%s ang nakalipas",s:"ilang segundo",ss:"%d segundo",m:"isang minuto",mm:"%d minuto",h:"isang oras",hh:"%d oras",d:"isang araw",dd:"%d araw",M:"isang buwan",MM:"%d buwan",y:"isang taon",yy:"%d taon"},dayOfMonthOrdinalParse:/\d{1,2}/,ordinal:function(e){return e},week:{dow:1,doy:4}});return t}))},"0ff2":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("eu",{months:"urtarrila_otsaila_martxoa_apirila_maiatza_ekaina_uztaila_abuztua_iraila_urria_azaroa_abendua".split("_"),monthsShort:"urt._ots._mar._api._mai._eka._uzt._abu._ira._urr._aza._abe.".split("_"),monthsParseExact:!0,weekdays:"igandea_astelehena_asteartea_asteazkena_osteguna_ostirala_larunbata".split("_"),weekdaysShort:"ig._al._ar._az._og._ol._lr.".split("_"),weekdaysMin:"ig_al_ar_az_og_ol_lr".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"YYYY[ko] MMMM[ren] D[a]",LLL:"YYYY[ko] MMMM[ren] D[a] HH:mm",LLLL:"dddd, YYYY[ko] MMMM[ren] D[a] HH:mm",l:"YYYY-M-D",ll:"YYYY[ko] MMM D[a]",lll:"YYYY[ko] MMM D[a] HH:mm",llll:"ddd, YYYY[ko] MMM D[a] HH:mm"},calendar:{sameDay:"[gaur] LT[etan]",nextDay:"[bihar] LT[etan]",nextWeek:"dddd LT[etan]",lastDay:"[atzo] LT[etan]",lastWeek:"[aurreko] dddd LT[etan]",sameElse:"L"},relativeTime:{future:"%s barru",past:"duela %s",s:"segundo batzuk",ss:"%d segundo",m:"minutu bat",mm:"%d minutu",h:"ordu bat",hh:"%d ordu",d:"egun bat",dd:"%d egun",M:"hilabete bat",MM:"%d hilabete",y:"urte bat",yy:"%d urte"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}});return t}))},"10e8":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("th",{months:"มกราคม_กุมภาพันธ์_มีนาคม_เมษายน_พฤษภาคม_มิถุนายน_กรกฎาคม_สิงหาคม_กันยายน_ตุลาคม_พฤศจิกายน_ธันวาคม".split("_"),monthsShort:"ม.ค._ก.พ._มี.ค._เม.ย._พ.ค._มิ.ย._ก.ค._ส.ค._ก.ย._ต.ค._พ.ย._ธ.ค.".split("_"),monthsParseExact:!0,weekdays:"อาทิตย์_จันทร์_อังคาร_พุธ_พฤหัสบดี_ศุกร์_เสาร์".split("_"),weekdaysShort:"อาทิตย์_จันทร์_อังคาร_พุธ_พฤหัส_ศุกร์_เสาร์".split("_"),weekdaysMin:"อา._จ._อ._พ._พฤ._ศ._ส.".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY เวลา H:mm",LLLL:"วันddddที่ D MMMM YYYY เวลา H:mm"},meridiemParse:/ก่อนเที่ยง|หลังเที่ยง/,isPM:function(e){return"หลังเที่ยง"===e},meridiem:function(e,t,n){return e<12?"ก่อนเที่ยง":"หลังเที่ยง"},calendar:{sameDay:"[วันนี้ เวลา] LT",nextDay:"[พรุ่งนี้ เวลา] LT",nextWeek:"dddd[หน้า เวลา] LT",lastDay:"[เมื่อวานนี้ เวลา] LT",lastWeek:"[วัน]dddd[ที่แล้ว เวลา] LT",sameElse:"L"},relativeTime:{future:"อีก %s",past:"%sที่แล้ว",s:"ไม่กี่วินาที",ss:"%d วินาที",m:"1 นาที",mm:"%d นาที",h:"1 ชั่วโมง",hh:"%d ชั่วโมง",d:"1 วัน",dd:"%d วัน",w:"1 สัปดาห์",ww:"%d สัปดาห์",M:"1 เดือน",MM:"%d เดือน",y:"1 ปี",yy:"%d ปี"}});return t}))},1115:function(e,t,n){"use strict";n("9b36")},1148:function(e,t,n){"use strict";var r=n("a691"),i=n("1d80");e.exports="".repeat||function(e){var t=String(i(this)),n="",a=r(e);if(a<0||a==1/0)throw RangeError("Wrong number of repetitions");for(;a>0;(a>>>=1)&&(t+=t))1&a&&(n+=t);return n}},1276:function(e,t,n){"use strict";var r=n("d784"),i=n("44e7"),a=n("825a"),o=n("1d80"),s=n("4840"),l=n("8aa5"),u=n("50c4"),c=n("14c3"),d=n("9263"),f=n("d039"),h=[].push,p=Math.min,m=4294967295,_=!f((function(){return!RegExp(m,"y")}));r("split",2,(function(e,t,n){var r;return r="c"=="abbc".split(/(b)*/)[1]||4!="test".split(/(?:)/,-1).length||2!="ab".split(/(?:ab)*/).length||4!=".".split(/(.?)(.?)/).length||".".split(/()()/).length>1||"".split(/.?/).length?function(e,n){var r=String(o(this)),a=void 0===n?m:n>>>0;if(0===a)return[];if(void 0===e)return[r];if(!i(e))return t.call(r,e,a);var s,l,u,c=[],f=(e.ignoreCase?"i":"")+(e.multiline?"m":"")+(e.unicode?"u":"")+(e.sticky?"y":""),p=0,_=new RegExp(e.source,f+"g");while(s=d.call(_,r)){if(l=_.lastIndex,l>p&&(c.push(r.slice(p,s.index)),s.length>1&&s.index<r.length&&h.apply(c,s.slice(1)),u=s[0].length,p=l,c.length>=a))break;_.lastIndex===s.index&&_.lastIndex++}return p===r.length?!u&&_.test("")||c.push(""):c.push(r.slice(p)),c.length>a?c.slice(0,a):c}:"0".split(void 0,0).length?function(e,n){return void 0===e&&0===n?[]:t.call(this,e,n)}:t,[function(t,n){var i=o(this),a=void 0==t?void 0:t[e];return void 0!==a?a.call(t,i,n):r.call(String(i),t,n)},function(e,i){var o=n(r,e,this,i,r!==t);if(o.done)return o.value;var d=a(e),f=String(this),h=s(d,RegExp),v=d.unicode,g=(d.ignoreCase?"i":"")+(d.multiline?"m":"")+(d.unicode?"u":"")+(_?"y":"g"),b=new h(_?d:"^(?:"+d.source+")",g),y=void 0===i?m:i>>>0;if(0===y)return[];if(0===f.length)return null===c(b,f)?[f]:[];var M=0,k=0,w=[];while(k<f.length){b.lastIndex=_?k:0;var L,x=c(b,_?f:f.slice(k));if(null===x||(L=p(u(b.lastIndex+(_?0:k)),f.length))===M)k=l(f,k,v);else{if(w.push(f.slice(M,k)),w.length===y)return w;for(var S=1;S<=x.length-1;S++)if(w.push(x[S]),w.length===y)return w;k=M=L}}return w.push(f.slice(M)),w}]}),!_)},"131a":function(e,t,n){var r=n("23e7"),i=n("d2bb");r({target:"Object",stat:!0},{setPrototypeOf:i})},"13d5":function(e,t,n){"use strict";var r=n("23e7"),i=n("d58f").left,a=n("a640"),o=n("ae40"),s=n("2d00"),l=n("605d"),u=a("reduce"),c=o("reduce",{1:0}),d=!l&&s>79&&s<83;r({target:"Array",proto:!0,forced:!u||!c||d},{reduce:function(e){return i(this,e,arguments.length,arguments.length>1?arguments[1]:void 0)}})},"13e9":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={words:{ss:["секунда","секунде","секунди"],m:["један минут","једне минуте"],mm:["минут","минуте","минута"],h:["један сат","једног сата"],hh:["сат","сата","сати"],dd:["дан","дана","дана"],MM:["месец","месеца","месеци"],yy:["година","године","година"]},correctGrammaticalCase:function(e,t){return 1===e?t[0]:e>=2&&e<=4?t[1]:t[2]},translate:function(e,n,r){var i=t.words[r];return 1===r.length?n?i[0]:i[1]:e+" "+t.correctGrammaticalCase(e,i)}},n=e.defineLocale("sr-cyrl",{months:"јануар_фебруар_март_април_мај_јун_јул_август_септембар_октобар_новембар_децембар".split("_"),monthsShort:"јан._феб._мар._апр._мај_јун_јул_авг._сеп._окт._нов._дец.".split("_"),monthsParseExact:!0,weekdays:"недеља_понедељак_уторак_среда_четвртак_петак_субота".split("_"),weekdaysShort:"нед._пон._уто._сре._чет._пет._суб.".split("_"),weekdaysMin:"не_по_ут_ср_че_пе_су".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"D. M. YYYY.",LL:"D. MMMM YYYY.",LLL:"D. MMMM YYYY. H:mm",LLLL:"dddd, D. MMMM YYYY. H:mm"},calendar:{sameDay:"[данас у] LT",nextDay:"[сутра у] LT",nextWeek:function(){switch(this.day()){case 0:return"[у] [недељу] [у] LT";case 3:return"[у] [среду] [у] LT";case 6:return"[у] [суботу] [у] LT";case 1:case 2:case 4:case 5:return"[у] dddd [у] LT"}},lastDay:"[јуче у] LT",lastWeek:function(){var e=["[прошле] [недеље] [у] LT","[прошлог] [понедељка] [у] LT","[прошлог] [уторка] [у] LT","[прошле] [среде] [у] LT","[прошлог] [четвртка] [у] LT","[прошлог] [петка] [у] LT","[прошле] [суботе] [у] LT"];return e[this.day()]},sameElse:"L"},relativeTime:{future:"за %s",past:"пре %s",s:"неколико секунди",ss:t.translate,m:t.translate,mm:t.translate,h:t.translate,hh:t.translate,d:"дан",dd:t.translate,M:"месец",MM:t.translate,y:"годину",yy:t.translate},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}});return n}))},"145e":function(e,t,n){"use strict";var r=n("7b0b"),i=n("23cb"),a=n("50c4"),o=Math.min;e.exports=[].copyWithin||function(e,t){var n=r(this),s=a(n.length),l=i(e,s),u=i(t,s),c=arguments.length>2?arguments[2]:void 0,d=o((void 0===c?s:i(c,s))-u,s-l),f=1;u<l&&l<u+d&&(f=-1,u+=d-1,l+=d-1);while(d-- >0)u in n?n[l]=n[u]:delete n[l],l+=f,u+=f;return n}},"14c3":function(e,t,n){var r=n("c6b6"),i=n("9263");e.exports=function(e,t){var n=e.exec;if("function"===typeof n){var a=n.call(e,t);if("object"!==typeof a)throw TypeError("RegExp exec method returned something other than an Object or null");return a}if("RegExp"!==r(e))throw TypeError("RegExp#exec called on incompatible receiver");return i.call(e,t)}},"153d":function(e,t,n){"use strict";n("56c0")},"159b":function(e,t,n){var r=n("da84"),i=n("fdbc"),a=n("17c2"),o=n("9112");for(var s in i){var l=r[s],u=l&&l.prototype;if(u&&u.forEach!==a)try{o(u,"forEach",a)}catch(c){u.forEach=a}}},"167b":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("oc-lnc",{months:{standalone:"genièr_febrièr_març_abril_mai_junh_julhet_agost_setembre_octòbre_novembre_decembre".split("_"),format:"de genièr_de febrièr_de març_d'abril_de mai_de junh_de julhet_d'agost_de setembre_d'octòbre_de novembre_de decembre".split("_"),isFormat:/D[oD]?(\s)+MMMM/},monthsShort:"gen._febr._març_abr._mai_junh_julh._ago._set._oct._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"dimenge_diluns_dimars_dimècres_dijòus_divendres_dissabte".split("_"),weekdaysShort:"dg._dl._dm._dc._dj._dv._ds.".split("_"),weekdaysMin:"dg_dl_dm_dc_dj_dv_ds".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM [de] YYYY",ll:"D MMM YYYY",LLL:"D MMMM [de] YYYY [a] H:mm",lll:"D MMM YYYY, H:mm",LLLL:"dddd D MMMM [de] YYYY [a] H:mm",llll:"ddd D MMM YYYY, H:mm"},calendar:{sameDay:"[uèi a] LT",nextDay:"[deman a] LT",nextWeek:"dddd [a] LT",lastDay:"[ièr a] LT",lastWeek:"dddd [passat a] LT",sameElse:"L"},relativeTime:{future:"d'aquí %s",past:"fa %s",s:"unas segondas",ss:"%d segondas",m:"una minuta",mm:"%d minutas",h:"una ora",hh:"%d oras",d:"un jorn",dd:"%d jorns",M:"un mes",MM:"%d meses",y:"un an",yy:"%d ans"},dayOfMonthOrdinalParse:/\d{1,2}(r|n|t|è|a)/,ordinal:function(e,t){var n=1===e?"r":2===e?"n":3===e?"r":4===e?"t":"è";return"w"!==t&&"W"!==t||(n="a"),e+n},week:{dow:1,doy:4}});return t}))},1683:function(e,t,n){},"170b":function(e,t,n){"use strict";var r=n("ebb5"),i=n("50c4"),a=n("23cb"),o=n("4840"),s=r.aTypedArray,l=r.exportTypedArrayMethod;l("subarray",(function(e,t){var n=s(this),r=n.length,l=a(e,r);return new(o(n,n.constructor))(n.buffer,n.byteOffset+l*n.BYTES_PER_ELEMENT,i((void 0===t?r:a(t,r))-l))}))},1773:function(e,t,n){"use strict";n("1683")},"17c2":function(e,t,n){"use strict";var r=n("b727").forEach,i=n("a640"),a=n("ae40"),o=i("forEach"),s=a("forEach");e.exports=o&&s?[].forEach:function(e){return r(this,e,arguments.length>1?arguments[1]:void 0)}},"17f1":function(e,t,n){"use strict";n("1a7b")},"182d":function(e,t,n){var r=n("f8cd");e.exports=function(e,t){var n=r(e);if(n%t)throw RangeError("Wrong offset");return n}},1881:function(e,t,n){!function(t,n){e.exports=n()}(window,(function(){return n={},e.m=t=[function(e,t,n){var r=n(6);"string"==typeof r&&(r=[[e.i,r,""]]),r.locals&&(e.exports=r.locals),(0,n(4).default)("27d83796",r,!1,{})},function(e,t,n){var r=n(8);"string"==typeof r&&(r=[[e.i,r,""]]),r.locals&&(e.exports=r.locals),(0,n(4).default)("0e783494",r,!1,{})},function(e,t,n){var r=n(10);"string"==typeof r&&(r=[[e.i,r,""]]),r.locals&&(e.exports=r.locals),(0,n(4).default)("17757f60",r,!1,{})},function(e,t){e.exports=function(e){var t=[];return t.toString=function(){return this.map((function(t){var n=function(e,t){var n=e[1]||"",r=e[3];if(!r)return n;if(t&&"function"==typeof btoa){var i=function(e){return"/*# sourceMappingURL=data:application/json;charset=utf-8;base64,"+btoa(unescape(encodeURIComponent(JSON.stringify(e))))+" */"}(r),a=r.sources.map((function(e){return"/*# sourceURL="+r.sourceRoot+e+" */"}));return[n].concat(a).concat([i]).join("\n")}return[n].join("\n")}(t,e);return t[2]?"@media "+t[2]+"{"+n+"}":n})).join("")},t.i=function(e,n){"string"==typeof e&&(e=[[null,e,""]]);for(var r={},i=0;i<this.length;i++){var a=this[i][0];"number"==typeof a&&(r[a]=!0)}for(i=0;i<e.length;i++){var o=e[i];"number"==typeof o[0]&&r[o[0]]||(n&&!o[2]?o[2]=n:n&&(o[2]="("+o[2]+") and ("+n+")"),t.push(o))}},t}},function(e,t,n){"use strict";function r(e,t){for(var n=[],r={},i=0;i<t.length;i++){var a=t[i],o=a[0],s={id:e+":"+i,css:a[1],media:a[2],sourceMap:a[3]};r[o]?r[o].parts.push(s):n.push(r[o]={id:o,parts:[s]})}return n}n.r(t),n.d(t,"default",(function(){return p}));var i="undefined"!=typeof document;if("undefined"!=typeof DEBUG&&DEBUG&&!i)throw new Error("vue-style-loader cannot be used in a non-browser environment. Use { target: 'node' } in your Webpack config to indicate a server-rendering environment.");var a={},o=i&&(document.head||document.getElementsByTagName("head")[0]),s=null,l=0,u=!1,c=function(){},d=null,f="data-vue-ssr-id",h="undefined"!=typeof navigator&&/msie [6-9]\b/.test(navigator.userAgent.toLowerCase());function p(e,t,n,i){u=n,d=i||{};var o=r(e,t);return m(o),function(t){for(var n=[],i=0;i<o.length;i++){var s=o[i];(l=a[s.id]).refs--,n.push(l)}for(t?m(o=r(e,t)):o=[],i=0;i<n.length;i++){var l;if(0===(l=n[i]).refs){for(var u=0;u<l.parts.length;u++)l.parts[u]();delete a[l.id]}}}}function m(e){for(var t=0;t<e.length;t++){var n=e[t],r=a[n.id];if(r){r.refs++;for(var i=0;i<r.parts.length;i++)r.parts[i](n.parts[i]);for(;i<n.parts.length;i++)r.parts.push(v(n.parts[i]));r.parts.length>n.parts.length&&(r.parts.length=n.parts.length)}else{var o=[];for(i=0;i<n.parts.length;i++)o.push(v(n.parts[i]));a[n.id]={id:n.id,refs:1,parts:o}}}}function _(){var e=document.createElement("style");return e.type="text/css",o.appendChild(e),e}function v(e){var t,n,r=document.querySelector("style["+f+'~="'+e.id+'"]');if(r){if(u)return c;r.parentNode.removeChild(r)}if(h){var i=l++;r=s=s||_(),t=y.bind(null,r,i,!1),n=y.bind(null,r,i,!0)}else r=_(),t=function(e,t){var n=t.css,r=t.media,i=t.sourceMap;if(r&&e.setAttribute("media",r),d.ssrId&&e.setAttribute(f,t.id),i&&(n+="\n/*# sourceURL="+i.sources[0]+" */",n+="\n/*# sourceMappingURL=data:application/json;base64,"+btoa(unescape(encodeURIComponent(JSON.stringify(i))))+" */"),e.styleSheet)e.styleSheet.cssText=n;else{for(;e.firstChild;)e.removeChild(e.firstChild);e.appendChild(document.createTextNode(n))}}.bind(null,r),n=function(){r.parentNode.removeChild(r)};return t(e),function(r){if(r){if(r.css===e.css&&r.media===e.media&&r.sourceMap===e.sourceMap)return;t(e=r)}else n()}}var g,b=(g=[],function(e,t){return g[e]=t,g.filter(Boolean).join("\n")});function y(e,t,n,r){var i=n?"":r.css;if(e.styleSheet)e.styleSheet.cssText=b(t,i);else{var a=document.createTextNode(i),o=e.childNodes;o[t]&&e.removeChild(o[t]),o.length?e.insertBefore(a,o[t]):e.appendChild(a)}}},function(e,t,n){"use strict";var r=n(0);n.n(r).a},function(e,t,n){(e.exports=n(3)(!1)).push([e.i,"\n.vue-modal-resizer {\n  display: block;\n  overflow: hidden;\n  position: absolute;\n  width: 12px;\n  height: 12px;\n  right: 0;\n  bottom: 0;\n  z-index: 9999999;\n  background: transparent;\n  cursor: se-resize;\n}\n.vue-modal-resizer::after {\n  display: block;\n  position: absolute;\n  content: '';\n  background: transparent;\n  left: 0;\n  top: 0;\n  width: 0;\n  height: 0;\n  border-bottom: 10px solid #ddd;\n  border-left: 10px solid transparent;\n}\n.vue-modal-resizer.clicked::after {\n  border-bottom: 10px solid #369be9;\n}\n",""])},function(e,t,n){"use strict";var r=n(1);n.n(r).a},function(e,t,n){(e.exports=n(3)(!1)).push([e.i,"\n.v--modal-block-scroll {\n  overflow: hidden;\n  width: 100vw;\n}\n.v--modal-overlay {\n  position: fixed;\n  box-sizing: border-box;\n  left: 0;\n  top: 0;\n  width: 100%;\n  height: 100vh;\n  background: rgba(0, 0, 0, 0.2);\n  z-index: 999;\n  opacity: 1;\n}\n.v--modal-overlay.scrollable {\n  height: 100%;\n  min-height: 100vh;\n  overflow-y: auto;\n  -webkit-overflow-scrolling: touch;\n}\n.v--modal-overlay .v--modal-background-click {\n  width: 100%;\n  min-height: 100%;\n  height: auto;\n}\n.v--modal-overlay .v--modal-box {\n  position: relative;\n  overflow: hidden;\n  box-sizing: border-box;\n}\n.v--modal-overlay.scrollable .v--modal-box {\n  margin-bottom: 2px;\n}\n.v--modal {\n  background-color: white;\n  text-align: left;\n  border-radius: 3px;\n  box-shadow: 0 20px 60px -2px rgba(27, 33, 58, 0.4);\n  padding: 0;\n}\n.v--modal.v--modal-fullscreen {\n  width: 100vw;\n  height: 100vh;\n  margin: 0;\n  left: 0;\n  top: 0;\n}\n.v--modal-top-right {\n  display: block;\n  position: absolute;\n  right: 0;\n  top: 0;\n}\n.overlay-fade-enter-active,\n.overlay-fade-leave-active {\n  transition: all 0.2s;\n}\n.overlay-fade-enter,\n.overlay-fade-leave-active {\n  opacity: 0;\n}\n.nice-modal-fade-enter-active,\n.nice-modal-fade-leave-active {\n  transition: all 0.4s;\n}\n.nice-modal-fade-enter,\n.nice-modal-fade-leave-active {\n  opacity: 0;\n  transform: translateY(-20px);\n}\n",""])},function(e,t,n){"use strict";var r=n(2);n.n(r).a},function(e,t,n){(e.exports=n(3)(!1)).push([e.i,"\n.vue-dialog div {\n  box-sizing: border-box;\n}\n.vue-dialog .dialog-flex {\n  width: 100%;\n  height: 100%;\n}\n.vue-dialog .dialog-content {\n  flex: 1 0 auto;\n  width: 100%;\n  padding: 15px;\n  font-size: 14px;\n}\n.vue-dialog .dialog-c-title {\n  font-weight: 600;\n  padding-bottom: 15px;\n}\n.vue-dialog .dialog-c-text {\n}\n.vue-dialog .vue-dialog-buttons {\n  display: flex;\n  flex: 0 1 auto;\n  width: 100%;\n  border-top: 1px solid #eee;\n}\n.vue-dialog .vue-dialog-buttons-none {\n  width: 100%;\n  padding-bottom: 15px;\n}\n.vue-dialog-button {\n  font-size: 12px !important;\n  background: transparent;\n  padding: 0;\n  margin: 0;\n  border: 0;\n  cursor: pointer;\n  box-sizing: border-box;\n  line-height: 40px;\n  height: 40px;\n  color: inherit;\n  font: inherit;\n  outline: none;\n}\n.vue-dialog-button:hover {\n  background: rgba(0, 0, 0, 0.01);\n}\n.vue-dialog-button:active {\n  background: rgba(0, 0, 0, 0.025);\n}\n.vue-dialog-button:not(:first-of-type) {\n  border-left: 1px solid #eee;\n}\n",""])},function(e,t,n){"use strict";function r(){var e=this,t=e.$createElement,n=e._self._c||t;return n("transition",{attrs:{name:e.overlayTransition}},[e.visibility.overlay?n("div",{ref:"overlay",class:e.overlayClass,attrs:{"aria-expanded":e.visibility.overlay.toString(),"data-modal":e.name}},[n("div",{staticClass:"v--modal-background-click",on:{mousedown:function(t){return t.target!==t.currentTarget?null:e.handleBackgroundClick(t)},touchstart:function(t){return t.target!==t.currentTarget?null:e.handleBackgroundClick(t)}}},[n("div",{staticClass:"v--modal-top-right"},[e._t("top-right")],2),e._v(" "),n("transition",{attrs:{name:e.transition},on:{"before-enter":e.beforeTransitionEnter,"after-enter":e.afterTransitionEnter,"after-leave":e.afterTransitionLeave}},[e.visibility.modal?n("div",{ref:"modal",class:e.modalClass,style:e.modalStyle,attrs:{role:"dialog","aria-modal":"true"}},[e._t("default"),e._v(" "),e.resizable&&!e.isAutoHeight?n("resizer",{attrs:{"min-width":e.minWidth,"min-height":e.minHeight,"max-width":e.maxWidth,"max-height":e.maxHeight},on:{resize:e.handleModalResize}}):e._e()],2):e._e()])],1)]):e._e()])}function i(){var e=this.$createElement;return(this._self._c||e)("div",{class:this.className})}function a(e,t){return function(e){if(Array.isArray(e))return e}(e)||function(e,t){var n=[],r=!0,i=!1,a=void 0;try{for(var o,s=e[Symbol.iterator]();!(r=(o=s.next()).done)&&(n.push(o.value),!t||n.length!==t);r=!0);}catch(e){i=!0,a=e}finally{try{r||null==s.return||s.return()}finally{if(i)throw a}}return n}(e,t)||function(){throw new TypeError("Invalid attempt to destructure non-iterable instance")}()}function o(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{},r=Object.keys(n);"function"==typeof Object.getOwnPropertySymbols&&(r=r.concat(Object.getOwnPropertySymbols(n).filter((function(e){return Object.getOwnPropertyDescriptor(n,e).enumerable})))),r.forEach((function(t){s(e,t,n[t])}))}return e}function s(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function l(e,t,n){return n<e?e:t<n?t:n}function u(){var e=window.innerWidth,t=document.documentElement.clientWidth;return e&&t?Math.min(e,t):t||e}n.r(t),n.d(t,"getModalsContainer",(function(){return P})),i._withStripped=r._withStripped=!0;var c=function(e){var t=0<arguments.length&&void 0!==e?e:0;return function(){return(t++).toString()}}(),d={name:"VueJsModalResizer",props:{minHeight:{type:Number,default:0},minWidth:{type:Number,default:0},maxWidth:{type:Number,default:Number.MAX_SAFE_INTEGER},maxHeight:{type:Number,default:Number.MAX_SAFE_INTEGER}},data:function(){return{clicked:!1,size:{}}},mounted:function(){this.$el.addEventListener("mousedown",this.start,!1)},computed:{className:function(){return{"vue-modal-resizer":!0,clicked:this.clicked}}},methods:{start:function(e){this.clicked=!0,window.addEventListener("mousemove",this.mousemove,!1),window.addEventListener("mouseup",this.stop,!1),e.stopPropagation(),e.preventDefault()},stop:function(){this.clicked=!1,window.removeEventListener("mousemove",this.mousemove,!1),window.removeEventListener("mouseup",this.stop,!1),this.$emit("resize-stop",{element:this.$el.parentElement,size:this.size})},mousemove:function(e){this.resize(e)},resize:function(e){var t=this.$el.parentElement;if(t){var n=e.clientX-t.offsetLeft,r=e.clientY-t.offsetTop,i=Math.min(u(),this.maxWidth),a=Math.min(window.innerHeight,this.maxHeight);n=l(this.minWidth,i,n),r=l(this.minHeight,a,r),this.size={width:n,height:r},t.style.width=n+"px",t.style.height=r+"px",this.$emit("resize",{element:t,size:this.size})}}}};function f(e,t,n,r,i,a,o,s){var l,u="function"==typeof e?e.options:e;if(t&&(u.render=t,u.staticRenderFns=n,u._compiled=!0),r&&(u.functional=!0),a&&(u._scopeId="data-v-"+a),o?(l=function(e){(e=e||this.$vnode&&this.$vnode.ssrContext||this.parent&&this.parent.$vnode&&this.parent.$vnode.ssrContext)||"undefined"==typeof __VUE_SSR_CONTEXT__||(e=__VUE_SSR_CONTEXT__),i&&i.call(this,e),e&&e._registeredComponents&&e._registeredComponents.add(o)},u._ssrRegister=l):i&&(l=s?function(){i.call(this,this.$root.$options.shadowRoot)}:i),l)if(u.functional){u._injectStyles=l;var c=u.render;u.render=function(e,t){return l.call(t),c(e,t)}}else{var d=u.beforeCreate;u.beforeCreate=d?[].concat(d,l):[l]}return{exports:e,options:u}}n(5);var h=f(d,i,[],!1,null,null,null);h.options.__file="src/Resizer.vue";var p=h.exports;function m(e){return(m="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function _(e){switch(m(e)){case"number":return{type:"px",value:e};case"string":return function(e){if("auto"===e)return{type:e,value:0};var t=b.find((function(t){return t.regexp.test(e)}));return t?{type:t.name,value:parseFloat(e)}:{type:"",value:e}}(e);default:return{type:"",value:e}}}function v(e){if("string"!=typeof e)return 0<=e;var t=_(e);return("%"===t.type||"px"===t.type)&&0<t.value}var g="[-+]?[0-9]*.?[0-9]+",b=[{name:"px",regexp:new RegExp("^".concat(g,"px$"))},{name:"%",regexp:new RegExp("^".concat(g,"%$"))},{name:"px",regexp:new RegExp("^".concat(g,"$"))}];function y(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}var M={name:"VueJsModal",props:{name:{required:!0,type:String},delay:{type:Number,default:0},resizable:{type:Boolean,default:!1},adaptive:{type:Boolean,default:!1},draggable:{type:[Boolean,String],default:!1},scrollable:{type:Boolean,default:!1},reset:{type:Boolean,default:!1},overlayTransition:{type:String,default:"overlay-fade"},transition:{type:String},clickToClose:{type:Boolean,default:!0},classes:{type:[String,Array],default:"v--modal"},styles:{type:[String,Array,Object]},minWidth:{type:Number,default:0,validator:function(e){return 0<=e}},minHeight:{type:Number,default:0,validator:function(e){return 0<=e}},maxWidth:{type:Number,default:Number.MAX_SAFE_INTEGER},maxHeight:{type:Number,default:Number.MAX_SAFE_INTEGER},width:{type:[Number,String],default:600,validator:v},height:{type:[Number,String],default:300,validator:function(e){return"auto"===e||v(e)}},pivotX:{type:Number,default:.5,validator:function(e){return 0<=e&&e<=1}},pivotY:{type:Number,default:.5,validator:function(e){return 0<=e&&e<=1}}},components:{Resizer:p},data:function(){return{visible:!1,visibility:{modal:!1,overlay:!1},shift:{left:0,top:0},modal:{width:0,widthType:"px",height:0,heightType:"px",renderedHeight:0},viewportHeight:0,viewportWidth:0,mutationObserver:null}},created:function(){this.setInitialSize()},beforeMount:function(){var e=this;if(I.event.$on("toggle",this.handleToggleEvent),window.addEventListener("resize",this.handleWindowResize),this.handleWindowResize(),this.scrollable&&!this.isAutoHeight&&console.warn('Modal "'.concat(this.name,'" has scrollable flag set to true ')+'but height is not "auto" ('.concat(this.height,")")),this.isAutoHeight){var t=function(){if("undefined"!=typeof window)for(var e=["","WebKit","Moz","O","Ms"],t=0;t<e.length;t++){var n=e[t]+"MutationObserver";if(n in window)return window[n]}return!1}();t?this.mutationObserver=new t((function(t){e.updateRenderedHeight()})):console.warn("MutationObserver was not found. Vue-js-modal automatic resizing relies heavily on MutationObserver. Please make sure to provide shim for it.")}this.clickToClose&&window.addEventListener("keyup",this.handleEscapeKeyUp)},beforeDestroy:function(){I.event.$off("toggle",this.handleToggleEvent),window.removeEventListener("resize",this.handleWindowResize),this.clickToClose&&window.removeEventListener("keyup",this.handleEscapeKeyUp),this.scrollable&&document.body.classList.remove("v--modal-block-scroll")},computed:{isAutoHeight:function(){return"auto"===this.modal.heightType},position:function(){var e=this.viewportHeight,t=this.viewportWidth,n=this.shift,r=this.pivotX,i=this.pivotY,a=this.trueModalWidth,o=this.trueModalHeight,s=t-a,u=Math.max(e-o,0),c=n.left+r*s,d=n.top+i*u;return{left:parseInt(l(0,s,c)),top:parseInt(l(0,u,d))}},trueModalWidth:function(){var e=this.viewportWidth,t=this.modal,n=this.adaptive,r=this.minWidth,i=this.maxWidth,a="%"===t.widthType?e/100*t.width:t.width,o=Math.max(r,Math.min(e,i));return n?l(r,o,a):a},trueModalHeight:function(){var e=this.viewportHeight,t=this.modal,n=this.isAutoHeight,r=this.adaptive,i=this.minHeight,a=this.maxHeight,o="%"===t.heightType?e/100*t.height:t.height;if(n)return this.modal.renderedHeight;var s=Math.max(i,Math.min(e,a));return r?l(i,s,o):o},overlayClass:function(){return{"v--modal-overlay":!0,scrollable:this.scrollable&&this.isAutoHeight}},modalClass:function(){return["v--modal-box",this.classes]},stylesProp:function(){return"string"==typeof this.styles?this.styles.split(";").map((function(e){return e.trim()})).filter(Boolean).map((function(e){return e.split(":")})).reduce((function(e,t){var n=a(t,2);return o({},e,s({},n[0],n[1]))}),{}):this.styles},modalStyle:function(){return[this.stylesProp,{top:this.position.top+"px",left:this.position.left+"px",width:this.trueModalWidth+"px",height:this.isAutoHeight?"auto":this.trueModalHeight+"px"}]}},watch:{visible:function(e){var t=this;e?(this.visibility.overlay=!0,setTimeout((function(){t.visibility.modal=!0,t.$nextTick((function(){t.addDraggableListeners(),t.callAfterEvent(!0)}))}),this.delay)):(this.visibility.modal=!1,setTimeout((function(){t.visibility.overlay=!1,t.$nextTick((function(){t.removeDraggableListeners(),t.callAfterEvent(!1)}))}),this.delay))}},methods:{handleToggleEvent:function(e,t,n){if(this.name===e){var r=void 0===t?!this.visible:t;this.toggle(r,n)}},setInitialSize:function(){var e=this.modal,t=_(this.width),n=_(this.height);e.width=t.value,e.widthType=t.type,e.height=n.value,e.heightType=n.type},handleEscapeKeyUp:function(e){27===e.which&&this.visible&&this.$modal.hide(this.name)},handleWindowResize:function(){this.viewportWidth=u(),this.viewportHeight=window.innerHeight,this.ensureShiftInWindowBounds()},createModalEvent:function(e){var t=0<arguments.length&&void 0!==e?e:{};return function(e){var t=0<arguments.length&&void 0!==e?e:{};return o({id:c(),timestamp:Date.now(),canceled:!1},t)}(function(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{},r=Object.keys(n);"function"==typeof Object.getOwnPropertySymbols&&(r=r.concat(Object.getOwnPropertySymbols(n).filter((function(e){return Object.getOwnPropertyDescriptor(n,e).enumerable})))),r.forEach((function(t){y(e,t,n[t])}))}return e}({name:this.name,ref:this.$refs.modal},t))},handleModalResize:function(e){this.modal.widthType="px",this.modal.width=e.size.width,this.modal.heightType="px",this.modal.height=e.size.height;var t=this.modal.size;this.$emit("resize",this.createModalEvent({size:t}))},toggle:function(e,t){var n=this.reset,r=this.scrollable,i=this.visible;if(i!==e){var a=i?"before-close":"before-open";"before-open"==a?(n&&(this.setInitialSize(),this.shift.left=0,this.shift.top=0),r&&document.body.classList.add("v--modal-block-scroll")):r&&document.body.classList.remove("v--modal-block-scroll");var o=!1,s=this.createModalEvent({stop:function(){o=!0},state:e,params:t});this.$emit(a,s),o||(this.visible=e,"before-open"==a&&"undefined"!=typeof document&&document.activeElement&&"BODY"!==document.activeElement.tagName&&document.activeElement.blur&&document.activeElement.blur())}},getDraggableElement:function(){var e="string"!=typeof this.draggable?".v--modal-box":this.draggable;return e?this.$refs.overlay.querySelector(e):null},handleBackgroundClick:function(){this.clickToClose&&this.toggle(!1)},callAfterEvent:function(e){e?this.connectObserver():this.disconnectObserver();var t=e?"opened":"closed",n=this.createModalEvent({state:e});this.$emit(t,n)},addDraggableListeners:function(){var e=this;if(this.draggable){var t=this.getDraggableElement();if(t){var n=0,r=0,i=0,a=0,o=function(e){return e.touches&&0<e.touches.length?e.touches[0]:e},s=function(t){var s=t.target;if(!s||"INPUT"!==s.nodeName&&"TEXTAREA"!==s.nodeName&&"SELECT"!==s.nodeName){var c=o(t),d=c.clientX,f=c.clientY;document.addEventListener("mousemove",l),document.addEventListener("touchmove",l),document.addEventListener("mouseup",u),document.addEventListener("touchend",u),n=d,r=f,i=e.shift.left,a=e.shift.top}},l=function(t){var s=o(t),l=s.clientX,u=s.clientY;e.shift.left=i+l-n,e.shift.top=a+u-r,t.preventDefault()},u=function t(n){e.ensureShiftInWindowBounds(),document.removeEventListener("mousemove",l),document.removeEventListener("touchmove",l),document.removeEventListener("mouseup",t),document.removeEventListener("touchend",t),n.preventDefault()};t.addEventListener("mousedown",s),t.addEventListener("touchstart",s)}}},removeDraggableListeners:function(){},updateRenderedHeight:function(){this.$refs.modal&&(this.modal.renderedHeight=this.$refs.modal.getBoundingClientRect().height)},connectObserver:function(){this.mutationObserver&&this.mutationObserver.observe(this.$refs.overlay,{childList:!0,attributes:!0,subtree:!0})},disconnectObserver:function(){this.mutationObserver&&this.mutationObserver.disconnect()},beforeTransitionEnter:function(){this.connectObserver()},afterTransitionEnter:function(){},afterTransitionLeave:function(){},ensureShiftInWindowBounds:function(){var e=this.viewportHeight,t=this.viewportWidth,n=this.shift,r=this.pivotX,i=this.pivotY,a=this.trueModalWidth,o=this.trueModalHeight,s=t-a,u=Math.max(e-o,0),c=n.left+r*s,d=n.top+i*u;this.shift.left-=c-l(0,s,c),this.shift.top-=d-l(0,u,d)}}},k=(n(7),f(M,r,[],!1,null,null,null));function w(){var e=this,t=e.$createElement,n=e._self._c||t;return n("modal",{attrs:{name:"dialog",height:"auto",classes:["v--modal","vue-dialog",this.params.class],width:e.width,"pivot-y":.3,adaptive:!0,clickToClose:e.clickToClose,transition:e.transition},on:{"before-open":e.beforeOpened,"before-close":e.beforeClosed,opened:function(t){e.$emit("opened",t)},closed:function(t){e.$emit("closed",t)}}},[n("div",{staticClass:"dialog-content"},[e.params.title?n("div",{staticClass:"dialog-c-title",domProps:{innerHTML:e._s(e.params.title||"")}}):e._e(),e._v(" "),e.params.component?n(e.params.component,e._b({tag:"component"},"component",e.params.props,!1)):n("div",{staticClass:"dialog-c-text",domProps:{innerHTML:e._s(e.params.text||"")}})],1),e._v(" "),e.buttons?n("div",{staticClass:"vue-dialog-buttons"},e._l(e.buttons,(function(t,r){return n("button",{key:r,class:t.class||"vue-dialog-button",style:e.buttonStyle,attrs:{type:"button"},domProps:{innerHTML:e._s(t.title)},on:{click:function(t){t.stopPropagation(),e.click(r,t)}}},[e._v("\n      "+e._s(t.title)+"\n    ")])}))):n("div",{staticClass:"vue-dialog-buttons-none"})])}k.options.__file="src/Modal.vue";var L=k.exports;w._withStripped=!0;var x={name:"VueJsDialog",props:{width:{type:[Number,String],default:400},clickToClose:{type:Boolean,default:!0},transition:{type:String,default:"fade"}},data:function(){return{params:{},defaultButtons:[{title:"CLOSE"}]}},computed:{buttons:function(){return this.params.buttons||this.defaultButtons},buttonStyle:function(){return{flex:"1 1 ".concat(100/this.buttons.length,"%")}}},methods:{beforeOpened:function(e){window.addEventListener("keyup",this.onKeyUp),this.params=e.params||{},this.$emit("before-opened",e)},beforeClosed:function(e){window.removeEventListener("keyup",this.onKeyUp),this.params={},this.$emit("before-closed",e)},click:function(e,t,n){var r=2<arguments.length&&void 0!==n?n:"click",i=this.buttons[e];i&&"function"==typeof i.handler?i.handler(e,t,{source:r}):this.$modal.hide("dialog")},onKeyUp:function(e){if(13===e.which&&0<this.buttons.length){var t=1===this.buttons.length?0:this.buttons.findIndex((function(e){return e.default}));-1!==t&&this.click(t,e,"keypress")}}}},S=(n(9),f(x,w,[],!1,null,null,null));function T(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{attrs:{id:"modals-container"}},e._l(e.modals,(function(t){return n("modal",e._g(e._b({key:t.id,on:{closed:function(n){e.remove(t.id)}}},"modal",t.modalAttrs,!1),t.modalListeners),[n(t.component,e._g(e._b({tag:"component",on:{close:function(n){e.$modal.hide(t.modalAttrs.name)}}},"component",t.componentAttrs,!1),e.$listeners))],1)})))}S.options.__file="src/Dialog.vue";var D=S.exports;function Y(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}T._withStripped=!0;var C=f({data:function(){return{modals:[]}},created:function(){this.$root._dynamicContainer=this},methods:{add:function(e,t,n,r){var i=this,a=1<arguments.length&&void 0!==t?t:{},o=2<arguments.length&&void 0!==n?n:{},s=3<arguments.length&&void 0!==r?r:{},l=c(),u=o.name||"_dynamic_modal_"+l;this.modals.push({id:l,modalAttrs:function(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{},r=Object.keys(n);"function"==typeof Object.getOwnPropertySymbols&&(r=r.concat(Object.getOwnPropertySymbols(n).filter((function(e){return Object.getOwnPropertyDescriptor(n,e).enumerable})))),r.forEach((function(t){Y(e,t,n[t])}))}return e}({},o,{name:u}),modalListeners:s,component:e,componentAttrs:a}),this.$nextTick((function(){i.$modal.show(u)}))},remove:function(e){var t=this.modals.findIndex((function(t){return t.id===e}));-1!==t&&this.modals.splice(t,1)}}},T,[],!1,null,null,null);C.options.__file="src/ModalsContainer.vue";var E=C.exports;function A(e){return(A="function"==typeof Symbol&&"symbol"==typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"==typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e})(e)}function O(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}var P=function(e,t,n){if(!n._dynamicContainer&&t.injectModalsContainer){var r=(i=document.createElement("div"),document.body.appendChild(i),i);new e({parent:n,render:function(e){return e(E)}}).$mount(r)}var i;return n._dynamicContainer},F={install:function(e,t){var n=1<arguments.length&&void 0!==t?t:{};if(!this.installed){this.installed=!0,this.event=new e,this.rootInstance=null;var r=n.componentName||"Modal",i=n.dynamicDefaults||{},a=function(t,r,a,o){var s=a&&a.root?a.root:F.rootInstance,l=P(e,n,s);l?l.add(t,r,function(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{},r=Object.keys(n);"function"==typeof Object.getOwnPropertySymbols&&(r=r.concat(Object.getOwnPropertySymbols(n).filter((function(e){return Object.getOwnPropertyDescriptor(n,e).enumerable})))),r.forEach((function(t){O(e,t,n[t])}))}return e}({},i,a),o):console.warn("[vue-js-modal] In order to render dynamic modals, a <modals-container> component must be present on the page.")};e.prototype.$modal={show:function(e){for(var t=arguments.length,r=new Array(1<t?t-1:0),i=1;i<t;i++)r[i-1]=arguments[i];switch(A(e)){case"string":return function(e,t){F.event.$emit("toggle",e,!0,t)}.apply(void 0,[e].concat(r));case"object":case"function":return n.dynamic?a.apply(void 0,[e].concat(r)):console.warn("[vue-js-modal] $modal() received object as a first argument, but dynamic modals are switched off. https://github.com/euvl/vue-js-modal/#dynamic-modals");default:console.warn("[vue-js-modal] $modal() received an unsupported argument as a first argument.",e)}},hide:function(e,t){F.event.$emit("toggle",e,!1,t)},toggle:function(e,t){F.event.$emit("toggle",e,void 0,t)}},e.component(r,L),n.dialog&&e.component("VDialog",D),n.dynamic&&(e.component("ModalsContainer",E),e.mixin({beforeMount:function(){null===F.rootInstance&&(F.rootInstance=this.$root)}}))}}},I=t.default=F}],e.c=n,e.d=function(t,n,r){e.o(t,n)||Object.defineProperty(t,n,{enumerable:!0,get:r})},e.r=function(e){"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})},e.t=function(t,n){if(1&n&&(t=e(t)),8&n)return t;if(4&n&&"object"==typeof t&&t&&t.__esModule)return t;var r=Object.create(null);if(e.r(r),Object.defineProperty(r,"default",{enumerable:!0,value:t}),2&n&&"string"!=typeof t)for(var i in t)e.d(r,i,function(e){return t[e]}.bind(null,i));return r},e.n=function(t){var n=t&&t.__esModule?function(){return t.default}:function(){return t};return e.d(n,"a",n),n},e.o=function(e,t){return Object.prototype.hasOwnProperty.call(e,t)},e.p="/dist/",e(e.s=11);function e(r){if(n[r])return n[r].exports;var i=n[r]={i:r,l:!1,exports:{}};return t[r].call(i.exports,i,i.exports,e),i.l=!0,i.exports}var t,n}))},"19aa":function(e,t){e.exports=function(e,t,n){if(!(e instanceof t))throw TypeError("Incorrect "+(n?n+" ":"")+"invocation");return e}},"1a7b":function(e,t,n){},"1a82":function(e,t,n){"use strict";n("8ed6")},"1ab7":function(e,t,n){},"1b45":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("mt",{months:"Jannar_Frar_Marzu_April_Mejju_Ġunju_Lulju_Awwissu_Settembru_Ottubru_Novembru_Diċembru".split("_"),monthsShort:"Jan_Fra_Mar_Apr_Mej_Ġun_Lul_Aww_Set_Ott_Nov_Diċ".split("_"),weekdays:"Il-Ħadd_It-Tnejn_It-Tlieta_L-Erbgħa_Il-Ħamis_Il-Ġimgħa_Is-Sibt".split("_"),weekdaysShort:"Ħad_Tne_Tli_Erb_Ħam_Ġim_Sib".split("_"),weekdaysMin:"Ħa_Tn_Tl_Er_Ħa_Ġi_Si".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Illum fil-]LT",nextDay:"[Għada fil-]LT",nextWeek:"dddd [fil-]LT",lastDay:"[Il-bieraħ fil-]LT",lastWeek:"dddd [li għadda] [fil-]LT",sameElse:"L"},relativeTime:{future:"f’ %s",past:"%s ilu",s:"ftit sekondi",ss:"%d sekondi",m:"minuta",mm:"%d minuti",h:"siegħa",hh:"%d siegħat",d:"ġurnata",dd:"%d ġranet",M:"xahar",MM:"%d xhur",y:"sena",yy:"%d sni"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,doy:4}});return t}))},"1be4":function(e,t,n){var r=n("d066");e.exports=r("document","documentElement")},"1c0b":function(e,t){e.exports=function(e){if("function"!=typeof e)throw TypeError(String(e)+" is not a function");return e}},"1c7e":function(e,t,n){var r=n("b622"),i=r("iterator"),a=!1;try{var o=0,s={next:function(){return{done:!!o++}},return:function(){a=!0}};s[i]=function(){return this},Array.from(s,(function(){throw 2}))}catch(l){}e.exports=function(e,t){if(!t&&!a)return!1;var n=!1;try{var r={};r[i]=function(){return{next:function(){return{done:n=!0}}}},e(r)}catch(l){}return n}},"1cdc":function(e,t,n){var r=n("342f");e.exports=/(iphone|ipod|ipad).*applewebkit/i.test(r)},"1cfd":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"1",2:"2",3:"3",4:"4",5:"5",6:"6",7:"7",8:"8",9:"9",0:"0"},n=function(e){return 0===e?0:1===e?1:2===e?2:e%100>=3&&e%100<=10?3:e%100>=11?4:5},r={s:["أقل من ثانية","ثانية واحدة",["ثانيتان","ثانيتين"],"%d ثوان","%d ثانية","%d ثانية"],m:["أقل من دقيقة","دقيقة واحدة",["دقيقتان","دقيقتين"],"%d دقائق","%d دقيقة","%d دقيقة"],h:["أقل من ساعة","ساعة واحدة",["ساعتان","ساعتين"],"%d ساعات","%d ساعة","%d ساعة"],d:["أقل من يوم","يوم واحد",["يومان","يومين"],"%d أيام","%d يومًا","%d يوم"],M:["أقل من شهر","شهر واحد",["شهران","شهرين"],"%d أشهر","%d شهرا","%d شهر"],y:["أقل من عام","عام واحد",["عامان","عامين"],"%d أعوام","%d عامًا","%d عام"]},i=function(e){return function(t,i,a,o){var s=n(t),l=r[e][n(t)];return 2===s&&(l=l[i?0:1]),l.replace(/%d/i,t)}},a=["يناير","فبراير","مارس","أبريل","مايو","يونيو","يوليو","أغسطس","سبتمبر","أكتوبر","نوفمبر","ديسمبر"],o=e.defineLocale("ar-ly",{months:a,monthsShort:a,weekdays:"الأحد_الإثنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت".split("_"),weekdaysShort:"أحد_إثنين_ثلاثاء_أربعاء_خميس_جمعة_سبت".split("_"),weekdaysMin:"ح_ن_ث_ر_خ_ج_س".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"D/‏M/‏YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},meridiemParse:/ص|م/,isPM:function(e){return"م"===e},meridiem:function(e,t,n){return e<12?"ص":"م"},calendar:{sameDay:"[اليوم عند الساعة] LT",nextDay:"[غدًا عند الساعة] LT",nextWeek:"dddd [عند الساعة] LT",lastDay:"[أمس عند الساعة] LT",lastWeek:"dddd [عند الساعة] LT",sameElse:"L"},relativeTime:{future:"بعد %s",past:"منذ %s",s:i("s"),ss:i("s"),m:i("m"),mm:i("m"),h:i("h"),hh:i("h"),d:i("d"),dd:i("d"),M:i("M"),MM:i("M"),y:i("y"),yy:i("y")},preparse:function(e){return e.replace(/،/g,",")},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]})).replace(/,/g,"،")},week:{dow:6,doy:12}});return o}))},"1d80":function(e,t){e.exports=function(e){if(void 0==e)throw TypeError("Can't call method on "+e);return e}},"1dde":function(e,t,n){var r=n("d039"),i=n("b622"),a=n("2d00"),o=i("species");e.exports=function(e){return a>=51||!r((function(){var t=[],n=t.constructor={};return n[o]=function(){return{foo:1}},1!==t[e](Boolean).foo}))}},"1f10":function(e,t,n){"use strict";n("88ee")},"1f38":function(e,t,n){"use strict";(function(e){n.d(t,"a",(function(){return i}));var r=function(){return"undefined"!==typeof window?window:e},i=function(){var e=r();return e&&e.tinymce?e.tinymce:null}}).call(this,n("c8ba"))},"1f5b":function(e,t,n){"use strict";n("8670")},"1fb5":function(e,t,n){"use strict";t.byteLength=c,t.toByteArray=f,t.fromByteArray=m;for(var r=[],i=[],a="undefined"!==typeof Uint8Array?Uint8Array:Array,o="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/",s=0,l=o.length;s<l;++s)r[s]=o[s],i[o.charCodeAt(s)]=s;function u(e){var t=e.length;if(t%4>0)throw new Error("Invalid string. Length must be a multiple of 4");var n=e.indexOf("=");-1===n&&(n=t);var r=n===t?0:4-n%4;return[n,r]}function c(e){var t=u(e),n=t[0],r=t[1];return 3*(n+r)/4-r}function d(e,t,n){return 3*(t+n)/4-n}function f(e){var t,n,r=u(e),o=r[0],s=r[1],l=new a(d(e,o,s)),c=0,f=s>0?o-4:o;for(n=0;n<f;n+=4)t=i[e.charCodeAt(n)]<<18|i[e.charCodeAt(n+1)]<<12|i[e.charCodeAt(n+2)]<<6|i[e.charCodeAt(n+3)],l[c++]=t>>16&255,l[c++]=t>>8&255,l[c++]=255&t;return 2===s&&(t=i[e.charCodeAt(n)]<<2|i[e.charCodeAt(n+1)]>>4,l[c++]=255&t),1===s&&(t=i[e.charCodeAt(n)]<<10|i[e.charCodeAt(n+1)]<<4|i[e.charCodeAt(n+2)]>>2,l[c++]=t>>8&255,l[c++]=255&t),l}function h(e){return r[e>>18&63]+r[e>>12&63]+r[e>>6&63]+r[63&e]}function p(e,t,n){for(var r,i=[],a=t;a<n;a+=3)r=(e[a]<<16&16711680)+(e[a+1]<<8&65280)+(255&e[a+2]),i.push(h(r));return i.join("")}function m(e){for(var t,n=e.length,i=n%3,a=[],o=16383,s=0,l=n-i;s<l;s+=o)a.push(p(e,s,s+o>l?l:s+o));return 1===i?(t=e[n-1],a.push(r[t>>2]+r[t<<4&63]+"==")):2===i&&(t=(e[n-2]<<8)+e[n-1],a.push(r[t>>10]+r[t>>4&63]+r[t<<2&63]+"=")),a.join("")}i["-".charCodeAt(0)]=62,i["_".charCodeAt(0)]=63},"1fc1":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e,t){var n=e.split("_");return t%10===1&&t%100!==11?n[0]:t%10>=2&&t%10<=4&&(t%100<10||t%100>=20)?n[1]:n[2]}function n(e,n,r){var i={ss:n?"секунда_секунды_секунд":"секунду_секунды_секунд",mm:n?"хвіліна_хвіліны_хвілін":"хвіліну_хвіліны_хвілін",hh:n?"гадзіна_гадзіны_гадзін":"гадзіну_гадзіны_гадзін",dd:"дзень_дні_дзён",MM:"месяц_месяцы_месяцаў",yy:"год_гады_гадоў"};return"m"===r?n?"хвіліна":"хвіліну":"h"===r?n?"гадзіна":"гадзіну":e+" "+t(i[r],+e)}var r=e.defineLocale("be",{months:{format:"студзеня_лютага_сакавіка_красавіка_траўня_чэрвеня_ліпеня_жніўня_верасня_кастрычніка_лістапада_снежня".split("_"),standalone:"студзень_люты_сакавік_красавік_травень_чэрвень_ліпень_жнівень_верасень_кастрычнік_лістапад_снежань".split("_")},monthsShort:"студ_лют_сак_крас_трав_чэрв_ліп_жнів_вер_каст_ліст_снеж".split("_"),weekdays:{format:"нядзелю_панядзелак_аўторак_сераду_чацвер_пятніцу_суботу".split("_"),standalone:"нядзеля_панядзелак_аўторак_серада_чацвер_пятніца_субота".split("_"),isFormat:/\[ ?[Ууў] ?(?:мінулую|наступную)? ?\] ?dddd/},weekdaysShort:"нд_пн_ат_ср_чц_пт_сб".split("_"),weekdaysMin:"нд_пн_ат_ср_чц_пт_сб".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY г.",LLL:"D MMMM YYYY г., HH:mm",LLLL:"dddd, D MMMM YYYY г., HH:mm"},calendar:{sameDay:"[Сёння ў] LT",nextDay:"[Заўтра ў] LT",lastDay:"[Учора ў] LT",nextWeek:function(){return"[У] dddd [ў] LT"},lastWeek:function(){switch(this.day()){case 0:case 3:case 5:case 6:return"[У мінулую] dddd [ў] LT";case 1:case 2:case 4:return"[У мінулы] dddd [ў] LT"}},sameElse:"L"},relativeTime:{future:"праз %s",past:"%s таму",s:"некалькі секунд",m:n,mm:n,h:n,hh:n,d:"дзень",dd:n,M:"месяц",MM:n,y:"год",yy:n},meridiemParse:/ночы|раніцы|дня|вечара/,isPM:function(e){return/^(дня|вечара)$/.test(e)},meridiem:function(e,t,n){return e<4?"ночы":e<12?"раніцы":e<17?"дня":"вечара"},dayOfMonthOrdinalParse:/\d{1,2}-(і|ы|га)/,ordinal:function(e,t){switch(t){case"M":case"d":case"DDD":case"w":case"W":return e%10!==2&&e%10!==3||e%100===12||e%100===13?e+"-ы":e+"-і";case"D":return e+"-га";default:return e}},week:{dow:1,doy:7}});return r}))},"201b":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("ka",{months:"იანვარი_თებერვალი_მარტი_აპრილი_მაისი_ივნისი_ივლისი_აგვისტო_სექტემბერი_ოქტომბერი_ნოემბერი_დეკემბერი".split("_"),monthsShort:"იან_თებ_მარ_აპრ_მაი_ივნ_ივლ_აგვ_სექ_ოქტ_ნოე_დეკ".split("_"),weekdays:{standalone:"კვირა_ორშაბათი_სამშაბათი_ოთხშაბათი_ხუთშაბათი_პარასკევი_შაბათი".split("_"),format:"კვირას_ორშაბათს_სამშაბათს_ოთხშაბათს_ხუთშაბათს_პარასკევს_შაბათს".split("_"),isFormat:/(წინა|შემდეგ)/},weekdaysShort:"კვი_ორშ_სამ_ოთხ_ხუთ_პარ_შაბ".split("_"),weekdaysMin:"კვ_ორ_სა_ოთ_ხუ_პა_შა".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[დღეს] LT[-ზე]",nextDay:"[ხვალ] LT[-ზე]",lastDay:"[გუშინ] LT[-ზე]",nextWeek:"[შემდეგ] dddd LT[-ზე]",lastWeek:"[წინა] dddd LT-ზე",sameElse:"L"},relativeTime:{future:function(e){return e.replace(/(წამ|წუთ|საათ|წელ|დღ|თვ)(ი|ე)/,(function(e,t,n){return"ი"===n?t+"ში":t+n+"ში"}))},past:function(e){return/(წამი|წუთი|საათი|დღე|თვე)/.test(e)?e.replace(/(ი|ე)$/,"ის წინ"):/წელი/.test(e)?e.replace(/წელი$/,"წლის წინ"):e},s:"რამდენიმე წამი",ss:"%d წამი",m:"წუთი",mm:"%d წუთი",h:"საათი",hh:"%d საათი",d:"დღე",dd:"%d დღე",M:"თვე",MM:"%d თვე",y:"წელი",yy:"%d წელი"},dayOfMonthOrdinalParse:/0|1-ლი|მე-\d{1,2}|\d{1,2}-ე/,ordinal:function(e){return 0===e?e:1===e?e+"-ლი":e<20||e<=100&&e%20===0||e%100===0?"მე-"+e:e+"-ე"},week:{dow:1,doy:7}});return t}))},"219c":function(e,t,n){"use strict";var r=n("ebb5"),i=r.aTypedArray,a=r.exportTypedArrayMethod,o=[].sort;a("sort",(function(e){return o.call(i(this),e)}))},"21a0":function(e,t,n){},"21fd":function(e,t,n){},2266:function(e,t,n){var r=n("825a"),i=n("e95a"),a=n("50c4"),o=n("0366"),s=n("35a1"),l=n("2a62"),u=function(e,t){this.stopped=e,this.result=t};e.exports=function(e,t,n){var c,d,f,h,p,m,_,v=n&&n.that,g=!(!n||!n.AS_ENTRIES),b=!(!n||!n.IS_ITERATOR),y=!(!n||!n.INTERRUPTED),M=o(t,v,1+g+y),k=function(e){return c&&l(c),new u(!0,e)},w=function(e){return g?(r(e),y?M(e[0],e[1],k):M(e[0],e[1])):y?M(e,k):M(e)};if(b)c=e;else{if(d=s(e),"function"!=typeof d)throw TypeError("Target is not iterable");if(i(d)){for(f=0,h=a(e.length);h>f;f++)if(p=w(e[f]),p&&p instanceof u)return p;return new u(!1)}c=d.call(e)}m=c.next;while(!(_=m.call(c)).done){try{p=w(_.value)}catch(L){throw l(c),L}if("object"==typeof p&&p&&p instanceof u)return p}return new u(!1)}},"22e3":function(e,t){function n(e){if(Array.isArray(e))return e}e.exports=n},"22f8":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("ko",{months:"1월_2월_3월_4월_5월_6월_7월_8월_9월_10월_11월_12월".split("_"),monthsShort:"1월_2월_3월_4월_5월_6월_7월_8월_9월_10월_11월_12월".split("_"),weekdays:"일요일_월요일_화요일_수요일_목요일_금요일_토요일".split("_"),weekdaysShort:"일_월_화_수_목_금_토".split("_"),weekdaysMin:"일_월_화_수_목_금_토".split("_"),longDateFormat:{LT:"A h:mm",LTS:"A h:mm:ss",L:"YYYY.MM.DD.",LL:"YYYY년 MMMM D일",LLL:"YYYY년 MMMM D일 A h:mm",LLLL:"YYYY년 MMMM D일 dddd A h:mm",l:"YYYY.MM.DD.",ll:"YYYY년 MMMM D일",lll:"YYYY년 MMMM D일 A h:mm",llll:"YYYY년 MMMM D일 dddd A h:mm"},calendar:{sameDay:"오늘 LT",nextDay:"내일 LT",nextWeek:"dddd LT",lastDay:"어제 LT",lastWeek:"지난주 dddd LT",sameElse:"L"},relativeTime:{future:"%s 후",past:"%s 전",s:"몇 초",ss:"%d초",m:"1분",mm:"%d분",h:"한 시간",hh:"%d시간",d:"하루",dd:"%d일",M:"한 달",MM:"%d달",y:"일 년",yy:"%d년"},dayOfMonthOrdinalParse:/\d{1,2}(일|월|주)/,ordinal:function(e,t){switch(t){case"d":case"D":case"DDD":return e+"일";case"M":return e+"월";case"w":case"W":return e+"주";default:return e}},meridiemParse:/오전|오후/,isPM:function(e){return"오후"===e},meridiem:function(e,t,n){return e<12?"오전":"오후"}});return t}))},"23bb":function(e,t,n){"use strict";n("9d48")},"23cb":function(e,t,n){var r=n("a691"),i=Math.max,a=Math.min;e.exports=function(e,t){var n=r(e);return n<0?i(n+t,0):a(n,t)}},"23e7":function(e,t,n){var r=n("da84"),i=n("06cf").f,a=n("9112"),o=n("6eeb"),s=n("ce4e"),l=n("e893"),u=n("94ca");e.exports=function(e,t){var n,c,d,f,h,p,m=e.target,_=e.global,v=e.stat;if(c=_?r:v?r[m]||s(m,{}):(r[m]||{}).prototype,c)for(d in t){if(h=t[d],e.noTargetGet?(p=i(c,d),f=p&&p.value):f=c[d],n=u(_?d:m+(v?".":"#")+d,e.forced),!n&&void 0!==f){if(typeof h===typeof f)continue;l(h,f)}(e.sham||f&&f.sham)&&a(h,"sham",!0),o(c,d,h,e)}}},"241c":function(e,t,n){var r=n("ca84"),i=n("7839"),a=i.concat("length","prototype");t.f=Object.getOwnPropertyNames||function(e){return r(e,a)}},2421:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"١",2:"٢",3:"٣",4:"٤",5:"٥",6:"٦",7:"٧",8:"٨",9:"٩",0:"٠"},n={"١":"1","٢":"2","٣":"3","٤":"4","٥":"5","٦":"6","٧":"7","٨":"8","٩":"9","٠":"0"},r=["کانونی دووەم","شوبات","ئازار","نیسان","ئایار","حوزەیران","تەمموز","ئاب","ئەیلوول","تشرینی یەكەم","تشرینی دووەم","كانونی یەکەم"],i=e.defineLocale("ku",{months:r,monthsShort:r,weekdays:"یه‌كشه‌ممه‌_دووشه‌ممه‌_سێشه‌ممه‌_چوارشه‌ممه‌_پێنجشه‌ممه‌_هه‌ینی_شه‌ممه‌".split("_"),weekdaysShort:"یه‌كشه‌م_دووشه‌م_سێشه‌م_چوارشه‌م_پێنجشه‌م_هه‌ینی_شه‌ممه‌".split("_"),weekdaysMin:"ی_د_س_چ_پ_ه_ش".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},meridiemParse:/ئێواره‌|به‌یانی/,isPM:function(e){return/ئێواره‌/.test(e)},meridiem:function(e,t,n){return e<12?"به‌یانی":"ئێواره‌"},calendar:{sameDay:"[ئه‌مرۆ كاتژمێر] LT",nextDay:"[به‌یانی كاتژمێر] LT",nextWeek:"dddd [كاتژمێر] LT",lastDay:"[دوێنێ كاتژمێر] LT",lastWeek:"dddd [كاتژمێر] LT",sameElse:"L"},relativeTime:{future:"له‌ %s",past:"%s",s:"چه‌ند چركه‌یه‌ك",ss:"چركه‌ %d",m:"یه‌ك خوله‌ك",mm:"%d خوله‌ك",h:"یه‌ك كاتژمێر",hh:"%d كاتژمێر",d:"یه‌ك ڕۆژ",dd:"%d ڕۆژ",M:"یه‌ك مانگ",MM:"%d مانگ",y:"یه‌ك ساڵ",yy:"%d ساڵ"},preparse:function(e){return e.replace(/[١٢٣٤٥٦٧٨٩٠]/g,(function(e){return n[e]})).replace(/،/g,",")},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]})).replace(/,/g,"،")},week:{dow:6,doy:12}});return i}))},"252a":function(e,t,n){"use strict";n("dd32")},"252c":function(e,t,n){"use strict";(function(e){function r(){var e=window.navigator.userAgent,t=e.indexOf("MSIE ");if(t>0)return parseInt(e.substring(t+5,e.indexOf(".",t)),10);var n=e.indexOf("Trident/");if(n>0){var r=e.indexOf("rv:");return parseInt(e.substring(r+3,e.indexOf(".",r)),10)}var i=e.indexOf("Edge/");return i>0?parseInt(e.substring(i+5,e.indexOf(".",i)),10):-1}n.d(t,"a",(function(){return o}));var i=void 0;function a(){a.init||(a.init=!0,i=-1!==r())}var o={render:function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"resize-observer",attrs:{tabindex:"-1"}})},staticRenderFns:[],_scopeId:"data-v-b329ee4c",name:"resize-observer",methods:{compareAndNotify:function(){this._w===this.$el.offsetWidth&&this._h===this.$el.offsetHeight||(this._w=this.$el.offsetWidth,this._h=this.$el.offsetHeight,this.$emit("notify"))},addResizeHandlers:function(){this._resizeObject.contentDocument.defaultView.addEventListener("resize",this.compareAndNotify),this.compareAndNotify()},removeResizeHandlers:function(){this._resizeObject&&this._resizeObject.onload&&(!i&&this._resizeObject.contentDocument&&this._resizeObject.contentDocument.defaultView.removeEventListener("resize",this.compareAndNotify),delete this._resizeObject.onload)}},mounted:function(){var e=this;a(),this.$nextTick((function(){e._w=e.$el.offsetWidth,e._h=e.$el.offsetHeight}));var t=document.createElement("object");this._resizeObject=t,t.setAttribute("aria-hidden","true"),t.setAttribute("tabindex",-1),t.onload=this.addResizeHandlers,t.type="text/html",i&&this.$el.appendChild(t),t.data="about:blank",i||this.$el.appendChild(t)},beforeDestroy:function(){this.removeResizeHandlers()}};function s(e){e.component("resize-observer",o),e.component("ResizeObserver",o)}var l={version:"0.4.5",install:s},u=null;"undefined"!==typeof window?u=window.Vue:"undefined"!==typeof e&&(u=e.Vue),u&&u.use(l)}).call(this,n("c8ba"))},2532:function(e,t,n){"use strict";var r=n("23e7"),i=n("5a34"),a=n("1d80"),o=n("ab13");r({target:"String",proto:!0,forced:!o("includes")},{includes:function(e){return!!~String(a(this)).indexOf(i(e),arguments.length>1?arguments[1]:void 0)}})},2554:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e,t,n){var r=e+" ";switch(n){case"ss":return r+=1===e?"sekunda":2===e||3===e||4===e?"sekunde":"sekundi",r;case"m":return t?"jedna minuta":"jedne minute";case"mm":return r+=1===e?"minuta":2===e||3===e||4===e?"minute":"minuta",r;case"h":return t?"jedan sat":"jednog sata";case"hh":return r+=1===e?"sat":2===e||3===e||4===e?"sata":"sati",r;case"dd":return r+=1===e?"dan":"dana",r;case"MM":return r+=1===e?"mjesec":2===e||3===e||4===e?"mjeseca":"mjeseci",r;case"yy":return r+=1===e?"godina":2===e||3===e||4===e?"godine":"godina",r}}var n=e.defineLocale("bs",{months:"januar_februar_mart_april_maj_juni_juli_august_septembar_oktobar_novembar_decembar".split("_"),monthsShort:"jan._feb._mar._apr._maj._jun._jul._aug._sep._okt._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"nedjelja_ponedjeljak_utorak_srijeda_četvrtak_petak_subota".split("_"),weekdaysShort:"ned._pon._uto._sri._čet._pet._sub.".split("_"),weekdaysMin:"ne_po_ut_sr_če_pe_su".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd, D. MMMM YYYY H:mm"},calendar:{sameDay:"[danas u] LT",nextDay:"[sutra u] LT",nextWeek:function(){switch(this.day()){case 0:return"[u] [nedjelju] [u] LT";case 3:return"[u] [srijedu] [u] LT";case 6:return"[u] [subotu] [u] LT";case 1:case 2:case 4:case 5:return"[u] dddd [u] LT"}},lastDay:"[jučer u] LT",lastWeek:function(){switch(this.day()){case 0:case 3:return"[prošlu] dddd [u] LT";case 6:return"[prošle] [subote] [u] LT";case 1:case 2:case 4:case 5:return"[prošli] dddd [u] LT"}},sameElse:"L"},relativeTime:{future:"za %s",past:"prije %s",s:"par sekundi",ss:t,m:t,mm:t,h:t,hh:t,d:"dan",dd:t,M:"mjesec",MM:t,y:"godinu",yy:t},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}});return n}))},"25a1":function(e,t,n){"use strict";var r=n("ebb5"),i=n("d58f").right,a=r.aTypedArray,o=r.exportTypedArrayMethod;o("reduceRight",(function(e){return i(a(this),e,arguments.length,arguments.length>1?arguments[1]:void 0)}))},"25f0":function(e,t,n){"use strict";var r=n("6eeb"),i=n("825a"),a=n("d039"),o=n("ad6d"),s="toString",l=RegExp.prototype,u=l[s],c=a((function(){return"/a/b"!=u.call({source:"a",flags:"b"})})),d=u.name!=s;(c||d)&&r(RegExp.prototype,s,(function(){var e=i(this),t=String(e.source),n=e.flags,r=String(void 0===n&&e instanceof RegExp&&!("flags"in l)?o.call(e):n);return"/"+t+"/"+r}),{unsafe:!0})},2626:function(e,t,n){"use strict";var r=n("d066"),i=n("9bf2"),a=n("b622"),o=n("83ab"),s=a("species");e.exports=function(e){var t=r(e),n=i.f;o&&t&&!t[s]&&n(t,s,{configurable:!0,get:function(){return this}})}},"26f9":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={ss:"sekundė_sekundžių_sekundes",m:"minutė_minutės_minutę",mm:"minutės_minučių_minutes",h:"valanda_valandos_valandą",hh:"valandos_valandų_valandas",d:"diena_dienos_dieną",dd:"dienos_dienų_dienas",M:"mėnuo_mėnesio_mėnesį",MM:"mėnesiai_mėnesių_mėnesius",y:"metai_metų_metus",yy:"metai_metų_metus"};function n(e,t,n,r){return t?"kelios sekundės":r?"kelių sekundžių":"kelias sekundes"}function r(e,t,n,r){return t?a(n)[0]:r?a(n)[1]:a(n)[2]}function i(e){return e%10===0||e>10&&e<20}function a(e){return t[e].split("_")}function o(e,t,n,o){var s=e+" ";return 1===e?s+r(e,t,n[0],o):t?s+(i(e)?a(n)[1]:a(n)[0]):o?s+a(n)[1]:s+(i(e)?a(n)[1]:a(n)[2])}var s=e.defineLocale("lt",{months:{format:"sausio_vasario_kovo_balandžio_gegužės_birželio_liepos_rugpjūčio_rugsėjo_spalio_lapkričio_gruodžio".split("_"),standalone:"sausis_vasaris_kovas_balandis_gegužė_birželis_liepa_rugpjūtis_rugsėjis_spalis_lapkritis_gruodis".split("_"),isFormat:/D[oD]?(\[[^\[\]]*\]|\s)+MMMM?|MMMM?(\[[^\[\]]*\]|\s)+D[oD]?/},monthsShort:"sau_vas_kov_bal_geg_bir_lie_rgp_rgs_spa_lap_grd".split("_"),weekdays:{format:"sekmadienį_pirmadienį_antradienį_trečiadienį_ketvirtadienį_penktadienį_šeštadienį".split("_"),standalone:"sekmadienis_pirmadienis_antradienis_trečiadienis_ketvirtadienis_penktadienis_šeštadienis".split("_"),isFormat:/dddd HH:mm/},weekdaysShort:"Sek_Pir_Ant_Tre_Ket_Pen_Šeš".split("_"),weekdaysMin:"S_P_A_T_K_Pn_Š".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"YYYY [m.] MMMM D [d.]",LLL:"YYYY [m.] MMMM D [d.], HH:mm [val.]",LLLL:"YYYY [m.] MMMM D [d.], dddd, HH:mm [val.]",l:"YYYY-MM-DD",ll:"YYYY [m.] MMMM D [d.]",lll:"YYYY [m.] MMMM D [d.], HH:mm [val.]",llll:"YYYY [m.] MMMM D [d.], ddd, HH:mm [val.]"},calendar:{sameDay:"[Šiandien] LT",nextDay:"[Rytoj] LT",nextWeek:"dddd LT",lastDay:"[Vakar] LT",lastWeek:"[Praėjusį] dddd LT",sameElse:"L"},relativeTime:{future:"po %s",past:"prieš %s",s:n,ss:o,m:r,mm:o,h:r,hh:o,d:r,dd:o,M:r,MM:o,y:r,yy:o},dayOfMonthOrdinalParse:/\d{1,2}-oji/,ordinal:function(e){return e+"-oji"},week:{dow:1,doy:4}});return s}))},"281c":function(e,t,n){},2921:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("vi",{months:"tháng 1_tháng 2_tháng 3_tháng 4_tháng 5_tháng 6_tháng 7_tháng 8_tháng 9_tháng 10_tháng 11_tháng 12".split("_"),monthsShort:"Thg 01_Thg 02_Thg 03_Thg 04_Thg 05_Thg 06_Thg 07_Thg 08_Thg 09_Thg 10_Thg 11_Thg 12".split("_"),monthsParseExact:!0,weekdays:"chủ nhật_thứ hai_thứ ba_thứ tư_thứ năm_thứ sáu_thứ bảy".split("_"),weekdaysShort:"CN_T2_T3_T4_T5_T6_T7".split("_"),weekdaysMin:"CN_T2_T3_T4_T5_T6_T7".split("_"),weekdaysParseExact:!0,meridiemParse:/sa|ch/i,isPM:function(e){return/^ch$/i.test(e)},meridiem:function(e,t,n){return e<12?n?"sa":"SA":n?"ch":"CH"},longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM [năm] YYYY",LLL:"D MMMM [năm] YYYY HH:mm",LLLL:"dddd, D MMMM [năm] YYYY HH:mm",l:"DD/M/YYYY",ll:"D MMM YYYY",lll:"D MMM YYYY HH:mm",llll:"ddd, D MMM YYYY HH:mm"},calendar:{sameDay:"[Hôm nay lúc] LT",nextDay:"[Ngày mai lúc] LT",nextWeek:"dddd [tuần tới lúc] LT",lastDay:"[Hôm qua lúc] LT",lastWeek:"dddd [tuần trước lúc] LT",sameElse:"L"},relativeTime:{future:"%s tới",past:"%s trước",s:"vài giây",ss:"%d giây",m:"một phút",mm:"%d phút",h:"một giờ",hh:"%d giờ",d:"một ngày",dd:"%d ngày",w:"một tuần",ww:"%d tuần",M:"một tháng",MM:"%d tháng",y:"một năm",yy:"%d năm"},dayOfMonthOrdinalParse:/\d{1,2}/,ordinal:function(e){return e},week:{dow:1,doy:4}});return t}))},"293c":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={words:{ss:["sekund","sekunda","sekundi"],m:["jedan minut","jednog minuta"],mm:["minut","minuta","minuta"],h:["jedan sat","jednog sata"],hh:["sat","sata","sati"],dd:["dan","dana","dana"],MM:["mjesec","mjeseca","mjeseci"],yy:["godina","godine","godina"]},correctGrammaticalCase:function(e,t){return 1===e?t[0]:e>=2&&e<=4?t[1]:t[2]},translate:function(e,n,r){var i=t.words[r];return 1===r.length?n?i[0]:i[1]:e+" "+t.correctGrammaticalCase(e,i)}},n=e.defineLocale("me",{months:"januar_februar_mart_april_maj_jun_jul_avgust_septembar_oktobar_novembar_decembar".split("_"),monthsShort:"jan._feb._mar._apr._maj_jun_jul_avg._sep._okt._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"nedjelja_ponedjeljak_utorak_srijeda_četvrtak_petak_subota".split("_"),weekdaysShort:"ned._pon._uto._sri._čet._pet._sub.".split("_"),weekdaysMin:"ne_po_ut_sr_če_pe_su".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd, D. MMMM YYYY H:mm"},calendar:{sameDay:"[danas u] LT",nextDay:"[sjutra u] LT",nextWeek:function(){switch(this.day()){case 0:return"[u] [nedjelju] [u] LT";case 3:return"[u] [srijedu] [u] LT";case 6:return"[u] [subotu] [u] LT";case 1:case 2:case 4:case 5:return"[u] dddd [u] LT"}},lastDay:"[juče u] LT",lastWeek:function(){var e=["[prošle] [nedjelje] [u] LT","[prošlog] [ponedjeljka] [u] LT","[prošlog] [utorka] [u] LT","[prošle] [srijede] [u] LT","[prošlog] [četvrtka] [u] LT","[prošlog] [petka] [u] LT","[prošle] [subote] [u] LT"];return e[this.day()]},sameElse:"L"},relativeTime:{future:"za %s",past:"prije %s",s:"nekoliko sekundi",ss:t.translate,m:t.translate,mm:t.translate,h:t.translate,hh:t.translate,d:"dan",dd:t.translate,M:"mjesec",MM:t.translate,y:"godinu",yy:t.translate},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}});return n}))},2954:function(e,t,n){"use strict";var r=n("ebb5"),i=n("4840"),a=n("d039"),o=r.aTypedArray,s=r.aTypedArrayConstructor,l=r.exportTypedArrayMethod,u=[].slice,c=a((function(){new Int8Array(1).slice()}));l("slice",(function(e,t){var n=u.call(o(this),e,t),r=i(this,this.constructor),a=0,l=n.length,c=new(s(r))(l);while(l>a)c[a]=n[a++];return c}),c)},"2a62":function(e,t,n){var r=n("825a");e.exports=function(e){var t=e["return"];if(void 0!==t)return r(t.call(e)).value}},"2b3d":function(e,t,n){"use strict";n("3ca3");var r,i=n("23e7"),a=n("83ab"),o=n("0d3b"),s=n("da84"),l=n("37e8"),u=n("6eeb"),c=n("19aa"),d=n("5135"),f=n("60da"),h=n("4df4"),p=n("6547").codeAt,m=n("5fb2"),_=n("d44e"),v=n("9861"),g=n("69f3"),b=s.URL,y=v.URLSearchParams,M=v.getState,k=g.set,w=g.getterFor("URL"),L=Math.floor,x=Math.pow,S="Invalid authority",T="Invalid scheme",D="Invalid host",Y="Invalid port",C=/[A-Za-z]/,E=/[\d+-.A-Za-z]/,A=/\d/,O=/^(0x|0X)/,P=/^[0-7]+$/,F=/^\d+$/,I=/^[\dA-Fa-f]+$/,j=/[\u0000\u0009\u000A\u000D #%/:?@[\\]]/,H=/[\u0000\u0009\u000A\u000D #/:?@[\\]]/,R=/^[\u0000-\u001F ]+|[\u0000-\u001F ]+$/g,N=/[\u0009\u000A\u000D]/g,$=function(e,t){var n,r,i;if("["==t.charAt(0)){if("]"!=t.charAt(t.length-1))return D;if(n=B(t.slice(1,-1)),!n)return D;e.host=n}else if(Q(e)){if(t=m(t),j.test(t))return D;if(n=z(t),null===n)return D;e.host=n}else{if(H.test(t))return D;for(n="",r=h(t),i=0;i<r.length;i++)n+=K(r[i],V);e.host=n}},z=function(e){var t,n,r,i,a,o,s,l=e.split(".");if(l.length&&""==l[l.length-1]&&l.pop(),t=l.length,t>4)return e;for(n=[],r=0;r<t;r++){if(i=l[r],""==i)return e;if(a=10,i.length>1&&"0"==i.charAt(0)&&(a=O.test(i)?16:8,i=i.slice(8==a?1:2)),""===i)o=0;else{if(!(10==a?F:8==a?P:I).test(i))return e;o=parseInt(i,a)}n.push(o)}for(r=0;r<t;r++)if(o=n[r],r==t-1){if(o>=x(256,5-t))return null}else if(o>255)return null;for(s=n.pop(),r=0;r<n.length;r++)s+=n[r]*x(256,3-r);return s},B=function(e){var t,n,r,i,a,o,s,l=[0,0,0,0,0,0,0,0],u=0,c=null,d=0,f=function(){return e.charAt(d)};if(":"==f()){if(":"!=e.charAt(1))return;d+=2,u++,c=u}while(f()){if(8==u)return;if(":"!=f()){t=n=0;while(n<4&&I.test(f()))t=16*t+parseInt(f(),16),d++,n++;if("."==f()){if(0==n)return;if(d-=n,u>6)return;r=0;while(f()){if(i=null,r>0){if(!("."==f()&&r<4))return;d++}if(!A.test(f()))return;while(A.test(f())){if(a=parseInt(f(),10),null===i)i=a;else{if(0==i)return;i=10*i+a}if(i>255)return;d++}l[u]=256*l[u]+i,r++,2!=r&&4!=r||u++}if(4!=r)return;break}if(":"==f()){if(d++,!f())return}else if(f())return;l[u++]=t}else{if(null!==c)return;d++,u++,c=u}}if(null!==c){o=u-c,u=7;while(0!=u&&o>0)s=l[u],l[u--]=l[c+o-1],l[c+--o]=s}else if(8!=u)return;return l},W=function(e){for(var t=null,n=1,r=null,i=0,a=0;a<8;a++)0!==e[a]?(i>n&&(t=r,n=i),r=null,i=0):(null===r&&(r=a),++i);return i>n&&(t=r,n=i),t},U=function(e){var t,n,r,i;if("number"==typeof e){for(t=[],n=0;n<4;n++)t.unshift(e%256),e=L(e/256);return t.join(".")}if("object"==typeof e){for(t="",r=W(e),n=0;n<8;n++)i&&0===e[n]||(i&&(i=!1),r===n?(t+=n?":":"::",i=!0):(t+=e[n].toString(16),n<7&&(t+=":")));return"["+t+"]"}return e},V={},q=f({},V,{" ":1,'"':1,"<":1,">":1,"`":1}),G=f({},q,{"#":1,"?":1,"{":1,"}":1}),J=f({},G,{"/":1,":":1,";":1,"=":1,"@":1,"[":1,"\\":1,"]":1,"^":1,"|":1}),K=function(e,t){var n=p(e,0);return n>32&&n<127&&!d(t,e)?e:encodeURIComponent(e)},X={ftp:21,file:null,http:80,https:443,ws:80,wss:443},Q=function(e){return d(X,e.scheme)},Z=function(e){return""!=e.username||""!=e.password},ee=function(e){return!e.host||e.cannotBeABaseURL||"file"==e.scheme},te=function(e,t){var n;return 2==e.length&&C.test(e.charAt(0))&&(":"==(n=e.charAt(1))||!t&&"|"==n)},ne=function(e){var t;return e.length>1&&te(e.slice(0,2))&&(2==e.length||"/"===(t=e.charAt(2))||"\\"===t||"?"===t||"#"===t)},re=function(e){var t=e.path,n=t.length;!n||"file"==e.scheme&&1==n&&te(t[0],!0)||t.pop()},ie=function(e){return"."===e||"%2e"===e.toLowerCase()},ae=function(e){return e=e.toLowerCase(),".."===e||"%2e."===e||".%2e"===e||"%2e%2e"===e},oe={},se={},le={},ue={},ce={},de={},fe={},he={},pe={},me={},_e={},ve={},ge={},be={},ye={},Me={},ke={},we={},Le={},xe={},Se={},Te=function(e,t,n,i){var a,o,s,l,u=n||oe,c=0,f="",p=!1,m=!1,_=!1;n||(e.scheme="",e.username="",e.password="",e.host=null,e.port=null,e.path=[],e.query=null,e.fragment=null,e.cannotBeABaseURL=!1,t=t.replace(R,"")),t=t.replace(N,""),a=h(t);while(c<=a.length){switch(o=a[c],u){case oe:if(!o||!C.test(o)){if(n)return T;u=le;continue}f+=o.toLowerCase(),u=se;break;case se:if(o&&(E.test(o)||"+"==o||"-"==o||"."==o))f+=o.toLowerCase();else{if(":"!=o){if(n)return T;f="",u=le,c=0;continue}if(n&&(Q(e)!=d(X,f)||"file"==f&&(Z(e)||null!==e.port)||"file"==e.scheme&&!e.host))return;if(e.scheme=f,n)return void(Q(e)&&X[e.scheme]==e.port&&(e.port=null));f="","file"==e.scheme?u=be:Q(e)&&i&&i.scheme==e.scheme?u=ue:Q(e)?u=he:"/"==a[c+1]?(u=ce,c++):(e.cannotBeABaseURL=!0,e.path.push(""),u=Le)}break;case le:if(!i||i.cannotBeABaseURL&&"#"!=o)return T;if(i.cannotBeABaseURL&&"#"==o){e.scheme=i.scheme,e.path=i.path.slice(),e.query=i.query,e.fragment="",e.cannotBeABaseURL=!0,u=Se;break}u="file"==i.scheme?be:de;continue;case ue:if("/"!=o||"/"!=a[c+1]){u=de;continue}u=pe,c++;break;case ce:if("/"==o){u=me;break}u=we;continue;case de:if(e.scheme=i.scheme,o==r)e.username=i.username,e.password=i.password,e.host=i.host,e.port=i.port,e.path=i.path.slice(),e.query=i.query;else if("/"==o||"\\"==o&&Q(e))u=fe;else if("?"==o)e.username=i.username,e.password=i.password,e.host=i.host,e.port=i.port,e.path=i.path.slice(),e.query="",u=xe;else{if("#"!=o){e.username=i.username,e.password=i.password,e.host=i.host,e.port=i.port,e.path=i.path.slice(),e.path.pop(),u=we;continue}e.username=i.username,e.password=i.password,e.host=i.host,e.port=i.port,e.path=i.path.slice(),e.query=i.query,e.fragment="",u=Se}break;case fe:if(!Q(e)||"/"!=o&&"\\"!=o){if("/"!=o){e.username=i.username,e.password=i.password,e.host=i.host,e.port=i.port,u=we;continue}u=me}else u=pe;break;case he:if(u=pe,"/"!=o||"/"!=f.charAt(c+1))continue;c++;break;case pe:if("/"!=o&&"\\"!=o){u=me;continue}break;case me:if("@"==o){p&&(f="%40"+f),p=!0,s=h(f);for(var v=0;v<s.length;v++){var g=s[v];if(":"!=g||_){var b=K(g,J);_?e.password+=b:e.username+=b}else _=!0}f=""}else if(o==r||"/"==o||"?"==o||"#"==o||"\\"==o&&Q(e)){if(p&&""==f)return S;c-=h(f).length+1,f="",u=_e}else f+=o;break;case _e:case ve:if(n&&"file"==e.scheme){u=Me;continue}if(":"!=o||m){if(o==r||"/"==o||"?"==o||"#"==o||"\\"==o&&Q(e)){if(Q(e)&&""==f)return D;if(n&&""==f&&(Z(e)||null!==e.port))return;if(l=$(e,f),l)return l;if(f="",u=ke,n)return;continue}"["==o?m=!0:"]"==o&&(m=!1),f+=o}else{if(""==f)return D;if(l=$(e,f),l)return l;if(f="",u=ge,n==ve)return}break;case ge:if(!A.test(o)){if(o==r||"/"==o||"?"==o||"#"==o||"\\"==o&&Q(e)||n){if(""!=f){var y=parseInt(f,10);if(y>65535)return Y;e.port=Q(e)&&y===X[e.scheme]?null:y,f=""}if(n)return;u=ke;continue}return Y}f+=o;break;case be:if(e.scheme="file","/"==o||"\\"==o)u=ye;else{if(!i||"file"!=i.scheme){u=we;continue}if(o==r)e.host=i.host,e.path=i.path.slice(),e.query=i.query;else if("?"==o)e.host=i.host,e.path=i.path.slice(),e.query="",u=xe;else{if("#"!=o){ne(a.slice(c).join(""))||(e.host=i.host,e.path=i.path.slice(),re(e)),u=we;continue}e.host=i.host,e.path=i.path.slice(),e.query=i.query,e.fragment="",u=Se}}break;case ye:if("/"==o||"\\"==o){u=Me;break}i&&"file"==i.scheme&&!ne(a.slice(c).join(""))&&(te(i.path[0],!0)?e.path.push(i.path[0]):e.host=i.host),u=we;continue;case Me:if(o==r||"/"==o||"\\"==o||"?"==o||"#"==o){if(!n&&te(f))u=we;else if(""==f){if(e.host="",n)return;u=ke}else{if(l=$(e,f),l)return l;if("localhost"==e.host&&(e.host=""),n)return;f="",u=ke}continue}f+=o;break;case ke:if(Q(e)){if(u=we,"/"!=o&&"\\"!=o)continue}else if(n||"?"!=o)if(n||"#"!=o){if(o!=r&&(u=we,"/"!=o))continue}else e.fragment="",u=Se;else e.query="",u=xe;break;case we:if(o==r||"/"==o||"\\"==o&&Q(e)||!n&&("?"==o||"#"==o)){if(ae(f)?(re(e),"/"==o||"\\"==o&&Q(e)||e.path.push("")):ie(f)?"/"==o||"\\"==o&&Q(e)||e.path.push(""):("file"==e.scheme&&!e.path.length&&te(f)&&(e.host&&(e.host=""),f=f.charAt(0)+":"),e.path.push(f)),f="","file"==e.scheme&&(o==r||"?"==o||"#"==o))while(e.path.length>1&&""===e.path[0])e.path.shift();"?"==o?(e.query="",u=xe):"#"==o&&(e.fragment="",u=Se)}else f+=K(o,G);break;case Le:"?"==o?(e.query="",u=xe):"#"==o?(e.fragment="",u=Se):o!=r&&(e.path[0]+=K(o,V));break;case xe:n||"#"!=o?o!=r&&("'"==o&&Q(e)?e.query+="%27":e.query+="#"==o?"%23":K(o,V)):(e.fragment="",u=Se);break;case Se:o!=r&&(e.fragment+=K(o,q));break}c++}},De=function(e){var t,n,r=c(this,De,"URL"),i=arguments.length>1?arguments[1]:void 0,o=String(e),s=k(r,{type:"URL"});if(void 0!==i)if(i instanceof De)t=w(i);else if(n=Te(t={},String(i)),n)throw TypeError(n);if(n=Te(s,o,null,t),n)throw TypeError(n);var l=s.searchParams=new y,u=M(l);u.updateSearchParams(s.query),u.updateURL=function(){s.query=String(l)||null},a||(r.href=Ce.call(r),r.origin=Ee.call(r),r.protocol=Ae.call(r),r.username=Oe.call(r),r.password=Pe.call(r),r.host=Fe.call(r),r.hostname=Ie.call(r),r.port=je.call(r),r.pathname=He.call(r),r.search=Re.call(r),r.searchParams=Ne.call(r),r.hash=$e.call(r))},Ye=De.prototype,Ce=function(){var e=w(this),t=e.scheme,n=e.username,r=e.password,i=e.host,a=e.port,o=e.path,s=e.query,l=e.fragment,u=t+":";return null!==i?(u+="//",Z(e)&&(u+=n+(r?":"+r:"")+"@"),u+=U(i),null!==a&&(u+=":"+a)):"file"==t&&(u+="//"),u+=e.cannotBeABaseURL?o[0]:o.length?"/"+o.join("/"):"",null!==s&&(u+="?"+s),null!==l&&(u+="#"+l),u},Ee=function(){var e=w(this),t=e.scheme,n=e.port;if("blob"==t)try{return new URL(t.path[0]).origin}catch(r){return"null"}return"file"!=t&&Q(e)?t+"://"+U(e.host)+(null!==n?":"+n:""):"null"},Ae=function(){return w(this).scheme+":"},Oe=function(){return w(this).username},Pe=function(){return w(this).password},Fe=function(){var e=w(this),t=e.host,n=e.port;return null===t?"":null===n?U(t):U(t)+":"+n},Ie=function(){var e=w(this).host;return null===e?"":U(e)},je=function(){var e=w(this).port;return null===e?"":String(e)},He=function(){var e=w(this),t=e.path;return e.cannotBeABaseURL?t[0]:t.length?"/"+t.join("/"):""},Re=function(){var e=w(this).query;return e?"?"+e:""},Ne=function(){return w(this).searchParams},$e=function(){var e=w(this).fragment;return e?"#"+e:""},ze=function(e,t){return{get:e,set:t,configurable:!0,enumerable:!0}};if(a&&l(Ye,{href:ze(Ce,(function(e){var t=w(this),n=String(e),r=Te(t,n);if(r)throw TypeError(r);M(t.searchParams).updateSearchParams(t.query)})),origin:ze(Ee),protocol:ze(Ae,(function(e){var t=w(this);Te(t,String(e)+":",oe)})),username:ze(Oe,(function(e){var t=w(this),n=h(String(e));if(!ee(t)){t.username="";for(var r=0;r<n.length;r++)t.username+=K(n[r],J)}})),password:ze(Pe,(function(e){var t=w(this),n=h(String(e));if(!ee(t)){t.password="";for(var r=0;r<n.length;r++)t.password+=K(n[r],J)}})),host:ze(Fe,(function(e){var t=w(this);t.cannotBeABaseURL||Te(t,String(e),_e)})),hostname:ze(Ie,(function(e){var t=w(this);t.cannotBeABaseURL||Te(t,String(e),ve)})),port:ze(je,(function(e){var t=w(this);ee(t)||(e=String(e),""==e?t.port=null:Te(t,e,ge))})),pathname:ze(He,(function(e){var t=w(this);t.cannotBeABaseURL||(t.path=[],Te(t,e+"",ke))})),search:ze(Re,(function(e){var t=w(this);e=String(e),""==e?t.query=null:("?"==e.charAt(0)&&(e=e.slice(1)),t.query="",Te(t,e,xe)),M(t.searchParams).updateSearchParams(t.query)})),searchParams:ze(Ne),hash:ze($e,(function(e){var t=w(this);e=String(e),""!=e?("#"==e.charAt(0)&&(e=e.slice(1)),t.fragment="",Te(t,e,Se)):t.fragment=null}))}),u(Ye,"toJSON",(function(){return Ce.call(this)}),{enumerable:!0}),u(Ye,"toString",(function(){return Ce.call(this)}),{enumerable:!0}),b){var Be=b.createObjectURL,We=b.revokeObjectURL;Be&&u(De,"createObjectURL",(function(e){return Be.apply(b,arguments)})),We&&u(De,"revokeObjectURL",(function(e){return We.apply(b,arguments)}))}_(De,"URL"),i({global:!0,forced:!o,sham:!a},{URL:De})},"2bfb":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("af",{months:"Januarie_Februarie_Maart_April_Mei_Junie_Julie_Augustus_September_Oktober_November_Desember".split("_"),monthsShort:"Jan_Feb_Mrt_Apr_Mei_Jun_Jul_Aug_Sep_Okt_Nov_Des".split("_"),weekdays:"Sondag_Maandag_Dinsdag_Woensdag_Donderdag_Vrydag_Saterdag".split("_"),weekdaysShort:"Son_Maa_Din_Woe_Don_Vry_Sat".split("_"),weekdaysMin:"So_Ma_Di_Wo_Do_Vr_Sa".split("_"),meridiemParse:/vm|nm/i,isPM:function(e){return/^nm$/i.test(e)},meridiem:function(e,t,n){return e<12?n?"vm":"VM":n?"nm":"NM"},longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Vandag om] LT",nextDay:"[Môre om] LT",nextWeek:"dddd [om] LT",lastDay:"[Gister om] LT",lastWeek:"[Laas] dddd [om] LT",sameElse:"L"},relativeTime:{future:"oor %s",past:"%s gelede",s:"'n paar sekondes",ss:"%d sekondes",m:"'n minuut",mm:"%d minute",h:"'n uur",hh:"%d ure",d:"'n dag",dd:"%d dae",M:"'n maand",MM:"%d maande",y:"'n jaar",yy:"%d jaar"},dayOfMonthOrdinalParse:/\d{1,2}(ste|de)/,ordinal:function(e){return e+(1===e||8===e||e>=20?"ste":"de")},week:{dow:1,doy:4}});return t}))},"2cf4":function(e,t,n){var r,i,a,o=n("da84"),s=n("d039"),l=n("0366"),u=n("1be4"),c=n("cc12"),d=n("1cdc"),f=n("605d"),h=o.location,p=o.setImmediate,m=o.clearImmediate,_=o.process,v=o.MessageChannel,g=o.Dispatch,b=0,y={},M="onreadystatechange",k=function(e){if(y.hasOwnProperty(e)){var t=y[e];delete y[e],t()}},w=function(e){return function(){k(e)}},L=function(e){k(e.data)},x=function(e){o.postMessage(e+"",h.protocol+"//"+h.host)};p&&m||(p=function(e){var t=[],n=1;while(arguments.length>n)t.push(arguments[n++]);return y[++b]=function(){("function"==typeof e?e:Function(e)).apply(void 0,t)},r(b),b},m=function(e){delete y[e]},f?r=function(e){_.nextTick(w(e))}:g&&g.now?r=function(e){g.now(w(e))}:v&&!d?(i=new v,a=i.port2,i.port1.onmessage=L,r=l(a.postMessage,a,1)):o.addEventListener&&"function"==typeof postMessage&&!o.importScripts&&h&&"file:"!==h.protocol&&!s(x)?(r=x,o.addEventListener("message",L,!1)):r=M in c("script")?function(e){u.appendChild(c("script"))[M]=function(){u.removeChild(this),k(e)}}:function(e){setTimeout(w(e),0)}),e.exports={set:p,clear:m}},"2d00":function(e,t,n){var r,i,a=n("da84"),o=n("342f"),s=a.process,l=s&&s.versions,u=l&&l.v8;u?(r=u.split("."),i=r[0]+r[1]):o&&(r=o.match(/Edge\/(\d+)/),(!r||r[1]>=74)&&(r=o.match(/Chrome\/(\d+)/),r&&(i=r[1]))),e.exports=i&&+i},"2e8c":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("uz",{months:"январ_феврал_март_апрел_май_июн_июл_август_сентябр_октябр_ноябр_декабр".split("_"),monthsShort:"янв_фев_мар_апр_май_июн_июл_авг_сен_окт_ноя_дек".split("_"),weekdays:"Якшанба_Душанба_Сешанба_Чоршанба_Пайшанба_Жума_Шанба".split("_"),weekdaysShort:"Якш_Душ_Сеш_Чор_Пай_Жум_Шан".split("_"),weekdaysMin:"Як_Ду_Се_Чо_Па_Жу_Ша".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"D MMMM YYYY, dddd HH:mm"},calendar:{sameDay:"[Бугун соат] LT [да]",nextDay:"[Эртага] LT [да]",nextWeek:"dddd [куни соат] LT [да]",lastDay:"[Кеча соат] LT [да]",lastWeek:"[Утган] dddd [куни соат] LT [да]",sameElse:"L"},relativeTime:{future:"Якин %s ичида",past:"Бир неча %s олдин",s:"фурсат",ss:"%d фурсат",m:"бир дакика",mm:"%d дакика",h:"бир соат",hh:"%d соат",d:"бир кун",dd:"%d кун",M:"бир ой",MM:"%d ой",y:"бир йил",yy:"%d йил"},week:{dow:1,doy:7}});return t}))},"30ef":function(e,t,n){
/*!
 * Chart.js v2.9.4
 * https://www.chartjs.org
 * (c) 2020 Chart.js Contributors
 * Released under the MIT License
 */
(function(t,r){e.exports=r(function(){try{return n("c1df")}catch(e){}}())})(0,(function(e){"use strict";function t(e,t){return t={exports:{}},e(t,t.exports),t.exports}function n(e){return e&&e["default"]||e}e=e&&e.hasOwnProperty("default")?e["default"]:e;var r={aliceblue:[240,248,255],antiquewhite:[250,235,215],aqua:[0,255,255],aquamarine:[127,255,212],azure:[240,255,255],beige:[245,245,220],bisque:[255,228,196],black:[0,0,0],blanchedalmond:[255,235,205],blue:[0,0,255],blueviolet:[138,43,226],brown:[165,42,42],burlywood:[222,184,135],cadetblue:[95,158,160],chartreuse:[127,255,0],chocolate:[210,105,30],coral:[255,127,80],cornflowerblue:[100,149,237],cornsilk:[255,248,220],crimson:[220,20,60],cyan:[0,255,255],darkblue:[0,0,139],darkcyan:[0,139,139],darkgoldenrod:[184,134,11],darkgray:[169,169,169],darkgreen:[0,100,0],darkgrey:[169,169,169],darkkhaki:[189,183,107],darkmagenta:[139,0,139],darkolivegreen:[85,107,47],darkorange:[255,140,0],darkorchid:[153,50,204],darkred:[139,0,0],darksalmon:[233,150,122],darkseagreen:[143,188,143],darkslateblue:[72,61,139],darkslategray:[47,79,79],darkslategrey:[47,79,79],darkturquoise:[0,206,209],darkviolet:[148,0,211],deeppink:[255,20,147],deepskyblue:[0,191,255],dimgray:[105,105,105],dimgrey:[105,105,105],dodgerblue:[30,144,255],firebrick:[178,34,34],floralwhite:[255,250,240],forestgreen:[34,139,34],fuchsia:[255,0,255],gainsboro:[220,220,220],ghostwhite:[248,248,255],gold:[255,215,0],goldenrod:[218,165,32],gray:[128,128,128],green:[0,128,0],greenyellow:[173,255,47],grey:[128,128,128],honeydew:[240,255,240],hotpink:[255,105,180],indianred:[205,92,92],indigo:[75,0,130],ivory:[255,255,240],khaki:[240,230,140],lavender:[230,230,250],lavenderblush:[255,240,245],lawngreen:[124,252,0],lemonchiffon:[255,250,205],lightblue:[173,216,230],lightcoral:[240,128,128],lightcyan:[224,255,255],lightgoldenrodyellow:[250,250,210],lightgray:[211,211,211],lightgreen:[144,238,144],lightgrey:[211,211,211],lightpink:[255,182,193],lightsalmon:[255,160,122],lightseagreen:[32,178,170],lightskyblue:[135,206,250],lightslategray:[119,136,153],lightslategrey:[119,136,153],lightsteelblue:[176,196,222],lightyellow:[255,255,224],lime:[0,255,0],limegreen:[50,205,50],linen:[250,240,230],magenta:[255,0,255],maroon:[128,0,0],mediumaquamarine:[102,205,170],mediumblue:[0,0,205],mediumorchid:[186,85,211],mediumpurple:[147,112,219],mediumseagreen:[60,179,113],mediumslateblue:[123,104,238],mediumspringgreen:[0,250,154],mediumturquoise:[72,209,204],mediumvioletred:[199,21,133],midnightblue:[25,25,112],mintcream:[245,255,250],mistyrose:[255,228,225],moccasin:[255,228,181],navajowhite:[255,222,173],navy:[0,0,128],oldlace:[253,245,230],olive:[128,128,0],olivedrab:[107,142,35],orange:[255,165,0],orangered:[255,69,0],orchid:[218,112,214],palegoldenrod:[238,232,170],palegreen:[152,251,152],paleturquoise:[175,238,238],palevioletred:[219,112,147],papayawhip:[255,239,213],peachpuff:[255,218,185],peru:[205,133,63],pink:[255,192,203],plum:[221,160,221],powderblue:[176,224,230],purple:[128,0,128],rebeccapurple:[102,51,153],red:[255,0,0],rosybrown:[188,143,143],royalblue:[65,105,225],saddlebrown:[139,69,19],salmon:[250,128,114],sandybrown:[244,164,96],seagreen:[46,139,87],seashell:[255,245,238],sienna:[160,82,45],silver:[192,192,192],skyblue:[135,206,235],slateblue:[106,90,205],slategray:[112,128,144],slategrey:[112,128,144],snow:[255,250,250],springgreen:[0,255,127],steelblue:[70,130,180],tan:[210,180,140],teal:[0,128,128],thistle:[216,191,216],tomato:[255,99,71],turquoise:[64,224,208],violet:[238,130,238],wheat:[245,222,179],white:[255,255,255],whitesmoke:[245,245,245],yellow:[255,255,0],yellowgreen:[154,205,50]},i=t((function(e){var t={};for(var n in r)r.hasOwnProperty(n)&&(t[r[n]]=n);var i=e.exports={rgb:{channels:3,labels:"rgb"},hsl:{channels:3,labels:"hsl"},hsv:{channels:3,labels:"hsv"},hwb:{channels:3,labels:"hwb"},cmyk:{channels:4,labels:"cmyk"},xyz:{channels:3,labels:"xyz"},lab:{channels:3,labels:"lab"},lch:{channels:3,labels:"lch"},hex:{channels:1,labels:["hex"]},keyword:{channels:1,labels:["keyword"]},ansi16:{channels:1,labels:["ansi16"]},ansi256:{channels:1,labels:["ansi256"]},hcg:{channels:3,labels:["h","c","g"]},apple:{channels:3,labels:["r16","g16","b16"]},gray:{channels:1,labels:["gray"]}};for(var a in i)if(i.hasOwnProperty(a)){if(!("channels"in i[a]))throw new Error("missing channels property: "+a);if(!("labels"in i[a]))throw new Error("missing channel labels property: "+a);if(i[a].labels.length!==i[a].channels)throw new Error("channel and label counts mismatch: "+a);var o=i[a].channels,s=i[a].labels;delete i[a].channels,delete i[a].labels,Object.defineProperty(i[a],"channels",{value:o}),Object.defineProperty(i[a],"labels",{value:s})}function l(e,t){return Math.pow(e[0]-t[0],2)+Math.pow(e[1]-t[1],2)+Math.pow(e[2]-t[2],2)}i.rgb.hsl=function(e){var t,n,r,i=e[0]/255,a=e[1]/255,o=e[2]/255,s=Math.min(i,a,o),l=Math.max(i,a,o),u=l-s;return l===s?t=0:i===l?t=(a-o)/u:a===l?t=2+(o-i)/u:o===l&&(t=4+(i-a)/u),t=Math.min(60*t,360),t<0&&(t+=360),r=(s+l)/2,n=l===s?0:r<=.5?u/(l+s):u/(2-l-s),[t,100*n,100*r]},i.rgb.hsv=function(e){var t,n,r,i,a,o=e[0]/255,s=e[1]/255,l=e[2]/255,u=Math.max(o,s,l),c=u-Math.min(o,s,l),d=function(e){return(u-e)/6/c+.5};return 0===c?i=a=0:(a=c/u,t=d(o),n=d(s),r=d(l),o===u?i=r-n:s===u?i=1/3+t-r:l===u&&(i=2/3+n-t),i<0?i+=1:i>1&&(i-=1)),[360*i,100*a,100*u]},i.rgb.hwb=function(e){var t=e[0],n=e[1],r=e[2],a=i.rgb.hsl(e)[0],o=1/255*Math.min(t,Math.min(n,r));return r=1-1/255*Math.max(t,Math.max(n,r)),[a,100*o,100*r]},i.rgb.cmyk=function(e){var t,n,r,i,a=e[0]/255,o=e[1]/255,s=e[2]/255;return i=Math.min(1-a,1-o,1-s),t=(1-a-i)/(1-i)||0,n=(1-o-i)/(1-i)||0,r=(1-s-i)/(1-i)||0,[100*t,100*n,100*r,100*i]},i.rgb.keyword=function(e){var n=t[e];if(n)return n;var i,a=1/0;for(var o in r)if(r.hasOwnProperty(o)){var s=r[o],u=l(e,s);u<a&&(a=u,i=o)}return i},i.keyword.rgb=function(e){return r[e]},i.rgb.xyz=function(e){var t=e[0]/255,n=e[1]/255,r=e[2]/255;t=t>.04045?Math.pow((t+.055)/1.055,2.4):t/12.92,n=n>.04045?Math.pow((n+.055)/1.055,2.4):n/12.92,r=r>.04045?Math.pow((r+.055)/1.055,2.4):r/12.92;var i=.4124*t+.3576*n+.1805*r,a=.2126*t+.7152*n+.0722*r,o=.0193*t+.1192*n+.9505*r;return[100*i,100*a,100*o]},i.rgb.lab=function(e){var t,n,r,a=i.rgb.xyz(e),o=a[0],s=a[1],l=a[2];return o/=95.047,s/=100,l/=108.883,o=o>.008856?Math.pow(o,1/3):7.787*o+16/116,s=s>.008856?Math.pow(s,1/3):7.787*s+16/116,l=l>.008856?Math.pow(l,1/3):7.787*l+16/116,t=116*s-16,n=500*(o-s),r=200*(s-l),[t,n,r]},i.hsl.rgb=function(e){var t,n,r,i,a,o=e[0]/360,s=e[1]/100,l=e[2]/100;if(0===s)return a=255*l,[a,a,a];n=l<.5?l*(1+s):l+s-l*s,t=2*l-n,i=[0,0,0];for(var u=0;u<3;u++)r=o+1/3*-(u-1),r<0&&r++,r>1&&r--,a=6*r<1?t+6*(n-t)*r:2*r<1?n:3*r<2?t+(n-t)*(2/3-r)*6:t,i[u]=255*a;return i},i.hsl.hsv=function(e){var t,n,r=e[0],i=e[1]/100,a=e[2]/100,o=i,s=Math.max(a,.01);return a*=2,i*=a<=1?a:2-a,o*=s<=1?s:2-s,n=(a+i)/2,t=0===a?2*o/(s+o):2*i/(a+i),[r,100*t,100*n]},i.hsv.rgb=function(e){var t=e[0]/60,n=e[1]/100,r=e[2]/100,i=Math.floor(t)%6,a=t-Math.floor(t),o=255*r*(1-n),s=255*r*(1-n*a),l=255*r*(1-n*(1-a));switch(r*=255,i){case 0:return[r,l,o];case 1:return[s,r,o];case 2:return[o,r,l];case 3:return[o,s,r];case 4:return[l,o,r];case 5:return[r,o,s]}},i.hsv.hsl=function(e){var t,n,r,i=e[0],a=e[1]/100,o=e[2]/100,s=Math.max(o,.01);return r=(2-a)*o,t=(2-a)*s,n=a*s,n/=t<=1?t:2-t,n=n||0,r/=2,[i,100*n,100*r]},i.hwb.rgb=function(e){var t,n,r,i,a,o,s,l=e[0]/360,u=e[1]/100,c=e[2]/100,d=u+c;switch(d>1&&(u/=d,c/=d),t=Math.floor(6*l),n=1-c,r=6*l-t,0!==(1&t)&&(r=1-r),i=u+r*(n-u),t){default:case 6:case 0:a=n,o=i,s=u;break;case 1:a=i,o=n,s=u;break;case 2:a=u,o=n,s=i;break;case 3:a=u,o=i,s=n;break;case 4:a=i,o=u,s=n;break;case 5:a=n,o=u,s=i;break}return[255*a,255*o,255*s]},i.cmyk.rgb=function(e){var t,n,r,i=e[0]/100,a=e[1]/100,o=e[2]/100,s=e[3]/100;return t=1-Math.min(1,i*(1-s)+s),n=1-Math.min(1,a*(1-s)+s),r=1-Math.min(1,o*(1-s)+s),[255*t,255*n,255*r]},i.xyz.rgb=function(e){var t,n,r,i=e[0]/100,a=e[1]/100,o=e[2]/100;return t=3.2406*i+-1.5372*a+-.4986*o,n=-.9689*i+1.8758*a+.0415*o,r=.0557*i+-.204*a+1.057*o,t=t>.0031308?1.055*Math.pow(t,1/2.4)-.055:12.92*t,n=n>.0031308?1.055*Math.pow(n,1/2.4)-.055:12.92*n,r=r>.0031308?1.055*Math.pow(r,1/2.4)-.055:12.92*r,t=Math.min(Math.max(0,t),1),n=Math.min(Math.max(0,n),1),r=Math.min(Math.max(0,r),1),[255*t,255*n,255*r]},i.xyz.lab=function(e){var t,n,r,i=e[0],a=e[1],o=e[2];return i/=95.047,a/=100,o/=108.883,i=i>.008856?Math.pow(i,1/3):7.787*i+16/116,a=a>.008856?Math.pow(a,1/3):7.787*a+16/116,o=o>.008856?Math.pow(o,1/3):7.787*o+16/116,t=116*a-16,n=500*(i-a),r=200*(a-o),[t,n,r]},i.lab.xyz=function(e){var t,n,r,i=e[0],a=e[1],o=e[2];n=(i+16)/116,t=a/500+n,r=n-o/200;var s=Math.pow(n,3),l=Math.pow(t,3),u=Math.pow(r,3);return n=s>.008856?s:(n-16/116)/7.787,t=l>.008856?l:(t-16/116)/7.787,r=u>.008856?u:(r-16/116)/7.787,t*=95.047,n*=100,r*=108.883,[t,n,r]},i.lab.lch=function(e){var t,n,r,i=e[0],a=e[1],o=e[2];return t=Math.atan2(o,a),n=360*t/2/Math.PI,n<0&&(n+=360),r=Math.sqrt(a*a+o*o),[i,r,n]},i.lch.lab=function(e){var t,n,r,i=e[0],a=e[1],o=e[2];return r=o/360*2*Math.PI,t=a*Math.cos(r),n=a*Math.sin(r),[i,t,n]},i.rgb.ansi16=function(e){var t=e[0],n=e[1],r=e[2],a=1 in arguments?arguments[1]:i.rgb.hsv(e)[2];if(a=Math.round(a/50),0===a)return 30;var o=30+(Math.round(r/255)<<2|Math.round(n/255)<<1|Math.round(t/255));return 2===a&&(o+=60),o},i.hsv.ansi16=function(e){return i.rgb.ansi16(i.hsv.rgb(e),e[2])},i.rgb.ansi256=function(e){var t=e[0],n=e[1],r=e[2];if(t===n&&n===r)return t<8?16:t>248?231:Math.round((t-8)/247*24)+232;var i=16+36*Math.round(t/255*5)+6*Math.round(n/255*5)+Math.round(r/255*5);return i},i.ansi16.rgb=function(e){var t=e%10;if(0===t||7===t)return e>50&&(t+=3.5),t=t/10.5*255,[t,t,t];var n=.5*(1+~~(e>50)),r=(1&t)*n*255,i=(t>>1&1)*n*255,a=(t>>2&1)*n*255;return[r,i,a]},i.ansi256.rgb=function(e){if(e>=232){var t=10*(e-232)+8;return[t,t,t]}var n;e-=16;var r=Math.floor(e/36)/5*255,i=Math.floor((n=e%36)/6)/5*255,a=n%6/5*255;return[r,i,a]},i.rgb.hex=function(e){var t=((255&Math.round(e[0]))<<16)+((255&Math.round(e[1]))<<8)+(255&Math.round(e[2])),n=t.toString(16).toUpperCase();return"000000".substring(n.length)+n},i.hex.rgb=function(e){var t=e.toString(16).match(/[a-f0-9]{6}|[a-f0-9]{3}/i);if(!t)return[0,0,0];var n=t[0];3===t[0].length&&(n=n.split("").map((function(e){return e+e})).join(""));var r=parseInt(n,16),i=r>>16&255,a=r>>8&255,o=255&r;return[i,a,o]},i.rgb.hcg=function(e){var t,n,r=e[0]/255,i=e[1]/255,a=e[2]/255,o=Math.max(Math.max(r,i),a),s=Math.min(Math.min(r,i),a),l=o-s;return t=l<1?s/(1-l):0,n=l<=0?0:o===r?(i-a)/l%6:o===i?2+(a-r)/l:4+(r-i)/l+4,n/=6,n%=1,[360*n,100*l,100*t]},i.hsl.hcg=function(e){var t=e[1]/100,n=e[2]/100,r=1,i=0;return r=n<.5?2*t*n:2*t*(1-n),r<1&&(i=(n-.5*r)/(1-r)),[e[0],100*r,100*i]},i.hsv.hcg=function(e){var t=e[1]/100,n=e[2]/100,r=t*n,i=0;return r<1&&(i=(n-r)/(1-r)),[e[0],100*r,100*i]},i.hcg.rgb=function(e){var t=e[0]/360,n=e[1]/100,r=e[2]/100;if(0===n)return[255*r,255*r,255*r];var i=[0,0,0],a=t%1*6,o=a%1,s=1-o,l=0;switch(Math.floor(a)){case 0:i[0]=1,i[1]=o,i[2]=0;break;case 1:i[0]=s,i[1]=1,i[2]=0;break;case 2:i[0]=0,i[1]=1,i[2]=o;break;case 3:i[0]=0,i[1]=s,i[2]=1;break;case 4:i[0]=o,i[1]=0,i[2]=1;break;default:i[0]=1,i[1]=0,i[2]=s}return l=(1-n)*r,[255*(n*i[0]+l),255*(n*i[1]+l),255*(n*i[2]+l)]},i.hcg.hsv=function(e){var t=e[1]/100,n=e[2]/100,r=t+n*(1-t),i=0;return r>0&&(i=t/r),[e[0],100*i,100*r]},i.hcg.hsl=function(e){var t=e[1]/100,n=e[2]/100,r=n*(1-t)+.5*t,i=0;return r>0&&r<.5?i=t/(2*r):r>=.5&&r<1&&(i=t/(2*(1-r))),[e[0],100*i,100*r]},i.hcg.hwb=function(e){var t=e[1]/100,n=e[2]/100,r=t+n*(1-t);return[e[0],100*(r-t),100*(1-r)]},i.hwb.hcg=function(e){var t=e[1]/100,n=e[2]/100,r=1-n,i=r-t,a=0;return i<1&&(a=(r-i)/(1-i)),[e[0],100*i,100*a]},i.apple.rgb=function(e){return[e[0]/65535*255,e[1]/65535*255,e[2]/65535*255]},i.rgb.apple=function(e){return[e[0]/255*65535,e[1]/255*65535,e[2]/255*65535]},i.gray.rgb=function(e){return[e[0]/100*255,e[0]/100*255,e[0]/100*255]},i.gray.hsl=i.gray.hsv=function(e){return[0,0,e[0]]},i.gray.hwb=function(e){return[0,100,e[0]]},i.gray.cmyk=function(e){return[0,0,0,e[0]]},i.gray.lab=function(e){return[e[0],0,0]},i.gray.hex=function(e){var t=255&Math.round(e[0]/100*255),n=(t<<16)+(t<<8)+t,r=n.toString(16).toUpperCase();return"000000".substring(r.length)+r},i.rgb.gray=function(e){var t=(e[0]+e[1]+e[2])/3;return[t/255*100]}}));i.rgb,i.hsl,i.hsv,i.hwb,i.cmyk,i.xyz,i.lab,i.lch,i.hex,i.keyword,i.ansi16,i.ansi256,i.hcg,i.apple,i.gray;function a(){for(var e={},t=Object.keys(i),n=t.length,r=0;r<n;r++)e[t[r]]={distance:-1,parent:null};return e}function o(e){var t=a(),n=[e];t[e].distance=0;while(n.length)for(var r=n.pop(),o=Object.keys(i[r]),s=o.length,l=0;l<s;l++){var u=o[l],c=t[u];-1===c.distance&&(c.distance=t[r].distance+1,c.parent=r,n.unshift(u))}return t}function s(e,t){return function(n){return t(e(n))}}function l(e,t){var n=[t[e].parent,e],r=i[t[e].parent][e],a=t[e].parent;while(t[a].parent)n.unshift(t[a].parent),r=s(i[t[a].parent][a],r),a=t[a].parent;return r.conversion=n,r}var u=function(e){for(var t=o(e),n={},r=Object.keys(t),i=r.length,a=0;a<i;a++){var s=r[a],u=t[s];null!==u.parent&&(n[s]=l(s,t))}return n},c={},d=Object.keys(i);function f(e){var t=function(t){return void 0===t||null===t?t:(arguments.length>1&&(t=Array.prototype.slice.call(arguments)),e(t))};return"conversion"in e&&(t.conversion=e.conversion),t}function h(e){var t=function(t){if(void 0===t||null===t)return t;arguments.length>1&&(t=Array.prototype.slice.call(arguments));var n=e(t);if("object"===typeof n)for(var r=n.length,i=0;i<r;i++)n[i]=Math.round(n[i]);return n};return"conversion"in e&&(t.conversion=e.conversion),t}d.forEach((function(e){c[e]={},Object.defineProperty(c[e],"channels",{value:i[e].channels}),Object.defineProperty(c[e],"labels",{value:i[e].labels});var t=u(e),n=Object.keys(t);n.forEach((function(n){var r=t[n];c[e][n]=h(r),c[e][n].raw=f(r)}))}));var p=c,m={aliceblue:[240,248,255],antiquewhite:[250,235,215],aqua:[0,255,255],aquamarine:[127,255,212],azure:[240,255,255],beige:[245,245,220],bisque:[255,228,196],black:[0,0,0],blanchedalmond:[255,235,205],blue:[0,0,255],blueviolet:[138,43,226],brown:[165,42,42],burlywood:[222,184,135],cadetblue:[95,158,160],chartreuse:[127,255,0],chocolate:[210,105,30],coral:[255,127,80],cornflowerblue:[100,149,237],cornsilk:[255,248,220],crimson:[220,20,60],cyan:[0,255,255],darkblue:[0,0,139],darkcyan:[0,139,139],darkgoldenrod:[184,134,11],darkgray:[169,169,169],darkgreen:[0,100,0],darkgrey:[169,169,169],darkkhaki:[189,183,107],darkmagenta:[139,0,139],darkolivegreen:[85,107,47],darkorange:[255,140,0],darkorchid:[153,50,204],darkred:[139,0,0],darksalmon:[233,150,122],darkseagreen:[143,188,143],darkslateblue:[72,61,139],darkslategray:[47,79,79],darkslategrey:[47,79,79],darkturquoise:[0,206,209],darkviolet:[148,0,211],deeppink:[255,20,147],deepskyblue:[0,191,255],dimgray:[105,105,105],dimgrey:[105,105,105],dodgerblue:[30,144,255],firebrick:[178,34,34],floralwhite:[255,250,240],forestgreen:[34,139,34],fuchsia:[255,0,255],gainsboro:[220,220,220],ghostwhite:[248,248,255],gold:[255,215,0],goldenrod:[218,165,32],gray:[128,128,128],green:[0,128,0],greenyellow:[173,255,47],grey:[128,128,128],honeydew:[240,255,240],hotpink:[255,105,180],indianred:[205,92,92],indigo:[75,0,130],ivory:[255,255,240],khaki:[240,230,140],lavender:[230,230,250],lavenderblush:[255,240,245],lawngreen:[124,252,0],lemonchiffon:[255,250,205],lightblue:[173,216,230],lightcoral:[240,128,128],lightcyan:[224,255,255],lightgoldenrodyellow:[250,250,210],lightgray:[211,211,211],lightgreen:[144,238,144],lightgrey:[211,211,211],lightpink:[255,182,193],lightsalmon:[255,160,122],lightseagreen:[32,178,170],lightskyblue:[135,206,250],lightslategray:[119,136,153],lightslategrey:[119,136,153],lightsteelblue:[176,196,222],lightyellow:[255,255,224],lime:[0,255,0],limegreen:[50,205,50],linen:[250,240,230],magenta:[255,0,255],maroon:[128,0,0],mediumaquamarine:[102,205,170],mediumblue:[0,0,205],mediumorchid:[186,85,211],mediumpurple:[147,112,219],mediumseagreen:[60,179,113],mediumslateblue:[123,104,238],mediumspringgreen:[0,250,154],mediumturquoise:[72,209,204],mediumvioletred:[199,21,133],midnightblue:[25,25,112],mintcream:[245,255,250],mistyrose:[255,228,225],moccasin:[255,228,181],navajowhite:[255,222,173],navy:[0,0,128],oldlace:[253,245,230],olive:[128,128,0],olivedrab:[107,142,35],orange:[255,165,0],orangered:[255,69,0],orchid:[218,112,214],palegoldenrod:[238,232,170],palegreen:[152,251,152],paleturquoise:[175,238,238],palevioletred:[219,112,147],papayawhip:[255,239,213],peachpuff:[255,218,185],peru:[205,133,63],pink:[255,192,203],plum:[221,160,221],powderblue:[176,224,230],purple:[128,0,128],rebeccapurple:[102,51,153],red:[255,0,0],rosybrown:[188,143,143],royalblue:[65,105,225],saddlebrown:[139,69,19],salmon:[250,128,114],sandybrown:[244,164,96],seagreen:[46,139,87],seashell:[255,245,238],sienna:[160,82,45],silver:[192,192,192],skyblue:[135,206,235],slateblue:[106,90,205],slategray:[112,128,144],slategrey:[112,128,144],snow:[255,250,250],springgreen:[0,255,127],steelblue:[70,130,180],tan:[210,180,140],teal:[0,128,128],thistle:[216,191,216],tomato:[255,99,71],turquoise:[64,224,208],violet:[238,130,238],wheat:[245,222,179],white:[255,255,255],whitesmoke:[245,245,245],yellow:[255,255,0],yellowgreen:[154,205,50]},_={getRgba:v,getHsla:g,getRgb:y,getHsl:M,getHwb:b,getAlpha:k,hexString:w,rgbString:L,rgbaString:x,percentString:S,percentaString:T,hslString:D,hslaString:Y,hwbString:C,keyword:E};function v(e){if(e){var t=/^#([a-fA-F0-9]{3,4})$/i,n=/^#([a-fA-F0-9]{6}([a-fA-F0-9]{2})?)$/i,r=/^rgba?\(\s*([+-]?\d+)\s*,\s*([+-]?\d+)\s*,\s*([+-]?\d+)\s*(?:,\s*([+-]?[\d\.]+)\s*)?\)$/i,i=/^rgba?\(\s*([+-]?[\d\.]+)\%\s*,\s*([+-]?[\d\.]+)\%\s*,\s*([+-]?[\d\.]+)\%\s*(?:,\s*([+-]?[\d\.]+)\s*)?\)$/i,a=/(\w+)/,o=[0,0,0],s=1,l=e.match(t),u="";if(l){l=l[1],u=l[3];for(var c=0;c<o.length;c++)o[c]=parseInt(l[c]+l[c],16);u&&(s=Math.round(parseInt(u+u,16)/255*100)/100)}else if(l=e.match(n)){u=l[2],l=l[1];for(c=0;c<o.length;c++)o[c]=parseInt(l.slice(2*c,2*c+2),16);u&&(s=Math.round(parseInt(u,16)/255*100)/100)}else if(l=e.match(r)){for(c=0;c<o.length;c++)o[c]=parseInt(l[c+1]);s=parseFloat(l[4])}else if(l=e.match(i)){for(c=0;c<o.length;c++)o[c]=Math.round(2.55*parseFloat(l[c+1]));s=parseFloat(l[4])}else if(l=e.match(a)){if("transparent"==l[1])return[0,0,0,0];if(o=m[l[1]],!o)return}for(c=0;c<o.length;c++)o[c]=A(o[c],0,255);return s=s||0==s?A(s,0,1):1,o[3]=s,o}}function g(e){if(e){var t=/^hsla?\(\s*([+-]?\d+)(?:deg)?\s*,\s*([+-]?[\d\.]+)%\s*,\s*([+-]?[\d\.]+)%\s*(?:,\s*([+-]?[\d\.]+)\s*)?\)/,n=e.match(t);if(n){var r=parseFloat(n[4]),i=A(parseInt(n[1]),0,360),a=A(parseFloat(n[2]),0,100),o=A(parseFloat(n[3]),0,100),s=A(isNaN(r)?1:r,0,1);return[i,a,o,s]}}}function b(e){if(e){var t=/^hwb\(\s*([+-]?\d+)(?:deg)?\s*,\s*([+-]?[\d\.]+)%\s*,\s*([+-]?[\d\.]+)%\s*(?:,\s*([+-]?[\d\.]+)\s*)?\)/,n=e.match(t);if(n){var r=parseFloat(n[4]),i=A(parseInt(n[1]),0,360),a=A(parseFloat(n[2]),0,100),o=A(parseFloat(n[3]),0,100),s=A(isNaN(r)?1:r,0,1);return[i,a,o,s]}}}function y(e){var t=v(e);return t&&t.slice(0,3)}function M(e){var t=g(e);return t&&t.slice(0,3)}function k(e){var t=v(e);return t||(t=g(e))||(t=b(e))?t[3]:void 0}function w(e,t){t=void 0!==t&&3===e.length?t:e[3];return"#"+O(e[0])+O(e[1])+O(e[2])+(t>=0&&t<1?O(Math.round(255*t)):"")}function L(e,t){return t<1||e[3]&&e[3]<1?x(e,t):"rgb("+e[0]+", "+e[1]+", "+e[2]+")"}function x(e,t){return void 0===t&&(t=void 0!==e[3]?e[3]:1),"rgba("+e[0]+", "+e[1]+", "+e[2]+", "+t+")"}function S(e,t){if(t<1||e[3]&&e[3]<1)return T(e,t);var n=Math.round(e[0]/255*100),r=Math.round(e[1]/255*100),i=Math.round(e[2]/255*100);return"rgb("+n+"%, "+r+"%, "+i+"%)"}function T(e,t){var n=Math.round(e[0]/255*100),r=Math.round(e[1]/255*100),i=Math.round(e[2]/255*100);return"rgba("+n+"%, "+r+"%, "+i+"%, "+(t||e[3]||1)+")"}function D(e,t){return t<1||e[3]&&e[3]<1?Y(e,t):"hsl("+e[0]+", "+e[1]+"%, "+e[2]+"%)"}function Y(e,t){return void 0===t&&(t=void 0!==e[3]?e[3]:1),"hsla("+e[0]+", "+e[1]+"%, "+e[2]+"%, "+t+")"}function C(e,t){return void 0===t&&(t=void 0!==e[3]?e[3]:1),"hwb("+e[0]+", "+e[1]+"%, "+e[2]+"%"+(void 0!==t&&1!==t?", "+t:"")+")"}function E(e){return P[e.slice(0,3)]}function A(e,t,n){return Math.min(Math.max(t,e),n)}function O(e){var t=e.toString(16).toUpperCase();return t.length<2?"0"+t:t}var P={};for(var F in m)P[m[F]]=F;var I=function(e){return e instanceof I?e:this instanceof I?(this.valid=!1,this.values={rgb:[0,0,0],hsl:[0,0,0],hsv:[0,0,0],hwb:[0,0,0],cmyk:[0,0,0,0],alpha:1},void("string"===typeof e?(t=_.getRgba(e),t?this.setValues("rgb",t):(t=_.getHsla(e))?this.setValues("hsl",t):(t=_.getHwb(e))&&this.setValues("hwb",t)):"object"===typeof e&&(t=e,void 0!==t.r||void 0!==t.red?this.setValues("rgb",t):void 0!==t.l||void 0!==t.lightness?this.setValues("hsl",t):void 0!==t.v||void 0!==t.value?this.setValues("hsv",t):void 0!==t.w||void 0!==t.whiteness?this.setValues("hwb",t):void 0===t.c&&void 0===t.cyan||this.setValues("cmyk",t)))):new I(e);var t};I.prototype={isValid:function(){return this.valid},rgb:function(){return this.setSpace("rgb",arguments)},hsl:function(){return this.setSpace("hsl",arguments)},hsv:function(){return this.setSpace("hsv",arguments)},hwb:function(){return this.setSpace("hwb",arguments)},cmyk:function(){return this.setSpace("cmyk",arguments)},rgbArray:function(){return this.values.rgb},hslArray:function(){return this.values.hsl},hsvArray:function(){return this.values.hsv},hwbArray:function(){var e=this.values;return 1!==e.alpha?e.hwb.concat([e.alpha]):e.hwb},cmykArray:function(){return this.values.cmyk},rgbaArray:function(){var e=this.values;return e.rgb.concat([e.alpha])},hslaArray:function(){var e=this.values;return e.hsl.concat([e.alpha])},alpha:function(e){return void 0===e?this.values.alpha:(this.setValues("alpha",e),this)},red:function(e){return this.setChannel("rgb",0,e)},green:function(e){return this.setChannel("rgb",1,e)},blue:function(e){return this.setChannel("rgb",2,e)},hue:function(e){return e&&(e%=360,e=e<0?360+e:e),this.setChannel("hsl",0,e)},saturation:function(e){return this.setChannel("hsl",1,e)},lightness:function(e){return this.setChannel("hsl",2,e)},saturationv:function(e){return this.setChannel("hsv",1,e)},whiteness:function(e){return this.setChannel("hwb",1,e)},blackness:function(e){return this.setChannel("hwb",2,e)},value:function(e){return this.setChannel("hsv",2,e)},cyan:function(e){return this.setChannel("cmyk",0,e)},magenta:function(e){return this.setChannel("cmyk",1,e)},yellow:function(e){return this.setChannel("cmyk",2,e)},black:function(e){return this.setChannel("cmyk",3,e)},hexString:function(){return _.hexString(this.values.rgb)},rgbString:function(){return _.rgbString(this.values.rgb,this.values.alpha)},rgbaString:function(){return _.rgbaString(this.values.rgb,this.values.alpha)},percentString:function(){return _.percentString(this.values.rgb,this.values.alpha)},hslString:function(){return _.hslString(this.values.hsl,this.values.alpha)},hslaString:function(){return _.hslaString(this.values.hsl,this.values.alpha)},hwbString:function(){return _.hwbString(this.values.hwb,this.values.alpha)},keyword:function(){return _.keyword(this.values.rgb,this.values.alpha)},rgbNumber:function(){var e=this.values.rgb;return e[0]<<16|e[1]<<8|e[2]},luminosity:function(){for(var e=this.values.rgb,t=[],n=0;n<e.length;n++){var r=e[n]/255;t[n]=r<=.03928?r/12.92:Math.pow((r+.055)/1.055,2.4)}return.2126*t[0]+.7152*t[1]+.0722*t[2]},contrast:function(e){var t=this.luminosity(),n=e.luminosity();return t>n?(t+.05)/(n+.05):(n+.05)/(t+.05)},level:function(e){var t=this.contrast(e);return t>=7.1?"AAA":t>=4.5?"AA":""},dark:function(){var e=this.values.rgb,t=(299*e[0]+587*e[1]+114*e[2])/1e3;return t<128},light:function(){return!this.dark()},negate:function(){for(var e=[],t=0;t<3;t++)e[t]=255-this.values.rgb[t];return this.setValues("rgb",e),this},lighten:function(e){var t=this.values.hsl;return t[2]+=t[2]*e,this.setValues("hsl",t),this},darken:function(e){var t=this.values.hsl;return t[2]-=t[2]*e,this.setValues("hsl",t),this},saturate:function(e){var t=this.values.hsl;return t[1]+=t[1]*e,this.setValues("hsl",t),this},desaturate:function(e){var t=this.values.hsl;return t[1]-=t[1]*e,this.setValues("hsl",t),this},whiten:function(e){var t=this.values.hwb;return t[1]+=t[1]*e,this.setValues("hwb",t),this},blacken:function(e){var t=this.values.hwb;return t[2]+=t[2]*e,this.setValues("hwb",t),this},greyscale:function(){var e=this.values.rgb,t=.3*e[0]+.59*e[1]+.11*e[2];return this.setValues("rgb",[t,t,t]),this},clearer:function(e){var t=this.values.alpha;return this.setValues("alpha",t-t*e),this},opaquer:function(e){var t=this.values.alpha;return this.setValues("alpha",t+t*e),this},rotate:function(e){var t=this.values.hsl,n=(t[0]+e)%360;return t[0]=n<0?360+n:n,this.setValues("hsl",t),this},mix:function(e,t){var n=this,r=e,i=void 0===t?.5:t,a=2*i-1,o=n.alpha()-r.alpha(),s=((a*o===-1?a:(a+o)/(1+a*o))+1)/2,l=1-s;return this.rgb(s*n.red()+l*r.red(),s*n.green()+l*r.green(),s*n.blue()+l*r.blue()).alpha(n.alpha()*i+r.alpha()*(1-i))},toJSON:function(){return this.rgb()},clone:function(){var e,t,n=new I,r=this.values,i=n.values;for(var a in r)r.hasOwnProperty(a)&&(e=r[a],t={}.toString.call(e),"[object Array]"===t?i[a]=e.slice(0):"[object Number]"===t?i[a]=e:console.error("unexpected color value:",e));return n}},I.prototype.spaces={rgb:["red","green","blue"],hsl:["hue","saturation","lightness"],hsv:["hue","saturation","value"],hwb:["hue","whiteness","blackness"],cmyk:["cyan","magenta","yellow","black"]},I.prototype.maxes={rgb:[255,255,255],hsl:[360,100,100],hsv:[360,100,100],hwb:[360,100,100],cmyk:[100,100,100,100]},I.prototype.getValues=function(e){for(var t=this.values,n={},r=0;r<e.length;r++)n[e.charAt(r)]=t[e][r];return 1!==t.alpha&&(n.a=t.alpha),n},I.prototype.setValues=function(e,t){var n,r,i=this.values,a=this.spaces,o=this.maxes,s=1;if(this.valid=!0,"alpha"===e)s=t;else if(t.length)i[e]=t.slice(0,e.length),s=t[e.length];else if(void 0!==t[e.charAt(0)]){for(n=0;n<e.length;n++)i[e][n]=t[e.charAt(n)];s=t.a}else if(void 0!==t[a[e][0]]){var l=a[e];for(n=0;n<e.length;n++)i[e][n]=t[l[n]];s=t.alpha}if(i.alpha=Math.max(0,Math.min(1,void 0===s?i.alpha:s)),"alpha"===e)return!1;for(n=0;n<e.length;n++)r=Math.max(0,Math.min(o[e][n],i[e][n])),i[e][n]=Math.round(r);for(var u in a)u!==e&&(i[u]=p[e][u](i[e]));return!0},I.prototype.setSpace=function(e,t){var n=t[0];return void 0===n?this.getValues(e):("number"===typeof n&&(n=Array.prototype.slice.call(t)),this.setValues(e,n),this)},I.prototype.setChannel=function(e,t,n){var r=this.values[e];return void 0===n?r[t]:(n===r[t]||(r[t]=n,this.setValues(e,r)),this)},"undefined"!==typeof window&&(window.Color=I);var j=I;function H(e){return-1===["__proto__","prototype","constructor"].indexOf(e)}var R={noop:function(){},uid:function(){var e=0;return function(){return e++}}(),isNullOrUndef:function(e){return null===e||"undefined"===typeof e},isArray:function(e){if(Array.isArray&&Array.isArray(e))return!0;var t=Object.prototype.toString.call(e);return"[object"===t.substr(0,7)&&"Array]"===t.substr(-6)},isObject:function(e){return null!==e&&"[object Object]"===Object.prototype.toString.call(e)},isFinite:function(e){return("number"===typeof e||e instanceof Number)&&isFinite(e)},valueOrDefault:function(e,t){return"undefined"===typeof e?t:e},valueAtIndexOrDefault:function(e,t,n){return R.valueOrDefault(R.isArray(e)?e[t]:e,n)},callback:function(e,t,n){if(e&&"function"===typeof e.call)return e.apply(n,t)},each:function(e,t,n,r){var i,a,o;if(R.isArray(e))if(a=e.length,r)for(i=a-1;i>=0;i--)t.call(n,e[i],i);else for(i=0;i<a;i++)t.call(n,e[i],i);else if(R.isObject(e))for(o=Object.keys(e),a=o.length,i=0;i<a;i++)t.call(n,e[o[i]],o[i])},arrayEquals:function(e,t){var n,r,i,a;if(!e||!t||e.length!==t.length)return!1;for(n=0,r=e.length;n<r;++n)if(i=e[n],a=t[n],i instanceof Array&&a instanceof Array){if(!R.arrayEquals(i,a))return!1}else if(i!==a)return!1;return!0},clone:function(e){if(R.isArray(e))return e.map(R.clone);if(R.isObject(e)){for(var t=Object.create(e),n=Object.keys(e),r=n.length,i=0;i<r;++i)t[n[i]]=R.clone(e[n[i]]);return t}return e},_merger:function(e,t,n,r){if(H(e)){var i=t[e],a=n[e];R.isObject(i)&&R.isObject(a)?R.merge(i,a,r):t[e]=R.clone(a)}},_mergerIf:function(e,t,n){if(H(e)){var r=t[e],i=n[e];R.isObject(r)&&R.isObject(i)?R.mergeIf(r,i):t.hasOwnProperty(e)||(t[e]=R.clone(i))}},merge:function(e,t,n){var r,i,a,o,s,l=R.isArray(t)?t:[t],u=l.length;if(!R.isObject(e))return e;for(n=n||{},r=n.merger||R._merger,i=0;i<u;++i)if(t=l[i],R.isObject(t))for(a=Object.keys(t),s=0,o=a.length;s<o;++s)r(a[s],e,t,n);return e},mergeIf:function(e,t){return R.merge(e,t,{merger:R._mergerIf})},extend:Object.assign||function(e){return R.merge(e,[].slice.call(arguments,1),{merger:function(e,t,n){t[e]=n[e]}})},inherits:function(e){var t=this,n=e&&e.hasOwnProperty("constructor")?e.constructor:function(){return t.apply(this,arguments)},r=function(){this.constructor=n};return r.prototype=t.prototype,n.prototype=new r,n.extend=R.inherits,e&&R.extend(n.prototype,e),n.__super__=t.prototype,n},_deprecated:function(e,t,n,r){void 0!==t&&console.warn(e+': "'+n+'" is deprecated. Please use "'+r+'" instead')}},N=R;R.callCallback=R.callback,R.indexOf=function(e,t,n){return Array.prototype.indexOf.call(e,t,n)},R.getValueOrDefault=R.valueOrDefault,R.getValueAtIndexOrDefault=R.valueAtIndexOrDefault;var $={linear:function(e){return e},easeInQuad:function(e){return e*e},easeOutQuad:function(e){return-e*(e-2)},easeInOutQuad:function(e){return(e/=.5)<1?.5*e*e:-.5*(--e*(e-2)-1)},easeInCubic:function(e){return e*e*e},easeOutCubic:function(e){return(e-=1)*e*e+1},easeInOutCubic:function(e){return(e/=.5)<1?.5*e*e*e:.5*((e-=2)*e*e+2)},easeInQuart:function(e){return e*e*e*e},easeOutQuart:function(e){return-((e-=1)*e*e*e-1)},easeInOutQuart:function(e){return(e/=.5)<1?.5*e*e*e*e:-.5*((e-=2)*e*e*e-2)},easeInQuint:function(e){return e*e*e*e*e},easeOutQuint:function(e){return(e-=1)*e*e*e*e+1},easeInOutQuint:function(e){return(e/=.5)<1?.5*e*e*e*e*e:.5*((e-=2)*e*e*e*e+2)},easeInSine:function(e){return 1-Math.cos(e*(Math.PI/2))},easeOutSine:function(e){return Math.sin(e*(Math.PI/2))},easeInOutSine:function(e){return-.5*(Math.cos(Math.PI*e)-1)},easeInExpo:function(e){return 0===e?0:Math.pow(2,10*(e-1))},easeOutExpo:function(e){return 1===e?1:1-Math.pow(2,-10*e)},easeInOutExpo:function(e){return 0===e?0:1===e?1:(e/=.5)<1?.5*Math.pow(2,10*(e-1)):.5*(2-Math.pow(2,-10*--e))},easeInCirc:function(e){return e>=1?e:-(Math.sqrt(1-e*e)-1)},easeOutCirc:function(e){return Math.sqrt(1-(e-=1)*e)},easeInOutCirc:function(e){return(e/=.5)<1?-.5*(Math.sqrt(1-e*e)-1):.5*(Math.sqrt(1-(e-=2)*e)+1)},easeInElastic:function(e){var t=1.70158,n=0,r=1;return 0===e?0:1===e?1:(n||(n=.3),r<1?(r=1,t=n/4):t=n/(2*Math.PI)*Math.asin(1/r),-r*Math.pow(2,10*(e-=1))*Math.sin((e-t)*(2*Math.PI)/n))},easeOutElastic:function(e){var t=1.70158,n=0,r=1;return 0===e?0:1===e?1:(n||(n=.3),r<1?(r=1,t=n/4):t=n/(2*Math.PI)*Math.asin(1/r),r*Math.pow(2,-10*e)*Math.sin((e-t)*(2*Math.PI)/n)+1)},easeInOutElastic:function(e){var t=1.70158,n=0,r=1;return 0===e?0:2===(e/=.5)?1:(n||(n=.45),r<1?(r=1,t=n/4):t=n/(2*Math.PI)*Math.asin(1/r),e<1?r*Math.pow(2,10*(e-=1))*Math.sin((e-t)*(2*Math.PI)/n)*-.5:r*Math.pow(2,-10*(e-=1))*Math.sin((e-t)*(2*Math.PI)/n)*.5+1)},easeInBack:function(e){var t=1.70158;return e*e*((t+1)*e-t)},easeOutBack:function(e){var t=1.70158;return(e-=1)*e*((t+1)*e+t)+1},easeInOutBack:function(e){var t=1.70158;return(e/=.5)<1?e*e*((1+(t*=1.525))*e-t)*.5:.5*((e-=2)*e*((1+(t*=1.525))*e+t)+2)},easeInBounce:function(e){return 1-$.easeOutBounce(1-e)},easeOutBounce:function(e){return e<1/2.75?7.5625*e*e:e<2/2.75?7.5625*(e-=1.5/2.75)*e+.75:e<2.5/2.75?7.5625*(e-=2.25/2.75)*e+.9375:7.5625*(e-=2.625/2.75)*e+.984375},easeInOutBounce:function(e){return e<.5?.5*$.easeInBounce(2*e):.5*$.easeOutBounce(2*e-1)+.5}},z={effects:$};N.easingEffects=$;var B=Math.PI,W=B/180,U=2*B,V=B/2,q=B/4,G=2*B/3,J={clear:function(e){e.ctx.clearRect(0,0,e.width,e.height)},roundedRect:function(e,t,n,r,i,a){if(a){var o=Math.min(a,i/2,r/2),s=t+o,l=n+o,u=t+r-o,c=n+i-o;e.moveTo(t,l),s<u&&l<c?(e.arc(s,l,o,-B,-V),e.arc(u,l,o,-V,0),e.arc(u,c,o,0,V),e.arc(s,c,o,V,B)):s<u?(e.moveTo(s,n),e.arc(u,l,o,-V,V),e.arc(s,l,o,V,B+V)):l<c?(e.arc(s,l,o,-B,0),e.arc(s,c,o,0,B)):e.arc(s,l,o,-B,B),e.closePath(),e.moveTo(t,n)}else e.rect(t,n,r,i)},drawPoint:function(e,t,n,r,i,a){var o,s,l,u,c,d=(a||0)*W;if(t&&"object"===typeof t&&(o=t.toString(),"[object HTMLImageElement]"===o||"[object HTMLCanvasElement]"===o))return e.save(),e.translate(r,i),e.rotate(d),e.drawImage(t,-t.width/2,-t.height/2,t.width,t.height),void e.restore();if(!(isNaN(n)||n<=0)){switch(e.beginPath(),t){default:e.arc(r,i,n,0,U),e.closePath();break;case"triangle":e.moveTo(r+Math.sin(d)*n,i-Math.cos(d)*n),d+=G,e.lineTo(r+Math.sin(d)*n,i-Math.cos(d)*n),d+=G,e.lineTo(r+Math.sin(d)*n,i-Math.cos(d)*n),e.closePath();break;case"rectRounded":c=.516*n,u=n-c,s=Math.cos(d+q)*u,l=Math.sin(d+q)*u,e.arc(r-s,i-l,c,d-B,d-V),e.arc(r+l,i-s,c,d-V,d),e.arc(r+s,i+l,c,d,d+V),e.arc(r-l,i+s,c,d+V,d+B),e.closePath();break;case"rect":if(!a){u=Math.SQRT1_2*n,e.rect(r-u,i-u,2*u,2*u);break}d+=q;case"rectRot":s=Math.cos(d)*n,l=Math.sin(d)*n,e.moveTo(r-s,i-l),e.lineTo(r+l,i-s),e.lineTo(r+s,i+l),e.lineTo(r-l,i+s),e.closePath();break;case"crossRot":d+=q;case"cross":s=Math.cos(d)*n,l=Math.sin(d)*n,e.moveTo(r-s,i-l),e.lineTo(r+s,i+l),e.moveTo(r+l,i-s),e.lineTo(r-l,i+s);break;case"star":s=Math.cos(d)*n,l=Math.sin(d)*n,e.moveTo(r-s,i-l),e.lineTo(r+s,i+l),e.moveTo(r+l,i-s),e.lineTo(r-l,i+s),d+=q,s=Math.cos(d)*n,l=Math.sin(d)*n,e.moveTo(r-s,i-l),e.lineTo(r+s,i+l),e.moveTo(r+l,i-s),e.lineTo(r-l,i+s);break;case"line":s=Math.cos(d)*n,l=Math.sin(d)*n,e.moveTo(r-s,i-l),e.lineTo(r+s,i+l);break;case"dash":e.moveTo(r,i),e.lineTo(r+Math.cos(d)*n,i+Math.sin(d)*n);break}e.fill(),e.stroke()}},_isPointInArea:function(e,t){var n=1e-6;return e.x>t.left-n&&e.x<t.right+n&&e.y>t.top-n&&e.y<t.bottom+n},clipArea:function(e,t){e.save(),e.beginPath(),e.rect(t.left,t.top,t.right-t.left,t.bottom-t.top),e.clip()},unclipArea:function(e){e.restore()},lineTo:function(e,t,n,r){var i=n.steppedLine;if(i){if("middle"===i){var a=(t.x+n.x)/2;e.lineTo(a,r?n.y:t.y),e.lineTo(a,r?t.y:n.y)}else"after"===i&&!r||"after"!==i&&r?e.lineTo(t.x,n.y):e.lineTo(n.x,t.y);e.lineTo(n.x,n.y)}else n.tension?e.bezierCurveTo(r?t.controlPointPreviousX:t.controlPointNextX,r?t.controlPointPreviousY:t.controlPointNextY,r?n.controlPointNextX:n.controlPointPreviousX,r?n.controlPointNextY:n.controlPointPreviousY,n.x,n.y):e.lineTo(n.x,n.y)}},K=J;N.clear=J.clear,N.drawRoundedRectangle=function(e){e.beginPath(),J.roundedRect.apply(J,arguments)};var X={_set:function(e,t){return N.merge(this[e]||(this[e]={}),t)}};X._set("global",{defaultColor:"rgba(0,0,0,0.1)",defaultFontColor:"#666",defaultFontFamily:"'Helvetica Neue', 'Helvetica', 'Arial', sans-serif",defaultFontSize:12,defaultFontStyle:"normal",defaultLineHeight:1.2,showLines:!0});var Q=X,Z=N.valueOrDefault;function ee(e){return!e||N.isNullOrUndef(e.size)||N.isNullOrUndef(e.family)?null:(e.style?e.style+" ":"")+(e.weight?e.weight+" ":"")+e.size+"px "+e.family}var te={toLineHeight:function(e,t){var n=(""+e).match(/^(normal|(\d+(?:\.\d+)?)(px|em|%)?)$/);if(!n||"normal"===n[1])return 1.2*t;switch(e=+n[2],n[3]){case"px":return e;case"%":e/=100;break}return t*e},toPadding:function(e){var t,n,r,i;return N.isObject(e)?(t=+e.top||0,n=+e.right||0,r=+e.bottom||0,i=+e.left||0):t=n=r=i=+e||0,{top:t,right:n,bottom:r,left:i,height:t+r,width:i+n}},_parseFont:function(e){var t=Q.global,n=Z(e.fontSize,t.defaultFontSize),r={family:Z(e.fontFamily,t.defaultFontFamily),lineHeight:N.options.toLineHeight(Z(e.lineHeight,t.defaultLineHeight),n),size:n,style:Z(e.fontStyle,t.defaultFontStyle),weight:null,string:""};return r.string=ee(r),r},resolve:function(e,t,n,r){var i,a,o,s=!0;for(i=0,a=e.length;i<a;++i)if(o=e[i],void 0!==o&&(void 0!==t&&"function"===typeof o&&(o=o(t),s=!1),void 0!==n&&N.isArray(o)&&(o=o[n],s=!1),void 0!==o))return r&&!s&&(r.cacheable=!1),o}},ne={_factorize:function(e){var t,n=[],r=Math.sqrt(e);for(t=1;t<r;t++)e%t===0&&(n.push(t),n.push(e/t));return r===(0|r)&&n.push(r),n.sort((function(e,t){return e-t})).pop(),n},log10:Math.log10||function(e){var t=Math.log(e)*Math.LOG10E,n=Math.round(t),r=e===Math.pow(10,n);return r?n:t}},re=ne;N.log10=ne.log10;var ie=function(e,t){return{x:function(n){return e+e+t-n},setWidth:function(e){t=e},textAlign:function(e){return"center"===e?e:"right"===e?"left":"right"},xPlus:function(e,t){return e-t},leftForLtr:function(e,t){return e-t}}},ae=function(){return{x:function(e){return e},setWidth:function(e){},textAlign:function(e){return e},xPlus:function(e,t){return e+t},leftForLtr:function(e,t){return e}}},oe=function(e,t,n){return e?ie(t,n):ae()},se=function(e,t){var n,r;"ltr"!==t&&"rtl"!==t||(n=e.canvas.style,r=[n.getPropertyValue("direction"),n.getPropertyPriority("direction")],n.setProperty("direction",t,"important"),e.prevTextDirection=r)},le=function(e){var t=e.prevTextDirection;void 0!==t&&(delete e.prevTextDirection,e.canvas.style.setProperty("direction",t[0],t[1]))},ue={getRtlAdapter:oe,overrideTextDirection:se,restoreTextDirection:le},ce=N,de=z,fe=K,he=te,pe=re,me=ue;function _e(e,t,n,r){var i,a,o,s,l,u,c,d,f,h=Object.keys(n);for(i=0,a=h.length;i<a;++i)if(o=h[i],u=n[o],t.hasOwnProperty(o)||(t[o]=u),s=t[o],s!==u&&"_"!==o[0]){if(e.hasOwnProperty(o)||(e[o]=s),l=e[o],c=typeof u,c===typeof l)if("string"===c){if(d=j(l),d.valid&&(f=j(u),f.valid)){t[o]=f.mix(d,r).rgbString();continue}}else if(ce.isFinite(l)&&ce.isFinite(u)){t[o]=l+(u-l)*r;continue}t[o]=u}}ce.easing=de,ce.canvas=fe,ce.options=he,ce.math=pe,ce.rtl=me;var ve=function(e){ce.extend(this,e),this.initialize.apply(this,arguments)};ce.extend(ve.prototype,{_type:void 0,initialize:function(){this.hidden=!1},pivot:function(){var e=this;return e._view||(e._view=ce.extend({},e._model)),e._start={},e},transition:function(e){var t=this,n=t._model,r=t._start,i=t._view;return n&&1!==e?(i||(i=t._view={}),r||(r=t._start={}),_e(r,i,n,e),t):(t._view=ce.extend({},n),t._start=null,t)},tooltipPosition:function(){return{x:this._model.x,y:this._model.y}},hasValue:function(){return ce.isNumber(this._model.x)&&ce.isNumber(this._model.y)}}),ve.extend=ce.inherits;var ge=ve,be=ge.extend({chart:null,currentStep:0,numSteps:60,easing:"",render:null,onAnimationProgress:null,onAnimationComplete:null}),ye=be;Object.defineProperty(be.prototype,"animationObject",{get:function(){return this}}),Object.defineProperty(be.prototype,"chartInstance",{get:function(){return this.chart},set:function(e){this.chart=e}}),Q._set("global",{animation:{duration:1e3,easing:"easeOutQuart",onProgress:ce.noop,onComplete:ce.noop}});var Me={animations:[],request:null,addAnimation:function(e,t,n,r){var i,a,o=this.animations;for(t.chart=e,t.startTime=Date.now(),t.duration=n,r||(e.animating=!0),i=0,a=o.length;i<a;++i)if(o[i].chart===e)return void(o[i]=t);o.push(t),1===o.length&&this.requestAnimationFrame()},cancelAnimation:function(e){var t=ce.findIndex(this.animations,(function(t){return t.chart===e}));-1!==t&&(this.animations.splice(t,1),e.animating=!1)},requestAnimationFrame:function(){var e=this;null===e.request&&(e.request=ce.requestAnimFrame.call(window,(function(){e.request=null,e.startDigest()})))},startDigest:function(){var e=this;e.advance(),e.animations.length>0&&e.requestAnimationFrame()},advance:function(){var e,t,n,r,i=this.animations,a=0;while(a<i.length)e=i[a],t=e.chart,n=e.numSteps,r=Math.floor((Date.now()-e.startTime)/e.duration*n)+1,e.currentStep=Math.min(r,n),ce.callback(e.render,[t,e],t),ce.callback(e.onAnimationProgress,[e],t),e.currentStep>=n?(ce.callback(e.onAnimationComplete,[e],t),t.animating=!1,i.splice(a,1)):++a}},ke=ce.options.resolve,we=["push","pop","shift","splice","unshift"];function Le(e,t){e._chartjs?e._chartjs.listeners.push(t):(Object.defineProperty(e,"_chartjs",{configurable:!0,enumerable:!1,value:{listeners:[t]}}),we.forEach((function(t){var n="onData"+t.charAt(0).toUpperCase()+t.slice(1),r=e[t];Object.defineProperty(e,t,{configurable:!0,enumerable:!1,value:function(){var t=Array.prototype.slice.call(arguments),i=r.apply(this,t);return ce.each(e._chartjs.listeners,(function(e){"function"===typeof e[n]&&e[n].apply(e,t)})),i}})})))}function xe(e,t){var n=e._chartjs;if(n){var r=n.listeners,i=r.indexOf(t);-1!==i&&r.splice(i,1),r.length>0||(we.forEach((function(t){delete e[t]})),delete e._chartjs)}}var Se=function(e,t){this.initialize(e,t)};ce.extend(Se.prototype,{datasetElementType:null,dataElementType:null,_datasetElementOptions:["backgroundColor","borderCapStyle","borderColor","borderDash","borderDashOffset","borderJoinStyle","borderWidth"],_dataElementOptions:["backgroundColor","borderColor","borderWidth","pointStyle"],initialize:function(e,t){var n=this;n.chart=e,n.index=t,n.linkScales(),n.addElements(),n._type=n.getMeta().type},updateIndex:function(e){this.index=e},linkScales:function(){var e=this,t=e.getMeta(),n=e.chart,r=n.scales,i=e.getDataset(),a=n.options.scales;null!==t.xAxisID&&t.xAxisID in r&&!i.xAxisID||(t.xAxisID=i.xAxisID||a.xAxes[0].id),null!==t.yAxisID&&t.yAxisID in r&&!i.yAxisID||(t.yAxisID=i.yAxisID||a.yAxes[0].id)},getDataset:function(){return this.chart.data.datasets[this.index]},getMeta:function(){return this.chart.getDatasetMeta(this.index)},getScaleForId:function(e){return this.chart.scales[e]},_getValueScaleId:function(){return this.getMeta().yAxisID},_getIndexScaleId:function(){return this.getMeta().xAxisID},_getValueScale:function(){return this.getScaleForId(this._getValueScaleId())},_getIndexScale:function(){return this.getScaleForId(this._getIndexScaleId())},reset:function(){this._update(!0)},destroy:function(){this._data&&xe(this._data,this)},createMetaDataset:function(){var e=this,t=e.datasetElementType;return t&&new t({_chart:e.chart,_datasetIndex:e.index})},createMetaData:function(e){var t=this,n=t.dataElementType;return n&&new n({_chart:t.chart,_datasetIndex:t.index,_index:e})},addElements:function(){var e,t,n=this,r=n.getMeta(),i=n.getDataset().data||[],a=r.data;for(e=0,t=i.length;e<t;++e)a[e]=a[e]||n.createMetaData(e);r.dataset=r.dataset||n.createMetaDataset()},addElementAndReset:function(e){var t=this.createMetaData(e);this.getMeta().data.splice(e,0,t),this.updateElement(t,e,!0)},buildOrUpdateElements:function(){var e=this,t=e.getDataset(),n=t.data||(t.data=[]);e._data!==n&&(e._data&&xe(e._data,e),n&&Object.isExtensible(n)&&Le(n,e),e._data=n),e.resyncElements()},_configure:function(){var e=this;e._config=ce.merge(Object.create(null),[e.chart.options.datasets[e._type],e.getDataset()],{merger:function(e,t,n){"_meta"!==e&&"data"!==e&&ce._merger(e,t,n)}})},_update:function(e){var t=this;t._configure(),t._cachedDataOpts=null,t.update(e)},update:ce.noop,transition:function(e){for(var t=this.getMeta(),n=t.data||[],r=n.length,i=0;i<r;++i)n[i].transition(e);t.dataset&&t.dataset.transition(e)},draw:function(){var e=this.getMeta(),t=e.data||[],n=t.length,r=0;for(e.dataset&&e.dataset.draw();r<n;++r)t[r].draw()},getStyle:function(e){var t,n=this,r=n.getMeta(),i=r.dataset;return n._configure(),i&&void 0===e?t=n._resolveDatasetElementOptions(i||{}):(e=e||0,t=n._resolveDataElementOptions(r.data[e]||{},e)),!1!==t.fill&&null!==t.fill||(t.backgroundColor=t.borderColor),t},_resolveDatasetElementOptions:function(e,t){var n,r,i,a,o=this,s=o.chart,l=o._config,u=e.custom||{},c=s.options.elements[o.datasetElementType.prototype._type]||{},d=o._datasetElementOptions,f={},h={chart:s,dataset:o.getDataset(),datasetIndex:o.index,hover:t};for(n=0,r=d.length;n<r;++n)i=d[n],a=t?"hover"+i.charAt(0).toUpperCase()+i.slice(1):i,f[i]=ke([u[a],l[a],c[a]],h);return f},_resolveDataElementOptions:function(e,t){var n=this,r=e&&e.custom,i=n._cachedDataOpts;if(i&&!r)return i;var a,o,s,l,u=n.chart,c=n._config,d=u.options.elements[n.dataElementType.prototype._type]||{},f=n._dataElementOptions,h={},p={chart:u,dataIndex:t,dataset:n.getDataset(),datasetIndex:n.index},m={cacheable:!r};if(r=r||{},ce.isArray(f))for(o=0,s=f.length;o<s;++o)l=f[o],h[l]=ke([r[l],c[l],d[l]],p,t,m);else for(a=Object.keys(f),o=0,s=a.length;o<s;++o)l=a[o],h[l]=ke([r[l],c[f[l]],c[l],d[l]],p,t,m);return m.cacheable&&(n._cachedDataOpts=Object.freeze(h)),h},removeHoverStyle:function(e){ce.merge(e._model,e.$previousStyle||{}),delete e.$previousStyle},setHoverStyle:function(e){var t=this.chart.data.datasets[e._datasetIndex],n=e._index,r=e.custom||{},i=e._model,a=ce.getHoverColor;e.$previousStyle={backgroundColor:i.backgroundColor,borderColor:i.borderColor,borderWidth:i.borderWidth},i.backgroundColor=ke([r.hoverBackgroundColor,t.hoverBackgroundColor,a(i.backgroundColor)],void 0,n),i.borderColor=ke([r.hoverBorderColor,t.hoverBorderColor,a(i.borderColor)],void 0,n),i.borderWidth=ke([r.hoverBorderWidth,t.hoverBorderWidth,i.borderWidth],void 0,n)},_removeDatasetHoverStyle:function(){var e=this.getMeta().dataset;e&&this.removeHoverStyle(e)},_setDatasetHoverStyle:function(){var e,t,n,r,i,a,o=this.getMeta().dataset,s={};if(o){for(a=o._model,i=this._resolveDatasetElementOptions(o,!0),r=Object.keys(i),e=0,t=r.length;e<t;++e)n=r[e],s[n]=a[n],a[n]=i[n];o.$previousStyle=s}},resyncElements:function(){var e=this,t=e.getMeta(),n=e.getDataset().data,r=t.data.length,i=n.length;i<r?t.data.splice(i,r-i):i>r&&e.insertElements(r,i-r)},insertElements:function(e,t){for(var n=0;n<t;++n)this.addElementAndReset(e+n)},onDataPush:function(){var e=arguments.length;this.insertElements(this.getDataset().data.length-e,e)},onDataPop:function(){this.getMeta().data.pop()},onDataShift:function(){this.getMeta().data.shift()},onDataSplice:function(e,t){this.getMeta().data.splice(e,t),this.insertElements(e,arguments.length-2)},onDataUnshift:function(){this.insertElements(0,arguments.length)}}),Se.extend=ce.inherits;var Te=Se,De=2*Math.PI;function Ye(e,t){var n=t.startAngle,r=t.endAngle,i=t.pixelMargin,a=i/t.outerRadius,o=t.x,s=t.y;e.beginPath(),e.arc(o,s,t.outerRadius,n-a,r+a),t.innerRadius>i?(a=i/t.innerRadius,e.arc(o,s,t.innerRadius-i,r+a,n-a,!0)):e.arc(o,s,i,r+Math.PI/2,n-Math.PI/2),e.closePath(),e.clip()}function Ce(e,t,n,r){var i,a=n.endAngle;for(r&&(n.endAngle=n.startAngle+De,Ye(e,n),n.endAngle=a,n.endAngle===n.startAngle&&n.fullCircles&&(n.endAngle+=De,n.fullCircles--)),e.beginPath(),e.arc(n.x,n.y,n.innerRadius,n.startAngle+De,n.startAngle,!0),i=0;i<n.fullCircles;++i)e.stroke();for(e.beginPath(),e.arc(n.x,n.y,t.outerRadius,n.startAngle,n.startAngle+De),i=0;i<n.fullCircles;++i)e.stroke()}function Ee(e,t,n){var r="inner"===t.borderAlign;r?(e.lineWidth=2*t.borderWidth,e.lineJoin="round"):(e.lineWidth=t.borderWidth,e.lineJoin="bevel"),n.fullCircles&&Ce(e,t,n,r),r&&Ye(e,n),e.beginPath(),e.arc(n.x,n.y,t.outerRadius,n.startAngle,n.endAngle),e.arc(n.x,n.y,n.innerRadius,n.endAngle,n.startAngle,!0),e.closePath(),e.stroke()}Q._set("global",{elements:{arc:{backgroundColor:Q.global.defaultColor,borderColor:"#fff",borderWidth:2,borderAlign:"center"}}});var Ae=ge.extend({_type:"arc",inLabelRange:function(e){var t=this._view;return!!t&&Math.pow(e-t.x,2)<Math.pow(t.radius+t.hoverRadius,2)},inRange:function(e,t){var n=this._view;if(n){var r=ce.getAngleFromPoint(n,{x:e,y:t}),i=r.angle,a=r.distance,o=n.startAngle,s=n.endAngle;while(s<o)s+=De;while(i>s)i-=De;while(i<o)i+=De;var l=i>=o&&i<=s,u=a>=n.innerRadius&&a<=n.outerRadius;return l&&u}return!1},getCenterPoint:function(){var e=this._view,t=(e.startAngle+e.endAngle)/2,n=(e.innerRadius+e.outerRadius)/2;return{x:e.x+Math.cos(t)*n,y:e.y+Math.sin(t)*n}},getArea:function(){var e=this._view;return Math.PI*((e.endAngle-e.startAngle)/(2*Math.PI))*(Math.pow(e.outerRadius,2)-Math.pow(e.innerRadius,2))},tooltipPosition:function(){var e=this._view,t=e.startAngle+(e.endAngle-e.startAngle)/2,n=(e.outerRadius-e.innerRadius)/2+e.innerRadius;return{x:e.x+Math.cos(t)*n,y:e.y+Math.sin(t)*n}},draw:function(){var e,t=this._chart.ctx,n=this._view,r="inner"===n.borderAlign?.33:0,i={x:n.x,y:n.y,innerRadius:n.innerRadius,outerRadius:Math.max(n.outerRadius-r,0),pixelMargin:r,startAngle:n.startAngle,endAngle:n.endAngle,fullCircles:Math.floor(n.circumference/De)};if(t.save(),t.fillStyle=n.backgroundColor,t.strokeStyle=n.borderColor,i.fullCircles){for(i.endAngle=i.startAngle+De,t.beginPath(),t.arc(i.x,i.y,i.outerRadius,i.startAngle,i.endAngle),t.arc(i.x,i.y,i.innerRadius,i.endAngle,i.startAngle,!0),t.closePath(),e=0;e<i.fullCircles;++e)t.fill();i.endAngle=i.startAngle+n.circumference%De}t.beginPath(),t.arc(i.x,i.y,i.outerRadius,i.startAngle,i.endAngle),t.arc(i.x,i.y,i.innerRadius,i.endAngle,i.startAngle,!0),t.closePath(),t.fill(),n.borderWidth&&Ee(t,n,i),t.restore()}}),Oe=ce.valueOrDefault,Pe=Q.global.defaultColor;Q._set("global",{elements:{line:{tension:.4,backgroundColor:Pe,borderWidth:3,borderColor:Pe,borderCapStyle:"butt",borderDash:[],borderDashOffset:0,borderJoinStyle:"miter",capBezierPoints:!0,fill:!0}}});var Fe=ge.extend({_type:"line",draw:function(){var e,t,n,r=this,i=r._view,a=r._chart.ctx,o=i.spanGaps,s=r._children.slice(),l=Q.global,u=l.elements.line,c=-1,d=r._loop;if(s.length){if(r._loop){for(e=0;e<s.length;++e)if(t=ce.previousItem(s,e),!s[e]._view.skip&&t._view.skip){s=s.slice(e).concat(s.slice(0,e)),d=o;break}d&&s.push(s[0])}for(a.save(),a.lineCap=i.borderCapStyle||u.borderCapStyle,a.setLineDash&&a.setLineDash(i.borderDash||u.borderDash),a.lineDashOffset=Oe(i.borderDashOffset,u.borderDashOffset),a.lineJoin=i.borderJoinStyle||u.borderJoinStyle,a.lineWidth=Oe(i.borderWidth,u.borderWidth),a.strokeStyle=i.borderColor||l.defaultColor,a.beginPath(),n=s[0]._view,n.skip||(a.moveTo(n.x,n.y),c=0),e=1;e<s.length;++e)n=s[e]._view,t=-1===c?ce.previousItem(s,e):s[c],n.skip||(c!==e-1&&!o||-1===c?a.moveTo(n.x,n.y):ce.canvas.lineTo(a,t._view,n),c=e);d&&a.closePath(),a.stroke(),a.restore()}}}),Ie=ce.valueOrDefault,je=Q.global.defaultColor;function He(e){var t=this._view;return!!t&&Math.abs(e-t.x)<t.radius+t.hitRadius}function Re(e){var t=this._view;return!!t&&Math.abs(e-t.y)<t.radius+t.hitRadius}Q._set("global",{elements:{point:{radius:3,pointStyle:"circle",backgroundColor:je,borderColor:je,borderWidth:1,hitRadius:1,hoverRadius:4,hoverBorderWidth:1}}});var Ne=ge.extend({_type:"point",inRange:function(e,t){var n=this._view;return!!n&&Math.pow(e-n.x,2)+Math.pow(t-n.y,2)<Math.pow(n.hitRadius+n.radius,2)},inLabelRange:He,inXRange:He,inYRange:Re,getCenterPoint:function(){var e=this._view;return{x:e.x,y:e.y}},getArea:function(){return Math.PI*Math.pow(this._view.radius,2)},tooltipPosition:function(){var e=this._view;return{x:e.x,y:e.y,padding:e.radius+e.borderWidth}},draw:function(e){var t=this._view,n=this._chart.ctx,r=t.pointStyle,i=t.rotation,a=t.radius,o=t.x,s=t.y,l=Q.global,u=l.defaultColor;t.skip||(void 0===e||ce.canvas._isPointInArea(t,e))&&(n.strokeStyle=t.borderColor||u,n.lineWidth=Ie(t.borderWidth,l.elements.point.borderWidth),n.fillStyle=t.backgroundColor||u,ce.canvas.drawPoint(n,r,a,o,s,i))}}),$e=Q.global.defaultColor;function ze(e){return e&&void 0!==e.width}function Be(e){var t,n,r,i,a;return ze(e)?(a=e.width/2,t=e.x-a,n=e.x+a,r=Math.min(e.y,e.base),i=Math.max(e.y,e.base)):(a=e.height/2,t=Math.min(e.x,e.base),n=Math.max(e.x,e.base),r=e.y-a,i=e.y+a),{left:t,top:r,right:n,bottom:i}}function We(e,t,n){return e===t?n:e===n?t:e}function Ue(e){var t=e.borderSkipped,n={};return t?(e.horizontal?e.base>e.x&&(t=We(t,"left","right")):e.base<e.y&&(t=We(t,"bottom","top")),n[t]=!0,n):n}function Ve(e,t,n){var r,i,a,o,s=e.borderWidth,l=Ue(e);return ce.isObject(s)?(r=+s.top||0,i=+s.right||0,a=+s.bottom||0,o=+s.left||0):r=i=a=o=+s||0,{t:l.top||r<0?0:r>n?n:r,r:l.right||i<0?0:i>t?t:i,b:l.bottom||a<0?0:a>n?n:a,l:l.left||o<0?0:o>t?t:o}}function qe(e){var t=Be(e),n=t.right-t.left,r=t.bottom-t.top,i=Ve(e,n/2,r/2);return{outer:{x:t.left,y:t.top,w:n,h:r},inner:{x:t.left+i.l,y:t.top+i.t,w:n-i.l-i.r,h:r-i.t-i.b}}}function Ge(e,t,n){var r=null===t,i=null===n,a=!(!e||r&&i)&&Be(e);return a&&(r||t>=a.left&&t<=a.right)&&(i||n>=a.top&&n<=a.bottom)}Q._set("global",{elements:{rectangle:{backgroundColor:$e,borderColor:$e,borderSkipped:"bottom",borderWidth:0}}});var Je=ge.extend({_type:"rectangle",draw:function(){var e=this._chart.ctx,t=this._view,n=qe(t),r=n.outer,i=n.inner;e.fillStyle=t.backgroundColor,e.fillRect(r.x,r.y,r.w,r.h),r.w===i.w&&r.h===i.h||(e.save(),e.beginPath(),e.rect(r.x,r.y,r.w,r.h),e.clip(),e.fillStyle=t.borderColor,e.rect(i.x,i.y,i.w,i.h),e.fill("evenodd"),e.restore())},height:function(){var e=this._view;return e.base-e.y},inRange:function(e,t){return Ge(this._view,e,t)},inLabelRange:function(e,t){var n=this._view;return ze(n)?Ge(n,e,null):Ge(n,null,t)},inXRange:function(e){return Ge(this._view,e,null)},inYRange:function(e){return Ge(this._view,null,e)},getCenterPoint:function(){var e,t,n=this._view;return ze(n)?(e=n.x,t=(n.y+n.base)/2):(e=(n.x+n.base)/2,t=n.y),{x:e,y:t}},getArea:function(){var e=this._view;return ze(e)?e.width*Math.abs(e.y-e.base):e.height*Math.abs(e.x-e.base)},tooltipPosition:function(){var e=this._view;return{x:e.x,y:e.y}}}),Ke={},Xe=Ae,Qe=Fe,Ze=Ne,et=Je;Ke.Arc=Xe,Ke.Line=Qe,Ke.Point=Ze,Ke.Rectangle=et;var tt=ce._deprecated,nt=ce.valueOrDefault;function rt(e,t){var n,r,i,a,o=e._length;for(i=1,a=t.length;i<a;++i)o=Math.min(o,Math.abs(t[i]-t[i-1]));for(i=0,a=e.getTicks().length;i<a;++i)r=e.getPixelForTick(i),o=i>0?Math.min(o,Math.abs(r-n)):o,n=r;return o}function it(e,t,n){var r,i,a=n.barThickness,o=t.stackCount,s=t.pixels[e],l=ce.isNullOrUndef(a)?rt(t.scale,t.pixels):-1;return ce.isNullOrUndef(a)?(r=l*n.categoryPercentage,i=n.barPercentage):(r=a*o,i=1),{chunk:r/o,ratio:i,start:s-r/2}}function at(e,t,n){var r,i,a=t.pixels,o=a[e],s=e>0?a[e-1]:null,l=e<a.length-1?a[e+1]:null,u=n.categoryPercentage;return null===s&&(s=o-(null===l?t.end-t.start:l-o)),null===l&&(l=o+o-s),r=o-(o-Math.min(s,l))/2*u,i=Math.abs(l-s)/2*u,{chunk:i/t.stackCount,ratio:n.barPercentage,start:r}}Q._set("bar",{hover:{mode:"label"},scales:{xAxes:[{type:"category",offset:!0,gridLines:{offsetGridLines:!0}}],yAxes:[{type:"linear"}]}}),Q._set("global",{datasets:{bar:{categoryPercentage:.8,barPercentage:.9}}});var ot=Te.extend({dataElementType:Ke.Rectangle,_dataElementOptions:["backgroundColor","borderColor","borderSkipped","borderWidth","barPercentage","barThickness","categoryPercentage","maxBarThickness","minBarLength"],initialize:function(){var e,t,n=this;Te.prototype.initialize.apply(n,arguments),e=n.getMeta(),e.stack=n.getDataset().stack,e.bar=!0,t=n._getIndexScale().options,tt("bar chart",t.barPercentage,"scales.[x/y]Axes.barPercentage","dataset.barPercentage"),tt("bar chart",t.barThickness,"scales.[x/y]Axes.barThickness","dataset.barThickness"),tt("bar chart",t.categoryPercentage,"scales.[x/y]Axes.categoryPercentage","dataset.categoryPercentage"),tt("bar chart",n._getValueScale().options.minBarLength,"scales.[x/y]Axes.minBarLength","dataset.minBarLength"),tt("bar chart",t.maxBarThickness,"scales.[x/y]Axes.maxBarThickness","dataset.maxBarThickness")},update:function(e){var t,n,r=this,i=r.getMeta().data;for(r._ruler=r.getRuler(),t=0,n=i.length;t<n;++t)r.updateElement(i[t],t,e)},updateElement:function(e,t,n){var r=this,i=r.getMeta(),a=r.getDataset(),o=r._resolveDataElementOptions(e,t);e._xScale=r.getScaleForId(i.xAxisID),e._yScale=r.getScaleForId(i.yAxisID),e._datasetIndex=r.index,e._index=t,e._model={backgroundColor:o.backgroundColor,borderColor:o.borderColor,borderSkipped:o.borderSkipped,borderWidth:o.borderWidth,datasetLabel:a.label,label:r.chart.data.labels[t]},ce.isArray(a.data[t])&&(e._model.borderSkipped=null),r._updateElementGeometry(e,t,n,o),e.pivot()},_updateElementGeometry:function(e,t,n,r){var i=this,a=e._model,o=i._getValueScale(),s=o.getBasePixel(),l=o.isHorizontal(),u=i._ruler||i.getRuler(),c=i.calculateBarValuePixels(i.index,t,r),d=i.calculateBarIndexPixels(i.index,t,u,r);a.horizontal=l,a.base=n?s:c.base,a.x=l?n?s:c.head:d.center,a.y=l?d.center:n?s:c.head,a.height=l?d.size:void 0,a.width=l?void 0:d.size},_getStacks:function(e){var t,n,r=this,i=r._getIndexScale(),a=i._getMatchingVisibleMetas(r._type),o=i.options.stacked,s=a.length,l=[];for(t=0;t<s;++t)if(n=a[t],(!1===o||-1===l.indexOf(n.stack)||void 0===o&&void 0===n.stack)&&l.push(n.stack),n.index===e)break;return l},getStackCount:function(){return this._getStacks().length},getStackIndex:function(e,t){var n=this._getStacks(e),r=void 0!==t?n.indexOf(t):-1;return-1===r?n.length-1:r},getRuler:function(){var e,t,n=this,r=n._getIndexScale(),i=[];for(e=0,t=n.getMeta().data.length;e<t;++e)i.push(r.getPixelForValue(null,e,n.index));return{pixels:i,start:r._startPixel,end:r._endPixel,stackCount:n.getStackCount(),scale:r}},calculateBarValuePixels:function(e,t,n){var r,i,a,o,s,l,u,c=this,d=c.chart,f=c._getValueScale(),h=f.isHorizontal(),p=d.data.datasets,m=f._getMatchingVisibleMetas(c._type),_=f._parseValue(p[e].data[t]),v=n.minBarLength,g=f.options.stacked,b=c.getMeta().stack,y=void 0===_.start?0:_.max>=0&&_.min>=0?_.min:_.max,M=void 0===_.start?_.end:_.max>=0&&_.min>=0?_.max-_.min:_.min-_.max,k=m.length;if(g||void 0===g&&void 0!==b)for(r=0;r<k;++r){if(i=m[r],i.index===e)break;i.stack===b&&(u=f._parseValue(p[i.index].data[t]),a=void 0===u.start?u.end:u.min>=0&&u.max>=0?u.max:u.min,(_.min<0&&a<0||_.max>=0&&a>0)&&(y+=a))}return o=f.getPixelForValue(y),s=f.getPixelForValue(y+M),l=s-o,void 0!==v&&Math.abs(l)<v&&(l=v,s=M>=0&&!h||M<0&&h?o-v:o+v),{size:l,base:o,head:s,center:s+l/2}},calculateBarIndexPixels:function(e,t,n,r){var i=this,a="flex"===r.barThickness?at(t,n,r):it(t,n,r),o=i.getStackIndex(e,i.getMeta().stack),s=a.start+a.chunk*o+a.chunk/2,l=Math.min(nt(r.maxBarThickness,1/0),a.chunk*a.ratio);return{base:s-l/2,head:s+l/2,center:s,size:l}},draw:function(){var e=this,t=e.chart,n=e._getValueScale(),r=e.getMeta().data,i=e.getDataset(),a=r.length,o=0;for(ce.canvas.clipArea(t.ctx,t.chartArea);o<a;++o){var s=n._parseValue(i.data[o]);isNaN(s.min)||isNaN(s.max)||r[o].draw()}ce.canvas.unclipArea(t.ctx)},_resolveDataElementOptions:function(){var e=this,t=ce.extend({},Te.prototype._resolveDataElementOptions.apply(e,arguments)),n=e._getIndexScale().options,r=e._getValueScale().options;return t.barPercentage=nt(n.barPercentage,t.barPercentage),t.barThickness=nt(n.barThickness,t.barThickness),t.categoryPercentage=nt(n.categoryPercentage,t.categoryPercentage),t.maxBarThickness=nt(n.maxBarThickness,t.maxBarThickness),t.minBarLength=nt(r.minBarLength,t.minBarLength),t}}),st=ce.valueOrDefault,lt=ce.options.resolve;Q._set("bubble",{hover:{mode:"single"},scales:{xAxes:[{type:"linear",position:"bottom",id:"x-axis-0"}],yAxes:[{type:"linear",position:"left",id:"y-axis-0"}]},tooltips:{callbacks:{title:function(){return""},label:function(e,t){var n=t.datasets[e.datasetIndex].label||"",r=t.datasets[e.datasetIndex].data[e.index];return n+": ("+e.xLabel+", "+e.yLabel+", "+r.r+")"}}}});var ut=Te.extend({dataElementType:Ke.Point,_dataElementOptions:["backgroundColor","borderColor","borderWidth","hoverBackgroundColor","hoverBorderColor","hoverBorderWidth","hoverRadius","hitRadius","pointStyle","rotation"],update:function(e){var t=this,n=t.getMeta(),r=n.data;ce.each(r,(function(n,r){t.updateElement(n,r,e)}))},updateElement:function(e,t,n){var r=this,i=r.getMeta(),a=e.custom||{},o=r.getScaleForId(i.xAxisID),s=r.getScaleForId(i.yAxisID),l=r._resolveDataElementOptions(e,t),u=r.getDataset().data[t],c=r.index,d=n?o.getPixelForDecimal(.5):o.getPixelForValue("object"===typeof u?u:NaN,t,c),f=n?s.getBasePixel():s.getPixelForValue(u,t,c);e._xScale=o,e._yScale=s,e._options=l,e._datasetIndex=c,e._index=t,e._model={backgroundColor:l.backgroundColor,borderColor:l.borderColor,borderWidth:l.borderWidth,hitRadius:l.hitRadius,pointStyle:l.pointStyle,rotation:l.rotation,radius:n?0:l.radius,skip:a.skip||isNaN(d)||isNaN(f),x:d,y:f},e.pivot()},setHoverStyle:function(e){var t=e._model,n=e._options,r=ce.getHoverColor;e.$previousStyle={backgroundColor:t.backgroundColor,borderColor:t.borderColor,borderWidth:t.borderWidth,radius:t.radius},t.backgroundColor=st(n.hoverBackgroundColor,r(n.backgroundColor)),t.borderColor=st(n.hoverBorderColor,r(n.borderColor)),t.borderWidth=st(n.hoverBorderWidth,n.borderWidth),t.radius=n.radius+n.hoverRadius},_resolveDataElementOptions:function(e,t){var n=this,r=n.chart,i=n.getDataset(),a=e.custom||{},o=i.data[t]||{},s=Te.prototype._resolveDataElementOptions.apply(n,arguments),l={chart:r,dataIndex:t,dataset:i,datasetIndex:n.index};return n._cachedDataOpts===s&&(s=ce.extend({},s)),s.radius=lt([a.radius,o.r,n._config.radius,r.options.elements.point.radius],l,t),s}}),ct=ce.valueOrDefault,dt=Math.PI,ft=2*dt,ht=dt/2;Q._set("doughnut",{animation:{animateRotate:!0,animateScale:!1},hover:{mode:"single"},legendCallback:function(e){var t,n,r,i,a=document.createElement("ul"),o=e.data,s=o.datasets,l=o.labels;if(a.setAttribute("class",e.id+"-legend"),s.length)for(t=0,n=s[0].data.length;t<n;++t)r=a.appendChild(document.createElement("li")),i=r.appendChild(document.createElement("span")),i.style.backgroundColor=s[0].backgroundColor[t],l[t]&&r.appendChild(document.createTextNode(l[t]));return a.outerHTML},legend:{labels:{generateLabels:function(e){var t=e.data;return t.labels.length&&t.datasets.length?t.labels.map((function(n,r){var i=e.getDatasetMeta(0),a=i.controller.getStyle(r);return{text:n,fillStyle:a.backgroundColor,strokeStyle:a.borderColor,lineWidth:a.borderWidth,hidden:isNaN(t.datasets[0].data[r])||i.data[r].hidden,index:r}})):[]}},onClick:function(e,t){var n,r,i,a=t.index,o=this.chart;for(n=0,r=(o.data.datasets||[]).length;n<r;++n)i=o.getDatasetMeta(n),i.data[a]&&(i.data[a].hidden=!i.data[a].hidden);o.update()}},cutoutPercentage:50,rotation:-ht,circumference:ft,tooltips:{callbacks:{title:function(){return""},label:function(e,t){var n=t.labels[e.index],r=": "+t.datasets[e.datasetIndex].data[e.index];return ce.isArray(n)?(n=n.slice(),n[0]+=r):n+=r,n}}}});var pt=Te.extend({dataElementType:Ke.Arc,linkScales:ce.noop,_dataElementOptions:["backgroundColor","borderColor","borderWidth","borderAlign","hoverBackgroundColor","hoverBorderColor","hoverBorderWidth"],getRingIndex:function(e){for(var t=0,n=0;n<e;++n)this.chart.isDatasetVisible(n)&&++t;return t},update:function(e){var t,n,r,i,a=this,o=a.chart,s=o.chartArea,l=o.options,u=1,c=1,d=0,f=0,h=a.getMeta(),p=h.data,m=l.cutoutPercentage/100||0,_=l.circumference,v=a._getRingWeight(a.index);if(_<ft){var g=l.rotation%ft;g+=g>=dt?-ft:g<-dt?ft:0;var b=g+_,y=Math.cos(g),M=Math.sin(g),k=Math.cos(b),w=Math.sin(b),L=g<=0&&b>=0||b>=ft,x=g<=ht&&b>=ht||b>=ft+ht,S=g===-dt||b>=dt,T=g<=-ht&&b>=-ht||b>=dt+ht,D=S?-1:Math.min(y,y*m,k,k*m),Y=T?-1:Math.min(M,M*m,w,w*m),C=L?1:Math.max(y,y*m,k,k*m),E=x?1:Math.max(M,M*m,w,w*m);u=(C-D)/2,c=(E-Y)/2,d=-(C+D)/2,f=-(E+Y)/2}for(r=0,i=p.length;r<i;++r)p[r]._options=a._resolveDataElementOptions(p[r],r);for(o.borderWidth=a.getMaxBorderWidth(),t=(s.right-s.left-o.borderWidth)/u,n=(s.bottom-s.top-o.borderWidth)/c,o.outerRadius=Math.max(Math.min(t,n)/2,0),o.innerRadius=Math.max(o.outerRadius*m,0),o.radiusLength=(o.outerRadius-o.innerRadius)/(a._getVisibleDatasetWeightTotal()||1),o.offsetX=d*o.outerRadius,o.offsetY=f*o.outerRadius,h.total=a.calculateTotal(),a.outerRadius=o.outerRadius-o.radiusLength*a._getRingWeightOffset(a.index),a.innerRadius=Math.max(a.outerRadius-o.radiusLength*v,0),r=0,i=p.length;r<i;++r)a.updateElement(p[r],r,e)},updateElement:function(e,t,n){var r=this,i=r.chart,a=i.chartArea,o=i.options,s=o.animation,l=(a.left+a.right)/2,u=(a.top+a.bottom)/2,c=o.rotation,d=o.rotation,f=r.getDataset(),h=n&&s.animateRotate||e.hidden?0:r.calculateCircumference(f.data[t])*(o.circumference/ft),p=n&&s.animateScale?0:r.innerRadius,m=n&&s.animateScale?0:r.outerRadius,_=e._options||{};ce.extend(e,{_datasetIndex:r.index,_index:t,_model:{backgroundColor:_.backgroundColor,borderColor:_.borderColor,borderWidth:_.borderWidth,borderAlign:_.borderAlign,x:l+i.offsetX,y:u+i.offsetY,startAngle:c,endAngle:d,circumference:h,outerRadius:m,innerRadius:p,label:ce.valueAtIndexOrDefault(f.label,t,i.data.labels[t])}});var v=e._model;n&&s.animateRotate||(v.startAngle=0===t?o.rotation:r.getMeta().data[t-1]._model.endAngle,v.endAngle=v.startAngle+v.circumference),e.pivot()},calculateTotal:function(){var e,t=this.getDataset(),n=this.getMeta(),r=0;return ce.each(n.data,(function(n,i){e=t.data[i],isNaN(e)||n.hidden||(r+=Math.abs(e))})),r},calculateCircumference:function(e){var t=this.getMeta().total;return t>0&&!isNaN(e)?ft*(Math.abs(e)/t):0},getMaxBorderWidth:function(e){var t,n,r,i,a,o,s,l,u=this,c=0,d=u.chart;if(!e)for(t=0,n=d.data.datasets.length;t<n;++t)if(d.isDatasetVisible(t)){r=d.getDatasetMeta(t),e=r.data,t!==u.index&&(a=r.controller);break}if(!e)return 0;for(t=0,n=e.length;t<n;++t)i=e[t],a?(a._configure(),o=a._resolveDataElementOptions(i,t)):o=i._options,"inner"!==o.borderAlign&&(s=o.borderWidth,l=o.hoverBorderWidth,c=s>c?s:c,c=l>c?l:c);return c},setHoverStyle:function(e){var t=e._model,n=e._options,r=ce.getHoverColor;e.$previousStyle={backgroundColor:t.backgroundColor,borderColor:t.borderColor,borderWidth:t.borderWidth},t.backgroundColor=ct(n.hoverBackgroundColor,r(n.backgroundColor)),t.borderColor=ct(n.hoverBorderColor,r(n.borderColor)),t.borderWidth=ct(n.hoverBorderWidth,n.borderWidth)},_getRingWeightOffset:function(e){for(var t=0,n=0;n<e;++n)this.chart.isDatasetVisible(n)&&(t+=this._getRingWeight(n));return t},_getRingWeight:function(e){return Math.max(ct(this.chart.data.datasets[e].weight,1),0)},_getVisibleDatasetWeightTotal:function(){return this._getRingWeightOffset(this.chart.data.datasets.length)}});Q._set("horizontalBar",{hover:{mode:"index",axis:"y"},scales:{xAxes:[{type:"linear",position:"bottom"}],yAxes:[{type:"category",position:"left",offset:!0,gridLines:{offsetGridLines:!0}}]},elements:{rectangle:{borderSkipped:"left"}},tooltips:{mode:"index",axis:"y"}}),Q._set("global",{datasets:{horizontalBar:{categoryPercentage:.8,barPercentage:.9}}});var mt=ot.extend({_getValueScaleId:function(){return this.getMeta().xAxisID},_getIndexScaleId:function(){return this.getMeta().yAxisID}}),_t=ce.valueOrDefault,vt=ce.options.resolve,gt=ce.canvas._isPointInArea;function bt(e,t){var n=e&&e.options.ticks||{},r=n.reverse,i=void 0===n.min?t:0,a=void 0===n.max?t:0;return{start:r?a:i,end:r?i:a}}function yt(e,t,n){var r=n/2,i=bt(e,r),a=bt(t,r);return{top:a.end,right:i.end,bottom:a.start,left:i.start}}function Mt(e){var t,n,r,i;return ce.isObject(e)?(t=e.top,n=e.right,r=e.bottom,i=e.left):t=n=r=i=e,{top:t,right:n,bottom:r,left:i}}Q._set("line",{showLines:!0,spanGaps:!1,hover:{mode:"label"},scales:{xAxes:[{type:"category",id:"x-axis-0"}],yAxes:[{type:"linear",id:"y-axis-0"}]}});var kt=Te.extend({datasetElementType:Ke.Line,dataElementType:Ke.Point,_datasetElementOptions:["backgroundColor","borderCapStyle","borderColor","borderDash","borderDashOffset","borderJoinStyle","borderWidth","cubicInterpolationMode","fill"],_dataElementOptions:{backgroundColor:"pointBackgroundColor",borderColor:"pointBorderColor",borderWidth:"pointBorderWidth",hitRadius:"pointHitRadius",hoverBackgroundColor:"pointHoverBackgroundColor",hoverBorderColor:"pointHoverBorderColor",hoverBorderWidth:"pointHoverBorderWidth",hoverRadius:"pointHoverRadius",pointStyle:"pointStyle",radius:"pointRadius",rotation:"pointRotation"},update:function(e){var t,n,r=this,i=r.getMeta(),a=i.dataset,o=i.data||[],s=r.chart.options,l=r._config,u=r._showLine=_t(l.showLine,s.showLines);for(r._xScale=r.getScaleForId(i.xAxisID),r._yScale=r.getScaleForId(i.yAxisID),u&&(void 0!==l.tension&&void 0===l.lineTension&&(l.lineTension=l.tension),a._scale=r._yScale,a._datasetIndex=r.index,a._children=o,a._model=r._resolveDatasetElementOptions(a),a.pivot()),t=0,n=o.length;t<n;++t)r.updateElement(o[t],t,e);for(u&&0!==a._model.tension&&r.updateBezierControlPoints(),t=0,n=o.length;t<n;++t)o[t].pivot()},updateElement:function(e,t,n){var r,i,a=this,o=a.getMeta(),s=e.custom||{},l=a.getDataset(),u=a.index,c=l.data[t],d=a._xScale,f=a._yScale,h=o.dataset._model,p=a._resolveDataElementOptions(e,t);r=d.getPixelForValue("object"===typeof c?c:NaN,t,u),i=n?f.getBasePixel():a.calculatePointY(c,t,u),e._xScale=d,e._yScale=f,e._options=p,e._datasetIndex=u,e._index=t,e._model={x:r,y:i,skip:s.skip||isNaN(r)||isNaN(i),radius:p.radius,pointStyle:p.pointStyle,rotation:p.rotation,backgroundColor:p.backgroundColor,borderColor:p.borderColor,borderWidth:p.borderWidth,tension:_t(s.tension,h?h.tension:0),steppedLine:!!h&&h.steppedLine,hitRadius:p.hitRadius}},_resolveDatasetElementOptions:function(e){var t=this,n=t._config,r=e.custom||{},i=t.chart.options,a=i.elements.line,o=Te.prototype._resolveDatasetElementOptions.apply(t,arguments);return o.spanGaps=_t(n.spanGaps,i.spanGaps),o.tension=_t(n.lineTension,a.tension),o.steppedLine=vt([r.steppedLine,n.steppedLine,a.stepped]),o.clip=Mt(_t(n.clip,yt(t._xScale,t._yScale,o.borderWidth))),o},calculatePointY:function(e,t,n){var r,i,a,o,s,l,u,c=this,d=c.chart,f=c._yScale,h=0,p=0;if(f.options.stacked){for(s=+f.getRightValue(e),l=d._getSortedVisibleDatasetMetas(),u=l.length,r=0;r<u;++r){if(a=l[r],a.index===n)break;i=d.data.datasets[a.index],"line"===a.type&&a.yAxisID===f.id&&(o=+f.getRightValue(i.data[t]),o<0?p+=o||0:h+=o||0)}return s<0?f.getPixelForValue(p+s):f.getPixelForValue(h+s)}return f.getPixelForValue(e)},updateBezierControlPoints:function(){var e,t,n,r,i=this,a=i.chart,o=i.getMeta(),s=o.dataset._model,l=a.chartArea,u=o.data||[];function c(e,t,n){return Math.max(Math.min(e,n),t)}if(s.spanGaps&&(u=u.filter((function(e){return!e._model.skip}))),"monotone"===s.cubicInterpolationMode)ce.splineCurveMonotone(u);else for(e=0,t=u.length;e<t;++e)n=u[e]._model,r=ce.splineCurve(ce.previousItem(u,e)._model,n,ce.nextItem(u,e)._model,s.tension),n.controlPointPreviousX=r.previous.x,n.controlPointPreviousY=r.previous.y,n.controlPointNextX=r.next.x,n.controlPointNextY=r.next.y;if(a.options.elements.line.capBezierPoints)for(e=0,t=u.length;e<t;++e)n=u[e]._model,gt(n,l)&&(e>0&&gt(u[e-1]._model,l)&&(n.controlPointPreviousX=c(n.controlPointPreviousX,l.left,l.right),n.controlPointPreviousY=c(n.controlPointPreviousY,l.top,l.bottom)),e<u.length-1&&gt(u[e+1]._model,l)&&(n.controlPointNextX=c(n.controlPointNextX,l.left,l.right),n.controlPointNextY=c(n.controlPointNextY,l.top,l.bottom)))},draw:function(){var e,t=this,n=t.chart,r=t.getMeta(),i=r.data||[],a=n.chartArea,o=n.canvas,s=0,l=i.length;for(t._showLine&&(e=r.dataset._model.clip,ce.canvas.clipArea(n.ctx,{left:!1===e.left?0:a.left-e.left,right:!1===e.right?o.width:a.right+e.right,top:!1===e.top?0:a.top-e.top,bottom:!1===e.bottom?o.height:a.bottom+e.bottom}),r.dataset.draw(),ce.canvas.unclipArea(n.ctx));s<l;++s)i[s].draw(a)},setHoverStyle:function(e){var t=e._model,n=e._options,r=ce.getHoverColor;e.$previousStyle={backgroundColor:t.backgroundColor,borderColor:t.borderColor,borderWidth:t.borderWidth,radius:t.radius},t.backgroundColor=_t(n.hoverBackgroundColor,r(n.backgroundColor)),t.borderColor=_t(n.hoverBorderColor,r(n.borderColor)),t.borderWidth=_t(n.hoverBorderWidth,n.borderWidth),t.radius=_t(n.hoverRadius,n.radius)}}),wt=ce.options.resolve;Q._set("polarArea",{scale:{type:"radialLinear",angleLines:{display:!1},gridLines:{circular:!0},pointLabels:{display:!1},ticks:{beginAtZero:!0}},animation:{animateRotate:!0,animateScale:!0},startAngle:-.5*Math.PI,legendCallback:function(e){var t,n,r,i,a=document.createElement("ul"),o=e.data,s=o.datasets,l=o.labels;if(a.setAttribute("class",e.id+"-legend"),s.length)for(t=0,n=s[0].data.length;t<n;++t)r=a.appendChild(document.createElement("li")),i=r.appendChild(document.createElement("span")),i.style.backgroundColor=s[0].backgroundColor[t],l[t]&&r.appendChild(document.createTextNode(l[t]));return a.outerHTML},legend:{labels:{generateLabels:function(e){var t=e.data;return t.labels.length&&t.datasets.length?t.labels.map((function(n,r){var i=e.getDatasetMeta(0),a=i.controller.getStyle(r);return{text:n,fillStyle:a.backgroundColor,strokeStyle:a.borderColor,lineWidth:a.borderWidth,hidden:isNaN(t.datasets[0].data[r])||i.data[r].hidden,index:r}})):[]}},onClick:function(e,t){var n,r,i,a=t.index,o=this.chart;for(n=0,r=(o.data.datasets||[]).length;n<r;++n)i=o.getDatasetMeta(n),i.data[a].hidden=!i.data[a].hidden;o.update()}},tooltips:{callbacks:{title:function(){return""},label:function(e,t){return t.labels[e.index]+": "+e.yLabel}}}});var Lt=Te.extend({dataElementType:Ke.Arc,linkScales:ce.noop,_dataElementOptions:["backgroundColor","borderColor","borderWidth","borderAlign","hoverBackgroundColor","hoverBorderColor","hoverBorderWidth"],_getIndexScaleId:function(){return this.chart.scale.id},_getValueScaleId:function(){return this.chart.scale.id},update:function(e){var t,n,r,i=this,a=i.getDataset(),o=i.getMeta(),s=i.chart.options.startAngle||0,l=i._starts=[],u=i._angles=[],c=o.data;for(i._updateRadius(),o.count=i.countVisibleElements(),t=0,n=a.data.length;t<n;t++)l[t]=s,r=i._computeAngle(t),u[t]=r,s+=r;for(t=0,n=c.length;t<n;++t)c[t]._options=i._resolveDataElementOptions(c[t],t),i.updateElement(c[t],t,e)},_updateRadius:function(){var e=this,t=e.chart,n=t.chartArea,r=t.options,i=Math.min(n.right-n.left,n.bottom-n.top);t.outerRadius=Math.max(i/2,0),t.innerRadius=Math.max(r.cutoutPercentage?t.outerRadius/100*r.cutoutPercentage:1,0),t.radiusLength=(t.outerRadius-t.innerRadius)/t.getVisibleDatasetCount(),e.outerRadius=t.outerRadius-t.radiusLength*e.index,e.innerRadius=e.outerRadius-t.radiusLength},updateElement:function(e,t,n){var r=this,i=r.chart,a=r.getDataset(),o=i.options,s=o.animation,l=i.scale,u=i.data.labels,c=l.xCenter,d=l.yCenter,f=o.startAngle,h=e.hidden?0:l.getDistanceFromCenterForValue(a.data[t]),p=r._starts[t],m=p+(e.hidden?0:r._angles[t]),_=s.animateScale?0:l.getDistanceFromCenterForValue(a.data[t]),v=e._options||{};ce.extend(e,{_datasetIndex:r.index,_index:t,_scale:l,_model:{backgroundColor:v.backgroundColor,borderColor:v.borderColor,borderWidth:v.borderWidth,borderAlign:v.borderAlign,x:c,y:d,innerRadius:0,outerRadius:n?_:h,startAngle:n&&s.animateRotate?f:p,endAngle:n&&s.animateRotate?f:m,label:ce.valueAtIndexOrDefault(u,t,u[t])}}),e.pivot()},countVisibleElements:function(){var e=this.getDataset(),t=this.getMeta(),n=0;return ce.each(t.data,(function(t,r){isNaN(e.data[r])||t.hidden||n++})),n},setHoverStyle:function(e){var t=e._model,n=e._options,r=ce.getHoverColor,i=ce.valueOrDefault;e.$previousStyle={backgroundColor:t.backgroundColor,borderColor:t.borderColor,borderWidth:t.borderWidth},t.backgroundColor=i(n.hoverBackgroundColor,r(n.backgroundColor)),t.borderColor=i(n.hoverBorderColor,r(n.borderColor)),t.borderWidth=i(n.hoverBorderWidth,n.borderWidth)},_computeAngle:function(e){var t=this,n=this.getMeta().count,r=t.getDataset(),i=t.getMeta();if(isNaN(r.data[e])||i.data[e].hidden)return 0;var a={chart:t.chart,dataIndex:e,dataset:r,datasetIndex:t.index};return wt([t.chart.options.elements.arc.angle,2*Math.PI/n],a,e)}});Q._set("pie",ce.clone(Q.doughnut)),Q._set("pie",{cutoutPercentage:0});var xt=pt,St=ce.valueOrDefault;Q._set("radar",{spanGaps:!1,scale:{type:"radialLinear"},elements:{line:{fill:"start",tension:0}}});var Tt=Te.extend({datasetElementType:Ke.Line,dataElementType:Ke.Point,linkScales:ce.noop,_datasetElementOptions:["backgroundColor","borderWidth","borderColor","borderCapStyle","borderDash","borderDashOffset","borderJoinStyle","fill"],_dataElementOptions:{backgroundColor:"pointBackgroundColor",borderColor:"pointBorderColor",borderWidth:"pointBorderWidth",hitRadius:"pointHitRadius",hoverBackgroundColor:"pointHoverBackgroundColor",hoverBorderColor:"pointHoverBorderColor",hoverBorderWidth:"pointHoverBorderWidth",hoverRadius:"pointHoverRadius",pointStyle:"pointStyle",radius:"pointRadius",rotation:"pointRotation"},_getIndexScaleId:function(){return this.chart.scale.id},_getValueScaleId:function(){return this.chart.scale.id},update:function(e){var t,n,r=this,i=r.getMeta(),a=i.dataset,o=i.data||[],s=r.chart.scale,l=r._config;for(void 0!==l.tension&&void 0===l.lineTension&&(l.lineTension=l.tension),a._scale=s,a._datasetIndex=r.index,a._children=o,a._loop=!0,a._model=r._resolveDatasetElementOptions(a),a.pivot(),t=0,n=o.length;t<n;++t)r.updateElement(o[t],t,e);for(r.updateBezierControlPoints(),t=0,n=o.length;t<n;++t)o[t].pivot()},updateElement:function(e,t,n){var r=this,i=e.custom||{},a=r.getDataset(),o=r.chart.scale,s=o.getPointPositionForValue(t,a.data[t]),l=r._resolveDataElementOptions(e,t),u=r.getMeta().dataset._model,c=n?o.xCenter:s.x,d=n?o.yCenter:s.y;e._scale=o,e._options=l,e._datasetIndex=r.index,e._index=t,e._model={x:c,y:d,skip:i.skip||isNaN(c)||isNaN(d),radius:l.radius,pointStyle:l.pointStyle,rotation:l.rotation,backgroundColor:l.backgroundColor,borderColor:l.borderColor,borderWidth:l.borderWidth,tension:St(i.tension,u?u.tension:0),hitRadius:l.hitRadius}},_resolveDatasetElementOptions:function(){var e=this,t=e._config,n=e.chart.options,r=Te.prototype._resolveDatasetElementOptions.apply(e,arguments);return r.spanGaps=St(t.spanGaps,n.spanGaps),r.tension=St(t.lineTension,n.elements.line.tension),r},updateBezierControlPoints:function(){var e,t,n,r,i=this,a=i.getMeta(),o=i.chart.chartArea,s=a.data||[];function l(e,t,n){return Math.max(Math.min(e,n),t)}for(a.dataset._model.spanGaps&&(s=s.filter((function(e){return!e._model.skip}))),e=0,t=s.length;e<t;++e)n=s[e]._model,r=ce.splineCurve(ce.previousItem(s,e,!0)._model,n,ce.nextItem(s,e,!0)._model,n.tension),n.controlPointPreviousX=l(r.previous.x,o.left,o.right),n.controlPointPreviousY=l(r.previous.y,o.top,o.bottom),n.controlPointNextX=l(r.next.x,o.left,o.right),n.controlPointNextY=l(r.next.y,o.top,o.bottom)},setHoverStyle:function(e){var t=e._model,n=e._options,r=ce.getHoverColor;e.$previousStyle={backgroundColor:t.backgroundColor,borderColor:t.borderColor,borderWidth:t.borderWidth,radius:t.radius},t.backgroundColor=St(n.hoverBackgroundColor,r(n.backgroundColor)),t.borderColor=St(n.hoverBorderColor,r(n.borderColor)),t.borderWidth=St(n.hoverBorderWidth,n.borderWidth),t.radius=St(n.hoverRadius,n.radius)}});Q._set("scatter",{hover:{mode:"single"},scales:{xAxes:[{id:"x-axis-1",type:"linear",position:"bottom"}],yAxes:[{id:"y-axis-1",type:"linear",position:"left"}]},tooltips:{callbacks:{title:function(){return""},label:function(e){return"("+e.xLabel+", "+e.yLabel+")"}}}}),Q._set("global",{datasets:{scatter:{showLine:!1}}});var Dt=kt,Yt={bar:ot,bubble:ut,doughnut:pt,horizontalBar:mt,line:kt,polarArea:Lt,pie:xt,radar:Tt,scatter:Dt};function Ct(e,t){return e.native?{x:e.x,y:e.y}:ce.getRelativePosition(e,t)}function Et(e,t){var n,r,i,a,o,s,l=e._getSortedVisibleDatasetMetas();for(r=0,a=l.length;r<a;++r)for(n=l[r].data,i=0,o=n.length;i<o;++i)s=n[i],s._view.skip||t(s)}function At(e,t){var n=[];return Et(e,(function(e){e.inRange(t.x,t.y)&&n.push(e)})),n}function Ot(e,t,n,r){var i=Number.POSITIVE_INFINITY,a=[];return Et(e,(function(e){if(!n||e.inRange(t.x,t.y)){var o=e.getCenterPoint(),s=r(t,o);s<i?(a=[e],i=s):s===i&&a.push(e)}})),a}function Pt(e){var t=-1!==e.indexOf("x"),n=-1!==e.indexOf("y");return function(e,r){var i=t?Math.abs(e.x-r.x):0,a=n?Math.abs(e.y-r.y):0;return Math.sqrt(Math.pow(i,2)+Math.pow(a,2))}}function Ft(e,t,n){var r=Ct(t,e);n.axis=n.axis||"x";var i=Pt(n.axis),a=n.intersect?At(e,r):Ot(e,r,!1,i),o=[];return a.length?(e._getSortedVisibleDatasetMetas().forEach((function(e){var t=e.data[a[0]._index];t&&!t._view.skip&&o.push(t)})),o):[]}var It={modes:{single:function(e,t){var n=Ct(t,e),r=[];return Et(e,(function(e){if(e.inRange(n.x,n.y))return r.push(e),r})),r.slice(0,1)},label:Ft,index:Ft,dataset:function(e,t,n){var r=Ct(t,e);n.axis=n.axis||"xy";var i=Pt(n.axis),a=n.intersect?At(e,r):Ot(e,r,!1,i);return a.length>0&&(a=e.getDatasetMeta(a[0]._datasetIndex).data),a},"x-axis":function(e,t){return Ft(e,t,{intersect:!1})},point:function(e,t){var n=Ct(t,e);return At(e,n)},nearest:function(e,t,n){var r=Ct(t,e);n.axis=n.axis||"xy";var i=Pt(n.axis);return Ot(e,r,n.intersect,i)},x:function(e,t,n){var r=Ct(t,e),i=[],a=!1;return Et(e,(function(e){e.inXRange(r.x)&&i.push(e),e.inRange(r.x,r.y)&&(a=!0)})),n.intersect&&!a&&(i=[]),i},y:function(e,t,n){var r=Ct(t,e),i=[],a=!1;return Et(e,(function(e){e.inYRange(r.y)&&i.push(e),e.inRange(r.x,r.y)&&(a=!0)})),n.intersect&&!a&&(i=[]),i}}},jt=ce.extend;function Ht(e,t){return ce.where(e,(function(e){return e.pos===t}))}function Rt(e,t){return e.sort((function(e,n){var r=t?n:e,i=t?e:n;return r.weight===i.weight?r.index-i.index:r.weight-i.weight}))}function Nt(e){var t,n,r,i=[];for(t=0,n=(e||[]).length;t<n;++t)r=e[t],i.push({index:t,box:r,pos:r.position,horizontal:r.isHorizontal(),weight:r.weight});return i}function $t(e,t){var n,r,i;for(n=0,r=e.length;n<r;++n)i=e[n],i.width=i.horizontal?i.box.fullWidth&&t.availableWidth:t.vBoxMaxWidth,i.height=i.horizontal&&t.hBoxMaxHeight}function zt(e){var t=Nt(e),n=Rt(Ht(t,"left"),!0),r=Rt(Ht(t,"right")),i=Rt(Ht(t,"top"),!0),a=Rt(Ht(t,"bottom"));return{leftAndTop:n.concat(i),rightAndBottom:r.concat(a),chartArea:Ht(t,"chartArea"),vertical:n.concat(r),horizontal:i.concat(a)}}function Bt(e,t,n,r){return Math.max(e[n],t[n])+Math.max(e[r],t[r])}function Wt(e,t,n){var r,i,a=n.box,o=e.maxPadding;if(n.size&&(e[n.pos]-=n.size),n.size=n.horizontal?a.height:a.width,e[n.pos]+=n.size,a.getPadding){var s=a.getPadding();o.top=Math.max(o.top,s.top),o.left=Math.max(o.left,s.left),o.bottom=Math.max(o.bottom,s.bottom),o.right=Math.max(o.right,s.right)}if(r=t.outerWidth-Bt(o,e,"left","right"),i=t.outerHeight-Bt(o,e,"top","bottom"),r!==e.w||i!==e.h){e.w=r,e.h=i;var l=n.horizontal?[r,e.w]:[i,e.h];return l[0]!==l[1]&&(!isNaN(l[0])||!isNaN(l[1]))}}function Ut(e){var t=e.maxPadding;function n(n){var r=Math.max(t[n]-e[n],0);return e[n]+=r,r}e.y+=n("top"),e.x+=n("left"),n("right"),n("bottom")}function Vt(e,t){var n=t.maxPadding;function r(e){var r={left:0,top:0,right:0,bottom:0};return e.forEach((function(e){r[e]=Math.max(t[e],n[e])})),r}return r(e?["left","right"]:["top","bottom"])}function qt(e,t,n){var r,i,a,o,s,l,u=[];for(r=0,i=e.length;r<i;++r)a=e[r],o=a.box,o.update(a.width||t.w,a.height||t.h,Vt(a.horizontal,t)),Wt(t,n,a)&&(l=!0,u.length&&(s=!0)),o.fullWidth||u.push(a);return s&&qt(u,t,n)||l}function Gt(e,t,n){var r,i,a,o,s=n.padding,l=t.x,u=t.y;for(r=0,i=e.length;r<i;++r)a=e[r],o=a.box,a.horizontal?(o.left=o.fullWidth?s.left:t.left,o.right=o.fullWidth?n.outerWidth-s.right:t.left+t.w,o.top=u,o.bottom=u+o.height,o.width=o.right-o.left,u=o.bottom):(o.left=l,o.right=l+o.width,o.top=t.top,o.bottom=t.top+t.h,o.height=o.bottom-o.top,l=o.right);t.x=l,t.y=u}Q._set("global",{layout:{padding:{top:0,right:0,bottom:0,left:0}}});var Jt={defaults:{},addBox:function(e,t){e.boxes||(e.boxes=[]),t.fullWidth=t.fullWidth||!1,t.position=t.position||"top",t.weight=t.weight||0,t._layers=t._layers||function(){return[{z:0,draw:function(){t.draw.apply(t,arguments)}}]},e.boxes.push(t)},removeBox:function(e,t){var n=e.boxes?e.boxes.indexOf(t):-1;-1!==n&&e.boxes.splice(n,1)},configure:function(e,t,n){for(var r,i=["fullWidth","position","weight"],a=i.length,o=0;o<a;++o)r=i[o],n.hasOwnProperty(r)&&(t[r]=n[r])},update:function(e,t,n){if(e){var r=e.options.layout||{},i=ce.options.toPadding(r.padding),a=t-i.width,o=n-i.height,s=zt(e.boxes),l=s.vertical,u=s.horizontal,c=Object.freeze({outerWidth:t,outerHeight:n,padding:i,availableWidth:a,vBoxMaxWidth:a/2/l.length,hBoxMaxHeight:o/2}),d=jt({maxPadding:jt({},i),w:a,h:o,x:i.left,y:i.top},i);$t(l.concat(u),c),qt(l,d,c),qt(u,d,c)&&qt(l,d,c),Ut(d),Gt(s.leftAndTop,d,c),d.x+=d.w,d.y+=d.h,Gt(s.rightAndBottom,d,c),e.chartArea={left:d.left,top:d.top,right:d.left+d.w,bottom:d.top+d.h},ce.each(s.chartArea,(function(t){var n=t.box;jt(n,e.chartArea),n.update(d.w,d.h)}))}}},Kt={acquireContext:function(e){return e&&e.canvas&&(e=e.canvas),e&&e.getContext("2d")||null}},Xt="/*\r\n * DOM element rendering detection\r\n * https://davidwalsh.name/detect-node-insertion\r\n */\r\n@keyframes chartjs-render-animation {\r\n\tfrom { opacity: 0.99; }\r\n\tto { opacity: 1; }\r\n}\r\n\r\n.chartjs-render-monitor {\r\n\tanimation: chartjs-render-animation 0.001s;\r\n}\r\n\r\n/*\r\n * DOM element resizing detection\r\n * https://github.com/marcj/css-element-queries\r\n */\r\n.chartjs-size-monitor,\r\n.chartjs-size-monitor-expand,\r\n.chartjs-size-monitor-shrink {\r\n\tposition: absolute;\r\n\tdirection: ltr;\r\n\tleft: 0;\r\n\ttop: 0;\r\n\tright: 0;\r\n\tbottom: 0;\r\n\toverflow: hidden;\r\n\tpointer-events: none;\r\n\tvisibility: hidden;\r\n\tz-index: -1;\r\n}\r\n\r\n.chartjs-size-monitor-expand > div {\r\n\tposition: absolute;\r\n\twidth: 1000000px;\r\n\theight: 1000000px;\r\n\tleft: 0;\r\n\ttop: 0;\r\n}\r\n\r\n.chartjs-size-monitor-shrink > div {\r\n\tposition: absolute;\r\n\twidth: 200%;\r\n\theight: 200%;\r\n\tleft: 0;\r\n\ttop: 0;\r\n}\r\n",Qt=Object.freeze({__proto__:null,default:Xt}),Zt=n(Qt),en="$chartjs",tn="chartjs-",nn=tn+"size-monitor",rn=tn+"render-monitor",an=tn+"render-animation",on=["animationstart","webkitAnimationStart"],sn={touchstart:"mousedown",touchmove:"mousemove",touchend:"mouseup",pointerenter:"mouseenter",pointerdown:"mousedown",pointermove:"mousemove",pointerup:"mouseup",pointerleave:"mouseout",pointerout:"mouseout"};function ln(e,t){var n=ce.getStyle(e,t),r=n&&n.match(/^(\d+)(\.\d+)?px$/);return r?Number(r[1]):void 0}function un(e,t){var n=e.style,r=e.getAttribute("height"),i=e.getAttribute("width");if(e[en]={initial:{height:r,width:i,style:{display:n.display,height:n.height,width:n.width}}},n.display=n.display||"block",null===i||""===i){var a=ln(e,"width");void 0!==a&&(e.width=a)}if(null===r||""===r)if(""===e.style.height)e.height=e.width/(t.options.aspectRatio||2);else{var o=ln(e,"height");void 0!==a&&(e.height=o)}return e}var cn=function(){var e=!1;try{var t=Object.defineProperty({},"passive",{get:function(){e=!0}});window.addEventListener("e",null,t)}catch(n){}return e}(),dn=!!cn&&{passive:!0};function fn(e,t,n){e.addEventListener(t,n,dn)}function hn(e,t,n){e.removeEventListener(t,n,dn)}function pn(e,t,n,r,i){return{type:e,chart:t,native:i||null,x:void 0!==n?n:null,y:void 0!==r?r:null}}function mn(e,t){var n=sn[e.type]||e.type,r=ce.getRelativePosition(e,t);return pn(n,t,r.x,r.y,e)}function _n(e,t){var n=!1,r=[];return function(){r=Array.prototype.slice.call(arguments),t=t||this,n||(n=!0,ce.requestAnimFrame.call(window,(function(){n=!1,e.apply(t,r)})))}}function vn(e){var t=document.createElement("div");return t.className=e||"",t}function gn(e){var t=1e6,n=vn(nn),r=vn(nn+"-expand"),i=vn(nn+"-shrink");r.appendChild(vn()),i.appendChild(vn()),n.appendChild(r),n.appendChild(i),n._reset=function(){r.scrollLeft=t,r.scrollTop=t,i.scrollLeft=t,i.scrollTop=t};var a=function(){n._reset(),e()};return fn(r,"scroll",a.bind(r,"expand")),fn(i,"scroll",a.bind(i,"shrink")),n}function bn(e,t){var n=e[en]||(e[en]={}),r=n.renderProxy=function(e){e.animationName===an&&t()};ce.each(on,(function(t){fn(e,t,r)})),n.reflow=!!e.offsetParent,e.classList.add(rn)}function yn(e){var t=e[en]||{},n=t.renderProxy;n&&(ce.each(on,(function(t){hn(e,t,n)})),delete t.renderProxy),e.classList.remove(rn)}function Mn(e,t,n){var r=e[en]||(e[en]={}),i=r.resizer=gn(_n((function(){if(r.resizer){var i=n.options.maintainAspectRatio&&e.parentNode,a=i?i.clientWidth:0;t(pn("resize",n)),i&&i.clientWidth<a&&n.canvas&&t(pn("resize",n))}})));bn(e,(function(){if(r.resizer){var t=e.parentNode;t&&t!==i.parentNode&&t.insertBefore(i,t.firstChild),i._reset()}}))}function kn(e){var t=e[en]||{},n=t.resizer;delete t.resizer,yn(e),n&&n.parentNode&&n.parentNode.removeChild(n)}function wn(e,t){var n=e[en]||(e[en]={});if(!n.containsStyles){n.containsStyles=!0,t="/* Chart.js */\n"+t;var r=document.createElement("style");r.setAttribute("type","text/css"),r.appendChild(document.createTextNode(t)),e.appendChild(r)}}var Ln={disableCSSInjection:!1,_enabled:"undefined"!==typeof window&&"undefined"!==typeof document,_ensureLoaded:function(e){if(!this.disableCSSInjection){var t=e.getRootNode?e.getRootNode():document,n=t.host?t:document.head;wn(n,Zt)}},acquireContext:function(e,t){"string"===typeof e?e=document.getElementById(e):e.length&&(e=e[0]),e&&e.canvas&&(e=e.canvas);var n=e&&e.getContext&&e.getContext("2d");return n&&n.canvas===e?(this._ensureLoaded(e),un(e,t),n):null},releaseContext:function(e){var t=e.canvas;if(t[en]){var n=t[en].initial;["height","width"].forEach((function(e){var r=n[e];ce.isNullOrUndef(r)?t.removeAttribute(e):t.setAttribute(e,r)})),ce.each(n.style||{},(function(e,n){t.style[n]=e})),t.width=t.width,delete t[en]}},addEventListener:function(e,t,n){var r=e.canvas;if("resize"!==t){var i=n[en]||(n[en]={}),a=i.proxies||(i.proxies={}),o=a[e.id+"_"+t]=function(t){n(mn(t,e))};fn(r,t,o)}else Mn(r,n,e)},removeEventListener:function(e,t,n){var r=e.canvas;if("resize"!==t){var i=n[en]||{},a=i.proxies||{},o=a[e.id+"_"+t];o&&hn(r,t,o)}else kn(r)}};ce.addEvent=fn,ce.removeEvent=hn;var xn=Ln._enabled?Ln:Kt,Sn=ce.extend({initialize:function(){},acquireContext:function(){},releaseContext:function(){},addEventListener:function(){},removeEventListener:function(){}},xn);Q._set("global",{plugins:{}});var Tn={_plugins:[],_cacheId:0,register:function(e){var t=this._plugins;[].concat(e).forEach((function(e){-1===t.indexOf(e)&&t.push(e)})),this._cacheId++},unregister:function(e){var t=this._plugins;[].concat(e).forEach((function(e){var n=t.indexOf(e);-1!==n&&t.splice(n,1)})),this._cacheId++},clear:function(){this._plugins=[],this._cacheId++},count:function(){return this._plugins.length},getAll:function(){return this._plugins},notify:function(e,t,n){var r,i,a,o,s,l=this.descriptors(e),u=l.length;for(r=0;r<u;++r)if(i=l[r],a=i.plugin,s=a[t],"function"===typeof s&&(o=[e].concat(n||[]),o.push(i.options),!1===s.apply(a,o)))return!1;return!0},descriptors:function(e){var t=e.$plugins||(e.$plugins={});if(t.id===this._cacheId)return t.descriptors;var n=[],r=[],i=e&&e.config||{},a=i.options&&i.options.plugins||{};return this._plugins.concat(i.plugins||[]).forEach((function(e){var t=n.indexOf(e);if(-1===t){var i=e.id,o=a[i];!1!==o&&(!0===o&&(o=ce.clone(Q.global.plugins[i])),n.push(e),r.push({plugin:e,options:o||{}}))}})),t.descriptors=r,t.id=this._cacheId,r},_invalidate:function(e){delete e.$plugins}},Dn={constructors:{},defaults:{},registerScaleType:function(e,t,n){this.constructors[e]=t,this.defaults[e]=ce.clone(n)},getScaleConstructor:function(e){return this.constructors.hasOwnProperty(e)?this.constructors[e]:void 0},getScaleDefaults:function(e){return this.defaults.hasOwnProperty(e)?ce.merge(Object.create(null),[Q.scale,this.defaults[e]]):{}},updateScaleDefaults:function(e,t){var n=this;n.defaults.hasOwnProperty(e)&&(n.defaults[e]=ce.extend(n.defaults[e],t))},addScalesToLayout:function(e){ce.each(e.scales,(function(t){t.fullWidth=t.options.fullWidth,t.position=t.options.position,t.weight=t.options.weight,Jt.addBox(e,t)}))}},Yn=ce.valueOrDefault,Cn=ce.rtl.getRtlAdapter;Q._set("global",{tooltips:{enabled:!0,custom:null,mode:"nearest",position:"average",intersect:!0,backgroundColor:"rgba(0,0,0,0.8)",titleFontStyle:"bold",titleSpacing:2,titleMarginBottom:6,titleFontColor:"#fff",titleAlign:"left",bodySpacing:2,bodyFontColor:"#fff",bodyAlign:"left",footerFontStyle:"bold",footerSpacing:2,footerMarginTop:6,footerFontColor:"#fff",footerAlign:"left",yPadding:6,xPadding:6,caretPadding:2,caretSize:5,cornerRadius:6,multiKeyBackground:"#fff",displayColors:!0,borderColor:"rgba(0,0,0,0)",borderWidth:0,callbacks:{beforeTitle:ce.noop,title:function(e,t){var n="",r=t.labels,i=r?r.length:0;if(e.length>0){var a=e[0];a.label?n=a.label:a.xLabel?n=a.xLabel:i>0&&a.index<i&&(n=r[a.index])}return n},afterTitle:ce.noop,beforeBody:ce.noop,beforeLabel:ce.noop,label:function(e,t){var n=t.datasets[e.datasetIndex].label||"";return n&&(n+=": "),ce.isNullOrUndef(e.value)?n+=e.yLabel:n+=e.value,n},labelColor:function(e,t){var n=t.getDatasetMeta(e.datasetIndex),r=n.data[e.index],i=r._view;return{borderColor:i.borderColor,backgroundColor:i.backgroundColor}},labelTextColor:function(){return this._options.bodyFontColor},afterLabel:ce.noop,afterBody:ce.noop,beforeFooter:ce.noop,footer:ce.noop,afterFooter:ce.noop}}});var En={average:function(e){if(!e.length)return!1;var t,n,r=0,i=0,a=0;for(t=0,n=e.length;t<n;++t){var o=e[t];if(o&&o.hasValue()){var s=o.tooltipPosition();r+=s.x,i+=s.y,++a}}return{x:r/a,y:i/a}},nearest:function(e,t){var n,r,i,a=t.x,o=t.y,s=Number.POSITIVE_INFINITY;for(n=0,r=e.length;n<r;++n){var l=e[n];if(l&&l.hasValue()){var u=l.getCenterPoint(),c=ce.distanceBetweenPoints(t,u);c<s&&(s=c,i=l)}}if(i){var d=i.tooltipPosition();a=d.x,o=d.y}return{x:a,y:o}}};function An(e,t){return t&&(ce.isArray(t)?Array.prototype.push.apply(e,t):e.push(t)),e}function On(e){return("string"===typeof e||e instanceof String)&&e.indexOf("\n")>-1?e.split("\n"):e}function Pn(e){var t=e._xScale,n=e._yScale||e._scale,r=e._index,i=e._datasetIndex,a=e._chart.getDatasetMeta(i).controller,o=a._getIndexScale(),s=a._getValueScale();return{xLabel:t?t.getLabelForIndex(r,i):"",yLabel:n?n.getLabelForIndex(r,i):"",label:o?""+o.getLabelForIndex(r,i):"",value:s?""+s.getLabelForIndex(r,i):"",index:r,datasetIndex:i,x:e._model.x,y:e._model.y}}function Fn(e){var t=Q.global;return{xPadding:e.xPadding,yPadding:e.yPadding,xAlign:e.xAlign,yAlign:e.yAlign,rtl:e.rtl,textDirection:e.textDirection,bodyFontColor:e.bodyFontColor,_bodyFontFamily:Yn(e.bodyFontFamily,t.defaultFontFamily),_bodyFontStyle:Yn(e.bodyFontStyle,t.defaultFontStyle),_bodyAlign:e.bodyAlign,bodyFontSize:Yn(e.bodyFontSize,t.defaultFontSize),bodySpacing:e.bodySpacing,titleFontColor:e.titleFontColor,_titleFontFamily:Yn(e.titleFontFamily,t.defaultFontFamily),_titleFontStyle:Yn(e.titleFontStyle,t.defaultFontStyle),titleFontSize:Yn(e.titleFontSize,t.defaultFontSize),_titleAlign:e.titleAlign,titleSpacing:e.titleSpacing,titleMarginBottom:e.titleMarginBottom,footerFontColor:e.footerFontColor,_footerFontFamily:Yn(e.footerFontFamily,t.defaultFontFamily),_footerFontStyle:Yn(e.footerFontStyle,t.defaultFontStyle),footerFontSize:Yn(e.footerFontSize,t.defaultFontSize),_footerAlign:e.footerAlign,footerSpacing:e.footerSpacing,footerMarginTop:e.footerMarginTop,caretSize:e.caretSize,cornerRadius:e.cornerRadius,backgroundColor:e.backgroundColor,opacity:0,legendColorBackground:e.multiKeyBackground,displayColors:e.displayColors,borderColor:e.borderColor,borderWidth:e.borderWidth}}function In(e,t){var n=e._chart.ctx,r=2*t.yPadding,i=0,a=t.body,o=a.reduce((function(e,t){return e+t.before.length+t.lines.length+t.after.length}),0);o+=t.beforeBody.length+t.afterBody.length;var s=t.title.length,l=t.footer.length,u=t.titleFontSize,c=t.bodyFontSize,d=t.footerFontSize;r+=s*u,r+=s?(s-1)*t.titleSpacing:0,r+=s?t.titleMarginBottom:0,r+=o*c,r+=o?(o-1)*t.bodySpacing:0,r+=l?t.footerMarginTop:0,r+=l*d,r+=l?(l-1)*t.footerSpacing:0;var f=0,h=function(e){i=Math.max(i,n.measureText(e).width+f)};return n.font=ce.fontString(u,t._titleFontStyle,t._titleFontFamily),ce.each(t.title,h),n.font=ce.fontString(c,t._bodyFontStyle,t._bodyFontFamily),ce.each(t.beforeBody.concat(t.afterBody),h),f=t.displayColors?c+2:0,ce.each(a,(function(e){ce.each(e.before,h),ce.each(e.lines,h),ce.each(e.after,h)})),f=0,n.font=ce.fontString(d,t._footerFontStyle,t._footerFontFamily),ce.each(t.footer,h),i+=2*t.xPadding,{width:i,height:r}}function jn(e,t){var n,r,i,a,o,s=e._model,l=e._chart,u=e._chart.chartArea,c="center",d="center";s.y<t.height?d="top":s.y>l.height-t.height&&(d="bottom");var f=(u.left+u.right)/2,h=(u.top+u.bottom)/2;"center"===d?(n=function(e){return e<=f},r=function(e){return e>f}):(n=function(e){return e<=t.width/2},r=function(e){return e>=l.width-t.width/2}),i=function(e){return e+t.width+s.caretSize+s.caretPadding>l.width},a=function(e){return e-t.width-s.caretSize-s.caretPadding<0},o=function(e){return e<=h?"top":"bottom"},n(s.x)?(c="left",i(s.x)&&(c="center",d=o(s.y))):r(s.x)&&(c="right",a(s.x)&&(c="center",d=o(s.y)));var p=e._options;return{xAlign:p.xAlign?p.xAlign:c,yAlign:p.yAlign?p.yAlign:d}}function Hn(e,t,n,r){var i=e.x,a=e.y,o=e.caretSize,s=e.caretPadding,l=e.cornerRadius,u=n.xAlign,c=n.yAlign,d=o+s,f=l+s;return"right"===u?i-=t.width:"center"===u&&(i-=t.width/2,i+t.width>r.width&&(i=r.width-t.width),i<0&&(i=0)),"top"===c?a+=d:a-="bottom"===c?t.height+d:t.height/2,"center"===c?"left"===u?i+=d:"right"===u&&(i-=d):"left"===u?i-=f:"right"===u&&(i+=f),{x:i,y:a}}function Rn(e,t){return"center"===t?e.x+e.width/2:"right"===t?e.x+e.width-e.xPadding:e.x+e.xPadding}function Nn(e){return An([],On(e))}var $n=ge.extend({initialize:function(){this._model=Fn(this._options),this._lastActive=[]},getTitle:function(){var e=this,t=e._options,n=t.callbacks,r=n.beforeTitle.apply(e,arguments),i=n.title.apply(e,arguments),a=n.afterTitle.apply(e,arguments),o=[];return o=An(o,On(r)),o=An(o,On(i)),o=An(o,On(a)),o},getBeforeBody:function(){return Nn(this._options.callbacks.beforeBody.apply(this,arguments))},getBody:function(e,t){var n=this,r=n._options.callbacks,i=[];return ce.each(e,(function(e){var a={before:[],lines:[],after:[]};An(a.before,On(r.beforeLabel.call(n,e,t))),An(a.lines,r.label.call(n,e,t)),An(a.after,On(r.afterLabel.call(n,e,t))),i.push(a)})),i},getAfterBody:function(){return Nn(this._options.callbacks.afterBody.apply(this,arguments))},getFooter:function(){var e=this,t=e._options.callbacks,n=t.beforeFooter.apply(e,arguments),r=t.footer.apply(e,arguments),i=t.afterFooter.apply(e,arguments),a=[];return a=An(a,On(n)),a=An(a,On(r)),a=An(a,On(i)),a},update:function(e){var t,n,r=this,i=r._options,a=r._model,o=r._model=Fn(i),s=r._active,l=r._data,u={xAlign:a.xAlign,yAlign:a.yAlign},c={x:a.x,y:a.y},d={width:a.width,height:a.height},f={x:a.caretX,y:a.caretY};if(s.length){o.opacity=1;var h=[],p=[];f=En[i.position].call(r,s,r._eventPosition);var m=[];for(t=0,n=s.length;t<n;++t)m.push(Pn(s[t]));i.filter&&(m=m.filter((function(e){return i.filter(e,l)}))),i.itemSort&&(m=m.sort((function(e,t){return i.itemSort(e,t,l)}))),ce.each(m,(function(e){h.push(i.callbacks.labelColor.call(r,e,r._chart)),p.push(i.callbacks.labelTextColor.call(r,e,r._chart))})),o.title=r.getTitle(m,l),o.beforeBody=r.getBeforeBody(m,l),o.body=r.getBody(m,l),o.afterBody=r.getAfterBody(m,l),o.footer=r.getFooter(m,l),o.x=f.x,o.y=f.y,o.caretPadding=i.caretPadding,o.labelColors=h,o.labelTextColors=p,o.dataPoints=m,d=In(this,o),u=jn(this,d),c=Hn(o,d,u,r._chart)}else o.opacity=0;return o.xAlign=u.xAlign,o.yAlign=u.yAlign,o.x=c.x,o.y=c.y,o.width=d.width,o.height=d.height,o.caretX=f.x,o.caretY=f.y,r._model=o,e&&i.custom&&i.custom.call(r,o),r},drawCaret:function(e,t){var n=this._chart.ctx,r=this._view,i=this.getCaretPosition(e,t,r);n.lineTo(i.x1,i.y1),n.lineTo(i.x2,i.y2),n.lineTo(i.x3,i.y3)},getCaretPosition:function(e,t,n){var r,i,a,o,s,l,u=n.caretSize,c=n.cornerRadius,d=n.xAlign,f=n.yAlign,h=e.x,p=e.y,m=t.width,_=t.height;if("center"===f)s=p+_/2,"left"===d?(r=h,i=r-u,a=r,o=s+u,l=s-u):(r=h+m,i=r+u,a=r,o=s-u,l=s+u);else if("left"===d?(i=h+c+u,r=i-u,a=i+u):"right"===d?(i=h+m-c-u,r=i-u,a=i+u):(i=n.caretX,r=i-u,a=i+u),"top"===f)o=p,s=o-u,l=o;else{o=p+_,s=o+u,l=o;var v=a;a=r,r=v}return{x1:r,x2:i,x3:a,y1:o,y2:s,y3:l}},drawTitle:function(e,t,n){var r,i,a,o=t.title,s=o.length;if(s){var l=Cn(t.rtl,t.x,t.width);for(e.x=Rn(t,t._titleAlign),n.textAlign=l.textAlign(t._titleAlign),n.textBaseline="middle",r=t.titleFontSize,i=t.titleSpacing,n.fillStyle=t.titleFontColor,n.font=ce.fontString(r,t._titleFontStyle,t._titleFontFamily),a=0;a<s;++a)n.fillText(o[a],l.x(e.x),e.y+r/2),e.y+=r+i,a+1===s&&(e.y+=t.titleMarginBottom-i)}},drawBody:function(e,t,n){var r,i,a,o,s,l,u,c,d=t.bodyFontSize,f=t.bodySpacing,h=t._bodyAlign,p=t.body,m=t.displayColors,_=0,v=m?Rn(t,"left"):0,g=Cn(t.rtl,t.x,t.width),b=function(t){n.fillText(t,g.x(e.x+_),e.y+d/2),e.y+=d+f},y=g.textAlign(h);for(n.textAlign=h,n.textBaseline="middle",n.font=ce.fontString(d,t._bodyFontStyle,t._bodyFontFamily),e.x=Rn(t,y),n.fillStyle=t.bodyFontColor,ce.each(t.beforeBody,b),_=m&&"right"!==y?"center"===h?d/2+1:d+2:0,s=0,u=p.length;s<u;++s){for(r=p[s],i=t.labelTextColors[s],a=t.labelColors[s],n.fillStyle=i,ce.each(r.before,b),o=r.lines,l=0,c=o.length;l<c;++l){if(m){var M=g.x(v);n.fillStyle=t.legendColorBackground,n.fillRect(g.leftForLtr(M,d),e.y,d,d),n.lineWidth=1,n.strokeStyle=a.borderColor,n.strokeRect(g.leftForLtr(M,d),e.y,d,d),n.fillStyle=a.backgroundColor,n.fillRect(g.leftForLtr(g.xPlus(M,1),d-2),e.y+1,d-2,d-2),n.fillStyle=i}b(o[l])}ce.each(r.after,b)}_=0,ce.each(t.afterBody,b),e.y-=f},drawFooter:function(e,t,n){var r,i,a=t.footer,o=a.length;if(o){var s=Cn(t.rtl,t.x,t.width);for(e.x=Rn(t,t._footerAlign),e.y+=t.footerMarginTop,n.textAlign=s.textAlign(t._footerAlign),n.textBaseline="middle",r=t.footerFontSize,n.fillStyle=t.footerFontColor,n.font=ce.fontString(r,t._footerFontStyle,t._footerFontFamily),i=0;i<o;++i)n.fillText(a[i],s.x(e.x),e.y+r/2),e.y+=r+t.footerSpacing}},drawBackground:function(e,t,n,r){n.fillStyle=t.backgroundColor,n.strokeStyle=t.borderColor,n.lineWidth=t.borderWidth;var i=t.xAlign,a=t.yAlign,o=e.x,s=e.y,l=r.width,u=r.height,c=t.cornerRadius;n.beginPath(),n.moveTo(o+c,s),"top"===a&&this.drawCaret(e,r),n.lineTo(o+l-c,s),n.quadraticCurveTo(o+l,s,o+l,s+c),"center"===a&&"right"===i&&this.drawCaret(e,r),n.lineTo(o+l,s+u-c),n.quadraticCurveTo(o+l,s+u,o+l-c,s+u),"bottom"===a&&this.drawCaret(e,r),n.lineTo(o+c,s+u),n.quadraticCurveTo(o,s+u,o,s+u-c),"center"===a&&"left"===i&&this.drawCaret(e,r),n.lineTo(o,s+c),n.quadraticCurveTo(o,s,o+c,s),n.closePath(),n.fill(),t.borderWidth>0&&n.stroke()},draw:function(){var e=this._chart.ctx,t=this._view;if(0!==t.opacity){var n={width:t.width,height:t.height},r={x:t.x,y:t.y},i=Math.abs(t.opacity<.001)?0:t.opacity,a=t.title.length||t.beforeBody.length||t.body.length||t.afterBody.length||t.footer.length;this._options.enabled&&a&&(e.save(),e.globalAlpha=i,this.drawBackground(r,t,e,n),r.y+=t.yPadding,ce.rtl.overrideTextDirection(e,t.textDirection),this.drawTitle(r,t,e),this.drawBody(r,t,e),this.drawFooter(r,t,e),ce.rtl.restoreTextDirection(e,t.textDirection),e.restore())}},handleEvent:function(e){var t=this,n=t._options,r=!1;return t._lastActive=t._lastActive||[],"mouseout"===e.type?t._active=[]:(t._active=t._chart.getElementsAtEventForMode(e,n.mode,n),n.reverse&&t._active.reverse()),r=!ce.arrayEquals(t._active,t._lastActive),r&&(t._lastActive=t._active,(n.enabled||n.custom)&&(t._eventPosition={x:e.x,y:e.y},t.update(!0),t.pivot())),r}}),zn=En,Bn=$n;Bn.positioners=zn;var Wn=ce.valueOrDefault;function Un(){return ce.merge(Object.create(null),[].slice.call(arguments),{merger:function(e,t,n,r){if("xAxes"===e||"yAxes"===e){var i,a,o,s=n[e].length;for(t[e]||(t[e]=[]),i=0;i<s;++i)o=n[e][i],a=Wn(o.type,"xAxes"===e?"category":"linear"),i>=t[e].length&&t[e].push({}),!t[e][i].type||o.type&&o.type!==t[e][i].type?ce.merge(t[e][i],[Dn.getScaleDefaults(a),o]):ce.merge(t[e][i],o)}else ce._merger(e,t,n,r)}})}function Vn(){return ce.merge(Object.create(null),[].slice.call(arguments),{merger:function(e,t,n,r){var i=t[e]||Object.create(null),a=n[e];"scales"===e?t[e]=Un(i,a):"scale"===e?t[e]=ce.merge(i,[Dn.getScaleDefaults(a.type),a]):ce._merger(e,t,n,r)}})}function qn(e){e=e||Object.create(null);var t=e.data=e.data||{};return t.datasets=t.datasets||[],t.labels=t.labels||[],e.options=Vn(Q.global,Q[e.type],e.options||{}),e}function Gn(e){var t=e.options;ce.each(e.scales,(function(t){Jt.removeBox(e,t)})),t=Vn(Q.global,Q[e.config.type],t),e.options=e.config.options=t,e.ensureScalesHaveIDs(),e.buildOrUpdateScales(),e.tooltip._options=t.tooltips,e.tooltip.initialize()}function Jn(e,t,n){var r,i=function(e){return e.id===r};do{r=t+n++}while(ce.findIndex(e,i)>=0);return r}function Kn(e){return"top"===e||"bottom"===e}function Xn(e,t){return function(n,r){return n[e]===r[e]?n[t]-r[t]:n[e]-r[e]}}Q._set("global",{elements:{},events:["mousemove","mouseout","click","touchstart","touchmove"],hover:{onHover:null,mode:"nearest",intersect:!0,animationDuration:400},onClick:null,maintainAspectRatio:!0,responsive:!0,responsiveAnimationDuration:0});var Qn=function(e,t){return this.construct(e,t),this};ce.extend(Qn.prototype,{construct:function(e,t){var n=this;t=qn(t);var r=Sn.acquireContext(e,t),i=r&&r.canvas,a=i&&i.height,o=i&&i.width;n.id=ce.uid(),n.ctx=r,n.canvas=i,n.config=t,n.width=o,n.height=a,n.aspectRatio=a?o/a:null,n.options=t.options,n._bufferedRender=!1,n._layers=[],n.chart=n,n.controller=n,Qn.instances[n.id]=n,Object.defineProperty(n,"data",{get:function(){return n.config.data},set:function(e){n.config.data=e}}),r&&i?(n.initialize(),n.update()):console.error("Failed to create chart: can't acquire context from the given item")},initialize:function(){var e=this;return Tn.notify(e,"beforeInit"),ce.retinaScale(e,e.options.devicePixelRatio),e.bindEvents(),e.options.responsive&&e.resize(!0),e.initToolTip(),Tn.notify(e,"afterInit"),e},clear:function(){return ce.canvas.clear(this),this},stop:function(){return Me.cancelAnimation(this),this},resize:function(e){var t=this,n=t.options,r=t.canvas,i=n.maintainAspectRatio&&t.aspectRatio||null,a=Math.max(0,Math.floor(ce.getMaximumWidth(r))),o=Math.max(0,Math.floor(i?a/i:ce.getMaximumHeight(r)));if((t.width!==a||t.height!==o)&&(r.width=t.width=a,r.height=t.height=o,r.style.width=a+"px",r.style.height=o+"px",ce.retinaScale(t,n.devicePixelRatio),!e)){var s={width:a,height:o};Tn.notify(t,"resize",[s]),n.onResize&&n.onResize(t,s),t.stop(),t.update({duration:n.responsiveAnimationDuration})}},ensureScalesHaveIDs:function(){var e=this.options,t=e.scales||{},n=e.scale;ce.each(t.xAxes,(function(e,n){e.id||(e.id=Jn(t.xAxes,"x-axis-",n))})),ce.each(t.yAxes,(function(e,n){e.id||(e.id=Jn(t.yAxes,"y-axis-",n))})),n&&(n.id=n.id||"scale")},buildOrUpdateScales:function(){var e=this,t=e.options,n=e.scales||{},r=[],i=Object.keys(n).reduce((function(e,t){return e[t]=!1,e}),{});t.scales&&(r=r.concat((t.scales.xAxes||[]).map((function(e){return{options:e,dtype:"category",dposition:"bottom"}})),(t.scales.yAxes||[]).map((function(e){return{options:e,dtype:"linear",dposition:"left"}})))),t.scale&&r.push({options:t.scale,dtype:"radialLinear",isDefault:!0,dposition:"chartArea"}),ce.each(r,(function(t){var r=t.options,a=r.id,o=Wn(r.type,t.dtype);Kn(r.position)!==Kn(t.dposition)&&(r.position=t.dposition),i[a]=!0;var s=null;if(a in n&&n[a].type===o)s=n[a],s.options=r,s.ctx=e.ctx,s.chart=e;else{var l=Dn.getScaleConstructor(o);if(!l)return;s=new l({id:a,type:o,options:r,ctx:e.ctx,chart:e}),n[s.id]=s}s.mergeTicksOptions(),t.isDefault&&(e.scale=s)})),ce.each(i,(function(e,t){e||delete n[t]})),e.scales=n,Dn.addScalesToLayout(this)},buildOrUpdateControllers:function(){var e,t,n=this,r=[],i=n.data.datasets;for(e=0,t=i.length;e<t;e++){var a=i[e],o=n.getDatasetMeta(e),s=a.type||n.config.type;if(o.type&&o.type!==s&&(n.destroyDatasetMeta(e),o=n.getDatasetMeta(e)),o.type=s,o.order=a.order||0,o.index=e,o.controller)o.controller.updateIndex(e),o.controller.linkScales();else{var l=Yt[o.type];if(void 0===l)throw new Error('"'+o.type+'" is not a chart type.');o.controller=new l(n,e),r.push(o.controller)}}return r},resetElements:function(){var e=this;ce.each(e.data.datasets,(function(t,n){e.getDatasetMeta(n).controller.reset()}),e)},reset:function(){this.resetElements(),this.tooltip.initialize()},update:function(e){var t,n,r=this;if(e&&"object"===typeof e||(e={duration:e,lazy:arguments[1]}),Gn(r),Tn._invalidate(r),!1!==Tn.notify(r,"beforeUpdate")){r.tooltip._data=r.data;var i=r.buildOrUpdateControllers();for(t=0,n=r.data.datasets.length;t<n;t++)r.getDatasetMeta(t).controller.buildOrUpdateElements();r.updateLayout(),r.options.animation&&r.options.animation.duration&&ce.each(i,(function(e){e.reset()})),r.updateDatasets(),r.tooltip.initialize(),r.lastActive=[],Tn.notify(r,"afterUpdate"),r._layers.sort(Xn("z","_idx")),r._bufferedRender?r._bufferedRequest={duration:e.duration,easing:e.easing,lazy:e.lazy}:r.render(e)}},updateLayout:function(){var e=this;!1!==Tn.notify(e,"beforeLayout")&&(Jt.update(this,this.width,this.height),e._layers=[],ce.each(e.boxes,(function(t){t._configure&&t._configure(),e._layers.push.apply(e._layers,t._layers())}),e),e._layers.forEach((function(e,t){e._idx=t})),Tn.notify(e,"afterScaleUpdate"),Tn.notify(e,"afterLayout"))},updateDatasets:function(){var e=this;if(!1!==Tn.notify(e,"beforeDatasetsUpdate")){for(var t=0,n=e.data.datasets.length;t<n;++t)e.updateDataset(t);Tn.notify(e,"afterDatasetsUpdate")}},updateDataset:function(e){var t=this,n=t.getDatasetMeta(e),r={meta:n,index:e};!1!==Tn.notify(t,"beforeDatasetUpdate",[r])&&(n.controller._update(),Tn.notify(t,"afterDatasetUpdate",[r]))},render:function(e){var t=this;e&&"object"===typeof e||(e={duration:e,lazy:arguments[1]});var n=t.options.animation,r=Wn(e.duration,n&&n.duration),i=e.lazy;if(!1!==Tn.notify(t,"beforeRender")){var a=function(e){Tn.notify(t,"afterRender"),ce.callback(n&&n.onComplete,[e],t)};if(n&&r){var o=new ye({numSteps:r/16.66,easing:e.easing||n.easing,render:function(e,t){var n=ce.easing.effects[t.easing],r=t.currentStep,i=r/t.numSteps;e.draw(n(i),i,r)},onAnimationProgress:n.onProgress,onAnimationComplete:a});Me.addAnimation(t,o,r,i)}else t.draw(),a(new ye({numSteps:0,chart:t}));return t}},draw:function(e){var t,n,r=this;if(r.clear(),ce.isNullOrUndef(e)&&(e=1),r.transition(e),!(r.width<=0||r.height<=0)&&!1!==Tn.notify(r,"beforeDraw",[e])){for(n=r._layers,t=0;t<n.length&&n[t].z<=0;++t)n[t].draw(r.chartArea);for(r.drawDatasets(e);t<n.length;++t)n[t].draw(r.chartArea);r._drawTooltip(e),Tn.notify(r,"afterDraw",[e])}},transition:function(e){for(var t=this,n=0,r=(t.data.datasets||[]).length;n<r;++n)t.isDatasetVisible(n)&&t.getDatasetMeta(n).controller.transition(e);t.tooltip.transition(e)},_getSortedDatasetMetas:function(e){var t,n,r=this,i=r.data.datasets||[],a=[];for(t=0,n=i.length;t<n;++t)e&&!r.isDatasetVisible(t)||a.push(r.getDatasetMeta(t));return a.sort(Xn("order","index")),a},_getSortedVisibleDatasetMetas:function(){return this._getSortedDatasetMetas(!0)},drawDatasets:function(e){var t,n,r=this;if(!1!==Tn.notify(r,"beforeDatasetsDraw",[e])){for(t=r._getSortedVisibleDatasetMetas(),n=t.length-1;n>=0;--n)r.drawDataset(t[n],e);Tn.notify(r,"afterDatasetsDraw",[e])}},drawDataset:function(e,t){var n=this,r={meta:e,index:e.index,easingValue:t};!1!==Tn.notify(n,"beforeDatasetDraw",[r])&&(e.controller.draw(t),Tn.notify(n,"afterDatasetDraw",[r]))},_drawTooltip:function(e){var t=this,n=t.tooltip,r={tooltip:n,easingValue:e};!1!==Tn.notify(t,"beforeTooltipDraw",[r])&&(n.draw(),Tn.notify(t,"afterTooltipDraw",[r]))},getElementAtEvent:function(e){return It.modes.single(this,e)},getElementsAtEvent:function(e){return It.modes.label(this,e,{intersect:!0})},getElementsAtXAxis:function(e){return It.modes["x-axis"](this,e,{intersect:!0})},getElementsAtEventForMode:function(e,t,n){var r=It.modes[t];return"function"===typeof r?r(this,e,n):[]},getDatasetAtEvent:function(e){return It.modes.dataset(this,e,{intersect:!0})},getDatasetMeta:function(e){var t=this,n=t.data.datasets[e];n._meta||(n._meta={});var r=n._meta[t.id];return r||(r=n._meta[t.id]={type:null,data:[],dataset:null,controller:null,hidden:null,xAxisID:null,yAxisID:null,order:n.order||0,index:e}),r},getVisibleDatasetCount:function(){for(var e=0,t=0,n=this.data.datasets.length;t<n;++t)this.isDatasetVisible(t)&&e++;return e},isDatasetVisible:function(e){var t=this.getDatasetMeta(e);return"boolean"===typeof t.hidden?!t.hidden:!this.data.datasets[e].hidden},generateLegend:function(){return this.options.legendCallback(this)},destroyDatasetMeta:function(e){var t=this.id,n=this.data.datasets[e],r=n._meta&&n._meta[t];r&&(r.controller.destroy(),delete n._meta[t])},destroy:function(){var e,t,n=this,r=n.canvas;for(n.stop(),e=0,t=n.data.datasets.length;e<t;++e)n.destroyDatasetMeta(e);r&&(n.unbindEvents(),ce.canvas.clear(n),Sn.releaseContext(n.ctx),n.canvas=null,n.ctx=null),Tn.notify(n,"destroy"),delete Qn.instances[n.id]},toBase64Image:function(){return this.canvas.toDataURL.apply(this.canvas,arguments)},initToolTip:function(){var e=this;e.tooltip=new Bn({_chart:e,_chartInstance:e,_data:e.data,_options:e.options.tooltips},e)},bindEvents:function(){var e=this,t=e._listeners={},n=function(){e.eventHandler.apply(e,arguments)};ce.each(e.options.events,(function(r){Sn.addEventListener(e,r,n),t[r]=n})),e.options.responsive&&(n=function(){e.resize()},Sn.addEventListener(e,"resize",n),t.resize=n)},unbindEvents:function(){var e=this,t=e._listeners;t&&(delete e._listeners,ce.each(t,(function(t,n){Sn.removeEventListener(e,n,t)})))},updateHoverStyle:function(e,t,n){var r,i,a,o=n?"set":"remove";for(i=0,a=e.length;i<a;++i)r=e[i],r&&this.getDatasetMeta(r._datasetIndex).controller[o+"HoverStyle"](r);"dataset"===t&&this.getDatasetMeta(e[0]._datasetIndex).controller["_"+o+"DatasetHoverStyle"]()},eventHandler:function(e){var t=this,n=t.tooltip;if(!1!==Tn.notify(t,"beforeEvent",[e])){t._bufferedRender=!0,t._bufferedRequest=null;var r=t.handleEvent(e);n&&(r=n._start?n.handleEvent(e):r|n.handleEvent(e)),Tn.notify(t,"afterEvent",[e]);var i=t._bufferedRequest;return i?t.render(i):r&&!t.animating&&(t.stop(),t.render({duration:t.options.hover.animationDuration,lazy:!0})),t._bufferedRender=!1,t._bufferedRequest=null,t}},handleEvent:function(e){var t=this,n=t.options||{},r=n.hover,i=!1;return t.lastActive=t.lastActive||[],"mouseout"===e.type?t.active=[]:t.active=t.getElementsAtEventForMode(e,r.mode,r),ce.callback(n.onHover||n.hover.onHover,[e.native,t.active],t),"mouseup"!==e.type&&"click"!==e.type||n.onClick&&n.onClick.call(t,e.native,t.active),t.lastActive.length&&t.updateHoverStyle(t.lastActive,r.mode,!1),t.active.length&&r.mode&&t.updateHoverStyle(t.active,r.mode,!0),i=!ce.arrayEquals(t.active,t.lastActive),t.lastActive=t.active,i}}),Qn.instances={};var Zn=Qn;Qn.Controller=Qn,Qn.types={},ce.configMerge=Vn,ce.scaleMerge=Un;var er=function(){function e(e,t,n){var r;return"string"===typeof e?(r=parseInt(e,10),-1!==e.indexOf("%")&&(r=r/100*t.parentNode[n])):r=e,r}function t(e){return void 0!==e&&null!==e&&"none"!==e}function n(n,r,i){var a=document.defaultView,o=ce._getParentNode(n),s=a.getComputedStyle(n)[r],l=a.getComputedStyle(o)[r],u=t(s),c=t(l),d=Number.POSITIVE_INFINITY;return u||c?Math.min(u?e(s,n,i):d,c?e(l,o,i):d):"none"}ce.where=function(e,t){if(ce.isArray(e)&&Array.prototype.filter)return e.filter(t);var n=[];return ce.each(e,(function(e){t(e)&&n.push(e)})),n},ce.findIndex=Array.prototype.findIndex?function(e,t,n){return e.findIndex(t,n)}:function(e,t,n){n=void 0===n?e:n;for(var r=0,i=e.length;r<i;++r)if(t.call(n,e[r],r,e))return r;return-1},ce.findNextWhere=function(e,t,n){ce.isNullOrUndef(n)&&(n=-1);for(var r=n+1;r<e.length;r++){var i=e[r];if(t(i))return i}},ce.findPreviousWhere=function(e,t,n){ce.isNullOrUndef(n)&&(n=e.length);for(var r=n-1;r>=0;r--){var i=e[r];if(t(i))return i}},ce.isNumber=function(e){return!isNaN(parseFloat(e))&&isFinite(e)},ce.almostEquals=function(e,t,n){return Math.abs(e-t)<n},ce.almostWhole=function(e,t){var n=Math.round(e);return n-t<=e&&n+t>=e},ce.max=function(e){return e.reduce((function(e,t){return isNaN(t)?e:Math.max(e,t)}),Number.NEGATIVE_INFINITY)},ce.min=function(e){return e.reduce((function(e,t){return isNaN(t)?e:Math.min(e,t)}),Number.POSITIVE_INFINITY)},ce.sign=Math.sign?function(e){return Math.sign(e)}:function(e){return e=+e,0===e||isNaN(e)?e:e>0?1:-1},ce.toRadians=function(e){return e*(Math.PI/180)},ce.toDegrees=function(e){return e*(180/Math.PI)},ce._decimalPlaces=function(e){if(ce.isFinite(e)){var t=1,n=0;while(Math.round(e*t)/t!==e)t*=10,n++;return n}},ce.getAngleFromPoint=function(e,t){var n=t.x-e.x,r=t.y-e.y,i=Math.sqrt(n*n+r*r),a=Math.atan2(r,n);return a<-.5*Math.PI&&(a+=2*Math.PI),{angle:a,distance:i}},ce.distanceBetweenPoints=function(e,t){return Math.sqrt(Math.pow(t.x-e.x,2)+Math.pow(t.y-e.y,2))},ce.aliasPixel=function(e){return e%2===0?0:.5},ce._alignPixel=function(e,t,n){var r=e.currentDevicePixelRatio,i=n/2;return Math.round((t-i)*r)/r+i},ce.splineCurve=function(e,t,n,r){var i=e.skip?t:e,a=t,o=n.skip?t:n,s=Math.sqrt(Math.pow(a.x-i.x,2)+Math.pow(a.y-i.y,2)),l=Math.sqrt(Math.pow(o.x-a.x,2)+Math.pow(o.y-a.y,2)),u=s/(s+l),c=l/(s+l);u=isNaN(u)?0:u,c=isNaN(c)?0:c;var d=r*u,f=r*c;return{previous:{x:a.x-d*(o.x-i.x),y:a.y-d*(o.y-i.y)},next:{x:a.x+f*(o.x-i.x),y:a.y+f*(o.y-i.y)}}},ce.EPSILON=Number.EPSILON||1e-14,ce.splineCurveMonotone=function(e){var t,n,r,i,a,o,s,l,u,c=(e||[]).map((function(e){return{model:e._model,deltaK:0,mK:0}})),d=c.length;for(t=0;t<d;++t)if(r=c[t],!r.model.skip){if(n=t>0?c[t-1]:null,i=t<d-1?c[t+1]:null,i&&!i.model.skip){var f=i.model.x-r.model.x;r.deltaK=0!==f?(i.model.y-r.model.y)/f:0}!n||n.model.skip?r.mK=r.deltaK:!i||i.model.skip?r.mK=n.deltaK:this.sign(n.deltaK)!==this.sign(r.deltaK)?r.mK=0:r.mK=(n.deltaK+r.deltaK)/2}for(t=0;t<d-1;++t)r=c[t],i=c[t+1],r.model.skip||i.model.skip||(ce.almostEquals(r.deltaK,0,this.EPSILON)?r.mK=i.mK=0:(a=r.mK/r.deltaK,o=i.mK/r.deltaK,l=Math.pow(a,2)+Math.pow(o,2),l<=9||(s=3/Math.sqrt(l),r.mK=a*s*r.deltaK,i.mK=o*s*r.deltaK)));for(t=0;t<d;++t)r=c[t],r.model.skip||(n=t>0?c[t-1]:null,i=t<d-1?c[t+1]:null,n&&!n.model.skip&&(u=(r.model.x-n.model.x)/3,r.model.controlPointPreviousX=r.model.x-u,r.model.controlPointPreviousY=r.model.y-u*r.mK),i&&!i.model.skip&&(u=(i.model.x-r.model.x)/3,r.model.controlPointNextX=r.model.x+u,r.model.controlPointNextY=r.model.y+u*r.mK))},ce.nextItem=function(e,t,n){return n?t>=e.length-1?e[0]:e[t+1]:t>=e.length-1?e[e.length-1]:e[t+1]},ce.previousItem=function(e,t,n){return n?t<=0?e[e.length-1]:e[t-1]:t<=0?e[0]:e[t-1]},ce.niceNum=function(e,t){var n,r=Math.floor(ce.log10(e)),i=e/Math.pow(10,r);return n=t?i<1.5?1:i<3?2:i<7?5:10:i<=1?1:i<=2?2:i<=5?5:10,n*Math.pow(10,r)},ce.requestAnimFrame=function(){return"undefined"===typeof window?function(e){e()}:window.requestAnimationFrame||window.webkitRequestAnimationFrame||window.mozRequestAnimationFrame||window.oRequestAnimationFrame||window.msRequestAnimationFrame||function(e){return window.setTimeout(e,1e3/60)}}(),ce.getRelativePosition=function(e,t){var n,r,i=e.originalEvent||e,a=e.target||e.srcElement,o=a.getBoundingClientRect(),s=i.touches;s&&s.length>0?(n=s[0].clientX,r=s[0].clientY):(n=i.clientX,r=i.clientY);var l=parseFloat(ce.getStyle(a,"padding-left")),u=parseFloat(ce.getStyle(a,"padding-top")),c=parseFloat(ce.getStyle(a,"padding-right")),d=parseFloat(ce.getStyle(a,"padding-bottom")),f=o.right-o.left-l-c,h=o.bottom-o.top-u-d;return n=Math.round((n-o.left-l)/f*a.width/t.currentDevicePixelRatio),r=Math.round((r-o.top-u)/h*a.height/t.currentDevicePixelRatio),{x:n,y:r}},ce.getConstraintWidth=function(e){return n(e,"max-width","clientWidth")},ce.getConstraintHeight=function(e){return n(e,"max-height","clientHeight")},ce._calculatePadding=function(e,t,n){return t=ce.getStyle(e,t),t.indexOf("%")>-1?n*parseInt(t,10)/100:parseInt(t,10)},ce._getParentNode=function(e){var t=e.parentNode;return t&&"[object ShadowRoot]"===t.toString()&&(t=t.host),t},ce.getMaximumWidth=function(e){var t=ce._getParentNode(e);if(!t)return e.clientWidth;var n=t.clientWidth,r=ce._calculatePadding(t,"padding-left",n),i=ce._calculatePadding(t,"padding-right",n),a=n-r-i,o=ce.getConstraintWidth(e);return isNaN(o)?a:Math.min(a,o)},ce.getMaximumHeight=function(e){var t=ce._getParentNode(e);if(!t)return e.clientHeight;var n=t.clientHeight,r=ce._calculatePadding(t,"padding-top",n),i=ce._calculatePadding(t,"padding-bottom",n),a=n-r-i,o=ce.getConstraintHeight(e);return isNaN(o)?a:Math.min(a,o)},ce.getStyle=function(e,t){return e.currentStyle?e.currentStyle[t]:document.defaultView.getComputedStyle(e,null).getPropertyValue(t)},ce.retinaScale=function(e,t){var n=e.currentDevicePixelRatio=t||"undefined"!==typeof window&&window.devicePixelRatio||1;if(1!==n){var r=e.canvas,i=e.height,a=e.width;r.height=i*n,r.width=a*n,e.ctx.scale(n,n),r.style.height||r.style.width||(r.style.height=i+"px",r.style.width=a+"px")}},ce.fontString=function(e,t,n){return t+" "+e+"px "+n},ce.longestText=function(e,t,n,r){r=r||{};var i=r.data=r.data||{},a=r.garbageCollect=r.garbageCollect||[];r.font!==t&&(i=r.data={},a=r.garbageCollect=[],r.font=t),e.font=t;var o,s,l,u,c,d=0,f=n.length;for(o=0;o<f;o++)if(u=n[o],void 0!==u&&null!==u&&!0!==ce.isArray(u))d=ce.measureText(e,i,a,d,u);else if(ce.isArray(u))for(s=0,l=u.length;s<l;s++)c=u[s],void 0===c||null===c||ce.isArray(c)||(d=ce.measureText(e,i,a,d,c));var h=a.length/2;if(h>n.length){for(o=0;o<h;o++)delete i[a[o]];a.splice(0,h)}return d},ce.measureText=function(e,t,n,r,i){var a=t[i];return a||(a=t[i]=e.measureText(i).width,n.push(i)),a>r&&(r=a),r},ce.numberOfLabelLines=function(e){var t=1;return ce.each(e,(function(e){ce.isArray(e)&&e.length>t&&(t=e.length)})),t},ce.color=j?function(e){return e instanceof CanvasGradient&&(e=Q.global.defaultColor),j(e)}:function(e){return console.error("Color.js not found!"),e},ce.getHoverColor=function(e){return e instanceof CanvasPattern||e instanceof CanvasGradient?e:ce.color(e).saturate(.5).darken(.1).rgbString()}};function tr(){throw new Error("This method is not implemented: either no adapter can be found or an incomplete integration was provided.")}function nr(e){this.options=e||{}}ce.extend(nr.prototype,{formats:tr,parse:tr,format:tr,add:tr,diff:tr,startOf:tr,endOf:tr,_create:function(e){return e}}),nr.override=function(e){ce.extend(nr.prototype,e)};var rr=nr,ir={_date:rr},ar={formatters:{values:function(e){return ce.isArray(e)?e:""+e},linear:function(e,t,n){var r=n.length>3?n[2]-n[1]:n[1]-n[0];Math.abs(r)>1&&e!==Math.floor(e)&&(r=e-Math.floor(e));var i=ce.log10(Math.abs(r)),a="";if(0!==e){var o=Math.max(Math.abs(n[0]),Math.abs(n[n.length-1]));if(o<1e-4){var s=ce.log10(Math.abs(e)),l=Math.floor(s)-Math.floor(i);l=Math.max(Math.min(l,20),0),a=e.toExponential(l)}else{var u=-1*Math.floor(i);u=Math.max(Math.min(u,20),0),a=e.toFixed(u)}}else a="0";return a},logarithmic:function(e,t,n){var r=e/Math.pow(10,Math.floor(ce.log10(e)));return 0===e?"0":1===r||2===r||5===r||0===t||t===n.length-1?e.toExponential():""}}},or=ce.isArray,sr=ce.isNullOrUndef,lr=ce.valueOrDefault,ur=ce.valueAtIndexOrDefault;function cr(e,t){for(var n=[],r=e.length/t,i=0,a=e.length;i<a;i+=r)n.push(e[Math.floor(i)]);return n}function dr(e,t,n){var r,i=e.getTicks().length,a=Math.min(t,i-1),o=e.getPixelForTick(a),s=e._startPixel,l=e._endPixel,u=1e-6;if(!(n&&(r=1===i?Math.max(o-s,l-o):0===t?(e.getPixelForTick(1)-o)/2:(o-e.getPixelForTick(a-1))/2,o+=a<t?r:-r,o<s-u||o>l+u)))return o}function fr(e,t){ce.each(e,(function(e){var n,r=e.gc,i=r.length/2;if(i>t){for(n=0;n<i;++n)delete e.data[r[n]];r.splice(0,i)}}))}function hr(e,t,n,r){var i,a,o,s,l,u,c,d,f,h,p,m,_,v=n.length,g=[],b=[],y=[],M=0,k=0;for(i=0;i<v;++i){if(s=n[i].label,l=n[i].major?t.major:t.minor,e.font=u=l.string,c=r[u]=r[u]||{data:{},gc:[]},d=l.lineHeight,f=h=0,sr(s)||or(s)){if(or(s))for(a=0,o=s.length;a<o;++a)p=s[a],sr(p)||or(p)||(f=ce.measureText(e,c.data,c.gc,f,p),h+=d)}else f=ce.measureText(e,c.data,c.gc,f,s),h=d;g.push(f),b.push(h),y.push(d/2),M=Math.max(f,M),k=Math.max(h,k)}function w(e){return{width:g[e]||0,height:b[e]||0,offset:y[e]||0}}return fr(r,v),m=g.indexOf(M),_=b.indexOf(k),{first:w(0),last:w(v-1),widest:w(m),highest:w(_)}}function pr(e){return e.drawTicks?e.tickMarkLength:0}function mr(e){var t,n;return e.display?(t=ce.options._parseFont(e),n=ce.options.toPadding(e.padding),t.lineHeight+n.height):0}function _r(e,t){return ce.extend(ce.options._parseFont({fontFamily:lr(t.fontFamily,e.fontFamily),fontSize:lr(t.fontSize,e.fontSize),fontStyle:lr(t.fontStyle,e.fontStyle),lineHeight:lr(t.lineHeight,e.lineHeight)}),{color:ce.options.resolve([t.fontColor,e.fontColor,Q.global.defaultFontColor])})}function vr(e){var t=_r(e,e.minor),n=e.major.enabled?_r(e,e.major):t;return{minor:t,major:n}}function gr(e){var t,n,r,i=[];for(n=0,r=e.length;n<r;++n)t=e[n],"undefined"!==typeof t._index&&i.push(t);return i}function br(e){var t,n,r=e.length;if(r<2)return!1;for(n=e[0],t=1;t<r;++t)if(e[t]-e[t-1]!==n)return!1;return n}function yr(e,t,n,r){var i,a,o,s,l=br(e),u=(t.length-1)/r;if(!l)return Math.max(u,1);for(i=ce.math._factorize(l),o=0,s=i.length-1;o<s;o++)if(a=i[o],a>u)return a;return Math.max(u,1)}function Mr(e){var t,n,r=[];for(t=0,n=e.length;t<n;t++)e[t].major&&r.push(t);return r}function kr(e,t,n){var r,i,a=0,o=t[0];for(n=Math.ceil(n),r=0;r<e.length;r++)i=e[r],r===o?(i._index=r,a++,o=t[a*n]):delete i.label}function wr(e,t,n,r){var i,a,o,s,l=lr(n,0),u=Math.min(lr(r,e.length),e.length),c=0;t=Math.ceil(t),r&&(i=r-n,t=i/Math.floor(i/t)),s=l;while(s<0)c++,s=Math.round(l+c*t);for(a=Math.max(l,0);a<u;a++)o=e[a],a===s?(o._index=a,c++,s=Math.round(l+c*t)):delete o.label}Q._set("scale",{display:!0,position:"left",offset:!1,gridLines:{display:!0,color:"rgba(0,0,0,0.1)",lineWidth:1,drawBorder:!0,drawOnChartArea:!0,drawTicks:!0,tickMarkLength:10,zeroLineWidth:1,zeroLineColor:"rgba(0,0,0,0.25)",zeroLineBorderDash:[],zeroLineBorderDashOffset:0,offsetGridLines:!1,borderDash:[],borderDashOffset:0},scaleLabel:{display:!1,labelString:"",padding:{top:4,bottom:4}},ticks:{beginAtZero:!1,minRotation:0,maxRotation:50,mirror:!1,padding:0,reverse:!1,display:!0,autoSkip:!0,autoSkipPadding:0,labelOffset:0,callback:ar.formatters.values,minor:{},major:{}}});var Lr=ge.extend({zeroLineIndex:0,getPadding:function(){var e=this;return{left:e.paddingLeft||0,top:e.paddingTop||0,right:e.paddingRight||0,bottom:e.paddingBottom||0}},getTicks:function(){return this._ticks},_getLabels:function(){var e=this.chart.data;return this.options.labels||(this.isHorizontal()?e.xLabels:e.yLabels)||e.labels||[]},mergeTicksOptions:function(){},beforeUpdate:function(){ce.callback(this.options.beforeUpdate,[this])},update:function(e,t,n){var r,i,a,o,s,l=this,u=l.options.ticks,c=u.sampleSize;if(l.beforeUpdate(),l.maxWidth=e,l.maxHeight=t,l.margins=ce.extend({left:0,right:0,top:0,bottom:0},n),l._ticks=null,l.ticks=null,l._labelSizes=null,l._maxLabelLines=0,l.longestLabelWidth=0,l.longestTextCache=l.longestTextCache||{},l._gridLineItems=null,l._labelItems=null,l.beforeSetDimensions(),l.setDimensions(),l.afterSetDimensions(),l.beforeDataLimits(),l.determineDataLimits(),l.afterDataLimits(),l.beforeBuildTicks(),o=l.buildTicks()||[],o=l.afterBuildTicks(o)||o,(!o||!o.length)&&l.ticks)for(o=[],r=0,i=l.ticks.length;r<i;++r)o.push({value:l.ticks[r],major:!1});return l._ticks=o,s=c<o.length,a=l._convertTicksToLabels(s?cr(o,c):o),l._configure(),l.beforeCalculateTickRotation(),l.calculateTickRotation(),l.afterCalculateTickRotation(),l.beforeFit(),l.fit(),l.afterFit(),l._ticksToDraw=u.display&&(u.autoSkip||"auto"===u.source)?l._autoSkip(o):o,s&&(a=l._convertTicksToLabels(l._ticksToDraw)),l.ticks=a,l.afterUpdate(),l.minSize},_configure:function(){var e,t,n=this,r=n.options.ticks.reverse;n.isHorizontal()?(e=n.left,t=n.right):(e=n.top,t=n.bottom,r=!r),n._startPixel=e,n._endPixel=t,n._reversePixels=r,n._length=t-e},afterUpdate:function(){ce.callback(this.options.afterUpdate,[this])},beforeSetDimensions:function(){ce.callback(this.options.beforeSetDimensions,[this])},setDimensions:function(){var e=this;e.isHorizontal()?(e.width=e.maxWidth,e.left=0,e.right=e.width):(e.height=e.maxHeight,e.top=0,e.bottom=e.height),e.paddingLeft=0,e.paddingTop=0,e.paddingRight=0,e.paddingBottom=0},afterSetDimensions:function(){ce.callback(this.options.afterSetDimensions,[this])},beforeDataLimits:function(){ce.callback(this.options.beforeDataLimits,[this])},determineDataLimits:ce.noop,afterDataLimits:function(){ce.callback(this.options.afterDataLimits,[this])},beforeBuildTicks:function(){ce.callback(this.options.beforeBuildTicks,[this])},buildTicks:ce.noop,afterBuildTicks:function(e){var t=this;return or(e)&&e.length?ce.callback(t.options.afterBuildTicks,[t,e]):(t.ticks=ce.callback(t.options.afterBuildTicks,[t,t.ticks])||t.ticks,e)},beforeTickToLabelConversion:function(){ce.callback(this.options.beforeTickToLabelConversion,[this])},convertTicksToLabels:function(){var e=this,t=e.options.ticks;e.ticks=e.ticks.map(t.userCallback||t.callback,this)},afterTickToLabelConversion:function(){ce.callback(this.options.afterTickToLabelConversion,[this])},beforeCalculateTickRotation:function(){ce.callback(this.options.beforeCalculateTickRotation,[this])},calculateTickRotation:function(){var e,t,n,r,i,a,o,s=this,l=s.options,u=l.ticks,c=s.getTicks().length,d=u.minRotation||0,f=u.maxRotation,h=d;!s._isVisible()||!u.display||d>=f||c<=1||!s.isHorizontal()?s.labelRotation=d:(e=s._getLabelSizes(),t=e.widest.width,n=e.highest.height-e.highest.offset,r=Math.min(s.maxWidth,s.chart.width-t),i=l.offset?s.maxWidth/c:r/(c-1),t+6>i&&(i=r/(c-(l.offset?.5:1)),a=s.maxHeight-pr(l.gridLines)-u.padding-mr(l.scaleLabel),o=Math.sqrt(t*t+n*n),h=ce.toDegrees(Math.min(Math.asin(Math.min((e.highest.height+6)/i,1)),Math.asin(Math.min(a/o,1))-Math.asin(n/o))),h=Math.max(d,Math.min(f,h))),s.labelRotation=h)},afterCalculateTickRotation:function(){ce.callback(this.options.afterCalculateTickRotation,[this])},beforeFit:function(){ce.callback(this.options.beforeFit,[this])},fit:function(){var e=this,t=e.minSize={width:0,height:0},n=e.chart,r=e.options,i=r.ticks,a=r.scaleLabel,o=r.gridLines,s=e._isVisible(),l="bottom"===r.position,u=e.isHorizontal();if(u?t.width=e.maxWidth:s&&(t.width=pr(o)+mr(a)),u?s&&(t.height=pr(o)+mr(a)):t.height=e.maxHeight,i.display&&s){var c=vr(i),d=e._getLabelSizes(),f=d.first,h=d.last,p=d.widest,m=d.highest,_=.4*c.minor.lineHeight,v=i.padding;if(u){var g=0!==e.labelRotation,b=ce.toRadians(e.labelRotation),y=Math.cos(b),M=Math.sin(b),k=M*p.width+y*(m.height-(g?m.offset:0))+(g?0:_);t.height=Math.min(e.maxHeight,t.height+k+v);var w,L,x=e.getPixelForTick(0)-e.left,S=e.right-e.getPixelForTick(e.getTicks().length-1);g?(w=l?y*f.width+M*f.offset:M*(f.height-f.offset),L=l?M*(h.height-h.offset):y*h.width+M*h.offset):(w=f.width/2,L=h.width/2),e.paddingLeft=Math.max((w-x)*e.width/(e.width-x),0)+3,e.paddingRight=Math.max((L-S)*e.width/(e.width-S),0)+3}else{var T=i.mirror?0:p.width+v+_;t.width=Math.min(e.maxWidth,t.width+T),e.paddingTop=f.height/2,e.paddingBottom=h.height/2}}e.handleMargins(),u?(e.width=e._length=n.width-e.margins.left-e.margins.right,e.height=t.height):(e.width=t.width,e.height=e._length=n.height-e.margins.top-e.margins.bottom)},handleMargins:function(){var e=this;e.margins&&(e.margins.left=Math.max(e.paddingLeft,e.margins.left),e.margins.top=Math.max(e.paddingTop,e.margins.top),e.margins.right=Math.max(e.paddingRight,e.margins.right),e.margins.bottom=Math.max(e.paddingBottom,e.margins.bottom))},afterFit:function(){ce.callback(this.options.afterFit,[this])},isHorizontal:function(){var e=this.options.position;return"top"===e||"bottom"===e},isFullWidth:function(){return this.options.fullWidth},getRightValue:function(e){if(sr(e))return NaN;if(("number"===typeof e||e instanceof Number)&&!isFinite(e))return NaN;if(e)if(this.isHorizontal()){if(void 0!==e.x)return this.getRightValue(e.x)}else if(void 0!==e.y)return this.getRightValue(e.y);return e},_convertTicksToLabels:function(e){var t,n,r,i=this;for(i.ticks=e.map((function(e){return e.value})),i.beforeTickToLabelConversion(),t=i.convertTicksToLabels(e)||i.ticks,i.afterTickToLabelConversion(),n=0,r=e.length;n<r;++n)e[n].label=t[n];return t},_getLabelSizes:function(){var e=this,t=e._labelSizes;return t||(e._labelSizes=t=hr(e.ctx,vr(e.options.ticks),e.getTicks(),e.longestTextCache),e.longestLabelWidth=t.widest.width),t},_parseValue:function(e){var t,n,r,i;return or(e)?(t=+this.getRightValue(e[0]),n=+this.getRightValue(e[1]),r=Math.min(t,n),i=Math.max(t,n)):(e=+this.getRightValue(e),t=void 0,n=e,r=e,i=e),{min:r,max:i,start:t,end:n}},_getScaleLabel:function(e){var t=this._parseValue(e);return void 0!==t.start?"["+t.start+", "+t.end+"]":+this.getRightValue(e)},getLabelForIndex:ce.noop,getPixelForValue:ce.noop,getValueForPixel:ce.noop,getPixelForTick:function(e){var t=this,n=t.options.offset,r=t._ticks.length,i=1/Math.max(r-(n?0:1),1);return e<0||e>r-1?null:t.getPixelForDecimal(e*i+(n?i/2:0))},getPixelForDecimal:function(e){var t=this;return t._reversePixels&&(e=1-e),t._startPixel+e*t._length},getDecimalForPixel:function(e){var t=(e-this._startPixel)/this._length;return this._reversePixels?1-t:t},getBasePixel:function(){return this.getPixelForValue(this.getBaseValue())},getBaseValue:function(){var e=this,t=e.min,n=e.max;return e.beginAtZero?0:t<0&&n<0?n:t>0&&n>0?t:0},_autoSkip:function(e){var t,n,r,i,a=this,o=a.options.ticks,s=a._length,l=o.maxTicksLimit||s/a._tickSize()+1,u=o.major.enabled?Mr(e):[],c=u.length,d=u[0],f=u[c-1];if(c>l)return kr(e,u,c/l),gr(e);if(r=yr(u,e,s,l),c>0){for(t=0,n=c-1;t<n;t++)wr(e,r,u[t],u[t+1]);return i=c>1?(f-d)/(c-1):null,wr(e,r,ce.isNullOrUndef(i)?0:d-i,d),wr(e,r,f,ce.isNullOrUndef(i)?e.length:f+i),gr(e)}return wr(e,r),gr(e)},_tickSize:function(){var e=this,t=e.options.ticks,n=ce.toRadians(e.labelRotation),r=Math.abs(Math.cos(n)),i=Math.abs(Math.sin(n)),a=e._getLabelSizes(),o=t.autoSkipPadding||0,s=a?a.widest.width+o:0,l=a?a.highest.height+o:0;return e.isHorizontal()?l*r>s*i?s/r:l/i:l*i<s*r?l/r:s/i},_isVisible:function(){var e,t,n,r=this,i=r.chart,a=r.options.display;if("auto"!==a)return!!a;for(e=0,t=i.data.datasets.length;e<t;++e)if(i.isDatasetVisible(e)&&(n=i.getDatasetMeta(e),n.xAxisID===r.id||n.yAxisID===r.id))return!0;return!1},_computeGridLineItems:function(e){var t,n,r,i,a,o,s,l,u,c,d,f,h,p,m,_,v,g=this,b=g.chart,y=g.options,M=y.gridLines,k=y.position,w=M.offsetGridLines,L=g.isHorizontal(),x=g._ticksToDraw,S=x.length+(w?1:0),T=pr(M),D=[],Y=M.drawBorder?ur(M.lineWidth,0,0):0,C=Y/2,E=ce._alignPixel,A=function(e){return E(b,e,Y)};for("top"===k?(t=A(g.bottom),s=g.bottom-T,u=t-C,d=A(e.top)+C,h=e.bottom):"bottom"===k?(t=A(g.top),d=e.top,h=A(e.bottom)-C,s=t+C,u=g.top+T):"left"===k?(t=A(g.right),o=g.right-T,l=t-C,c=A(e.left)+C,f=e.right):(t=A(g.left),c=e.left,f=A(e.right)-C,o=t+C,l=g.left+T),n=0;n<S;++n)r=x[n]||{},sr(r.label)&&n<x.length||(n===g.zeroLineIndex&&y.offset===w?(p=M.zeroLineWidth,m=M.zeroLineColor,_=M.zeroLineBorderDash||[],v=M.zeroLineBorderDashOffset||0):(p=ur(M.lineWidth,n,1),m=ur(M.color,n,"rgba(0,0,0,0.1)"),_=M.borderDash||[],v=M.borderDashOffset||0),i=dr(g,r._index||n,w),void 0!==i&&(a=E(b,i,p),L?o=l=c=f=a:s=u=d=h=a,D.push({tx1:o,ty1:s,tx2:l,ty2:u,x1:c,y1:d,x2:f,y2:h,width:p,color:m,borderDash:_,borderDashOffset:v})));return D.ticksLength=S,D.borderValue=t,D},_computeLabelItems:function(){var e,t,n,r,i,a,o,s,l,u,c,d,f=this,h=f.options,p=h.ticks,m=h.position,_=p.mirror,v=f.isHorizontal(),g=f._ticksToDraw,b=vr(p),y=p.padding,M=pr(h.gridLines),k=-ce.toRadians(f.labelRotation),w=[];for("top"===m?(a=f.bottom-M-y,o=k?"left":"center"):"bottom"===m?(a=f.top+M+y,o=k?"right":"center"):"left"===m?(i=f.right-(_?0:M)-y,o=_?"left":"right"):(i=f.left+(_?0:M)+y,o=_?"right":"left"),e=0,t=g.length;e<t;++e)n=g[e],r=n.label,sr(r)||(s=f.getPixelForTick(n._index||e)+p.labelOffset,l=n.major?b.major:b.minor,u=l.lineHeight,c=or(r)?r.length:1,v?(i=s,d="top"===m?((k?1:.5)-c)*u:(k?0:.5)*u):(a=s,d=(1-c)*u/2),w.push({x:i,y:a,rotation:k,label:r,font:l,textOffset:d,textAlign:o}));return w},_drawGrid:function(e){var t=this,n=t.options.gridLines;if(n.display){var r,i,a,o,s,l=t.ctx,u=t.chart,c=ce._alignPixel,d=n.drawBorder?ur(n.lineWidth,0,0):0,f=t._gridLineItems||(t._gridLineItems=t._computeGridLineItems(e));for(a=0,o=f.length;a<o;++a)s=f[a],r=s.width,i=s.color,r&&i&&(l.save(),l.lineWidth=r,l.strokeStyle=i,l.setLineDash&&(l.setLineDash(s.borderDash),l.lineDashOffset=s.borderDashOffset),l.beginPath(),n.drawTicks&&(l.moveTo(s.tx1,s.ty1),l.lineTo(s.tx2,s.ty2)),n.drawOnChartArea&&(l.moveTo(s.x1,s.y1),l.lineTo(s.x2,s.y2)),l.stroke(),l.restore());if(d){var h,p,m,_,v=d,g=ur(n.lineWidth,f.ticksLength-1,1),b=f.borderValue;t.isHorizontal()?(h=c(u,t.left,v)-v/2,p=c(u,t.right,g)+g/2,m=_=b):(m=c(u,t.top,v)-v/2,_=c(u,t.bottom,g)+g/2,h=p=b),l.lineWidth=d,l.strokeStyle=ur(n.color,0),l.beginPath(),l.moveTo(h,m),l.lineTo(p,_),l.stroke()}}},_drawLabels:function(){var e=this,t=e.options.ticks;if(t.display){var n,r,i,a,o,s,l,u,c=e.ctx,d=e._labelItems||(e._labelItems=e._computeLabelItems());for(n=0,i=d.length;n<i;++n){if(o=d[n],s=o.font,c.save(),c.translate(o.x,o.y),c.rotate(o.rotation),c.font=s.string,c.fillStyle=s.color,c.textBaseline="middle",c.textAlign=o.textAlign,l=o.label,u=o.textOffset,or(l))for(r=0,a=l.length;r<a;++r)c.fillText(""+l[r],0,u),u+=s.lineHeight;else c.fillText(l,0,u);c.restore()}}},_drawTitle:function(){var e=this,t=e.ctx,n=e.options,r=n.scaleLabel;if(r.display){var i,a,o=lr(r.fontColor,Q.global.defaultFontColor),s=ce.options._parseFont(r),l=ce.options.toPadding(r.padding),u=s.lineHeight/2,c=n.position,d=0;if(e.isHorizontal())i=e.left+e.width/2,a="bottom"===c?e.bottom-u-l.bottom:e.top+u+l.top;else{var f="left"===c;i=f?e.left+u+l.top:e.right-u-l.top,a=e.top+e.height/2,d=f?-.5*Math.PI:.5*Math.PI}t.save(),t.translate(i,a),t.rotate(d),t.textAlign="center",t.textBaseline="middle",t.fillStyle=o,t.font=s.string,t.fillText(r.labelString,0,0),t.restore()}},draw:function(e){var t=this;t._isVisible()&&(t._drawGrid(e),t._drawTitle(),t._drawLabels())},_layers:function(){var e=this,t=e.options,n=t.ticks&&t.ticks.z||0,r=t.gridLines&&t.gridLines.z||0;return e._isVisible()&&n!==r&&e.draw===e._draw?[{z:r,draw:function(){e._drawGrid.apply(e,arguments),e._drawTitle.apply(e,arguments)}},{z:n,draw:function(){e._drawLabels.apply(e,arguments)}}]:[{z:n,draw:function(){e.draw.apply(e,arguments)}}]},_getMatchingVisibleMetas:function(e){var t=this,n=t.isHorizontal();return t.chart._getSortedVisibleDatasetMetas().filter((function(r){return(!e||r.type===e)&&(n?r.xAxisID===t.id:r.yAxisID===t.id)}))}});Lr.prototype._draw=Lr.prototype.draw;var xr=Lr,Sr=ce.isNullOrUndef,Tr={position:"bottom"},Dr=xr.extend({determineDataLimits:function(){var e,t=this,n=t._getLabels(),r=t.options.ticks,i=r.min,a=r.max,o=0,s=n.length-1;void 0!==i&&(e=n.indexOf(i),e>=0&&(o=e)),void 0!==a&&(e=n.indexOf(a),e>=0&&(s=e)),t.minIndex=o,t.maxIndex=s,t.min=n[o],t.max=n[s]},buildTicks:function(){var e=this,t=e._getLabels(),n=e.minIndex,r=e.maxIndex;e.ticks=0===n&&r===t.length-1?t:t.slice(n,r+1)},getLabelForIndex:function(e,t){var n=this,r=n.chart;return r.getDatasetMeta(t).controller._getValueScaleId()===n.id?n.getRightValue(r.data.datasets[t].data[e]):n._getLabels()[e]},_configure:function(){var e=this,t=e.options.offset,n=e.ticks;xr.prototype._configure.call(e),e.isHorizontal()||(e._reversePixels=!e._reversePixels),n&&(e._startValue=e.minIndex-(t?.5:0),e._valueRange=Math.max(n.length-(t?0:1),1))},getPixelForValue:function(e,t,n){var r,i,a,o=this;return Sr(t)||Sr(n)||(e=o.chart.data.datasets[n].data[t]),Sr(e)||(r=o.isHorizontal()?e.x:e.y),(void 0!==r||void 0!==e&&isNaN(t))&&(i=o._getLabels(),e=ce.valueOrDefault(r,e),a=i.indexOf(e),t=-1!==a?a:t,isNaN(t)&&(t=e)),o.getPixelForDecimal((t-o._startValue)/o._valueRange)},getPixelForTick:function(e){var t=this.ticks;return e<0||e>t.length-1?null:this.getPixelForValue(t[e],e+this.minIndex)},getValueForPixel:function(e){var t=this,n=Math.round(t._startValue+t.getDecimalForPixel(e)*t._valueRange);return Math.min(Math.max(n,0),t.ticks.length-1)},getBasePixel:function(){return this.bottom}}),Yr=Tr;Dr._defaults=Yr;var Cr=ce.noop,Er=ce.isNullOrUndef;function Ar(e,t){var n,r,i,a,o=[],s=1e-14,l=e.stepSize,u=l||1,c=e.maxTicks-1,d=e.min,f=e.max,h=e.precision,p=t.min,m=t.max,_=ce.niceNum((m-p)/c/u)*u;if(_<s&&Er(d)&&Er(f))return[p,m];a=Math.ceil(m/_)-Math.floor(p/_),a>c&&(_=ce.niceNum(a*_/c/u)*u),l||Er(h)?n=Math.pow(10,ce._decimalPlaces(_)):(n=Math.pow(10,h),_=Math.ceil(_*n)/n),r=Math.floor(p/_)*_,i=Math.ceil(m/_)*_,l&&(!Er(d)&&ce.almostWhole(d/_,_/1e3)&&(r=d),!Er(f)&&ce.almostWhole(f/_,_/1e3)&&(i=f)),a=(i-r)/_,a=ce.almostEquals(a,Math.round(a),_/1e3)?Math.round(a):Math.ceil(a),r=Math.round(r*n)/n,i=Math.round(i*n)/n,o.push(Er(d)?r:d);for(var v=1;v<a;++v)o.push(Math.round((r+v*_)*n)/n);return o.push(Er(f)?i:f),o}var Or=xr.extend({getRightValue:function(e){return"string"===typeof e?+e:xr.prototype.getRightValue.call(this,e)},handleTickRangeOptions:function(){var e=this,t=e.options,n=t.ticks;if(n.beginAtZero){var r=ce.sign(e.min),i=ce.sign(e.max);r<0&&i<0?e.max=0:r>0&&i>0&&(e.min=0)}var a=void 0!==n.min||void 0!==n.suggestedMin,o=void 0!==n.max||void 0!==n.suggestedMax;void 0!==n.min?e.min=n.min:void 0!==n.suggestedMin&&(null===e.min?e.min=n.suggestedMin:e.min=Math.min(e.min,n.suggestedMin)),void 0!==n.max?e.max=n.max:void 0!==n.suggestedMax&&(null===e.max?e.max=n.suggestedMax:e.max=Math.max(e.max,n.suggestedMax)),a!==o&&e.min>=e.max&&(a?e.max=e.min+1:e.min=e.max-1),e.min===e.max&&(e.max++,n.beginAtZero||e.min--)},getTickLimit:function(){var e,t=this,n=t.options.ticks,r=n.stepSize,i=n.maxTicksLimit;return r?e=Math.ceil(t.max/r)-Math.floor(t.min/r)+1:(e=t._computeTickLimit(),i=i||11),i&&(e=Math.min(i,e)),e},_computeTickLimit:function(){return Number.POSITIVE_INFINITY},handleDirectionalChanges:Cr,buildTicks:function(){var e=this,t=e.options,n=t.ticks,r=e.getTickLimit();r=Math.max(2,r);var i={maxTicks:r,min:n.min,max:n.max,precision:n.precision,stepSize:ce.valueOrDefault(n.fixedStepSize,n.stepSize)},a=e.ticks=Ar(i,e);e.handleDirectionalChanges(),e.max=ce.max(a),e.min=ce.min(a),n.reverse?(a.reverse(),e.start=e.max,e.end=e.min):(e.start=e.min,e.end=e.max)},convertTicksToLabels:function(){var e=this;e.ticksAsNumbers=e.ticks.slice(),e.zeroLineIndex=e.ticks.indexOf(0),xr.prototype.convertTicksToLabels.call(e)},_configure:function(){var e,t=this,n=t.getTicks(),r=t.min,i=t.max;xr.prototype._configure.call(t),t.options.offset&&n.length&&(e=(i-r)/Math.max(n.length-1,1)/2,r-=e,i+=e),t._startValue=r,t._endValue=i,t._valueRange=i-r}}),Pr={position:"left",ticks:{callback:ar.formatters.linear}},Fr=0,Ir=1;function jr(e,t,n){var r=[n.type,void 0===t&&void 0===n.stack?n.index:"",n.stack].join(".");return void 0===e[r]&&(e[r]={pos:[],neg:[]}),e[r]}function Hr(e,t,n,r){var i,a,o=e.options,s=o.stacked,l=jr(t,s,n),u=l.pos,c=l.neg,d=r.length;for(i=0;i<d;++i)a=e._parseValue(r[i]),isNaN(a.min)||isNaN(a.max)||n.data[i].hidden||(u[i]=u[i]||0,c[i]=c[i]||0,o.relativePoints?u[i]=100:a.min<0||a.max<0?c[i]+=a.min:u[i]+=a.max)}function Rr(e,t,n){var r,i,a=n.length;for(r=0;r<a;++r)i=e._parseValue(n[r]),isNaN(i.min)||isNaN(i.max)||t.data[r].hidden||(e.min=Math.min(e.min,i.min),e.max=Math.max(e.max,i.max))}var Nr=Or.extend({determineDataLimits:function(){var e,t,n,r,i=this,a=i.options,o=i.chart,s=o.data.datasets,l=i._getMatchingVisibleMetas(),u=a.stacked,c={},d=l.length;if(i.min=Number.POSITIVE_INFINITY,i.max=Number.NEGATIVE_INFINITY,void 0===u)for(e=0;!u&&e<d;++e)t=l[e],u=void 0!==t.stack;for(e=0;e<d;++e)t=l[e],n=s[t.index].data,u?Hr(i,c,t,n):Rr(i,t,n);ce.each(c,(function(e){r=e.pos.concat(e.neg),i.min=Math.min(i.min,ce.min(r)),i.max=Math.max(i.max,ce.max(r))})),i.min=ce.isFinite(i.min)&&!isNaN(i.min)?i.min:Fr,i.max=ce.isFinite(i.max)&&!isNaN(i.max)?i.max:Ir,i.handleTickRangeOptions()},_computeTickLimit:function(){var e,t=this;return t.isHorizontal()?Math.ceil(t.width/40):(e=ce.options._parseFont(t.options.ticks),Math.ceil(t.height/e.lineHeight))},handleDirectionalChanges:function(){this.isHorizontal()||this.ticks.reverse()},getLabelForIndex:function(e,t){return this._getScaleLabel(this.chart.data.datasets[t].data[e])},getPixelForValue:function(e){var t=this;return t.getPixelForDecimal((+t.getRightValue(e)-t._startValue)/t._valueRange)},getValueForPixel:function(e){return this._startValue+this.getDecimalForPixel(e)*this._valueRange},getPixelForTick:function(e){var t=this.ticksAsNumbers;return e<0||e>t.length-1?null:this.getPixelForValue(t[e])}}),$r=Pr;Nr._defaults=$r;var zr=ce.valueOrDefault,Br=ce.math.log10;function Wr(e,t){var n,r,i=[],a=zr(e.min,Math.pow(10,Math.floor(Br(t.min)))),o=Math.floor(Br(t.max)),s=Math.ceil(t.max/Math.pow(10,o));0===a?(n=Math.floor(Br(t.minNotZero)),r=Math.floor(t.minNotZero/Math.pow(10,n)),i.push(a),a=r*Math.pow(10,n)):(n=Math.floor(Br(a)),r=Math.floor(a/Math.pow(10,n)));var l=n<0?Math.pow(10,Math.abs(n)):1;do{i.push(a),++r,10===r&&(r=1,++n,l=n>=0?1:l),a=Math.round(r*Math.pow(10,n)*l)/l}while(n<o||n===o&&r<s);var u=zr(e.max,a);return i.push(u),i}var Ur={position:"left",ticks:{callback:ar.formatters.logarithmic}};function Vr(e,t){return ce.isFinite(e)&&e>=0?e:t}var qr=xr.extend({determineDataLimits:function(){var e,t,n,r,i,a,o=this,s=o.options,l=o.chart,u=l.data.datasets,c=o.isHorizontal();function d(e){return c?e.xAxisID===o.id:e.yAxisID===o.id}o.min=Number.POSITIVE_INFINITY,o.max=Number.NEGATIVE_INFINITY,o.minNotZero=Number.POSITIVE_INFINITY;var f=s.stacked;if(void 0===f)for(e=0;e<u.length;e++)if(t=l.getDatasetMeta(e),l.isDatasetVisible(e)&&d(t)&&void 0!==t.stack){f=!0;break}if(s.stacked||f){var h={};for(e=0;e<u.length;e++){t=l.getDatasetMeta(e);var p=[t.type,void 0===s.stacked&&void 0===t.stack?e:"",t.stack].join(".");if(l.isDatasetVisible(e)&&d(t))for(void 0===h[p]&&(h[p]=[]),r=u[e].data,i=0,a=r.length;i<a;i++){var m=h[p];n=o._parseValue(r[i]),isNaN(n.min)||isNaN(n.max)||t.data[i].hidden||n.min<0||n.max<0||(m[i]=m[i]||0,m[i]+=n.max)}}ce.each(h,(function(e){if(e.length>0){var t=ce.min(e),n=ce.max(e);o.min=Math.min(o.min,t),o.max=Math.max(o.max,n)}}))}else for(e=0;e<u.length;e++)if(t=l.getDatasetMeta(e),l.isDatasetVisible(e)&&d(t))for(r=u[e].data,i=0,a=r.length;i<a;i++)n=o._parseValue(r[i]),isNaN(n.min)||isNaN(n.max)||t.data[i].hidden||n.min<0||n.max<0||(o.min=Math.min(n.min,o.min),o.max=Math.max(n.max,o.max),0!==n.min&&(o.minNotZero=Math.min(n.min,o.minNotZero)));o.min=ce.isFinite(o.min)?o.min:null,o.max=ce.isFinite(o.max)?o.max:null,o.minNotZero=ce.isFinite(o.minNotZero)?o.minNotZero:null,this.handleTickRangeOptions()},handleTickRangeOptions:function(){var e=this,t=e.options.ticks,n=1,r=10;e.min=Vr(t.min,e.min),e.max=Vr(t.max,e.max),e.min===e.max&&(0!==e.min&&null!==e.min?(e.min=Math.pow(10,Math.floor(Br(e.min))-1),e.max=Math.pow(10,Math.floor(Br(e.max))+1)):(e.min=n,e.max=r)),null===e.min&&(e.min=Math.pow(10,Math.floor(Br(e.max))-1)),null===e.max&&(e.max=0!==e.min?Math.pow(10,Math.floor(Br(e.min))+1):r),null===e.minNotZero&&(e.min>0?e.minNotZero=e.min:e.max<1?e.minNotZero=Math.pow(10,Math.floor(Br(e.max))):e.minNotZero=n)},buildTicks:function(){var e=this,t=e.options.ticks,n=!e.isHorizontal(),r={min:Vr(t.min),max:Vr(t.max)},i=e.ticks=Wr(r,e);e.max=ce.max(i),e.min=ce.min(i),t.reverse?(n=!n,e.start=e.max,e.end=e.min):(e.start=e.min,e.end=e.max),n&&i.reverse()},convertTicksToLabels:function(){this.tickValues=this.ticks.slice(),xr.prototype.convertTicksToLabels.call(this)},getLabelForIndex:function(e,t){return this._getScaleLabel(this.chart.data.datasets[t].data[e])},getPixelForTick:function(e){var t=this.tickValues;return e<0||e>t.length-1?null:this.getPixelForValue(t[e])},_getFirstTickValue:function(e){var t=Math.floor(Br(e)),n=Math.floor(e/Math.pow(10,t));return n*Math.pow(10,t)},_configure:function(){var e=this,t=e.min,n=0;xr.prototype._configure.call(e),0===t&&(t=e._getFirstTickValue(e.minNotZero),n=zr(e.options.ticks.fontSize,Q.global.defaultFontSize)/e._length),e._startValue=Br(t),e._valueOffset=n,e._valueRange=(Br(e.max)-Br(t))/(1-n)},getPixelForValue:function(e){var t=this,n=0;return e=+t.getRightValue(e),e>t.min&&e>0&&(n=(Br(e)-t._startValue)/t._valueRange+t._valueOffset),t.getPixelForDecimal(n)},getValueForPixel:function(e){var t=this,n=t.getDecimalForPixel(e);return 0===n&&0===t.min?0:Math.pow(10,t._startValue+(n-t._valueOffset)*t._valueRange)}}),Gr=Ur;qr._defaults=Gr;var Jr=ce.valueOrDefault,Kr=ce.valueAtIndexOrDefault,Xr=ce.options.resolve,Qr={display:!0,animate:!0,position:"chartArea",angleLines:{display:!0,color:"rgba(0,0,0,0.1)",lineWidth:1,borderDash:[],borderDashOffset:0},gridLines:{circular:!1},ticks:{showLabelBackdrop:!0,backdropColor:"rgba(255,255,255,0.75)",backdropPaddingY:2,backdropPaddingX:2,callback:ar.formatters.linear},pointLabels:{display:!0,fontSize:10,callback:function(e){return e}}};function Zr(e){var t=e.ticks;return t.display&&e.display?Jr(t.fontSize,Q.global.defaultFontSize)+2*t.backdropPaddingY:0}function ei(e,t,n){return ce.isArray(n)?{w:ce.longestText(e,e.font,n),h:n.length*t}:{w:e.measureText(n).width,h:t}}function ti(e,t,n,r,i){return e===r||e===i?{start:t-n/2,end:t+n/2}:e<r||e>i?{start:t-n,end:t}:{start:t,end:t+n}}function ni(e){var t,n,r,i=ce.options._parseFont(e.options.pointLabels),a={l:0,r:e.width,t:0,b:e.height-e.paddingTop},o={};e.ctx.font=i.string,e._pointLabelSizes=[];var s=e.chart.data.labels.length;for(t=0;t<s;t++){r=e.getPointPosition(t,e.drawingArea+5),n=ei(e.ctx,i.lineHeight,e.pointLabels[t]),e._pointLabelSizes[t]=n;var l=e.getIndexAngle(t),u=ce.toDegrees(l)%360,c=ti(u,r.x,n.w,0,180),d=ti(u,r.y,n.h,90,270);c.start<a.l&&(a.l=c.start,o.l=l),c.end>a.r&&(a.r=c.end,o.r=l),d.start<a.t&&(a.t=d.start,o.t=l),d.end>a.b&&(a.b=d.end,o.b=l)}e.setReductions(e.drawingArea,a,o)}function ri(e){return 0===e||180===e?"center":e<180?"left":"right"}function ii(e,t,n,r){var i,a,o=n.y+r/2;if(ce.isArray(t))for(i=0,a=t.length;i<a;++i)e.fillText(t[i],n.x,o),o+=r;else e.fillText(t,n.x,o)}function ai(e,t,n){90===e||270===e?n.y-=t.h/2:(e>270||e<90)&&(n.y-=t.h)}function oi(e){var t=e.ctx,n=e.options,r=n.pointLabels,i=Zr(n),a=e.getDistanceFromCenterForValue(n.ticks.reverse?e.min:e.max),o=ce.options._parseFont(r);t.save(),t.font=o.string,t.textBaseline="middle";for(var s=e.chart.data.labels.length-1;s>=0;s--){var l=0===s?i/2:0,u=e.getPointPosition(s,a+l+5),c=Kr(r.fontColor,s,Q.global.defaultFontColor);t.fillStyle=c;var d=e.getIndexAngle(s),f=ce.toDegrees(d);t.textAlign=ri(f),ai(f,e._pointLabelSizes[s],u),ii(t,e.pointLabels[s],u,o.lineHeight)}t.restore()}function si(e,t,n,r){var i,a=e.ctx,o=t.circular,s=e.chart.data.labels.length,l=Kr(t.color,r-1),u=Kr(t.lineWidth,r-1);if((o||s)&&l&&u){if(a.save(),a.strokeStyle=l,a.lineWidth=u,a.setLineDash&&(a.setLineDash(t.borderDash||[]),a.lineDashOffset=t.borderDashOffset||0),a.beginPath(),o)a.arc(e.xCenter,e.yCenter,n,0,2*Math.PI);else{i=e.getPointPosition(0,n),a.moveTo(i.x,i.y);for(var c=1;c<s;c++)i=e.getPointPosition(c,n),a.lineTo(i.x,i.y)}a.closePath(),a.stroke(),a.restore()}}function li(e){return ce.isNumber(e)?e:0}var ui=Or.extend({setDimensions:function(){var e=this;e.width=e.maxWidth,e.height=e.maxHeight,e.paddingTop=Zr(e.options)/2,e.xCenter=Math.floor(e.width/2),e.yCenter=Math.floor((e.height-e.paddingTop)/2),e.drawingArea=Math.min(e.height-e.paddingTop,e.width)/2},determineDataLimits:function(){var e=this,t=e.chart,n=Number.POSITIVE_INFINITY,r=Number.NEGATIVE_INFINITY;ce.each(t.data.datasets,(function(i,a){if(t.isDatasetVisible(a)){var o=t.getDatasetMeta(a);ce.each(i.data,(function(t,i){var a=+e.getRightValue(t);isNaN(a)||o.data[i].hidden||(n=Math.min(a,n),r=Math.max(a,r))}))}})),e.min=n===Number.POSITIVE_INFINITY?0:n,e.max=r===Number.NEGATIVE_INFINITY?0:r,e.handleTickRangeOptions()},_computeTickLimit:function(){return Math.ceil(this.drawingArea/Zr(this.options))},convertTicksToLabels:function(){var e=this;Or.prototype.convertTicksToLabels.call(e),e.pointLabels=e.chart.data.labels.map((function(){var t=ce.callback(e.options.pointLabels.callback,arguments,e);return t||0===t?t:""}))},getLabelForIndex:function(e,t){return+this.getRightValue(this.chart.data.datasets[t].data[e])},fit:function(){var e=this,t=e.options;t.display&&t.pointLabels.display?ni(e):e.setCenterPoint(0,0,0,0)},setReductions:function(e,t,n){var r=this,i=t.l/Math.sin(n.l),a=Math.max(t.r-r.width,0)/Math.sin(n.r),o=-t.t/Math.cos(n.t),s=-Math.max(t.b-(r.height-r.paddingTop),0)/Math.cos(n.b);i=li(i),a=li(a),o=li(o),s=li(s),r.drawingArea=Math.min(Math.floor(e-(i+a)/2),Math.floor(e-(o+s)/2)),r.setCenterPoint(i,a,o,s)},setCenterPoint:function(e,t,n,r){var i=this,a=i.width-t-i.drawingArea,o=e+i.drawingArea,s=n+i.drawingArea,l=i.height-i.paddingTop-r-i.drawingArea;i.xCenter=Math.floor((o+a)/2+i.left),i.yCenter=Math.floor((s+l)/2+i.top+i.paddingTop)},getIndexAngle:function(e){var t=this.chart,n=360/t.data.labels.length,r=t.options||{},i=r.startAngle||0,a=(e*n+i)%360;return(a<0?a+360:a)*Math.PI*2/360},getDistanceFromCenterForValue:function(e){var t=this;if(ce.isNullOrUndef(e))return NaN;var n=t.drawingArea/(t.max-t.min);return t.options.ticks.reverse?(t.max-e)*n:(e-t.min)*n},getPointPosition:function(e,t){var n=this,r=n.getIndexAngle(e)-Math.PI/2;return{x:Math.cos(r)*t+n.xCenter,y:Math.sin(r)*t+n.yCenter}},getPointPositionForValue:function(e,t){return this.getPointPosition(e,this.getDistanceFromCenterForValue(t))},getBasePosition:function(e){var t=this,n=t.min,r=t.max;return t.getPointPositionForValue(e||0,t.beginAtZero?0:n<0&&r<0?r:n>0&&r>0?n:0)},_drawGrid:function(){var e,t,n,r=this,i=r.ctx,a=r.options,o=a.gridLines,s=a.angleLines,l=Jr(s.lineWidth,o.lineWidth),u=Jr(s.color,o.color);if(a.pointLabels.display&&oi(r),o.display&&ce.each(r.ticks,(function(e,n){0!==n&&(t=r.getDistanceFromCenterForValue(r.ticksAsNumbers[n]),si(r,o,t,n))})),s.display&&l&&u){for(i.save(),i.lineWidth=l,i.strokeStyle=u,i.setLineDash&&(i.setLineDash(Xr([s.borderDash,o.borderDash,[]])),i.lineDashOffset=Xr([s.borderDashOffset,o.borderDashOffset,0])),e=r.chart.data.labels.length-1;e>=0;e--)t=r.getDistanceFromCenterForValue(a.ticks.reverse?r.min:r.max),n=r.getPointPosition(e,t),i.beginPath(),i.moveTo(r.xCenter,r.yCenter),i.lineTo(n.x,n.y),i.stroke();i.restore()}},_drawLabels:function(){var e=this,t=e.ctx,n=e.options,r=n.ticks;if(r.display){var i,a,o=e.getIndexAngle(0),s=ce.options._parseFont(r),l=Jr(r.fontColor,Q.global.defaultFontColor);t.save(),t.font=s.string,t.translate(e.xCenter,e.yCenter),t.rotate(o),t.textAlign="center",t.textBaseline="middle",ce.each(e.ticks,(function(n,o){(0!==o||r.reverse)&&(i=e.getDistanceFromCenterForValue(e.ticksAsNumbers[o]),r.showLabelBackdrop&&(a=t.measureText(n).width,t.fillStyle=r.backdropColor,t.fillRect(-a/2-r.backdropPaddingX,-i-s.size/2-r.backdropPaddingY,a+2*r.backdropPaddingX,s.size+2*r.backdropPaddingY)),t.fillStyle=l,t.fillText(n,0,-i))})),t.restore()}},_drawTitle:ce.noop}),ci=Qr;ui._defaults=ci;var di=ce._deprecated,fi=ce.options.resolve,hi=ce.valueOrDefault,pi=Number.MIN_SAFE_INTEGER||-9007199254740991,mi=Number.MAX_SAFE_INTEGER||9007199254740991,_i={millisecond:{common:!0,size:1,steps:1e3},second:{common:!0,size:1e3,steps:60},minute:{common:!0,size:6e4,steps:60},hour:{common:!0,size:36e5,steps:24},day:{common:!0,size:864e5,steps:30},week:{common:!1,size:6048e5,steps:4},month:{common:!0,size:2628e6,steps:12},quarter:{common:!1,size:7884e6,steps:4},year:{common:!0,size:3154e7}},vi=Object.keys(_i);function gi(e,t){return e-t}function bi(e){var t,n,r,i={},a=[];for(t=0,n=e.length;t<n;++t)r=e[t],i[r]||(i[r]=!0,a.push(r));return a}function yi(e){return ce.valueOrDefault(e.time.min,e.ticks.min)}function Mi(e){return ce.valueOrDefault(e.time.max,e.ticks.max)}function ki(e,t,n,r){if("linear"===r||!e.length)return[{time:t,pos:0},{time:n,pos:1}];var i,a,o,s,l,u=[],c=[t];for(i=0,a=e.length;i<a;++i)s=e[i],s>t&&s<n&&c.push(s);for(c.push(n),i=0,a=c.length;i<a;++i)l=c[i+1],o=c[i-1],s=c[i],void 0!==o&&void 0!==l&&Math.round((l+o)/2)===s||u.push({time:s,pos:i/(a-1)});return u}function wi(e,t,n){var r,i,a,o=0,s=e.length-1;while(o>=0&&o<=s){if(r=o+s>>1,i=e[r-1]||null,a=e[r],!i)return{lo:null,hi:a};if(a[t]<n)o=r+1;else{if(!(i[t]>n))return{lo:i,hi:a};s=r-1}}return{lo:a,hi:null}}function Li(e,t,n,r){var i=wi(e,t,n),a=i.lo?i.hi?i.lo:e[e.length-2]:e[0],o=i.lo?i.hi?i.hi:e[e.length-1]:e[1],s=o[t]-a[t],l=s?(n-a[t])/s:0,u=(o[r]-a[r])*l;return a[r]+u}function xi(e,t){var n=e._adapter,r=e.options.time,i=r.parser,a=i||r.format,o=t;return"function"===typeof i&&(o=i(o)),ce.isFinite(o)||(o="string"===typeof a?n.parse(o,a):n.parse(o)),null!==o?+o:(i||"function"!==typeof a||(o=a(t),ce.isFinite(o)||(o=n.parse(o))),o)}function Si(e,t){if(ce.isNullOrUndef(t))return null;var n=e.options.time,r=xi(e,e.getRightValue(t));return null===r||n.round&&(r=+e._adapter.startOf(r,n.round)),r}function Ti(e,t,n,r){var i,a,o,s=vi.length;for(i=vi.indexOf(e);i<s-1;++i)if(a=_i[vi[i]],o=a.steps?a.steps:mi,a.common&&Math.ceil((n-t)/(o*a.size))<=r)return vi[i];return vi[s-1]}function Di(e,t,n,r,i){var a,o;for(a=vi.length-1;a>=vi.indexOf(n);a--)if(o=vi[a],_i[o].common&&e._adapter.diff(i,r,o)>=t-1)return o;return vi[n?vi.indexOf(n):0]}function Yi(e){for(var t=vi.indexOf(e)+1,n=vi.length;t<n;++t)if(_i[vi[t]].common)return vi[t]}function Ci(e,t,n,r){var i,a=e._adapter,o=e.options,s=o.time,l=s.unit||Ti(s.minUnit,t,n,r),u=fi([s.stepSize,s.unitStepSize,1]),c="week"===l&&s.isoWeekday,d=t,f=[];if(c&&(d=+a.startOf(d,"isoWeek",c)),d=+a.startOf(d,c?"day":l),a.diff(n,t,l)>1e5*u)throw t+" and "+n+" are too far apart with stepSize of "+u+" "+l;for(i=d;i<n;i=+a.add(i,u,l))f.push(i);return i!==n&&"ticks"!==o.bounds||f.push(i),f}function Ei(e,t,n,r,i){var a,o,s=0,l=0;return i.offset&&t.length&&(a=Li(e,"time",t[0],"pos"),s=1===t.length?1-a:(Li(e,"time",t[1],"pos")-a)/2,o=Li(e,"time",t[t.length-1],"pos"),l=1===t.length?o:(o-Li(e,"time",t[t.length-2],"pos"))/2),{start:s,end:l,factor:1/(s+1+l)}}function Ai(e,t,n,r){var i,a,o=e._adapter,s=+o.startOf(t[0].value,r),l=t[t.length-1].value;for(i=s;i<=l;i=+o.add(i,1,r))a=n[i],a>=0&&(t[a].major=!0);return t}function Oi(e,t,n){var r,i,a=[],o={},s=t.length;for(r=0;r<s;++r)i=t[r],o[i]=r,a.push({value:i,major:!1});return 0!==s&&n?Ai(e,a,o,n):a}var Pi={position:"bottom",distribution:"linear",bounds:"data",adapters:{},time:{parser:!1,unit:!1,round:!1,displayFormat:!1,isoWeekday:!1,minUnit:"millisecond",displayFormats:{}},ticks:{autoSkip:!1,source:"auto",major:{enabled:!1}}},Fi=xr.extend({initialize:function(){this.mergeTicksOptions(),xr.prototype.initialize.call(this)},update:function(){var e=this,t=e.options,n=t.time||(t.time={}),r=e._adapter=new ir._date(t.adapters.date);return di("time scale",n.format,"time.format","time.parser"),di("time scale",n.min,"time.min","ticks.min"),di("time scale",n.max,"time.max","ticks.max"),ce.mergeIf(n.displayFormats,r.formats()),xr.prototype.update.apply(e,arguments)},getRightValue:function(e){return e&&void 0!==e.t&&(e=e.t),xr.prototype.getRightValue.call(this,e)},determineDataLimits:function(){var e,t,n,r,i,a,o,s=this,l=s.chart,u=s._adapter,c=s.options,d=c.time.unit||"day",f=mi,h=pi,p=[],m=[],_=[],v=s._getLabels();for(e=0,n=v.length;e<n;++e)_.push(Si(s,v[e]));for(e=0,n=(l.data.datasets||[]).length;e<n;++e)if(l.isDatasetVisible(e))if(i=l.data.datasets[e].data,ce.isObject(i[0]))for(m[e]=[],t=0,r=i.length;t<r;++t)a=Si(s,i[t]),p.push(a),m[e][t]=a;else m[e]=_.slice(0),o||(p=p.concat(_),o=!0);else m[e]=[];_.length&&(f=Math.min(f,_[0]),h=Math.max(h,_[_.length-1])),p.length&&(p=n>1?bi(p).sort(gi):p.sort(gi),f=Math.min(f,p[0]),h=Math.max(h,p[p.length-1])),f=Si(s,yi(c))||f,h=Si(s,Mi(c))||h,f=f===mi?+u.startOf(Date.now(),d):f,h=h===pi?+u.endOf(Date.now(),d)+1:h,s.min=Math.min(f,h),s.max=Math.max(f+1,h),s._table=[],s._timestamps={data:p,datasets:m,labels:_}},buildTicks:function(){var e,t,n,r=this,i=r.min,a=r.max,o=r.options,s=o.ticks,l=o.time,u=r._timestamps,c=[],d=r.getLabelCapacity(i),f=s.source,h=o.distribution;for(u="data"===f||"auto"===f&&"series"===h?u.data:"labels"===f?u.labels:Ci(r,i,a,d),"ticks"===o.bounds&&u.length&&(i=u[0],a=u[u.length-1]),i=Si(r,yi(o))||i,a=Si(r,Mi(o))||a,e=0,t=u.length;e<t;++e)n=u[e],n>=i&&n<=a&&c.push(n);return r.min=i,r.max=a,r._unit=l.unit||(s.autoSkip?Ti(l.minUnit,r.min,r.max,d):Di(r,c.length,l.minUnit,r.min,r.max)),r._majorUnit=s.major.enabled&&"year"!==r._unit?Yi(r._unit):void 0,r._table=ki(r._timestamps.data,i,a,h),r._offsets=Ei(r._table,c,i,a,o),s.reverse&&c.reverse(),Oi(r,c,r._majorUnit)},getLabelForIndex:function(e,t){var n=this,r=n._adapter,i=n.chart.data,a=n.options.time,o=i.labels&&e<i.labels.length?i.labels[e]:"",s=i.datasets[t].data[e];return ce.isObject(s)&&(o=n.getRightValue(s)),a.tooltipFormat?r.format(xi(n,o),a.tooltipFormat):"string"===typeof o?o:r.format(xi(n,o),a.displayFormats.datetime)},tickFormatFunction:function(e,t,n,r){var i=this,a=i._adapter,o=i.options,s=o.time.displayFormats,l=s[i._unit],u=i._majorUnit,c=s[u],d=n[t],f=o.ticks,h=u&&c&&d&&d.major,p=a.format(e,r||(h?c:l)),m=h?f.major:f.minor,_=fi([m.callback,m.userCallback,f.callback,f.userCallback]);return _?_(p,t,n):p},convertTicksToLabels:function(e){var t,n,r=[];for(t=0,n=e.length;t<n;++t)r.push(this.tickFormatFunction(e[t].value,t,e));return r},getPixelForOffset:function(e){var t=this,n=t._offsets,r=Li(t._table,"time",e,"pos");return t.getPixelForDecimal((n.start+r)*n.factor)},getPixelForValue:function(e,t,n){var r=this,i=null;if(void 0!==t&&void 0!==n&&(i=r._timestamps.datasets[n][t]),null===i&&(i=Si(r,e)),null!==i)return r.getPixelForOffset(i)},getPixelForTick:function(e){var t=this.getTicks();return e>=0&&e<t.length?this.getPixelForOffset(t[e].value):null},getValueForPixel:function(e){var t=this,n=t._offsets,r=t.getDecimalForPixel(e)/n.factor-n.end,i=Li(t._table,"pos",r,"time");return t._adapter._create(i)},_getLabelSize:function(e){var t=this,n=t.options.ticks,r=t.ctx.measureText(e).width,i=ce.toRadians(t.isHorizontal()?n.maxRotation:n.minRotation),a=Math.cos(i),o=Math.sin(i),s=hi(n.fontSize,Q.global.defaultFontSize);return{w:r*a+s*o,h:r*o+s*a}},getLabelWidth:function(e){return this._getLabelSize(e).w},getLabelCapacity:function(e){var t=this,n=t.options.time,r=n.displayFormats,i=r[n.unit]||r.millisecond,a=t.tickFormatFunction(e,0,Oi(t,[e],t._majorUnit),i),o=t._getLabelSize(a),s=Math.floor(t.isHorizontal()?t.width/o.w:t.height/o.h);return t.options.offset&&s--,s>0?s:1}}),Ii=Pi;Fi._defaults=Ii;var ji={category:Dr,linear:Nr,logarithmic:qr,radialLinear:ui,time:Fi},Hi={datetime:"MMM D, YYYY, h:mm:ss a",millisecond:"h:mm:ss.SSS a",second:"h:mm:ss a",minute:"h:mm a",hour:"hA",day:"MMM D",week:"ll",month:"MMM YYYY",quarter:"[Q]Q - YYYY",year:"YYYY"};ir._date.override("function"===typeof e?{_id:"moment",formats:function(){return Hi},parse:function(t,n){return"string"===typeof t&&"string"===typeof n?t=e(t,n):t instanceof e||(t=e(t)),t.isValid()?t.valueOf():null},format:function(t,n){return e(t).format(n)},add:function(t,n,r){return e(t).add(n,r).valueOf()},diff:function(t,n,r){return e(t).diff(e(n),r)},startOf:function(t,n,r){return t=e(t),"isoWeek"===n?t.isoWeekday(r).valueOf():t.startOf(n).valueOf()},endOf:function(t,n){return e(t).endOf(n).valueOf()},_create:function(t){return e(t)}}:{}),Q._set("global",{plugins:{filler:{propagate:!0}}});var Ri={dataset:function(e){var t=e.fill,n=e.chart,r=n.getDatasetMeta(t),i=r&&n.isDatasetVisible(t),a=i&&r.dataset._children||[],o=a.length||0;return o?function(e,t){return t<o&&a[t]._view||null}:null},boundary:function(e){var t=e.boundary,n=t?t.x:null,r=t?t.y:null;return ce.isArray(t)?function(e,n){return t[n]}:function(e){return{x:null===n?e.x:n,y:null===r?e.y:r}}}};function Ni(e,t,n){var r,i=e._model||{},a=i.fill;if(void 0===a&&(a=!!i.backgroundColor),!1===a||null===a)return!1;if(!0===a)return"origin";if(r=parseFloat(a,10),isFinite(r)&&Math.floor(r)===r)return"-"!==a[0]&&"+"!==a[0]||(r=t+r),!(r===t||r<0||r>=n)&&r;switch(a){case"bottom":return"start";case"top":return"end";case"zero":return"origin";case"origin":case"start":case"end":return a;default:return!1}}function $i(e){var t,n=e.el._model||{},r=e.el._scale||{},i=e.fill,a=null;if(isFinite(i))return null;if("start"===i?a=void 0===n.scaleBottom?r.bottom:n.scaleBottom:"end"===i?a=void 0===n.scaleTop?r.top:n.scaleTop:void 0!==n.scaleZero?a=n.scaleZero:r.getBasePixel&&(a=r.getBasePixel()),void 0!==a&&null!==a){if(void 0!==a.x&&void 0!==a.y)return a;if(ce.isFinite(a))return t=r.isHorizontal(),{x:t?a:null,y:t?null:a}}return null}function zi(e){var t,n,r,i,a,o=e.el._scale,s=o.options,l=o.chart.data.labels.length,u=e.fill,c=[];if(!l)return null;for(t=s.ticks.reverse?o.max:o.min,n=s.ticks.reverse?o.min:o.max,r=o.getPointPositionForValue(0,t),i=0;i<l;++i)a="start"===u||"end"===u?o.getPointPositionForValue(i,"start"===u?t:n):o.getBasePosition(i),s.gridLines.circular&&(a.cx=r.x,a.cy=r.y,a.angle=o.getIndexAngle(i)-Math.PI/2),c.push(a);return c}function Bi(e){var t=e.el._scale||{};return t.getPointPositionForValue?zi(e):$i(e)}function Wi(e,t,n){var r,i=e[t],a=i.fill,o=[t];if(!n)return a;while(!1!==a&&-1===o.indexOf(a)){if(!isFinite(a))return a;if(r=e[a],!r)return!1;if(r.visible)return a;o.push(a),a=r.fill}return!1}function Ui(e){var t=e.fill,n="dataset";return!1===t?null:(isFinite(t)||(n="boundary"),Ri[n](e))}function Vi(e){return e&&!e.skip}function qi(e,t,n,r,i){var a,o,s,l;if(r&&i){for(e.moveTo(t[0].x,t[0].y),a=1;a<r;++a)ce.canvas.lineTo(e,t[a-1],t[a]);if(void 0===n[0].angle)for(e.lineTo(n[i-1].x,n[i-1].y),a=i-1;a>0;--a)ce.canvas.lineTo(e,n[a],n[a-1],!0);else for(o=n[0].cx,s=n[0].cy,l=Math.sqrt(Math.pow(n[0].x-o,2)+Math.pow(n[0].y-s,2)),a=i-1;a>0;--a)e.arc(o,s,l,n[a].angle,n[a-1].angle,!0)}}function Gi(e,t,n,r,i,a){var o,s,l,u,c,d,f,h,p=t.length,m=r.spanGaps,_=[],v=[],g=0,b=0;for(e.beginPath(),o=0,s=p;o<s;++o)l=o%p,u=t[l]._view,c=n(u,l,r),d=Vi(u),f=Vi(c),a&&void 0===h&&d&&(h=o+1,s=p+h),d&&f?(g=_.push(u),b=v.push(c)):g&&b&&(m?(d&&_.push(u),f&&v.push(c)):(qi(e,_,v,g,b),g=b=0,_=[],v=[]));qi(e,_,v,g,b),e.closePath(),e.fillStyle=i,e.fill()}var Ji={id:"filler",afterDatasetsUpdate:function(e,t){var n,r,i,a,o=(e.data.datasets||[]).length,s=t.propagate,l=[];for(r=0;r<o;++r)n=e.getDatasetMeta(r),i=n.dataset,a=null,i&&i._model&&i instanceof Ke.Line&&(a={visible:e.isDatasetVisible(r),fill:Ni(i,r,o),chart:e,el:i}),n.$filler=a,l.push(a);for(r=0;r<o;++r)a=l[r],a&&(a.fill=Wi(l,r,s),a.boundary=Bi(a),a.mapper=Ui(a))},beforeDatasetsDraw:function(e){var t,n,r,i,a,o,s,l=e._getSortedVisibleDatasetMetas(),u=e.ctx;for(n=l.length-1;n>=0;--n)t=l[n].$filler,t&&t.visible&&(r=t.el,i=r._view,a=r._children||[],o=t.mapper,s=i.backgroundColor||Q.global.defaultColor,o&&s&&a.length&&(ce.canvas.clipArea(u,e.chartArea),Gi(u,a,o,i,s,r._loop),ce.canvas.unclipArea(u)))}},Ki=ce.rtl.getRtlAdapter,Xi=ce.noop,Qi=ce.valueOrDefault;function Zi(e,t){return e.usePointStyle&&e.boxWidth>t?t:e.boxWidth}Q._set("global",{legend:{display:!0,position:"top",align:"center",fullWidth:!0,reverse:!1,weight:1e3,onClick:function(e,t){var n=t.datasetIndex,r=this.chart,i=r.getDatasetMeta(n);i.hidden=null===i.hidden?!r.data.datasets[n].hidden:null,r.update()},onHover:null,onLeave:null,labels:{boxWidth:40,padding:10,generateLabels:function(e){var t=e.data.datasets,n=e.options.legend||{},r=n.labels&&n.labels.usePointStyle;return e._getSortedDatasetMetas().map((function(n){var i=n.controller.getStyle(r?0:void 0);return{text:t[n.index].label,fillStyle:i.backgroundColor,hidden:!e.isDatasetVisible(n.index),lineCap:i.borderCapStyle,lineDash:i.borderDash,lineDashOffset:i.borderDashOffset,lineJoin:i.borderJoinStyle,lineWidth:i.borderWidth,strokeStyle:i.borderColor,pointStyle:i.pointStyle,rotation:i.rotation,datasetIndex:n.index}}),this)}}},legendCallback:function(e){var t,n,r,i,a=document.createElement("ul"),o=e.data.datasets;for(a.setAttribute("class",e.id+"-legend"),t=0,n=o.length;t<n;t++)r=a.appendChild(document.createElement("li")),i=r.appendChild(document.createElement("span")),i.style.backgroundColor=o[t].backgroundColor,o[t].label&&r.appendChild(document.createTextNode(o[t].label));return a.outerHTML}});var ea=ge.extend({initialize:function(e){var t=this;ce.extend(t,e),t.legendHitBoxes=[],t._hoveredItem=null,t.doughnutMode=!1},beforeUpdate:Xi,update:function(e,t,n){var r=this;return r.beforeUpdate(),r.maxWidth=e,r.maxHeight=t,r.margins=n,r.beforeSetDimensions(),r.setDimensions(),r.afterSetDimensions(),r.beforeBuildLabels(),r.buildLabels(),r.afterBuildLabels(),r.beforeFit(),r.fit(),r.afterFit(),r.afterUpdate(),r.minSize},afterUpdate:Xi,beforeSetDimensions:Xi,setDimensions:function(){var e=this;e.isHorizontal()?(e.width=e.maxWidth,e.left=0,e.right=e.width):(e.height=e.maxHeight,e.top=0,e.bottom=e.height),e.paddingLeft=0,e.paddingTop=0,e.paddingRight=0,e.paddingBottom=0,e.minSize={width:0,height:0}},afterSetDimensions:Xi,beforeBuildLabels:Xi,buildLabels:function(){var e=this,t=e.options.labels||{},n=ce.callback(t.generateLabels,[e.chart],e)||[];t.filter&&(n=n.filter((function(n){return t.filter(n,e.chart.data)}))),e.options.reverse&&n.reverse(),e.legendItems=n},afterBuildLabels:Xi,beforeFit:Xi,fit:function(){var e=this,t=e.options,n=t.labels,r=t.display,i=e.ctx,a=ce.options._parseFont(n),o=a.size,s=e.legendHitBoxes=[],l=e.minSize,u=e.isHorizontal();if(u?(l.width=e.maxWidth,l.height=r?10:0):(l.width=r?10:0,l.height=e.maxHeight),r){if(i.font=a.string,u){var c=e.lineWidths=[0],d=0;i.textAlign="left",i.textBaseline="middle",ce.each(e.legendItems,(function(e,t){var r=Zi(n,o),a=r+o/2+i.measureText(e.text).width;(0===t||c[c.length-1]+a+2*n.padding>l.width)&&(d+=o+n.padding,c[c.length-(t>0?0:1)]=0),s[t]={left:0,top:0,width:a,height:o},c[c.length-1]+=a+n.padding})),l.height+=d}else{var f=n.padding,h=e.columnWidths=[],p=e.columnHeights=[],m=n.padding,_=0,v=0;ce.each(e.legendItems,(function(e,t){var r=Zi(n,o),a=r+o/2+i.measureText(e.text).width;t>0&&v+o+2*f>l.height&&(m+=_+n.padding,h.push(_),p.push(v),_=0,v=0),_=Math.max(_,a),v+=o+f,s[t]={left:0,top:0,width:a,height:o}})),m+=_,h.push(_),p.push(v),l.width+=m}e.width=l.width,e.height=l.height}else e.width=l.width=e.height=l.height=0},afterFit:Xi,isHorizontal:function(){return"top"===this.options.position||"bottom"===this.options.position},draw:function(){var e=this,t=e.options,n=t.labels,r=Q.global,i=r.defaultColor,a=r.elements.line,o=e.height,s=e.columnHeights,l=e.width,u=e.lineWidths;if(t.display){var c,d=Ki(t.rtl,e.left,e.minSize.width),f=e.ctx,h=Qi(n.fontColor,r.defaultFontColor),p=ce.options._parseFont(n),m=p.size;f.textAlign=d.textAlign("left"),f.textBaseline="middle",f.lineWidth=.5,f.strokeStyle=h,f.fillStyle=h,f.font=p.string;var _=Zi(n,m),v=e.legendHitBoxes,g=function(e,t,r){if(!(isNaN(_)||_<=0)){f.save();var o=Qi(r.lineWidth,a.borderWidth);if(f.fillStyle=Qi(r.fillStyle,i),f.lineCap=Qi(r.lineCap,a.borderCapStyle),f.lineDashOffset=Qi(r.lineDashOffset,a.borderDashOffset),f.lineJoin=Qi(r.lineJoin,a.borderJoinStyle),f.lineWidth=o,f.strokeStyle=Qi(r.strokeStyle,i),f.setLineDash&&f.setLineDash(Qi(r.lineDash,a.borderDash)),n&&n.usePointStyle){var s=_*Math.SQRT2/2,l=d.xPlus(e,_/2),u=t+m/2;ce.canvas.drawPoint(f,r.pointStyle,s,l,u,r.rotation)}else f.fillRect(d.leftForLtr(e,_),t,_,m),0!==o&&f.strokeRect(d.leftForLtr(e,_),t,_,m);f.restore()}},b=function(e,t,n,r){var i=m/2,a=d.xPlus(e,_+i),o=t+i;f.fillText(n.text,a,o),n.hidden&&(f.beginPath(),f.lineWidth=2,f.moveTo(a,o),f.lineTo(d.xPlus(a,r),o),f.stroke())},y=function(e,r){switch(t.align){case"start":return n.padding;case"end":return e-r;default:return(e-r+n.padding)/2}},M=e.isHorizontal();c=M?{x:e.left+y(l,u[0]),y:e.top+n.padding,line:0}:{x:e.left+n.padding,y:e.top+y(o,s[0]),line:0},ce.rtl.overrideTextDirection(e.ctx,t.textDirection);var k=m+n.padding;ce.each(e.legendItems,(function(t,r){var i=f.measureText(t.text).width,a=_+m/2+i,h=c.x,p=c.y;d.setWidth(e.minSize.width),M?r>0&&h+a+n.padding>e.left+e.minSize.width&&(p=c.y+=k,c.line++,h=c.x=e.left+y(l,u[c.line])):r>0&&p+k>e.top+e.minSize.height&&(h=c.x=h+e.columnWidths[c.line]+n.padding,c.line++,p=c.y=e.top+y(o,s[c.line]));var w=d.x(h);g(w,p,t),v[r].left=d.leftForLtr(w,v[r].width),v[r].top=p,b(w,p,t,i),M?c.x+=a+n.padding:c.y+=k})),ce.rtl.restoreTextDirection(e.ctx,t.textDirection)}},_getLegendItemAt:function(e,t){var n,r,i,a=this;if(e>=a.left&&e<=a.right&&t>=a.top&&t<=a.bottom)for(i=a.legendHitBoxes,n=0;n<i.length;++n)if(r=i[n],e>=r.left&&e<=r.left+r.width&&t>=r.top&&t<=r.top+r.height)return a.legendItems[n];return null},handleEvent:function(e){var t,n=this,r=n.options,i="mouseup"===e.type?"click":e.type;if("mousemove"===i){if(!r.onHover&&!r.onLeave)return}else{if("click"!==i)return;if(!r.onClick)return}t=n._getLegendItemAt(e.x,e.y),"click"===i?t&&r.onClick&&r.onClick.call(n,e.native,t):(r.onLeave&&t!==n._hoveredItem&&(n._hoveredItem&&r.onLeave.call(n,e.native,n._hoveredItem),n._hoveredItem=t),r.onHover&&t&&r.onHover.call(n,e.native,t))}});function ta(e,t){var n=new ea({ctx:e.ctx,options:t,chart:e});Jt.configure(e,n,t),Jt.addBox(e,n),e.legend=n}var na={id:"legend",_element:ea,beforeInit:function(e){var t=e.options.legend;t&&ta(e,t)},beforeUpdate:function(e){var t=e.options.legend,n=e.legend;t?(ce.mergeIf(t,Q.global.legend),n?(Jt.configure(e,n,t),n.options=t):ta(e,t)):n&&(Jt.removeBox(e,n),delete e.legend)},afterEvent:function(e,t){var n=e.legend;n&&n.handleEvent(t)}},ra=ce.noop;Q._set("global",{title:{display:!1,fontStyle:"bold",fullWidth:!0,padding:10,position:"top",text:"",weight:2e3}});var ia=ge.extend({initialize:function(e){var t=this;ce.extend(t,e),t.legendHitBoxes=[]},beforeUpdate:ra,update:function(e,t,n){var r=this;return r.beforeUpdate(),r.maxWidth=e,r.maxHeight=t,r.margins=n,r.beforeSetDimensions(),r.setDimensions(),r.afterSetDimensions(),r.beforeBuildLabels(),r.buildLabels(),r.afterBuildLabels(),r.beforeFit(),r.fit(),r.afterFit(),r.afterUpdate(),r.minSize},afterUpdate:ra,beforeSetDimensions:ra,setDimensions:function(){var e=this;e.isHorizontal()?(e.width=e.maxWidth,e.left=0,e.right=e.width):(e.height=e.maxHeight,e.top=0,e.bottom=e.height),e.paddingLeft=0,e.paddingTop=0,e.paddingRight=0,e.paddingBottom=0,e.minSize={width:0,height:0}},afterSetDimensions:ra,beforeBuildLabels:ra,buildLabels:ra,afterBuildLabels:ra,beforeFit:ra,fit:function(){var e,t,n=this,r=n.options,i=n.minSize={},a=n.isHorizontal();r.display?(e=ce.isArray(r.text)?r.text.length:1,t=e*ce.options._parseFont(r).lineHeight+2*r.padding,n.width=i.width=a?n.maxWidth:t,n.height=i.height=a?t:n.maxHeight):n.width=i.width=n.height=i.height=0},afterFit:ra,isHorizontal:function(){var e=this.options.position;return"top"===e||"bottom"===e},draw:function(){var e=this,t=e.ctx,n=e.options;if(n.display){var r,i,a,o=ce.options._parseFont(n),s=o.lineHeight,l=s/2+n.padding,u=0,c=e.top,d=e.left,f=e.bottom,h=e.right;t.fillStyle=ce.valueOrDefault(n.fontColor,Q.global.defaultFontColor),t.font=o.string,e.isHorizontal()?(i=d+(h-d)/2,a=c+l,r=h-d):(i="left"===n.position?d+l:h-l,a=c+(f-c)/2,r=f-c,u=Math.PI*("left"===n.position?-.5:.5)),t.save(),t.translate(i,a),t.rotate(u),t.textAlign="center",t.textBaseline="middle";var p=n.text;if(ce.isArray(p))for(var m=0,_=0;_<p.length;++_)t.fillText(p[_],0,m,r),m+=s;else t.fillText(p,0,0,r);t.restore()}}});function aa(e,t){var n=new ia({ctx:e.ctx,options:t,chart:e});Jt.configure(e,n,t),Jt.addBox(e,n),e.titleBlock=n}var oa={id:"title",_element:ia,beforeInit:function(e){var t=e.options.title;t&&aa(e,t)},beforeUpdate:function(e){var t=e.options.title,n=e.titleBlock;t?(ce.mergeIf(t,Q.global.title),n?(Jt.configure(e,n,t),n.options=t):aa(e,t)):n&&(Jt.removeBox(e,n),delete e.titleBlock)}},sa={},la=Ji,ua=na,ca=oa;for(var da in sa.filler=la,sa.legend=ua,sa.title=ca,Zn.helpers=ce,er(),Zn._adapters=ir,Zn.Animation=ye,Zn.animationService=Me,Zn.controllers=Yt,Zn.DatasetController=Te,Zn.defaults=Q,Zn.Element=ge,Zn.elements=Ke,Zn.Interaction=It,Zn.layouts=Jt,Zn.platform=Sn,Zn.plugins=Tn,Zn.Scale=xr,Zn.scaleService=Dn,Zn.Ticks=ar,Zn.Tooltip=Bn,Zn.helpers.each(ji,(function(e,t){Zn.scaleService.registerScaleType(t,e,e._defaults)})),sa)sa.hasOwnProperty(da)&&Zn.plugins.register(sa[da]);Zn.platform.initialize();var fa=Zn;return"undefined"!==typeof window&&(window.Chart=Zn),Zn.Chart=Zn,Zn.Legend=sa.legend._element,Zn.Title=sa.title._element,Zn.pluginService=Zn.plugins,Zn.PluginBase=Zn.Element.extend({}),Zn.canvasHelpers=Zn.helpers.canvas,Zn.layoutService=Zn.layouts,Zn.LinearScaleBase=Or,Zn.helpers.each(["Bar","Bubble","Doughnut","Line","PolarArea","Radar","Scatter"],(function(e){Zn[e]=function(t,n){return new Zn(t,Zn.helpers.merge(n||{},{type:e.charAt(0).toLowerCase()+e.slice(1)}))}})),fa}))},3169:function(e,t,n){},"31a5":function(e,t,n){"use strict";n("21a0")},3280:function(e,t,n){"use strict";var r=n("ebb5"),i=n("e58c"),a=r.aTypedArray,o=r.exportTypedArrayMethod;o("lastIndexOf",(function(e){return i.apply(a(this),arguments)}))},"32ba":function(e,t,n){},3410:function(e,t,n){var r=n("23e7"),i=n("d039"),a=n("7b0b"),o=n("e163"),s=n("e177"),l=i((function(){o(1)}));r({target:"Object",stat:!0,forced:l,sham:!s},{getPrototypeOf:function(e){return o(a(e))}})},"342f":function(e,t,n){var r=n("d066");e.exports=r("navigator","userAgent")||""},3546:function(e,t,n){},"356a":function(e,t,n){},"35a1":function(e,t,n){var r=n("f5df"),i=n("3f8c"),a=n("b622"),o=a("iterator");e.exports=function(e){if(void 0!=e)return e[o]||e["@@iterator"]||i[r(e)]}},"36a4":function(e,t,n){},"376d":function(e,t,n){},"37e8":function(e,t,n){var r=n("83ab"),i=n("9bf2"),a=n("825a"),o=n("df75");e.exports=r?Object.defineProperties:function(e,t){a(e);var n,r=o(t),s=r.length,l=0;while(s>l)i.f(e,n=r[l++],t[n]);return e}},3886:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("en-ca",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"YYYY-MM-DD",LL:"MMMM D, YYYY",LLL:"MMMM D, YYYY h:mm A",LLLL:"dddd, MMMM D, YYYY h:mm A"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(e){var t=e%10,n=1===~~(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th";return e+n}});return t}))},"39a6":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("en-gb",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(e){var t=e%10,n=1===~~(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th";return e+n},week:{dow:1,doy:4}});return t}))},"39bd":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"१",2:"२",3:"३",4:"४",5:"५",6:"६",7:"७",8:"८",9:"९",0:"०"},n={"१":"1","२":"2","३":"3","४":"4","५":"5","६":"6","७":"7","८":"8","९":"9","०":"0"};function r(e,t,n,r){var i="";if(t)switch(n){case"s":i="काही सेकंद";break;case"ss":i="%d सेकंद";break;case"m":i="एक मिनिट";break;case"mm":i="%d मिनिटे";break;case"h":i="एक तास";break;case"hh":i="%d तास";break;case"d":i="एक दिवस";break;case"dd":i="%d दिवस";break;case"M":i="एक महिना";break;case"MM":i="%d महिने";break;case"y":i="एक वर्ष";break;case"yy":i="%d वर्षे";break}else switch(n){case"s":i="काही सेकंदां";break;case"ss":i="%d सेकंदां";break;case"m":i="एका मिनिटा";break;case"mm":i="%d मिनिटां";break;case"h":i="एका तासा";break;case"hh":i="%d तासां";break;case"d":i="एका दिवसा";break;case"dd":i="%d दिवसां";break;case"M":i="एका महिन्या";break;case"MM":i="%d महिन्यां";break;case"y":i="एका वर्षा";break;case"yy":i="%d वर्षां";break}return i.replace(/%d/i,e)}var i=e.defineLocale("mr",{months:"जानेवारी_फेब्रुवारी_मार्च_एप्रिल_मे_जून_जुलै_ऑगस्ट_सप्टेंबर_ऑक्टोबर_नोव्हेंबर_डिसेंबर".split("_"),monthsShort:"जाने._फेब्रु._मार्च._एप्रि._मे._जून._जुलै._ऑग._सप्टें._ऑक्टो._नोव्हें._डिसें.".split("_"),monthsParseExact:!0,weekdays:"रविवार_सोमवार_मंगळवार_बुधवार_गुरूवार_शुक्रवार_शनिवार".split("_"),weekdaysShort:"रवि_सोम_मंगळ_बुध_गुरू_शुक्र_शनि".split("_"),weekdaysMin:"र_सो_मं_बु_गु_शु_श".split("_"),longDateFormat:{LT:"A h:mm वाजता",LTS:"A h:mm:ss वाजता",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm वाजता",LLLL:"dddd, D MMMM YYYY, A h:mm वाजता"},calendar:{sameDay:"[आज] LT",nextDay:"[उद्या] LT",nextWeek:"dddd, LT",lastDay:"[काल] LT",lastWeek:"[मागील] dddd, LT",sameElse:"L"},relativeTime:{future:"%sमध्ये",past:"%sपूर्वी",s:r,ss:r,m:r,mm:r,h:r,hh:r,d:r,dd:r,M:r,MM:r,y:r,yy:r},preparse:function(e){return e.replace(/[१२३४५६७८९०]/g,(function(e){return n[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},meridiemParse:/पहाटे|सकाळी|दुपारी|सायंकाळी|रात्री/,meridiemHour:function(e,t){return 12===e&&(e=0),"पहाटे"===t||"सकाळी"===t?e:"दुपारी"===t||"सायंकाळी"===t||"रात्री"===t?e>=12?e:e+12:void 0},meridiem:function(e,t,n){return e>=0&&e<6?"पहाटे":e<12?"सकाळी":e<17?"दुपारी":e<20?"सायंकाळी":"रात्री"},week:{dow:0,doy:6}});return i}))},"3a39":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"१",2:"२",3:"३",4:"४",5:"५",6:"६",7:"७",8:"८",9:"९",0:"०"},n={"१":"1","२":"2","३":"3","४":"4","५":"5","६":"6","७":"7","८":"8","९":"9","०":"0"},r=e.defineLocale("ne",{months:"जनवरी_फेब्रुवरी_मार्च_अप्रिल_मई_जुन_जुलाई_अगष्ट_सेप्टेम्बर_अक्टोबर_नोभेम्बर_डिसेम्बर".split("_"),monthsShort:"जन._फेब्रु._मार्च_अप्रि._मई_जुन_जुलाई._अग._सेप्ट._अक्टो._नोभे._डिसे.".split("_"),monthsParseExact:!0,weekdays:"आइतबार_सोमबार_मङ्गलबार_बुधबार_बिहिबार_शुक्रबार_शनिबार".split("_"),weekdaysShort:"आइत._सोम._मङ्गल._बुध._बिहि._शुक्र._शनि.".split("_"),weekdaysMin:"आ._सो._मं._बु._बि._शु._श.".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"Aको h:mm बजे",LTS:"Aको h:mm:ss बजे",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, Aको h:mm बजे",LLLL:"dddd, D MMMM YYYY, Aको h:mm बजे"},preparse:function(e){return e.replace(/[१२३४५६७८९०]/g,(function(e){return n[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},meridiemParse:/राति|बिहान|दिउँसो|साँझ/,meridiemHour:function(e,t){return 12===e&&(e=0),"राति"===t?e<4?e:e+12:"बिहान"===t?e:"दिउँसो"===t?e>=10?e:e+12:"साँझ"===t?e+12:void 0},meridiem:function(e,t,n){return e<3?"राति":e<12?"बिहान":e<16?"दिउँसो":e<20?"साँझ":"राति"},calendar:{sameDay:"[आज] LT",nextDay:"[भोलि] LT",nextWeek:"[आउँदो] dddd[,] LT",lastDay:"[हिजो] LT",lastWeek:"[गएको] dddd[,] LT",sameElse:"L"},relativeTime:{future:"%sमा",past:"%s अगाडि",s:"केही क्षण",ss:"%d सेकेण्ड",m:"एक मिनेट",mm:"%d मिनेट",h:"एक घण्टा",hh:"%d घण्टा",d:"एक दिन",dd:"%d दिन",M:"एक महिना",MM:"%d महिना",y:"एक बर्ष",yy:"%d बर्ष"},week:{dow:0,doy:6}});return r}))},"3a6c":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("zh-mo",{months:"一月_二月_三月_四月_五月_六月_七月_八月_九月_十月_十一月_十二月".split("_"),monthsShort:"1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月".split("_"),weekdays:"星期日_星期一_星期二_星期三_星期四_星期五_星期六".split("_"),weekdaysShort:"週日_週一_週二_週三_週四_週五_週六".split("_"),weekdaysMin:"日_一_二_三_四_五_六".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"YYYY年M月D日",LLL:"YYYY年M月D日 HH:mm",LLLL:"YYYY年M月D日dddd HH:mm",l:"D/M/YYYY",ll:"YYYY年M月D日",lll:"YYYY年M月D日 HH:mm",llll:"YYYY年M月D日dddd HH:mm"},meridiemParse:/凌晨|早上|上午|中午|下午|晚上/,meridiemHour:function(e,t){return 12===e&&(e=0),"凌晨"===t||"早上"===t||"上午"===t?e:"中午"===t?e>=11?e:e+12:"下午"===t||"晚上"===t?e+12:void 0},meridiem:function(e,t,n){var r=100*e+t;return r<600?"凌晨":r<900?"早上":r<1130?"上午":r<1230?"中午":r<1800?"下午":"晚上"},calendar:{sameDay:"[今天] LT",nextDay:"[明天] LT",nextWeek:"[下]dddd LT",lastDay:"[昨天] LT",lastWeek:"[上]dddd LT",sameElse:"L"},dayOfMonthOrdinalParse:/\d{1,2}(日|月|週)/,ordinal:function(e,t){switch(t){case"d":case"D":case"DDD":return e+"日";case"M":return e+"月";case"w":case"W":return e+"週";default:return e}},relativeTime:{future:"%s內",past:"%s前",s:"幾秒",ss:"%d 秒",m:"1 分鐘",mm:"%d 分鐘",h:"1 小時",hh:"%d 小時",d:"1 天",dd:"%d 天",M:"1 個月",MM:"%d 個月",y:"1 年",yy:"%d 年"}});return t}))},"3a7b":function(e,t,n){"use strict";var r=n("ebb5"),i=n("b727").findIndex,a=r.aTypedArray,o=r.exportTypedArrayMethod;o("findIndex",(function(e){return i(a(this),e,arguments.length>1?arguments[1]:void 0)}))},"3b02":function(e,t,n){"use strict";n("5194")},"3b1b":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={0:"-ум",1:"-ум",2:"-юм",3:"-юм",4:"-ум",5:"-ум",6:"-ум",7:"-ум",8:"-ум",9:"-ум",10:"-ум",12:"-ум",13:"-ум",20:"-ум",30:"-юм",40:"-ум",50:"-ум",60:"-ум",70:"-ум",80:"-ум",90:"-ум",100:"-ум"},n=e.defineLocale("tg",{months:{format:"январи_феврали_марти_апрели_майи_июни_июли_августи_сентябри_октябри_ноябри_декабри".split("_"),standalone:"январ_феврал_март_апрел_май_июн_июл_август_сентябр_октябр_ноябр_декабр".split("_")},monthsShort:"янв_фев_мар_апр_май_июн_июл_авг_сен_окт_ноя_дек".split("_"),weekdays:"якшанбе_душанбе_сешанбе_чоршанбе_панҷшанбе_ҷумъа_шанбе".split("_"),weekdaysShort:"яшб_дшб_сшб_чшб_пшб_ҷум_шнб".split("_"),weekdaysMin:"яш_дш_сш_чш_пш_ҷм_шб".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Имрӯз соати] LT",nextDay:"[Фардо соати] LT",lastDay:"[Дирӯз соати] LT",nextWeek:"dddd[и] [ҳафтаи оянда соати] LT",lastWeek:"dddd[и] [ҳафтаи гузашта соати] LT",sameElse:"L"},relativeTime:{future:"баъди %s",past:"%s пеш",s:"якчанд сония",m:"як дақиқа",mm:"%d дақиқа",h:"як соат",hh:"%d соат",d:"як рӯз",dd:"%d рӯз",M:"як моҳ",MM:"%d моҳ",y:"як сол",yy:"%d сол"},meridiemParse:/шаб|субҳ|рӯз|бегоҳ/,meridiemHour:function(e,t){return 12===e&&(e=0),"шаб"===t?e<4?e:e+12:"субҳ"===t?e:"рӯз"===t?e>=11?e:e+12:"бегоҳ"===t?e+12:void 0},meridiem:function(e,t,n){return e<4?"шаб":e<11?"субҳ":e<16?"рӯз":e<19?"бегоҳ":"шаб"},dayOfMonthOrdinalParse:/\d{1,2}-(ум|юм)/,ordinal:function(e){var n=e%10,r=e>=100?100:null;return e+(t[e]||t[n]||t[r])},week:{dow:1,doy:7}});return n}))},"3b3b":function(e,t,n){"use strict";n("4b10")},"3bbe":function(e,t,n){var r=n("861d");e.exports=function(e){if(!r(e)&&null!==e)throw TypeError("Can't set "+String(e)+" as a prototype");return e}},"3bdf":function(e,t,n){"use strict";n("4301")},"3c0d":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t="leden_únor_březen_duben_květen_červen_červenec_srpen_září_říjen_listopad_prosinec".split("_"),n="led_úno_bře_dub_kvě_čvn_čvc_srp_zář_říj_lis_pro".split("_"),r=[/^led/i,/^úno/i,/^bře/i,/^dub/i,/^kvě/i,/^(čvn|červen$|června)/i,/^(čvc|červenec|července)/i,/^srp/i,/^zář/i,/^říj/i,/^lis/i,/^pro/i],i=/^(leden|únor|březen|duben|květen|červenec|července|červen|června|srpen|září|říjen|listopad|prosinec|led|úno|bře|dub|kvě|čvn|čvc|srp|zář|říj|lis|pro)/i;function a(e){return e>1&&e<5&&1!==~~(e/10)}function o(e,t,n,r){var i=e+" ";switch(n){case"s":return t||r?"pár sekund":"pár sekundami";case"ss":return t||r?i+(a(e)?"sekundy":"sekund"):i+"sekundami";case"m":return t?"minuta":r?"minutu":"minutou";case"mm":return t||r?i+(a(e)?"minuty":"minut"):i+"minutami";case"h":return t?"hodina":r?"hodinu":"hodinou";case"hh":return t||r?i+(a(e)?"hodiny":"hodin"):i+"hodinami";case"d":return t||r?"den":"dnem";case"dd":return t||r?i+(a(e)?"dny":"dní"):i+"dny";case"M":return t||r?"měsíc":"měsícem";case"MM":return t||r?i+(a(e)?"měsíce":"měsíců"):i+"měsíci";case"y":return t||r?"rok":"rokem";case"yy":return t||r?i+(a(e)?"roky":"let"):i+"lety"}}var s=e.defineLocale("cs",{months:t,monthsShort:n,monthsRegex:i,monthsShortRegex:i,monthsStrictRegex:/^(leden|ledna|února|únor|březen|března|duben|dubna|květen|května|červenec|července|červen|června|srpen|srpna|září|říjen|října|listopadu|listopad|prosinec|prosince)/i,monthsShortStrictRegex:/^(led|úno|bře|dub|kvě|čvn|čvc|srp|zář|říj|lis|pro)/i,monthsParse:r,longMonthsParse:r,shortMonthsParse:r,weekdays:"neděle_pondělí_úterý_středa_čtvrtek_pátek_sobota".split("_"),weekdaysShort:"ne_po_út_st_čt_pá_so".split("_"),weekdaysMin:"ne_po_út_st_čt_pá_so".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd D. MMMM YYYY H:mm",l:"D. M. YYYY"},calendar:{sameDay:"[dnes v] LT",nextDay:"[zítra v] LT",nextWeek:function(){switch(this.day()){case 0:return"[v neděli v] LT";case 1:case 2:return"[v] dddd [v] LT";case 3:return"[ve středu v] LT";case 4:return"[ve čtvrtek v] LT";case 5:return"[v pátek v] LT";case 6:return"[v sobotu v] LT"}},lastDay:"[včera v] LT",lastWeek:function(){switch(this.day()){case 0:return"[minulou neděli v] LT";case 1:case 2:return"[minulé] dddd [v] LT";case 3:return"[minulou středu v] LT";case 4:case 5:return"[minulý] dddd [v] LT";case 6:return"[minulou sobotu v] LT"}},sameElse:"L"},relativeTime:{future:"za %s",past:"před %s",s:o,ss:o,m:o,mm:o,h:o,hh:o,d:o,dd:o,M:o,MM:o,y:o,yy:o},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return s}))},"3c5d":function(e,t,n){"use strict";var r=n("ebb5"),i=n("50c4"),a=n("182d"),o=n("7b0b"),s=n("d039"),l=r.aTypedArray,u=r.exportTypedArrayMethod,c=s((function(){new Int8Array(1).set({})}));u("set",(function(e){l(this);var t=a(arguments.length>1?arguments[1]:void 0,1),n=this.length,r=o(e),s=i(r.length),u=0;if(s+t>n)throw RangeError("Wrong length");while(u<s)this[t+u]=r[u++]}),c)},"3ca3":function(e,t,n){"use strict";var r=n("6547").charAt,i=n("69f3"),a=n("7dd0"),o="String Iterator",s=i.set,l=i.getterFor(o);a(String,"String",(function(e){s(this,{type:o,string:String(e),index:0})}),(function(){var e,t=l(this),n=t.string,i=t.index;return i>=n.length?{value:void 0,done:!0}:(e=r(n,i),t.index+=e.length,{value:e,done:!1})}))},"3d1f":function(e,t,n){"use strict";n("4c89")},"3d42":function(e,t,n){},"3d5c":function(e,t){function n(){throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}e.exports=n},"3de5":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"௧",2:"௨",3:"௩",4:"௪",5:"௫",6:"௬",7:"௭",8:"௮",9:"௯",0:"௦"},n={"௧":"1","௨":"2","௩":"3","௪":"4","௫":"5","௬":"6","௭":"7","௮":"8","௯":"9","௦":"0"},r=e.defineLocale("ta",{months:"ஜனவரி_பிப்ரவரி_மார்ச்_ஏப்ரல்_மே_ஜூன்_ஜூலை_ஆகஸ்ட்_செப்டெம்பர்_அக்டோபர்_நவம்பர்_டிசம்பர்".split("_"),monthsShort:"ஜனவரி_பிப்ரவரி_மார்ச்_ஏப்ரல்_மே_ஜூன்_ஜூலை_ஆகஸ்ட்_செப்டெம்பர்_அக்டோபர்_நவம்பர்_டிசம்பர்".split("_"),weekdays:"ஞாயிற்றுக்கிழமை_திங்கட்கிழமை_செவ்வாய்கிழமை_புதன்கிழமை_வியாழக்கிழமை_வெள்ளிக்கிழமை_சனிக்கிழமை".split("_"),weekdaysShort:"ஞாயிறு_திங்கள்_செவ்வாய்_புதன்_வியாழன்_வெள்ளி_சனி".split("_"),weekdaysMin:"ஞா_தி_செ_பு_வி_வெ_ச".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, HH:mm",LLLL:"dddd, D MMMM YYYY, HH:mm"},calendar:{sameDay:"[இன்று] LT",nextDay:"[நாளை] LT",nextWeek:"dddd, LT",lastDay:"[நேற்று] LT",lastWeek:"[கடந்த வாரம்] dddd, LT",sameElse:"L"},relativeTime:{future:"%s இல்",past:"%s முன்",s:"ஒரு சில விநாடிகள்",ss:"%d விநாடிகள்",m:"ஒரு நிமிடம்",mm:"%d நிமிடங்கள்",h:"ஒரு மணி நேரம்",hh:"%d மணி நேரம்",d:"ஒரு நாள்",dd:"%d நாட்கள்",M:"ஒரு மாதம்",MM:"%d மாதங்கள்",y:"ஒரு வருடம்",yy:"%d ஆண்டுகள்"},dayOfMonthOrdinalParse:/\d{1,2}வது/,ordinal:function(e){return e+"வது"},preparse:function(e){return e.replace(/[௧௨௩௪௫௬௭௮௯௦]/g,(function(e){return n[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},meridiemParse:/யாமம்|வைகறை|காலை|நண்பகல்|எற்பாடு|மாலை/,meridiem:function(e,t,n){return e<2?" யாமம்":e<6?" வைகறை":e<10?" காலை":e<14?" நண்பகல்":e<18?" எற்பாடு":e<22?" மாலை":" யாமம்"},meridiemHour:function(e,t){return 12===e&&(e=0),"யாமம்"===t?e<2?e:e+12:"வைகறை"===t||"காலை"===t||"நண்பகல்"===t&&e>=10?e:e+12},week:{dow:0,doy:6}});return r}))},"3e92":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"೧",2:"೨",3:"೩",4:"೪",5:"೫",6:"೬",7:"೭",8:"೮",9:"೯",0:"೦"},n={"೧":"1","೨":"2","೩":"3","೪":"4","೫":"5","೬":"6","೭":"7","೮":"8","೯":"9","೦":"0"},r=e.defineLocale("kn",{months:"ಜನವರಿ_ಫೆಬ್ರವರಿ_ಮಾರ್ಚ್_ಏಪ್ರಿಲ್_ಮೇ_ಜೂನ್_ಜುಲೈ_ಆಗಸ್ಟ್_ಸೆಪ್ಟೆಂಬರ್_ಅಕ್ಟೋಬರ್_ನವೆಂಬರ್_ಡಿಸೆಂಬರ್".split("_"),monthsShort:"ಜನ_ಫೆಬ್ರ_ಮಾರ್ಚ್_ಏಪ್ರಿಲ್_ಮೇ_ಜೂನ್_ಜುಲೈ_ಆಗಸ್ಟ್_ಸೆಪ್ಟೆಂ_ಅಕ್ಟೋ_ನವೆಂ_ಡಿಸೆಂ".split("_"),monthsParseExact:!0,weekdays:"ಭಾನುವಾರ_ಸೋಮವಾರ_ಮಂಗಳವಾರ_ಬುಧವಾರ_ಗುರುವಾರ_ಶುಕ್ರವಾರ_ಶನಿವಾರ".split("_"),weekdaysShort:"ಭಾನು_ಸೋಮ_ಮಂಗಳ_ಬುಧ_ಗುರು_ಶುಕ್ರ_ಶನಿ".split("_"),weekdaysMin:"ಭಾ_ಸೋ_ಮಂ_ಬು_ಗು_ಶು_ಶ".split("_"),longDateFormat:{LT:"A h:mm",LTS:"A h:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm",LLLL:"dddd, D MMMM YYYY, A h:mm"},calendar:{sameDay:"[ಇಂದು] LT",nextDay:"[ನಾಳೆ] LT",nextWeek:"dddd, LT",lastDay:"[ನಿನ್ನೆ] LT",lastWeek:"[ಕೊನೆಯ] dddd, LT",sameElse:"L"},relativeTime:{future:"%s ನಂತರ",past:"%s ಹಿಂದೆ",s:"ಕೆಲವು ಕ್ಷಣಗಳು",ss:"%d ಸೆಕೆಂಡುಗಳು",m:"ಒಂದು ನಿಮಿಷ",mm:"%d ನಿಮಿಷ",h:"ಒಂದು ಗಂಟೆ",hh:"%d ಗಂಟೆ",d:"ಒಂದು ದಿನ",dd:"%d ದಿನ",M:"ಒಂದು ತಿಂಗಳು",MM:"%d ತಿಂಗಳು",y:"ಒಂದು ವರ್ಷ",yy:"%d ವರ್ಷ"},preparse:function(e){return e.replace(/[೧೨೩೪೫೬೭೮೯೦]/g,(function(e){return n[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},meridiemParse:/ರಾತ್ರಿ|ಬೆಳಿಗ್ಗೆ|ಮಧ್ಯಾಹ್ನ|ಸಂಜೆ/,meridiemHour:function(e,t){return 12===e&&(e=0),"ರಾತ್ರಿ"===t?e<4?e:e+12:"ಬೆಳಿಗ್ಗೆ"===t?e:"ಮಧ್ಯಾಹ್ನ"===t?e>=10?e:e+12:"ಸಂಜೆ"===t?e+12:void 0},meridiem:function(e,t,n){return e<4?"ರಾತ್ರಿ":e<10?"ಬೆಳಿಗ್ಗೆ":e<17?"ಮಧ್ಯಾಹ್ನ":e<20?"ಸಂಜೆ":"ರಾತ್ರಿ"},dayOfMonthOrdinalParse:/\d{1,2}(ನೇ)/,ordinal:function(e){return e+"ನೇ"},week:{dow:0,doy:6}});return r}))},"3eb3":function(e,t,n){},"3f8c":function(e,t){e.exports={}},"3fcc":function(e,t,n){"use strict";var r=n("ebb5"),i=n("b727").map,a=n("4840"),o=r.aTypedArray,s=r.aTypedArrayConstructor,l=r.exportTypedArrayMethod;l("map",(function(e){return i(o(this),e,arguments.length>1?arguments[1]:void 0,(function(e,t){return new(s(a(e,e.constructor)))(t)}))}))},"3fed":function(e,t,n){"use strict";n("74fb")},"408a":function(e,t,n){var r=n("c6b6");e.exports=function(e){if("number"!=typeof e&&"Number"!=r(e))throw TypeError("Incorrect invocation");return+e}},4160:function(e,t,n){"use strict";var r=n("23e7"),i=n("17c2");r({target:"Array",proto:!0,forced:[].forEach!=i},{forEach:i})},"423e":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("ar-kw",{months:"يناير_فبراير_مارس_أبريل_ماي_يونيو_يوليوز_غشت_شتنبر_أكتوبر_نونبر_دجنبر".split("_"),monthsShort:"يناير_فبراير_مارس_أبريل_ماي_يونيو_يوليوز_غشت_شتنبر_أكتوبر_نونبر_دجنبر".split("_"),weekdays:"الأحد_الإتنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت".split("_"),weekdaysShort:"احد_اتنين_ثلاثاء_اربعاء_خميس_جمعة_سبت".split("_"),weekdaysMin:"ح_ن_ث_ر_خ_ج_س".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[اليوم على الساعة] LT",nextDay:"[غدا على الساعة] LT",nextWeek:"dddd [على الساعة] LT",lastDay:"[أمس على الساعة] LT",lastWeek:"dddd [على الساعة] LT",sameElse:"L"},relativeTime:{future:"في %s",past:"منذ %s",s:"ثوان",ss:"%d ثانية",m:"دقيقة",mm:"%d دقائق",h:"ساعة",hh:"%d ساعات",d:"يوم",dd:"%d أيام",M:"شهر",MM:"%d أشهر",y:"سنة",yy:"%d سنوات"},week:{dow:0,doy:12}});return t}))},"428f":function(e,t,n){var r=n("da84");e.exports=r},4301:function(e,t,n){},"440c":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e,t,n,r){var i={m:["eng Minutt","enger Minutt"],h:["eng Stonn","enger Stonn"],d:["een Dag","engem Dag"],M:["ee Mount","engem Mount"],y:["ee Joer","engem Joer"]};return t?i[n][0]:i[n][1]}function n(e){var t=e.substr(0,e.indexOf(" "));return i(t)?"a "+e:"an "+e}function r(e){var t=e.substr(0,e.indexOf(" "));return i(t)?"viru "+e:"virun "+e}function i(e){if(e=parseInt(e,10),isNaN(e))return!1;if(e<0)return!0;if(e<10)return 4<=e&&e<=7;if(e<100){var t=e%10,n=e/10;return i(0===t?n:t)}if(e<1e4){while(e>=10)e/=10;return i(e)}return e/=1e3,i(e)}var a=e.defineLocale("lb",{months:"Januar_Februar_Mäerz_Abrëll_Mee_Juni_Juli_August_September_Oktober_November_Dezember".split("_"),monthsShort:"Jan._Febr._Mrz._Abr._Mee_Jun._Jul._Aug._Sept._Okt._Nov._Dez.".split("_"),monthsParseExact:!0,weekdays:"Sonndeg_Méindeg_Dënschdeg_Mëttwoch_Donneschdeg_Freideg_Samschdeg".split("_"),weekdaysShort:"So._Mé._Dë._Më._Do._Fr._Sa.".split("_"),weekdaysMin:"So_Mé_Dë_Më_Do_Fr_Sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm [Auer]",LTS:"H:mm:ss [Auer]",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm [Auer]",LLLL:"dddd, D. MMMM YYYY H:mm [Auer]"},calendar:{sameDay:"[Haut um] LT",sameElse:"L",nextDay:"[Muer um] LT",nextWeek:"dddd [um] LT",lastDay:"[Gëschter um] LT",lastWeek:function(){switch(this.day()){case 2:case 4:return"[Leschten] dddd [um] LT";default:return"[Leschte] dddd [um] LT"}}},relativeTime:{future:n,past:r,s:"e puer Sekonnen",ss:"%d Sekonnen",m:t,mm:"%d Minutten",h:t,hh:"%d Stonnen",d:t,dd:"%d Deeg",M:t,MM:"%d Méint",y:t,yy:"%d Joer"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return a}))},"44ad":function(e,t,n){var r=n("d039"),i=n("c6b6"),a="".split;e.exports=r((function(){return!Object("z").propertyIsEnumerable(0)}))?function(e){return"String"==i(e)?a.call(e,""):Object(e)}:Object},"44d2":function(e,t,n){var r=n("b622"),i=n("7c73"),a=n("9bf2"),o=r("unscopables"),s=Array.prototype;void 0==s[o]&&a.f(s,o,{configurable:!0,value:i(null)}),e.exports=function(e){s[o][e]=!0}},"44dc":function(e,t,n){"use strict";n("9e3a")},"44de":function(e,t,n){var r=n("da84");e.exports=function(e,t){var n=r.console;n&&n.error&&(1===arguments.length?n.error(e):n.error(e,t))}},"44e7":function(e,t,n){var r=n("861d"),i=n("c6b6"),a=n("b622"),o=a("match");e.exports=function(e){var t;return r(e)&&(void 0!==(t=e[o])?!!t:"RegExp"==i(e))}},"45fc":function(e,t,n){"use strict";var r=n("23e7"),i=n("b727").some,a=n("a640"),o=n("ae40"),s=a("some"),l=o("some");r({target:"Array",proto:!0,forced:!s||!l},{some:function(e){return i(this,e,arguments.length>1?arguments[1]:void 0)}})},"466d":function(e,t,n){"use strict";var r=n("d784"),i=n("825a"),a=n("50c4"),o=n("1d80"),s=n("8aa5"),l=n("14c3");r("match",1,(function(e,t,n){return[function(t){var n=o(this),r=void 0==t?void 0:t[e];return void 0!==r?r.call(t,n):new RegExp(t)[e](String(n))},function(e){var r=n(t,e,this);if(r.done)return r.value;var o=i(e),u=String(this);if(!o.global)return l(o,u);var c=o.unicode;o.lastIndex=0;var d,f=[],h=0;while(null!==(d=l(o,u))){var p=String(d[0]);f[h]=p,""===p&&(o.lastIndex=s(u,a(o.lastIndex),c)),h++}return 0===h?null:f}]}))},4678:function(e,t,n){var r={"./af":"2bfb","./af.js":"2bfb","./ar":"8e73","./ar-dz":"a356","./ar-dz.js":"a356","./ar-kw":"423e","./ar-kw.js":"423e","./ar-ly":"1cfd","./ar-ly.js":"1cfd","./ar-ma":"0a84","./ar-ma.js":"0a84","./ar-sa":"8230","./ar-sa.js":"8230","./ar-tn":"6d83","./ar-tn.js":"6d83","./ar.js":"8e73","./az":"485c","./az.js":"485c","./be":"1fc1","./be.js":"1fc1","./bg":"84aa","./bg.js":"84aa","./bm":"a7fa","./bm.js":"a7fa","./bn":"9043","./bn-bd":"9686","./bn-bd.js":"9686","./bn.js":"9043","./bo":"d26a","./bo.js":"d26a","./br":"6887","./br.js":"6887","./bs":"2554","./bs.js":"2554","./ca":"d716","./ca.js":"d716","./cs":"3c0d","./cs.js":"3c0d","./cv":"03ec","./cv.js":"03ec","./cy":"9797","./cy.js":"9797","./da":"0f14","./da.js":"0f14","./de":"b469","./de-at":"b3eb","./de-at.js":"b3eb","./de-ch":"bb71","./de-ch.js":"bb71","./de.js":"b469","./dv":"598a","./dv.js":"598a","./el":"8d47","./el.js":"8d47","./en-au":"0e6b","./en-au.js":"0e6b","./en-ca":"3886","./en-ca.js":"3886","./en-gb":"39a6","./en-gb.js":"39a6","./en-ie":"e1d3","./en-ie.js":"e1d3","./en-il":"7333","./en-il.js":"7333","./en-in":"ec2e","./en-in.js":"ec2e","./en-nz":"6f50","./en-nz.js":"6f50","./en-sg":"b7e9","./en-sg.js":"b7e9","./eo":"65db","./eo.js":"65db","./es":"898b","./es-do":"0a3c","./es-do.js":"0a3c","./es-mx":"b5b7","./es-mx.js":"b5b7","./es-us":"55c9","./es-us.js":"55c9","./es.js":"898b","./et":"ec18","./et.js":"ec18","./eu":"0ff2","./eu.js":"0ff2","./fa":"8df4","./fa.js":"8df4","./fi":"81e9","./fi.js":"81e9","./fil":"d69a","./fil.js":"d69a","./fo":"0721","./fo.js":"0721","./fr":"9f26","./fr-ca":"d9f8","./fr-ca.js":"d9f8","./fr-ch":"0e49","./fr-ch.js":"0e49","./fr.js":"9f26","./fy":"7118","./fy.js":"7118","./ga":"5120","./ga.js":"5120","./gd":"f6b4","./gd.js":"f6b4","./gl":"8840","./gl.js":"8840","./gom-deva":"aaf2","./gom-deva.js":"aaf2","./gom-latn":"0caa","./gom-latn.js":"0caa","./gu":"e0c5","./gu.js":"e0c5","./he":"c7aa","./he.js":"c7aa","./hi":"dc4d","./hi.js":"dc4d","./hr":"4ba9","./hr.js":"4ba9","./hu":"5b14","./hu.js":"5b14","./hy-am":"d6b6","./hy-am.js":"d6b6","./id":"5038","./id.js":"5038","./is":"0558","./is.js":"0558","./it":"6e98","./it-ch":"6f12","./it-ch.js":"6f12","./it.js":"6e98","./ja":"079e","./ja.js":"079e","./jv":"b540","./jv.js":"b540","./ka":"201b","./ka.js":"201b","./kk":"6d79","./kk.js":"6d79","./km":"e81d","./km.js":"e81d","./kn":"3e92","./kn.js":"3e92","./ko":"22f8","./ko.js":"22f8","./ku":"2421","./ku.js":"2421","./ky":"9609","./ky.js":"9609","./lb":"440c","./lb.js":"440c","./lo":"b29d","./lo.js":"b29d","./lt":"26f9","./lt.js":"26f9","./lv":"b97c","./lv.js":"b97c","./me":"293c","./me.js":"293c","./mi":"688b","./mi.js":"688b","./mk":"6909","./mk.js":"6909","./ml":"02fb","./ml.js":"02fb","./mn":"958b","./mn.js":"958b","./mr":"39bd","./mr.js":"39bd","./ms":"ebe4","./ms-my":"6403","./ms-my.js":"6403","./ms.js":"ebe4","./mt":"1b45","./mt.js":"1b45","./my":"8689","./my.js":"8689","./nb":"6ce3","./nb.js":"6ce3","./ne":"3a39","./ne.js":"3a39","./nl":"facd","./nl-be":"db29","./nl-be.js":"db29","./nl.js":"facd","./nn":"b84c","./nn.js":"b84c","./oc-lnc":"167b","./oc-lnc.js":"167b","./pa-in":"f3ff","./pa-in.js":"f3ff","./pl":"8d57","./pl.js":"8d57","./pt":"f260","./pt-br":"d2d4","./pt-br.js":"d2d4","./pt.js":"f260","./ro":"972c","./ro.js":"972c","./ru":"957c","./ru.js":"957c","./sd":"6784","./sd.js":"6784","./se":"ffff","./se.js":"ffff","./si":"eda5","./si.js":"eda5","./sk":"7be6","./sk.js":"7be6","./sl":"8155","./sl.js":"8155","./sq":"c8f3","./sq.js":"c8f3","./sr":"cf1e","./sr-cyrl":"13e9","./sr-cyrl.js":"13e9","./sr.js":"cf1e","./ss":"52bd","./ss.js":"52bd","./sv":"5fbd","./sv.js":"5fbd","./sw":"74dc","./sw.js":"74dc","./ta":"3de5","./ta.js":"3de5","./te":"5cbb","./te.js":"5cbb","./tet":"576c","./tet.js":"576c","./tg":"3b1b","./tg.js":"3b1b","./th":"10e8","./th.js":"10e8","./tk":"5aff","./tk.js":"5aff","./tl-ph":"0f38","./tl-ph.js":"0f38","./tlh":"cf75","./tlh.js":"cf75","./tr":"0e81","./tr.js":"0e81","./tzl":"cf51","./tzl.js":"cf51","./tzm":"c109","./tzm-latn":"b53d","./tzm-latn.js":"b53d","./tzm.js":"c109","./ug-cn":"6117","./ug-cn.js":"6117","./uk":"ada2","./uk.js":"ada2","./ur":"5294","./ur.js":"5294","./uz":"2e8c","./uz-latn":"010e","./uz-latn.js":"010e","./uz.js":"2e8c","./vi":"2921","./vi.js":"2921","./x-pseudo":"fd7e","./x-pseudo.js":"fd7e","./yo":"7f33","./yo.js":"7f33","./zh-cn":"5c3a","./zh-cn.js":"5c3a","./zh-hk":"49ab","./zh-hk.js":"49ab","./zh-mo":"3a6c","./zh-mo.js":"3a6c","./zh-tw":"90ea","./zh-tw.js":"90ea"};function i(e){var t=a(e);return n(t)}function a(e){if(!n.o(r,e)){var t=new Error("Cannot find module '"+e+"'");throw t.code="MODULE_NOT_FOUND",t}return r[e]}i.keys=function(){return Object.keys(r)},i.resolve=a,e.exports=i,i.id="4678"},"46fd":function(e,t,n){var r=n("22e3"),i=n("faf4"),a=n("6abd"),o=n("3d5c");function s(e,t){return r(e)||i(e,t)||a(e,t)||o()}e.exports=s},"481b":function(e,t,n){"use strict";n("32ba")},4840:function(e,t,n){var r=n("825a"),i=n("1c0b"),a=n("b622"),o=a("species");e.exports=function(e,t){var n,a=r(e).constructor;return void 0===a||void 0==(n=r(a)[o])?t:i(n)}},"485c":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"-inci",5:"-inci",8:"-inci",70:"-inci",80:"-inci",2:"-nci",7:"-nci",20:"-nci",50:"-nci",3:"-üncü",4:"-üncü",100:"-üncü",6:"-ncı",9:"-uncu",10:"-uncu",30:"-uncu",60:"-ıncı",90:"-ıncı"},n=e.defineLocale("az",{months:"yanvar_fevral_mart_aprel_may_iyun_iyul_avqust_sentyabr_oktyabr_noyabr_dekabr".split("_"),monthsShort:"yan_fev_mar_apr_may_iyn_iyl_avq_sen_okt_noy_dek".split("_"),weekdays:"Bazar_Bazar ertəsi_Çərşənbə axşamı_Çərşənbə_Cümə axşamı_Cümə_Şənbə".split("_"),weekdaysShort:"Baz_BzE_ÇAx_Çər_CAx_Cüm_Şən".split("_"),weekdaysMin:"Bz_BE_ÇA_Çə_CA_Cü_Şə".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[bugün saat] LT",nextDay:"[sabah saat] LT",nextWeek:"[gələn həftə] dddd [saat] LT",lastDay:"[dünən] LT",lastWeek:"[keçən həftə] dddd [saat] LT",sameElse:"L"},relativeTime:{future:"%s sonra",past:"%s əvvəl",s:"bir neçə saniyə",ss:"%d saniyə",m:"bir dəqiqə",mm:"%d dəqiqə",h:"bir saat",hh:"%d saat",d:"bir gün",dd:"%d gün",M:"bir ay",MM:"%d ay",y:"bir il",yy:"%d il"},meridiemParse:/gecə|səhər|gündüz|axşam/,isPM:function(e){return/^(gündüz|axşam)$/.test(e)},meridiem:function(e,t,n){return e<4?"gecə":e<12?"səhər":e<17?"gündüz":"axşam"},dayOfMonthOrdinalParse:/\d{1,2}-(ıncı|inci|nci|üncü|ncı|uncu)/,ordinal:function(e){if(0===e)return e+"-ıncı";var n=e%10,r=e%100-n,i=e>=100?100:null;return e+(t[n]||t[r]||t[i])},week:{dow:1,doy:7}});return n}))},4885:function(e,t,n){},4930:function(e,t,n){var r=n("d039");e.exports=!!Object.getOwnPropertySymbols&&!r((function(){return!String(Symbol())}))},4946:function(e,t,n){},"498a":function(e,t,n){"use strict";var r=n("23e7"),i=n("58a8").trim,a=n("c8d2");r({target:"String",proto:!0,forced:a("trim")},{trim:function(){return i(this)}})},"49ab":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("zh-hk",{months:"一月_二月_三月_四月_五月_六月_七月_八月_九月_十月_十一月_十二月".split("_"),monthsShort:"1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月".split("_"),weekdays:"星期日_星期一_星期二_星期三_星期四_星期五_星期六".split("_"),weekdaysShort:"週日_週一_週二_週三_週四_週五_週六".split("_"),weekdaysMin:"日_一_二_三_四_五_六".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY/MM/DD",LL:"YYYY年M月D日",LLL:"YYYY年M月D日 HH:mm",LLLL:"YYYY年M月D日dddd HH:mm",l:"YYYY/M/D",ll:"YYYY年M月D日",lll:"YYYY年M月D日 HH:mm",llll:"YYYY年M月D日dddd HH:mm"},meridiemParse:/凌晨|早上|上午|中午|下午|晚上/,meridiemHour:function(e,t){return 12===e&&(e=0),"凌晨"===t||"早上"===t||"上午"===t?e:"中午"===t?e>=11?e:e+12:"下午"===t||"晚上"===t?e+12:void 0},meridiem:function(e,t,n){var r=100*e+t;return r<600?"凌晨":r<900?"早上":r<1200?"上午":1200===r?"中午":r<1800?"下午":"晚上"},calendar:{sameDay:"[今天]LT",nextDay:"[明天]LT",nextWeek:"[下]ddddLT",lastDay:"[昨天]LT",lastWeek:"[上]ddddLT",sameElse:"L"},dayOfMonthOrdinalParse:/\d{1,2}(日|月|週)/,ordinal:function(e,t){switch(t){case"d":case"D":case"DDD":return e+"日";case"M":return e+"月";case"w":case"W":return e+"週";default:return e}},relativeTime:{future:"%s後",past:"%s前",s:"幾秒",ss:"%d 秒",m:"1 分鐘",mm:"%d 分鐘",h:"1 小時",hh:"%d 小時",d:"1 天",dd:"%d 天",M:"1 個月",MM:"%d 個月",y:"1 年",yy:"%d 年"}});return t}))},"49c8":function(e,t,n){},"49dd":function(e,t,n){"use strict";n("d69e")},"4b10":function(e,t,n){},"4ba9":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e,t,n){var r=e+" ";switch(n){case"ss":return r+=1===e?"sekunda":2===e||3===e||4===e?"sekunde":"sekundi",r;case"m":return t?"jedna minuta":"jedne minute";case"mm":return r+=1===e?"minuta":2===e||3===e||4===e?"minute":"minuta",r;case"h":return t?"jedan sat":"jednog sata";case"hh":return r+=1===e?"sat":2===e||3===e||4===e?"sata":"sati",r;case"dd":return r+=1===e?"dan":"dana",r;case"MM":return r+=1===e?"mjesec":2===e||3===e||4===e?"mjeseca":"mjeseci",r;case"yy":return r+=1===e?"godina":2===e||3===e||4===e?"godine":"godina",r}}var n=e.defineLocale("hr",{months:{format:"siječnja_veljače_ožujka_travnja_svibnja_lipnja_srpnja_kolovoza_rujna_listopada_studenoga_prosinca".split("_"),standalone:"siječanj_veljača_ožujak_travanj_svibanj_lipanj_srpanj_kolovoz_rujan_listopad_studeni_prosinac".split("_")},monthsShort:"sij._velj._ožu._tra._svi._lip._srp._kol._ruj._lis._stu._pro.".split("_"),monthsParseExact:!0,weekdays:"nedjelja_ponedjeljak_utorak_srijeda_četvrtak_petak_subota".split("_"),weekdaysShort:"ned._pon._uto._sri._čet._pet._sub.".split("_"),weekdaysMin:"ne_po_ut_sr_če_pe_su".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"Do MMMM YYYY",LLL:"Do MMMM YYYY H:mm",LLLL:"dddd, Do MMMM YYYY H:mm"},calendar:{sameDay:"[danas u] LT",nextDay:"[sutra u] LT",nextWeek:function(){switch(this.day()){case 0:return"[u] [nedjelju] [u] LT";case 3:return"[u] [srijedu] [u] LT";case 6:return"[u] [subotu] [u] LT";case 1:case 2:case 4:case 5:return"[u] dddd [u] LT"}},lastDay:"[jučer u] LT",lastWeek:function(){switch(this.day()){case 0:return"[prošlu] [nedjelju] [u] LT";case 3:return"[prošlu] [srijedu] [u] LT";case 6:return"[prošle] [subote] [u] LT";case 1:case 2:case 4:case 5:return"[prošli] dddd [u] LT"}},sameElse:"L"},relativeTime:{future:"za %s",past:"prije %s",s:"par sekundi",ss:t,m:t,mm:t,h:t,hh:t,d:"dan",dd:t,M:"mjesec",MM:t,y:"godinu",yy:t},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}});return n}))},"4c89":function(e,t,n){},"4d63":function(e,t,n){var r=n("83ab"),i=n("da84"),a=n("94ca"),o=n("7156"),s=n("9bf2").f,l=n("241c").f,u=n("44e7"),c=n("ad6d"),d=n("9f7f"),f=n("6eeb"),h=n("d039"),p=n("69f3").set,m=n("2626"),_=n("b622"),v=_("match"),g=i.RegExp,b=g.prototype,y=/a/g,M=/a/g,k=new g(y)!==y,w=d.UNSUPPORTED_Y,L=r&&a("RegExp",!k||w||h((function(){return M[v]=!1,g(y)!=y||g(M)==M||"/a/i"!=g(y,"i")})));if(L){var x=function(e,t){var n,r=this instanceof x,i=u(e),a=void 0===t;if(!r&&i&&e.constructor===x&&a)return e;k?i&&!a&&(e=e.source):e instanceof x&&(a&&(t=c.call(e)),e=e.source),w&&(n=!!t&&t.indexOf("y")>-1,n&&(t=t.replace(/y/g,"")));var s=o(k?new g(e,t):g(e,t),r?this:b,x);return w&&n&&p(s,{sticky:n}),s},S=function(e){e in x||s(x,e,{configurable:!0,get:function(){return g[e]},set:function(t){g[e]=t}})},T=l(g),D=0;while(T.length>D)S(T[D++]);b.constructor=x,x.prototype=b,f(i,"RegExp",x)}m("RegExp")},"4d64":function(e,t,n){var r=n("fc6a"),i=n("50c4"),a=n("23cb"),o=function(e){return function(t,n,o){var s,l=r(t),u=i(l.length),c=a(o,u);if(e&&n!=n){while(u>c)if(s=l[c++],s!=s)return!0}else for(;u>c;c++)if((e||c in l)&&l[c]===n)return e||c||0;return!e&&-1}};e.exports={includes:o(!0),indexOf:o(!1)}},"4db6":function(e,t,n){"use strict";n("c9ff")},"4de4":function(e,t,n){"use strict";var r=n("23e7"),i=n("b727").filter,a=n("1dde"),o=n("ae40"),s=a("filter"),l=o("filter");r({target:"Array",proto:!0,forced:!s||!l},{filter:function(e){return i(this,e,arguments.length>1?arguments[1]:void 0)}})},"4dea":function(e,t,n){"use strict";n("7957")},"4df4":function(e,t,n){"use strict";var r=n("0366"),i=n("7b0b"),a=n("9bdd"),o=n("e95a"),s=n("50c4"),l=n("8418"),u=n("35a1");e.exports=function(e){var t,n,c,d,f,h,p=i(e),m="function"==typeof this?this:Array,_=arguments.length,v=_>1?arguments[1]:void 0,g=void 0!==v,b=u(p),y=0;if(g&&(v=r(v,_>2?arguments[2]:void 0,2)),void 0==b||m==Array&&o(b))for(t=s(p.length),n=new m(t);t>y;y++)h=g?v(p[y],y):p[y],l(n,y,h);else for(d=b.call(p),f=d.next,n=new m;!(c=f.call(d)).done;y++)h=g?a(d,v,[c.value,y],!0):c.value,l(n,y,h);return n.length=y,n}},"4eb9":function(e,t,n){"use strict";n("21fd")},"4ec9":function(e,t,n){"use strict";var r=n("6d61"),i=n("6566");e.exports=r("Map",(function(e){return function(){return e(this,arguments.length?arguments[0]:void 0)}}),i)},"4fad":function(e,t,n){var r=n("23e7"),i=n("6f53").entries;r({target:"Object",stat:!0},{entries:function(e){return i(e)}})},"4fbc":function(e,t,n){"use strict";n("648a")},5038:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("id",{months:"Januari_Februari_Maret_April_Mei_Juni_Juli_Agustus_September_Oktober_November_Desember".split("_"),monthsShort:"Jan_Feb_Mar_Apr_Mei_Jun_Jul_Agt_Sep_Okt_Nov_Des".split("_"),weekdays:"Minggu_Senin_Selasa_Rabu_Kamis_Jumat_Sabtu".split("_"),weekdaysShort:"Min_Sen_Sel_Rab_Kam_Jum_Sab".split("_"),weekdaysMin:"Mg_Sn_Sl_Rb_Km_Jm_Sb".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [pukul] HH.mm",LLLL:"dddd, D MMMM YYYY [pukul] HH.mm"},meridiemParse:/pagi|siang|sore|malam/,meridiemHour:function(e,t){return 12===e&&(e=0),"pagi"===t?e:"siang"===t?e>=11?e:e+12:"sore"===t||"malam"===t?e+12:void 0},meridiem:function(e,t,n){return e<11?"pagi":e<15?"siang":e<19?"sore":"malam"},calendar:{sameDay:"[Hari ini pukul] LT",nextDay:"[Besok pukul] LT",nextWeek:"dddd [pukul] LT",lastDay:"[Kemarin pukul] LT",lastWeek:"dddd [lalu pukul] LT",sameElse:"L"},relativeTime:{future:"dalam %s",past:"%s yang lalu",s:"beberapa detik",ss:"%d detik",m:"semenit",mm:"%d menit",h:"sejam",hh:"%d jam",d:"sehari",dd:"%d hari",M:"sebulan",MM:"%d bulan",y:"setahun",yy:"%d tahun"},week:{dow:0,doy:6}});return t}))},"50c4":function(e,t,n){var r=n("a691"),i=Math.min;e.exports=function(e){return e>0?i(r(e),9007199254740991):0}},5120:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=["Eanáir","Feabhra","Márta","Aibreán","Bealtaine","Meitheamh","Iúil","Lúnasa","Meán Fómhair","Deireadh Fómhair","Samhain","Nollaig"],n=["Ean","Feabh","Márt","Aib","Beal","Meith","Iúil","Lún","M.F.","D.F.","Samh","Noll"],r=["Dé Domhnaigh","Dé Luain","Dé Máirt","Dé Céadaoin","Déardaoin","Dé hAoine","Dé Sathairn"],i=["Domh","Luan","Máirt","Céad","Déar","Aoine","Sath"],a=["Do","Lu","Má","Cé","Dé","A","Sa"],o=e.defineLocale("ga",{months:t,monthsShort:n,monthsParseExact:!0,weekdays:r,weekdaysShort:i,weekdaysMin:a,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Inniu ag] LT",nextDay:"[Amárach ag] LT",nextWeek:"dddd [ag] LT",lastDay:"[Inné ag] LT",lastWeek:"dddd [seo caite] [ag] LT",sameElse:"L"},relativeTime:{future:"i %s",past:"%s ó shin",s:"cúpla soicind",ss:"%d soicind",m:"nóiméad",mm:"%d nóiméad",h:"uair an chloig",hh:"%d uair an chloig",d:"lá",dd:"%d lá",M:"mí",MM:"%d míonna",y:"bliain",yy:"%d bliain"},dayOfMonthOrdinalParse:/\d{1,2}(d|na|mh)/,ordinal:function(e){var t=1===e?"d":e%10===2?"na":"mh";return e+t},week:{dow:1,doy:4}});return o}))},5135:function(e,t){var n={}.hasOwnProperty;e.exports=function(e,t){return n.call(e,t)}},5194:function(e,t,n){},5294:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=["جنوری","فروری","مارچ","اپریل","مئی","جون","جولائی","اگست","ستمبر","اکتوبر","نومبر","دسمبر"],n=["اتوار","پیر","منگل","بدھ","جمعرات","جمعہ","ہفتہ"],r=e.defineLocale("ur",{months:t,monthsShort:t,weekdays:n,weekdaysShort:n,weekdaysMin:n,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd، D MMMM YYYY HH:mm"},meridiemParse:/صبح|شام/,isPM:function(e){return"شام"===e},meridiem:function(e,t,n){return e<12?"صبح":"شام"},calendar:{sameDay:"[آج بوقت] LT",nextDay:"[کل بوقت] LT",nextWeek:"dddd [بوقت] LT",lastDay:"[گذشتہ روز بوقت] LT",lastWeek:"[گذشتہ] dddd [بوقت] LT",sameElse:"L"},relativeTime:{future:"%s بعد",past:"%s قبل",s:"چند سیکنڈ",ss:"%d سیکنڈ",m:"ایک منٹ",mm:"%d منٹ",h:"ایک گھنٹہ",hh:"%d گھنٹے",d:"ایک دن",dd:"%d دن",M:"ایک ماہ",MM:"%d ماہ",y:"ایک سال",yy:"%d سال"},preparse:function(e){return e.replace(/،/g,",")},postformat:function(e){return e.replace(/,/g,"،")},week:{dow:1,doy:4}});return r}))},"52bd":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("ss",{months:"Bhimbidvwane_Indlovana_Indlov'lenkhulu_Mabasa_Inkhwekhweti_Inhlaba_Kholwane_Ingci_Inyoni_Imphala_Lweti_Ingongoni".split("_"),monthsShort:"Bhi_Ina_Inu_Mab_Ink_Inh_Kho_Igc_Iny_Imp_Lwe_Igo".split("_"),weekdays:"Lisontfo_Umsombuluko_Lesibili_Lesitsatfu_Lesine_Lesihlanu_Umgcibelo".split("_"),weekdaysShort:"Lis_Umb_Lsb_Les_Lsi_Lsh_Umg".split("_"),weekdaysMin:"Li_Us_Lb_Lt_Ls_Lh_Ug".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendar:{sameDay:"[Namuhla nga] LT",nextDay:"[Kusasa nga] LT",nextWeek:"dddd [nga] LT",lastDay:"[Itolo nga] LT",lastWeek:"dddd [leliphelile] [nga] LT",sameElse:"L"},relativeTime:{future:"nga %s",past:"wenteka nga %s",s:"emizuzwana lomcane",ss:"%d mzuzwana",m:"umzuzu",mm:"%d emizuzu",h:"lihora",hh:"%d emahora",d:"lilanga",dd:"%d emalanga",M:"inyanga",MM:"%d tinyanga",y:"umnyaka",yy:"%d iminyaka"},meridiemParse:/ekuseni|emini|entsambama|ebusuku/,meridiem:function(e,t,n){return e<11?"ekuseni":e<15?"emini":e<19?"entsambama":"ebusuku"},meridiemHour:function(e,t){return 12===e&&(e=0),"ekuseni"===t?e:"emini"===t?e>=11?e:e+12:"entsambama"===t||"ebusuku"===t?0===e?0:e+12:void 0},dayOfMonthOrdinalParse:/\d{1,2}/,ordinal:"%d",week:{dow:1,doy:4}});return t}))},"52f6":function(e,t,n){"use strict";(function(t){
/*!
 * shallow-clone <https://github.com/jonschlinkert/shallow-clone>
 *
 * Copyright (c) 2015-present, Jon Schlinkert.
 * Released under the MIT License.
 */
const r=Symbol.prototype.valueOf,i=n("ef5d");function a(e,t){switch(i(e)){case"array":return e.slice();case"object":return Object.assign({},e);case"date":return new e.constructor(Number(e));case"map":return new Map(e);case"set":return new Set(e);case"buffer":return u(e);case"symbol":return c(e);case"arraybuffer":return s(e);case"float32array":case"float64array":case"int16array":case"int32array":case"int8array":case"uint16array":case"uint32array":case"uint8clampedarray":case"uint8array":return l(e);case"regexp":return o(e);case"error":return Object.create(e);default:return e}}function o(e){const t=void 0!==e.flags?e.flags:/\w+$/.exec(e)||void 0,n=new e.constructor(e.source,t);return n.lastIndex=e.lastIndex,n}function s(e){const t=new e.constructor(e.byteLength);return new Uint8Array(t).set(new Uint8Array(e)),t}function l(e,t){return new e.constructor(e.buffer,e.byteOffset,e.length)}function u(e){const n=e.length,r=t.allocUnsafe?t.allocUnsafe(n):t.from(n);return e.copy(r),r}function c(e){return r?Object(r.call(e)):{}}e.exports=a}).call(this,n("b639").Buffer)},5319:function(e,t,n){"use strict";var r=n("d784"),i=n("825a"),a=n("7b0b"),o=n("50c4"),s=n("a691"),l=n("1d80"),u=n("8aa5"),c=n("14c3"),d=Math.max,f=Math.min,h=Math.floor,p=/\$([$&'`]|\d\d?|<[^>]*>)/g,m=/\$([$&'`]|\d\d?)/g,_=function(e){return void 0===e?e:String(e)};r("replace",2,(function(e,t,n,r){var v=r.REGEXP_REPLACE_SUBSTITUTES_UNDEFINED_CAPTURE,g=r.REPLACE_KEEPS_$0,b=v?"$":"$0";return[function(n,r){var i=l(this),a=void 0==n?void 0:n[e];return void 0!==a?a.call(n,i,r):t.call(String(i),n,r)},function(e,r){if(!v&&g||"string"===typeof r&&-1===r.indexOf(b)){var a=n(t,e,this,r);if(a.done)return a.value}var l=i(e),h=String(this),p="function"===typeof r;p||(r=String(r));var m=l.global;if(m){var M=l.unicode;l.lastIndex=0}var k=[];while(1){var w=c(l,h);if(null===w)break;if(k.push(w),!m)break;var L=String(w[0]);""===L&&(l.lastIndex=u(h,o(l.lastIndex),M))}for(var x="",S=0,T=0;T<k.length;T++){w=k[T];for(var D=String(w[0]),Y=d(f(s(w.index),h.length),0),C=[],E=1;E<w.length;E++)C.push(_(w[E]));var A=w.groups;if(p){var O=[D].concat(C,Y,h);void 0!==A&&O.push(A);var P=String(r.apply(void 0,O))}else P=y(D,h,Y,C,A,r);Y>=S&&(x+=h.slice(S,Y)+P,S=Y+D.length)}return x+h.slice(S)}];function y(e,n,r,i,o,s){var l=r+e.length,u=i.length,c=m;return void 0!==o&&(o=a(o),c=p),t.call(s,c,(function(t,a){var s;switch(a.charAt(0)){case"$":return"$";case"&":return e;case"`":return n.slice(0,r);case"'":return n.slice(l);case"<":s=o[a.slice(1,-1)];break;default:var c=+a;if(0===c)return t;if(c>u){var d=h(c/10);return 0===d?t:d<=u?void 0===i[d-1]?a.charAt(1):i[d-1]+a.charAt(1):t}s=i[c-1]}return void 0===s?"":s}))}}))},5387:function(e,t,n){},"544b":function(e,t,n){"use strict";n("3d42")},"55c9":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t="ene._feb._mar._abr._may._jun._jul._ago._sep._oct._nov._dic.".split("_"),n="ene_feb_mar_abr_may_jun_jul_ago_sep_oct_nov_dic".split("_"),r=[/^ene/i,/^feb/i,/^mar/i,/^abr/i,/^may/i,/^jun/i,/^jul/i,/^ago/i,/^sep/i,/^oct/i,/^nov/i,/^dic/i],i=/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre|ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,a=e.defineLocale("es-us",{months:"enero_febrero_marzo_abril_mayo_junio_julio_agosto_septiembre_octubre_noviembre_diciembre".split("_"),monthsShort:function(e,r){return e?/-MMM-/.test(r)?n[e.month()]:t[e.month()]:t},monthsRegex:i,monthsShortRegex:i,monthsStrictRegex:/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre)/i,monthsShortStrictRegex:/^(ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,monthsParse:r,longMonthsParse:r,shortMonthsParse:r,weekdays:"domingo_lunes_martes_miércoles_jueves_viernes_sábado".split("_"),weekdaysShort:"dom._lun._mar._mié._jue._vie._sáb.".split("_"),weekdaysMin:"do_lu_ma_mi_ju_vi_sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"MM/DD/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY h:mm A",LLLL:"dddd, D [de] MMMM [de] YYYY h:mm A"},calendar:{sameDay:function(){return"[hoy a la"+(1!==this.hours()?"s":"")+"] LT"},nextDay:function(){return"[mañana a la"+(1!==this.hours()?"s":"")+"] LT"},nextWeek:function(){return"dddd [a la"+(1!==this.hours()?"s":"")+"] LT"},lastDay:function(){return"[ayer a la"+(1!==this.hours()?"s":"")+"] LT"},lastWeek:function(){return"[el] dddd [pasado a la"+(1!==this.hours()?"s":"")+"] LT"},sameElse:"L"},relativeTime:{future:"en %s",past:"hace %s",s:"unos segundos",ss:"%d segundos",m:"un minuto",mm:"%d minutos",h:"una hora",hh:"%d horas",d:"un día",dd:"%d días",w:"una semana",ww:"%d semanas",M:"un mes",MM:"%d meses",y:"un año",yy:"%d años"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:0,doy:6}});return a}))},5659:function(e,t,n){},5692:function(e,t,n){var r=n("c430"),i=n("c6cd");(e.exports=function(e,t){return i[e]||(i[e]=void 0!==t?t:{})})("versions",[]).push({version:"3.8.0",mode:r?"pure":"global",copyright:"© 2020 Denis Pushkarev (zloirock.ru)"})},"56c0":function(e,t,n){},"56ef":function(e,t,n){var r=n("d066"),i=n("241c"),a=n("7418"),o=n("825a");e.exports=r("Reflect","ownKeys")||function(e){var t=i.f(o(e)),n=a.f;return n?t.concat(n(e)):t}},5729:function(e,t,n){},"576c":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("tet",{months:"Janeiru_Fevereiru_Marsu_Abril_Maiu_Juñu_Jullu_Agustu_Setembru_Outubru_Novembru_Dezembru".split("_"),monthsShort:"Jan_Fev_Mar_Abr_Mai_Jun_Jul_Ago_Set_Out_Nov_Dez".split("_"),weekdays:"Domingu_Segunda_Tersa_Kuarta_Kinta_Sesta_Sabadu".split("_"),weekdaysShort:"Dom_Seg_Ters_Kua_Kint_Sest_Sab".split("_"),weekdaysMin:"Do_Seg_Te_Ku_Ki_Ses_Sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Ohin iha] LT",nextDay:"[Aban iha] LT",nextWeek:"dddd [iha] LT",lastDay:"[Horiseik iha] LT",lastWeek:"dddd [semana kotuk] [iha] LT",sameElse:"L"},relativeTime:{future:"iha %s",past:"%s liuba",s:"segundu balun",ss:"segundu %d",m:"minutu ida",mm:"minutu %d",h:"oras ida",hh:"oras %d",d:"loron ida",dd:"loron %d",M:"fulan ida",MM:"fulan %d",y:"tinan ida",yy:"tinan %d"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(e){var t=e%10,n=1===~~(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th";return e+n},week:{dow:1,doy:4}});return t}))},5899:function(e,t){e.exports="\t\n\v\f\r                　\u2028\u2029\ufeff"},"58a8":function(e,t,n){var r=n("1d80"),i=n("5899"),a="["+i+"]",o=RegExp("^"+a+a+"*"),s=RegExp(a+a+"*$"),l=function(e){return function(t){var n=String(r(t));return 1&e&&(n=n.replace(o,"")),2&e&&(n=n.replace(s,"")),n}};e.exports={start:l(1),end:l(2),trim:l(3)}},"58be":function(e,t,n){"use strict";n("643b")},"598a":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=["ޖެނުއަރީ","ފެބްރުއަރީ","މާރިޗު","އޭޕްރީލު","މޭ","ޖޫން","ޖުލައި","އޯގަސްޓު","ސެޕްޓެމްބަރު","އޮކްޓޯބަރު","ނޮވެމްބަރު","ޑިސެމްބަރު"],n=["އާދިއްތަ","ހޯމަ","އަންގާރަ","ބުދަ","ބުރާސްފަތި","ހުކުރު","ހޮނިހިރު"],r=e.defineLocale("dv",{months:t,monthsShort:t,weekdays:n,weekdaysShort:n,weekdaysMin:"އާދި_ހޯމަ_އަން_ބުދަ_ބުރާ_ހުކު_ހޮނި".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"D/M/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},meridiemParse:/މކ|މފ/,isPM:function(e){return"މފ"===e},meridiem:function(e,t,n){return e<12?"މކ":"މފ"},calendar:{sameDay:"[މިއަދު] LT",nextDay:"[މާދަމާ] LT",nextWeek:"dddd LT",lastDay:"[އިއްޔެ] LT",lastWeek:"[ފާއިތުވި] dddd LT",sameElse:"L"},relativeTime:{future:"ތެރޭގައި %s",past:"ކުރިން %s",s:"ސިކުންތުކޮޅެއް",ss:"d% ސިކުންތު",m:"މިނިޓެއް",mm:"މިނިޓު %d",h:"ގަޑިއިރެއް",hh:"ގަޑިއިރު %d",d:"ދުވަހެއް",dd:"ދުވަސް %d",M:"މަހެއް",MM:"މަސް %d",y:"އަހަރެއް",yy:"އަހަރު %d"},preparse:function(e){return e.replace(/،/g,",")},postformat:function(e){return e.replace(/,/g,"،")},week:{dow:7,doy:12}});return r}))},"5a34":function(e,t,n){var r=n("44e7");e.exports=function(e){if(r(e))throw TypeError("The method doesn't accept regular expressions");return e}},"5a6e":function(e,t,n){"use strict";n("281c")},"5aff":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"'inji",5:"'inji",8:"'inji",70:"'inji",80:"'inji",2:"'nji",7:"'nji",20:"'nji",50:"'nji",3:"'ünji",4:"'ünji",100:"'ünji",6:"'njy",9:"'unjy",10:"'unjy",30:"'unjy",60:"'ynjy",90:"'ynjy"},n=e.defineLocale("tk",{months:"Ýanwar_Fewral_Mart_Aprel_Maý_Iýun_Iýul_Awgust_Sentýabr_Oktýabr_Noýabr_Dekabr".split("_"),monthsShort:"Ýan_Few_Mar_Apr_Maý_Iýn_Iýl_Awg_Sen_Okt_Noý_Dek".split("_"),weekdays:"Ýekşenbe_Duşenbe_Sişenbe_Çarşenbe_Penşenbe_Anna_Şenbe".split("_"),weekdaysShort:"Ýek_Duş_Siş_Çar_Pen_Ann_Şen".split("_"),weekdaysMin:"Ýk_Dş_Sş_Çr_Pn_An_Şn".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[bugün sagat] LT",nextDay:"[ertir sagat] LT",nextWeek:"[indiki] dddd [sagat] LT",lastDay:"[düýn] LT",lastWeek:"[geçen] dddd [sagat] LT",sameElse:"L"},relativeTime:{future:"%s soň",past:"%s öň",s:"birnäçe sekunt",m:"bir minut",mm:"%d minut",h:"bir sagat",hh:"%d sagat",d:"bir gün",dd:"%d gün",M:"bir aý",MM:"%d aý",y:"bir ýyl",yy:"%d ýyl"},ordinal:function(e,n){switch(n){case"d":case"D":case"Do":case"DD":return e;default:if(0===e)return e+"'unjy";var r=e%10,i=e%100-r,a=e>=100?100:null;return e+(t[r]||t[i]||t[a])}},week:{dow:1,doy:7}});return n}))},"5b14":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t="vasárnap hétfőn kedden szerdán csütörtökön pénteken szombaton".split(" ");function n(e,t,n,r){var i=e;switch(n){case"s":return r||t?"néhány másodperc":"néhány másodperce";case"ss":return i+(r||t)?" másodperc":" másodperce";case"m":return"egy"+(r||t?" perc":" perce");case"mm":return i+(r||t?" perc":" perce");case"h":return"egy"+(r||t?" óra":" órája");case"hh":return i+(r||t?" óra":" órája");case"d":return"egy"+(r||t?" nap":" napja");case"dd":return i+(r||t?" nap":" napja");case"M":return"egy"+(r||t?" hónap":" hónapja");case"MM":return i+(r||t?" hónap":" hónapja");case"y":return"egy"+(r||t?" év":" éve");case"yy":return i+(r||t?" év":" éve")}return""}function r(e){return(e?"":"[múlt] ")+"["+t[this.day()]+"] LT[-kor]"}var i=e.defineLocale("hu",{months:"január_február_március_április_május_június_július_augusztus_szeptember_október_november_december".split("_"),monthsShort:"jan._feb._márc._ápr._máj._jún._júl._aug._szept._okt._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"vasárnap_hétfő_kedd_szerda_csütörtök_péntek_szombat".split("_"),weekdaysShort:"vas_hét_kedd_sze_csüt_pén_szo".split("_"),weekdaysMin:"v_h_k_sze_cs_p_szo".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"YYYY.MM.DD.",LL:"YYYY. MMMM D.",LLL:"YYYY. MMMM D. H:mm",LLLL:"YYYY. MMMM D., dddd H:mm"},meridiemParse:/de|du/i,isPM:function(e){return"u"===e.charAt(1).toLowerCase()},meridiem:function(e,t,n){return e<12?!0===n?"de":"DE":!0===n?"du":"DU"},calendar:{sameDay:"[ma] LT[-kor]",nextDay:"[holnap] LT[-kor]",nextWeek:function(){return r.call(this,!0)},lastDay:"[tegnap] LT[-kor]",lastWeek:function(){return r.call(this,!1)},sameElse:"L"},relativeTime:{future:"%s múlva",past:"%s",s:n,ss:n,m:n,mm:n,h:n,hh:n,d:n,dd:n,M:n,MM:n,y:n,yy:n},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return i}))},"5c3a":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("zh-cn",{months:"一月_二月_三月_四月_五月_六月_七月_八月_九月_十月_十一月_十二月".split("_"),monthsShort:"1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月".split("_"),weekdays:"星期日_星期一_星期二_星期三_星期四_星期五_星期六".split("_"),weekdaysShort:"周日_周一_周二_周三_周四_周五_周六".split("_"),weekdaysMin:"日_一_二_三_四_五_六".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY/MM/DD",LL:"YYYY年M月D日",LLL:"YYYY年M月D日Ah点mm分",LLLL:"YYYY年M月D日ddddAh点mm分",l:"YYYY/M/D",ll:"YYYY年M月D日",lll:"YYYY年M月D日 HH:mm",llll:"YYYY年M月D日dddd HH:mm"},meridiemParse:/凌晨|早上|上午|中午|下午|晚上/,meridiemHour:function(e,t){return 12===e&&(e=0),"凌晨"===t||"早上"===t||"上午"===t?e:"下午"===t||"晚上"===t?e+12:e>=11?e:e+12},meridiem:function(e,t,n){var r=100*e+t;return r<600?"凌晨":r<900?"早上":r<1130?"上午":r<1230?"中午":r<1800?"下午":"晚上"},calendar:{sameDay:"[今天]LT",nextDay:"[明天]LT",nextWeek:function(e){return e.week()!==this.week()?"[下]dddLT":"[本]dddLT"},lastDay:"[昨天]LT",lastWeek:function(e){return this.week()!==e.week()?"[上]dddLT":"[本]dddLT"},sameElse:"L"},dayOfMonthOrdinalParse:/\d{1,2}(日|月|周)/,ordinal:function(e,t){switch(t){case"d":case"D":case"DDD":return e+"日";case"M":return e+"月";case"w":case"W":return e+"周";default:return e}},relativeTime:{future:"%s后",past:"%s前",s:"几秒",ss:"%d 秒",m:"1 分钟",mm:"%d 分钟",h:"1 小时",hh:"%d 小时",d:"1 天",dd:"%d 天",w:"1 周",ww:"%d 周",M:"1 个月",MM:"%d 个月",y:"1 年",yy:"%d 年"},week:{dow:1,doy:4}});return t}))},"5c6c":function(e,t){e.exports=function(e,t){return{enumerable:!(1&e),configurable:!(2&e),writable:!(4&e),value:t}}},"5cbb":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("te",{months:"జనవరి_ఫిబ్రవరి_మార్చి_ఏప్రిల్_మే_జూన్_జులై_ఆగస్టు_సెప్టెంబర్_అక్టోబర్_నవంబర్_డిసెంబర్".split("_"),monthsShort:"జన._ఫిబ్ర._మార్చి_ఏప్రి._మే_జూన్_జులై_ఆగ._సెప్._అక్టో._నవ._డిసె.".split("_"),monthsParseExact:!0,weekdays:"ఆదివారం_సోమవారం_మంగళవారం_బుధవారం_గురువారం_శుక్రవారం_శనివారం".split("_"),weekdaysShort:"ఆది_సోమ_మంగళ_బుధ_గురు_శుక్ర_శని".split("_"),weekdaysMin:"ఆ_సో_మం_బు_గు_శు_శ".split("_"),longDateFormat:{LT:"A h:mm",LTS:"A h:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm",LLLL:"dddd, D MMMM YYYY, A h:mm"},calendar:{sameDay:"[నేడు] LT",nextDay:"[రేపు] LT",nextWeek:"dddd, LT",lastDay:"[నిన్న] LT",lastWeek:"[గత] dddd, LT",sameElse:"L"},relativeTime:{future:"%s లో",past:"%s క్రితం",s:"కొన్ని క్షణాలు",ss:"%d సెకన్లు",m:"ఒక నిమిషం",mm:"%d నిమిషాలు",h:"ఒక గంట",hh:"%d గంటలు",d:"ఒక రోజు",dd:"%d రోజులు",M:"ఒక నెల",MM:"%d నెలలు",y:"ఒక సంవత్సరం",yy:"%d సంవత్సరాలు"},dayOfMonthOrdinalParse:/\d{1,2}వ/,ordinal:"%dవ",meridiemParse:/రాత్రి|ఉదయం|మధ్యాహ్నం|సాయంత్రం/,meridiemHour:function(e,t){return 12===e&&(e=0),"రాత్రి"===t?e<4?e:e+12:"ఉదయం"===t?e:"మధ్యాహ్నం"===t?e>=10?e:e+12:"సాయంత్రం"===t?e+12:void 0},meridiem:function(e,t,n){return e<4?"రాత్రి":e<10?"ఉదయం":e<17?"మధ్యాహ్నం":e<20?"సాయంత్రం":"రాత్రి"},week:{dow:0,doy:6}});return t}))},"5cc6":function(e,t,n){var r=n("74e8");r("Uint8",(function(e){return function(t,n,r){return e(this,t,n,r)}}))},"5d07":function(e,t,n){"use strict";n("0aa5")},"5d3c":function(e,t,n){"use strict";n("97ee")},"5e33":function(e,t,n){"use strict";n("db59")},"5e43":function(e,t,n){},"5f96":function(e,t,n){"use strict";var r=n("ebb5"),i=r.aTypedArray,a=r.exportTypedArrayMethod,o=[].join;a("join",(function(e){return o.apply(i(this),arguments)}))},"5fb2":function(e,t,n){"use strict";var r=2147483647,i=36,a=1,o=26,s=38,l=700,u=72,c=128,d="-",f=/[^\0-\u007E]/,h=/[.\u3002\uFF0E\uFF61]/g,p="Overflow: input needs wider integers to process",m=i-a,_=Math.floor,v=String.fromCharCode,g=function(e){var t=[],n=0,r=e.length;while(n<r){var i=e.charCodeAt(n++);if(i>=55296&&i<=56319&&n<r){var a=e.charCodeAt(n++);56320==(64512&a)?t.push(((1023&i)<<10)+(1023&a)+65536):(t.push(i),n--)}else t.push(i)}return t},b=function(e){return e+22+75*(e<26)},y=function(e,t,n){var r=0;for(e=n?_(e/l):e>>1,e+=_(e/t);e>m*o>>1;r+=i)e=_(e/m);return _(r+(m+1)*e/(e+s))},M=function(e){var t=[];e=g(e);var n,s,l=e.length,f=c,h=0,m=u;for(n=0;n<e.length;n++)s=e[n],s<128&&t.push(v(s));var M=t.length,k=M;M&&t.push(d);while(k<l){var w=r;for(n=0;n<e.length;n++)s=e[n],s>=f&&s<w&&(w=s);var L=k+1;if(w-f>_((r-h)/L))throw RangeError(p);for(h+=(w-f)*L,f=w,n=0;n<e.length;n++){if(s=e[n],s<f&&++h>r)throw RangeError(p);if(s==f){for(var x=h,S=i;;S+=i){var T=S<=m?a:S>=m+o?o:S-m;if(x<T)break;var D=x-T,Y=i-T;t.push(v(b(T+D%Y))),x=_(D/Y)}t.push(v(b(x))),m=y(h,L,k==M),h=0,++k}}++h,++f}return t.join("")};e.exports=function(e){var t,n,r=[],i=e.toLowerCase().replace(h,".").split(".");for(t=0;t<i.length;t++)n=i[t],r.push(f.test(n)?"xn--"+M(n):n);return r.join(".")}},"5fbd":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("sv",{months:"januari_februari_mars_april_maj_juni_juli_augusti_september_oktober_november_december".split("_"),monthsShort:"jan_feb_mar_apr_maj_jun_jul_aug_sep_okt_nov_dec".split("_"),weekdays:"söndag_måndag_tisdag_onsdag_torsdag_fredag_lördag".split("_"),weekdaysShort:"sön_mån_tis_ons_tor_fre_lör".split("_"),weekdaysMin:"sö_må_ti_on_to_fr_lö".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [kl.] HH:mm",LLLL:"dddd D MMMM YYYY [kl.] HH:mm",lll:"D MMM YYYY HH:mm",llll:"ddd D MMM YYYY HH:mm"},calendar:{sameDay:"[Idag] LT",nextDay:"[Imorgon] LT",lastDay:"[Igår] LT",nextWeek:"[På] dddd LT",lastWeek:"[I] dddd[s] LT",sameElse:"L"},relativeTime:{future:"om %s",past:"för %s sedan",s:"några sekunder",ss:"%d sekunder",m:"en minut",mm:"%d minuter",h:"en timme",hh:"%d timmar",d:"en dag",dd:"%d dagar",M:"en månad",MM:"%d månader",y:"ett år",yy:"%d år"},dayOfMonthOrdinalParse:/\d{1,2}(\:e|\:a)/,ordinal:function(e){var t=e%10,n=1===~~(e%100/10)?":e":1===t||2===t?":a":":e";return e+n},week:{dow:1,doy:4}});return t}))},"5fd4":function(e,t,n){},"5ff4":function(e,t,n){},"605d":function(e,t,n){var r=n("c6b6"),i=n("da84");e.exports="process"==r(i.process)},"60bd":function(e,t,n){"use strict";var r=n("da84"),i=n("ebb5"),a=n("e260"),o=n("b622"),s=o("iterator"),l=r.Uint8Array,u=a.values,c=a.keys,d=a.entries,f=i.aTypedArray,h=i.exportTypedArrayMethod,p=l&&l.prototype[s],m=!!p&&("values"==p.name||void 0==p.name),_=function(){return u.call(f(this))};h("entries",(function(){return d.call(f(this))})),h("keys",(function(){return c.call(f(this))})),h("values",_,!m),h(s,_,!m)},"60da":function(e,t,n){"use strict";var r=n("83ab"),i=n("d039"),a=n("df75"),o=n("7418"),s=n("d1e7"),l=n("7b0b"),u=n("44ad"),c=Object.assign,d=Object.defineProperty;e.exports=!c||i((function(){if(r&&1!==c({b:1},c(d({},"a",{enumerable:!0,get:function(){d(this,"b",{value:3,enumerable:!1})}}),{b:2})).b)return!0;var e={},t={},n=Symbol(),i="abcdefghijklmnopqrst";return e[n]=7,i.split("").forEach((function(e){t[e]=e})),7!=c({},e)[n]||a(c({},t)).join("")!=i}))?function(e,t){var n=l(e),i=arguments.length,c=1,d=o.f,f=s.f;while(i>c){var h,p=u(arguments[c++]),m=d?a(p).concat(d(p)):a(p),_=m.length,v=0;while(_>v)h=m[v++],r&&!f.call(p,h)||(n[h]=p[h])}return n}:c},6117:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("ug-cn",{months:"يانۋار_فېۋرال_مارت_ئاپرېل_ماي_ئىيۇن_ئىيۇل_ئاۋغۇست_سېنتەبىر_ئۆكتەبىر_نويابىر_دېكابىر".split("_"),monthsShort:"يانۋار_فېۋرال_مارت_ئاپرېل_ماي_ئىيۇن_ئىيۇل_ئاۋغۇست_سېنتەبىر_ئۆكتەبىر_نويابىر_دېكابىر".split("_"),weekdays:"يەكشەنبە_دۈشەنبە_سەيشەنبە_چارشەنبە_پەيشەنبە_جۈمە_شەنبە".split("_"),weekdaysShort:"يە_دۈ_سە_چا_پە_جۈ_شە".split("_"),weekdaysMin:"يە_دۈ_سە_چا_پە_جۈ_شە".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"YYYY-يىلىM-ئاينىڭD-كۈنى",LLL:"YYYY-يىلىM-ئاينىڭD-كۈنى، HH:mm",LLLL:"dddd، YYYY-يىلىM-ئاينىڭD-كۈنى، HH:mm"},meridiemParse:/يېرىم كېچە|سەھەر|چۈشتىن بۇرۇن|چۈش|چۈشتىن كېيىن|كەچ/,meridiemHour:function(e,t){return 12===e&&(e=0),"يېرىم كېچە"===t||"سەھەر"===t||"چۈشتىن بۇرۇن"===t?e:"چۈشتىن كېيىن"===t||"كەچ"===t?e+12:e>=11?e:e+12},meridiem:function(e,t,n){var r=100*e+t;return r<600?"يېرىم كېچە":r<900?"سەھەر":r<1130?"چۈشتىن بۇرۇن":r<1230?"چۈش":r<1800?"چۈشتىن كېيىن":"كەچ"},calendar:{sameDay:"[بۈگۈن سائەت] LT",nextDay:"[ئەتە سائەت] LT",nextWeek:"[كېلەركى] dddd [سائەت] LT",lastDay:"[تۆنۈگۈن] LT",lastWeek:"[ئالدىنقى] dddd [سائەت] LT",sameElse:"L"},relativeTime:{future:"%s كېيىن",past:"%s بۇرۇن",s:"نەچچە سېكونت",ss:"%d سېكونت",m:"بىر مىنۇت",mm:"%d مىنۇت",h:"بىر سائەت",hh:"%d سائەت",d:"بىر كۈن",dd:"%d كۈن",M:"بىر ئاي",MM:"%d ئاي",y:"بىر يىل",yy:"%d يىل"},dayOfMonthOrdinalParse:/\d{1,2}(-كۈنى|-ئاي|-ھەپتە)/,ordinal:function(e,t){switch(t){case"d":case"D":case"DDD":return e+"-كۈنى";case"w":case"W":return e+"-ھەپتە";default:return e}},preparse:function(e){return e.replace(/،/g,",")},postformat:function(e){return e.replace(/,/g,"،")},week:{dow:1,doy:7}});return t}))},6133:function(e,t,n){"use strict";n("5fd4")},"61fd":function(e,t,n){"use strict";n("5729")},"621a":function(e,t,n){"use strict";var r=n("da84"),i=n("83ab"),a=n("a981"),o=n("9112"),s=n("e2cc"),l=n("d039"),u=n("19aa"),c=n("a691"),d=n("50c4"),f=n("0b25"),h=n("77a7"),p=n("e163"),m=n("d2bb"),_=n("241c").f,v=n("9bf2").f,g=n("81d5"),b=n("d44e"),y=n("69f3"),M=y.get,k=y.set,w="ArrayBuffer",L="DataView",x="prototype",S="Wrong length",T="Wrong index",D=r[w],Y=D,C=r[L],E=C&&C[x],A=Object.prototype,O=r.RangeError,P=h.pack,F=h.unpack,I=function(e){return[255&e]},j=function(e){return[255&e,e>>8&255]},H=function(e){return[255&e,e>>8&255,e>>16&255,e>>24&255]},R=function(e){return e[3]<<24|e[2]<<16|e[1]<<8|e[0]},N=function(e){return P(e,23,4)},$=function(e){return P(e,52,8)},z=function(e,t){v(e[x],t,{get:function(){return M(this)[t]}})},B=function(e,t,n,r){var i=f(n),a=M(e);if(i+t>a.byteLength)throw O(T);var o=M(a.buffer).bytes,s=i+a.byteOffset,l=o.slice(s,s+t);return r?l:l.reverse()},W=function(e,t,n,r,i,a){var o=f(n),s=M(e);if(o+t>s.byteLength)throw O(T);for(var l=M(s.buffer).bytes,u=o+s.byteOffset,c=r(+i),d=0;d<t;d++)l[u+d]=c[a?d:t-d-1]};if(a){if(!l((function(){D(1)}))||!l((function(){new D(-1)}))||l((function(){return new D,new D(1.5),new D(NaN),D.name!=w}))){Y=function(e){return u(this,Y),new D(f(e))};for(var U,V=Y[x]=D[x],q=_(D),G=0;q.length>G;)(U=q[G++])in Y||o(Y,U,D[U]);V.constructor=Y}m&&p(E)!==A&&m(E,A);var J=new C(new Y(2)),K=E.setInt8;J.setInt8(0,2147483648),J.setInt8(1,2147483649),!J.getInt8(0)&&J.getInt8(1)||s(E,{setInt8:function(e,t){K.call(this,e,t<<24>>24)},setUint8:function(e,t){K.call(this,e,t<<24>>24)}},{unsafe:!0})}else Y=function(e){u(this,Y,w);var t=f(e);k(this,{bytes:g.call(new Array(t),0),byteLength:t}),i||(this.byteLength=t)},C=function(e,t,n){u(this,C,L),u(e,Y,L);var r=M(e).byteLength,a=c(t);if(a<0||a>r)throw O("Wrong offset");if(n=void 0===n?r-a:d(n),a+n>r)throw O(S);k(this,{buffer:e,byteLength:n,byteOffset:a}),i||(this.buffer=e,this.byteLength=n,this.byteOffset=a)},i&&(z(Y,"byteLength"),z(C,"buffer"),z(C,"byteLength"),z(C,"byteOffset")),s(C[x],{getInt8:function(e){return B(this,1,e)[0]<<24>>24},getUint8:function(e){return B(this,1,e)[0]},getInt16:function(e){var t=B(this,2,e,arguments.length>1?arguments[1]:void 0);return(t[1]<<8|t[0])<<16>>16},getUint16:function(e){var t=B(this,2,e,arguments.length>1?arguments[1]:void 0);return t[1]<<8|t[0]},getInt32:function(e){return R(B(this,4,e,arguments.length>1?arguments[1]:void 0))},getUint32:function(e){return R(B(this,4,e,arguments.length>1?arguments[1]:void 0))>>>0},getFloat32:function(e){return F(B(this,4,e,arguments.length>1?arguments[1]:void 0),23)},getFloat64:function(e){return F(B(this,8,e,arguments.length>1?arguments[1]:void 0),52)},setInt8:function(e,t){W(this,1,e,I,t)},setUint8:function(e,t){W(this,1,e,I,t)},setInt16:function(e,t){W(this,2,e,j,t,arguments.length>2?arguments[2]:void 0)},setUint16:function(e,t){W(this,2,e,j,t,arguments.length>2?arguments[2]:void 0)},setInt32:function(e,t){W(this,4,e,H,t,arguments.length>2?arguments[2]:void 0)},setUint32:function(e,t){W(this,4,e,H,t,arguments.length>2?arguments[2]:void 0)},setFloat32:function(e,t){W(this,4,e,N,t,arguments.length>2?arguments[2]:void 0)},setFloat64:function(e,t){W(this,8,e,$,t,arguments.length>2?arguments[2]:void 0)}});b(Y,w),b(C,L),e.exports={ArrayBuffer:Y,DataView:C}},"62e4":function(e,t){e.exports=function(e){return e.webpackPolyfill||(e.deprecate=function(){},e.paths=[],e.children||(e.children=[]),Object.defineProperty(e,"loaded",{enumerable:!0,get:function(){return e.l}}),Object.defineProperty(e,"id",{enumerable:!0,get:function(){return e.i}}),e.webpackPolyfill=1),e}},"62f5":function(e,t,n){function r(t){return"function"===typeof Symbol&&"symbol"===typeof Symbol.iterator?e.exports=r=function(e){return typeof e}:e.exports=r=function(e){return e&&"function"===typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e},r(t)}n("a4d3"),n("e01a"),n("d28b"),n("d3b7"),n("3ca3"),n("ddb0"),e.exports=r},6403:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("ms-my",{months:"Januari_Februari_Mac_April_Mei_Jun_Julai_Ogos_September_Oktober_November_Disember".split("_"),monthsShort:"Jan_Feb_Mac_Apr_Mei_Jun_Jul_Ogs_Sep_Okt_Nov_Dis".split("_"),weekdays:"Ahad_Isnin_Selasa_Rabu_Khamis_Jumaat_Sabtu".split("_"),weekdaysShort:"Ahd_Isn_Sel_Rab_Kha_Jum_Sab".split("_"),weekdaysMin:"Ah_Is_Sl_Rb_Km_Jm_Sb".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [pukul] HH.mm",LLLL:"dddd, D MMMM YYYY [pukul] HH.mm"},meridiemParse:/pagi|tengahari|petang|malam/,meridiemHour:function(e,t){return 12===e&&(e=0),"pagi"===t?e:"tengahari"===t?e>=11?e:e+12:"petang"===t||"malam"===t?e+12:void 0},meridiem:function(e,t,n){return e<11?"pagi":e<15?"tengahari":e<19?"petang":"malam"},calendar:{sameDay:"[Hari ini pukul] LT",nextDay:"[Esok pukul] LT",nextWeek:"dddd [pukul] LT",lastDay:"[Kelmarin pukul] LT",lastWeek:"dddd [lepas pukul] LT",sameElse:"L"},relativeTime:{future:"dalam %s",past:"%s yang lepas",s:"beberapa saat",ss:"%d saat",m:"seminit",mm:"%d minit",h:"sejam",hh:"%d jam",d:"sehari",dd:"%d hari",M:"sebulan",MM:"%d bulan",y:"setahun",yy:"%d tahun"},week:{dow:1,doy:7}});return t}))},"643b":function(e,t,n){},"648a":function(e,t,n){},"649e":function(e,t,n){"use strict";var r=n("ebb5"),i=n("b727").some,a=r.aTypedArray,o=r.exportTypedArrayMethod;o("some",(function(e){return i(a(this),e,arguments.length>1?arguments[1]:void 0)}))},6547:function(e,t,n){var r=n("a691"),i=n("1d80"),a=function(e){return function(t,n){var a,o,s=String(i(t)),l=r(n),u=s.length;return l<0||l>=u?e?"":void 0:(a=s.charCodeAt(l),a<55296||a>56319||l+1===u||(o=s.charCodeAt(l+1))<56320||o>57343?e?s.charAt(l):a:e?s.slice(l,l+2):o-56320+(a-55296<<10)+65536)}};e.exports={codeAt:a(!1),charAt:a(!0)}},6566:function(e,t,n){"use strict";var r=n("9bf2").f,i=n("7c73"),a=n("e2cc"),o=n("0366"),s=n("19aa"),l=n("2266"),u=n("7dd0"),c=n("2626"),d=n("83ab"),f=n("f183").fastKey,h=n("69f3"),p=h.set,m=h.getterFor;e.exports={getConstructor:function(e,t,n,u){var c=e((function(e,r){s(e,c,t),p(e,{type:t,index:i(null),first:void 0,last:void 0,size:0}),d||(e.size=0),void 0!=r&&l(r,e[u],{that:e,AS_ENTRIES:n})})),h=m(t),_=function(e,t,n){var r,i,a=h(e),o=v(e,t);return o?o.value=n:(a.last=o={index:i=f(t,!0),key:t,value:n,previous:r=a.last,next:void 0,removed:!1},a.first||(a.first=o),r&&(r.next=o),d?a.size++:e.size++,"F"!==i&&(a.index[i]=o)),e},v=function(e,t){var n,r=h(e),i=f(t);if("F"!==i)return r.index[i];for(n=r.first;n;n=n.next)if(n.key==t)return n};return a(c.prototype,{clear:function(){var e=this,t=h(e),n=t.index,r=t.first;while(r)r.removed=!0,r.previous&&(r.previous=r.previous.next=void 0),delete n[r.index],r=r.next;t.first=t.last=void 0,d?t.size=0:e.size=0},delete:function(e){var t=this,n=h(t),r=v(t,e);if(r){var i=r.next,a=r.previous;delete n.index[r.index],r.removed=!0,a&&(a.next=i),i&&(i.previous=a),n.first==r&&(n.first=i),n.last==r&&(n.last=a),d?n.size--:t.size--}return!!r},forEach:function(e){var t,n=h(this),r=o(e,arguments.length>1?arguments[1]:void 0,3);while(t=t?t.next:n.first){r(t.value,t.key,this);while(t&&t.removed)t=t.previous}},has:function(e){return!!v(this,e)}}),a(c.prototype,n?{get:function(e){var t=v(this,e);return t&&t.value},set:function(e,t){return _(this,0===e?0:e,t)}}:{add:function(e){return _(this,e=0===e?0:e,e)}}),d&&r(c.prototype,"size",{get:function(){return h(this).size}}),c},setStrong:function(e,t,n){var r=t+" Iterator",i=m(t),a=m(r);u(e,t,(function(e,t){p(this,{type:r,target:e,state:i(e),kind:t,last:void 0})}),(function(){var e=a(this),t=e.kind,n=e.last;while(n&&n.removed)n=n.previous;return e.target&&(e.last=n=n?n.next:e.state.first)?"keys"==t?{value:n.key,done:!1}:"values"==t?{value:n.value,done:!1}:{value:[n.key,n.value],done:!1}:(e.target=void 0,{value:void 0,done:!0})}),n?"entries":"values",!n,!0),c(t)}}},"65db":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("eo",{months:"januaro_februaro_marto_aprilo_majo_junio_julio_aŭgusto_septembro_oktobro_novembro_decembro".split("_"),monthsShort:"jan_feb_mart_apr_maj_jun_jul_aŭg_sept_okt_nov_dec".split("_"),weekdays:"dimanĉo_lundo_mardo_merkredo_ĵaŭdo_vendredo_sabato".split("_"),weekdaysShort:"dim_lun_mard_merk_ĵaŭ_ven_sab".split("_"),weekdaysMin:"di_lu_ma_me_ĵa_ve_sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"[la] D[-an de] MMMM, YYYY",LLL:"[la] D[-an de] MMMM, YYYY HH:mm",LLLL:"dddd[n], [la] D[-an de] MMMM, YYYY HH:mm",llll:"ddd, [la] D[-an de] MMM, YYYY HH:mm"},meridiemParse:/[ap]\.t\.m/i,isPM:function(e){return"p"===e.charAt(0).toLowerCase()},meridiem:function(e,t,n){return e>11?n?"p.t.m.":"P.T.M.":n?"a.t.m.":"A.T.M."},calendar:{sameDay:"[Hodiaŭ je] LT",nextDay:"[Morgaŭ je] LT",nextWeek:"dddd[n je] LT",lastDay:"[Hieraŭ je] LT",lastWeek:"[pasintan] dddd[n je] LT",sameElse:"L"},relativeTime:{future:"post %s",past:"antaŭ %s",s:"kelkaj sekundoj",ss:"%d sekundoj",m:"unu minuto",mm:"%d minutoj",h:"unu horo",hh:"%d horoj",d:"unu tago",dd:"%d tagoj",M:"unu monato",MM:"%d monatoj",y:"unu jaro",yy:"%d jaroj"},dayOfMonthOrdinalParse:/\d{1,2}a/,ordinal:"%da",week:{dow:1,doy:7}});return t}))},"65f0":function(e,t,n){var r=n("861d"),i=n("e8b5"),a=n("b622"),o=a("species");e.exports=function(e,t){var n;return i(e)&&(n=e.constructor,"function"!=typeof n||n!==Array&&!i(n.prototype)?r(n)&&(n=n[o],null===n&&(n=void 0)):n=void 0),new(void 0===n?Array:n)(0===t?0:t)}},6784:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=["جنوري","فيبروري","مارچ","اپريل","مئي","جون","جولاءِ","آگسٽ","سيپٽمبر","آڪٽوبر","نومبر","ڊسمبر"],n=["آچر","سومر","اڱارو","اربع","خميس","جمع","ڇنڇر"],r=e.defineLocale("sd",{months:t,monthsShort:t,weekdays:n,weekdaysShort:n,weekdaysMin:n,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd، D MMMM YYYY HH:mm"},meridiemParse:/صبح|شام/,isPM:function(e){return"شام"===e},meridiem:function(e,t,n){return e<12?"صبح":"شام"},calendar:{sameDay:"[اڄ] LT",nextDay:"[سڀاڻي] LT",nextWeek:"dddd [اڳين هفتي تي] LT",lastDay:"[ڪالهه] LT",lastWeek:"[گزريل هفتي] dddd [تي] LT",sameElse:"L"},relativeTime:{future:"%s پوء",past:"%s اڳ",s:"چند سيڪنڊ",ss:"%d سيڪنڊ",m:"هڪ منٽ",mm:"%d منٽ",h:"هڪ ڪلاڪ",hh:"%d ڪلاڪ",d:"هڪ ڏينهن",dd:"%d ڏينهن",M:"هڪ مهينو",MM:"%d مهينا",y:"هڪ سال",yy:"%d سال"},preparse:function(e){return e.replace(/،/g,",")},postformat:function(e){return e.replace(/,/g,"،")},week:{dow:1,doy:4}});return r}))},"679d":function(e,t,n){"use strict";n("a414")},"67fd":function(e,t,n){"use strict";n("5ff4")},6887:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e,t,n){var r={mm:"munutenn",MM:"miz",dd:"devezh"};return e+" "+i(r[n],e)}function n(e){switch(r(e)){case 1:case 3:case 4:case 5:case 9:return e+" bloaz";default:return e+" vloaz"}}function r(e){return e>9?r(e%10):e}function i(e,t){return 2===t?a(e):e}function a(e){var t={m:"v",b:"v",d:"z"};return void 0===t[e.charAt(0)]?e:t[e.charAt(0)]+e.substring(1)}var o=[/^gen/i,/^c[ʼ\']hwe/i,/^meu/i,/^ebr/i,/^mae/i,/^(mez|eve)/i,/^gou/i,/^eos/i,/^gwe/i,/^her/i,/^du/i,/^ker/i],s=/^(genver|c[ʼ\']hwevrer|meurzh|ebrel|mae|mezheven|gouere|eost|gwengolo|here|du|kerzu|gen|c[ʼ\']hwe|meu|ebr|mae|eve|gou|eos|gwe|her|du|ker)/i,l=/^(genver|c[ʼ\']hwevrer|meurzh|ebrel|mae|mezheven|gouere|eost|gwengolo|here|du|kerzu)/i,u=/^(gen|c[ʼ\']hwe|meu|ebr|mae|eve|gou|eos|gwe|her|du|ker)/i,c=[/^sul/i,/^lun/i,/^meurzh/i,/^merc[ʼ\']her/i,/^yaou/i,/^gwener/i,/^sadorn/i],d=[/^Sul/i,/^Lun/i,/^Meu/i,/^Mer/i,/^Yao/i,/^Gwe/i,/^Sad/i],f=[/^Su/i,/^Lu/i,/^Me([^r]|$)/i,/^Mer/i,/^Ya/i,/^Gw/i,/^Sa/i],h=e.defineLocale("br",{months:"Genver_Cʼhwevrer_Meurzh_Ebrel_Mae_Mezheven_Gouere_Eost_Gwengolo_Here_Du_Kerzu".split("_"),monthsShort:"Gen_Cʼhwe_Meu_Ebr_Mae_Eve_Gou_Eos_Gwe_Her_Du_Ker".split("_"),weekdays:"Sul_Lun_Meurzh_Mercʼher_Yaou_Gwener_Sadorn".split("_"),weekdaysShort:"Sul_Lun_Meu_Mer_Yao_Gwe_Sad".split("_"),weekdaysMin:"Su_Lu_Me_Mer_Ya_Gw_Sa".split("_"),weekdaysParse:f,fullWeekdaysParse:c,shortWeekdaysParse:d,minWeekdaysParse:f,monthsRegex:s,monthsShortRegex:s,monthsStrictRegex:l,monthsShortStrictRegex:u,monthsParse:o,longMonthsParse:o,shortMonthsParse:o,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D [a viz] MMMM YYYY",LLL:"D [a viz] MMMM YYYY HH:mm",LLLL:"dddd, D [a viz] MMMM YYYY HH:mm"},calendar:{sameDay:"[Hiziv da] LT",nextDay:"[Warcʼhoazh da] LT",nextWeek:"dddd [da] LT",lastDay:"[Decʼh da] LT",lastWeek:"dddd [paset da] LT",sameElse:"L"},relativeTime:{future:"a-benn %s",past:"%s ʼzo",s:"un nebeud segondennoù",ss:"%d eilenn",m:"ur vunutenn",mm:t,h:"un eur",hh:"%d eur",d:"un devezh",dd:t,M:"ur miz",MM:t,y:"ur bloaz",yy:n},dayOfMonthOrdinalParse:/\d{1,2}(añ|vet)/,ordinal:function(e){var t=1===e?"añ":"vet";return e+t},week:{dow:1,doy:4},meridiemParse:/a.m.|g.m./,isPM:function(e){return"g.m."===e},meridiem:function(e,t,n){return e<12?"a.m.":"g.m."}});return h}))},"688b":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("mi",{months:"Kohi-tāte_Hui-tanguru_Poutū-te-rangi_Paenga-whāwhā_Haratua_Pipiri_Hōngoingoi_Here-turi-kōkā_Mahuru_Whiringa-ā-nuku_Whiringa-ā-rangi_Hakihea".split("_"),monthsShort:"Kohi_Hui_Pou_Pae_Hara_Pipi_Hōngoi_Here_Mahu_Whi-nu_Whi-ra_Haki".split("_"),monthsRegex:/(?:['a-z\u0101\u014D\u016B]+\-?){1,3}/i,monthsStrictRegex:/(?:['a-z\u0101\u014D\u016B]+\-?){1,3}/i,monthsShortRegex:/(?:['a-z\u0101\u014D\u016B]+\-?){1,3}/i,monthsShortStrictRegex:/(?:['a-z\u0101\u014D\u016B]+\-?){1,2}/i,weekdays:"Rātapu_Mane_Tūrei_Wenerei_Tāite_Paraire_Hātarei".split("_"),weekdaysShort:"Ta_Ma_Tū_We_Tāi_Pa_Hā".split("_"),weekdaysMin:"Ta_Ma_Tū_We_Tāi_Pa_Hā".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [i] HH:mm",LLLL:"dddd, D MMMM YYYY [i] HH:mm"},calendar:{sameDay:"[i teie mahana, i] LT",nextDay:"[apopo i] LT",nextWeek:"dddd [i] LT",lastDay:"[inanahi i] LT",lastWeek:"dddd [whakamutunga i] LT",sameElse:"L"},relativeTime:{future:"i roto i %s",past:"%s i mua",s:"te hēkona ruarua",ss:"%d hēkona",m:"he meneti",mm:"%d meneti",h:"te haora",hh:"%d haora",d:"he ra",dd:"%d ra",M:"he marama",MM:"%d marama",y:"he tau",yy:"%d tau"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,doy:4}});return t}))},6909:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("mk",{months:"јануари_февруари_март_април_мај_јуни_јули_август_септември_октомври_ноември_декември".split("_"),monthsShort:"јан_фев_мар_апр_мај_јун_јул_авг_сеп_окт_ное_дек".split("_"),weekdays:"недела_понеделник_вторник_среда_четврток_петок_сабота".split("_"),weekdaysShort:"нед_пон_вто_сре_чет_пет_саб".split("_"),weekdaysMin:"нe_пo_вт_ср_че_пе_сa".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"D.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY H:mm",LLLL:"dddd, D MMMM YYYY H:mm"},calendar:{sameDay:"[Денес во] LT",nextDay:"[Утре во] LT",nextWeek:"[Во] dddd [во] LT",lastDay:"[Вчера во] LT",lastWeek:function(){switch(this.day()){case 0:case 3:case 6:return"[Изминатата] dddd [во] LT";case 1:case 2:case 4:case 5:return"[Изминатиот] dddd [во] LT"}},sameElse:"L"},relativeTime:{future:"за %s",past:"пред %s",s:"неколку секунди",ss:"%d секунди",m:"една минута",mm:"%d минути",h:"еден час",hh:"%d часа",d:"еден ден",dd:"%d дена",M:"еден месец",MM:"%d месеци",y:"една година",yy:"%d години"},dayOfMonthOrdinalParse:/\d{1,2}-(ев|ен|ти|ви|ри|ми)/,ordinal:function(e){var t=e%10,n=e%100;return 0===e?e+"-ев":0===n?e+"-ен":n>10&&n<20?e+"-ти":1===t?e+"-ви":2===t?e+"-ри":7===t||8===t?e+"-ми":e+"-ти"},week:{dow:1,doy:7}});return t}))},"69e7":function(e,t,n){"use strict";n("d8da")},"69f3":function(e,t,n){var r,i,a,o=n("7f9a"),s=n("da84"),l=n("861d"),u=n("9112"),c=n("5135"),d=n("c6cd"),f=n("f772"),h=n("d012"),p=s.WeakMap,m=function(e){return a(e)?i(e):r(e,{})},_=function(e){return function(t){var n;if(!l(t)||(n=i(t)).type!==e)throw TypeError("Incompatible receiver, "+e+" required");return n}};if(o){var v=d.state||(d.state=new p),g=v.get,b=v.has,y=v.set;r=function(e,t){return t.facade=e,y.call(v,e,t),t},i=function(e){return g.call(v,e)||{}},a=function(e){return b.call(v,e)}}else{var M=f("state");h[M]=!0,r=function(e,t){return t.facade=e,u(e,M,t),t},i=function(e){return c(e,M)?e[M]:{}},a=function(e){return c(e,M)}}e.exports={set:r,get:i,has:a,enforce:m,getterFor:_}},"6a5c":function(e,t,n){"use strict";n("c738")},"6abd":function(e,t,n){n("a630"),n("fb6a"),n("b0c0"),n("d3b7"),n("25f0"),n("3ca3");var r=n("e6f9");function i(e,t){if(e){if("string"===typeof e)return r(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?r(e,t):void 0}}e.exports=i},"6c67":function(e,t,n){"use strict";n("8cc4")},"6ce3":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("nb",{months:"januar_februar_mars_april_mai_juni_juli_august_september_oktober_november_desember".split("_"),monthsShort:"jan._feb._mars_apr._mai_juni_juli_aug._sep._okt._nov._des.".split("_"),monthsParseExact:!0,weekdays:"søndag_mandag_tirsdag_onsdag_torsdag_fredag_lørdag".split("_"),weekdaysShort:"sø._ma._ti._on._to._fr._lø.".split("_"),weekdaysMin:"sø_ma_ti_on_to_fr_lø".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY [kl.] HH:mm",LLLL:"dddd D. MMMM YYYY [kl.] HH:mm"},calendar:{sameDay:"[i dag kl.] LT",nextDay:"[i morgen kl.] LT",nextWeek:"dddd [kl.] LT",lastDay:"[i går kl.] LT",lastWeek:"[forrige] dddd [kl.] LT",sameElse:"L"},relativeTime:{future:"om %s",past:"%s siden",s:"noen sekunder",ss:"%d sekunder",m:"ett minutt",mm:"%d minutter",h:"en time",hh:"%d timer",d:"en dag",dd:"%d dager",w:"en uke",ww:"%d uker",M:"en måned",MM:"%d måneder",y:"ett år",yy:"%d år"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return t}))},"6d61":function(e,t,n){"use strict";var r=n("23e7"),i=n("da84"),a=n("94ca"),o=n("6eeb"),s=n("f183"),l=n("2266"),u=n("19aa"),c=n("861d"),d=n("d039"),f=n("1c7e"),h=n("d44e"),p=n("7156");e.exports=function(e,t,n){var m=-1!==e.indexOf("Map"),_=-1!==e.indexOf("Weak"),v=m?"set":"add",g=i[e],b=g&&g.prototype,y=g,M={},k=function(e){var t=b[e];o(b,e,"add"==e?function(e){return t.call(this,0===e?0:e),this}:"delete"==e?function(e){return!(_&&!c(e))&&t.call(this,0===e?0:e)}:"get"==e?function(e){return _&&!c(e)?void 0:t.call(this,0===e?0:e)}:"has"==e?function(e){return!(_&&!c(e))&&t.call(this,0===e?0:e)}:function(e,n){return t.call(this,0===e?0:e,n),this})};if(a(e,"function"!=typeof g||!(_||b.forEach&&!d((function(){(new g).entries().next()})))))y=n.getConstructor(t,e,m,v),s.REQUIRED=!0;else if(a(e,!0)){var w=new y,L=w[v](_?{}:-0,1)!=w,x=d((function(){w.has(1)})),S=f((function(e){new g(e)})),T=!_&&d((function(){var e=new g,t=5;while(t--)e[v](t,t);return!e.has(-0)}));S||(y=t((function(t,n){u(t,y,e);var r=p(new g,t,y);return void 0!=n&&l(n,r[v],{that:r,AS_ENTRIES:m}),r})),y.prototype=b,b.constructor=y),(x||T)&&(k("delete"),k("has"),m&&k("get")),(T||L)&&k(v),_&&b.clear&&delete b.clear}return M[e]=y,r({global:!0,forced:y!=g},M),h(y,e),_||n.setStrong(y,e,m),y}},"6d79":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={0:"-ші",1:"-ші",2:"-ші",3:"-ші",4:"-ші",5:"-ші",6:"-шы",7:"-ші",8:"-ші",9:"-шы",10:"-шы",20:"-шы",30:"-шы",40:"-шы",50:"-ші",60:"-шы",70:"-ші",80:"-ші",90:"-шы",100:"-ші"},n=e.defineLocale("kk",{months:"қаңтар_ақпан_наурыз_сәуір_мамыр_маусым_шілде_тамыз_қыркүйек_қазан_қараша_желтоқсан".split("_"),monthsShort:"қаң_ақп_нау_сәу_мам_мау_шіл_там_қыр_қаз_қар_жел".split("_"),weekdays:"жексенбі_дүйсенбі_сейсенбі_сәрсенбі_бейсенбі_жұма_сенбі".split("_"),weekdaysShort:"жек_дүй_сей_сәр_бей_жұм_сен".split("_"),weekdaysMin:"жк_дй_сй_ср_бй_жм_сн".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Бүгін сағат] LT",nextDay:"[Ертең сағат] LT",nextWeek:"dddd [сағат] LT",lastDay:"[Кеше сағат] LT",lastWeek:"[Өткен аптаның] dddd [сағат] LT",sameElse:"L"},relativeTime:{future:"%s ішінде",past:"%s бұрын",s:"бірнеше секунд",ss:"%d секунд",m:"бір минут",mm:"%d минут",h:"бір сағат",hh:"%d сағат",d:"бір күн",dd:"%d күн",M:"бір ай",MM:"%d ай",y:"бір жыл",yy:"%d жыл"},dayOfMonthOrdinalParse:/\d{1,2}-(ші|шы)/,ordinal:function(e){var n=e%10,r=e>=100?100:null;return e+(t[e]||t[n]||t[r])},week:{dow:1,doy:7}});return n}))},"6d83":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("ar-tn",{months:"جانفي_فيفري_مارس_أفريل_ماي_جوان_جويلية_أوت_سبتمبر_أكتوبر_نوفمبر_ديسمبر".split("_"),monthsShort:"جانفي_فيفري_مارس_أفريل_ماي_جوان_جويلية_أوت_سبتمبر_أكتوبر_نوفمبر_ديسمبر".split("_"),weekdays:"الأحد_الإثنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت".split("_"),weekdaysShort:"أحد_إثنين_ثلاثاء_أربعاء_خميس_جمعة_سبت".split("_"),weekdaysMin:"ح_ن_ث_ر_خ_ج_س".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[اليوم على الساعة] LT",nextDay:"[غدا على الساعة] LT",nextWeek:"dddd [على الساعة] LT",lastDay:"[أمس على الساعة] LT",lastWeek:"dddd [على الساعة] LT",sameElse:"L"},relativeTime:{future:"في %s",past:"منذ %s",s:"ثوان",ss:"%d ثانية",m:"دقيقة",mm:"%d دقائق",h:"ساعة",hh:"%d ساعات",d:"يوم",dd:"%d أيام",M:"شهر",MM:"%d أشهر",y:"سنة",yy:"%d سنوات"},week:{dow:1,doy:4}});return t}))},"6e7c":function(e,t,n){"use strict";n("5e43")},"6e98":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("it",{months:"gennaio_febbraio_marzo_aprile_maggio_giugno_luglio_agosto_settembre_ottobre_novembre_dicembre".split("_"),monthsShort:"gen_feb_mar_apr_mag_giu_lug_ago_set_ott_nov_dic".split("_"),weekdays:"domenica_lunedì_martedì_mercoledì_giovedì_venerdì_sabato".split("_"),weekdaysShort:"dom_lun_mar_mer_gio_ven_sab".split("_"),weekdaysMin:"do_lu_ma_me_gi_ve_sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:function(){return"[Oggi a"+(this.hours()>1?"lle ":0===this.hours()?" ":"ll'")+"]LT"},nextDay:function(){return"[Domani a"+(this.hours()>1?"lle ":0===this.hours()?" ":"ll'")+"]LT"},nextWeek:function(){return"dddd [a"+(this.hours()>1?"lle ":0===this.hours()?" ":"ll'")+"]LT"},lastDay:function(){return"[Ieri a"+(this.hours()>1?"lle ":0===this.hours()?" ":"ll'")+"]LT"},lastWeek:function(){switch(this.day()){case 0:return"[La scorsa] dddd [a"+(this.hours()>1?"lle ":0===this.hours()?" ":"ll'")+"]LT";default:return"[Lo scorso] dddd [a"+(this.hours()>1?"lle ":0===this.hours()?" ":"ll'")+"]LT"}},sameElse:"L"},relativeTime:{future:"tra %s",past:"%s fa",s:"alcuni secondi",ss:"%d secondi",m:"un minuto",mm:"%d minuti",h:"un'ora",hh:"%d ore",d:"un giorno",dd:"%d giorni",w:"una settimana",ww:"%d settimane",M:"un mese",MM:"%d mesi",y:"un anno",yy:"%d anni"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,doy:4}});return t}))},"6eeb":function(e,t,n){var r=n("da84"),i=n("9112"),a=n("5135"),o=n("ce4e"),s=n("8925"),l=n("69f3"),u=l.get,c=l.enforce,d=String(String).split("String");(e.exports=function(e,t,n,s){var l,u=!!s&&!!s.unsafe,f=!!s&&!!s.enumerable,h=!!s&&!!s.noTargetGet;"function"==typeof n&&("string"!=typeof t||a(n,"name")||i(n,"name",t),l=c(n),l.source||(l.source=d.join("string"==typeof t?t:""))),e!==r?(u?!h&&e[t]&&(f=!0):delete e[t],f?e[t]=n:i(e,t,n)):f?e[t]=n:o(t,n)})(Function.prototype,"toString",(function(){return"function"==typeof this&&u(this).source||s(this)}))},"6f12":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("it-ch",{months:"gennaio_febbraio_marzo_aprile_maggio_giugno_luglio_agosto_settembre_ottobre_novembre_dicembre".split("_"),monthsShort:"gen_feb_mar_apr_mag_giu_lug_ago_set_ott_nov_dic".split("_"),weekdays:"domenica_lunedì_martedì_mercoledì_giovedì_venerdì_sabato".split("_"),weekdaysShort:"dom_lun_mar_mer_gio_ven_sab".split("_"),weekdaysMin:"do_lu_ma_me_gi_ve_sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[Oggi alle] LT",nextDay:"[Domani alle] LT",nextWeek:"dddd [alle] LT",lastDay:"[Ieri alle] LT",lastWeek:function(){switch(this.day()){case 0:return"[la scorsa] dddd [alle] LT";default:return"[lo scorso] dddd [alle] LT"}},sameElse:"L"},relativeTime:{future:function(e){return(/^[0-9].+$/.test(e)?"tra":"in")+" "+e},past:"%s fa",s:"alcuni secondi",ss:"%d secondi",m:"un minuto",mm:"%d minuti",h:"un'ora",hh:"%d ore",d:"un giorno",dd:"%d giorni",M:"un mese",MM:"%d mesi",y:"un anno",yy:"%d anni"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,doy:4}});return t}))},"6f50":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("en-nz",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(e){var t=e%10,n=1===~~(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th";return e+n},week:{dow:1,doy:4}});return t}))},"6f53":function(e,t,n){var r=n("83ab"),i=n("df75"),a=n("fc6a"),o=n("d1e7").f,s=function(e){return function(t){var n,s=a(t),l=i(s),u=l.length,c=0,d=[];while(u>c)n=l[c++],r&&!o.call(s,n)||d.push(e?[n,s[n]]:s[n]);return d}};e.exports={entries:s(!0),values:s(!1)}},"710e":function(e,t,n){"use strict";n("5659")},7118:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t="jan._feb._mrt._apr._mai_jun._jul._aug._sep._okt._nov._des.".split("_"),n="jan_feb_mrt_apr_mai_jun_jul_aug_sep_okt_nov_des".split("_"),r=e.defineLocale("fy",{months:"jannewaris_febrewaris_maart_april_maaie_juny_july_augustus_septimber_oktober_novimber_desimber".split("_"),monthsShort:function(e,r){return e?/-MMM-/.test(r)?n[e.month()]:t[e.month()]:t},monthsParseExact:!0,weekdays:"snein_moandei_tiisdei_woansdei_tongersdei_freed_sneon".split("_"),weekdaysShort:"si._mo._ti._wo._to._fr._so.".split("_"),weekdaysMin:"Si_Mo_Ti_Wo_To_Fr_So".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD-MM-YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[hjoed om] LT",nextDay:"[moarn om] LT",nextWeek:"dddd [om] LT",lastDay:"[juster om] LT",lastWeek:"[ôfrûne] dddd [om] LT",sameElse:"L"},relativeTime:{future:"oer %s",past:"%s lyn",s:"in pear sekonden",ss:"%d sekonden",m:"ien minút",mm:"%d minuten",h:"ien oere",hh:"%d oeren",d:"ien dei",dd:"%d dagen",M:"ien moanne",MM:"%d moannen",y:"ien jier",yy:"%d jierren"},dayOfMonthOrdinalParse:/\d{1,2}(ste|de)/,ordinal:function(e){return e+(1===e||8===e||e>=20?"ste":"de")},week:{dow:1,doy:4}});return r}))},7156:function(e,t,n){var r=n("861d"),i=n("d2bb");e.exports=function(e,t,n){var a,o;return i&&"function"==typeof(a=t.constructor)&&a!==n&&r(o=a.prototype)&&o!==n.prototype&&i(e,o),e}},"72f7":function(e,t,n){"use strict";var r=n("ebb5").exportTypedArrayMethod,i=n("d039"),a=n("da84"),o=a.Uint8Array,s=o&&o.prototype||{},l=[].toString,u=[].join;i((function(){l.call({})}))&&(l=function(){return u.call(this)});var c=s.toString!=l;r("toString",l,c)},7333:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("en-il",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(e){var t=e%10,n=1===~~(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th";return e+n}});return t}))},"735e":function(e,t,n){"use strict";var r=n("ebb5"),i=n("81d5"),a=r.aTypedArray,o=r.exportTypedArrayMethod;o("fill",(function(e){return i.apply(a(this),arguments)}))},7418:function(e,t){t.f=Object.getOwnPropertySymbols},"741d":function(e,t,n){},"746a":function(e,t,n){"use strict";n.r(t);n("e260"),n("e6cf"),n("cca6"),n("a79d");var r=n("a026");n("ac1f"),n("5319"),n("a4d3"),n("4de4"),n("4160"),n("e439"),n("dbb4"),n("b64b"),n("159b");function i(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function a(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),n.push.apply(n,r)}return n}function o(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?a(Object(n),!0).forEach((function(t){i(e,t,n[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):a(Object(n)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))}))}return e}var s=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"modal modal--dialog"},[n("span",{attrs:{tabindex:"0"},on:{focus:function(t){return e.setFocusIn(e.$refs.keyboardTrap,!0)}}}),n("div",{ref:"keyboardTrap"},[n("div",{staticClass:"modal__header"},[e.title?n("h2",{staticClass:"modal__title"},[e._v(" "+e._s(e.title)+" ")]):e._e(),e.isLoading?n("spinner"):e._e(),n("button",{staticClass:"modal__closeButton",on:{click:function(t){return e.$modal.hide(e.modalName)}}},[n("span",{attrs:{"aria-hidden":!0}},[e._v("×")]),n("span",{staticClass:"-screenReader"},[e._v(e._s(e.closeLabel))])])],1),n("div",{staticClass:"modal__content"},[e._t("default",[n("div",{domProps:{innerHTML:e._s(e.message)}})]),n("div",{staticClass:"modal__footer"},[e._t("actions",[n("pkp-button",{attrs:{isDisabled:e.isLoading},on:{click:e.fireCallback}},[e._v(" "+e._s(e.confirmLabel)+" ")]),e.cancelLabel?n("pkp-button",{attrs:{isWarnable:!0,isDisabled:e.isLoading},on:{click:function(t){return e.$modal.hide(e.modalName)}}},[e._v(" "+e._s(e.cancelLabel)+" ")]):e._e()])],2)],2)]),n("span",{attrs:{tabindex:"0"},on:{focus:function(t){return e.setFocusIn(e.$refs.keyboardTrap)}}})])},l=[],u=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"modal"},[n("span",{attrs:{tabindex:"0"},on:{focus:function(t){return e.setFocusIn(e.$refs.keyboardTrap,!0)}}}),n("div",{ref:"keyboardTrap"},[n("div",{staticClass:"modal__header"},[e.title?n("h2",{staticClass:"modal__title"},[e._v(" "+e._s(e.title)+" ")]):e._e(),n("button",{staticClass:"modal__closeButton",on:{click:function(t){return e.$modal.hide(e.modalName)}}},[n("span",{attrs:{"aria-hidden":!0}},[e._v("×")]),n("span",{staticClass:"-screenReader"},[e._v(e._s(e.closeLabel))])])]),n("div",{staticClass:"modal__content"},[e._t("default")],2)]),n("span",{attrs:{tabindex:"0"},on:{focus:function(t){return e.setFocusIn(e.$refs.keyboardTrap)}}})])},c=[],d={props:{closeLabel:{type:String,required:!0},modalName:{type:String,reqired:!0},title:{type:String,default:function(){return""}}},mounted:function(){this.setFocusIn(this.$refs.keyboardTrap)}},f=d;n("4eb9");function h(e,t,n,r,i,a,o,s){var l,u="function"===typeof e?e.options:e;if(t&&(u.render=t,u.staticRenderFns=n,u._compiled=!0),r&&(u.functional=!0),a&&(u._scopeId="data-v-"+a),o?(l=function(e){e=e||this.$vnode&&this.$vnode.ssrContext||this.parent&&this.parent.$vnode&&this.parent.$vnode.ssrContext,e||"undefined"===typeof __VUE_SSR_CONTEXT__||(e=__VUE_SSR_CONTEXT__),i&&i.call(this,e),e&&e._registeredComponents&&e._registeredComponents.add(o)},u._ssrRegister=l):i&&(l=s?function(){i.call(this,(u.functional?this.parent:this).$root.$options.shadowRoot)}:i),l)if(u.functional){u._injectStyles=l;var c=u.render;u.render=function(e,t){return l.call(t),c(e,t)}}else{var d=u.beforeCreate;u.beforeCreate=d?[].concat(d,l):[l]}return{exports:e,options:u}}var p=h(f,u,c,!1,null,null,null),m=p.exports,_={extends:m,props:{actions:{type:Array,default:function(){return[]}},callback:{type:Function,required:!0},cancelLabel:{type:String,default:function(){return""}},closeCallback:{type:Function,default:function(){return function(){}}},confirmLabel:{type:String,reqired:!0},message:{type:String,required:!0},title:{type:String,default:function(){return{}}}},data:function(){return{isLoading:!1}},methods:{fireCallback:function(){this.isLoading=!0,this.callback()}},destroyed:function(){"function"===typeof this.closeCallback&&this.closeCallback()}},v=_,g=h(v,s,l,!1,null,null,null),b=g.exports,y={height:"auto",scrollable:!0},M=o(o({},y),{},{classes:"v--modal v--modal-dialog",scrollable:!1}),k={components:{ModalContent:m},data:function(){return{MODAL_PROPS:y,MODAL_PROPS_DIALOG:M}},methods:{openDialog:function(e,t,n){var r=document.activeElement;e.closeLabel=e.closeLabel||this.__("common.close"),e.confirmLabel=e.confirmLabel||this.__("common.yes"),t=t||{},n=n||{},this.$modal.show(b,o(o({},e),{},{closeCallback:function(){r&&r.focus(),e.closeCallback&&e.closeCallback()}}),o(o(o({},M),t),{},{name:e.modalName}),n)},setFocusToRef:function(e){this.$refs[e]&&this.$refs[e].$el.focus()}}},w={mixins:[k],methods:{__:function(e,t){return"undefined"===typeof pkp.localeKeys[e]?"":"undefined"===typeof t?pkp.localeKeys[e]:this.replaceLocaleParams(pkp.localeKeys[e],t)},localize:function(e,t){if(!e)return"";if(void 0!==t)return e.hasOwnProperty(t)?e[t]:"";if(e.hasOwnProperty($.pkp.app.currentLocale)&&e[$.pkp.app.currentLocale])return e[$.pkp.app.currentLocale];if(e.hasOwnProperty($.pkp.app.primaryLocale)&&e[$.pkp.app.primaryLocale])return e[$.pkp.app.primaryLocale];for(var n in e)if(e[n])return e[n];return""},localizeDate:function(e,t){if(!e)return"";e=new Date(e);var n=void 0!==t?t.replace("_","-"):$.pkp.app.currentLocale.replace("_","-");return e.toLocaleDateString(n,{weekday:"long",year:"numeric",month:"long",day:"numeric"})},replaceLocaleParams:function(e,t){for(var n in t){var r=t[n];r===Object(r)&&(r=this.localize(r)),e=e.replace("{$"+n+"}",r)}return e},setFocusIn:function(e,t){var n=e.querySelectorAll('button, [href], input, select, textarea, [tabindex]:not([tabindex="-1"])');n.length&&(t?n[n.length-1].focus():n[0].focus())}}},L=n("1881"),x=n.n(L),S=n("e37d"),T=n("f13c"),D=n.n(T),Y=function(){var e=this,t=e.$createElement,n=e._self._c||t;return e.isButton?n("button",{staticClass:"pkpBadge pkpBadge--button",class:e.classes,on:{click:e.click}},[e._t("default"),e.label?n("span",{staticClass:"-screenReader"},[e._v(e._s(e.label))]):e._e()],2):n("span",{staticClass:"pkpBadge",class:e.classes},[e._t("default"),e.label?n("span",{staticClass:"-screenReader"},[e._v(e._s(e.label))]):e._e()],2)},C=[],E={name:"Badge",props:["label","isPrimary","isSuccess","isWarnable","isButton","hasDot","stage"],computed:{classes:function(){var e=[];return this.isPrimary&&e.push("pkpBadge--isPrimary"),this.isSuccess&&e.push("pkpBadge--isSuccess"),this.isWarnable&&e.push("pkpBadge--isWarnable"),(this.hasDot||this.stage)&&e.push("pkpBadge--dot"),this.stage&&e.push("pkpBadge--"+this.stage),e}},methods:{click:function(){this.$emit("click")}}},A=E,O=(n("58be"),h(A,Y,C,!1,null,null,null)),P=O.exports,F=function(){var e=this,t=e.$createElement,n=e._self._c||t;return"orcid"===e.icon?n("svg",{staticClass:"pkpIcon--orcid",class:e.classes,attrs:{viewBox:"0 0 256 256","aria-hidden":"true"}},[n("path",{staticClass:"pkpIcon--orcid__bg",attrs:{d:"M256,128c0,70.7-57.3,128-128,128C57.3,256,0,198.7,0,128C0,57.3,57.3,0,128,0C198.7,0,256,57.3,256,128z"}}),n("g",[n("path",{staticClass:"pkpIcon--orcid__fill",attrs:{d:"M86.3,186.2H70.9V79.1h15.4v48.4V186.2z"}}),n("path",{staticClass:"pkpIcon--orcid__fill",attrs:{d:"M108.9,79.1h41.6c39.6,0,57,28.3,57,53.6c0,27.5-21.5,53.6-56.8,53.6h-41.8V79.1z M124.3,172.4h24.5\n\t\t\tc34.9,0,42.9-26.5,42.9-39.7c0-21.5-13.7-39.7-43.7-39.7h-23.7V172.4z"}}),n("path",{staticClass:"pkpIcon--orcid__fill",attrs:{d:"M88.7,56.8c0,5.5-4.5,10.1-10.1,10.1c-5.6,0-10.1-4.6-10.1-10.1c0-5.6,4.5-10.1,10.1-10.1\n\t\t\tC84.2,46.7,88.7,51.3,88.7,56.8z"}})])]):n("span",{staticClass:"fa",class:e.classes,attrs:{"aria-hidden":"true"}})},I=[],j={name:"Icon",props:{icon:String,inline:Boolean},computed:{classes:function(){var e=[];return"orcid"!==this.icon&&e.push("fa-"+this.icon),this.inline&&e.push("pkpIcon--inline"),e}}},H=j,R=(n("e82e"),h(H,F,I,!1,null,null,null)),N=R.exports,z=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n(e.element,{tag:"component",staticClass:"pkpButton",class:e.classes,attrs:{href:"a"===e.element&&e.href,disabled:e.isDisabled},on:{click:e.click,focus:function(t){return e.$emit("focus")},blur:function(t){return e.$emit("blur")}}},[e._t("default")],2)},B=[],W={name:"pkpButton",props:{element:{type:String,default:"button"},href:String,isPrimary:Boolean,isWarnable:Boolean,isActive:Boolean,isLink:Boolean,isDisabled:Boolean},computed:{classes:function(){var e=[];return this.isPrimary&&e.push("pkpButton--isPrimary"),this.isWarnable&&e.push("pkpButton--isWarnable"),this.isActive&&e.push("pkpButton--isActive"),this.isLink&&e.push("pkpButton--isLink"),e}},methods:{click:function(e){"button"===this.element&&e.preventDefault(),this.$emit("click")}}},U=W,V=(n("8217"),h(U,z,B,!1,null,null,null)),q=V.exports,G=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("span",{staticClass:"pkpSpinner",attrs:{"aria-hidden":"true"}})},J=[],K=(n("f009"),{}),X=h(K,G,J,!1,null,null,null),Q=X.exports,Z=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"pkpTab",class:e.classes,attrs:{role:"tabpanel",id:e.id,"aria-labelledby":e.id+"-button",hidden:!e.isActive}},[e._t("default")],2)},ee=[],te=(n("a9e3"),{props:{icon:{type:String,default:function(){return""}},id:{type:String,required:!0},label:{type:String,default:function(){return""}},badge:{type:[String,Number],default:function(){return""}}},data:function(){return{isActive:!1}},computed:{classes:function(){var e=[];return this.isActive&&e.push("pkpTab--isActive"),this.isDisabled&&e.push("pkpTab--isDisabled"),e}}}),ne=te,re=h(ne,Z,ee,!1,null,null,null),ie=re.exports,ae=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"pkpTabs",class:{"pkpTabs--side -pkpClearfix":e.isSideTabs}},[n("div",{staticClass:"pkpTabs__buttons",attrs:{role:"tablist","aria-label":e.label}},e._l(e.tabs,(function(t){return n("button",{key:t.id,ref:"button"+t.id,refInFor:!0,staticClass:"pkpTabs__button",attrs:{"aria-selected":e.currentTab===t.id,"aria-controls":t.id,id:t.id+"-button",role:"tab",tabindex:e.currentTab===t.id?"":-1},on:{click:function(n){return e.setCurrentTab(t.id)},keydown:[function(t){return t.type.indexOf("key")||35===t.keyCode?(t.preventDefault(),e.setLastTab(t)):null},function(t){return t.type.indexOf("key")||36===t.keyCode?(t.preventDefault(),e.setFirstTab(t)):null},function(t){return!t.type.indexOf("key")&&e._k(t.keyCode,"left",37,t.key,["Left","ArrowLeft"])||"button"in t&&0!==t.button||t.ctrlKey||t.shiftKey||t.altKey||t.metaKey?null:(t.preventDefault(),e.setPreviousTab(t))},function(t){return!t.type.indexOf("key")&&e._k(t.keyCode,"right",39,t.key,["Right","ArrowRight"])||"button"in t&&2!==t.button||t.ctrlKey||t.shiftKey||t.altKey||t.metaKey?null:(t.preventDefault(),e.setNextTab(t))}]}},[t.icon?[n("icon",{attrs:{icon:t.icon}}),n("span",{staticClass:"-screenReader"},[e._v(e._s(t.label))])]:[e._v(" "+e._s(t.label)+" ")],t.badge?[n("badge",[e._v(e._s(t.badge))])]:e._e()],2)})),0),e._t("default")],2)},oe=[],se=(n("7db0"),n("c740"),n("b0c0"),n("b012")),le=n.n(se),ue={props:{defaultTab:{type:String,default:function(){return""}},isSideTabs:{type:Boolean,default:function(){return!1}},label:{type:String,default:function(){return""}},trackHistory:{type:Boolean,default:function(){return!1}}},data:function(){return{currentTab:"",tabs:[]}},methods:{setCurrentTab:function(e){var t=this;this.currentTab=e,this.$nextTick((function(){$(t.$refs["button"+e]).focus(),t.updateUrl()}))},setFirstTab:function(){this.setCurrentTab(this.tabs[0].id)},setLastTab:function(){this.setCurrentTab(this.tabs[this.tabs.length-1].id)},setNextTab:function(){var e=this,t=this.tabs.findIndex((function(t){return t.id===e.currentTab})),n=this.tabs[t+1]||this.tabs[0];this.setCurrentTab(n.id)},setPreviousTab:function(){var e=this,t=this.tabs.findIndex((function(t){return t.id===e.currentTab})),n=this.tabs[t-1]||this.tabs[this.tabs.length-1];this.setCurrentTab(n.id)},updateUrl:le()((function(){var e=this;if(this.trackHistory){var t="Tab"===this.$parent.$options.name?"#"+this.$parent.id+"/"+this.currentTab:"#"+this.currentTab;if(t!==window.location.hash){var n=this.tabs.find((function(t){return t.id===e.currentTab}));window.history.pushState({},n.label,t)}}}),100)},watch:{currentTab:function(e,t){this.tabs.forEach((function(t){return t.isActive=t.id===e}))}},mounted:function(){var e=this;this.tabs=this.$children.filter((function(e){return"tab"===e.$options._componentTag})),this.currentTab=this.defaultTab||this.tabs[0].id,pkp.eventBus.$on("open-tab",(function(t){e.tabs.forEach((function(n){n.id===t&&e.setCurrentTab(t)}))}))},destroyed:function(){pkp.eventBus.$off("open-tab")}},ce=ue,de=(n("1115"),h(ce,ae,oe,!1,null,null,null)),fe=de.exports,he={_instances:{},init:function(e,t,n){if(void 0!==pkp.controllers[t]){var r={};"function"===typeof pkp.controllers[t].data&&(r=pkp.controllers[t].data());var i=$.extend(!0,{},pkp.controllers[t],{el:"#"+e,data:$.extend(!0,{},r,n,{id:e})});pkp.registry._instances[e]=new pkp.Vue(i);var a=$(pkp.registry._instances[e].$el).parents();a.each((function(t){if($.pkp.classes.Handler.hasHandler($(a[t])))return $.pkp.classes.Handler.getHandler($(a[t])).handlerChildren_.push(pkp.registry._instances[e]),!1}))}}};r["a"].use(x.a,{dynamic:!0,injectModalsContainer:!0}),r["a"].use(S["a"],{defaultTrigger:"click"}),r["a"].use(D.a),r["a"].mixin(w),r["a"].component("Badge",P),r["a"].component("Icon",N),r["a"].component("PkpButton",q),r["a"].component("Spinner",Q),r["a"].component("Tab",ie),r["a"].component("Tabs",fe);var pe={Vue:r["a"],registry:he,eventBus:new r["a"],const:{},localeKeys:{},currentUser:null};n("07ac");function me(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,r=new Array(t);n<t;n++)r[n]=e[n];return r}function _e(e){if(Array.isArray(e))return me(e)}n("e01a"),n("d28b"),n("a630"),n("d3b7"),n("3ca3"),n("ddb0");function ve(e){if("undefined"!==typeof Symbol&&Symbol.iterator in Object(e))return Array.from(e)}n("fb6a"),n("25f0");function ge(e,t){if(e){if("string"===typeof e)return me(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?me(e,t):void 0}}function be(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function ye(e){return _e(e)||ve(e)||ge(e)||be()}var Me=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"listPanel"},[n("div",{staticClass:"listPanel__header"},[e._t("header",[n("pkp-header",[n(e.headingLevel,{tag:"component"},[e._v(e._s(e.title))])],1)])],2),n("div",{staticClass:"listPanel__body"},[n("transition",{attrs:{name:"listPanel__sidebar"}},[e.$slots.sidebar&&e.isSidebarVisible?n("div",{ref:"sidebar",staticClass:"listPanel__sidebar"},[e._t("sidebar")],2):e._e()]),n("div",{staticClass:"listPanel__items"},[e.items.length?n("ul",{staticClass:"listPanel__itemsList"},e._l(e.items,(function(t){return n("li",{key:t.id,staticClass:"listPanel__item"},[e._t("item",[n("div",{staticClass:"listPanel__itemSummary"},[n("div",{staticClass:"listPanel__itemIdentity"},[n("div",{staticClass:"listPanel__itemTitle"},[e._t("itemTitle",[e._v(e._s(t.title))],{item:t})],2),e.$scopedSlots.itemSubtitle||t.subtitle?n("div",{staticClass:"listPanel__itemSubtitle"},[e._t("itemSubtitle",[e._v(" "+e._s(t.subtitle)+" ")],{item:t})],2):e._e()]),e.$scopedSlots.itemActions?n("div",{staticClass:"listPanel__itemActions"},[e._t("itemActions",null,{item:t})],2):e._e()])],{item:t}),e.expanded.includes(t.id)?n("div",{staticClass:"listPanel__itemExpanded"},[e._t("itemExpanded",null,{item:t})],2):e._e()],2)})),0):n("div",{staticClass:"listPanel__empty"},[e._t("itemsEmpty",[e._v(e._s(e.currentEmptyLabel))])],2)])],1),e.$slots.footer?n("div",{staticClass:"listPanel__footer"},[e._t("footer")],2):e._e()])},ke=[],we=(n("caad"),n("2532"),function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"pkpNotification",class:e.classes},[e._t("default"),e.canDismiss?n("button",{staticClass:"pkpNotification__closeButton",on:{click:function(t){return e.$emit("dismiss")}}},[n("span",{attrs:{"aria-hidden":!0}},[e._v("×")]),n("span",{staticClass:"-screenReader"},[e._v(e._s(e.__("common.close")))])]):e._e()],2)}),Le=[],xe={props:{canDismiss:{type:Boolean,default:function(){return!1}},type:{type:String,default:function(){return""}}},computed:{classes:function(){var e=[];return this.type&&e.push("pkpNotification--"+this.type),this.canDismiss&&e.push("pkpNotification--canDismiss"),e}}},Se=xe,Te=(n("e4d7"),h(Se,we,Le,!1,null,null,null)),De=Te.exports,Ye=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"pkpHeader",class:{"-isOneLine":e.isOneLine,"-pkpClearfix":!e.isOneLine}},[n("span",{staticClass:"pkpHeader__title"},[e._t("default")],2),e.hasActions?n("div",{staticClass:"pkpHeader__actions"},[e._t("actions")],2):e._e()])},Ce=[],Ee={props:{isOneLine:{type:Boolean,default:!0}},computed:{hasActions:function(){return this.$slots.actions}}},Ae=Ee,Oe=(n("fb13"),h(Ae,Ye,Ce,!1,null,null,null)),Pe=Oe.exports,Fe={name:"ListPanel",components:{Notification:De,PkpHeader:Pe},props:{description:{type:String,default:function(){return""}},emptyLabel:{type:String,default:function(){return""}},expanded:{type:Array,default:function(){return[]}},headingLevel:{type:String,default:function(){return"h2"}},isSidebarVisible:{type:Boolean,default:function(){return!1}},items:{type:Array,required:!0},title:{type:String,default:function(){return""}}},computed:{currentEmptyLabel:function(){return this.emptyLabel?this.emptyLabel:this.__("common.noItemsFound")}},watch:{isSidebarVisible:function(e,t){var n=this;e&&this.$nextTick((function(){e&&Object.keys(n.$refs).includes("sidebar")&&n.setFocusIn(n.$refs.sidebar)}))}}},Ie=Fe,je=(n("a639"),h(Ie,Me,ke,!1,null,null,null)),He=je.exports,Re=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("form",{staticClass:"pkpForm -pkpClearfix",attrs:{method:e.method,action:e.action}},[e.availableLocales.length>1?n("form-locales",{attrs:{primaryLocaleKey:e.primaryLocale,locales:e.availableLocales,visible:e.visibleLocales},on:{updateLocales:e.setVisibleLocales}}):e._e(),e.pages.length>1?n("div",{staticClass:"pkpForm__pageNav"},[n("ol",{staticClass:"pkpForm__pageNavList"},e._l(e.pages,(function(t){return n("li",{key:t.id,staticClass:"pkpForm__pageNavListItem"},[n("button",{staticClass:"pkpForm__pageNavPage",class:{"pkpForm__pageNavPage--current":t.id===e.currentPage},on:{click:function(n){return n.preventDefault(),e.setCurrentPage(t.id)}}},[e._v(" "+e._s(t.label)+" ")]),e.pageIdsWithErrors.includes(t.id)?n("icon",{attrs:{icon:"exclamation-triangle",inline:!0}}):e._e()],1)})),0)]):e._e(),n("div",{staticClass:"pkpFormPages",class:e.classes},e._l(e.pages,(function(t,r){return n("form-page",e._b({key:t.id,attrs:{groups:e.groups,fields:e.fields,errors:e.errors,formId:e.id,canSubmit:e.canSubmit,isCurrentPage:e.currentPage===t.id,isLastPage:r===e.pages.length-1,lastSaveTimestamp:e.lastSaveTimestamp,primaryLocale:e.primaryLocale,visibleLocales:e.visibleLocales,availableLocales:e.availableLocales,isSaving:e.isSaving},on:{change:e.fieldChanged,pageSubmitted:e.nextPage,previousPage:function(t){return e.setCurrentPage(!1)},showField:e.showField,showLocale:e.showLocale,"set-errors":e.setErrors}},"form-page",t,!1))})),1)],1)},Ne=[];n("d81d");function $e(e){return $e="function"===typeof Symbol&&"symbol"===typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"===typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e},$e(e)}var ze,Be,We=function(){var e=this,t=e.$createElement,n=e._self._c||t;return e.locales.length?n("div",{staticClass:"pkpFormLocales"},[e._l(e.additionalLocales,(function(t){return n("button",{key:t.key,staticClass:"pkpFormLocales__locale",class:{"pkpFormLocales__locale--isActive":e.visible.includes(t.key)},attrs:{type:"button",label:t.label},on:{click:function(n){return n.preventDefault(),e.toggleLocale(t.key)}}},[e._v(" "+e._s(t.label)+" ")])})),n("span",{staticClass:"pkpFormLocales__locale pkpFormLocales__locale--isPrimary"},[e._v(" "+e._s(this.primaryLocale.label)+" ")])],2):e._e()},Ue=[],Ve={name:"FormLocales",props:{locales:{type:Array,required:!0},visible:{type:Array,required:!0},primaryLocaleKey:{type:String,required:!0}},computed:{primaryLocale:function(){var e=this;return this.locales.find((function(t){return t.key===e.primaryLocaleKey}))},additionalLocales:function(){var e=this;return this.locales.filter((function(t){return t.key!==e.primaryLocaleKey}))}},methods:{toggleLocale:function(e){var t=[this.primaryLocaleKey];this.visible.includes(e)||t.push(e),this.updateLocales(t)},updateLocales:function(e){this.$emit("updateLocales",e)}}},qe=Ve,Ge=(n("0da8"),h(qe,We,Ue,!1,null,null,null)),Je=Ge.exports,Ke=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"pkpFormPage",class:{"pkpFormPage--current":e.isCurrentPage}},[e._l(e.groupsInPage,(function(t){return n("form-group",e._b({key:t.id,attrs:{fields:e.fields,errors:e.errors,primaryLocale:e.primaryLocale,visibleLocales:e.visibleLocales,availableLocales:e.availableLocales,formId:e.formId},on:{change:e.fieldChanged,"set-errors":e.setErrors}},"form-group",t,!1))})),e.hasFooter?n("div",{ref:"footer",staticClass:"pkpFormPage__footer",attrs:{"aria-live":"polite"}},[Object.keys(e.errors).length?n("form-errors",{attrs:{errors:e.errors,fields:e.fields},on:{showField:e.showField,showLocale:e.showLocale}}):e._e(),n("span",{attrs:{role:"status","aria-live":"polite","aria-atomic":"true"}},[n("transition",{attrs:{name:"pkpFormPage__status"}},[e.isSaving?n("span",{staticClass:"pkpFormPage__status"},[n("spinner"),e._v(" "+e._s(e.__("common.saving"))+" ")],1):e.hasRecentSave?n("span",{staticClass:"pkpFormPage__status"},[n("icon",{attrs:{icon:"check",inline:!0}}),e._v(" "+e._s(e.__("form.saved"))+" ")],1):e._e()])],1),n("div",{ref:"buttons",staticClass:"pkpFormPage__buttons"},[[e.previousButton?n("pkp-button",e._b({on:{click:e.previousPage}},"pkp-button",e.previousButton,!1),[e._v(" "+e._s(e.previousButton.label)+" ")]):e._e(),e.submitButton?n("pkp-button",e._b({attrs:{disabled:e.isSaving||!e.canSubmit||e.isLastPage&&!!Object.keys(e.errors).length},on:{click:e.submit}},"pkp-button",e.submitButton,!1),[e._v(" "+e._s(e.submitButton.label)+" ")]):e._e()]],2)],1):e._e()],2)},Xe=[],Qe=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"pkpFormErrors"},[n("icon",{attrs:{icon:"exclamation-triangle",inline:!0}}),e._v(" "+e._s(e.message)+" "),n("ul",{staticClass:"-screenReader"},e._l(e.errorList,(function(t,r){return n("li",{key:r},[n("button",{on:{click:function(n){return n.preventDefault(),e.showError(t.fieldName)}}},[e._v(" "+e._s(e.__("form.errorA11y",{fieldLabel:t.label,errorMessage:t.message}))+" ")])])})),0),n("button",{staticClass:"pkpFormErrors__goTo",on:{click:function(t){return t.preventDefault(),e.showNextError(t)}}},[e._v(" "+e._s(e.__("form.errorGoTo"))+" ")])],1)},Ze=[],et={name:"FormErrors",props:{errors:Object,fields:Array},computed:{message:function(){return 1===Object.keys(this.errors).length?this.__("form.errorOne"):this.__("form.errorMany",{count:Object.keys(this.errors).length})},errorList:function(){var e=this;return Object.keys(this.errors).map((function(t){var n,r=e.fields.find((function(e){return e.name===t})),i=r?r.label:t;return n=null!==e.errors[t]&&"object"===$e(e.errors[t])?e.errors[t][Object.keys(e.errors[t])[0]]:e.errors[t],{fieldName:t,label:i,message:n}}))}},methods:{showNextError:function(){this.showError(Object.keys(this.errors)[0])},showError:function(e){var t=this.errors[e];t&&"object"===$e(t)&&t.constructor===Object&&this.$emit("showLocale",Object.keys(t)[0]),this.$emit("showField",e)}}},tt=et,nt=(n("3bdf"),h(tt,Qe,Ze,!1,null,null,null)),rt=nt.exports,it=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("fieldset",{staticClass:"pkpFormGroup -pkpClearfix"},[e.label?n("div",{staticClass:"pkpFormGroup__heading"},[n("legend",{staticClass:"pkpFormGroup__legend"},[e._v(e._s(e.label))]),e.description?n("div",{staticClass:"pkpFormGroup__description",domProps:{innerHTML:e._s(e.description)}}):e._e()]):e._e(),n("div",{staticClass:"pkpFormGroup__fields"},[e._l(e.fieldsInGroup,(function(t){return[t.isMultilingual?[n("div",{key:t.name,staticClass:"pkpFormGroup__localeGroup -pkpClearfix"},e._l(e.availableLocales,(function(r){return n("div",{key:r.key,staticClass:"pkpFormGroup__locale",class:{"pkpFormGroup__locale--isVisible":e.visibleLocales.includes(r.key)}},[n(t.component,e._b({tag:"component",attrs:{allErrors:e.errors,localeKey:r.key,formId:e.formId,primaryLocale:e.primaryLocale,locales:e.availableLocales},on:{change:e.fieldChanged,"set-errors":e.setFieldErrors}},"component",t,!1))],1)})),0)]:[n(t.component,e._b({key:t.name,tag:"component",attrs:{allErrors:e.errors,formId:e.formId},on:{change:e.fieldChanged,"set-errors":e.setFieldErrors}},"component",t,!1))]]}))],2)])},at=[],ot=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"pkpFormField pkpFormField--autosuggest"},[n("div",{staticClass:"pkpFormField__heading"},[n("form-field-label",{attrs:{controlId:e.controlId,label:e.label,localeLabel:e.localeLabel,isRequired:e.isRequired,requiredLabel:e.__("common.required"),multilingualLabel:e.multilingualLabel}}),e.tooltip?n("tooltip",{attrs:{"aria-hidden":"true",tooltip:e.tooltip,label:""}}):e._e(),e.tooltip?n("span",{staticClass:"-screenReader",attrs:{id:e.describedByTooltipId},domProps:{innerHTML:e._s(e.tooltip)}}):e._e(),e.helpTopic?n("help-button",{attrs:{id:e.describedByHelpId,topic:e.helpTopic,section:e.helpSection,label:e.__("help.help")}}):e._e()],1),e.isPrimaryLocale&&e.description?n("div",{staticClass:"pkpFormField__description",attrs:{id:e.describedByDescriptionId},domProps:{innerHTML:e._s(e.description)}}):e._e(),n("div",{staticClass:"pkpFormField__control pkpFormField--autosuggest__control",class:{"pkpFormField__control--hasMultilingualIndicator":e.isMultilingual&&e.locales.length>1}},["inline"===e.currentPosition?n("div",{ref:"values",staticClass:"pkpFormField--autosuggest__values pkpFormField--autosuggest__values--inline",attrs:{id:e.describedBySelectedId}},[n("span",{staticClass:"-screenReader"},[e._v(e._s(e.selectedLabel))]),e.currentValue.length?e._l(e.currentSelected,(function(t){return n("pkp-badge",{key:t.value},[e._v(" "+e._s(t.label)+" "),n("button",{staticClass:"pkpFormField--autosuggest__valueButton",on:{click:function(n){return n.stopPropagation(),n.preventDefault(),e.deselect(t)}}},[n("icon",{attrs:{icon:"times"}}),n("span",{staticClass:"-screenReader"},[e._v(" "+e._s(e.deselectLabel.replace("{$item}",t.label))+" ")])],1)])})):n("span",{staticClass:"-screenReader"},[e._v(" "+e._s(e.__("common.none"))+" ")])],2):e._e(),n("vue-autosuggest",e._b({ref:"autosuggest",staticClass:"pkpFormField--autosuggest__autosuggest",on:{selected:e.selectSuggestion},model:{value:e.inputValue,callback:function(t){e.inputValue=t},expression:"inputValue"}},"vue-autosuggest",e.autosuggestOptions,!1)),"below"===e.currentPosition?n("div",{ref:"values",staticClass:"pkpFormField--autosuggest__values pkpFormField--autosuggest__values--below",attrs:{id:e.describedBySelectedId}},[n("span",{staticClass:"-screenReader"},[e._v(e._s(e.selectedLabel))]),e.currentValue.length?e._l(e.currentSelected,(function(t){return n("pkp-badge",{key:t.value},[e._v(" "+e._s(t.label)+" "),n("button",{staticClass:"pkpFormField--autosuggest__valueButton",on:{click:function(n){return n.stopPropagation(),n.preventDefault(),e.deselect(t)}}},[n("icon",{attrs:{icon:"times"}}),n("span",{staticClass:"-screenReader"},[e._v(" "+e._s(e.deselectLabel.replace("{$item}",t.label))+" ")])],1)])})):n("span",{staticClass:"-screenReader"},[e._v(" "+e._s(e.__("common.none"))+" ")])],2):e._e(),e.isMultilingual&&e.locales.length>1?n("multilingual-progress",{attrs:{id:e.multilingualProgressId,count:e.multilingualFieldsCompleted,total:e.locales.length}}):e._e(),e.errors&&e.errors.length?n("field-error",{attrs:{id:e.describedByErrorId,messages:e.errors}}):e._e()],1)])},st=[],lt=(n("a434"),n("a15b"),function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"pkpFieldError",attrs:{"aria-live":"polite"}},e._l(e.messages,(function(t){return n("div",{key:t,staticClass:"pkpFieldError__message"},[n("icon",{attrs:{icon:"exclamation-triangle",inline:!0}}),n("span",{domProps:{innerHTML:e._s(t)}})],1)})),0)}),ut=[],ct={name:"FieldError",props:{messages:Array}},dt=ct,ft=(n("aaab"),h(dt,lt,ut,!1,null,null,null)),ht=ft.exports,pt=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("label",{staticClass:"pkpFormFieldLabel",attrs:{for:e.controlId}},[e.localeLabel?[n("span",{staticClass:"aria-hidden"},[e._v(e._s(e.localeLabel))]),n("span",{staticClass:"-screenReader"},[e._v(e._s(e.multilingualLabel))])]:[e._v(" "+e._s(e.label)+" ")],e.isRequired?n("span",{staticClass:"pkpFormFieldLabel__required"},[n("span",{staticClass:"aria-hidden"},[e._v("*")]),n("span",{staticClass:"-screenReader"},[e._v(e._s(e.requiredLabel))])]):e._e()],2)},mt=[],_t={name:"FormFieldLabel",props:{controlId:String,label:String,localeLabel:String,multilingualLabel:String,isRequired:{type:Boolean,default:!1},requiredLabel:String}},vt=_t,gt=(n("252a"),h(vt,pt,mt,!1,null,null,null)),bt=gt.exports,yt=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("button",{staticClass:"helpButton requestHelpPanel",attrs:{"data-topic":e.topic,"data-section":e.section},on:{click:function(e){e.preventDefault()}}},[n("icon",{attrs:{icon:"info-circle"}}),n("span",{staticClass:"-screenReader"},[e._v(e._s(e.label))])],1)},Mt=[],kt={name:"HelpButton",props:{topic:{type:String,required:!0},section:String,label:{type:String,required:!0}}},wt=kt,Lt=(n("710e"),h(wt,yt,Mt,!1,null,null,null)),xt=Lt.exports,St=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("button",{directives:[{name:"tooltip",rawName:"v-tooltip",value:e.tooltip,expression:"tooltip"}],staticClass:"tooltipButton",on:{click:function(e){e.preventDefault()}}},[n("icon",{attrs:{icon:"question-circle"}}),n("span",{staticClass:"-screenReader"},[e._v(e._s(e.label))])],1)},Tt=[],Dt={name:"Tooltip",props:{tooltip:{type:String,required:!0},label:{type:String,required:!0}}},Yt=Dt,Ct=(n("1a82"),h(Yt,St,Tt,!1,null,null,null)),Et=Ct.exports,At=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("span",{staticClass:"multilingualProgress",class:e.classes},[n("button",{directives:[{name:"tooltip",rawName:"v-tooltip",value:e.tooltip,expression:"tooltip"}],attrs:{type:"button","aria-hidden":"true"},on:{click:function(e){e.preventDefault()}}},[n("icon",{attrs:{icon:"globe"}})],1),n("span",{staticClass:"-screenReader"},[e._v(e._s(e.tooltip))])])},Ot=[],Pt={name:"MultilingualProgress",props:{count:{type:Number,required:!0},total:{type:Number,required:!0}},computed:{classes:function(){var e=[];return this.count===this.total?e.push("multilingualProgress--isComplete"):this.count?e.push("multilingualProgress--isIncomplete"):e.push("multilingualProgress--isEmpty"),e},tooltip:function(){return this.__("form.multilingualProgress",{count:this.count,total:this.total})}}},Ft=Pt,It=(n("679d"),h(Ft,At,Ot,!1,null,null,null)),jt=It.exports,Ht={name:"FieldBase",components:{FieldError:ht,FormFieldLabel:bt,HelpButton:xt,Tooltip:Et,MultilingualProgress:jt},props:{name:String,component:String,label:String,description:String,tooltip:String,helpTopic:String,helpSection:String,groupId:String,formId:String,isMultilingual:Boolean,isRequired:Boolean,showWhen:[String,Array],primaryLocale:String,localeKey:String,locales:Array,value:{required:!0},allErrors:{type:Object,default:function(){return{}}}},computed:{currentValue:{get:function(){return this.isMultilingual?this.value[this.localeKey]:this.value},set:function(e){this.$emit("change",this.name,"value",e,this.localeKey)}},errors:function(){if(!Object.keys(this.allErrors).includes(this.name))return[];var e=this.allErrors[this.name];return this.isMultilingual&&Object.keys(e).includes(this.localeKey)?e[this.localeKey]:this.isMultilingual?[]:e},localizedName:function(){return this.isMultilingual?this.name+"-"+this.localeKey:this.name},controlId:function(){return this.compileId("control")},describedByTooltipId:function(){return this.compileId("tooltip")},describedByHelpId:function(){return this.compileId("help")},describedByDescriptionId:function(){return this.compileId("description")},describedByErrorId:function(){return this.compileId("error")},multilingualProgressId:function(){return this.compileId("multilingualProgress")},describedByIds:function(){var e=[];return this.description&&e.push(this.describedByDescriptionId),this.tooltip&&e.push(this.describedByTooltipId),this.helpTopic&&e.push(this.describedByHelpId),this.error&&e.push(this.describedByErrorId),this.isMultilingual&&e.push(this.multilingualProgressId),!!e.length&&e.join(" ")},multilingualFieldsCompleted:function(){return this.isMultilingual?Object.values(this.value).filter((function(e){return Array.isArray(e)?e.length:e})).length:0},isPrimaryLocale:function(){return!this.isMultilingual||this.localeKey===this.primaryLocale},localeLabel:function(){var e=this;return!this.isMultilingual||this.isPrimaryLocale?"":this.locales.find((function(t){return t.key===e.localeKey})).label},multilingualLabel:function(){return this.__("form.multilingualLabel",{label:this.label,localeName:this.localeLabel})}},methods:{compileId:function(e){var t=[this.formId,this.name,e];return this.isMultilingual&&t.push(this.localeKey.replace("@","_")),t.join("-")}}},Rt=Ht,Nt=(n("b406"),h(Rt,ze,Be,!1,null,null,null)),$t=Nt.exports,zt={name:"default-section",props:{section:{type:Object,required:!0},currentIndex:{type:[Number,String],required:!1,default:1/0},renderSuggestion:{type:Function,required:!1},normalizeItemFunction:{type:Function,required:!0},componentAttrPrefix:{type:String,required:!0},componentAttrIdAutosuggest:{type:String,required:!0}},data:function(){return{_currentIndex:this.currentIndex}},computed:{list:function(){var e=this.section,t=e.limit,n=e.data;return n.length<t&&(t=n.length),n.slice(0,t)}},methods:{getItemIndex:function(e){return this.section.start_index+e},getItemByIndex:function(e){return this.section.data[e]},onMouseEnter:function(e){var t=parseInt(e.currentTarget.getAttribute("data-suggestion-index"));this._currentIndex=t,this.$emit("updateCurrentIndex",t)},onMouseLeave:function(){this.$emit("updateCurrentIndex",null)}},render:function(e){var t=this,n=this.componentAttrPrefix,r={beforeSection:this.$scopedSlots["before-section-"+this.section.name],afterSectionDefault:this.$scopedSlots["after-section"],afterSectionNamed:this.$scopedSlots["after-section-"+this.section.name]},i=n+"__results-before "+n+"__results-before--"+this.section.name,a=r.beforeSection&&r.beforeSection({section:this.section,className:i})||[];return e("ul",{attrs:{role:"listbox","aria-labelledby":this.section.label&&this.componentAttrIdAutosuggest+"-"+this.section.label},class:this.section.ulClass},[a[0]&&a[0]||this.section.label&&e("li",{class:i,id:this.componentAttrIdAutosuggest+"-"+this.section.label},this.section.label)||"",this.list.map((function(r,i){var a,o=t.normalizeItemFunction(t.section.name,t.section.type,t.section.label,t.section.liClass,r),s=t.getItemIndex(i),l=t._currentIndex===s||parseInt(t.currentIndex)===s;return e("li",{attrs:Object.assign({},{role:"option","data-suggestion-index":s,"data-section-name":o.name,id:n+"__results-item--"+s},o.liAttributes),key:s,class:Object.assign((a={},a[n+"__results-item--highlighted"]=l,a[n+"__results-item"]=!0,a),o.liClass),on:{mouseenter:t.onMouseEnter,mouseleave:t.onMouseLeave}},[t.renderSuggestion?t.renderSuggestion(o):t.$scopedSlots.default&&t.$scopedSlots.default({_key:i,suggestion:o})])})),r.afterSectionDefault&&r.afterSectionDefault({section:this.section,className:n+"__results-after "+n+"__results-after--"+this.section.name}),r.afterSectionNamed&&r.afterSectionNamed({section:this.section,className:n+"__results_after "+n+"__results-after--"+this.section.name})])}};function Bt(e,t){return!!e.className.match(new RegExp("(\\s|^)"+t+"(\\s|$)"))}function Wt(e,t){Bt(e,t)||(e.className+=" "+t)}function Ut(e,t){e.classList&&e.classList.remove(t)}var Vt=-1,qt={name:"default",type:"default-section"},Gt={render:function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{attrs:{id:e.componentAttrIdAutosuggest}},[e._t("before-input"),n("div",{attrs:{role:"combobox","aria-expanded":e.isOpen?"true":"false","aria-haspopup":"listbox","aria-owns":e.componentAttrIdAutosuggest+"-"+e.componentAttrPrefix+"__results"}},[n("input",e._g(e._b({class:[e.isOpen?e.componentAttrPrefix+"__input--open":"",e.internal_inputProps.class],attrs:{type:e.internal_inputProps.type,autocomplete:e.internal_inputProps.autocomplete,"aria-autocomplete":"list","aria-activedescendant":e.isOpen&&null!==e.currentIndex?e.componentAttrPrefix+"__results-item--"+e.currentIndex:"","aria-controls":e.componentAttrIdAutosuggest+"-"+e.componentAttrPrefix+"__results"},domProps:{value:e.internalValue},on:{input:e.inputHandler,keydown:e.handleKeyStroke}},"input",e.internal_inputProps,!1),e.listeners))]),e._t("after-input"),e._v(" "),n("div",{class:e._componentAttrClassAutosuggestResultsContainer,attrs:{id:e.componentAttrIdAutosuggest+"-"+e.componentAttrPrefix+"__results"}},[e.isOpen?n("div",{class:e._componentAttrClassAutosuggestResults,attrs:{"aria-labelledby":e.componentAttrIdAutosuggest}},[e._t("before-suggestions"),e._v(" "),e._l(e.computedSections,(function(t,r){return n(t.type,{key:e.getSectionRef(""+t.name+r),ref:e.getSectionRef(""+t.name+r),refInFor:!0,tag:"component",attrs:{"current-index":e.currentIndex,"normalize-item-function":e.normalizeItem,"render-suggestion":e.renderSuggestion,section:t,"component-attr-prefix":e.componentAttrPrefix,"component-attr-id-autosuggest":e.componentAttrIdAutosuggest},on:{updateCurrentIndex:e.updateCurrentIndex},scopedSlots:e._u([{key:"before-section-"+(t.name||t.label),fn:function(n){var r=n.section,i=n.className;return[e._t("before-section-"+(t.name||t.label),null,{section:r,className:i})]}},{key:"default",fn:function(t){var n=t.suggestion,r=t._key;return[e._t("default",[e._v(" "+e._s(n.item)+" ")],{suggestion:n,index:r})]}},{key:"after-section-"+(t.name||t.label),fn:function(n){var r=n.section;return[e._t("after-section-"+(t.name||t.label),null,{section:r})]}},{key:"after-section",fn:function(t){var n=t.section;return[e._t("after-section",null,{section:n})]}}])})})),e._v(" "),e._t("after-suggestions")],2):e._e(),e._v(" "),e._t("after-suggestions-container")],2)],2)},staticRenderFns:[],name:"Autosuggest",components:{DefaultSection:zt},props:{value:{type:String,default:null},inputProps:{type:Object,required:!0},limit:{type:Number,required:!1,default:1/0},suggestions:{type:Array,required:!0},renderSuggestion:{type:Function,required:!1,default:null},getSuggestionValue:{type:Function,required:!1,default:function(e){var t=e.item;return"object"==typeof t&&t.hasOwnProperty("name")?t.name:t}},shouldRenderSuggestions:{type:Function,required:!1,default:function(e,t){return e>0&&!t}},sectionConfigs:{type:Object,required:!1,default:function(){return{default:{onSelected:null}}}},onSelected:{type:Function,required:!1,default:null},componentAttrIdAutosuggest:{type:String,required:!1,default:"autosuggest"},componentAttrClassAutosuggestResultsContainer:{type:String,required:!1,default:null},componentAttrClassAutosuggestResults:{type:String,required:!1,default:null},componentAttrPrefix:{type:String,required:!1,default:"autosuggest"}},data:function(){return{internalValue:null,searchInputOriginal:null,currentIndex:null,currentItem:null,loading:!1,didSelectFromOptions:!1,defaultInputProps:{type:"text",autocomplete:"off"},clientXMouseDownInitial:null}},computed:{internal_inputProps:function(){return Object.assign({},this.defaultInputProps,this.inputProps)},listeners:function(){var e=this;return Object.assign({},this.$listeners,{input:function(e){},click:function(){e.loading=!1,e.$listeners.click&&e.$listeners.click(e.currentItem),e.$nextTick((function(){e.ensureItemVisible(e.currentItem,e.currentIndex)}))},selected:function(){e.currentItem&&e.sectionConfigs[e.currentItem.name]&&e.sectionConfigs[e.currentItem.name].onSelected?e.sectionConfigs[e.currentItem.name].onSelected(e.currentItem,e.searchInputOriginal):e.sectionConfigs.default.onSelected?e.sectionConfigs.default.onSelected(null,e.searchInputOriginal):e.$listeners.selected&&e.$emit("selected",e.currentItem,e.currentIndex),e.setChangeItem(null)}})},isOpen:function(){return this.shouldRenderSuggestions(this.totalResults,this.loading)},computedSections:function(){var e=this,t=0;return this.suggestions.map((function(n){if(n.data){var r,i,a,o,s=n.name?n.name:qt.name,l=null;e.sectionConfigs[s]&&(r=e.sectionConfigs[s].limit,a=e.sectionConfigs[s].type,i=e.sectionConfigs[s].label,o=e.sectionConfigs[s].ulClass,l=e.sectionConfigs[s].liClass),a=a||qt.type,r=(r=r||e.limit)||1/0,r=n.data.length<r?n.data.length:r;var u={name:s,label:i=i||n.label,type:a,limit:r,data:n.data,start_index:t,end_index:t+r-1,ulClass:o,liClass:l};return t+=r,u}}))},totalResults:function(){return this.computedSections.reduce((function(e,t){if(!t)return e;var n=t.limit,r=t.data;return e+(r.length>=n?n:r.length)}),0)},_componentAttrClassAutosuggestResultsContainer:function(){return this.componentAttrClassAutosuggestResultsContainer||this.componentAttrPrefix+"__results-container"},_componentAttrClassAutosuggestResults:function(){return this.componentAttrClassAutosuggestResults||this.componentAttrPrefix+"__results"}},watch:{value:{handler:function(e){this.internalValue=e},immediate:!0},isOpen:{handler:function(e,t){e!==t&&this.$emit(e?"opened":"closed")},immediate:!0}},created:function(){this.loading=!0},mounted:function(){document.addEventListener("mouseup",this.onDocumentMouseUp),document.addEventListener("mousedown",this.onDocumentMouseDown)},beforeDestroy:function(){document.removeEventListener("mouseup",this.onDocumentMouseUp),document.removeEventListener("mousedown",this.onDocumentMouseDown)},methods:{inputHandler:function(e){var t=e.target.value;this.$emit("input",t),this.internalValue=t,this.didSelectFromOptions||(this.searchInputOriginal=t,this.currentIndex=null)},getSectionRef:function(e){return"computed_section_"+e},getItemByIndex:function(e){var t=!1;if(null===e)return t;for(var n=0;n<this.computedSections.length;n++)if(e>=this.computedSections[n].start_index&&e<=this.computedSections[n].end_index){var r=e-this.computedSections[n].start_index,i=this.computedSections[n].name,a=this.$refs[this.getSectionRef(""+i+n)][0];if(a){t=this.normalizeItem(this.computedSections[n].name,this.computedSections[n].type,a.section.label,a.section.liClass,a.getItemByIndex(r));break}}return t},handleKeyStroke:function(e){var t=this,n=e.keyCode;if(!([16,9,17,18,91,93].indexOf(n)>-1)){var r=!this.isOpen;if(this.loading=!1,this.didSelectFromOptions=!1,this.isOpen)switch(n){case 40:case 38:if(e.preventDefault(),38===n&&null===this.currentIndex)break;var i=40===n?1:-1,a=Math.max((parseInt(this.currentIndex)||0)+(r?0:i),Vt);this.setCurrentIndex(a,this.totalResults),this.didSelectFromOptions=!0,this.totalResults>0&&this.currentIndex>=0?(this.setChangeItem(this.getItemByIndex(this.currentIndex)),this.didSelectFromOptions=!0):this.currentIndex===Vt&&(this.setChangeItem(null),this.internalValue=this.searchInputOriginal,e.preventDefault()),this.$nextTick((function(){t.ensureItemVisible(t.currentItem,t.currentIndex)}));break;case 13:e.preventDefault(),this.totalResults>0&&this.currentIndex>=0&&(this.setChangeItem(this.getItemByIndex(this.currentIndex),!0),this.didSelectFromOptions=!0),this.loading=!0,this.listeners.selected(this.didSelectFromOptions);break;case 27:this.loading=!0,this.currentIndex=null,this.internalValue=this.searchInputOriginal,this.$emit("input",this.searchInputOriginal),e.preventDefault()}}},setChangeItem:function(e,t){if(void 0===t&&(t=!1),null!==this.currentIndex&&e){if(e){this.currentItem=e,this.$emit("item-changed",e,this.currentIndex);var n=this.getSuggestionValue(e);this.internalValue=n,t&&(this.searchInputOriginal=n),this.ensureItemVisible(e,this.currentIndex)}}else this.currentItem=null,this.$emit("item-changed",null,null)},normalizeItem:function(e,t,n,r,i){return{name:e,type:t,label:n,liClass:i.liClass||r,item:i}},ensureItemVisible:function(e,t,n){var r=this.$el.querySelector(n||"."+this._componentAttrClassAutosuggestResults);if(r){var i=r.querySelector("#"+this.componentAttrPrefix+"__results-item--"+t);if(i){var a=r.clientHeight,o=r.scrollTop,s=i.clientHeight,l=i.offsetTop;s+l>=o+a?r.scrollTop=s+l-a:l<o&&o>0&&(r.scrollTop=l)}}},updateCurrentIndex:function(e){this.setCurrentIndex(e,-1,!0)},clickedOnScrollbar:function(e,t){var n=this.$el.querySelector("."+this._componentAttrClassAutosuggestResults),r=n&&n.clientWidth<=t+17&&t+17<=n.clientWidth+34;return"DIV"===e.target.tagName&&n&&r||!1},onDocumentMouseDown:function(e){var t=e.target.getBoundingClientRect?e.target.getBoundingClientRect():0;this.clientXMouseDownInitial=e.clientX-t.left},onDocumentMouseUp:function(e){if(!this.$el.contains(e.target))return this.loading=!0,void(this.currentIndex=null);"INPUT"===e.target.tagName||this.clickedOnScrollbar(e,this.clientXMouseDownInitial)||(this.loading=!0,this.didSelectFromOptions=!0,this.setChangeItem(this.getItemByIndex(this.currentIndex),!0),this.listeners.selected(!0))},setCurrentIndex:function(e,t,n){void 0===t&&(t=-1),void 0===n&&(n=!1);var r=e;n||(null===this.currentIndex||e>=t)&&(r=0),this.currentIndex=r;var i=this.$el.querySelector("#"+this.componentAttrPrefix+"__results-item--"+this.currentIndex),a=this.componentAttrPrefix+"__results-item--highlighted";this.$el.querySelector("."+a)&&Ut(this.$el.querySelector("."+a),a),i&&Wt(i,a)}}},Jt={install:function(e){e.component("vue-autosuggest-default-section",zt),e.component("vue-autosuggest",Gt)}};"undefined"!=typeof window&&window.Vue&&window.Vue.use(Jt);var Kt,Xt,Qt,Zt,en,tn,nn,rn,an,on,sn,ln,un={mixins:[k],methods:{ajaxErrorCallback:function(e){var t,n=this;"status"in e&&0==e.status||(t="responseJSON"in e&&"errorMessage"in e.responseJSON?e.responseJSON.errorMessage:this.__("common.unknownError"),this.openDialog({modalName:"ajaxError",confirmLabel:this.__("common.ok"),title:this.__("common.error"),message:t,callback:function(){return n.$modal.hide("ajaxError")}}))}}},cn=n("89e8"),dn=n.n(cn),fn={name:"FieldBaseAutosuggest",extends:$t,mixins:[un],components:{PkpBadge:P,VueAutosuggest:Gt},props:{apiUrl:{type:String,default:function(){return""}},deselectLabel:{type:String,required:!0},getParams:{type:Object,default:function(){return{}}},initialPosition:{type:String,default:function(){return"inline"},validator:function(e){return["inline","below"].includes(e)}},selected:{type:[Array,Object],default:function(){return this.isMultilingual?{}:[]}},selectedLabel:{type:String,required:!0}},data:function(){return{currentPosition:this.initialPosition,inputValue:"",suggestions:[]}},computed:{autosuggestId:function(){return this.compileId("autosuggest")},autosuggestOptions:function(){return{id:this.autosuggestId,inputProps:this.inputProps,key:this.autosuggestId,suggestions:[{data:this.suggestions}],getSuggestionValue:function(e){return e.item.label},renderSuggestion:function(e){return e.item.label}}},currentValue:function(){var e=this.isMultilingual?this.selected[this.localeKey]:this.selected;return e.map((function(e){return e.value}))},currentSelected:{get:function(){return this.isMultilingual?this.selected[this.localeKey]:this.selected},set:function(e){this.$emit("change",this.name,"selected",e,this.localeKey)}},describedBySelectedId:function(){return this.compileId("selected")},describedByIds:function(){return this.describedBySelectedId+" "+$t.computed.describedByIds.apply(this)},inputProps:function(){return{"aria-describedby":this.describedByIds,class:"pkpFormField__input pkpFormField--autosuggest__input",id:this.controlId,name:this.name}}},methods:{deselect:function(e){this.currentSelected.splice(this.currentSelected.findIndex((function(t){return t.value===e.value})),1)},getSuggestions:le()((function(){if(this.inputValue){var e=this;$.ajax({url:this.apiUrl,type:"GET",data:o(o({},this.getParams),{},{searchPhrase:this.inputValue}),error:function(t){e.ajaxErrorCallback(t)},success:function(t){e.setSuggestions(t.items)}})}else this.suggestions=[]}),250),select:function(e){var t=this;if(!e){if(!this.inputValue||!this.suggestions.length)return;e=this.suggestions[0]}this.currentSelected.push(e),this.inputValue="",this.$nextTick((function(){t.$nextTick((function(){t.$nextTick((function(){return t.$el.querySelector("#"+t.controlId).focus()}))}))}))},selectSuggestion:function(e){this.select(e?e.item:null)},setSuggestions:function(e){throw new Error("The setSuggestions method must be implemented in any component that extends FieldBaseAutosuggest.")},updateInputPadding:function(){var e=this.isMultilingual&&this.locales.length>1?48:16;if(this.currentValue.length){var t=this.$refs.autosuggest.$el.querySelector("#"+this.controlId),n=this.isMultilingual&&this.locales.length>1?48:16,r=n+this.$refs.values.offsetWidth,i=this.$refs.values.offsetHeight>50;i||t.offsetWidth-r<140?(this.currentPosition="below",t.style.paddingLeft=e+"px"):(this.currentPosition="inline",t.style.paddingLeft=r+"px")}else{var a=this.$refs.autosuggest.$el.querySelector("#"+this.controlId);this.currentPosition="inline",a.style.paddingLeft=e+"px"}}},watch:{currentPosition:function(e,t){},currentValue:function(e,t){var n=this;this.$emit("change",this.name,"value",e,this.localeKey),this.$nextTick((function(){return n.updateInputPadding()}))},inputValue:function(e,t){e!==t&&this.getSuggestions()}},mounted:function(){var e=this;this.updateInputPadding(),dn()(this.$refs.autosuggest.$el,le()((function(){return e.updateInputPadding()}),100))},beforeDestroy:function(){dn.a.unbind(this.$refs.autosuggest.$el)}},hn=fn,pn=(n("61fd"),h(hn,ot,st,!1,null,null,null)),mn=pn.exports,_n=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("fieldset",{staticClass:"pkpFormField pkpFormField--options pkpFormField--archivingPn",class:e.classes},[n("legend",{staticClass:"pkpFormField--options__legend"},[e.localeLabel?[n("span",{staticClass:"aria-hidden"},[e._v(e._s(e.localeLabel))]),n("span",{staticClass:"-screenReader"},[e._v(e._s(e.multilingualLabel))])]:[e._v(" "+e._s(e.label)+" ")],e.isRequired?n("span",{staticClass:"pkpFormFieldLabel__required"},[e._v(" * "),n("span",{staticClass:"-screenReader"},[e._v(e._s(e.__("common.required")))])]):e._e(),e.tooltip?n("tooltip",{attrs:{"aria-hidden":"true",tooltip:e.tooltip,label:""}}):e._e(),e.tooltip?n("span",{staticClass:"-screenReader",attrs:{id:e.describedByTooltipId},domProps:{innerHTML:e._s(e.tooltip)}}):e._e(),e.helpTopic?n("help-button",{attrs:{id:e.describedByHelpId,topic:e.helpTopic,section:e.helpSection,label:e.__("help.help")}}):e._e()],2),e.description?n("div",{staticClass:"pkpFormField__description pkpFormField--options__description",attrs:{id:e.describedByDescriptionId},domProps:{innerHTML:e._s(e.description)}}):e._e(),e.terms&&e.value?n("div",{staticClass:"pkpFormField__description pkpFormField--options__description pkpFormField--archivingPn__terms",domProps:{innerHTML:e._s(e.terms)}}):e._e(),e.errors&&e.errors.length?n("field-error",{attrs:{id:e.describedByErrorId,messages:e.errors}}):e._e(),n("input",{directives:[{name:"model",rawName:"v-model",value:e.selectedValue,expression:"selectedValue"}],attrs:{type:"hidden"},domProps:{value:e.selectedValue},on:{input:function(t){t.target.composing||(e.selectedValue=t.target.value)}}}),n("div",{staticClass:"pkpFormField__control"},e._l(e.localizedOptions,(function(t){return n("label",{key:t.value,staticClass:"pkpFormField--options__option"},[n("input",{directives:[{name:"model",rawName:"v-model",value:e.selectedValue,expression:"selectedValue"}],staticClass:"pkpFormField--options__input",attrs:{type:"checkbox",name:e.name,"aria-describedby":e.describedByIds,"aria-invalid":e.errors&&e.errors.length,disabled:e.isSaving||t.disabled},domProps:{value:t.value,checked:Array.isArray(e.selectedValue)?e._i(e.selectedValue,t.value)>-1:e.selectedValue},on:{change:function(n){var r=e.selectedValue,i=n.target,a=!!i.checked;if(Array.isArray(r)){var o=t.value,s=e._i(r,o);i.checked?s<0&&(e.selectedValue=r.concat([o])):s>-1&&(e.selectedValue=r.slice(0,s).concat(r.slice(s+1)))}else e.selectedValue=a}}}),e._v(" "+e._s(t.label)+" "),e.isSaving?n("span",{staticClass:"pkpSpinner",attrs:{"aria-hidden":"true"}}):e._e()])})),0)],1)},vn=[],gn=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("fieldset",{staticClass:"pkpFormField pkpFormField--options",class:e.classes},[n("legend",{staticClass:"pkpFormField--options__legend"},[e.localeLabel?[n("span",{staticClass:"aria-hidden"},[e._v(e._s(e.localeLabel))]),n("span",{staticClass:"-screenReader"},[e._v(e._s(e.multilingualLabel))])]:[e._v(" "+e._s(e.label)+" ")],e.isRequired?n("span",{staticClass:"pkpFormFieldLabel__required"},[e._v(" * "),n("span",{staticClass:"-screenReader"},[e._v(e._s(e.__("common.required")))])]):e._e(),e.isPrimaryLocale&&e.tooltip?n("tooltip",{attrs:{"aria-hidden":"true",tooltip:e.tooltip,label:""}}):e._e(),e.isPrimaryLocale&&e.tooltip?n("span",{staticClass:"-screenReader",attrs:{id:e.describedByTooltipId},domProps:{innerHTML:e._s(e.tooltip)}}):e._e(),e.isPrimaryLocale&&e.helpTopic?n("help-button",{attrs:{id:e.describedByHelpId,topic:e.helpTopic,section:e.helpSection,label:e.__("help.help")}}):e._e()],2),e.isPrimaryLocale&&e.description?n("div",{staticClass:"pkpFormField__description pkpFormField--options__description",attrs:{id:e.describedByDescriptionId},domProps:{innerHTML:e._s(e.description)}}):e._e(),e.errors&&e.errors.length?n("field-error",{attrs:{id:e.describedByErrorId,messages:e.errors}}):e._e(),n("div",{staticClass:"pkpFormField__control"},[n("draggable",{attrs:{disabled:!e.isOrderable},on:{end:e.updateValueOrder},model:{value:e.localizedOptions,callback:function(t){e.localizedOptions=t},expression:"localizedOptions"}},e._l(e.localizedOptions,(function(t){return n("label",{key:t.value,staticClass:"pkpFormField--options__option"},["checkbox"===e.type?n("input",{directives:[{name:"model",rawName:"v-model",value:e.selectedValue,expression:"selectedValue"}],staticClass:"pkpFormField--options__input",attrs:{type:"checkbox",name:e.localizedName,"aria-describedby":e.describedByIds,"aria-invalid":e.errors&&e.errors.length,disabled:t.disabled},domProps:{value:t.value,checked:Array.isArray(e.selectedValue)?e._i(e.selectedValue,t.value)>-1:e.selectedValue},on:{change:function(n){var r=e.selectedValue,i=n.target,a=!!i.checked;if(Array.isArray(r)){var o=t.value,s=e._i(r,o);i.checked?s<0&&(e.selectedValue=r.concat([o])):s>-1&&(e.selectedValue=r.slice(0,s).concat(r.slice(s+1)))}else e.selectedValue=a}}}):n("input",{directives:[{name:"model",rawName:"v-model",value:e.selectedValue,expression:"selectedValue"}],staticClass:"pkpFormField--options__input",attrs:{type:"radio",name:e.localizedName,"aria-describedby":e.describedByIds,"aria-invalid":e.errors&&e.errors.length,disabled:t.disabled},domProps:{value:t.value,checked:e._q(e.selectedValue,t.value)},on:{change:function(n){e.selectedValue=t.value}}}),n("span",{staticClass:"pkpFormField--options__optionLabel",domProps:{innerHTML:e._s(t.label)}}),e.isOrderable?n("orderer",{attrs:{itemId:t.value,itemTitle:t.label},on:{up:e.optionOrderUp,down:e.optionOrderDown}}):e._e()],1)})),0),e.isMultilingual&&e.locales.length>1?n("multilingual-progress",{attrs:{id:e.multilingualProgressId,count:e.multilingualFieldsCompleted,total:e.locales.length}}):e._e()],1)],1)},bn=[],yn=n("b76a"),Mn=n.n(yn),kn=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"orderer"},[e.isDraggable?n("span",{staticClass:"orderer__dragDrop",attrs:{"aria-hidden":"true"}},[n("icon",{attrs:{icon:"sort"}})],1):e._e(),n("button",{staticClass:"orderer__up",on:{click:function(t){return t.preventDefault(),e.up(t)}}},[n("icon",{attrs:{icon:"angle-up"}}),n("span",{staticClass:"-screenReader"},[e._v(" "+e._s(e.__("common.orderUp",{itemTitle:e.itemTitle}))+" ")])],1),n("button",{staticClass:"orderer__down",on:{click:function(t){return t.preventDefault(),e.down(t)}}},[n("icon",{attrs:{icon:"angle-down"}}),n("span",{staticClass:"-screenReader"},[e._v(" "+e._s(e.__("common.orderDown",{itemTitle:e.itemTitle}))+" ")])],1)])},wn=[],Ln={name:"Orderer",props:{itemId:[String,Number],itemTitle:String,isDraggable:{type:Boolean,default:!0}},methods:{up:function(){var e=this;this.$emit("up",this.itemId),this.$nextTick((function(){e.setFocus("up")}))},down:function(){var e=this;this.$emit("down",this.itemId),this.$nextTick((function(){e.setFocus("down")}))},setFocus:function(e){var t="up"===e?".orderer__up":".orderer__down";this.$el.querySelector(t).focus()}}},xn=Ln,Sn=(n("3b02"),h(xn,kn,wn,!1,null,null,null)),Tn=Sn.exports,Dn={name:"FieldOptions",extends:$t,components:{draggable:Mn.a,Orderer:Tn},props:{type:{validator:function(e){return["checkbox","radio"].includes(e)},default:"checkbox"},isOrderable:{type:Boolean,default:!1},options:{type:Array,required:!0},value:{type:[Array,Number,String,Boolean],required:!0}},data:function(){return{localizedOptions:this.isMultilingual?this.options[this.localeKey]:this.options,selectedValue:this.isMultilingual?this.value[this.localeKey]:this.value}},computed:{classes:function(){var e=[];return this.isOrderable&&e.push("pkpFormField--optionsOrderable"),e}},methods:{optionOrderUp:function(e){var t=this.localizedOptions.findIndex((function(t){return t.value===e}));t&&(this.localizedOptions.splice(t-1,0,this.localizedOptions.splice(t,1)[0]),this.updateValueOrder())},optionOrderDown:function(e){var t=this.localizedOptions.findIndex((function(t){return t.value===e}));t<0||t>=this.localizedOptions.length-1||(this.localizedOptions.splice(t+1,0,this.localizedOptions.splice(t,1)[0]),this.updateValueOrder())},updateValueOrder:function(){var e=this;this.selectedValue=this.localizedOptions.filter((function(t){return e.selectedValue.includes(t.value)})).map((function(e){return e.value}))}},watch:{value:function(e,t){JSON.stringify(e)!==JSON.stringify(t)&&(this.selectedValue=this.isMultilingual?this.value[this.localeKey]:this.value)},selectedValue:function(e,t){JSON.stringify(e)!==JSON.stringify(t)&&(this.isOrderable&&(e=this.localizedOptions.filter((function(t){return e.includes(t.value)})).map((function(e){return e.value}))),this.$emit("change",this.name,"value",e,this.localeKey))},options:function(e){this.localizedOptions=this.isMultilingual?e[this.localeKey]:e}}},Yn=Dn,Cn=(n("4fbc"),h(Yn,gn,bn,!1,null,null,null)),En=Cn.exports,An={name:"FieldArchivingPn",extends:En,props:{value:{type:Boolean,required:!0},terms:{type:String,required:!0},disablePluginSuccess:{type:String,required:!0},enablePluginSuccess:{type:String,required:!0},enablePluginUrl:{type:String,required:!0},disablePluginUrl:{type:String,required:!0},settingsUrl:{type:String,required:!0}},data:function(){return{isSaving:!1}},methods:{addSettingsListener:function(){var e=this;this.$nextTick((function(){$(".pkpFormField--archivingPn__terms button",e.$el).click((function(t){t.stopPropagation(),t.preventDefault();var n={title:"Test title",url:e.settingsUrl,closeCallback:e.resetFocus};return $('<div id="'+$.pkp.classes.Helper.uuid()+'" class="pkp_modal pkpModalWrapper" tabindex="-1"></div>').pkpHandler("$.pkp.controllers.modal.AjaxModalHandler",n),!1}))}))},removeSettingsListener:function(){$(".pkpFormField--archivingPn__terms button",this.$el).off()},resetFocus:function(){$(".pkpFormField--archivingPn__terms button",this.$el).focus()},success:function(e){e.status?pkp.eventBus.$emit("notify",this.value?this.enablePluginSuccess:this.disablePluginSuccess,"success"):pkp.eventBus.$emit("notify",this.__("common.unknownError"),"warning")},error:function(){pkp.eventBus.$emit("notify",this.enablePluginError,"warning")},complete:function(){this.isSaving=!1}},watch:{value:function(e,t){e!==t&&(e?this.addSettingsListener():this.removeSettingsListener(),this.isSaving=!0,$.ajax({method:"POST",url:e?this.enablePluginUrl:this.disablePluginUrl,data:{csrfToken:pkp.currentUser.csrfToken,disableNotification:!1},success:this.success,error:this.error,complete:this.complete}))}},mounted:function(){this.value&&this.addSettingsListener()},beforeDestroy:function(){this.value||$(".pkpFormField--archivingPn__terms button",this.$el).off()}},On=An,Pn=(n("6a5c"),h(On,_n,vn,!1,null,null,null)),Fn=Pn.exports,In=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("fieldset",{staticClass:"pkpFormField pkpFormField--color"},[n("legend",{staticClass:"pkpFormField__heading--legend"},[e.localeLabel?[n("span",{staticClass:"aria-hidden"},[e._v(e._s(e.localeLabel))]),n("span",{staticClass:"-screenReader"},[e._v(e._s(e.multilingualLabel))])]:[e._v(" "+e._s(e.label)+" ")],e.isRequired?n("span",{staticClass:"pkpFormFieldLabel__required"},[e._v(" * "),n("span",{staticClass:"-screenReader"},[e._v(e._s(e.__("common.required")))])]):e._e(),e.isPrimaryLocale&&e.tooltip?n("tooltip",{attrs:{"aria-hidden":"true",tooltip:e.tooltip,label:""}}):e._e(),e.isPrimaryLocale&&e.tooltip?n("span",{staticClass:"-screenReader",attrs:{id:e.describedByTooltipId},domProps:{innerHTML:e._s(e.tooltip)}}):e._e(),e.isPrimaryLocale&&e.helpTopic?n("help-button",{attrs:{id:e.describedByHelpId,topic:e.helpTopic,section:e.helpSection,label:e.__("help.help")}}):e._e()],2),e.isPrimaryLocale&&e.description?n("div",{staticClass:"pkpFormField__description",attrs:{id:e.describedByDescriptionId},domProps:{innerHTML:e._s(e.description)}}):e._e(),n("div",{staticClass:"pkpFormField__control"},[n("color-picker",{attrs:{value:e.currentValue,disableAlpha:!0},on:{input:e.setValue}}),e.errors&&e.errors.length?n("field-error",{attrs:{id:e.describedByErrorId,messages:e.errors}}):e._e()],1)])},jn=[],Hn=n("c345"),Rn={name:"FieldColor",extends:$t,components:{"color-picker":Hn["Chrome"]},methods:{setValue:function(e){e.hex!==this.currentValue&&(this.currentValue=e.hex)}}},Nn=Rn,$n=(n("85cd"),h(Nn,In,jn,!1,null,null,null)),zn=$n.exports,Bn=(n("4d63"),n("466d"),{name:"FieldControlledVocab",extends:mn,data:function(){return{allSuggestions:[],suggestionsLoaded:!1}},methods:{getSuggestions:function(){if(this.inputValue){if(!this.suggestionsLoaded){var e=this;$.ajax({url:this.apiUrl,type:"GET",data:this.isMultilingual?{locale:this.localeKey}:{},error:function(t){e.ajaxErrorCallback(t)},success:function(t){e.allSuggestions=t.map((function(e){return{value:e,label:e}})),e.setSuggestions.apply(e),e.suggestionsLoaded=!0}})}this.setSuggestions()}else this.suggestions=[]},selectSuggestion:function(e){e?this.select(e.item):this.inputValue&&this.select({value:this.inputValue,label:this.inputValue})},setSuggestions:le()((function(){var e=this,t=new RegExp(this.inputValue.replace(/[-/\\^$*+?.()|[\]{}]/g,"\\$&"),"gi");this.suggestions=this.allSuggestions.filter((function(n){return!e.inputValue||e.inputValue!==n.value&&n.value.match(t)})),this.inputValue&&!this.suggestions.includes(this.inputValue)&&this.suggestions.unshift({value:this.inputValue,label:this.inputValue})}),250)}}),Wn=Bn,Un=h(Wn,Kt,Xt,!1,null,null,null),Vn=Un.exports,qn=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"pkpFormField pkpFormField--pubid"},[n("div",{staticClass:"pkpFormField__heading"},[n("form-field-label",{attrs:{controlId:e.controlId,label:e.label,isRequired:e.isRequired,requiredLabel:e.__("common.required")}}),e.tooltip?n("tooltip",{attrs:{"aria-hidden":"true",tooltip:e.tooltip,label:""}}):e._e(),e.tooltip?n("span",{staticClass:"-screenReader",attrs:{id:e.describedByTooltipId},domProps:{innerHTML:e._s(e.tooltip)}}):e._e(),e.helpTopic?n("help-button",{attrs:{id:e.describedByHelpId,topic:e.helpTopic,section:e.helpSection,label:e.__("help.help")}}):e._e()],1),e.description?n("div",{staticClass:"pkpFormField__description",attrs:{id:e.describedByDescriptionId},domProps:{innerHTML:e._s(e.description)}}):e._e(),n("div",{staticClass:"pkpFormField__control"},[n("input",{directives:[{name:"model",rawName:"v-model",value:e.currentValue,expression:"currentValue"}],ref:"input",staticClass:"pkpFormField__input pkpFormField--text__input pkpFormField--pubid__input",attrs:{id:e.controlId,name:e.localizedName,"aria-describedby":e.describedByIds,"aria-invalid":e.errors&&e.errors.length,disabled:!!e.pattern,required:e.isRequired},domProps:{value:e.currentValue},on:{input:function(t){t.target.composing||(e.currentValue=t.target.value)}}}),e.pattern&&e.canGenerateId&&!e.currentValue?n("pkp-button",{staticClass:"pkpFormField--pubid__button",on:{click:e.assignId}},[e._v(" "+e._s(e.assignIdLabel)+" ")]):e.pattern&&e.currentValue?n("pkp-button",{staticClass:"pkpFormField--pubid__button",attrs:{isWarnable:!0},on:{click:function(){return e.currentValue=""}}},[e._v(" "+e._s(e.clearIdLabel)+" ")]):e._e(),!e.pattern||e.canGenerateId||e.currentValue?e._e():n("div",{staticClass:"pkpFormField--pubid__warning",attrs:{id:e.describedByDescriptionId}},[n("icon",{attrs:{icon:"exclamation-triangle",inline:!0}}),e._v(" "+e._s(e.missingPartsLabel)+" ")],1),e.errors&&e.errors.length?n("field-error",{attrs:{id:e.describedByErrorId,messages:e.errors}}):e._e()],1)])},Gn=[],Jn={name:"FieldPubId",extends:$t,props:{assignIdLabel:{type:String,required:!0},clearIdLabel:{type:String,required:!0},contextInitials:{type:String,default:function(){return""}},isPForPress:{type:Boolean,default:function(){return!1}},issueNumber:{type:[String,Number],default:function(){return""}},issueVolume:{type:[String,Number],default:function(){return""}},missingPartsLabel:{type:String,required:!0},pages:{type:String,default:function(){return""}},pattern:{type:String,required:!0},prefix:{type:String,required:!0},publisherId:{type:String,default:function(){return""}},separator:{type:String,default:function(){return""}},submissionId:{type:Number,default:function(){return""}},publicationId:{type:Number,default:function(){return 0}},year:{type:[String,Number],default:function(){return""}}},computed:{canGenerateId:function(){return(this.issueNumber||!this.pattern.includes("%i"))&&(this.issueVolume||!this.pattern.includes("%v"))&&(!this.isPForPress&&(this.contextInitials||!this.pattern.includes("%j"))&&(this.pages||!this.pattern.includes("%p"))||this.isPForPress&&(this.contextInitials||!this.pattern.includes("%j")))&&(this.publisherId||!this.pattern.includes("%x"))&&(this.submissionId||!this.pattern.includes("%a"))&&(this.submissionId||!this.pattern.includes("%m"))&&(this.publicationId||!this.pattern.includes("%b"))&&(this.year||!this.pattern.includes("%Y"))}},methods:{assignId:function(){this.currentValue=this.generateId()},generateId:function(){var e=this.prefix+this.separator+this.pattern.replace("%i",this.issueNumber).replace("%v",this.issueVolume).replace("%x",this.publisherId).replace("%Y",this.year);return e=this.isPForPress?e.replace("%p",this.contextInitials).replace("%m",this.submissionId):e.replace("%j",this.contextInitials).replace("%a",this.submissionId).replace("%b",this.publicationId).replace("%p",this.pages),e}}},Kn=Jn,Xn=(n("4db6"),h(Kn,qn,Gn,!1,null,null,null)),Qn=Xn.exports,Zn=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"pkpFormField pkpFormField--html"},[n("div",{staticClass:"pkpFormField__heading"},[n("span",{staticClass:"pkpFormFieldLabel"},[e._v(" "+e._s(e.label)+" ")]),e.tooltip?n("tooltip",{attrs:{"aria-hidden":"true",tooltip:e.tooltip,label:""}}):e._e(),e.tooltip?n("span",{staticClass:"-screenReader",domProps:{innerHTML:e._s(e.tooltip)}}):e._e(),e.helpTopic?n("help-button",{attrs:{topic:e.helpTopic,section:e.helpSection,label:e.__("help.help")}}):e._e()],1),n("div",{staticClass:"pkpFormField__control pkpFormField__control--html",domProps:{innerHTML:e._s(e.description)}})])},er=[],tr={name:"FieldHtml",extends:$t},nr=tr,rr=(n("9396"),h(nr,Zn,er,!1,null,null,null)),ir=rr.exports,ar=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("fieldset",{staticClass:"pkpFormField pkpFormField--options pkpFormField--metadata"},[n("legend",{staticClass:"pkpFormField--options__legend"},[e._v(" "+e._s(e.label)+" "),e.tooltip?n("tooltip",{attrs:{"aria-hidden":"true",tooltip:e.tooltip,label:""}}):e._e(),e.tooltip?n("span",{staticClass:"-screenReader",attrs:{id:e.describedByTooltipId},domProps:{innerHTML:e._s(e.tooltip)}}):e._e(),e.helpTopic?n("help-button",{attrs:{id:e.describedByHelpId,topic:e.helpTopic,section:e.helpSection,label:e.__("help.help")}}):e._e()],1),e.description?n("div",{staticClass:"pkpFormField__description pkpFormField--options__description",attrs:{id:e.describedByDescriptionId},domProps:{innerHTML:e._s(e.description)}}):e._e(),e.errors&&e.errors.length?n("field-error",{attrs:{id:e.describedByErrorId,messages:e.errors}}):e._e(),n("div",{staticClass:"pkpFormField__control"},[e._l(e.options,(function(t){return n("label",{key:t.value,staticClass:"pkpFormField--options__option"},[n("input",{directives:[{name:"model",rawName:"v-model",value:e.isEnabled,expression:"isEnabled"}],staticClass:"pkpFormField--options__input",attrs:{type:"checkbox","aria-describedby":e.describedByIds,"aria-invalid":e.errors&&e.errors.length,disabled:t.disabled},domProps:{value:t.value,checked:Array.isArray(e.isEnabled)?e._i(e.isEnabled,t.value)>-1:e.isEnabled},on:{change:function(n){var r=e.isEnabled,i=n.target,a=!!i.checked;if(Array.isArray(r)){var o=t.value,s=e._i(r,o);i.checked?s<0&&(e.isEnabled=r.concat([o])):s>-1&&(e.isEnabled=r.slice(0,s).concat(r.slice(s+1)))}else e.isEnabled=a}}}),n("span",{staticClass:"pkpFormField--options__optionLabel",domProps:{innerHTML:e._s(t.label)}})])})),e.isEnabled?n("div",{staticClass:"pkpFormField--metadata__submissionOptions"},e._l(e.submissionOptions,(function(t){return n("label",{key:"submission"+t.value,staticClass:"pkpFormField--options__option"},[n("input",{directives:[{name:"model",rawName:"v-model",value:e.selectedValue,expression:"selectedValue"}],staticClass:"pkpFormField--options__input pkpFormField--metadata__submissionInput",attrs:{type:"radio","aria-invalid":e.errors&&e.errors.length,disabled:t.disabled},domProps:{value:t.value,checked:e._q(e.selectedValue,t.value)},on:{change:function(n){e.selectedValue=t.value}}}),n("span",{staticClass:"pkpFormField--options__optionLabel",domProps:{innerHTML:e._s(t.label)}})])})),0):e._e()],2)],1)},or=[],sr={name:"FieldMetadataSetting",extends:En,props:{disabledValue:{type:Number,required:!0},enabledOnlyValue:{type:String,required:!0},value:{type:[Number,String],required:!0},submissionOptions:{type:Array,required:!0}},data:function(){return{isEnabled:this.disabledValue!==this.value}},watch:{isEnabled:function(e,t){e!==t&&(this.selectedValue=e?this.enabledOnlyValue:this.disabledValue)}}},lr=sr,ur=(n("1f5b"),h(lr,ar,or,!1,null,null,null)),cr=ur.exports,dr=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("fieldset",{staticClass:"pkpFormField pkpFormField--options",class:e.classes},[n("legend",{staticClass:"pkpFormField--options__legend"},[e.localeLabel?[n("span",{staticClass:"aria-hidden"},[e._v(e._s(e.localeLabel))]),n("span",{staticClass:"-screenReader"},[e._v(e._s(e.multilingualLabel))])]:[e._v(" "+e._s(e.label)+" ")],e.isRequired?n("span",{staticClass:"pkpFormFieldLabel__required"},[e._v(" * "),n("span",{staticClass:"-screenReader"},[e._v(e._s(e.__("common.required")))])]):e._e(),e.isPrimaryLocale&&e.tooltip?n("tooltip",{attrs:{"aria-hidden":"true",tooltip:e.tooltip,label:""}}):e._e(),e.isPrimaryLocale&&e.tooltip?n("span",{staticClass:"-screenReader",attrs:{id:e.describedByTooltipId},domProps:{innerHTML:e._s(e.tooltip)}}):e._e(),e.isPrimaryLocale&&e.helpTopic?n("help-button",{attrs:{id:e.describedByHelpId,topic:e.helpTopic,section:e.helpSection,label:e.__("help.help")}}):e._e()],2),e.isPrimaryLocale&&e.description?n("div",{staticClass:"pkpFormField__description pkpFormField--options__description",attrs:{id:e.describedByDescriptionId},domProps:{innerHTML:e._s(e.description)}}):e._e(),e.errors&&e.errors.length?n("field-error",{attrs:{id:e.describedByErrorId,messages:e.errors}}):e._e(),n("input",{directives:[{name:"model",rawName:"v-model",value:e.selectedValue,expression:"selectedValue"}],attrs:{type:"hidden"},domProps:{value:e.selectedValue},on:{input:function(t){t.target.composing||(e.selectedValue=t.target.value)}}}),n("div",{staticClass:"pkpFormField__control"},[e._l(e.localizedOptions,(function(t){return n("label",{key:t.value,staticClass:"pkpFormField--options__option"},[t.isInput?[n("input",{ref:"inputRadio",refInFor:!0,staticClass:"pkpFormField--options__input",attrs:{type:"radio",name:e.localizedName,"aria-describedby":e.describedByIds,"aria-invalid":e.errors&&e.errors.length,disabled:t.disabled},on:{change:e.selectInput}}),t.label?n("span",[e._v(e._s(t.label))]):e._e(),n("input",{directives:[{name:"model",rawName:"v-model",value:e.inputValue,expression:"inputValue"}],ref:"inputText",refInFor:!0,staticClass:"pkpFormField__input pkpFormField--options__input--text",attrs:{type:"text","aria-describedby":e.describedByIds,"aria-invalid":e.errors&&e.errors.length,disabled:t.disabled},domProps:{value:e.inputValue},on:{focus:e.selectInput,input:function(t){t.target.composing||(e.inputValue=t.target.value)}}})]:[n("input",{directives:[{name:"model",rawName:"v-model",value:e.selectedValue,expression:"selectedValue"}],staticClass:"pkpFormField--options__input",attrs:{type:"radio",name:e.localizedName,"aria-describedby":e.describedByIds,"aria-invalid":e.errors&&e.errors.length,disabled:t.disabled},domProps:{value:t.value,checked:e._q(e.selectedValue,t.value)},on:{change:function(n){e.selectedValue=t.value}}}),e._v(" "+e._s(t.label)+" ")]],2)})),e.isMultilingual&&e.locales.length>1?n("multilingual-progress",{attrs:{id:e.multilingualProgressId,count:e.multilingualFieldsCompleted,total:e.locales.length}}):e._e()],2)],1)},fr=[],hr={name:"FieldRadioInput",extends:En,props:{value:{type:String,required:!0}},data:function(){return{inputValue:""}},computed:{isInputSelected:function(){return!this.localizedOptions.filter((function(e){return!e.isInput})).map((function(e){return e.value})).includes(this.selectedValue)}},methods:{selectInput:function(){this.selectedValue=this.inputValue,this.$refs.inputRadio[0].checked=!0,this.$refs.inputText[0].focus()},setInputToSelected:function(){this.selectedValue=this.inputValue}},watch:{inputValue:function(e,t){e!==t&&this.isInputSelected&&(this.selectedValue=this.inputValue)}},mounted:function(){this.isInputSelected&&(this.inputValue=this.isMultilingual?this.value[this.localeKey]:this.value,this.$refs.inputRadio[0].checked=!0)}},pr=hr,mr=(n("5a6e"),h(pr,dr,fr,!1,null,null,null)),_r=mr.exports,vr=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"pkpFormField pkpFormField--richTextarea",class:{"-isFocused":e.isFocused}},[n("div",{staticClass:"pkpFormField__heading"},[n("form-field-label",{attrs:{controlId:e.controlId,label:e.label,localeLabel:e.localeLabel,isRequired:e.isRequired,requiredLabel:e.__("common.required"),multilingualLabel:e.multilingualLabel}}),e.isPrimaryLocale&&e.tooltip?n("tooltip",{attrs:{"aria-hidden":"true",tooltip:e.tooltip,label:""}}):e._e(),e.isPrimaryLocale&&e.tooltip?n("span",{staticClass:"-screenReader",attrs:{id:e.describedByTooltipId},domProps:{innerHTML:e._s(e.tooltip)}}):e._e(),e.isPrimaryLocale&&e.helpTopic?n("help-button",{attrs:{id:e.describedByHelpId,topic:e.helpTopic,section:e.helpSection,label:e.__("help.help")}}):e._e()],1),e.isPrimaryLocale&&e.description?n("div",{staticClass:"pkpFormField__description",attrs:{id:e.describedByDescriptionId},domProps:{innerHTML:e._s(e.description)}}):e._e(),n("div",{staticClass:"pkpFormField__control pkpFormField--richTextarea__control",class:"pkpFormField--richTextArea__control--"+this.size},[n("div",{staticClass:"pkpFormField--richTextarea__toolbar",attrs:{id:e.toolbarId}}),n("editor",{ref:"editor",staticClass:"pkpFormField__input pkpFormField--richTextarea__input",attrs:{id:e.controlId,toolbar:e.toolbar,plugins:e.plugins,init:e.compiledInit},on:{onFocus:e.focus,onBlur:e.blur},model:{value:e.editorValue,callback:function(t){e.editorValue=t},expression:"editorValue"}}),e.isMultilingual&&e.locales.length>1||e.wordLimit?n("div",{staticClass:"pkpFormField--richTextarea__controlFooter"},[e.isMultilingual&&e.locales.length>1?n("multilingual-progress",{attrs:{id:e.multilingualProgressId,count:e.multilingualFieldsCompleted,total:e.locales.length}}):e._e(),e.wordLimit?n("div",{staticClass:"pkpFormField--richTextarea__wordLimit"},[e.wordCount>e.wordLimit?n("icon",{attrs:{icon:"exclamation-triangle",inline:!0}}):e._e(),e._v(" "+e._s(e.replaceLocaleParams(e.wordCountLabel,{count:e.wordCount,limit:e.wordLimit}))+" ")],1):e._e()],1):e._e()],1),e.errors&&e.errors.length?n("field-error",{attrs:{id:e.describedByErrorId,messages:e.errors}}):e._e()],1)},gr=[],br=(n("99af"),["onActivate","onAddUndo","onBeforeAddUndo","onBeforeExecCommand","onBeforeGetContent","onBeforeRenderUI","onBeforeSetContent","onBeforePaste","onBlur","onChange","onClearUndos","onClick","onContextMenu","onCopy","onCut","onDblclick","onDeactivate","onDirty","onDrag","onDragDrop","onDragEnd","onDragGesture","onDragOver","onDrop","onExecCommand","onFocus","onFocusIn","onFocusOut","onGetContent","onHide","onInit","onKeyDown","onKeyPress","onKeyUp","onLoadContent","onMouseDown","onMouseEnter","onMouseLeave","onMouseMove","onMouseOut","onMouseOver","onMouseUp","onNodeChange","onObjectResizeStart","onObjectResized","onObjectSelected","onPaste","onPostProcess","onPostRender","onPreProcess","onProgressState","onRedo","onRemove","onReset","onSaveContent","onSelectionChange","onSetAttrib","onSetContent","onShow","onSubmit","onUndo","onVisualAid"]),yr=function(e){return-1!==br.indexOf(e)},Mr=function(e,t,n){Object.keys(t).filter(yr).forEach((function(r){var i=t[r];"function"===typeof i&&("onInit"===r?i(e,n):n.on(r.substring(2),(function(e){return i(e,n)})))}))},kr=function(e,t){var n,r=e.$props.modelEvents?e.$props.modelEvents:null,i=Array.isArray(r)?r.join(" "):r;e.$watch("value",(function(e,r){t&&"string"===typeof e&&e!==n&&e!==r&&(t.setContent(e),n=e)})),t.on(i||"change keyup undo redo",(function(){n=t.getContent(),e.$emit("input",n)}))},wr=function(e,t,n){var r=t.$props.value?t.$props.value:"",i=t.$props.initialValue?t.$props.initialValue:"";n.setContent(r||i),t.$listeners.input&&kr(t,n),Mr(e,t.$listeners,n)},Lr=0,xr=function(e){var t=Date.now(),n=Math.floor(1e9*Math.random());return Lr++,e+"_"+n+Lr+String(t)},Sr=function(e){return null!==e&&"textarea"===e.tagName.toLowerCase()},Tr=function(e){return"undefined"===typeof e||""===e?[]:Array.isArray(e)?e:e.split(" ")},Dr=function(e,t){return Tr(e).concat(Tr(t))},Yr=function(e,t,n,r){var i=t.createElement("script");i.type="application/javascript",i.id=e,i.addEventListener("load",r),i.src=n,t.head&&t.head.appendChild(i)},Cr=function(){return{listeners:[],scriptId:xr("tiny-script"),scriptLoaded:!1}},Er=function(e,t,n,r){e.scriptLoaded?r():(e.listeners.push(r),t.getElementById(e.scriptId)||Yr(e.scriptId,t,n,(function(){e.listeners.forEach((function(e){return e()})),e.scriptLoaded=!0})))},Ar=n("1f38"),Or={apiKey:String,cloudChannel:String,id:String,init:Object,initialValue:String,inline:Boolean,modelEvents:[String,Array],plugins:[String,Array],tagName:String,toolbar:[String,Array],value:String,disabled:Boolean},Pr=function(){return Pr=Object.assign||function(e){for(var t,n=1,r=arguments.length;n<r;n++)for(var i in t=arguments[n],t)Object.prototype.hasOwnProperty.call(t,i)&&(e[i]=t[i]);return e},Pr.apply(this,arguments)},Fr=Cr(),Ir=function(e,t,n){return e(n||"div",{attrs:{id:t}})},jr=function(e,t){return e("textarea",{attrs:{id:t},style:{visibility:"hidden"}})},Hr=function(e){return function(){var t=Pr({},e.$props.init,{readonly:e.$props.disabled,selector:"#"+e.elementId,plugins:Dr(e.$props.init&&e.$props.init.plugins,e.$props.plugins),toolbar:e.$props.toolbar||e.$props.init&&e.$props.init.toolbar,inline:e.inlineEditor,setup:function(t){e.editor=t,t.on("init",(function(n){return wr(n,e,t)})),e.$props.init&&"function"===typeof e.$props.init.setup&&e.$props.init.setup(t)}});Sr(e.element)&&(e.element.style.visibility=""),Object(Ar["a"])().init(t)}},Rr={props:Or,created:function(){this.elementId=this.$props.id||xr("tiny-vue"),this.inlineEditor=this.$props.init&&this.$props.init.inline||this.$props.inline},watch:{disabled:function(){this.editor.setMode(this.disabled?"readonly":"design")}},mounted:function(){if(this.element=this.$el,null!==Object(Ar["a"])())Hr(this)();else if(this.element&&this.element.ownerDocument){var e=this.element.ownerDocument,t=this.$props.cloudChannel?this.$props.cloudChannel:"stable",n=this.$props.apiKey?this.$props.apiKey:"",r="https://cloud.tinymce.com/"+t+"/tinymce.min.js?apiKey="+n;Er(Fr,e,r,Hr(this))}},beforeDestroy:function(){null!==Object(Ar["a"])()&&Object(Ar["a"])().remove(this.editor)},render:function(e){return this.inlineEditor?Ir(e,this.elementId,this.$props.tagName):jr(e,this.elementId)}},Nr=Rr,$r={name:"FieldRichTextarea",extends:$t,components:{Editor:Nr},props:{init:{type:Object,default:function(){return{menubar:!1,statusbar:!1,entity_encoding:"raw"}}},plugins:{type:String,required:!0},preparedContent:{type:Object,default:function(){return{}}},renderPreparedContent:{type:Boolean,default:function(){return!1}},size:{type:String,default:function(){return"default"},validator:function(e){return["default","large"].includes(e)}},toolbar:{type:String,required:!0},uploadUrl:{type:String,default:function(){return""}},wordCountLabel:{type:String,default:function(){return""}},wordLimit:{type:Number,default:function(){return 0}}},data:function(){return{editorValue:"",isFocused:!1,wordCount:0}},computed:{toolbarId:function(){return this.compileId("toolbar")},compiledInit:function(){var e=this;return o({inline:!0,paste_data_images:!0,relative_urls:!1,remove_script_host:!1,convert_urls:!0,urlconverter_callback:function(e){var t=/^http[s]*:\/\//i.test(e),n=/\{\$(\w*)\}/.exec(e);return t||(e=window.location.href+"/"+e),n&&(e=n[0]),e},directionality:$.pkp.app.rtlLocales.includes(this.localeKey)?"rtl":null,images_upload_handler:function(t,n,r){var i=new FormData;i.append("file",t.blob(),t.filename()),$.ajax({method:"POST",url:e.uploadUrl,data:i,processData:!1,contentType:!1,headers:{"X-Csrf-Token":pkp.currentUser.csrfToken},success:function(e){n(e.url)},error:function(e){r(e.responseJSON.errorMessage)}})},fixed_toolbar_container:"#"+this.toolbarId,init_instance_callback:function(e){e.fire("focus"),e.fire("blur")},setup:function(t){Object.keys(e.preparedContentSanitized).length&&(t.addButton("pkpPreparedContent",{icon:"nonbreaking",type:"panelbutton",panel:{html:function(){var t='<ul class="pkpFormField--richTextarea__tinymcePanel">';return Object.keys(e.preparedContentSanitized).forEach((function(n){return t+='<li>\n\t\t\t\t\t\t\t\t\t\t\t\t\t<button data-symbolic="'.concat(n,'">\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t').concat(e.preparedContentSanitized[n],"\n\t\t\t\t\t\t\t\t\t\t\t\t\t</button>\n\t\t\t\t\t\t\t\t\t\t\t\t</li>")})),t+"</ul>"},onclick:function(n){if("BUTTON"===n.target.tagName){var r=n.target.dataset.symbolic;e.renderPreparedContent?t.insertContent(e.preparedContentSanitized[r]):t.insertContent(e.getPlaceholder(r,e.preparedContentSanitized[r])),this.hide()}}}}),t.settings.toolbar+=" | pkpPreparedContent")}},this.init)},preparedContentSanitized:function(){var e=this;if(!Object.keys(this.preparedContent).length)return{};var t={};return Object.keys(this.preparedContent).forEach((function(n){var r=n.replace(/[^\w]/gi,""),i={"<":"&lt;",">":"&gt;"},a=e.preparedContent[n].replace(/[<>]/g,(function(e){return i[e]}));r&&a&&(t[r]=a)})),t}},methods:{blur:function(){if(this.isFocused=!1,!this.renderPreparedContent){var e=Object.keys(this.preparedContentSanitized);if(e.length){var t=document.createElement("div");t.innerHTML=this.editorValue,t.querySelectorAll(".pkpTag[data-symbolic]").forEach((function(e){e.replaceWith("{$"+e.dataset.symbolic+"}")})),this.currentValue=t.innerHTML}else this.currentValue=this.editorValue}},focus:function(){this.isFocused=!0},getPlaceholder:function(e,t){return'<span class="pkpTag mceNonEditable" data-symbolic="'.concat(e,'">').concat(t,"</span>")},setEditorValue:function(){var e=this;this.editorValue=this.currentValue;var t=Object.keys(this.preparedContentSanitized);if(t.length){var n=this.editorValue;this.renderPreparedContent?t.forEach((function(t){n=n.replace(new RegExp("\\{\\$"+t+"\\}","g"),e.preparedContentSanitized[t])})):t.forEach((function(t){n=n.replace(new RegExp("\\{\\$"+t+"\\}","g"),e.getPlaceholder(t,e.preparedContentSanitized[t]))})),this.editorValue=n}},setWordCount:le()((function(){if(this.wordLimit&&this.editorValue&&this.$refs.editor){var e=this.$refs.editor.editor.getContent({format:"text"}).match(/\S+/g);this.wordCount=null===e?0:e.length}else this.wordCount=0}),250)},watch:{editorValue:function(e,t){e!==t&&this.setWordCount()},currentValue:function(e,t){e!==t&&this.setEditorValue()}},created:function(){this.setEditorValue()},mounted:function(){this.setWordCount()}},zr=$r,Br=(n("3b3b"),h(zr,vr,gr,!1,null,null,null)),Wr=Br.exports,Ur=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"pkpFormField pkpFormField--select"},[n("div",{staticClass:"pkpFormField__heading"},[n("form-field-label",{attrs:{controlId:e.controlId,label:e.label,localeLabel:e.localeLabel,isRequired:e.isRequired,requiredLabel:e.__("common.required"),multilingualLabel:e.multilingualLabel}}),e.tooltip?n("tooltip",{attrs:{"aria-hidden":"true",tooltip:e.tooltip,label:""}}):e._e(),e.tooltip?n("span",{staticClass:"-screenReader",attrs:{id:e.describedByTooltipId},domProps:{innerHTML:e._s(e.tooltip)}}):e._e(),e.helpTopic?n("help-button",{attrs:{id:e.describedByHelpId,topic:e.helpTopic,section:e.helpSection,label:e.__("help.help")}}):e._e()],1),e.isPrimaryLocale&&e.description?n("div",{staticClass:"pkpFormField__description",attrs:{id:e.describedByDescriptionId},domProps:{innerHTML:e._s(e.description)}}):e._e(),n("div",{staticClass:"pkpFormField__control",class:{"pkpFormField__control--hasMultilingualIndicator":e.isMultilingual&&e.locales.length>1}},[n("select",{directives:[{name:"model",rawName:"v-model",value:e.currentValue,expression:"currentValue"}],staticClass:"pkpFormField__input pkpFormField--select__input",attrs:{id:e.controlId,name:e.localizedName,"aria-describedby":e.describedByIds,"aria-invalid":e.errors&&e.errors.length,required:e.isRequired},on:{change:function(t){var n=Array.prototype.filter.call(t.target.options,(function(e){return e.selected})).map((function(e){var t="_value"in e?e._value:e.value;return t}));e.currentValue=t.target.multiple?n:n[0]}}},e._l(e.localizedOptions,(function(t){return n("option",e._b({key:t.value},"option",t,!1),[e._v(" "+e._s(t.label)+" ")])})),0),e.isMultilingual&&e.locales.length>1?n("multilingual-progress",{attrs:{id:e.multilingualProgressId,count:e.multilingualFieldsCompleted,total:e.locales.length}}):e._e(),e.errors&&e.errors.length?n("field-error",{attrs:{id:e.describedByErrorId,messages:e.errors}}):e._e()],1)])},Vr=[],qr={name:"FieldSelect",extends:$t,props:{options:Array},computed:{localizedOptions:function(){return this.isMultilingual?this.options[this.localeKey]:this.options}}},Gr=qr,Jr=(n("49dd"),h(Gr,Ur,Vr,!1,null,null,null)),Kr=Jr.exports,Xr=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"pkpFormField pkpFormField--select pkpFormField--selectIssue"},[n("div",{staticClass:"pkpFormField__heading"},[n("form-field-label",{attrs:{controlId:e.controlId,label:e.label,isRequired:e.isRequired,requiredLabel:e.__("common.required")}}),e.tooltip?n("tooltip",{attrs:{"aria-hidden":"true",tooltip:e.tooltip,label:""}}):e._e(),e.tooltip?n("span",{staticClass:"-screenReader",attrs:{id:e.describedByTooltipId},domProps:{innerHTML:e._s(e.tooltip)}}):e._e(),e.helpTopic?n("help-button",{attrs:{id:e.describedByHelpId,topic:e.helpTopic,section:e.helpSection,label:e.__("help.help")}}):e._e()],1),e.description?n("div",{staticClass:"pkpFormField__description",attrs:{id:e.describedByDescriptionId},domProps:{innerHTML:e._s(e.description)}}):e._e(),n("div",{staticClass:"pkpFormField__control"},[n("span",{staticClass:"pkpFormField__description"},[n("span",{domProps:{innerHTML:e._s(e.notice)}}),e.button?n("pkp-button",e._b({staticClass:"pkpFormField--selectIssue__button",on:{click:function(t){return e.emitGlobal(e.button.event)}}},"pkp-button",e.button,!1),[e._v(" "+e._s(e.button.label)+" ")]):e._e()],1),e.errors&&e.errors.length?n("field-error",{attrs:{id:e.describedByErrorId,messages:e.errors}}):e._e()],1)])},Qr=[],Zr={name:"FieldSelectIssue",extends:Kr,props:{assignedNoticeBase:{type:String,required:!0},assignLabel:{type:String,required:!0},changeIssueLabel:{type:String,required:!0},publicationStatus:{type:Number,required:!0},publishedNoticeBase:{type:String,required:!0},scheduledNoticeBase:{type:String,required:!0},unscheduledNotice:{type:String,required:!0},unscheduleLabel:{type:String,required:!0}},computed:{button:function(){var e=null;return this.publicationStatus===pkp.const.STATUS_SCHEDULED?e={event:"unpublish:publication",isWarnable:!0,label:this.unscheduleLabel}:this.publicationStatus!==pkp.const.STATUS_PUBLISHED&&(e={event:"schedule:publication",label:this.value?this.changeIssueLabel:this.assignLabel}),e},notice:function(){var e="";if(this.publicationStatus===pkp.const.STATUS_PUBLISHED)e=this.publishedNoticeBase;else if(this.publicationStatus===pkp.const.STATUS_SCHEDULED)e=this.scheduledNoticeBase;else{if(!this.value)return this.unscheduledNotice;e=this.assignedNoticeBase}return e.replace("__issueId__",this.value).replace("{$issueName}",this.selectedIssueName)},selectedIssueName:function(){var e=this;if(!this.value)return"";var t=this.options.find((function(t){return t.value===e.value}));return t?t.label:""}},methods:{emitGlobal:function(e){pkp.eventBus.$emit(e)}}},ei=Zr,ti=(n("82d0"),h(ei,Xr,Qr,!1,null,null,null)),ni=ti.exports,ri={name:"FieldSelectSubmissions",extends:mn,methods:{setSuggestions:function(e){var t=this,n=e.filter((function(e){return!t.selected.find((function(t){return t.id===e.id}))})).map((function(e){var n=e.publications.find((function(t){return t.id===e.currentPublicationId}));return n?{value:e.id,label:t.localize(n.fullTitle)}:{value:0,label:""}}));this.suggestions=n}}},ii=ri,ai=h(ii,Qt,Zt,!1,null,null,null),oi=ai.exports,si={name:"FieldShowEnsuringLink",extends:En,mixins:[k],props:{message:String,modalTitle:String},computed:{classes:function(){return["pkpFormField--showEnsuringLink"]}},mounted:function(){var e=this;$(".pkpFormField--options__option button",this.$el).click((function(){return e.openDialog({modalName:"ensureAnonymousReview",confirmLabel:e.__("common.ok"),message:e.message,title:e.modalTitle,callback:function(){e.$modal.hide("ensureAnonymousReview")}},{height:"auto",scrollable:!0}),!1}))},beforeDestroy:function(){$(".pkpFormField--options__option button",this.$el).off()}},li=si,ui=(n("86da"),h(li,en,tn,!1,null,null,null)),ci=ui.exports,di=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"pkpFormField pkpFormField--text",class:e.classes},[n("div",{staticClass:"pkpFormField__heading"},[n("form-field-label",{attrs:{controlId:e.controlId,label:e.label,localeLabel:e.localeLabel,isRequired:e.isRequired,requiredLabel:e.__("common.required"),multilingualLabel:e.multilingualLabel}}),e.isPrimaryLocale&&e.tooltip?n("tooltip",{attrs:{"aria-hidden":"true",tooltip:e.tooltip,label:""}}):e._e(),e.isPrimaryLocale&&e.tooltip?n("span",{staticClass:"-screenReader",attrs:{id:e.describedByTooltipId},domProps:{innerHTML:e._s(e.tooltip)}}):e._e(),e.isPrimaryLocale&&e.helpTopic?n("help-button",{attrs:{id:e.describedByHelpId,topic:e.helpTopic,section:e.helpSection,label:e.__("help.help")}}):e._e()],1),e.isPrimaryLocale&&e.description?n("div",{staticClass:"pkpFormField__description",attrs:{id:e.describedByDescriptionId},domProps:{innerHTML:e._s(e.description)}}):e._e(),n("div",{staticClass:"pkpFormField__control",class:e.controlClasses},[n("div",{staticClass:"pkpFormField__control_top"},["checkbox"===e.inputType?n("input",{directives:[{name:"model",rawName:"v-model",value:e.currentValue,expression:"currentValue"}],ref:"input",staticClass:"pkpFormField__input pkpFormField--text__input",style:e.inputStyles,attrs:{id:e.controlId,name:e.localizedName,"aria-describedby":e.describedByIds,"aria-invalid":e.errors&&e.errors.length,disabled:e.isDisabled,required:e.isRequired,type:"checkbox"},domProps:{checked:Array.isArray(e.currentValue)?e._i(e.currentValue,null)>-1:e.currentValue},on:{change:function(t){var n=e.currentValue,r=t.target,i=!!r.checked;if(Array.isArray(n)){var a=null,o=e._i(n,a);r.checked?o<0&&(e.currentValue=n.concat([a])):o>-1&&(e.currentValue=n.slice(0,o).concat(n.slice(o+1)))}else e.currentValue=i}}}):"radio"===e.inputType?n("input",{directives:[{name:"model",rawName:"v-model",value:e.currentValue,expression:"currentValue"}],ref:"input",staticClass:"pkpFormField__input pkpFormField--text__input",style:e.inputStyles,attrs:{id:e.controlId,name:e.localizedName,"aria-describedby":e.describedByIds,"aria-invalid":e.errors&&e.errors.length,disabled:e.isDisabled,required:e.isRequired,type:"radio"},domProps:{checked:e._q(e.currentValue,null)},on:{change:function(t){e.currentValue=null}}}):n("input",{directives:[{name:"model",rawName:"v-model",value:e.currentValue,expression:"currentValue"}],ref:"input",staticClass:"pkpFormField__input pkpFormField--text__input",style:e.inputStyles,attrs:{id:e.controlId,name:e.localizedName,"aria-describedby":e.describedByIds,"aria-invalid":e.errors&&e.errors.length,disabled:e.isDisabled,required:e.isRequired,type:e.inputType},domProps:{value:e.currentValue},on:{input:function(t){t.target.composing||(e.currentValue=t.target.value)}}}),e.prefix?n("span",{ref:"prefix",staticClass:"pkpFormField__inputPrefix",style:e.prefixStyles,domProps:{innerHTML:e._s(e.prefix)},on:{click:e.setFocus}}):e._e(),e.isMultilingual&&e.locales.length>1?n("multilingual-progress",{attrs:{id:e.multilingualProgressId,count:e.multilingualFieldsCompleted,total:e.locales.length}}):e._e(),e.optIntoEdit&&e.isDisabled?n("pkp-button",{staticClass:"pkpFormField--text__optIntoEdit",on:{click:function(t){e.isDisabled=!1}}},[e._v(" "+e._s(e.optIntoEditLabel)+" ")]):e._e()],1),e.errors&&e.errors.length?n("field-error",{attrs:{id:e.describedByErrorId,messages:e.errors}}):e._e()],1)])},fi=[],hi=(n("c975"),{name:"FieldText",extends:$t,props:{inputType:String,optIntoEdit:Boolean,optIntoEditLabel:String,size:{default:"normal",validator:function(e){return-1!==["small","normal","large"].indexOf(e)}},prefix:String},data:function(){return{inputStyles:{},isDisabled:!1,prefixStyles:{}}},computed:{classes:function(){return["pkpFormField--size"+this.size]},controlClasses:function(){var e=[];return this.isMultilingual&&this.locales.length>1&&e.push("pkpFormField__control--hasMultilingualIndicator"),this.prefix&&e.push("pkpFormField__control--hasPrefix"),e}},methods:{setFocus:function(){this.$refs.input.focus()}},mounted:function(){var e=this;this.$nextTick((function(){setTimeout((function(){e.prefix&&(e.inputStyles={"padding-left":e.$refs.prefix.clientWidth+e.$refs.prefix.offsetLeft+"px"},e.$nextTick((function(){var t=e.$refs.prefix.clientWidth+e.$refs.prefix.offsetLeft;t>e.$refs.input.clientWidth-20&&(e.prefixStyles={width:e.$refs.input.clientWidth-e.$refs.prefix.offsetLeft-80+"px",display:"block","white-space":"nowrap","overflow-x":"hidden","text-overflow":"ellipsis"},e.$nextTick((function(){e.inputStyles={"padding-left":e.$refs.prefix.clientWidth+e.$refs.prefix.offsetLeft+"px"}})))})))}),700)})),this.optIntoEdit&&(this.isDisabled=!0)}}),pi=hi,mi=(n("5e33"),h(pi,di,fi,!1,null,null,null)),_i=mi.exports,vi=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"pkpFormField pkpFormField--textarea",class:e.classes},[n("div",{staticClass:"pkpFormField__heading"},[n("form-field-label",{attrs:{controlId:e.controlId,label:e.label,localeLabel:e.localeLabel,isRequired:e.isRequired,requiredLabel:e.__("common.required"),multilingualLabel:e.multilingualLabel}}),e.isPrimaryLocale&&e.tooltip?n("tooltip",{attrs:{"aria-hidden":"true",tooltip:e.tooltip,label:""}}):e._e(),e.isPrimaryLocale&&e.tooltip?n("span",{staticClass:"-screenReader",attrs:{id:e.describedByTooltipId},domProps:{innerHTML:e._s(e.tooltip)}}):e._e(),e.isPrimaryLocale&&e.helpTopic?n("help-button",{attrs:{id:e.describedByHelpId,topic:e.helpTopic,section:e.helpSection,label:e.__("help.help")}}):e._e()],1),e.isPrimaryLocale&&e.description?n("div",{staticClass:"pkpFormField__description",attrs:{id:e.describedByDescriptionId},domProps:{innerHTML:e._s(e.description)}}):e._e(),n("div",{staticClass:"pkpFormField__control"},[n("textarea",{directives:[{name:"model",rawName:"v-model",value:e.currentValue,expression:"currentValue"}],staticClass:"pkpFormField__input pkpFormField--textarea__input",attrs:{id:e.controlId,name:e.localizedName,"aria-describedby":e.describedByIds,"aria-invalid":e.errors&&e.errors.length,required:e.isRequired},domProps:{value:e.currentValue},on:{input:function(t){t.target.composing||(e.currentValue=t.target.value)}}}),e.isMultilingual&&e.locales.length>1?n("multilingual-progress",{attrs:{id:e.multilingualProgressId,count:e.multilingualFieldsCompleted,total:e.locales.length}}):e._e()],1),e.errors&&e.errors.length?n("field-error",{attrs:{id:e.describedByErrorId,messages:e.errors}}):e._e()],1)},gi=[],bi={name:"FieldTextarea",extends:$t,props:{size:{type:String,default:"normal",validator:function(e){return-1!==["small","normal","large"].indexOf(e)}}},computed:{classes:function(){return["pkpFormField--"+this.size]}}},yi=bi,Mi=(n("153d"),h(yi,vi,gi,!1,null,null,null)),ki=Mi.exports,wi=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"pkpFormField pkpFormField--upload"},[n("div",{staticClass:"pkpFormField__heading"},[n("form-field-label",{attrs:{controlId:e.dropzoneHiddenFileId,label:e.label,localeLabel:e.localeLabel,isRequired:e.isRequired,requiredLabel:e.__("common.required"),multilingualLabel:e.multilingualLabel}}),e.tooltip?n("tooltip",{attrs:{"aria-hidden":"true",tooltip:e.tooltip,label:""}}):e._e(),e.tooltip?n("span",{staticClass:"-screenReader",attrs:{id:e.describedByTooltipId},domProps:{innerHTML:e._s(e.tooltip)}}):e._e(),e.helpTopic?n("help-button",{attrs:{id:e.describedByHelpId,topic:e.helpTopic,section:e.helpSection,label:e.__("help.help")}}):e._e()],1),e.isPrimaryLocale&&e.description?n("div",{staticClass:"pkpFormField__description",attrs:{id:e.describedByDescriptionId},domProps:{innerHTML:e._s(e.description)}}):e._e(),n("div",{ref:"control",staticClass:"pkpFormField__control pkpFormField--upload__control",attrs:{id:e.controlId,"aria-live":"polite"}},[e.currentValue?n("div",{staticClass:"pkpFormField--upload__preview -pkpClearfix"},[n("span",{staticClass:"pkpFormField--upload__fileName"},[e._v(" "+e._s(e.fileName)+" ")]),n("div",{staticClass:"pkpFormField--upload__previewActions"},[n("pkp-button",{attrs:{isWarnable:!0},on:{click:e.clear}},[e._v(" "+e._s(e.__("common.remove"))+" ")]),e.initialValue&&!e.isInitialValue?n("pkp-button",{on:{click:e.revert}},[e._v(" "+e._s(e.restoreLabel)+" ")]):e._e(),e.isMultilingual&&e.locales.length>1?n("multilingual-progress",{attrs:{id:e.multilingualProgressId,count:e.multilingualFieldsCompleted,total:e.locales.length}}):e._e()],1)]):e._e(),e.currentValue&&e.errors&&e.errors.length?n("field-error",{attrs:{id:e.describedByErrorId,messages:e.errors}}):e._e(),n("div",{class:{"-screenReader":e.currentValue}},[n("vue-dropzone",{ref:"dropzone",attrs:{id:e.dropzoneId,options:e.dropzoneOptions},on:{"vdropzone-file-added":e.onAddFile,"vdropzone-success":e.success,"vdropzone-error":e.error,"vdropzone-removed-file":e.onRemoveFile}}),!e.currentValue&&e.errors&&e.errors.length?n("field-error",{attrs:{id:e.describedByErrorId,messages:e.errors}}):e._e(),n("div",{staticClass:"pkpFormField--upload__uploadActions"},[n("pkp-button",{staticClass:"pkpFormField--upload__addFile",attrs:{disabled:!!e.uploadFile,id:e.dropzoneClickableId}},[e._v(" "+e._s(e.uploadFileLabel)+" ")]),e.currentValue||!e.initialValue||e.isInitialValue?e._e():n("pkp-button",{staticClass:"pkpFormField--upload__restore",on:{click:e.revert}},[e._v(" "+e._s(e.restoreLabel)+" ")]),e.isMultilingual&&e.locales.length>1?n("multilingual-progress",{attrs:{id:e.multilingualProgressId,count:e.multilingualFieldsCompleted,total:e.locales.length}}):e._e()],1)],1)],1)])},Li=[],xi=n("92c3"),Si=n.n(xi),Ti={name:"FieldUpload",extends:$t,components:{VueDropzone:Si.a},props:{options:Object,restoreLabel:String,uploadFileLabel:String},data:function(){return{initialValue:null,uploadFile:null}},computed:{isInitialValue:function(){return!!this.currentValue&&!!this.initialValue&&!!this.currentValue.dateUploaded&&!!this.initialValue.dateUploaded&&this.currentValue.dateUploaded===this.initialValue.dateUploaded},fileName:function(){return this.uploadFile?this.uploadFile.upload.filename:this.currentValue.uploadName},controlId:function(){return this.compileId("control")},dropzoneClickableId:function(){return this.compileId("clickable")},dropzoneId:function(){return this.compileId("dropzone")},dropzoneHiddenFileId:function(){return this.compileId("hiddenFileId")},dropzoneOptions:function(){return o({method:"POST",thumbnailWidth:240,maxFiles:1,hiddenInputContainer:"#"+this.controlId,clickable:["#"+this.controlId+" .dropzone","#"+this.dropzoneClickableId],addRemoveLinks:!0,previewTemplate:'<div class="dz-preview">\n\t\t\t\t\t<img class="dz-image" data-dz-thumbnail />\n\t\t\t\t\t<div class="dz-details">\n\t\t\t\t\t\t<div class="dz-filename"><span data-dz-name></span></div>\n\t\t\t\t\t\t<div class="dz-progress"><span class="dz-upload" data-dz-uploadprogress></span></div>\n\t\t\t\t\t</div>\n\t\t\t\t</div>',headers:{"X-Csrf-Token":pkp.currentUser.csrfToken}},this.options)}},methods:{clear:function(){this.$emit("change",this.name,"value",null,this.localeKey),this.uploadFile=null,this.$refs.dropzone.dropzone.removeAllFiles(),this.setFocusToControl(),this.setErrors([])},revert:function(){this.$emit("change",this.name,"value",this.initialValue,this.localeKey),this.uploadFile=null,this.setFocusToControl(),this.setErrors([])},success:function(e,t){this.$emit("change",this.name,"value",{temporaryFileId:t.id},this.localeKey),this.setFocusToControl()},onAddFile:function(e){this.uploadFile=o({},e),this.setErrors([])},onRemoveFile:function(){this.uploadFile=null,this.setErrors([])},error:function(e,t){var n=this.errors.slice();"string"===typeof t?n.push(t):"object"===$e(t)&&n.push(t.errorMessage),this.setErrors(n)},setErrors:function(e){this.$emit("set-errors",this.name,e,this.localeKey)},setFocusToControl:function(){var e=this;this.$nextTick((function(){var t=e.$refs.control.querySelectorAll("input,button");t.length?t[0].focus():e.$el.focus()}))}},watch:{value:function(e,t){e!==t&&this.setFocusToControl()}},mounted:function(){this.$refs.dropzone.dropzone.hiddenFileInput.id=this.dropzoneHiddenFileId,this.$refs.dropzone.dropzone.hiddenFileInput.setAttribute("aria-describedby",this.describedByIds),this.initialValue=this.currentValue?this.currentValue:null}},Di=Ti,Yi=(n("ae1f"),h(Di,wi,Li,!1,null,null,null)),Ci=Yi.exports,Ei=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"pkpFormField pkpFormField--upload pkpFormField--uploadImage"},[n("div",{staticClass:"pkpFormField__heading"},[n("form-field-label",{attrs:{controlId:e.dropzoneHiddenFileId,label:e.label,localeLabel:e.localeLabel,isRequired:e.isRequired,requiredLabel:e.__("common.required"),multilingualLabel:e.multilingualLabel}}),e.tooltip?n("tooltip",{attrs:{"aria-hidden":"true",tooltip:e.tooltip,label:""}}):e._e(),e.tooltip?n("span",{staticClass:"-screenReader",attrs:{id:e.describedByTooltipId},domProps:{innerHTML:e._s(e.tooltip)}}):e._e(),e.helpTopic?n("help-button",{attrs:{id:e.describedByHelpId,topic:e.helpTopic,section:e.helpSection,label:e.__("help.help")}}):e._e()],1),e.isPrimaryLocale&&e.description?n("div",{staticClass:"pkpFormField__description",attrs:{id:e.describedByDescriptionId},domProps:{innerHTML:e._s(e.description)}}):e._e(),n("div",{ref:"control",staticClass:"pkpFormField__control pkpFormField--upload__control",attrs:{id:e.controlId,"aria-live":"polite"}},[e.currentValue?n("div",{staticClass:"pkpFormField--upload__preview -pkpClearfix"},[this.thumbnail?n("div",{staticClass:"pkpFormField--uploadImage__thumbnailWrapper"},[n("img",{staticClass:"pkpFormField--uploadImage__thumbnail",attrs:{alt:e.thumbnailDescription,src:e.thumbnail}})]):e._e(),n("div",{staticClass:"pkpFormField--upload__details"},[n("label",{staticClass:"pkpFormFieldLabel",attrs:{for:e.altTextId}},[e._v(" "+e._s(e.altTextLabel)+" ")]),n("input",{directives:[{name:"model",rawName:"v-model",value:e.altTextValue,expression:"altTextValue"}],staticClass:"pkpFormField__input pkpFormField--uploadImage__altTextInput",attrs:{type:"text",id:e.altTextId,"aria-describedby":e.altTextDescriptionId},domProps:{value:e.altTextValue},on:{input:function(t){t.target.composing||(e.altTextValue=t.target.value)}}}),n("div",{staticClass:"pkpFormField--uploadImage__altTextDescription",attrs:{id:e.altTextDescriptionId},domProps:{innerHTML:e._s(e.altTextDescription)}})]),n("div",{staticClass:"pkpFormField--upload__previewActions"},[n("pkp-button",{attrs:{isWarnable:!0},on:{click:e.clear}},[e._v(" "+e._s(e.__("common.remove"))+" ")]),e.initialValue&&!e.isInitialValue?n("pkp-button",{on:{click:e.revert}},[e._v(" "+e._s(e.restoreLabel)+" ")]):e._e(),e.isMultilingual&&e.locales.length>1?n("multilingual-progress",{attrs:{id:e.multilingualProgressId,count:e.multilingualFieldsCompleted,total:e.locales.length}}):e._e()],1)]):e._e(),e.currentValue&&e.errors&&e.errors.length?n("field-error",{attrs:{id:e.describedByErrorId,messages:e.errors}}):e._e(),n("div",{class:{"-screenReader":e.currentValue}},[n("vue-dropzone",{ref:"dropzone",attrs:{id:e.dropzoneId,options:e.dropzoneOptions},on:{"vdropzone-file-added":e.onAddFile,"vdropzone-thumbnail":e.onThumbnail,"vdropzone-success":e.success,"vdropzone-error":e.error,"vdropzone-removed-file":e.onRemoveFile}}),e.errors&&e.errors.length?n("field-error",{attrs:{id:e.describedByErrorId,messages:e.errors}}):e._e(),n("div",{staticClass:"pkpFormField--upload__uploadActions"},[n("pkp-button",{staticClass:"pkpFormField--upload__addFile",attrs:{disabled:!!e.uploadFile,id:e.dropzoneClickableId}},[e._v(" "+e._s(e.uploadFileLabel)+" ")]),e.initialValue&&!e.isInitialValue?n("pkp-button",{staticClass:"pkpFormField--upload__restore",on:{click:e.revert}},[e._v(" "+e._s(e.restoreLabel)+" ")]):e._e(),e.isMultilingual&&e.locales.length>1?n("multilingual-progress",{attrs:{id:e.multilingualProgressId,count:e.multilingualFieldsCompleted,total:e.locales.length}}):e._e()],1)],1)],1)])},Ai=[],Oi={name:"FieldUploadImage",extends:Ci,props:{altTextDescription:String,altTextLabel:String,baseUrl:String,thumbnailDescription:String},data:function(){return{altTextValue:"",initialValue:null}},computed:{thumbnail:function(){return this.uploadFile?this.uploadFile.dataURL:this.currentValue&&this.currentValue.uploadName?this.baseUrl+"/"+this.currentValue.uploadName:""},altTextId:function(){return this.compileId("altText")},altTextDescriptionId:function(){return this.compileId("altTextDescription")}},methods:{success:function(e,t){this.isUploading=!1,this.$emit("change",this.name,"value",{temporaryFileId:t.id,altText:""},this.localeKey),this.setFocusToControl()},onThumbnail:function(e){this.uploadFile=o({},e),this.setFocusToControl()}},watch:{currentValue:function(e,t){e!==t&&(this.altTextValue=e&&e.altText?e.altText:"")},altTextValue:function(e,t){e!==t&&(e||this.currentValue)&&this.$emit("change",this.name,"value",o(o({},this.currentValue),{},{altText:e}),this.localeKey)}},mounted:function(){this.$refs.dropzone.dropzone.hiddenFileInput.id=this.dropzoneHiddenFileId,this.$refs.dropzone.dropzone.hiddenFileInput.setAttribute("aria-describedby",this.describedByIds),this.altTextValue=this.currentValue?this.currentValue.altText:""}},Pi=Oi,Fi=(n("0e37"),h(Pi,Ei,Ai,!1,null,null,null)),Ii=Fi.exports,ji={name:"FormGroup",components:{FieldBaseAutosuggest:mn,FieldArchivingPn:Fn,FieldColor:zn,FieldControlledVocab:Vn,FieldPubId:Qn,FieldHtml:ir,FieldMetadataSetting:cr,FieldOptions:En,FieldRadioInput:_r,FieldRichTextarea:Wr,FieldSelect:Kr,FieldSelectIssue:ni,FieldSelectSubmissions:oi,FieldShowEnsuringLink:ci,FieldText:_i,FieldTextarea:ki,FieldUpload:Ci,FieldUploadImage:Ii},props:{id:String,label:String,description:String,pageId:String,fields:Array,errors:Object,formId:String,primaryLocale:String,visibleLocales:Array,availableLocales:Array,showWhen:[String,Array]},computed:{fieldsInGroup:function(){var e=this;return this.fields.filter((function(t){return t.groupId===e.id&&e.shouldShowField(t)}))},hasErrors:function(){for(var e in this.fieldsInGroup)if(e in this.errors)return!0;return!1}},methods:{fieldChanged:function(e,t,n,r){this.$emit("change",e,t,n,r)},shouldShowField:function(e){if("undefined"===typeof e.showWhen)return!0;var t="string"===typeof e.showWhen?e.showWhen:e.showWhen[0],n=this.fields.find((function(e){return e.name===t}));return!!n&&("string"===typeof e.showWhen?!!n.value:n.value===e.showWhen[1])},setFieldErrors:function(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:"",r=o({},this.errors);t&&t.length?n?(r[e]=r[e]||{},r[e][n]=t):r[e]=t:n&&r[e]&&r[e][n]?delete r[e][n]:r[e]&&delete r[e],this.$emit("set-errors",r)}}},Hi=ji,Ri=(n("ec70"),h(Hi,it,at,!1,null,null,null)),Ni=Ri.exports,$i={name:"FormPage",components:{FormErrors:rt,FormGroup:Ni},data:function(){return{hasRecentSave:!1,recentSaveInterval:null}},props:{id:String,groups:Array,fields:Array,errors:Object,formId:String,isCurrentPage:Boolean,isLastPage:Boolean,lastSaveTimestamp:Number,primaryLocale:String,visibleLocales:Array,availableLocales:Array,submitButton:Object,previousButton:Object,canSubmit:{type:Boolean,default:function(){return!0}},isSaving:Boolean},computed:{groupsInPage:function(){var e=this;return this.groups.filter((function(t){return t.pageId===e.id&&e.shouldShowGroup(t)}))},hasFooter:function(){return this.previousButton||this.submitButton||Object.keys(this.errors).length}},watch:{isSaving:function(){this.$refs.buttons.focus()}},methods:{fieldChanged:function(e,t,n,r){this.$emit("change",e,t,n,r)},submit:function(){this.$emit("pageSubmitted",this.id)},previousPage:function(){this.$emit("previousPage",this.id)},showField:function(e){this.$emit("showField",e)},showLocale:function(e){this.$emit("showLocale",e)},shouldShowGroup:function(e){if("undefined"===typeof e.showWhen)return!0;var t="string"===typeof e.showWhen?e.showWhen:e.showWhen[0],n=this.fields.find((function(e){return e.name===t}));return!!n&&("string"===typeof e.showWhen?!!n.value:n.value===e.showWhen[1])},setErrors:function(e){this.$emit("set-errors",e)}},mounted:function(){var e=this;this.recentSaveInterval=setInterval((function(){var t=e.lastSaveTimestamp+5e3;e.hasRecentSave&&t<Date.now()?e.hasRecentSave=!1:!e.hasRecentSave&&t>Date.now()&&(e.hasRecentSave=!0)}),250)},destroyed:function(){clearInterval(this.recentSaveInterval)}},zi=$i,Bi=(n("44dc"),h(zi,Ke,Xe,!1,null,null,null)),Wi=Bi.exports,Ui={name:"PkpForm",components:{FormLocales:Je,FormPage:Wi},props:{id:String,method:String,action:String,canSubmit:{type:Boolean,default:function(){return!0}},errors:{type:Object,default:function(){return{}}},fields:Array,groups:Array,pages:Array,primaryLocale:String,visibleLocales:Array,supportedFormLocales:Array},data:function(){return{currentPage:"",isSaving:!1,lastSaveTimestamp:-1}},computed:{classes:function(){var e=[];return this.visibleLocales.length>1&&e.push("pkpForm--hasManyVisibleLocales"),e},availableLocales:function(){return this.hasMultilingualFields?this.supportedFormLocales:[]},hasMultilingualFields:function(){return!!this.fields.find((function(e){return e.isMultilingual}))},groupIdsWithErrors:function(){var e=this,t=[];return Object.keys(this.errors).forEach((function(n){var r=e.fields.find((function(e){return e.name===n}));r&&!t.includes(r.groupId)&&t.push(r.groupId)})),t},pageIdsWithErrors:function(){var e=this,t=[];return this.groupIdsWithErrors.forEach((function(n){var r=e.groups.find((function(e){return e.id===n}));r&&!t.includes(r.pageId)&&t.push(r.pageId)})),t},submitValues:function(){var e=this,t={};return this.fields.forEach((function(n){if(n.isMultilingual){var r={};Object.keys(n.value).forEach((function(t){e.supportedFormLocales.filter((function(e){return e.key===t})).length&&(Array.isArray(n.value[t])&&!n.value[t].length?r[t]="":r[t]=n.value[t])})),Object.keys(r).length&&(t[n.name]=r)}else Array.isArray(n.value)&&!n.value.length?t[n.name]="":t[n.name]=n.value})),t}},methods:{nextPage:function(e){var t=this.pages.findIndex((function(t){return t.id===e}));1!==this.pages.length&&t!==this.pages.length-1?this.setCurrentPage(!0):this.submit()},submit:function(){if(!this.canSubmit)return!1;this.isSaving=!0;var e=this.validate();if(Object.keys(e).length)return this.$emit("set",this.id,{errors:o(o({},this.errors),e)}),void(this.isSaving=!1);var t=this.method;"DELETE"!==t&&"PUT"!==t||(t="POST"),$.ajax({context:this,method:t,url:this.action,headers:{"X-Csrf-Token":pkp.currentUser.csrfToken,"X-Http-Method-Override":this.method},data:this.submitValues,success:this.success,error:this.error,complete:this.complete})},validate:function(){return this.validateRequired()},validateRequired:function(){var e=this,t={};return this.fields.forEach((function(n){if(n.isRequired){var r=!1,i=n.isMultilingual?n.value[e.primaryLocale]:n.value;switch($e(i)){case"undefined":r=!0;break;case"string":case"array":i.length||(r=!0);break;case"object":i||(r=!0);break}r&&(n.isMultilingual?(t[n.name]={},t[n.name][e.primaryLocale]=[e.__("validator.required")]):t[n.name]=[e.__("validator.required")])}})),t},success:function(e){this.$emit("success",e),this.lastSaveTimestamp=Date.now(),pkp.eventBus.$emit("form-success",this.id,e);var t=this.fields.map((function(t){return"undefined"!==typeof e[t.name]&&(t.value=e[t.name]),t}));this.$emit("set",this.id,{fields:t})},error:function(e){e.status&&400===e.status?(pkp.eventBus.$emit("notify",this.__("form.errors",{count:Object.keys(e.responseJSON).length}),"warning"),this.$emit("set",this.id,{errors:e.responseJSON})):e.status&&[403,404].includes(e.status)&&e.responseJSON&&e.responseJSON.errorMessage?pkp.eventBus.$emit("notify",e.responseJSON.errorMessage,"warning"):pkp.eventBus.$emit("notify",this.__("common.unknownError","warning"))},complete:function(){this.isSaving=!1},fieldChanged:function(e,t,n,r){var i=this.fields.map((function(i){return i.name===e&&(r?i[t][r]=n:i[t]=n),i}));this.$emit("set",this.id,{fields:i}),this.removeError(e,r)},setCurrentPage:function(e){var t=this,n=this.pages.findIndex((function(e){return e.id===t.currentPage}));if(!0===e){if(this.pages.length<=n)return;e=this.pages[n+1].id}else if(!1===e){if(!n)return;e=this.pages[n-1].id}this.currentPage=e,this.$scrollTo(this.$el,500,{offset:-50})},setVisibleLocales:function(e){this.$emit("set",this.id,{visibleLocales:e})},showLocale:function(e){this.visibleLocales.includes(e)||this.setVisibleLocales([this.primaryLocale,e])},showField:function(e){var t=this,n=this.fields.find((function(t){return t.name===e}));if(n){var r=this.groups.find((function(e){return e.id===n.groupId}));r&&(this.currentPage=r.pageId,this.$nextTick((function(){var e=document.querySelector('[id*="'+t.id+"-"+n.name+'"]');e?t.$scrollTo(e,500,{offset:-50}):t.setCurrentPage(r.pageId)})))}},removeError:function(e,t){if(this.errors[e]){var n=o({},this.errors);t&&"string"!==typeof n[e]?(n[e][t]&&delete n[e][t],Object.keys(n[e]).length||delete n[e]):delete n[e],this.$emit("set",this.id,{errors:n})}},setErrors:function(e){this.$emit("set",this.id,{errors:e})}},mounted:function(){this.currentPage||(this.currentPage=this.pages[0].id)}},Vi=Ui,qi=(n("f174"),h(Vi,Re,Ne,!1,null,null,null)),Gi=qi.exports,Ji=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",[e._t("default",[n("list-panel",{staticClass:"listPanel--selectReviewer",attrs:{isSidebarVisible:e.isSidebarVisible,items:e.items},scopedSlots:e._u([{key:"item",fn:function(t){var r=t.item;return[n("select-reviewer-list-item",{key:r.id,attrs:{activeReviewsCountLabel:e.activeReviewsCountLabel,activeReviewsLabel:e.activeReviewsLabel,averageCompletionLabel:e.averageCompletionLabel,biographyLabel:e.biographyLabel,cancelledReviewsLabel:e.cancelledReviewsLabel,completedReviewsLabel:e.completedReviewsLabel,currentlyAssigned:e.currentlyAssigned.includes(r.id),currentlyAssignedLabel:e.currentlyAssignedLabel,daySinceLastAssignmentLabel:e.daySinceLastAssignmentLabel,daysSinceLastAssignmentLabel:e.daysSinceLastAssignmentLabel,daysSinceLastAssignmentDescriptionLabel:e.daysSinceLastAssignmentDescriptionLabel,declinedReviewsLabel:e.declinedReviewsLabel,gossipLabel:e.gossipLabel,item:r,neverAssignedLabel:e.neverAssignedLabel,reviewerRatingLabel:e.reviewerRatingLabel,reviewInterestsLabel:e.reviewInterestsLabel,selectReviewerLabel:e.selectReviewerLabel,selectorName:e.selectorName,warnOnAssignment:e.warnOnAssignment.includes(r.id),warnOnAssignmentLabel:e.warnOnAssignmentLabel,warnOnAssignmentUnlockLabel:e.warnOnAssignmentUnlockLabel}})]}}])},[n("pkp-header",{attrs:{slot:"header"},slot:"header"},[n("h2",[e._v(" "+e._s(e.title)+" ")]),e.isLoading?n("spinner"):e._e(),n("template",{slot:"actions"},[n("search",{attrs:{searchPhrase:e.searchPhrase},on:{"search-phrase-changed":e.setSearchPhrase}}),n("pkp-button",{attrs:{isActive:e.isSidebarVisible},on:{click:function(t){e.isSidebarVisible=!e.isSidebarVisible}}},[n("icon",{attrs:{icon:"filter",inline:!0}}),e._v(" "+e._s(e.__("common.filter"))+" ")],1)],1)],2),n("template",{slot:"sidebar"},[n("pkp-header",{attrs:{isOneLine:!1}},[n("h2",[n("icon",{attrs:{icon:"filter",inline:!0}}),e._v(" "+e._s(e.__("common.filter"))+" ")],1)]),e._l(e.filters,(function(t){return n(t.filterType||"filter-slider",e._b({key:t.param,tag:"component",attrs:{isFilterActive:e.isFilterActive(t.param,t.value)},on:{"add-filter":e.addFilter,"update-filter":e.addFilter,"remove-filter":e.removeFilter}},"component",t,!1))}))],2),e.isLoading?n("template",{slot:"itemsEmpty"},[e.isLoading?[n("spinner"),e._v(" "+e._s(e.__("common.loading"))+" ")]:[e._v(" "+e._s(e.emptyLabel)+" ")]],2):e._e(),n("template",{slot:"footer"},[e.lastPage>1?n("pagination",{attrs:{currentPage:e.currentPage,isLoading:e.isLoading,lastPage:e.lastPage},on:{"set-page":e.setPage}}):e._e()],1)],2)])],2)},Ki=[],Xi=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"pkpFilter pkpFilter--slider",class:e.classes},[e.isFilterActive?n("button",{staticClass:"pkpFilter__remove",on:{click:e.remove}},[n("icon",{attrs:{icon:"times-circle-o"}}),n("span",{staticClass:"-screenReader"},[e._v(" "+e._s(e.__("common.filterRemove",{filterTitle:e.title}))+" ")])],1):n("button",{staticClass:"pkpFilter__add",on:{click:e.enable}},[n("icon",{attrs:{icon:"plus-square-o"}}),n("span",{staticClass:"-screenReader"},[e._v(" "+e._s(e.__("common.filterAdd",{filterTitle:e.title}))+" ")])],1),n("div",{staticClass:"pkpFilter__inputTitle",attrs:{"aria-hidden":"true"},on:{click:e.toggle}},[e._v(" "+e._s(e.title)+" ")]),n("div",{staticClass:"pkpFilter__input pkpFilter__input--slider"},[n("label",{staticClass:"-screenReader",attrs:{for:"slider"}},[e._v(e._s(e.title))]),n("input",{directives:[{name:"model",rawName:"v-model.number",value:e.currentValue,expression:"currentValue",modifiers:{number:!0}}],attrs:{type:"range",max:e.max,min:e.min,disabled:!e.isFilterActive,id:"slider"},domProps:{value:e.currentValue},on:{__r:function(t){e.currentValue=e._n(t.target.value)},blur:function(t){return e.$forceUpdate()}}}),e.isFilterActive?n("output",{ref:"output",staticClass:"pkpFilter__value",style:e.valueStyles,attrs:{for:"slider"}},[n("span",{staticClass:"pkpFilter__valueCaret",attrs:{"aria-hidden":"true"}}),e.useStars?n("span",{staticClass:"pkpFilter__value--stars",attrs:{"aria-hidden":"true"}},e._l(5,(function(t){return n("icon",{key:t,attrs:{icon:t<=e.currentValue?"star":"star-o"}})})),1):[e._v(" "+e._s(e.currentValueLabel)+" ")]],2):e._e()])])},Qi=[],Zi=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{class:e.classes},[n("button",{staticClass:"pkpFilter__label",class:{"-isActive":e.isFilterActive},on:{click:function(t){return t.preventDefault(),t.stopPropagation(),e.toggle(e.param,e.value)}}},[e._v(" "+e._s(e.title)+" ")]),e.isFilterActive?n("button",{staticClass:"pkpFilter__remove",on:{click:function(t){return t.preventDefault(),t.stopPropagation(),e.remove(e.param,e.value)}}},[n("icon",{attrs:{icon:"times-circle-o"}}),n("span",{staticClass:"-screenReader"},[e._v(" "+e._s(e.__("common.filterRemove",{filterTitle:e.title}))+" ")])],1):e._e()])},ea=[],ta={props:{isFilterActive:{type:Boolean,default:function(){return!1}},param:{type:String,required:!0},title:{type:String,required:!0},value:{type:[String,Number,Boolean,Array,Object],required:!0}},computed:{classes:function(){var e=["pkpFilter"];return this.isFilterActive||e.push("pkpFilter--disabled"),e}},methods:{toggle:function(){this.isFilterActive?this.remove():this.$emit("add-filter",this.param,this.value)},remove:function(){this.$emit("remove-filter",this.param,this.value)}}},na=ta,ra=(n("31a5"),h(na,Zi,ea,!1,null,null,null)),ia=ra.exports,aa={extends:ia,components:{Icon:N},props:{max:{type:Number,required:!0},min:{type:Number,required:!0},useStars:{type:Boolean,default:function(){return!1}},valueLabel:{type:String,default:function(){return"{$value}"}}},data:function(){return{currentValue:this.value}},computed:{currentValueLabel:function(){return this.valueLabel.replace("{$value}",this.currentValue)},valueStyles:function(){if(this.isFilterActive){var e=Number(100*(this.currentValue-this.min)/(this.max-this.min)),t=8+e/100*-17;return{left:"calc(".concat(e,"% + ").concat(t,"px)")}}return{}}},methods:{enable:function(){this.$emit("add-filter",this.param,this.currentValue)},updateCurrentValue:le()((function(e){this.$emit("update-filter",this.param,e)}),250),remove:function(){this.$emit("remove-filter",this.param)}},watch:{currentValue:function(e,t){this.updateCurrentValue(e)}}},oa=aa,sa=(n("e6d9"),h(oa,Xi,Qi,!1,null,null,null)),la=sa.exports,ua=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"pkpFilter--slider pkpFilter--sliderMultirange",class:e.classes},[e.isFilterActive?n("button",{staticClass:"pkpFilter__remove",on:{click:e.remove}},[n("icon",{attrs:{icon:"times-circle-o"}}),n("span",{staticClass:"-screenReader"},[e._v(" "+e._s(e.__("filterRemove",{filterTitle:e.title}))+" ")])],1):n("button",{staticClass:"pkpFilter__add",on:{click:e.enable}},[n("icon",{attrs:{icon:"plus-square-o"}}),n("span",{staticClass:"-screenReader"},[e._v(" "+e._s(e.__("filterAdd",{filterTitle:e.title}))+" ")])],1),n("div",{staticClass:"pkpFilter__inputTitle",attrs:{tabindex:!e.isFilterActive&&-1,"aria-hidden":"true"},on:{click:e.toggle}},[e._v(" "+e._s(e.title)+" ")]),n("fieldset",{staticClass:"pkpFilter__input pkpFilter__input--slider pkpFilter__input--sliderMultirange"},[n("legend",{staticClass:"-screenReader"},[e._v(e._s(e.title))]),n("div",{staticClass:"pkpFilter__multirangeInput"},[n("label",{staticClass:"-screenReader",attrs:{for:"sliderMore"}},[e._v(" "+e._s(e.moreThanLabel)+" ")]),n("input",{directives:[{name:"model",rawName:"v-model.number",value:e.currentValue[0],expression:"currentValue[0]",modifiers:{number:!0}}],attrs:{type:"range",max:e.moreThanMax,min:e.min,disabled:!e.isFilterActive,id:"sliderMore"},domProps:{value:e.currentValue[0]},on:{__r:function(t){e.$set(e.currentValue,0,e._n(t.target.value))},blur:function(t){return e.$forceUpdate()}}})]),n("div",{staticClass:"pkpFilter__multirangeInput"},[n("label",{staticClass:"-screenReader",attrs:{for:"sliderLess"}},[e._v(" "+e._s(e.lessThanLabel)+" ")]),n("input",{directives:[{name:"model",rawName:"v-model.number",value:e.currentValue[1],expression:"currentValue[1]",modifiers:{number:!0}}],attrs:{type:"range",max:e.max,min:e.lessThanMin,disabled:!e.isFilterActive,id:"sliderLess"},domProps:{value:e.currentValue[1]},on:{__r:function(t){e.$set(e.currentValue,1,e._n(t.target.value))},blur:function(t){return e.$forceUpdate()}}})])]),n("div",{staticClass:"pkpFilter__value pkpFilter__value--multirange",attrs:{"aria-hidden":"true"}},[n("span",{staticClass:"pkpFilter__valueCaret",attrs:{"aria-hidden":"true"}}),e._v(" "+e._s(e.currentValueLabel)+" ")])])},ca=[],da={extends:la,props:{lessThanLabel:{type:String,required:!0},moreThanLabel:{type:String,required:!0},valueLabel:{type:String,required:!0}},computed:{lessThanMin:function(){return Math.max(this.min,this.currentValue[0])},moreThanMax:function(){return Math.min(this.max,this.currentValue[1])},currentValueLabel:function(){return this.valueLabel.replace("{$min}",this.currentValue[0]).replace("{$max}",this.currentValue[1])}}},fa=da,ha=(n("fd86"),h(fa,ua,ca,!1,null,null,null)),pa=ha.exports,ma=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("nav",{staticClass:"pkpPagination",attrs:{role:"navigation","aria-label":e.__("common.pagination.label")}},[n("ul",[n("li",[n("pkp-button",{attrs:{disabled:1===e.currentPage,"aria-label":e.__("common.pagination.goToPage",{page:e.__("common.pagination.previous")})},on:{click:function(t){return e.setPage("previous")}}},[e._v(" "+e._s(e.__("common.pagination.previous"))+" ")])],1),e._l(e.items,(function(t,r){return n("li",{key:r},[t.isSeparator?n("span",{staticClass:"pkpPagination__separator",attrs:{"aria-hidden":!0}},[e._v(" ··· ")]):e.isLoading&&t.isCurrent?n("spinner",{staticClass:"pkpPagination__loading"}):n("pkp-button",{attrs:{disabled:t.isDisabled,"aria-label":t.ariaLabel,"aria-current":t.isCurrent,"is-link":!t.isCurrent&&!["previous","next"].includes(t.value),"is-active":t.isCurrent},on:{click:function(n){return e.setPage(t.value)}}},[e._v(" "+e._s(t.label)+" ")])],1)})),n("li",[n("pkp-button",{attrs:{disabled:e.currentPage===e.lastPage},on:{click:function(t){return e.setPage("next")}}},[e._v(" "+e._s(e.__("common.pagination.next"))+" ")])],1)],2)])},_a=[],va={props:{currentPage:{type:Number,required:!0},isLoading:{type:Boolean,default:!1},lastPage:{type:Number,required:!0},showAdjacentPages:{type:Number,default:1}},computed:{items:function(){var e=[],t=Math.min(this.currentPage+this.showAdjacentPages,this.lastPage),n=Math.max(this.currentPage-this.showAdjacentPages,1);if(t-n<this.showAdjacentPages){var r=this.showAdjacentPages-(t-n);1===n?t=Math.min(t+r,this.lastPage):t===this.lastPage&&(n=Math.max(n-r,1))}n>1&&e.push({value:1,label:1,ariaLabel:this.getNumberAriaLabel(1)}),n>2&&e.push({isSeparator:!0});for(var i=n;i<=t;i++)e.push({value:i,isCurrent:this.currentPage===i,label:i,ariaLabel:this.getNumberAriaLabel(i)});return t<this.lastPage-1&&e.push({isSeparator:!0}),t<this.lastPage&&e.push({value:this.lastPage,label:this.lastPage,ariaLabel:this.getNumberAriaLabel(this.lastPage)}),e}},methods:{getNumberAriaLabel:function(e){var t=this.__("common.pageNumber",{pageNumber:e});return this.__("common.pagination.goToPage",{page:t})},setPage:function(e){"previous"===e?e=this.currentPage-1:"next"===e&&(e=this.currentPage+1),e>=1&&e<=this.lastPage&&this.$emit("set-page",e)}}},ga=va,ba=(n("6c67"),h(ga,ma,_a,!1,null,null,null)),ya=ba.exports,Ma=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"pkpSearch"},[n("label",[n("span",{staticClass:"-screenReader"},[e._v(e._s(e.currentSearchLabel))]),n("input",{staticClass:"pkpSearch__input",attrs:{type:"search",id:e.inputId,placeholder:e.currentSearchLabel},domProps:{value:e.searchPhrase},on:{keyup:e.searchPhraseKeyUp}}),n("span",{staticClass:"pkpSearch__icons"},[n("icon",{staticClass:"pkpSearch__icons--search",attrs:{icon:"search",inline:!0}})],1)]),e.searchPhrase?n("button",{staticClass:"pkpSearch__clear",attrs:{"aria-controls":e.inputId},on:{click:function(t){return t.preventDefault(),e.clearSearchPhrase(t)}}},[n("icon",{attrs:{icon:"times"}}),n("span",{staticClass:"-screenReader"},[e._v(e._s(e.__("common.clearSearch")))])],1):e._e()])},ka=[],wa={props:{searchLabel:{type:String,default:function(){return""}},searchPhrase:{type:String,default:function(){return""}}},computed:{inputId:function(){return this._uid},currentSearchLabel:function(){return this.searchLabel?this.searchLabel:this.__("common.search")}},methods:{searchPhraseKeyUp:le()((function(e){this.$emit("search-phrase-changed",e.target.value)}),250),clearSearchPhrase:function(){this.$emit("search-phrase-changed",""),this.$nextTick((function(){this.$el.querySelector('input[type="search"]').focus()}))}}},La=wa,xa=(n("f4f7"),h(La,Ma,ka,!1,null,null,null)),Sa=xa.exports,Ta=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"listPanel__item--reviewer",class:e.currentlyAssigned?"-isAssigned":""},[n("div",{staticClass:"listPanel__itemSummary"},[n("div",{staticClass:"listPanel__itemIdentity"},[n("div",{staticClass:"listPanel__itemTitle"},[e.item.reviewsActive&&e.canSelect?n("badge",{staticClass:"listPanel__item--reviewer__active"},[e._v(" "+e._s(e.activeReviewsCountLabel.replace("{$count}",e.item.reviewsActive))+" ")]):e._e(),e._v(" "+e._s(e.item.fullName)+" "),null!==e.item.reviewerRating&&e.canSelect?n("span",{staticClass:"listPanel__item--reviewer__rating"},[e._l(e.stars,(function(e,t){return n("icon",{key:t,class:e?"listPanel__item--reviewer__star--on":"listPanel__item--reviewer__star--off",attrs:{icon:"star"}})})),n("span",{staticClass:"-screenReader"},[e._v(" "+e._s(e.reviewerRatingLabel.replace("{$rating}",e.item.reviewerRating))+" ")])],2):e._e()],1),n("div",{staticClass:"listPanel__itemSubtitle"},[e.item.affiliation||e.item.orcid?n("div",{staticClass:"listPanel__item--reviewer__affiliation"},[e._v(" "+e._s(e.localize(e.item.affiliation))+" "),e.item.orcid?n("a",{staticClass:"listPanel__item--reviewer__orcid",attrs:{href:e.item.orcid,target:"_blank"}},[n("icon",{attrs:{icon:"orcid",inline:!0}}),e._v(" "+e._s(e.item.orcid)+" ")],1):e._e()]):e._e()]),e.canSelect?n("div",{staticClass:"listPanel__item--reviewer__brief",attrs:{"aria-hidden":"true"}},[n("span",{staticClass:"listPanel__item--reviewer__complete"},[n("icon",{attrs:{icon:"check-circle-o",inline:!0}}),e._v(" "+e._s(e.item.reviewsCompleted)+" ")],1),n("span",{staticClass:"listPanel__item--reviewer__last"},[n("icon",{attrs:{icon:"history",inline:!0}}),e._v(" "+e._s(e.daysSinceLastAssignmentLabelCompiled)+" ")],1),e.item.interests.length?n("span",{staticClass:"listPanel__item--reviewer__interests"},[n("icon",{attrs:{icon:"book",inline:!0}}),e._v(" "+e._s(e.interestsString)+" ")],1):e._e()]):e._e(),e.currentlyAssigned?n("div",{staticClass:"listPanel__item--reviewer__notice"},[n("icon",{attrs:{icon:"exclamation-triangle",inline:!0}}),e._v(" "+e._s(e.currentlyAssignedLabel)+" ")],1):e.warnOnAssignment&&!e.isWarningBypassed?n("div",{staticClass:"listPanel__item--reviewer__notice"},[n("icon",{attrs:{icon:"lock",inline:!0}}),e._v(" "+e._s(e.warnOnAssignmentLabel)+" "),n("button",{staticClass:"listPanel__item--reviewer__noticeAction",on:{click:function(t){return t.preventDefault(),e.unlockAssignment(t)}}},[e._v(" "+e._s(e.warnOnAssignmentUnlockLabel)+" ")])],1):e._e()]),n("div",{staticClass:"listPanel__itemActions"},[e.canSelect?n("pkp-button",{on:{click:e.select}},[n("span",{attrs:{"aria-hidden":"true"}},[e._v(e._s(e.selectReviewerLabel))]),n("span",{staticClass:"-screenReader"},[e._v(" "+e._s(e.__("common.selectWithName",{name:e.item.fullName}))+" ")])]):e._e(),n("expander",{attrs:{isExpanded:e.isExpanded,itemName:e.item.fullName},on:{toggle:function(t){e.isExpanded=!e.isExpanded}}})],1)]),e.isExpanded?n("div",{staticClass:"listPanel__itemExpanded listPanel__itemExpanded--reviewer"},[n("list",[n("list-item",[n("template",{slot:"value"},[n("icon",{attrs:{icon:"clock-o",inline:!0}}),e._v(" "+e._s(e.item.reviewsActive)+" ")],1),e._v(" "+e._s(e.activeReviewsLabel)+" ")],2),n("list-item",[n("template",{slot:"value"},[n("icon",{attrs:{icon:"check-circle-o",inline:!0}}),e._v(" "+e._s(e.item.reviewsCompleted)+" ")],1),e._v(" "+e._s(e.completedReviewsLabel)+" ")],2),n("list-item",[n("template",{slot:"value"},[n("icon",{attrs:{icon:"times-circle-o",inline:!0}}),e._v(" "+e._s(e.item.reviewsDeclined)+" ")],1),e._v(" "+e._s(e.declinedReviewsLabel)+" ")],2),n("list-item",[n("template",{slot:"value"},[n("icon",{attrs:{icon:"ban",inline:!0}}),e._v(" "+e._s(e.item.reviewsCancelled)+" ")],1),e._v(" "+e._s(e.cancelledReviewsLabel)+" ")],2),n("list-item",[n("template",{slot:"value"},[n("icon",{attrs:{icon:"history",inline:!0}}),e._v(" "+e._s(e.daysSinceLastAssignment)+" ")],1),e._v(" "+e._s(e.daysSinceLastAssignmentDescriptionLabel)+" ")],2),n("list-item",[n("template",{slot:"value"},[n("icon",{attrs:{icon:"calendar",inline:!0}}),e._v(" "+e._s(e.item.averageReviewCompletionDays)+" ")],1),e._v(" "+e._s(e.averageCompletionLabel)+" ")],2),e.item.interests.length?n("list-item",[n("div",{staticClass:"listPanel__item--reviewer__detailHeading"},[n("icon",{attrs:{icon:"book",inline:!0}}),e._v(" "+e._s(e.reviewInterestsLabel)+" ")],1),e._v(" "+e._s(e.interestsString)+" ")]):e._e(),e.item.gossip?n("list-item",[n("div",{staticClass:"listPanel__item--reviewer__detailHeading"},[e._v(" "+e._s(e.gossipLabel)+" ")]),n("div",{domProps:{innerHTML:e._s(e.item.gossip)}})]):e._e(),e.localize(e.item.biography)?n("list-item",[n("div",{staticClass:"listPanel__item--reviewer__detailHeading"},[e._v(" "+e._s(e.biographyLabel)+" ")]),n("div",{domProps:{innerHTML:e._s(e.localize(e.item.biography))}})]):e._e()],1)],1):e._e()])},Da=[],Ya=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("button",{staticClass:"expander",on:{click:function(t){return e.$emit("toggle")}}},[e.isExpanded?n("icon",{attrs:{icon:"angle-up"}}):n("icon",{attrs:{icon:"angle-down"}}),e.isExpanded?n("span",{staticClass:"-screenReader"},[e._v(" "+e._s(e.__("common.viewLess",{name:e.itemName}))+" ")]):n("span",{staticClass:"-screenReader"},[e._v(" "+e._s(e.__("common.viewMore",{name:e.itemName}))+" ")])],1)},Ca=[],Ea={props:{isExpanded:{type:Boolean,default:function(){return!1}},itemName:{type:String,required:!0}}},Aa=Ea,Oa=(n("7b1e"),h(Aa,Ya,Ca,!1,null,null,null)),Pa=Oa.exports,Fa=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("ul",{staticClass:"list"},[e._t("default")],2)},Ia=[],ja={name:"List"},Ha=ja,Ra=(n("ff77"),h(Ha,Fa,Ia,!1,null,null,null)),Na=Ra.exports,$a=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("li",{staticClass:"listItem",class:e.classes},[e.hasValueSlot?n("span",{staticClass:"listItem__value"},[e._t("value")],2):e._e(),n("span",{staticClass:"listItem__label"},[e._t("default")],2)])},za=[],Ba={name:"List",computed:{classes:function(){var e=[];return this.hasValueSlot&&e.push("listItem--hasLabel"),e},hasValueSlot:function(){return this.$slots.value}}},Wa=Ba,Ua=(n("5d3c"),h(Wa,$a,za,!1,null,null,null)),Va=Ua.exports,qa={components:{Expander:Pa,List:Na,ListItem:Va},props:{activeReviewsCountLabel:{type:String,required:!0},activeReviewsLabel:{type:String,required:!0},averageCompletionLabel:{type:String,required:!0},biographyLabel:{type:String,required:!0},cancelledReviewsLabel:{type:String,required:!0},completedReviewsLabel:{type:String,required:!0},currentlyAssigned:{type:Boolean,required:!0},currentlyAssignedLabel:{type:String,required:!0},daySinceLastAssignmentLabel:{type:String,required:!0},daysSinceLastAssignmentLabel:{type:String,required:!0},daysSinceLastAssignmentDescriptionLabel:{type:String,required:!0},declinedReviewsLabel:{type:String,required:!0},gossipLabel:{type:String,required:!0},item:{type:Object,required:!0},neverAssignedLabel:{type:String,required:!0},reviewerRatingLabel:{type:String,required:!0},reviewInterestsLabel:{type:String,required:!0},selectReviewerLabel:{type:String,required:!0},warnOnAssignment:{type:Boolean,required:!0},warnOnAssignmentLabel:{type:String,required:!0},warnOnAssignmentUnlockLabel:{type:String,required:!0}},data:function(){return{isExpanded:!1,isWarningBypassed:!1}},computed:{canSelect:function(){return!this.currentlyAssigned&&(!this.warnOnAssignment||this.isWarningBypassed)},daysSinceLastAssignment:function(){return this.item.dateLastReviewAssignment?Math.floor((Date.parse(this.item.dateLastReviewAssignment)-Date.now())/864e5*-1):0},daysSinceLastAssignmentLabelCompiled:function(){return this.daysSinceLastAssignment?this.daysSinceLastAssignment>1?this.daysSinceLastAssignmentLabel.replace("{$days}",this.daysSinceLastAssignment):this.daySinceLastAssignmentLabel:this.neverAssignedLabel},interestsString:function(){return this.item.interests&&this.item.interests.length?this.item.interests.map((function(e){return e.interest})).join(this.__("common.commaListSeparator")):""},stars:function(){var e=[];if(this.item.reviewerRating)for(var t=0;t<5;t++)e.push(t<this.item.reviewerRating);return e}},methods:{select:function(){this.$emit("select",this.item),pkp.eventBus.$emit("selected:reviewer",this.item)},unlockAssignment:function(){this.isWarningBypassed=!0}}},Ga=qa,Ja=(n("4dea"),h(Ga,Ta,Da,!1,null,null,null)),Ka=Ja.exports,Xa={mixins:[un],props:{apiUrl:{type:String,required:!0},count:{type:Number,default:function(){return 30}},getParams:{type:Object,default:function(){return{}}},lazyLoad:{type:Boolean,default:function(){return!1}}},data:function(){return{activeFilters:{},isLoading:!1,latestGetRequest:"",offset:0,searchPhrase:""}},computed:{currentPage:function(){return Math.floor(this.offset/this.count)+1},lastPage:function(){return Math.ceil(this.itemsMax/this.count)}},methods:{get:function(){if(this.apiUrl){var e=this;this.isLoading=!0,this.latestGetRequest=$.pkp.classes.Helper.uuid(),$.ajax({url:this.apiUrl,type:"GET",data:o(o(o({},this.getParams),this.activeFilters),{},{searchPhrase:this.searchPhrase,count:this.count,offset:this.offset}),_uuid:this.latestGetRequest,error:function(t){e.latestGetRequest===this._uuid&&e.ajaxErrorCallback(t)},success:function(t){e.latestGetRequest===this._uuid&&e.setItems(t.items,t.itemsMax)},complete:function(){e.latestGetRequest===this._uuid&&(e.isLoading=!1)}})}},setItems:function(e,t){throw new Error("Missing function to set items after get request. This mixin requires that the component implements a `setItems` method which handles the items received from the get request.")},setPage:function(e){this.offset=e*this.count-this.count},setSearchPhrase:function(e){this.searchPhrase=e}},watch:{activeFilters:function(e,t){var n=this;this.offset?this.offset=0:this.$nextTick((function(){return n.get()}))},offset:function(e,t){var n=this;e!==t&&this.$nextTick((function(){return n.get()}))},searchPhrase:function(e,t){var n=this;e!==t&&(this.offset?this.offset=0:this.$nextTick((function(){return n.get()})))}},mounted:function(){if(this.lazyLoad)if("complete"===document.readyState)this.get();else{var e=this;$((function(){e.get()}))}}},Qa={components:{FilterSlider:la,FilterSliderMultirange:pa,ListPanel:He,Pagination:ya,PkpHeader:Pe,Search:Sa,SelectReviewerListItem:Ka},mixins:[Xa],props:{activeReviewsCountLabel:{type:String,required:!0},activeReviewsLabel:{type:String,required:!0},averageCompletionLabel:{type:String,required:!0},biographyLabel:{type:String,required:!0},cancelledReviewsLabel:{type:String,required:!0},completedReviewsLabel:{type:String,required:!0},currentlyAssigned:{type:Array,default:function(){return[]}},currentlyAssignedLabel:{type:String,required:!0},daySinceLastAssignmentLabel:{type:String,required:!0},daysSinceLastAssignmentLabel:{type:String,required:!0},daysSinceLastAssignmentDescriptionLabel:{type:String,required:!0},declinedReviewsLabel:{type:String,required:!0},emptyLabel:{type:String,required:!0},filters:{type:Array,default:function(){return[]}},gossipLabel:{type:String,required:!0},id:{type:String,required:!0},items:{type:Array,default:function(){return[]}},itemsMax:{type:Number,default:function(){return 0}},neverAssignedLabel:{type:String,required:!0},reviewerRatingLabel:{type:String,required:!0},reviewInterestsLabel:{type:String,required:!0},selectorName:{type:String,required:!0},selectReviewerLabel:{type:String,required:!0},title:{type:String,required:!0},warnOnAssignment:{type:Array,default:function(){return[]}},warnOnAssignmentLabel:{type:String,required:!0},warnOnAssignmentUnlockLabel:{type:String,required:!0}},data:function(){return{isLoading:!1,isSidebarVisible:!1}},methods:{addFilter:function(e,t){var n=o({},this.activeFilters);n[e]=t,this.activeFilters=n},isFilterActive:function(e,t){return this.activeFilters.hasOwnProperty(e)},removeFilter:function(e,t){if(this.activeFilters.hasOwnProperty(e)){var n=o({},this.activeFilters);delete n[e],this.activeFilters=n}},setItems:function(e,t){this.$emit("set",this.id,{items:e,itemsMax:t})}},watch:{activeFilters:function(e,t){this.offset=0,e&&Object.keys(e).length&&(this.isSidebarVisible=!0)}}},Za=Qa,eo=h(Za,Ji,Ki,!1,null,null,null),to=eo.exports,no=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"submissionsListPanel"},[n("list-panel",{staticClass:"listPanel--submissions",attrs:{isSidebarVisible:e.isSidebarVisible,items:e.items},scopedSlots:e._u([{key:"item",fn:function(t){var r=t.item;return[e._t("item",[n("submissions-list-item",{key:r.id,attrs:{item:r,apiUrl:e.apiUrl,infoUrl:e.infoUrl,assignParticipantUrl:e.assignParticipantUrl},on:{addFilter:e.addFilter}})],{item:r})]}}],null,!0)},[n("pkp-header",{attrs:{slot:"header"},slot:"header"},[n("h2",[e._v(e._s(e.title))]),e.isLoading?n("spinner"):e._e(),n("template",{slot:"actions"},[n("search",{attrs:{searchPhrase:e.searchPhrase},on:{"search-phrase-changed":e.setSearchPhrase}}),n("pkp-button",{attrs:{isActive:e.isSidebarVisible},on:{click:function(t){e.isSidebarVisible=!e.isSidebarVisible}}},[n("icon",{attrs:{icon:"filter",inline:!0}}),e._v(" "+e._s(e.__("common.filter"))+" ")],1),e.addUrl&&e.currentUserCanAddSubmission?n("pkp-button",{attrs:{element:"a",href:e.addUrl}},[e._v(" "+e._s(e.__("submission.submit.newSubmissionSingle"))+" ")]):e._e()],1)],2),n("template",{slot:"sidebar"},[n("pkp-header",{attrs:{isOneLine:!1}},[n("h3",[n("icon",{attrs:{icon:"filter",inline:!0}}),e._v(" "+e._s(e.__("common.filter"))+" ")],1)]),e._l(e.filters,(function(t,r){return n("div",{key:r,staticClass:"listPanel__block"},[t.heading?n("pkp-header",[n("h4",[e._v(e._s(t.heading))])]):e._e(),e._l(t.filters,(function(t){return n(t.filterType||"pkp-filter",e._b({key:t.param+t.value,tag:"component",attrs:{isFilterActive:e.isFilterActive(t.param,t.value)},on:{"add-filter":e.addFilter,"remove-filter":e.removeFilter,"update-filter":e.addFilter}},"component",t,!1))}))],2)}))],2),n("template",{slot:"itemsEmpty"},[e.isLoading?[n("spinner"),e._v(" "+e._s(e.__("common.loading"))+" ")]:[e._v(" "+e._s(e.__("submission.list.empty"))+" ")]],2),e.lastPage>1?n("pagination",{attrs:{slot:"footer",currentPage:e.currentPage,isLoading:e.isLoading,lastPage:e.lastPage},on:{"set-page":e.setPage},slot:"footer"}):e._e()],2)],1)},ro=[],io=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"pkpFilter--autosuggest",class:e.classes},[n(e.component,e._b({tag:"component",on:{change:e.toggle}},"component",e.autosuggestProps,!1))],1)},ao=[],oo={name:"FieldAutosuggestPreset",extends:mn,props:{options:{type:Array,required:!0}},methods:{getSuggestions:function(){var e=this;if(this.inputValue&&this.options.length){var t=new RegExp(this.inputValue.replace(/[-/\\^$*+?.()|[\]{}]/g,"\\$&"),"gi");this.suggestions=this.options.filter((function(n){if(e.currentValue.includes(n.value))return!1;var r="string"===typeof n.value?n.value.match(t):n.value==e.inputValue;return r||n.label.match(t)}))}else this.suggestions=[]}}},so=oo,lo=h(so,nn,rn,!1,null,null,null),uo=lo.exports,co={name:"FieldSelectUsers",extends:mn,methods:{setSuggestions:function(e){var t=this,n=e.filter((function(e){return!t.selected.find((function(t){return t.id===e.id}))})).map((function(e){return{value:e.id,label:e.fullName}}));this.suggestions=n}}},fo=co,ho=h(fo,an,on,!1,null,null,null),po=ho.exports,mo={name:"FieldSelectIssues",extends:mn,methods:{setSuggestions:function(e){var t=this,n=e.filter((function(e){return!t.selected.find((function(t){return t.value===e.id}))})).map((function(e){return{value:e.id,label:e.identification}}));this.suggestions=n}}},_o=mo,vo=h(_o,sn,ln,!1,null,null,null),go=vo.exports,bo={extends:ia,components:{FieldAutosuggestPreset:uo,FieldSelectUsers:po,FieldSelectIssues:go},props:{component:{type:String,required:!0},autosuggestProps:{type:Object,required:!0}},methods:{toggle:function(e,t,n){n.length?this.$emit("add-filter",this.param,n):this.remove()},remove:function(){this.$emit("remove-filter",this.param)}}},yo=bo,Mo=(n("6133"),h(yo,io,ao,!1,null,null,null)),ko=Mo.exports,wo=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"listPanel__item--submission"},[n("div",{staticClass:"listPanel__itemSummary"},[n("div",{staticClass:"listPanel__itemIdentity listPanel__itemIdentity--submission"},[n("div",{staticClass:"listPanel__item--submission__id"},[e._v(" "+e._s(e.item.id)+" ")]),n("div",{staticClass:"listPanel__itemTitle"},[e.currentUserIsReviewer?n("span",[e._v(" "+e._s(e.__("submission.list.reviewAssignment"))+" ")]):e.currentPublication.authorsStringShort?n("span",[e._v(" "+e._s(e.currentPublication.authorsStringShort)+" ")]):e._e()]),n("div",{staticClass:"listPanel__itemSubtitle"},[e._v(" "+e._s(e.localizeSubmission(e.currentPublication.fullTitle,e.currentPublication.locale))+" ")]),e.currentUserIsReviewer?n("div",{staticClass:"listPanel__item--submission__reviewDetails"},[e.currentUserLatestReviewAssignment.responsePending?n("span",{staticClass:"listPanel__item--submission__dueDate"},[e._v(" "+e._s(e.__("submission.list.responseDue",{date:e.currentUserLatestReviewAssignment.responseDue}))+" ")]):e._e(),e.currentUserLatestReviewAssignment.reviewPending?n("span",{staticClass:"listPanel__item--submission__dueDate"},[e._v(" "+e._s(e.__("submission.list.reviewDue",{date:e.currentUserLatestReviewAssignment.due}))+" ")]):e._e()]):e._e(),e.reviewerWorkflowLink?n("div",{staticClass:"listPanel__item--submission__notice"},[n("span",{domProps:{innerHTML:e._s(e.reviewerWorkflowLink)}})]):e.notice?n("div",{staticClass:"listPanel__item--submission__notice"},[n("icon",{attrs:{icon:"exclamation-triangle",inline:!0}}),e._v(" "+e._s(e.notice)+" "),e.shouldAssignEditor?n("button",{staticClass:"-linkButton",on:{click:function(t){return t.stopPropagation(),t.preventDefault(),e.openAssignParticipant(t)}}},[e._v(" "+e._s(e.__("submission.list.assignEditor"))+" ")]):e._e()],1):e._e()]),n("div",{staticClass:"listPanel__itemActions"},[e.currentUserIsReviewer?[e.currentUserLatestReviewAssignment.reviewCancelled?n("div",{staticClass:"listPanel__item--submission__reviewCancelled"},[n("icon",{attrs:{icon:"exclamation-triangle",inline:!0}}),e._v(" "+e._s(e.__("submission.list.reviewCancelled"))+" ")],1):e._e(),e.currentUserLatestReviewAssignment.reviewComplete?n("div",{staticClass:"listPanel__item--submission__reviewComplete"},[n("icon",{attrs:{icon:"check",inline:!0}}),e._v(" "+e._s(e.__("submission.list.reviewComplete"))+" ")],1):e._e()]:n("div",{staticClass:"listPanel__item--submission__itemSummaryDetails"},[n("div",{staticClass:"listPanel__item--submission__itemSummaryDetailsRow"},[n("div",{staticClass:"listPanel__item--submission__flags",attrs:{"aria-hidden":"true"}},[e.isReviewStage?n("span",[n("icon",{attrs:{icon:"user-o",inline:!0}}),e._v(" "+e._s(e.completedReviewsCount)+"/"+e._s(e.currentReviewAssignments.length)+" ")],1):e._e(),e.activeStage.files.count?n("span",[n("icon",{attrs:{icon:"file-text-o",inline:!0}}),e._v(" "+e._s(e.activeStage.files.count)+" ")],1):e._e(),e.openQueryCount?n("span",[n("icon",{attrs:{icon:"comment-o",inline:!0}}),e._v(" "+e._s(e.openQueryCount)+" ")],1):e._e()]),n("badge",{staticClass:"listPanel__item--submission__stage",attrs:{isButton:!e.isArchived,label:e.currentStageDescription,stage:e.isArchived?"":e.currentStage,isPrimary:e.isScheduled,isSuccess:e.isPublished,isWarnable:e.isDeclined},on:{click:function(t){return e.filterByStage(e.activeStage.id)}}},[e._v(" "+e._s(e.currentStageLabel)+" ")])],1)]),n("pkp-button",{attrs:{element:"a",href:e.item.urlWorkflow}},[n("span",{attrs:{"aria-hidden":"true"}},[e._v(e._s(e.__("common.view")))]),e.currentUserIsReviewer?n("span",{staticClass:"-screenReader"},[e._v(" "+e._s(e.__("common.viewWithName",{name:e.localizeSubmission(e.currentPublication.fullTitle,e.currentPublication.locale)}))+" ")]):n("span",{staticClass:"-screenReader"},[e._v(" "+e._s(e.__("common.viewWithName",{name:e.currentPublication.authorsStringShort}))+" ")])]),e.currentUserIsReviewer?e._e():n("expander",{attrs:{isExpanded:e.isExpanded,itemName:e.localize(e.currentPublication.authorsStringShort)},on:{toggle:function(t){e.isExpanded=!e.isExpanded}}})],2)]),e.isExpanded?n("div",{staticClass:"listPanel__itemExpanded listPanel__itemExpanded--submission"},[n("list",[e.isReviewStage?n("list-item",[n("template",{slot:"value"},[n("icon",{attrs:{icon:"user-o",inline:!0}}),e._v(" "+e._s(e.completedReviewsCount)+"/"+e._s(e.currentReviewAssignments.length)+" ")],1),e._v(" "+e._s(e.__("submission.list.reviewsCompleted"))+" ")],2):e._e(),e.isSubmissionStage?e._e():n("list-item",[n("template",{slot:"value"},[n("icon",{attrs:{icon:"file-text-o",inline:!0}}),e._v(" "+e._s(e.activeStage.files.count)+" ")],1),e._v(" "+e._s(e.activeStageFilesLabel)+" ")],2),e.item.submissionProgress?e._e():n("list-item",[n("template",{slot:"value"},[n("icon",{attrs:{icon:"comment-o",inline:!0}}),e._v(" "+e._s(e.openQueryCount)+" ")],1),e._v(" "+e._s(e.__("submission.list.discussions"))+" ")],2),e.dualWorkflowLinks?n("list-item",[n("span",{domProps:{innerHTML:e._s(e.dualWorkflowLinks)}})]):e._e(),n("list-item",[n("span",[e._v(" "+e._s(e.__("common.lastActivity",{date:e.localizeDate(e.item.dateLastActivity)}))+" ")])])],1),n("div",{staticClass:"listPanel__itemExpandedActions"},[e.currentUserCanViewInfoCenter?n("pkp-button",{on:{click:e.openInfoCenter}},[e._v(" "+e._s(e.__("submission.list.infoCenter"))+" ")]):e._e(),e.currentUserCanDelete?n("pkp-button",{attrs:{isWarnable:!0},on:{click:e.deleteSubmissionPrompt}},[e._v(" "+e._s(e.__("common.delete"))+" ")]):e._e()],1)],1):e._e()])},Lo=[];n("13d5"),n("45fc");function xo(e,t){var n;if("undefined"===typeof Symbol||null==e[Symbol.iterator]){if(Array.isArray(e)||(n=ge(e))||t&&e&&"number"===typeof e.length){n&&(e=n);var r=0,i=function(){};return{s:i,n:function(){return r>=e.length?{done:!0}:{done:!1,value:e[r++]}},e:function(e){throw e},f:i}}throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}var a,o=!0,s=!1;return{s:function(){n=e[Symbol.iterator]()},n:function(){var e=n.next();return o=e.done,e},e:function(e){s=!0,a=e},f:function(){try{o||null==n["return"]||n["return"]()}finally{if(s)throw a}}}}var So,To,Do,Yo,Co,Eo,Ao,Oo,Po,Fo,Io,jo,Ho,Ro,No,$o,zo,Bo,Wo,Uo,Vo,qo,Go,Jo,Ko={methods:{localizeSubmission:function(e,t){if("undefined"===typeof e)return"";if(e.hasOwnProperty($.pkp.app.currentLocale)&&e[$.pkp.app.currentLocale])return e[$.pkp.app.currentLocale];if(e.hasOwnProperty(t)&&e[t])return e[t];if(e.hasOwnProperty($.pkp.app.primaryLocale)&&e[$.pkp.app.primaryLocale])return e[$.pkp.app.primaryLocale];for(var n in e)if(e[n])return e[n];return""}}},Xo={name:"SubmissionsListItem",mixins:[un,Xa,Ko,k],components:{Expander:Pa,List:Na,ListItem:Va},props:{apiUrl:{type:String,required:!0},assignParticipantUrl:{type:String,required:!0},infoUrl:{type:String,required:!0},item:{type:Object,required:!0}},data:function(){return{isExpanded:!1,mask:null,noticeActions:[],noticeActionLabels:[]}},computed:{currentPublication:function(){var e=this;return this.item.publications.find((function(t){return t.id===e.item.currentPublicationId}))},currentUserCanDelete:function(){return!(this.userAssignedRole(pkp.const.ROLE_ID_AUTHOR)||!this.userAssignedRole([pkp.const.ROLE_ID_MANAGER,pkp.const.ROLE_ID_SITE_ADMIN])||this.item.status!==pkp.const.STATUS_DECLINED)||!(!this.userAssignedRole(pkp.const.ROLE_ID_AUTHOR)||0===this.item.submissionProgress)},currentUserCanViewInfoCenter:function(){return this.userAssignedRole([pkp.const.ROLE_ID_SITE_ADMIN,pkp.const.ROLE_ID_MANAGER,pkp.const.ROLE_ID_SUB_EDITOR])},currentUserIsReviewer:function(){var e,t=xo(this.item.reviewAssignments);try{for(t.s();!(e=t.n()).done;){var n=e.value;if(n.isCurrentUserAssigned)return!0}}catch(r){t.e(r)}finally{t.f()}return!1},activeStage:function(){return this.item.stages.find((function(e){return!0===e.isActiveStage}))},notice:function(){var e="";if(this.item.submissionProgress>0)return e;if(this.shouldAssignEditor&&(e=this.activeStage.status),this.userAssignedRole([pkp.const.ROLE_ID_MANAGER,pkp.const.ROLE_ID_SUB_EDITOR])&&this.isReviewStage){switch(this.activeStage.statusId){case pkp.const.REVIEW_ROUND_STATUS_PENDING_REVIEWERS:case pkp.const.REVIEW_ROUND_STATUS_REVIEWS_READY:case pkp.const.REVIEW_ROUND_STATUS_REVIEWS_COMPLETED:case pkp.const.REVIEW_ROUND_STATUS_REVIEWS_OVERDUE:case pkp.const.REVIEW_ROUND_STATUS_REVISIONS_SUBMITTED:case pkp.const.REVIEW_ROUND_STATUS_RESUBMIT_FOR_REVIEW_SUBMITTED:e=this.activeStage.status;break}if(!this.activeStage.currentUserCanRecommendOnly)switch(this.activeStage.statusId){case pkp.const.REVIEW_ROUND_STATUS_RECOMMENDATIONS_READY:case pkp.const.REVIEW_ROUND_STATUS_RECOMMENDATIONS_COMPLETED:e=this.activeStage.status;break}}if(this.userAssignedRole(pkp.const.ROLE_ID_AUTHOR)&&this.isReviewStage)switch(this.activeStage.statusId){case pkp.const.REVIEW_ROUND_STATUS_REVISIONS_REQUESTED:case pkp.const.REVIEW_ROUND_STATUS_RESUBMIT_FOR_REVIEW:e=this.activeStage.status;break}if(this.currentUserIsReviewer)switch(this.currentUserLatestReviewAssignment.statusId){case pkp.const.REVIEW_ASSIGNMENT_STATUS_AWAITING_RESPONSE:case pkp.const.REVIEW_ASSIGNMENT_STATUS_RESPONSE_OVERDUE:case pkp.const.REVIEW_ASSIGNMENT_STATUS_REVIEW_OVERDUE:e=this.currentUserLatestReviewAssignment.status;break}return e},shouldAssignEditor:function(){return this.userAssignedRole(pkp.const.ROLE_ID_MANAGER)&&this.activeStage.id===pkp.const.WORKFLOW_STAGE_ID_SUBMISSION&&this.activeStage.statusId===pkp.const.STAGE_STATUS_SUBMISSION_UNASSIGNED},currentStage:function(){switch(this.activeStage.id){case pkp.const.WORKFLOW_STAGE_ID_SUBMISSION:return"submission";case pkp.const.WORKFLOW_STAGE_ID_INTERNAL_REVIEW:case pkp.const.WORKFLOW_STAGE_ID_EXTERNAL_REVIEW:return"review";case pkp.const.WORKFLOW_STAGE_ID_EDITING:return"copyediting";case pkp.const.WORKFLOW_STAGE_ID_PRODUCTION:return"production"}return""},currentStageLabel:function(){return this.isArchived?this.item.statusLabel:this.item.submissionProgress>0?this.__("submissions.incomplete"):this.activeStage.label},currentStageDescription:function(){return this.__("submission.list.currentStage",{stage:this.currentStageLabel})},isArchived:function(){return this.item.status===pkp.const.STATUS_SCHEDULED||this.item.status===pkp.const.STATUS_PUBLISHED||this.item.status===pkp.const.STATUS_DECLINED},isDeclined:function(){return this.item.status===pkp.const.STATUS_DECLINED},isScheduled:function(){return this.item.status===pkp.const.STATUS_SCHEDULED},isPublished:function(){return this.item.status===pkp.const.STATUS_PUBLISHED},openQueryCount:function(){return this.activeStage.queries.filter((function(e){return!e.closed})).length},activeStageFilesLabel:function(){switch(this.activeStage.id){case pkp.const.WORKFLOW_STAGE_ID_INTERNAL_REVIEW:case pkp.const.WORKFLOW_STAGE_ID_EXTERNAL_REVIEW:return this.__("submission.list.revisionsSubmitted");case pkp.const.WORKFLOW_STAGE_ID_EDITING:return this.__("submission.list.copyeditsSubmitted");case pkp.const.WORKFLOW_STAGE_ID_PRODUCTION:return this.__("submission.list.galleysCreated")}return""},isSubmissionStage:function(){return this.activeStage.id===pkp.const.WORKFLOW_STAGE_ID_SUBMISSION},isReviewStage:function(){return this.activeStage.id===pkp.const.WORKFLOW_STAGE_ID_INTERNAL_REVIEW||this.activeStage.id===pkp.const.WORKFLOW_STAGE_ID_EXTERNAL_REVIEW},currentRoleAssignments:function(){var e=[];return this.item.stages.forEach((function(t){t.currentUserAssignedRoles.forEach((function(t){-1===e.indexOf(t)&&e.push(t)}))})),e},dualWorkflowLinks:function(){return this.userAssignedRole(pkp.const.ROLE_ID_AUTHOR)&&this.userAssignedRole([pkp.const.ROLE_ID_MANAGER,pkp.const.ROLE_ID_SUB_EDITOR,pkp.const.ROLE_ID_ASSISTANT])?this.__("submission.list.dualWorkflowLinks",{urlAuthorWorkflow:this.item.urlAuthorWorkflow,urlEditorialWorkflow:this.item.urlEditorialWorkflow}):""},reviewerWorkflowLink:function(){return this.currentUserIsReviewer&&this.userAssignedRole([pkp.const.ROLE_ID_MANAGER,pkp.const.ROLE_ID_SUB_EDITOR,pkp.const.ROLE_ID_ASSISTANT])?this.__("submission.list.reviewerWorkflowLink",{urlEditorialWorkflow:this.item.urlEditorialWorkflow}):""},currentReviewAssignments:function(){if(!this.item.reviewRounds.length||!this.item.reviewAssignments.length)return[];var e=this.item.reviewRounds[this.item.reviewRounds.length-1].id;return this.item.reviewAssignments.filter((function(t){return t.roundId===e}))},currentUserLatestReviewAssignment:function(){if(!this.currentUserIsReviewer)return!1;var e=this.item.reviewAssignments.filter((function(e){return!0===e.isCurrentUserAssigned}));if(!e.length)return!1;var t=e.reduce((function(e,t){return e.round>t.round?e:t}));switch(t.statusId){case pkp.const.REVIEW_ASSIGNMENT_STATUS_AWAITING_RESPONSE:case pkp.const.REVIEW_ASSIGNMENT_STATUS_RESPONSE_OVERDUE:t.responsePending=!0,t.reviewPending=!0;break;case pkp.const.REVIEW_ASSIGNMENT_STATUS_ACCEPTED:case pkp.const.REVIEW_ASSIGNMENT_STATUS_REVIEW_OVERDUE:t.reviewPending=!0;break;case pkp.const.REVIEW_ASSIGNMENT_STATUS_RECEIVED:case pkp.const.REVIEW_ASSIGNMENT_STATUS_COMPLETE:case pkp.const.REVIEW_ASSIGNMENT_STATUS_THANKED:t.reviewComplete=!0;break}return t},completedReviewsCount:function(){return this.isReviewStage?this.currentReviewAssignments.filter((function(e){return e.statusId>=pkp.const.REVIEW_ASSIGNMENT_STATUS_RECEIVED})).length:0}},methods:{userAssignedRole:function(e){var t=this;if(Array.isArray(e)||(e=[e]),this.currentRoleAssignments.length)return e.some((function(e){return t.currentRoleAssignments.includes(e)}));var n=e.filter((function(e){return e===pkp.const.ROLE_ID_SITE_ADMIN||e===pkp.const.ROLE_ID_MANAGER}));return!!n.length&&this.userHasRole(n)},filterByStage:function(e){this.$emit("addFilter","stageIds",e)},openInfoCenter:function(){var e={title:this.localizeSubmission(this.currentPublication.fullTitle,this.currentPublication.locale),url:this.infoUrl.replace("__id__",this.item.id),closeCallback:this.resetFocusInfoCenter};$('<div id="'+$.pkp.classes.Helper.uuid()+'" class="pkp_modal pkpModalWrapper" tabindex="-1"></div>').pkpHandler("$.pkp.controllers.modal.AjaxModalHandler",e)},resetFocusInfoCenter:function(){this.$el.querySelector(".listPanel__item__openInfoCenter").focus()},openAssignParticipant:function(){var e={title:this.__("submission.list.assignEditor"),url:this.assignParticipantUrl.replace("__id__",this.item.id).replace("__stageId__",this.activeStage.id),closeCallback:this.resetFocusAssignParticipant};$('<div id="'+$.pkp.classes.Helper.uuid()+'" class="pkp_modal pkpModalWrapper" tabIndex="-1"></div>').pkpHandler("$.pkp.controllers.modal.AjaxModalHandler",e)},resetFocusAssignParticipant:function(){this.$el.querySelector(".listPanel__item--submission__notice button").focus()},deleteSubmissionPrompt:function(){this.openDialog({cancelLabel:this.__("common.no"),modalName:"deleteSubmission",message:this.__("editor.submissionArchive.confirmDelete"),title:this.__("common.delete"),callback:this.deleteSubmission})},deleteSubmission:function(){var e=this;$.ajax({url:this.apiUrl+"/"+this.item.id,type:"POST",headers:{"X-Csrf-Token":pkp.currentUser.csrfToken,"X-Http-Method-Override":"DELETE"},error:this.ajaxErrorCallback,success:function(){pkp.eventBus.$emit("deleted:submission",{id:e.item.id})},complete:function(){e.$modal.hide("deleteSubmission")}})},userHasRole:function(e){Array.isArray(e)||(e=[e]);var t=!1;return e.forEach((function(e){pkp.currentUser.roles.indexOf(e)>-1&&(t=!0)})),t}}},Qo=Xo,Zo=(n("544b"),h(Qo,wo,Lo,!1,null,null,null)),es=Zo.exports,ts={mixins:[Xa],components:{ListPanel:He,Pagination:ya,PkpFilter:ia,PkpFilterSlider:la,PkpFilterAutosuggest:ko,PkpHeader:Pe,Search:Sa,SubmissionsListItem:es},props:{addUrl:{type:String,required:!0},assignParticipantUrl:{type:String,default:function(){return""}},filters:{type:Array,default:function(){return[]}},id:{type:String,required:!0},infoUrl:{type:String,required:!0},items:{type:Array,default:function(){return[]}},itemsMax:{type:Number,defaut:function(){return 0}},title:{type:String,required:!0},allowSubmissions:{type:Boolean,default:function(){return!0}}},data:function(){return{isSidebarVisible:!1}},computed:{currentUserCanFilter:function(){return this.userHasRole([pkp.const.ROLE_ID_MANAGER,pkp.const.ROLE_ID_SUB_EDITOR,pkp.const.ROLE_ID_ASSISTANT])},currentUserCanAddSubmission:function(){return this.allowSubmissions&&this.userHasRole([pkp.const.ROLE_ID_MANAGER,pkp.const.ROLE_ID_SUB_EDITOR,pkp.const.ROLE_ID_ASSISTANT,pkp.const.ROLE_ID_AUTHOR,pkp.const.ROLE_ID_REVIEWER])}},methods:{addFilter:function(e,t){if("isIncomplete"===e)this.activeFilters={isIncomplete:t},this.get();else{var n=o({},this.activeFilters);n.hasOwnProperty("isIncomplete")&&delete n.isIncomplete,["isOverdue","daysInactive","assignedTo","issueIds","sectionIds","assignedTo"].includes(e)?n[e]=t:(n[e]||(n[e]=[]),n[e].push(t)),this.activeFilters=n}},isFilterActive:function(e,t){return!!Object.keys(this.activeFilters).includes(e)&&(Array.isArray(this.activeFilters[e])?this.activeFilters[e].includes(t):!!["isOverdue","daysInactive"].includes(e)||this.activeFilters[e]===t)},removeFilter:function(e,t){var n=o({},this.activeFilters);["isIncomplete","isOverdue","daysInactive","issueIds","sectionIds","assignedTo"].includes(e)?delete n[e]:n[e]=n[e].filter((function(e){return e!==t})),this.activeFilters=n},setItems:function(e,t){this.$emit("set",this.id,{items:e,itemsMax:t})},userHasRole:function(e){Array.isArray(e)||(e=[e]);var t=!1;return e.forEach((function(e){pkp.currentUser.roles.indexOf(e)>-1&&(t=!0)})),t}},mounted:function(){var e=this;pkp.eventBus.$on("updated:submission",(function(){return e.get()})),pkp.eventBus.$on("deleted:submission",(function(t){t.id&&e.items.find((function(e){return e.id===t.id}))&&(e.items=e.items.filter((function(e){return t.id!==e.id})),e.itemsMax--)}))},destroyed:function(){pkp.eventBus.$off("updated:submission"),pkp.eventBus.$off("deleted:submission")}},ns=ts,rs=h(ns,no,ro,!1,null,null,null),is=rs.exports,as=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"submissionFilesListPanel",attrs:{id:e.controlId}},[n("list-panel",{staticClass:"listPanel--submissionFiles",attrs:{items:e.items},scopedSlots:e._u([{key:"item",fn:function(t){var r=t.item;return[e._t("item",[n("submission-files-list-item",{attrs:{apiUrl:e.apiUrl,cancelUploadLabel:e.cancelUploadLabel,genrePromptLabel:e.genrePromptLabel,documentTypes:e.documentTypes,fileStage:e.fileStage,genres:e.genres,item:r,otherLabel:e.otherLabel,stageId:e.stageId},on:{cancel:e.dropzoneCancelUpload,edit:e.edit,remove:e.remove,update:e.updateItem}})],{item:r})]}}],null,!0)},[n("pkp-header",{attrs:{slot:"header"},slot:"header"},[n("h2",[e._v(e._s(e.title))]),n("template",{slot:"actions"},[n("pkp-button",{ref:"addFileButton",on:{click:e.openFileBrowser}},[e._v(" "+e._s(e.addFileLabel)+" ")])],1)],2),n("template",{slot:"itemsEmpty"},[e._v(" "+e._s(e.emptyLabel)+" "),n("button",{staticClass:"-linkButton",on:{click:e.openFileBrowser}},[e._v(" "+e._s(e.emptyAddLabel)+" ")])])],2),n("vue-dropzone",{ref:"dropzone",class:{"-isDragging":e.isDragging},attrs:{id:e.dropzoneId,options:e.dropzoneOptions},on:{"vdropzone-error":e.dropzoneError,"vdropzone-files-added":e.dropzoneFilesAdded,"vdropzone-removed-file":e.dropzoneRemovedFile,"vdropzone-success":e.dropzoneSuccess,"vdropzone-upload-progress":e.dropzoneUploadProgress,"vdropzone-total-upload-progress":e.dropzoneTotalUploadProgress,"vdropzone-drop":e.drop}}),e.status?n("span",{staticClass:"-screenReader",attrs:{role:"status"}},[e._v(" "+e._s(e.status)+" ")]):e._e(),n("modal",e._b({attrs:{name:"form"}},"modal",e.MODAL_PROPS,!1),[n("modal-content",{attrs:{closeLabel:e.__("common.close"),modalName:"form",title:e.editingLabel}},[n("pkp-form",e._b({on:{set:e.setForm,success:e.formSuccess}},"pkp-form",e.activeForm,!1))],1)],1)],1)},os=[],ss=(n("b680"),function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"listPanel__item--submissionFile"},[e.item.fileId?[n("div",{staticClass:"listPanel__itemSummary"},[n("div",{staticClass:"listPanel__itemIdentity"},[n("div",{staticClass:"listPanel__itemSubtitle"},[n("a",{staticClass:"listPanel__item--submissionFile__link",attrs:{href:e.item.url,id:e.nameId}},[n("icon",{staticClass:"listPanel--submissionFiles__itemDocumentType",attrs:{icon:e.getIcon(e.item),inline:!0}}),e._v(" "+e._s(e.localize(e.item.name))+" ")],1)])]),n("div",{staticClass:"listPanel__itemActions"},[e.getGenre(e.item.genreId)?n("badge",{staticClass:"listPanel--submissionFiles__itemGenre",attrs:{"is-primary":e.getGenre(e.item.genreId).isPrimary}},[e._v(" "+e._s(e.getGenre(e.item.genreId).name)+" ")]):e._e(),n("pkp-button",{attrs:{id:e.editId,"aria-describedby":e.nameId},on:{click:function(t){return e.$emit("edit",e.item)}}},[e._v(" "+e._s(e.__("common.edit"))+" ")]),n("pkp-button",{attrs:{"is-warnable":!0,"aria-describedby":e.nameId},on:{click:function(t){return e.$emit("remove",e.item)}}},[e._v(" "+e._s(e.__("common.remove"))+" ")])],1)]),e.item.genreId?e._e():n("div",{staticClass:"listPanel--submissionFiles__setGenre"},[n("span",{attrs:{role:"alert"}},[n("icon",{attrs:{icon:"exclamation-triangle",inline:!0}}),n("span",{staticClass:"listPanel--submissionFiles__setGenreLabel",attrs:{id:e.genrePromptId}},[e._v(" "+e._s(e.genrePromptLabel)+" ")]),e._l(e.primaryGenres,(function(t){return n("button",{key:t.id,staticClass:"-linkButton listPanel--submissionFiles__setGenreButton",attrs:{"aria-describedby":e.genrePromptId+" "+e.nameId},on:{click:function(n){return e.setGenre(t.id)}}},[e._v(" "+e._s(t.name)+" ")])})),n("button",{staticClass:"-linkButton listPanel--submissionFiles__setGenreButton",attrs:{"aria-describedby":e.genrePromptId+" "+e.nameId},on:{click:function(t){return e.$emit("edit",e.item)}}},[e._v(" "+e._s(e.otherLabel)+" ")])],2),e.isSavingGenreId?[n("spinner",{staticClass:"listPanel--submissionFiles__genreSpinner"}),n("span",{staticClass:"-screenReader",attrs:{role:"status","aria-live":"polite","aria-atomic":"true"}},[e._v(" "+e._s(e.status)+" ")])]:e._e()],2)]:[n("div",{staticClass:"listPanel__itemSummary"},[n("div",{staticClass:"listPanel__itemIdentity"},[n("div",{staticClass:"listPanel__itemSubtitle"},[e._v(" "+e._s(e.item.name)+" ")])]),n("div",{staticClass:"listPanel__itemActions"},[n("pkp-button",{attrs:{"is-warnable":!0,"aria-describedby":e.nameId},on:{click:function(t){return e.$emit("cancel",e.item.uuid)}}},[e._v(" "+e._s(e.cancelUploadLabel)+" ")])],1)]),e.item.error?n("field-error",{attrs:{messages:[e.item.error]}}):n("progress-bar",{attrs:{value:e.item.progress}})]],2)}),ls=[],us=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"progressBar"},[n("div",{staticClass:"progressBar__progress",style:{width:e.progress+"%"},attrs:{"aria-valuenow":e.value,"aria-valuemin":e.min,"aria-valuemax":e.max,role:"progressbar"}})])},cs=[],ds={props:{max:{type:Number,default:function(){return 100}},min:{type:Number,default:function(){return 100}},value:{type:Number,required:!0}},computed:{progress:function(){return(this.value/this.max*100).toFixed(0)}}},fs=ds,hs=(n("69e7"),h(fs,us,cs,!1,null,null,null)),ps=hs.exports,ms={components:{Badge:P,FieldError:ht,ProgressBar:ps},props:{apiUrl:{type:String,required:!0},cancelUploadLabel:{type:String,required:!0},documentTypes:{type:Object,required:!0},genrePromptLabel:{type:String,required:!0},genres:{type:Array,required:!0},item:{type:Object,required:!0},otherLabel:{type:String,required:!0},stageId:{type:Number,required:!0}},data:function(){return{isSavingGenreId:!1}},computed:{editId:function(){return this.compileId("edit")},genrePromptId:function(){return this.compileId("setGenre")},nameId:function(){return this.compileId("name")},primaryGenres:function(){return this.genres.filter((function(e){return!!e.isPrimary}))}},methods:{compileId:function(e){var t=this.item.id||this.item.uuid;return[e,t].join("-")},getGenre:function(e){return this.genres.find((function(t){return t.id===e}))},getIcon:function(e){switch(e.documentType){case this.documentTypes.DOCUMENT_TYPE_EXCEL:return"file-excel-o";case this.documentTypes.DOCUMENT_TYPE_HTML:return"file-code-o";case this.documentTypes.DOCUMENT_TYPE_IMAGE:return"file-image-o";case this.documentTypes.DOCUMENT_TYPE_PDF:return"file-pdf-o";case this.documentTypes.DOCUMENT_TYPE_WORD:return"file-word-o";case this.documentTypes.DOCUMENT_TYPE_EPUB:return"file-text-o";case this.documentTypes.DOCUMENT_TYPE_ZIP:return"file-archive-o";case this.documentTypes.DOCUMENT_TYPE_AUDIO:return"file-audio-o";case this.documentTypes.DOCUMENT_TYPE_VIDEO:return"file-video-o";default:return"file-o"}},setGenre:function(e){this.isSavingGenreId=!0,this.status=this.__("common.saving");var t=this;$.ajax({url:this.apiUrl+"/"+this.item.id+"?stageId="+this.stageId,type:"PUT",data:{genreId:e},headers:{"X-Csrf-Token":pkp.currentUser.csrfToken},error:this.ajaxErrorCallback,success:function(e){t.status=t.__("form.saved"),setTimeout((function(){t.$emit("update",e),t.$el.querySelector("#"+t.editId).focus()}),500)},complete:function(e){setTimeout((function(){return t.isSavingGenreId=!1}),500)}})}}},_s=ms,vs=(n("67fd"),h(_s,ss,ls,!1,null,null,null)),gs=vs.exports,bs=n("9675"),ys=n.n(bs),Ms={mixins:[k],components:{ListPanel:He,PkpForm:Gi,PkpHeader:Pe,SubmissionFilesListItem:gs,VueDropzone:Si.a},props:{addFileLabel:{type:String,required:!0},apiUrl:{type:String,required:!0},cancelUploadLabel:{type:String,required:!0},documentTypes:{type:Object,required:!0},emptyLabel:{type:String,required:!0},emptyAddLabel:{type:String,required:!0},fileStage:{type:Number,required:!0},form:{type:Object},genrePromptLabel:{type:String,required:!0},genres:{type:Array,required:!0},id:{type:String,required:!0},items:{type:Array,default:function(){return[]}},options:{type:Object,default:function(){return{}}},otherLabel:{type:String,required:!0},removeConfirmLabel:{type:String,required:!0},stageId:{type:Number,required:!0},title:{type:String,required:!0},uploadProgressLabel:{type:String,required:!0}},data:function(){return{activeForm:{},dragEventCounter:0,editingLabel:"",isDragging:!1,status:""}},computed:{controlId:function(){return this.compileId("control")},dropzoneId:function(){return this.compileId("dropzone")},dropzoneOptions:function(){var e=this;return o({method:"POST",url:this.apiUrl,thumbnailWidth:240,hiddenInputContainer:"#"+this.controlId,addRemoveLinks:!0,previewTemplate:"<p></p>",headers:{"X-Csrf-Token":pkp.currentUser.csrfToken},params:function(t,n){var r="name["+$.pkp.app.primaryLocale+"]";return i({fileStage:e.fileStage},r,t[0].name)}},this.options)}},methods:{compileId:function(e){return[this.id,e].join("-")},dragenter:function(e){this.dragEventCounter=this.dragEventCounter+1,this.isDragging=e.dataTransfer.types.includes("Files")},dragleave:function(e){this.dragEventCounter=this.dragEventCounter-1,this.isDragging=this.dragEventCounter>0},drop:function(e){e.preventDefault(),"drop"===e.type&&(this.isDragging=!1,this.dragEventCounter=0)},dropzoneCancelUpload:function(e){var t=this.$refs.dropzone.dropzone.files.find((function(t){return t.upload.uuid===e}));t&&this.$refs.dropzone.removeFile(t)},dropzoneError:function(e,t){var n=this;t="string"===typeof t?t:t.errorMessage,this.$nextTick((function(){n.$nextTick((function(){n.$nextTick((function(){var r=n.items.map((function(n){return n.uuid&&n.uuid===e.upload.uuid&&(n.error=t),n}));n.$emit("set",n.id,{items:ye(r)})}))}))}))},dropzoneFilesAdded:function(e){var t=this;this.$nextTick((function(){var n=Array.from(e).map((function(e){return{uuid:e.upload.uuid,name:e.upload.filename,progress:e.upload.progress,error:""}})),r=t.items.concat(n);t.$emit("set",t.id,{items:r})}))},dropzoneRemovedFile:function(e){var t=this.items.filter((function(t){return!t.uuid||t.uuid!==e.upload.uuid}));this.$emit("set",this.id,{items:t})},dropzoneSuccess:function(e,t){var n=this.items.map((function(n){return n.uuid===e.upload.uuid?t:n}));this.$emit("set",this.id,{items:n})},dropzoneTotalUploadProgress:function(e){this.status=this.uploadProgressLabel.replace("{$percent}",e.toFixed(0))},dropzoneUploadProgress:function(e,t){var n=this.items.map((function(n){return n.uuid===e.upload.uuid&&(n.progress=t),n}));this.$emit("set",this.id,{items:n})},edit:function(e){var t=o({},ys()(this.form));t.action=t.action+"/"+e.id+"?stageId="+this.stageId,t.fields=t.fields.map((function(t){return null!=e[t.name]&&(t.value=e[t.name]),t})),this.activeForm=t,this.editingLabel=this.__("common.editItem",{name:this.localize(e.name)}),this.$modal.show("form")},formSuccess:function(e){this.updateItem(e),this.$modal.hide("form"),this.activeForm={},this.$el.querySelector("#edit-"+e.id).focus()},openFileBrowser:function(){this.$refs.dropzone.dropzone.hiddenFileInput.click()},remove:function(e){var t=this;this.openDialog({cancelLabel:this.__("common.no"),modalName:"remove",message:this.removeConfirmLabel,title:this.__("common.remove"),callback:function(){var n=t;$.ajax({url:t.apiUrl+"/"+e.id+"?stageId="+t.stageId,type:"POST",headers:{"X-Csrf-Token":pkp.currentUser.csrfToken,"X-Http-Method-Override":"DELETE"},error:n.ajaxErrorCallback,success:function(e){var t=n.items.filter((function(t){return t.id!==e.id}));n.$emit("set",n.id,{items:t}),n.$modal.hide("remove"),n.$refs.addFileButton.$el.focus()}})}})},setForm:function(e,t){var n=o({},this.activeForm);Object.keys(t).forEach((function(e){n[e]=t[e]})),this.activeForm=n},updateItem:function(e){var t=this.items.map((function(t){return t.id===e.id?e:t}));this.$emit("set",this.id,{items:t})}},mounted:function(){document.addEventListener("dragenter",this.dragenter,!0),document.addEventListener("dragleave",this.dragleave,!0),document.addEventListener("dragover",this.drop,!0),document.addEventListener("drop",this.drop)},destroyed:function(){document.removeEventListener("dragenter",this.dragenter,!0),document.removeEventListener("dragleave",this.dragleave,!0),document.removeEventListener("dragover",this.drop,!0),document.removeEventListener("drop",this.drop)}},ks=Ms,ws=(n("23bb"),h(ks,as,os,!1,null,null,null)),Ls=ws.exports,xs={name:"Container",components:{ListPanel:He,PkpForm:Gi,SelectReviewerListPanel:to,SubmissionsListPanel:is,SubmissionFilesListPanel:Ls},data:function(){return{components:{}}},methods:{get:function(e){return this.components[e]?this.components[e]:{}},set:function(e,t){var n=o({},this.get(e));Object.keys(t).forEach((function(e){n[e]=t[e]})),this.components[e]=n}},mounted:function(){var e=this;pkp.eventBus.$on("set-form-languages",(function(t){var n;Array.isArray(t)?n=ye(t):(n=o({},t),n.isGlobalEvent&&delete n.isGlobalEvent,n.handler&&delete n.handler,n=Object.values(n)),Object.keys(e.components).forEach((function(t){"undefined"!==typeof e.components[t].supportedFormLocales&&(e.components[t].supportedFormLocales=n)}))}))},destroyed:function(){pkp.eventBus.$off("set-form-languages")}},Ss=xs,Ts=(n("481b"),h(Ss,So,To,!1,null,null,null)),Ds=Ts.exports,Ys=(n("1276"),function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"pkpDropdown"},[n("pkp-button",{ref:"button",attrs:{isActive:e.isActive,isLink:e.isLink,isPrimary:e.isPrimary,isWarnable:e.isWarnable,"aria-describedby":e.ariaDescribedBy,"aria-controls":e.id,"aria-expanded":e.isOpen},on:{click:e.toggle,blur:e.closeOnBlur}},[e._t("button",[e.icon?n("icon",{attrs:{icon:e.icon,inline:!0}}):e._e(),e._v(" "+e._s(e.label)+" ")])],2),e.isOpen?n("div",{staticClass:"pkpDropdown__content",attrs:{id:e.id},on:{click:e.preserveFocus}},[e._t("default")],2):e._e(),e.isOpen?n("span",{staticClass:"pkpDropdown__caret"}):e._e()],1)}),Cs=[],Es={props:{ariaDescribedBy:{type:String,default:""},icon:{type:String,default:""},label:{type:String,default:""},isActive:{type:Boolean,default:!1},isLink:{type:Boolean,default:!1},isPrimary:{type:Boolean,default:!1},isWarnable:{type:Boolean,default:!1}},data:function(){return{isOpen:!1}},computed:{id:function(){return this._uid}},methods:{preserveFocus:function(e){document.activeElement===document.body&&this.$refs.button.$el.focus()},toggle:function(){this.isOpen=!this.isOpen},closeOnBlur:function(){var e=this;setTimeout((function(){if(e.$el.contains(document.activeElement))var t=setInterval((function(){e.$el.contains(document.activeElement)||(e.isOpen=!1,window.clearInterval(t))}),1e3);else e.isOpen=!1}),100)}}},As=Es,Os=(n("f167"),h(As,Ys,Cs,!1,null,null,null)),Ps=Os.exports,Fs={extends:Ds,name:"Page",mixins:[k],components:{Dropdown:Ps,Notification:De,PkpHeader:Pe},data:function(){return{breadcrumbs:[],menu:[],notifications:[],notificationInterval:null,tasksUrl:"",unreadTasksCount:0}},computed:{backToDashboardLink:function(){return this.useBackToDashboard?this.menu[Object.keys(this.menu)[0]]:null},backToDashboardLabel:function(){return this.backToDashboardLink?this.__("navigation.backTo",{page:this.backToDashboardLink.name}):null},useBackToDashboard:function(){return!!this.menu&&1===Object.keys(this.menu).length&&!this.menu[Object.keys(this.menu)[0]].isCurrent}},methods:{dismissNotification:function(e){this.notifications=this.notifications.filter((function(t){return t.key!==e}))},openUrlTab:function(){var e=window.location.hash.slice(1).split("/");while(e.length)pkp.eventBus.$emit("open-tab",e.shift())},openTasks:function(){var e={url:this.tasksUrl,closeCallback:this.$refs.tasksButton.focus()};$('<div id="'+$.pkp.classes.Helper.uuid()+'" class="pkp_modal pkpModalWrapper" tabindex="-1"></div>').pkpHandler("$.pkp.controllers.modal.AjaxModalHandler",e)}},mounted:function(){var e=this;window.location.hash&&this.openUrlTab(),window.onhashchange=this.openUrlTab;var t=0;pkp.eventBus.$on("notify",(function(n,r){e.notifications.push({key:t,message:n,type:null!==r&&void 0!==r?r:"notice",expire:Date.now()+5e3}),t++})),this.notificationInterval=setInterval((function(){e.notifications.length&&(e.$refs.notifications.matches(":hover")||(e.notifications=e.notifications.filter((function(e){return e.expire>Date.now()}))))}),250),pkp.eventBus.$on("clear-all-notify",(function(){return e.notifications=[]})),pkp.eventBus.$on("update:unread-tasks-count",(function(t){return e.unreadTasksCount=t.count}))},destroyed:function(){pkp.eventBus.$off("notify"),pkp.eventBus.$off("clear-all-notify"),clearInterval(this.notificationInterval),pkp.eventBus.$off("update:unread-tasks-count")}},Is=Fs,js=(n("3fed"),h(Is,Do,Yo,!1,null,null,null)),Hs=js.exports,Rs={name:"NotifyUsersForm",extends:Gi,props:{confirmLabel:{type:String,required:!0},userGroupCounts:{type:Object,required:!0},sendLabel:{type:String,required:!0}},methods:{nextPage:function(e){var t=this,n=0;this.submitValues.userGroupIds&&(n=this.submitValues.userGroupIds.reduce((function(e,n){return e+t.userGroupCounts[n]}),0)),this.openDialog({cancelLabel:this.__("common.cancel"),confirmLabel:this.sendLabel,message:this.confirmLabel.replace("{$total}",n),modalName:"confirmNotify",title:this.sendLabel,callback:function(){t.submit(),t.$modal.hide("confirmNotify")}})}}},Ns=Rs,$s=h(Ns,Co,Eo,!1,null,null,null),zs=$s.exports,Bs={name:"AccessPage",extends:Hs,components:{NotifyUsersForm:zs,ProgressBar:ps},data:function(){return{completedJobs:0,progressUrl:"",queueId:"",totalJobs:0}},methods:{processQueue:function(e){var t=this;$.ajax({url:this.progressUrl.replace("{queueId}",e),method:"PUT",headers:{"X-Csrf-Token":pkp.currentUser.csrfToken,"X-Http-Method-Override":"PUT"},error:this.ajaxErrorCallback,success:function(n){t.completedJobs=t.totalJobs-n.pendingJobs,n.pendingJobs&&t.processQueue(e)}})},reload:function(){window.location.reload()}},mounted:function(){var e=this;pkp.eventBus.$on("form-success",(function(t,n){t===pkp.const.FORM_NOTIFY_USERS&&(e.queueId=n.queueId,e.totalJobs=n.totalJobs,e.processQueue(e.queueId))}))},destroyed:function(){pkp.eventBus.$off("form-success")}},Ws=Bs,Us=(n("1f10"),h(Ws,Ao,Oo,!1,null,null,null)),Vs=Us.exports,qs={name:"AddContextForm",extends:Gi,props:{editContextUrl:{type:String,required:!0}},methods:{success:function(e){pkp.eventBus.$emit("form-success",this.id,e),e.id&&(window.location.href=this.editContextUrl.replace("__id__",e.id))}}},Gs=qs,Js=h(Gs,Po,Fo,!1,null,null,null),Ks=Js.exports,Xs={name:"AddContextContainer",extends:Ds,components:{AddContextForm:Ks}},Qs=Xs,Zs=h(Qs,Io,jo,!1,null,null,null),el=Zs.exports,tl={name:"ThemeForm",extends:Gi,props:{themeFields:Object},methods:{fieldChanged:function(e,t,n,r){if("themePluginPath"===e)this.changeTheme(n);else{var i=this.fields.map((function(i){return i.name===e&&(r?i[t][r]=n:i[t]=n),i}));this.$emit("set",this.id,{fields:i})}this.removeError(e,r)},changeTheme:function(e){var t=this;this.$emit("set",this.id,{fields:this.fields.slice(0,1)}),this.$nextTick((function(){var n=t.themeFields[e]||[];t.$emit("set",t.id,{fields:[o(o({},t.fields[0]),{},{value:e})].concat(ye(n))})}))}}},nl=tl,rl=h(nl,Ho,Ro,!1,null,null,null),il=rl.exports,al={extends:Hs,name:"AdminPage",components:{ThemeForm:il}},ol=al,sl=(n("acb6"),h(ol,No,$o,!1,null,null,null)),ll=sl.exports,ul={name:"ImportExportPage",extends:Hs,data:function(){return{selectedSubmissions:[]}},methods:{submit:function(e){$(e).submit()},toggleSelectAll:function(){this.selectedSubmissions.length>=this.components.submissions.itemsMax?this.selectedSubmissions=[]:this.selectedSubmissions=this.components.submissions.items.map((function(e){return e.id}))}}},cl=ul,dl=(n("5d07"),h(cl,zo,Bo,!1,null,null,null)),fl=dl.exports,hl=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"announcementsListPanel"},[e._t("default",[n("list-panel",{attrs:{items:e.items},scopedSlots:e._u([{key:"itemTitle",fn:function(t){var n=t.item;return[e._v(" "+e._s(e.localize(n.title))+" ")]}},{key:"itemActions",fn:function(t){var r=t.item;return[n("pkp-button",{attrs:{element:"a",href:e.urlBase.replace("__id__",r.id)}},[e._v(" "+e._s(e.__("common.view"))+" ")]),n("pkp-button",{on:{click:function(t){return e.openEditModal(r.id)}}},[e._v(" "+e._s(e.__("common.edit"))+" ")]),n("pkp-button",{attrs:{isWarnable:!0},on:{click:function(t){return e.openDeleteModal(r.id)}}},[e._v(" "+e._s(e.__("common.delete"))+" ")])]}}])},[n("pkp-header",{attrs:{slot:"header"},slot:"header"},[n("h2",[e._v(e._s(e.title))]),e.isLoading?n("spinner"):e._e(),n("template",{slot:"actions"},[n("search",{attrs:{searchPhrase:e.searchPhrase},on:{"search-phrase-changed":e.setSearchPhrase}}),n("pkp-button",{on:{click:e.openAddModal}},[e._v(" "+e._s(e.addAnnouncementLabel)+" ")])],1)],2),e.lastPage>1?n("pagination",{attrs:{slot:"footer",currentPage:e.currentPage,isLoading:e.isLoading,lastPage:e.lastPage},on:{"set-page":e.setPage},slot:"footer"}):e._e()],1),n("modal",e._b({attrs:{name:"form"},on:{closed:e.formModalClosed}},"modal",e.MODAL_PROPS,!1),[n("modal-content",{attrs:{closeLabel:e.__("common.close"),modalName:"form",title:e.activeFormTitle}},[n("pkp-form",e._b({on:{set:e.updateForm,success:e.formSuccess}},"pkp-form",e.activeForm,!1))],1)],1)])],2)},pl=[],ml={components:{ListPanel:He,Pagination:ya,PkpForm:Gi,PkpHeader:Pe,Search:Sa},mixins:[Xa,k,un],props:{addAnnouncementLabel:{type:String,required:!0},confirmDeleteMessage:{type:String,required:!0},deleteAnnouncementLabel:{type:String,required:!0},editAnnouncementLabel:{type:String,required:!0},form:{type:Object,required:!0},id:{type:String,required:!0},items:{type:Array,default:function(){return[]}},itemsMax:{type:Number,default:function(){return 0}},title:{type:String,required:!0},urlBase:{type:String,required:!0}},data:function(){return{activeForm:null,activeFormTitle:"",resetFocusTo:null}},methods:{formModalClosed:function(e){this.activeForm=null,this.activeFormTitle="",this.resetFocusTo&&this.resetFocusTo.focus()},formSuccess:function(e){"POST"===this.activeForm.method?(this.offset=0,this.get(),pkp.eventBus.$emit("add:announcement",e)):(this.setItems(this.items.map((function(t){return t.id===e.id?e:t})),this.itemsMax),pkp.eventBus.$emit("update:announcement",e)),this.$modal.hide("form")},openAddModal:function(){this.resetFocusTo=document.activeElement;var e=ys()(this.form);e.action=this.apiUrl,e.method="POST",this.activeForm=e,this.activeFormTitle=this.addAnnouncementLabel,this.$modal.show("form")},openDeleteModal:function(e){var t=this,n=this.items.find((function(t){return t.id===e}));"undefined"!==typeof n?this.openDialog({cancelLabel:this.__("common.no"),modalName:"delete",title:this.deleteAnnouncementLabel,message:this.replaceLocaleParams(this.confirmDeleteMessage,{title:this.localize(n.title)}),callback:function(){var n=t;$.ajax({url:t.apiUrl+"/"+e,type:"POST",headers:{"X-Csrf-Token":pkp.currentUser.csrfToken,"X-Http-Method-Override":"DELETE"},error:n.ajaxErrorCallback,success:function(t){n.setItems(n.items.filter((function(t){return t.id!==e})),n.itemsMax),n.$modal.hide("delete"),n.setFocusIn(n.$el)}})}}):this.openDialog({confirmLabel:this.__("common.ok"),modalName:"unknownError",message:this.__("common.unknownError"),title:this.__("common.error"),callback:function(){t.$modal.hide("unknownError")}})},openEditModal:function(e){var t=this;this.resetFocusTo=document.activeElement;var n=this.items.find((function(t){return t.id===e}));n||this.openDialog({confirmLabel:this.__("common.ok"),modalName:"unknownError",message:this.__("common.unknownError"),callback:function(){t.$modal.hide("unknownError")}});var r=ys()(this.form);r.action=this.apiUrl+"/"+e,r.method="PUT",r.fields=r.fields.map((function(e){return Object.keys(n).includes(e.name)&&(e.value=n[e.name]),e})),this.activeForm=r,this.activeFormTitle=this.editAnnouncementLabel,this.$modal.show("form")},setItems:function(e,t){this.$emit("set",this.id,{items:e,itemsMax:t})},updateForm:function(e,t){var n=o({},this.activeForm);Object.keys(t).forEach((function(e){n[e]=t[e]})),this.activeForm=n}}},_l=ml,vl=(n("c755"),h(_l,hl,pl,!1,null,null,null)),gl=vl.exports,bl=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",[n("list-panel",{staticClass:"listPanel--emailTemplates",attrs:{isSidebarVisible:e.isSidebarVisible,items:e.items},scopedSlots:e._u([{key:"item",fn:function(t){var r=t.item;return[n("email-templates-list-item",{attrs:{apiUrl:e.apiUrl,deleteConfirmMessage:e.deleteConfirmMessage,disableLabel:e.disableLabel,disabledLabel:e.disabledLabel,enableLabel:e.enableLabel,fromLabel:e.fromLabel,resetCompleteLabel:e.resetCompleteLabel,resetConfirmLabel:e.resetConfirmLabel,resetLabel:e.resetLabel,subjectLabel:e.subjectLabel,toLabel:e.toLabel,item:r,roles:e.roles},on:{"delete:item":e.removeItem,"edit:item":e.openEditTemplateModal,"update:item":e.updateItem}})]}}])},[n("pkp-header",{attrs:{slot:"header"},slot:"header"},[n("h2",[e._v(e._s(e.title))]),e.isLoading?n("spinner"):e._e(),n("template",{slot:"actions"},[n("search",{attrs:{searchPhrase:e.searchPhrase},on:{"search-phrase-changed":e.setSearchPhrase}}),n("pkp-button",{attrs:{isActive:e.isSidebarVisible},on:{click:function(t){e.isSidebarVisible=!e.isSidebarVisible}}},[n("icon",{attrs:{icon:"filter",inline:!0}}),e._v(" "+e._s(e.__("common.filter"))+" ")],1),n("pkp-button",{on:{click:e.openAddTemplateModal}},[e._v(e._s(e.addLabel))]),n("pkp-button",{attrs:{isWarnable:!0},on:{click:e.openResetAllModal}},[e._v(" "+e._s(e.resetAllLabel)+" ")])],1)],2),n("template",{slot:"sidebar"},[n("pkp-header",{attrs:{isOneLine:!1}},[n("h3",[n("icon",{attrs:{icon:"filter",inline:!0}}),e._v(" "+e._s(e.__("common.filter"))+" ")],1)]),e._l(e.filters,(function(t,r){return n("div",{key:r,staticClass:"listPanel__block"},[t.heading?n("pkp-header",[n("h4",[e._v(e._s(t.heading))])]):e._e(),e._l(t.filters,(function(t){return n("pkp-filter",e._b({key:t.param+t.value,attrs:{isFilterActive:e.isFilterActive(t.param,t.value)},on:{"add-filter":e.addFilter,"remove-filter":e.removeFilter}},"pkp-filter",t,!1))}))],2)}))],2),e.isLoading?n("template",{slot:"itemsEmpty"},[n("spinner"),e._v(" "+e._s(e.__("common.loading"))+" ")],1):e._e()],2),n("modal",e._b({attrs:{name:"form"},on:{closed:e.formModalClosed}},"modal",e.MODAL_PROPS,!1),[n("modal-content",{attrs:{closeLabel:e.closeLabel,modalName:"form",title:e.activeFormTitle}},[n("pkp-form",e._b({on:{set:e.updateForm,success:e.formSuccess}},"pkp-form",e.activeForm,!1))],1)],1)],1)},yl=[],Ml=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"listPanel__item--emailTemplate"},[n("div",{staticClass:"listPanel__itemSummary "},[n("div",{staticClass:"listPanel__itemIdentity"},[n("div",{staticClass:"listPanel__itemTitle"},[!1===e.item.enabled?n("badge",{attrs:{isWarnable:!0}},[e._v(" "+e._s(e.disabledLabel)+" ")]):e._e(),e._v(" "+e._s(e.localize(e.item.subject))+" ")],1),n("div",{staticClass:"listPanel__itemSubtitle"},[e._v(" "+e._s(e.localize(e.item.description))+" ")]),n("span",{staticClass:"listPanel--emailTemplates__key"},[e._v(" "+e._s(e.item.key)+" ")])]),n("div",{staticClass:"listPanel__itemActions"},[n("expander",{attrs:{isExpanded:e.isExpanded,itemName:e.localize(e.item.subject)},on:{toggle:function(t){e.isExpanded=!e.isExpanded}}})],1)]),e.isExpanded?n("div",{staticClass:"listPanel__itemExpanded"},[n("list",[n("list-item",[e._v(" "+e._s(e.replaceLocaleParams(this.subjectLabel,{subject:e.item.subject}))+" ")]),e.item.fromRoleId?n("list-item",[e._v(" "+e._s(e.replaceLocaleParams(this.fromLabel,{value:e.getRoleLabel(e.item.fromRoleId)}))+" ")]):e._e(),e.item.toRoleId?n("list-item",[e._v(" "+e._s(e.replaceLocaleParams(this.toLabel,{value:e.getRoleLabel(e.item.toRoleId)}))+" ")]):e._e(),n("list-item",{staticClass:"listPanel__item--emailTemplates__body",domProps:{innerHTML:e._s(e.localize(e.item.body))}})],1),n("div",{staticClass:"listPanel__itemExpandedActions"},[e.isEnabling?n("spinner"):e._e(),e.canEdit?n("pkp-button",{ref:"editButton",on:{click:function(t){return e.$emit("edit:item",e.item.key)}}},[e._v(" "+e._s(e.__("common.edit"))+" ")]):e._e(),e.item.canDisable?n("pkp-button",{ref:"toggleEnabledButton",attrs:{isWarnable:!1!==e.item.enabled,disabled:e.isEnabling},on:{click:e.toggleEnabled}},[e._v(" "+e._s(!1===e.item.enabled?e.enableLabel:e.disableLabel)+" ")]):e._e(),e.canReset?n("pkp-button",{ref:"resetButton",attrs:{isWarnable:!0},on:{click:e.openResetModal}},[e._v(" "+e._s(e.resetLabel)+" ")]):e.isCustom?n("pkp-button",{ref:"deleteButton",attrs:{isWarnable:!0},on:{click:e.openDeleteModal}},[e._v(" "+e._s(e.__("common.delete"))+" ")]):e._e()],1)],1):e._e(),e.isLoading?n("div",{staticClass:"listPanel__item--emailTemplates__loadingMask"},[n("spinner")],1):e._e()])},kl=[],wl={components:{Expander:Pa,List:Na,ListItem:Va},mixins:[k,un],props:{apiUrl:{type:String,required:!0},deleteConfirmMessage:{type:String,required:!0},disableLabel:{type:String,required:!0},disabledLabel:{type:String,required:!0},enableLabel:{type:String,required:!0},fromLabel:{type:String,required:!0},item:{type:Object,required:!0},resetCompleteLabel:{type:String,required:!0},resetConfirmLabel:{type:String,required:!0},resetLabel:{type:String,required:!0},roles:{type:Object,required:!0},subjectLabel:{type:String,required:!0},toLabel:{type:String,required:!0}},data:function(){return{isEnabling:!1,isExpanded:!1,isLoading:!1}},computed:{canEdit:function(){return!1!==this.item.canEdit},canReset:function(){return!0===this.item.canEdit&&this.item.id},isCustom:function(){return!this.canReset&&this.item.id}},methods:{getRoleLabel:function(e){return this.roles[e]||""},openDeleteModal:function(){var e=this,t=this.$el.parentNode;this.openDialog({modalName:"delete",cancelLabel:this.__("common.cancel"),confirmLabel:this.__("common.delete"),message:this.deleteConfirmMessage,title:this.__("common.delete"),callback:function(){var n=e;e.isLoading=!0,$.ajax({url:e.apiUrl+"/"+e.item.key,type:"POST",headers:{"X-Csrf-Token":pkp.currentUser.csrfToken,"X-Http-Method-Override":"DELETE"},error:n.ajaxErrorCallback,success:function(e){n.$modal.hide("delete"),n.$emit("delete:item",e),pkp.eventBus.$emit("delete:emailTemplate",e),n.setFocusIn(t)},complete:function(e){n.isLoading=!1}})}})},openResetModal:function(){var e=this,t=this.$el.parentNode;this.openDialog({modalName:"reset",cancelLabel:this.__("common.cancel"),confirmLabel:this.resetLabel,message:this.resetConfirmLabel,title:this.resetLabel,callback:function(){var n=e;e.isLoading=!0,$.ajax({url:e.apiUrl+"/"+e.item.key,type:"POST",headers:{"X-Csrf-Token":pkp.currentUser.csrfToken,"X-Http-Method-Override":"DELETE"},error:n.ajaxErrorCallback,success:function(e){$.ajax({url:n.apiUrl+"/"+n.item.key,type:"GET",error:n.ajaxErrorCallback,success:function(e){n.$modal.hide("reset"),n.$emit("update:item",e),pkp.eventBus.$emit("update:emailTemplate",e),pkp.eventBus.$emit("notify",n.resetCompleteLabel,"success"),n.setFocusIn(t)},complete:function(){n.isLoading=!1}})}})}})},toggleEnabled:function(){var e=this;this.isEnabling=!0,$.ajax({url:this.apiUrl+"/"+this.item.key,type:"POST",headers:{"X-Csrf-Token":pkp.currentUser.csrfToken,"X-Http-Method-Override":"PUT"},data:{enabled:!this.item.enabled},error:function(t){e.ajaxErrorCallback(t)},success:function(t){e.$emit("update:item",t),pkp.eventBus.$emit("update:emailTemplate",t)},complete:function(){e.isEnabling=!1}})}}},Ll=wl,xl=(n("ce9e"),h(Ll,Ml,kl,!1,null,null,null)),Sl=xl.exports,Tl={components:{EmailTemplatesListItem:Sl,ListPanel:He,PkpFilter:ia,PkpForm:Gi,PkpHeader:Pe,Search:Sa},mixins:[un,Xa,k],props:{addLabel:{type:String,required:!0},deleteConfirmMessage:{type:String,required:!0},descriptionLabel:{type:String,required:!0},disableLabel:{type:String,required:!0},disabledLabel:{type:String,required:!0},editTemplateLabel:{type:String,required:!0},enableLabel:{type:String,required:!0},filters:{type:Array,default:function(){return[]}},form:{type:Object,required:!0},fromLabel:{type:String,required:!0},id:{type:String,required:!0},items:{type:Array,default:function(){return[]}},itemsMax:{type:Number,defaut:function(){return 0}},resetAllLabel:{type:String,required:!0},resetAllCompleteLabel:{type:String,required:!0},resetAllConfirmLabel:{type:String,required:!0},resetCompleteLabel:{type:String,required:!0},resetConfirmLabel:{type:String,required:!0},resetLabel:{type:String,required:!0},roles:{type:Object,required:!0},subjectLabel:{type:String,required:!0},title:{type:String,default:function(){return""}},toLabel:{type:String,required:!0}},data:function(){return{activeForm:null,activeFormTitle:"",isSidebarVisible:!1,resetFocusTo:null}},computed:{closeLabel:function(){return this.__("common.close")}},methods:{addFilter:function(e,t){if(!this.isFilterActive(e,t)){var n=o({},this.activeFilters);"isEnabled"===e||"isCustom"===e?n[e]=t:Object.keys(n).includes(e)?n[e].push(t):n[e]=[t],this.activeFilters=n,this.get()}},formModalClosed:function(e){this.activeForm=null,this.activeFormTitle="",this.resetFocusTo&&this.resetFocusTo.focus()},formSuccess:function(e){var t=this;"POST"===this.activeForm.method?(this.offset=0,this.get(),pkp.eventBus.$emit("add:emailTemplate",e)):(this.setItems(this.items.map((function(t){return t.id===e.id?e:t})),this.itemsMax),pkp.eventBus.$emit("update:emailTemplate",e)),setTimeout((function(){t.$modal.hide("form")}),1500)},isFilterActive:function(e,t){return!!Object.keys(this.activeFilters).includes(e)&&(Array.isArray(this.activeFilters[e])?this.activeFilters[e].includes(t):this.activeFilters[e]===t)},openAddTemplateModal:function(){this.resetFocusTo=document.activeElement;var e=ys()(this.form);e.action=this.apiUrl,e.method="POST",this.activeForm=e,this.activeFormTitle=this.addLabel,this.$modal.show("form")},openEditTemplateModal:function(e){var t=this;this.resetFocusTo=document.activeElement;var n=this.items.find((function(t){return t.key===e}));n||this.openDialog({confirmLabel:this.__("common.ok"),modalName:"unknownError",message:this.__("common.unknownError"),callback:function(){t.$modal.hide("unknownError")}});var r=ys()(this.form);r.action=this.apiUrl+"/"+e,r.method="PUT",r.fields=r.fields.filter((function(e){return"key"!==e.name})),this.localize(n.description)&&r.fields.unshift({component:"field-html",name:"description",description:this.localize(n.description),label:this.descriptionLabel,groupId:"default"}),r.fields=r.fields.map((function(e){return Object.keys(n).includes(e.name)&&(e.value=n[e.name]),e})),this.activeForm=r,this.activeFormTitle=this.editTemplateLabel,this.$modal.show("form")},openResetAllModal:function(){var e=this,t=document.activeElement;this.openDialog({modalName:"resetAll",cancelLabel:this.__("common.cancel"),confirmLabel:this.resetAllLabel,message:this.resetAllConfirmLabel,title:this.resetAllLabel,callback:function(){var n=e;e.isLoading=!0,$.ajax({url:e.apiUrl+"/restoreDefaults",type:"POST",headers:{"X-Csrf-Token":pkp.currentUser.csrfToken,"X-Http-Method-Override":"DELETE"},error:n.ajaxErrorCallback,success:function(e){n.$modal.hide("resetAll"),n.get(),pkp.eventBus.$emit("notify",n.resetAllCompleteLabel,"success"),t.focus()},complete:function(e){n.isLoading=!1}})}})},removeFilter:function(e,t){if(Object.keys(this.activeFilters).includes(e)){var n=o({},this.activeFilters);"isEnabled"===e||"isCustom"===e?delete n[e]:n[e]=n[e].filter((function(e){return e!==t})),this.activeFilters=n,this.get()}},removeItem:function(e){this.$emit("set",this.id,{items:this.items.filter((function(t){return t.key!==e.key}))})},setItems:function(e,t){this.$emit("set",this.id,{items:e,itemsMax:t})},updateForm:function(e,t){var n=o({},this.activeForm);Object.keys(t).forEach((function(e){n[e]=t[e]})),this.activeForm=n},updateItem:function(e){this.$emit("set",this.id,{items:this.items.map((function(t){return t.key===e.key?e:t}))})}},watch:{activeFilters:function(e,t){this.offset=0,e&&Object.keys(e).length&&(this.isSidebarVisible=!0)}}},Dl=Tl,Yl=h(Dl,bl,yl,!1,null,null,null),Cl=Yl.exports,El=(n("4ec9"),n("c1df")),Al=n.n(El),Ol={name:"DateTimeForm",extends:Gi,data:function(){return{fieldBeforeSetEvent:{name:null,value:null},formatMap:new Map([["%a","ddd"],["%A","dddd"],["%d","DD"],["%e","D"],["%j","DDDD"],["%W","w"],["%V","W"],["%b","MMM"],["%B","MMMM"],["%m","MM"],["%y","YY"],["%Y","YYYY"],["%H","HH"],["%k","H"],["%I","hh"],["%l","h"],["%M","mm"],["%p","A"],["%P","a"]])}},methods:{fieldChanged:function(e,t,n,r){var i=this,a=this.fields.map((function(a){return a.name===e&&(i.fieldBeforeSetEvent.name=e,i.fieldBeforeSetEvent.value=a[t][r],a[t][r]=n),a}));if(["dateFormatShort","dateFormatLong","timeFormat"].find((function(t){return t===e}))){var o=this.fields.find((function(e){return"dateFormatShort"===e.name})),s=this.fields.find((function(e){return"dateFormatLong"===e.name})),l=this.fields.find((function(e){return"timeFormat"===e.name})),u={date:o.value[r],time:l.value[r]},c={date:s.value[r],time:l.value[r]};"dateFormatShort"===e?u.date=n:"dateFormatLong"===e?c.date=n:"timeFormat"===e&&(u.time=n,c.time=n);var d=o.options[r].find((function(e){return e.value===o.value[r]})),f=s.options[r].find((function(e){return e.value===s.value[r]})),h=l.options[r].find((function(e){return e.value===l.value[r]})),p=d&&h?{date:d.label,time:h.label}:u,m=f&&h?{date:f.label,time:h.label}:c;a=this.updateFields(a,e,r,{value:u,label:p},{value:c,label:m}),this.$emit("set",this.id,{fields:a})}else this.$emit("set",this.id,{fields:a})},updateFields:function(e,t,n,r,i){var a=this;return e.map((function(e){if(("dateFormatShort"===t||"timeFormat"===t)&&"datetimeFormatShort"===e.name){var o="dateFormatShort"===t?a.fieldBeforeSetEvent.value+" "+r.value.time:r.value.date+" "+a.fieldBeforeSetEvent.value;o===e.value[n]&&(e.value[n]=r.value.date+" "+r.value.time),e.options[n][0].label=r.label.date+" "+r.label.time}if(("dateFormatLong"===t||"timeFormat"===t)&&"datetimeFormatLong"===e.name){var s="dateFormatLong"===t?a.fieldBeforeSetEvent.value+" - "+i.value.time:i.value.date+" - "+a.fieldBeforeSetEvent.value;s===e.value[n]&&(e.value[n]=i.value.date+" - "+i.value.time),e.options[n][0].label=i.label.date+" - "+i.label.time}return e}))},convertDateFormat:function(e){for(var t="",n=e.split(""),r=0;r<n.length;r++){var i=n[r];if("%"===i){var a=n[r+1],o=i+a;if(this.formatMap.has(o))t+=this.formatMap.get(o),r++;else{if(" "!==a)return null;t+=i}}else t+=i}return t===e?null:t}},mounted:function(){this.$nextTick((function(){var e=this;this.fields.forEach((function(t){e.availableLocales.forEach((function(n){t.options[n.key].forEach((function(t){var r=e.convertDateFormat(t.label);if(r){var i=Al()();i.locale(n.key),t.label=i.format(r)}}))}))}))}))}},Pl=Ol,Fl=h(Pl,Wo,Uo,!1,null,null,null),Il=Fl.exports,jl={name:"SettingsPage",extends:Hs,components:{AnnouncementsListPanel:gl,EmailTemplatesListPanel:Cl,PkpForm:Gi,ThemeForm:il,DateTimeForm:Il},data:function(){return{announcementsNavLink:{},paymentsNavLink:null}},mounted:function(){var e=this;pkp.eventBus.$on("form-success",(function(t,n){if(t===pkp.const.FORM_PAYMENT_SETTINGS&&e.paymentsNavLink)if(!n.paymentsEnabled&&e.menu["payments"]){var r=o({},e.menu);delete r.payments,e.menu=r}else if(n.paymentsEnabled&&!Object.keys(e.menu).includes("payments")){var i={};Object.keys(e.menu).forEach((function(t){"settings"===t&&(i.payments=e.paymentsNavLink),i[t]=e.menu[t]})),e.menu=i}if(t===pkp.const.FORM_ANNOUNCEMENT_SETTINGS)if(!n.enableAnnouncements&&e.menu["announcements"]){var a=o({},e.menu);delete a.announcements,e.menu=a}else if(n.enableAnnouncements&&!Object.keys(e.menu).includes("announcements")){var s={};Object.keys(e.menu).forEach((function(t){"settings"!==t&&"payments"!==t||(s.announcements=e.announcementsNavLink),s[t]=e.menu[t]})),e.menu=s}}))},destroyed:function(){pkp.eventBus.$off("form-success")}},Hl=jl,Rl=(n("17f1"),h(Hl,Vo,qo,!1,null,null,null)),Nl=Rl.exports,$l=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"pkpDateRange"},[n("span",{staticClass:"-screenReader"},[e._v(e._s(e.dateRangeLabel))]),n("span",{staticClass:"pkpDateRange__current",domProps:{innerHTML:e._s(e.currentRange)}}),n("button",{ref:"toggleButton",staticClass:"pkpDateRange__button",on:{click:e.toggle,blur:e.closeOnBlur}},[n("icon",{attrs:{icon:"calendar"}}),n("span",{staticClass:"-screenReader"},[e._v(" "+e._s(e.changeDateRangeLabel)+" ")])],1),e.options&&this.isOpen?n("div",{staticClass:"pkpDateRange__options"},[n("div",{staticClass:"pkpDateRange__optionGroup"},e._l(e.options,(function(t){return n("div",{key:t.value},[n("button",{ref:t.value,refInFor:!0,staticClass:"pkpDateRange__option",on:{click:function(n){return e.selectOption(t.dateStart,t.dateEnd)}}},[e._v(" "+e._s(t.label)+" ")])])})),0),n("form",{staticClass:"pkpDateRange__optionGroup pkpDateRange__form",on:{submit:e.applyCustomRange}},[n("fieldset",{staticClass:"pkpDateRange__inputGroup"},[n("legend",[e._v(e._s(e.customRangeLabel))]),n("div",{staticClass:"-screenReader",attrs:{id:e.uniqueId+"-describedBy"}},[e._v(" "+e._s(e.dateFormatInstructionsLabel)+" ")]),n("label",[n("span",{staticClass:"-screenReader"},[e._v(e._s(e.fromDateLabel))]),n("input",{directives:[{name:"model",rawName:"v-model",value:e.localDateStart,expression:"localDateStart"}],ref:"dateStart",staticClass:"pkpDateRange__input pkpDateRange__input--start",attrs:{placeholder:"YYYY-MM-DD","aria-describedby":e.uniqueId+"-describedBy","aria-invalid":!!e.errorMessage},domProps:{value:e.localDateStart},on:{focus:e.focus,input:function(t){t.target.composing||(e.localDateStart=t.target.value)}}})]),n("span",{staticClass:"pkpDateRange__separator"},[e._v("—")]),n("label",[n("span",{staticClass:"-screenReader"},[e._v(e._s(e.toDateLabel))]),n("input",{directives:[{name:"model",rawName:"v-model",value:e.localDateEnd,expression:"localDateEnd"}],ref:"dateEnd",staticClass:"pkpDateRange__input pkpDateRange__input--end",attrs:{placeholder:"YYYY-MM-DD","aria-describedby":e.uniqueId+"-describedBy","aria-invalid":!!e.errorMessage},domProps:{value:e.localDateEnd},on:{focus:e.focus,input:function(t){t.target.composing||(e.localDateEnd=t.target.value)}}})])]),n("pkp-button",{on:{click:e.applyCustomRange}},[e._v(e._s(e.applyLabel))]),e.errorMessage?n("div",{staticClass:"pkpDateRange__error"},[n("icon",{attrs:{icon:"exclamation-triangle",inline:!0}}),n("span",{domProps:{innerHTML:e._s(e.errorMessage)}})],1):e._e()],1)]):e._e()])},zl=[],Bl={props:{uniqueId:{type:String,required:!0},dateStart:{type:String,default:""},dateStartMin:{type:String,default:""},dateEnd:{type:String,default:""},dateEndMax:{type:String,default:""},dateRangeLabel:{type:String,required:!0},dateFormatInstructionsLabel:{type:String,required:!0},changeDateRangeLabel:{type:String,required:!0},sinceDateLabel:{type:String,required:!0},untilDateLabel:{type:String,required:!0},allDatesLabel:{type:String,required:!0},customRangeLabel:{type:String,required:!0},fromDateLabel:{type:String,required:!0},toDateLabel:{type:String,required:!0},applyLabel:{type:String,required:!0},invalidDateLabel:{type:String,required:!0},dateDoesNotExistLabel:{type:String,required:!0},invalidDateRangeLabel:{type:String,required:!0},invalidStartDateMinLabel:{type:String,required:!0},options:Array},data:function(){return{isOpen:!1,localDateStart:"",localDateEnd:"",errorMessage:""}},computed:{currentRange:function(){return this.dateStart&&!this.dateEnd?this.replaceLocaleParams(this.sinceDateLabel,{date:this.dateStart}):!this.dateStart&&this.dateEnd?this.replaceLocaleParams(this.untilDateLabel,{date:this.dateEnd}):this.dateStart||this.dateEnd?this.dateStart+" — "+this.dateEnd:this.allDatesLabel}},methods:{toggle:function(){this.isOpen=!this.isOpen},focus:function(){this.isOpen=!0},set:function(e,t){this.$emit("set-range",e,t)},selectOption:function(e,t){this.set(e,t),this.$refs.toggleButton.focus(),this.isOpen=!1},applyCustomRange:function(){if(this.errorMessage="",this.localDateStart.length){if(!this.validateFormat(this.localDateStart))return void(this.errorMessage=this.invalidDateLabel);if(!this.validateDateExists(this.localDateStart))return void(this.errorMessage=this.dateDoesNotExistLabel)}if(this.localDateEnd.length){if(!this.validateFormat(this.localDateEnd))return void(this.errorMessage=this.invalidDateLabel);if(!this.validateDateExists(this.localDateEnd))return void(this.errorMessage=this.dateDoesNotExistLabel)}this.validateRange(this.localDateStart,this.localDateEnd)?this.validateDateStartMin(this.localDateStart)?this.validateDateEndMax(this.localDateEnd)?(this.set(this.localDateStart,this.localDateEnd),this.$refs.toggleButton.focus(),this.isOpen=!1):this.errorMessage=this.replaceLocaleParams(this.invalidEndDateMaxLabel,{date:this.dateEndMax}):this.errorMessage=this.replaceLocaleParams(this.invalidStartDateMinLabel,{date:this.dateStartMin}):this.errorMessage=this.invalidDateRangeLabel},closeOnBlur:function(){var e=this;setTimeout((function(){if(e.$el.contains(document.activeElement))var t=setInterval((function(){e.$el.contains(document.activeElement)||(e.isOpen=!1,window.clearInterval(t))}),1e3);else e.isOpen=!1}),10)},validateFormat:function(e){return null!==e.match(/^[\d]{4}-[\d]{2}-[\d]{2}$/)},validateDateExists:function(e){var t=new Date(e);return!isNaN(t.getTime())},validateRange:function(e,t){if(!e.length||!t.length)return!0;var n=parseInt(e.replace(/-/g,"")),r=parseInt(t.replace(/-/g,""));return!(n>r)},validateDateStartMin:function(e){return!this.dateStartMin||parseInt(e.replace(/-/g,""))>=parseInt(this.dateStartMin.replace(/-/g,""))},validateDateEndMax:function(e){return!this.dateEndMax||parseInt(e.replace(/-/g,""))<=parseInt(this.dateEndMax.replace(/-/g,""))}},mounted:function(){this.localDateStart=this.dateStart,this.localDateEnd=this.dateEnd},watch:{currentRange:function(e,t){e!==t&&this.$emit("updated:current-range",e)}}},Wl=Bl,Ul=(n("1773"),h(Wl,$l,zl,!1,null,null,null)),Vl=Ul.exports,ql=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("table",{staticClass:"pkpTable",attrs:{role:"grid","aria-labelledBy":!!e.labelledBy&&e.labelledBy,"aria-describedBy":!!e.describedBy&&e.describedBy}},[e.label||e.description?n("caption",[e.label?n("div",{staticClass:"pkpTable__label"},[e._v(e._s(e.label))]):e._e(),e.description?n("div",{staticClass:"pkpTable__description",domProps:{innerHTML:e._s(e.description)}}):e._e()]):e._e(),n("thead",[n("tr",e._l(e.columns,(function(t){return n("th",{key:t.name,class:{"-isActive":e.orderBy===t.orderBy},attrs:{scope:"col","aria-sort":!!t.orderBy}},[t.orderBy?n("button",{on:{click:function(n){return e.setOrderBy(t)}}},[e._v(" "+e._s(t.label)+" "),n("icon",{staticClass:"pkpTable__sortIcon",attrs:{icon:e.getIconDirection(t)?"caret-down":"caret-up"}})],1):[e._v(" "+e._s(t.label)+" ")],e._t("thead-"+t.name,null,{column:t})],2)})),0)]),n("tbody",{on:{keydown:[function(t){return(t.type.indexOf("key")||35===t.keyCode)&&t.ctrlKey?t.shiftKey||t.altKey||t.metaKey?null:(t.preventDefault(),e.focusEnd(t)):null},function(t){return(t.type.indexOf("key")||36===t.keyCode)&&t.ctrlKey?t.shiftKey||t.altKey||t.metaKey?null:(t.preventDefault(),e.focusStart(t)):null},function(t){return t.type.indexOf("key")||35===t.keyCode?t.ctrlKey||t.shiftKey||t.altKey||t.metaKey?null:(t.preventDefault(),e.focusLastCell(t)):null},function(t){return t.type.indexOf("key")||36===t.keyCode?t.ctrlKey||t.shiftKey||t.altKey||t.metaKey?null:(t.preventDefault(),e.focusFirstCell(t)):null},function(t){return!t.type.indexOf("key")&&e._k(t.keyCode,"left",37,t.key,["Left","ArrowLeft"])||"button"in t&&0!==t.button||t.ctrlKey||t.shiftKey||t.altKey||t.metaKey?null:(t.preventDefault(),e.focusPreviousCell(t))},function(t){return!t.type.indexOf("key")&&e._k(t.keyCode,"up",38,t.key,["Up","ArrowUp"])||t.ctrlKey||t.shiftKey||t.altKey||t.metaKey?null:(t.preventDefault(),e.focusPreviousRow(t))},function(t){return!t.type.indexOf("key")&&e._k(t.keyCode,"right",39,t.key,["Right","ArrowRight"])||"button"in t&&2!==t.button||t.ctrlKey||t.shiftKey||t.altKey||t.metaKey?null:(t.preventDefault(),e.focusNextCell(t))},function(t){return!t.type.indexOf("key")&&e._k(t.keyCode,"down",40,t.key,["Down","ArrowDown"])||t.ctrlKey||t.shiftKey||t.altKey||t.metaKey?null:(t.preventDefault(),e.focusNextRow(t))}]}},e._l(e.rows,(function(t,r){return n("tr",{key:"row"+r,staticClass:"pkpTable__row"},[e._t("default",e._l(e.columns,(function(e,i){return n("table-cell",{key:e.name,attrs:{column:e,row:t,tabindex:r||i?-1:0}})})),{row:t,rowIndex:r})],2)})),0)])},Gl=[],Jl=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n(e.tag,{tag:"component",class:e.classes,attrs:{scope:"th"===e.tag&&"row",tabindex:e.tabindex}},[e.hasSlot?[e._t("default",null,{column:e.column,row:e.row})]:[n("span",{domProps:{innerHTML:e._s(e.value)}})]],2)},Kl=[],Xl={props:{column:Object,row:Object,tabindex:[Number,Boolean]},computed:{tag:function(){return this.column.isRowHeader?"th":"td"},classes:function(){var e=["pkpTable__cell"];return this.column.truncate&&(e.push("-truncate"),e.push("-truncate-"+this.column.truncate)),e},hasSlot:function(){return this.$slots.default},value:function(){return"function"===typeof this.column.value?this.column.value(this.row):"undefined"!==typeof this.row[this.column.value]?this.row[this.column.value]:""}}},Ql=Xl,Zl=(n("796e"),h(Ql,Jl,Kl,!1,null,null,null)),eu=Zl.exports,tu={components:{TableCell:eu},props:{columns:{type:Array,default:function(){return[]}},describedBy:{type:String,default:function(){return""}},description:{type:String,default:function(){return""}},label:{type:String,default:function(){return""}},labelledBy:{type:String,default:function(){return""}},orderBy:{type:String,default:function(){return""}},orderDirection:{type:Boolean,default:function(){return!1}},rows:{type:Array,default:function(){return[]}}},methods:{setOrderBy:function(e){this.orderBy===e.orderBy?this.$emit("order-by",e.orderBy,!this.orderDirection):this.$emit("order-by",e.orderBy,e.initialOrderDirection)},getIconDirection:function(e){return this.orderBy===e.orderBy?this.orderDirection:null!==e.initialOrderDirection&&e.initialOrderDirection},focusNextCell:function(e){$(e.target).closest("td, th").next().focus()},focusPreviousCell:function(e){$(e.target).closest("td, th").prev().focus()},focusNextRow:function(e){var t=$(e.target),n=t.closest("td, th"),r=n.closest("tr"),i=r.children().index(n),a=r.next().children();a[i]&&a[i].focus()},focusPreviousRow:function(e){var t=$(e.target),n=t.closest("td, th"),r=n.closest("tr"),i=r.children().index(n),a=r.prev().children();a[i]&&a[i].focus()},focusFirstCell:function(e){$(e.target).parents("tr").children("td, th").first().focus()},focusLastCell:function(e){$(e.target).parents("tr").children("td, th").last().focus()},focusStart:function(e){$(e.target).parents("tbody").children("tr").first().children("td, th").first().focus()},focusEnd:function(e){$(e.target).parents("tbody").children("tr").last().children("td, th").last().focus()}}},nu=tu,ru=(n("d88f"),h(nu,ql,Gl,!1,null,null,null)),iu=ru.exports,au={name:"StatsPage",extends:Hs,mixins:[un],components:{DateRange:Vl,PkpFilter:ia,Pagination:ya,PkpHeader:Pe,PkpTable:iu,TableCell:eu},data:function(){return{apiUrl:"",tableColumns:[],dateStart:"",dateEnd:"",dateEndMax:"",dateRangeOptions:[],filters:[],activeFilters:{},isSidebarVisible:!1,isLoadingItems:!1,itemsOfTotalLabel:"",latestItemsGetRequest:""}},computed:{sidebarClasses:function(){var e=[];return this.isSidebarVisible&&e.push("-isVisible"),e},tableClasses:function(){var e=[];return this.isLoadingItems&&e.push("-isLoading"),e}},methods:{isFilterActive:function(e,t){return!!Object.keys(this.activeFilters).includes(e)&&(Array.isArray(this.activeFilters[e])?this.activeFilters[e].includes(t):this.activeFilters[e]===t)},setDateRange:function(e,t){this.dateStart=e,this.dateEnd=t},toggleSidebar:function(){this.isSidebarVisible=!this.isSidebarVisible,this.isSidebarVisible||(this.activeFilters={},this.get())},addFilter:function(e,t){if(!this.isFilterActive(e,t)){var n=o({},this.activeFilters);void 0===n[e]&&(n[e]=[]),n[e].push(t),this.activeFilters=n,this.get()}},removeFilter:function(e,t){if(void 0!==this.activeFilters[e]){var n=o({},this.activeFilters);n[e]=n[e].filter((function(e){return e!==t})),this.activeFilters=n,this.get()}},getDaysBetween:function(e,t){var n=864e5;return e.setMinutes(e.getMinutes()-e.getTimezoneOffset()),t.setMinutes(t.getMinutes()-t.getTimezoneOffset()),(t-e)/n},setSidebarFocus:function(){if(this.$refs.sidebar){var e=this.isSidebarVisible?0:-1;this.$refs.sidebar.querySelectorAll("button, [href], input, select, textarea").forEach((function(t){return t.tabIndex=e}))}}},watch:{activeFilters:function(e,t){e!==t&&(this.offset=0,this.get())},dateEnd:function(e,t){e!==t&&(this.offset=0,this.get())},dateStart:function(e,t){e!==t&&(this.offset=0,this.get())},isSidebarVisible:function(e,t){var n=this;e!==t&&(this.setSidebarFocus(),e&&this.$nextTick((function(){e&&Object.keys(n.$refs).includes("sidebar")&&n.setFocusIn(n.$refs.sidebar)})))}},mounted:function(){this.setSidebarFocus()}},ou=au,su=(n("3d1f"),h(ou,Go,Jo,!1,null,null,null)),lu=su.exports;function uu(e,t){if(t){var n=this.$data._chart,r=e.datasets.map((function(e){return e.label})),i=t.datasets.map((function(e){return e.label})),a=JSON.stringify(i),o=JSON.stringify(r);o===a&&t.datasets.length===e.datasets.length?(e.datasets.forEach((function(e,r){var i=Object.keys(t.datasets[r]),a=Object.keys(e),o=i.filter((function(e){return"_meta"!==e&&-1===a.indexOf(e)}));for(var s in o.forEach((function(e){delete n.data.datasets[r][e]})),e)e.hasOwnProperty(s)&&(n.data.datasets[r][s]=e[s])})),e.hasOwnProperty("labels")&&(n.data.labels=e.labels,this.$emit("labels:update")),e.hasOwnProperty("xLabels")&&(n.data.xLabels=e.xLabels,this.$emit("xlabels:update")),e.hasOwnProperty("yLabels")&&(n.data.yLabels=e.yLabels,this.$emit("ylabels:update")),n.update(),this.$emit("chart:update")):(n&&(n.destroy(),this.$emit("chart:destroy")),this.renderChart(this.chartData,this.options),this.$emit("chart:render"))}else this.$data._chart&&(this.$data._chart.destroy(),this.$emit("chart:destroy")),this.renderChart(this.chartData,this.options),this.$emit("chart:render")}var cu={data:function(){return{chartData:null}},watch:{chartData:uu}},du={props:{chartData:{type:Object,required:!0,default:function(){}}},watch:{chartData:uu}},fu={reactiveData:cu,reactiveProp:du},hu=n("30ef"),pu=n.n(hu);function mu(e,t){return{render:function(e){return e("div",{style:this.styles,class:this.cssClasses},[e("canvas",{attrs:{id:this.chartId,width:this.width,height:this.height},ref:"canvas"})])},props:{chartId:{default:e,type:String},width:{default:400,type:Number},height:{default:400,type:Number},cssClasses:{type:String,default:""},styles:{type:Object},plugins:{type:Array,default:function(){return[]}}},data:function(){return{_chart:null,_plugins:this.plugins}},methods:{addPlugin:function(e){this.$data._plugins.push(e)},generateLegend:function(){if(this.$data._chart)return this.$data._chart.generateLegend()},renderChart:function(e,n){if(this.$data._chart&&this.$data._chart.destroy(),!this.$refs.canvas)throw new Error("Please remove the <template></template> tags from your chart component. See https://vue-chartjs.org/guide/#vue-single-file-components");this.$data._chart=new pu.a(this.$refs.canvas.getContext("2d"),{type:t,data:e,options:n,plugins:this.$data._plugins})}},beforeDestroy:function(){this.$data._chart&&this.$data._chart.destroy()}}}mu("bar-chart","bar"),mu("horizontalbar-chart","horizontalBar");var _u,vu,gu,bu,yu,Mu,ku,wu,Lu,xu,Su,Tu,Du,Yu,Cu=mu("doughnut-chart","doughnut"),Eu=mu("line-chart","line"),Au=(mu("pie-chart","pie"),mu("polar-chart","polarArea"),mu("radar-chart","radar"),mu("bubble-chart","bubble"),mu("scatter-chart","scatter"),{extends:Cu,mixins:[fu.reactiveProp],props:{chartData:{type:Object,required:!0},chartOptions:{type:Object,default:function(){return{}}}},computed:{options:function(){return o({responsive:!0,maintainAspectRatio:!1,cutoutPercentage:80,layout:{padding:{top:32,bottom:32,left:32,right:32}},legend:{display:!1},elements:{arc:{borderWidth:0}}},this.chartOptions)}},mounted:function(){this.renderChart(this.chartData,this.options)}}),Ou=Au,Pu=h(Ou,_u,vu,!1,null,null,null),Fu=Pu.exports,Iu={name:"StatsEditorialPage",extends:lu,components:{DoughnutChart:Fu,Tooltip:Et},data:function(){return{activeByStage:[],averages:{},averagesApiUrl:"",isLoading:!1,percentageStats:[],tableRows:[]}},computed:{chartData:function(){return this.activeByStage.length?{labels:this.activeByStage.map((function(e){return e.name})),datasets:[{data:this.activeByStage.map((function(e){return e.count})),backgroundColor:this.activeByStage.map((function(e){return e.color}))}]}:null},getParams:function(){var e=o({},this.activeFilters);return this.dateStart&&(e.dateStart=this.dateStart),this.dateEnd&&(e.dateEnd=this.dateEnd),e},totalActive:function(){return this.activeByStage.reduce((function(e,t){return e+t.count}),0)}},methods:{get:le()((function(){var e=this,t=this,n=$.pkp.classes.Helper.uuid(),r=$.pkp.classes.Helper.uuid(),i=$.pkp.classes.Helper.uuid();this.isLoading=!0;var a=!1,s=!1,l=!1,u=[],c=[],d={};$.ajax({url:this.apiUrl,type:"GET",data:this.getParams,_uuid:n,error:function(e){n===this._uuid&&t.ajaxErrorCallback(e)},success:function(e){n===this._uuid&&(u=e)},complete:function(e){n===this._uuid&&(a=!0)}});var f=o({},this.getParams);delete f["dateStart"],delete f["dateEnd"],$.ajax({url:this.apiUrl,type:"GET",data:f,_uuid:r,error:function(e){r===this._uuid&&t.ajaxErrorCallback(e)},success:function(e){r===this._uuid&&(c=e)},complete:function(e){r===this._uuid&&(s=!0)}}),$.ajax({url:this.averagesApiUrl,type:"GET",data:this.getParams,_uuid:i,error:function(e){i===this._uuid&&t.ajaxErrorCallback(e)},success:function(e){i===this._uuid&&(d=e)},complete:function(e){i===this._uuid&&(l=!0)}});var h=setInterval((function(){if(a&&s&&l){var t=e.tableRows.map((function(t){var n=u.find((function(e){return e.key===t.key})),r=c.find((function(e){return e.key===t.key}));return e.percentageStats.includes(t.key)?(t.dateRange=Number(100*n.value).toFixed(0)+"%",t.total=Number(100*r.value).toFixed(0)+"%"):(t.dateRange=n.value,t.total=r.value),d.hasOwnProperty(t.key)&&d[t.key]>-1&&r.value>0&&(t.total=e.__("stats.countWithYearlyAverage",{count:t.total,average:d[t.key]})),t}));e.tableRows=[],e.tableRows=t,e.isLoading=!1,clearInterval(h)}}),20)}),0),setCurrentDateRange:function(e){this.tableColumns[1].label=e}}},ju=Iu,Hu=(n("edc4"),h(ju,gu,bu,!1,null,null,null)),Ru=Hu.exports,Nu={extends:Eu,mixins:[fu.reactiveProp],props:{chartData:{type:Object,required:!0},chartOptions:{type:Object,default:function(){return{}}}},computed:{options:function(){return o({responsive:!0,maintainAspectRatio:!1,layout:{padding:{top:32,bottom:8,left:32,right:64}},legend:{display:!1},elements:{point:{borderWidth:0,backgroundColor:"#D00A6C"},line:{tension:.2,borderWidth:2,borderColor:"#D00A6C",backgroundColor:"transparent"}},scales:{xAxes:[{gridLines:{color:"transparent",zeroLineColor:"transparent"},ticks:{fontColor:"rgba(255,255,255,0.85)",fontFamily:'"Noto Sans", "Noto Kufi Arabic", -apple-system, BlinkMacSystemFont, "Segoe UI", "Roboto", "Oxygen-Sans", "Ubuntu", "Cantarell", "Helvetica Neue", sans-serif',fontStyle:700,padding:8,maxTicksLimit:5,maxRotation:0,minRotation:0}}],yAxes:[{gridLines:{color:"#006798",borderDash:[1,10],zeroLineColor:"#006798",drawBorder:!1},ticks:{fontColor:"rgba(255,255,255,0.85)"}}]}},this.chartOptions)}},mounted:function(){this.renderChart(this.chartData,this.options)}},$u=Nu,zu=(n("7a5d"),h($u,yu,Mu,!1,null,null,null)),Bu=zu.exports,Wu={name:"StatsPublicationsPage",extends:lu,components:{LineChart:Bu,Search:Sa},data:function(){return{count:30,items:[],itemsMax:0,offset:0,searchPhrase:"",timeline:[],timelineInterval:"",timelineType:"",orderBy:"",orderDirection:!1,isLoadingTimeline:!1,latestTimelineGetRequest:""}},computed:{chartData:function(){return this.timeline.length?{labels:this.timeline.map((function(e){return e.label})),datasets:[{data:this.timeline.map((function(e){return e.value}))}]}:null},currentPage:function(){return Math.floor(this.offset/this.count)+1},lastPage:function(){return Math.ceil(this.itemsMax/this.count)},isDailyIntervalEnabled:function(){return!(!this.dateStart||!this.dateEnd)&&this.getDaysBetween(new Date(this.dateStart),new Date(this.dateEnd))<91},isMonthlyIntervalEnabled:function(){return!(this.dateStart&&this.dateEnd&&this.isDailyIntervalEnabled)||this.getDaysBetween(new Date(this.dateStart),new Date(this.dateEndMax))>31},getParams:function(){var e=o(o({},this.activeFilters),{},{count:this.count,offset:this.offset});return this.dateStart&&(e.dateStart=this.dateStart),this.dateEnd&&(e.dateEnd=this.dateEnd),this.timelineInterval&&(e.timelineInterval=this.timelineInterval),this.searchPhrase&&(e.searchPhrase=this.searchPhrase),this.orderBy&&(e.orderBy=this.orderBy,e.orderDirection=this.orderDirection?"DESC":"ASC"),e}},methods:{get:le()((function(){this.getItems(),this.getTimeline()}),0),getItems:function(){var e=this;this.isLoadingItems=!0,this.latestItemsGetRequest=$.pkp.classes.Helper.uuid(),$.ajax({url:this.apiUrl,type:"GET",data:this.getParams,_uuid:this.latestItemsGetRequest,error:function(t){e.latestItemsGetRequest===this._uuid&&e.ajaxErrorCallback(t)},success:function(t){e.latestItemsGetRequest===this._uuid&&(e.items=t.items.map((function(e){return e.total=e.abstractViews+e.galleyViews,e})),e.itemsMax=t.itemsMax)},complete:function(t){e.latestItemsGetRequest===this._uuid&&(e.isLoadingItems=!1)}})},getTimeline:function(){var e=this;this.isLoadingTimeline=!0,this.latestTimelineGetRequest=$.pkp.classes.Helper.uuid(),$.ajax({url:this.apiUrl+"/"+this.timelineType,type:"GET",data:this.getParams,_uuid:this.latestTimelineGetRequest,error:function(t){e.latestTimelineGetRequest===this._uuid&&e.ajaxErrorCallback(t)},success:function(t){e.latestTimelineGetRequest===this._uuid&&(e.timeline=t)},complete:function(t){e.latestTimelineGetRequest===this._uuid&&(e.isLoadingTimeline=!1)}})},setOrderBy:function(e,t){this.orderBy=e,this.orderDirection=t},setPage:function(e){this.offset=(e-1)*this.count},setSearchPhrase:function(e){this.searchPhrase=e},setTimelineType:function(e){this.timelineType=e},setTimelineInterval:function(e){this.timelineInterval=e}},mounted:function(){this.getItems()},watch:{count:function(e,t){e!==t&&this.getItems()},isDailyIntervalEnabled:function(e,t){e!==t&&(e||(this.timelineInterval="month"))},isMonthlyIntervalEnabled:function(e,t){e!==t&&(e||(this.timelineInterval="day"))},orderBy:function(e,t){e!==t&&(this.offset=0,this.getItems())},orderDirection:function(e,t){e!==t&&(this.offset=0,this.getItems())},offset:function(e,t){e!==t&&this.getItems()},searchPhrase:function(e,t){e!==t&&(this.offset=0,le()(this.get(),250))},timelineInterval:function(e,t){e!==t&&this.getTimeline()},timelineType:function(e,t){e!==t&&this.getTimeline()}}},Uu=Wu,Vu=(n("d7d9"),h(Uu,ku,wu,!1,null,null,null)),qu=Vu.exports,Gu={name:"StatsUsersPage",extends:lu,methods:{loadExport:function(e){window.location=e,this.$modal.hide("export")}}},Ju=Gu,Ku=h(Ju,Lu,xu,!1,null,null,null),Xu=Ku.exports,Qu={name:"WorkflowPage",extends:Hs,mixins:[Ko,k,un],components:{Dropdown:Ps,PkpHeader:Pe},data:function(){return{activityLogLabel:"",canAccessPublication:!1,canEditPublication:!1,contributorsGridUrl:"",currentPublication:null,editorialHistoryUrl:"",isLoadingVersion:!1,publicationFormIds:[],publicationList:[],publicationTabsLabel:"",publishLabel:"",publishUrl:"",representationsGridUrl:"",schedulePublicationLabel:"",submission:null,submissionApiUrl:"",submissionLibraryLabel:"",submissionLibraryUrl:"",supportsReferences:!1,statusLabel:"",unpublishConfirmLabel:"",unpublishLabel:"",unscheduleConfirmLabel:"",unscheduleLabel:"",uploadFileModalLabel:"",uploadFileUrl:"",versionLabel:"",versionConfirmMessage:"",versionConfirmTitle:"",workingPublication:null}},computed:{canCreateNewVersion:function(){return this.submission.status===pkp.const.STATUS_PUBLISHED&&this.latestPublicationId<=this.currentPublication.id},latestPublicationId:function(){return this.publicationList.reduce((function(e,t){return e<t.id?t.id:e}),0)},publicationMaskClasses:function(){var e=[];return this.isLoadingVersion&&e.push("-isVisible"),e},currentPublicationTabsLabel:function(){return this.replaceLocaleParams(this.publicationTabsLabel,{version:this.workingPublication.id})}},methods:{createVersion:function(){this.isLoadingVersion=!0;var e=new Date,t=this;$.ajax({url:this.submissionApiUrl+"/publications/"+this.latestPublicationId+"/version",type:"POST",headers:{"X-Csrf-Token":pkp.currentUser.csrfToken},error:function(e){t.isLoadingVersion=!1,t.ajaxErrorCallback(e)},success:function(n){t.publicationList.push({id:n.id,datePublished:n.datePublished,status:n.status,version:n.version}),t.workingPublication={},t.workingPublication=n,t.$nextTick((function(){t.setFocusIn(t.$refs.publication);var n=new Date,r=n-e;r/1e3>=3?t.isLoadingVersion=!1:setTimeout((function(){t.isLoadingVersion=!1}),r)}))}})},getConstant:function(e){return pkp.const[e]},loadContributorsGrid:function(e){if(this.$refs.contributors){var t=$(this.$refs.contributors),n=this.contributorsGridUrl.replace("__publicationId__",e.id);if($.pkp.classes.Handler.hasHandler(t)){var r=$.pkp.classes.Handler.getHandler(t);r.setSourceUrl(n),r.reload()}else t.pkpHandler("$.pkp.controllers.UrlInDivHandler",{sourceUrl:n,refreshOn:"form-success"})}},loadRepresentationsGrid:function(e){if(this.$refs.representations){var t=$(this.$refs.representations),n=this.representationsGridUrl.replace("__publicationId__",e.id);if($.pkp.classes.Handler.hasHandler(t)){var r=$.pkp.classes.Handler.getHandler(t);r.setSourceUrl(n),r.reload()}else t.pkpHandler("$.pkp.controllers.UrlInDivHandler",{sourceUrl:n,refreshOn:"form-success"})}},openActivity:function(){var e=this,t={title:this.activityLogLabel,url:this.editorialHistoryUrl,closeCallback:function(){return e.$refs.activityButton.$el.focus()}};$('<div id="'+$.pkp.classes.Helper.uuid()+'" class="pkp_modal pkpModalWrapper" tabIndex="-1"></div>').pkpHandler("$.pkp.controllers.modal.AjaxModalHandler",t)},openCreateVersionPrompt:function(){var e=this;this.openDialog({cancelLabel:this.__("common.no"),modalName:"createVersion",message:this.versionConfirmMessage,title:this.versionConfirmTitle,callback:function(){e.createVersion(),e.$modal.hide("createVersion")}})},openFileUpload:function(){var e=this,t={title:this.uploadFileModalLabel,url:this.uploadFileUrl,closeCallback:function(){pkp.eventBus.$emit("refreshRevisionsGrid"),e.$refs.uploadFileButton.$el.focus()}};$('<div id="'+$.pkp.classes.Helper.uuid()+'" class="pkp_modal pkpModalWrapper" tabIndex="-1"></div>').pkpHandler("$.pkp.controllers.modal.WizardModalHandler",t)},openLibrary:function(){var e=this,t={title:this.submissionLibraryLabel,url:this.submissionLibraryUrl,closeCallback:function(){return e.$refs.library.$el.focus()}};$('<div id="'+$.pkp.classes.Helper.uuid()+'" class="pkp_modal pkpModalWrapper" tabIndex="-1"></div>').pkpHandler("$.pkp.controllers.modal.AjaxModalHandler",t)},openPublish:function(){var e=this,t=this.publishUrl.replace("__publicationId__",this.workingPublication.id),n={title:this.publishLabel,url:t,closeCallback:function(){e.$refs.publish?e.$refs.publish.$el.focus():e.$refs.createVersion&&e.$refs.createVersion.$el.focus(),e.setFocusIn(e.$refs.publication)},closeOnFormSuccessId:pkp.const.FORM_PUBLISH};$('<div id="'+$.pkp.classes.Helper.uuid()+'" class="pkp_modal pkpModalWrapper" tabIndex="-1"></div>').pkpHandler("$.pkp.controllers.modal.AjaxModalHandler",n)},openUnpublish:function(){var e=this;this.openDialog({modalName:"confirmUnpublish",cancelLabel:this.__("common.cancel"),confirmLabel:this.workingPublication.status===pkp.const.STATUS_SCHEDULED?this.unscheduleLabel:this.unpublishLabel,message:this.workingPublication.status===pkp.const.STATUS_SCHEDULED?this.unscheduleConfirmLabel:this.unpublishConfirmLabel,title:this.workingPublication.status===pkp.const.STATUS_SCHEDULED?this.unscheduleLabel:this.unpublishLabel,callback:function(){e.unpublish(e.workingPublication),e.$modal.hide("confirmUnpublish")}})},refreshSubmission:function(){var e=this;$.ajax({url:this.submissionApiUrl,type:"GET",success:function(t){t.publications.forEach((function(t){return e.updatePublicationInList(t)})),delete t.publications,e.submission={},e.submission=t},error:function(t){e.ajaxErrorCallback(t)}})},setPublicationForms:function(e){var t=this;this.publicationFormIds.forEach((function(n){if(t.components[n]){var r=o({},t.components[n]);r.action=t.submissionApiUrl+"/publications/"+e.id,r.fields=r.fields.map((function(t){return Object.keys(e).includes(t.name)&&(t.value=e[t.name]),t})),r.canSubmit=t.canEditPublication&&e.status!==pkp.const.STATUS_PUBLISHED,n===pkp.const.FORM_ISSUE_ENTRY&&(r.fields=r.fields.map((function(t){return"issueId"===t.name&&(t.publicationStatus=e.status),t}))),n===pkp.const.FORM_PUBLICATION_IDENTIFIERS&&(r.fields=r.fields.map((function(t){return["pub-id::doi","pub-id::other::urn"].includes(t.name)&&(t.pages=e.pages||"",t.publisherId=e["pub-id::publisher-id"]||""),t}))),t.components[n]={},t.components[n]=r}}))},setWorkingPublicationById:function(e){this.isLoadingVersion=!0;var t=this;$.ajax({url:this.submissionApiUrl+"/publications/"+e,type:"GET",error:function(e){t.isLoadingVersion=!1,t.ajaxErrorCallback(e)},success:function(e){t.workingPublication={},t.workingPublication=e,t.updatePublicationInList(e),t.$nextTick((function(){t.setFocusIn(t.$refs.publication),t.isLoadingVersion=!1}))}})},unpublish:function(e){this.isLoadingVersion=!0;var t=this;$.ajax({url:this.submissionApiUrl+"/publications/"+e.id+"/unpublish",type:"POST",headers:{"X-Csrf-Token":pkp.currentUser.csrfToken,"X-Http-Method-Override":"PUT"},error:function(e){t.isLoadingVersion=!1,t.ajaxErrorCallback(e)},success:function(e){t.workingPublication={},t.workingPublication=e,t.updatePublicationInList(e),t.setPublicationForms(e),t.loadContributorsGrid(e),t.loadRepresentationsGrid(e),t.isLoadingVersion=!1,t.setFocusIn(t.$refs.publication),t.refreshSubmission()}})},updatePublicationInList:function(e){this.publicationList.forEach((function(t){t.id===e.id&&(t.datePublished=e.datePublished,t.status=e.status,t.version=e.version)}))}},watch:{workingPublication:function(e,t){this.setPublicationForms(e),this.loadContributorsGrid(e),this.loadRepresentationsGrid(e),e.id===this.currentPublication.id&&(this.currentPublication={},this.currentPublication=e)}},created:function(){var e=this;pkp.eventBus.$on("form-success",(function(t,n){e.publicationFormIds.includes(t)&&(e.workingPublication={},e.workingPublication=n),t===pkp.const.FORM_PUBLISH&&(e.setPublicationForms(n),e.currentPublication={},e.currentPublication=n,e.submission.currentPublicationId=n.id,e.refreshSubmission())})),pkp.eventBus.$on("authorsUpdated",(function(){var t=e;$.ajax({url:e.submissionApiUrl+"/publications/"+e.workingPublication.id,type:"GET",success:function(e){t.workingPublication={},t.workingPublication=e,t.updatePublicationInList(e)}})})),this.setPublicationForms(this.workingPublication)},mounted:function(){var e=this;setTimeout((function(){e.loadContributorsGrid(e.workingPublication),e.loadRepresentationsGrid(e.workingPublication)}),1e3),pkp.eventBus.$on("unpublish:publication",this.openUnpublish)},destroyed:function(){pkp.eventBus.$off("form-success"),pkp.eventBus.$off("authorsUpdated"),pkp.eventBus.$off("unpublish:publication")}},Zu=Qu,ec=(n("6e7c"),h(Zu,Su,Tu,!1,null,null,null)),tc=ec.exports,nc={name:"WorkflowPageOJS",extends:tc,data:function(){return{assignToIssueUrl:"",issueApiUrl:"",sectionWordLimits:{},selectIssueLabel:""}},methods:{openAssignToIssue:function(){var e=this.assignToIssueUrl.replace("__publicationId__",this.workingPublication.id),t={title:this.selectIssueLabel,url:e,closeOnFormSuccessId:pkp.const.FORM_ASSIGN_TO_ISSUE};$('<div id="'+$.pkp.classes.Helper.uuid()+'" class="pkp_modal pkpModalWrapper" tabIndex="-1"></div>').pkpHandler("$.pkp.controllers.modal.AjaxModalHandler",t)}},watch:{workingPublication:function(e,t){var n=this;if(e.sectionId!==t.sectionId&&this.components[pkp.const.FORM_TITLE_ABSTRACT]){var r=this.sectionWordLimits[e.sectionId]||0,i=o({},this.components[pkp.const.FORM_TITLE_ABSTRACT]);i.fields=i.fields.map((function(e){return"abstract"===e.name&&(e.wordLimit=r),e})),this.components[pkp.const.FORM_TITLE_ABSTRACT]={},this.components[pkp.const.FORM_TITLE_ABSTRACT]=i}if(e.issueId!==t.issueId){var a=this.components[pkp.const.FORM_ISSUE_ENTRY];if(a.fields=a.fields.map((function(t){return"issueId"===t.name&&(t.value=e.issueId),t})),e.issueId){var s=this;$.ajax({url:this.issueApiUrl.replace("__issueId__",e.issueId),type:"GET",error:function(e){s.ajaxErrorCallback(e)},success:function(e){s.publicationFormIds.forEach((function(t){if(t===pkp.const.FORM_PUBLICATION_IDENTIFIERS){var n=o({},s.components[t]);n.fields=n.fields.map((function(t){return"pub-id::doi"===t.name&&(t.issueNumber=e.number||"",t.issueVolume=e.volume||"",s.workingPublication.datePublished||(t.year=e.year||"")),t})),s.components[t]={},s.components[t]=n}}))}})}else this.publicationFormIds.forEach((function(e){if(e===pkp.const.FORM_PUBLICATION_IDENTIFIERS){var t=o({},n.components[e]);t.fields=t.fields.map((function(e){return"pub-id::doi"===e.name&&(e.issueNumber="",e.issueVolume="",n.workingPublication.datePublished||(e.year="")),e})),n.components[e]={},n.components[e]=t}}))}}},mounted:function(){var e=this;pkp.eventBus.$on("form-success",(function(t,n){pkp.const.FORM_ASSIGN_TO_ISSUE&&t===pkp.const.FORM_ASSIGN_TO_ISSUE&&(e.workingPublication=n,e.workingPublication.issueId&&e.$nextTick((function(){return e.openPublish()})))})),pkp.eventBus.$on("schedule:publication",this.openAssignToIssue)}},rc=nc,ic=(n("b4c0"),h(rc,Du,Yu,!1,null,null,null)),ac=ic.exports;window.pkp=Object.assign(pe,{controllers:{Container:Ds,Page:Hs,AccessPage:Vs,AddContextContainer:el,AdminPage:ll,ImportExportPage:fl,SettingsPage:Nl,StatsEditorialPage:Ru,StatsPublicationsPage:qu,StatsUsersPage:Xu,WorkflowPage:ac}}),window.pkp.Vue.component("field-text",_i)},"746f":function(e,t,n){var r=n("428f"),i=n("5135"),a=n("e538"),o=n("9bf2").f;e.exports=function(e){var t=r.Symbol||(r.Symbol={});i(t,e)||o(t,e,{value:a.f(e)})}},"74dc":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("sw",{months:"Januari_Februari_Machi_Aprili_Mei_Juni_Julai_Agosti_Septemba_Oktoba_Novemba_Desemba".split("_"),monthsShort:"Jan_Feb_Mac_Apr_Mei_Jun_Jul_Ago_Sep_Okt_Nov_Des".split("_"),weekdays:"Jumapili_Jumatatu_Jumanne_Jumatano_Alhamisi_Ijumaa_Jumamosi".split("_"),weekdaysShort:"Jpl_Jtat_Jnne_Jtan_Alh_Ijm_Jmos".split("_"),weekdaysMin:"J2_J3_J4_J5_Al_Ij_J1".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"hh:mm A",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[leo saa] LT",nextDay:"[kesho saa] LT",nextWeek:"[wiki ijayo] dddd [saat] LT",lastDay:"[jana] LT",lastWeek:"[wiki iliyopita] dddd [saat] LT",sameElse:"L"},relativeTime:{future:"%s baadaye",past:"tokea %s",s:"hivi punde",ss:"sekunde %d",m:"dakika moja",mm:"dakika %d",h:"saa limoja",hh:"masaa %d",d:"siku moja",dd:"siku %d",M:"mwezi mmoja",MM:"miezi %d",y:"mwaka mmoja",yy:"miaka %d"},week:{dow:1,doy:7}});return t}))},"74e8":function(e,t,n){"use strict";var r=n("23e7"),i=n("da84"),a=n("83ab"),o=n("8aa7"),s=n("ebb5"),l=n("621a"),u=n("19aa"),c=n("5c6c"),d=n("9112"),f=n("50c4"),h=n("0b25"),p=n("182d"),m=n("c04e"),_=n("5135"),v=n("f5df"),g=n("861d"),b=n("7c73"),y=n("d2bb"),M=n("241c").f,k=n("a078"),w=n("b727").forEach,L=n("2626"),x=n("9bf2"),S=n("06cf"),T=n("69f3"),D=n("7156"),Y=T.get,C=T.set,E=x.f,A=S.f,O=Math.round,P=i.RangeError,F=l.ArrayBuffer,I=l.DataView,j=s.NATIVE_ARRAY_BUFFER_VIEWS,H=s.TYPED_ARRAY_TAG,R=s.TypedArray,N=s.TypedArrayPrototype,$=s.aTypedArrayConstructor,z=s.isTypedArray,B="BYTES_PER_ELEMENT",W="Wrong length",U=function(e,t){var n=0,r=t.length,i=new($(e))(r);while(r>n)i[n]=t[n++];return i},V=function(e,t){E(e,t,{get:function(){return Y(this)[t]}})},q=function(e){var t;return e instanceof F||"ArrayBuffer"==(t=v(e))||"SharedArrayBuffer"==t},G=function(e,t){return z(e)&&"symbol"!=typeof t&&t in e&&String(+t)==String(t)},J=function(e,t){return G(e,t=m(t,!0))?c(2,e[t]):A(e,t)},K=function(e,t,n){return!(G(e,t=m(t,!0))&&g(n)&&_(n,"value"))||_(n,"get")||_(n,"set")||n.configurable||_(n,"writable")&&!n.writable||_(n,"enumerable")&&!n.enumerable?E(e,t,n):(e[t]=n.value,e)};a?(j||(S.f=J,x.f=K,V(N,"buffer"),V(N,"byteOffset"),V(N,"byteLength"),V(N,"length")),r({target:"Object",stat:!0,forced:!j},{getOwnPropertyDescriptor:J,defineProperty:K}),e.exports=function(e,t,n){var a=e.match(/\d+$/)[0]/8,s=e+(n?"Clamped":"")+"Array",l="get"+e,c="set"+e,m=i[s],_=m,v=_&&_.prototype,x={},S=function(e,t){var n=Y(e);return n.view[l](t*a+n.byteOffset,!0)},T=function(e,t,r){var i=Y(e);n&&(r=(r=O(r))<0?0:r>255?255:255&r),i.view[c](t*a+i.byteOffset,r,!0)},A=function(e,t){E(e,t,{get:function(){return S(this,t)},set:function(e){return T(this,t,e)},enumerable:!0})};j?o&&(_=t((function(e,t,n,r){return u(e,_,s),D(function(){return g(t)?q(t)?void 0!==r?new m(t,p(n,a),r):void 0!==n?new m(t,p(n,a)):new m(t):z(t)?U(_,t):k.call(_,t):new m(h(t))}(),e,_)})),y&&y(_,R),w(M(m),(function(e){e in _||d(_,e,m[e])})),_.prototype=v):(_=t((function(e,t,n,r){u(e,_,s);var i,o,l,c=0,d=0;if(g(t)){if(!q(t))return z(t)?U(_,t):k.call(_,t);i=t,d=p(n,a);var m=t.byteLength;if(void 0===r){if(m%a)throw P(W);if(o=m-d,o<0)throw P(W)}else if(o=f(r)*a,o+d>m)throw P(W);l=o/a}else l=h(t),o=l*a,i=new F(o);C(e,{buffer:i,byteOffset:d,byteLength:o,length:l,view:new I(i)});while(c<l)A(e,c++)})),y&&y(_,R),v=_.prototype=b(N)),v.constructor!==_&&d(v,"constructor",_),H&&d(v,H,s),x[s]=_,r({global:!0,forced:_!=m,sham:!j},x),B in _||d(_,B,a),B in v||d(v,B,a),L(s)}):e.exports=function(){}},"74fb":function(e,t,n){},7758:function(e,t,n){},"77a7":function(e,t){var n=1/0,r=Math.abs,i=Math.pow,a=Math.floor,o=Math.log,s=Math.LN2,l=function(e,t,l){var u,c,d,f=new Array(l),h=8*l-t-1,p=(1<<h)-1,m=p>>1,_=23===t?i(2,-24)-i(2,-77):0,v=e<0||0===e&&1/e<0?1:0,g=0;for(e=r(e),e!=e||e===n?(c=e!=e?1:0,u=p):(u=a(o(e)/s),e*(d=i(2,-u))<1&&(u--,d*=2),e+=u+m>=1?_/d:_*i(2,1-m),e*d>=2&&(u++,d/=2),u+m>=p?(c=0,u=p):u+m>=1?(c=(e*d-1)*i(2,t),u+=m):(c=e*i(2,m-1)*i(2,t),u=0));t>=8;f[g++]=255&c,c/=256,t-=8);for(u=u<<t|c,h+=t;h>0;f[g++]=255&u,u/=256,h-=8);return f[--g]|=128*v,f},u=function(e,t){var r,a=e.length,o=8*a-t-1,s=(1<<o)-1,l=s>>1,u=o-7,c=a-1,d=e[c--],f=127&d;for(d>>=7;u>0;f=256*f+e[c],c--,u-=8);for(r=f&(1<<-u)-1,f>>=-u,u+=t;u>0;r=256*r+e[c],c--,u-=8);if(0===f)f=1-l;else{if(f===s)return r?NaN:d?-n:n;r+=i(2,t),f-=l}return(d?-1:1)*r*i(2,f-t)};e.exports={pack:l,unpack:u}},7839:function(e,t){e.exports=["constructor","hasOwnProperty","isPrototypeOf","propertyIsEnumerable","toLocaleString","toString","valueOf"]},7957:function(e,t,n){},"796e":function(e,t,n){"use strict";n("376d")},"7a5d":function(e,t,n){"use strict";n("b073")},"7b0b":function(e,t,n){var r=n("1d80");e.exports=function(e){return Object(r(e))}},"7b1e":function(e,t,n){"use strict";n("eedf")},"7be6":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t="január_február_marec_apríl_máj_jún_júl_august_september_október_november_december".split("_"),n="jan_feb_mar_apr_máj_jún_júl_aug_sep_okt_nov_dec".split("_");function r(e){return e>1&&e<5}function i(e,t,n,i){var a=e+" ";switch(n){case"s":return t||i?"pár sekúnd":"pár sekundami";case"ss":return t||i?a+(r(e)?"sekundy":"sekúnd"):a+"sekundami";case"m":return t?"minúta":i?"minútu":"minútou";case"mm":return t||i?a+(r(e)?"minúty":"minút"):a+"minútami";case"h":return t?"hodina":i?"hodinu":"hodinou";case"hh":return t||i?a+(r(e)?"hodiny":"hodín"):a+"hodinami";case"d":return t||i?"deň":"dňom";case"dd":return t||i?a+(r(e)?"dni":"dní"):a+"dňami";case"M":return t||i?"mesiac":"mesiacom";case"MM":return t||i?a+(r(e)?"mesiace":"mesiacov"):a+"mesiacmi";case"y":return t||i?"rok":"rokom";case"yy":return t||i?a+(r(e)?"roky":"rokov"):a+"rokmi"}}var a=e.defineLocale("sk",{months:t,monthsShort:n,weekdays:"nedeľa_pondelok_utorok_streda_štvrtok_piatok_sobota".split("_"),weekdaysShort:"ne_po_ut_st_št_pi_so".split("_"),weekdaysMin:"ne_po_ut_st_št_pi_so".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd D. MMMM YYYY H:mm"},calendar:{sameDay:"[dnes o] LT",nextDay:"[zajtra o] LT",nextWeek:function(){switch(this.day()){case 0:return"[v nedeľu o] LT";case 1:case 2:return"[v] dddd [o] LT";case 3:return"[v stredu o] LT";case 4:return"[vo štvrtok o] LT";case 5:return"[v piatok o] LT";case 6:return"[v sobotu o] LT"}},lastDay:"[včera o] LT",lastWeek:function(){switch(this.day()){case 0:return"[minulú nedeľu o] LT";case 1:case 2:return"[minulý] dddd [o] LT";case 3:return"[minulú stredu o] LT";case 4:case 5:return"[minulý] dddd [o] LT";case 6:return"[minulú sobotu o] LT"}},sameElse:"L"},relativeTime:{future:"za %s",past:"pred %s",s:i,ss:i,m:i,mm:i,h:i,hh:i,d:i,dd:i,M:i,MM:i,y:i,yy:i},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return a}))},"7c73":function(e,t,n){var r,i=n("825a"),a=n("37e8"),o=n("7839"),s=n("d012"),l=n("1be4"),u=n("cc12"),c=n("f772"),d=">",f="<",h="prototype",p="script",m=c("IE_PROTO"),_=function(){},v=function(e){return f+p+d+e+f+"/"+p+d},g=function(e){e.write(v("")),e.close();var t=e.parentWindow.Object;return e=null,t},b=function(){var e,t=u("iframe"),n="java"+p+":";return t.style.display="none",l.appendChild(t),t.src=String(n),e=t.contentWindow.document,e.open(),e.write(v("document.F=Object")),e.close(),e.F},y=function(){try{r=document.domain&&new ActiveXObject("htmlfile")}catch(t){}y=r?g(r):b();var e=o.length;while(e--)delete y[h][o[e]];return y()};s[m]=!0,e.exports=Object.create||function(e,t){var n;return null!==e?(_[h]=i(e),n=new _,_[h]=null,n[m]=e):n=y(),void 0===t?n:a(n,t)}},"7db0":function(e,t,n){"use strict";var r=n("23e7"),i=n("b727").find,a=n("44d2"),o=n("ae40"),s="find",l=!0,u=o(s);s in[]&&Array(1)[s]((function(){l=!1})),r({target:"Array",proto:!0,forced:l||!u},{find:function(e){return i(this,e,arguments.length>1?arguments[1]:void 0)}}),a(s)},"7dd0":function(e,t,n){"use strict";var r=n("23e7"),i=n("9ed3"),a=n("e163"),o=n("d2bb"),s=n("d44e"),l=n("9112"),u=n("6eeb"),c=n("b622"),d=n("c430"),f=n("3f8c"),h=n("ae93"),p=h.IteratorPrototype,m=h.BUGGY_SAFARI_ITERATORS,_=c("iterator"),v="keys",g="values",b="entries",y=function(){return this};e.exports=function(e,t,n,c,h,M,k){i(n,t,c);var w,L,x,S=function(e){if(e===h&&E)return E;if(!m&&e in Y)return Y[e];switch(e){case v:return function(){return new n(this,e)};case g:return function(){return new n(this,e)};case b:return function(){return new n(this,e)}}return function(){return new n(this)}},T=t+" Iterator",D=!1,Y=e.prototype,C=Y[_]||Y["@@iterator"]||h&&Y[h],E=!m&&C||S(h),A="Array"==t&&Y.entries||C;if(A&&(w=a(A.call(new e)),p!==Object.prototype&&w.next&&(d||a(w)===p||(o?o(w,p):"function"!=typeof w[_]&&l(w,_,y)),s(w,T,!0,!0),d&&(f[T]=y))),h==g&&C&&C.name!==g&&(D=!0,E=function(){return C.call(this)}),d&&!k||Y[_]===E||l(Y,_,E),f[t]=E,h)if(L={values:S(g),keys:M?E:S(v),entries:S(b)},k)for(x in L)(m||D||!(x in Y))&&u(Y,x,L[x]);else r({target:t,proto:!0,forced:m||D},L);return L}},"7f33":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("yo",{months:"Sẹ́rẹ́_Èrèlè_Ẹrẹ̀nà_Ìgbé_Èbibi_Òkùdu_Agẹmo_Ògún_Owewe_Ọ̀wàrà_Bélú_Ọ̀pẹ̀̀".split("_"),monthsShort:"Sẹ́r_Èrl_Ẹrn_Ìgb_Èbi_Òkù_Agẹ_Ògú_Owe_Ọ̀wà_Bél_Ọ̀pẹ̀̀".split("_"),weekdays:"Àìkú_Ajé_Ìsẹ́gun_Ọjọ́rú_Ọjọ́bọ_Ẹtì_Àbámẹ́ta".split("_"),weekdaysShort:"Àìk_Ajé_Ìsẹ́_Ọjr_Ọjb_Ẹtì_Àbá".split("_"),weekdaysMin:"Àì_Aj_Ìs_Ọr_Ọb_Ẹt_Àb".split("_"),longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendar:{sameDay:"[Ònì ni] LT",nextDay:"[Ọ̀la ni] LT",nextWeek:"dddd [Ọsẹ̀ tón'bọ] [ni] LT",lastDay:"[Àna ni] LT",lastWeek:"dddd [Ọsẹ̀ tólọ́] [ni] LT",sameElse:"L"},relativeTime:{future:"ní %s",past:"%s kọjá",s:"ìsẹjú aayá die",ss:"aayá %d",m:"ìsẹjú kan",mm:"ìsẹjú %d",h:"wákati kan",hh:"wákati %d",d:"ọjọ́ kan",dd:"ọjọ́ %d",M:"osù kan",MM:"osù %d",y:"ọdún kan",yy:"ọdún %d"},dayOfMonthOrdinalParse:/ọjọ́\s\d{1,2}/,ordinal:"ọjọ́ %d",week:{dow:1,doy:4}});return t}))},"7f9a":function(e,t,n){var r=n("da84"),i=n("8925"),a=r.WeakMap;e.exports="function"===typeof a&&/native code/.test(i(a))},8155:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e,t,n,r){var i=e+" ";switch(n){case"s":return t||r?"nekaj sekund":"nekaj sekundami";case"ss":return i+=1===e?t?"sekundo":"sekundi":2===e?t||r?"sekundi":"sekundah":e<5?t||r?"sekunde":"sekundah":"sekund",i;case"m":return t?"ena minuta":"eno minuto";case"mm":return i+=1===e?t?"minuta":"minuto":2===e?t||r?"minuti":"minutama":e<5?t||r?"minute":"minutami":t||r?"minut":"minutami",i;case"h":return t?"ena ura":"eno uro";case"hh":return i+=1===e?t?"ura":"uro":2===e?t||r?"uri":"urama":e<5?t||r?"ure":"urami":t||r?"ur":"urami",i;case"d":return t||r?"en dan":"enim dnem";case"dd":return i+=1===e?t||r?"dan":"dnem":2===e?t||r?"dni":"dnevoma":t||r?"dni":"dnevi",i;case"M":return t||r?"en mesec":"enim mesecem";case"MM":return i+=1===e?t||r?"mesec":"mesecem":2===e?t||r?"meseca":"mesecema":e<5?t||r?"mesece":"meseci":t||r?"mesecev":"meseci",i;case"y":return t||r?"eno leto":"enim letom";case"yy":return i+=1===e?t||r?"leto":"letom":2===e?t||r?"leti":"letoma":e<5?t||r?"leta":"leti":t||r?"let":"leti",i}}var n=e.defineLocale("sl",{months:"januar_februar_marec_april_maj_junij_julij_avgust_september_oktober_november_december".split("_"),monthsShort:"jan._feb._mar._apr._maj._jun._jul._avg._sep._okt._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"nedelja_ponedeljek_torek_sreda_četrtek_petek_sobota".split("_"),weekdaysShort:"ned._pon._tor._sre._čet._pet._sob.".split("_"),weekdaysMin:"ne_po_to_sr_če_pe_so".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD. MM. YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd, D. MMMM YYYY H:mm"},calendar:{sameDay:"[danes ob] LT",nextDay:"[jutri ob] LT",nextWeek:function(){switch(this.day()){case 0:return"[v] [nedeljo] [ob] LT";case 3:return"[v] [sredo] [ob] LT";case 6:return"[v] [soboto] [ob] LT";case 1:case 2:case 4:case 5:return"[v] dddd [ob] LT"}},lastDay:"[včeraj ob] LT",lastWeek:function(){switch(this.day()){case 0:return"[prejšnjo] [nedeljo] [ob] LT";case 3:return"[prejšnjo] [sredo] [ob] LT";case 6:return"[prejšnjo] [soboto] [ob] LT";case 1:case 2:case 4:case 5:return"[prejšnji] dddd [ob] LT"}},sameElse:"L"},relativeTime:{future:"čez %s",past:"pred %s",s:t,ss:t,m:t,mm:t,h:t,hh:t,d:t,dd:t,M:t,MM:t,y:t,yy:t},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}});return n}))},"81d5":function(e,t,n){"use strict";var r=n("7b0b"),i=n("23cb"),a=n("50c4");e.exports=function(e){var t=r(this),n=a(t.length),o=arguments.length,s=i(o>1?arguments[1]:void 0,n),l=o>2?arguments[2]:void 0,u=void 0===l?n:i(l,n);while(u>s)t[s++]=e;return t}},"81e9":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t="nolla yksi kaksi kolme neljä viisi kuusi seitsemän kahdeksan yhdeksän".split(" "),n=["nolla","yhden","kahden","kolmen","neljän","viiden","kuuden",t[7],t[8],t[9]];function r(e,t,n,r){var a="";switch(n){case"s":return r?"muutaman sekunnin":"muutama sekunti";case"ss":a=r?"sekunnin":"sekuntia";break;case"m":return r?"minuutin":"minuutti";case"mm":a=r?"minuutin":"minuuttia";break;case"h":return r?"tunnin":"tunti";case"hh":a=r?"tunnin":"tuntia";break;case"d":return r?"päivän":"päivä";case"dd":a=r?"päivän":"päivää";break;case"M":return r?"kuukauden":"kuukausi";case"MM":a=r?"kuukauden":"kuukautta";break;case"y":return r?"vuoden":"vuosi";case"yy":a=r?"vuoden":"vuotta";break}return a=i(e,r)+" "+a,a}function i(e,r){return e<10?r?n[e]:t[e]:e}var a=e.defineLocale("fi",{months:"tammikuu_helmikuu_maaliskuu_huhtikuu_toukokuu_kesäkuu_heinäkuu_elokuu_syyskuu_lokakuu_marraskuu_joulukuu".split("_"),monthsShort:"tammi_helmi_maalis_huhti_touko_kesä_heinä_elo_syys_loka_marras_joulu".split("_"),weekdays:"sunnuntai_maanantai_tiistai_keskiviikko_torstai_perjantai_lauantai".split("_"),weekdaysShort:"su_ma_ti_ke_to_pe_la".split("_"),weekdaysMin:"su_ma_ti_ke_to_pe_la".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD.MM.YYYY",LL:"Do MMMM[ta] YYYY",LLL:"Do MMMM[ta] YYYY, [klo] HH.mm",LLLL:"dddd, Do MMMM[ta] YYYY, [klo] HH.mm",l:"D.M.YYYY",ll:"Do MMM YYYY",lll:"Do MMM YYYY, [klo] HH.mm",llll:"ddd, Do MMM YYYY, [klo] HH.mm"},calendar:{sameDay:"[tänään] [klo] LT",nextDay:"[huomenna] [klo] LT",nextWeek:"dddd [klo] LT",lastDay:"[eilen] [klo] LT",lastWeek:"[viime] dddd[na] [klo] LT",sameElse:"L"},relativeTime:{future:"%s päästä",past:"%s sitten",s:r,ss:r,m:r,mm:r,h:r,hh:r,d:r,dd:r,M:r,MM:r,y:r,yy:r},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return a}))},8217:function(e,t,n){"use strict";n("0450")},8230:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"١",2:"٢",3:"٣",4:"٤",5:"٥",6:"٦",7:"٧",8:"٨",9:"٩",0:"٠"},n={"١":"1","٢":"2","٣":"3","٤":"4","٥":"5","٦":"6","٧":"7","٨":"8","٩":"9","٠":"0"},r=e.defineLocale("ar-sa",{months:"يناير_فبراير_مارس_أبريل_مايو_يونيو_يوليو_أغسطس_سبتمبر_أكتوبر_نوفمبر_ديسمبر".split("_"),monthsShort:"يناير_فبراير_مارس_أبريل_مايو_يونيو_يوليو_أغسطس_سبتمبر_أكتوبر_نوفمبر_ديسمبر".split("_"),weekdays:"الأحد_الإثنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت".split("_"),weekdaysShort:"أحد_إثنين_ثلاثاء_أربعاء_خميس_جمعة_سبت".split("_"),weekdaysMin:"ح_ن_ث_ر_خ_ج_س".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},meridiemParse:/ص|م/,isPM:function(e){return"م"===e},meridiem:function(e,t,n){return e<12?"ص":"م"},calendar:{sameDay:"[اليوم على الساعة] LT",nextDay:"[غدا على الساعة] LT",nextWeek:"dddd [على الساعة] LT",lastDay:"[أمس على الساعة] LT",lastWeek:"dddd [على الساعة] LT",sameElse:"L"},relativeTime:{future:"في %s",past:"منذ %s",s:"ثوان",ss:"%d ثانية",m:"دقيقة",mm:"%d دقائق",h:"ساعة",hh:"%d ساعات",d:"يوم",dd:"%d أيام",M:"شهر",MM:"%d أشهر",y:"سنة",yy:"%d سنوات"},preparse:function(e){return e.replace(/[١٢٣٤٥٦٧٨٩٠]/g,(function(e){return n[e]})).replace(/،/g,",")},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]})).replace(/,/g,"،")},week:{dow:0,doy:6}});return r}))},"825a":function(e,t,n){var r=n("861d");e.exports=function(e){if(!r(e))throw TypeError(String(e)+" is not an object");return e}},"82d0":function(e,t,n){"use strict";n("971f")},"82f8":function(e,t,n){"use strict";var r=n("ebb5"),i=n("4d64").includes,a=r.aTypedArray,o=r.exportTypedArrayMethod;o("includes",(function(e){return i(a(this),e,arguments.length>1?arguments[1]:void 0)}))},8300:function(e,t,n){},"83ab":function(e,t,n){var r=n("d039");e.exports=!r((function(){return 7!=Object.defineProperty({},1,{get:function(){return 7}})[1]}))},8418:function(e,t,n){"use strict";var r=n("c04e"),i=n("9bf2"),a=n("5c6c");e.exports=function(e,t,n){var o=r(t);o in e?i.f(e,o,a(0,n)):e[o]=n}},"84aa":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("bg",{months:"януари_февруари_март_април_май_юни_юли_август_септември_октомври_ноември_декември".split("_"),monthsShort:"яну_фев_мар_апр_май_юни_юли_авг_сеп_окт_ное_дек".split("_"),weekdays:"неделя_понеделник_вторник_сряда_четвъртък_петък_събота".split("_"),weekdaysShort:"нед_пон_вто_сря_чет_пет_съб".split("_"),weekdaysMin:"нд_пн_вт_ср_чт_пт_сб".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"D.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY H:mm",LLLL:"dddd, D MMMM YYYY H:mm"},calendar:{sameDay:"[Днес в] LT",nextDay:"[Утре в] LT",nextWeek:"dddd [в] LT",lastDay:"[Вчера в] LT",lastWeek:function(){switch(this.day()){case 0:case 3:case 6:return"[Миналата] dddd [в] LT";case 1:case 2:case 4:case 5:return"[Миналия] dddd [в] LT"}},sameElse:"L"},relativeTime:{future:"след %s",past:"преди %s",s:"няколко секунди",ss:"%d секунди",m:"минута",mm:"%d минути",h:"час",hh:"%d часа",d:"ден",dd:"%d дена",w:"седмица",ww:"%d седмици",M:"месец",MM:"%d месеца",y:"година",yy:"%d години"},dayOfMonthOrdinalParse:/\d{1,2}-(ев|ен|ти|ви|ри|ми)/,ordinal:function(e){var t=e%10,n=e%100;return 0===e?e+"-ев":0===n?e+"-ен":n>10&&n<20?e+"-ти":1===t?e+"-ви":2===t?e+"-ри":7===t||8===t?e+"-ми":e+"-ти"},week:{dow:1,doy:7}});return t}))},"85cd":function(e,t,n){"use strict";n("9a5c")},"85d9":function(e,t,n){},"861d":function(e,t){e.exports=function(e){return"object"===typeof e?null!==e:"function"===typeof e}},8670:function(e,t,n){},8689:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"၁",2:"၂",3:"၃",4:"၄",5:"၅",6:"၆",7:"၇",8:"၈",9:"၉",0:"၀"},n={"၁":"1","၂":"2","၃":"3","၄":"4","၅":"5","၆":"6","၇":"7","၈":"8","၉":"9","၀":"0"},r=e.defineLocale("my",{months:"ဇန်နဝါရီ_ဖေဖော်ဝါရီ_မတ်_ဧပြီ_မေ_ဇွန်_ဇူလိုင်_သြဂုတ်_စက်တင်ဘာ_အောက်တိုဘာ_နိုဝင်ဘာ_ဒီဇင်ဘာ".split("_"),monthsShort:"ဇန်_ဖေ_မတ်_ပြီ_မေ_ဇွန်_လိုင်_သြ_စက်_အောက်_နို_ဒီ".split("_"),weekdays:"တနင်္ဂနွေ_တနင်္လာ_အင်္ဂါ_ဗုဒ္ဓဟူး_ကြာသပတေး_သောကြာ_စနေ".split("_"),weekdaysShort:"နွေ_လာ_ဂါ_ဟူး_ကြာ_သော_နေ".split("_"),weekdaysMin:"နွေ_လာ_ဂါ_ဟူး_ကြာ_သော_နေ".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[ယနေ.] LT [မှာ]",nextDay:"[မနက်ဖြန်] LT [မှာ]",nextWeek:"dddd LT [မှာ]",lastDay:"[မနေ.က] LT [မှာ]",lastWeek:"[ပြီးခဲ့သော] dddd LT [မှာ]",sameElse:"L"},relativeTime:{future:"လာမည့် %s မှာ",past:"လွန်ခဲ့သော %s က",s:"စက္ကန်.အနည်းငယ်",ss:"%d စက္ကန့်",m:"တစ်မိနစ်",mm:"%d မိနစ်",h:"တစ်နာရီ",hh:"%d နာရီ",d:"တစ်ရက်",dd:"%d ရက်",M:"တစ်လ",MM:"%d လ",y:"တစ်နှစ်",yy:"%d နှစ်"},preparse:function(e){return e.replace(/[၁၂၃၄၅၆၇၈၉၀]/g,(function(e){return n[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},week:{dow:1,doy:4}});return r}))},"86b0":function(e,t,n){},"86da":function(e,t,n){"use strict";n("36a4")},8840:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("gl",{months:"xaneiro_febreiro_marzo_abril_maio_xuño_xullo_agosto_setembro_outubro_novembro_decembro".split("_"),monthsShort:"xan._feb._mar._abr._mai._xuñ._xul._ago._set._out._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"domingo_luns_martes_mércores_xoves_venres_sábado".split("_"),weekdaysShort:"dom._lun._mar._mér._xov._ven._sáb.".split("_"),weekdaysMin:"do_lu_ma_mé_xo_ve_sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY H:mm",LLLL:"dddd, D [de] MMMM [de] YYYY H:mm"},calendar:{sameDay:function(){return"[hoxe "+(1!==this.hours()?"ás":"á")+"] LT"},nextDay:function(){return"[mañá "+(1!==this.hours()?"ás":"á")+"] LT"},nextWeek:function(){return"dddd ["+(1!==this.hours()?"ás":"a")+"] LT"},lastDay:function(){return"[onte "+(1!==this.hours()?"á":"a")+"] LT"},lastWeek:function(){return"[o] dddd [pasado "+(1!==this.hours()?"ás":"a")+"] LT"},sameElse:"L"},relativeTime:{future:function(e){return 0===e.indexOf("un")?"n"+e:"en "+e},past:"hai %s",s:"uns segundos",ss:"%d segundos",m:"un minuto",mm:"%d minutos",h:"unha hora",hh:"%d horas",d:"un día",dd:"%d días",M:"un mes",MM:"%d meses",y:"un ano",yy:"%d anos"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,doy:4}});return t}))},"88ee":function(e,t,n){},8925:function(e,t,n){var r=n("c6cd"),i=Function.toString;"function"!=typeof r.inspectSource&&(r.inspectSource=function(e){return i.call(e)}),e.exports=r.inspectSource},"898b":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t="ene._feb._mar._abr._may._jun._jul._ago._sep._oct._nov._dic.".split("_"),n="ene_feb_mar_abr_may_jun_jul_ago_sep_oct_nov_dic".split("_"),r=[/^ene/i,/^feb/i,/^mar/i,/^abr/i,/^may/i,/^jun/i,/^jul/i,/^ago/i,/^sep/i,/^oct/i,/^nov/i,/^dic/i],i=/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre|ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,a=e.defineLocale("es",{months:"enero_febrero_marzo_abril_mayo_junio_julio_agosto_septiembre_octubre_noviembre_diciembre".split("_"),monthsShort:function(e,r){return e?/-MMM-/.test(r)?n[e.month()]:t[e.month()]:t},monthsRegex:i,monthsShortRegex:i,monthsStrictRegex:/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre)/i,monthsShortStrictRegex:/^(ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,monthsParse:r,longMonthsParse:r,shortMonthsParse:r,weekdays:"domingo_lunes_martes_miércoles_jueves_viernes_sábado".split("_"),weekdaysShort:"dom._lun._mar._mié._jue._vie._sáb.".split("_"),weekdaysMin:"do_lu_ma_mi_ju_vi_sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY H:mm",LLLL:"dddd, D [de] MMMM [de] YYYY H:mm"},calendar:{sameDay:function(){return"[hoy a la"+(1!==this.hours()?"s":"")+"] LT"},nextDay:function(){return"[mañana a la"+(1!==this.hours()?"s":"")+"] LT"},nextWeek:function(){return"dddd [a la"+(1!==this.hours()?"s":"")+"] LT"},lastDay:function(){return"[ayer a la"+(1!==this.hours()?"s":"")+"] LT"},lastWeek:function(){return"[el] dddd [pasado a la"+(1!==this.hours()?"s":"")+"] LT"},sameElse:"L"},relativeTime:{future:"en %s",past:"hace %s",s:"unos segundos",ss:"%d segundos",m:"un minuto",mm:"%d minutos",h:"una hora",hh:"%d horas",d:"un día",dd:"%d días",w:"una semana",ww:"%d semanas",M:"un mes",MM:"%d meses",y:"un año",yy:"%d años"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,doy:4},invalidDate:"Fecha inválida"});return a}))},"89e8":function(e,t){function n(e){var t=e.target||e.srcElement;t.__resizeRAF__&&cancelAnimationFrame(t.__resizeRAF__),t.__resizeRAF__=requestAnimationFrame((function(){var n=t.__resizeTrigger__,r=n&&n.__resizeListeners__;r&&r.forEach((function(t){t.call(n,e)}))}))}t=function(e,t){var r,i=this,a=i.document,o=a.attachEvent;function s(){this.contentDocument.defaultView.__resizeTrigger__=this.__resizeElement__,this.contentDocument.defaultView.addEventListener("resize",n)}if("undefined"!==typeof navigator&&(r=navigator.userAgent.match(/Trident/)||navigator.userAgent.match(/Edge/)),!e.__resizeListeners__)if(e.__resizeListeners__=[],o)e.__resizeTrigger__=e,e.attachEvent("onresize",n);else{"static"===getComputedStyle(e).position&&(e.style.position="relative");var l=e.__resizeTrigger__=a.createElement("object");l.setAttribute("style","position: absolute; top: 0; left: 0; height: 100%; width: 100%; pointer-events: none; z-index: -1; opacity: 0;"),l.setAttribute("class","resize-sensor"),l.setAttribute("tabindex","-1"),l.__resizeElement__=e,l.onload=s,l.type="text/html",r&&e.appendChild(l),l.data="about:blank",r||e.appendChild(l)}e.__resizeListeners__.push(t)};e.exports="undefined"===typeof window?t:t.bind(window),e.exports.unbind=function(e,t){var r=document.attachEvent,i=e.__resizeListeners__||[];if(t){var a=i.indexOf(t);-1!==a&&i.splice(a,1)}else i=e.__resizeListeners__=[];if(!i.length){if(r)e.detachEvent("onresize",n);else if(e.__resizeTrigger__){var o=e.__resizeTrigger__.contentDocument,s=o&&o.defaultView;s&&(s.removeEventListener("resize",n),delete s.__resizeTrigger__),e.__resizeTrigger__=!e.removeChild(e.__resizeTrigger__)}delete e.__resizeListeners__}}},"8aa5":function(e,t,n){"use strict";var r=n("6547").charAt;e.exports=function(e,t,n){return t+(n?r(e,t).length:1)}},"8aa7":function(e,t,n){var r=n("da84"),i=n("d039"),a=n("1c7e"),o=n("ebb5").NATIVE_ARRAY_BUFFER_VIEWS,s=r.ArrayBuffer,l=r.Int8Array;e.exports=!o||!i((function(){l(1)}))||!i((function(){new l(-1)}))||!a((function(e){new l,new l(null),new l(1.5),new l(e)}),!0)||i((function(){return 1!==new l(new s(2),1,void 0).length}))},"8cc4":function(e,t,n){},"8d47":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e){return"undefined"!==typeof Function&&e instanceof Function||"[object Function]"===Object.prototype.toString.call(e)}var n=e.defineLocale("el",{monthsNominativeEl:"Ιανουάριος_Φεβρουάριος_Μάρτιος_Απρίλιος_Μάιος_Ιούνιος_Ιούλιος_Αύγουστος_Σεπτέμβριος_Οκτώβριος_Νοέμβριος_Δεκέμβριος".split("_"),monthsGenitiveEl:"Ιανουαρίου_Φεβρουαρίου_Μαρτίου_Απριλίου_Μαΐου_Ιουνίου_Ιουλίου_Αυγούστου_Σεπτεμβρίου_Οκτωβρίου_Νοεμβρίου_Δεκεμβρίου".split("_"),months:function(e,t){return e?"string"===typeof t&&/D/.test(t.substring(0,t.indexOf("MMMM")))?this._monthsGenitiveEl[e.month()]:this._monthsNominativeEl[e.month()]:this._monthsNominativeEl},monthsShort:"Ιαν_Φεβ_Μαρ_Απρ_Μαϊ_Ιουν_Ιουλ_Αυγ_Σεπ_Οκτ_Νοε_Δεκ".split("_"),weekdays:"Κυριακή_Δευτέρα_Τρίτη_Τετάρτη_Πέμπτη_Παρασκευή_Σάββατο".split("_"),weekdaysShort:"Κυρ_Δευ_Τρι_Τετ_Πεμ_Παρ_Σαβ".split("_"),weekdaysMin:"Κυ_Δε_Τρ_Τε_Πε_Πα_Σα".split("_"),meridiem:function(e,t,n){return e>11?n?"μμ":"ΜΜ":n?"πμ":"ΠΜ"},isPM:function(e){return"μ"===(e+"").toLowerCase()[0]},meridiemParse:/[ΠΜ]\.?Μ?\.?/i,longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendarEl:{sameDay:"[Σήμερα {}] LT",nextDay:"[Αύριο {}] LT",nextWeek:"dddd [{}] LT",lastDay:"[Χθες {}] LT",lastWeek:function(){switch(this.day()){case 6:return"[το προηγούμενο] dddd [{}] LT";default:return"[την προηγούμενη] dddd [{}] LT"}},sameElse:"L"},calendar:function(e,n){var r=this._calendarEl[e],i=n&&n.hours();return t(r)&&(r=r.apply(n)),r.replace("{}",i%12===1?"στη":"στις")},relativeTime:{future:"σε %s",past:"%s πριν",s:"λίγα δευτερόλεπτα",ss:"%d δευτερόλεπτα",m:"ένα λεπτό",mm:"%d λεπτά",h:"μία ώρα",hh:"%d ώρες",d:"μία μέρα",dd:"%d μέρες",M:"ένας μήνας",MM:"%d μήνες",y:"ένας χρόνος",yy:"%d χρόνια"},dayOfMonthOrdinalParse:/\d{1,2}η/,ordinal:"%dη",week:{dow:1,doy:4}});return n}))},"8d57":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t="styczeń_luty_marzec_kwiecień_maj_czerwiec_lipiec_sierpień_wrzesień_październik_listopad_grudzień".split("_"),n="stycznia_lutego_marca_kwietnia_maja_czerwca_lipca_sierpnia_września_października_listopada_grudnia".split("_"),r=[/^sty/i,/^lut/i,/^mar/i,/^kwi/i,/^maj/i,/^cze/i,/^lip/i,/^sie/i,/^wrz/i,/^paź/i,/^lis/i,/^gru/i];function i(e){return e%10<5&&e%10>1&&~~(e/10)%10!==1}function a(e,t,n){var r=e+" ";switch(n){case"ss":return r+(i(e)?"sekundy":"sekund");case"m":return t?"minuta":"minutę";case"mm":return r+(i(e)?"minuty":"minut");case"h":return t?"godzina":"godzinę";case"hh":return r+(i(e)?"godziny":"godzin");case"ww":return r+(i(e)?"tygodnie":"tygodni");case"MM":return r+(i(e)?"miesiące":"miesięcy");case"yy":return r+(i(e)?"lata":"lat")}}var o=e.defineLocale("pl",{months:function(e,r){return e?/D MMMM/.test(r)?n[e.month()]:t[e.month()]:t},monthsShort:"sty_lut_mar_kwi_maj_cze_lip_sie_wrz_paź_lis_gru".split("_"),monthsParse:r,longMonthsParse:r,shortMonthsParse:r,weekdays:"niedziela_poniedziałek_wtorek_środa_czwartek_piątek_sobota".split("_"),weekdaysShort:"ndz_pon_wt_śr_czw_pt_sob".split("_"),weekdaysMin:"Nd_Pn_Wt_Śr_Cz_Pt_So".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Dziś o] LT",nextDay:"[Jutro o] LT",nextWeek:function(){switch(this.day()){case 0:return"[W niedzielę o] LT";case 2:return"[We wtorek o] LT";case 3:return"[W środę o] LT";case 6:return"[W sobotę o] LT";default:return"[W] dddd [o] LT"}},lastDay:"[Wczoraj o] LT",lastWeek:function(){switch(this.day()){case 0:return"[W zeszłą niedzielę o] LT";case 3:return"[W zeszłą środę o] LT";case 6:return"[W zeszłą sobotę o] LT";default:return"[W zeszły] dddd [o] LT"}},sameElse:"L"},relativeTime:{future:"za %s",past:"%s temu",s:"kilka sekund",ss:a,m:a,mm:a,h:a,hh:a,d:"1 dzień",dd:"%d dni",w:"tydzień",ww:a,M:"miesiąc",MM:a,y:"rok",yy:a},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return o}))},"8df4":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"۱",2:"۲",3:"۳",4:"۴",5:"۵",6:"۶",7:"۷",8:"۸",9:"۹",0:"۰"},n={"۱":"1","۲":"2","۳":"3","۴":"4","۵":"5","۶":"6","۷":"7","۸":"8","۹":"9","۰":"0"},r=e.defineLocale("fa",{months:"ژانویه_فوریه_مارس_آوریل_مه_ژوئن_ژوئیه_اوت_سپتامبر_اکتبر_نوامبر_دسامبر".split("_"),monthsShort:"ژانویه_فوریه_مارس_آوریل_مه_ژوئن_ژوئیه_اوت_سپتامبر_اکتبر_نوامبر_دسامبر".split("_"),weekdays:"یک‌شنبه_دوشنبه_سه‌شنبه_چهارشنبه_پنج‌شنبه_جمعه_شنبه".split("_"),weekdaysShort:"یک‌شنبه_دوشنبه_سه‌شنبه_چهارشنبه_پنج‌شنبه_جمعه_شنبه".split("_"),weekdaysMin:"ی_د_س_چ_پ_ج_ش".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},meridiemParse:/قبل از ظهر|بعد از ظهر/,isPM:function(e){return/بعد از ظهر/.test(e)},meridiem:function(e,t,n){return e<12?"قبل از ظهر":"بعد از ظهر"},calendar:{sameDay:"[امروز ساعت] LT",nextDay:"[فردا ساعت] LT",nextWeek:"dddd [ساعت] LT",lastDay:"[دیروز ساعت] LT",lastWeek:"dddd [پیش] [ساعت] LT",sameElse:"L"},relativeTime:{future:"در %s",past:"%s پیش",s:"چند ثانیه",ss:"%d ثانیه",m:"یک دقیقه",mm:"%d دقیقه",h:"یک ساعت",hh:"%d ساعت",d:"یک روز",dd:"%d روز",M:"یک ماه",MM:"%d ماه",y:"یک سال",yy:"%d سال"},preparse:function(e){return e.replace(/[۰-۹]/g,(function(e){return n[e]})).replace(/،/g,",")},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]})).replace(/,/g,"،")},dayOfMonthOrdinalParse:/\d{1,2}م/,ordinal:"%dم",week:{dow:6,doy:12}});return r}))},"8e73":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"١",2:"٢",3:"٣",4:"٤",5:"٥",6:"٦",7:"٧",8:"٨",9:"٩",0:"٠"},n={"١":"1","٢":"2","٣":"3","٤":"4","٥":"5","٦":"6","٧":"7","٨":"8","٩":"9","٠":"0"},r=function(e){return 0===e?0:1===e?1:2===e?2:e%100>=3&&e%100<=10?3:e%100>=11?4:5},i={s:["أقل من ثانية","ثانية واحدة",["ثانيتان","ثانيتين"],"%d ثوان","%d ثانية","%d ثانية"],m:["أقل من دقيقة","دقيقة واحدة",["دقيقتان","دقيقتين"],"%d دقائق","%d دقيقة","%d دقيقة"],h:["أقل من ساعة","ساعة واحدة",["ساعتان","ساعتين"],"%d ساعات","%d ساعة","%d ساعة"],d:["أقل من يوم","يوم واحد",["يومان","يومين"],"%d أيام","%d يومًا","%d يوم"],M:["أقل من شهر","شهر واحد",["شهران","شهرين"],"%d أشهر","%d شهرا","%d شهر"],y:["أقل من عام","عام واحد",["عامان","عامين"],"%d أعوام","%d عامًا","%d عام"]},a=function(e){return function(t,n,a,o){var s=r(t),l=i[e][r(t)];return 2===s&&(l=l[n?0:1]),l.replace(/%d/i,t)}},o=["يناير","فبراير","مارس","أبريل","مايو","يونيو","يوليو","أغسطس","سبتمبر","أكتوبر","نوفمبر","ديسمبر"],s=e.defineLocale("ar",{months:o,monthsShort:o,weekdays:"الأحد_الإثنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت".split("_"),weekdaysShort:"أحد_إثنين_ثلاثاء_أربعاء_خميس_جمعة_سبت".split("_"),weekdaysMin:"ح_ن_ث_ر_خ_ج_س".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"D/‏M/‏YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},meridiemParse:/ص|م/,isPM:function(e){return"م"===e},meridiem:function(e,t,n){return e<12?"ص":"م"},calendar:{sameDay:"[اليوم عند الساعة] LT",nextDay:"[غدًا عند الساعة] LT",nextWeek:"dddd [عند الساعة] LT",lastDay:"[أمس عند الساعة] LT",lastWeek:"dddd [عند الساعة] LT",sameElse:"L"},relativeTime:{future:"بعد %s",past:"منذ %s",s:a("s"),ss:a("s"),m:a("m"),mm:a("m"),h:a("h"),hh:a("h"),d:a("d"),dd:a("d"),M:a("M"),MM:a("M"),y:a("y"),yy:a("y")},preparse:function(e){return e.replace(/[١٢٣٤٥٦٧٨٩٠]/g,(function(e){return n[e]})).replace(/،/g,",")},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]})).replace(/,/g,"،")},week:{dow:6,doy:12}});return s}))},"8ed6":function(e,t,n){},9043:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"১",2:"২",3:"৩",4:"৪",5:"৫",6:"৬",7:"৭",8:"৮",9:"৯",0:"০"},n={"১":"1","২":"2","৩":"3","৪":"4","৫":"5","৬":"6","৭":"7","৮":"8","৯":"9","০":"0"},r=e.defineLocale("bn",{months:"জানুয়ারি_ফেব্রুয়ারি_মার্চ_এপ্রিল_মে_জুন_জুলাই_আগস্ট_সেপ্টেম্বর_অক্টোবর_নভেম্বর_ডিসেম্বর".split("_"),monthsShort:"জানু_ফেব্রু_মার্চ_এপ্রিল_মে_জুন_জুলাই_আগস্ট_সেপ্ট_অক্টো_নভে_ডিসে".split("_"),weekdays:"রবিবার_সোমবার_মঙ্গলবার_বুধবার_বৃহস্পতিবার_শুক্রবার_শনিবার".split("_"),weekdaysShort:"রবি_সোম_মঙ্গল_বুধ_বৃহস্পতি_শুক্র_শনি".split("_"),weekdaysMin:"রবি_সোম_মঙ্গল_বুধ_বৃহ_শুক্র_শনি".split("_"),longDateFormat:{LT:"A h:mm সময়",LTS:"A h:mm:ss সময়",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm সময়",LLLL:"dddd, D MMMM YYYY, A h:mm সময়"},calendar:{sameDay:"[আজ] LT",nextDay:"[আগামীকাল] LT",nextWeek:"dddd, LT",lastDay:"[গতকাল] LT",lastWeek:"[গত] dddd, LT",sameElse:"L"},relativeTime:{future:"%s পরে",past:"%s আগে",s:"কয়েক সেকেন্ড",ss:"%d সেকেন্ড",m:"এক মিনিট",mm:"%d মিনিট",h:"এক ঘন্টা",hh:"%d ঘন্টা",d:"এক দিন",dd:"%d দিন",M:"এক মাস",MM:"%d মাস",y:"এক বছর",yy:"%d বছর"},preparse:function(e){return e.replace(/[১২৩৪৫৬৭৮৯০]/g,(function(e){return n[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},meridiemParse:/রাত|সকাল|দুপুর|বিকাল|রাত/,meridiemHour:function(e,t){return 12===e&&(e=0),"রাত"===t&&e>=4||"দুপুর"===t&&e<5||"বিকাল"===t?e+12:e},meridiem:function(e,t,n){return e<4?"রাত":e<10?"সকাল":e<17?"দুপুর":e<20?"বিকাল":"রাত"},week:{dow:0,doy:6}});return r}))},"90e3":function(e,t){var n=0,r=Math.random();e.exports=function(e){return"Symbol("+String(void 0===e?"":e)+")_"+(++n+r).toString(36)}},"90ea":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("zh-tw",{months:"一月_二月_三月_四月_五月_六月_七月_八月_九月_十月_十一月_十二月".split("_"),monthsShort:"1月_2月_3月_4月_5月_6月_7月_8月_9月_10月_11月_12月".split("_"),weekdays:"星期日_星期一_星期二_星期三_星期四_星期五_星期六".split("_"),weekdaysShort:"週日_週一_週二_週三_週四_週五_週六".split("_"),weekdaysMin:"日_一_二_三_四_五_六".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY/MM/DD",LL:"YYYY年M月D日",LLL:"YYYY年M月D日 HH:mm",LLLL:"YYYY年M月D日dddd HH:mm",l:"YYYY/M/D",ll:"YYYY年M月D日",lll:"YYYY年M月D日 HH:mm",llll:"YYYY年M月D日dddd HH:mm"},meridiemParse:/凌晨|早上|上午|中午|下午|晚上/,meridiemHour:function(e,t){return 12===e&&(e=0),"凌晨"===t||"早上"===t||"上午"===t?e:"中午"===t?e>=11?e:e+12:"下午"===t||"晚上"===t?e+12:void 0},meridiem:function(e,t,n){var r=100*e+t;return r<600?"凌晨":r<900?"早上":r<1130?"上午":r<1230?"中午":r<1800?"下午":"晚上"},calendar:{sameDay:"[今天] LT",nextDay:"[明天] LT",nextWeek:"[下]dddd LT",lastDay:"[昨天] LT",lastWeek:"[上]dddd LT",sameElse:"L"},dayOfMonthOrdinalParse:/\d{1,2}(日|月|週)/,ordinal:function(e,t){switch(t){case"d":case"D":case"DDD":return e+"日";case"M":return e+"月";case"w":case"W":return e+"週";default:return e}},relativeTime:{future:"%s後",past:"%s前",s:"幾秒",ss:"%d 秒",m:"1 分鐘",mm:"%d 分鐘",h:"1 小時",hh:"%d 小時",d:"1 天",dd:"%d 天",M:"1 個月",MM:"%d 個月",y:"1 年",yy:"%d 年"}});return t}))},9112:function(e,t,n){var r=n("83ab"),i=n("9bf2"),a=n("5c6c");e.exports=r?function(e,t,n){return i.f(e,t,a(1,n))}:function(e,t,n){return e[t]=n,e}},9152:function(e,t){
/*! ieee754. BSD-3-Clause License. Feross Aboukhadijeh <https://feross.org/opensource> */
t.read=function(e,t,n,r,i){var a,o,s=8*i-r-1,l=(1<<s)-1,u=l>>1,c=-7,d=n?i-1:0,f=n?-1:1,h=e[t+d];for(d+=f,a=h&(1<<-c)-1,h>>=-c,c+=s;c>0;a=256*a+e[t+d],d+=f,c-=8);for(o=a&(1<<-c)-1,a>>=-c,c+=r;c>0;o=256*o+e[t+d],d+=f,c-=8);if(0===a)a=1-u;else{if(a===l)return o?NaN:1/0*(h?-1:1);o+=Math.pow(2,r),a-=u}return(h?-1:1)*o*Math.pow(2,a-r)},t.write=function(e,t,n,r,i,a){var o,s,l,u=8*a-i-1,c=(1<<u)-1,d=c>>1,f=23===i?Math.pow(2,-24)-Math.pow(2,-77):0,h=r?0:a-1,p=r?1:-1,m=t<0||0===t&&1/t<0?1:0;for(t=Math.abs(t),isNaN(t)||t===1/0?(s=isNaN(t)?1:0,o=c):(o=Math.floor(Math.log(t)/Math.LN2),t*(l=Math.pow(2,-o))<1&&(o--,l*=2),t+=o+d>=1?f/l:f*Math.pow(2,1-d),t*l>=2&&(o++,l/=2),o+d>=c?(s=0,o=c):o+d>=1?(s=(t*l-1)*Math.pow(2,i),o+=d):(s=t*Math.pow(2,d-1)*Math.pow(2,i),o=0));i>=8;e[n+h]=255&s,h+=p,s/=256,i-=8);for(o=o<<i|s,u+=i;u>0;e[n+h]=255&o,h+=p,o/=256,u-=8);e[n+h-p]|=128*m}},9182:function(e,t,n){},9263:function(e,t,n){"use strict";var r=n("ad6d"),i=n("9f7f"),a=RegExp.prototype.exec,o=String.prototype.replace,s=a,l=function(){var e=/a/,t=/b*/g;return a.call(e,"a"),a.call(t,"a"),0!==e.lastIndex||0!==t.lastIndex}(),u=i.UNSUPPORTED_Y||i.BROKEN_CARET,c=void 0!==/()??/.exec("")[1],d=l||c||u;d&&(s=function(e){var t,n,i,s,d=this,f=u&&d.sticky,h=r.call(d),p=d.source,m=0,_=e;return f&&(h=h.replace("y",""),-1===h.indexOf("g")&&(h+="g"),_=String(e).slice(d.lastIndex),d.lastIndex>0&&(!d.multiline||d.multiline&&"\n"!==e[d.lastIndex-1])&&(p="(?: "+p+")",_=" "+_,m++),n=new RegExp("^(?:"+p+")",h)),c&&(n=new RegExp("^"+p+"$(?!\\s)",h)),l&&(t=d.lastIndex),i=a.call(f?n:d,_),f?i?(i.input=i.input.slice(m),i[0]=i[0].slice(m),i.index=d.lastIndex,d.lastIndex+=i[0].length):d.lastIndex=0:l&&i&&(d.lastIndex=d.global?i.index+i[0].length:t),c&&i&&i.length>1&&o.call(i[0],n,(function(){for(s=1;s<arguments.length-2;s++)void 0===arguments[s]&&(i[s]=void 0)})),i}),e.exports=s},"92c3":function(e,t,n){var r,i;n("99af"),n("4de4"),n("4160"),n("c975"),n("d81d"),n("fb6a"),n("a434"),n("c19f"),n("ace4"),n("b0c0"),n("4fad"),n("3410"),n("b64b"),n("131a"),n("d3b7"),n("ac1f"),n("25f0"),n("3ca3"),n("466d"),n("5319"),n("1276"),n("498a"),n("5cc6"),n("9a8c"),n("a975"),n("735e"),n("c1ac"),n("d139"),n("3a7b"),n("d5d6"),n("82f8"),n("e91f"),n("60bd"),n("5f96"),n("3280"),n("3fcc"),n("ca91"),n("25a1"),n("cd26"),n("3c5d"),n("2954"),n("649e"),n("219c"),n("170b"),n("b39a"),n("72f7"),n("159b"),n("ddb0"),n("2b3d");var a=n("46fd"),o=n("62f5");!function(a,s){"object"==o(t)&&"undefined"!=typeof e?e.exports=s():(r=s,i="function"===typeof r?r.call(t,n,t,e):r,void 0===i||(e.exports=i))}(0,(function(){"use strict";var e,t=(function(e){var t=function(){function e(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}return function(t,n,r){return n&&e(t.prototype,n),r&&e(t,r),t}}();function n(e,t){if(!e)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return!t||"object"!=o(t)&&"function"!=typeof t?e:t}function r(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}var i=function(){function e(){r(this,e)}return t(e,[{key:"on",value:function(e,t){return this._callbacks=this._callbacks||{},this._callbacks[e]||(this._callbacks[e]=[]),this._callbacks[e].push(t),this}},{key:"emit",value:function(e){this._callbacks=this._callbacks||{};var t=this._callbacks[e];if(t){for(var n=arguments.length,r=Array(n>1?n-1:0),i=1;i<n;i++)r[i-1]=arguments[i];for(var a=0,o=o=t;;){if(a>=o.length)break;o[a++].apply(this,r)}}return this}},{key:"off",value:function(e,t){if(!this._callbacks||0===arguments.length)return this._callbacks={},this;var n=this._callbacks[e];if(!n)return this;if(1===arguments.length)return delete this._callbacks[e],this;for(var r=0;r<n.length;r++)if(n[r]===t){n.splice(r,1);break}return this}}]),e}(),a=function(e){function a(e,t){r(this,a);var i,o=n(this,(a.__proto__||Object.getPrototypeOf(a)).call(this)),s=void 0;if(o.element=e,o.version=a.version,o.defaultOptions.previewTemplate=o.defaultOptions.previewTemplate.replace(/\n*/g,""),o.clickableElements=[],o.listeners=[],o.files=[],"string"==typeof o.element&&(o.element=document.querySelector(o.element)),!o.element||null==o.element.nodeType)throw new Error("Invalid dropzone element.");if(o.element.dropzone)throw new Error("Dropzone already attached.");a.instances.push(o),o.element.dropzone=o;var l,u=null!=(i=a.optionsForElement(o.element))?i:{};if(o.options=a.extend({},o.defaultOptions,u,null!=t?t:{}),o.options.forceFallback||!a.isBrowserSupported())return l=o.options.fallback.call(o),n(o,l);if(null==o.options.url&&(o.options.url=o.element.getAttribute("action")),!o.options.url)throw new Error("No URL provided.");if(o.options.acceptedFiles&&o.options.acceptedMimeTypes)throw new Error("You can't provide both 'acceptedFiles' and 'acceptedMimeTypes'. 'acceptedMimeTypes' is deprecated.");if(o.options.uploadMultiple&&o.options.chunking)throw new Error("You cannot set both: uploadMultiple and chunking.");return o.options.acceptedMimeTypes&&(o.options.acceptedFiles=o.options.acceptedMimeTypes,delete o.options.acceptedMimeTypes),null!=o.options.renameFilename&&(o.options.renameFile=function(e){return o.options.renameFilename.call(o,e.name,e)}),o.options.method=o.options.method.toUpperCase(),(s=o.getExistingFallback())&&s.parentNode&&s.parentNode.removeChild(s),!1!==o.options.previewsContainer&&(o.options.previewsContainer?o.previewsContainer=a.getElement(o.options.previewsContainer,"previewsContainer"):o.previewsContainer=o.element),o.options.clickable&&(!0===o.options.clickable?o.clickableElements=[o.element]:o.clickableElements=a.getElements(o.options.clickable,"clickable")),o.init(),o}return function(e,t){if("function"!=typeof t&&null!==t)throw new TypeError("Super expression must either be null or a function, not "+o(t));e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,enumerable:!1,writable:!0,configurable:!0}}),t&&(Object.setPrototypeOf?Object.setPrototypeOf(e,t):e.__proto__=t)}(a,i),t(a,null,[{key:"initClass",value:function(){this.prototype.Emitter=i,this.prototype.events=["drop","dragstart","dragend","dragenter","dragover","dragleave","addedfile","addedfiles","removedfile","thumbnail","error","errormultiple","processing","processingmultiple","uploadprogress","totaluploadprogress","sending","sendingmultiple","success","successmultiple","canceled","canceledmultiple","complete","completemultiple","reset","maxfilesexceeded","maxfilesreached","queuecomplete"],this.prototype.defaultOptions={url:null,method:"post",withCredentials:!1,timeout:3e4,parallelUploads:2,uploadMultiple:!1,chunking:!1,forceChunking:!1,chunkSize:2e6,parallelChunkUploads:!1,retryChunks:!1,retryChunksLimit:3,maxFilesize:256,paramName:"file",createImageThumbnails:!0,maxThumbnailFilesize:10,thumbnailWidth:120,thumbnailHeight:120,thumbnailMethod:"crop",resizeWidth:null,resizeHeight:null,resizeMimeType:null,resizeQuality:.8,resizeMethod:"contain",filesizeBase:1e3,maxFiles:null,headers:null,clickable:!0,ignoreHiddenFiles:!0,acceptedFiles:null,acceptedMimeTypes:null,autoProcessQueue:!0,autoQueue:!0,addRemoveLinks:!1,previewsContainer:null,hiddenInputContainer:"body",capture:null,renameFilename:null,renameFile:null,forceFallback:!1,dictDefaultMessage:"Drop files here to upload",dictFallbackMessage:"Your browser does not support drag'n'drop file uploads.",dictFallbackText:"Please use the fallback form below to upload your files like in the olden days.",dictFileTooBig:"File is too big ({{filesize}}MiB). Max filesize: {{maxFilesize}}MiB.",dictInvalidFileType:"You can't upload files of this type.",dictResponseError:"Server responded with {{statusCode}} code.",dictCancelUpload:"Cancel upload",dictUploadCanceled:"Upload canceled.",dictCancelUploadConfirmation:"Are you sure you want to cancel this upload?",dictRemoveFile:"Remove file",dictRemoveFileConfirmation:null,dictMaxFilesExceeded:"You can not upload any more files.",dictFileSizeUnits:{tb:"TB",gb:"GB",mb:"MB",kb:"KB",b:"b"},init:function(){},params:function(e,t,n){if(n)return{dzuuid:n.file.upload.uuid,dzchunkindex:n.index,dztotalfilesize:n.file.size,dzchunksize:this.options.chunkSize,dztotalchunkcount:n.file.upload.totalChunkCount,dzchunkbyteoffset:n.index*this.options.chunkSize}},accept:function(e,t){return t()},chunksUploaded:function(e,t){t()},fallback:function(){var e=void 0;this.element.className=this.element.className+" dz-browser-not-supported";for(var t=0,n=n=this.element.getElementsByTagName("div");;){if(t>=n.length)break;var r=n[t++];if(/(^| )dz-message($| )/.test(r.className)){e=r,r.className="dz-message";break}}e||(e=a.createElement('<div class="dz-message"><span></span></div>'),this.element.appendChild(e));var i=e.getElementsByTagName("span")[0];return i&&(null!=i.textContent?i.textContent=this.options.dictFallbackMessage:null!=i.innerText&&(i.innerText=this.options.dictFallbackMessage)),this.element.appendChild(this.getFallbackForm())},resize:function(e,t,n,r){var i={srcX:0,srcY:0,srcWidth:e.width,srcHeight:e.height},a=e.width/e.height;null==t&&null==n?(t=i.srcWidth,n=i.srcHeight):null==t?t=n*a:null==n&&(n=t/a);var o=(t=Math.min(t,i.srcWidth))/(n=Math.min(n,i.srcHeight));if(i.srcWidth>t||i.srcHeight>n)if("crop"===r)a>o?(i.srcHeight=e.height,i.srcWidth=i.srcHeight*o):(i.srcWidth=e.width,i.srcHeight=i.srcWidth/o);else{if("contain"!==r)throw new Error("Unknown resizeMethod '"+r+"'");a>o?n=t/a:t=n*a}return i.srcX=(e.width-i.srcWidth)/2,i.srcY=(e.height-i.srcHeight)/2,i.trgWidth=t,i.trgHeight=n,i},transformFile:function(e,t){return(this.options.resizeWidth||this.options.resizeHeight)&&e.type.match(/image.*/)?this.resizeImage(e,this.options.resizeWidth,this.options.resizeHeight,this.options.resizeMethod,t):t(e)},previewTemplate:'<div class="dz-preview dz-file-preview">\n  <div class="dz-image"><img data-dz-thumbnail /></div>\n  <div class="dz-details">\n    <div class="dz-size"><span data-dz-size></span></div>\n    <div class="dz-filename"><span data-dz-name></span></div>\n  </div>\n  <div class="dz-progress"><span class="dz-upload" data-dz-uploadprogress></span></div>\n  <div class="dz-error-message"><span data-dz-errormessage></span></div>\n  <div class="dz-success-mark">\n    <svg width="54px" height="54px" viewBox="0 0 54 54" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" xmlns:sketch="http://www.bohemiancoding.com/sketch/ns">\n      <title>Check</title>\n      <defs></defs>\n      <g id="Page-1" stroke="none" stroke-width="1" fill="none" fill-rule="evenodd" sketch:type="MSPage">\n        <path d="M23.5,31.8431458 L17.5852419,25.9283877 C16.0248253,24.3679711 13.4910294,24.366835 11.9289322,25.9289322 C10.3700136,27.4878508 10.3665912,30.0234455 11.9283877,31.5852419 L20.4147581,40.0716123 C20.5133999,40.1702541 20.6159315,40.2626649 20.7218615,40.3488435 C22.2835669,41.8725651 24.794234,41.8626202 26.3461564,40.3106978 L43.3106978,23.3461564 C44.8771021,21.7797521 44.8758057,19.2483887 43.3137085,17.6862915 C41.7547899,16.1273729 39.2176035,16.1255422 37.6538436,17.6893022 L23.5,31.8431458 Z M27,53 C41.3594035,53 53,41.3594035 53,27 C53,12.6405965 41.3594035,1 27,1 C12.6405965,1 1,12.6405965 1,27 C1,41.3594035 12.6405965,53 27,53 Z" id="Oval-2" stroke-opacity="0.198794158" stroke="#747474" fill-opacity="0.816519475" fill="#FFFFFF" sketch:type="MSShapeGroup"></path>\n      </g>\n    </svg>\n  </div>\n  <div class="dz-error-mark">\n    <svg width="54px" height="54px" viewBox="0 0 54 54" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" xmlns:sketch="http://www.bohemiancoding.com/sketch/ns">\n      <title>Error</title>\n      <defs></defs>\n      <g id="Page-1" stroke="none" stroke-width="1" fill="none" fill-rule="evenodd" sketch:type="MSPage">\n        <g id="Check-+-Oval-2" sketch:type="MSLayerGroup" stroke="#747474" stroke-opacity="0.198794158" fill="#FFFFFF" fill-opacity="0.816519475">\n          <path d="M32.6568542,29 L38.3106978,23.3461564 C39.8771021,21.7797521 39.8758057,19.2483887 38.3137085,17.6862915 C36.7547899,16.1273729 34.2176035,16.1255422 32.6538436,17.6893022 L27,23.3431458 L21.3461564,17.6893022 C19.7823965,16.1255422 17.2452101,16.1273729 15.6862915,17.6862915 C14.1241943,19.2483887 14.1228979,21.7797521 15.6893022,23.3461564 L21.3431458,29 L15.6893022,34.6538436 C14.1228979,36.2202479 14.1241943,38.7516113 15.6862915,40.3137085 C17.2452101,41.8726271 19.7823965,41.8744578 21.3461564,40.3106978 L27,34.6568542 L32.6538436,40.3106978 C34.2176035,41.8744578 36.7547899,41.8726271 38.3137085,40.3137085 C39.8758057,38.7516113 39.8771021,36.2202479 38.3106978,34.6538436 L32.6568542,29 Z M27,53 C41.3594035,53 53,41.3594035 53,27 C53,12.6405965 41.3594035,1 27,1 C12.6405965,1 1,12.6405965 1,27 C1,41.3594035 12.6405965,53 27,53 Z" id="Oval-2" sketch:type="MSShapeGroup"></path>\n        </g>\n      </g>\n    </svg>\n  </div>\n</div>',drop:function(e){return this.element.classList.remove("dz-drag-hover")},dragstart:function(e){},dragend:function(e){return this.element.classList.remove("dz-drag-hover")},dragenter:function(e){return this.element.classList.add("dz-drag-hover")},dragover:function(e){return this.element.classList.add("dz-drag-hover")},dragleave:function(e){return this.element.classList.remove("dz-drag-hover")},paste:function(e){},reset:function(){return this.element.classList.remove("dz-started")},addedfile:function(e){var t=this;if(this.element===this.previewsContainer&&this.element.classList.add("dz-started"),this.previewsContainer){e.previewElement=a.createElement(this.options.previewTemplate.trim()),e.previewTemplate=e.previewElement,this.previewsContainer.appendChild(e.previewElement);for(var n=0,r=r=e.previewElement.querySelectorAll("[data-dz-name]");;){if(n>=r.length)break;var i=r[n++];i.textContent=e.name}for(var o=0,s=s=e.previewElement.querySelectorAll("[data-dz-size]");!(o>=s.length);)(i=s[o++]).innerHTML=this.filesize(e.size);this.options.addRemoveLinks&&(e._removeLink=a.createElement('<a class="dz-remove" href="javascript:undefined;" data-dz-remove>'+this.options.dictRemoveFile+"</a>"),e.previewElement.appendChild(e._removeLink));for(var l=function(n){return n.preventDefault(),n.stopPropagation(),e.status===a.UPLOADING?a.confirm(t.options.dictCancelUploadConfirmation,(function(){return t.removeFile(e)})):t.options.dictRemoveFileConfirmation?a.confirm(t.options.dictRemoveFileConfirmation,(function(){return t.removeFile(e)})):t.removeFile(e)},u=0,c=c=e.previewElement.querySelectorAll("[data-dz-remove]");;){if(u>=c.length)break;c[u++].addEventListener("click",l)}}},removedfile:function(e){return null!=e.previewElement&&null!=e.previewElement.parentNode&&e.previewElement.parentNode.removeChild(e.previewElement),this._updateMaxFilesReachedClass()},thumbnail:function(e,t){if(e.previewElement){e.previewElement.classList.remove("dz-file-preview");for(var n=0,r=r=e.previewElement.querySelectorAll("[data-dz-thumbnail]");;){if(n>=r.length)break;var i=r[n++];i.alt=e.name,i.src=t}return setTimeout((function(){return e.previewElement.classList.add("dz-image-preview")}),1)}},error:function(e,t){if(e.previewElement){e.previewElement.classList.add("dz-error"),"String"!=typeof t&&t.error&&(t=t.error);for(var n=0,r=r=e.previewElement.querySelectorAll("[data-dz-errormessage]");;){if(n>=r.length)break;r[n++].textContent=t}}},errormultiple:function(){},processing:function(e){if(e.previewElement&&(e.previewElement.classList.add("dz-processing"),e._removeLink))return e._removeLink.innerHTML=this.options.dictCancelUpload},processingmultiple:function(){},uploadprogress:function(e,t,n){if(e.previewElement)for(var r=0,i=i=e.previewElement.querySelectorAll("[data-dz-uploadprogress]");;){if(r>=i.length)break;var a=i[r++];"PROGRESS"===a.nodeName?a.value=t:a.style.width=t+"%"}},totaluploadprogress:function(){},sending:function(){},sendingmultiple:function(){},success:function(e){if(e.previewElement)return e.previewElement.classList.add("dz-success")},successmultiple:function(){},canceled:function(e){return this.emit("error",e,this.options.dictUploadCanceled)},canceledmultiple:function(){},complete:function(e){if(e._removeLink&&(e._removeLink.innerHTML=this.options.dictRemoveFile),e.previewElement)return e.previewElement.classList.add("dz-complete")},completemultiple:function(){},maxfilesexceeded:function(){},maxfilesreached:function(){},queuecomplete:function(){},addedfiles:function(){}},this.prototype._thumbnailQueue=[],this.prototype._processingThumbnail=!1}},{key:"extend",value:function(e){for(var t=arguments.length,n=Array(t>1?t-1:0),r=1;r<t;r++)n[r-1]=arguments[r];for(var i=0,a=a=n;;){if(i>=a.length)break;var o=a[i++];for(var s in o){var l=o[s];e[s]=l}}return e}}]),t(a,[{key:"getAcceptedFiles",value:function(){return this.files.filter((function(e){return e.accepted})).map((function(e){return e}))}},{key:"getRejectedFiles",value:function(){return this.files.filter((function(e){return!e.accepted})).map((function(e){return e}))}},{key:"getFilesWithStatus",value:function(e){return this.files.filter((function(t){return t.status===e})).map((function(e){return e}))}},{key:"getQueuedFiles",value:function(){return this.getFilesWithStatus(a.QUEUED)}},{key:"getUploadingFiles",value:function(){return this.getFilesWithStatus(a.UPLOADING)}},{key:"getAddedFiles",value:function(){return this.getFilesWithStatus(a.ADDED)}},{key:"getActiveFiles",value:function(){return this.files.filter((function(e){return e.status===a.UPLOADING||e.status===a.QUEUED})).map((function(e){return e}))}},{key:"init",value:function(){var e=this;"form"===this.element.tagName&&this.element.setAttribute("enctype","multipart/form-data"),this.element.classList.contains("dropzone")&&!this.element.querySelector(".dz-message")&&this.element.appendChild(a.createElement('<div class="dz-default dz-message"><span>'+this.options.dictDefaultMessage+"</span></div>")),this.clickableElements.length&&function t(){return e.hiddenFileInput&&e.hiddenFileInput.parentNode.removeChild(e.hiddenFileInput),e.hiddenFileInput=document.createElement("input"),e.hiddenFileInput.setAttribute("type","file"),(null===e.options.maxFiles||e.options.maxFiles>1)&&e.hiddenFileInput.setAttribute("multiple","multiple"),e.hiddenFileInput.className="dz-hidden-input",null!==e.options.acceptedFiles&&e.hiddenFileInput.setAttribute("accept",e.options.acceptedFiles),null!==e.options.capture&&e.hiddenFileInput.setAttribute("capture",e.options.capture),e.hiddenFileInput.style.visibility="hidden",e.hiddenFileInput.style.position="absolute",e.hiddenFileInput.style.top="0",e.hiddenFileInput.style.left="0",e.hiddenFileInput.style.height="0",e.hiddenFileInput.style.width="0",a.getElement(e.options.hiddenInputContainer,"hiddenInputContainer").appendChild(e.hiddenFileInput),e.hiddenFileInput.addEventListener("change",(function(){var n=e.hiddenFileInput.files;if(n.length)for(var r=0,i=i=n;!(r>=i.length);){var a=i[r++];e.addFile(a)}return e.emit("addedfiles",n),t()}))}(),this.URL=null!==window.URL?window.URL:window.webkitURL;for(var t=0,n=n=this.events;;){if(t>=n.length)break;var r=n[t++];this.on(r,this.options[r])}this.on("uploadprogress",(function(){return e.updateTotalUploadProgress()})),this.on("removedfile",(function(){return e.updateTotalUploadProgress()})),this.on("canceled",(function(t){return e.emit("complete",t)})),this.on("complete",(function(t){if(0===e.getAddedFiles().length&&0===e.getUploadingFiles().length&&0===e.getQueuedFiles().length)return setTimeout((function(){return e.emit("queuecomplete")}),0)}));var i=function(e){return e.stopPropagation(),e.preventDefault?e.preventDefault():e.returnValue=!1};return this.listeners=[{element:this.element,events:{dragstart:function(t){return e.emit("dragstart",t)},dragenter:function(t){return i(t),e.emit("dragenter",t)},dragover:function(t){var n=void 0;try{n=t.dataTransfer.effectAllowed}catch(e){}return t.dataTransfer.dropEffect="move"===n||"linkMove"===n?"move":"copy",i(t),e.emit("dragover",t)},dragleave:function(t){return e.emit("dragleave",t)},drop:function(t){return i(t),e.drop(t)},dragend:function(t){return e.emit("dragend",t)}}}],this.clickableElements.forEach((function(t){return e.listeners.push({element:t,events:{click:function(n){return(t!==e.element||n.target===e.element||a.elementInside(n.target,e.element.querySelector(".dz-message")))&&e.hiddenFileInput.click(),!0}}})})),this.enable(),this.options.init.call(this)}},{key:"destroy",value:function(){return this.disable(),this.removeAllFiles(!0),(null!=this.hiddenFileInput?this.hiddenFileInput.parentNode:void 0)&&(this.hiddenFileInput.parentNode.removeChild(this.hiddenFileInput),this.hiddenFileInput=null),delete this.element.dropzone,a.instances.splice(a.instances.indexOf(this),1)}},{key:"updateTotalUploadProgress",value:function(){var e=void 0,t=0,n=0;if(this.getActiveFiles().length){for(var r=0,i=i=this.getActiveFiles();;){if(r>=i.length)break;var a=i[r++];t+=a.upload.bytesSent,n+=a.upload.total}e=100*t/n}else e=100;return this.emit("totaluploadprogress",e,n,t)}},{key:"_getParamName",value:function(e){return"function"==typeof this.options.paramName?this.options.paramName(e):this.options.paramName+(this.options.uploadMultiple?"["+e+"]":"")}},{key:"_renameFile",value:function(e){return"function"!=typeof this.options.renameFile?e.name:this.options.renameFile(e)}},{key:"getFallbackForm",value:function(){var e,t=void 0;if(e=this.getExistingFallback())return e;var n='<div class="dz-fallback">';this.options.dictFallbackText&&(n+="<p>"+this.options.dictFallbackText+"</p>"),n+='<input type="file" name="'+this._getParamName(0)+'" '+(this.options.uploadMultiple?'multiple="multiple"':void 0)+' /><input type="submit" value="Upload!"></div>';var r=a.createElement(n);return"FORM"!==this.element.tagName?(t=a.createElement('<form action="'+this.options.url+'" enctype="multipart/form-data" method="'+this.options.method+'"></form>')).appendChild(r):(this.element.setAttribute("enctype","multipart/form-data"),this.element.setAttribute("method",this.options.method)),null!=t?t:r}},{key:"getExistingFallback",value:function(){for(var e=function(e){for(var t=0,n=n=e;;){if(t>=n.length)break;var r=n[t++];if(/(^| )fallback($| )/.test(r.className))return r}},t=["div","form"],n=0;n<t.length;n++){var r,i=t[n];if(r=e(this.element.getElementsByTagName(i)))return r}}},{key:"setupEventListeners",value:function(){return this.listeners.map((function(e){return function(){var t=[];for(var n in e.events){var r=e.events[n];t.push(e.element.addEventListener(n,r,!1))}return t}()}))}},{key:"removeEventListeners",value:function(){return this.listeners.map((function(e){return function(){var t=[];for(var n in e.events){var r=e.events[n];t.push(e.element.removeEventListener(n,r,!1))}return t}()}))}},{key:"disable",value:function(){var e=this;return this.clickableElements.forEach((function(e){return e.classList.remove("dz-clickable")})),this.removeEventListeners(),this.disabled=!0,this.files.map((function(t){return e.cancelUpload(t)}))}},{key:"enable",value:function(){return delete this.disabled,this.clickableElements.forEach((function(e){return e.classList.add("dz-clickable")})),this.setupEventListeners()}},{key:"filesize",value:function(e){var t=0,n="b";if(e>0){for(var r=["tb","gb","mb","kb","b"],i=0;i<r.length;i++){var a=r[i];if(e>=Math.pow(this.options.filesizeBase,4-i)/10){t=e/Math.pow(this.options.filesizeBase,4-i),n=a;break}}t=Math.round(10*t)/10}return"<strong>"+t+"</strong> "+this.options.dictFileSizeUnits[n]}},{key:"_updateMaxFilesReachedClass",value:function(){return null!=this.options.maxFiles&&this.getAcceptedFiles().length>=this.options.maxFiles?(this.getAcceptedFiles().length===this.options.maxFiles&&this.emit("maxfilesreached",this.files),this.element.classList.add("dz-max-files-reached")):this.element.classList.remove("dz-max-files-reached")}},{key:"drop",value:function(e){if(e.dataTransfer){this.emit("drop",e);for(var t=[],n=0;n<e.dataTransfer.files.length;n++)t[n]=e.dataTransfer.files[n];if(this.emit("addedfiles",t),t.length){var r=e.dataTransfer.items;r&&r.length&&null!=r[0].webkitGetAsEntry?this._addFilesFromItems(r):this.handleFiles(t)}}}},{key:"paste",value:function(e){if(null!=(t=null!=e?e.clipboardData:void 0,n=function(e){return e.items},null!=t?n(t):void 0)){var t,n;this.emit("paste",e);var r=e.clipboardData.items;return r.length?this._addFilesFromItems(r):void 0}}},{key:"handleFiles",value:function(e){for(var t=0,n=n=e;;){if(t>=n.length)break;var r=n[t++];this.addFile(r)}}},{key:"_addFilesFromItems",value:function(e){var t=this;return function(){for(var n=[],r=0,i=i=e;;){if(r>=i.length)break;var a,o=i[r++];null!=o.webkitGetAsEntry&&(a=o.webkitGetAsEntry())?a.isFile?n.push(t.addFile(o.getAsFile())):a.isDirectory?n.push(t._addFilesFromDirectory(a,a.name)):n.push(void 0):null==o.getAsFile||null!=o.kind&&"file"!==o.kind?n.push(void 0):n.push(t.addFile(o.getAsFile()))}return n}()}},{key:"_addFilesFromDirectory",value:function(e,t){var n=this,r=e.createReader(),i=function(e){return t=console,n="log",r=function(t){return t.log(e)},null!=t&&"function"==typeof t[n]?r(t,n):void 0;var t,n,r};return function e(){return r.readEntries((function(r){if(r.length>0){for(var i=0,a=a=r;!(i>=a.length);){var o=a[i++];o.isFile?o.file((function(e){if(!n.options.ignoreHiddenFiles||"."!==e.name.substring(0,1))return e.fullPath=t+"/"+e.name,n.addFile(e)})):o.isDirectory&&n._addFilesFromDirectory(o,t+"/"+o.name)}e()}return null}),i)}()}},{key:"accept",value:function(e,t){return this.options.maxFilesize&&e.size>1024*this.options.maxFilesize*1024?t(this.options.dictFileTooBig.replace("{{filesize}}",Math.round(e.size/1024/10.24)/100).replace("{{maxFilesize}}",this.options.maxFilesize)):a.isValidFile(e,this.options.acceptedFiles)?null!=this.options.maxFiles&&this.getAcceptedFiles().length>=this.options.maxFiles?(t(this.options.dictMaxFilesExceeded.replace("{{maxFiles}}",this.options.maxFiles)),this.emit("maxfilesexceeded",e)):this.options.accept.call(this,e,t):t(this.options.dictInvalidFileType)}},{key:"addFile",value:function(e){var t=this;return e.upload={uuid:a.uuidv4(),progress:0,total:e.size,bytesSent:0,filename:this._renameFile(e),chunked:this.options.chunking&&(this.options.forceChunking||e.size>this.options.chunkSize),totalChunkCount:Math.ceil(e.size/this.options.chunkSize)},this.files.push(e),e.status=a.ADDED,this.emit("addedfile",e),this._enqueueThumbnail(e),this.accept(e,(function(n){return n?(e.accepted=!1,t._errorProcessing([e],n)):(e.accepted=!0,t.options.autoQueue&&t.enqueueFile(e)),t._updateMaxFilesReachedClass()}))}},{key:"enqueueFiles",value:function(e){for(var t=0,n=n=e;;){if(t>=n.length)break;var r=n[t++];this.enqueueFile(r)}return null}},{key:"enqueueFile",value:function(e){var t=this;if(e.status!==a.ADDED||!0!==e.accepted)throw new Error("This file can't be queued because it has already been processed or was rejected.");if(e.status=a.QUEUED,this.options.autoProcessQueue)return setTimeout((function(){return t.processQueue()}),0)}},{key:"_enqueueThumbnail",value:function(e){var t=this;if(this.options.createImageThumbnails&&e.type.match(/image.*/)&&e.size<=1024*this.options.maxThumbnailFilesize*1024)return this._thumbnailQueue.push(e),setTimeout((function(){return t._processThumbnailQueue()}),0)}},{key:"_processThumbnailQueue",value:function(){var e=this;if(!this._processingThumbnail&&0!==this._thumbnailQueue.length){this._processingThumbnail=!0;var t=this._thumbnailQueue.shift();return this.createThumbnail(t,this.options.thumbnailWidth,this.options.thumbnailHeight,this.options.thumbnailMethod,!0,(function(n){return e.emit("thumbnail",t,n),e._processingThumbnail=!1,e._processThumbnailQueue()}))}}},{key:"removeFile",value:function(e){if(e.status===a.UPLOADING&&this.cancelUpload(e),this.files=s(this.files,e),this.emit("removedfile",e),0===this.files.length)return this.emit("reset")}},{key:"removeAllFiles",value:function(e){null==e&&(e=!1);for(var t=0,n=n=this.files.slice();;){if(t>=n.length)break;var r=n[t++];(r.status!==a.UPLOADING||e)&&this.removeFile(r)}return null}},{key:"resizeImage",value:function(e,t,n,r,i){var o=this;return this.createThumbnail(e,t,n,r,!0,(function(t,n){if(null==n)return i(e);var r=o.options.resizeMimeType;null==r&&(r=e.type);var s=n.toDataURL(r,o.options.resizeQuality);return"image/jpeg"!==r&&"image/jpg"!==r||(s=c.restore(e.dataURL,s)),i(a.dataURItoBlob(s))}))}},{key:"createThumbnail",value:function(e,t,n,r,i,a){var o=this,s=new FileReader;return s.onload=function(){if(e.dataURL=s.result,"image/svg+xml"!==e.type)return o.createThumbnailFromUrl(e,t,n,r,i,a);null!=a&&a(s.result)},s.readAsDataURL(e)}},{key:"createThumbnailFromUrl",value:function(e,t,n,r,i,a,o){var s=this,l=document.createElement("img");return o&&(l.crossOrigin=o),l.onload=function(){var o=function(e){return e(1)};return"undefined"!=typeof EXIF&&null!==EXIF&&i&&(o=function(e){return EXIF.getData(l,(function(){return e(EXIF.getTag(this,"Orientation"))}))}),o((function(i){e.width=l.width,e.height=l.height;var o=s.options.resize.call(s,e,t,n,r),c=document.createElement("canvas"),d=c.getContext("2d");switch(c.width=o.trgWidth,c.height=o.trgHeight,i>4&&(c.width=o.trgHeight,c.height=o.trgWidth),i){case 2:d.translate(c.width,0),d.scale(-1,1);break;case 3:d.translate(c.width,c.height),d.rotate(Math.PI);break;case 4:d.translate(0,c.height),d.scale(1,-1);break;case 5:d.rotate(.5*Math.PI),d.scale(1,-1);break;case 6:d.rotate(.5*Math.PI),d.translate(0,-c.width);break;case 7:d.rotate(.5*Math.PI),d.translate(c.height,-c.width),d.scale(-1,1);break;case 8:d.rotate(-.5*Math.PI),d.translate(-c.height,0)}u(d,l,null!=o.srcX?o.srcX:0,null!=o.srcY?o.srcY:0,o.srcWidth,o.srcHeight,null!=o.trgX?o.trgX:0,null!=o.trgY?o.trgY:0,o.trgWidth,o.trgHeight);var f=c.toDataURL("image/png");if(null!=a)return a(f,c)}))},null!=a&&(l.onerror=a),l.src=e.dataURL}},{key:"processQueue",value:function(){var e=this.options.parallelUploads,t=this.getUploadingFiles().length,n=t;if(!(t>=e)){var r=this.getQueuedFiles();if(r.length>0){if(this.options.uploadMultiple)return this.processFiles(r.slice(0,e-t));for(;n<e;){if(!r.length)return;this.processFile(r.shift()),n++}}}}},{key:"processFile",value:function(e){return this.processFiles([e])}},{key:"processFiles",value:function(e){for(var t=0,n=n=e;;){if(t>=n.length)break;var r=n[t++];r.processing=!0,r.status=a.UPLOADING,this.emit("processing",r)}return this.options.uploadMultiple&&this.emit("processingmultiple",e),this.uploadFiles(e)}},{key:"_getFilesWithXhr",value:function(e){return this.files.filter((function(t){return t.xhr===e})).map((function(e){return e}))}},{key:"cancelUpload",value:function(e){if(e.status===a.UPLOADING){for(var t=this._getFilesWithXhr(e.xhr),n=0,r=r=t;;){if(n>=r.length)break;r[n++].status=a.CANCELED}void 0!==e.xhr&&e.xhr.abort();for(var i=0,o=o=t;;){if(i>=o.length)break;var s=o[i++];this.emit("canceled",s)}this.options.uploadMultiple&&this.emit("canceledmultiple",t)}else e.status!==a.ADDED&&e.status!==a.QUEUED||(e.status=a.CANCELED,this.emit("canceled",e),this.options.uploadMultiple&&this.emit("canceledmultiple",[e]));if(this.options.autoProcessQueue)return this.processQueue()}},{key:"resolveOption",value:function(e){if("function"==typeof e){for(var t=arguments.length,n=Array(t>1?t-1:0),r=1;r<t;r++)n[r-1]=arguments[r];return e.apply(this,n)}return e}},{key:"uploadFile",value:function(e){return this.uploadFiles([e])}},{key:"uploadFiles",value:function(e){var t=this;this._transformFiles(e,(function(n){if(e[0].upload.chunked){var r=e[0],i=n[0];r.upload.chunks=[];var o=function(){for(var n=0;void 0!==r.upload.chunks[n];)n++;if(!(n>=r.upload.totalChunkCount)){var o=n*t.options.chunkSize,s=Math.min(o+t.options.chunkSize,r.size),l={name:t._getParamName(0),data:i.webkitSlice?i.webkitSlice(o,s):i.slice(o,s),filename:r.upload.filename,chunkIndex:n};r.upload.chunks[n]={file:r,index:n,dataBlock:l,status:a.UPLOADING,progress:0,retries:0},t._uploadData(e,[l])}};if(r.upload.finishedChunkUpload=function(n){var i=!0;n.status=a.SUCCESS,n.dataBlock=null,n.xhr=null;for(var s=0;s<r.upload.totalChunkCount;s++){if(void 0===r.upload.chunks[s])return o();r.upload.chunks[s].status!==a.SUCCESS&&(i=!1)}i&&t.options.chunksUploaded(r,(function(){t._finished(e,"",null)}))},t.options.parallelChunkUploads)for(var s=0;s<r.upload.totalChunkCount;s++)o();else o()}else{for(var l=[],u=0;u<e.length;u++)l[u]={name:t._getParamName(u),data:n[u],filename:e[u].upload.filename};t._uploadData(e,l)}}))}},{key:"_getChunk",value:function(e,t){for(var n=0;n<e.upload.totalChunkCount;n++)if(void 0!==e.upload.chunks[n]&&e.upload.chunks[n].xhr===t)return e.upload.chunks[n]}},{key:"_uploadData",value:function(e,t){for(var n=this,r=new XMLHttpRequest,i=0,o=o=e;;){if(i>=o.length)break;o[i++].xhr=r}e[0].upload.chunked&&(e[0].upload.chunks[t[0].chunkIndex].xhr=r);var s=this.resolveOption(this.options.method,e),l=this.resolveOption(this.options.url,e);r.open(s,l,!0),r.timeout=this.resolveOption(this.options.timeout,e),r.withCredentials=!!this.options.withCredentials,r.onload=function(t){n._finishedUploading(e,r,t)},r.onerror=function(){n._handleUploadError(e,r)},(null!=r.upload?r.upload:r).onprogress=function(t){return n._updateFilesUploadProgress(e,r,t)};var u={Accept:"application/json","Cache-Control":"no-cache","X-Requested-With":"XMLHttpRequest"};for(var c in this.options.headers&&a.extend(u,this.options.headers),u){var d=u[c];d&&r.setRequestHeader(c,d)}var f=new FormData;if(this.options.params){var h=this.options.params;for(var p in"function"==typeof h&&(h=h.call(this,e,r,e[0].upload.chunked?this._getChunk(e[0],r):null)),h){var m=h[p];f.append(p,m)}}for(var _=0,v=v=e;;){if(_>=v.length)break;var g=v[_++];this.emit("sending",g,r,f)}this.options.uploadMultiple&&this.emit("sendingmultiple",e,r,f),this._addFormElementData(f);for(var b=0;b<t.length;b++){var y=t[b];f.append(y.name,y.data,y.filename)}this.submitRequest(r,f,e)}},{key:"_transformFiles",value:function(e,t){for(var n=this,r=[],i=0,a=function(a){n.options.transformFile.call(n,e[a],(function(n){r[a]=n,++i===e.length&&t(r)}))},o=0;o<e.length;o++)a(o)}},{key:"_addFormElementData",value:function(e){if("FORM"===this.element.tagName)for(var t=0,n=n=this.element.querySelectorAll("input, textarea, select, button");;){if(t>=n.length)break;var r=n[t++],i=r.getAttribute("name"),a=r.getAttribute("type");if(a&&(a=a.toLowerCase()),null!=i)if("SELECT"===r.tagName&&r.hasAttribute("multiple"))for(var o=0,s=s=r.options;;){if(o>=s.length)break;var l=s[o++];l.selected&&e.append(i,l.value)}else(!a||"checkbox"!==a&&"radio"!==a||r.checked)&&e.append(i,r.value)}}},{key:"_updateFilesUploadProgress",value:function(e,t,n){var r=void 0;if(void 0!==n){if(r=100*n.loaded/n.total,e[0].upload.chunked){var i=e[0],a=this._getChunk(i,t);a.progress=r,a.total=n.total,a.bytesSent=n.loaded,i.upload.progress=0,i.upload.total=0,i.upload.bytesSent=0;for(var o=0;o<i.upload.totalChunkCount;o++)void 0!==i.upload.chunks[o]&&void 0!==i.upload.chunks[o].progress&&(i.upload.progress+=i.upload.chunks[o].progress,i.upload.total+=i.upload.chunks[o].total,i.upload.bytesSent+=i.upload.chunks[o].bytesSent);i.upload.progress=i.upload.progress/i.upload.totalChunkCount}else for(var s=0,l=l=e;;){if(s>=l.length)break;var u=l[s++];u.upload.progress=r,u.upload.total=n.total,u.upload.bytesSent=n.loaded}for(var c=0,d=d=e;;){if(c>=d.length)break;var f=d[c++];this.emit("uploadprogress",f,f.upload.progress,f.upload.bytesSent)}}else{var h=!0;r=100;for(var p=0,m=m=e;;){if(p>=m.length)break;var _=m[p++];100===_.upload.progress&&_.upload.bytesSent===_.upload.total||(h=!1),_.upload.progress=r,_.upload.bytesSent=_.upload.total}if(h)return;for(var v=0,g=g=e;;){if(v>=g.length)break;var b=g[v++];this.emit("uploadprogress",b,r,b.upload.bytesSent)}}}},{key:"_finishedUploading",value:function(e,t,n){var r=void 0;if(e[0].status!==a.CANCELED&&4===t.readyState){if("arraybuffer"!==t.responseType&&"blob"!==t.responseType&&(r=t.responseText,t.getResponseHeader("content-type")&&~t.getResponseHeader("content-type").indexOf("application/json")))try{r=JSON.parse(r)}catch(e){n=e,r="Invalid JSON response from server."}this._updateFilesUploadProgress(e),200<=t.status&&t.status<300?e[0].upload.chunked?e[0].upload.finishedChunkUpload(this._getChunk(e[0],t)):this._finished(e,r,n):this._handleUploadError(e,t,r)}}},{key:"_handleUploadError",value:function(e,t,n){if(e[0].status!==a.CANCELED){if(e[0].upload.chunked&&this.options.retryChunks){var r=this._getChunk(e[0],t);if(r.retries++<this.options.retryChunksLimit)return void this._uploadData(e,[r.dataBlock]);console.warn("Retried this chunk too often. Giving up.")}for(var i=0,o=o=e;;){if(i>=o.length)break;o[i++],this._errorProcessing(e,n||this.options.dictResponseError.replace("{{statusCode}}",t.status),t)}}}},{key:"submitRequest",value:function(e,t,n){e.send(t)}},{key:"_finished",value:function(e,t,n){for(var r=0,i=i=e;;){if(r>=i.length)break;var o=i[r++];o.status=a.SUCCESS,this.emit("success",o,t,n),this.emit("complete",o)}if(this.options.uploadMultiple&&(this.emit("successmultiple",e,t,n),this.emit("completemultiple",e)),this.options.autoProcessQueue)return this.processQueue()}},{key:"_errorProcessing",value:function(e,t,n){for(var r=0,i=i=e;;){if(r>=i.length)break;var o=i[r++];o.status=a.ERROR,this.emit("error",o,t,n),this.emit("complete",o)}if(this.options.uploadMultiple&&(this.emit("errormultiple",e,t,n),this.emit("completemultiple",e)),this.options.autoProcessQueue)return this.processQueue()}}],[{key:"uuidv4",value:function(){return"xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx".replace(/[xy]/g,(function(e){var t=16*Math.random()|0;return("x"===e?t:3&t|8).toString(16)}))}}]),a}();a.initClass(),a.version="5.5.1",a.options={},a.optionsForElement=function(e){return e.getAttribute("id")?a.options[l(e.getAttribute("id"))]:void 0},a.instances=[],a.forElement=function(e){if("string"==typeof e&&(e=document.querySelector(e)),null==(null!=e?e.dropzone:void 0))throw new Error("No Dropzone found for given element. This is probably because you're trying to access it before Dropzone had the time to initialize. Use the `init` option to setup any additional observers on your Dropzone.");return e.dropzone},a.autoDiscover=!0,a.discover=function(){var e=void 0;if(document.querySelectorAll)e=document.querySelectorAll(".dropzone");else{e=[];var t=function(t){return function(){for(var n=[],r=0,i=i=t;;){if(r>=i.length)break;var a=i[r++];/(^| )dropzone($| )/.test(a.className)?n.push(e.push(a)):n.push(void 0)}return n}()};t(document.getElementsByTagName("div")),t(document.getElementsByTagName("form"))}return function(){for(var t=[],n=0,r=r=e;;){if(n>=r.length)break;var i=r[n++];!1!==a.optionsForElement(i)?t.push(new a(i)):t.push(void 0)}return t}()},a.blacklistedBrowsers=[/opera.*(Macintosh|Windows Phone).*version\/12/i],a.isBrowserSupported=function(){var e=!0;if(window.File&&window.FileReader&&window.FileList&&window.Blob&&window.FormData&&document.querySelector)if("classList"in document.createElement("a"))for(var t=0,n=n=a.blacklistedBrowsers;;){if(t>=n.length)break;n[t++].test(navigator.userAgent)&&(e=!1)}else e=!1;else e=!1;return e},a.dataURItoBlob=function(e){for(var t=atob(e.split(",")[1]),n=e.split(",")[0].split(":")[1].split(";")[0],r=new ArrayBuffer(t.length),i=new Uint8Array(r),a=0,o=t.length,s=0<=o;s?a<=o:a>=o;s?a++:a--)i[a]=t.charCodeAt(a);return new Blob([r],{type:n})};var s=function(e,t){return e.filter((function(e){return e!==t})).map((function(e){return e}))},l=function(e){return e.replace(/[\-_](\w)/g,(function(e){return e.charAt(1).toUpperCase()}))};a.createElement=function(e){var t=document.createElement("div");return t.innerHTML=e,t.childNodes[0]},a.elementInside=function(e,t){if(e===t)return!0;for(;e=e.parentNode;)if(e===t)return!0;return!1},a.getElement=function(e,t){var n=void 0;if("string"==typeof e?n=document.querySelector(e):null!=e.nodeType&&(n=e),null==n)throw new Error("Invalid `"+t+"` option provided. Please provide a CSS selector or a plain HTML element.");return n},a.getElements=function(e,t){var n=void 0,r=void 0;if(e instanceof Array){r=[];try{for(var i=0,a=a=e;!(i>=a.length);)n=a[i++],r.push(this.getElement(n,t))}catch(e){r=null}}else if("string"==typeof e){r=[];for(var o=0,s=s=document.querySelectorAll(e);!(o>=s.length);)n=s[o++],r.push(n)}else null!=e.nodeType&&(r=[e]);if(null==r||!r.length)throw new Error("Invalid `"+t+"` option provided. Please provide a CSS selector, a plain HTML element or a list of those.");return r},a.confirm=function(e,t,n){return window.confirm(e)?t():null!=n?n():void 0},a.isValidFile=function(e,t){if(!t)return!0;t=t.split(",");for(var n=e.type,r=n.replace(/\/.*$/,""),i=0,a=a=t;;){if(i>=a.length)break;var o=a[i++];if("."===(o=o.trim()).charAt(0)){if(-1!==e.name.toLowerCase().indexOf(o.toLowerCase(),e.name.length-o.length))return!0}else if(/\/\*$/.test(o)){if(r===o.replace(/\/.*$/,""))return!0}else if(n===o)return!0}return!1},"undefined"!=typeof jQuery&&null!==jQuery&&(jQuery.fn.dropzone=function(e){return this.each((function(){return new a(this,e)}))}),null!==e?e.exports=a:window.Dropzone=a,a.ADDED="added",a.QUEUED="queued",a.ACCEPTED=a.QUEUED,a.UPLOADING="uploading",a.PROCESSING=a.UPLOADING,a.CANCELED="canceled",a.ERROR="error",a.SUCCESS="success";var u=function(e,t,n,r,i,a,o,s,l,u){var c=function(e){e.naturalWidth;var t=e.naturalHeight,n=document.createElement("canvas");n.width=1,n.height=t;var r=n.getContext("2d");r.drawImage(e,0,0);for(var i=r.getImageData(1,0,1,t).data,a=0,o=t,s=t;s>a;)0===i[4*(s-1)+3]?o=s:a=s,s=o+a>>1;var l=s/t;return 0===l?1:l}(t);return e.drawImage(t,n,r,i,a,o,s,l,u/c)},c=function(){function e(){r(this,e)}return t(e,null,[{key:"initClass",value:function(){this.KEY_STR="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/="}},{key:"encode64",value:function(e){for(var t="",n=void 0,r=void 0,i="",a=void 0,o=void 0,s=void 0,l="",u=0;a=(n=e[u++])>>2,o=(3&n)<<4|(r=e[u++])>>4,s=(15&r)<<2|(i=e[u++])>>6,l=63&i,isNaN(r)?s=l=64:isNaN(i)&&(l=64),t=t+this.KEY_STR.charAt(a)+this.KEY_STR.charAt(o)+this.KEY_STR.charAt(s)+this.KEY_STR.charAt(l),n=r=i="",a=o=s=l="",u<e.length;);return t}},{key:"restore",value:function(e,t){if(!e.match("data:image/jpeg;base64,"))return t;var n=this.decode64(e.replace("data:image/jpeg;base64,","")),r=this.slice2Segments(n),i=this.exifManipulation(t,r);return"data:image/jpeg;base64,"+this.encode64(i)}},{key:"exifManipulation",value:function(e,t){var n=this.getExifArray(t),r=this.insertExif(e,n);return new Uint8Array(r)}},{key:"getExifArray",value:function(e){for(var t=void 0,n=0;n<e.length;){if(255===(t=e[n])[0]&225===t[1])return t;n++}return[]}},{key:"insertExif",value:function(e,t){var n=e.replace("data:image/jpeg;base64,",""),r=this.decode64(n),i=r.indexOf(255,3),a=r.slice(0,i),o=r.slice(i),s=a;return(s=s.concat(t)).concat(o)}},{key:"slice2Segments",value:function(e){for(var t=0,n=[];;){if(255===e[t]&218===e[t+1])break;if(255===e[t]&216===e[t+1])t+=2;else{var r=t+(256*e[t+2]+e[t+3])+2,i=e.slice(t,r);n.push(i),t=r}if(t>e.length)break}return n}},{key:"decode64",value:function(e){var t=void 0,n=void 0,r="",i=void 0,a=void 0,o="",s=0,l=[];for(/[^A-Za-z0-9\+\/\=]/g.exec(e)&&console.warn("There were invalid base64 characters in the input text.\nValid base64 characters are A-Z, a-z, 0-9, '+', '/',and '='\nExpect errors in decoding."),e=e.replace(/[^A-Za-z0-9\+\/\=]/g,"");t=this.KEY_STR.indexOf(e.charAt(s++))<<2|(i=this.KEY_STR.indexOf(e.charAt(s++)))>>4,n=(15&i)<<4|(a=this.KEY_STR.indexOf(e.charAt(s++)))>>2,r=(3&a)<<6|(o=this.KEY_STR.indexOf(e.charAt(s++))),l.push(t),64!==a&&l.push(n),64!==o&&l.push(r),t=n=r="",i=a=o="",s<e.length;);return l}}]),e}();c.initClass(),a._autoDiscoverFunction=function(){if(a.autoDiscover)return a.discover()},function(e,t){var n=!1,r=!0,i=e.document,a=i.documentElement,o=i.addEventListener?"addEventListener":"attachEvent",s=i.addEventListener?"removeEventListener":"detachEvent",l=i.addEventListener?"":"on",u=function r(a){if("readystatechange"!==a.type||"complete"===i.readyState)return("load"===a.type?e:i)[s](l+a.type,r,!1),!n&&(n=!0)?t.call(e,a.type||a):void 0};if("complete"!==i.readyState){if(i.createEventObject&&a.doScroll){try{r=!e.frameElement}catch(e){}r&&function e(){try{a.doScroll("left")}catch(t){return void setTimeout(e,50)}return u("poll")}()}i[o](l+"DOMContentLoaded",u,!1),i[o](l+"readystatechange",u,!1),e[o](l+"load",u,!1)}}(window,a._autoDiscoverFunction)}(e={exports:{}},e.exports),e.exports),n={getSignedURL:function(e,t){var n={filePath:e.name,contentType:e.type};return new Promise((function(r,i){var o=new FormData,s=new XMLHttpRequest,l="function"==typeof t.signingURL?t.signingURL(e):t.signingURL;s.open("POST",l),s.onload=function(){200==s.status?r(JSON.parse(s.response)):i(s.statusText)},s.onerror=function(e){console.error("Network Error : Could not send request to AWS (Maybe CORS errors)"),i(e)},!0===t.withCredentials&&(s.withCredentials=!0),Object.entries(t.headers||{}).forEach((function(e){var t=a(e,2),n=t[0],r=t[1];s.setRequestHeader(n,r)})),n=Object.assign(n,t.params||{}),Object.entries(n).forEach((function(e){var t=a(e,2),n=t[0],r=t[1];o.append(n,r)})),s.send(o)}))},sendFile:function(e,t,n){var r=n?this.setResponseHandler:this.sendS3Handler;return this.getSignedURL(e,t).then((function(t){return r(t,e)})).catch((function(e){return e}))},setResponseHandler:function(e,t){t.s3Signature=e.signature,t.s3Url=e.postEndpoint},sendS3Handler:function(e,t){var n=new FormData,r=e.signature;return Object.keys(r).forEach((function(e){n.append(e,r[e])})),n.append("file",t),new Promise((function(t,r){var i=new XMLHttpRequest;i.open("POST",e.postEndpoint),i.onload=function(){if(201==i.status){var e=(new window.DOMParser).parseFromString(i.response,"text/xml").firstChild.children[0].innerHTML;t({success:!0,message:e})}else{var n=(new window.DOMParser).parseFromString(i.response,"text/xml").firstChild.children[0].innerHTML;r({success:!1,message:n+". Request is marked as resolved when returns as status 201"})}},i.onerror=function(e){var t=(new window.DOMParser).parseFromString(i.response,"text/xml").firstChild.children[1].innerHTML;r({success:!1,message:t})},i.send(n)}))}};return t.autoDiscover=!1,function(e,t,n,r,i,a,o,s,l,u){"boolean"!=typeof o&&(l=s,s=o,o=!1);var c,d="function"==typeof n?n.options:n;if(e&&e.render&&(d.render=e.render,d.staticRenderFns=e.staticRenderFns,d._compiled=!0,i&&(d.functional=!0)),r&&(d._scopeId=r),a?(c=function(e){(e=e||this.$vnode&&this.$vnode.ssrContext||this.parent&&this.parent.$vnode&&this.parent.$vnode.ssrContext)||"undefined"==typeof __VUE_SSR_CONTEXT__||(e=__VUE_SSR_CONTEXT__),t&&t.call(this,l(e)),e&&e._registeredComponents&&e._registeredComponents.add(a)},d._ssrRegister=c):t&&(c=o?function(){t.call(this,u(this.$root.$options.shadowRoot))}:function(e){t.call(this,s(e))}),c)if(d.functional){var f=d.render;d.render=function(e,t){return c.call(t),f(e,t)}}else{var h=d.beforeCreate;d.beforeCreate=h?[].concat(h,c):[c]}return n}({render:function(){var e=this.$createElement,t=this._self._c||e;return t("div",{ref:"dropzoneElement",class:{"vue-dropzone dropzone":this.includeStyling},attrs:{id:this.id}},[this.useCustomSlot?t("div",{staticClass:"dz-message"},[this._t("default",[this._v("Drop files here to upload")])],2):this._e()])},staticRenderFns:[]},void 0,{props:{id:{type:String,required:!0,default:"dropzone"},options:{type:Object,required:!0},includeStyling:{type:Boolean,default:!0,required:!1},awss3:{type:Object,required:!1,default:null},destroyDropzone:{type:Boolean,default:!0,required:!1},duplicateCheck:{type:Boolean,default:!1,required:!1},useCustomSlot:{type:Boolean,default:!1,required:!1}},data:function(){return{isS3:!1,isS3OverridesServerPropagation:!1,wasQueueAutoProcess:!0}},computed:{dropzoneSettings:function(){var e={thumbnailWidth:200,thumbnailHeight:200};return Object.keys(this.options).forEach((function(t){e[t]=this.options[t]}),this),null!==this.awss3&&(e.autoProcessQueue=!1,this.isS3=!0,this.isS3OverridesServerPropagation=!1===this.awss3.sendFileToServer,void 0!==this.options.autoProcessQueue&&(this.wasQueueAutoProcess=this.options.autoProcessQueue),this.isS3OverridesServerPropagation&&(e.url=function(e){return e[0].s3Url})),e}},mounted:function(){if(!this.$isServer||!this.hasBeenMounted){this.hasBeenMounted=!0,this.dropzone=new t(this.$refs.dropzoneElement,this.dropzoneSettings);var e=this;this.dropzone.on("thumbnail",(function(t,n){e.$emit("vdropzone-thumbnail",t,n)})),this.dropzone.on("addedfile",(function(t){var n,r;if(e.duplicateCheck&&this.files.length)for(n=0,r=this.files.length;n<r-1;n++)this.files[n].name===t.name&&this.files[n].size===t.size&&this.files[n].lastModifiedDate.toString()===t.lastModifiedDate.toString()&&(this.removeFile(t),e.$emit("vdropzone-duplicate-file",t));e.$emit("vdropzone-file-added",t),e.isS3&&e.wasQueueAutoProcess&&!t.manuallyAdded&&e.getSignedAndUploadToS3(t)})),this.dropzone.on("addedfiles",(function(t){e.$emit("vdropzone-files-added",t)})),this.dropzone.on("removedfile",(function(t){e.$emit("vdropzone-removed-file",t),t.manuallyAdded&&null!==e.dropzone.options.maxFiles&&e.dropzone.options.maxFiles++})),this.dropzone.on("success",(function(t,n){if(e.$emit("vdropzone-success",t,n),e.isS3){if(e.isS3OverridesServerPropagation){var r=(new window.DOMParser).parseFromString(n,"text/xml").firstChild.children[0].innerHTML;e.$emit("vdropzone-s3-upload-success",r)}e.wasQueueAutoProcess&&e.setOption("autoProcessQueue",!1)}})),this.dropzone.on("successmultiple",(function(t,n){e.$emit("vdropzone-success-multiple",t,n)})),this.dropzone.on("error",(function(t,n,r){e.$emit("vdropzone-error",t,n,r),this.isS3&&e.$emit("vdropzone-s3-upload-error")})),this.dropzone.on("errormultiple",(function(t,n,r){e.$emit("vdropzone-error-multiple",t,n,r)})),this.dropzone.on("sending",(function(t,n,r){if(e.isS3)if(e.isS3OverridesServerPropagation){var i=t.s3Signature;Object.keys(i).forEach((function(e){r.append(e,i[e])}))}else r.append("s3ObjectLocation",t.s3ObjectLocation);e.$emit("vdropzone-sending",t,n,r)})),this.dropzone.on("sendingmultiple",(function(t,n,r){e.$emit("vdropzone-sending-multiple",t,n,r)})),this.dropzone.on("complete",(function(t){e.$emit("vdropzone-complete",t)})),this.dropzone.on("completemultiple",(function(t){e.$emit("vdropzone-complete-multiple",t)})),this.dropzone.on("canceled",(function(t){e.$emit("vdropzone-canceled",t)})),this.dropzone.on("canceledmultiple",(function(t){e.$emit("vdropzone-canceled-multiple",t)})),this.dropzone.on("maxfilesreached",(function(t){e.$emit("vdropzone-max-files-reached",t)})),this.dropzone.on("maxfilesexceeded",(function(t){e.$emit("vdropzone-max-files-exceeded",t)})),this.dropzone.on("processing",(function(t){e.$emit("vdropzone-processing",t)})),this.dropzone.on("processingmultiple",(function(t){e.$emit("vdropzone-processing-multiple",t)})),this.dropzone.on("uploadprogress",(function(t,n,r){e.$emit("vdropzone-upload-progress",t,n,r)})),this.dropzone.on("totaluploadprogress",(function(t,n,r){e.$emit("vdropzone-total-upload-progress",t,n,r)})),this.dropzone.on("reset",(function(){e.$emit("vdropzone-reset")})),this.dropzone.on("queuecomplete",(function(){e.$emit("vdropzone-queue-complete")})),this.dropzone.on("drop",(function(t){e.$emit("vdropzone-drop",t)})),this.dropzone.on("dragstart",(function(t){e.$emit("vdropzone-drag-start",t)})),this.dropzone.on("dragend",(function(t){e.$emit("vdropzone-drag-end",t)})),this.dropzone.on("dragenter",(function(t){e.$emit("vdropzone-drag-enter",t)})),this.dropzone.on("dragover",(function(t){e.$emit("vdropzone-drag-over",t)})),this.dropzone.on("dragleave",(function(t){e.$emit("vdropzone-drag-leave",t)})),e.$emit("vdropzone-mounted")}},beforeDestroy:function(){this.destroyDropzone&&this.dropzone.destroy()},methods:{manuallyAddFile:function(e,t){e.manuallyAdded=!0,this.dropzone.emit("addedfile",e);var n=!1;if((t.indexOf(".svg")>-1||t.indexOf(".png")>-1||t.indexOf(".jpg")>-1||t.indexOf(".jpeg")>-1||t.indexOf(".gif")>-1||t.indexOf(".webp")>-1)&&(n=!0),this.dropzone.options.createImageThumbnails&&n&&e.size<=1024*this.dropzone.options.maxThumbnailFilesize*1024){t&&this.dropzone.emit("thumbnail",e,t);for(var r=e.previewElement.querySelectorAll("[data-dz-thumbnail]"),i=0;i<r.length;i++)r[i].style.width=this.dropzoneSettings.thumbnailWidth+"px",r[i].style.height=this.dropzoneSettings.thumbnailHeight+"px",r[i].style["object-fit"]="contain"}this.dropzone.emit("complete",e),this.dropzone.options.maxFiles&&this.dropzone.options.maxFiles--,this.dropzone.files.push(e),this.$emit("vdropzone-file-added-manually",e)},setOption:function(e,t){this.dropzone.options[e]=t},removeAllFiles:function(e){this.dropzone.removeAllFiles(e)},processQueue:function(){var e=this,t=this.dropzone;this.isS3&&!this.wasQueueAutoProcess?this.getQueuedFiles().forEach((function(t){e.getSignedAndUploadToS3(t)})):this.dropzone.processQueue(),this.dropzone.on("success",(function(){t.options.autoProcessQueue=!0})),this.dropzone.on("queuecomplete",(function(){t.options.autoProcessQueue=!1}))},init:function(){return this.dropzone.init()},destroy:function(){return this.dropzone.destroy()},updateTotalUploadProgress:function(){return this.dropzone.updateTotalUploadProgress()},getFallbackForm:function(){return this.dropzone.getFallbackForm()},getExistingFallback:function(){return this.dropzone.getExistingFallback()},setupEventListeners:function(){return this.dropzone.setupEventListeners()},removeEventListeners:function(){return this.dropzone.removeEventListeners()},disable:function(){return this.dropzone.disable()},enable:function(){return this.dropzone.enable()},filesize:function(e){return this.dropzone.filesize(e)},accept:function(e,t){return this.dropzone.accept(e,t)},addFile:function(e){return this.dropzone.addFile(e)},removeFile:function(e){this.dropzone.removeFile(e)},getAcceptedFiles:function(){return this.dropzone.getAcceptedFiles()},getRejectedFiles:function(){return this.dropzone.getRejectedFiles()},getFilesWithStatus:function(){return this.dropzone.getFilesWithStatus()},getQueuedFiles:function(){return this.dropzone.getQueuedFiles()},getUploadingFiles:function(){return this.dropzone.getUploadingFiles()},getAddedFiles:function(){return this.dropzone.getAddedFiles()},getActiveFiles:function(){return this.dropzone.getActiveFiles()},getSignedAndUploadToS3:function(e){var t=this,r=n.sendFile(e,this.awss3,this.isS3OverridesServerPropagation);this.isS3OverridesServerPropagation?r.then((function(){setTimeout((function(){return t.dropzone.processFile(e)}))})):r.then((function(n){n.success?(e.s3ObjectLocation=n.message,setTimeout((function(){return t.dropzone.processFile(e)})),t.$emit("vdropzone-s3-upload-success",n.message)):void 0!==n.message?t.$emit("vdropzone-s3-upload-error",n.message):t.$emit("vdropzone-s3-upload-error","Network Error : Could not send request to AWS. (Maybe CORS error)")})),r.catch((function(e){alert(e)}))},setAWSSigningURL:function(e){this.isS3&&(this.awss3.signingURL=e)}}},void 0,!1,void 0,void 0,void 0)}))},9396:function(e,t,n){"use strict";n("1ab7")},"94ca":function(e,t,n){var r=n("d039"),i=/#|\.prototype\./,a=function(e,t){var n=s[o(e)];return n==u||n!=l&&("function"==typeof t?r(t):!!t)},o=a.normalize=function(e){return String(e).replace(i,".").toLowerCase()},s=a.data={},l=a.NATIVE="N",u=a.POLYFILL="P";e.exports=a},"957c":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e,t){var n=e.split("_");return t%10===1&&t%100!==11?n[0]:t%10>=2&&t%10<=4&&(t%100<10||t%100>=20)?n[1]:n[2]}function n(e,n,r){var i={ss:n?"секунда_секунды_секунд":"секунду_секунды_секунд",mm:n?"минута_минуты_минут":"минуту_минуты_минут",hh:"час_часа_часов",dd:"день_дня_дней",ww:"неделя_недели_недель",MM:"месяц_месяца_месяцев",yy:"год_года_лет"};return"m"===r?n?"минута":"минуту":e+" "+t(i[r],+e)}var r=[/^янв/i,/^фев/i,/^мар/i,/^апр/i,/^ма[йя]/i,/^июн/i,/^июл/i,/^авг/i,/^сен/i,/^окт/i,/^ноя/i,/^дек/i],i=e.defineLocale("ru",{months:{format:"января_февраля_марта_апреля_мая_июня_июля_августа_сентября_октября_ноября_декабря".split("_"),standalone:"январь_февраль_март_апрель_май_июнь_июль_август_сентябрь_октябрь_ноябрь_декабрь".split("_")},monthsShort:{format:"янв._февр._мар._апр._мая_июня_июля_авг._сент._окт._нояб._дек.".split("_"),standalone:"янв._февр._март_апр._май_июнь_июль_авг._сент._окт._нояб._дек.".split("_")},weekdays:{standalone:"воскресенье_понедельник_вторник_среда_четверг_пятница_суббота".split("_"),format:"воскресенье_понедельник_вторник_среду_четверг_пятницу_субботу".split("_"),isFormat:/\[ ?[Вв] ?(?:прошлую|следующую|эту)? ?] ?dddd/},weekdaysShort:"вс_пн_вт_ср_чт_пт_сб".split("_"),weekdaysMin:"вс_пн_вт_ср_чт_пт_сб".split("_"),monthsParse:r,longMonthsParse:r,shortMonthsParse:r,monthsRegex:/^(январ[ья]|янв\.?|феврал[ья]|февр?\.?|марта?|мар\.?|апрел[ья]|апр\.?|ма[йя]|июн[ья]|июн\.?|июл[ья]|июл\.?|августа?|авг\.?|сентябр[ья]|сент?\.?|октябр[ья]|окт\.?|ноябр[ья]|нояб?\.?|декабр[ья]|дек\.?)/i,monthsShortRegex:/^(январ[ья]|янв\.?|феврал[ья]|февр?\.?|марта?|мар\.?|апрел[ья]|апр\.?|ма[йя]|июн[ья]|июн\.?|июл[ья]|июл\.?|августа?|авг\.?|сентябр[ья]|сент?\.?|октябр[ья]|окт\.?|ноябр[ья]|нояб?\.?|декабр[ья]|дек\.?)/i,monthsStrictRegex:/^(январ[яь]|феврал[яь]|марта?|апрел[яь]|ма[яй]|июн[яь]|июл[яь]|августа?|сентябр[яь]|октябр[яь]|ноябр[яь]|декабр[яь])/i,monthsShortStrictRegex:/^(янв\.|февр?\.|мар[т.]|апр\.|ма[яй]|июн[ья.]|июл[ья.]|авг\.|сент?\.|окт\.|нояб?\.|дек\.)/i,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY г.",LLL:"D MMMM YYYY г., H:mm",LLLL:"dddd, D MMMM YYYY г., H:mm"},calendar:{sameDay:"[Сегодня, в] LT",nextDay:"[Завтра, в] LT",lastDay:"[Вчера, в] LT",nextWeek:function(e){if(e.week()===this.week())return 2===this.day()?"[Во] dddd, [в] LT":"[В] dddd, [в] LT";switch(this.day()){case 0:return"[В следующее] dddd, [в] LT";case 1:case 2:case 4:return"[В следующий] dddd, [в] LT";case 3:case 5:case 6:return"[В следующую] dddd, [в] LT"}},lastWeek:function(e){if(e.week()===this.week())return 2===this.day()?"[Во] dddd, [в] LT":"[В] dddd, [в] LT";switch(this.day()){case 0:return"[В прошлое] dddd, [в] LT";case 1:case 2:case 4:return"[В прошлый] dddd, [в] LT";case 3:case 5:case 6:return"[В прошлую] dddd, [в] LT"}},sameElse:"L"},relativeTime:{future:"через %s",past:"%s назад",s:"несколько секунд",ss:n,m:n,mm:n,h:"час",hh:n,d:"день",dd:n,w:"неделя",ww:n,M:"месяц",MM:n,y:"год",yy:n},meridiemParse:/ночи|утра|дня|вечера/i,isPM:function(e){return/^(дня|вечера)$/.test(e)},meridiem:function(e,t,n){return e<4?"ночи":e<12?"утра":e<17?"дня":"вечера"},dayOfMonthOrdinalParse:/\d{1,2}-(й|го|я)/,ordinal:function(e,t){switch(t){case"M":case"d":case"DDD":return e+"-й";case"D":return e+"-го";case"w":case"W":return e+"-я";default:return e}},week:{dow:1,doy:4}});return i}))},"958b":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e,t,n,r){switch(n){case"s":return t?"хэдхэн секунд":"хэдхэн секундын";case"ss":return e+(t?" секунд":" секундын");case"m":case"mm":return e+(t?" минут":" минутын");case"h":case"hh":return e+(t?" цаг":" цагийн");case"d":case"dd":return e+(t?" өдөр":" өдрийн");case"M":case"MM":return e+(t?" сар":" сарын");case"y":case"yy":return e+(t?" жил":" жилийн");default:return e}}var n=e.defineLocale("mn",{months:"Нэгдүгээр сар_Хоёрдугаар сар_Гуравдугаар сар_Дөрөвдүгээр сар_Тавдугаар сар_Зургадугаар сар_Долдугаар сар_Наймдугаар сар_Есдүгээр сар_Аравдугаар сар_Арван нэгдүгээр сар_Арван хоёрдугаар сар".split("_"),monthsShort:"1 сар_2 сар_3 сар_4 сар_5 сар_6 сар_7 сар_8 сар_9 сар_10 сар_11 сар_12 сар".split("_"),monthsParseExact:!0,weekdays:"Ням_Даваа_Мягмар_Лхагва_Пүрэв_Баасан_Бямба".split("_"),weekdaysShort:"Ням_Дав_Мяг_Лха_Пүр_Баа_Бям".split("_"),weekdaysMin:"Ня_Да_Мя_Лх_Пү_Ба_Бя".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"YYYY оны MMMMын D",LLL:"YYYY оны MMMMын D HH:mm",LLLL:"dddd, YYYY оны MMMMын D HH:mm"},meridiemParse:/ҮӨ|ҮХ/i,isPM:function(e){return"ҮХ"===e},meridiem:function(e,t,n){return e<12?"ҮӨ":"ҮХ"},calendar:{sameDay:"[Өнөөдөр] LT",nextDay:"[Маргааш] LT",nextWeek:"[Ирэх] dddd LT",lastDay:"[Өчигдөр] LT",lastWeek:"[Өнгөрсөн] dddd LT",sameElse:"L"},relativeTime:{future:"%s дараа",past:"%s өмнө",s:t,ss:t,m:t,mm:t,h:t,hh:t,d:t,dd:t,M:t,MM:t,y:t,yy:t},dayOfMonthOrdinalParse:/\d{1,2} өдөр/,ordinal:function(e,t){switch(t){case"d":case"D":case"DDD":return e+" өдөр";default:return e}}});return n}))},9609:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={0:"-чү",1:"-чи",2:"-чи",3:"-чү",4:"-чү",5:"-чи",6:"-чы",7:"-чи",8:"-чи",9:"-чу",10:"-чу",20:"-чы",30:"-чу",40:"-чы",50:"-чү",60:"-чы",70:"-чи",80:"-чи",90:"-чу",100:"-чү"},n=e.defineLocale("ky",{months:"январь_февраль_март_апрель_май_июнь_июль_август_сентябрь_октябрь_ноябрь_декабрь".split("_"),monthsShort:"янв_фев_март_апр_май_июнь_июль_авг_сен_окт_ноя_дек".split("_"),weekdays:"Жекшемби_Дүйшөмбү_Шейшемби_Шаршемби_Бейшемби_Жума_Ишемби".split("_"),weekdaysShort:"Жек_Дүй_Шей_Шар_Бей_Жум_Ише".split("_"),weekdaysMin:"Жк_Дй_Шй_Шр_Бй_Жм_Иш".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Бүгүн саат] LT",nextDay:"[Эртең саат] LT",nextWeek:"dddd [саат] LT",lastDay:"[Кечээ саат] LT",lastWeek:"[Өткөн аптанын] dddd [күнү] [саат] LT",sameElse:"L"},relativeTime:{future:"%s ичинде",past:"%s мурун",s:"бирнече секунд",ss:"%d секунд",m:"бир мүнөт",mm:"%d мүнөт",h:"бир саат",hh:"%d саат",d:"бир күн",dd:"%d күн",M:"бир ай",MM:"%d ай",y:"бир жыл",yy:"%d жыл"},dayOfMonthOrdinalParse:/\d{1,2}-(чи|чы|чү|чу)/,ordinal:function(e){var n=e%10,r=e>=100?100:null;return e+(t[e]||t[n]||t[r])},week:{dow:1,doy:7}});return n}))},9675:function(e,t,n){"use strict";const r=n("52f6"),i=n("ef5d"),a=n("fb48");function o(e,t){switch(i(e)){case"object":return s(e,t);case"array":return l(e,t);default:return r(e)}}function s(e,t){if("function"===typeof t)return t(e);if(t||a(e)){const n=new e.constructor;for(let r in e)n[r]=o(e[r],t);return n}return e}function l(e,t){const n=new e.constructor(e.length);for(let r=0;r<e.length;r++)n[r]=o(e[r],t);return n}e.exports=o},9686:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"১",2:"২",3:"৩",4:"৪",5:"৫",6:"৬",7:"৭",8:"৮",9:"৯",0:"০"},n={"১":"1","২":"2","৩":"3","৪":"4","৫":"5","৬":"6","৭":"7","৮":"8","৯":"9","০":"0"},r=e.defineLocale("bn-bd",{months:"জানুয়ারি_ফেব্রুয়ারি_মার্চ_এপ্রিল_মে_জুন_জুলাই_আগস্ট_সেপ্টেম্বর_অক্টোবর_নভেম্বর_ডিসেম্বর".split("_"),monthsShort:"জানু_ফেব্রু_মার্চ_এপ্রিল_মে_জুন_জুলাই_আগস্ট_সেপ্ট_অক্টো_নভে_ডিসে".split("_"),weekdays:"রবিবার_সোমবার_মঙ্গলবার_বুধবার_বৃহস্পতিবার_শুক্রবার_শনিবার".split("_"),weekdaysShort:"রবি_সোম_মঙ্গল_বুধ_বৃহস্পতি_শুক্র_শনি".split("_"),weekdaysMin:"রবি_সোম_মঙ্গল_বুধ_বৃহ_শুক্র_শনি".split("_"),longDateFormat:{LT:"A h:mm সময়",LTS:"A h:mm:ss সময়",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm সময়",LLLL:"dddd, D MMMM YYYY, A h:mm সময়"},calendar:{sameDay:"[আজ] LT",nextDay:"[আগামীকাল] LT",nextWeek:"dddd, LT",lastDay:"[গতকাল] LT",lastWeek:"[গত] dddd, LT",sameElse:"L"},relativeTime:{future:"%s পরে",past:"%s আগে",s:"কয়েক সেকেন্ড",ss:"%d সেকেন্ড",m:"এক মিনিট",mm:"%d মিনিট",h:"এক ঘন্টা",hh:"%d ঘন্টা",d:"এক দিন",dd:"%d দিন",M:"এক মাস",MM:"%d মাস",y:"এক বছর",yy:"%d বছর"},preparse:function(e){return e.replace(/[১২৩৪৫৬৭৮৯০]/g,(function(e){return n[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},meridiemParse:/রাত|ভোর|সকাল|দুপুর|বিকাল|সন্ধ্যা|রাত/,meridiemHour:function(e,t){return 12===e&&(e=0),"রাত"===t?e<4?e:e+12:"ভোর"===t||"সকাল"===t?e:"দুপুর"===t?e>=3?e:e+12:"বিকাল"===t||"সন্ধ্যা"===t?e+12:void 0},meridiem:function(e,t,n){return e<4?"রাত":e<6?"ভোর":e<12?"সকাল":e<15?"দুপুর":e<18?"বিকাল":e<20?"সন্ধ্যা":"রাত"},week:{dow:0,doy:6}});return r}))},"971f":function(e,t,n){},"972c":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e,t,n){var r={ss:"secunde",mm:"minute",hh:"ore",dd:"zile",ww:"săptămâni",MM:"luni",yy:"ani"},i=" ";return(e%100>=20||e>=100&&e%100===0)&&(i=" de "),e+i+r[n]}var n=e.defineLocale("ro",{months:"ianuarie_februarie_martie_aprilie_mai_iunie_iulie_august_septembrie_octombrie_noiembrie_decembrie".split("_"),monthsShort:"ian._feb._mart._apr._mai_iun._iul._aug._sept._oct._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"duminică_luni_marți_miercuri_joi_vineri_sâmbătă".split("_"),weekdaysShort:"Dum_Lun_Mar_Mie_Joi_Vin_Sâm".split("_"),weekdaysMin:"Du_Lu_Ma_Mi_Jo_Vi_Sâ".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY H:mm",LLLL:"dddd, D MMMM YYYY H:mm"},calendar:{sameDay:"[azi la] LT",nextDay:"[mâine la] LT",nextWeek:"dddd [la] LT",lastDay:"[ieri la] LT",lastWeek:"[fosta] dddd [la] LT",sameElse:"L"},relativeTime:{future:"peste %s",past:"%s în urmă",s:"câteva secunde",ss:t,m:"un minut",mm:t,h:"o oră",hh:t,d:"o zi",dd:t,w:"o săptămână",ww:t,M:"o lună",MM:t,y:"un an",yy:t},week:{dow:1,doy:7}});return n}))},9797:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("cy",{months:"Ionawr_Chwefror_Mawrth_Ebrill_Mai_Mehefin_Gorffennaf_Awst_Medi_Hydref_Tachwedd_Rhagfyr".split("_"),monthsShort:"Ion_Chwe_Maw_Ebr_Mai_Meh_Gor_Aws_Med_Hyd_Tach_Rhag".split("_"),weekdays:"Dydd Sul_Dydd Llun_Dydd Mawrth_Dydd Mercher_Dydd Iau_Dydd Gwener_Dydd Sadwrn".split("_"),weekdaysShort:"Sul_Llun_Maw_Mer_Iau_Gwe_Sad".split("_"),weekdaysMin:"Su_Ll_Ma_Me_Ia_Gw_Sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Heddiw am] LT",nextDay:"[Yfory am] LT",nextWeek:"dddd [am] LT",lastDay:"[Ddoe am] LT",lastWeek:"dddd [diwethaf am] LT",sameElse:"L"},relativeTime:{future:"mewn %s",past:"%s yn ôl",s:"ychydig eiliadau",ss:"%d eiliad",m:"munud",mm:"%d munud",h:"awr",hh:"%d awr",d:"diwrnod",dd:"%d diwrnod",M:"mis",MM:"%d mis",y:"blwyddyn",yy:"%d flynedd"},dayOfMonthOrdinalParse:/\d{1,2}(fed|ain|af|il|ydd|ed|eg)/,ordinal:function(e){var t=e,n="",r=["","af","il","ydd","ydd","ed","ed","ed","fed","fed","fed","eg","fed","eg","eg","fed","eg","eg","fed","eg","fed"];return t>20?n=40===t||50===t||60===t||80===t||100===t?"fed":"ain":t>0&&(n=r[t]),e+n},week:{dow:1,doy:4}});return t}))},"97ee":function(e,t,n){},9861:function(e,t,n){"use strict";n("e260");var r=n("23e7"),i=n("d066"),a=n("0d3b"),o=n("6eeb"),s=n("e2cc"),l=n("d44e"),u=n("9ed3"),c=n("69f3"),d=n("19aa"),f=n("5135"),h=n("0366"),p=n("f5df"),m=n("825a"),_=n("861d"),v=n("7c73"),g=n("5c6c"),b=n("9a1f"),y=n("35a1"),M=n("b622"),k=i("fetch"),w=i("Headers"),L=M("iterator"),x="URLSearchParams",S=x+"Iterator",T=c.set,D=c.getterFor(x),Y=c.getterFor(S),C=/\+/g,E=Array(4),A=function(e){return E[e-1]||(E[e-1]=RegExp("((?:%[\\da-f]{2}){"+e+"})","gi"))},O=function(e){try{return decodeURIComponent(e)}catch(t){return e}},P=function(e){var t=e.replace(C," "),n=4;try{return decodeURIComponent(t)}catch(r){while(n)t=t.replace(A(n--),O);return t}},F=/[!'()~]|%20/g,I={"!":"%21","'":"%27","(":"%28",")":"%29","~":"%7E","%20":"+"},j=function(e){return I[e]},H=function(e){return encodeURIComponent(e).replace(F,j)},R=function(e,t){if(t){var n,r,i=t.split("&"),a=0;while(a<i.length)n=i[a++],n.length&&(r=n.split("="),e.push({key:P(r.shift()),value:P(r.join("="))}))}},N=function(e){this.entries.length=0,R(this.entries,e)},$=function(e,t){if(e<t)throw TypeError("Not enough arguments")},z=u((function(e,t){T(this,{type:S,iterator:b(D(e).entries),kind:t})}),"Iterator",(function(){var e=Y(this),t=e.kind,n=e.iterator.next(),r=n.value;return n.done||(n.value="keys"===t?r.key:"values"===t?r.value:[r.key,r.value]),n})),B=function(){d(this,B,x);var e,t,n,r,i,a,o,s,l,u=arguments.length>0?arguments[0]:void 0,c=this,h=[];if(T(c,{type:x,entries:h,updateURL:function(){},updateSearchParams:N}),void 0!==u)if(_(u))if(e=y(u),"function"===typeof e){t=e.call(u),n=t.next;while(!(r=n.call(t)).done){if(i=b(m(r.value)),a=i.next,(o=a.call(i)).done||(s=a.call(i)).done||!a.call(i).done)throw TypeError("Expected sequence with length 2");h.push({key:o.value+"",value:s.value+""})}}else for(l in u)f(u,l)&&h.push({key:l,value:u[l]+""});else R(h,"string"===typeof u?"?"===u.charAt(0)?u.slice(1):u:u+"")},W=B.prototype;s(W,{append:function(e,t){$(arguments.length,2);var n=D(this);n.entries.push({key:e+"",value:t+""}),n.updateURL()},delete:function(e){$(arguments.length,1);var t=D(this),n=t.entries,r=e+"",i=0;while(i<n.length)n[i].key===r?n.splice(i,1):i++;t.updateURL()},get:function(e){$(arguments.length,1);for(var t=D(this).entries,n=e+"",r=0;r<t.length;r++)if(t[r].key===n)return t[r].value;return null},getAll:function(e){$(arguments.length,1);for(var t=D(this).entries,n=e+"",r=[],i=0;i<t.length;i++)t[i].key===n&&r.push(t[i].value);return r},has:function(e){$(arguments.length,1);var t=D(this).entries,n=e+"",r=0;while(r<t.length)if(t[r++].key===n)return!0;return!1},set:function(e,t){$(arguments.length,1);for(var n,r=D(this),i=r.entries,a=!1,o=e+"",s=t+"",l=0;l<i.length;l++)n=i[l],n.key===o&&(a?i.splice(l--,1):(a=!0,n.value=s));a||i.push({key:o,value:s}),r.updateURL()},sort:function(){var e,t,n,r=D(this),i=r.entries,a=i.slice();for(i.length=0,n=0;n<a.length;n++){for(e=a[n],t=0;t<n;t++)if(i[t].key>e.key){i.splice(t,0,e);break}t===n&&i.push(e)}r.updateURL()},forEach:function(e){var t,n=D(this).entries,r=h(e,arguments.length>1?arguments[1]:void 0,3),i=0;while(i<n.length)t=n[i++],r(t.value,t.key,this)},keys:function(){return new z(this,"keys")},values:function(){return new z(this,"values")},entries:function(){return new z(this,"entries")}},{enumerable:!0}),o(W,L,W.entries),o(W,"toString",(function(){var e,t=D(this).entries,n=[],r=0;while(r<t.length)e=t[r++],n.push(H(e.key)+"="+H(e.value));return n.join("&")}),{enumerable:!0}),l(B,x),r({global:!0,forced:!a},{URLSearchParams:B}),a||"function"!=typeof k||"function"!=typeof w||r({global:!0,enumerable:!0,forced:!0},{fetch:function(e){var t,n,r,i=[e];return arguments.length>1&&(t=arguments[1],_(t)&&(n=t.body,p(n)===x&&(r=t.headers?new w(t.headers):new w,r.has("content-type")||r.set("content-type","application/x-www-form-urlencoded;charset=UTF-8"),t=v(t,{body:g(0,String(n)),headers:g(0,r)}))),i.push(t)),k.apply(this,i)}}),e.exports={URLSearchParams:B,getState:D}},"99af":function(e,t,n){"use strict";var r=n("23e7"),i=n("d039"),a=n("e8b5"),o=n("861d"),s=n("7b0b"),l=n("50c4"),u=n("8418"),c=n("65f0"),d=n("1dde"),f=n("b622"),h=n("2d00"),p=f("isConcatSpreadable"),m=9007199254740991,_="Maximum allowed index exceeded",v=h>=51||!i((function(){var e=[];return e[p]=!1,e.concat()[0]!==e})),g=d("concat"),b=function(e){if(!o(e))return!1;var t=e[p];return void 0!==t?!!t:a(e)},y=!v||!g;r({target:"Array",proto:!0,forced:y},{concat:function(e){var t,n,r,i,a,o=s(this),d=c(o,0),f=0;for(t=-1,r=arguments.length;t<r;t++)if(a=-1===t?o:arguments[t],b(a)){if(i=l(a.length),f+i>m)throw TypeError(_);for(n=0;n<i;n++,f++)n in a&&u(d,f,a[n])}else{if(f>=m)throw TypeError(_);u(d,f++,a)}return d.length=f,d}})},"9a1f":function(e,t,n){var r=n("825a"),i=n("35a1");e.exports=function(e){var t=i(e);if("function"!=typeof t)throw TypeError(String(e)+" is not iterable");return r(t.call(e))}},"9a5c":function(e,t,n){},"9a70":function(e,t,n){},"9a8c":function(e,t,n){"use strict";var r=n("ebb5"),i=n("145e"),a=r.aTypedArray,o=r.exportTypedArrayMethod;o("copyWithin",(function(e,t){return i.call(a(this),e,t,arguments.length>2?arguments[2]:void 0)}))},"9b36":function(e,t,n){},"9bdd":function(e,t,n){var r=n("825a"),i=n("2a62");e.exports=function(e,t,n,a){try{return a?t(r(n)[0],n[1]):t(n)}catch(o){throw i(e),o}}},"9bf2":function(e,t,n){var r=n("83ab"),i=n("0cfb"),a=n("825a"),o=n("c04e"),s=Object.defineProperty;t.f=r?s:function(e,t,n){if(a(e),t=o(t,!0),a(n),i)try{return s(e,t,n)}catch(r){}if("get"in n||"set"in n)throw TypeError("Accessors not supported");return"value"in n&&(e[t]=n.value),e}},"9d48":function(e,t,n){},"9e3a":function(e,t,n){},"9ed3":function(e,t,n){"use strict";var r=n("ae93").IteratorPrototype,i=n("7c73"),a=n("5c6c"),o=n("d44e"),s=n("3f8c"),l=function(){return this};e.exports=function(e,t,n){var u=t+" Iterator";return e.prototype=i(r,{next:a(1,n)}),o(e,u,!1,!0),s[u]=l,e}},"9f26":function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=/^(janvier|février|mars|avril|mai|juin|juillet|août|septembre|octobre|novembre|décembre)/i,n=/(janv\.?|févr\.?|mars|avr\.?|mai|juin|juil\.?|août|sept\.?|oct\.?|nov\.?|déc\.?)/i,r=/(janv\.?|févr\.?|mars|avr\.?|mai|juin|juil\.?|août|sept\.?|oct\.?|nov\.?|déc\.?|janvier|février|mars|avril|mai|juin|juillet|août|septembre|octobre|novembre|décembre)/i,i=[/^janv/i,/^févr/i,/^mars/i,/^avr/i,/^mai/i,/^juin/i,/^juil/i,/^août/i,/^sept/i,/^oct/i,/^nov/i,/^déc/i],a=e.defineLocale("fr",{months:"janvier_février_mars_avril_mai_juin_juillet_août_septembre_octobre_novembre_décembre".split("_"),monthsShort:"janv._févr._mars_avr._mai_juin_juil._août_sept._oct._nov._déc.".split("_"),monthsRegex:r,monthsShortRegex:r,monthsStrictRegex:t,monthsShortStrictRegex:n,monthsParse:i,longMonthsParse:i,shortMonthsParse:i,weekdays:"dimanche_lundi_mardi_mercredi_jeudi_vendredi_samedi".split("_"),weekdaysShort:"dim._lun._mar._mer._jeu._ven._sam.".split("_"),weekdaysMin:"di_lu_ma_me_je_ve_sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[Aujourd’hui à] LT",nextDay:"[Demain à] LT",nextWeek:"dddd [à] LT",lastDay:"[Hier à] LT",lastWeek:"dddd [dernier à] LT",sameElse:"L"},relativeTime:{future:"dans %s",past:"il y a %s",s:"quelques secondes",ss:"%d secondes",m:"une minute",mm:"%d minutes",h:"une heure",hh:"%d heures",d:"un jour",dd:"%d jours",w:"une semaine",ww:"%d semaines",M:"un mois",MM:"%d mois",y:"un an",yy:"%d ans"},dayOfMonthOrdinalParse:/\d{1,2}(er|)/,ordinal:function(e,t){switch(t){case"D":return e+(1===e?"er":"");default:case"M":case"Q":case"DDD":case"d":return e+(1===e?"er":"e");case"w":case"W":return e+(1===e?"re":"e")}},week:{dow:1,doy:4}});return a}))},"9f7f":function(e,t,n){"use strict";var r=n("d039");function i(e,t){return RegExp(e,t)}t.UNSUPPORTED_Y=r((function(){var e=i("a","y");return e.lastIndex=2,null!=e.exec("abcd")})),t.BROKEN_CARET=r((function(){var e=i("^r","gy");return e.lastIndex=2,null!=e.exec("str")}))},a026:function(e,t,n){"use strict";(function(e){
/*!
 * Vue.js v2.6.12
 * (c) 2014-2020 Evan You
 * Released under the MIT License.
 */
var n=Object.freeze({});function r(e){return void 0===e||null===e}function i(e){return void 0!==e&&null!==e}function a(e){return!0===e}function o(e){return!1===e}function s(e){return"string"===typeof e||"number"===typeof e||"symbol"===typeof e||"boolean"===typeof e}function l(e){return null!==e&&"object"===typeof e}var u=Object.prototype.toString;function c(e){return"[object Object]"===u.call(e)}function d(e){return"[object RegExp]"===u.call(e)}function f(e){var t=parseFloat(String(e));return t>=0&&Math.floor(t)===t&&isFinite(e)}function h(e){return i(e)&&"function"===typeof e.then&&"function"===typeof e.catch}function p(e){return null==e?"":Array.isArray(e)||c(e)&&e.toString===u?JSON.stringify(e,null,2):String(e)}function m(e){var t=parseFloat(e);return isNaN(t)?e:t}function _(e,t){for(var n=Object.create(null),r=e.split(","),i=0;i<r.length;i++)n[r[i]]=!0;return t?function(e){return n[e.toLowerCase()]}:function(e){return n[e]}}var v=_("slot,component",!0),g=_("key,ref,slot,slot-scope,is");function b(e,t){if(e.length){var n=e.indexOf(t);if(n>-1)return e.splice(n,1)}}var y=Object.prototype.hasOwnProperty;function M(e,t){return y.call(e,t)}function k(e){var t=Object.create(null);return function(n){var r=t[n];return r||(t[n]=e(n))}}var w=/-(\w)/g,L=k((function(e){return e.replace(w,(function(e,t){return t?t.toUpperCase():""}))})),x=k((function(e){return e.charAt(0).toUpperCase()+e.slice(1)})),S=/\B([A-Z])/g,T=k((function(e){return e.replace(S,"-$1").toLowerCase()}));function D(e,t){function n(n){var r=arguments.length;return r?r>1?e.apply(t,arguments):e.call(t,n):e.call(t)}return n._length=e.length,n}function Y(e,t){return e.bind(t)}var C=Function.prototype.bind?Y:D;function E(e,t){t=t||0;var n=e.length-t,r=new Array(n);while(n--)r[n]=e[n+t];return r}function A(e,t){for(var n in t)e[n]=t[n];return e}function O(e){for(var t={},n=0;n<e.length;n++)e[n]&&A(t,e[n]);return t}function P(e,t,n){}var F=function(e,t,n){return!1},I=function(e){return e};function j(e){return e.reduce((function(e,t){return e.concat(t.staticKeys||[])}),[]).join(",")}function H(e,t){if(e===t)return!0;var n=l(e),r=l(t);if(!n||!r)return!n&&!r&&String(e)===String(t);try{var i=Array.isArray(e),a=Array.isArray(t);if(i&&a)return e.length===t.length&&e.every((function(e,n){return H(e,t[n])}));if(e instanceof Date&&t instanceof Date)return e.getTime()===t.getTime();if(i||a)return!1;var o=Object.keys(e),s=Object.keys(t);return o.length===s.length&&o.every((function(n){return H(e[n],t[n])}))}catch(u){return!1}}function R(e,t){for(var n=0;n<e.length;n++)if(H(e[n],t))return n;return-1}function N(e){var t=!1;return function(){t||(t=!0,e.apply(this,arguments))}}var $="data-server-rendered",z=["component","directive","filter"],B=["beforeCreate","created","beforeMount","mounted","beforeUpdate","updated","beforeDestroy","destroyed","activated","deactivated","errorCaptured","serverPrefetch"],W={optionMergeStrategies:Object.create(null),silent:!1,productionTip:!1,devtools:!1,performance:!1,errorHandler:null,warnHandler:null,ignoredElements:[],keyCodes:Object.create(null),isReservedTag:F,isReservedAttr:F,isUnknownElement:F,getTagNamespace:P,parsePlatformTagName:I,mustUseProp:F,async:!0,_lifecycleHooks:B},U=/a-zA-Z\u00B7\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u037D\u037F-\u1FFF\u200C-\u200D\u203F-\u2040\u2070-\u218F\u2C00-\u2FEF\u3001-\uD7FF\uF900-\uFDCF\uFDF0-\uFFFD/;function V(e){var t=(e+"").charCodeAt(0);return 36===t||95===t}function q(e,t,n,r){Object.defineProperty(e,t,{value:n,enumerable:!!r,writable:!0,configurable:!0})}var G=new RegExp("[^"+U.source+".$_\\d]");function J(e){if(!G.test(e)){var t=e.split(".");return function(e){for(var n=0;n<t.length;n++){if(!e)return;e=e[t[n]]}return e}}}var K,X="__proto__"in{},Q="undefined"!==typeof window,Z="undefined"!==typeof WXEnvironment&&!!WXEnvironment.platform,ee=Z&&WXEnvironment.platform.toLowerCase(),te=Q&&window.navigator.userAgent.toLowerCase(),ne=te&&/msie|trident/.test(te),re=te&&te.indexOf("msie 9.0")>0,ie=te&&te.indexOf("edge/")>0,ae=(te&&te.indexOf("android"),te&&/iphone|ipad|ipod|ios/.test(te)||"ios"===ee),oe=(te&&/chrome\/\d+/.test(te),te&&/phantomjs/.test(te),te&&te.match(/firefox\/(\d+)/)),se={}.watch,le=!1;if(Q)try{var ue={};Object.defineProperty(ue,"passive",{get:function(){le=!0}}),window.addEventListener("test-passive",null,ue)}catch(Xu){}var ce=function(){return void 0===K&&(K=!Q&&!Z&&"undefined"!==typeof e&&(e["process"]&&"server"===e["process"].env.VUE_ENV)),K},de=Q&&window.__VUE_DEVTOOLS_GLOBAL_HOOK__;function fe(e){return"function"===typeof e&&/native code/.test(e.toString())}var he,pe="undefined"!==typeof Symbol&&fe(Symbol)&&"undefined"!==typeof Reflect&&fe(Reflect.ownKeys);he="undefined"!==typeof Set&&fe(Set)?Set:function(){function e(){this.set=Object.create(null)}return e.prototype.has=function(e){return!0===this.set[e]},e.prototype.add=function(e){this.set[e]=!0},e.prototype.clear=function(){this.set=Object.create(null)},e}();var me=P,_e=0,ve=function(){this.id=_e++,this.subs=[]};ve.prototype.addSub=function(e){this.subs.push(e)},ve.prototype.removeSub=function(e){b(this.subs,e)},ve.prototype.depend=function(){ve.target&&ve.target.addDep(this)},ve.prototype.notify=function(){var e=this.subs.slice();for(var t=0,n=e.length;t<n;t++)e[t].update()},ve.target=null;var ge=[];function be(e){ge.push(e),ve.target=e}function ye(){ge.pop(),ve.target=ge[ge.length-1]}var Me=function(e,t,n,r,i,a,o,s){this.tag=e,this.data=t,this.children=n,this.text=r,this.elm=i,this.ns=void 0,this.context=a,this.fnContext=void 0,this.fnOptions=void 0,this.fnScopeId=void 0,this.key=t&&t.key,this.componentOptions=o,this.componentInstance=void 0,this.parent=void 0,this.raw=!1,this.isStatic=!1,this.isRootInsert=!0,this.isComment=!1,this.isCloned=!1,this.isOnce=!1,this.asyncFactory=s,this.asyncMeta=void 0,this.isAsyncPlaceholder=!1},ke={child:{configurable:!0}};ke.child.get=function(){return this.componentInstance},Object.defineProperties(Me.prototype,ke);var we=function(e){void 0===e&&(e="");var t=new Me;return t.text=e,t.isComment=!0,t};function Le(e){return new Me(void 0,void 0,void 0,String(e))}function xe(e){var t=new Me(e.tag,e.data,e.children&&e.children.slice(),e.text,e.elm,e.context,e.componentOptions,e.asyncFactory);return t.ns=e.ns,t.isStatic=e.isStatic,t.key=e.key,t.isComment=e.isComment,t.fnContext=e.fnContext,t.fnOptions=e.fnOptions,t.fnScopeId=e.fnScopeId,t.asyncMeta=e.asyncMeta,t.isCloned=!0,t}var Se=Array.prototype,Te=Object.create(Se),De=["push","pop","shift","unshift","splice","sort","reverse"];De.forEach((function(e){var t=Se[e];q(Te,e,(function(){var n=[],r=arguments.length;while(r--)n[r]=arguments[r];var i,a=t.apply(this,n),o=this.__ob__;switch(e){case"push":case"unshift":i=n;break;case"splice":i=n.slice(2);break}return i&&o.observeArray(i),o.dep.notify(),a}))}));var Ye=Object.getOwnPropertyNames(Te),Ce=!0;function Ee(e){Ce=e}var Ae=function(e){this.value=e,this.dep=new ve,this.vmCount=0,q(e,"__ob__",this),Array.isArray(e)?(X?Oe(e,Te):Pe(e,Te,Ye),this.observeArray(e)):this.walk(e)};function Oe(e,t){e.__proto__=t}function Pe(e,t,n){for(var r=0,i=n.length;r<i;r++){var a=n[r];q(e,a,t[a])}}function Fe(e,t){var n;if(l(e)&&!(e instanceof Me))return M(e,"__ob__")&&e.__ob__ instanceof Ae?n=e.__ob__:Ce&&!ce()&&(Array.isArray(e)||c(e))&&Object.isExtensible(e)&&!e._isVue&&(n=new Ae(e)),t&&n&&n.vmCount++,n}function Ie(e,t,n,r,i){var a=new ve,o=Object.getOwnPropertyDescriptor(e,t);if(!o||!1!==o.configurable){var s=o&&o.get,l=o&&o.set;s&&!l||2!==arguments.length||(n=e[t]);var u=!i&&Fe(n);Object.defineProperty(e,t,{enumerable:!0,configurable:!0,get:function(){var t=s?s.call(e):n;return ve.target&&(a.depend(),u&&(u.dep.depend(),Array.isArray(t)&&Re(t))),t},set:function(t){var r=s?s.call(e):n;t===r||t!==t&&r!==r||s&&!l||(l?l.call(e,t):n=t,u=!i&&Fe(t),a.notify())}})}}function je(e,t,n){if(Array.isArray(e)&&f(t))return e.length=Math.max(e.length,t),e.splice(t,1,n),n;if(t in e&&!(t in Object.prototype))return e[t]=n,n;var r=e.__ob__;return e._isVue||r&&r.vmCount?n:r?(Ie(r.value,t,n),r.dep.notify(),n):(e[t]=n,n)}function He(e,t){if(Array.isArray(e)&&f(t))e.splice(t,1);else{var n=e.__ob__;e._isVue||n&&n.vmCount||M(e,t)&&(delete e[t],n&&n.dep.notify())}}function Re(e){for(var t=void 0,n=0,r=e.length;n<r;n++)t=e[n],t&&t.__ob__&&t.__ob__.dep.depend(),Array.isArray(t)&&Re(t)}Ae.prototype.walk=function(e){for(var t=Object.keys(e),n=0;n<t.length;n++)Ie(e,t[n])},Ae.prototype.observeArray=function(e){for(var t=0,n=e.length;t<n;t++)Fe(e[t])};var Ne=W.optionMergeStrategies;function $e(e,t){if(!t)return e;for(var n,r,i,a=pe?Reflect.ownKeys(t):Object.keys(t),o=0;o<a.length;o++)n=a[o],"__ob__"!==n&&(r=e[n],i=t[n],M(e,n)?r!==i&&c(r)&&c(i)&&$e(r,i):je(e,n,i));return e}function ze(e,t,n){return n?function(){var r="function"===typeof t?t.call(n,n):t,i="function"===typeof e?e.call(n,n):e;return r?$e(r,i):i}:t?e?function(){return $e("function"===typeof t?t.call(this,this):t,"function"===typeof e?e.call(this,this):e)}:t:e}function Be(e,t){var n=t?e?e.concat(t):Array.isArray(t)?t:[t]:e;return n?We(n):n}function We(e){for(var t=[],n=0;n<e.length;n++)-1===t.indexOf(e[n])&&t.push(e[n]);return t}function Ue(e,t,n,r){var i=Object.create(e||null);return t?A(i,t):i}Ne.data=function(e,t,n){return n?ze(e,t,n):t&&"function"!==typeof t?e:ze(e,t)},B.forEach((function(e){Ne[e]=Be})),z.forEach((function(e){Ne[e+"s"]=Ue})),Ne.watch=function(e,t,n,r){if(e===se&&(e=void 0),t===se&&(t=void 0),!t)return Object.create(e||null);if(!e)return t;var i={};for(var a in A(i,e),t){var o=i[a],s=t[a];o&&!Array.isArray(o)&&(o=[o]),i[a]=o?o.concat(s):Array.isArray(s)?s:[s]}return i},Ne.props=Ne.methods=Ne.inject=Ne.computed=function(e,t,n,r){if(!e)return t;var i=Object.create(null);return A(i,e),t&&A(i,t),i},Ne.provide=ze;var Ve=function(e,t){return void 0===t?e:t};function qe(e,t){var n=e.props;if(n){var r,i,a,o={};if(Array.isArray(n)){r=n.length;while(r--)i=n[r],"string"===typeof i&&(a=L(i),o[a]={type:null})}else if(c(n))for(var s in n)i=n[s],a=L(s),o[a]=c(i)?i:{type:i};else 0;e.props=o}}function Ge(e,t){var n=e.inject;if(n){var r=e.inject={};if(Array.isArray(n))for(var i=0;i<n.length;i++)r[n[i]]={from:n[i]};else if(c(n))for(var a in n){var o=n[a];r[a]=c(o)?A({from:a},o):{from:o}}else 0}}function Je(e){var t=e.directives;if(t)for(var n in t){var r=t[n];"function"===typeof r&&(t[n]={bind:r,update:r})}}function Ke(e,t,n){if("function"===typeof t&&(t=t.options),qe(t,n),Ge(t,n),Je(t),!t._base&&(t.extends&&(e=Ke(e,t.extends,n)),t.mixins))for(var r=0,i=t.mixins.length;r<i;r++)e=Ke(e,t.mixins[r],n);var a,o={};for(a in e)s(a);for(a in t)M(e,a)||s(a);function s(r){var i=Ne[r]||Ve;o[r]=i(e[r],t[r],n,r)}return o}function Xe(e,t,n,r){if("string"===typeof n){var i=e[t];if(M(i,n))return i[n];var a=L(n);if(M(i,a))return i[a];var o=x(a);if(M(i,o))return i[o];var s=i[n]||i[a]||i[o];return s}}function Qe(e,t,n,r){var i=t[e],a=!M(n,e),o=n[e],s=nt(Boolean,i.type);if(s>-1)if(a&&!M(i,"default"))o=!1;else if(""===o||o===T(e)){var l=nt(String,i.type);(l<0||s<l)&&(o=!0)}if(void 0===o){o=Ze(r,i,e);var u=Ce;Ee(!0),Fe(o),Ee(u)}return o}function Ze(e,t,n){if(M(t,"default")){var r=t.default;return e&&e.$options.propsData&&void 0===e.$options.propsData[n]&&void 0!==e._props[n]?e._props[n]:"function"===typeof r&&"Function"!==et(t.type)?r.call(e):r}}function et(e){var t=e&&e.toString().match(/^\s*function (\w+)/);return t?t[1]:""}function tt(e,t){return et(e)===et(t)}function nt(e,t){if(!Array.isArray(t))return tt(t,e)?0:-1;for(var n=0,r=t.length;n<r;n++)if(tt(t[n],e))return n;return-1}function rt(e,t,n){be();try{if(t){var r=t;while(r=r.$parent){var i=r.$options.errorCaptured;if(i)for(var a=0;a<i.length;a++)try{var o=!1===i[a].call(r,e,t,n);if(o)return}catch(Xu){at(Xu,r,"errorCaptured hook")}}}at(e,t,n)}finally{ye()}}function it(e,t,n,r,i){var a;try{a=n?e.apply(t,n):e.call(t),a&&!a._isVue&&h(a)&&!a._handled&&(a.catch((function(e){return rt(e,r,i+" (Promise/async)")})),a._handled=!0)}catch(Xu){rt(Xu,r,i)}return a}function at(e,t,n){if(W.errorHandler)try{return W.errorHandler.call(null,e,t,n)}catch(Xu){Xu!==e&&ot(Xu,null,"config.errorHandler")}ot(e,t,n)}function ot(e,t,n){if(!Q&&!Z||"undefined"===typeof console)throw e;console.error(e)}var st,lt=!1,ut=[],ct=!1;function dt(){ct=!1;var e=ut.slice(0);ut.length=0;for(var t=0;t<e.length;t++)e[t]()}if("undefined"!==typeof Promise&&fe(Promise)){var ft=Promise.resolve();st=function(){ft.then(dt),ae&&setTimeout(P)},lt=!0}else if(ne||"undefined"===typeof MutationObserver||!fe(MutationObserver)&&"[object MutationObserverConstructor]"!==MutationObserver.toString())st="undefined"!==typeof setImmediate&&fe(setImmediate)?function(){setImmediate(dt)}:function(){setTimeout(dt,0)};else{var ht=1,pt=new MutationObserver(dt),mt=document.createTextNode(String(ht));pt.observe(mt,{characterData:!0}),st=function(){ht=(ht+1)%2,mt.data=String(ht)},lt=!0}function _t(e,t){var n;if(ut.push((function(){if(e)try{e.call(t)}catch(Xu){rt(Xu,t,"nextTick")}else n&&n(t)})),ct||(ct=!0,st()),!e&&"undefined"!==typeof Promise)return new Promise((function(e){n=e}))}var vt=new he;function gt(e){bt(e,vt),vt.clear()}function bt(e,t){var n,r,i=Array.isArray(e);if(!(!i&&!l(e)||Object.isFrozen(e)||e instanceof Me)){if(e.__ob__){var a=e.__ob__.dep.id;if(t.has(a))return;t.add(a)}if(i){n=e.length;while(n--)bt(e[n],t)}else{r=Object.keys(e),n=r.length;while(n--)bt(e[r[n]],t)}}}var yt=k((function(e){var t="&"===e.charAt(0);e=t?e.slice(1):e;var n="~"===e.charAt(0);e=n?e.slice(1):e;var r="!"===e.charAt(0);return e=r?e.slice(1):e,{name:e,once:n,capture:r,passive:t}}));function Mt(e,t){function n(){var e=arguments,r=n.fns;if(!Array.isArray(r))return it(r,null,arguments,t,"v-on handler");for(var i=r.slice(),a=0;a<i.length;a++)it(i[a],null,e,t,"v-on handler")}return n.fns=e,n}function kt(e,t,n,i,o,s){var l,u,c,d;for(l in e)u=e[l],c=t[l],d=yt(l),r(u)||(r(c)?(r(u.fns)&&(u=e[l]=Mt(u,s)),a(d.once)&&(u=e[l]=o(d.name,u,d.capture)),n(d.name,u,d.capture,d.passive,d.params)):u!==c&&(c.fns=u,e[l]=c));for(l in t)r(e[l])&&(d=yt(l),i(d.name,t[l],d.capture))}function wt(e,t,n){var o;e instanceof Me&&(e=e.data.hook||(e.data.hook={}));var s=e[t];function l(){n.apply(this,arguments),b(o.fns,l)}r(s)?o=Mt([l]):i(s.fns)&&a(s.merged)?(o=s,o.fns.push(l)):o=Mt([s,l]),o.merged=!0,e[t]=o}function Lt(e,t,n){var a=t.options.props;if(!r(a)){var o={},s=e.attrs,l=e.props;if(i(s)||i(l))for(var u in a){var c=T(u);xt(o,l,u,c,!0)||xt(o,s,u,c,!1)}return o}}function xt(e,t,n,r,a){if(i(t)){if(M(t,n))return e[n]=t[n],a||delete t[n],!0;if(M(t,r))return e[n]=t[r],a||delete t[r],!0}return!1}function St(e){for(var t=0;t<e.length;t++)if(Array.isArray(e[t]))return Array.prototype.concat.apply([],e);return e}function Tt(e){return s(e)?[Le(e)]:Array.isArray(e)?Yt(e):void 0}function Dt(e){return i(e)&&i(e.text)&&o(e.isComment)}function Yt(e,t){var n,o,l,u,c=[];for(n=0;n<e.length;n++)o=e[n],r(o)||"boolean"===typeof o||(l=c.length-1,u=c[l],Array.isArray(o)?o.length>0&&(o=Yt(o,(t||"")+"_"+n),Dt(o[0])&&Dt(u)&&(c[l]=Le(u.text+o[0].text),o.shift()),c.push.apply(c,o)):s(o)?Dt(u)?c[l]=Le(u.text+o):""!==o&&c.push(Le(o)):Dt(o)&&Dt(u)?c[l]=Le(u.text+o.text):(a(e._isVList)&&i(o.tag)&&r(o.key)&&i(t)&&(o.key="__vlist"+t+"_"+n+"__"),c.push(o)));return c}function Ct(e){var t=e.$options.provide;t&&(e._provided="function"===typeof t?t.call(e):t)}function Et(e){var t=At(e.$options.inject,e);t&&(Ee(!1),Object.keys(t).forEach((function(n){Ie(e,n,t[n])})),Ee(!0))}function At(e,t){if(e){for(var n=Object.create(null),r=pe?Reflect.ownKeys(e):Object.keys(e),i=0;i<r.length;i++){var a=r[i];if("__ob__"!==a){var o=e[a].from,s=t;while(s){if(s._provided&&M(s._provided,o)){n[a]=s._provided[o];break}s=s.$parent}if(!s)if("default"in e[a]){var l=e[a].default;n[a]="function"===typeof l?l.call(t):l}else 0}}return n}}function Ot(e,t){if(!e||!e.length)return{};for(var n={},r=0,i=e.length;r<i;r++){var a=e[r],o=a.data;if(o&&o.attrs&&o.attrs.slot&&delete o.attrs.slot,a.context!==t&&a.fnContext!==t||!o||null==o.slot)(n.default||(n.default=[])).push(a);else{var s=o.slot,l=n[s]||(n[s]=[]);"template"===a.tag?l.push.apply(l,a.children||[]):l.push(a)}}for(var u in n)n[u].every(Pt)&&delete n[u];return n}function Pt(e){return e.isComment&&!e.asyncFactory||" "===e.text}function Ft(e,t,r){var i,a=Object.keys(t).length>0,o=e?!!e.$stable:!a,s=e&&e.$key;if(e){if(e._normalized)return e._normalized;if(o&&r&&r!==n&&s===r.$key&&!a&&!r.$hasNormal)return r;for(var l in i={},e)e[l]&&"$"!==l[0]&&(i[l]=It(t,l,e[l]))}else i={};for(var u in t)u in i||(i[u]=jt(t,u));return e&&Object.isExtensible(e)&&(e._normalized=i),q(i,"$stable",o),q(i,"$key",s),q(i,"$hasNormal",a),i}function It(e,t,n){var r=function(){var e=arguments.length?n.apply(null,arguments):n({});return e=e&&"object"===typeof e&&!Array.isArray(e)?[e]:Tt(e),e&&(0===e.length||1===e.length&&e[0].isComment)?void 0:e};return n.proxy&&Object.defineProperty(e,t,{get:r,enumerable:!0,configurable:!0}),r}function jt(e,t){return function(){return e[t]}}function Ht(e,t){var n,r,a,o,s;if(Array.isArray(e)||"string"===typeof e)for(n=new Array(e.length),r=0,a=e.length;r<a;r++)n[r]=t(e[r],r);else if("number"===typeof e)for(n=new Array(e),r=0;r<e;r++)n[r]=t(r+1,r);else if(l(e))if(pe&&e[Symbol.iterator]){n=[];var u=e[Symbol.iterator](),c=u.next();while(!c.done)n.push(t(c.value,n.length)),c=u.next()}else for(o=Object.keys(e),n=new Array(o.length),r=0,a=o.length;r<a;r++)s=o[r],n[r]=t(e[s],s,r);return i(n)||(n=[]),n._isVList=!0,n}function Rt(e,t,n,r){var i,a=this.$scopedSlots[e];a?(n=n||{},r&&(n=A(A({},r),n)),i=a(n)||t):i=this.$slots[e]||t;var o=n&&n.slot;return o?this.$createElement("template",{slot:o},i):i}function Nt(e){return Xe(this.$options,"filters",e,!0)||I}function $t(e,t){return Array.isArray(e)?-1===e.indexOf(t):e!==t}function zt(e,t,n,r,i){var a=W.keyCodes[t]||n;return i&&r&&!W.keyCodes[t]?$t(i,r):a?$t(a,e):r?T(r)!==t:void 0}function Bt(e,t,n,r,i){if(n)if(l(n)){var a;Array.isArray(n)&&(n=O(n));var o=function(o){if("class"===o||"style"===o||g(o))a=e;else{var s=e.attrs&&e.attrs.type;a=r||W.mustUseProp(t,s,o)?e.domProps||(e.domProps={}):e.attrs||(e.attrs={})}var l=L(o),u=T(o);if(!(l in a)&&!(u in a)&&(a[o]=n[o],i)){var c=e.on||(e.on={});c["update:"+o]=function(e){n[o]=e}}};for(var s in n)o(s)}else;return e}function Wt(e,t){var n=this._staticTrees||(this._staticTrees=[]),r=n[e];return r&&!t||(r=n[e]=this.$options.staticRenderFns[e].call(this._renderProxy,null,this),Vt(r,"__static__"+e,!1)),r}function Ut(e,t,n){return Vt(e,"__once__"+t+(n?"_"+n:""),!0),e}function Vt(e,t,n){if(Array.isArray(e))for(var r=0;r<e.length;r++)e[r]&&"string"!==typeof e[r]&&qt(e[r],t+"_"+r,n);else qt(e,t,n)}function qt(e,t,n){e.isStatic=!0,e.key=t,e.isOnce=n}function Gt(e,t){if(t)if(c(t)){var n=e.on=e.on?A({},e.on):{};for(var r in t){var i=n[r],a=t[r];n[r]=i?[].concat(i,a):a}}else;return e}function Jt(e,t,n,r){t=t||{$stable:!n};for(var i=0;i<e.length;i++){var a=e[i];Array.isArray(a)?Jt(a,t,n):a&&(a.proxy&&(a.fn.proxy=!0),t[a.key]=a.fn)}return r&&(t.$key=r),t}function Kt(e,t){for(var n=0;n<t.length;n+=2){var r=t[n];"string"===typeof r&&r&&(e[t[n]]=t[n+1])}return e}function Xt(e,t){return"string"===typeof e?t+e:e}function Qt(e){e._o=Ut,e._n=m,e._s=p,e._l=Ht,e._t=Rt,e._q=H,e._i=R,e._m=Wt,e._f=Nt,e._k=zt,e._b=Bt,e._v=Le,e._e=we,e._u=Jt,e._g=Gt,e._d=Kt,e._p=Xt}function Zt(e,t,r,i,o){var s,l=this,u=o.options;M(i,"_uid")?(s=Object.create(i),s._original=i):(s=i,i=i._original);var c=a(u._compiled),d=!c;this.data=e,this.props=t,this.children=r,this.parent=i,this.listeners=e.on||n,this.injections=At(u.inject,i),this.slots=function(){return l.$slots||Ft(e.scopedSlots,l.$slots=Ot(r,i)),l.$slots},Object.defineProperty(this,"scopedSlots",{enumerable:!0,get:function(){return Ft(e.scopedSlots,this.slots())}}),c&&(this.$options=u,this.$slots=this.slots(),this.$scopedSlots=Ft(e.scopedSlots,this.$slots)),u._scopeId?this._c=function(e,t,n,r){var a=hn(s,e,t,n,r,d);return a&&!Array.isArray(a)&&(a.fnScopeId=u._scopeId,a.fnContext=i),a}:this._c=function(e,t,n,r){return hn(s,e,t,n,r,d)}}function en(e,t,r,a,o){var s=e.options,l={},u=s.props;if(i(u))for(var c in u)l[c]=Qe(c,u,t||n);else i(r.attrs)&&nn(l,r.attrs),i(r.props)&&nn(l,r.props);var d=new Zt(r,l,o,a,e),f=s.render.call(null,d._c,d);if(f instanceof Me)return tn(f,r,d.parent,s,d);if(Array.isArray(f)){for(var h=Tt(f)||[],p=new Array(h.length),m=0;m<h.length;m++)p[m]=tn(h[m],r,d.parent,s,d);return p}}function tn(e,t,n,r,i){var a=xe(e);return a.fnContext=n,a.fnOptions=r,t.slot&&((a.data||(a.data={})).slot=t.slot),a}function nn(e,t){for(var n in t)e[L(n)]=t[n]}Qt(Zt.prototype);var rn={init:function(e,t){if(e.componentInstance&&!e.componentInstance._isDestroyed&&e.data.keepAlive){var n=e;rn.prepatch(n,n)}else{var r=e.componentInstance=sn(e,An);r.$mount(t?e.elm:void 0,t)}},prepatch:function(e,t){var n=t.componentOptions,r=t.componentInstance=e.componentInstance;jn(r,n.propsData,n.listeners,t,n.children)},insert:function(e){var t=e.context,n=e.componentInstance;n._isMounted||(n._isMounted=!0,$n(n,"mounted")),e.data.keepAlive&&(t._isMounted?er(n):Rn(n,!0))},destroy:function(e){var t=e.componentInstance;t._isDestroyed||(e.data.keepAlive?Nn(t,!0):t.$destroy())}},an=Object.keys(rn);function on(e,t,n,o,s){if(!r(e)){var u=n.$options._base;if(l(e)&&(e=u.extend(e)),"function"===typeof e){var c;if(r(e.cid)&&(c=e,e=wn(c,u),void 0===e))return kn(c,t,n,o,s);t=t||{},wr(e),i(t.model)&&cn(e.options,t);var d=Lt(t,e,s);if(a(e.options.functional))return en(e,d,t,n,o);var f=t.on;if(t.on=t.nativeOn,a(e.options.abstract)){var h=t.slot;t={},h&&(t.slot=h)}ln(t);var p=e.options.name||s,m=new Me("vue-component-"+e.cid+(p?"-"+p:""),t,void 0,void 0,void 0,n,{Ctor:e,propsData:d,listeners:f,tag:s,children:o},c);return m}}}function sn(e,t){var n={_isComponent:!0,_parentVnode:e,parent:t},r=e.data.inlineTemplate;return i(r)&&(n.render=r.render,n.staticRenderFns=r.staticRenderFns),new e.componentOptions.Ctor(n)}function ln(e){for(var t=e.hook||(e.hook={}),n=0;n<an.length;n++){var r=an[n],i=t[r],a=rn[r];i===a||i&&i._merged||(t[r]=i?un(a,i):a)}}function un(e,t){var n=function(n,r){e(n,r),t(n,r)};return n._merged=!0,n}function cn(e,t){var n=e.model&&e.model.prop||"value",r=e.model&&e.model.event||"input";(t.attrs||(t.attrs={}))[n]=t.model.value;var a=t.on||(t.on={}),o=a[r],s=t.model.callback;i(o)?(Array.isArray(o)?-1===o.indexOf(s):o!==s)&&(a[r]=[s].concat(o)):a[r]=s}var dn=1,fn=2;function hn(e,t,n,r,i,o){return(Array.isArray(n)||s(n))&&(i=r,r=n,n=void 0),a(o)&&(i=fn),pn(e,t,n,r,i)}function pn(e,t,n,r,a){if(i(n)&&i(n.__ob__))return we();if(i(n)&&i(n.is)&&(t=n.is),!t)return we();var o,s,l;(Array.isArray(r)&&"function"===typeof r[0]&&(n=n||{},n.scopedSlots={default:r[0]},r.length=0),a===fn?r=Tt(r):a===dn&&(r=St(r)),"string"===typeof t)?(s=e.$vnode&&e.$vnode.ns||W.getTagNamespace(t),o=W.isReservedTag(t)?new Me(W.parsePlatformTagName(t),n,r,void 0,void 0,e):n&&n.pre||!i(l=Xe(e.$options,"components",t))?new Me(t,n,r,void 0,void 0,e):on(l,n,e,r,t)):o=on(t,n,e,r);return Array.isArray(o)?o:i(o)?(i(s)&&mn(o,s),i(n)&&_n(n),o):we()}function mn(e,t,n){if(e.ns=t,"foreignObject"===e.tag&&(t=void 0,n=!0),i(e.children))for(var o=0,s=e.children.length;o<s;o++){var l=e.children[o];i(l.tag)&&(r(l.ns)||a(n)&&"svg"!==l.tag)&&mn(l,t,n)}}function _n(e){l(e.style)&&gt(e.style),l(e.class)&&gt(e.class)}function vn(e){e._vnode=null,e._staticTrees=null;var t=e.$options,r=e.$vnode=t._parentVnode,i=r&&r.context;e.$slots=Ot(t._renderChildren,i),e.$scopedSlots=n,e._c=function(t,n,r,i){return hn(e,t,n,r,i,!1)},e.$createElement=function(t,n,r,i){return hn(e,t,n,r,i,!0)};var a=r&&r.data;Ie(e,"$attrs",a&&a.attrs||n,null,!0),Ie(e,"$listeners",t._parentListeners||n,null,!0)}var gn,bn=null;function yn(e){Qt(e.prototype),e.prototype.$nextTick=function(e){return _t(e,this)},e.prototype._render=function(){var e,t=this,n=t.$options,r=n.render,i=n._parentVnode;i&&(t.$scopedSlots=Ft(i.data.scopedSlots,t.$slots,t.$scopedSlots)),t.$vnode=i;try{bn=t,e=r.call(t._renderProxy,t.$createElement)}catch(Xu){rt(Xu,t,"render"),e=t._vnode}finally{bn=null}return Array.isArray(e)&&1===e.length&&(e=e[0]),e instanceof Me||(e=we()),e.parent=i,e}}function Mn(e,t){return(e.__esModule||pe&&"Module"===e[Symbol.toStringTag])&&(e=e.default),l(e)?t.extend(e):e}function kn(e,t,n,r,i){var a=we();return a.asyncFactory=e,a.asyncMeta={data:t,context:n,children:r,tag:i},a}function wn(e,t){if(a(e.error)&&i(e.errorComp))return e.errorComp;if(i(e.resolved))return e.resolved;var n=bn;if(n&&i(e.owners)&&-1===e.owners.indexOf(n)&&e.owners.push(n),a(e.loading)&&i(e.loadingComp))return e.loadingComp;if(n&&!i(e.owners)){var o=e.owners=[n],s=!0,u=null,c=null;n.$on("hook:destroyed",(function(){return b(o,n)}));var d=function(e){for(var t=0,n=o.length;t<n;t++)o[t].$forceUpdate();e&&(o.length=0,null!==u&&(clearTimeout(u),u=null),null!==c&&(clearTimeout(c),c=null))},f=N((function(n){e.resolved=Mn(n,t),s?o.length=0:d(!0)})),p=N((function(t){i(e.errorComp)&&(e.error=!0,d(!0))})),m=e(f,p);return l(m)&&(h(m)?r(e.resolved)&&m.then(f,p):h(m.component)&&(m.component.then(f,p),i(m.error)&&(e.errorComp=Mn(m.error,t)),i(m.loading)&&(e.loadingComp=Mn(m.loading,t),0===m.delay?e.loading=!0:u=setTimeout((function(){u=null,r(e.resolved)&&r(e.error)&&(e.loading=!0,d(!1))}),m.delay||200)),i(m.timeout)&&(c=setTimeout((function(){c=null,r(e.resolved)&&p(null)}),m.timeout)))),s=!1,e.loading?e.loadingComp:e.resolved}}function Ln(e){return e.isComment&&e.asyncFactory}function xn(e){if(Array.isArray(e))for(var t=0;t<e.length;t++){var n=e[t];if(i(n)&&(i(n.componentOptions)||Ln(n)))return n}}function Sn(e){e._events=Object.create(null),e._hasHookEvent=!1;var t=e.$options._parentListeners;t&&Cn(e,t)}function Tn(e,t){gn.$on(e,t)}function Dn(e,t){gn.$off(e,t)}function Yn(e,t){var n=gn;return function r(){var i=t.apply(null,arguments);null!==i&&n.$off(e,r)}}function Cn(e,t,n){gn=e,kt(t,n||{},Tn,Dn,Yn,e),gn=void 0}function En(e){var t=/^hook:/;e.prototype.$on=function(e,n){var r=this;if(Array.isArray(e))for(var i=0,a=e.length;i<a;i++)r.$on(e[i],n);else(r._events[e]||(r._events[e]=[])).push(n),t.test(e)&&(r._hasHookEvent=!0);return r},e.prototype.$once=function(e,t){var n=this;function r(){n.$off(e,r),t.apply(n,arguments)}return r.fn=t,n.$on(e,r),n},e.prototype.$off=function(e,t){var n=this;if(!arguments.length)return n._events=Object.create(null),n;if(Array.isArray(e)){for(var r=0,i=e.length;r<i;r++)n.$off(e[r],t);return n}var a,o=n._events[e];if(!o)return n;if(!t)return n._events[e]=null,n;var s=o.length;while(s--)if(a=o[s],a===t||a.fn===t){o.splice(s,1);break}return n},e.prototype.$emit=function(e){var t=this,n=t._events[e];if(n){n=n.length>1?E(n):n;for(var r=E(arguments,1),i='event handler for "'+e+'"',a=0,o=n.length;a<o;a++)it(n[a],t,r,t,i)}return t}}var An=null;function On(e){var t=An;return An=e,function(){An=t}}function Pn(e){var t=e.$options,n=t.parent;if(n&&!t.abstract){while(n.$options.abstract&&n.$parent)n=n.$parent;n.$children.push(e)}e.$parent=n,e.$root=n?n.$root:e,e.$children=[],e.$refs={},e._watcher=null,e._inactive=null,e._directInactive=!1,e._isMounted=!1,e._isDestroyed=!1,e._isBeingDestroyed=!1}function Fn(e){e.prototype._update=function(e,t){var n=this,r=n.$el,i=n._vnode,a=On(n);n._vnode=e,n.$el=i?n.__patch__(i,e):n.__patch__(n.$el,e,t,!1),a(),r&&(r.__vue__=null),n.$el&&(n.$el.__vue__=n),n.$vnode&&n.$parent&&n.$vnode===n.$parent._vnode&&(n.$parent.$el=n.$el)},e.prototype.$forceUpdate=function(){var e=this;e._watcher&&e._watcher.update()},e.prototype.$destroy=function(){var e=this;if(!e._isBeingDestroyed){$n(e,"beforeDestroy"),e._isBeingDestroyed=!0;var t=e.$parent;!t||t._isBeingDestroyed||e.$options.abstract||b(t.$children,e),e._watcher&&e._watcher.teardown();var n=e._watchers.length;while(n--)e._watchers[n].teardown();e._data.__ob__&&e._data.__ob__.vmCount--,e._isDestroyed=!0,e.__patch__(e._vnode,null),$n(e,"destroyed"),e.$off(),e.$el&&(e.$el.__vue__=null),e.$vnode&&(e.$vnode.parent=null)}}}function In(e,t,n){var r;return e.$el=t,e.$options.render||(e.$options.render=we),$n(e,"beforeMount"),r=function(){e._update(e._render(),n)},new ir(e,r,P,{before:function(){e._isMounted&&!e._isDestroyed&&$n(e,"beforeUpdate")}},!0),n=!1,null==e.$vnode&&(e._isMounted=!0,$n(e,"mounted")),e}function jn(e,t,r,i,a){var o=i.data.scopedSlots,s=e.$scopedSlots,l=!!(o&&!o.$stable||s!==n&&!s.$stable||o&&e.$scopedSlots.$key!==o.$key),u=!!(a||e.$options._renderChildren||l);if(e.$options._parentVnode=i,e.$vnode=i,e._vnode&&(e._vnode.parent=i),e.$options._renderChildren=a,e.$attrs=i.data.attrs||n,e.$listeners=r||n,t&&e.$options.props){Ee(!1);for(var c=e._props,d=e.$options._propKeys||[],f=0;f<d.length;f++){var h=d[f],p=e.$options.props;c[h]=Qe(h,p,t,e)}Ee(!0),e.$options.propsData=t}r=r||n;var m=e.$options._parentListeners;e.$options._parentListeners=r,Cn(e,r,m),u&&(e.$slots=Ot(a,i.context),e.$forceUpdate())}function Hn(e){while(e&&(e=e.$parent))if(e._inactive)return!0;return!1}function Rn(e,t){if(t){if(e._directInactive=!1,Hn(e))return}else if(e._directInactive)return;if(e._inactive||null===e._inactive){e._inactive=!1;for(var n=0;n<e.$children.length;n++)Rn(e.$children[n]);$n(e,"activated")}}function Nn(e,t){if((!t||(e._directInactive=!0,!Hn(e)))&&!e._inactive){e._inactive=!0;for(var n=0;n<e.$children.length;n++)Nn(e.$children[n]);$n(e,"deactivated")}}function $n(e,t){be();var n=e.$options[t],r=t+" hook";if(n)for(var i=0,a=n.length;i<a;i++)it(n[i],e,null,e,r);e._hasHookEvent&&e.$emit("hook:"+t),ye()}var zn=[],Bn=[],Wn={},Un=!1,Vn=!1,qn=0;function Gn(){qn=zn.length=Bn.length=0,Wn={},Un=Vn=!1}var Jn=0,Kn=Date.now;if(Q&&!ne){var Xn=window.performance;Xn&&"function"===typeof Xn.now&&Kn()>document.createEvent("Event").timeStamp&&(Kn=function(){return Xn.now()})}function Qn(){var e,t;for(Jn=Kn(),Vn=!0,zn.sort((function(e,t){return e.id-t.id})),qn=0;qn<zn.length;qn++)e=zn[qn],e.before&&e.before(),t=e.id,Wn[t]=null,e.run();var n=Bn.slice(),r=zn.slice();Gn(),tr(n),Zn(r),de&&W.devtools&&de.emit("flush")}function Zn(e){var t=e.length;while(t--){var n=e[t],r=n.vm;r._watcher===n&&r._isMounted&&!r._isDestroyed&&$n(r,"updated")}}function er(e){e._inactive=!1,Bn.push(e)}function tr(e){for(var t=0;t<e.length;t++)e[t]._inactive=!0,Rn(e[t],!0)}function nr(e){var t=e.id;if(null==Wn[t]){if(Wn[t]=!0,Vn){var n=zn.length-1;while(n>qn&&zn[n].id>e.id)n--;zn.splice(n+1,0,e)}else zn.push(e);Un||(Un=!0,_t(Qn))}}var rr=0,ir=function(e,t,n,r,i){this.vm=e,i&&(e._watcher=this),e._watchers.push(this),r?(this.deep=!!r.deep,this.user=!!r.user,this.lazy=!!r.lazy,this.sync=!!r.sync,this.before=r.before):this.deep=this.user=this.lazy=this.sync=!1,this.cb=n,this.id=++rr,this.active=!0,this.dirty=this.lazy,this.deps=[],this.newDeps=[],this.depIds=new he,this.newDepIds=new he,this.expression="","function"===typeof t?this.getter=t:(this.getter=J(t),this.getter||(this.getter=P)),this.value=this.lazy?void 0:this.get()};ir.prototype.get=function(){var e;be(this);var t=this.vm;try{e=this.getter.call(t,t)}catch(Xu){if(!this.user)throw Xu;rt(Xu,t,'getter for watcher "'+this.expression+'"')}finally{this.deep&&gt(e),ye(),this.cleanupDeps()}return e},ir.prototype.addDep=function(e){var t=e.id;this.newDepIds.has(t)||(this.newDepIds.add(t),this.newDeps.push(e),this.depIds.has(t)||e.addSub(this))},ir.prototype.cleanupDeps=function(){var e=this.deps.length;while(e--){var t=this.deps[e];this.newDepIds.has(t.id)||t.removeSub(this)}var n=this.depIds;this.depIds=this.newDepIds,this.newDepIds=n,this.newDepIds.clear(),n=this.deps,this.deps=this.newDeps,this.newDeps=n,this.newDeps.length=0},ir.prototype.update=function(){this.lazy?this.dirty=!0:this.sync?this.run():nr(this)},ir.prototype.run=function(){if(this.active){var e=this.get();if(e!==this.value||l(e)||this.deep){var t=this.value;if(this.value=e,this.user)try{this.cb.call(this.vm,e,t)}catch(Xu){rt(Xu,this.vm,'callback for watcher "'+this.expression+'"')}else this.cb.call(this.vm,e,t)}}},ir.prototype.evaluate=function(){this.value=this.get(),this.dirty=!1},ir.prototype.depend=function(){var e=this.deps.length;while(e--)this.deps[e].depend()},ir.prototype.teardown=function(){if(this.active){this.vm._isBeingDestroyed||b(this.vm._watchers,this);var e=this.deps.length;while(e--)this.deps[e].removeSub(this);this.active=!1}};var ar={enumerable:!0,configurable:!0,get:P,set:P};function or(e,t,n){ar.get=function(){return this[t][n]},ar.set=function(e){this[t][n]=e},Object.defineProperty(e,n,ar)}function sr(e){e._watchers=[];var t=e.$options;t.props&&lr(e,t.props),t.methods&&_r(e,t.methods),t.data?ur(e):Fe(e._data={},!0),t.computed&&fr(e,t.computed),t.watch&&t.watch!==se&&vr(e,t.watch)}function lr(e,t){var n=e.$options.propsData||{},r=e._props={},i=e.$options._propKeys=[],a=!e.$parent;a||Ee(!1);var o=function(a){i.push(a);var o=Qe(a,t,n,e);Ie(r,a,o),a in e||or(e,"_props",a)};for(var s in t)o(s);Ee(!0)}function ur(e){var t=e.$options.data;t=e._data="function"===typeof t?cr(t,e):t||{},c(t)||(t={});var n=Object.keys(t),r=e.$options.props,i=(e.$options.methods,n.length);while(i--){var a=n[i];0,r&&M(r,a)||V(a)||or(e,"_data",a)}Fe(t,!0)}function cr(e,t){be();try{return e.call(t,t)}catch(Xu){return rt(Xu,t,"data()"),{}}finally{ye()}}var dr={lazy:!0};function fr(e,t){var n=e._computedWatchers=Object.create(null),r=ce();for(var i in t){var a=t[i],o="function"===typeof a?a:a.get;0,r||(n[i]=new ir(e,o||P,P,dr)),i in e||hr(e,i,a)}}function hr(e,t,n){var r=!ce();"function"===typeof n?(ar.get=r?pr(t):mr(n),ar.set=P):(ar.get=n.get?r&&!1!==n.cache?pr(t):mr(n.get):P,ar.set=n.set||P),Object.defineProperty(e,t,ar)}function pr(e){return function(){var t=this._computedWatchers&&this._computedWatchers[e];if(t)return t.dirty&&t.evaluate(),ve.target&&t.depend(),t.value}}function mr(e){return function(){return e.call(this,this)}}function _r(e,t){e.$options.props;for(var n in t)e[n]="function"!==typeof t[n]?P:C(t[n],e)}function vr(e,t){for(var n in t){var r=t[n];if(Array.isArray(r))for(var i=0;i<r.length;i++)gr(e,n,r[i]);else gr(e,n,r)}}function gr(e,t,n,r){return c(n)&&(r=n,n=n.handler),"string"===typeof n&&(n=e[n]),e.$watch(t,n,r)}function br(e){var t={get:function(){return this._data}},n={get:function(){return this._props}};Object.defineProperty(e.prototype,"$data",t),Object.defineProperty(e.prototype,"$props",n),e.prototype.$set=je,e.prototype.$delete=He,e.prototype.$watch=function(e,t,n){var r=this;if(c(t))return gr(r,e,t,n);n=n||{},n.user=!0;var i=new ir(r,e,t,n);if(n.immediate)try{t.call(r,i.value)}catch(a){rt(a,r,'callback for immediate watcher "'+i.expression+'"')}return function(){i.teardown()}}}var yr=0;function Mr(e){e.prototype._init=function(e){var t=this;t._uid=yr++,t._isVue=!0,e&&e._isComponent?kr(t,e):t.$options=Ke(wr(t.constructor),e||{},t),t._renderProxy=t,t._self=t,Pn(t),Sn(t),vn(t),$n(t,"beforeCreate"),Et(t),sr(t),Ct(t),$n(t,"created"),t.$options.el&&t.$mount(t.$options.el)}}function kr(e,t){var n=e.$options=Object.create(e.constructor.options),r=t._parentVnode;n.parent=t.parent,n._parentVnode=r;var i=r.componentOptions;n.propsData=i.propsData,n._parentListeners=i.listeners,n._renderChildren=i.children,n._componentTag=i.tag,t.render&&(n.render=t.render,n.staticRenderFns=t.staticRenderFns)}function wr(e){var t=e.options;if(e.super){var n=wr(e.super),r=e.superOptions;if(n!==r){e.superOptions=n;var i=Lr(e);i&&A(e.extendOptions,i),t=e.options=Ke(n,e.extendOptions),t.name&&(t.components[t.name]=e)}}return t}function Lr(e){var t,n=e.options,r=e.sealedOptions;for(var i in n)n[i]!==r[i]&&(t||(t={}),t[i]=n[i]);return t}function xr(e){this._init(e)}function Sr(e){e.use=function(e){var t=this._installedPlugins||(this._installedPlugins=[]);if(t.indexOf(e)>-1)return this;var n=E(arguments,1);return n.unshift(this),"function"===typeof e.install?e.install.apply(e,n):"function"===typeof e&&e.apply(null,n),t.push(e),this}}function Tr(e){e.mixin=function(e){return this.options=Ke(this.options,e),this}}function Dr(e){e.cid=0;var t=1;e.extend=function(e){e=e||{};var n=this,r=n.cid,i=e._Ctor||(e._Ctor={});if(i[r])return i[r];var a=e.name||n.options.name;var o=function(e){this._init(e)};return o.prototype=Object.create(n.prototype),o.prototype.constructor=o,o.cid=t++,o.options=Ke(n.options,e),o["super"]=n,o.options.props&&Yr(o),o.options.computed&&Cr(o),o.extend=n.extend,o.mixin=n.mixin,o.use=n.use,z.forEach((function(e){o[e]=n[e]})),a&&(o.options.components[a]=o),o.superOptions=n.options,o.extendOptions=e,o.sealedOptions=A({},o.options),i[r]=o,o}}function Yr(e){var t=e.options.props;for(var n in t)or(e.prototype,"_props",n)}function Cr(e){var t=e.options.computed;for(var n in t)hr(e.prototype,n,t[n])}function Er(e){z.forEach((function(t){e[t]=function(e,n){return n?("component"===t&&c(n)&&(n.name=n.name||e,n=this.options._base.extend(n)),"directive"===t&&"function"===typeof n&&(n={bind:n,update:n}),this.options[t+"s"][e]=n,n):this.options[t+"s"][e]}}))}function Ar(e){return e&&(e.Ctor.options.name||e.tag)}function Or(e,t){return Array.isArray(e)?e.indexOf(t)>-1:"string"===typeof e?e.split(",").indexOf(t)>-1:!!d(e)&&e.test(t)}function Pr(e,t){var n=e.cache,r=e.keys,i=e._vnode;for(var a in n){var o=n[a];if(o){var s=Ar(o.componentOptions);s&&!t(s)&&Fr(n,a,r,i)}}}function Fr(e,t,n,r){var i=e[t];!i||r&&i.tag===r.tag||i.componentInstance.$destroy(),e[t]=null,b(n,t)}Mr(xr),br(xr),En(xr),Fn(xr),yn(xr);var Ir=[String,RegExp,Array],jr={name:"keep-alive",abstract:!0,props:{include:Ir,exclude:Ir,max:[String,Number]},created:function(){this.cache=Object.create(null),this.keys=[]},destroyed:function(){for(var e in this.cache)Fr(this.cache,e,this.keys)},mounted:function(){var e=this;this.$watch("include",(function(t){Pr(e,(function(e){return Or(t,e)}))})),this.$watch("exclude",(function(t){Pr(e,(function(e){return!Or(t,e)}))}))},render:function(){var e=this.$slots.default,t=xn(e),n=t&&t.componentOptions;if(n){var r=Ar(n),i=this,a=i.include,o=i.exclude;if(a&&(!r||!Or(a,r))||o&&r&&Or(o,r))return t;var s=this,l=s.cache,u=s.keys,c=null==t.key?n.Ctor.cid+(n.tag?"::"+n.tag:""):t.key;l[c]?(t.componentInstance=l[c].componentInstance,b(u,c),u.push(c)):(l[c]=t,u.push(c),this.max&&u.length>parseInt(this.max)&&Fr(l,u[0],u,this._vnode)),t.data.keepAlive=!0}return t||e&&e[0]}},Hr={KeepAlive:jr};function Rr(e){var t={get:function(){return W}};Object.defineProperty(e,"config",t),e.util={warn:me,extend:A,mergeOptions:Ke,defineReactive:Ie},e.set=je,e.delete=He,e.nextTick=_t,e.observable=function(e){return Fe(e),e},e.options=Object.create(null),z.forEach((function(t){e.options[t+"s"]=Object.create(null)})),e.options._base=e,A(e.options.components,Hr),Sr(e),Tr(e),Dr(e),Er(e)}Rr(xr),Object.defineProperty(xr.prototype,"$isServer",{get:ce}),Object.defineProperty(xr.prototype,"$ssrContext",{get:function(){return this.$vnode&&this.$vnode.ssrContext}}),Object.defineProperty(xr,"FunctionalRenderContext",{value:Zt}),xr.version="2.6.12";var Nr=_("style,class"),$r=_("input,textarea,option,select,progress"),zr=function(e,t,n){return"value"===n&&$r(e)&&"button"!==t||"selected"===n&&"option"===e||"checked"===n&&"input"===e||"muted"===n&&"video"===e},Br=_("contenteditable,draggable,spellcheck"),Wr=_("events,caret,typing,plaintext-only"),Ur=function(e,t){return Kr(t)||"false"===t?"false":"contenteditable"===e&&Wr(t)?t:"true"},Vr=_("allowfullscreen,async,autofocus,autoplay,checked,compact,controls,declare,default,defaultchecked,defaultmuted,defaultselected,defer,disabled,enabled,formnovalidate,hidden,indeterminate,inert,ismap,itemscope,loop,multiple,muted,nohref,noresize,noshade,novalidate,nowrap,open,pauseonexit,readonly,required,reversed,scoped,seamless,selected,sortable,translate,truespeed,typemustmatch,visible"),qr="http://www.w3.org/1999/xlink",Gr=function(e){return":"===e.charAt(5)&&"xlink"===e.slice(0,5)},Jr=function(e){return Gr(e)?e.slice(6,e.length):""},Kr=function(e){return null==e||!1===e};function Xr(e){var t=e.data,n=e,r=e;while(i(r.componentInstance))r=r.componentInstance._vnode,r&&r.data&&(t=Qr(r.data,t));while(i(n=n.parent))n&&n.data&&(t=Qr(t,n.data));return Zr(t.staticClass,t.class)}function Qr(e,t){return{staticClass:ei(e.staticClass,t.staticClass),class:i(e.class)?[e.class,t.class]:t.class}}function Zr(e,t){return i(e)||i(t)?ei(e,ti(t)):""}function ei(e,t){return e?t?e+" "+t:e:t||""}function ti(e){return Array.isArray(e)?ni(e):l(e)?ri(e):"string"===typeof e?e:""}function ni(e){for(var t,n="",r=0,a=e.length;r<a;r++)i(t=ti(e[r]))&&""!==t&&(n&&(n+=" "),n+=t);return n}function ri(e){var t="";for(var n in e)e[n]&&(t&&(t+=" "),t+=n);return t}var ii={svg:"http://www.w3.org/2000/svg",math:"http://www.w3.org/1998/Math/MathML"},ai=_("html,body,base,head,link,meta,style,title,address,article,aside,footer,header,h1,h2,h3,h4,h5,h6,hgroup,nav,section,div,dd,dl,dt,figcaption,figure,picture,hr,img,li,main,ol,p,pre,ul,a,b,abbr,bdi,bdo,br,cite,code,data,dfn,em,i,kbd,mark,q,rp,rt,rtc,ruby,s,samp,small,span,strong,sub,sup,time,u,var,wbr,area,audio,map,track,video,embed,object,param,source,canvas,script,noscript,del,ins,caption,col,colgroup,table,thead,tbody,td,th,tr,button,datalist,fieldset,form,input,label,legend,meter,optgroup,option,output,progress,select,textarea,details,dialog,menu,menuitem,summary,content,element,shadow,template,blockquote,iframe,tfoot"),oi=_("svg,animate,circle,clippath,cursor,defs,desc,ellipse,filter,font-face,foreignObject,g,glyph,image,line,marker,mask,missing-glyph,path,pattern,polygon,polyline,rect,switch,symbol,text,textpath,tspan,use,view",!0),si=function(e){return"pre"===e},li=function(e){return ai(e)||oi(e)};function ui(e){return oi(e)?"svg":"math"===e?"math":void 0}var ci=Object.create(null);function di(e){if(!Q)return!0;if(li(e))return!1;if(e=e.toLowerCase(),null!=ci[e])return ci[e];var t=document.createElement(e);return e.indexOf("-")>-1?ci[e]=t.constructor===window.HTMLUnknownElement||t.constructor===window.HTMLElement:ci[e]=/HTMLUnknownElement/.test(t.toString())}var fi=_("text,number,password,search,email,tel,url");function hi(e){if("string"===typeof e){var t=document.querySelector(e);return t||document.createElement("div")}return e}function pi(e,t){var n=document.createElement(e);return"select"!==e||t.data&&t.data.attrs&&void 0!==t.data.attrs.multiple&&n.setAttribute("multiple","multiple"),n}function mi(e,t){return document.createElementNS(ii[e],t)}function _i(e){return document.createTextNode(e)}function vi(e){return document.createComment(e)}function gi(e,t,n){e.insertBefore(t,n)}function bi(e,t){e.removeChild(t)}function yi(e,t){e.appendChild(t)}function Mi(e){return e.parentNode}function ki(e){return e.nextSibling}function wi(e){return e.tagName}function Li(e,t){e.textContent=t}function xi(e,t){e.setAttribute(t,"")}var Si=Object.freeze({createElement:pi,createElementNS:mi,createTextNode:_i,createComment:vi,insertBefore:gi,removeChild:bi,appendChild:yi,parentNode:Mi,nextSibling:ki,tagName:wi,setTextContent:Li,setStyleScope:xi}),Ti={create:function(e,t){Di(t)},update:function(e,t){e.data.ref!==t.data.ref&&(Di(e,!0),Di(t))},destroy:function(e){Di(e,!0)}};function Di(e,t){var n=e.data.ref;if(i(n)){var r=e.context,a=e.componentInstance||e.elm,o=r.$refs;t?Array.isArray(o[n])?b(o[n],a):o[n]===a&&(o[n]=void 0):e.data.refInFor?Array.isArray(o[n])?o[n].indexOf(a)<0&&o[n].push(a):o[n]=[a]:o[n]=a}}var Yi=new Me("",{},[]),Ci=["create","activate","update","remove","destroy"];function Ei(e,t){return e.key===t.key&&(e.tag===t.tag&&e.isComment===t.isComment&&i(e.data)===i(t.data)&&Ai(e,t)||a(e.isAsyncPlaceholder)&&e.asyncFactory===t.asyncFactory&&r(t.asyncFactory.error))}function Ai(e,t){if("input"!==e.tag)return!0;var n,r=i(n=e.data)&&i(n=n.attrs)&&n.type,a=i(n=t.data)&&i(n=n.attrs)&&n.type;return r===a||fi(r)&&fi(a)}function Oi(e,t,n){var r,a,o={};for(r=t;r<=n;++r)a=e[r].key,i(a)&&(o[a]=r);return o}function Pi(e){var t,n,o={},l=e.modules,u=e.nodeOps;for(t=0;t<Ci.length;++t)for(o[Ci[t]]=[],n=0;n<l.length;++n)i(l[n][Ci[t]])&&o[Ci[t]].push(l[n][Ci[t]]);function c(e){return new Me(u.tagName(e).toLowerCase(),{},[],void 0,e)}function d(e,t){function n(){0===--n.listeners&&f(e)}return n.listeners=t,n}function f(e){var t=u.parentNode(e);i(t)&&u.removeChild(t,e)}function h(e,t,n,r,o,s,l){if(i(e.elm)&&i(s)&&(e=s[l]=xe(e)),e.isRootInsert=!o,!p(e,t,n,r)){var c=e.data,d=e.children,f=e.tag;i(f)?(e.elm=e.ns?u.createElementNS(e.ns,f):u.createElement(f,e),k(e),b(e,d,t),i(c)&&M(e,t),g(n,e.elm,r)):a(e.isComment)?(e.elm=u.createComment(e.text),g(n,e.elm,r)):(e.elm=u.createTextNode(e.text),g(n,e.elm,r))}}function p(e,t,n,r){var o=e.data;if(i(o)){var s=i(e.componentInstance)&&o.keepAlive;if(i(o=o.hook)&&i(o=o.init)&&o(e,!1),i(e.componentInstance))return m(e,t),g(n,e.elm,r),a(s)&&v(e,t,n,r),!0}}function m(e,t){i(e.data.pendingInsert)&&(t.push.apply(t,e.data.pendingInsert),e.data.pendingInsert=null),e.elm=e.componentInstance.$el,y(e)?(M(e,t),k(e)):(Di(e),t.push(e))}function v(e,t,n,r){var a,s=e;while(s.componentInstance)if(s=s.componentInstance._vnode,i(a=s.data)&&i(a=a.transition)){for(a=0;a<o.activate.length;++a)o.activate[a](Yi,s);t.push(s);break}g(n,e.elm,r)}function g(e,t,n){i(e)&&(i(n)?u.parentNode(n)===e&&u.insertBefore(e,t,n):u.appendChild(e,t))}function b(e,t,n){if(Array.isArray(t)){0;for(var r=0;r<t.length;++r)h(t[r],n,e.elm,null,!0,t,r)}else s(e.text)&&u.appendChild(e.elm,u.createTextNode(String(e.text)))}function y(e){while(e.componentInstance)e=e.componentInstance._vnode;return i(e.tag)}function M(e,n){for(var r=0;r<o.create.length;++r)o.create[r](Yi,e);t=e.data.hook,i(t)&&(i(t.create)&&t.create(Yi,e),i(t.insert)&&n.push(e))}function k(e){var t;if(i(t=e.fnScopeId))u.setStyleScope(e.elm,t);else{var n=e;while(n)i(t=n.context)&&i(t=t.$options._scopeId)&&u.setStyleScope(e.elm,t),n=n.parent}i(t=An)&&t!==e.context&&t!==e.fnContext&&i(t=t.$options._scopeId)&&u.setStyleScope(e.elm,t)}function w(e,t,n,r,i,a){for(;r<=i;++r)h(n[r],a,e,t,!1,n,r)}function L(e){var t,n,r=e.data;if(i(r))for(i(t=r.hook)&&i(t=t.destroy)&&t(e),t=0;t<o.destroy.length;++t)o.destroy[t](e);if(i(t=e.children))for(n=0;n<e.children.length;++n)L(e.children[n])}function x(e,t,n){for(;t<=n;++t){var r=e[t];i(r)&&(i(r.tag)?(S(r),L(r)):f(r.elm))}}function S(e,t){if(i(t)||i(e.data)){var n,r=o.remove.length+1;for(i(t)?t.listeners+=r:t=d(e.elm,r),i(n=e.componentInstance)&&i(n=n._vnode)&&i(n.data)&&S(n,t),n=0;n<o.remove.length;++n)o.remove[n](e,t);i(n=e.data.hook)&&i(n=n.remove)?n(e,t):t()}else f(e.elm)}function T(e,t,n,a,o){var s,l,c,d,f=0,p=0,m=t.length-1,_=t[0],v=t[m],g=n.length-1,b=n[0],y=n[g],M=!o;while(f<=m&&p<=g)r(_)?_=t[++f]:r(v)?v=t[--m]:Ei(_,b)?(Y(_,b,a,n,p),_=t[++f],b=n[++p]):Ei(v,y)?(Y(v,y,a,n,g),v=t[--m],y=n[--g]):Ei(_,y)?(Y(_,y,a,n,g),M&&u.insertBefore(e,_.elm,u.nextSibling(v.elm)),_=t[++f],y=n[--g]):Ei(v,b)?(Y(v,b,a,n,p),M&&u.insertBefore(e,v.elm,_.elm),v=t[--m],b=n[++p]):(r(s)&&(s=Oi(t,f,m)),l=i(b.key)?s[b.key]:D(b,t,f,m),r(l)?h(b,a,e,_.elm,!1,n,p):(c=t[l],Ei(c,b)?(Y(c,b,a,n,p),t[l]=void 0,M&&u.insertBefore(e,c.elm,_.elm)):h(b,a,e,_.elm,!1,n,p)),b=n[++p]);f>m?(d=r(n[g+1])?null:n[g+1].elm,w(e,d,n,p,g,a)):p>g&&x(t,f,m)}function D(e,t,n,r){for(var a=n;a<r;a++){var o=t[a];if(i(o)&&Ei(e,o))return a}}function Y(e,t,n,s,l,c){if(e!==t){i(t.elm)&&i(s)&&(t=s[l]=xe(t));var d=t.elm=e.elm;if(a(e.isAsyncPlaceholder))i(t.asyncFactory.resolved)?A(e.elm,t,n):t.isAsyncPlaceholder=!0;else if(a(t.isStatic)&&a(e.isStatic)&&t.key===e.key&&(a(t.isCloned)||a(t.isOnce)))t.componentInstance=e.componentInstance;else{var f,h=t.data;i(h)&&i(f=h.hook)&&i(f=f.prepatch)&&f(e,t);var p=e.children,m=t.children;if(i(h)&&y(t)){for(f=0;f<o.update.length;++f)o.update[f](e,t);i(f=h.hook)&&i(f=f.update)&&f(e,t)}r(t.text)?i(p)&&i(m)?p!==m&&T(d,p,m,n,c):i(m)?(i(e.text)&&u.setTextContent(d,""),w(d,null,m,0,m.length-1,n)):i(p)?x(p,0,p.length-1):i(e.text)&&u.setTextContent(d,""):e.text!==t.text&&u.setTextContent(d,t.text),i(h)&&i(f=h.hook)&&i(f=f.postpatch)&&f(e,t)}}}function C(e,t,n){if(a(n)&&i(e.parent))e.parent.data.pendingInsert=t;else for(var r=0;r<t.length;++r)t[r].data.hook.insert(t[r])}var E=_("attrs,class,staticClass,staticStyle,key");function A(e,t,n,r){var o,s=t.tag,l=t.data,u=t.children;if(r=r||l&&l.pre,t.elm=e,a(t.isComment)&&i(t.asyncFactory))return t.isAsyncPlaceholder=!0,!0;if(i(l)&&(i(o=l.hook)&&i(o=o.init)&&o(t,!0),i(o=t.componentInstance)))return m(t,n),!0;if(i(s)){if(i(u))if(e.hasChildNodes())if(i(o=l)&&i(o=o.domProps)&&i(o=o.innerHTML)){if(o!==e.innerHTML)return!1}else{for(var c=!0,d=e.firstChild,f=0;f<u.length;f++){if(!d||!A(d,u[f],n,r)){c=!1;break}d=d.nextSibling}if(!c||d)return!1}else b(t,u,n);if(i(l)){var h=!1;for(var p in l)if(!E(p)){h=!0,M(t,n);break}!h&&l["class"]&&gt(l["class"])}}else e.data!==t.text&&(e.data=t.text);return!0}return function(e,t,n,s){if(!r(t)){var l=!1,d=[];if(r(e))l=!0,h(t,d);else{var f=i(e.nodeType);if(!f&&Ei(e,t))Y(e,t,d,null,null,s);else{if(f){if(1===e.nodeType&&e.hasAttribute($)&&(e.removeAttribute($),n=!0),a(n)&&A(e,t,d))return C(t,d,!0),e;e=c(e)}var p=e.elm,m=u.parentNode(p);if(h(t,d,p._leaveCb?null:m,u.nextSibling(p)),i(t.parent)){var _=t.parent,v=y(t);while(_){for(var g=0;g<o.destroy.length;++g)o.destroy[g](_);if(_.elm=t.elm,v){for(var b=0;b<o.create.length;++b)o.create[b](Yi,_);var M=_.data.hook.insert;if(M.merged)for(var k=1;k<M.fns.length;k++)M.fns[k]()}else Di(_);_=_.parent}}i(m)?x([e],0,0):i(e.tag)&&L(e)}}return C(t,d,l),t.elm}i(e)&&L(e)}}var Fi={create:Ii,update:Ii,destroy:function(e){Ii(e,Yi)}};function Ii(e,t){(e.data.directives||t.data.directives)&&ji(e,t)}function ji(e,t){var n,r,i,a=e===Yi,o=t===Yi,s=Ri(e.data.directives,e.context),l=Ri(t.data.directives,t.context),u=[],c=[];for(n in l)r=s[n],i=l[n],r?(i.oldValue=r.value,i.oldArg=r.arg,$i(i,"update",t,e),i.def&&i.def.componentUpdated&&c.push(i)):($i(i,"bind",t,e),i.def&&i.def.inserted&&u.push(i));if(u.length){var d=function(){for(var n=0;n<u.length;n++)$i(u[n],"inserted",t,e)};a?wt(t,"insert",d):d()}if(c.length&&wt(t,"postpatch",(function(){for(var n=0;n<c.length;n++)$i(c[n],"componentUpdated",t,e)})),!a)for(n in s)l[n]||$i(s[n],"unbind",e,e,o)}var Hi=Object.create(null);function Ri(e,t){var n,r,i=Object.create(null);if(!e)return i;for(n=0;n<e.length;n++)r=e[n],r.modifiers||(r.modifiers=Hi),i[Ni(r)]=r,r.def=Xe(t.$options,"directives",r.name,!0);return i}function Ni(e){return e.rawName||e.name+"."+Object.keys(e.modifiers||{}).join(".")}function $i(e,t,n,r,i){var a=e.def&&e.def[t];if(a)try{a(n.elm,e,n,r,i)}catch(Xu){rt(Xu,n.context,"directive "+e.name+" "+t+" hook")}}var zi=[Ti,Fi];function Bi(e,t){var n=t.componentOptions;if((!i(n)||!1!==n.Ctor.options.inheritAttrs)&&(!r(e.data.attrs)||!r(t.data.attrs))){var a,o,s,l=t.elm,u=e.data.attrs||{},c=t.data.attrs||{};for(a in i(c.__ob__)&&(c=t.data.attrs=A({},c)),c)o=c[a],s=u[a],s!==o&&Wi(l,a,o);for(a in(ne||ie)&&c.value!==u.value&&Wi(l,"value",c.value),u)r(c[a])&&(Gr(a)?l.removeAttributeNS(qr,Jr(a)):Br(a)||l.removeAttribute(a))}}function Wi(e,t,n){e.tagName.indexOf("-")>-1?Ui(e,t,n):Vr(t)?Kr(n)?e.removeAttribute(t):(n="allowfullscreen"===t&&"EMBED"===e.tagName?"true":t,e.setAttribute(t,n)):Br(t)?e.setAttribute(t,Ur(t,n)):Gr(t)?Kr(n)?e.removeAttributeNS(qr,Jr(t)):e.setAttributeNS(qr,t,n):Ui(e,t,n)}function Ui(e,t,n){if(Kr(n))e.removeAttribute(t);else{if(ne&&!re&&"TEXTAREA"===e.tagName&&"placeholder"===t&&""!==n&&!e.__ieph){var r=function(t){t.stopImmediatePropagation(),e.removeEventListener("input",r)};e.addEventListener("input",r),e.__ieph=!0}e.setAttribute(t,n)}}var Vi={create:Bi,update:Bi};function qi(e,t){var n=t.elm,a=t.data,o=e.data;if(!(r(a.staticClass)&&r(a.class)&&(r(o)||r(o.staticClass)&&r(o.class)))){var s=Xr(t),l=n._transitionClasses;i(l)&&(s=ei(s,ti(l))),s!==n._prevClass&&(n.setAttribute("class",s),n._prevClass=s)}}var Gi,Ji,Ki,Xi,Qi,Zi,ea={create:qi,update:qi},ta=/[\w).+\-_$\]]/;function na(e){var t,n,r,i,a,o=!1,s=!1,l=!1,u=!1,c=0,d=0,f=0,h=0;for(r=0;r<e.length;r++)if(n=t,t=e.charCodeAt(r),o)39===t&&92!==n&&(o=!1);else if(s)34===t&&92!==n&&(s=!1);else if(l)96===t&&92!==n&&(l=!1);else if(u)47===t&&92!==n&&(u=!1);else if(124!==t||124===e.charCodeAt(r+1)||124===e.charCodeAt(r-1)||c||d||f){switch(t){case 34:s=!0;break;case 39:o=!0;break;case 96:l=!0;break;case 40:f++;break;case 41:f--;break;case 91:d++;break;case 93:d--;break;case 123:c++;break;case 125:c--;break}if(47===t){for(var p=r-1,m=void 0;p>=0;p--)if(m=e.charAt(p)," "!==m)break;m&&ta.test(m)||(u=!0)}}else void 0===i?(h=r+1,i=e.slice(0,r).trim()):_();function _(){(a||(a=[])).push(e.slice(h,r).trim()),h=r+1}if(void 0===i?i=e.slice(0,r).trim():0!==h&&_(),a)for(r=0;r<a.length;r++)i=ra(i,a[r]);return i}function ra(e,t){var n=t.indexOf("(");if(n<0)return'_f("'+t+'")('+e+")";var r=t.slice(0,n),i=t.slice(n+1);return'_f("'+r+'")('+e+(")"!==i?","+i:i)}function ia(e,t){console.error("[Vue compiler]: "+e)}function aa(e,t){return e?e.map((function(e){return e[t]})).filter((function(e){return e})):[]}function oa(e,t,n,r,i){(e.props||(e.props=[])).push(_a({name:t,value:n,dynamic:i},r)),e.plain=!1}function sa(e,t,n,r,i){var a=i?e.dynamicAttrs||(e.dynamicAttrs=[]):e.attrs||(e.attrs=[]);a.push(_a({name:t,value:n,dynamic:i},r)),e.plain=!1}function la(e,t,n,r){e.attrsMap[t]=n,e.attrsList.push(_a({name:t,value:n},r))}function ua(e,t,n,r,i,a,o,s){(e.directives||(e.directives=[])).push(_a({name:t,rawName:n,value:r,arg:i,isDynamicArg:a,modifiers:o},s)),e.plain=!1}function ca(e,t,n){return n?"_p("+t+',"'+e+'")':e+t}function da(e,t,r,i,a,o,s,l){var u;i=i||n,i.right?l?t="("+t+")==='click'?'contextmenu':("+t+")":"click"===t&&(t="contextmenu",delete i.right):i.middle&&(l?t="("+t+")==='click'?'mouseup':("+t+")":"click"===t&&(t="mouseup")),i.capture&&(delete i.capture,t=ca("!",t,l)),i.once&&(delete i.once,t=ca("~",t,l)),i.passive&&(delete i.passive,t=ca("&",t,l)),i.native?(delete i.native,u=e.nativeEvents||(e.nativeEvents={})):u=e.events||(e.events={});var c=_a({value:r.trim(),dynamic:l},s);i!==n&&(c.modifiers=i);var d=u[t];Array.isArray(d)?a?d.unshift(c):d.push(c):u[t]=d?a?[c,d]:[d,c]:c,e.plain=!1}function fa(e,t){return e.rawAttrsMap[":"+t]||e.rawAttrsMap["v-bind:"+t]||e.rawAttrsMap[t]}function ha(e,t,n){var r=pa(e,":"+t)||pa(e,"v-bind:"+t);if(null!=r)return na(r);if(!1!==n){var i=pa(e,t);if(null!=i)return JSON.stringify(i)}}function pa(e,t,n){var r;if(null!=(r=e.attrsMap[t]))for(var i=e.attrsList,a=0,o=i.length;a<o;a++)if(i[a].name===t){i.splice(a,1);break}return n&&delete e.attrsMap[t],r}function ma(e,t){for(var n=e.attrsList,r=0,i=n.length;r<i;r++){var a=n[r];if(t.test(a.name))return n.splice(r,1),a}}function _a(e,t){return t&&(null!=t.start&&(e.start=t.start),null!=t.end&&(e.end=t.end)),e}function va(e,t,n){var r=n||{},i=r.number,a=r.trim,o="$$v",s=o;a&&(s="(typeof "+o+" === 'string'? "+o+".trim(): "+o+")"),i&&(s="_n("+s+")");var l=ga(t,s);e.model={value:"("+t+")",expression:JSON.stringify(t),callback:"function ("+o+") {"+l+"}"}}function ga(e,t){var n=ba(e);return null===n.key?e+"="+t:"$set("+n.exp+", "+n.key+", "+t+")"}function ba(e){if(e=e.trim(),Gi=e.length,e.indexOf("[")<0||e.lastIndexOf("]")<Gi-1)return Xi=e.lastIndexOf("."),Xi>-1?{exp:e.slice(0,Xi),key:'"'+e.slice(Xi+1)+'"'}:{exp:e,key:null};Ji=e,Xi=Qi=Zi=0;while(!Ma())Ki=ya(),ka(Ki)?La(Ki):91===Ki&&wa(Ki);return{exp:e.slice(0,Qi),key:e.slice(Qi+1,Zi)}}function ya(){return Ji.charCodeAt(++Xi)}function Ma(){return Xi>=Gi}function ka(e){return 34===e||39===e}function wa(e){var t=1;Qi=Xi;while(!Ma())if(e=ya(),ka(e))La(e);else if(91===e&&t++,93===e&&t--,0===t){Zi=Xi;break}}function La(e){var t=e;while(!Ma())if(e=ya(),e===t)break}var xa,Sa="__r",Ta="__c";function Da(e,t,n){n;var r=t.value,i=t.modifiers,a=e.tag,o=e.attrsMap.type;if(e.component)return va(e,r,i),!1;if("select"===a)Ea(e,r,i);else if("input"===a&&"checkbox"===o)Ya(e,r,i);else if("input"===a&&"radio"===o)Ca(e,r,i);else if("input"===a||"textarea"===a)Aa(e,r,i);else{if(!W.isReservedTag(a))return va(e,r,i),!1}return!0}function Ya(e,t,n){var r=n&&n.number,i=ha(e,"value")||"null",a=ha(e,"true-value")||"true",o=ha(e,"false-value")||"false";oa(e,"checked","Array.isArray("+t+")?_i("+t+","+i+")>-1"+("true"===a?":("+t+")":":_q("+t+","+a+")")),da(e,"change","var $$a="+t+",$$el=$event.target,$$c=$$el.checked?("+a+"):("+o+");if(Array.isArray($$a)){var $$v="+(r?"_n("+i+")":i)+",$$i=_i($$a,$$v);if($$el.checked){$$i<0&&("+ga(t,"$$a.concat([$$v])")+")}else{$$i>-1&&("+ga(t,"$$a.slice(0,$$i).concat($$a.slice($$i+1))")+")}}else{"+ga(t,"$$c")+"}",null,!0)}function Ca(e,t,n){var r=n&&n.number,i=ha(e,"value")||"null";i=r?"_n("+i+")":i,oa(e,"checked","_q("+t+","+i+")"),da(e,"change",ga(t,i),null,!0)}function Ea(e,t,n){var r=n&&n.number,i='Array.prototype.filter.call($event.target.options,function(o){return o.selected}).map(function(o){var val = "_value" in o ? o._value : o.value;return '+(r?"_n(val)":"val")+"})",a="$event.target.multiple ? $$selectedVal : $$selectedVal[0]",o="var $$selectedVal = "+i+";";o=o+" "+ga(t,a),da(e,"change",o,null,!0)}function Aa(e,t,n){var r=e.attrsMap.type,i=n||{},a=i.lazy,o=i.number,s=i.trim,l=!a&&"range"!==r,u=a?"change":"range"===r?Sa:"input",c="$event.target.value";s&&(c="$event.target.value.trim()"),o&&(c="_n("+c+")");var d=ga(t,c);l&&(d="if($event.target.composing)return;"+d),oa(e,"value","("+t+")"),da(e,u,d,null,!0),(s||o)&&da(e,"blur","$forceUpdate()")}function Oa(e){if(i(e[Sa])){var t=ne?"change":"input";e[t]=[].concat(e[Sa],e[t]||[]),delete e[Sa]}i(e[Ta])&&(e.change=[].concat(e[Ta],e.change||[]),delete e[Ta])}function Pa(e,t,n){var r=xa;return function i(){var a=t.apply(null,arguments);null!==a&&ja(e,i,n,r)}}var Fa=lt&&!(oe&&Number(oe[1])<=53);function Ia(e,t,n,r){if(Fa){var i=Jn,a=t;t=a._wrapper=function(e){if(e.target===e.currentTarget||e.timeStamp>=i||e.timeStamp<=0||e.target.ownerDocument!==document)return a.apply(this,arguments)}}xa.addEventListener(e,t,le?{capture:n,passive:r}:n)}function ja(e,t,n,r){(r||xa).removeEventListener(e,t._wrapper||t,n)}function Ha(e,t){if(!r(e.data.on)||!r(t.data.on)){var n=t.data.on||{},i=e.data.on||{};xa=t.elm,Oa(n),kt(n,i,Ia,ja,Pa,t.context),xa=void 0}}var Ra,Na={create:Ha,update:Ha};function $a(e,t){if(!r(e.data.domProps)||!r(t.data.domProps)){var n,a,o=t.elm,s=e.data.domProps||{},l=t.data.domProps||{};for(n in i(l.__ob__)&&(l=t.data.domProps=A({},l)),s)n in l||(o[n]="");for(n in l){if(a=l[n],"textContent"===n||"innerHTML"===n){if(t.children&&(t.children.length=0),a===s[n])continue;1===o.childNodes.length&&o.removeChild(o.childNodes[0])}if("value"===n&&"PROGRESS"!==o.tagName){o._value=a;var u=r(a)?"":String(a);za(o,u)&&(o.value=u)}else if("innerHTML"===n&&oi(o.tagName)&&r(o.innerHTML)){Ra=Ra||document.createElement("div"),Ra.innerHTML="<svg>"+a+"</svg>";var c=Ra.firstChild;while(o.firstChild)o.removeChild(o.firstChild);while(c.firstChild)o.appendChild(c.firstChild)}else if(a!==s[n])try{o[n]=a}catch(Xu){}}}}function za(e,t){return!e.composing&&("OPTION"===e.tagName||Ba(e,t)||Wa(e,t))}function Ba(e,t){var n=!0;try{n=document.activeElement!==e}catch(Xu){}return n&&e.value!==t}function Wa(e,t){var n=e.value,r=e._vModifiers;if(i(r)){if(r.number)return m(n)!==m(t);if(r.trim)return n.trim()!==t.trim()}return n!==t}var Ua={create:$a,update:$a},Va=k((function(e){var t={},n=/;(?![^(]*\))/g,r=/:(.+)/;return e.split(n).forEach((function(e){if(e){var n=e.split(r);n.length>1&&(t[n[0].trim()]=n[1].trim())}})),t}));function qa(e){var t=Ga(e.style);return e.staticStyle?A(e.staticStyle,t):t}function Ga(e){return Array.isArray(e)?O(e):"string"===typeof e?Va(e):e}function Ja(e,t){var n,r={};if(t){var i=e;while(i.componentInstance)i=i.componentInstance._vnode,i&&i.data&&(n=qa(i.data))&&A(r,n)}(n=qa(e.data))&&A(r,n);var a=e;while(a=a.parent)a.data&&(n=qa(a.data))&&A(r,n);return r}var Ka,Xa=/^--/,Qa=/\s*!important$/,Za=function(e,t,n){if(Xa.test(t))e.style.setProperty(t,n);else if(Qa.test(n))e.style.setProperty(T(t),n.replace(Qa,""),"important");else{var r=to(t);if(Array.isArray(n))for(var i=0,a=n.length;i<a;i++)e.style[r]=n[i];else e.style[r]=n}},eo=["Webkit","Moz","ms"],to=k((function(e){if(Ka=Ka||document.createElement("div").style,e=L(e),"filter"!==e&&e in Ka)return e;for(var t=e.charAt(0).toUpperCase()+e.slice(1),n=0;n<eo.length;n++){var r=eo[n]+t;if(r in Ka)return r}}));function no(e,t){var n=t.data,a=e.data;if(!(r(n.staticStyle)&&r(n.style)&&r(a.staticStyle)&&r(a.style))){var o,s,l=t.elm,u=a.staticStyle,c=a.normalizedStyle||a.style||{},d=u||c,f=Ga(t.data.style)||{};t.data.normalizedStyle=i(f.__ob__)?A({},f):f;var h=Ja(t,!0);for(s in d)r(h[s])&&Za(l,s,"");for(s in h)o=h[s],o!==d[s]&&Za(l,s,null==o?"":o)}}var ro={create:no,update:no},io=/\s+/;function ao(e,t){if(t&&(t=t.trim()))if(e.classList)t.indexOf(" ")>-1?t.split(io).forEach((function(t){return e.classList.add(t)})):e.classList.add(t);else{var n=" "+(e.getAttribute("class")||"")+" ";n.indexOf(" "+t+" ")<0&&e.setAttribute("class",(n+t).trim())}}function oo(e,t){if(t&&(t=t.trim()))if(e.classList)t.indexOf(" ")>-1?t.split(io).forEach((function(t){return e.classList.remove(t)})):e.classList.remove(t),e.classList.length||e.removeAttribute("class");else{var n=" "+(e.getAttribute("class")||"")+" ",r=" "+t+" ";while(n.indexOf(r)>=0)n=n.replace(r," ");n=n.trim(),n?e.setAttribute("class",n):e.removeAttribute("class")}}function so(e){if(e){if("object"===typeof e){var t={};return!1!==e.css&&A(t,lo(e.name||"v")),A(t,e),t}return"string"===typeof e?lo(e):void 0}}var lo=k((function(e){return{enterClass:e+"-enter",enterToClass:e+"-enter-to",enterActiveClass:e+"-enter-active",leaveClass:e+"-leave",leaveToClass:e+"-leave-to",leaveActiveClass:e+"-leave-active"}})),uo=Q&&!re,co="transition",fo="animation",ho="transition",po="transitionend",mo="animation",_o="animationend";uo&&(void 0===window.ontransitionend&&void 0!==window.onwebkittransitionend&&(ho="WebkitTransition",po="webkitTransitionEnd"),void 0===window.onanimationend&&void 0!==window.onwebkitanimationend&&(mo="WebkitAnimation",_o="webkitAnimationEnd"));var vo=Q?window.requestAnimationFrame?window.requestAnimationFrame.bind(window):setTimeout:function(e){return e()};function go(e){vo((function(){vo(e)}))}function bo(e,t){var n=e._transitionClasses||(e._transitionClasses=[]);n.indexOf(t)<0&&(n.push(t),ao(e,t))}function yo(e,t){e._transitionClasses&&b(e._transitionClasses,t),oo(e,t)}function Mo(e,t,n){var r=wo(e,t),i=r.type,a=r.timeout,o=r.propCount;if(!i)return n();var s=i===co?po:_o,l=0,u=function(){e.removeEventListener(s,c),n()},c=function(t){t.target===e&&++l>=o&&u()};setTimeout((function(){l<o&&u()}),a+1),e.addEventListener(s,c)}var ko=/\b(transform|all)(,|$)/;function wo(e,t){var n,r=window.getComputedStyle(e),i=(r[ho+"Delay"]||"").split(", "),a=(r[ho+"Duration"]||"").split(", "),o=Lo(i,a),s=(r[mo+"Delay"]||"").split(", "),l=(r[mo+"Duration"]||"").split(", "),u=Lo(s,l),c=0,d=0;t===co?o>0&&(n=co,c=o,d=a.length):t===fo?u>0&&(n=fo,c=u,d=l.length):(c=Math.max(o,u),n=c>0?o>u?co:fo:null,d=n?n===co?a.length:l.length:0);var f=n===co&&ko.test(r[ho+"Property"]);return{type:n,timeout:c,propCount:d,hasTransform:f}}function Lo(e,t){while(e.length<t.length)e=e.concat(e);return Math.max.apply(null,t.map((function(t,n){return xo(t)+xo(e[n])})))}function xo(e){return 1e3*Number(e.slice(0,-1).replace(",","."))}function So(e,t){var n=e.elm;i(n._leaveCb)&&(n._leaveCb.cancelled=!0,n._leaveCb());var a=so(e.data.transition);if(!r(a)&&!i(n._enterCb)&&1===n.nodeType){var o=a.css,s=a.type,u=a.enterClass,c=a.enterToClass,d=a.enterActiveClass,f=a.appearClass,h=a.appearToClass,p=a.appearActiveClass,_=a.beforeEnter,v=a.enter,g=a.afterEnter,b=a.enterCancelled,y=a.beforeAppear,M=a.appear,k=a.afterAppear,w=a.appearCancelled,L=a.duration,x=An,S=An.$vnode;while(S&&S.parent)x=S.context,S=S.parent;var T=!x._isMounted||!e.isRootInsert;if(!T||M||""===M){var D=T&&f?f:u,Y=T&&p?p:d,C=T&&h?h:c,E=T&&y||_,A=T&&"function"===typeof M?M:v,O=T&&k||g,P=T&&w||b,F=m(l(L)?L.enter:L);0;var I=!1!==o&&!re,j=Yo(A),H=n._enterCb=N((function(){I&&(yo(n,C),yo(n,Y)),H.cancelled?(I&&yo(n,D),P&&P(n)):O&&O(n),n._enterCb=null}));e.data.show||wt(e,"insert",(function(){var t=n.parentNode,r=t&&t._pending&&t._pending[e.key];r&&r.tag===e.tag&&r.elm._leaveCb&&r.elm._leaveCb(),A&&A(n,H)})),E&&E(n),I&&(bo(n,D),bo(n,Y),go((function(){yo(n,D),H.cancelled||(bo(n,C),j||(Do(F)?setTimeout(H,F):Mo(n,s,H)))}))),e.data.show&&(t&&t(),A&&A(n,H)),I||j||H()}}}function To(e,t){var n=e.elm;i(n._enterCb)&&(n._enterCb.cancelled=!0,n._enterCb());var a=so(e.data.transition);if(r(a)||1!==n.nodeType)return t();if(!i(n._leaveCb)){var o=a.css,s=a.type,u=a.leaveClass,c=a.leaveToClass,d=a.leaveActiveClass,f=a.beforeLeave,h=a.leave,p=a.afterLeave,_=a.leaveCancelled,v=a.delayLeave,g=a.duration,b=!1!==o&&!re,y=Yo(h),M=m(l(g)?g.leave:g);0;var k=n._leaveCb=N((function(){n.parentNode&&n.parentNode._pending&&(n.parentNode._pending[e.key]=null),b&&(yo(n,c),yo(n,d)),k.cancelled?(b&&yo(n,u),_&&_(n)):(t(),p&&p(n)),n._leaveCb=null}));v?v(w):w()}function w(){k.cancelled||(!e.data.show&&n.parentNode&&((n.parentNode._pending||(n.parentNode._pending={}))[e.key]=e),f&&f(n),b&&(bo(n,u),bo(n,d),go((function(){yo(n,u),k.cancelled||(bo(n,c),y||(Do(M)?setTimeout(k,M):Mo(n,s,k)))}))),h&&h(n,k),b||y||k())}}function Do(e){return"number"===typeof e&&!isNaN(e)}function Yo(e){if(r(e))return!1;var t=e.fns;return i(t)?Yo(Array.isArray(t)?t[0]:t):(e._length||e.length)>1}function Co(e,t){!0!==t.data.show&&So(t)}var Eo=Q?{create:Co,activate:Co,remove:function(e,t){!0!==e.data.show?To(e,t):t()}}:{},Ao=[Vi,ea,Na,Ua,ro,Eo],Oo=Ao.concat(zi),Po=Pi({nodeOps:Si,modules:Oo});re&&document.addEventListener("selectionchange",(function(){var e=document.activeElement;e&&e.vmodel&&zo(e,"input")}));var Fo={inserted:function(e,t,n,r){"select"===n.tag?(r.elm&&!r.elm._vOptions?wt(n,"postpatch",(function(){Fo.componentUpdated(e,t,n)})):Io(e,t,n.context),e._vOptions=[].map.call(e.options,Ro)):("textarea"===n.tag||fi(e.type))&&(e._vModifiers=t.modifiers,t.modifiers.lazy||(e.addEventListener("compositionstart",No),e.addEventListener("compositionend",$o),e.addEventListener("change",$o),re&&(e.vmodel=!0)))},componentUpdated:function(e,t,n){if("select"===n.tag){Io(e,t,n.context);var r=e._vOptions,i=e._vOptions=[].map.call(e.options,Ro);if(i.some((function(e,t){return!H(e,r[t])}))){var a=e.multiple?t.value.some((function(e){return Ho(e,i)})):t.value!==t.oldValue&&Ho(t.value,i);a&&zo(e,"change")}}}};function Io(e,t,n){jo(e,t,n),(ne||ie)&&setTimeout((function(){jo(e,t,n)}),0)}function jo(e,t,n){var r=t.value,i=e.multiple;if(!i||Array.isArray(r)){for(var a,o,s=0,l=e.options.length;s<l;s++)if(o=e.options[s],i)a=R(r,Ro(o))>-1,o.selected!==a&&(o.selected=a);else if(H(Ro(o),r))return void(e.selectedIndex!==s&&(e.selectedIndex=s));i||(e.selectedIndex=-1)}}function Ho(e,t){return t.every((function(t){return!H(t,e)}))}function Ro(e){return"_value"in e?e._value:e.value}function No(e){e.target.composing=!0}function $o(e){e.target.composing&&(e.target.composing=!1,zo(e.target,"input"))}function zo(e,t){var n=document.createEvent("HTMLEvents");n.initEvent(t,!0,!0),e.dispatchEvent(n)}function Bo(e){return!e.componentInstance||e.data&&e.data.transition?e:Bo(e.componentInstance._vnode)}var Wo={bind:function(e,t,n){var r=t.value;n=Bo(n);var i=n.data&&n.data.transition,a=e.__vOriginalDisplay="none"===e.style.display?"":e.style.display;r&&i?(n.data.show=!0,So(n,(function(){e.style.display=a}))):e.style.display=r?a:"none"},update:function(e,t,n){var r=t.value,i=t.oldValue;if(!r!==!i){n=Bo(n);var a=n.data&&n.data.transition;a?(n.data.show=!0,r?So(n,(function(){e.style.display=e.__vOriginalDisplay})):To(n,(function(){e.style.display="none"}))):e.style.display=r?e.__vOriginalDisplay:"none"}},unbind:function(e,t,n,r,i){i||(e.style.display=e.__vOriginalDisplay)}},Uo={model:Fo,show:Wo},Vo={name:String,appear:Boolean,css:Boolean,mode:String,type:String,enterClass:String,leaveClass:String,enterToClass:String,leaveToClass:String,enterActiveClass:String,leaveActiveClass:String,appearClass:String,appearActiveClass:String,appearToClass:String,duration:[Number,String,Object]};function qo(e){var t=e&&e.componentOptions;return t&&t.Ctor.options.abstract?qo(xn(t.children)):e}function Go(e){var t={},n=e.$options;for(var r in n.propsData)t[r]=e[r];var i=n._parentListeners;for(var a in i)t[L(a)]=i[a];return t}function Jo(e,t){if(/\d-keep-alive$/.test(t.tag))return e("keep-alive",{props:t.componentOptions.propsData})}function Ko(e){while(e=e.parent)if(e.data.transition)return!0}function Xo(e,t){return t.key===e.key&&t.tag===e.tag}var Qo=function(e){return e.tag||Ln(e)},Zo=function(e){return"show"===e.name},es={name:"transition",props:Vo,abstract:!0,render:function(e){var t=this,n=this.$slots.default;if(n&&(n=n.filter(Qo),n.length)){0;var r=this.mode;0;var i=n[0];if(Ko(this.$vnode))return i;var a=qo(i);if(!a)return i;if(this._leaving)return Jo(e,i);var o="__transition-"+this._uid+"-";a.key=null==a.key?a.isComment?o+"comment":o+a.tag:s(a.key)?0===String(a.key).indexOf(o)?a.key:o+a.key:a.key;var l=(a.data||(a.data={})).transition=Go(this),u=this._vnode,c=qo(u);if(a.data.directives&&a.data.directives.some(Zo)&&(a.data.show=!0),c&&c.data&&!Xo(a,c)&&!Ln(c)&&(!c.componentInstance||!c.componentInstance._vnode.isComment)){var d=c.data.transition=A({},l);if("out-in"===r)return this._leaving=!0,wt(d,"afterLeave",(function(){t._leaving=!1,t.$forceUpdate()})),Jo(e,i);if("in-out"===r){if(Ln(a))return u;var f,h=function(){f()};wt(l,"afterEnter",h),wt(l,"enterCancelled",h),wt(d,"delayLeave",(function(e){f=e}))}}return i}}},ts=A({tag:String,moveClass:String},Vo);delete ts.mode;var ns={props:ts,beforeMount:function(){var e=this,t=this._update;this._update=function(n,r){var i=On(e);e.__patch__(e._vnode,e.kept,!1,!0),e._vnode=e.kept,i(),t.call(e,n,r)}},render:function(e){for(var t=this.tag||this.$vnode.data.tag||"span",n=Object.create(null),r=this.prevChildren=this.children,i=this.$slots.default||[],a=this.children=[],o=Go(this),s=0;s<i.length;s++){var l=i[s];if(l.tag)if(null!=l.key&&0!==String(l.key).indexOf("__vlist"))a.push(l),n[l.key]=l,(l.data||(l.data={})).transition=o;else;}if(r){for(var u=[],c=[],d=0;d<r.length;d++){var f=r[d];f.data.transition=o,f.data.pos=f.elm.getBoundingClientRect(),n[f.key]?u.push(f):c.push(f)}this.kept=e(t,null,u),this.removed=c}return e(t,null,a)},updated:function(){var e=this.prevChildren,t=this.moveClass||(this.name||"v")+"-move";e.length&&this.hasMove(e[0].elm,t)&&(e.forEach(rs),e.forEach(is),e.forEach(as),this._reflow=document.body.offsetHeight,e.forEach((function(e){if(e.data.moved){var n=e.elm,r=n.style;bo(n,t),r.transform=r.WebkitTransform=r.transitionDuration="",n.addEventListener(po,n._moveCb=function e(r){r&&r.target!==n||r&&!/transform$/.test(r.propertyName)||(n.removeEventListener(po,e),n._moveCb=null,yo(n,t))})}})))},methods:{hasMove:function(e,t){if(!uo)return!1;if(this._hasMove)return this._hasMove;var n=e.cloneNode();e._transitionClasses&&e._transitionClasses.forEach((function(e){oo(n,e)})),ao(n,t),n.style.display="none",this.$el.appendChild(n);var r=wo(n);return this.$el.removeChild(n),this._hasMove=r.hasTransform}}};function rs(e){e.elm._moveCb&&e.elm._moveCb(),e.elm._enterCb&&e.elm._enterCb()}function is(e){e.data.newPos=e.elm.getBoundingClientRect()}function as(e){var t=e.data.pos,n=e.data.newPos,r=t.left-n.left,i=t.top-n.top;if(r||i){e.data.moved=!0;var a=e.elm.style;a.transform=a.WebkitTransform="translate("+r+"px,"+i+"px)",a.transitionDuration="0s"}}var os={Transition:es,TransitionGroup:ns};xr.config.mustUseProp=zr,xr.config.isReservedTag=li,xr.config.isReservedAttr=Nr,xr.config.getTagNamespace=ui,xr.config.isUnknownElement=di,A(xr.options.directives,Uo),A(xr.options.components,os),xr.prototype.__patch__=Q?Po:P,xr.prototype.$mount=function(e,t){return e=e&&Q?hi(e):void 0,In(this,e,t)},Q&&setTimeout((function(){W.devtools&&de&&de.emit("init",xr)}),0);var ss=/\{\{((?:.|\r?\n)+?)\}\}/g,ls=/[-.*+?^${}()|[\]\/\\]/g,us=k((function(e){var t=e[0].replace(ls,"\\$&"),n=e[1].replace(ls,"\\$&");return new RegExp(t+"((?:.|\\n)+?)"+n,"g")}));function cs(e,t){var n=t?us(t):ss;if(n.test(e)){var r,i,a,o=[],s=[],l=n.lastIndex=0;while(r=n.exec(e)){i=r.index,i>l&&(s.push(a=e.slice(l,i)),o.push(JSON.stringify(a)));var u=na(r[1].trim());o.push("_s("+u+")"),s.push({"@binding":u}),l=i+r[0].length}return l<e.length&&(s.push(a=e.slice(l)),o.push(JSON.stringify(a))),{expression:o.join("+"),tokens:s}}}function ds(e,t){t.warn;var n=pa(e,"class");n&&(e.staticClass=JSON.stringify(n));var r=ha(e,"class",!1);r&&(e.classBinding=r)}function fs(e){var t="";return e.staticClass&&(t+="staticClass:"+e.staticClass+","),e.classBinding&&(t+="class:"+e.classBinding+","),t}var hs={staticKeys:["staticClass"],transformNode:ds,genData:fs};function ps(e,t){t.warn;var n=pa(e,"style");n&&(e.staticStyle=JSON.stringify(Va(n)));var r=ha(e,"style",!1);r&&(e.styleBinding=r)}function ms(e){var t="";return e.staticStyle&&(t+="staticStyle:"+e.staticStyle+","),e.styleBinding&&(t+="style:("+e.styleBinding+"),"),t}var _s,vs={staticKeys:["staticStyle"],transformNode:ps,genData:ms},gs={decode:function(e){return _s=_s||document.createElement("div"),_s.innerHTML=e,_s.textContent}},bs=_("area,base,br,col,embed,frame,hr,img,input,isindex,keygen,link,meta,param,source,track,wbr"),ys=_("colgroup,dd,dt,li,options,p,td,tfoot,th,thead,tr,source"),Ms=_("address,article,aside,base,blockquote,body,caption,col,colgroup,dd,details,dialog,div,dl,dt,fieldset,figcaption,figure,footer,form,h1,h2,h3,h4,h5,h6,head,header,hgroup,hr,html,legend,li,menuitem,meta,optgroup,option,param,rp,rt,source,style,summary,tbody,td,tfoot,th,thead,title,tr,track"),ks=/^\s*([^\s"'<>\/=]+)(?:\s*(=)\s*(?:"([^"]*)"+|'([^']*)'+|([^\s"'=<>`]+)))?/,ws=/^\s*((?:v-[\w-]+:|@|:|#)\[[^=]+\][^\s"'<>\/=]*)(?:\s*(=)\s*(?:"([^"]*)"+|'([^']*)'+|([^\s"'=<>`]+)))?/,Ls="[a-zA-Z_][\\-\\.0-9_a-zA-Z"+U.source+"]*",xs="((?:"+Ls+"\\:)?"+Ls+")",Ss=new RegExp("^<"+xs),Ts=/^\s*(\/?)>/,Ds=new RegExp("^<\\/"+xs+"[^>]*>"),Ys=/^<!DOCTYPE [^>]+>/i,Cs=/^<!\--/,Es=/^<!\[/,As=_("script,style,textarea",!0),Os={},Ps={"&lt;":"<","&gt;":">","&quot;":'"',"&amp;":"&","&#10;":"\n","&#9;":"\t","&#39;":"'"},Fs=/&(?:lt|gt|quot|amp|#39);/g,Is=/&(?:lt|gt|quot|amp|#39|#10|#9);/g,js=_("pre,textarea",!0),Hs=function(e,t){return e&&js(e)&&"\n"===t[0]};function Rs(e,t){var n=t?Is:Fs;return e.replace(n,(function(e){return Ps[e]}))}function Ns(e,t){var n,r,i=[],a=t.expectHTML,o=t.isUnaryTag||F,s=t.canBeLeftOpenTag||F,l=0;while(e){if(n=e,r&&As(r)){var u=0,c=r.toLowerCase(),d=Os[c]||(Os[c]=new RegExp("([\\s\\S]*?)(</"+c+"[^>]*>)","i")),f=e.replace(d,(function(e,n,r){return u=r.length,As(c)||"noscript"===c||(n=n.replace(/<!\--([\s\S]*?)-->/g,"$1").replace(/<!\[CDATA\[([\s\S]*?)]]>/g,"$1")),Hs(c,n)&&(n=n.slice(1)),t.chars&&t.chars(n),""}));l+=e.length-f.length,e=f,S(c,l-u,l)}else{var h=e.indexOf("<");if(0===h){if(Cs.test(e)){var p=e.indexOf("--\x3e");if(p>=0){t.shouldKeepComment&&t.comment(e.substring(4,p),l,l+p+3),w(p+3);continue}}if(Es.test(e)){var m=e.indexOf("]>");if(m>=0){w(m+2);continue}}var _=e.match(Ys);if(_){w(_[0].length);continue}var v=e.match(Ds);if(v){var g=l;w(v[0].length),S(v[1],g,l);continue}var b=L();if(b){x(b),Hs(b.tagName,e)&&w(1);continue}}var y=void 0,M=void 0,k=void 0;if(h>=0){M=e.slice(h);while(!Ds.test(M)&&!Ss.test(M)&&!Cs.test(M)&&!Es.test(M)){if(k=M.indexOf("<",1),k<0)break;h+=k,M=e.slice(h)}y=e.substring(0,h)}h<0&&(y=e),y&&w(y.length),t.chars&&y&&t.chars(y,l-y.length,l)}if(e===n){t.chars&&t.chars(e);break}}function w(t){l+=t,e=e.substring(t)}function L(){var t=e.match(Ss);if(t){var n,r,i={tagName:t[1],attrs:[],start:l};w(t[0].length);while(!(n=e.match(Ts))&&(r=e.match(ws)||e.match(ks)))r.start=l,w(r[0].length),r.end=l,i.attrs.push(r);if(n)return i.unarySlash=n[1],w(n[0].length),i.end=l,i}}function x(e){var n=e.tagName,l=e.unarySlash;a&&("p"===r&&Ms(n)&&S(r),s(n)&&r===n&&S(n));for(var u=o(n)||!!l,c=e.attrs.length,d=new Array(c),f=0;f<c;f++){var h=e.attrs[f],p=h[3]||h[4]||h[5]||"",m="a"===n&&"href"===h[1]?t.shouldDecodeNewlinesForHref:t.shouldDecodeNewlines;d[f]={name:h[1],value:Rs(p,m)}}u||(i.push({tag:n,lowerCasedTag:n.toLowerCase(),attrs:d,start:e.start,end:e.end}),r=n),t.start&&t.start(n,d,u,e.start,e.end)}function S(e,n,a){var o,s;if(null==n&&(n=l),null==a&&(a=l),e){for(s=e.toLowerCase(),o=i.length-1;o>=0;o--)if(i[o].lowerCasedTag===s)break}else o=0;if(o>=0){for(var u=i.length-1;u>=o;u--)t.end&&t.end(i[u].tag,n,a);i.length=o,r=o&&i[o-1].tag}else"br"===s?t.start&&t.start(e,[],!0,n,a):"p"===s&&(t.start&&t.start(e,[],!1,n,a),t.end&&t.end(e,n,a))}S()}var $s,zs,Bs,Ws,Us,Vs,qs,Gs,Js=/^@|^v-on:/,Ks=/^v-|^@|^:|^#/,Xs=/([\s\S]*?)\s+(?:in|of)\s+([\s\S]*)/,Qs=/,([^,\}\]]*)(?:,([^,\}\]]*))?$/,Zs=/^\(|\)$/g,el=/^\[.*\]$/,tl=/:(.*)$/,nl=/^:|^\.|^v-bind:/,rl=/\.[^.\]]+(?=[^\]]*$)/g,il=/^v-slot(:|$)|^#/,al=/[\r\n]/,ol=/\s+/g,sl=k(gs.decode),ll="_empty_";function ul(e,t,n){return{type:1,tag:e,attrsList:t,attrsMap:Cl(t),rawAttrsMap:{},parent:n,children:[]}}function cl(e,t){$s=t.warn||ia,Vs=t.isPreTag||F,qs=t.mustUseProp||F,Gs=t.getTagNamespace||F;var n=t.isReservedTag||F;(function(e){return!!e.component||!n(e.tag)}),Bs=aa(t.modules,"transformNode"),Ws=aa(t.modules,"preTransformNode"),Us=aa(t.modules,"postTransformNode"),zs=t.delimiters;var r,i,a=[],o=!1!==t.preserveWhitespace,s=t.whitespace,l=!1,u=!1;function c(e){if(d(e),l||e.processed||(e=hl(e,t)),a.length||e===r||r.if&&(e.elseif||e.else)&&Ml(r,{exp:e.elseif,block:e}),i&&!e.forbidden)if(e.elseif||e.else)bl(e,i);else{if(e.slotScope){var n=e.slotTarget||'"default"';(i.scopedSlots||(i.scopedSlots={}))[n]=e}i.children.push(e),e.parent=i}e.children=e.children.filter((function(e){return!e.slotScope})),d(e),e.pre&&(l=!1),Vs(e.tag)&&(u=!1);for(var o=0;o<Us.length;o++)Us[o](e,t)}function d(e){var t;if(!u)while((t=e.children[e.children.length-1])&&3===t.type&&" "===t.text)e.children.pop()}return Ns(e,{warn:$s,expectHTML:t.expectHTML,isUnaryTag:t.isUnaryTag,canBeLeftOpenTag:t.canBeLeftOpenTag,shouldDecodeNewlines:t.shouldDecodeNewlines,shouldDecodeNewlinesForHref:t.shouldDecodeNewlinesForHref,shouldKeepComment:t.comments,outputSourceRange:t.outputSourceRange,start:function(e,n,o,s,d){var f=i&&i.ns||Gs(e);ne&&"svg"===f&&(n=Fl(n));var h=ul(e,n,i);f&&(h.ns=f),Al(h)&&!ce()&&(h.forbidden=!0);for(var p=0;p<Ws.length;p++)h=Ws[p](h,t)||h;l||(dl(h),h.pre&&(l=!0)),Vs(h.tag)&&(u=!0),l?fl(h):h.processed||(_l(h),gl(h),kl(h)),r||(r=h),o?c(h):(i=h,a.push(h))},end:function(e,t,n){var r=a[a.length-1];a.length-=1,i=a[a.length-1],c(r)},chars:function(e,t,n){if(i&&(!ne||"textarea"!==i.tag||i.attrsMap.placeholder!==e)){var r,a,c=i.children;if(e=u||e.trim()?El(i)?e:sl(e):c.length?s?"condense"===s&&al.test(e)?"":" ":o?" ":"":"",e)u||"condense"!==s||(e=e.replace(ol," ")),!l&&" "!==e&&(r=cs(e,zs))?a={type:2,expression:r.expression,tokens:r.tokens,text:e}:" "===e&&c.length&&" "===c[c.length-1].text||(a={type:3,text:e}),a&&c.push(a)}},comment:function(e,t,n){if(i){var r={type:3,text:e,isComment:!0};0,i.children.push(r)}}}),r}function dl(e){null!=pa(e,"v-pre")&&(e.pre=!0)}function fl(e){var t=e.attrsList,n=t.length;if(n)for(var r=e.attrs=new Array(n),i=0;i<n;i++)r[i]={name:t[i].name,value:JSON.stringify(t[i].value)},null!=t[i].start&&(r[i].start=t[i].start,r[i].end=t[i].end);else e.pre||(e.plain=!0)}function hl(e,t){pl(e),e.plain=!e.key&&!e.scopedSlots&&!e.attrsList.length,ml(e),wl(e),xl(e),Sl(e);for(var n=0;n<Bs.length;n++)e=Bs[n](e,t)||e;return Tl(e),e}function pl(e){var t=ha(e,"key");t&&(e.key=t)}function ml(e){var t=ha(e,"ref");t&&(e.ref=t,e.refInFor=Dl(e))}function _l(e){var t;if(t=pa(e,"v-for")){var n=vl(t);n&&A(e,n)}}function vl(e){var t=e.match(Xs);if(t){var n={};n.for=t[2].trim();var r=t[1].trim().replace(Zs,""),i=r.match(Qs);return i?(n.alias=r.replace(Qs,"").trim(),n.iterator1=i[1].trim(),i[2]&&(n.iterator2=i[2].trim())):n.alias=r,n}}function gl(e){var t=pa(e,"v-if");if(t)e.if=t,Ml(e,{exp:t,block:e});else{null!=pa(e,"v-else")&&(e.else=!0);var n=pa(e,"v-else-if");n&&(e.elseif=n)}}function bl(e,t){var n=yl(t.children);n&&n.if&&Ml(n,{exp:e.elseif,block:e})}function yl(e){var t=e.length;while(t--){if(1===e[t].type)return e[t];e.pop()}}function Ml(e,t){e.ifConditions||(e.ifConditions=[]),e.ifConditions.push(t)}function kl(e){var t=pa(e,"v-once");null!=t&&(e.once=!0)}function wl(e){var t;"template"===e.tag?(t=pa(e,"scope"),e.slotScope=t||pa(e,"slot-scope")):(t=pa(e,"slot-scope"))&&(e.slotScope=t);var n=ha(e,"slot");if(n&&(e.slotTarget='""'===n?'"default"':n,e.slotTargetDynamic=!(!e.attrsMap[":slot"]&&!e.attrsMap["v-bind:slot"]),"template"===e.tag||e.slotScope||sa(e,"slot",n,fa(e,"slot"))),"template"===e.tag){var r=ma(e,il);if(r){0;var i=Ll(r),a=i.name,o=i.dynamic;e.slotTarget=a,e.slotTargetDynamic=o,e.slotScope=r.value||ll}}else{var s=ma(e,il);if(s){0;var l=e.scopedSlots||(e.scopedSlots={}),u=Ll(s),c=u.name,d=u.dynamic,f=l[c]=ul("template",[],e);f.slotTarget=c,f.slotTargetDynamic=d,f.children=e.children.filter((function(e){if(!e.slotScope)return e.parent=f,!0})),f.slotScope=s.value||ll,e.children=[],e.plain=!1}}}function Ll(e){var t=e.name.replace(il,"");return t||"#"!==e.name[0]&&(t="default"),el.test(t)?{name:t.slice(1,-1),dynamic:!0}:{name:'"'+t+'"',dynamic:!1}}function xl(e){"slot"===e.tag&&(e.slotName=ha(e,"name"))}function Sl(e){var t;(t=ha(e,"is"))&&(e.component=t),null!=pa(e,"inline-template")&&(e.inlineTemplate=!0)}function Tl(e){var t,n,r,i,a,o,s,l,u=e.attrsList;for(t=0,n=u.length;t<n;t++){if(r=i=u[t].name,a=u[t].value,Ks.test(r))if(e.hasBindings=!0,o=Yl(r.replace(Ks,"")),o&&(r=r.replace(rl,"")),nl.test(r))r=r.replace(nl,""),a=na(a),l=el.test(r),l&&(r=r.slice(1,-1)),o&&(o.prop&&!l&&(r=L(r),"innerHtml"===r&&(r="innerHTML")),o.camel&&!l&&(r=L(r)),o.sync&&(s=ga(a,"$event"),l?da(e,'"update:"+('+r+")",s,null,!1,$s,u[t],!0):(da(e,"update:"+L(r),s,null,!1,$s,u[t]),T(r)!==L(r)&&da(e,"update:"+T(r),s,null,!1,$s,u[t])))),o&&o.prop||!e.component&&qs(e.tag,e.attrsMap.type,r)?oa(e,r,a,u[t],l):sa(e,r,a,u[t],l);else if(Js.test(r))r=r.replace(Js,""),l=el.test(r),l&&(r=r.slice(1,-1)),da(e,r,a,o,!1,$s,u[t],l);else{r=r.replace(Ks,"");var c=r.match(tl),d=c&&c[1];l=!1,d&&(r=r.slice(0,-(d.length+1)),el.test(d)&&(d=d.slice(1,-1),l=!0)),ua(e,r,i,a,d,l,o,u[t])}else sa(e,r,JSON.stringify(a),u[t]),!e.component&&"muted"===r&&qs(e.tag,e.attrsMap.type,r)&&oa(e,r,"true",u[t])}}function Dl(e){var t=e;while(t){if(void 0!==t.for)return!0;t=t.parent}return!1}function Yl(e){var t=e.match(rl);if(t){var n={};return t.forEach((function(e){n[e.slice(1)]=!0})),n}}function Cl(e){for(var t={},n=0,r=e.length;n<r;n++)t[e[n].name]=e[n].value;return t}function El(e){return"script"===e.tag||"style"===e.tag}function Al(e){return"style"===e.tag||"script"===e.tag&&(!e.attrsMap.type||"text/javascript"===e.attrsMap.type)}var Ol=/^xmlns:NS\d+/,Pl=/^NS\d+:/;function Fl(e){for(var t=[],n=0;n<e.length;n++){var r=e[n];Ol.test(r.name)||(r.name=r.name.replace(Pl,""),t.push(r))}return t}function Il(e,t){if("input"===e.tag){var n,r=e.attrsMap;if(!r["v-model"])return;if((r[":type"]||r["v-bind:type"])&&(n=ha(e,"type")),r.type||n||!r["v-bind"]||(n="("+r["v-bind"]+").type"),n){var i=pa(e,"v-if",!0),a=i?"&&("+i+")":"",o=null!=pa(e,"v-else",!0),s=pa(e,"v-else-if",!0),l=jl(e);_l(l),la(l,"type","checkbox"),hl(l,t),l.processed=!0,l.if="("+n+")==='checkbox'"+a,Ml(l,{exp:l.if,block:l});var u=jl(e);pa(u,"v-for",!0),la(u,"type","radio"),hl(u,t),Ml(l,{exp:"("+n+")==='radio'"+a,block:u});var c=jl(e);return pa(c,"v-for",!0),la(c,":type",n),hl(c,t),Ml(l,{exp:i,block:c}),o?l.else=!0:s&&(l.elseif=s),l}}}function jl(e){return ul(e.tag,e.attrsList.slice(),e.parent)}var Hl={preTransformNode:Il},Rl=[hs,vs,Hl];function Nl(e,t){t.value&&oa(e,"textContent","_s("+t.value+")",t)}function $l(e,t){t.value&&oa(e,"innerHTML","_s("+t.value+")",t)}var zl,Bl,Wl={model:Da,text:Nl,html:$l},Ul={expectHTML:!0,modules:Rl,directives:Wl,isPreTag:si,isUnaryTag:bs,mustUseProp:zr,canBeLeftOpenTag:ys,isReservedTag:li,getTagNamespace:ui,staticKeys:j(Rl)},Vl=k(Gl);function ql(e,t){e&&(zl=Vl(t.staticKeys||""),Bl=t.isReservedTag||F,Jl(e),Kl(e,!1))}function Gl(e){return _("type,tag,attrsList,attrsMap,plain,parent,children,attrs,start,end,rawAttrsMap"+(e?","+e:""))}function Jl(e){if(e.static=Xl(e),1===e.type){if(!Bl(e.tag)&&"slot"!==e.tag&&null==e.attrsMap["inline-template"])return;for(var t=0,n=e.children.length;t<n;t++){var r=e.children[t];Jl(r),r.static||(e.static=!1)}if(e.ifConditions)for(var i=1,a=e.ifConditions.length;i<a;i++){var o=e.ifConditions[i].block;Jl(o),o.static||(e.static=!1)}}}function Kl(e,t){if(1===e.type){if((e.static||e.once)&&(e.staticInFor=t),e.static&&e.children.length&&(1!==e.children.length||3!==e.children[0].type))return void(e.staticRoot=!0);if(e.staticRoot=!1,e.children)for(var n=0,r=e.children.length;n<r;n++)Kl(e.children[n],t||!!e.for);if(e.ifConditions)for(var i=1,a=e.ifConditions.length;i<a;i++)Kl(e.ifConditions[i].block,t)}}function Xl(e){return 2!==e.type&&(3===e.type||!(!e.pre&&(e.hasBindings||e.if||e.for||v(e.tag)||!Bl(e.tag)||Ql(e)||!Object.keys(e).every(zl))))}function Ql(e){while(e.parent){if(e=e.parent,"template"!==e.tag)return!1;if(e.for)return!0}return!1}var Zl=/^([\w$_]+|\([^)]*?\))\s*=>|^function(?:\s+[\w$]+)?\s*\(/,eu=/\([^)]*?\);*$/,tu=/^[A-Za-z_$][\w$]*(?:\.[A-Za-z_$][\w$]*|\['[^']*?']|\["[^"]*?"]|\[\d+]|\[[A-Za-z_$][\w$]*])*$/,nu={esc:27,tab:9,enter:13,space:32,up:38,left:37,right:39,down:40,delete:[8,46]},ru={esc:["Esc","Escape"],tab:"Tab",enter:"Enter",space:[" ","Spacebar"],up:["Up","ArrowUp"],left:["Left","ArrowLeft"],right:["Right","ArrowRight"],down:["Down","ArrowDown"],delete:["Backspace","Delete","Del"]},iu=function(e){return"if("+e+")return null;"},au={stop:"$event.stopPropagation();",prevent:"$event.preventDefault();",self:iu("$event.target !== $event.currentTarget"),ctrl:iu("!$event.ctrlKey"),shift:iu("!$event.shiftKey"),alt:iu("!$event.altKey"),meta:iu("!$event.metaKey"),left:iu("'button' in $event && $event.button !== 0"),middle:iu("'button' in $event && $event.button !== 1"),right:iu("'button' in $event && $event.button !== 2")};function ou(e,t){var n=t?"nativeOn:":"on:",r="",i="";for(var a in e){var o=su(e[a]);e[a]&&e[a].dynamic?i+=a+","+o+",":r+='"'+a+'":'+o+","}return r="{"+r.slice(0,-1)+"}",i?n+"_d("+r+",["+i.slice(0,-1)+"])":n+r}function su(e){if(!e)return"function(){}";if(Array.isArray(e))return"["+e.map((function(e){return su(e)})).join(",")+"]";var t=tu.test(e.value),n=Zl.test(e.value),r=tu.test(e.value.replace(eu,""));if(e.modifiers){var i="",a="",o=[];for(var s in e.modifiers)if(au[s])a+=au[s],nu[s]&&o.push(s);else if("exact"===s){var l=e.modifiers;a+=iu(["ctrl","shift","alt","meta"].filter((function(e){return!l[e]})).map((function(e){return"$event."+e+"Key"})).join("||"))}else o.push(s);o.length&&(i+=lu(o)),a&&(i+=a);var u=t?"return "+e.value+"($event)":n?"return ("+e.value+")($event)":r?"return "+e.value:e.value;return"function($event){"+i+u+"}"}return t||n?e.value:"function($event){"+(r?"return "+e.value:e.value)+"}"}function lu(e){return"if(!$event.type.indexOf('key')&&"+e.map(uu).join("&&")+")return null;"}function uu(e){var t=parseInt(e,10);if(t)return"$event.keyCode!=="+t;var n=nu[e],r=ru[e];return"_k($event.keyCode,"+JSON.stringify(e)+","+JSON.stringify(n)+",$event.key,"+JSON.stringify(r)+")"}function cu(e,t){e.wrapListeners=function(e){return"_g("+e+","+t.value+")"}}function du(e,t){e.wrapData=function(n){return"_b("+n+",'"+e.tag+"',"+t.value+","+(t.modifiers&&t.modifiers.prop?"true":"false")+(t.modifiers&&t.modifiers.sync?",true":"")+")"}}var fu={on:cu,bind:du,cloak:P},hu=function(e){this.options=e,this.warn=e.warn||ia,this.transforms=aa(e.modules,"transformCode"),this.dataGenFns=aa(e.modules,"genData"),this.directives=A(A({},fu),e.directives);var t=e.isReservedTag||F;this.maybeComponent=function(e){return!!e.component||!t(e.tag)},this.onceId=0,this.staticRenderFns=[],this.pre=!1};function pu(e,t){var n=new hu(t),r=e?mu(e,n):'_c("div")';return{render:"with(this){return "+r+"}",staticRenderFns:n.staticRenderFns}}function mu(e,t){if(e.parent&&(e.pre=e.pre||e.parent.pre),e.staticRoot&&!e.staticProcessed)return _u(e,t);if(e.once&&!e.onceProcessed)return vu(e,t);if(e.for&&!e.forProcessed)return yu(e,t);if(e.if&&!e.ifProcessed)return gu(e,t);if("template"!==e.tag||e.slotTarget||t.pre){if("slot"===e.tag)return Pu(e,t);var n;if(e.component)n=Fu(e.component,e,t);else{var r;(!e.plain||e.pre&&t.maybeComponent(e))&&(r=Mu(e,t));var i=e.inlineTemplate?null:Du(e,t,!0);n="_c('"+e.tag+"'"+(r?","+r:"")+(i?","+i:"")+")"}for(var a=0;a<t.transforms.length;a++)n=t.transforms[a](e,n);return n}return Du(e,t)||"void 0"}function _u(e,t){e.staticProcessed=!0;var n=t.pre;return e.pre&&(t.pre=e.pre),t.staticRenderFns.push("with(this){return "+mu(e,t)+"}"),t.pre=n,"_m("+(t.staticRenderFns.length-1)+(e.staticInFor?",true":"")+")"}function vu(e,t){if(e.onceProcessed=!0,e.if&&!e.ifProcessed)return gu(e,t);if(e.staticInFor){var n="",r=e.parent;while(r){if(r.for){n=r.key;break}r=r.parent}return n?"_o("+mu(e,t)+","+t.onceId+++","+n+")":mu(e,t)}return _u(e,t)}function gu(e,t,n,r){return e.ifProcessed=!0,bu(e.ifConditions.slice(),t,n,r)}function bu(e,t,n,r){if(!e.length)return r||"_e()";var i=e.shift();return i.exp?"("+i.exp+")?"+a(i.block)+":"+bu(e,t,n,r):""+a(i.block);function a(e){return n?n(e,t):e.once?vu(e,t):mu(e,t)}}function yu(e,t,n,r){var i=e.for,a=e.alias,o=e.iterator1?","+e.iterator1:"",s=e.iterator2?","+e.iterator2:"";return e.forProcessed=!0,(r||"_l")+"(("+i+"),function("+a+o+s+"){return "+(n||mu)(e,t)+"})"}function Mu(e,t){var n="{",r=ku(e,t);r&&(n+=r+","),e.key&&(n+="key:"+e.key+","),e.ref&&(n+="ref:"+e.ref+","),e.refInFor&&(n+="refInFor:true,"),e.pre&&(n+="pre:true,"),e.component&&(n+='tag:"'+e.tag+'",');for(var i=0;i<t.dataGenFns.length;i++)n+=t.dataGenFns[i](e);if(e.attrs&&(n+="attrs:"+Iu(e.attrs)+","),e.props&&(n+="domProps:"+Iu(e.props)+","),e.events&&(n+=ou(e.events,!1)+","),e.nativeEvents&&(n+=ou(e.nativeEvents,!0)+","),e.slotTarget&&!e.slotScope&&(n+="slot:"+e.slotTarget+","),e.scopedSlots&&(n+=Lu(e,e.scopedSlots,t)+","),e.model&&(n+="model:{value:"+e.model.value+",callback:"+e.model.callback+",expression:"+e.model.expression+"},"),e.inlineTemplate){var a=wu(e,t);a&&(n+=a+",")}return n=n.replace(/,$/,"")+"}",e.dynamicAttrs&&(n="_b("+n+',"'+e.tag+'",'+Iu(e.dynamicAttrs)+")"),e.wrapData&&(n=e.wrapData(n)),e.wrapListeners&&(n=e.wrapListeners(n)),n}function ku(e,t){var n=e.directives;if(n){var r,i,a,o,s="directives:[",l=!1;for(r=0,i=n.length;r<i;r++){a=n[r],o=!0;var u=t.directives[a.name];u&&(o=!!u(e,a,t.warn)),o&&(l=!0,s+='{name:"'+a.name+'",rawName:"'+a.rawName+'"'+(a.value?",value:("+a.value+"),expression:"+JSON.stringify(a.value):"")+(a.arg?",arg:"+(a.isDynamicArg?a.arg:'"'+a.arg+'"'):"")+(a.modifiers?",modifiers:"+JSON.stringify(a.modifiers):"")+"},")}return l?s.slice(0,-1)+"]":void 0}}function wu(e,t){var n=e.children[0];if(n&&1===n.type){var r=pu(n,t.options);return"inlineTemplate:{render:function(){"+r.render+"},staticRenderFns:["+r.staticRenderFns.map((function(e){return"function(){"+e+"}"})).join(",")+"]}"}}function Lu(e,t,n){var r=e.for||Object.keys(t).some((function(e){var n=t[e];return n.slotTargetDynamic||n.if||n.for||Su(n)})),i=!!e.if;if(!r){var a=e.parent;while(a){if(a.slotScope&&a.slotScope!==ll||a.for){r=!0;break}a.if&&(i=!0),a=a.parent}}var o=Object.keys(t).map((function(e){return Tu(t[e],n)})).join(",");return"scopedSlots:_u(["+o+"]"+(r?",null,true":"")+(!r&&i?",null,false,"+xu(o):"")+")"}function xu(e){var t=5381,n=e.length;while(n)t=33*t^e.charCodeAt(--n);return t>>>0}function Su(e){return 1===e.type&&("slot"===e.tag||e.children.some(Su))}function Tu(e,t){var n=e.attrsMap["slot-scope"];if(e.if&&!e.ifProcessed&&!n)return gu(e,t,Tu,"null");if(e.for&&!e.forProcessed)return yu(e,t,Tu);var r=e.slotScope===ll?"":String(e.slotScope),i="function("+r+"){return "+("template"===e.tag?e.if&&n?"("+e.if+")?"+(Du(e,t)||"undefined")+":undefined":Du(e,t)||"undefined":mu(e,t))+"}",a=r?"":",proxy:true";return"{key:"+(e.slotTarget||'"default"')+",fn:"+i+a+"}"}function Du(e,t,n,r,i){var a=e.children;if(a.length){var o=a[0];if(1===a.length&&o.for&&"template"!==o.tag&&"slot"!==o.tag){var s=n?t.maybeComponent(o)?",1":",0":"";return""+(r||mu)(o,t)+s}var l=n?Yu(a,t.maybeComponent):0,u=i||Eu;return"["+a.map((function(e){return u(e,t)})).join(",")+"]"+(l?","+l:"")}}function Yu(e,t){for(var n=0,r=0;r<e.length;r++){var i=e[r];if(1===i.type){if(Cu(i)||i.ifConditions&&i.ifConditions.some((function(e){return Cu(e.block)}))){n=2;break}(t(i)||i.ifConditions&&i.ifConditions.some((function(e){return t(e.block)})))&&(n=1)}}return n}function Cu(e){return void 0!==e.for||"template"===e.tag||"slot"===e.tag}function Eu(e,t){return 1===e.type?mu(e,t):3===e.type&&e.isComment?Ou(e):Au(e)}function Au(e){return"_v("+(2===e.type?e.expression:ju(JSON.stringify(e.text)))+")"}function Ou(e){return"_e("+JSON.stringify(e.text)+")"}function Pu(e,t){var n=e.slotName||'"default"',r=Du(e,t),i="_t("+n+(r?","+r:""),a=e.attrs||e.dynamicAttrs?Iu((e.attrs||[]).concat(e.dynamicAttrs||[]).map((function(e){return{name:L(e.name),value:e.value,dynamic:e.dynamic}}))):null,o=e.attrsMap["v-bind"];return!a&&!o||r||(i+=",null"),a&&(i+=","+a),o&&(i+=(a?"":",null")+","+o),i+")"}function Fu(e,t,n){var r=t.inlineTemplate?null:Du(t,n,!0);return"_c("+e+","+Mu(t,n)+(r?","+r:"")+")"}function Iu(e){for(var t="",n="",r=0;r<e.length;r++){var i=e[r],a=ju(i.value);i.dynamic?n+=i.name+","+a+",":t+='"'+i.name+'":'+a+","}return t="{"+t.slice(0,-1)+"}",n?"_d("+t+",["+n.slice(0,-1)+"])":t}function ju(e){return e.replace(/\u2028/g,"\\u2028").replace(/\u2029/g,"\\u2029")}new RegExp("\\b"+"do,if,for,let,new,try,var,case,else,with,await,break,catch,class,const,super,throw,while,yield,delete,export,import,return,switch,default,extends,finally,continue,debugger,function,arguments".split(",").join("\\b|\\b")+"\\b"),new RegExp("\\b"+"delete,typeof,void".split(",").join("\\s*\\([^\\)]*\\)|\\b")+"\\s*\\([^\\)]*\\)");function Hu(e,t){try{return new Function(e)}catch(n){return t.push({err:n,code:e}),P}}function Ru(e){var t=Object.create(null);return function(n,r,i){r=A({},r);r.warn;delete r.warn;var a=r.delimiters?String(r.delimiters)+n:n;if(t[a])return t[a];var o=e(n,r);var s={},l=[];return s.render=Hu(o.render,l),s.staticRenderFns=o.staticRenderFns.map((function(e){return Hu(e,l)})),t[a]=s}}function Nu(e){return function(t){function n(n,r){var i=Object.create(t),a=[],o=[],s=function(e,t,n){(n?o:a).push(e)};if(r)for(var l in r.modules&&(i.modules=(t.modules||[]).concat(r.modules)),r.directives&&(i.directives=A(Object.create(t.directives||null),r.directives)),r)"modules"!==l&&"directives"!==l&&(i[l]=r[l]);i.warn=s;var u=e(n.trim(),i);return u.errors=a,u.tips=o,u}return{compile:n,compileToFunctions:Ru(n)}}}var $u,zu=Nu((function(e,t){var n=cl(e.trim(),t);!1!==t.optimize&&ql(n,t);var r=pu(n,t);return{ast:n,render:r.render,staticRenderFns:r.staticRenderFns}})),Bu=zu(Ul),Wu=(Bu.compile,Bu.compileToFunctions);function Uu(e){return $u=$u||document.createElement("div"),$u.innerHTML=e?'<a href="\n"/>':'<div a="\n"/>',$u.innerHTML.indexOf("&#10;")>0}var Vu=!!Q&&Uu(!1),qu=!!Q&&Uu(!0),Gu=k((function(e){var t=hi(e);return t&&t.innerHTML})),Ju=xr.prototype.$mount;function Ku(e){if(e.outerHTML)return e.outerHTML;var t=document.createElement("div");return t.appendChild(e.cloneNode(!0)),t.innerHTML}xr.prototype.$mount=function(e,t){if(e=e&&hi(e),e===document.body||e===document.documentElement)return this;var n=this.$options;if(!n.render){var r=n.template;if(r)if("string"===typeof r)"#"===r.charAt(0)&&(r=Gu(r));else{if(!r.nodeType)return this;r=r.innerHTML}else e&&(r=Ku(e));if(r){0;var i=Wu(r,{outputSourceRange:!1,shouldDecodeNewlines:Vu,shouldDecodeNewlinesForHref:qu,delimiters:n.delimiters,comments:n.comments},this),a=i.render,o=i.staticRenderFns;n.render=a,n.staticRenderFns=o}}return Ju.call(this,e,t)},xr.compile=Wu,t["a"]=xr}).call(this,n("c8ba"))},a078:function(e,t,n){var r=n("7b0b"),i=n("50c4"),a=n("35a1"),o=n("e95a"),s=n("0366"),l=n("ebb5").aTypedArrayConstructor;e.exports=function(e){var t,n,u,c,d,f,h=r(e),p=arguments.length,m=p>1?arguments[1]:void 0,_=void 0!==m,v=a(h);if(void 0!=v&&!o(v)){d=v.call(h),f=d.next,h=[];while(!(c=f.call(d)).done)h.push(c.value)}for(_&&p>2&&(m=s(m,arguments[2],2)),n=i(h.length),u=new(l(this))(n),t=0;n>t;t++)u[t]=_?m(h[t],t):h[t];return u}},a15b:function(e,t,n){"use strict";var r=n("23e7"),i=n("44ad"),a=n("fc6a"),o=n("a640"),s=[].join,l=i!=Object,u=o("join",",");r({target:"Array",proto:!0,forced:l||!u},{join:function(e){return s.call(a(this),void 0===e?",":e)}})},a356:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=function(e){return 0===e?0:1===e?1:2===e?2:e%100>=3&&e%100<=10?3:e%100>=11?4:5},n={s:["أقل من ثانية","ثانية واحدة",["ثانيتان","ثانيتين"],"%d ثوان","%d ثانية","%d ثانية"],m:["أقل من دقيقة","دقيقة واحدة",["دقيقتان","دقيقتين"],"%d دقائق","%d دقيقة","%d دقيقة"],h:["أقل من ساعة","ساعة واحدة",["ساعتان","ساعتين"],"%d ساعات","%d ساعة","%d ساعة"],d:["أقل من يوم","يوم واحد",["يومان","يومين"],"%d أيام","%d يومًا","%d يوم"],M:["أقل من شهر","شهر واحد",["شهران","شهرين"],"%d أشهر","%d شهرا","%d شهر"],y:["أقل من عام","عام واحد",["عامان","عامين"],"%d أعوام","%d عامًا","%d عام"]},r=function(e){return function(r,i,a,o){var s=t(r),l=n[e][t(r)];return 2===s&&(l=l[i?0:1]),l.replace(/%d/i,r)}},i=["جانفي","فيفري","مارس","أفريل","ماي","جوان","جويلية","أوت","سبتمبر","أكتوبر","نوفمبر","ديسمبر"],a=e.defineLocale("ar-dz",{months:i,monthsShort:i,weekdays:"الأحد_الإثنين_الثلاثاء_الأربعاء_الخميس_الجمعة_السبت".split("_"),weekdaysShort:"أحد_إثنين_ثلاثاء_أربعاء_خميس_جمعة_سبت".split("_"),weekdaysMin:"ح_ن_ث_ر_خ_ج_س".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"D/‏M/‏YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},meridiemParse:/ص|م/,isPM:function(e){return"م"===e},meridiem:function(e,t,n){return e<12?"ص":"م"},calendar:{sameDay:"[اليوم عند الساعة] LT",nextDay:"[غدًا عند الساعة] LT",nextWeek:"dddd [عند الساعة] LT",lastDay:"[أمس عند الساعة] LT",lastWeek:"dddd [عند الساعة] LT",sameElse:"L"},relativeTime:{future:"بعد %s",past:"منذ %s",s:r("s"),ss:r("s"),m:r("m"),mm:r("m"),h:r("h"),hh:r("h"),d:r("d"),dd:r("d"),M:r("M"),MM:r("M"),y:r("y"),yy:r("y")},postformat:function(e){return e.replace(/,/g,"،")},week:{dow:0,doy:4}});return a}))},a414:function(e,t,n){},a434:function(e,t,n){"use strict";var r=n("23e7"),i=n("23cb"),a=n("a691"),o=n("50c4"),s=n("7b0b"),l=n("65f0"),u=n("8418"),c=n("1dde"),d=n("ae40"),f=c("splice"),h=d("splice",{ACCESSORS:!0,0:0,1:2}),p=Math.max,m=Math.min,_=9007199254740991,v="Maximum allowed length exceeded";r({target:"Array",proto:!0,forced:!f||!h},{splice:function(e,t){var n,r,c,d,f,h,g=s(this),b=o(g.length),y=i(e,b),M=arguments.length;if(0===M?n=r=0:1===M?(n=0,r=b-y):(n=M-2,r=m(p(a(t),0),b-y)),b+n-r>_)throw TypeError(v);for(c=l(g,r),d=0;d<r;d++)f=y+d,f in g&&u(c,d,g[f]);if(c.length=r,n<r){for(d=y;d<b-r;d++)f=d+r,h=d+n,f in g?g[h]=g[f]:delete g[h];for(d=b;d>b-r+n;d--)delete g[d-1]}else if(n>r)for(d=b-r;d>y;d--)f=d+r-1,h=d+n-1,f in g?g[h]=g[f]:delete g[h];for(d=0;d<n;d++)g[d+y]=arguments[d+2];return g.length=b-r+n,c}})},a4d3:function(e,t,n){"use strict";var r=n("23e7"),i=n("da84"),a=n("d066"),o=n("c430"),s=n("83ab"),l=n("4930"),u=n("fdbf"),c=n("d039"),d=n("5135"),f=n("e8b5"),h=n("861d"),p=n("825a"),m=n("7b0b"),_=n("fc6a"),v=n("c04e"),g=n("5c6c"),b=n("7c73"),y=n("df75"),M=n("241c"),k=n("057f"),w=n("7418"),L=n("06cf"),x=n("9bf2"),S=n("d1e7"),T=n("9112"),D=n("6eeb"),Y=n("5692"),C=n("f772"),E=n("d012"),A=n("90e3"),O=n("b622"),P=n("e538"),F=n("746f"),I=n("d44e"),j=n("69f3"),H=n("b727").forEach,R=C("hidden"),N="Symbol",$="prototype",z=O("toPrimitive"),B=j.set,W=j.getterFor(N),U=Object[$],V=i.Symbol,q=a("JSON","stringify"),G=L.f,J=x.f,K=k.f,X=S.f,Q=Y("symbols"),Z=Y("op-symbols"),ee=Y("string-to-symbol-registry"),te=Y("symbol-to-string-registry"),ne=Y("wks"),re=i.QObject,ie=!re||!re[$]||!re[$].findChild,ae=s&&c((function(){return 7!=b(J({},"a",{get:function(){return J(this,"a",{value:7}).a}})).a}))?function(e,t,n){var r=G(U,t);r&&delete U[t],J(e,t,n),r&&e!==U&&J(U,t,r)}:J,oe=function(e,t){var n=Q[e]=b(V[$]);return B(n,{type:N,tag:e,description:t}),s||(n.description=t),n},se=u?function(e){return"symbol"==typeof e}:function(e){return Object(e)instanceof V},le=function(e,t,n){e===U&&le(Z,t,n),p(e);var r=v(t,!0);return p(n),d(Q,r)?(n.enumerable?(d(e,R)&&e[R][r]&&(e[R][r]=!1),n=b(n,{enumerable:g(0,!1)})):(d(e,R)||J(e,R,g(1,{})),e[R][r]=!0),ae(e,r,n)):J(e,r,n)},ue=function(e,t){p(e);var n=_(t),r=y(n).concat(pe(n));return H(r,(function(t){s&&!de.call(n,t)||le(e,t,n[t])})),e},ce=function(e,t){return void 0===t?b(e):ue(b(e),t)},de=function(e){var t=v(e,!0),n=X.call(this,t);return!(this===U&&d(Q,t)&&!d(Z,t))&&(!(n||!d(this,t)||!d(Q,t)||d(this,R)&&this[R][t])||n)},fe=function(e,t){var n=_(e),r=v(t,!0);if(n!==U||!d(Q,r)||d(Z,r)){var i=G(n,r);return!i||!d(Q,r)||d(n,R)&&n[R][r]||(i.enumerable=!0),i}},he=function(e){var t=K(_(e)),n=[];return H(t,(function(e){d(Q,e)||d(E,e)||n.push(e)})),n},pe=function(e){var t=e===U,n=K(t?Z:_(e)),r=[];return H(n,(function(e){!d(Q,e)||t&&!d(U,e)||r.push(Q[e])})),r};if(l||(V=function(){if(this instanceof V)throw TypeError("Symbol is not a constructor");var e=arguments.length&&void 0!==arguments[0]?String(arguments[0]):void 0,t=A(e),n=function(e){this===U&&n.call(Z,e),d(this,R)&&d(this[R],t)&&(this[R][t]=!1),ae(this,t,g(1,e))};return s&&ie&&ae(U,t,{configurable:!0,set:n}),oe(t,e)},D(V[$],"toString",(function(){return W(this).tag})),D(V,"withoutSetter",(function(e){return oe(A(e),e)})),S.f=de,x.f=le,L.f=fe,M.f=k.f=he,w.f=pe,P.f=function(e){return oe(O(e),e)},s&&(J(V[$],"description",{configurable:!0,get:function(){return W(this).description}}),o||D(U,"propertyIsEnumerable",de,{unsafe:!0}))),r({global:!0,wrap:!0,forced:!l,sham:!l},{Symbol:V}),H(y(ne),(function(e){F(e)})),r({target:N,stat:!0,forced:!l},{for:function(e){var t=String(e);if(d(ee,t))return ee[t];var n=V(t);return ee[t]=n,te[n]=t,n},keyFor:function(e){if(!se(e))throw TypeError(e+" is not a symbol");if(d(te,e))return te[e]},useSetter:function(){ie=!0},useSimple:function(){ie=!1}}),r({target:"Object",stat:!0,forced:!l,sham:!s},{create:ce,defineProperty:le,defineProperties:ue,getOwnPropertyDescriptor:fe}),r({target:"Object",stat:!0,forced:!l},{getOwnPropertyNames:he,getOwnPropertySymbols:pe}),r({target:"Object",stat:!0,forced:c((function(){w.f(1)}))},{getOwnPropertySymbols:function(e){return w.f(m(e))}}),q){var me=!l||c((function(){var e=V();return"[null]"!=q([e])||"{}"!=q({a:e})||"{}"!=q(Object(e))}));r({target:"JSON",stat:!0,forced:me},{stringify:function(e,t,n){var r,i=[e],a=1;while(arguments.length>a)i.push(arguments[a++]);if(r=t,(h(t)||void 0!==e)&&!se(e))return f(t)||(t=function(e,t){if("function"==typeof r&&(t=r.call(this,e,t)),!se(t))return t}),i[1]=t,q.apply(null,i)}})}V[$][z]||T(V[$],z,V[$].valueOf),I(V,N),E[R]=!0},a630:function(e,t,n){var r=n("23e7"),i=n("4df4"),a=n("1c7e"),o=!a((function(e){Array.from(e)}));r({target:"Array",stat:!0,forced:o},{from:i})},a639:function(e,t,n){"use strict";n("0d8e")},a640:function(e,t,n){"use strict";var r=n("d039");e.exports=function(e,t){var n=[][e];return!!n&&r((function(){n.call(null,t||function(){throw 1},1)}))}},a691:function(e,t){var n=Math.ceil,r=Math.floor;e.exports=function(e){return isNaN(e=+e)?0:(e>0?r:n)(e)}},a718:function(e,t,n){},a79d:function(e,t,n){"use strict";var r=n("23e7"),i=n("c430"),a=n("fea9"),o=n("d039"),s=n("d066"),l=n("4840"),u=n("cdf9"),c=n("6eeb"),d=!!a&&o((function(){a.prototype["finally"].call({then:function(){}},(function(){}))}));r({target:"Promise",proto:!0,real:!0,forced:d},{finally:function(e){var t=l(this,s("Promise")),n="function"==typeof e;return this.then(n?function(n){return u(t,e()).then((function(){return n}))}:e,n?function(n){return u(t,e()).then((function(){throw n}))}:e)}}),i||"function"!=typeof a||a.prototype["finally"]||c(a.prototype,"finally",s("Promise").prototype["finally"])},a7fa:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("bm",{months:"Zanwuyekalo_Fewuruyekalo_Marisikalo_Awirilikalo_Mɛkalo_Zuwɛnkalo_Zuluyekalo_Utikalo_Sɛtanburukalo_ɔkutɔburukalo_Nowanburukalo_Desanburukalo".split("_"),monthsShort:"Zan_Few_Mar_Awi_Mɛ_Zuw_Zul_Uti_Sɛt_ɔku_Now_Des".split("_"),weekdays:"Kari_Ntɛnɛn_Tarata_Araba_Alamisa_Juma_Sibiri".split("_"),weekdaysShort:"Kar_Ntɛ_Tar_Ara_Ala_Jum_Sib".split("_"),weekdaysMin:"Ka_Nt_Ta_Ar_Al_Ju_Si".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"MMMM [tile] D [san] YYYY",LLL:"MMMM [tile] D [san] YYYY [lɛrɛ] HH:mm",LLLL:"dddd MMMM [tile] D [san] YYYY [lɛrɛ] HH:mm"},calendar:{sameDay:"[Bi lɛrɛ] LT",nextDay:"[Sini lɛrɛ] LT",nextWeek:"dddd [don lɛrɛ] LT",lastDay:"[Kunu lɛrɛ] LT",lastWeek:"dddd [tɛmɛnen lɛrɛ] LT",sameElse:"L"},relativeTime:{future:"%s kɔnɔ",past:"a bɛ %s bɔ",s:"sanga dama dama",ss:"sekondi %d",m:"miniti kelen",mm:"miniti %d",h:"lɛrɛ kelen",hh:"lɛrɛ %d",d:"tile kelen",dd:"tile %d",M:"kalo kelen",MM:"kalo %d",y:"san kelen",yy:"san %d"},week:{dow:1,doy:4}});return t}))},a832:function(e,t,n){"use strict";
/*!
 * isobject <https://github.com/jonschlinkert/isobject>
 *
 * Copyright (c) 2014-2017, Jon Schlinkert.
 * Released under the MIT License.
 */e.exports=function(e){return null!=e&&"object"===typeof e&&!1===Array.isArray(e)}},a975:function(e,t,n){"use strict";var r=n("ebb5"),i=n("b727").every,a=r.aTypedArray,o=r.exportTypedArrayMethod;o("every",(function(e){return i(a(this),e,arguments.length>1?arguments[1]:void 0)}))},a981:function(e,t){e.exports="undefined"!==typeof ArrayBuffer&&"undefined"!==typeof DataView},a9e3:function(e,t,n){"use strict";var r=n("83ab"),i=n("da84"),a=n("94ca"),o=n("6eeb"),s=n("5135"),l=n("c6b6"),u=n("7156"),c=n("c04e"),d=n("d039"),f=n("7c73"),h=n("241c").f,p=n("06cf").f,m=n("9bf2").f,_=n("58a8").trim,v="Number",g=i[v],b=g.prototype,y=l(f(b))==v,M=function(e){var t,n,r,i,a,o,s,l,u=c(e,!1);if("string"==typeof u&&u.length>2)if(u=_(u),t=u.charCodeAt(0),43===t||45===t){if(n=u.charCodeAt(2),88===n||120===n)return NaN}else if(48===t){switch(u.charCodeAt(1)){case 66:case 98:r=2,i=49;break;case 79:case 111:r=8,i=55;break;default:return+u}for(a=u.slice(2),o=a.length,s=0;s<o;s++)if(l=a.charCodeAt(s),l<48||l>i)return NaN;return parseInt(a,r)}return+u};if(a(v,!g(" 0o1")||!g("0b1")||g("+0x1"))){for(var k,w=function(e){var t=arguments.length<1?0:e,n=this;return n instanceof w&&(y?d((function(){b.valueOf.call(n)})):l(n)!=v)?u(new g(M(t)),n,w):M(t)},L=r?h(g):"MAX_VALUE,MIN_VALUE,NaN,NEGATIVE_INFINITY,POSITIVE_INFINITY,EPSILON,isFinite,isInteger,isNaN,isSafeInteger,MAX_SAFE_INTEGER,MIN_SAFE_INTEGER,parseFloat,parseInt,isInteger".split(","),x=0;L.length>x;x++)s(g,k=L[x])&&!s(w,k)&&m(w,k,p(g,k));w.prototype=b,b.constructor=w,o(i,v,w)}},aa47:function(e,t,n){"use strict";
/**!
 * Sortable 1.10.2
 * @author	RubaXa   <trash@rubaxa.org>
 * @author	owenm    <owen23355@gmail.com>
 * @license MIT
 */
function r(e){return r="function"===typeof Symbol&&"symbol"===typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"===typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e},r(e)}function i(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function a(){return a=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},a.apply(this,arguments)}function o(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{},r=Object.keys(n);"function"===typeof Object.getOwnPropertySymbols&&(r=r.concat(Object.getOwnPropertySymbols(n).filter((function(e){return Object.getOwnPropertyDescriptor(n,e).enumerable})))),r.forEach((function(t){i(e,t,n[t])}))}return e}function s(e,t){if(null==e)return{};var n,r,i={},a=Object.keys(e);for(r=0;r<a.length;r++)n=a[r],t.indexOf(n)>=0||(i[n]=e[n]);return i}function l(e,t){if(null==e)return{};var n,r,i=s(e,t);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(r=0;r<a.length;r++)n=a[r],t.indexOf(n)>=0||Object.prototype.propertyIsEnumerable.call(e,n)&&(i[n]=e[n])}return i}function u(e){return c(e)||d(e)||f()}function c(e){if(Array.isArray(e)){for(var t=0,n=new Array(e.length);t<e.length;t++)n[t]=e[t];return n}}function d(e){if(Symbol.iterator in Object(e)||"[object Arguments]"===Object.prototype.toString.call(e))return Array.from(e)}function f(){throw new TypeError("Invalid attempt to spread non-iterable instance")}n.r(t),n.d(t,"MultiDrag",(function(){return Ht})),n.d(t,"Sortable",(function(){return Qe})),n.d(t,"Swap",(function(){return St}));var h="1.10.2";function p(e){if("undefined"!==typeof window&&window.navigator)return!!navigator.userAgent.match(e)}var m=p(/(?:Trident.*rv[ :]?11\.|msie|iemobile|Windows Phone)/i),_=p(/Edge/i),v=p(/firefox/i),g=p(/safari/i)&&!p(/chrome/i)&&!p(/android/i),b=p(/iP(ad|od|hone)/i),y=p(/chrome/i)&&p(/android/i),M={capture:!1,passive:!1};function k(e,t,n){e.addEventListener(t,n,!m&&M)}function w(e,t,n){e.removeEventListener(t,n,!m&&M)}function L(e,t){if(t){if(">"===t[0]&&(t=t.substring(1)),e)try{if(e.matches)return e.matches(t);if(e.msMatchesSelector)return e.msMatchesSelector(t);if(e.webkitMatchesSelector)return e.webkitMatchesSelector(t)}catch(n){return!1}return!1}}function x(e){return e.host&&e!==document&&e.host.nodeType?e.host:e.parentNode}function S(e,t,n,r){if(e){n=n||document;do{if(null!=t&&(">"===t[0]?e.parentNode===n&&L(e,t):L(e,t))||r&&e===n)return e;if(e===n)break}while(e=x(e))}return null}var T,D=/\s+/g;function Y(e,t,n){if(e&&t)if(e.classList)e.classList[n?"add":"remove"](t);else{var r=(" "+e.className+" ").replace(D," ").replace(" "+t+" "," ");e.className=(r+(n?" "+t:"")).replace(D," ")}}function C(e,t,n){var r=e&&e.style;if(r){if(void 0===n)return document.defaultView&&document.defaultView.getComputedStyle?n=document.defaultView.getComputedStyle(e,""):e.currentStyle&&(n=e.currentStyle),void 0===t?n:n[t];t in r||-1!==t.indexOf("webkit")||(t="-webkit-"+t),r[t]=n+("string"===typeof n?"":"px")}}function E(e,t){var n="";if("string"===typeof e)n=e;else do{var r=C(e,"transform");r&&"none"!==r&&(n=r+" "+n)}while(!t&&(e=e.parentNode));var i=window.DOMMatrix||window.WebKitCSSMatrix||window.CSSMatrix||window.MSCSSMatrix;return i&&new i(n)}function A(e,t,n){if(e){var r=e.getElementsByTagName(t),i=0,a=r.length;if(n)for(;i<a;i++)n(r[i],i);return r}return[]}function O(){var e=document.scrollingElement;return e||document.documentElement}function P(e,t,n,r,i){if(e.getBoundingClientRect||e===window){var a,o,s,l,u,c,d;if(e!==window&&e!==O()?(a=e.getBoundingClientRect(),o=a.top,s=a.left,l=a.bottom,u=a.right,c=a.height,d=a.width):(o=0,s=0,l=window.innerHeight,u=window.innerWidth,c=window.innerHeight,d=window.innerWidth),(t||n)&&e!==window&&(i=i||e.parentNode,!m))do{if(i&&i.getBoundingClientRect&&("none"!==C(i,"transform")||n&&"static"!==C(i,"position"))){var f=i.getBoundingClientRect();o-=f.top+parseInt(C(i,"border-top-width")),s-=f.left+parseInt(C(i,"border-left-width")),l=o+a.height,u=s+a.width;break}}while(i=i.parentNode);if(r&&e!==window){var h=E(i||e),p=h&&h.a,_=h&&h.d;h&&(o/=_,s/=p,d/=p,c/=_,l=o+c,u=s+d)}return{top:o,left:s,bottom:l,right:u,width:d,height:c}}}function F(e,t,n){var r=$(e,!0),i=P(e)[t];while(r){var a=P(r)[n],o=void 0;if(o="top"===n||"left"===n?i>=a:i<=a,!o)return r;if(r===O())break;r=$(r,!1)}return!1}function I(e,t,n){var r=0,i=0,a=e.children;while(i<a.length){if("none"!==a[i].style.display&&a[i]!==Qe.ghost&&a[i]!==Qe.dragged&&S(a[i],n.draggable,e,!1)){if(r===t)return a[i];r++}i++}return null}function j(e,t){var n=e.lastElementChild;while(n&&(n===Qe.ghost||"none"===C(n,"display")||t&&!L(n,t)))n=n.previousElementSibling;return n||null}function H(e,t){var n=0;if(!e||!e.parentNode)return-1;while(e=e.previousElementSibling)"TEMPLATE"===e.nodeName.toUpperCase()||e===Qe.clone||t&&!L(e,t)||n++;return n}function R(e){var t=0,n=0,r=O();if(e)do{var i=E(e),a=i.a,o=i.d;t+=e.scrollLeft*a,n+=e.scrollTop*o}while(e!==r&&(e=e.parentNode));return[t,n]}function N(e,t){for(var n in e)if(e.hasOwnProperty(n))for(var r in t)if(t.hasOwnProperty(r)&&t[r]===e[n][r])return Number(n);return-1}function $(e,t){if(!e||!e.getBoundingClientRect)return O();var n=e,r=!1;do{if(n.clientWidth<n.scrollWidth||n.clientHeight<n.scrollHeight){var i=C(n);if(n.clientWidth<n.scrollWidth&&("auto"==i.overflowX||"scroll"==i.overflowX)||n.clientHeight<n.scrollHeight&&("auto"==i.overflowY||"scroll"==i.overflowY)){if(!n.getBoundingClientRect||n===document.body)return O();if(r||t)return n;r=!0}}}while(n=n.parentNode);return O()}function z(e,t){if(e&&t)for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n]);return e}function B(e,t){return Math.round(e.top)===Math.round(t.top)&&Math.round(e.left)===Math.round(t.left)&&Math.round(e.height)===Math.round(t.height)&&Math.round(e.width)===Math.round(t.width)}function W(e,t){return function(){if(!T){var n=arguments,r=this;1===n.length?e.call(r,n[0]):e.apply(r,n),T=setTimeout((function(){T=void 0}),t)}}}function U(){clearTimeout(T),T=void 0}function V(e,t,n){e.scrollLeft+=t,e.scrollTop+=n}function q(e){var t=window.Polymer,n=window.jQuery||window.Zepto;return t&&t.dom?t.dom(e).cloneNode(!0):n?n(e).clone(!0)[0]:e.cloneNode(!0)}function G(e,t){C(e,"position","absolute"),C(e,"top",t.top),C(e,"left",t.left),C(e,"width",t.width),C(e,"height",t.height)}function J(e){C(e,"position",""),C(e,"top",""),C(e,"left",""),C(e,"width",""),C(e,"height","")}var K="Sortable"+(new Date).getTime();function X(){var e,t=[];return{captureAnimationState:function(){if(t=[],this.options.animation){var e=[].slice.call(this.el.children);e.forEach((function(e){if("none"!==C(e,"display")&&e!==Qe.ghost){t.push({target:e,rect:P(e)});var n=o({},t[t.length-1].rect);if(e.thisAnimationDuration){var r=E(e,!0);r&&(n.top-=r.f,n.left-=r.e)}e.fromRect=n}}))}},addAnimationState:function(e){t.push(e)},removeAnimationState:function(e){t.splice(N(t,{target:e}),1)},animateAll:function(n){var r=this;if(!this.options.animation)return clearTimeout(e),void("function"===typeof n&&n());var i=!1,a=0;t.forEach((function(e){var t=0,n=e.target,o=n.fromRect,s=P(n),l=n.prevFromRect,u=n.prevToRect,c=e.rect,d=E(n,!0);d&&(s.top-=d.f,s.left-=d.e),n.toRect=s,n.thisAnimationDuration&&B(l,s)&&!B(o,s)&&(c.top-s.top)/(c.left-s.left)===(o.top-s.top)/(o.left-s.left)&&(t=Z(c,l,u,r.options)),B(s,o)||(n.prevFromRect=o,n.prevToRect=s,t||(t=r.options.animation),r.animate(n,c,s,t)),t&&(i=!0,a=Math.max(a,t),clearTimeout(n.animationResetTimer),n.animationResetTimer=setTimeout((function(){n.animationTime=0,n.prevFromRect=null,n.fromRect=null,n.prevToRect=null,n.thisAnimationDuration=null}),t),n.thisAnimationDuration=t)})),clearTimeout(e),i?e=setTimeout((function(){"function"===typeof n&&n()}),a):"function"===typeof n&&n(),t=[]},animate:function(e,t,n,r){if(r){C(e,"transition",""),C(e,"transform","");var i=E(this.el),a=i&&i.a,o=i&&i.d,s=(t.left-n.left)/(a||1),l=(t.top-n.top)/(o||1);e.animatingX=!!s,e.animatingY=!!l,C(e,"transform","translate3d("+s+"px,"+l+"px,0)"),Q(e),C(e,"transition","transform "+r+"ms"+(this.options.easing?" "+this.options.easing:"")),C(e,"transform","translate3d(0,0,0)"),"number"===typeof e.animated&&clearTimeout(e.animated),e.animated=setTimeout((function(){C(e,"transition",""),C(e,"transform",""),e.animated=!1,e.animatingX=!1,e.animatingY=!1}),r)}}}}function Q(e){return e.offsetWidth}function Z(e,t,n,r){return Math.sqrt(Math.pow(t.top-e.top,2)+Math.pow(t.left-e.left,2))/Math.sqrt(Math.pow(t.top-n.top,2)+Math.pow(t.left-n.left,2))*r.animation}var ee=[],te={initializeByDefault:!0},ne={mount:function(e){for(var t in te)te.hasOwnProperty(t)&&!(t in e)&&(e[t]=te[t]);ee.push(e)},pluginEvent:function(e,t,n){var r=this;this.eventCanceled=!1,n.cancel=function(){r.eventCanceled=!0};var i=e+"Global";ee.forEach((function(r){t[r.pluginName]&&(t[r.pluginName][i]&&t[r.pluginName][i](o({sortable:t},n)),t.options[r.pluginName]&&t[r.pluginName][e]&&t[r.pluginName][e](o({sortable:t},n)))}))},initializePlugins:function(e,t,n,r){for(var i in ee.forEach((function(r){var i=r.pluginName;if(e.options[i]||r.initializeByDefault){var o=new r(e,t,e.options);o.sortable=e,o.options=e.options,e[i]=o,a(n,o.defaults)}})),e.options)if(e.options.hasOwnProperty(i)){var o=this.modifyOption(e,i,e.options[i]);"undefined"!==typeof o&&(e.options[i]=o)}},getEventProperties:function(e,t){var n={};return ee.forEach((function(r){"function"===typeof r.eventProperties&&a(n,r.eventProperties.call(t[r.pluginName],e))})),n},modifyOption:function(e,t,n){var r;return ee.forEach((function(i){e[i.pluginName]&&i.optionListeners&&"function"===typeof i.optionListeners[t]&&(r=i.optionListeners[t].call(e[i.pluginName],n))})),r}};function re(e){var t=e.sortable,n=e.rootEl,r=e.name,i=e.targetEl,a=e.cloneEl,s=e.toEl,l=e.fromEl,u=e.oldIndex,c=e.newIndex,d=e.oldDraggableIndex,f=e.newDraggableIndex,h=e.originalEvent,p=e.putSortable,v=e.extraEventProperties;if(t=t||n&&n[K],t){var g,b=t.options,y="on"+r.charAt(0).toUpperCase()+r.substr(1);!window.CustomEvent||m||_?(g=document.createEvent("Event"),g.initEvent(r,!0,!0)):g=new CustomEvent(r,{bubbles:!0,cancelable:!0}),g.to=s||n,g.from=l||n,g.item=i||n,g.clone=a,g.oldIndex=u,g.newIndex=c,g.oldDraggableIndex=d,g.newDraggableIndex=f,g.originalEvent=h,g.pullMode=p?p.lastPutMode:void 0;var M=o({},v,ne.getEventProperties(r,t));for(var k in M)g[k]=M[k];n&&n.dispatchEvent(g),b[y]&&b[y].call(t,g)}}var ie=function(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},r=n.evt,i=l(n,["evt"]);ne.pluginEvent.bind(Qe)(e,t,o({dragEl:oe,parentEl:se,ghostEl:le,rootEl:ue,nextEl:ce,lastDownEl:de,cloneEl:fe,cloneHidden:he,dragStarted:Se,putSortable:be,activeSortable:Qe.active,originalEvent:r,oldIndex:pe,oldDraggableIndex:_e,newIndex:me,newDraggableIndex:ve,hideGhostForTarget:Ge,unhideGhostForTarget:Je,cloneNowHidden:function(){he=!0},cloneNowShown:function(){he=!1},dispatchSortableEvent:function(e){ae({sortable:t,name:e,originalEvent:r})}},i))};function ae(e){re(o({putSortable:be,cloneEl:fe,targetEl:oe,rootEl:ue,oldIndex:pe,oldDraggableIndex:_e,newIndex:me,newDraggableIndex:ve},e))}var oe,se,le,ue,ce,de,fe,he,pe,me,_e,ve,ge,be,ye,Me,ke,we,Le,xe,Se,Te,De,Ye,Ce,Ee=!1,Ae=!1,Oe=[],Pe=!1,Fe=!1,Ie=[],je=!1,He=[],Re="undefined"!==typeof document,Ne=b,$e=_||m?"cssFloat":"float",ze=Re&&!y&&!b&&"draggable"in document.createElement("div"),Be=function(){if(Re){if(m)return!1;var e=document.createElement("x");return e.style.cssText="pointer-events:auto","auto"===e.style.pointerEvents}}(),We=function(e,t){var n=C(e),r=parseInt(n.width)-parseInt(n.paddingLeft)-parseInt(n.paddingRight)-parseInt(n.borderLeftWidth)-parseInt(n.borderRightWidth),i=I(e,0,t),a=I(e,1,t),o=i&&C(i),s=a&&C(a),l=o&&parseInt(o.marginLeft)+parseInt(o.marginRight)+P(i).width,u=s&&parseInt(s.marginLeft)+parseInt(s.marginRight)+P(a).width;if("flex"===n.display)return"column"===n.flexDirection||"column-reverse"===n.flexDirection?"vertical":"horizontal";if("grid"===n.display)return n.gridTemplateColumns.split(" ").length<=1?"vertical":"horizontal";if(i&&o["float"]&&"none"!==o["float"]){var c="left"===o["float"]?"left":"right";return!a||"both"!==s.clear&&s.clear!==c?"horizontal":"vertical"}return i&&("block"===o.display||"flex"===o.display||"table"===o.display||"grid"===o.display||l>=r&&"none"===n[$e]||a&&"none"===n[$e]&&l+u>r)?"vertical":"horizontal"},Ue=function(e,t,n){var r=n?e.left:e.top,i=n?e.right:e.bottom,a=n?e.width:e.height,o=n?t.left:t.top,s=n?t.right:t.bottom,l=n?t.width:t.height;return r===o||i===s||r+a/2===o+l/2},Ve=function(e,t){var n;return Oe.some((function(r){if(!j(r)){var i=P(r),a=r[K].options.emptyInsertThreshold,o=e>=i.left-a&&e<=i.right+a,s=t>=i.top-a&&t<=i.bottom+a;return a&&o&&s?n=r:void 0}})),n},qe=function(e){function t(e,n){return function(r,i,a,o){var s=r.options.group.name&&i.options.group.name&&r.options.group.name===i.options.group.name;if(null==e&&(n||s))return!0;if(null==e||!1===e)return!1;if(n&&"clone"===e)return e;if("function"===typeof e)return t(e(r,i,a,o),n)(r,i,a,o);var l=(n?r:i).options.group.name;return!0===e||"string"===typeof e&&e===l||e.join&&e.indexOf(l)>-1}}var n={},i=e.group;i&&"object"==r(i)||(i={name:i}),n.name=i.name,n.checkPull=t(i.pull,!0),n.checkPut=t(i.put),n.revertClone=i.revertClone,e.group=n},Ge=function(){!Be&&le&&C(le,"display","none")},Je=function(){!Be&&le&&C(le,"display","")};Re&&document.addEventListener("click",(function(e){if(Ae)return e.preventDefault(),e.stopPropagation&&e.stopPropagation(),e.stopImmediatePropagation&&e.stopImmediatePropagation(),Ae=!1,!1}),!0);var Ke=function(e){if(oe){e=e.touches?e.touches[0]:e;var t=Ve(e.clientX,e.clientY);if(t){var n={};for(var r in e)e.hasOwnProperty(r)&&(n[r]=e[r]);n.target=n.rootEl=t,n.preventDefault=void 0,n.stopPropagation=void 0,t[K]._onDragOver(n)}}},Xe=function(e){oe&&oe.parentNode[K]._isOutsideThisEl(e.target)};function Qe(e,t){if(!e||!e.nodeType||1!==e.nodeType)throw"Sortable: `el` must be an HTMLElement, not ".concat({}.toString.call(e));this.el=e,this.options=t=a({},t),e[K]=this;var n={group:null,sort:!0,disabled:!1,store:null,handle:null,draggable:/^[uo]l$/i.test(e.nodeName)?">li":">*",swapThreshold:1,invertSwap:!1,invertedSwapThreshold:null,removeCloneOnHide:!0,direction:function(){return We(e,this.options)},ghostClass:"sortable-ghost",chosenClass:"sortable-chosen",dragClass:"sortable-drag",ignore:"a, img",filter:null,preventOnFilter:!0,animation:0,easing:null,setData:function(e,t){e.setData("Text",t.textContent)},dropBubble:!1,dragoverBubble:!1,dataIdAttr:"data-id",delay:0,delayOnTouchOnly:!1,touchStartThreshold:(Number.parseInt?Number:window).parseInt(window.devicePixelRatio,10)||1,forceFallback:!1,fallbackClass:"sortable-fallback",fallbackOnBody:!1,fallbackTolerance:0,fallbackOffset:{x:0,y:0},supportPointer:!1!==Qe.supportPointer&&"PointerEvent"in window,emptyInsertThreshold:5};for(var r in ne.initializePlugins(this,e,n),n)!(r in t)&&(t[r]=n[r]);for(var i in qe(t),this)"_"===i.charAt(0)&&"function"===typeof this[i]&&(this[i]=this[i].bind(this));this.nativeDraggable=!t.forceFallback&&ze,this.nativeDraggable&&(this.options.touchStartThreshold=1),t.supportPointer?k(e,"pointerdown",this._onTapStart):(k(e,"mousedown",this._onTapStart),k(e,"touchstart",this._onTapStart)),this.nativeDraggable&&(k(e,"dragover",this),k(e,"dragenter",this)),Oe.push(this.el),t.store&&t.store.get&&this.sort(t.store.get(this)||[]),a(this,X())}function Ze(e){e.dataTransfer&&(e.dataTransfer.dropEffect="move"),e.cancelable&&e.preventDefault()}function et(e,t,n,r,i,a,o,s){var l,u,c=e[K],d=c.options.onMove;return!window.CustomEvent||m||_?(l=document.createEvent("Event"),l.initEvent("move",!0,!0)):l=new CustomEvent("move",{bubbles:!0,cancelable:!0}),l.to=t,l.from=e,l.dragged=n,l.draggedRect=r,l.related=i||t,l.relatedRect=a||P(t),l.willInsertAfter=s,l.originalEvent=o,e.dispatchEvent(l),d&&(u=d.call(c,l,o)),u}function tt(e){e.draggable=!1}function nt(){je=!1}function rt(e,t,n){var r=P(j(n.el,n.options.draggable)),i=10;return t?e.clientX>r.right+i||e.clientX<=r.right&&e.clientY>r.bottom&&e.clientX>=r.left:e.clientX>r.right&&e.clientY>r.top||e.clientX<=r.right&&e.clientY>r.bottom+i}function it(e,t,n,r,i,a,o,s){var l=r?e.clientY:e.clientX,u=r?n.height:n.width,c=r?n.top:n.left,d=r?n.bottom:n.right,f=!1;if(!o)if(s&&Ye<u*i){if(!Pe&&(1===De?l>c+u*a/2:l<d-u*a/2)&&(Pe=!0),Pe)f=!0;else if(1===De?l<c+Ye:l>d-Ye)return-De}else if(l>c+u*(1-i)/2&&l<d-u*(1-i)/2)return at(t);return f=f||o,f&&(l<c+u*a/2||l>d-u*a/2)?l>c+u/2?1:-1:0}function at(e){return H(oe)<H(e)?1:-1}function ot(e){var t=e.tagName+e.className+e.src+e.href+e.textContent,n=t.length,r=0;while(n--)r+=t.charCodeAt(n);return r.toString(36)}function st(e){He.length=0;var t=e.getElementsByTagName("input"),n=t.length;while(n--){var r=t[n];r.checked&&He.push(r)}}function lt(e){return setTimeout(e,0)}function ut(e){return clearTimeout(e)}Qe.prototype={constructor:Qe,_isOutsideThisEl:function(e){this.el.contains(e)||e===this.el||(Te=null)},_getDirection:function(e,t){return"function"===typeof this.options.direction?this.options.direction.call(this,e,t,oe):this.options.direction},_onTapStart:function(e){if(e.cancelable){var t=this,n=this.el,r=this.options,i=r.preventOnFilter,a=e.type,o=e.touches&&e.touches[0]||e.pointerType&&"touch"===e.pointerType&&e,s=(o||e).target,l=e.target.shadowRoot&&(e.path&&e.path[0]||e.composedPath&&e.composedPath()[0])||s,u=r.filter;if(st(n),!oe&&!(/mousedown|pointerdown/.test(a)&&0!==e.button||r.disabled)&&!l.isContentEditable&&(s=S(s,r.draggable,n,!1),(!s||!s.animated)&&de!==s)){if(pe=H(s),_e=H(s,r.draggable),"function"===typeof u){if(u.call(this,e,s,this))return ae({sortable:t,rootEl:l,name:"filter",targetEl:s,toEl:n,fromEl:n}),ie("filter",t,{evt:e}),void(i&&e.cancelable&&e.preventDefault())}else if(u&&(u=u.split(",").some((function(r){if(r=S(l,r.trim(),n,!1),r)return ae({sortable:t,rootEl:r,name:"filter",targetEl:s,fromEl:n,toEl:n}),ie("filter",t,{evt:e}),!0})),u))return void(i&&e.cancelable&&e.preventDefault());r.handle&&!S(l,r.handle,n,!1)||this._prepareDragStart(e,o,s)}}},_prepareDragStart:function(e,t,n){var r,i=this,a=i.el,o=i.options,s=a.ownerDocument;if(n&&!oe&&n.parentNode===a){var l=P(n);if(ue=a,oe=n,se=oe.parentNode,ce=oe.nextSibling,de=n,ge=o.group,Qe.dragged=oe,ye={target:oe,clientX:(t||e).clientX,clientY:(t||e).clientY},Le=ye.clientX-l.left,xe=ye.clientY-l.top,this._lastX=(t||e).clientX,this._lastY=(t||e).clientY,oe.style["will-change"]="all",r=function(){ie("delayEnded",i,{evt:e}),Qe.eventCanceled?i._onDrop():(i._disableDelayedDragEvents(),!v&&i.nativeDraggable&&(oe.draggable=!0),i._triggerDragStart(e,t),ae({sortable:i,name:"choose",originalEvent:e}),Y(oe,o.chosenClass,!0))},o.ignore.split(",").forEach((function(e){A(oe,e.trim(),tt)})),k(s,"dragover",Ke),k(s,"mousemove",Ke),k(s,"touchmove",Ke),k(s,"mouseup",i._onDrop),k(s,"touchend",i._onDrop),k(s,"touchcancel",i._onDrop),v&&this.nativeDraggable&&(this.options.touchStartThreshold=4,oe.draggable=!0),ie("delayStart",this,{evt:e}),!o.delay||o.delayOnTouchOnly&&!t||this.nativeDraggable&&(_||m))r();else{if(Qe.eventCanceled)return void this._onDrop();k(s,"mouseup",i._disableDelayedDrag),k(s,"touchend",i._disableDelayedDrag),k(s,"touchcancel",i._disableDelayedDrag),k(s,"mousemove",i._delayedDragTouchMoveHandler),k(s,"touchmove",i._delayedDragTouchMoveHandler),o.supportPointer&&k(s,"pointermove",i._delayedDragTouchMoveHandler),i._dragStartTimer=setTimeout(r,o.delay)}}},_delayedDragTouchMoveHandler:function(e){var t=e.touches?e.touches[0]:e;Math.max(Math.abs(t.clientX-this._lastX),Math.abs(t.clientY-this._lastY))>=Math.floor(this.options.touchStartThreshold/(this.nativeDraggable&&window.devicePixelRatio||1))&&this._disableDelayedDrag()},_disableDelayedDrag:function(){oe&&tt(oe),clearTimeout(this._dragStartTimer),this._disableDelayedDragEvents()},_disableDelayedDragEvents:function(){var e=this.el.ownerDocument;w(e,"mouseup",this._disableDelayedDrag),w(e,"touchend",this._disableDelayedDrag),w(e,"touchcancel",this._disableDelayedDrag),w(e,"mousemove",this._delayedDragTouchMoveHandler),w(e,"touchmove",this._delayedDragTouchMoveHandler),w(e,"pointermove",this._delayedDragTouchMoveHandler)},_triggerDragStart:function(e,t){t=t||"touch"==e.pointerType&&e,!this.nativeDraggable||t?this.options.supportPointer?k(document,"pointermove",this._onTouchMove):k(document,t?"touchmove":"mousemove",this._onTouchMove):(k(oe,"dragend",this),k(ue,"dragstart",this._onDragStart));try{document.selection?lt((function(){document.selection.empty()})):window.getSelection().removeAllRanges()}catch(n){}},_dragStarted:function(e,t){if(Ee=!1,ue&&oe){ie("dragStarted",this,{evt:t}),this.nativeDraggable&&k(document,"dragover",Xe);var n=this.options;!e&&Y(oe,n.dragClass,!1),Y(oe,n.ghostClass,!0),Qe.active=this,e&&this._appendGhost(),ae({sortable:this,name:"start",originalEvent:t})}else this._nulling()},_emulateDragOver:function(){if(Me){this._lastX=Me.clientX,this._lastY=Me.clientY,Ge();var e=document.elementFromPoint(Me.clientX,Me.clientY),t=e;while(e&&e.shadowRoot){if(e=e.shadowRoot.elementFromPoint(Me.clientX,Me.clientY),e===t)break;t=e}if(oe.parentNode[K]._isOutsideThisEl(e),t)do{if(t[K]){var n=void 0;if(n=t[K]._onDragOver({clientX:Me.clientX,clientY:Me.clientY,target:e,rootEl:t}),n&&!this.options.dragoverBubble)break}e=t}while(t=t.parentNode);Je()}},_onTouchMove:function(e){if(ye){var t=this.options,n=t.fallbackTolerance,r=t.fallbackOffset,i=e.touches?e.touches[0]:e,a=le&&E(le,!0),o=le&&a&&a.a,s=le&&a&&a.d,l=Ne&&Ce&&R(Ce),u=(i.clientX-ye.clientX+r.x)/(o||1)+(l?l[0]-Ie[0]:0)/(o||1),c=(i.clientY-ye.clientY+r.y)/(s||1)+(l?l[1]-Ie[1]:0)/(s||1);if(!Qe.active&&!Ee){if(n&&Math.max(Math.abs(i.clientX-this._lastX),Math.abs(i.clientY-this._lastY))<n)return;this._onDragStart(e,!0)}if(le){a?(a.e+=u-(ke||0),a.f+=c-(we||0)):a={a:1,b:0,c:0,d:1,e:u,f:c};var d="matrix(".concat(a.a,",").concat(a.b,",").concat(a.c,",").concat(a.d,",").concat(a.e,",").concat(a.f,")");C(le,"webkitTransform",d),C(le,"mozTransform",d),C(le,"msTransform",d),C(le,"transform",d),ke=u,we=c,Me=i}e.cancelable&&e.preventDefault()}},_appendGhost:function(){if(!le){var e=this.options.fallbackOnBody?document.body:ue,t=P(oe,!0,Ne,!0,e),n=this.options;if(Ne){Ce=e;while("static"===C(Ce,"position")&&"none"===C(Ce,"transform")&&Ce!==document)Ce=Ce.parentNode;Ce!==document.body&&Ce!==document.documentElement?(Ce===document&&(Ce=O()),t.top+=Ce.scrollTop,t.left+=Ce.scrollLeft):Ce=O(),Ie=R(Ce)}le=oe.cloneNode(!0),Y(le,n.ghostClass,!1),Y(le,n.fallbackClass,!0),Y(le,n.dragClass,!0),C(le,"transition",""),C(le,"transform",""),C(le,"box-sizing","border-box"),C(le,"margin",0),C(le,"top",t.top),C(le,"left",t.left),C(le,"width",t.width),C(le,"height",t.height),C(le,"opacity","0.8"),C(le,"position",Ne?"absolute":"fixed"),C(le,"zIndex","100000"),C(le,"pointerEvents","none"),Qe.ghost=le,e.appendChild(le),C(le,"transform-origin",Le/parseInt(le.style.width)*100+"% "+xe/parseInt(le.style.height)*100+"%")}},_onDragStart:function(e,t){var n=this,r=e.dataTransfer,i=n.options;ie("dragStart",this,{evt:e}),Qe.eventCanceled?this._onDrop():(ie("setupClone",this),Qe.eventCanceled||(fe=q(oe),fe.draggable=!1,fe.style["will-change"]="",this._hideClone(),Y(fe,this.options.chosenClass,!1),Qe.clone=fe),n.cloneId=lt((function(){ie("clone",n),Qe.eventCanceled||(n.options.removeCloneOnHide||ue.insertBefore(fe,oe),n._hideClone(),ae({sortable:n,name:"clone"}))})),!t&&Y(oe,i.dragClass,!0),t?(Ae=!0,n._loopId=setInterval(n._emulateDragOver,50)):(w(document,"mouseup",n._onDrop),w(document,"touchend",n._onDrop),w(document,"touchcancel",n._onDrop),r&&(r.effectAllowed="move",i.setData&&i.setData.call(n,r,oe)),k(document,"drop",n),C(oe,"transform","translateZ(0)")),Ee=!0,n._dragStartId=lt(n._dragStarted.bind(n,t,e)),k(document,"selectstart",n),Se=!0,g&&C(document.body,"user-select","none"))},_onDragOver:function(e){var t,n,r,i,a=this.el,s=e.target,l=this.options,u=l.group,c=Qe.active,d=ge===u,f=l.sort,h=be||c,p=this,m=!1;if(!je){if(void 0!==e.preventDefault&&e.cancelable&&e.preventDefault(),s=S(s,l.draggable,a,!0),A("dragOver"),Qe.eventCanceled)return m;if(oe.contains(e.target)||s.animated&&s.animatingX&&s.animatingY||p._ignoreWhileAnimating===s)return I(!1);if(Ae=!1,c&&!l.disabled&&(d?f||(r=!ue.contains(oe)):be===this||(this.lastPutMode=ge.checkPull(this,c,oe,e))&&u.checkPut(this,c,oe,e))){if(i="vertical"===this._getDirection(e,s),t=P(oe),A("dragOverValid"),Qe.eventCanceled)return m;if(r)return se=ue,O(),this._hideClone(),A("revert"),Qe.eventCanceled||(ce?ue.insertBefore(oe,ce):ue.appendChild(oe)),I(!0);var _=j(a,l.draggable);if(!_||rt(e,i,this)&&!_.animated){if(_===oe)return I(!1);if(_&&a===e.target&&(s=_),s&&(n=P(s)),!1!==et(ue,a,oe,t,s,n,e,!!s))return O(),a.appendChild(oe),se=a,R(),I(!0)}else if(s.parentNode===a){n=P(s);var v,g,b=0,y=oe.parentNode!==a,M=!Ue(oe.animated&&oe.toRect||t,s.animated&&s.toRect||n,i),k=i?"top":"left",w=F(s,"top","top")||F(oe,"top","top"),L=w?w.scrollTop:void 0;if(Te!==s&&(v=n[k],Pe=!1,Fe=!M&&l.invertSwap||y),b=it(e,s,n,i,M?1:l.swapThreshold,null==l.invertedSwapThreshold?l.swapThreshold:l.invertedSwapThreshold,Fe,Te===s),0!==b){var x=H(oe);do{x-=b,g=se.children[x]}while(g&&("none"===C(g,"display")||g===le))}if(0===b||g===s)return I(!1);Te=s,De=b;var T=s.nextElementSibling,D=!1;D=1===b;var E=et(ue,a,oe,t,s,n,e,D);if(!1!==E)return 1!==E&&-1!==E||(D=1===E),je=!0,setTimeout(nt,30),O(),D&&!T?a.appendChild(oe):s.parentNode.insertBefore(oe,D?T:s),w&&V(w,0,L-w.scrollTop),se=oe.parentNode,void 0===v||Fe||(Ye=Math.abs(v-P(s)[k])),R(),I(!0)}if(a.contains(oe))return I(!1)}return!1}function A(l,u){ie(l,p,o({evt:e,isOwner:d,axis:i?"vertical":"horizontal",revert:r,dragRect:t,targetRect:n,canSort:f,fromSortable:h,target:s,completed:I,onMove:function(n,r){return et(ue,a,oe,t,n,P(n),e,r)},changed:R},u))}function O(){A("dragOverAnimationCapture"),p.captureAnimationState(),p!==h&&h.captureAnimationState()}function I(t){return A("dragOverCompleted",{insertion:t}),t&&(d?c._hideClone():c._showClone(p),p!==h&&(Y(oe,be?be.options.ghostClass:c.options.ghostClass,!1),Y(oe,l.ghostClass,!0)),be!==p&&p!==Qe.active?be=p:p===Qe.active&&be&&(be=null),h===p&&(p._ignoreWhileAnimating=s),p.animateAll((function(){A("dragOverAnimationComplete"),p._ignoreWhileAnimating=null})),p!==h&&(h.animateAll(),h._ignoreWhileAnimating=null)),(s===oe&&!oe.animated||s===a&&!s.animated)&&(Te=null),l.dragoverBubble||e.rootEl||s===document||(oe.parentNode[K]._isOutsideThisEl(e.target),!t&&Ke(e)),!l.dragoverBubble&&e.stopPropagation&&e.stopPropagation(),m=!0}function R(){me=H(oe),ve=H(oe,l.draggable),ae({sortable:p,name:"change",toEl:a,newIndex:me,newDraggableIndex:ve,originalEvent:e})}},_ignoreWhileAnimating:null,_offMoveEvents:function(){w(document,"mousemove",this._onTouchMove),w(document,"touchmove",this._onTouchMove),w(document,"pointermove",this._onTouchMove),w(document,"dragover",Ke),w(document,"mousemove",Ke),w(document,"touchmove",Ke)},_offUpEvents:function(){var e=this.el.ownerDocument;w(e,"mouseup",this._onDrop),w(e,"touchend",this._onDrop),w(e,"pointerup",this._onDrop),w(e,"touchcancel",this._onDrop),w(document,"selectstart",this)},_onDrop:function(e){var t=this.el,n=this.options;me=H(oe),ve=H(oe,n.draggable),ie("drop",this,{evt:e}),se=oe&&oe.parentNode,me=H(oe),ve=H(oe,n.draggable),Qe.eventCanceled||(Ee=!1,Fe=!1,Pe=!1,clearInterval(this._loopId),clearTimeout(this._dragStartTimer),ut(this.cloneId),ut(this._dragStartId),this.nativeDraggable&&(w(document,"drop",this),w(t,"dragstart",this._onDragStart)),this._offMoveEvents(),this._offUpEvents(),g&&C(document.body,"user-select",""),C(oe,"transform",""),e&&(Se&&(e.cancelable&&e.preventDefault(),!n.dropBubble&&e.stopPropagation()),le&&le.parentNode&&le.parentNode.removeChild(le),(ue===se||be&&"clone"!==be.lastPutMode)&&fe&&fe.parentNode&&fe.parentNode.removeChild(fe),oe&&(this.nativeDraggable&&w(oe,"dragend",this),tt(oe),oe.style["will-change"]="",Se&&!Ee&&Y(oe,be?be.options.ghostClass:this.options.ghostClass,!1),Y(oe,this.options.chosenClass,!1),ae({sortable:this,name:"unchoose",toEl:se,newIndex:null,newDraggableIndex:null,originalEvent:e}),ue!==se?(me>=0&&(ae({rootEl:se,name:"add",toEl:se,fromEl:ue,originalEvent:e}),ae({sortable:this,name:"remove",toEl:se,originalEvent:e}),ae({rootEl:se,name:"sort",toEl:se,fromEl:ue,originalEvent:e}),ae({sortable:this,name:"sort",toEl:se,originalEvent:e})),be&&be.save()):me!==pe&&me>=0&&(ae({sortable:this,name:"update",toEl:se,originalEvent:e}),ae({sortable:this,name:"sort",toEl:se,originalEvent:e})),Qe.active&&(null!=me&&-1!==me||(me=pe,ve=_e),ae({sortable:this,name:"end",toEl:se,originalEvent:e}),this.save())))),this._nulling()},_nulling:function(){ie("nulling",this),ue=oe=se=le=ce=fe=de=he=ye=Me=Se=me=ve=pe=_e=Te=De=be=ge=Qe.dragged=Qe.ghost=Qe.clone=Qe.active=null,He.forEach((function(e){e.checked=!0})),He.length=ke=we=0},handleEvent:function(e){switch(e.type){case"drop":case"dragend":this._onDrop(e);break;case"dragenter":case"dragover":oe&&(this._onDragOver(e),Ze(e));break;case"selectstart":e.preventDefault();break}},toArray:function(){for(var e,t=[],n=this.el.children,r=0,i=n.length,a=this.options;r<i;r++)e=n[r],S(e,a.draggable,this.el,!1)&&t.push(e.getAttribute(a.dataIdAttr)||ot(e));return t},sort:function(e){var t={},n=this.el;this.toArray().forEach((function(e,r){var i=n.children[r];S(i,this.options.draggable,n,!1)&&(t[e]=i)}),this),e.forEach((function(e){t[e]&&(n.removeChild(t[e]),n.appendChild(t[e]))}))},save:function(){var e=this.options.store;e&&e.set&&e.set(this)},closest:function(e,t){return S(e,t||this.options.draggable,this.el,!1)},option:function(e,t){var n=this.options;if(void 0===t)return n[e];var r=ne.modifyOption(this,e,t);n[e]="undefined"!==typeof r?r:t,"group"===e&&qe(n)},destroy:function(){ie("destroy",this);var e=this.el;e[K]=null,w(e,"mousedown",this._onTapStart),w(e,"touchstart",this._onTapStart),w(e,"pointerdown",this._onTapStart),this.nativeDraggable&&(w(e,"dragover",this),w(e,"dragenter",this)),Array.prototype.forEach.call(e.querySelectorAll("[draggable]"),(function(e){e.removeAttribute("draggable")})),this._onDrop(),this._disableDelayedDragEvents(),Oe.splice(Oe.indexOf(this.el),1),this.el=e=null},_hideClone:function(){if(!he){if(ie("hideClone",this),Qe.eventCanceled)return;C(fe,"display","none"),this.options.removeCloneOnHide&&fe.parentNode&&fe.parentNode.removeChild(fe),he=!0}},_showClone:function(e){if("clone"===e.lastPutMode){if(he){if(ie("showClone",this),Qe.eventCanceled)return;ue.contains(oe)&&!this.options.group.revertClone?ue.insertBefore(fe,oe):ce?ue.insertBefore(fe,ce):ue.appendChild(fe),this.options.group.revertClone&&this.animate(oe,fe),C(fe,"display",""),he=!1}}else this._hideClone()}},Re&&k(document,"touchmove",(function(e){(Qe.active||Ee)&&e.cancelable&&e.preventDefault()})),Qe.utils={on:k,off:w,css:C,find:A,is:function(e,t){return!!S(e,t,e,!1)},extend:z,throttle:W,closest:S,toggleClass:Y,clone:q,index:H,nextTick:lt,cancelNextTick:ut,detectDirection:We,getChild:I},Qe.get=function(e){return e[K]},Qe.mount=function(){for(var e=arguments.length,t=new Array(e),n=0;n<e;n++)t[n]=arguments[n];t[0].constructor===Array&&(t=t[0]),t.forEach((function(e){if(!e.prototype||!e.prototype.constructor)throw"Sortable: Mounted plugin must be a constructor function, not ".concat({}.toString.call(e));e.utils&&(Qe.utils=o({},Qe.utils,e.utils)),ne.mount(e)}))},Qe.create=function(e,t){return new Qe(e,t)},Qe.version=h;var ct,dt,ft,ht,pt,mt,_t=[],vt=!1;function gt(){function e(){for(var e in this.defaults={scroll:!0,scrollSensitivity:30,scrollSpeed:10,bubbleScroll:!0},this)"_"===e.charAt(0)&&"function"===typeof this[e]&&(this[e]=this[e].bind(this))}return e.prototype={dragStarted:function(e){var t=e.originalEvent;this.sortable.nativeDraggable?k(document,"dragover",this._handleAutoScroll):this.options.supportPointer?k(document,"pointermove",this._handleFallbackAutoScroll):t.touches?k(document,"touchmove",this._handleFallbackAutoScroll):k(document,"mousemove",this._handleFallbackAutoScroll)},dragOverCompleted:function(e){var t=e.originalEvent;this.options.dragOverBubble||t.rootEl||this._handleAutoScroll(t)},drop:function(){this.sortable.nativeDraggable?w(document,"dragover",this._handleAutoScroll):(w(document,"pointermove",this._handleFallbackAutoScroll),w(document,"touchmove",this._handleFallbackAutoScroll),w(document,"mousemove",this._handleFallbackAutoScroll)),yt(),bt(),U()},nulling:function(){pt=dt=ct=vt=mt=ft=ht=null,_t.length=0},_handleFallbackAutoScroll:function(e){this._handleAutoScroll(e,!0)},_handleAutoScroll:function(e,t){var n=this,r=(e.touches?e.touches[0]:e).clientX,i=(e.touches?e.touches[0]:e).clientY,a=document.elementFromPoint(r,i);if(pt=e,t||_||m||g){kt(e,this.options,a,t);var o=$(a,!0);!vt||mt&&r===ft&&i===ht||(mt&&yt(),mt=setInterval((function(){var a=$(document.elementFromPoint(r,i),!0);a!==o&&(o=a,bt()),kt(e,n.options,a,t)}),10),ft=r,ht=i)}else{if(!this.options.bubbleScroll||$(a,!0)===O())return void bt();kt(e,this.options,$(a,!1),!1)}}},a(e,{pluginName:"scroll",initializeByDefault:!0})}function bt(){_t.forEach((function(e){clearInterval(e.pid)})),_t=[]}function yt(){clearInterval(mt)}var Mt,kt=W((function(e,t,n,r){if(t.scroll){var i,a=(e.touches?e.touches[0]:e).clientX,o=(e.touches?e.touches[0]:e).clientY,s=t.scrollSensitivity,l=t.scrollSpeed,u=O(),c=!1;dt!==n&&(dt=n,bt(),ct=t.scroll,i=t.scrollFn,!0===ct&&(ct=$(n,!0)));var d=0,f=ct;do{var h=f,p=P(h),m=p.top,_=p.bottom,v=p.left,g=p.right,b=p.width,y=p.height,M=void 0,k=void 0,w=h.scrollWidth,L=h.scrollHeight,x=C(h),S=h.scrollLeft,T=h.scrollTop;h===u?(M=b<w&&("auto"===x.overflowX||"scroll"===x.overflowX||"visible"===x.overflowX),k=y<L&&("auto"===x.overflowY||"scroll"===x.overflowY||"visible"===x.overflowY)):(M=b<w&&("auto"===x.overflowX||"scroll"===x.overflowX),k=y<L&&("auto"===x.overflowY||"scroll"===x.overflowY));var D=M&&(Math.abs(g-a)<=s&&S+b<w)-(Math.abs(v-a)<=s&&!!S),Y=k&&(Math.abs(_-o)<=s&&T+y<L)-(Math.abs(m-o)<=s&&!!T);if(!_t[d])for(var E=0;E<=d;E++)_t[E]||(_t[E]={});_t[d].vx==D&&_t[d].vy==Y&&_t[d].el===h||(_t[d].el=h,_t[d].vx=D,_t[d].vy=Y,clearInterval(_t[d].pid),0==D&&0==Y||(c=!0,_t[d].pid=setInterval(function(){r&&0===this.layer&&Qe.active._onTouchMove(pt);var t=_t[this.layer].vy?_t[this.layer].vy*l:0,n=_t[this.layer].vx?_t[this.layer].vx*l:0;"function"===typeof i&&"continue"!==i.call(Qe.dragged.parentNode[K],n,t,e,pt,_t[this.layer].el)||V(_t[this.layer].el,n,t)}.bind({layer:d}),24))),d++}while(t.bubbleScroll&&f!==u&&(f=$(f,!1)));vt=c}}),30),wt=function(e){var t=e.originalEvent,n=e.putSortable,r=e.dragEl,i=e.activeSortable,a=e.dispatchSortableEvent,o=e.hideGhostForTarget,s=e.unhideGhostForTarget;if(t){var l=n||i;o();var u=t.changedTouches&&t.changedTouches.length?t.changedTouches[0]:t,c=document.elementFromPoint(u.clientX,u.clientY);s(),l&&!l.el.contains(c)&&(a("spill"),this.onSpill({dragEl:r,putSortable:n}))}};function Lt(){}function xt(){}function St(){function e(){this.defaults={swapClass:"sortable-swap-highlight"}}return e.prototype={dragStart:function(e){var t=e.dragEl;Mt=t},dragOverValid:function(e){var t=e.completed,n=e.target,r=e.onMove,i=e.activeSortable,a=e.changed,o=e.cancel;if(i.options.swap){var s=this.sortable.el,l=this.options;if(n&&n!==s){var u=Mt;!1!==r(n)?(Y(n,l.swapClass,!0),Mt=n):Mt=null,u&&u!==Mt&&Y(u,l.swapClass,!1)}a(),t(!0),o()}},drop:function(e){var t=e.activeSortable,n=e.putSortable,r=e.dragEl,i=n||this.sortable,a=this.options;Mt&&Y(Mt,a.swapClass,!1),Mt&&(a.swap||n&&n.options.swap)&&r!==Mt&&(i.captureAnimationState(),i!==t&&t.captureAnimationState(),Tt(r,Mt),i.animateAll(),i!==t&&t.animateAll())},nulling:function(){Mt=null}},a(e,{pluginName:"swap",eventProperties:function(){return{swapItem:Mt}}})}function Tt(e,t){var n,r,i=e.parentNode,a=t.parentNode;i&&a&&!i.isEqualNode(t)&&!a.isEqualNode(e)&&(n=H(e),r=H(t),i.isEqualNode(a)&&n<r&&r++,i.insertBefore(t,i.children[n]),a.insertBefore(e,a.children[r]))}Lt.prototype={startIndex:null,dragStart:function(e){var t=e.oldDraggableIndex;this.startIndex=t},onSpill:function(e){var t=e.dragEl,n=e.putSortable;this.sortable.captureAnimationState(),n&&n.captureAnimationState();var r=I(this.sortable.el,this.startIndex,this.options);r?this.sortable.el.insertBefore(t,r):this.sortable.el.appendChild(t),this.sortable.animateAll(),n&&n.animateAll()},drop:wt},a(Lt,{pluginName:"revertOnSpill"}),xt.prototype={onSpill:function(e){var t=e.dragEl,n=e.putSortable,r=n||this.sortable;r.captureAnimationState(),t.parentNode&&t.parentNode.removeChild(t),r.animateAll()},drop:wt},a(xt,{pluginName:"removeOnSpill"});var Dt,Yt,Ct,Et,At,Ot=[],Pt=[],Ft=!1,It=!1,jt=!1;function Ht(){function e(e){for(var t in this)"_"===t.charAt(0)&&"function"===typeof this[t]&&(this[t]=this[t].bind(this));e.options.supportPointer?k(document,"pointerup",this._deselectMultiDrag):(k(document,"mouseup",this._deselectMultiDrag),k(document,"touchend",this._deselectMultiDrag)),k(document,"keydown",this._checkKeyDown),k(document,"keyup",this._checkKeyUp),this.defaults={selectedClass:"sortable-selected",multiDragKey:null,setData:function(t,n){var r="";Ot.length&&Yt===e?Ot.forEach((function(e,t){r+=(t?", ":"")+e.textContent})):r=n.textContent,t.setData("Text",r)}}}return e.prototype={multiDragKeyDown:!1,isMultiDrag:!1,delayStartGlobal:function(e){var t=e.dragEl;Ct=t},delayEnded:function(){this.isMultiDrag=~Ot.indexOf(Ct)},setupClone:function(e){var t=e.sortable,n=e.cancel;if(this.isMultiDrag){for(var r=0;r<Ot.length;r++)Pt.push(q(Ot[r])),Pt[r].sortableIndex=Ot[r].sortableIndex,Pt[r].draggable=!1,Pt[r].style["will-change"]="",Y(Pt[r],this.options.selectedClass,!1),Ot[r]===Ct&&Y(Pt[r],this.options.chosenClass,!1);t._hideClone(),n()}},clone:function(e){var t=e.sortable,n=e.rootEl,r=e.dispatchSortableEvent,i=e.cancel;this.isMultiDrag&&(this.options.removeCloneOnHide||Ot.length&&Yt===t&&(Nt(!0,n),r("clone"),i()))},showClone:function(e){var t=e.cloneNowShown,n=e.rootEl,r=e.cancel;this.isMultiDrag&&(Nt(!1,n),Pt.forEach((function(e){C(e,"display","")})),t(),At=!1,r())},hideClone:function(e){var t=this,n=(e.sortable,e.cloneNowHidden),r=e.cancel;this.isMultiDrag&&(Pt.forEach((function(e){C(e,"display","none"),t.options.removeCloneOnHide&&e.parentNode&&e.parentNode.removeChild(e)})),n(),At=!0,r())},dragStartGlobal:function(e){e.sortable;!this.isMultiDrag&&Yt&&Yt.multiDrag._deselectMultiDrag(),Ot.forEach((function(e){e.sortableIndex=H(e)})),Ot=Ot.sort((function(e,t){return e.sortableIndex-t.sortableIndex})),jt=!0},dragStarted:function(e){var t=this,n=e.sortable;if(this.isMultiDrag){if(this.options.sort&&(n.captureAnimationState(),this.options.animation)){Ot.forEach((function(e){e!==Ct&&C(e,"position","absolute")}));var r=P(Ct,!1,!0,!0);Ot.forEach((function(e){e!==Ct&&G(e,r)})),It=!0,Ft=!0}n.animateAll((function(){It=!1,Ft=!1,t.options.animation&&Ot.forEach((function(e){J(e)})),t.options.sort&&$t()}))}},dragOver:function(e){var t=e.target,n=e.completed,r=e.cancel;It&&~Ot.indexOf(t)&&(n(!1),r())},revert:function(e){var t=e.fromSortable,n=e.rootEl,r=e.sortable,i=e.dragRect;Ot.length>1&&(Ot.forEach((function(e){r.addAnimationState({target:e,rect:It?P(e):i}),J(e),e.fromRect=i,t.removeAnimationState(e)})),It=!1,Rt(!this.options.removeCloneOnHide,n))},dragOverCompleted:function(e){var t=e.sortable,n=e.isOwner,r=e.insertion,i=e.activeSortable,a=e.parentEl,o=e.putSortable,s=this.options;if(r){if(n&&i._hideClone(),Ft=!1,s.animation&&Ot.length>1&&(It||!n&&!i.options.sort&&!o)){var l=P(Ct,!1,!0,!0);Ot.forEach((function(e){e!==Ct&&(G(e,l),a.appendChild(e))})),It=!0}if(!n)if(It||$t(),Ot.length>1){var u=At;i._showClone(t),i.options.animation&&!At&&u&&Pt.forEach((function(e){i.addAnimationState({target:e,rect:Et}),e.fromRect=Et,e.thisAnimationDuration=null}))}else i._showClone(t)}},dragOverAnimationCapture:function(e){var t=e.dragRect,n=e.isOwner,r=e.activeSortable;if(Ot.forEach((function(e){e.thisAnimationDuration=null})),r.options.animation&&!n&&r.multiDrag.isMultiDrag){Et=a({},t);var i=E(Ct,!0);Et.top-=i.f,Et.left-=i.e}},dragOverAnimationComplete:function(){It&&(It=!1,$t())},drop:function(e){var t=e.originalEvent,n=e.rootEl,r=e.parentEl,i=e.sortable,a=e.dispatchSortableEvent,o=e.oldIndex,s=e.putSortable,l=s||this.sortable;if(t){var u=this.options,c=r.children;if(!jt)if(u.multiDragKey&&!this.multiDragKeyDown&&this._deselectMultiDrag(),Y(Ct,u.selectedClass,!~Ot.indexOf(Ct)),~Ot.indexOf(Ct))Ot.splice(Ot.indexOf(Ct),1),Dt=null,re({sortable:i,rootEl:n,name:"deselect",targetEl:Ct,originalEvt:t});else{if(Ot.push(Ct),re({sortable:i,rootEl:n,name:"select",targetEl:Ct,originalEvt:t}),t.shiftKey&&Dt&&i.el.contains(Dt)){var d,f,h=H(Dt),p=H(Ct);if(~h&&~p&&h!==p)for(p>h?(f=h,d=p):(f=p,d=h+1);f<d;f++)~Ot.indexOf(c[f])||(Y(c[f],u.selectedClass,!0),Ot.push(c[f]),re({sortable:i,rootEl:n,name:"select",targetEl:c[f],originalEvt:t}))}else Dt=Ct;Yt=l}if(jt&&this.isMultiDrag){if((r[K].options.sort||r!==n)&&Ot.length>1){var m=P(Ct),_=H(Ct,":not(."+this.options.selectedClass+")");if(!Ft&&u.animation&&(Ct.thisAnimationDuration=null),l.captureAnimationState(),!Ft&&(u.animation&&(Ct.fromRect=m,Ot.forEach((function(e){if(e.thisAnimationDuration=null,e!==Ct){var t=It?P(e):m;e.fromRect=t,l.addAnimationState({target:e,rect:t})}}))),$t(),Ot.forEach((function(e){c[_]?r.insertBefore(e,c[_]):r.appendChild(e),_++})),o===H(Ct))){var v=!1;Ot.forEach((function(e){e.sortableIndex===H(e)||(v=!0)})),v&&a("update")}Ot.forEach((function(e){J(e)})),l.animateAll()}Yt=l}(n===r||s&&"clone"!==s.lastPutMode)&&Pt.forEach((function(e){e.parentNode&&e.parentNode.removeChild(e)}))}},nullingGlobal:function(){this.isMultiDrag=jt=!1,Pt.length=0},destroyGlobal:function(){this._deselectMultiDrag(),w(document,"pointerup",this._deselectMultiDrag),w(document,"mouseup",this._deselectMultiDrag),w(document,"touchend",this._deselectMultiDrag),w(document,"keydown",this._checkKeyDown),w(document,"keyup",this._checkKeyUp)},_deselectMultiDrag:function(e){if(("undefined"===typeof jt||!jt)&&Yt===this.sortable&&(!e||!S(e.target,this.options.draggable,this.sortable.el,!1))&&(!e||0===e.button))while(Ot.length){var t=Ot[0];Y(t,this.options.selectedClass,!1),Ot.shift(),re({sortable:this.sortable,rootEl:this.sortable.el,name:"deselect",targetEl:t,originalEvt:e})}},_checkKeyDown:function(e){e.key===this.options.multiDragKey&&(this.multiDragKeyDown=!0)},_checkKeyUp:function(e){e.key===this.options.multiDragKey&&(this.multiDragKeyDown=!1)}},a(e,{pluginName:"multiDrag",utils:{select:function(e){var t=e.parentNode[K];t&&t.options.multiDrag&&!~Ot.indexOf(e)&&(Yt&&Yt!==t&&(Yt.multiDrag._deselectMultiDrag(),Yt=t),Y(e,t.options.selectedClass,!0),Ot.push(e))},deselect:function(e){var t=e.parentNode[K],n=Ot.indexOf(e);t&&t.options.multiDrag&&~n&&(Y(e,t.options.selectedClass,!1),Ot.splice(n,1))}},eventProperties:function(){var e=this,t=[],n=[];return Ot.forEach((function(r){var i;t.push({multiDragElement:r,index:r.sortableIndex}),i=It&&r!==Ct?-1:It?H(r,":not(."+e.options.selectedClass+")"):H(r),n.push({multiDragElement:r,index:i})})),{items:u(Ot),clones:[].concat(Pt),oldIndicies:t,newIndicies:n}},optionListeners:{multiDragKey:function(e){return e=e.toLowerCase(),"ctrl"===e?e="Control":e.length>1&&(e=e.charAt(0).toUpperCase()+e.substr(1)),e}}})}function Rt(e,t){Ot.forEach((function(n,r){var i=t.children[n.sortableIndex+(e?Number(r):0)];i?t.insertBefore(n,i):t.appendChild(n)}))}function Nt(e,t){Pt.forEach((function(n,r){var i=t.children[n.sortableIndex+(e?Number(r):0)];i?t.insertBefore(n,i):t.appendChild(n)}))}function $t(){Ot.forEach((function(e){e!==Ct&&e.parentNode&&e.parentNode.removeChild(e)}))}Qe.mount(new gt),Qe.mount(xt,Lt),t["default"]=Qe},aaab:function(e,t,n){"use strict";n("d8a8")},aaf2:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e,t,n,r){var i={s:["थोडया सॅकंडांनी","थोडे सॅकंड"],ss:[e+" सॅकंडांनी",e+" सॅकंड"],m:["एका मिणटान","एक मिनूट"],mm:[e+" मिणटांनी",e+" मिणटां"],h:["एका वरान","एक वर"],hh:[e+" वरांनी",e+" वरां"],d:["एका दिसान","एक दीस"],dd:[e+" दिसांनी",e+" दीस"],M:["एका म्हयन्यान","एक म्हयनो"],MM:[e+" म्हयन्यानी",e+" म्हयने"],y:["एका वर्सान","एक वर्स"],yy:[e+" वर्सांनी",e+" वर्सां"]};return r?i[n][0]:i[n][1]}var n=e.defineLocale("gom-deva",{months:{standalone:"जानेवारी_फेब्रुवारी_मार्च_एप्रील_मे_जून_जुलय_ऑगस्ट_सप्टेंबर_ऑक्टोबर_नोव्हेंबर_डिसेंबर".split("_"),format:"जानेवारीच्या_फेब्रुवारीच्या_मार्चाच्या_एप्रीलाच्या_मेयाच्या_जूनाच्या_जुलयाच्या_ऑगस्टाच्या_सप्टेंबराच्या_ऑक्टोबराच्या_नोव्हेंबराच्या_डिसेंबराच्या".split("_"),isFormat:/MMMM(\s)+D[oD]?/},monthsShort:"जाने._फेब्रु._मार्च_एप्री._मे_जून_जुल._ऑग._सप्टें._ऑक्टो._नोव्हें._डिसें.".split("_"),monthsParseExact:!0,weekdays:"आयतार_सोमार_मंगळार_बुधवार_बिरेस्तार_सुक्रार_शेनवार".split("_"),weekdaysShort:"आयत._सोम._मंगळ._बुध._ब्रेस्त._सुक्र._शेन.".split("_"),weekdaysMin:"आ_सो_मं_बु_ब्रे_सु_शे".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"A h:mm [वाजतां]",LTS:"A h:mm:ss [वाजतां]",L:"DD-MM-YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY A h:mm [वाजतां]",LLLL:"dddd, MMMM Do, YYYY, A h:mm [वाजतां]",llll:"ddd, D MMM YYYY, A h:mm [वाजतां]"},calendar:{sameDay:"[आयज] LT",nextDay:"[फाल्यां] LT",nextWeek:"[फुडलो] dddd[,] LT",lastDay:"[काल] LT",lastWeek:"[फाटलो] dddd[,] LT",sameElse:"L"},relativeTime:{future:"%s",past:"%s आदीं",s:t,ss:t,m:t,mm:t,h:t,hh:t,d:t,dd:t,M:t,MM:t,y:t,yy:t},dayOfMonthOrdinalParse:/\d{1,2}(वेर)/,ordinal:function(e,t){switch(t){case"D":return e+"वेर";default:case"M":case"Q":case"DDD":case"d":case"w":case"W":return e}},week:{dow:0,doy:3},meridiemParse:/राती|सकाळीं|दनपारां|सांजे/,meridiemHour:function(e,t){return 12===e&&(e=0),"राती"===t?e<4?e:e+12:"सकाळीं"===t?e:"दनपारां"===t?e>12?e:e+12:"सांजे"===t?e+12:void 0},meridiem:function(e,t,n){return e<4?"राती":e<12?"सकाळीं":e<16?"दनपारां":e<20?"सांजे":"राती"}});return n}))},ab13:function(e,t,n){var r=n("b622"),i=r("match");e.exports=function(e){var t=/./;try{"/./"[e](t)}catch(n){try{return t[i]=!1,"/./"[e](t)}catch(r){}}return!1}},ac1f:function(e,t,n){"use strict";var r=n("23e7"),i=n("9263");r({target:"RegExp",proto:!0,forced:/./.exec!==i},{exec:i})},acb6:function(e,t,n){"use strict";n("ee83")},ace4:function(e,t,n){"use strict";var r=n("23e7"),i=n("d039"),a=n("621a"),o=n("825a"),s=n("23cb"),l=n("50c4"),u=n("4840"),c=a.ArrayBuffer,d=a.DataView,f=c.prototype.slice,h=i((function(){return!new c(2).slice(1,void 0).byteLength}));r({target:"ArrayBuffer",proto:!0,unsafe:!0,forced:h},{slice:function(e,t){if(void 0!==f&&void 0===t)return f.call(o(this),e);var n=o(this).byteLength,r=s(e,n),i=s(void 0===t?n:t,n),a=new(u(this,c))(l(i-r)),h=new d(this),p=new d(a),m=0;while(r<i)p.setUint8(m++,h.getUint8(r++));return a}})},ad6d:function(e,t,n){"use strict";var r=n("825a");e.exports=function(){var e=r(this),t="";return e.global&&(t+="g"),e.ignoreCase&&(t+="i"),e.multiline&&(t+="m"),e.dotAll&&(t+="s"),e.unicode&&(t+="u"),e.sticky&&(t+="y"),t}},ada2:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e,t){var n=e.split("_");return t%10===1&&t%100!==11?n[0]:t%10>=2&&t%10<=4&&(t%100<10||t%100>=20)?n[1]:n[2]}function n(e,n,r){var i={ss:n?"секунда_секунди_секунд":"секунду_секунди_секунд",mm:n?"хвилина_хвилини_хвилин":"хвилину_хвилини_хвилин",hh:n?"година_години_годин":"годину_години_годин",dd:"день_дні_днів",MM:"місяць_місяці_місяців",yy:"рік_роки_років"};return"m"===r?n?"хвилина":"хвилину":"h"===r?n?"година":"годину":e+" "+t(i[r],+e)}function r(e,t){var n,r={nominative:"неділя_понеділок_вівторок_середа_четвер_п’ятниця_субота".split("_"),accusative:"неділю_понеділок_вівторок_середу_четвер_п’ятницю_суботу".split("_"),genitive:"неділі_понеділка_вівторка_середи_четверга_п’ятниці_суботи".split("_")};return!0===e?r["nominative"].slice(1,7).concat(r["nominative"].slice(0,1)):e?(n=/(\[[ВвУу]\]) ?dddd/.test(t)?"accusative":/\[?(?:минулої|наступної)? ?\] ?dddd/.test(t)?"genitive":"nominative",r[n][e.day()]):r["nominative"]}function i(e){return function(){return e+"о"+(11===this.hours()?"б":"")+"] LT"}}var a=e.defineLocale("uk",{months:{format:"січня_лютого_березня_квітня_травня_червня_липня_серпня_вересня_жовтня_листопада_грудня".split("_"),standalone:"січень_лютий_березень_квітень_травень_червень_липень_серпень_вересень_жовтень_листопад_грудень".split("_")},monthsShort:"січ_лют_бер_квіт_трав_черв_лип_серп_вер_жовт_лист_груд".split("_"),weekdays:r,weekdaysShort:"нд_пн_вт_ср_чт_пт_сб".split("_"),weekdaysMin:"нд_пн_вт_ср_чт_пт_сб".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY р.",LLL:"D MMMM YYYY р., HH:mm",LLLL:"dddd, D MMMM YYYY р., HH:mm"},calendar:{sameDay:i("[Сьогодні "),nextDay:i("[Завтра "),lastDay:i("[Вчора "),nextWeek:i("[У] dddd ["),lastWeek:function(){switch(this.day()){case 0:case 3:case 5:case 6:return i("[Минулої] dddd [").call(this);case 1:case 2:case 4:return i("[Минулого] dddd [").call(this)}},sameElse:"L"},relativeTime:{future:"за %s",past:"%s тому",s:"декілька секунд",ss:n,m:n,mm:n,h:"годину",hh:n,d:"день",dd:n,M:"місяць",MM:n,y:"рік",yy:n},meridiemParse:/ночі|ранку|дня|вечора/,isPM:function(e){return/^(дня|вечора)$/.test(e)},meridiem:function(e,t,n){return e<4?"ночі":e<12?"ранку":e<17?"дня":"вечора"},dayOfMonthOrdinalParse:/\d{1,2}-(й|го)/,ordinal:function(e,t){switch(t){case"M":case"d":case"DDD":case"w":case"W":return e+"-й";case"D":return e+"-го";default:return e}},week:{dow:1,doy:7}});return a}))},ae1f:function(e,t,n){"use strict";n("4885")},ae40:function(e,t,n){var r=n("83ab"),i=n("d039"),a=n("5135"),o=Object.defineProperty,s={},l=function(e){throw e};e.exports=function(e,t){if(a(s,e))return s[e];t||(t={});var n=[][e],u=!!a(t,"ACCESSORS")&&t.ACCESSORS,c=a(t,0)?t[0]:l,d=a(t,1)?t[1]:void 0;return s[e]=!!n&&!i((function(){if(u&&!r)return!0;var e={length:-1};u?o(e,1,{enumerable:!0,get:l}):e[1]=1,n.call(e,c,d)}))}},ae93:function(e,t,n){"use strict";var r,i,a,o=n("e163"),s=n("9112"),l=n("5135"),u=n("b622"),c=n("c430"),d=u("iterator"),f=!1,h=function(){return this};[].keys&&(a=[].keys(),"next"in a?(i=o(o(a)),i!==Object.prototype&&(r=i)):f=!0),void 0==r&&(r={}),c||l(r,d)||s(r,d,h),e.exports={IteratorPrototype:r,BUGGY_SAFARI_ITERATORS:f}},b012:function(e,t){function n(e,t,n){var r,i,a,o,s;function l(){var u=Date.now()-o;u<t&&u>=0?r=setTimeout(l,t-u):(r=null,n||(s=e.apply(a,i),a=i=null))}null==t&&(t=100);var u=function(){a=this,i=arguments,o=Date.now();var u=n&&!r;return r||(r=setTimeout(l,t)),u&&(s=e.apply(a,i),a=i=null),s};return u.clear=function(){r&&(clearTimeout(r),r=null)},u.flush=function(){r&&(s=e.apply(a,i),a=i=null,clearTimeout(r),r=null)},u}n.debounce=n,e.exports=n},b041:function(e,t,n){"use strict";var r=n("00ee"),i=n("f5df");e.exports=r?{}.toString:function(){return"[object "+i(this)+"]"}},b073:function(e,t,n){},b0c0:function(e,t,n){var r=n("83ab"),i=n("9bf2").f,a=Function.prototype,o=a.toString,s=/^\s*function ([^ (]*)/,l="name";r&&!(l in a)&&i(a,l,{configurable:!0,get:function(){try{return o.call(this).match(s)[1]}catch(e){return""}}})},b29d:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("lo",{months:"ມັງກອນ_ກຸມພາ_ມີນາ_ເມສາ_ພຶດສະພາ_ມິຖຸນາ_ກໍລະກົດ_ສິງຫາ_ກັນຍາ_ຕຸລາ_ພະຈິກ_ທັນວາ".split("_"),monthsShort:"ມັງກອນ_ກຸມພາ_ມີນາ_ເມສາ_ພຶດສະພາ_ມິຖຸນາ_ກໍລະກົດ_ສິງຫາ_ກັນຍາ_ຕຸລາ_ພະຈິກ_ທັນວາ".split("_"),weekdays:"ອາທິດ_ຈັນ_ອັງຄານ_ພຸດ_ພະຫັດ_ສຸກ_ເສົາ".split("_"),weekdaysShort:"ທິດ_ຈັນ_ອັງຄານ_ພຸດ_ພະຫັດ_ສຸກ_ເສົາ".split("_"),weekdaysMin:"ທ_ຈ_ອຄ_ພ_ພຫ_ສກ_ສ".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"ວັນdddd D MMMM YYYY HH:mm"},meridiemParse:/ຕອນເຊົ້າ|ຕອນແລງ/,isPM:function(e){return"ຕອນແລງ"===e},meridiem:function(e,t,n){return e<12?"ຕອນເຊົ້າ":"ຕອນແລງ"},calendar:{sameDay:"[ມື້ນີ້ເວລາ] LT",nextDay:"[ມື້ອື່ນເວລາ] LT",nextWeek:"[ວັນ]dddd[ໜ້າເວລາ] LT",lastDay:"[ມື້ວານນີ້ເວລາ] LT",lastWeek:"[ວັນ]dddd[ແລ້ວນີ້ເວລາ] LT",sameElse:"L"},relativeTime:{future:"ອີກ %s",past:"%sຜ່ານມາ",s:"ບໍ່ເທົ່າໃດວິນາທີ",ss:"%d ວິນາທີ",m:"1 ນາທີ",mm:"%d ນາທີ",h:"1 ຊົ່ວໂມງ",hh:"%d ຊົ່ວໂມງ",d:"1 ມື້",dd:"%d ມື້",M:"1 ເດືອນ",MM:"%d ເດືອນ",y:"1 ປີ",yy:"%d ປີ"},dayOfMonthOrdinalParse:/(ທີ່)\d{1,2}/,ordinal:function(e){return"ທີ່"+e}});return t}))},b2c4:function(e,t,n){},b39a:function(e,t,n){"use strict";var r=n("da84"),i=n("ebb5"),a=n("d039"),o=r.Int8Array,s=i.aTypedArray,l=i.exportTypedArrayMethod,u=[].toLocaleString,c=[].slice,d=!!o&&a((function(){u.call(new o(1))})),f=a((function(){return[1,2].toLocaleString()!=new o([1,2]).toLocaleString()}))||!a((function(){o.prototype.toLocaleString.call([1,2])}));l("toLocaleString",(function(){return u.apply(d?c.call(s(this)):s(this),arguments)}),f)},b3eb:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e,t,n,r){var i={m:["eine Minute","einer Minute"],h:["eine Stunde","einer Stunde"],d:["ein Tag","einem Tag"],dd:[e+" Tage",e+" Tagen"],w:["eine Woche","einer Woche"],M:["ein Monat","einem Monat"],MM:[e+" Monate",e+" Monaten"],y:["ein Jahr","einem Jahr"],yy:[e+" Jahre",e+" Jahren"]};return t?i[n][0]:i[n][1]}var n=e.defineLocale("de-at",{months:"Jänner_Februar_März_April_Mai_Juni_Juli_August_September_Oktober_November_Dezember".split("_"),monthsShort:"Jän._Feb._März_Apr._Mai_Juni_Juli_Aug._Sep._Okt._Nov._Dez.".split("_"),monthsParseExact:!0,weekdays:"Sonntag_Montag_Dienstag_Mittwoch_Donnerstag_Freitag_Samstag".split("_"),weekdaysShort:"So._Mo._Di._Mi._Do._Fr._Sa.".split("_"),weekdaysMin:"So_Mo_Di_Mi_Do_Fr_Sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY HH:mm",LLLL:"dddd, D. MMMM YYYY HH:mm"},calendar:{sameDay:"[heute um] LT [Uhr]",sameElse:"L",nextDay:"[morgen um] LT [Uhr]",nextWeek:"dddd [um] LT [Uhr]",lastDay:"[gestern um] LT [Uhr]",lastWeek:"[letzten] dddd [um] LT [Uhr]"},relativeTime:{future:"in %s",past:"vor %s",s:"ein paar Sekunden",ss:"%d Sekunden",m:t,mm:"%d Minuten",h:t,hh:"%d Stunden",d:t,dd:t,w:t,ww:"%d Wochen",M:t,MM:t,y:t,yy:t},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return n}))},b406:function(e,t,n){"use strict";n("f24f")},b469:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e,t,n,r){var i={m:["eine Minute","einer Minute"],h:["eine Stunde","einer Stunde"],d:["ein Tag","einem Tag"],dd:[e+" Tage",e+" Tagen"],w:["eine Woche","einer Woche"],M:["ein Monat","einem Monat"],MM:[e+" Monate",e+" Monaten"],y:["ein Jahr","einem Jahr"],yy:[e+" Jahre",e+" Jahren"]};return t?i[n][0]:i[n][1]}var n=e.defineLocale("de",{months:"Januar_Februar_März_April_Mai_Juni_Juli_August_September_Oktober_November_Dezember".split("_"),monthsShort:"Jan._Feb._März_Apr._Mai_Juni_Juli_Aug._Sep._Okt._Nov._Dez.".split("_"),monthsParseExact:!0,weekdays:"Sonntag_Montag_Dienstag_Mittwoch_Donnerstag_Freitag_Samstag".split("_"),weekdaysShort:"So._Mo._Di._Mi._Do._Fr._Sa.".split("_"),weekdaysMin:"So_Mo_Di_Mi_Do_Fr_Sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY HH:mm",LLLL:"dddd, D. MMMM YYYY HH:mm"},calendar:{sameDay:"[heute um] LT [Uhr]",sameElse:"L",nextDay:"[morgen um] LT [Uhr]",nextWeek:"dddd [um] LT [Uhr]",lastDay:"[gestern um] LT [Uhr]",lastWeek:"[letzten] dddd [um] LT [Uhr]"},relativeTime:{future:"in %s",past:"vor %s",s:"ein paar Sekunden",ss:"%d Sekunden",m:t,mm:"%d Minuten",h:t,hh:"%d Stunden",d:t,dd:t,w:t,ww:"%d Wochen",M:t,MM:t,y:t,yy:t},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return n}))},b4c0:function(e,t,n){"use strict";n("e59d")},b53d:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("tzm-latn",{months:"innayr_brˤayrˤ_marˤsˤ_ibrir_mayyw_ywnyw_ywlywz_ɣwšt_šwtanbir_ktˤwbrˤ_nwwanbir_dwjnbir".split("_"),monthsShort:"innayr_brˤayrˤ_marˤsˤ_ibrir_mayyw_ywnyw_ywlywz_ɣwšt_šwtanbir_ktˤwbrˤ_nwwanbir_dwjnbir".split("_"),weekdays:"asamas_aynas_asinas_akras_akwas_asimwas_asiḍyas".split("_"),weekdaysShort:"asamas_aynas_asinas_akras_akwas_asimwas_asiḍyas".split("_"),weekdaysMin:"asamas_aynas_asinas_akras_akwas_asimwas_asiḍyas".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[asdkh g] LT",nextDay:"[aska g] LT",nextWeek:"dddd [g] LT",lastDay:"[assant g] LT",lastWeek:"dddd [g] LT",sameElse:"L"},relativeTime:{future:"dadkh s yan %s",past:"yan %s",s:"imik",ss:"%d imik",m:"minuḍ",mm:"%d minuḍ",h:"saɛa",hh:"%d tassaɛin",d:"ass",dd:"%d ossan",M:"ayowr",MM:"%d iyyirn",y:"asgas",yy:"%d isgasn"},week:{dow:6,doy:12}});return t}))},b540:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("jv",{months:"Januari_Februari_Maret_April_Mei_Juni_Juli_Agustus_September_Oktober_Nopember_Desember".split("_"),monthsShort:"Jan_Feb_Mar_Apr_Mei_Jun_Jul_Ags_Sep_Okt_Nop_Des".split("_"),weekdays:"Minggu_Senen_Seloso_Rebu_Kemis_Jemuwah_Septu".split("_"),weekdaysShort:"Min_Sen_Sel_Reb_Kem_Jem_Sep".split("_"),weekdaysMin:"Mg_Sn_Sl_Rb_Km_Jm_Sp".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [pukul] HH.mm",LLLL:"dddd, D MMMM YYYY [pukul] HH.mm"},meridiemParse:/enjing|siyang|sonten|ndalu/,meridiemHour:function(e,t){return 12===e&&(e=0),"enjing"===t?e:"siyang"===t?e>=11?e:e+12:"sonten"===t||"ndalu"===t?e+12:void 0},meridiem:function(e,t,n){return e<11?"enjing":e<15?"siyang":e<19?"sonten":"ndalu"},calendar:{sameDay:"[Dinten puniko pukul] LT",nextDay:"[Mbenjang pukul] LT",nextWeek:"dddd [pukul] LT",lastDay:"[Kala wingi pukul] LT",lastWeek:"dddd [kepengker pukul] LT",sameElse:"L"},relativeTime:{future:"wonten ing %s",past:"%s ingkang kepengker",s:"sawetawis detik",ss:"%d detik",m:"setunggal menit",mm:"%d menit",h:"setunggal jam",hh:"%d jam",d:"sedinten",dd:"%d dinten",M:"sewulan",MM:"%d wulan",y:"setaun",yy:"%d taun"},week:{dow:1,doy:7}});return t}))},b575:function(e,t,n){var r,i,a,o,s,l,u,c,d=n("da84"),f=n("06cf").f,h=n("2cf4").set,p=n("1cdc"),m=n("605d"),_=d.MutationObserver||d.WebKitMutationObserver,v=d.document,g=d.process,b=d.Promise,y=f(d,"queueMicrotask"),M=y&&y.value;M||(r=function(){var e,t;m&&(e=g.domain)&&e.exit();while(i){t=i.fn,i=i.next;try{t()}catch(n){throw i?o():a=void 0,n}}a=void 0,e&&e.enter()},!p&&!m&&_&&v?(s=!0,l=v.createTextNode(""),new _(r).observe(l,{characterData:!0}),o=function(){l.data=s=!s}):b&&b.resolve?(u=b.resolve(void 0),c=u.then,o=function(){c.call(u,r)}):o=m?function(){g.nextTick(r)}:function(){h.call(d,r)}),e.exports=M||function(e){var t={fn:e,next:void 0};a&&(a.next=t),i||(i=t,o()),a=t}},b5b7:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t="ene._feb._mar._abr._may._jun._jul._ago._sep._oct._nov._dic.".split("_"),n="ene_feb_mar_abr_may_jun_jul_ago_sep_oct_nov_dic".split("_"),r=[/^ene/i,/^feb/i,/^mar/i,/^abr/i,/^may/i,/^jun/i,/^jul/i,/^ago/i,/^sep/i,/^oct/i,/^nov/i,/^dic/i],i=/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre|ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,a=e.defineLocale("es-mx",{months:"enero_febrero_marzo_abril_mayo_junio_julio_agosto_septiembre_octubre_noviembre_diciembre".split("_"),monthsShort:function(e,r){return e?/-MMM-/.test(r)?n[e.month()]:t[e.month()]:t},monthsRegex:i,monthsShortRegex:i,monthsStrictRegex:/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre)/i,monthsShortStrictRegex:/^(ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,monthsParse:r,longMonthsParse:r,shortMonthsParse:r,weekdays:"domingo_lunes_martes_miércoles_jueves_viernes_sábado".split("_"),weekdaysShort:"dom._lun._mar._mié._jue._vie._sáb.".split("_"),weekdaysMin:"do_lu_ma_mi_ju_vi_sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY H:mm",LLLL:"dddd, D [de] MMMM [de] YYYY H:mm"},calendar:{sameDay:function(){return"[hoy a la"+(1!==this.hours()?"s":"")+"] LT"},nextDay:function(){return"[mañana a la"+(1!==this.hours()?"s":"")+"] LT"},nextWeek:function(){return"dddd [a la"+(1!==this.hours()?"s":"")+"] LT"},lastDay:function(){return"[ayer a la"+(1!==this.hours()?"s":"")+"] LT"},lastWeek:function(){return"[el] dddd [pasado a la"+(1!==this.hours()?"s":"")+"] LT"},sameElse:"L"},relativeTime:{future:"en %s",past:"hace %s",s:"unos segundos",ss:"%d segundos",m:"un minuto",mm:"%d minutos",h:"una hora",hh:"%d horas",d:"un día",dd:"%d días",w:"una semana",ww:"%d semanas",M:"un mes",MM:"%d meses",y:"un año",yy:"%d años"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:0,doy:4},invalidDate:"Fecha inválida"});return a}))},b622:function(e,t,n){var r=n("da84"),i=n("5692"),a=n("5135"),o=n("90e3"),s=n("4930"),l=n("fdbf"),u=i("wks"),c=r.Symbol,d=l?c:c&&c.withoutSetter||o;e.exports=function(e){return a(u,e)||(s&&a(c,e)?u[e]=c[e]:u[e]=d("Symbol."+e)),u[e]}},b639:function(e,t,n){"use strict";(function(e){
/*!
 * The buffer module from node.js, for the browser.
 *
 * @author   Feross Aboukhadijeh <http://feross.org>
 * @license  MIT
 */
var r=n("1fb5"),i=n("9152"),a=n("e3db");function o(){try{var e=new Uint8Array(1);return e.__proto__={__proto__:Uint8Array.prototype,foo:function(){return 42}},42===e.foo()&&"function"===typeof e.subarray&&0===e.subarray(1,1).byteLength}catch(t){return!1}}function s(){return u.TYPED_ARRAY_SUPPORT?2147483647:1073741823}function l(e,t){if(s()<t)throw new RangeError("Invalid typed array length");return u.TYPED_ARRAY_SUPPORT?(e=new Uint8Array(t),e.__proto__=u.prototype):(null===e&&(e=new u(t)),e.length=t),e}function u(e,t,n){if(!u.TYPED_ARRAY_SUPPORT&&!(this instanceof u))return new u(e,t,n);if("number"===typeof e){if("string"===typeof t)throw new Error("If encoding is specified then the first argument must be a string");return h(this,e)}return c(this,e,t,n)}function c(e,t,n,r){if("number"===typeof t)throw new TypeError('"value" argument must not be a number');return"undefined"!==typeof ArrayBuffer&&t instanceof ArrayBuffer?_(e,t,n,r):"string"===typeof t?p(e,t,n):v(e,t)}function d(e){if("number"!==typeof e)throw new TypeError('"size" argument must be a number');if(e<0)throw new RangeError('"size" argument must not be negative')}function f(e,t,n,r){return d(t),t<=0?l(e,t):void 0!==n?"string"===typeof r?l(e,t).fill(n,r):l(e,t).fill(n):l(e,t)}function h(e,t){if(d(t),e=l(e,t<0?0:0|g(t)),!u.TYPED_ARRAY_SUPPORT)for(var n=0;n<t;++n)e[n]=0;return e}function p(e,t,n){if("string"===typeof n&&""!==n||(n="utf8"),!u.isEncoding(n))throw new TypeError('"encoding" must be a valid string encoding');var r=0|y(t,n);e=l(e,r);var i=e.write(t,n);return i!==r&&(e=e.slice(0,i)),e}function m(e,t){var n=t.length<0?0:0|g(t.length);e=l(e,n);for(var r=0;r<n;r+=1)e[r]=255&t[r];return e}function _(e,t,n,r){if(t.byteLength,n<0||t.byteLength<n)throw new RangeError("'offset' is out of bounds");if(t.byteLength<n+(r||0))throw new RangeError("'length' is out of bounds");return t=void 0===n&&void 0===r?new Uint8Array(t):void 0===r?new Uint8Array(t,n):new Uint8Array(t,n,r),u.TYPED_ARRAY_SUPPORT?(e=t,e.__proto__=u.prototype):e=m(e,t),e}function v(e,t){if(u.isBuffer(t)){var n=0|g(t.length);return e=l(e,n),0===e.length?e:(t.copy(e,0,0,n),e)}if(t){if("undefined"!==typeof ArrayBuffer&&t.buffer instanceof ArrayBuffer||"length"in t)return"number"!==typeof t.length||te(t.length)?l(e,0):m(e,t);if("Buffer"===t.type&&a(t.data))return m(e,t.data)}throw new TypeError("First argument must be a string, Buffer, ArrayBuffer, Array, or array-like object.")}function g(e){if(e>=s())throw new RangeError("Attempt to allocate Buffer larger than maximum size: 0x"+s().toString(16)+" bytes");return 0|e}function b(e){return+e!=e&&(e=0),u.alloc(+e)}function y(e,t){if(u.isBuffer(e))return e.length;if("undefined"!==typeof ArrayBuffer&&"function"===typeof ArrayBuffer.isView&&(ArrayBuffer.isView(e)||e instanceof ArrayBuffer))return e.byteLength;"string"!==typeof e&&(e=""+e);var n=e.length;if(0===n)return 0;for(var r=!1;;)switch(t){case"ascii":case"latin1":case"binary":return n;case"utf8":case"utf-8":case void 0:return K(e).length;case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return 2*n;case"hex":return n>>>1;case"base64":return Z(e).length;default:if(r)return K(e).length;t=(""+t).toLowerCase(),r=!0}}function M(e,t,n){var r=!1;if((void 0===t||t<0)&&(t=0),t>this.length)return"";if((void 0===n||n>this.length)&&(n=this.length),n<=0)return"";if(n>>>=0,t>>>=0,n<=t)return"";e||(e="utf8");while(1)switch(e){case"hex":return j(this,t,n);case"utf8":case"utf-8":return A(this,t,n);case"ascii":return F(this,t,n);case"latin1":case"binary":return I(this,t,n);case"base64":return E(this,t,n);case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return H(this,t,n);default:if(r)throw new TypeError("Unknown encoding: "+e);e=(e+"").toLowerCase(),r=!0}}function k(e,t,n){var r=e[t];e[t]=e[n],e[n]=r}function w(e,t,n,r,i){if(0===e.length)return-1;if("string"===typeof n?(r=n,n=0):n>2147483647?n=2147483647:n<-2147483648&&(n=-2147483648),n=+n,isNaN(n)&&(n=i?0:e.length-1),n<0&&(n=e.length+n),n>=e.length){if(i)return-1;n=e.length-1}else if(n<0){if(!i)return-1;n=0}if("string"===typeof t&&(t=u.from(t,r)),u.isBuffer(t))return 0===t.length?-1:L(e,t,n,r,i);if("number"===typeof t)return t&=255,u.TYPED_ARRAY_SUPPORT&&"function"===typeof Uint8Array.prototype.indexOf?i?Uint8Array.prototype.indexOf.call(e,t,n):Uint8Array.prototype.lastIndexOf.call(e,t,n):L(e,[t],n,r,i);throw new TypeError("val must be string, number or Buffer")}function L(e,t,n,r,i){var a,o=1,s=e.length,l=t.length;if(void 0!==r&&(r=String(r).toLowerCase(),"ucs2"===r||"ucs-2"===r||"utf16le"===r||"utf-16le"===r)){if(e.length<2||t.length<2)return-1;o=2,s/=2,l/=2,n/=2}function u(e,t){return 1===o?e[t]:e.readUInt16BE(t*o)}if(i){var c=-1;for(a=n;a<s;a++)if(u(e,a)===u(t,-1===c?0:a-c)){if(-1===c&&(c=a),a-c+1===l)return c*o}else-1!==c&&(a-=a-c),c=-1}else for(n+l>s&&(n=s-l),a=n;a>=0;a--){for(var d=!0,f=0;f<l;f++)if(u(e,a+f)!==u(t,f)){d=!1;break}if(d)return a}return-1}function x(e,t,n,r){n=Number(n)||0;var i=e.length-n;r?(r=Number(r),r>i&&(r=i)):r=i;var a=t.length;if(a%2!==0)throw new TypeError("Invalid hex string");r>a/2&&(r=a/2);for(var o=0;o<r;++o){var s=parseInt(t.substr(2*o,2),16);if(isNaN(s))return o;e[n+o]=s}return o}function S(e,t,n,r){return ee(K(t,e.length-n),e,n,r)}function T(e,t,n,r){return ee(X(t),e,n,r)}function D(e,t,n,r){return T(e,t,n,r)}function Y(e,t,n,r){return ee(Z(t),e,n,r)}function C(e,t,n,r){return ee(Q(t,e.length-n),e,n,r)}function E(e,t,n){return 0===t&&n===e.length?r.fromByteArray(e):r.fromByteArray(e.slice(t,n))}function A(e,t,n){n=Math.min(e.length,n);var r=[],i=t;while(i<n){var a,o,s,l,u=e[i],c=null,d=u>239?4:u>223?3:u>191?2:1;if(i+d<=n)switch(d){case 1:u<128&&(c=u);break;case 2:a=e[i+1],128===(192&a)&&(l=(31&u)<<6|63&a,l>127&&(c=l));break;case 3:a=e[i+1],o=e[i+2],128===(192&a)&&128===(192&o)&&(l=(15&u)<<12|(63&a)<<6|63&o,l>2047&&(l<55296||l>57343)&&(c=l));break;case 4:a=e[i+1],o=e[i+2],s=e[i+3],128===(192&a)&&128===(192&o)&&128===(192&s)&&(l=(15&u)<<18|(63&a)<<12|(63&o)<<6|63&s,l>65535&&l<1114112&&(c=l))}null===c?(c=65533,d=1):c>65535&&(c-=65536,r.push(c>>>10&1023|55296),c=56320|1023&c),r.push(c),i+=d}return P(r)}t.Buffer=u,t.SlowBuffer=b,t.INSPECT_MAX_BYTES=50,u.TYPED_ARRAY_SUPPORT=void 0!==e.TYPED_ARRAY_SUPPORT?e.TYPED_ARRAY_SUPPORT:o(),t.kMaxLength=s(),u.poolSize=8192,u._augment=function(e){return e.__proto__=u.prototype,e},u.from=function(e,t,n){return c(null,e,t,n)},u.TYPED_ARRAY_SUPPORT&&(u.prototype.__proto__=Uint8Array.prototype,u.__proto__=Uint8Array,"undefined"!==typeof Symbol&&Symbol.species&&u[Symbol.species]===u&&Object.defineProperty(u,Symbol.species,{value:null,configurable:!0})),u.alloc=function(e,t,n){return f(null,e,t,n)},u.allocUnsafe=function(e){return h(null,e)},u.allocUnsafeSlow=function(e){return h(null,e)},u.isBuffer=function(e){return!(null==e||!e._isBuffer)},u.compare=function(e,t){if(!u.isBuffer(e)||!u.isBuffer(t))throw new TypeError("Arguments must be Buffers");if(e===t)return 0;for(var n=e.length,r=t.length,i=0,a=Math.min(n,r);i<a;++i)if(e[i]!==t[i]){n=e[i],r=t[i];break}return n<r?-1:r<n?1:0},u.isEncoding=function(e){switch(String(e).toLowerCase()){case"hex":case"utf8":case"utf-8":case"ascii":case"latin1":case"binary":case"base64":case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return!0;default:return!1}},u.concat=function(e,t){if(!a(e))throw new TypeError('"list" argument must be an Array of Buffers');if(0===e.length)return u.alloc(0);var n;if(void 0===t)for(t=0,n=0;n<e.length;++n)t+=e[n].length;var r=u.allocUnsafe(t),i=0;for(n=0;n<e.length;++n){var o=e[n];if(!u.isBuffer(o))throw new TypeError('"list" argument must be an Array of Buffers');o.copy(r,i),i+=o.length}return r},u.byteLength=y,u.prototype._isBuffer=!0,u.prototype.swap16=function(){var e=this.length;if(e%2!==0)throw new RangeError("Buffer size must be a multiple of 16-bits");for(var t=0;t<e;t+=2)k(this,t,t+1);return this},u.prototype.swap32=function(){var e=this.length;if(e%4!==0)throw new RangeError("Buffer size must be a multiple of 32-bits");for(var t=0;t<e;t+=4)k(this,t,t+3),k(this,t+1,t+2);return this},u.prototype.swap64=function(){var e=this.length;if(e%8!==0)throw new RangeError("Buffer size must be a multiple of 64-bits");for(var t=0;t<e;t+=8)k(this,t,t+7),k(this,t+1,t+6),k(this,t+2,t+5),k(this,t+3,t+4);return this},u.prototype.toString=function(){var e=0|this.length;return 0===e?"":0===arguments.length?A(this,0,e):M.apply(this,arguments)},u.prototype.equals=function(e){if(!u.isBuffer(e))throw new TypeError("Argument must be a Buffer");return this===e||0===u.compare(this,e)},u.prototype.inspect=function(){var e="",n=t.INSPECT_MAX_BYTES;return this.length>0&&(e=this.toString("hex",0,n).match(/.{2}/g).join(" "),this.length>n&&(e+=" ... ")),"<Buffer "+e+">"},u.prototype.compare=function(e,t,n,r,i){if(!u.isBuffer(e))throw new TypeError("Argument must be a Buffer");if(void 0===t&&(t=0),void 0===n&&(n=e?e.length:0),void 0===r&&(r=0),void 0===i&&(i=this.length),t<0||n>e.length||r<0||i>this.length)throw new RangeError("out of range index");if(r>=i&&t>=n)return 0;if(r>=i)return-1;if(t>=n)return 1;if(t>>>=0,n>>>=0,r>>>=0,i>>>=0,this===e)return 0;for(var a=i-r,o=n-t,s=Math.min(a,o),l=this.slice(r,i),c=e.slice(t,n),d=0;d<s;++d)if(l[d]!==c[d]){a=l[d],o=c[d];break}return a<o?-1:o<a?1:0},u.prototype.includes=function(e,t,n){return-1!==this.indexOf(e,t,n)},u.prototype.indexOf=function(e,t,n){return w(this,e,t,n,!0)},u.prototype.lastIndexOf=function(e,t,n){return w(this,e,t,n,!1)},u.prototype.write=function(e,t,n,r){if(void 0===t)r="utf8",n=this.length,t=0;else if(void 0===n&&"string"===typeof t)r=t,n=this.length,t=0;else{if(!isFinite(t))throw new Error("Buffer.write(string, encoding, offset[, length]) is no longer supported");t|=0,isFinite(n)?(n|=0,void 0===r&&(r="utf8")):(r=n,n=void 0)}var i=this.length-t;if((void 0===n||n>i)&&(n=i),e.length>0&&(n<0||t<0)||t>this.length)throw new RangeError("Attempt to write outside buffer bounds");r||(r="utf8");for(var a=!1;;)switch(r){case"hex":return x(this,e,t,n);case"utf8":case"utf-8":return S(this,e,t,n);case"ascii":return T(this,e,t,n);case"latin1":case"binary":return D(this,e,t,n);case"base64":return Y(this,e,t,n);case"ucs2":case"ucs-2":case"utf16le":case"utf-16le":return C(this,e,t,n);default:if(a)throw new TypeError("Unknown encoding: "+r);r=(""+r).toLowerCase(),a=!0}},u.prototype.toJSON=function(){return{type:"Buffer",data:Array.prototype.slice.call(this._arr||this,0)}};var O=4096;function P(e){var t=e.length;if(t<=O)return String.fromCharCode.apply(String,e);var n="",r=0;while(r<t)n+=String.fromCharCode.apply(String,e.slice(r,r+=O));return n}function F(e,t,n){var r="";n=Math.min(e.length,n);for(var i=t;i<n;++i)r+=String.fromCharCode(127&e[i]);return r}function I(e,t,n){var r="";n=Math.min(e.length,n);for(var i=t;i<n;++i)r+=String.fromCharCode(e[i]);return r}function j(e,t,n){var r=e.length;(!t||t<0)&&(t=0),(!n||n<0||n>r)&&(n=r);for(var i="",a=t;a<n;++a)i+=J(e[a]);return i}function H(e,t,n){for(var r=e.slice(t,n),i="",a=0;a<r.length;a+=2)i+=String.fromCharCode(r[a]+256*r[a+1]);return i}function R(e,t,n){if(e%1!==0||e<0)throw new RangeError("offset is not uint");if(e+t>n)throw new RangeError("Trying to access beyond buffer length")}function N(e,t,n,r,i,a){if(!u.isBuffer(e))throw new TypeError('"buffer" argument must be a Buffer instance');if(t>i||t<a)throw new RangeError('"value" argument is out of bounds');if(n+r>e.length)throw new RangeError("Index out of range")}function $(e,t,n,r){t<0&&(t=65535+t+1);for(var i=0,a=Math.min(e.length-n,2);i<a;++i)e[n+i]=(t&255<<8*(r?i:1-i))>>>8*(r?i:1-i)}function z(e,t,n,r){t<0&&(t=4294967295+t+1);for(var i=0,a=Math.min(e.length-n,4);i<a;++i)e[n+i]=t>>>8*(r?i:3-i)&255}function B(e,t,n,r,i,a){if(n+r>e.length)throw new RangeError("Index out of range");if(n<0)throw new RangeError("Index out of range")}function W(e,t,n,r,a){return a||B(e,t,n,4,34028234663852886e22,-34028234663852886e22),i.write(e,t,n,r,23,4),n+4}function U(e,t,n,r,a){return a||B(e,t,n,8,17976931348623157e292,-17976931348623157e292),i.write(e,t,n,r,52,8),n+8}u.prototype.slice=function(e,t){var n,r=this.length;if(e=~~e,t=void 0===t?r:~~t,e<0?(e+=r,e<0&&(e=0)):e>r&&(e=r),t<0?(t+=r,t<0&&(t=0)):t>r&&(t=r),t<e&&(t=e),u.TYPED_ARRAY_SUPPORT)n=this.subarray(e,t),n.__proto__=u.prototype;else{var i=t-e;n=new u(i,void 0);for(var a=0;a<i;++a)n[a]=this[a+e]}return n},u.prototype.readUIntLE=function(e,t,n){e|=0,t|=0,n||R(e,t,this.length);var r=this[e],i=1,a=0;while(++a<t&&(i*=256))r+=this[e+a]*i;return r},u.prototype.readUIntBE=function(e,t,n){e|=0,t|=0,n||R(e,t,this.length);var r=this[e+--t],i=1;while(t>0&&(i*=256))r+=this[e+--t]*i;return r},u.prototype.readUInt8=function(e,t){return t||R(e,1,this.length),this[e]},u.prototype.readUInt16LE=function(e,t){return t||R(e,2,this.length),this[e]|this[e+1]<<8},u.prototype.readUInt16BE=function(e,t){return t||R(e,2,this.length),this[e]<<8|this[e+1]},u.prototype.readUInt32LE=function(e,t){return t||R(e,4,this.length),(this[e]|this[e+1]<<8|this[e+2]<<16)+16777216*this[e+3]},u.prototype.readUInt32BE=function(e,t){return t||R(e,4,this.length),16777216*this[e]+(this[e+1]<<16|this[e+2]<<8|this[e+3])},u.prototype.readIntLE=function(e,t,n){e|=0,t|=0,n||R(e,t,this.length);var r=this[e],i=1,a=0;while(++a<t&&(i*=256))r+=this[e+a]*i;return i*=128,r>=i&&(r-=Math.pow(2,8*t)),r},u.prototype.readIntBE=function(e,t,n){e|=0,t|=0,n||R(e,t,this.length);var r=t,i=1,a=this[e+--r];while(r>0&&(i*=256))a+=this[e+--r]*i;return i*=128,a>=i&&(a-=Math.pow(2,8*t)),a},u.prototype.readInt8=function(e,t){return t||R(e,1,this.length),128&this[e]?-1*(255-this[e]+1):this[e]},u.prototype.readInt16LE=function(e,t){t||R(e,2,this.length);var n=this[e]|this[e+1]<<8;return 32768&n?4294901760|n:n},u.prototype.readInt16BE=function(e,t){t||R(e,2,this.length);var n=this[e+1]|this[e]<<8;return 32768&n?4294901760|n:n},u.prototype.readInt32LE=function(e,t){return t||R(e,4,this.length),this[e]|this[e+1]<<8|this[e+2]<<16|this[e+3]<<24},u.prototype.readInt32BE=function(e,t){return t||R(e,4,this.length),this[e]<<24|this[e+1]<<16|this[e+2]<<8|this[e+3]},u.prototype.readFloatLE=function(e,t){return t||R(e,4,this.length),i.read(this,e,!0,23,4)},u.prototype.readFloatBE=function(e,t){return t||R(e,4,this.length),i.read(this,e,!1,23,4)},u.prototype.readDoubleLE=function(e,t){return t||R(e,8,this.length),i.read(this,e,!0,52,8)},u.prototype.readDoubleBE=function(e,t){return t||R(e,8,this.length),i.read(this,e,!1,52,8)},u.prototype.writeUIntLE=function(e,t,n,r){if(e=+e,t|=0,n|=0,!r){var i=Math.pow(2,8*n)-1;N(this,e,t,n,i,0)}var a=1,o=0;this[t]=255&e;while(++o<n&&(a*=256))this[t+o]=e/a&255;return t+n},u.prototype.writeUIntBE=function(e,t,n,r){if(e=+e,t|=0,n|=0,!r){var i=Math.pow(2,8*n)-1;N(this,e,t,n,i,0)}var a=n-1,o=1;this[t+a]=255&e;while(--a>=0&&(o*=256))this[t+a]=e/o&255;return t+n},u.prototype.writeUInt8=function(e,t,n){return e=+e,t|=0,n||N(this,e,t,1,255,0),u.TYPED_ARRAY_SUPPORT||(e=Math.floor(e)),this[t]=255&e,t+1},u.prototype.writeUInt16LE=function(e,t,n){return e=+e,t|=0,n||N(this,e,t,2,65535,0),u.TYPED_ARRAY_SUPPORT?(this[t]=255&e,this[t+1]=e>>>8):$(this,e,t,!0),t+2},u.prototype.writeUInt16BE=function(e,t,n){return e=+e,t|=0,n||N(this,e,t,2,65535,0),u.TYPED_ARRAY_SUPPORT?(this[t]=e>>>8,this[t+1]=255&e):$(this,e,t,!1),t+2},u.prototype.writeUInt32LE=function(e,t,n){return e=+e,t|=0,n||N(this,e,t,4,4294967295,0),u.TYPED_ARRAY_SUPPORT?(this[t+3]=e>>>24,this[t+2]=e>>>16,this[t+1]=e>>>8,this[t]=255&e):z(this,e,t,!0),t+4},u.prototype.writeUInt32BE=function(e,t,n){return e=+e,t|=0,n||N(this,e,t,4,4294967295,0),u.TYPED_ARRAY_SUPPORT?(this[t]=e>>>24,this[t+1]=e>>>16,this[t+2]=e>>>8,this[t+3]=255&e):z(this,e,t,!1),t+4},u.prototype.writeIntLE=function(e,t,n,r){if(e=+e,t|=0,!r){var i=Math.pow(2,8*n-1);N(this,e,t,n,i-1,-i)}var a=0,o=1,s=0;this[t]=255&e;while(++a<n&&(o*=256))e<0&&0===s&&0!==this[t+a-1]&&(s=1),this[t+a]=(e/o>>0)-s&255;return t+n},u.prototype.writeIntBE=function(e,t,n,r){if(e=+e,t|=0,!r){var i=Math.pow(2,8*n-1);N(this,e,t,n,i-1,-i)}var a=n-1,o=1,s=0;this[t+a]=255&e;while(--a>=0&&(o*=256))e<0&&0===s&&0!==this[t+a+1]&&(s=1),this[t+a]=(e/o>>0)-s&255;return t+n},u.prototype.writeInt8=function(e,t,n){return e=+e,t|=0,n||N(this,e,t,1,127,-128),u.TYPED_ARRAY_SUPPORT||(e=Math.floor(e)),e<0&&(e=255+e+1),this[t]=255&e,t+1},u.prototype.writeInt16LE=function(e,t,n){return e=+e,t|=0,n||N(this,e,t,2,32767,-32768),u.TYPED_ARRAY_SUPPORT?(this[t]=255&e,this[t+1]=e>>>8):$(this,e,t,!0),t+2},u.prototype.writeInt16BE=function(e,t,n){return e=+e,t|=0,n||N(this,e,t,2,32767,-32768),u.TYPED_ARRAY_SUPPORT?(this[t]=e>>>8,this[t+1]=255&e):$(this,e,t,!1),t+2},u.prototype.writeInt32LE=function(e,t,n){return e=+e,t|=0,n||N(this,e,t,4,2147483647,-2147483648),u.TYPED_ARRAY_SUPPORT?(this[t]=255&e,this[t+1]=e>>>8,this[t+2]=e>>>16,this[t+3]=e>>>24):z(this,e,t,!0),t+4},u.prototype.writeInt32BE=function(e,t,n){return e=+e,t|=0,n||N(this,e,t,4,2147483647,-2147483648),e<0&&(e=4294967295+e+1),u.TYPED_ARRAY_SUPPORT?(this[t]=e>>>24,this[t+1]=e>>>16,this[t+2]=e>>>8,this[t+3]=255&e):z(this,e,t,!1),t+4},u.prototype.writeFloatLE=function(e,t,n){return W(this,e,t,!0,n)},u.prototype.writeFloatBE=function(e,t,n){return W(this,e,t,!1,n)},u.prototype.writeDoubleLE=function(e,t,n){return U(this,e,t,!0,n)},u.prototype.writeDoubleBE=function(e,t,n){return U(this,e,t,!1,n)},u.prototype.copy=function(e,t,n,r){if(n||(n=0),r||0===r||(r=this.length),t>=e.length&&(t=e.length),t||(t=0),r>0&&r<n&&(r=n),r===n)return 0;if(0===e.length||0===this.length)return 0;if(t<0)throw new RangeError("targetStart out of bounds");if(n<0||n>=this.length)throw new RangeError("sourceStart out of bounds");if(r<0)throw new RangeError("sourceEnd out of bounds");r>this.length&&(r=this.length),e.length-t<r-n&&(r=e.length-t+n);var i,a=r-n;if(this===e&&n<t&&t<r)for(i=a-1;i>=0;--i)e[i+t]=this[i+n];else if(a<1e3||!u.TYPED_ARRAY_SUPPORT)for(i=0;i<a;++i)e[i+t]=this[i+n];else Uint8Array.prototype.set.call(e,this.subarray(n,n+a),t);return a},u.prototype.fill=function(e,t,n,r){if("string"===typeof e){if("string"===typeof t?(r=t,t=0,n=this.length):"string"===typeof n&&(r=n,n=this.length),1===e.length){var i=e.charCodeAt(0);i<256&&(e=i)}if(void 0!==r&&"string"!==typeof r)throw new TypeError("encoding must be a string");if("string"===typeof r&&!u.isEncoding(r))throw new TypeError("Unknown encoding: "+r)}else"number"===typeof e&&(e&=255);if(t<0||this.length<t||this.length<n)throw new RangeError("Out of range index");if(n<=t)return this;var a;if(t>>>=0,n=void 0===n?this.length:n>>>0,e||(e=0),"number"===typeof e)for(a=t;a<n;++a)this[a]=e;else{var o=u.isBuffer(e)?e:K(new u(e,r).toString()),s=o.length;for(a=0;a<n-t;++a)this[a+t]=o[a%s]}return this};var V=/[^+\/0-9A-Za-z-_]/g;function q(e){if(e=G(e).replace(V,""),e.length<2)return"";while(e.length%4!==0)e+="=";return e}function G(e){return e.trim?e.trim():e.replace(/^\s+|\s+$/g,"")}function J(e){return e<16?"0"+e.toString(16):e.toString(16)}function K(e,t){var n;t=t||1/0;for(var r=e.length,i=null,a=[],o=0;o<r;++o){if(n=e.charCodeAt(o),n>55295&&n<57344){if(!i){if(n>56319){(t-=3)>-1&&a.push(239,191,189);continue}if(o+1===r){(t-=3)>-1&&a.push(239,191,189);continue}i=n;continue}if(n<56320){(t-=3)>-1&&a.push(239,191,189),i=n;continue}n=65536+(i-55296<<10|n-56320)}else i&&(t-=3)>-1&&a.push(239,191,189);if(i=null,n<128){if((t-=1)<0)break;a.push(n)}else if(n<2048){if((t-=2)<0)break;a.push(n>>6|192,63&n|128)}else if(n<65536){if((t-=3)<0)break;a.push(n>>12|224,n>>6&63|128,63&n|128)}else{if(!(n<1114112))throw new Error("Invalid code point");if((t-=4)<0)break;a.push(n>>18|240,n>>12&63|128,n>>6&63|128,63&n|128)}}return a}function X(e){for(var t=[],n=0;n<e.length;++n)t.push(255&e.charCodeAt(n));return t}function Q(e,t){for(var n,r,i,a=[],o=0;o<e.length;++o){if((t-=2)<0)break;n=e.charCodeAt(o),r=n>>8,i=n%256,a.push(i),a.push(r)}return a}function Z(e){return r.toByteArray(q(e))}function ee(e,t,n,r){for(var i=0;i<r;++i){if(i+n>=t.length||i>=e.length)break;t[i+n]=e[i]}return i}function te(e){return e!==e}}).call(this,n("c8ba"))},b64b:function(e,t,n){var r=n("23e7"),i=n("7b0b"),a=n("df75"),o=n("d039"),s=o((function(){a(1)}));r({target:"Object",stat:!0,forced:s},{keys:function(e){return a(i(e))}})},b680:function(e,t,n){"use strict";var r=n("23e7"),i=n("a691"),a=n("408a"),o=n("1148"),s=n("d039"),l=1..toFixed,u=Math.floor,c=function(e,t,n){return 0===t?n:t%2===1?c(e,t-1,n*e):c(e*e,t/2,n)},d=function(e){var t=0,n=e;while(n>=4096)t+=12,n/=4096;while(n>=2)t+=1,n/=2;return t},f=l&&("0.000"!==8e-5.toFixed(3)||"1"!==.9.toFixed(0)||"1.25"!==1.255.toFixed(2)||"1000000000000000128"!==(0xde0b6b3a7640080).toFixed(0))||!s((function(){l.call({})}));r({target:"Number",proto:!0,forced:f},{toFixed:function(e){var t,n,r,s,l=a(this),f=i(e),h=[0,0,0,0,0,0],p="",m="0",_=function(e,t){var n=-1,r=t;while(++n<6)r+=e*h[n],h[n]=r%1e7,r=u(r/1e7)},v=function(e){var t=6,n=0;while(--t>=0)n+=h[t],h[t]=u(n/e),n=n%e*1e7},g=function(){var e=6,t="";while(--e>=0)if(""!==t||0===e||0!==h[e]){var n=String(h[e]);t=""===t?n:t+o.call("0",7-n.length)+n}return t};if(f<0||f>20)throw RangeError("Incorrect fraction digits");if(l!=l)return"NaN";if(l<=-1e21||l>=1e21)return String(l);if(l<0&&(p="-",l=-l),l>1e-21)if(t=d(l*c(2,69,1))-69,n=t<0?l*c(2,-t,1):l/c(2,t,1),n*=4503599627370496,t=52-t,t>0){_(0,n),r=f;while(r>=7)_(1e7,0),r-=7;_(c(10,r,1),0),r=t-1;while(r>=23)v(1<<23),r-=23;v(1<<r),_(1,1),v(2),m=g()}else _(0,n),_(1<<-t,0),m=g()+o.call("0",f);return f>0?(s=m.length,m=p+(s<=f?"0."+o.call("0",f-s)+m:m.slice(0,s-f)+"."+m.slice(s-f))):m=p+m,m}})},b727:function(e,t,n){var r=n("0366"),i=n("44ad"),a=n("7b0b"),o=n("50c4"),s=n("65f0"),l=[].push,u=function(e){var t=1==e,n=2==e,u=3==e,c=4==e,d=6==e,f=7==e,h=5==e||d;return function(p,m,_,v){for(var g,b,y=a(p),M=i(y),k=r(m,_,3),w=o(M.length),L=0,x=v||s,S=t?x(p,w):n||f?x(p,0):void 0;w>L;L++)if((h||L in M)&&(g=M[L],b=k(g,L,y),e))if(t)S[L]=b;else if(b)switch(e){case 3:return!0;case 5:return g;case 6:return L;case 2:l.call(S,g)}else switch(e){case 4:return!1;case 7:l.call(S,g)}return d?-1:u||c?c:S}};e.exports={forEach:u(0),map:u(1),filter:u(2),some:u(3),every:u(4),find:u(5),findIndex:u(6),filterOut:u(7)}},b76a:function(e,t,n){(function(t,r){e.exports=r(n("aa47"))})("undefined"!==typeof self&&self,(function(e){return function(e){var t={};function n(r){if(t[r])return t[r].exports;var i=t[r]={i:r,l:!1,exports:{}};return e[r].call(i.exports,i,i.exports,n),i.l=!0,i.exports}return n.m=e,n.c=t,n.d=function(e,t,r){n.o(e,t)||Object.defineProperty(e,t,{enumerable:!0,get:r})},n.r=function(e){"undefined"!==typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})},n.t=function(e,t){if(1&t&&(e=n(e)),8&t)return e;if(4&t&&"object"===typeof e&&e&&e.__esModule)return e;var r=Object.create(null);if(n.r(r),Object.defineProperty(r,"default",{enumerable:!0,value:e}),2&t&&"string"!=typeof e)for(var i in e)n.d(r,i,function(t){return e[t]}.bind(null,i));return r},n.n=function(e){var t=e&&e.__esModule?function(){return e["default"]}:function(){return e};return n.d(t,"a",t),t},n.o=function(e,t){return Object.prototype.hasOwnProperty.call(e,t)},n.p="",n(n.s="fb15")}({"01f9":function(e,t,n){"use strict";var r=n("2d00"),i=n("5ca1"),a=n("2aba"),o=n("32e9"),s=n("84f2"),l=n("41a0"),u=n("7f20"),c=n("38fd"),d=n("2b4c")("iterator"),f=!([].keys&&"next"in[].keys()),h="@@iterator",p="keys",m="values",_=function(){return this};e.exports=function(e,t,n,v,g,b,y){l(n,t,v);var M,k,w,L=function(e){if(!f&&e in D)return D[e];switch(e){case p:return function(){return new n(this,e)};case m:return function(){return new n(this,e)}}return function(){return new n(this,e)}},x=t+" Iterator",S=g==m,T=!1,D=e.prototype,Y=D[d]||D[h]||g&&D[g],C=Y||L(g),E=g?S?L("entries"):C:void 0,A="Array"==t&&D.entries||Y;if(A&&(w=c(A.call(new e)),w!==Object.prototype&&w.next&&(u(w,x,!0),r||"function"==typeof w[d]||o(w,d,_))),S&&Y&&Y.name!==m&&(T=!0,C=function(){return Y.call(this)}),r&&!y||!f&&!T&&D[d]||o(D,d,C),s[t]=C,s[x]=_,g)if(M={values:S?C:L(m),keys:b?C:L(p),entries:E},y)for(k in M)k in D||a(D,k,M[k]);else i(i.P+i.F*(f||T),t,M);return M}},"02f4":function(e,t,n){var r=n("4588"),i=n("be13");e.exports=function(e){return function(t,n){var a,o,s=String(i(t)),l=r(n),u=s.length;return l<0||l>=u?e?"":void 0:(a=s.charCodeAt(l),a<55296||a>56319||l+1===u||(o=s.charCodeAt(l+1))<56320||o>57343?e?s.charAt(l):a:e?s.slice(l,l+2):o-56320+(a-55296<<10)+65536)}}},"0390":function(e,t,n){"use strict";var r=n("02f4")(!0);e.exports=function(e,t,n){return t+(n?r(e,t).length:1)}},"0bfb":function(e,t,n){"use strict";var r=n("cb7c");e.exports=function(){var e=r(this),t="";return e.global&&(t+="g"),e.ignoreCase&&(t+="i"),e.multiline&&(t+="m"),e.unicode&&(t+="u"),e.sticky&&(t+="y"),t}},"0d58":function(e,t,n){var r=n("ce10"),i=n("e11e");e.exports=Object.keys||function(e){return r(e,i)}},1495:function(e,t,n){var r=n("86cc"),i=n("cb7c"),a=n("0d58");e.exports=n("9e1e")?Object.defineProperties:function(e,t){i(e);var n,o=a(t),s=o.length,l=0;while(s>l)r.f(e,n=o[l++],t[n]);return e}},"214f":function(e,t,n){"use strict";n("b0c5");var r=n("2aba"),i=n("32e9"),a=n("79e5"),o=n("be13"),s=n("2b4c"),l=n("520a"),u=s("species"),c=!a((function(){var e=/./;return e.exec=function(){var e=[];return e.groups={a:"7"},e},"7"!=="".replace(e,"$<a>")})),d=function(){var e=/(?:)/,t=e.exec;e.exec=function(){return t.apply(this,arguments)};var n="ab".split(e);return 2===n.length&&"a"===n[0]&&"b"===n[1]}();e.exports=function(e,t,n){var f=s(e),h=!a((function(){var t={};return t[f]=function(){return 7},7!=""[e](t)})),p=h?!a((function(){var t=!1,n=/a/;return n.exec=function(){return t=!0,null},"split"===e&&(n.constructor={},n.constructor[u]=function(){return n}),n[f](""),!t})):void 0;if(!h||!p||"replace"===e&&!c||"split"===e&&!d){var m=/./[f],_=n(o,f,""[e],(function(e,t,n,r,i){return t.exec===l?h&&!i?{done:!0,value:m.call(t,n,r)}:{done:!0,value:e.call(n,t,r)}:{done:!1}})),v=_[0],g=_[1];r(String.prototype,e,v),i(RegExp.prototype,f,2==t?function(e,t){return g.call(e,this,t)}:function(e){return g.call(e,this)})}}},"230e":function(e,t,n){var r=n("d3f4"),i=n("7726").document,a=r(i)&&r(i.createElement);e.exports=function(e){return a?i.createElement(e):{}}},"23c6":function(e,t,n){var r=n("2d95"),i=n("2b4c")("toStringTag"),a="Arguments"==r(function(){return arguments}()),o=function(e,t){try{return e[t]}catch(n){}};e.exports=function(e){var t,n,s;return void 0===e?"Undefined":null===e?"Null":"string"==typeof(n=o(t=Object(e),i))?n:a?r(t):"Object"==(s=r(t))&&"function"==typeof t.callee?"Arguments":s}},2621:function(e,t){t.f=Object.getOwnPropertySymbols},"2aba":function(e,t,n){var r=n("7726"),i=n("32e9"),a=n("69a8"),o=n("ca5a")("src"),s=n("fa5b"),l="toString",u=(""+s).split(l);n("8378").inspectSource=function(e){return s.call(e)},(e.exports=function(e,t,n,s){var l="function"==typeof n;l&&(a(n,"name")||i(n,"name",t)),e[t]!==n&&(l&&(a(n,o)||i(n,o,e[t]?""+e[t]:u.join(String(t)))),e===r?e[t]=n:s?e[t]?e[t]=n:i(e,t,n):(delete e[t],i(e,t,n)))})(Function.prototype,l,(function(){return"function"==typeof this&&this[o]||s.call(this)}))},"2aeb":function(e,t,n){var r=n("cb7c"),i=n("1495"),a=n("e11e"),o=n("613b")("IE_PROTO"),s=function(){},l="prototype",u=function(){var e,t=n("230e")("iframe"),r=a.length,i="<",o=">";t.style.display="none",n("fab2").appendChild(t),t.src="javascript:",e=t.contentWindow.document,e.open(),e.write(i+"script"+o+"document.F=Object"+i+"/script"+o),e.close(),u=e.F;while(r--)delete u[l][a[r]];return u()};e.exports=Object.create||function(e,t){var n;return null!==e?(s[l]=r(e),n=new s,s[l]=null,n[o]=e):n=u(),void 0===t?n:i(n,t)}},"2b4c":function(e,t,n){var r=n("5537")("wks"),i=n("ca5a"),a=n("7726").Symbol,o="function"==typeof a,s=e.exports=function(e){return r[e]||(r[e]=o&&a[e]||(o?a:i)("Symbol."+e))};s.store=r},"2d00":function(e,t){e.exports=!1},"2d95":function(e,t){var n={}.toString;e.exports=function(e){return n.call(e).slice(8,-1)}},"2fdb":function(e,t,n){"use strict";var r=n("5ca1"),i=n("d2c8"),a="includes";r(r.P+r.F*n("5147")(a),"String",{includes:function(e){return!!~i(this,e,a).indexOf(e,arguments.length>1?arguments[1]:void 0)}})},"32e9":function(e,t,n){var r=n("86cc"),i=n("4630");e.exports=n("9e1e")?function(e,t,n){return r.f(e,t,i(1,n))}:function(e,t,n){return e[t]=n,e}},"38fd":function(e,t,n){var r=n("69a8"),i=n("4bf8"),a=n("613b")("IE_PROTO"),o=Object.prototype;e.exports=Object.getPrototypeOf||function(e){return e=i(e),r(e,a)?e[a]:"function"==typeof e.constructor&&e instanceof e.constructor?e.constructor.prototype:e instanceof Object?o:null}},"41a0":function(e,t,n){"use strict";var r=n("2aeb"),i=n("4630"),a=n("7f20"),o={};n("32e9")(o,n("2b4c")("iterator"),(function(){return this})),e.exports=function(e,t,n){e.prototype=r(o,{next:i(1,n)}),a(e,t+" Iterator")}},"456d":function(e,t,n){var r=n("4bf8"),i=n("0d58");n("5eda")("keys",(function(){return function(e){return i(r(e))}}))},4588:function(e,t){var n=Math.ceil,r=Math.floor;e.exports=function(e){return isNaN(e=+e)?0:(e>0?r:n)(e)}},4630:function(e,t){e.exports=function(e,t){return{enumerable:!(1&e),configurable:!(2&e),writable:!(4&e),value:t}}},"4bf8":function(e,t,n){var r=n("be13");e.exports=function(e){return Object(r(e))}},5147:function(e,t,n){var r=n("2b4c")("match");e.exports=function(e){var t=/./;try{"/./"[e](t)}catch(n){try{return t[r]=!1,!"/./"[e](t)}catch(i){}}return!0}},"520a":function(e,t,n){"use strict";var r=n("0bfb"),i=RegExp.prototype.exec,a=String.prototype.replace,o=i,s="lastIndex",l=function(){var e=/a/,t=/b*/g;return i.call(e,"a"),i.call(t,"a"),0!==e[s]||0!==t[s]}(),u=void 0!==/()??/.exec("")[1],c=l||u;c&&(o=function(e){var t,n,o,c,d=this;return u&&(n=new RegExp("^"+d.source+"$(?!\\s)",r.call(d))),l&&(t=d[s]),o=i.call(d,e),l&&o&&(d[s]=d.global?o.index+o[0].length:t),u&&o&&o.length>1&&a.call(o[0],n,(function(){for(c=1;c<arguments.length-2;c++)void 0===arguments[c]&&(o[c]=void 0)})),o}),e.exports=o},"52a7":function(e,t){t.f={}.propertyIsEnumerable},5537:function(e,t,n){var r=n("8378"),i=n("7726"),a="__core-js_shared__",o=i[a]||(i[a]={});(e.exports=function(e,t){return o[e]||(o[e]=void 0!==t?t:{})})("versions",[]).push({version:r.version,mode:n("2d00")?"pure":"global",copyright:"© 2019 Denis Pushkarev (zloirock.ru)"})},"5ca1":function(e,t,n){var r=n("7726"),i=n("8378"),a=n("32e9"),o=n("2aba"),s=n("9b43"),l="prototype",u=function(e,t,n){var c,d,f,h,p=e&u.F,m=e&u.G,_=e&u.S,v=e&u.P,g=e&u.B,b=m?r:_?r[t]||(r[t]={}):(r[t]||{})[l],y=m?i:i[t]||(i[t]={}),M=y[l]||(y[l]={});for(c in m&&(n=t),n)d=!p&&b&&void 0!==b[c],f=(d?b:n)[c],h=g&&d?s(f,r):v&&"function"==typeof f?s(Function.call,f):f,b&&o(b,c,f,e&u.U),y[c]!=f&&a(y,c,h),v&&M[c]!=f&&(M[c]=f)};r.core=i,u.F=1,u.G=2,u.S=4,u.P=8,u.B=16,u.W=32,u.U=64,u.R=128,e.exports=u},"5eda":function(e,t,n){var r=n("5ca1"),i=n("8378"),a=n("79e5");e.exports=function(e,t){var n=(i.Object||{})[e]||Object[e],o={};o[e]=t(n),r(r.S+r.F*a((function(){n(1)})),"Object",o)}},"5f1b":function(e,t,n){"use strict";var r=n("23c6"),i=RegExp.prototype.exec;e.exports=function(e,t){var n=e.exec;if("function"===typeof n){var a=n.call(e,t);if("object"!==typeof a)throw new TypeError("RegExp exec method returned something other than an Object or null");return a}if("RegExp"!==r(e))throw new TypeError("RegExp#exec called on incompatible receiver");return i.call(e,t)}},"613b":function(e,t,n){var r=n("5537")("keys"),i=n("ca5a");e.exports=function(e){return r[e]||(r[e]=i(e))}},"626a":function(e,t,n){var r=n("2d95");e.exports=Object("z").propertyIsEnumerable(0)?Object:function(e){return"String"==r(e)?e.split(""):Object(e)}},6762:function(e,t,n){"use strict";var r=n("5ca1"),i=n("c366")(!0);r(r.P,"Array",{includes:function(e){return i(this,e,arguments.length>1?arguments[1]:void 0)}}),n("9c6c")("includes")},6821:function(e,t,n){var r=n("626a"),i=n("be13");e.exports=function(e){return r(i(e))}},"69a8":function(e,t){var n={}.hasOwnProperty;e.exports=function(e,t){return n.call(e,t)}},"6a99":function(e,t,n){var r=n("d3f4");e.exports=function(e,t){if(!r(e))return e;var n,i;if(t&&"function"==typeof(n=e.toString)&&!r(i=n.call(e)))return i;if("function"==typeof(n=e.valueOf)&&!r(i=n.call(e)))return i;if(!t&&"function"==typeof(n=e.toString)&&!r(i=n.call(e)))return i;throw TypeError("Can't convert object to primitive value")}},7333:function(e,t,n){"use strict";var r=n("0d58"),i=n("2621"),a=n("52a7"),o=n("4bf8"),s=n("626a"),l=Object.assign;e.exports=!l||n("79e5")((function(){var e={},t={},n=Symbol(),r="abcdefghijklmnopqrst";return e[n]=7,r.split("").forEach((function(e){t[e]=e})),7!=l({},e)[n]||Object.keys(l({},t)).join("")!=r}))?function(e,t){var n=o(e),l=arguments.length,u=1,c=i.f,d=a.f;while(l>u){var f,h=s(arguments[u++]),p=c?r(h).concat(c(h)):r(h),m=p.length,_=0;while(m>_)d.call(h,f=p[_++])&&(n[f]=h[f])}return n}:l},7726:function(e,t){var n=e.exports="undefined"!=typeof window&&window.Math==Math?window:"undefined"!=typeof self&&self.Math==Math?self:Function("return this")();"number"==typeof __g&&(__g=n)},"77f1":function(e,t,n){var r=n("4588"),i=Math.max,a=Math.min;e.exports=function(e,t){return e=r(e),e<0?i(e+t,0):a(e,t)}},"79e5":function(e,t){e.exports=function(e){try{return!!e()}catch(t){return!0}}},"7f20":function(e,t,n){var r=n("86cc").f,i=n("69a8"),a=n("2b4c")("toStringTag");e.exports=function(e,t,n){e&&!i(e=n?e:e.prototype,a)&&r(e,a,{configurable:!0,value:t})}},8378:function(e,t){var n=e.exports={version:"2.6.5"};"number"==typeof __e&&(__e=n)},"84f2":function(e,t){e.exports={}},"86cc":function(e,t,n){var r=n("cb7c"),i=n("c69a"),a=n("6a99"),o=Object.defineProperty;t.f=n("9e1e")?Object.defineProperty:function(e,t,n){if(r(e),t=a(t,!0),r(n),i)try{return o(e,t,n)}catch(s){}if("get"in n||"set"in n)throw TypeError("Accessors not supported!");return"value"in n&&(e[t]=n.value),e}},"9b43":function(e,t,n){var r=n("d8e8");e.exports=function(e,t,n){if(r(e),void 0===t)return e;switch(n){case 1:return function(n){return e.call(t,n)};case 2:return function(n,r){return e.call(t,n,r)};case 3:return function(n,r,i){return e.call(t,n,r,i)}}return function(){return e.apply(t,arguments)}}},"9c6c":function(e,t,n){var r=n("2b4c")("unscopables"),i=Array.prototype;void 0==i[r]&&n("32e9")(i,r,{}),e.exports=function(e){i[r][e]=!0}},"9def":function(e,t,n){var r=n("4588"),i=Math.min;e.exports=function(e){return e>0?i(r(e),9007199254740991):0}},"9e1e":function(e,t,n){e.exports=!n("79e5")((function(){return 7!=Object.defineProperty({},"a",{get:function(){return 7}}).a}))},a352:function(t,n){t.exports=e},a481:function(e,t,n){"use strict";var r=n("cb7c"),i=n("4bf8"),a=n("9def"),o=n("4588"),s=n("0390"),l=n("5f1b"),u=Math.max,c=Math.min,d=Math.floor,f=/\$([$&`']|\d\d?|<[^>]*>)/g,h=/\$([$&`']|\d\d?)/g,p=function(e){return void 0===e?e:String(e)};n("214f")("replace",2,(function(e,t,n,m){return[function(r,i){var a=e(this),o=void 0==r?void 0:r[t];return void 0!==o?o.call(r,a,i):n.call(String(a),r,i)},function(e,t){var i=m(n,e,this,t);if(i.done)return i.value;var d=r(e),f=String(this),h="function"===typeof t;h||(t=String(t));var v=d.global;if(v){var g=d.unicode;d.lastIndex=0}var b=[];while(1){var y=l(d,f);if(null===y)break;if(b.push(y),!v)break;var M=String(y[0]);""===M&&(d.lastIndex=s(f,a(d.lastIndex),g))}for(var k="",w=0,L=0;L<b.length;L++){y=b[L];for(var x=String(y[0]),S=u(c(o(y.index),f.length),0),T=[],D=1;D<y.length;D++)T.push(p(y[D]));var Y=y.groups;if(h){var C=[x].concat(T,S,f);void 0!==Y&&C.push(Y);var E=String(t.apply(void 0,C))}else E=_(x,f,S,T,Y,t);S>=w&&(k+=f.slice(w,S)+E,w=S+x.length)}return k+f.slice(w)}];function _(e,t,r,a,o,s){var l=r+e.length,u=a.length,c=h;return void 0!==o&&(o=i(o),c=f),n.call(s,c,(function(n,i){var s;switch(i.charAt(0)){case"$":return"$";case"&":return e;case"`":return t.slice(0,r);case"'":return t.slice(l);case"<":s=o[i.slice(1,-1)];break;default:var c=+i;if(0===c)return n;if(c>u){var f=d(c/10);return 0===f?n:f<=u?void 0===a[f-1]?i.charAt(1):a[f-1]+i.charAt(1):n}s=a[c-1]}return void 0===s?"":s}))}}))},aae3:function(e,t,n){var r=n("d3f4"),i=n("2d95"),a=n("2b4c")("match");e.exports=function(e){var t;return r(e)&&(void 0!==(t=e[a])?!!t:"RegExp"==i(e))}},ac6a:function(e,t,n){for(var r=n("cadf"),i=n("0d58"),a=n("2aba"),o=n("7726"),s=n("32e9"),l=n("84f2"),u=n("2b4c"),c=u("iterator"),d=u("toStringTag"),f=l.Array,h={CSSRuleList:!0,CSSStyleDeclaration:!1,CSSValueList:!1,ClientRectList:!1,DOMRectList:!1,DOMStringList:!1,DOMTokenList:!0,DataTransferItemList:!1,FileList:!1,HTMLAllCollection:!1,HTMLCollection:!1,HTMLFormElement:!1,HTMLSelectElement:!1,MediaList:!0,MimeTypeArray:!1,NamedNodeMap:!1,NodeList:!0,PaintRequestList:!1,Plugin:!1,PluginArray:!1,SVGLengthList:!1,SVGNumberList:!1,SVGPathSegList:!1,SVGPointList:!1,SVGStringList:!1,SVGTransformList:!1,SourceBufferList:!1,StyleSheetList:!0,TextTrackCueList:!1,TextTrackList:!1,TouchList:!1},p=i(h),m=0;m<p.length;m++){var _,v=p[m],g=h[v],b=o[v],y=b&&b.prototype;if(y&&(y[c]||s(y,c,f),y[d]||s(y,d,v),l[v]=f,g))for(_ in r)y[_]||a(y,_,r[_],!0)}},b0c5:function(e,t,n){"use strict";var r=n("520a");n("5ca1")({target:"RegExp",proto:!0,forced:r!==/./.exec},{exec:r})},be13:function(e,t){e.exports=function(e){if(void 0==e)throw TypeError("Can't call method on  "+e);return e}},c366:function(e,t,n){var r=n("6821"),i=n("9def"),a=n("77f1");e.exports=function(e){return function(t,n,o){var s,l=r(t),u=i(l.length),c=a(o,u);if(e&&n!=n){while(u>c)if(s=l[c++],s!=s)return!0}else for(;u>c;c++)if((e||c in l)&&l[c]===n)return e||c||0;return!e&&-1}}},c649:function(e,t,n){"use strict";(function(e){n.d(t,"c",(function(){return u})),n.d(t,"a",(function(){return s})),n.d(t,"b",(function(){return i})),n.d(t,"d",(function(){return l}));n("a481");function r(){return"undefined"!==typeof window?window.console:e.console}var i=r();function a(e){var t=Object.create(null);return function(n){var r=t[n];return r||(t[n]=e(n))}}var o=/-(\w)/g,s=a((function(e){return e.replace(o,(function(e,t){return t?t.toUpperCase():""}))}));function l(e){null!==e.parentElement&&e.parentElement.removeChild(e)}function u(e,t,n){var r=0===n?e.children[0]:e.children[n-1].nextSibling;e.insertBefore(t,r)}}).call(this,n("c8ba"))},c69a:function(e,t,n){e.exports=!n("9e1e")&&!n("79e5")((function(){return 7!=Object.defineProperty(n("230e")("div"),"a",{get:function(){return 7}}).a}))},c8ba:function(e,t){var n;n=function(){return this}();try{n=n||new Function("return this")()}catch(r){"object"===typeof window&&(n=window)}e.exports=n},ca5a:function(e,t){var n=0,r=Math.random();e.exports=function(e){return"Symbol(".concat(void 0===e?"":e,")_",(++n+r).toString(36))}},cadf:function(e,t,n){"use strict";var r=n("9c6c"),i=n("d53b"),a=n("84f2"),o=n("6821");e.exports=n("01f9")(Array,"Array",(function(e,t){this._t=o(e),this._i=0,this._k=t}),(function(){var e=this._t,t=this._k,n=this._i++;return!e||n>=e.length?(this._t=void 0,i(1)):i(0,"keys"==t?n:"values"==t?e[n]:[n,e[n]])}),"values"),a.Arguments=a.Array,r("keys"),r("values"),r("entries")},cb7c:function(e,t,n){var r=n("d3f4");e.exports=function(e){if(!r(e))throw TypeError(e+" is not an object!");return e}},ce10:function(e,t,n){var r=n("69a8"),i=n("6821"),a=n("c366")(!1),o=n("613b")("IE_PROTO");e.exports=function(e,t){var n,s=i(e),l=0,u=[];for(n in s)n!=o&&r(s,n)&&u.push(n);while(t.length>l)r(s,n=t[l++])&&(~a(u,n)||u.push(n));return u}},d2c8:function(e,t,n){var r=n("aae3"),i=n("be13");e.exports=function(e,t,n){if(r(t))throw TypeError("String#"+n+" doesn't accept regex!");return String(i(e))}},d3f4:function(e,t){e.exports=function(e){return"object"===typeof e?null!==e:"function"===typeof e}},d53b:function(e,t){e.exports=function(e,t){return{value:t,done:!!e}}},d8e8:function(e,t){e.exports=function(e){if("function"!=typeof e)throw TypeError(e+" is not a function!");return e}},e11e:function(e,t){e.exports="constructor,hasOwnProperty,isPrototypeOf,propertyIsEnumerable,toLocaleString,toString,valueOf".split(",")},f559:function(e,t,n){"use strict";var r=n("5ca1"),i=n("9def"),a=n("d2c8"),o="startsWith",s=""[o];r(r.P+r.F*n("5147")(o),"String",{startsWith:function(e){var t=a(this,e,o),n=i(Math.min(arguments.length>1?arguments[1]:void 0,t.length)),r=String(e);return s?s.call(t,r,n):t.slice(n,n+r.length)===r}})},f6fd:function(e,t){(function(e){var t="currentScript",n=e.getElementsByTagName("script");t in e||Object.defineProperty(e,t,{get:function(){try{throw new Error}catch(r){var e,t=(/.*at [^\(]*\((.*):.+:.+\)$/gi.exec(r.stack)||[!1])[1];for(e in n)if(n[e].src==t||"interactive"==n[e].readyState)return n[e];return null}}})})(document)},f751:function(e,t,n){var r=n("5ca1");r(r.S+r.F,"Object",{assign:n("7333")})},fa5b:function(e,t,n){e.exports=n("5537")("native-function-to-string",Function.toString)},fab2:function(e,t,n){var r=n("7726").document;e.exports=r&&r.documentElement},fb15:function(e,t,n){"use strict";var r;(n.r(t),"undefined"!==typeof window)&&(n("f6fd"),(r=window.document.currentScript)&&(r=r.src.match(/(.+\/)[^/]+\.js(\?.*)?$/))&&(n.p=r[1]));n("f751"),n("f559"),n("ac6a"),n("cadf"),n("456d");function i(e){if(Array.isArray(e))return e}function a(e,t){if("undefined"!==typeof Symbol&&Symbol.iterator in Object(e)){var n=[],r=!0,i=!1,a=void 0;try{for(var o,s=e[Symbol.iterator]();!(r=(o=s.next()).done);r=!0)if(n.push(o.value),t&&n.length===t)break}catch(l){i=!0,a=l}finally{try{r||null==s["return"]||s["return"]()}finally{if(i)throw a}}return n}}function o(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,r=new Array(t);n<t;n++)r[n]=e[n];return r}function s(e,t){if(e){if("string"===typeof e)return o(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);return"Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n?Array.from(e):"Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?o(e,t):void 0}}function l(){throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function u(e,t){return i(e)||a(e,t)||s(e,t)||l()}n("6762"),n("2fdb");function c(e){if(Array.isArray(e))return o(e)}function d(e){if("undefined"!==typeof Symbol&&Symbol.iterator in Object(e))return Array.from(e)}function f(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function h(e){return c(e)||d(e)||s(e)||f()}var p=n("a352"),m=n.n(p),_=n("c649");function v(e,t,n){return void 0===n||(e=e||{},e[t]=n),e}function g(e,t){return e.map((function(e){return e.elm})).indexOf(t)}function b(e,t,n,r){if(!e)return[];var i=e.map((function(e){return e.elm})),a=t.length-r,o=h(t).map((function(e,t){return t>=a?i.length:i.indexOf(e)}));return n?o.filter((function(e){return-1!==e})):o}function y(e,t){var n=this;this.$nextTick((function(){return n.$emit(e.toLowerCase(),t)}))}function M(e){var t=this;return function(n){null!==t.realList&&t["onDrag"+e](n),y.call(t,e,n)}}function k(e){return["transition-group","TransitionGroup"].includes(e)}function w(e){if(!e||1!==e.length)return!1;var t=u(e,1),n=t[0].componentOptions;return!!n&&k(n.tag)}function L(e,t,n){return e[n]||(t[n]?t[n]():void 0)}function x(e,t,n){var r=0,i=0,a=L(t,n,"header");a&&(r=a.length,e=e?[].concat(h(a),h(e)):h(a));var o=L(t,n,"footer");return o&&(i=o.length,e=e?[].concat(h(e),h(o)):h(o)),{children:e,headerOffset:r,footerOffset:i}}function S(e,t){var n=null,r=function(e,t){n=v(n,e,t)},i=Object.keys(e).filter((function(e){return"id"===e||e.startsWith("data-")})).reduce((function(t,n){return t[n]=e[n],t}),{});if(r("attrs",i),!t)return n;var a=t.on,o=t.props,s=t.attrs;return r("on",a),r("props",o),Object.assign(n.attrs,s),n}var T=["Start","Add","Remove","Update","End"],D=["Choose","Unchoose","Sort","Filter","Clone"],Y=["Move"].concat(T,D).map((function(e){return"on"+e})),C=null,E={options:Object,list:{type:Array,required:!1,default:null},value:{type:Array,required:!1,default:null},noTransitionOnDrag:{type:Boolean,default:!1},clone:{type:Function,default:function(e){return e}},element:{type:String,default:"div"},tag:{type:String,default:null},move:{type:Function,default:null},componentData:{type:Object,required:!1,default:null}},A={name:"draggable",inheritAttrs:!1,props:E,data:function(){return{transitionMode:!1,noneFunctionalComponentMode:!1}},render:function(e){var t=this.$slots.default;this.transitionMode=w(t);var n=x(t,this.$slots,this.$scopedSlots),r=n.children,i=n.headerOffset,a=n.footerOffset;this.headerOffset=i,this.footerOffset=a;var o=S(this.$attrs,this.componentData);return e(this.getTag(),o,r)},created:function(){null!==this.list&&null!==this.value&&_["b"].error("Value and list props are mutually exclusive! Please set one or another."),"div"!==this.element&&_["b"].warn("Element props is deprecated please use tag props instead. See https://github.com/SortableJS/Vue.Draggable/blob/master/documentation/migrate.md#element-props"),void 0!==this.options&&_["b"].warn("Options props is deprecated, add sortable options directly as vue.draggable item, or use v-bind. See https://github.com/SortableJS/Vue.Draggable/blob/master/documentation/migrate.md#options-props")},mounted:function(){var e=this;if(this.noneFunctionalComponentMode=this.getTag().toLowerCase()!==this.$el.nodeName.toLowerCase()&&!this.getIsFunctional(),this.noneFunctionalComponentMode&&this.transitionMode)throw new Error("Transition-group inside component is not supported. Please alter tag value or remove transition-group. Current tag value: ".concat(this.getTag()));var t={};T.forEach((function(n){t["on"+n]=M.call(e,n)})),D.forEach((function(n){t["on"+n]=y.bind(e,n)}));var n=Object.keys(this.$attrs).reduce((function(t,n){return t[Object(_["a"])(n)]=e.$attrs[n],t}),{}),r=Object.assign({},this.options,n,t,{onMove:function(t,n){return e.onDragMove(t,n)}});!("draggable"in r)&&(r.draggable=">*"),this._sortable=new m.a(this.rootContainer,r),this.computeIndexes()},beforeDestroy:function(){void 0!==this._sortable&&this._sortable.destroy()},computed:{rootContainer:function(){return this.transitionMode?this.$el.children[0]:this.$el},realList:function(){return this.list?this.list:this.value}},watch:{options:{handler:function(e){this.updateOptions(e)},deep:!0},$attrs:{handler:function(e){this.updateOptions(e)},deep:!0},realList:function(){this.computeIndexes()}},methods:{getIsFunctional:function(){var e=this._vnode.fnOptions;return e&&e.functional},getTag:function(){return this.tag||this.element},updateOptions:function(e){for(var t in e){var n=Object(_["a"])(t);-1===Y.indexOf(n)&&this._sortable.option(n,e[t])}},getChildrenNodes:function(){if(this.noneFunctionalComponentMode)return this.$children[0].$slots.default;var e=this.$slots.default;return this.transitionMode?e[0].child.$slots.default:e},computeIndexes:function(){var e=this;this.$nextTick((function(){e.visibleIndexes=b(e.getChildrenNodes(),e.rootContainer.children,e.transitionMode,e.footerOffset)}))},getUnderlyingVm:function(e){var t=g(this.getChildrenNodes()||[],e);if(-1===t)return null;var n=this.realList[t];return{index:t,element:n}},getUnderlyingPotencialDraggableComponent:function(e){var t=e.__vue__;return t&&t.$options&&k(t.$options._componentTag)?t.$parent:!("realList"in t)&&1===t.$children.length&&"realList"in t.$children[0]?t.$children[0]:t},emitChanges:function(e){var t=this;this.$nextTick((function(){t.$emit("change",e)}))},alterList:function(e){if(this.list)e(this.list);else{var t=h(this.value);e(t),this.$emit("input",t)}},spliceList:function(){var e=arguments,t=function(t){return t.splice.apply(t,h(e))};this.alterList(t)},updatePosition:function(e,t){var n=function(n){return n.splice(t,0,n.splice(e,1)[0])};this.alterList(n)},getRelatedContextFromMoveEvent:function(e){var t=e.to,n=e.related,r=this.getUnderlyingPotencialDraggableComponent(t);if(!r)return{component:r};var i=r.realList,a={list:i,component:r};if(t!==n&&i&&r.getUnderlyingVm){var o=r.getUnderlyingVm(n);if(o)return Object.assign(o,a)}return a},getVmIndex:function(e){var t=this.visibleIndexes,n=t.length;return e>n-1?n:t[e]},getComponent:function(){return this.$slots.default[0].componentInstance},resetTransitionData:function(e){if(this.noTransitionOnDrag&&this.transitionMode){var t=this.getChildrenNodes();t[e].data=null;var n=this.getComponent();n.children=[],n.kept=void 0}},onDragStart:function(e){this.context=this.getUnderlyingVm(e.item),e.item._underlying_vm_=this.clone(this.context.element),C=e.item},onDragAdd:function(e){var t=e.item._underlying_vm_;if(void 0!==t){Object(_["d"])(e.item);var n=this.getVmIndex(e.newIndex);this.spliceList(n,0,t),this.computeIndexes();var r={element:t,newIndex:n};this.emitChanges({added:r})}},onDragRemove:function(e){if(Object(_["c"])(this.rootContainer,e.item,e.oldIndex),"clone"!==e.pullMode){var t=this.context.index;this.spliceList(t,1);var n={element:this.context.element,oldIndex:t};this.resetTransitionData(t),this.emitChanges({removed:n})}else Object(_["d"])(e.clone)},onDragUpdate:function(e){Object(_["d"])(e.item),Object(_["c"])(e.from,e.item,e.oldIndex);var t=this.context.index,n=this.getVmIndex(e.newIndex);this.updatePosition(t,n);var r={element:this.context.element,oldIndex:t,newIndex:n};this.emitChanges({moved:r})},updateProperty:function(e,t){e.hasOwnProperty(t)&&(e[t]+=this.headerOffset)},computeFutureIndex:function(e,t){if(!e.element)return 0;var n=h(t.to.children).filter((function(e){return"none"!==e.style["display"]})),r=n.indexOf(t.related),i=e.component.getVmIndex(r),a=-1!==n.indexOf(C);return a||!t.willInsertAfter?i:i+1},onDragMove:function(e,t){var n=this.move;if(!n||!this.realList)return!0;var r=this.getRelatedContextFromMoveEvent(e),i=this.context,a=this.computeFutureIndex(r,e);Object.assign(i,{futureIndex:a});var o=Object.assign({},e,{relatedContext:r,draggedContext:i});return n(o,t)},onDragEnd:function(){this.computeIndexes(),C=null}}};"undefined"!==typeof window&&"Vue"in window&&window.Vue.component("draggable",A);var O=A;t["default"]=O}})["default"]}))},b7e9:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("en-sg",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(e){var t=e%10,n=1===~~(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th";return e+n},week:{dow:1,doy:4}});return t}))},b84c:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("nn",{months:"januar_februar_mars_april_mai_juni_juli_august_september_oktober_november_desember".split("_"),monthsShort:"jan._feb._mars_apr._mai_juni_juli_aug._sep._okt._nov._des.".split("_"),monthsParseExact:!0,weekdays:"sundag_måndag_tysdag_onsdag_torsdag_fredag_laurdag".split("_"),weekdaysShort:"su._må._ty._on._to._fr._lau.".split("_"),weekdaysMin:"su_må_ty_on_to_fr_la".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY [kl.] H:mm",LLLL:"dddd D. MMMM YYYY [kl.] HH:mm"},calendar:{sameDay:"[I dag klokka] LT",nextDay:"[I morgon klokka] LT",nextWeek:"dddd [klokka] LT",lastDay:"[I går klokka] LT",lastWeek:"[Føregåande] dddd [klokka] LT",sameElse:"L"},relativeTime:{future:"om %s",past:"%s sidan",s:"nokre sekund",ss:"%d sekund",m:"eit minutt",mm:"%d minutt",h:"ein time",hh:"%d timar",d:"ein dag",dd:"%d dagar",w:"ei veke",ww:"%d veker",M:"ein månad",MM:"%d månader",y:"eit år",yy:"%d år"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return t}))},b97c:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={ss:"sekundes_sekundēm_sekunde_sekundes".split("_"),m:"minūtes_minūtēm_minūte_minūtes".split("_"),mm:"minūtes_minūtēm_minūte_minūtes".split("_"),h:"stundas_stundām_stunda_stundas".split("_"),hh:"stundas_stundām_stunda_stundas".split("_"),d:"dienas_dienām_diena_dienas".split("_"),dd:"dienas_dienām_diena_dienas".split("_"),M:"mēneša_mēnešiem_mēnesis_mēneši".split("_"),MM:"mēneša_mēnešiem_mēnesis_mēneši".split("_"),y:"gada_gadiem_gads_gadi".split("_"),yy:"gada_gadiem_gads_gadi".split("_")};function n(e,t,n){return n?t%10===1&&t%100!==11?e[2]:e[3]:t%10===1&&t%100!==11?e[0]:e[1]}function r(e,r,i){return e+" "+n(t[i],e,r)}function i(e,r,i){return n(t[i],e,r)}function a(e,t){return t?"dažas sekundes":"dažām sekundēm"}var o=e.defineLocale("lv",{months:"janvāris_februāris_marts_aprīlis_maijs_jūnijs_jūlijs_augusts_septembris_oktobris_novembris_decembris".split("_"),monthsShort:"jan_feb_mar_apr_mai_jūn_jūl_aug_sep_okt_nov_dec".split("_"),weekdays:"svētdiena_pirmdiena_otrdiena_trešdiena_ceturtdiena_piektdiena_sestdiena".split("_"),weekdaysShort:"Sv_P_O_T_C_Pk_S".split("_"),weekdaysMin:"Sv_P_O_T_C_Pk_S".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY.",LL:"YYYY. [gada] D. MMMM",LLL:"YYYY. [gada] D. MMMM, HH:mm",LLLL:"YYYY. [gada] D. MMMM, dddd, HH:mm"},calendar:{sameDay:"[Šodien pulksten] LT",nextDay:"[Rīt pulksten] LT",nextWeek:"dddd [pulksten] LT",lastDay:"[Vakar pulksten] LT",lastWeek:"[Pagājušā] dddd [pulksten] LT",sameElse:"L"},relativeTime:{future:"pēc %s",past:"pirms %s",s:a,ss:r,m:i,mm:r,h:i,hh:r,d:i,dd:r,M:i,MM:r,y:i,yy:r},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return o}))},bb2f:function(e,t,n){var r=n("d039");e.exports=!r((function(){return Object.isExtensible(Object.preventExtensions({}))}))},bb71:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e,t,n,r){var i={m:["eine Minute","einer Minute"],h:["eine Stunde","einer Stunde"],d:["ein Tag","einem Tag"],dd:[e+" Tage",e+" Tagen"],w:["eine Woche","einer Woche"],M:["ein Monat","einem Monat"],MM:[e+" Monate",e+" Monaten"],y:["ein Jahr","einem Jahr"],yy:[e+" Jahre",e+" Jahren"]};return t?i[n][0]:i[n][1]}var n=e.defineLocale("de-ch",{months:"Januar_Februar_März_April_Mai_Juni_Juli_August_September_Oktober_November_Dezember".split("_"),monthsShort:"Jan._Feb._März_Apr._Mai_Juni_Juli_Aug._Sep._Okt._Nov._Dez.".split("_"),monthsParseExact:!0,weekdays:"Sonntag_Montag_Dienstag_Mittwoch_Donnerstag_Freitag_Samstag".split("_"),weekdaysShort:"So_Mo_Di_Mi_Do_Fr_Sa".split("_"),weekdaysMin:"So_Mo_Di_Mi_Do_Fr_Sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY HH:mm",LLLL:"dddd, D. MMMM YYYY HH:mm"},calendar:{sameDay:"[heute um] LT [Uhr]",sameElse:"L",nextDay:"[morgen um] LT [Uhr]",nextWeek:"dddd [um] LT [Uhr]",lastDay:"[gestern um] LT [Uhr]",lastWeek:"[letzten] dddd [um] LT [Uhr]"},relativeTime:{future:"in %s",past:"vor %s",s:"ein paar Sekunden",ss:"%d Sekunden",m:t,mm:"%d Minuten",h:t,hh:"%d Stunden",d:t,dd:t,w:t,ww:"%d Wochen",M:t,MM:t,y:t,yy:t},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return n}))},c04e:function(e,t,n){var r=n("861d");e.exports=function(e,t){if(!r(e))return e;var n,i;if(t&&"function"==typeof(n=e.toString)&&!r(i=n.call(e)))return i;if("function"==typeof(n=e.valueOf)&&!r(i=n.call(e)))return i;if(!t&&"function"==typeof(n=e.toString)&&!r(i=n.call(e)))return i;throw TypeError("Can't convert object to primitive value")}},c109:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("tzm",{months:"ⵉⵏⵏⴰⵢⵔ_ⴱⵕⴰⵢⵕ_ⵎⴰⵕⵚ_ⵉⴱⵔⵉⵔ_ⵎⴰⵢⵢⵓ_ⵢⵓⵏⵢⵓ_ⵢⵓⵍⵢⵓⵣ_ⵖⵓⵛⵜ_ⵛⵓⵜⴰⵏⴱⵉⵔ_ⴽⵟⵓⴱⵕ_ⵏⵓⵡⴰⵏⴱⵉⵔ_ⴷⵓⵊⵏⴱⵉⵔ".split("_"),monthsShort:"ⵉⵏⵏⴰⵢⵔ_ⴱⵕⴰⵢⵕ_ⵎⴰⵕⵚ_ⵉⴱⵔⵉⵔ_ⵎⴰⵢⵢⵓ_ⵢⵓⵏⵢⵓ_ⵢⵓⵍⵢⵓⵣ_ⵖⵓⵛⵜ_ⵛⵓⵜⴰⵏⴱⵉⵔ_ⴽⵟⵓⴱⵕ_ⵏⵓⵡⴰⵏⴱⵉⵔ_ⴷⵓⵊⵏⴱⵉⵔ".split("_"),weekdays:"ⴰⵙⴰⵎⴰⵙ_ⴰⵢⵏⴰⵙ_ⴰⵙⵉⵏⴰⵙ_ⴰⴽⵔⴰⵙ_ⴰⴽⵡⴰⵙ_ⴰⵙⵉⵎⵡⴰⵙ_ⴰⵙⵉⴹⵢⴰⵙ".split("_"),weekdaysShort:"ⴰⵙⴰⵎⴰⵙ_ⴰⵢⵏⴰⵙ_ⴰⵙⵉⵏⴰⵙ_ⴰⴽⵔⴰⵙ_ⴰⴽⵡⴰⵙ_ⴰⵙⵉⵎⵡⴰⵙ_ⴰⵙⵉⴹⵢⴰⵙ".split("_"),weekdaysMin:"ⴰⵙⴰⵎⴰⵙ_ⴰⵢⵏⴰⵙ_ⴰⵙⵉⵏⴰⵙ_ⴰⴽⵔⴰⵙ_ⴰⴽⵡⴰⵙ_ⴰⵙⵉⵎⵡⴰⵙ_ⴰⵙⵉⴹⵢⴰⵙ".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[ⴰⵙⴷⵅ ⴴ] LT",nextDay:"[ⴰⵙⴽⴰ ⴴ] LT",nextWeek:"dddd [ⴴ] LT",lastDay:"[ⴰⵚⴰⵏⵜ ⴴ] LT",lastWeek:"dddd [ⴴ] LT",sameElse:"L"},relativeTime:{future:"ⴷⴰⴷⵅ ⵙ ⵢⴰⵏ %s",past:"ⵢⴰⵏ %s",s:"ⵉⵎⵉⴽ",ss:"%d ⵉⵎⵉⴽ",m:"ⵎⵉⵏⵓⴺ",mm:"%d ⵎⵉⵏⵓⴺ",h:"ⵙⴰⵄⴰ",hh:"%d ⵜⴰⵙⵙⴰⵄⵉⵏ",d:"ⴰⵙⵙ",dd:"%d oⵙⵙⴰⵏ",M:"ⴰⵢoⵓⵔ",MM:"%d ⵉⵢⵢⵉⵔⵏ",y:"ⴰⵙⴳⴰⵙ",yy:"%d ⵉⵙⴳⴰⵙⵏ"},week:{dow:6,doy:12}});return t}))},c19f:function(e,t,n){"use strict";var r=n("23e7"),i=n("da84"),a=n("621a"),o=n("2626"),s="ArrayBuffer",l=a[s],u=i[s];r({global:!0,forced:u!==l},{ArrayBuffer:l}),o(s)},c1ac:function(e,t,n){"use strict";var r=n("ebb5"),i=n("b727").filter,a=n("4840"),o=r.aTypedArray,s=r.aTypedArrayConstructor,l=r.exportTypedArrayMethod;l("filter",(function(e){var t=i(o(this),e,arguments.length>1?arguments[1]:void 0),n=a(this,this.constructor),r=0,l=t.length,u=new(s(n))(l);while(l>r)u[r]=t[r++];return u}))},c1df:function(e,t,n){(function(e){var t;//! moment.js
//! version : 2.29.1
//! authors : Tim Wood, Iskren Chernev, Moment.js contributors
//! license : MIT
//! momentjs.com
(function(t,n){e.exports=n()})(0,(function(){"use strict";var r,i;function a(){return r.apply(null,arguments)}function o(e){r=e}function s(e){return e instanceof Array||"[object Array]"===Object.prototype.toString.call(e)}function l(e){return null!=e&&"[object Object]"===Object.prototype.toString.call(e)}function u(e,t){return Object.prototype.hasOwnProperty.call(e,t)}function c(e){if(Object.getOwnPropertyNames)return 0===Object.getOwnPropertyNames(e).length;var t;for(t in e)if(u(e,t))return!1;return!0}function d(e){return void 0===e}function f(e){return"number"===typeof e||"[object Number]"===Object.prototype.toString.call(e)}function h(e){return e instanceof Date||"[object Date]"===Object.prototype.toString.call(e)}function p(e,t){var n,r=[];for(n=0;n<e.length;++n)r.push(t(e[n],n));return r}function m(e,t){for(var n in t)u(t,n)&&(e[n]=t[n]);return u(t,"toString")&&(e.toString=t.toString),u(t,"valueOf")&&(e.valueOf=t.valueOf),e}function _(e,t,n,r){return Jn(e,t,n,r,!0).utc()}function v(){return{empty:!1,unusedTokens:[],unusedInput:[],overflow:-2,charsLeftOver:0,nullInput:!1,invalidEra:null,invalidMonth:null,invalidFormat:!1,userInvalidated:!1,iso:!1,parsedDateParts:[],era:null,meridiem:null,rfc2822:!1,weekdayMismatch:!1}}function g(e){return null==e._pf&&(e._pf=v()),e._pf}function b(e){if(null==e._isValid){var t=g(e),n=i.call(t.parsedDateParts,(function(e){return null!=e})),r=!isNaN(e._d.getTime())&&t.overflow<0&&!t.empty&&!t.invalidEra&&!t.invalidMonth&&!t.invalidWeekday&&!t.weekdayMismatch&&!t.nullInput&&!t.invalidFormat&&!t.userInvalidated&&(!t.meridiem||t.meridiem&&n);if(e._strict&&(r=r&&0===t.charsLeftOver&&0===t.unusedTokens.length&&void 0===t.bigHour),null!=Object.isFrozen&&Object.isFrozen(e))return r;e._isValid=r}return e._isValid}function y(e){var t=_(NaN);return null!=e?m(g(t),e):g(t).userInvalidated=!0,t}i=Array.prototype.some?Array.prototype.some:function(e){var t,n=Object(this),r=n.length>>>0;for(t=0;t<r;t++)if(t in n&&e.call(this,n[t],t,n))return!0;return!1};var M=a.momentProperties=[],k=!1;function w(e,t){var n,r,i;if(d(t._isAMomentObject)||(e._isAMomentObject=t._isAMomentObject),d(t._i)||(e._i=t._i),d(t._f)||(e._f=t._f),d(t._l)||(e._l=t._l),d(t._strict)||(e._strict=t._strict),d(t._tzm)||(e._tzm=t._tzm),d(t._isUTC)||(e._isUTC=t._isUTC),d(t._offset)||(e._offset=t._offset),d(t._pf)||(e._pf=g(t)),d(t._locale)||(e._locale=t._locale),M.length>0)for(n=0;n<M.length;n++)r=M[n],i=t[r],d(i)||(e[r]=i);return e}function L(e){w(this,e),this._d=new Date(null!=e._d?e._d.getTime():NaN),this.isValid()||(this._d=new Date(NaN)),!1===k&&(k=!0,a.updateOffset(this),k=!1)}function x(e){return e instanceof L||null!=e&&null!=e._isAMomentObject}function S(e){!1===a.suppressDeprecationWarnings&&"undefined"!==typeof console&&console.warn&&console.warn("Deprecation warning: "+e)}function T(e,t){var n=!0;return m((function(){if(null!=a.deprecationHandler&&a.deprecationHandler(null,e),n){var r,i,o,s=[];for(i=0;i<arguments.length;i++){if(r="","object"===typeof arguments[i]){for(o in r+="\n["+i+"] ",arguments[0])u(arguments[0],o)&&(r+=o+": "+arguments[0][o]+", ");r=r.slice(0,-2)}else r=arguments[i];s.push(r)}S(e+"\nArguments: "+Array.prototype.slice.call(s).join("")+"\n"+(new Error).stack),n=!1}return t.apply(this,arguments)}),t)}var D,Y={};function C(e,t){null!=a.deprecationHandler&&a.deprecationHandler(e,t),Y[e]||(S(t),Y[e]=!0)}function E(e){return"undefined"!==typeof Function&&e instanceof Function||"[object Function]"===Object.prototype.toString.call(e)}function A(e){var t,n;for(n in e)u(e,n)&&(t=e[n],E(t)?this[n]=t:this["_"+n]=t);this._config=e,this._dayOfMonthOrdinalParseLenient=new RegExp((this._dayOfMonthOrdinalParse.source||this._ordinalParse.source)+"|"+/\d{1,2}/.source)}function O(e,t){var n,r=m({},e);for(n in t)u(t,n)&&(l(e[n])&&l(t[n])?(r[n]={},m(r[n],e[n]),m(r[n],t[n])):null!=t[n]?r[n]=t[n]:delete r[n]);for(n in e)u(e,n)&&!u(t,n)&&l(e[n])&&(r[n]=m({},r[n]));return r}function P(e){null!=e&&this.set(e)}a.suppressDeprecationWarnings=!1,a.deprecationHandler=null,D=Object.keys?Object.keys:function(e){var t,n=[];for(t in e)u(e,t)&&n.push(t);return n};var F={sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"};function I(e,t,n){var r=this._calendar[e]||this._calendar["sameElse"];return E(r)?r.call(t,n):r}function j(e,t,n){var r=""+Math.abs(e),i=t-r.length,a=e>=0;return(a?n?"+":"":"-")+Math.pow(10,Math.max(0,i)).toString().substr(1)+r}var H=/(\[[^\[]*\])|(\\)?([Hh]mm(ss)?|Mo|MM?M?M?|Do|DDDo|DD?D?D?|ddd?d?|do?|w[o|w]?|W[o|W]?|Qo?|N{1,5}|YYYYYY|YYYYY|YYYY|YY|y{2,4}|yo?|gg(ggg?)?|GG(GGG?)?|e|E|a|A|hh?|HH?|kk?|mm?|ss?|S{1,9}|x|X|zz?|ZZ?|.)/g,R=/(\[[^\[]*\])|(\\)?(LTS|LT|LL?L?L?|l{1,4})/g,N={},$={};function z(e,t,n,r){var i=r;"string"===typeof r&&(i=function(){return this[r]()}),e&&($[e]=i),t&&($[t[0]]=function(){return j(i.apply(this,arguments),t[1],t[2])}),n&&($[n]=function(){return this.localeData().ordinal(i.apply(this,arguments),e)})}function B(e){return e.match(/\[[\s\S]/)?e.replace(/^\[|\]$/g,""):e.replace(/\\/g,"")}function W(e){var t,n,r=e.match(H);for(t=0,n=r.length;t<n;t++)$[r[t]]?r[t]=$[r[t]]:r[t]=B(r[t]);return function(t){var i,a="";for(i=0;i<n;i++)a+=E(r[i])?r[i].call(t,e):r[i];return a}}function U(e,t){return e.isValid()?(t=V(t,e.localeData()),N[t]=N[t]||W(t),N[t](e)):e.localeData().invalidDate()}function V(e,t){var n=5;function r(e){return t.longDateFormat(e)||e}R.lastIndex=0;while(n>=0&&R.test(e))e=e.replace(R,r),R.lastIndex=0,n-=1;return e}var q={LTS:"h:mm:ss A",LT:"h:mm A",L:"MM/DD/YYYY",LL:"MMMM D, YYYY",LLL:"MMMM D, YYYY h:mm A",LLLL:"dddd, MMMM D, YYYY h:mm A"};function G(e){var t=this._longDateFormat[e],n=this._longDateFormat[e.toUpperCase()];return t||!n?t:(this._longDateFormat[e]=n.match(H).map((function(e){return"MMMM"===e||"MM"===e||"DD"===e||"dddd"===e?e.slice(1):e})).join(""),this._longDateFormat[e])}var J="Invalid date";function K(){return this._invalidDate}var X="%d",Q=/\d{1,2}/;function Z(e){return this._ordinal.replace("%d",e)}var ee={future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",w:"a week",ww:"%d weeks",M:"a month",MM:"%d months",y:"a year",yy:"%d years"};function te(e,t,n,r){var i=this._relativeTime[n];return E(i)?i(e,t,n,r):i.replace(/%d/i,e)}function ne(e,t){var n=this._relativeTime[e>0?"future":"past"];return E(n)?n(t):n.replace(/%s/i,t)}var re={};function ie(e,t){var n=e.toLowerCase();re[n]=re[n+"s"]=re[t]=e}function ae(e){return"string"===typeof e?re[e]||re[e.toLowerCase()]:void 0}function oe(e){var t,n,r={};for(n in e)u(e,n)&&(t=ae(n),t&&(r[t]=e[n]));return r}var se={};function le(e,t){se[e]=t}function ue(e){var t,n=[];for(t in e)u(e,t)&&n.push({unit:t,priority:se[t]});return n.sort((function(e,t){return e.priority-t.priority})),n}function ce(e){return e%4===0&&e%100!==0||e%400===0}function de(e){return e<0?Math.ceil(e)||0:Math.floor(e)}function fe(e){var t=+e,n=0;return 0!==t&&isFinite(t)&&(n=de(t)),n}function he(e,t){return function(n){return null!=n?(me(this,e,n),a.updateOffset(this,t),this):pe(this,e)}}function pe(e,t){return e.isValid()?e._d["get"+(e._isUTC?"UTC":"")+t]():NaN}function me(e,t,n){e.isValid()&&!isNaN(n)&&("FullYear"===t&&ce(e.year())&&1===e.month()&&29===e.date()?(n=fe(n),e._d["set"+(e._isUTC?"UTC":"")+t](n,e.month(),tt(n,e.month()))):e._d["set"+(e._isUTC?"UTC":"")+t](n))}function _e(e){return e=ae(e),E(this[e])?this[e]():this}function ve(e,t){if("object"===typeof e){e=oe(e);var n,r=ue(e);for(n=0;n<r.length;n++)this[r[n].unit](e[r[n].unit])}else if(e=ae(e),E(this[e]))return this[e](t);return this}var ge,be=/\d/,ye=/\d\d/,Me=/\d{3}/,ke=/\d{4}/,we=/[+-]?\d{6}/,Le=/\d\d?/,xe=/\d\d\d\d?/,Se=/\d\d\d\d\d\d?/,Te=/\d{1,3}/,De=/\d{1,4}/,Ye=/[+-]?\d{1,6}/,Ce=/\d+/,Ee=/[+-]?\d+/,Ae=/Z|[+-]\d\d:?\d\d/gi,Oe=/Z|[+-]\d\d(?::?\d\d)?/gi,Pe=/[+-]?\d+(\.\d{1,3})?/,Fe=/[0-9]{0,256}['a-z\u00A0-\u05FF\u0700-\uD7FF\uF900-\uFDCF\uFDF0-\uFF07\uFF10-\uFFEF]{1,256}|[\u0600-\u06FF\/]{1,256}(\s*?[\u0600-\u06FF]{1,256}){1,2}/i;function Ie(e,t,n){ge[e]=E(t)?t:function(e,r){return e&&n?n:t}}function je(e,t){return u(ge,e)?ge[e](t._strict,t._locale):new RegExp(He(e))}function He(e){return Re(e.replace("\\","").replace(/\\(\[)|\\(\])|\[([^\]\[]*)\]|\\(.)/g,(function(e,t,n,r,i){return t||n||r||i})))}function Re(e){return e.replace(/[-\/\\^$*+?.()|[\]{}]/g,"\\$&")}ge={};var Ne={};function $e(e,t){var n,r=t;for("string"===typeof e&&(e=[e]),f(t)&&(r=function(e,n){n[t]=fe(e)}),n=0;n<e.length;n++)Ne[e[n]]=r}function ze(e,t){$e(e,(function(e,n,r,i){r._w=r._w||{},t(e,r._w,r,i)}))}function Be(e,t,n){null!=t&&u(Ne,e)&&Ne[e](t,n._a,n,e)}var We,Ue=0,Ve=1,qe=2,Ge=3,Je=4,Ke=5,Xe=6,Qe=7,Ze=8;function et(e,t){return(e%t+t)%t}function tt(e,t){if(isNaN(e)||isNaN(t))return NaN;var n=et(t,12);return e+=(t-n)/12,1===n?ce(e)?29:28:31-n%7%2}We=Array.prototype.indexOf?Array.prototype.indexOf:function(e){var t;for(t=0;t<this.length;++t)if(this[t]===e)return t;return-1},z("M",["MM",2],"Mo",(function(){return this.month()+1})),z("MMM",0,0,(function(e){return this.localeData().monthsShort(this,e)})),z("MMMM",0,0,(function(e){return this.localeData().months(this,e)})),ie("month","M"),le("month",8),Ie("M",Le),Ie("MM",Le,ye),Ie("MMM",(function(e,t){return t.monthsShortRegex(e)})),Ie("MMMM",(function(e,t){return t.monthsRegex(e)})),$e(["M","MM"],(function(e,t){t[Ve]=fe(e)-1})),$e(["MMM","MMMM"],(function(e,t,n,r){var i=n._locale.monthsParse(e,r,n._strict);null!=i?t[Ve]=i:g(n).invalidMonth=e}));var nt="January_February_March_April_May_June_July_August_September_October_November_December".split("_"),rt="Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),it=/D[oD]?(\[[^\[\]]*\]|\s)+MMMM?/,at=Fe,ot=Fe;function st(e,t){return e?s(this._months)?this._months[e.month()]:this._months[(this._months.isFormat||it).test(t)?"format":"standalone"][e.month()]:s(this._months)?this._months:this._months["standalone"]}function lt(e,t){return e?s(this._monthsShort)?this._monthsShort[e.month()]:this._monthsShort[it.test(t)?"format":"standalone"][e.month()]:s(this._monthsShort)?this._monthsShort:this._monthsShort["standalone"]}function ut(e,t,n){var r,i,a,o=e.toLocaleLowerCase();if(!this._monthsParse)for(this._monthsParse=[],this._longMonthsParse=[],this._shortMonthsParse=[],r=0;r<12;++r)a=_([2e3,r]),this._shortMonthsParse[r]=this.monthsShort(a,"").toLocaleLowerCase(),this._longMonthsParse[r]=this.months(a,"").toLocaleLowerCase();return n?"MMM"===t?(i=We.call(this._shortMonthsParse,o),-1!==i?i:null):(i=We.call(this._longMonthsParse,o),-1!==i?i:null):"MMM"===t?(i=We.call(this._shortMonthsParse,o),-1!==i?i:(i=We.call(this._longMonthsParse,o),-1!==i?i:null)):(i=We.call(this._longMonthsParse,o),-1!==i?i:(i=We.call(this._shortMonthsParse,o),-1!==i?i:null))}function ct(e,t,n){var r,i,a;if(this._monthsParseExact)return ut.call(this,e,t,n);for(this._monthsParse||(this._monthsParse=[],this._longMonthsParse=[],this._shortMonthsParse=[]),r=0;r<12;r++){if(i=_([2e3,r]),n&&!this._longMonthsParse[r]&&(this._longMonthsParse[r]=new RegExp("^"+this.months(i,"").replace(".","")+"$","i"),this._shortMonthsParse[r]=new RegExp("^"+this.monthsShort(i,"").replace(".","")+"$","i")),n||this._monthsParse[r]||(a="^"+this.months(i,"")+"|^"+this.monthsShort(i,""),this._monthsParse[r]=new RegExp(a.replace(".",""),"i")),n&&"MMMM"===t&&this._longMonthsParse[r].test(e))return r;if(n&&"MMM"===t&&this._shortMonthsParse[r].test(e))return r;if(!n&&this._monthsParse[r].test(e))return r}}function dt(e,t){var n;if(!e.isValid())return e;if("string"===typeof t)if(/^\d+$/.test(t))t=fe(t);else if(t=e.localeData().monthsParse(t),!f(t))return e;return n=Math.min(e.date(),tt(e.year(),t)),e._d["set"+(e._isUTC?"UTC":"")+"Month"](t,n),e}function ft(e){return null!=e?(dt(this,e),a.updateOffset(this,!0),this):pe(this,"Month")}function ht(){return tt(this.year(),this.month())}function pt(e){return this._monthsParseExact?(u(this,"_monthsRegex")||_t.call(this),e?this._monthsShortStrictRegex:this._monthsShortRegex):(u(this,"_monthsShortRegex")||(this._monthsShortRegex=at),this._monthsShortStrictRegex&&e?this._monthsShortStrictRegex:this._monthsShortRegex)}function mt(e){return this._monthsParseExact?(u(this,"_monthsRegex")||_t.call(this),e?this._monthsStrictRegex:this._monthsRegex):(u(this,"_monthsRegex")||(this._monthsRegex=ot),this._monthsStrictRegex&&e?this._monthsStrictRegex:this._monthsRegex)}function _t(){function e(e,t){return t.length-e.length}var t,n,r=[],i=[],a=[];for(t=0;t<12;t++)n=_([2e3,t]),r.push(this.monthsShort(n,"")),i.push(this.months(n,"")),a.push(this.months(n,"")),a.push(this.monthsShort(n,""));for(r.sort(e),i.sort(e),a.sort(e),t=0;t<12;t++)r[t]=Re(r[t]),i[t]=Re(i[t]);for(t=0;t<24;t++)a[t]=Re(a[t]);this._monthsRegex=new RegExp("^("+a.join("|")+")","i"),this._monthsShortRegex=this._monthsRegex,this._monthsStrictRegex=new RegExp("^("+i.join("|")+")","i"),this._monthsShortStrictRegex=new RegExp("^("+r.join("|")+")","i")}function vt(e){return ce(e)?366:365}z("Y",0,0,(function(){var e=this.year();return e<=9999?j(e,4):"+"+e})),z(0,["YY",2],0,(function(){return this.year()%100})),z(0,["YYYY",4],0,"year"),z(0,["YYYYY",5],0,"year"),z(0,["YYYYYY",6,!0],0,"year"),ie("year","y"),le("year",1),Ie("Y",Ee),Ie("YY",Le,ye),Ie("YYYY",De,ke),Ie("YYYYY",Ye,we),Ie("YYYYYY",Ye,we),$e(["YYYYY","YYYYYY"],Ue),$e("YYYY",(function(e,t){t[Ue]=2===e.length?a.parseTwoDigitYear(e):fe(e)})),$e("YY",(function(e,t){t[Ue]=a.parseTwoDigitYear(e)})),$e("Y",(function(e,t){t[Ue]=parseInt(e,10)})),a.parseTwoDigitYear=function(e){return fe(e)+(fe(e)>68?1900:2e3)};var gt=he("FullYear",!0);function bt(){return ce(this.year())}function yt(e,t,n,r,i,a,o){var s;return e<100&&e>=0?(s=new Date(e+400,t,n,r,i,a,o),isFinite(s.getFullYear())&&s.setFullYear(e)):s=new Date(e,t,n,r,i,a,o),s}function Mt(e){var t,n;return e<100&&e>=0?(n=Array.prototype.slice.call(arguments),n[0]=e+400,t=new Date(Date.UTC.apply(null,n)),isFinite(t.getUTCFullYear())&&t.setUTCFullYear(e)):t=new Date(Date.UTC.apply(null,arguments)),t}function kt(e,t,n){var r=7+t-n,i=(7+Mt(e,0,r).getUTCDay()-t)%7;return-i+r-1}function wt(e,t,n,r,i){var a,o,s=(7+n-r)%7,l=kt(e,r,i),u=1+7*(t-1)+s+l;return u<=0?(a=e-1,o=vt(a)+u):u>vt(e)?(a=e+1,o=u-vt(e)):(a=e,o=u),{year:a,dayOfYear:o}}function Lt(e,t,n){var r,i,a=kt(e.year(),t,n),o=Math.floor((e.dayOfYear()-a-1)/7)+1;return o<1?(i=e.year()-1,r=o+xt(i,t,n)):o>xt(e.year(),t,n)?(r=o-xt(e.year(),t,n),i=e.year()+1):(i=e.year(),r=o),{week:r,year:i}}function xt(e,t,n){var r=kt(e,t,n),i=kt(e+1,t,n);return(vt(e)-r+i)/7}function St(e){return Lt(e,this._week.dow,this._week.doy).week}z("w",["ww",2],"wo","week"),z("W",["WW",2],"Wo","isoWeek"),ie("week","w"),ie("isoWeek","W"),le("week",5),le("isoWeek",5),Ie("w",Le),Ie("ww",Le,ye),Ie("W",Le),Ie("WW",Le,ye),ze(["w","ww","W","WW"],(function(e,t,n,r){t[r.substr(0,1)]=fe(e)}));var Tt={dow:0,doy:6};function Dt(){return this._week.dow}function Yt(){return this._week.doy}function Ct(e){var t=this.localeData().week(this);return null==e?t:this.add(7*(e-t),"d")}function Et(e){var t=Lt(this,1,4).week;return null==e?t:this.add(7*(e-t),"d")}function At(e,t){return"string"!==typeof e?e:isNaN(e)?(e=t.weekdaysParse(e),"number"===typeof e?e:null):parseInt(e,10)}function Ot(e,t){return"string"===typeof e?t.weekdaysParse(e)%7||7:isNaN(e)?null:e}function Pt(e,t){return e.slice(t,7).concat(e.slice(0,t))}z("d",0,"do","day"),z("dd",0,0,(function(e){return this.localeData().weekdaysMin(this,e)})),z("ddd",0,0,(function(e){return this.localeData().weekdaysShort(this,e)})),z("dddd",0,0,(function(e){return this.localeData().weekdays(this,e)})),z("e",0,0,"weekday"),z("E",0,0,"isoWeekday"),ie("day","d"),ie("weekday","e"),ie("isoWeekday","E"),le("day",11),le("weekday",11),le("isoWeekday",11),Ie("d",Le),Ie("e",Le),Ie("E",Le),Ie("dd",(function(e,t){return t.weekdaysMinRegex(e)})),Ie("ddd",(function(e,t){return t.weekdaysShortRegex(e)})),Ie("dddd",(function(e,t){return t.weekdaysRegex(e)})),ze(["dd","ddd","dddd"],(function(e,t,n,r){var i=n._locale.weekdaysParse(e,r,n._strict);null!=i?t.d=i:g(n).invalidWeekday=e})),ze(["d","e","E"],(function(e,t,n,r){t[r]=fe(e)}));var Ft="Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),It="Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),jt="Su_Mo_Tu_We_Th_Fr_Sa".split("_"),Ht=Fe,Rt=Fe,Nt=Fe;function $t(e,t){var n=s(this._weekdays)?this._weekdays:this._weekdays[e&&!0!==e&&this._weekdays.isFormat.test(t)?"format":"standalone"];return!0===e?Pt(n,this._week.dow):e?n[e.day()]:n}function zt(e){return!0===e?Pt(this._weekdaysShort,this._week.dow):e?this._weekdaysShort[e.day()]:this._weekdaysShort}function Bt(e){return!0===e?Pt(this._weekdaysMin,this._week.dow):e?this._weekdaysMin[e.day()]:this._weekdaysMin}function Wt(e,t,n){var r,i,a,o=e.toLocaleLowerCase();if(!this._weekdaysParse)for(this._weekdaysParse=[],this._shortWeekdaysParse=[],this._minWeekdaysParse=[],r=0;r<7;++r)a=_([2e3,1]).day(r),this._minWeekdaysParse[r]=this.weekdaysMin(a,"").toLocaleLowerCase(),this._shortWeekdaysParse[r]=this.weekdaysShort(a,"").toLocaleLowerCase(),this._weekdaysParse[r]=this.weekdays(a,"").toLocaleLowerCase();return n?"dddd"===t?(i=We.call(this._weekdaysParse,o),-1!==i?i:null):"ddd"===t?(i=We.call(this._shortWeekdaysParse,o),-1!==i?i:null):(i=We.call(this._minWeekdaysParse,o),-1!==i?i:null):"dddd"===t?(i=We.call(this._weekdaysParse,o),-1!==i?i:(i=We.call(this._shortWeekdaysParse,o),-1!==i?i:(i=We.call(this._minWeekdaysParse,o),-1!==i?i:null))):"ddd"===t?(i=We.call(this._shortWeekdaysParse,o),-1!==i?i:(i=We.call(this._weekdaysParse,o),-1!==i?i:(i=We.call(this._minWeekdaysParse,o),-1!==i?i:null))):(i=We.call(this._minWeekdaysParse,o),-1!==i?i:(i=We.call(this._weekdaysParse,o),-1!==i?i:(i=We.call(this._shortWeekdaysParse,o),-1!==i?i:null)))}function Ut(e,t,n){var r,i,a;if(this._weekdaysParseExact)return Wt.call(this,e,t,n);for(this._weekdaysParse||(this._weekdaysParse=[],this._minWeekdaysParse=[],this._shortWeekdaysParse=[],this._fullWeekdaysParse=[]),r=0;r<7;r++){if(i=_([2e3,1]).day(r),n&&!this._fullWeekdaysParse[r]&&(this._fullWeekdaysParse[r]=new RegExp("^"+this.weekdays(i,"").replace(".","\\.?")+"$","i"),this._shortWeekdaysParse[r]=new RegExp("^"+this.weekdaysShort(i,"").replace(".","\\.?")+"$","i"),this._minWeekdaysParse[r]=new RegExp("^"+this.weekdaysMin(i,"").replace(".","\\.?")+"$","i")),this._weekdaysParse[r]||(a="^"+this.weekdays(i,"")+"|^"+this.weekdaysShort(i,"")+"|^"+this.weekdaysMin(i,""),this._weekdaysParse[r]=new RegExp(a.replace(".",""),"i")),n&&"dddd"===t&&this._fullWeekdaysParse[r].test(e))return r;if(n&&"ddd"===t&&this._shortWeekdaysParse[r].test(e))return r;if(n&&"dd"===t&&this._minWeekdaysParse[r].test(e))return r;if(!n&&this._weekdaysParse[r].test(e))return r}}function Vt(e){if(!this.isValid())return null!=e?this:NaN;var t=this._isUTC?this._d.getUTCDay():this._d.getDay();return null!=e?(e=At(e,this.localeData()),this.add(e-t,"d")):t}function qt(e){if(!this.isValid())return null!=e?this:NaN;var t=(this.day()+7-this.localeData()._week.dow)%7;return null==e?t:this.add(e-t,"d")}function Gt(e){if(!this.isValid())return null!=e?this:NaN;if(null!=e){var t=Ot(e,this.localeData());return this.day(this.day()%7?t:t-7)}return this.day()||7}function Jt(e){return this._weekdaysParseExact?(u(this,"_weekdaysRegex")||Qt.call(this),e?this._weekdaysStrictRegex:this._weekdaysRegex):(u(this,"_weekdaysRegex")||(this._weekdaysRegex=Ht),this._weekdaysStrictRegex&&e?this._weekdaysStrictRegex:this._weekdaysRegex)}function Kt(e){return this._weekdaysParseExact?(u(this,"_weekdaysRegex")||Qt.call(this),e?this._weekdaysShortStrictRegex:this._weekdaysShortRegex):(u(this,"_weekdaysShortRegex")||(this._weekdaysShortRegex=Rt),this._weekdaysShortStrictRegex&&e?this._weekdaysShortStrictRegex:this._weekdaysShortRegex)}function Xt(e){return this._weekdaysParseExact?(u(this,"_weekdaysRegex")||Qt.call(this),e?this._weekdaysMinStrictRegex:this._weekdaysMinRegex):(u(this,"_weekdaysMinRegex")||(this._weekdaysMinRegex=Nt),this._weekdaysMinStrictRegex&&e?this._weekdaysMinStrictRegex:this._weekdaysMinRegex)}function Qt(){function e(e,t){return t.length-e.length}var t,n,r,i,a,o=[],s=[],l=[],u=[];for(t=0;t<7;t++)n=_([2e3,1]).day(t),r=Re(this.weekdaysMin(n,"")),i=Re(this.weekdaysShort(n,"")),a=Re(this.weekdays(n,"")),o.push(r),s.push(i),l.push(a),u.push(r),u.push(i),u.push(a);o.sort(e),s.sort(e),l.sort(e),u.sort(e),this._weekdaysRegex=new RegExp("^("+u.join("|")+")","i"),this._weekdaysShortRegex=this._weekdaysRegex,this._weekdaysMinRegex=this._weekdaysRegex,this._weekdaysStrictRegex=new RegExp("^("+l.join("|")+")","i"),this._weekdaysShortStrictRegex=new RegExp("^("+s.join("|")+")","i"),this._weekdaysMinStrictRegex=new RegExp("^("+o.join("|")+")","i")}function Zt(){return this.hours()%12||12}function en(){return this.hours()||24}function tn(e,t){z(e,0,0,(function(){return this.localeData().meridiem(this.hours(),this.minutes(),t)}))}function nn(e,t){return t._meridiemParse}function rn(e){return"p"===(e+"").toLowerCase().charAt(0)}z("H",["HH",2],0,"hour"),z("h",["hh",2],0,Zt),z("k",["kk",2],0,en),z("hmm",0,0,(function(){return""+Zt.apply(this)+j(this.minutes(),2)})),z("hmmss",0,0,(function(){return""+Zt.apply(this)+j(this.minutes(),2)+j(this.seconds(),2)})),z("Hmm",0,0,(function(){return""+this.hours()+j(this.minutes(),2)})),z("Hmmss",0,0,(function(){return""+this.hours()+j(this.minutes(),2)+j(this.seconds(),2)})),tn("a",!0),tn("A",!1),ie("hour","h"),le("hour",13),Ie("a",nn),Ie("A",nn),Ie("H",Le),Ie("h",Le),Ie("k",Le),Ie("HH",Le,ye),Ie("hh",Le,ye),Ie("kk",Le,ye),Ie("hmm",xe),Ie("hmmss",Se),Ie("Hmm",xe),Ie("Hmmss",Se),$e(["H","HH"],Ge),$e(["k","kk"],(function(e,t,n){var r=fe(e);t[Ge]=24===r?0:r})),$e(["a","A"],(function(e,t,n){n._isPm=n._locale.isPM(e),n._meridiem=e})),$e(["h","hh"],(function(e,t,n){t[Ge]=fe(e),g(n).bigHour=!0})),$e("hmm",(function(e,t,n){var r=e.length-2;t[Ge]=fe(e.substr(0,r)),t[Je]=fe(e.substr(r)),g(n).bigHour=!0})),$e("hmmss",(function(e,t,n){var r=e.length-4,i=e.length-2;t[Ge]=fe(e.substr(0,r)),t[Je]=fe(e.substr(r,2)),t[Ke]=fe(e.substr(i)),g(n).bigHour=!0})),$e("Hmm",(function(e,t,n){var r=e.length-2;t[Ge]=fe(e.substr(0,r)),t[Je]=fe(e.substr(r))})),$e("Hmmss",(function(e,t,n){var r=e.length-4,i=e.length-2;t[Ge]=fe(e.substr(0,r)),t[Je]=fe(e.substr(r,2)),t[Ke]=fe(e.substr(i))}));var an=/[ap]\.?m?\.?/i,on=he("Hours",!0);function sn(e,t,n){return e>11?n?"pm":"PM":n?"am":"AM"}var ln,un={calendar:F,longDateFormat:q,invalidDate:J,ordinal:X,dayOfMonthOrdinalParse:Q,relativeTime:ee,months:nt,monthsShort:rt,week:Tt,weekdays:Ft,weekdaysMin:jt,weekdaysShort:It,meridiemParse:an},cn={},dn={};function fn(e,t){var n,r=Math.min(e.length,t.length);for(n=0;n<r;n+=1)if(e[n]!==t[n])return n;return r}function hn(e){return e?e.toLowerCase().replace("_","-"):e}function pn(e){var t,n,r,i,a=0;while(a<e.length){i=hn(e[a]).split("-"),t=i.length,n=hn(e[a+1]),n=n?n.split("-"):null;while(t>0){if(r=mn(i.slice(0,t).join("-")),r)return r;if(n&&n.length>=t&&fn(i,n)>=t-1)break;t--}a++}return ln}function mn(r){var i=null;if(void 0===cn[r]&&"undefined"!==typeof e&&e&&e.exports)try{i=ln._abbr,t,n("4678")("./"+r),_n(i)}catch(a){cn[r]=null}return cn[r]}function _n(e,t){var n;return e&&(n=d(t)?bn(e):vn(e,t),n?ln=n:"undefined"!==typeof console&&console.warn&&console.warn("Locale "+e+" not found. Did you forget to load it?")),ln._abbr}function vn(e,t){if(null!==t){var n,r=un;if(t.abbr=e,null!=cn[e])C("defineLocaleOverride","use moment.updateLocale(localeName, config) to change an existing locale. moment.defineLocale(localeName, config) should only be used for creating a new locale See http://momentjs.com/guides/#/warnings/define-locale/ for more info."),r=cn[e]._config;else if(null!=t.parentLocale)if(null!=cn[t.parentLocale])r=cn[t.parentLocale]._config;else{if(n=mn(t.parentLocale),null==n)return dn[t.parentLocale]||(dn[t.parentLocale]=[]),dn[t.parentLocale].push({name:e,config:t}),null;r=n._config}return cn[e]=new P(O(r,t)),dn[e]&&dn[e].forEach((function(e){vn(e.name,e.config)})),_n(e),cn[e]}return delete cn[e],null}function gn(e,t){if(null!=t){var n,r,i=un;null!=cn[e]&&null!=cn[e].parentLocale?cn[e].set(O(cn[e]._config,t)):(r=mn(e),null!=r&&(i=r._config),t=O(i,t),null==r&&(t.abbr=e),n=new P(t),n.parentLocale=cn[e],cn[e]=n),_n(e)}else null!=cn[e]&&(null!=cn[e].parentLocale?(cn[e]=cn[e].parentLocale,e===_n()&&_n(e)):null!=cn[e]&&delete cn[e]);return cn[e]}function bn(e){var t;if(e&&e._locale&&e._locale._abbr&&(e=e._locale._abbr),!e)return ln;if(!s(e)){if(t=mn(e),t)return t;e=[e]}return pn(e)}function yn(){return D(cn)}function Mn(e){var t,n=e._a;return n&&-2===g(e).overflow&&(t=n[Ve]<0||n[Ve]>11?Ve:n[qe]<1||n[qe]>tt(n[Ue],n[Ve])?qe:n[Ge]<0||n[Ge]>24||24===n[Ge]&&(0!==n[Je]||0!==n[Ke]||0!==n[Xe])?Ge:n[Je]<0||n[Je]>59?Je:n[Ke]<0||n[Ke]>59?Ke:n[Xe]<0||n[Xe]>999?Xe:-1,g(e)._overflowDayOfYear&&(t<Ue||t>qe)&&(t=qe),g(e)._overflowWeeks&&-1===t&&(t=Qe),g(e)._overflowWeekday&&-1===t&&(t=Ze),g(e).overflow=t),e}var kn=/^\s*((?:[+-]\d{6}|\d{4})-(?:\d\d-\d\d|W\d\d-\d|W\d\d|\d\d\d|\d\d))(?:(T| )(\d\d(?::\d\d(?::\d\d(?:[.,]\d+)?)?)?)([+-]\d\d(?::?\d\d)?|\s*Z)?)?$/,wn=/^\s*((?:[+-]\d{6}|\d{4})(?:\d\d\d\d|W\d\d\d|W\d\d|\d\d\d|\d\d|))(?:(T| )(\d\d(?:\d\d(?:\d\d(?:[.,]\d+)?)?)?)([+-]\d\d(?::?\d\d)?|\s*Z)?)?$/,Ln=/Z|[+-]\d\d(?::?\d\d)?/,xn=[["YYYYYY-MM-DD",/[+-]\d{6}-\d\d-\d\d/],["YYYY-MM-DD",/\d{4}-\d\d-\d\d/],["GGGG-[W]WW-E",/\d{4}-W\d\d-\d/],["GGGG-[W]WW",/\d{4}-W\d\d/,!1],["YYYY-DDD",/\d{4}-\d{3}/],["YYYY-MM",/\d{4}-\d\d/,!1],["YYYYYYMMDD",/[+-]\d{10}/],["YYYYMMDD",/\d{8}/],["GGGG[W]WWE",/\d{4}W\d{3}/],["GGGG[W]WW",/\d{4}W\d{2}/,!1],["YYYYDDD",/\d{7}/],["YYYYMM",/\d{6}/,!1],["YYYY",/\d{4}/,!1]],Sn=[["HH:mm:ss.SSSS",/\d\d:\d\d:\d\d\.\d+/],["HH:mm:ss,SSSS",/\d\d:\d\d:\d\d,\d+/],["HH:mm:ss",/\d\d:\d\d:\d\d/],["HH:mm",/\d\d:\d\d/],["HHmmss.SSSS",/\d\d\d\d\d\d\.\d+/],["HHmmss,SSSS",/\d\d\d\d\d\d,\d+/],["HHmmss",/\d\d\d\d\d\d/],["HHmm",/\d\d\d\d/],["HH",/\d\d/]],Tn=/^\/?Date\((-?\d+)/i,Dn=/^(?:(Mon|Tue|Wed|Thu|Fri|Sat|Sun),?\s)?(\d{1,2})\s(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)\s(\d{2,4})\s(\d\d):(\d\d)(?::(\d\d))?\s(?:(UT|GMT|[ECMP][SD]T)|([Zz])|([+-]\d{4}))$/,Yn={UT:0,GMT:0,EDT:-240,EST:-300,CDT:-300,CST:-360,MDT:-360,MST:-420,PDT:-420,PST:-480};function Cn(e){var t,n,r,i,a,o,s=e._i,l=kn.exec(s)||wn.exec(s);if(l){for(g(e).iso=!0,t=0,n=xn.length;t<n;t++)if(xn[t][1].exec(l[1])){i=xn[t][0],r=!1!==xn[t][2];break}if(null==i)return void(e._isValid=!1);if(l[3]){for(t=0,n=Sn.length;t<n;t++)if(Sn[t][1].exec(l[3])){a=(l[2]||" ")+Sn[t][0];break}if(null==a)return void(e._isValid=!1)}if(!r&&null!=a)return void(e._isValid=!1);if(l[4]){if(!Ln.exec(l[4]))return void(e._isValid=!1);o="Z"}e._f=i+(a||"")+(o||""),zn(e)}else e._isValid=!1}function En(e,t,n,r,i,a){var o=[An(e),rt.indexOf(t),parseInt(n,10),parseInt(r,10),parseInt(i,10)];return a&&o.push(parseInt(a,10)),o}function An(e){var t=parseInt(e,10);return t<=49?2e3+t:t<=999?1900+t:t}function On(e){return e.replace(/\([^)]*\)|[\n\t]/g," ").replace(/(\s\s+)/g," ").replace(/^\s\s*/,"").replace(/\s\s*$/,"")}function Pn(e,t,n){if(e){var r=It.indexOf(e),i=new Date(t[0],t[1],t[2]).getDay();if(r!==i)return g(n).weekdayMismatch=!0,n._isValid=!1,!1}return!0}function Fn(e,t,n){if(e)return Yn[e];if(t)return 0;var r=parseInt(n,10),i=r%100,a=(r-i)/100;return 60*a+i}function In(e){var t,n=Dn.exec(On(e._i));if(n){if(t=En(n[4],n[3],n[2],n[5],n[6],n[7]),!Pn(n[1],t,e))return;e._a=t,e._tzm=Fn(n[8],n[9],n[10]),e._d=Mt.apply(null,e._a),e._d.setUTCMinutes(e._d.getUTCMinutes()-e._tzm),g(e).rfc2822=!0}else e._isValid=!1}function jn(e){var t=Tn.exec(e._i);null===t?(Cn(e),!1===e._isValid&&(delete e._isValid,In(e),!1===e._isValid&&(delete e._isValid,e._strict?e._isValid=!1:a.createFromInputFallback(e)))):e._d=new Date(+t[1])}function Hn(e,t,n){return null!=e?e:null!=t?t:n}function Rn(e){var t=new Date(a.now());return e._useUTC?[t.getUTCFullYear(),t.getUTCMonth(),t.getUTCDate()]:[t.getFullYear(),t.getMonth(),t.getDate()]}function Nn(e){var t,n,r,i,a,o=[];if(!e._d){for(r=Rn(e),e._w&&null==e._a[qe]&&null==e._a[Ve]&&$n(e),null!=e._dayOfYear&&(a=Hn(e._a[Ue],r[Ue]),(e._dayOfYear>vt(a)||0===e._dayOfYear)&&(g(e)._overflowDayOfYear=!0),n=Mt(a,0,e._dayOfYear),e._a[Ve]=n.getUTCMonth(),e._a[qe]=n.getUTCDate()),t=0;t<3&&null==e._a[t];++t)e._a[t]=o[t]=r[t];for(;t<7;t++)e._a[t]=o[t]=null==e._a[t]?2===t?1:0:e._a[t];24===e._a[Ge]&&0===e._a[Je]&&0===e._a[Ke]&&0===e._a[Xe]&&(e._nextDay=!0,e._a[Ge]=0),e._d=(e._useUTC?Mt:yt).apply(null,o),i=e._useUTC?e._d.getUTCDay():e._d.getDay(),null!=e._tzm&&e._d.setUTCMinutes(e._d.getUTCMinutes()-e._tzm),e._nextDay&&(e._a[Ge]=24),e._w&&"undefined"!==typeof e._w.d&&e._w.d!==i&&(g(e).weekdayMismatch=!0)}}function $n(e){var t,n,r,i,a,o,s,l,u;t=e._w,null!=t.GG||null!=t.W||null!=t.E?(a=1,o=4,n=Hn(t.GG,e._a[Ue],Lt(Kn(),1,4).year),r=Hn(t.W,1),i=Hn(t.E,1),(i<1||i>7)&&(l=!0)):(a=e._locale._week.dow,o=e._locale._week.doy,u=Lt(Kn(),a,o),n=Hn(t.gg,e._a[Ue],u.year),r=Hn(t.w,u.week),null!=t.d?(i=t.d,(i<0||i>6)&&(l=!0)):null!=t.e?(i=t.e+a,(t.e<0||t.e>6)&&(l=!0)):i=a),r<1||r>xt(n,a,o)?g(e)._overflowWeeks=!0:null!=l?g(e)._overflowWeekday=!0:(s=wt(n,r,i,a,o),e._a[Ue]=s.year,e._dayOfYear=s.dayOfYear)}function zn(e){if(e._f!==a.ISO_8601)if(e._f!==a.RFC_2822){e._a=[],g(e).empty=!0;var t,n,r,i,o,s,l=""+e._i,u=l.length,c=0;for(r=V(e._f,e._locale).match(H)||[],t=0;t<r.length;t++)i=r[t],n=(l.match(je(i,e))||[])[0],n&&(o=l.substr(0,l.indexOf(n)),o.length>0&&g(e).unusedInput.push(o),l=l.slice(l.indexOf(n)+n.length),c+=n.length),$[i]?(n?g(e).empty=!1:g(e).unusedTokens.push(i),Be(i,n,e)):e._strict&&!n&&g(e).unusedTokens.push(i);g(e).charsLeftOver=u-c,l.length>0&&g(e).unusedInput.push(l),e._a[Ge]<=12&&!0===g(e).bigHour&&e._a[Ge]>0&&(g(e).bigHour=void 0),g(e).parsedDateParts=e._a.slice(0),g(e).meridiem=e._meridiem,e._a[Ge]=Bn(e._locale,e._a[Ge],e._meridiem),s=g(e).era,null!==s&&(e._a[Ue]=e._locale.erasConvertYear(s,e._a[Ue])),Nn(e),Mn(e)}else In(e);else Cn(e)}function Bn(e,t,n){var r;return null==n?t:null!=e.meridiemHour?e.meridiemHour(t,n):null!=e.isPM?(r=e.isPM(n),r&&t<12&&(t+=12),r||12!==t||(t=0),t):t}function Wn(e){var t,n,r,i,a,o,s=!1;if(0===e._f.length)return g(e).invalidFormat=!0,void(e._d=new Date(NaN));for(i=0;i<e._f.length;i++)a=0,o=!1,t=w({},e),null!=e._useUTC&&(t._useUTC=e._useUTC),t._f=e._f[i],zn(t),b(t)&&(o=!0),a+=g(t).charsLeftOver,a+=10*g(t).unusedTokens.length,g(t).score=a,s?a<r&&(r=a,n=t):(null==r||a<r||o)&&(r=a,n=t,o&&(s=!0));m(e,n||t)}function Un(e){if(!e._d){var t=oe(e._i),n=void 0===t.day?t.date:t.day;e._a=p([t.year,t.month,n,t.hour,t.minute,t.second,t.millisecond],(function(e){return e&&parseInt(e,10)})),Nn(e)}}function Vn(e){var t=new L(Mn(qn(e)));return t._nextDay&&(t.add(1,"d"),t._nextDay=void 0),t}function qn(e){var t=e._i,n=e._f;return e._locale=e._locale||bn(e._l),null===t||void 0===n&&""===t?y({nullInput:!0}):("string"===typeof t&&(e._i=t=e._locale.preparse(t)),x(t)?new L(Mn(t)):(h(t)?e._d=t:s(n)?Wn(e):n?zn(e):Gn(e),b(e)||(e._d=null),e))}function Gn(e){var t=e._i;d(t)?e._d=new Date(a.now()):h(t)?e._d=new Date(t.valueOf()):"string"===typeof t?jn(e):s(t)?(e._a=p(t.slice(0),(function(e){return parseInt(e,10)})),Nn(e)):l(t)?Un(e):f(t)?e._d=new Date(t):a.createFromInputFallback(e)}function Jn(e,t,n,r,i){var a={};return!0!==t&&!1!==t||(r=t,t=void 0),!0!==n&&!1!==n||(r=n,n=void 0),(l(e)&&c(e)||s(e)&&0===e.length)&&(e=void 0),a._isAMomentObject=!0,a._useUTC=a._isUTC=i,a._l=n,a._i=e,a._f=t,a._strict=r,Vn(a)}function Kn(e,t,n,r){return Jn(e,t,n,r,!1)}a.createFromInputFallback=T("value provided is not in a recognized RFC2822 or ISO format. moment construction falls back to js Date(), which is not reliable across all browsers and versions. Non RFC2822/ISO date formats are discouraged. Please refer to http://momentjs.com/guides/#/warnings/js-date/ for more info.",(function(e){e._d=new Date(e._i+(e._useUTC?" UTC":""))})),a.ISO_8601=function(){},a.RFC_2822=function(){};var Xn=T("moment().min is deprecated, use moment.max instead. http://momentjs.com/guides/#/warnings/min-max/",(function(){var e=Kn.apply(null,arguments);return this.isValid()&&e.isValid()?e<this?this:e:y()})),Qn=T("moment().max is deprecated, use moment.min instead. http://momentjs.com/guides/#/warnings/min-max/",(function(){var e=Kn.apply(null,arguments);return this.isValid()&&e.isValid()?e>this?this:e:y()}));function Zn(e,t){var n,r;if(1===t.length&&s(t[0])&&(t=t[0]),!t.length)return Kn();for(n=t[0],r=1;r<t.length;++r)t[r].isValid()&&!t[r][e](n)||(n=t[r]);return n}function er(){var e=[].slice.call(arguments,0);return Zn("isBefore",e)}function tr(){var e=[].slice.call(arguments,0);return Zn("isAfter",e)}var nr=function(){return Date.now?Date.now():+new Date},rr=["year","quarter","month","week","day","hour","minute","second","millisecond"];function ir(e){var t,n,r=!1;for(t in e)if(u(e,t)&&(-1===We.call(rr,t)||null!=e[t]&&isNaN(e[t])))return!1;for(n=0;n<rr.length;++n)if(e[rr[n]]){if(r)return!1;parseFloat(e[rr[n]])!==fe(e[rr[n]])&&(r=!0)}return!0}function ar(){return this._isValid}function or(){return Yr(NaN)}function sr(e){var t=oe(e),n=t.year||0,r=t.quarter||0,i=t.month||0,a=t.week||t.isoWeek||0,o=t.day||0,s=t.hour||0,l=t.minute||0,u=t.second||0,c=t.millisecond||0;this._isValid=ir(t),this._milliseconds=+c+1e3*u+6e4*l+1e3*s*60*60,this._days=+o+7*a,this._months=+i+3*r+12*n,this._data={},this._locale=bn(),this._bubble()}function lr(e){return e instanceof sr}function ur(e){return e<0?-1*Math.round(-1*e):Math.round(e)}function cr(e,t,n){var r,i=Math.min(e.length,t.length),a=Math.abs(e.length-t.length),o=0;for(r=0;r<i;r++)(n&&e[r]!==t[r]||!n&&fe(e[r])!==fe(t[r]))&&o++;return o+a}function dr(e,t){z(e,0,0,(function(){var e=this.utcOffset(),n="+";return e<0&&(e=-e,n="-"),n+j(~~(e/60),2)+t+j(~~e%60,2)}))}dr("Z",":"),dr("ZZ",""),Ie("Z",Oe),Ie("ZZ",Oe),$e(["Z","ZZ"],(function(e,t,n){n._useUTC=!0,n._tzm=hr(Oe,e)}));var fr=/([\+\-]|\d\d)/gi;function hr(e,t){var n,r,i,a=(t||"").match(e);return null===a?null:(n=a[a.length-1]||[],r=(n+"").match(fr)||["-",0,0],i=60*r[1]+fe(r[2]),0===i?0:"+"===r[0]?i:-i)}function pr(e,t){var n,r;return t._isUTC?(n=t.clone(),r=(x(e)||h(e)?e.valueOf():Kn(e).valueOf())-n.valueOf(),n._d.setTime(n._d.valueOf()+r),a.updateOffset(n,!1),n):Kn(e).local()}function mr(e){return-Math.round(e._d.getTimezoneOffset())}function _r(e,t,n){var r,i=this._offset||0;if(!this.isValid())return null!=e?this:NaN;if(null!=e){if("string"===typeof e){if(e=hr(Oe,e),null===e)return this}else Math.abs(e)<16&&!n&&(e*=60);return!this._isUTC&&t&&(r=mr(this)),this._offset=e,this._isUTC=!0,null!=r&&this.add(r,"m"),i!==e&&(!t||this._changeInProgress?Pr(this,Yr(e-i,"m"),1,!1):this._changeInProgress||(this._changeInProgress=!0,a.updateOffset(this,!0),this._changeInProgress=null)),this}return this._isUTC?i:mr(this)}function vr(e,t){return null!=e?("string"!==typeof e&&(e=-e),this.utcOffset(e,t),this):-this.utcOffset()}function gr(e){return this.utcOffset(0,e)}function br(e){return this._isUTC&&(this.utcOffset(0,e),this._isUTC=!1,e&&this.subtract(mr(this),"m")),this}function yr(){if(null!=this._tzm)this.utcOffset(this._tzm,!1,!0);else if("string"===typeof this._i){var e=hr(Ae,this._i);null!=e?this.utcOffset(e):this.utcOffset(0,!0)}return this}function Mr(e){return!!this.isValid()&&(e=e?Kn(e).utcOffset():0,(this.utcOffset()-e)%60===0)}function kr(){return this.utcOffset()>this.clone().month(0).utcOffset()||this.utcOffset()>this.clone().month(5).utcOffset()}function wr(){if(!d(this._isDSTShifted))return this._isDSTShifted;var e,t={};return w(t,this),t=qn(t),t._a?(e=t._isUTC?_(t._a):Kn(t._a),this._isDSTShifted=this.isValid()&&cr(t._a,e.toArray())>0):this._isDSTShifted=!1,this._isDSTShifted}function Lr(){return!!this.isValid()&&!this._isUTC}function xr(){return!!this.isValid()&&this._isUTC}function Sr(){return!!this.isValid()&&(this._isUTC&&0===this._offset)}a.updateOffset=function(){};var Tr=/^(-|\+)?(?:(\d*)[. ])?(\d+):(\d+)(?::(\d+)(\.\d*)?)?$/,Dr=/^(-|\+)?P(?:([-+]?[0-9,.]*)Y)?(?:([-+]?[0-9,.]*)M)?(?:([-+]?[0-9,.]*)W)?(?:([-+]?[0-9,.]*)D)?(?:T(?:([-+]?[0-9,.]*)H)?(?:([-+]?[0-9,.]*)M)?(?:([-+]?[0-9,.]*)S)?)?$/;function Yr(e,t){var n,r,i,a=e,o=null;return lr(e)?a={ms:e._milliseconds,d:e._days,M:e._months}:f(e)||!isNaN(+e)?(a={},t?a[t]=+e:a.milliseconds=+e):(o=Tr.exec(e))?(n="-"===o[1]?-1:1,a={y:0,d:fe(o[qe])*n,h:fe(o[Ge])*n,m:fe(o[Je])*n,s:fe(o[Ke])*n,ms:fe(ur(1e3*o[Xe]))*n}):(o=Dr.exec(e))?(n="-"===o[1]?-1:1,a={y:Cr(o[2],n),M:Cr(o[3],n),w:Cr(o[4],n),d:Cr(o[5],n),h:Cr(o[6],n),m:Cr(o[7],n),s:Cr(o[8],n)}):null==a?a={}:"object"===typeof a&&("from"in a||"to"in a)&&(i=Ar(Kn(a.from),Kn(a.to)),a={},a.ms=i.milliseconds,a.M=i.months),r=new sr(a),lr(e)&&u(e,"_locale")&&(r._locale=e._locale),lr(e)&&u(e,"_isValid")&&(r._isValid=e._isValid),r}function Cr(e,t){var n=e&&parseFloat(e.replace(",","."));return(isNaN(n)?0:n)*t}function Er(e,t){var n={};return n.months=t.month()-e.month()+12*(t.year()-e.year()),e.clone().add(n.months,"M").isAfter(t)&&--n.months,n.milliseconds=+t-+e.clone().add(n.months,"M"),n}function Ar(e,t){var n;return e.isValid()&&t.isValid()?(t=pr(t,e),e.isBefore(t)?n=Er(e,t):(n=Er(t,e),n.milliseconds=-n.milliseconds,n.months=-n.months),n):{milliseconds:0,months:0}}function Or(e,t){return function(n,r){var i,a;return null===r||isNaN(+r)||(C(t,"moment()."+t+"(period, number) is deprecated. Please use moment()."+t+"(number, period). See http://momentjs.com/guides/#/warnings/add-inverted-param/ for more info."),a=n,n=r,r=a),i=Yr(n,r),Pr(this,i,e),this}}function Pr(e,t,n,r){var i=t._milliseconds,o=ur(t._days),s=ur(t._months);e.isValid()&&(r=null==r||r,s&&dt(e,pe(e,"Month")+s*n),o&&me(e,"Date",pe(e,"Date")+o*n),i&&e._d.setTime(e._d.valueOf()+i*n),r&&a.updateOffset(e,o||s))}Yr.fn=sr.prototype,Yr.invalid=or;var Fr=Or(1,"add"),Ir=Or(-1,"subtract");function jr(e){return"string"===typeof e||e instanceof String}function Hr(e){return x(e)||h(e)||jr(e)||f(e)||Nr(e)||Rr(e)||null===e||void 0===e}function Rr(e){var t,n,r=l(e)&&!c(e),i=!1,a=["years","year","y","months","month","M","days","day","d","dates","date","D","hours","hour","h","minutes","minute","m","seconds","second","s","milliseconds","millisecond","ms"];for(t=0;t<a.length;t+=1)n=a[t],i=i||u(e,n);return r&&i}function Nr(e){var t=s(e),n=!1;return t&&(n=0===e.filter((function(t){return!f(t)&&jr(e)})).length),t&&n}function $r(e){var t,n,r=l(e)&&!c(e),i=!1,a=["sameDay","nextDay","lastDay","nextWeek","lastWeek","sameElse"];for(t=0;t<a.length;t+=1)n=a[t],i=i||u(e,n);return r&&i}function zr(e,t){var n=e.diff(t,"days",!0);return n<-6?"sameElse":n<-1?"lastWeek":n<0?"lastDay":n<1?"sameDay":n<2?"nextDay":n<7?"nextWeek":"sameElse"}function Br(e,t){1===arguments.length&&(arguments[0]?Hr(arguments[0])?(e=arguments[0],t=void 0):$r(arguments[0])&&(t=arguments[0],e=void 0):(e=void 0,t=void 0));var n=e||Kn(),r=pr(n,this).startOf("day"),i=a.calendarFormat(this,r)||"sameElse",o=t&&(E(t[i])?t[i].call(this,n):t[i]);return this.format(o||this.localeData().calendar(i,this,Kn(n)))}function Wr(){return new L(this)}function Ur(e,t){var n=x(e)?e:Kn(e);return!(!this.isValid()||!n.isValid())&&(t=ae(t)||"millisecond","millisecond"===t?this.valueOf()>n.valueOf():n.valueOf()<this.clone().startOf(t).valueOf())}function Vr(e,t){var n=x(e)?e:Kn(e);return!(!this.isValid()||!n.isValid())&&(t=ae(t)||"millisecond","millisecond"===t?this.valueOf()<n.valueOf():this.clone().endOf(t).valueOf()<n.valueOf())}function qr(e,t,n,r){var i=x(e)?e:Kn(e),a=x(t)?t:Kn(t);return!!(this.isValid()&&i.isValid()&&a.isValid())&&(r=r||"()",("("===r[0]?this.isAfter(i,n):!this.isBefore(i,n))&&(")"===r[1]?this.isBefore(a,n):!this.isAfter(a,n)))}function Gr(e,t){var n,r=x(e)?e:Kn(e);return!(!this.isValid()||!r.isValid())&&(t=ae(t)||"millisecond","millisecond"===t?this.valueOf()===r.valueOf():(n=r.valueOf(),this.clone().startOf(t).valueOf()<=n&&n<=this.clone().endOf(t).valueOf()))}function Jr(e,t){return this.isSame(e,t)||this.isAfter(e,t)}function Kr(e,t){return this.isSame(e,t)||this.isBefore(e,t)}function Xr(e,t,n){var r,i,a;if(!this.isValid())return NaN;if(r=pr(e,this),!r.isValid())return NaN;switch(i=6e4*(r.utcOffset()-this.utcOffset()),t=ae(t),t){case"year":a=Qr(this,r)/12;break;case"month":a=Qr(this,r);break;case"quarter":a=Qr(this,r)/3;break;case"second":a=(this-r)/1e3;break;case"minute":a=(this-r)/6e4;break;case"hour":a=(this-r)/36e5;break;case"day":a=(this-r-i)/864e5;break;case"week":a=(this-r-i)/6048e5;break;default:a=this-r}return n?a:de(a)}function Qr(e,t){if(e.date()<t.date())return-Qr(t,e);var n,r,i=12*(t.year()-e.year())+(t.month()-e.month()),a=e.clone().add(i,"months");return t-a<0?(n=e.clone().add(i-1,"months"),r=(t-a)/(a-n)):(n=e.clone().add(i+1,"months"),r=(t-a)/(n-a)),-(i+r)||0}function Zr(){return this.clone().locale("en").format("ddd MMM DD YYYY HH:mm:ss [GMT]ZZ")}function ei(e){if(!this.isValid())return null;var t=!0!==e,n=t?this.clone().utc():this;return n.year()<0||n.year()>9999?U(n,t?"YYYYYY-MM-DD[T]HH:mm:ss.SSS[Z]":"YYYYYY-MM-DD[T]HH:mm:ss.SSSZ"):E(Date.prototype.toISOString)?t?this.toDate().toISOString():new Date(this.valueOf()+60*this.utcOffset()*1e3).toISOString().replace("Z",U(n,"Z")):U(n,t?"YYYY-MM-DD[T]HH:mm:ss.SSS[Z]":"YYYY-MM-DD[T]HH:mm:ss.SSSZ")}function ti(){if(!this.isValid())return"moment.invalid(/* "+this._i+" */)";var e,t,n,r,i="moment",a="";return this.isLocal()||(i=0===this.utcOffset()?"moment.utc":"moment.parseZone",a="Z"),e="["+i+'("]',t=0<=this.year()&&this.year()<=9999?"YYYY":"YYYYYY",n="-MM-DD[T]HH:mm:ss.SSS",r=a+'[")]',this.format(e+t+n+r)}function ni(e){e||(e=this.isUtc()?a.defaultFormatUtc:a.defaultFormat);var t=U(this,e);return this.localeData().postformat(t)}function ri(e,t){return this.isValid()&&(x(e)&&e.isValid()||Kn(e).isValid())?Yr({to:this,from:e}).locale(this.locale()).humanize(!t):this.localeData().invalidDate()}function ii(e){return this.from(Kn(),e)}function ai(e,t){return this.isValid()&&(x(e)&&e.isValid()||Kn(e).isValid())?Yr({from:this,to:e}).locale(this.locale()).humanize(!t):this.localeData().invalidDate()}function oi(e){return this.to(Kn(),e)}function si(e){var t;return void 0===e?this._locale._abbr:(t=bn(e),null!=t&&(this._locale=t),this)}a.defaultFormat="YYYY-MM-DDTHH:mm:ssZ",a.defaultFormatUtc="YYYY-MM-DDTHH:mm:ss[Z]";var li=T("moment().lang() is deprecated. Instead, use moment().localeData() to get the language configuration. Use moment().locale() to change languages.",(function(e){return void 0===e?this.localeData():this.locale(e)}));function ui(){return this._locale}var ci=1e3,di=60*ci,fi=60*di,hi=3506328*fi;function pi(e,t){return(e%t+t)%t}function mi(e,t,n){return e<100&&e>=0?new Date(e+400,t,n)-hi:new Date(e,t,n).valueOf()}function _i(e,t,n){return e<100&&e>=0?Date.UTC(e+400,t,n)-hi:Date.UTC(e,t,n)}function vi(e){var t,n;if(e=ae(e),void 0===e||"millisecond"===e||!this.isValid())return this;switch(n=this._isUTC?_i:mi,e){case"year":t=n(this.year(),0,1);break;case"quarter":t=n(this.year(),this.month()-this.month()%3,1);break;case"month":t=n(this.year(),this.month(),1);break;case"week":t=n(this.year(),this.month(),this.date()-this.weekday());break;case"isoWeek":t=n(this.year(),this.month(),this.date()-(this.isoWeekday()-1));break;case"day":case"date":t=n(this.year(),this.month(),this.date());break;case"hour":t=this._d.valueOf(),t-=pi(t+(this._isUTC?0:this.utcOffset()*di),fi);break;case"minute":t=this._d.valueOf(),t-=pi(t,di);break;case"second":t=this._d.valueOf(),t-=pi(t,ci);break}return this._d.setTime(t),a.updateOffset(this,!0),this}function gi(e){var t,n;if(e=ae(e),void 0===e||"millisecond"===e||!this.isValid())return this;switch(n=this._isUTC?_i:mi,e){case"year":t=n(this.year()+1,0,1)-1;break;case"quarter":t=n(this.year(),this.month()-this.month()%3+3,1)-1;break;case"month":t=n(this.year(),this.month()+1,1)-1;break;case"week":t=n(this.year(),this.month(),this.date()-this.weekday()+7)-1;break;case"isoWeek":t=n(this.year(),this.month(),this.date()-(this.isoWeekday()-1)+7)-1;break;case"day":case"date":t=n(this.year(),this.month(),this.date()+1)-1;break;case"hour":t=this._d.valueOf(),t+=fi-pi(t+(this._isUTC?0:this.utcOffset()*di),fi)-1;break;case"minute":t=this._d.valueOf(),t+=di-pi(t,di)-1;break;case"second":t=this._d.valueOf(),t+=ci-pi(t,ci)-1;break}return this._d.setTime(t),a.updateOffset(this,!0),this}function bi(){return this._d.valueOf()-6e4*(this._offset||0)}function yi(){return Math.floor(this.valueOf()/1e3)}function Mi(){return new Date(this.valueOf())}function ki(){var e=this;return[e.year(),e.month(),e.date(),e.hour(),e.minute(),e.second(),e.millisecond()]}function wi(){var e=this;return{years:e.year(),months:e.month(),date:e.date(),hours:e.hours(),minutes:e.minutes(),seconds:e.seconds(),milliseconds:e.milliseconds()}}function Li(){return this.isValid()?this.toISOString():null}function xi(){return b(this)}function Si(){return m({},g(this))}function Ti(){return g(this).overflow}function Di(){return{input:this._i,format:this._f,locale:this._locale,isUTC:this._isUTC,strict:this._strict}}function Yi(e,t){var n,r,i,o=this._eras||bn("en")._eras;for(n=0,r=o.length;n<r;++n){switch(typeof o[n].since){case"string":i=a(o[n].since).startOf("day"),o[n].since=i.valueOf();break}switch(typeof o[n].until){case"undefined":o[n].until=1/0;break;case"string":i=a(o[n].until).startOf("day").valueOf(),o[n].until=i.valueOf();break}}return o}function Ci(e,t,n){var r,i,a,o,s,l=this.eras();for(e=e.toUpperCase(),r=0,i=l.length;r<i;++r)if(a=l[r].name.toUpperCase(),o=l[r].abbr.toUpperCase(),s=l[r].narrow.toUpperCase(),n)switch(t){case"N":case"NN":case"NNN":if(o===e)return l[r];break;case"NNNN":if(a===e)return l[r];break;case"NNNNN":if(s===e)return l[r];break}else if([a,o,s].indexOf(e)>=0)return l[r]}function Ei(e,t){var n=e.since<=e.until?1:-1;return void 0===t?a(e.since).year():a(e.since).year()+(t-e.offset)*n}function Ai(){var e,t,n,r=this.localeData().eras();for(e=0,t=r.length;e<t;++e){if(n=this.clone().startOf("day").valueOf(),r[e].since<=n&&n<=r[e].until)return r[e].name;if(r[e].until<=n&&n<=r[e].since)return r[e].name}return""}function Oi(){var e,t,n,r=this.localeData().eras();for(e=0,t=r.length;e<t;++e){if(n=this.clone().startOf("day").valueOf(),r[e].since<=n&&n<=r[e].until)return r[e].narrow;if(r[e].until<=n&&n<=r[e].since)return r[e].narrow}return""}function Pi(){var e,t,n,r=this.localeData().eras();for(e=0,t=r.length;e<t;++e){if(n=this.clone().startOf("day").valueOf(),r[e].since<=n&&n<=r[e].until)return r[e].abbr;if(r[e].until<=n&&n<=r[e].since)return r[e].abbr}return""}function Fi(){var e,t,n,r,i=this.localeData().eras();for(e=0,t=i.length;e<t;++e)if(n=i[e].since<=i[e].until?1:-1,r=this.clone().startOf("day").valueOf(),i[e].since<=r&&r<=i[e].until||i[e].until<=r&&r<=i[e].since)return(this.year()-a(i[e].since).year())*n+i[e].offset;return this.year()}function Ii(e){return u(this,"_erasNameRegex")||Bi.call(this),e?this._erasNameRegex:this._erasRegex}function ji(e){return u(this,"_erasAbbrRegex")||Bi.call(this),e?this._erasAbbrRegex:this._erasRegex}function Hi(e){return u(this,"_erasNarrowRegex")||Bi.call(this),e?this._erasNarrowRegex:this._erasRegex}function Ri(e,t){return t.erasAbbrRegex(e)}function Ni(e,t){return t.erasNameRegex(e)}function $i(e,t){return t.erasNarrowRegex(e)}function zi(e,t){return t._eraYearOrdinalRegex||Ce}function Bi(){var e,t,n=[],r=[],i=[],a=[],o=this.eras();for(e=0,t=o.length;e<t;++e)r.push(Re(o[e].name)),n.push(Re(o[e].abbr)),i.push(Re(o[e].narrow)),a.push(Re(o[e].name)),a.push(Re(o[e].abbr)),a.push(Re(o[e].narrow));this._erasRegex=new RegExp("^("+a.join("|")+")","i"),this._erasNameRegex=new RegExp("^("+r.join("|")+")","i"),this._erasAbbrRegex=new RegExp("^("+n.join("|")+")","i"),this._erasNarrowRegex=new RegExp("^("+i.join("|")+")","i")}function Wi(e,t){z(0,[e,e.length],0,t)}function Ui(e){return Xi.call(this,e,this.week(),this.weekday(),this.localeData()._week.dow,this.localeData()._week.doy)}function Vi(e){return Xi.call(this,e,this.isoWeek(),this.isoWeekday(),1,4)}function qi(){return xt(this.year(),1,4)}function Gi(){return xt(this.isoWeekYear(),1,4)}function Ji(){var e=this.localeData()._week;return xt(this.year(),e.dow,e.doy)}function Ki(){var e=this.localeData()._week;return xt(this.weekYear(),e.dow,e.doy)}function Xi(e,t,n,r,i){var a;return null==e?Lt(this,r,i).year:(a=xt(e,r,i),t>a&&(t=a),Qi.call(this,e,t,n,r,i))}function Qi(e,t,n,r,i){var a=wt(e,t,n,r,i),o=Mt(a.year,0,a.dayOfYear);return this.year(o.getUTCFullYear()),this.month(o.getUTCMonth()),this.date(o.getUTCDate()),this}function Zi(e){return null==e?Math.ceil((this.month()+1)/3):this.month(3*(e-1)+this.month()%3)}z("N",0,0,"eraAbbr"),z("NN",0,0,"eraAbbr"),z("NNN",0,0,"eraAbbr"),z("NNNN",0,0,"eraName"),z("NNNNN",0,0,"eraNarrow"),z("y",["y",1],"yo","eraYear"),z("y",["yy",2],0,"eraYear"),z("y",["yyy",3],0,"eraYear"),z("y",["yyyy",4],0,"eraYear"),Ie("N",Ri),Ie("NN",Ri),Ie("NNN",Ri),Ie("NNNN",Ni),Ie("NNNNN",$i),$e(["N","NN","NNN","NNNN","NNNNN"],(function(e,t,n,r){var i=n._locale.erasParse(e,r,n._strict);i?g(n).era=i:g(n).invalidEra=e})),Ie("y",Ce),Ie("yy",Ce),Ie("yyy",Ce),Ie("yyyy",Ce),Ie("yo",zi),$e(["y","yy","yyy","yyyy"],Ue),$e(["yo"],(function(e,t,n,r){var i;n._locale._eraYearOrdinalRegex&&(i=e.match(n._locale._eraYearOrdinalRegex)),n._locale.eraYearOrdinalParse?t[Ue]=n._locale.eraYearOrdinalParse(e,i):t[Ue]=parseInt(e,10)})),z(0,["gg",2],0,(function(){return this.weekYear()%100})),z(0,["GG",2],0,(function(){return this.isoWeekYear()%100})),Wi("gggg","weekYear"),Wi("ggggg","weekYear"),Wi("GGGG","isoWeekYear"),Wi("GGGGG","isoWeekYear"),ie("weekYear","gg"),ie("isoWeekYear","GG"),le("weekYear",1),le("isoWeekYear",1),Ie("G",Ee),Ie("g",Ee),Ie("GG",Le,ye),Ie("gg",Le,ye),Ie("GGGG",De,ke),Ie("gggg",De,ke),Ie("GGGGG",Ye,we),Ie("ggggg",Ye,we),ze(["gggg","ggggg","GGGG","GGGGG"],(function(e,t,n,r){t[r.substr(0,2)]=fe(e)})),ze(["gg","GG"],(function(e,t,n,r){t[r]=a.parseTwoDigitYear(e)})),z("Q",0,"Qo","quarter"),ie("quarter","Q"),le("quarter",7),Ie("Q",be),$e("Q",(function(e,t){t[Ve]=3*(fe(e)-1)})),z("D",["DD",2],"Do","date"),ie("date","D"),le("date",9),Ie("D",Le),Ie("DD",Le,ye),Ie("Do",(function(e,t){return e?t._dayOfMonthOrdinalParse||t._ordinalParse:t._dayOfMonthOrdinalParseLenient})),$e(["D","DD"],qe),$e("Do",(function(e,t){t[qe]=fe(e.match(Le)[0])}));var ea=he("Date",!0);function ta(e){var t=Math.round((this.clone().startOf("day")-this.clone().startOf("year"))/864e5)+1;return null==e?t:this.add(e-t,"d")}z("DDD",["DDDD",3],"DDDo","dayOfYear"),ie("dayOfYear","DDD"),le("dayOfYear",4),Ie("DDD",Te),Ie("DDDD",Me),$e(["DDD","DDDD"],(function(e,t,n){n._dayOfYear=fe(e)})),z("m",["mm",2],0,"minute"),ie("minute","m"),le("minute",14),Ie("m",Le),Ie("mm",Le,ye),$e(["m","mm"],Je);var na=he("Minutes",!1);z("s",["ss",2],0,"second"),ie("second","s"),le("second",15),Ie("s",Le),Ie("ss",Le,ye),$e(["s","ss"],Ke);var ra,ia,aa=he("Seconds",!1);for(z("S",0,0,(function(){return~~(this.millisecond()/100)})),z(0,["SS",2],0,(function(){return~~(this.millisecond()/10)})),z(0,["SSS",3],0,"millisecond"),z(0,["SSSS",4],0,(function(){return 10*this.millisecond()})),z(0,["SSSSS",5],0,(function(){return 100*this.millisecond()})),z(0,["SSSSSS",6],0,(function(){return 1e3*this.millisecond()})),z(0,["SSSSSSS",7],0,(function(){return 1e4*this.millisecond()})),z(0,["SSSSSSSS",8],0,(function(){return 1e5*this.millisecond()})),z(0,["SSSSSSSSS",9],0,(function(){return 1e6*this.millisecond()})),ie("millisecond","ms"),le("millisecond",16),Ie("S",Te,be),Ie("SS",Te,ye),Ie("SSS",Te,Me),ra="SSSS";ra.length<=9;ra+="S")Ie(ra,Ce);function oa(e,t){t[Xe]=fe(1e3*("0."+e))}for(ra="S";ra.length<=9;ra+="S")$e(ra,oa);function sa(){return this._isUTC?"UTC":""}function la(){return this._isUTC?"Coordinated Universal Time":""}ia=he("Milliseconds",!1),z("z",0,0,"zoneAbbr"),z("zz",0,0,"zoneName");var ua=L.prototype;function ca(e){return Kn(1e3*e)}function da(){return Kn.apply(null,arguments).parseZone()}function fa(e){return e}ua.add=Fr,ua.calendar=Br,ua.clone=Wr,ua.diff=Xr,ua.endOf=gi,ua.format=ni,ua.from=ri,ua.fromNow=ii,ua.to=ai,ua.toNow=oi,ua.get=_e,ua.invalidAt=Ti,ua.isAfter=Ur,ua.isBefore=Vr,ua.isBetween=qr,ua.isSame=Gr,ua.isSameOrAfter=Jr,ua.isSameOrBefore=Kr,ua.isValid=xi,ua.lang=li,ua.locale=si,ua.localeData=ui,ua.max=Qn,ua.min=Xn,ua.parsingFlags=Si,ua.set=ve,ua.startOf=vi,ua.subtract=Ir,ua.toArray=ki,ua.toObject=wi,ua.toDate=Mi,ua.toISOString=ei,ua.inspect=ti,"undefined"!==typeof Symbol&&null!=Symbol.for&&(ua[Symbol.for("nodejs.util.inspect.custom")]=function(){return"Moment<"+this.format()+">"}),ua.toJSON=Li,ua.toString=Zr,ua.unix=yi,ua.valueOf=bi,ua.creationData=Di,ua.eraName=Ai,ua.eraNarrow=Oi,ua.eraAbbr=Pi,ua.eraYear=Fi,ua.year=gt,ua.isLeapYear=bt,ua.weekYear=Ui,ua.isoWeekYear=Vi,ua.quarter=ua.quarters=Zi,ua.month=ft,ua.daysInMonth=ht,ua.week=ua.weeks=Ct,ua.isoWeek=ua.isoWeeks=Et,ua.weeksInYear=Ji,ua.weeksInWeekYear=Ki,ua.isoWeeksInYear=qi,ua.isoWeeksInISOWeekYear=Gi,ua.date=ea,ua.day=ua.days=Vt,ua.weekday=qt,ua.isoWeekday=Gt,ua.dayOfYear=ta,ua.hour=ua.hours=on,ua.minute=ua.minutes=na,ua.second=ua.seconds=aa,ua.millisecond=ua.milliseconds=ia,ua.utcOffset=_r,ua.utc=gr,ua.local=br,ua.parseZone=yr,ua.hasAlignedHourOffset=Mr,ua.isDST=kr,ua.isLocal=Lr,ua.isUtcOffset=xr,ua.isUtc=Sr,ua.isUTC=Sr,ua.zoneAbbr=sa,ua.zoneName=la,ua.dates=T("dates accessor is deprecated. Use date instead.",ea),ua.months=T("months accessor is deprecated. Use month instead",ft),ua.years=T("years accessor is deprecated. Use year instead",gt),ua.zone=T("moment().zone is deprecated, use moment().utcOffset instead. http://momentjs.com/guides/#/warnings/zone/",vr),ua.isDSTShifted=T("isDSTShifted is deprecated. See http://momentjs.com/guides/#/warnings/dst-shifted/ for more information",wr);var ha=P.prototype;function pa(e,t,n,r){var i=bn(),a=_().set(r,t);return i[n](a,e)}function ma(e,t,n){if(f(e)&&(t=e,e=void 0),e=e||"",null!=t)return pa(e,t,n,"month");var r,i=[];for(r=0;r<12;r++)i[r]=pa(e,r,n,"month");return i}function _a(e,t,n,r){"boolean"===typeof e?(f(t)&&(n=t,t=void 0),t=t||""):(t=e,n=t,e=!1,f(t)&&(n=t,t=void 0),t=t||"");var i,a=bn(),o=e?a._week.dow:0,s=[];if(null!=n)return pa(t,(n+o)%7,r,"day");for(i=0;i<7;i++)s[i]=pa(t,(i+o)%7,r,"day");return s}function va(e,t){return ma(e,t,"months")}function ga(e,t){return ma(e,t,"monthsShort")}function ba(e,t,n){return _a(e,t,n,"weekdays")}function ya(e,t,n){return _a(e,t,n,"weekdaysShort")}function Ma(e,t,n){return _a(e,t,n,"weekdaysMin")}ha.calendar=I,ha.longDateFormat=G,ha.invalidDate=K,ha.ordinal=Z,ha.preparse=fa,ha.postformat=fa,ha.relativeTime=te,ha.pastFuture=ne,ha.set=A,ha.eras=Yi,ha.erasParse=Ci,ha.erasConvertYear=Ei,ha.erasAbbrRegex=ji,ha.erasNameRegex=Ii,ha.erasNarrowRegex=Hi,ha.months=st,ha.monthsShort=lt,ha.monthsParse=ct,ha.monthsRegex=mt,ha.monthsShortRegex=pt,ha.week=St,ha.firstDayOfYear=Yt,ha.firstDayOfWeek=Dt,ha.weekdays=$t,ha.weekdaysMin=Bt,ha.weekdaysShort=zt,ha.weekdaysParse=Ut,ha.weekdaysRegex=Jt,ha.weekdaysShortRegex=Kt,ha.weekdaysMinRegex=Xt,ha.isPM=rn,ha.meridiem=sn,_n("en",{eras:[{since:"0001-01-01",until:1/0,offset:1,name:"Anno Domini",narrow:"AD",abbr:"AD"},{since:"0000-12-31",until:-1/0,offset:1,name:"Before Christ",narrow:"BC",abbr:"BC"}],dayOfMonthOrdinalParse:/\d{1,2}(th|st|nd|rd)/,ordinal:function(e){var t=e%10,n=1===fe(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th";return e+n}}),a.lang=T("moment.lang is deprecated. Use moment.locale instead.",_n),a.langData=T("moment.langData is deprecated. Use moment.localeData instead.",bn);var ka=Math.abs;function wa(){var e=this._data;return this._milliseconds=ka(this._milliseconds),this._days=ka(this._days),this._months=ka(this._months),e.milliseconds=ka(e.milliseconds),e.seconds=ka(e.seconds),e.minutes=ka(e.minutes),e.hours=ka(e.hours),e.months=ka(e.months),e.years=ka(e.years),this}function La(e,t,n,r){var i=Yr(t,n);return e._milliseconds+=r*i._milliseconds,e._days+=r*i._days,e._months+=r*i._months,e._bubble()}function xa(e,t){return La(this,e,t,1)}function Sa(e,t){return La(this,e,t,-1)}function Ta(e){return e<0?Math.floor(e):Math.ceil(e)}function Da(){var e,t,n,r,i,a=this._milliseconds,o=this._days,s=this._months,l=this._data;return a>=0&&o>=0&&s>=0||a<=0&&o<=0&&s<=0||(a+=864e5*Ta(Ca(s)+o),o=0,s=0),l.milliseconds=a%1e3,e=de(a/1e3),l.seconds=e%60,t=de(e/60),l.minutes=t%60,n=de(t/60),l.hours=n%24,o+=de(n/24),i=de(Ya(o)),s+=i,o-=Ta(Ca(i)),r=de(s/12),s%=12,l.days=o,l.months=s,l.years=r,this}function Ya(e){return 4800*e/146097}function Ca(e){return 146097*e/4800}function Ea(e){if(!this.isValid())return NaN;var t,n,r=this._milliseconds;if(e=ae(e),"month"===e||"quarter"===e||"year"===e)switch(t=this._days+r/864e5,n=this._months+Ya(t),e){case"month":return n;case"quarter":return n/3;case"year":return n/12}else switch(t=this._days+Math.round(Ca(this._months)),e){case"week":return t/7+r/6048e5;case"day":return t+r/864e5;case"hour":return 24*t+r/36e5;case"minute":return 1440*t+r/6e4;case"second":return 86400*t+r/1e3;case"millisecond":return Math.floor(864e5*t)+r;default:throw new Error("Unknown unit "+e)}}function Aa(){return this.isValid()?this._milliseconds+864e5*this._days+this._months%12*2592e6+31536e6*fe(this._months/12):NaN}function Oa(e){return function(){return this.as(e)}}var Pa=Oa("ms"),Fa=Oa("s"),Ia=Oa("m"),ja=Oa("h"),Ha=Oa("d"),Ra=Oa("w"),Na=Oa("M"),$a=Oa("Q"),za=Oa("y");function Ba(){return Yr(this)}function Wa(e){return e=ae(e),this.isValid()?this[e+"s"]():NaN}function Ua(e){return function(){return this.isValid()?this._data[e]:NaN}}var Va=Ua("milliseconds"),qa=Ua("seconds"),Ga=Ua("minutes"),Ja=Ua("hours"),Ka=Ua("days"),Xa=Ua("months"),Qa=Ua("years");function Za(){return de(this.days()/7)}var eo=Math.round,to={ss:44,s:45,m:45,h:22,d:26,w:null,M:11};function no(e,t,n,r,i){return i.relativeTime(t||1,!!n,e,r)}function ro(e,t,n,r){var i=Yr(e).abs(),a=eo(i.as("s")),o=eo(i.as("m")),s=eo(i.as("h")),l=eo(i.as("d")),u=eo(i.as("M")),c=eo(i.as("w")),d=eo(i.as("y")),f=a<=n.ss&&["s",a]||a<n.s&&["ss",a]||o<=1&&["m"]||o<n.m&&["mm",o]||s<=1&&["h"]||s<n.h&&["hh",s]||l<=1&&["d"]||l<n.d&&["dd",l];return null!=n.w&&(f=f||c<=1&&["w"]||c<n.w&&["ww",c]),f=f||u<=1&&["M"]||u<n.M&&["MM",u]||d<=1&&["y"]||["yy",d],f[2]=t,f[3]=+e>0,f[4]=r,no.apply(null,f)}function io(e){return void 0===e?eo:"function"===typeof e&&(eo=e,!0)}function ao(e,t){return void 0!==to[e]&&(void 0===t?to[e]:(to[e]=t,"s"===e&&(to.ss=t-1),!0))}function oo(e,t){if(!this.isValid())return this.localeData().invalidDate();var n,r,i=!1,a=to;return"object"===typeof e&&(t=e,e=!1),"boolean"===typeof e&&(i=e),"object"===typeof t&&(a=Object.assign({},to,t),null!=t.s&&null==t.ss&&(a.ss=t.s-1)),n=this.localeData(),r=ro(this,!i,a,n),i&&(r=n.pastFuture(+this,r)),n.postformat(r)}var so=Math.abs;function lo(e){return(e>0)-(e<0)||+e}function uo(){if(!this.isValid())return this.localeData().invalidDate();var e,t,n,r,i,a,o,s,l=so(this._milliseconds)/1e3,u=so(this._days),c=so(this._months),d=this.asSeconds();return d?(e=de(l/60),t=de(e/60),l%=60,e%=60,n=de(c/12),c%=12,r=l?l.toFixed(3).replace(/\.?0+$/,""):"",i=d<0?"-":"",a=lo(this._months)!==lo(d)?"-":"",o=lo(this._days)!==lo(d)?"-":"",s=lo(this._milliseconds)!==lo(d)?"-":"",i+"P"+(n?a+n+"Y":"")+(c?a+c+"M":"")+(u?o+u+"D":"")+(t||e||l?"T":"")+(t?s+t+"H":"")+(e?s+e+"M":"")+(l?s+r+"S":"")):"P0D"}var co=sr.prototype;return co.isValid=ar,co.abs=wa,co.add=xa,co.subtract=Sa,co.as=Ea,co.asMilliseconds=Pa,co.asSeconds=Fa,co.asMinutes=Ia,co.asHours=ja,co.asDays=Ha,co.asWeeks=Ra,co.asMonths=Na,co.asQuarters=$a,co.asYears=za,co.valueOf=Aa,co._bubble=Da,co.clone=Ba,co.get=Wa,co.milliseconds=Va,co.seconds=qa,co.minutes=Ga,co.hours=Ja,co.days=Ka,co.weeks=Za,co.months=Xa,co.years=Qa,co.humanize=oo,co.toISOString=uo,co.toString=uo,co.toJSON=uo,co.locale=si,co.localeData=ui,co.toIsoString=T("toIsoString() is deprecated. Please use toISOString() instead (notice the capitals)",uo),co.lang=li,z("X",0,0,"unix"),z("x",0,0,"valueOf"),Ie("x",Ee),Ie("X",Pe),$e("X",(function(e,t,n){n._d=new Date(1e3*parseFloat(e))})),$e("x",(function(e,t,n){n._d=new Date(fe(e))})),
//! moment.js
a.version="2.29.1",o(Kn),a.fn=ua,a.min=er,a.max=tr,a.now=nr,a.utc=_,a.unix=ca,a.months=va,a.isDate=h,a.locale=_n,a.invalid=y,a.duration=Yr,a.isMoment=x,a.weekdays=ba,a.parseZone=da,a.localeData=bn,a.isDuration=lr,a.monthsShort=ga,a.weekdaysMin=Ma,a.defineLocale=vn,a.updateLocale=gn,a.locales=yn,a.weekdaysShort=ya,a.normalizeUnits=ae,a.relativeTimeRounding=io,a.relativeTimeThreshold=ao,a.calendarFormat=zr,a.prototype=ua,a.HTML5_FMT={DATETIME_LOCAL:"YYYY-MM-DDTHH:mm",DATETIME_LOCAL_SECONDS:"YYYY-MM-DDTHH:mm:ss",DATETIME_LOCAL_MS:"YYYY-MM-DDTHH:mm:ss.SSS",DATE:"YYYY-MM-DD",TIME:"HH:mm",TIME_SECONDS:"HH:mm:ss",TIME_MS:"HH:mm:ss.SSS",WEEK:"GGGG-[W]WW",MONTH:"YYYY-MM"},a}))}).call(this,n("62e4")(e))},c345:function(e,t,n){(function(t){!function(t,n){e.exports=n()}("undefined"!=typeof self&&self,(function(){return function(e){function t(r){if(n[r])return n[r].exports;var i=n[r]={i:r,l:!1,exports:{}};return e[r].call(i.exports,i,i.exports,t),i.l=!0,i.exports}var n={};return t.m=e,t.c=n,t.d=function(e,n,r){t.o(e,n)||Object.defineProperty(e,n,{configurable:!1,enumerable:!0,get:r})},t.n=function(e){var n=e&&e.__esModule?function(){return e.default}:function(){return e};return t.d(n,"a",n),n},t.o=function(e,t){return Object.prototype.hasOwnProperty.call(e,t)},t.p="",t(t.s=22)}([function(e,t){function n(e,t){var n=e[1]||"",i=e[3];if(!i)return n;if(t&&"function"==typeof btoa){var a=r(i);return[n].concat(i.sources.map((function(e){return"/*# sourceURL="+i.sourceRoot+e+" */"}))).concat([a]).join("\n")}return[n].join("\n")}function r(e){return"/*# sourceMappingURL=data:application/json;charset=utf-8;base64,"+btoa(unescape(encodeURIComponent(JSON.stringify(e))))+" */"}e.exports=function(e){var t=[];return t.toString=function(){return this.map((function(t){var r=n(t,e);return t[2]?"@media "+t[2]+"{"+r+"}":r})).join("")},t.i=function(e,n){"string"==typeof e&&(e=[[null,e,""]]);for(var r={},i=0;i<this.length;i++){var a=this[i][0];"number"==typeof a&&(r[a]=!0)}for(i=0;i<e.length;i++){var o=e[i];"number"==typeof o[0]&&r[o[0]]||(n&&!o[2]?o[2]=n:n&&(o[2]="("+o[2]+") and ("+n+")"),t.push(o))}},t}},function(e,t,n){function r(e){for(var t=0;t<e.length;t++){var n=e[t],r=c[n.id];if(r){r.refs++;for(var i=0;i<r.parts.length;i++)r.parts[i](n.parts[i]);for(;i<n.parts.length;i++)r.parts.push(a(n.parts[i]));r.parts.length>n.parts.length&&(r.parts.length=n.parts.length)}else{var o=[];for(i=0;i<n.parts.length;i++)o.push(a(n.parts[i]));c[n.id]={id:n.id,refs:1,parts:o}}}}function i(){var e=document.createElement("style");return e.type="text/css",d.appendChild(e),e}function a(e){var t,n,r=document.querySelector("style["+v+'~="'+e.id+'"]');if(r){if(p)return m;r.parentNode.removeChild(r)}if(g){var a=h++;r=f||(f=i()),t=o.bind(null,r,a,!1),n=o.bind(null,r,a,!0)}else r=i(),t=s.bind(null,r),n=function(){r.parentNode.removeChild(r)};return t(e),function(r){if(r){if(r.css===e.css&&r.media===e.media&&r.sourceMap===e.sourceMap)return;t(e=r)}else n()}}function o(e,t,n,r){var i=n?"":r.css;if(e.styleSheet)e.styleSheet.cssText=b(t,i);else{var a=document.createTextNode(i),o=e.childNodes;o[t]&&e.removeChild(o[t]),o.length?e.insertBefore(a,o[t]):e.appendChild(a)}}function s(e,t){var n=t.css,r=t.media,i=t.sourceMap;if(r&&e.setAttribute("media",r),_.ssrId&&e.setAttribute(v,t.id),i&&(n+="\n/*# sourceURL="+i.sources[0]+" */",n+="\n/*# sourceMappingURL=data:application/json;base64,"+btoa(unescape(encodeURIComponent(JSON.stringify(i))))+" */"),e.styleSheet)e.styleSheet.cssText=n;else{for(;e.firstChild;)e.removeChild(e.firstChild);e.appendChild(document.createTextNode(n))}}var l="undefined"!=typeof document;if("undefined"!=typeof DEBUG&&DEBUG&&!l)throw new Error("vue-style-loader cannot be used in a non-browser environment. Use { target: 'node' } in your Webpack config to indicate a server-rendering environment.");var u=n(26),c={},d=l&&(document.head||document.getElementsByTagName("head")[0]),f=null,h=0,p=!1,m=function(){},_=null,v="data-vue-ssr-id",g="undefined"!=typeof navigator&&/msie [6-9]\b/.test(navigator.userAgent.toLowerCase());e.exports=function(e,t,n,i){p=n,_=i||{};var a=u(e,t);return r(a),function(t){for(var n=[],i=0;i<a.length;i++){var o=a[i],s=c[o.id];s.refs--,n.push(s)}t?(a=u(e,t),r(a)):a=[];for(i=0;i<n.length;i++){s=n[i];if(0===s.refs){for(var l=0;l<s.parts.length;l++)s.parts[l]();delete c[s.id]}}}};var b=function(){var e=[];return function(t,n){return e[t]=n,e.filter(Boolean).join("\n")}}()},function(e,t){e.exports=function(e,t,n,r,i,a){var o,s=e=e||{},l=typeof e.default;"object"!==l&&"function"!==l||(o=e,s=e.default);var u,c="function"==typeof s?s.options:s;if(t&&(c.render=t.render,c.staticRenderFns=t.staticRenderFns,c._compiled=!0),n&&(c.functional=!0),i&&(c._scopeId=i),a?(u=function(e){e=e||this.$vnode&&this.$vnode.ssrContext||this.parent&&this.parent.$vnode&&this.parent.$vnode.ssrContext,e||"undefined"==typeof __VUE_SSR_CONTEXT__||(e=__VUE_SSR_CONTEXT__),r&&r.call(this,e),e&&e._registeredComponents&&e._registeredComponents.add(a)},c._ssrRegister=u):r&&(u=r),u){var d=c.functional,f=d?c.render:c.beforeCreate;d?(c._injectStyles=u,c.render=function(e,t){return u.call(t),f(e,t)}):c.beforeCreate=f?[].concat(f,u):[u]}return{esModule:o,exports:s,options:c}}},function(e,t,n){"use strict";function r(e,t){var n,r=e&&e.a;!(n=e&&e.hsl?(0,a.default)(e.hsl):e&&e.hex&&e.hex.length>0?(0,a.default)(e.hex):e&&e.hsv?(0,a.default)(e.hsv):e&&e.rgba?(0,a.default)(e.rgba):e&&e.rgb?(0,a.default)(e.rgb):(0,a.default)(e))||void 0!==n._a&&null!==n._a||n.setAlpha(r||1);var i=n.toHsl(),o=n.toHsv();return 0===i.s&&(o.h=i.h=e.h||e.hsl&&e.hsl.h||t||0),{hsl:i,hex:n.toHexString().toUpperCase(),hex8:n.toHex8String().toUpperCase(),rgba:n.toRgb(),hsv:o,oldHue:e.h||t||i.h,source:e.source,a:e.a||n.getAlpha()}}Object.defineProperty(t,"__esModule",{value:!0});var i=n(27),a=function(e){return e&&e.__esModule?e:{default:e}}(i);t.default={props:["value"],data:function(){return{val:r(this.value)}},computed:{colors:{get:function(){return this.val},set:function(e){this.val=e,this.$emit("input",e)}}},watch:{value:function(e){this.val=r(e)}},methods:{colorChange:function(e,t){this.oldHue=this.colors.hsl.h,this.colors=r(e,t||this.oldHue)},isValidHex:function(e){return(0,a.default)(e).isValid()},simpleCheckForValidColor:function(e){for(var t=["r","g","b","a","h","s","l","v"],n=0,r=0,i=0;i<t.length;i++){var a=t[i];e[a]&&(n++,isNaN(e[a])||r++)}if(n===r)return e},paletteUpperCase:function(e){return e.map((function(e){return e.toUpperCase()}))},isTransparent:function(e){return 0===(0,a.default)(e).getAlpha()}}}},function(e,t,n){"use strict";function r(e){l||n(28)}Object.defineProperty(t,"__esModule",{value:!0});var i=n(10),a=n.n(i);for(var o in i)"default"!==o&&function(e){n.d(t,e,(function(){return i[e]}))}(o);var s=n(30),l=!1,u=n(2),c=r,d=u(a.a,s.a,!1,c,null,null);d.options.__file="src/components/common/EditableInput.vue",t.default=d.exports},function(e,t,n){"use strict";function r(e){l||n(43)}Object.defineProperty(t,"__esModule",{value:!0});var i=n(14),a=n.n(i);for(var o in i)"default"!==o&&function(e){n.d(t,e,(function(){return i[e]}))}(o);var s=n(45),l=!1,u=n(2),c=r,d=u(a.a,s.a,!1,c,null,null);d.options.__file="src/components/common/Hue.vue",t.default=d.exports},function(e,t,n){"use strict";function r(e){l||n(55)}Object.defineProperty(t,"__esModule",{value:!0});var i=n(17),a=n.n(i);for(var o in i)"default"!==o&&function(e){n.d(t,e,(function(){return i[e]}))}(o);var s=n(59),l=!1,u=n(2),c=r,d=u(a.a,s.a,!1,c,null,null);d.options.__file="src/components/common/Saturation.vue",t.default=d.exports},function(e,t,n){"use strict";function r(e){l||n(60)}Object.defineProperty(t,"__esModule",{value:!0});var i=n(18),a=n.n(i);for(var o in i)"default"!==o&&function(e){n.d(t,e,(function(){return i[e]}))}(o);var s=n(65),l=!1,u=n(2),c=r,d=u(a.a,s.a,!1,c,null,null);d.options.__file="src/components/common/Alpha.vue",t.default=d.exports},function(e,t,n){"use strict";function r(e){l||n(62)}Object.defineProperty(t,"__esModule",{value:!0});var i=n(19),a=n.n(i);for(var o in i)"default"!==o&&function(e){n.d(t,e,(function(){return i[e]}))}(o);var s=n(64),l=!1,u=n(2),c=r,d=u(a.a,s.a,!1,c,null,null);d.options.__file="src/components/common/Checkboard.vue",t.default=d.exports},function(e,t,n){"use strict";function r(e){return e&&e.__esModule?e:{default:e}}Object.defineProperty(t,"__esModule",{value:!0});var i=n(3),a=r(i),o=n(4),s=r(o),l=["#4D4D4D","#999999","#FFFFFF","#F44E3B","#FE9200","#FCDC00","#DBDF00","#A4DD00","#68CCCA","#73D8FF","#AEA1FF","#FDA1FF","#333333","#808080","#CCCCCC","#D33115","#E27300","#FCC400","#B0BC00","#68BC00","#16A5A5","#009CE0","#7B64FF","#FA28FF","#000000","#666666","#B3B3B3","#9F0500","#C45100","#FB9E00","#808900","#194D33","#0C797D","#0062B1","#653294","#AB149E"];t.default={name:"Compact",mixins:[a.default],props:{palette:{type:Array,default:function(){return l}}},components:{"ed-in":s.default},computed:{pick:function(){return this.colors.hex.toUpperCase()}},methods:{handlerClick:function(e){this.colorChange({hex:e,source:"hex"})}}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default={name:"editableInput",props:{label:String,labelText:String,desc:String,value:[String,Number],max:Number,min:Number,arrowOffset:{type:Number,default:1}},computed:{val:{get:function(){return this.value},set:function(e){if(!(void 0!==this.max&&+e>this.max))return e;this.$refs.input.value=this.max}},labelId:function(){return"input__label__"+this.label+"__"+Math.random().toString().slice(2,5)},labelSpanText:function(){return this.labelText||this.label}},methods:{update:function(e){this.handleChange(e.target.value)},handleChange:function(e){var t={};t[this.label]=e,(void 0===t.hex&&void 0===t["#"]||e.length>5)&&this.$emit("change",t)},handleKeyDown:function(e){var t=this.val,n=Number(t);if(n){var r=this.arrowOffset||1;38===e.keyCode&&(t=n+r,this.handleChange(t),e.preventDefault()),40===e.keyCode&&(t=n-r,this.handleChange(t),e.preventDefault())}}}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(3),i=function(e){return e&&e.__esModule?e:{default:e}}(r),a=["#FFFFFF","#F2F2F2","#E6E6E6","#D9D9D9","#CCCCCC","#BFBFBF","#B3B3B3","#A6A6A6","#999999","#8C8C8C","#808080","#737373","#666666","#595959","#4D4D4D","#404040","#333333","#262626","#0D0D0D","#000000"];t.default={name:"Grayscale",mixins:[i.default],props:{palette:{type:Array,default:function(){return a}}},components:{},computed:{pick:function(){return this.colors.hex.toUpperCase()}},methods:{handlerClick:function(e){this.colorChange({hex:e,source:"hex"})}}}},function(e,t,n){"use strict";function r(e){return e&&e.__esModule?e:{default:e}}Object.defineProperty(t,"__esModule",{value:!0});var i=n(4),a=r(i),o=n(3),s=r(o);t.default={name:"Material",mixins:[s.default],components:{"ed-in":a.default},methods:{onChange:function(e){e&&(e.hex?this.isValidHex(e.hex)&&this.colorChange({hex:e.hex,source:"hex"}):(e.r||e.g||e.b)&&this.colorChange({r:e.r||this.colors.rgba.r,g:e.g||this.colors.rgba.g,b:e.b||this.colors.rgba.b,a:e.a||this.colors.rgba.a,source:"rgba"}))}}}},function(e,t,n){"use strict";function r(e){return e&&e.__esModule?e:{default:e}}Object.defineProperty(t,"__esModule",{value:!0});var i=n(3),a=r(i),o=n(5),s=r(o);t.default={name:"Slider",mixins:[a.default],props:{swatches:{type:Array,default:function(){return[".80",".65",".50",".35",".20"]}}},components:{hue:s.default},computed:{activeOffset:function(){var e=this.swatches.includes("0"),t=this.swatches.includes("1"),n=this.colors.hsl;return Math.round(100*n.s)/100==.5?Math.round(100*n.l)/100:e&&0===n.l?0:t&&1===n.l?1:-1}},methods:{hueChange:function(e){this.colorChange(e)},handleSwClick:function(e,t){this.colorChange({h:this.colors.hsl.h,s:.5,l:t,source:"hsl"})}}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),t.default={name:"Hue",props:{value:Object,direction:{type:String,default:"horizontal"}},data:function(){return{oldHue:0,pullDirection:""}},computed:{colors:function(){var e=this.value.hsl.h;return 0!==e&&e-this.oldHue>0&&(this.pullDirection="right"),0!==e&&e-this.oldHue<0&&(this.pullDirection="left"),this.oldHue=e,this.value},directionClass:function(){return{"vc-hue--horizontal":"horizontal"===this.direction,"vc-hue--vertical":"vertical"===this.direction}},pointerTop:function(){return"vertical"===this.direction?0===this.colors.hsl.h&&"right"===this.pullDirection?0:-100*this.colors.hsl.h/360+100+"%":0},pointerLeft:function(){return"vertical"===this.direction?0:0===this.colors.hsl.h&&"right"===this.pullDirection?"100%":100*this.colors.hsl.h/360+"%"}},methods:{handleChange:function(e,t){!t&&e.preventDefault();var n,r,i=this.$refs.container,a=i.clientWidth,o=i.clientHeight,s=i.getBoundingClientRect().left+window.pageXOffset,l=i.getBoundingClientRect().top+window.pageYOffset,u=e.pageX||(e.touches?e.touches[0].pageX:0),c=e.pageY||(e.touches?e.touches[0].pageY:0),d=u-s,f=c-l;"vertical"===this.direction?(f<0?n=360:f>o?n=0:(r=-100*f/o+100,n=360*r/100),this.colors.hsl.h!==n&&this.$emit("change",{h:n,s:this.colors.hsl.s,l:this.colors.hsl.l,a:this.colors.hsl.a,source:"hsl"})):(d<0?n=0:d>a?n=360:(r=100*d/a,n=360*r/100),this.colors.hsl.h!==n&&this.$emit("change",{h:n,s:this.colors.hsl.s,l:this.colors.hsl.l,a:this.colors.hsl.a,source:"hsl"}))},handleMouseDown:function(e){this.handleChange(e,!0),window.addEventListener("mousemove",this.handleChange),window.addEventListener("mouseup",this.handleMouseUp)},handleMouseUp:function(e){this.unbindEventListeners()},unbindEventListeners:function(){window.removeEventListener("mousemove",this.handleChange),window.removeEventListener("mouseup",this.handleMouseUp)}}}},function(e,t,n){"use strict";function r(e){return e&&e.__esModule?e:{default:e}}Object.defineProperty(t,"__esModule",{value:!0});var i=n(50),a=r(i),o=n(3),s=r(o),l=["red","pink","purple","deepPurple","indigo","blue","lightBlue","cyan","teal","green","lightGreen","lime","yellow","amber","orange","deepOrange","brown","blueGrey","black"],u=["900","700","500","300","100"],c=function(){var e=[];return l.forEach((function(t){var n=[];"black"===t.toLowerCase()||"white"===t.toLowerCase()?n=n.concat(["#000000","#FFFFFF"]):u.forEach((function(e){var r=a.default[t][e];n.push(r.toUpperCase())})),e.push(n)})),e}();t.default={name:"Swatches",mixins:[s.default],props:{palette:{type:Array,default:function(){return c}}},computed:{pick:function(){return this.colors.hex}},methods:{equal:function(e){return e.toLowerCase()===this.colors.hex.toLowerCase()},handlerClick:function(e){this.colorChange({hex:e,source:"hex"})}}}},function(e,t,n){"use strict";function r(e){return e&&e.__esModule?e:{default:e}}Object.defineProperty(t,"__esModule",{value:!0});var i=n(3),a=r(i),o=n(4),s=r(o),l=n(6),u=r(l),c=n(5),d=r(c),f=n(7),h=r(f);t.default={name:"Photoshop",mixins:[a.default],props:{head:{type:String,default:"Color Picker"},disableFields:{type:Boolean,default:!1},hasResetButton:{type:Boolean,default:!1},acceptLabel:{type:String,default:"OK"},cancelLabel:{type:String,default:"Cancel"},resetLabel:{type:String,default:"Reset"},newLabel:{type:String,default:"new"},currentLabel:{type:String,default:"current"}},components:{saturation:u.default,hue:d.default,alpha:h.default,"ed-in":s.default},data:function(){return{currentColor:"#FFF"}},computed:{hsv:function(){var e=this.colors.hsv;return{h:e.h.toFixed(),s:(100*e.s).toFixed(),v:(100*e.v).toFixed()}},hex:function(){var e=this.colors.hex;return e&&e.replace("#","")}},created:function(){this.currentColor=this.colors.hex},methods:{childChange:function(e){this.colorChange(e)},inputChange:function(e){e&&(e["#"]?this.isValidHex(e["#"])&&this.colorChange({hex:e["#"],source:"hex"}):e.r||e.g||e.b||e.a?this.colorChange({r:e.r||this.colors.rgba.r,g:e.g||this.colors.rgba.g,b:e.b||this.colors.rgba.b,a:e.a||this.colors.rgba.a,source:"rgba"}):(e.h||e.s||e.v)&&this.colorChange({h:e.h||this.colors.hsv.h,s:e.s/100||this.colors.hsv.s,v:e.v/100||this.colors.hsv.v,source:"hsv"}))},clickCurrentColor:function(){this.colorChange({hex:this.currentColor,source:"hex"})},handleAccept:function(){this.$emit("ok")},handleCancel:function(){this.$emit("cancel")},handleReset:function(){this.$emit("reset")}}}},function(e,t,n){"use strict";function r(e){return e&&e.__esModule?e:{default:e}}Object.defineProperty(t,"__esModule",{value:!0});var i=n(57),a=r(i),o=n(58),s=r(o);t.default={name:"Saturation",props:{value:Object},computed:{colors:function(){return this.value},bgColor:function(){return"hsl("+this.colors.hsv.h+", 100%, 50%)"},pointerTop:function(){return-100*this.colors.hsv.v+1+100+"%"},pointerLeft:function(){return 100*this.colors.hsv.s+"%"}},methods:{throttle:(0,s.default)((function(e,t){e(t)}),20,{leading:!0,trailing:!1}),handleChange:function(e,t){!t&&e.preventDefault();var n=this.$refs.container,r=n.clientWidth,i=n.clientHeight,o=n.getBoundingClientRect().left+window.pageXOffset,s=n.getBoundingClientRect().top+window.pageYOffset,l=e.pageX||(e.touches?e.touches[0].pageX:0),u=e.pageY||(e.touches?e.touches[0].pageY:0),c=(0,a.default)(l-o,0,r),d=(0,a.default)(u-s,0,i),f=c/r,h=(0,a.default)(-d/i+1,0,1);this.throttle(this.onChange,{h:this.colors.hsv.h,s:f,v:h,a:this.colors.hsv.a,source:"hsva"})},onChange:function(e){this.$emit("change",e)},handleMouseDown:function(e){window.addEventListener("mousemove",this.handleChange),window.addEventListener("mouseup",this.handleChange),window.addEventListener("mouseup",this.handleMouseUp)},handleMouseUp:function(e){this.unbindEventListeners()},unbindEventListeners:function(){window.removeEventListener("mousemove",this.handleChange),window.removeEventListener("mouseup",this.handleChange),window.removeEventListener("mouseup",this.handleMouseUp)}}}},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(8),i=function(e){return e&&e.__esModule?e:{default:e}}(r);t.default={name:"Alpha",props:{value:Object,onChange:Function},components:{checkboard:i.default},computed:{colors:function(){return this.value},gradientColor:function(){var e=this.colors.rgba,t=[e.r,e.g,e.b].join(",");return"linear-gradient(to right, rgba("+t+", 0) 0%, rgba("+t+", 1) 100%)"}},methods:{handleChange:function(e,t){!t&&e.preventDefault();var n,r=this.$refs.container,i=r.clientWidth,a=r.getBoundingClientRect().left+window.pageXOffset,o=e.pageX||(e.touches?e.touches[0].pageX:0),s=o-a;n=s<0?0:s>i?1:Math.round(100*s/i)/100,this.colors.a!==n&&this.$emit("change",{h:this.colors.hsl.h,s:this.colors.hsl.s,l:this.colors.hsl.l,a:n,source:"rgba"})},handleMouseDown:function(e){this.handleChange(e,!0),window.addEventListener("mousemove",this.handleChange),window.addEventListener("mouseup",this.handleMouseUp)},handleMouseUp:function(){this.unbindEventListeners()},unbindEventListeners:function(){window.removeEventListener("mousemove",this.handleChange),window.removeEventListener("mouseup",this.handleMouseUp)}}}},function(e,t,n){"use strict";function r(e,t,n){if("undefined"==typeof document)return null;var r=document.createElement("canvas");r.width=r.height=2*n;var i=r.getContext("2d");return i?(i.fillStyle=e,i.fillRect(0,0,r.width,r.height),i.fillStyle=t,i.fillRect(0,0,n,n),i.translate(n,n),i.fillRect(0,0,n,n),r.toDataURL()):null}function i(e,t,n){var i=e+","+t+","+n;if(a[i])return a[i];var o=r(e,t,n);return a[i]=o,o}Object.defineProperty(t,"__esModule",{value:!0});var a={};t.default={name:"Checkboard",props:{size:{type:[Number,String],default:8},white:{type:String,default:"#fff"},grey:{type:String,default:"#e6e6e6"}},computed:{bgStyle:function(){return{"background-image":"url("+i(this.white,this.grey,this.size)+")"}}}}},function(e,t,n){"use strict";function r(e){return e&&e.__esModule?e:{default:e}}Object.defineProperty(t,"__esModule",{value:!0});var i=n(3),a=r(i),o=n(4),s=r(o),l=n(6),u=r(l),c=n(5),d=r(c),f=n(7),h=r(f),p=n(8),m=r(p),_=["#D0021B","#F5A623","#F8E71C","#8B572A","#7ED321","#417505","#BD10E0","#9013FE","#4A90E2","#50E3C2","#B8E986","#000000","#4A4A4A","#9B9B9B","#FFFFFF","rgba(0,0,0,0)"];t.default={name:"Sketch",mixins:[a.default],components:{saturation:u.default,hue:d.default,alpha:h.default,"ed-in":s.default,checkboard:m.default},props:{presetColors:{type:Array,default:function(){return _}},disableAlpha:{type:Boolean,default:!1},disableFields:{type:Boolean,default:!1}},computed:{hex:function(){var e=void 0;return e=this.colors.a<1?this.colors.hex8:this.colors.hex,e.replace("#","")},activeColor:function(){var e=this.colors.rgba;return"rgba("+[e.r,e.g,e.b,e.a].join(",")+")"}},methods:{handlePreset:function(e){this.colorChange({hex:e,source:"hex"})},childChange:function(e){this.colorChange(e)},inputChange:function(e){e&&(e.hex?this.isValidHex(e.hex)&&this.colorChange({hex:e.hex,source:"hex"}):(e.r||e.g||e.b||e.a)&&this.colorChange({r:e.r||this.colors.rgba.r,g:e.g||this.colors.rgba.g,b:e.b||this.colors.rgba.b,a:e.a||this.colors.rgba.a,source:"rgba"}))}}}},function(e,t,n){"use strict";function r(e){return e&&e.__esModule?e:{default:e}}Object.defineProperty(t,"__esModule",{value:!0});var i=n(3),a=r(i),o=n(4),s=r(o),l=n(6),u=r(l),c=n(5),d=r(c),f=n(7),h=r(f),p=n(8),m=r(p);t.default={name:"Chrome",mixins:[a.default],props:{disableAlpha:{type:Boolean,default:!1},disableFields:{type:Boolean,default:!1}},components:{saturation:u.default,hue:d.default,alpha:h.default,"ed-in":s.default,checkboard:m.default},data:function(){return{fieldsIndex:0,highlight:!1}},computed:{hsl:function(){var e=this.colors.hsl,t=e.h,n=e.s,r=e.l;return{h:t.toFixed(),s:(100*n).toFixed()+"%",l:(100*r).toFixed()+"%"}},activeColor:function(){var e=this.colors.rgba;return"rgba("+[e.r,e.g,e.b,e.a].join(",")+")"},hasAlpha:function(){return this.colors.a<1}},methods:{childChange:function(e){this.colorChange(e)},inputChange:function(e){if(e)if(e.hex)this.isValidHex(e.hex)&&this.colorChange({hex:e.hex,source:"hex"});else if(e.r||e.g||e.b||e.a)this.colorChange({r:e.r||this.colors.rgba.r,g:e.g||this.colors.rgba.g,b:e.b||this.colors.rgba.b,a:e.a||this.colors.rgba.a,source:"rgba"});else if(e.h||e.s||e.l){var t=e.s?e.s.replace("%","")/100:this.colors.hsl.s,n=e.l?e.l.replace("%","")/100:this.colors.hsl.l;this.colorChange({h:e.h||this.colors.hsl.h,s:t,l:n,source:"hsl"})}},toggleViews:function(){this.fieldsIndex>=2?this.fieldsIndex=0:this.fieldsIndex++},showHighlight:function(){this.highlight=!0},hideHighlight:function(){this.highlight=!1}}}},function(e,t,n){"use strict";function r(e){return e&&e.__esModule?e:{default:e}}var i=n(23),a=r(i),o=n(32),s=r(o),l=n(36),u=r(l),c=n(40),d=r(c),f=n(47),h=r(f),p=n(52),m=r(p),_=n(67),v=r(_),g=n(71),b=r(g),y=n(7),M=r(y),k=n(8),w=r(k),L=n(4),x=r(L),S=n(5),T=r(S),D=n(6),Y=r(D),C=n(3),E=r(C),A={version:"2.7.1",Compact:a.default,Grayscale:s.default,Material:u.default,Slider:d.default,Swatches:h.default,Photoshop:m.default,Sketch:v.default,Chrome:b.default,Alpha:M.default,Checkboard:w.default,EditableInput:x.default,Hue:T.default,Saturation:Y.default,ColorMixin:E.default};e.exports=A},function(e,t,n){"use strict";function r(e){l||n(24)}Object.defineProperty(t,"__esModule",{value:!0});var i=n(9),a=n.n(i);for(var o in i)"default"!==o&&function(e){n.d(t,e,(function(){return i[e]}))}(o);var s=n(31),l=!1,u=n(2),c=r,d=u(a.a,s.a,!1,c,null,null);d.options.__file="src/components/Compact.vue",t.default=d.exports},function(e,t,n){var r=n(25);"string"==typeof r&&(r=[[e.i,r,""]]),r.locals&&(e.exports=r.locals),n(1)("6ce8a5a8",r,!1,{})},function(e,t,n){t=e.exports=n(0)(!1),t.push([e.i,"\n.vc-compact {\n  padding-top: 5px;\n  padding-left: 5px;\n  width: 245px;\n  border-radius: 2px;\n  box-sizing: border-box;\n  box-shadow: 0 2px 10px rgba(0,0,0,.12), 0 2px 5px rgba(0,0,0,.16);\n  background-color: #fff;\n}\n.vc-compact-colors {\n  overflow: hidden;\n  padding: 0;\n  margin: 0;\n}\n.vc-compact-color-item {\n  list-style: none;\n  width: 15px;\n  height: 15px;\n  float: left;\n  margin-right: 5px;\n  margin-bottom: 5px;\n  position: relative;\n  cursor: pointer;\n}\n.vc-compact-color-item--white {\n  box-shadow: inset 0 0 0 1px #ddd;\n}\n.vc-compact-color-item--white .vc-compact-dot {\n  background: #000;\n}\n.vc-compact-dot {\n  position: absolute;\n  top: 5px;\n  right: 5px;\n  bottom: 5px;\n  left: 5px;\n  border-radius: 50%;\n  opacity: 1;\n  background: #fff;\n}\n",""])},function(e,t){e.exports=function(e,t){for(var n=[],r={},i=0;i<t.length;i++){var a=t[i],o=a[0],s=a[1],l=a[2],u=a[3],c={id:e+":"+i,css:s,media:l,sourceMap:u};r[o]?r[o].parts.push(c):n.push(r[o]={id:o,parts:[c]})}return n}},function(e,t,n){var r;!function(i){function a(e,t){if(e=e||"",t=t||{},e instanceof a)return e;if(!(this instanceof a))return new a(e,t);var n=o(e);this._originalInput=e,this._r=n.r,this._g=n.g,this._b=n.b,this._a=n.a,this._roundA=W(100*this._a)/100,this._format=t.format||n.format,this._gradientType=t.gradientType,this._r<1&&(this._r=W(this._r)),this._g<1&&(this._g=W(this._g)),this._b<1&&(this._b=W(this._b)),this._ok=n.ok,this._tc_id=B++}function o(e){var t={r:0,g:0,b:0},n=1,r=null,i=null,a=null,o=!1,l=!1;return"string"==typeof e&&(e=R(e)),"object"==typeof e&&(H(e.r)&&H(e.g)&&H(e.b)?(t=s(e.r,e.g,e.b),o=!0,l="%"===String(e.r).substr(-1)?"prgb":"rgb"):H(e.h)&&H(e.s)&&H(e.v)?(r=F(e.s),i=F(e.v),t=d(e.h,r,i),o=!0,l="hsv"):H(e.h)&&H(e.s)&&H(e.l)&&(r=F(e.s),a=F(e.l),t=u(e.h,r,a),o=!0,l="hsl"),e.hasOwnProperty("a")&&(n=e.a)),n=D(n),{ok:o,format:e.format||l,r:U(255,V(t.r,0)),g:U(255,V(t.g,0)),b:U(255,V(t.b,0)),a:n}}function s(e,t,n){return{r:255*Y(e,255),g:255*Y(t,255),b:255*Y(n,255)}}function l(e,t,n){e=Y(e,255),t=Y(t,255),n=Y(n,255);var r,i,a=V(e,t,n),o=U(e,t,n),s=(a+o)/2;if(a==o)r=i=0;else{var l=a-o;switch(i=s>.5?l/(2-a-o):l/(a+o),a){case e:r=(t-n)/l+(t<n?6:0);break;case t:r=(n-e)/l+2;break;case n:r=(e-t)/l+4}r/=6}return{h:r,s:i,l:s}}function u(e,t,n){function r(e,t,n){return n<0&&(n+=1),n>1&&(n-=1),n<1/6?e+6*(t-e)*n:n<.5?t:n<2/3?e+(t-e)*(2/3-n)*6:e}var i,a,o;if(e=Y(e,360),t=Y(t,100),n=Y(n,100),0===t)i=a=o=n;else{var s=n<.5?n*(1+t):n+t-n*t,l=2*n-s;i=r(l,s,e+1/3),a=r(l,s,e),o=r(l,s,e-1/3)}return{r:255*i,g:255*a,b:255*o}}function c(e,t,n){e=Y(e,255),t=Y(t,255),n=Y(n,255);var r,i,a=V(e,t,n),o=U(e,t,n),s=a,l=a-o;if(i=0===a?0:l/a,a==o)r=0;else{switch(a){case e:r=(t-n)/l+(t<n?6:0);break;case t:r=(n-e)/l+2;break;case n:r=(e-t)/l+4}r/=6}return{h:r,s:i,v:s}}function d(e,t,n){e=6*Y(e,360),t=Y(t,100),n=Y(n,100);var r=i.floor(e),a=e-r,o=n*(1-t),s=n*(1-a*t),l=n*(1-(1-a)*t),u=r%6;return{r:255*[n,s,o,o,l,n][u],g:255*[l,n,n,s,o,o][u],b:255*[o,o,l,n,n,s][u]}}function f(e,t,n,r){var i=[P(W(e).toString(16)),P(W(t).toString(16)),P(W(n).toString(16))];return r&&i[0].charAt(0)==i[0].charAt(1)&&i[1].charAt(0)==i[1].charAt(1)&&i[2].charAt(0)==i[2].charAt(1)?i[0].charAt(0)+i[1].charAt(0)+i[2].charAt(0):i.join("")}function h(e,t,n,r,i){var a=[P(W(e).toString(16)),P(W(t).toString(16)),P(W(n).toString(16)),P(I(r))];return i&&a[0].charAt(0)==a[0].charAt(1)&&a[1].charAt(0)==a[1].charAt(1)&&a[2].charAt(0)==a[2].charAt(1)&&a[3].charAt(0)==a[3].charAt(1)?a[0].charAt(0)+a[1].charAt(0)+a[2].charAt(0)+a[3].charAt(0):a.join("")}function p(e,t,n,r){return[P(I(r)),P(W(e).toString(16)),P(W(t).toString(16)),P(W(n).toString(16))].join("")}function m(e,t){t=0===t?0:t||10;var n=a(e).toHsl();return n.s-=t/100,n.s=C(n.s),a(n)}function _(e,t){t=0===t?0:t||10;var n=a(e).toHsl();return n.s+=t/100,n.s=C(n.s),a(n)}function v(e){return a(e).desaturate(100)}function g(e,t){t=0===t?0:t||10;var n=a(e).toHsl();return n.l+=t/100,n.l=C(n.l),a(n)}function b(e,t){t=0===t?0:t||10;var n=a(e).toRgb();return n.r=V(0,U(255,n.r-W(-t/100*255))),n.g=V(0,U(255,n.g-W(-t/100*255))),n.b=V(0,U(255,n.b-W(-t/100*255))),a(n)}function y(e,t){t=0===t?0:t||10;var n=a(e).toHsl();return n.l-=t/100,n.l=C(n.l),a(n)}function M(e,t){var n=a(e).toHsl(),r=(n.h+t)%360;return n.h=r<0?360+r:r,a(n)}function k(e){var t=a(e).toHsl();return t.h=(t.h+180)%360,a(t)}function w(e){var t=a(e).toHsl(),n=t.h;return[a(e),a({h:(n+120)%360,s:t.s,l:t.l}),a({h:(n+240)%360,s:t.s,l:t.l})]}function L(e){var t=a(e).toHsl(),n=t.h;return[a(e),a({h:(n+90)%360,s:t.s,l:t.l}),a({h:(n+180)%360,s:t.s,l:t.l}),a({h:(n+270)%360,s:t.s,l:t.l})]}function x(e){var t=a(e).toHsl(),n=t.h;return[a(e),a({h:(n+72)%360,s:t.s,l:t.l}),a({h:(n+216)%360,s:t.s,l:t.l})]}function S(e,t,n){t=t||6,n=n||30;var r=a(e).toHsl(),i=360/n,o=[a(e)];for(r.h=(r.h-(i*t>>1)+720)%360;--t;)r.h=(r.h+i)%360,o.push(a(r));return o}function T(e,t){t=t||6;for(var n=a(e).toHsv(),r=n.h,i=n.s,o=n.v,s=[],l=1/t;t--;)s.push(a({h:r,s:i,v:o})),o=(o+l)%1;return s}function D(e){return e=parseFloat(e),(isNaN(e)||e<0||e>1)&&(e=1),e}function Y(e,t){A(e)&&(e="100%");var n=O(e);return e=U(t,V(0,parseFloat(e))),n&&(e=parseInt(e*t,10)/100),i.abs(e-t)<1e-6?1:e%t/parseFloat(t)}function C(e){return U(1,V(0,e))}function E(e){return parseInt(e,16)}function A(e){return"string"==typeof e&&-1!=e.indexOf(".")&&1===parseFloat(e)}function O(e){return"string"==typeof e&&-1!=e.indexOf("%")}function P(e){return 1==e.length?"0"+e:""+e}function F(e){return e<=1&&(e=100*e+"%"),e}function I(e){return i.round(255*parseFloat(e)).toString(16)}function j(e){return E(e)/255}function H(e){return!!K.CSS_UNIT.exec(e)}function R(e){e=e.replace($,"").replace(z,"").toLowerCase();var t,n=!1;if(G[e])e=G[e],n=!0;else if("transparent"==e)return{r:0,g:0,b:0,a:0,format:"name"};return(t=K.rgb.exec(e))?{r:t[1],g:t[2],b:t[3]}:(t=K.rgba.exec(e))?{r:t[1],g:t[2],b:t[3],a:t[4]}:(t=K.hsl.exec(e))?{h:t[1],s:t[2],l:t[3]}:(t=K.hsla.exec(e))?{h:t[1],s:t[2],l:t[3],a:t[4]}:(t=K.hsv.exec(e))?{h:t[1],s:t[2],v:t[3]}:(t=K.hsva.exec(e))?{h:t[1],s:t[2],v:t[3],a:t[4]}:(t=K.hex8.exec(e))?{r:E(t[1]),g:E(t[2]),b:E(t[3]),a:j(t[4]),format:n?"name":"hex8"}:(t=K.hex6.exec(e))?{r:E(t[1]),g:E(t[2]),b:E(t[3]),format:n?"name":"hex"}:(t=K.hex4.exec(e))?{r:E(t[1]+""+t[1]),g:E(t[2]+""+t[2]),b:E(t[3]+""+t[3]),a:j(t[4]+""+t[4]),format:n?"name":"hex8"}:!!(t=K.hex3.exec(e))&&{r:E(t[1]+""+t[1]),g:E(t[2]+""+t[2]),b:E(t[3]+""+t[3]),format:n?"name":"hex"}}function N(e){var t,n;return e=e||{level:"AA",size:"small"},t=(e.level||"AA").toUpperCase(),n=(e.size||"small").toLowerCase(),"AA"!==t&&"AAA"!==t&&(t="AA"),"small"!==n&&"large"!==n&&(n="small"),{level:t,size:n}}var $=/^\s+/,z=/\s+$/,B=0,W=i.round,U=i.min,V=i.max,q=i.random;a.prototype={isDark:function(){return this.getBrightness()<128},isLight:function(){return!this.isDark()},isValid:function(){return this._ok},getOriginalInput:function(){return this._originalInput},getFormat:function(){return this._format},getAlpha:function(){return this._a},getBrightness:function(){var e=this.toRgb();return(299*e.r+587*e.g+114*e.b)/1e3},getLuminance:function(){var e,t,n,r,a,o,s=this.toRgb();return e=s.r/255,t=s.g/255,n=s.b/255,r=e<=.03928?e/12.92:i.pow((e+.055)/1.055,2.4),a=t<=.03928?t/12.92:i.pow((t+.055)/1.055,2.4),o=n<=.03928?n/12.92:i.pow((n+.055)/1.055,2.4),.2126*r+.7152*a+.0722*o},setAlpha:function(e){return this._a=D(e),this._roundA=W(100*this._a)/100,this},toHsv:function(){var e=c(this._r,this._g,this._b);return{h:360*e.h,s:e.s,v:e.v,a:this._a}},toHsvString:function(){var e=c(this._r,this._g,this._b),t=W(360*e.h),n=W(100*e.s),r=W(100*e.v);return 1==this._a?"hsv("+t+", "+n+"%, "+r+"%)":"hsva("+t+", "+n+"%, "+r+"%, "+this._roundA+")"},toHsl:function(){var e=l(this._r,this._g,this._b);return{h:360*e.h,s:e.s,l:e.l,a:this._a}},toHslString:function(){var e=l(this._r,this._g,this._b),t=W(360*e.h),n=W(100*e.s),r=W(100*e.l);return 1==this._a?"hsl("+t+", "+n+"%, "+r+"%)":"hsla("+t+", "+n+"%, "+r+"%, "+this._roundA+")"},toHex:function(e){return f(this._r,this._g,this._b,e)},toHexString:function(e){return"#"+this.toHex(e)},toHex8:function(e){return h(this._r,this._g,this._b,this._a,e)},toHex8String:function(e){return"#"+this.toHex8(e)},toRgb:function(){return{r:W(this._r),g:W(this._g),b:W(this._b),a:this._a}},toRgbString:function(){return 1==this._a?"rgb("+W(this._r)+", "+W(this._g)+", "+W(this._b)+")":"rgba("+W(this._r)+", "+W(this._g)+", "+W(this._b)+", "+this._roundA+")"},toPercentageRgb:function(){return{r:W(100*Y(this._r,255))+"%",g:W(100*Y(this._g,255))+"%",b:W(100*Y(this._b,255))+"%",a:this._a}},toPercentageRgbString:function(){return 1==this._a?"rgb("+W(100*Y(this._r,255))+"%, "+W(100*Y(this._g,255))+"%, "+W(100*Y(this._b,255))+"%)":"rgba("+W(100*Y(this._r,255))+"%, "+W(100*Y(this._g,255))+"%, "+W(100*Y(this._b,255))+"%, "+this._roundA+")"},toName:function(){return 0===this._a?"transparent":!(this._a<1)&&(J[f(this._r,this._g,this._b,!0)]||!1)},toFilter:function(e){var t="#"+p(this._r,this._g,this._b,this._a),n=t,r=this._gradientType?"GradientType = 1, ":"";if(e){var i=a(e);n="#"+p(i._r,i._g,i._b,i._a)}return"progid:DXImageTransform.Microsoft.gradient("+r+"startColorstr="+t+",endColorstr="+n+")"},toString:function(e){var t=!!e;e=e||this._format;var n=!1,r=this._a<1&&this._a>=0;return t||!r||"hex"!==e&&"hex6"!==e&&"hex3"!==e&&"hex4"!==e&&"hex8"!==e&&"name"!==e?("rgb"===e&&(n=this.toRgbString()),"prgb"===e&&(n=this.toPercentageRgbString()),"hex"!==e&&"hex6"!==e||(n=this.toHexString()),"hex3"===e&&(n=this.toHexString(!0)),"hex4"===e&&(n=this.toHex8String(!0)),"hex8"===e&&(n=this.toHex8String()),"name"===e&&(n=this.toName()),"hsl"===e&&(n=this.toHslString()),"hsv"===e&&(n=this.toHsvString()),n||this.toHexString()):"name"===e&&0===this._a?this.toName():this.toRgbString()},clone:function(){return a(this.toString())},_applyModification:function(e,t){var n=e.apply(null,[this].concat([].slice.call(t)));return this._r=n._r,this._g=n._g,this._b=n._b,this.setAlpha(n._a),this},lighten:function(){return this._applyModification(g,arguments)},brighten:function(){return this._applyModification(b,arguments)},darken:function(){return this._applyModification(y,arguments)},desaturate:function(){return this._applyModification(m,arguments)},saturate:function(){return this._applyModification(_,arguments)},greyscale:function(){return this._applyModification(v,arguments)},spin:function(){return this._applyModification(M,arguments)},_applyCombination:function(e,t){return e.apply(null,[this].concat([].slice.call(t)))},analogous:function(){return this._applyCombination(S,arguments)},complement:function(){return this._applyCombination(k,arguments)},monochromatic:function(){return this._applyCombination(T,arguments)},splitcomplement:function(){return this._applyCombination(x,arguments)},triad:function(){return this._applyCombination(w,arguments)},tetrad:function(){return this._applyCombination(L,arguments)}},a.fromRatio=function(e,t){if("object"==typeof e){var n={};for(var r in e)e.hasOwnProperty(r)&&(n[r]="a"===r?e[r]:F(e[r]));e=n}return a(e,t)},a.equals=function(e,t){return!(!e||!t)&&a(e).toRgbString()==a(t).toRgbString()},a.random=function(){return a.fromRatio({r:q(),g:q(),b:q()})},a.mix=function(e,t,n){n=0===n?0:n||50;var r=a(e).toRgb(),i=a(t).toRgb(),o=n/100;return a({r:(i.r-r.r)*o+r.r,g:(i.g-r.g)*o+r.g,b:(i.b-r.b)*o+r.b,a:(i.a-r.a)*o+r.a})},a.readability=function(e,t){var n=a(e),r=a(t);return(i.max(n.getLuminance(),r.getLuminance())+.05)/(i.min(n.getLuminance(),r.getLuminance())+.05)},a.isReadable=function(e,t,n){var r,i,o=a.readability(e,t);switch(i=!1,r=N(n),r.level+r.size){case"AAsmall":case"AAAlarge":i=o>=4.5;break;case"AAlarge":i=o>=3;break;case"AAAsmall":i=o>=7}return i},a.mostReadable=function(e,t,n){var r,i,o,s,l=null,u=0;n=n||{},i=n.includeFallbackColors,o=n.level,s=n.size;for(var c=0;c<t.length;c++)(r=a.readability(e,t[c]))>u&&(u=r,l=a(t[c]));return a.isReadable(e,l,{level:o,size:s})||!i?l:(n.includeFallbackColors=!1,a.mostReadable(e,["#fff","#000"],n))};var G=a.names={aliceblue:"f0f8ff",antiquewhite:"faebd7",aqua:"0ff",aquamarine:"7fffd4",azure:"f0ffff",beige:"f5f5dc",bisque:"ffe4c4",black:"000",blanchedalmond:"ffebcd",blue:"00f",blueviolet:"8a2be2",brown:"a52a2a",burlywood:"deb887",burntsienna:"ea7e5d",cadetblue:"5f9ea0",chartreuse:"7fff00",chocolate:"d2691e",coral:"ff7f50",cornflowerblue:"6495ed",cornsilk:"fff8dc",crimson:"dc143c",cyan:"0ff",darkblue:"00008b",darkcyan:"008b8b",darkgoldenrod:"b8860b",darkgray:"a9a9a9",darkgreen:"006400",darkgrey:"a9a9a9",darkkhaki:"bdb76b",darkmagenta:"8b008b",darkolivegreen:"556b2f",darkorange:"ff8c00",darkorchid:"9932cc",darkred:"8b0000",darksalmon:"e9967a",darkseagreen:"8fbc8f",darkslateblue:"483d8b",darkslategray:"2f4f4f",darkslategrey:"2f4f4f",darkturquoise:"00ced1",darkviolet:"9400d3",deeppink:"ff1493",deepskyblue:"00bfff",dimgray:"696969",dimgrey:"696969",dodgerblue:"1e90ff",firebrick:"b22222",floralwhite:"fffaf0",forestgreen:"228b22",fuchsia:"f0f",gainsboro:"dcdcdc",ghostwhite:"f8f8ff",gold:"ffd700",goldenrod:"daa520",gray:"808080",green:"008000",greenyellow:"adff2f",grey:"808080",honeydew:"f0fff0",hotpink:"ff69b4",indianred:"cd5c5c",indigo:"4b0082",ivory:"fffff0",khaki:"f0e68c",lavender:"e6e6fa",lavenderblush:"fff0f5",lawngreen:"7cfc00",lemonchiffon:"fffacd",lightblue:"add8e6",lightcoral:"f08080",lightcyan:"e0ffff",lightgoldenrodyellow:"fafad2",lightgray:"d3d3d3",lightgreen:"90ee90",lightgrey:"d3d3d3",lightpink:"ffb6c1",lightsalmon:"ffa07a",lightseagreen:"20b2aa",lightskyblue:"87cefa",lightslategray:"789",lightslategrey:"789",lightsteelblue:"b0c4de",lightyellow:"ffffe0",lime:"0f0",limegreen:"32cd32",linen:"faf0e6",magenta:"f0f",maroon:"800000",mediumaquamarine:"66cdaa",mediumblue:"0000cd",mediumorchid:"ba55d3",mediumpurple:"9370db",mediumseagreen:"3cb371",mediumslateblue:"7b68ee",mediumspringgreen:"00fa9a",mediumturquoise:"48d1cc",mediumvioletred:"c71585",midnightblue:"191970",mintcream:"f5fffa",mistyrose:"ffe4e1",moccasin:"ffe4b5",navajowhite:"ffdead",navy:"000080",oldlace:"fdf5e6",olive:"808000",olivedrab:"6b8e23",orange:"ffa500",orangered:"ff4500",orchid:"da70d6",palegoldenrod:"eee8aa",palegreen:"98fb98",paleturquoise:"afeeee",palevioletred:"db7093",papayawhip:"ffefd5",peachpuff:"ffdab9",peru:"cd853f",pink:"ffc0cb",plum:"dda0dd",powderblue:"b0e0e6",purple:"800080",rebeccapurple:"663399",red:"f00",rosybrown:"bc8f8f",royalblue:"4169e1",saddlebrown:"8b4513",salmon:"fa8072",sandybrown:"f4a460",seagreen:"2e8b57",seashell:"fff5ee",sienna:"a0522d",silver:"c0c0c0",skyblue:"87ceeb",slateblue:"6a5acd",slategray:"708090",slategrey:"708090",snow:"fffafa",springgreen:"00ff7f",steelblue:"4682b4",tan:"d2b48c",teal:"008080",thistle:"d8bfd8",tomato:"ff6347",turquoise:"40e0d0",violet:"ee82ee",wheat:"f5deb3",white:"fff",whitesmoke:"f5f5f5",yellow:"ff0",yellowgreen:"9acd32"},J=a.hexNames=function(e){var t={};for(var n in e)e.hasOwnProperty(n)&&(t[e[n]]=n);return t}(G),K=function(){var e="(?:[-\\+]?\\d*\\.\\d+%?)|(?:[-\\+]?\\d+%?)",t="[\\s|\\(]+("+e+")[,|\\s]+("+e+")[,|\\s]+("+e+")\\s*\\)?",n="[\\s|\\(]+("+e+")[,|\\s]+("+e+")[,|\\s]+("+e+")[,|\\s]+("+e+")\\s*\\)?";return{CSS_UNIT:new RegExp(e),rgb:new RegExp("rgb"+t),rgba:new RegExp("rgba"+n),hsl:new RegExp("hsl"+t),hsla:new RegExp("hsla"+n),hsv:new RegExp("hsv"+t),hsva:new RegExp("hsva"+n),hex3:/^#?([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})$/,hex6:/^#?([0-9a-fA-F]{2})([0-9a-fA-F]{2})([0-9a-fA-F]{2})$/,hex4:/^#?([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})([0-9a-fA-F]{1})$/,hex8:/^#?([0-9a-fA-F]{2})([0-9a-fA-F]{2})([0-9a-fA-F]{2})([0-9a-fA-F]{2})$/}}();void 0!==e&&e.exports?e.exports=a:void 0!==(r=function(){return a}.call(t,n,t,e))&&(e.exports=r)}(Math)},function(e,t,n){var r=n(29);"string"==typeof r&&(r=[[e.i,r,""]]),r.locals&&(e.exports=r.locals),n(1)("0f73e73c",r,!1,{})},function(e,t,n){t=e.exports=n(0)(!1),t.push([e.i,"\n.vc-editable-input {\n  position: relative;\n}\n.vc-input__input {\n  padding: 0;\n  border: 0;\n  outline: none;\n}\n.vc-input__label {\n  text-transform: capitalize;\n}\n",""])},function(e,t,n){"use strict";var r=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"vc-editable-input"},[n("input",{directives:[{name:"model",rawName:"v-model",value:e.val,expression:"val"}],ref:"input",staticClass:"vc-input__input",attrs:{"aria-labelledby":e.labelId},domProps:{value:e.val},on:{keydown:e.handleKeyDown,input:[function(t){t.target.composing||(e.val=t.target.value)},e.update]}}),e._v(" "),n("span",{staticClass:"vc-input__label",attrs:{for:e.label,id:e.labelId}},[e._v(e._s(e.labelSpanText))]),e._v(" "),n("span",{staticClass:"vc-input__desc"},[e._v(e._s(e.desc))])])},i=[];r._withStripped=!0;var a={render:r,staticRenderFns:i};t.a=a},function(e,t,n){"use strict";var r=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"vc-compact",attrs:{role:"application","aria-label":"Compact color picker"}},[n("ul",{staticClass:"vc-compact-colors",attrs:{role:"listbox"}},e._l(e.paletteUpperCase(e.palette),(function(t){return n("li",{key:t,staticClass:"vc-compact-color-item",class:{"vc-compact-color-item--white":"#FFFFFF"===t},style:{background:t},attrs:{role:"option","aria-label":"color:"+t,"aria-selected":t===e.pick},on:{click:function(n){e.handlerClick(t)}}},[n("div",{directives:[{name:"show",rawName:"v-show",value:t===e.pick,expression:"c === pick"}],staticClass:"vc-compact-dot"})])})))])},i=[];r._withStripped=!0;var a={render:r,staticRenderFns:i};t.a=a},function(e,t,n){"use strict";function r(e){l||n(33)}Object.defineProperty(t,"__esModule",{value:!0});var i=n(11),a=n.n(i);for(var o in i)"default"!==o&&function(e){n.d(t,e,(function(){return i[e]}))}(o);var s=n(35),l=!1,u=n(2),c=r,d=u(a.a,s.a,!1,c,null,null);d.options.__file="src/components/Grayscale.vue",t.default=d.exports},function(e,t,n){var r=n(34);"string"==typeof r&&(r=[[e.i,r,""]]),r.locals&&(e.exports=r.locals),n(1)("21ddbb74",r,!1,{})},function(e,t,n){t=e.exports=n(0)(!1),t.push([e.i,"\n.vc-grayscale {\n  width: 125px;\n  border-radius: 2px;\n  box-shadow: 0 2px 15px rgba(0,0,0,.12), 0 2px 10px rgba(0,0,0,.16);\n  background-color: #fff;\n}\n.vc-grayscale-colors {\n  border-radius: 2px;\n  overflow: hidden;\n  padding: 0;\n  margin: 0;\n}\n.vc-grayscale-color-item {\n  list-style: none;\n  width: 25px;\n  height: 25px;\n  float: left;\n  position: relative;\n  cursor: pointer;\n}\n.vc-grayscale-color-item--white .vc-grayscale-dot {\n  background: #000;\n}\n.vc-grayscale-dot {\n  position: absolute;\n  top: 50%;\n  left: 50%;\n  width: 6px;\n  height: 6px;\n  margin: -3px 0 0 -2px;\n  border-radius: 50%;\n  opacity: 1;\n  background: #fff;\n}\n",""])},function(e,t,n){"use strict";var r=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"vc-grayscale",attrs:{role:"application","aria-label":"Grayscale color picker"}},[n("ul",{staticClass:"vc-grayscale-colors",attrs:{role:"listbox"}},e._l(e.paletteUpperCase(e.palette),(function(t){return n("li",{key:t,staticClass:"vc-grayscale-color-item",class:{"vc-grayscale-color-item--white":"#FFFFFF"==t},style:{background:t},attrs:{role:"option","aria-label":"Color:"+t,"aria-selected":t===e.pick},on:{click:function(n){e.handlerClick(t)}}},[n("div",{directives:[{name:"show",rawName:"v-show",value:t===e.pick,expression:"c === pick"}],staticClass:"vc-grayscale-dot"})])})))])},i=[];r._withStripped=!0;var a={render:r,staticRenderFns:i};t.a=a},function(e,t,n){"use strict";function r(e){l||n(37)}Object.defineProperty(t,"__esModule",{value:!0});var i=n(12),a=n.n(i);for(var o in i)"default"!==o&&function(e){n.d(t,e,(function(){return i[e]}))}(o);var s=n(39),l=!1,u=n(2),c=r,d=u(a.a,s.a,!1,c,null,null);d.options.__file="src/components/Material.vue",t.default=d.exports},function(e,t,n){var r=n(38);"string"==typeof r&&(r=[[e.i,r,""]]),r.locals&&(e.exports=r.locals),n(1)("1ff3af73",r,!1,{})},function(e,t,n){t=e.exports=n(0)(!1),t.push([e.i,'\n.vc-material {\n  width: 98px;\n  height: 98px;\n  padding: 16px;\n  font-family: "Roboto";\n  position: relative;\n  border-radius: 2px;\n  box-shadow: 0 2px 10px rgba(0,0,0,.12), 0 2px 5px rgba(0,0,0,.16);\n  background-color: #fff;\n}\n.vc-material .vc-input__input {\n  width: 100%;\n  margin-top: 12px;\n  font-size: 15px;\n  color: #333;\n  height: 30px;\n}\n.vc-material .vc-input__label {\n  position: absolute;\n  top: 0;\n  left: 0;\n  font-size: 11px;\n  color: #999;\n  text-transform: capitalize;\n}\n.vc-material-hex {\n  border-bottom-width: 2px;\n  border-bottom-style: solid;\n}\n.vc-material-split {\n  display: flex;\n  margin-right: -10px;\n  padding-top: 11px;\n}\n.vc-material-third {\n  flex: 1;\n  padding-right: 10px;\n}\n',""])},function(e,t,n){"use strict";var r=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"vc-material",attrs:{role:"application","aria-label":"Material color picker"}},[n("ed-in",{staticClass:"vc-material-hex",style:{borderColor:e.colors.hex},attrs:{label:"hex"},on:{change:e.onChange},model:{value:e.colors.hex,callback:function(t){e.$set(e.colors,"hex",t)},expression:"colors.hex"}}),e._v(" "),n("div",{staticClass:"vc-material-split"},[n("div",{staticClass:"vc-material-third"},[n("ed-in",{attrs:{label:"r"},on:{change:e.onChange},model:{value:e.colors.rgba.r,callback:function(t){e.$set(e.colors.rgba,"r",t)},expression:"colors.rgba.r"}})],1),e._v(" "),n("div",{staticClass:"vc-material-third"},[n("ed-in",{attrs:{label:"g"},on:{change:e.onChange},model:{value:e.colors.rgba.g,callback:function(t){e.$set(e.colors.rgba,"g",t)},expression:"colors.rgba.g"}})],1),e._v(" "),n("div",{staticClass:"vc-material-third"},[n("ed-in",{attrs:{label:"b"},on:{change:e.onChange},model:{value:e.colors.rgba.b,callback:function(t){e.$set(e.colors.rgba,"b",t)},expression:"colors.rgba.b"}})],1)])],1)},i=[];r._withStripped=!0;var a={render:r,staticRenderFns:i};t.a=a},function(e,t,n){"use strict";function r(e){l||n(41)}Object.defineProperty(t,"__esModule",{value:!0});var i=n(13),a=n.n(i);for(var o in i)"default"!==o&&function(e){n.d(t,e,(function(){return i[e]}))}(o);var s=n(46),l=!1,u=n(2),c=r,d=u(a.a,s.a,!1,c,null,null);d.options.__file="src/components/Slider.vue",t.default=d.exports},function(e,t,n){var r=n(42);"string"==typeof r&&(r=[[e.i,r,""]]),r.locals&&(e.exports=r.locals),n(1)("7982aa43",r,!1,{})},function(e,t,n){t=e.exports=n(0)(!1),t.push([e.i,"\n.vc-slider {\n  position: relative;\n  width: 410px;\n}\n.vc-slider-hue-warp {\n  height: 12px;\n  position: relative;\n}\n.vc-slider-hue-warp .vc-hue-picker {\n  width: 14px;\n  height: 14px;\n  border-radius: 6px;\n  transform: translate(-7px, -2px);\n  background-color: rgb(248, 248, 248);\n  box-shadow: 0 1px 4px 0 rgba(0, 0, 0, 0.37);\n}\n.vc-slider-swatches {\n  display: flex;\n  margin-top: 20px;\n}\n.vc-slider-swatch {\n  margin-right: 1px;\n  flex: 1;\n  width: 20%;\n}\n.vc-slider-swatch:first-child {\n  margin-right: 1px;\n}\n.vc-slider-swatch:first-child .vc-slider-swatch-picker {\n  border-radius: 2px 0px 0px 2px;\n}\n.vc-slider-swatch:last-child {\n  margin-right: 0;\n}\n.vc-slider-swatch:last-child .vc-slider-swatch-picker {\n  border-radius: 0px 2px 2px 0px;\n}\n.vc-slider-swatch-picker {\n  cursor: pointer;\n  height: 12px;\n}\n.vc-slider-swatch:nth-child(n) .vc-slider-swatch-picker.vc-slider-swatch-picker--active {\n  transform: scaleY(1.8);\n  border-radius: 3.6px/2px;\n}\n.vc-slider-swatch-picker--white {\n  box-shadow: inset 0 0 0 1px #ddd;\n}\n.vc-slider-swatch-picker--active.vc-slider-swatch-picker--white {\n  box-shadow: inset 0 0 0 0.6px #ddd;\n}\n",""])},function(e,t,n){var r=n(44);"string"==typeof r&&(r=[[e.i,r,""]]),r.locals&&(e.exports=r.locals),n(1)("7c5f1a1c",r,!1,{})},function(e,t,n){t=e.exports=n(0)(!1),t.push([e.i,"\n.vc-hue {\n  position: absolute;\n  top: 0px;\n  right: 0px;\n  bottom: 0px;\n  left: 0px;\n  border-radius: 2px;\n}\n.vc-hue--horizontal {\n  background: linear-gradient(to right, #f00 0%, #ff0 17%, #0f0 33%, #0ff 50%, #00f 67%, #f0f 83%, #f00 100%);\n}\n.vc-hue--vertical {\n  background: linear-gradient(to top, #f00 0%, #ff0 17%, #0f0 33%, #0ff 50%, #00f 67%, #f0f 83%, #f00 100%);\n}\n.vc-hue-container {\n  cursor: pointer;\n  margin: 0 2px;\n  position: relative;\n  height: 100%;\n}\n.vc-hue-pointer {\n  z-index: 2;\n  position: absolute;\n}\n.vc-hue-picker {\n  cursor: pointer;\n  margin-top: 1px;\n  width: 4px;\n  border-radius: 1px;\n  height: 8px;\n  box-shadow: 0 0 2px rgba(0, 0, 0, .6);\n  background: #fff;\n  transform: translateX(-2px) ;\n}\n",""])},function(e,t,n){"use strict";var r=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{class:["vc-hue",e.directionClass]},[n("div",{ref:"container",staticClass:"vc-hue-container",attrs:{role:"slider","aria-valuenow":e.colors.hsl.h,"aria-valuemin":"0","aria-valuemax":"360"},on:{mousedown:e.handleMouseDown,touchmove:e.handleChange,touchstart:e.handleChange}},[n("div",{staticClass:"vc-hue-pointer",style:{top:e.pointerTop,left:e.pointerLeft},attrs:{role:"presentation"}},[n("div",{staticClass:"vc-hue-picker"})])])])},i=[];r._withStripped=!0;var a={render:r,staticRenderFns:i};t.a=a},function(e,t,n){"use strict";var r=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"vc-slider",attrs:{role:"application","aria-label":"Slider color picker"}},[n("div",{staticClass:"vc-slider-hue-warp"},[n("hue",{on:{change:e.hueChange},model:{value:e.colors,callback:function(t){e.colors=t},expression:"colors"}})],1),e._v(" "),n("div",{staticClass:"vc-slider-swatches",attrs:{role:"group"}},e._l(e.swatches,(function(t,r){return n("div",{key:r,staticClass:"vc-slider-swatch",attrs:{"data-index":r,"aria-label":"color:"+e.colors.hex,role:"button"},on:{click:function(n){e.handleSwClick(r,t)}}},[n("div",{staticClass:"vc-slider-swatch-picker",class:{"vc-slider-swatch-picker--active":t==e.activeOffset,"vc-slider-swatch-picker--white":"1"===t},style:{background:"hsl("+e.colors.hsl.h+", 50%, "+100*t+"%)"}})])})))])},i=[];r._withStripped=!0;var a={render:r,staticRenderFns:i};t.a=a},function(e,t,n){"use strict";function r(e){l||n(48)}Object.defineProperty(t,"__esModule",{value:!0});var i=n(15),a=n.n(i);for(var o in i)"default"!==o&&function(e){n.d(t,e,(function(){return i[e]}))}(o);var s=n(51),l=!1,u=n(2),c=r,d=u(a.a,s.a,!1,c,null,null);d.options.__file="src/components/Swatches.vue",t.default=d.exports},function(e,t,n){var r=n(49);"string"==typeof r&&(r=[[e.i,r,""]]),r.locals&&(e.exports=r.locals),n(1)("10f839a2",r,!1,{})},function(e,t,n){t=e.exports=n(0)(!1),t.push([e.i,"\n.vc-swatches {\n  width: 320px;\n  height: 240px;\n  overflow-y: scroll;\n  background-color: #fff;\n  box-shadow: 0 2px 10px rgba(0,0,0,.12), 0 2px 5px rgba(0,0,0,.16);\n}\n.vc-swatches-box {\n  padding: 16px 0 6px 16px;\n  overflow: hidden;\n}\n.vc-swatches-color-group {\n  padding-bottom: 10px;\n  width: 40px;\n  float: left;\n  margin-right: 10px;\n}\n.vc-swatches-color-it {\n  box-sizing: border-box;\n  width: 40px;\n  height: 24px;\n  cursor: pointer;\n  background: #880e4f;\n  margin-bottom: 1px;\n  overflow: hidden;\n  -ms-border-radius: 2px 2px 0 0;\n  -moz-border-radius: 2px 2px 0 0;\n  -o-border-radius: 2px 2px 0 0;\n  -webkit-border-radius: 2px 2px 0 0;\n  border-radius: 2px 2px 0 0;\n}\n.vc-swatches-color--white {\n  border: 1px solid #DDD;\n}\n.vc-swatches-pick {\n  fill: rgb(255, 255, 255);\n  margin-left: 8px;\n  display: block;\n}\n.vc-swatches-color--white .vc-swatches-pick {\n  fill: rgb(51, 51, 51);\n}\n",""])},function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0}),n.d(t,"red",(function(){return r})),n.d(t,"pink",(function(){return i})),n.d(t,"purple",(function(){return a})),n.d(t,"deepPurple",(function(){return o})),n.d(t,"indigo",(function(){return s})),n.d(t,"blue",(function(){return l})),n.d(t,"lightBlue",(function(){return u})),n.d(t,"cyan",(function(){return c})),n.d(t,"teal",(function(){return d})),n.d(t,"green",(function(){return f})),n.d(t,"lightGreen",(function(){return h})),n.d(t,"lime",(function(){return p})),n.d(t,"yellow",(function(){return m})),n.d(t,"amber",(function(){return _})),n.d(t,"orange",(function(){return v})),n.d(t,"deepOrange",(function(){return g})),n.d(t,"brown",(function(){return b})),n.d(t,"grey",(function(){return y})),n.d(t,"blueGrey",(function(){return M})),n.d(t,"darkText",(function(){return k})),n.d(t,"lightText",(function(){return w})),n.d(t,"darkIcons",(function(){return L})),n.d(t,"lightIcons",(function(){return x})),n.d(t,"white",(function(){return S})),n.d(t,"black",(function(){return T}));var r={50:"#ffebee",100:"#ffcdd2",200:"#ef9a9a",300:"#e57373",400:"#ef5350",500:"#f44336",600:"#e53935",700:"#d32f2f",800:"#c62828",900:"#b71c1c",a100:"#ff8a80",a200:"#ff5252",a400:"#ff1744",a700:"#d50000"},i={50:"#fce4ec",100:"#f8bbd0",200:"#f48fb1",300:"#f06292",400:"#ec407a",500:"#e91e63",600:"#d81b60",700:"#c2185b",800:"#ad1457",900:"#880e4f",a100:"#ff80ab",a200:"#ff4081",a400:"#f50057",a700:"#c51162"},a={50:"#f3e5f5",100:"#e1bee7",200:"#ce93d8",300:"#ba68c8",400:"#ab47bc",500:"#9c27b0",600:"#8e24aa",700:"#7b1fa2",800:"#6a1b9a",900:"#4a148c",a100:"#ea80fc",a200:"#e040fb",a400:"#d500f9",a700:"#aa00ff"},o={50:"#ede7f6",100:"#d1c4e9",200:"#b39ddb",300:"#9575cd",400:"#7e57c2",500:"#673ab7",600:"#5e35b1",700:"#512da8",800:"#4527a0",900:"#311b92",a100:"#b388ff",a200:"#7c4dff",a400:"#651fff",a700:"#6200ea"},s={50:"#e8eaf6",100:"#c5cae9",200:"#9fa8da",300:"#7986cb",400:"#5c6bc0",500:"#3f51b5",600:"#3949ab",700:"#303f9f",800:"#283593",900:"#1a237e",a100:"#8c9eff",a200:"#536dfe",a400:"#3d5afe",a700:"#304ffe"},l={50:"#e3f2fd",100:"#bbdefb",200:"#90caf9",300:"#64b5f6",400:"#42a5f5",500:"#2196f3",600:"#1e88e5",700:"#1976d2",800:"#1565c0",900:"#0d47a1",a100:"#82b1ff",a200:"#448aff",a400:"#2979ff",a700:"#2962ff"},u={50:"#e1f5fe",100:"#b3e5fc",200:"#81d4fa",300:"#4fc3f7",400:"#29b6f6",500:"#03a9f4",600:"#039be5",700:"#0288d1",800:"#0277bd",900:"#01579b",a100:"#80d8ff",a200:"#40c4ff",a400:"#00b0ff",a700:"#0091ea"},c={50:"#e0f7fa",100:"#b2ebf2",200:"#80deea",300:"#4dd0e1",400:"#26c6da",500:"#00bcd4",600:"#00acc1",700:"#0097a7",800:"#00838f",900:"#006064",a100:"#84ffff",a200:"#18ffff",a400:"#00e5ff",a700:"#00b8d4"},d={50:"#e0f2f1",100:"#b2dfdb",200:"#80cbc4",300:"#4db6ac",400:"#26a69a",500:"#009688",600:"#00897b",700:"#00796b",800:"#00695c",900:"#004d40",a100:"#a7ffeb",a200:"#64ffda",a400:"#1de9b6",a700:"#00bfa5"},f={50:"#e8f5e9",100:"#c8e6c9",200:"#a5d6a7",300:"#81c784",400:"#66bb6a",500:"#4caf50",600:"#43a047",700:"#388e3c",800:"#2e7d32",900:"#1b5e20",a100:"#b9f6ca",a200:"#69f0ae",a400:"#00e676",a700:"#00c853"},h={50:"#f1f8e9",100:"#dcedc8",200:"#c5e1a5",300:"#aed581",400:"#9ccc65",500:"#8bc34a",600:"#7cb342",700:"#689f38",800:"#558b2f",900:"#33691e",a100:"#ccff90",a200:"#b2ff59",a400:"#76ff03",a700:"#64dd17"},p={50:"#f9fbe7",100:"#f0f4c3",200:"#e6ee9c",300:"#dce775",400:"#d4e157",500:"#cddc39",600:"#c0ca33",700:"#afb42b",800:"#9e9d24",900:"#827717",a100:"#f4ff81",a200:"#eeff41",a400:"#c6ff00",a700:"#aeea00"},m={50:"#fffde7",100:"#fff9c4",200:"#fff59d",300:"#fff176",400:"#ffee58",500:"#ffeb3b",600:"#fdd835",700:"#fbc02d",800:"#f9a825",900:"#f57f17",a100:"#ffff8d",a200:"#ffff00",a400:"#ffea00",a700:"#ffd600"},_={50:"#fff8e1",100:"#ffecb3",200:"#ffe082",300:"#ffd54f",400:"#ffca28",500:"#ffc107",600:"#ffb300",700:"#ffa000",800:"#ff8f00",900:"#ff6f00",a100:"#ffe57f",a200:"#ffd740",a400:"#ffc400",a700:"#ffab00"},v={50:"#fff3e0",100:"#ffe0b2",200:"#ffcc80",300:"#ffb74d",400:"#ffa726",500:"#ff9800",600:"#fb8c00",700:"#f57c00",800:"#ef6c00",900:"#e65100",a100:"#ffd180",a200:"#ffab40",a400:"#ff9100",a700:"#ff6d00"},g={50:"#fbe9e7",100:"#ffccbc",200:"#ffab91",300:"#ff8a65",400:"#ff7043",500:"#ff5722",600:"#f4511e",700:"#e64a19",800:"#d84315",900:"#bf360c",a100:"#ff9e80",a200:"#ff6e40",a400:"#ff3d00",a700:"#dd2c00"},b={50:"#efebe9",100:"#d7ccc8",200:"#bcaaa4",300:"#a1887f",400:"#8d6e63",500:"#795548",600:"#6d4c41",700:"#5d4037",800:"#4e342e",900:"#3e2723"},y={50:"#fafafa",100:"#f5f5f5",200:"#eeeeee",300:"#e0e0e0",400:"#bdbdbd",500:"#9e9e9e",600:"#757575",700:"#616161",800:"#424242",900:"#212121"},M={50:"#eceff1",100:"#cfd8dc",200:"#b0bec5",300:"#90a4ae",400:"#78909c",500:"#607d8b",600:"#546e7a",700:"#455a64",800:"#37474f",900:"#263238"},k={primary:"rgba(0, 0, 0, 0.87)",secondary:"rgba(0, 0, 0, 0.54)",disabled:"rgba(0, 0, 0, 0.38)",dividers:"rgba(0, 0, 0, 0.12)"},w={primary:"rgba(255, 255, 255, 1)",secondary:"rgba(255, 255, 255, 0.7)",disabled:"rgba(255, 255, 255, 0.5)",dividers:"rgba(255, 255, 255, 0.12)"},L={active:"rgba(0, 0, 0, 0.54)",inactive:"rgba(0, 0, 0, 0.38)"},x={active:"rgba(255, 255, 255, 1)",inactive:"rgba(255, 255, 255, 0.5)"},S="#ffffff",T="#000000";t.default={red:r,pink:i,purple:a,deepPurple:o,indigo:s,blue:l,lightBlue:u,cyan:c,teal:d,green:f,lightGreen:h,lime:p,yellow:m,amber:_,orange:v,deepOrange:g,brown:b,grey:y,blueGrey:M,darkText:k,lightText:w,darkIcons:L,lightIcons:x,white:S,black:T}},function(e,t,n){"use strict";var r=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"vc-swatches",attrs:{role:"application","aria-label":"Swatches color picker","data-pick":e.pick}},[n("div",{staticClass:"vc-swatches-box",attrs:{role:"listbox"}},e._l(e.palette,(function(t,r){return n("div",{key:r,staticClass:"vc-swatches-color-group"},e._l(t,(function(t){return n("div",{key:t,class:["vc-swatches-color-it",{"vc-swatches-color--white":"#FFFFFF"===t}],style:{background:t},attrs:{role:"option","aria-label":"Color:"+t,"aria-selected":e.equal(t),"data-color":t},on:{click:function(n){e.handlerClick(t)}}},[n("div",{directives:[{name:"show",rawName:"v-show",value:e.equal(t),expression:"equal(c)"}],staticClass:"vc-swatches-pick"},[n("svg",{staticStyle:{width:"24px",height:"24px"},attrs:{viewBox:"0 0 24 24"}},[n("path",{attrs:{d:"M21,7L9,19L3.5,13.5L4.91,12.09L9,16.17L19.59,5.59L21,7Z"}})])])])})))})))])},i=[];r._withStripped=!0;var a={render:r,staticRenderFns:i};t.a=a},function(e,t,n){"use strict";function r(e){l||n(53)}Object.defineProperty(t,"__esModule",{value:!0});var i=n(16),a=n.n(i);for(var o in i)"default"!==o&&function(e){n.d(t,e,(function(){return i[e]}))}(o);var s=n(66),l=!1,u=n(2),c=r,d=u(a.a,s.a,!1,c,null,null);d.options.__file="src/components/Photoshop.vue",t.default=d.exports},function(e,t,n){var r=n(54);"string"==typeof r&&(r=[[e.i,r,""]]),r.locals&&(e.exports=r.locals),n(1)("080365d4",r,!1,{})},function(e,t,n){t=e.exports=n(0)(!1),t.push([e.i,'\n.vc-photoshop {\n  background: #DCDCDC;\n  border-radius: 4px;\n  box-shadow: 0 0 0 1px rgba(0,0,0,.25), 0 8px 16px rgba(0,0,0,.15);\n  box-sizing: initial;\n  width: 513px;\n  font-family: Roboto;\n}\n.vc-photoshop__disable-fields {\n  width: 390px;\n}\n.vc-ps-head {\n  background-image: linear-gradient(-180deg, #F0F0F0 0%, #D4D4D4 100%);\n  border-bottom: 1px solid #B1B1B1;\n  box-shadow: inset 0 1px 0 0 rgba(255,255,255,.2), inset 0 -1px 0 0 rgba(0,0,0,.02);\n  height: 23px;\n  line-height: 24px;\n  border-radius: 4px 4px 0 0;\n  font-size: 13px;\n  color: #4D4D4D;\n  text-align: center;\n}\n.vc-ps-body {\n  padding: 15px;\n  display: flex;\n}\n.vc-ps-saturation-wrap {\n  width: 256px;\n  height: 256px;\n  position: relative;\n  border: 2px solid #B3B3B3;\n  border-bottom: 2px solid #F0F0F0;\n  overflow: hidden;\n}\n.vc-ps-saturation-wrap .vc-saturation-circle {\n  width: 12px;\n  height: 12px;\n}\n.vc-ps-hue-wrap {\n  position: relative;\n  height: 256px;\n  width: 19px;\n  margin-left: 10px;\n  border: 2px solid #B3B3B3;\n  border-bottom: 2px solid #F0F0F0;\n}\n.vc-ps-hue-pointer {\n  position: relative;\n}\n.vc-ps-hue-pointer--left,\n.vc-ps-hue-pointer--right {\n  position: absolute;\n  width: 0;\n  height: 0;\n  border-style: solid;\n  border-width: 5px 0 5px 8px;\n  border-color: transparent transparent transparent #555;\n}\n.vc-ps-hue-pointer--left:after,\n.vc-ps-hue-pointer--right:after {\n  content: "";\n  width: 0;\n  height: 0;\n  border-style: solid;\n  border-width: 4px 0 4px 6px;\n  border-color: transparent transparent transparent #fff;\n  position: absolute;\n  top: 1px;\n  left: 1px;\n  transform: translate(-8px, -5px);\n}\n.vc-ps-hue-pointer--left {\n  transform: translate(-13px, -4px);\n}\n.vc-ps-hue-pointer--right {\n  transform: translate(20px, -4px) rotate(180deg);\n}\n.vc-ps-controls {\n  width: 180px;\n  margin-left: 10px;\n  display: flex;\n}\n.vc-ps-controls__disable-fields {\n  width: auto;\n}\n.vc-ps-actions {\n  margin-left: 20px;\n  flex: 1;\n}\n.vc-ps-ac-btn {\n  cursor: pointer;\n  background-image: linear-gradient(-180deg, #FFFFFF 0%, #E6E6E6 100%);\n  border: 1px solid #878787;\n  border-radius: 2px;\n  height: 20px;\n  box-shadow: 0 1px 0 0 #EAEAEA;\n  font-size: 14px;\n  color: #000;\n  line-height: 20px;\n  text-align: center;\n  margin-bottom: 10px;\n}\n.vc-ps-previews {\n  width: 60px;\n}\n.vc-ps-previews__swatches {\n  border: 1px solid #B3B3B3;\n  border-bottom: 1px solid #F0F0F0;\n  margin-bottom: 2px;\n  margin-top: 1px;\n}\n.vc-ps-previews__pr-color {\n  height: 34px;\n  box-shadow: inset 1px 0 0 #000, inset -1px 0 0 #000, inset 0 1px 0 #000;\n}\n.vc-ps-previews__label {\n  font-size: 14px;\n  color: #000;\n  text-align: center;\n}\n.vc-ps-fields {\n  padding-top: 5px;\n  padding-bottom: 9px;\n  width: 80px;\n  position: relative;\n}\n.vc-ps-fields .vc-input__input {\n  margin-left: 40%;\n  width: 40%;\n  height: 18px;\n  border: 1px solid #888888;\n  box-shadow: inset 0 1px 1px rgba(0,0,0,.1), 0 1px 0 0 #ECECEC;\n  margin-bottom: 5px;\n  font-size: 13px;\n  padding-left: 3px;\n  margin-right: 10px;\n}\n.vc-ps-fields .vc-input__label, .vc-ps-fields .vc-input__desc {\n  top: 0;\n  text-transform: uppercase;\n  font-size: 13px;\n  height: 18px;\n  line-height: 22px;\n  position: absolute;\n}\n.vc-ps-fields .vc-input__label {\n  left: 0;\n  width: 34px;\n}\n.vc-ps-fields .vc-input__desc {\n  right: 0;\n  width: 0;\n}\n.vc-ps-fields__divider {\n  height: 5px;\n}\n.vc-ps-fields__hex .vc-input__input {\n  margin-left: 20%;\n  width: 80%;\n  height: 18px;\n  border: 1px solid #888888;\n  box-shadow: inset 0 1px 1px rgba(0,0,0,.1), 0 1px 0 0 #ECECEC;\n  margin-bottom: 6px;\n  font-size: 13px;\n  padding-left: 3px;\n}\n.vc-ps-fields__hex .vc-input__label {\n  position: absolute;\n  top: 0;\n  left: 0;\n  width: 14px;\n  text-transform: uppercase;\n  font-size: 13px;\n  height: 18px;\n  line-height: 22px;\n}\n',""])},function(e,t,n){var r=n(56);"string"==typeof r&&(r=[[e.i,r,""]]),r.locals&&(e.exports=r.locals),n(1)("b5380e52",r,!1,{})},function(e,t,n){t=e.exports=n(0)(!1),t.push([e.i,"\n.vc-saturation,\n.vc-saturation--white,\n.vc-saturation--black {\n  cursor: pointer;\n  position: absolute;\n  top: 0;\n  left: 0;\n  right: 0;\n  bottom: 0;\n}\n.vc-saturation--white {\n  background: linear-gradient(to right, #fff, rgba(255,255,255,0));\n}\n.vc-saturation--black {\n  background: linear-gradient(to top, #000, rgba(0,0,0,0));\n}\n.vc-saturation-pointer {\n  cursor: pointer;\n  position: absolute;\n}\n.vc-saturation-circle {\n  cursor: head;\n  width: 4px;\n  height: 4px;\n  box-shadow: 0 0 0 1.5px #fff, inset 0 0 1px 1px rgba(0,0,0,.3), 0 0 1px 2px rgba(0,0,0,.4);\n  border-radius: 50%;\n  transform: translate(-2px, -2px);\n}\n",""])},function(e,t){function n(e,t,n){return t<n?e<t?t:e>n?n:e:e<n?n:e>t?t:e}e.exports=n},function(e,n){function r(e,t,n){function r(t){var n=m,r=_;return m=_=void 0,M=t,g=e.apply(r,n)}function i(e){return M=e,b=setTimeout(c,t),x?r(e):g}function o(e){var n=e-y,r=e-M,i=t-n;return S?w(i,v-r):i}function s(e){var n=e-y,r=e-M;return void 0===y||n>=t||n<0||S&&r>=v}function c(){var e=L();if(s(e))return d(e);b=setTimeout(c,o(e))}function d(e){return b=void 0,T&&m?r(e):(m=_=void 0,g)}function f(){void 0!==b&&clearTimeout(b),M=0,m=y=_=b=void 0}function h(){return void 0===b?g:d(L())}function p(){var e=L(),n=s(e);if(m=arguments,_=this,y=e,n){if(void 0===b)return i(y);if(S)return b=setTimeout(c,t),r(y)}return void 0===b&&(b=setTimeout(c,t)),g}var m,_,v,g,b,y,M=0,x=!1,S=!1,T=!0;if("function"!=typeof e)throw new TypeError(u);return t=l(t)||0,a(n)&&(x=!!n.leading,S="maxWait"in n,v=S?k(l(n.maxWait)||0,t):v,T="trailing"in n?!!n.trailing:T),p.cancel=f,p.flush=h,p}function i(e,t,n){var i=!0,o=!0;if("function"!=typeof e)throw new TypeError(u);return a(n)&&(i="leading"in n?!!n.leading:i,o="trailing"in n?!!n.trailing:o),r(e,t,{leading:i,maxWait:t,trailing:o})}function a(e){var t=typeof e;return!!e&&("object"==t||"function"==t)}function o(e){return!!e&&"object"==typeof e}function s(e){return"symbol"==typeof e||o(e)&&M.call(e)==d}function l(e){if("number"==typeof e)return e;if(s(e))return c;if(a(e)){var t="function"==typeof e.valueOf?e.valueOf():e;e=a(t)?t+"":t}if("string"!=typeof e)return 0===e?e:+e;e=e.replace(f,"");var n=p.test(e);return n||m.test(e)?_(e.slice(2),n?2:8):h.test(e)?c:+e}var u="Expected a function",c=NaN,d="[object Symbol]",f=/^\s+|\s+$/g,h=/^[-+]0x[0-9a-f]+$/i,p=/^0b[01]+$/i,m=/^0o[0-7]+$/i,_=parseInt,v="object"==typeof t&&t&&t.Object===Object&&t,g="object"==typeof self&&self&&self.Object===Object&&self,b=v||g||Function("return this")(),y=Object.prototype,M=y.toString,k=Math.max,w=Math.min,L=function(){return b.Date.now()};e.exports=i},function(e,t,n){"use strict";var r=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{ref:"container",staticClass:"vc-saturation",style:{background:e.bgColor},on:{mousedown:e.handleMouseDown,touchmove:e.handleChange,touchstart:e.handleChange}},[n("div",{staticClass:"vc-saturation--white"}),e._v(" "),n("div",{staticClass:"vc-saturation--black"}),e._v(" "),n("div",{staticClass:"vc-saturation-pointer",style:{top:e.pointerTop,left:e.pointerLeft}},[n("div",{staticClass:"vc-saturation-circle"})])])},i=[];r._withStripped=!0;var a={render:r,staticRenderFns:i};t.a=a},function(e,t,n){var r=n(61);"string"==typeof r&&(r=[[e.i,r,""]]),r.locals&&(e.exports=r.locals),n(1)("4dc1b086",r,!1,{})},function(e,t,n){t=e.exports=n(0)(!1),t.push([e.i,"\n.vc-alpha {\n  position: absolute;\n  top: 0px;\n  right: 0px;\n  bottom: 0px;\n  left: 0px;\n}\n.vc-alpha-checkboard-wrap {\n  position: absolute;\n  top: 0px;\n  right: 0px;\n  bottom: 0px;\n  left: 0px;\n  overflow: hidden;\n}\n.vc-alpha-gradient {\n  position: absolute;\n  top: 0px;\n  right: 0px;\n  bottom: 0px;\n  left: 0px;\n}\n.vc-alpha-container {\n  cursor: pointer;\n  position: relative;\n  z-index: 2;\n  height: 100%;\n  margin: 0 3px;\n}\n.vc-alpha-pointer {\n  z-index: 2;\n  position: absolute;\n}\n.vc-alpha-picker {\n  cursor: pointer;\n  width: 4px;\n  border-radius: 1px;\n  height: 8px;\n  box-shadow: 0 0 2px rgba(0, 0, 0, .6);\n  background: #fff;\n  margin-top: 1px;\n  transform: translateX(-2px);\n}\n",""])},function(e,t,n){var r=n(63);"string"==typeof r&&(r=[[e.i,r,""]]),r.locals&&(e.exports=r.locals),n(1)("7e15c05b",r,!1,{})},function(e,t,n){t=e.exports=n(0)(!1),t.push([e.i,"\n.vc-checkerboard {\n  position: absolute;\n  top: 0px;\n  right: 0px;\n  bottom: 0px;\n  left: 0px;\n  background-size: contain;\n}\n",""])},function(e,t,n){"use strict";var r=function(){var e=this,t=e.$createElement;return(e._self._c||t)("div",{staticClass:"vc-checkerboard",style:e.bgStyle})},i=[];r._withStripped=!0;var a={render:r,staticRenderFns:i};t.a=a},function(e,t,n){"use strict";var r=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"vc-alpha"},[n("div",{staticClass:"vc-alpha-checkboard-wrap"},[n("checkboard")],1),e._v(" "),n("div",{staticClass:"vc-alpha-gradient",style:{background:e.gradientColor}}),e._v(" "),n("div",{ref:"container",staticClass:"vc-alpha-container",on:{mousedown:e.handleMouseDown,touchmove:e.handleChange,touchstart:e.handleChange}},[n("div",{staticClass:"vc-alpha-pointer",style:{left:100*e.colors.a+"%"}},[n("div",{staticClass:"vc-alpha-picker"})])])])},i=[];r._withStripped=!0;var a={render:r,staticRenderFns:i};t.a=a},function(e,t,n){"use strict";var r=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{class:["vc-photoshop",e.disableFields?"vc-photoshop__disable-fields":""],attrs:{role:"application","aria-label":"PhotoShop color picker"}},[n("div",{staticClass:"vc-ps-head",attrs:{role:"heading"}},[e._v(e._s(e.head))]),e._v(" "),n("div",{staticClass:"vc-ps-body"},[n("div",{staticClass:"vc-ps-saturation-wrap"},[n("saturation",{on:{change:e.childChange},model:{value:e.colors,callback:function(t){e.colors=t},expression:"colors"}})],1),e._v(" "),n("div",{staticClass:"vc-ps-hue-wrap"},[n("hue",{attrs:{direction:"vertical"},on:{change:e.childChange},model:{value:e.colors,callback:function(t){e.colors=t},expression:"colors"}},[n("div",{staticClass:"vc-ps-hue-pointer"},[n("i",{staticClass:"vc-ps-hue-pointer--left"}),n("i",{staticClass:"vc-ps-hue-pointer--right"})])])],1),e._v(" "),n("div",{class:["vc-ps-controls",e.disableFields?"vc-ps-controls__disable-fields":""]},[n("div",{staticClass:"vc-ps-previews"},[n("div",{staticClass:"vc-ps-previews__label"},[e._v(e._s(e.newLabel))]),e._v(" "),n("div",{staticClass:"vc-ps-previews__swatches"},[n("div",{staticClass:"vc-ps-previews__pr-color",style:{background:e.colors.hex},attrs:{"aria-label":"New color is "+e.colors.hex}}),e._v(" "),n("div",{staticClass:"vc-ps-previews__pr-color",style:{background:e.currentColor},attrs:{"aria-label":"Current color is "+e.currentColor},on:{click:e.clickCurrentColor}})]),e._v(" "),n("div",{staticClass:"vc-ps-previews__label"},[e._v(e._s(e.currentLabel))])]),e._v(" "),e.disableFields?e._e():n("div",{staticClass:"vc-ps-actions"},[n("div",{staticClass:"vc-ps-ac-btn",attrs:{role:"button","aria-label":e.acceptLabel},on:{click:e.handleAccept}},[e._v(e._s(e.acceptLabel))]),e._v(" "),n("div",{staticClass:"vc-ps-ac-btn",attrs:{role:"button","aria-label":e.cancelLabel},on:{click:e.handleCancel}},[e._v(e._s(e.cancelLabel))]),e._v(" "),n("div",{staticClass:"vc-ps-fields"},[n("ed-in",{attrs:{label:"h",desc:"°",value:e.hsv.h},on:{change:e.inputChange}}),e._v(" "),n("ed-in",{attrs:{label:"s",desc:"%",value:e.hsv.s,max:100},on:{change:e.inputChange}}),e._v(" "),n("ed-in",{attrs:{label:"v",desc:"%",value:e.hsv.v,max:100},on:{change:e.inputChange}}),e._v(" "),n("div",{staticClass:"vc-ps-fields__divider"}),e._v(" "),n("ed-in",{attrs:{label:"r",value:e.colors.rgba.r},on:{change:e.inputChange}}),e._v(" "),n("ed-in",{attrs:{label:"g",value:e.colors.rgba.g},on:{change:e.inputChange}}),e._v(" "),n("ed-in",{attrs:{label:"b",value:e.colors.rgba.b},on:{change:e.inputChange}}),e._v(" "),n("div",{staticClass:"vc-ps-fields__divider"}),e._v(" "),n("ed-in",{staticClass:"vc-ps-fields__hex",attrs:{label:"#",value:e.hex},on:{change:e.inputChange}})],1),e._v(" "),e.hasResetButton?n("div",{staticClass:"vc-ps-ac-btn",attrs:{"aria-label":"reset"},on:{click:e.handleReset}},[e._v(e._s(e.resetLabel))]):e._e()])])])])},i=[];r._withStripped=!0;var a={render:r,staticRenderFns:i};t.a=a},function(e,t,n){"use strict";function r(e){l||n(68)}Object.defineProperty(t,"__esModule",{value:!0});var i=n(20),a=n.n(i);for(var o in i)"default"!==o&&function(e){n.d(t,e,(function(){return i[e]}))}(o);var s=n(70),l=!1,u=n(2),c=r,d=u(a.a,s.a,!1,c,null,null);d.options.__file="src/components/Sketch.vue",t.default=d.exports},function(e,t,n){var r=n(69);"string"==typeof r&&(r=[[e.i,r,""]]),r.locals&&(e.exports=r.locals),n(1)("612c6604",r,!1,{})},function(e,t,n){t=e.exports=n(0)(!1),t.push([e.i,"\n.vc-sketch {\n  position: relative;\n  width: 200px;\n  padding: 10px 10px 0;\n  box-sizing: initial;\n  background: #fff;\n  border-radius: 4px;\n  box-shadow: 0 0 0 1px rgba(0, 0, 0, .15), 0 8px 16px rgba(0, 0, 0, .15);\n}\n.vc-sketch-saturation-wrap {\n  width: 100%;\n  padding-bottom: 75%;\n  position: relative;\n  overflow: hidden;\n}\n.vc-sketch-controls {\n  display: flex;\n}\n.vc-sketch-sliders {\n  padding: 4px 0;\n  flex: 1;\n}\n.vc-sketch-sliders .vc-hue,\n.vc-sketch-sliders .vc-alpha-gradient {\n  border-radius: 2px;\n}\n.vc-sketch-hue-wrap {\n  position: relative;\n  height: 10px;\n}\n.vc-sketch-alpha-wrap {\n  position: relative;\n  height: 10px;\n  margin-top: 4px;\n  overflow: hidden;\n}\n.vc-sketch-color-wrap {\n  width: 24px;\n  height: 24px;\n  position: relative;\n  margin-top: 4px;\n  margin-left: 4px;\n  border-radius: 3px;\n}\n.vc-sketch-active-color {\n  position: absolute;\n  top: 0;\n  left: 0;\n  right: 0;\n  bottom: 0;\n  border-radius: 2px;\n  box-shadow: inset 0 0 0 1px rgba(0, 0, 0, .15), inset 0 0 4px rgba(0, 0, 0, .25);\n  z-index: 2;\n}\n.vc-sketch-color-wrap .vc-checkerboard {\n  background-size: auto;\n}\n.vc-sketch-field {\n  display: flex;\n  padding-top: 4px;\n}\n.vc-sketch-field .vc-input__input {\n  width: 90%;\n  padding: 4px 0 3px 10%;\n  border: none;\n  box-shadow: inset 0 0 0 1px #ccc;\n  font-size: 10px;\n}\n.vc-sketch-field .vc-input__label {\n  display: block;\n  text-align: center;\n  font-size: 11px;\n  color: #222;\n  padding-top: 3px;\n  padding-bottom: 4px;\n  text-transform: capitalize;\n}\n.vc-sketch-field--single {\n  flex: 1;\n  padding-left: 6px;\n}\n.vc-sketch-field--double {\n  flex: 2;\n}\n.vc-sketch-presets {\n  margin-right: -10px;\n  margin-left: -10px;\n  padding-left: 10px;\n  padding-top: 10px;\n  border-top: 1px solid #eee;\n}\n.vc-sketch-presets-color {\n  border-radius: 3px;\n  overflow: hidden;\n  position: relative;\n  display: inline-block;\n  margin: 0 10px 10px 0;\n  vertical-align: top;\n  cursor: pointer;\n  width: 16px;\n  height: 16px;\n  box-shadow: inset 0 0 0 1px rgba(0, 0, 0, .15);\n}\n.vc-sketch-presets-color .vc-checkerboard {\n  box-shadow: inset 0 0 0 1px rgba(0, 0, 0, .15);\n  border-radius: 3px;\n}\n.vc-sketch__disable-alpha .vc-sketch-color-wrap {\n  height: 10px;\n}\n",""])},function(e,t,n){"use strict";var r=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{class:["vc-sketch",e.disableAlpha?"vc-sketch__disable-alpha":""],attrs:{role:"application","aria-label":"Sketch color picker"}},[n("div",{staticClass:"vc-sketch-saturation-wrap"},[n("saturation",{on:{change:e.childChange},model:{value:e.colors,callback:function(t){e.colors=t},expression:"colors"}})],1),e._v(" "),n("div",{staticClass:"vc-sketch-controls"},[n("div",{staticClass:"vc-sketch-sliders"},[n("div",{staticClass:"vc-sketch-hue-wrap"},[n("hue",{on:{change:e.childChange},model:{value:e.colors,callback:function(t){e.colors=t},expression:"colors"}})],1),e._v(" "),e.disableAlpha?e._e():n("div",{staticClass:"vc-sketch-alpha-wrap"},[n("alpha",{on:{change:e.childChange},model:{value:e.colors,callback:function(t){e.colors=t},expression:"colors"}})],1)]),e._v(" "),n("div",{staticClass:"vc-sketch-color-wrap"},[n("div",{staticClass:"vc-sketch-active-color",style:{background:e.activeColor},attrs:{"aria-label":"Current color is "+e.activeColor}}),e._v(" "),n("checkboard")],1)]),e._v(" "),e.disableFields?e._e():n("div",{staticClass:"vc-sketch-field"},[n("div",{staticClass:"vc-sketch-field--double"},[n("ed-in",{attrs:{label:"hex",value:e.hex},on:{change:e.inputChange}})],1),e._v(" "),n("div",{staticClass:"vc-sketch-field--single"},[n("ed-in",{attrs:{label:"r",value:e.colors.rgba.r},on:{change:e.inputChange}})],1),e._v(" "),n("div",{staticClass:"vc-sketch-field--single"},[n("ed-in",{attrs:{label:"g",value:e.colors.rgba.g},on:{change:e.inputChange}})],1),e._v(" "),n("div",{staticClass:"vc-sketch-field--single"},[n("ed-in",{attrs:{label:"b",value:e.colors.rgba.b},on:{change:e.inputChange}})],1),e._v(" "),e.disableAlpha?e._e():n("div",{staticClass:"vc-sketch-field--single"},[n("ed-in",{attrs:{label:"a",value:e.colors.a,"arrow-offset":.01,max:1},on:{change:e.inputChange}})],1)]),e._v(" "),n("div",{staticClass:"vc-sketch-presets",attrs:{role:"group","aria-label":"A color preset, pick one to set as current color"}},[e._l(e.presetColors,(function(t){return[e.isTransparent(t)?n("div",{key:t,staticClass:"vc-sketch-presets-color",attrs:{"aria-label":"Color:"+t},on:{click:function(n){e.handlePreset(t)}}},[n("checkboard")],1):n("div",{key:t,staticClass:"vc-sketch-presets-color",style:{background:t},attrs:{"aria-label":"Color:"+t},on:{click:function(n){e.handlePreset(t)}}})]}))],2)])},i=[];r._withStripped=!0;var a={render:r,staticRenderFns:i};t.a=a},function(e,t,n){"use strict";function r(e){l||n(72)}Object.defineProperty(t,"__esModule",{value:!0});var i=n(21),a=n.n(i);for(var o in i)"default"!==o&&function(e){n.d(t,e,(function(){return i[e]}))}(o);var s=n(74),l=!1,u=n(2),c=r,d=u(a.a,s.a,!1,c,null,null);d.options.__file="src/components/Chrome.vue",t.default=d.exports},function(e,t,n){var r=n(73);"string"==typeof r&&(r=[[e.i,r,""]]),r.locals&&(e.exports=r.locals),n(1)("1cd16048",r,!1,{})},function(e,t,n){t=e.exports=n(0)(!1),t.push([e.i,"\n.vc-chrome {\n  background: #fff;\n  border-radius: 2px;\n  box-shadow: 0 0 2px rgba(0,0,0,.3), 0 4px 8px rgba(0,0,0,.3);\n  box-sizing: initial;\n  width: 225px;\n  font-family: Menlo;\n  background-color: #fff;\n}\n.vc-chrome-controls {\n  display: flex;\n}\n.vc-chrome-color-wrap {\n  position: relative;\n  width: 36px;\n}\n.vc-chrome-active-color {\n  position: relative;\n  width: 30px;\n  height: 30px;\n  border-radius: 15px;\n  overflow: hidden;\n  z-index: 1;\n}\n.vc-chrome-color-wrap .vc-checkerboard {\n  width: 30px;\n  height: 30px;\n  border-radius: 15px;\n  background-size: auto;\n}\n.vc-chrome-sliders {\n  flex: 1;\n}\n.vc-chrome-fields-wrap {\n  display: flex;\n  padding-top: 16px;\n}\n.vc-chrome-fields {\n  display: flex;\n  margin-left: -6px;\n  flex: 1;\n}\n.vc-chrome-field {\n  padding-left: 6px;\n  width: 100%;\n}\n.vc-chrome-toggle-btn {\n  width: 32px;\n  text-align: right;\n  position: relative;\n}\n.vc-chrome-toggle-icon {\n  margin-right: -4px;\n  margin-top: 12px;\n  cursor: pointer;\n  position: relative;\n  z-index: 2;\n}\n.vc-chrome-toggle-icon-highlight {\n  position: absolute;\n  width: 24px;\n  height: 28px;\n  background: #eee;\n  border-radius: 4px;\n  top: 10px;\n  left: 12px;\n}\n.vc-chrome-hue-wrap {\n  position: relative;\n  height: 10px;\n  margin-bottom: 8px;\n}\n.vc-chrome-alpha-wrap {\n  position: relative;\n  height: 10px;\n}\n.vc-chrome-hue-wrap .vc-hue {\n  border-radius: 2px;\n}\n.vc-chrome-alpha-wrap .vc-alpha-gradient {\n  border-radius: 2px;\n}\n.vc-chrome-hue-wrap .vc-hue-picker, .vc-chrome-alpha-wrap .vc-alpha-picker {\n  width: 12px;\n  height: 12px;\n  border-radius: 6px;\n  transform: translate(-6px, -2px);\n  background-color: rgb(248, 248, 248);\n  box-shadow: 0 1px 4px 0 rgba(0, 0, 0, 0.37);\n}\n.vc-chrome-body {\n  padding: 16px 16px 12px;\n  background-color: #fff;\n}\n.vc-chrome-saturation-wrap {\n  width: 100%;\n  padding-bottom: 55%;\n  position: relative;\n  border-radius: 2px 2px 0 0;\n  overflow: hidden;\n}\n.vc-chrome-saturation-wrap .vc-saturation-circle {\n  width: 12px;\n  height: 12px;\n}\n.vc-chrome-fields .vc-input__input {\n  font-size: 11px;\n  color: #333;\n  width: 100%;\n  border-radius: 2px;\n  border: none;\n  box-shadow: inset 0 0 0 1px #dadada;\n  height: 21px;\n  text-align: center;\n}\n.vc-chrome-fields .vc-input__label {\n  text-transform: uppercase;\n  font-size: 11px;\n  line-height: 11px;\n  color: #969696;\n  text-align: center;\n  display: block;\n  margin-top: 12px;\n}\n.vc-chrome__disable-alpha .vc-chrome-active-color {\n  width: 18px;\n  height: 18px;\n}\n.vc-chrome__disable-alpha .vc-chrome-color-wrap {\n  width: 30px;\n}\n.vc-chrome__disable-alpha .vc-chrome-hue-wrap {\n  margin-top: 4px;\n  margin-bottom: 4px;\n}\n",""])},function(e,t,n){"use strict";var r=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{class:["vc-chrome",e.disableAlpha?"vc-chrome__disable-alpha":""],attrs:{role:"application","aria-label":"Chrome color picker"}},[n("div",{staticClass:"vc-chrome-saturation-wrap"},[n("saturation",{on:{change:e.childChange},model:{value:e.colors,callback:function(t){e.colors=t},expression:"colors"}})],1),e._v(" "),n("div",{staticClass:"vc-chrome-body"},[n("div",{staticClass:"vc-chrome-controls"},[n("div",{staticClass:"vc-chrome-color-wrap"},[n("div",{staticClass:"vc-chrome-active-color",style:{background:e.activeColor},attrs:{"aria-label":"current color is "+e.colors.hex}}),e._v(" "),e.disableAlpha?e._e():n("checkboard")],1),e._v(" "),n("div",{staticClass:"vc-chrome-sliders"},[n("div",{staticClass:"vc-chrome-hue-wrap"},[n("hue",{on:{change:e.childChange},model:{value:e.colors,callback:function(t){e.colors=t},expression:"colors"}})],1),e._v(" "),e.disableAlpha?e._e():n("div",{staticClass:"vc-chrome-alpha-wrap"},[n("alpha",{on:{change:e.childChange},model:{value:e.colors,callback:function(t){e.colors=t},expression:"colors"}})],1)])]),e._v(" "),e.disableFields?e._e():n("div",{staticClass:"vc-chrome-fields-wrap"},[n("div",{directives:[{name:"show",rawName:"v-show",value:0===e.fieldsIndex,expression:"fieldsIndex === 0"}],staticClass:"vc-chrome-fields"},[n("div",{staticClass:"vc-chrome-field"},[e.hasAlpha?e._e():n("ed-in",{attrs:{label:"hex",value:e.colors.hex},on:{change:e.inputChange}}),e._v(" "),e.hasAlpha?n("ed-in",{attrs:{label:"hex",value:e.colors.hex8},on:{change:e.inputChange}}):e._e()],1)]),e._v(" "),n("div",{directives:[{name:"show",rawName:"v-show",value:1===e.fieldsIndex,expression:"fieldsIndex === 1"}],staticClass:"vc-chrome-fields"},[n("div",{staticClass:"vc-chrome-field"},[n("ed-in",{attrs:{label:"r",value:e.colors.rgba.r},on:{change:e.inputChange}})],1),e._v(" "),n("div",{staticClass:"vc-chrome-field"},[n("ed-in",{attrs:{label:"g",value:e.colors.rgba.g},on:{change:e.inputChange}})],1),e._v(" "),n("div",{staticClass:"vc-chrome-field"},[n("ed-in",{attrs:{label:"b",value:e.colors.rgba.b},on:{change:e.inputChange}})],1),e._v(" "),e.disableAlpha?e._e():n("div",{staticClass:"vc-chrome-field"},[n("ed-in",{attrs:{label:"a",value:e.colors.a,"arrow-offset":.01,max:1},on:{change:e.inputChange}})],1)]),e._v(" "),n("div",{directives:[{name:"show",rawName:"v-show",value:2===e.fieldsIndex,expression:"fieldsIndex === 2"}],staticClass:"vc-chrome-fields"},[n("div",{staticClass:"vc-chrome-field"},[n("ed-in",{attrs:{label:"h",value:e.hsl.h},on:{change:e.inputChange}})],1),e._v(" "),n("div",{staticClass:"vc-chrome-field"},[n("ed-in",{attrs:{label:"s",value:e.hsl.s},on:{change:e.inputChange}})],1),e._v(" "),n("div",{staticClass:"vc-chrome-field"},[n("ed-in",{attrs:{label:"l",value:e.hsl.l},on:{change:e.inputChange}})],1),e._v(" "),e.disableAlpha?e._e():n("div",{staticClass:"vc-chrome-field"},[n("ed-in",{attrs:{label:"a",value:e.colors.a,"arrow-offset":.01,max:1},on:{change:e.inputChange}})],1)]),e._v(" "),n("div",{staticClass:"vc-chrome-toggle-btn",attrs:{role:"button","aria-label":"Change another color definition"},on:{click:e.toggleViews}},[n("div",{staticClass:"vc-chrome-toggle-icon"},[n("svg",{staticStyle:{width:"24px",height:"24px"},attrs:{viewBox:"0 0 24 24"},on:{mouseover:e.showHighlight,mouseenter:e.showHighlight,mouseout:e.hideHighlight}},[n("path",{attrs:{fill:"#333",d:"M12,18.17L8.83,15L7.42,16.41L12,21L16.59,16.41L15.17,15M12,5.83L15.17,9L16.58,7.59L12,3L7.41,7.59L8.83,9L12,5.83Z"}})])]),e._v(" "),n("div",{directives:[{name:"show",rawName:"v-show",value:e.highlight,expression:"highlight"}],staticClass:"vc-chrome-toggle-icon-highlight"})])])])])},i=[];r._withStripped=!0;var a={render:r,staticRenderFns:i};t.a=a}])}))}).call(this,n("c8ba"))},c430:function(e,t){e.exports=!1},c516:function(e,t,n){},c6b6:function(e,t){var n={}.toString;e.exports=function(e){return n.call(e).slice(8,-1)}},c6cd:function(e,t,n){var r=n("da84"),i=n("ce4e"),a="__core-js_shared__",o=r[a]||i(a,{});e.exports=o},c738:function(e,t,n){},c740:function(e,t,n){"use strict";var r=n("23e7"),i=n("b727").findIndex,a=n("44d2"),o=n("ae40"),s="findIndex",l=!0,u=o(s);s in[]&&Array(1)[s]((function(){l=!1})),r({target:"Array",proto:!0,forced:l||!u},{findIndex:function(e){return i(this,e,arguments.length>1?arguments[1]:void 0)}}),a(s)},c755:function(e,t,n){"use strict";n("5387")},c7aa:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("he",{months:"ינואר_פברואר_מרץ_אפריל_מאי_יוני_יולי_אוגוסט_ספטמבר_אוקטובר_נובמבר_דצמבר".split("_"),monthsShort:"ינו׳_פבר׳_מרץ_אפר׳_מאי_יוני_יולי_אוג׳_ספט׳_אוק׳_נוב׳_דצמ׳".split("_"),weekdays:"ראשון_שני_שלישי_רביעי_חמישי_שישי_שבת".split("_"),weekdaysShort:"א׳_ב׳_ג׳_ד׳_ה׳_ו׳_ש׳".split("_"),weekdaysMin:"א_ב_ג_ד_ה_ו_ש".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D [ב]MMMM YYYY",LLL:"D [ב]MMMM YYYY HH:mm",LLLL:"dddd, D [ב]MMMM YYYY HH:mm",l:"D/M/YYYY",ll:"D MMM YYYY",lll:"D MMM YYYY HH:mm",llll:"ddd, D MMM YYYY HH:mm"},calendar:{sameDay:"[היום ב־]LT",nextDay:"[מחר ב־]LT",nextWeek:"dddd [בשעה] LT",lastDay:"[אתמול ב־]LT",lastWeek:"[ביום] dddd [האחרון בשעה] LT",sameElse:"L"},relativeTime:{future:"בעוד %s",past:"לפני %s",s:"מספר שניות",ss:"%d שניות",m:"דקה",mm:"%d דקות",h:"שעה",hh:function(e){return 2===e?"שעתיים":e+" שעות"},d:"יום",dd:function(e){return 2===e?"יומיים":e+" ימים"},M:"חודש",MM:function(e){return 2===e?"חודשיים":e+" חודשים"},y:"שנה",yy:function(e){return 2===e?"שנתיים":e%10===0&&10!==e?e+" שנה":e+" שנים"}},meridiemParse:/אחה"צ|לפנה"צ|אחרי הצהריים|לפני הצהריים|לפנות בוקר|בבוקר|בערב/i,isPM:function(e){return/^(אחה"צ|אחרי הצהריים|בערב)$/.test(e)},meridiem:function(e,t,n){return e<5?"לפנות בוקר":e<10?"בבוקר":e<12?n?'לפנה"צ':"לפני הצהריים":e<18?n?'אחה"צ':"אחרי הצהריים":"בערב"}});return t}))},c8ba:function(e,t){var n;n=function(){return this}();try{n=n||new Function("return this")()}catch(r){"object"===typeof window&&(n=window)}e.exports=n},c8d2:function(e,t,n){var r=n("d039"),i=n("5899"),a="​᠎";e.exports=function(e){return r((function(){return!!i[e]()||a[e]()!=a||i[e].name!==e}))}},c8f3:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("sq",{months:"Janar_Shkurt_Mars_Prill_Maj_Qershor_Korrik_Gusht_Shtator_Tetor_Nëntor_Dhjetor".split("_"),monthsShort:"Jan_Shk_Mar_Pri_Maj_Qer_Kor_Gus_Sht_Tet_Nën_Dhj".split("_"),weekdays:"E Diel_E Hënë_E Martë_E Mërkurë_E Enjte_E Premte_E Shtunë".split("_"),weekdaysShort:"Die_Hën_Mar_Mër_Enj_Pre_Sht".split("_"),weekdaysMin:"D_H_Ma_Më_E_P_Sh".split("_"),weekdaysParseExact:!0,meridiemParse:/PD|MD/,isPM:function(e){return"M"===e.charAt(0)},meridiem:function(e,t,n){return e<12?"PD":"MD"},longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Sot në] LT",nextDay:"[Nesër në] LT",nextWeek:"dddd [në] LT",lastDay:"[Dje në] LT",lastWeek:"dddd [e kaluar në] LT",sameElse:"L"},relativeTime:{future:"në %s",past:"%s më parë",s:"disa sekonda",ss:"%d sekonda",m:"një minutë",mm:"%d minuta",h:"një orë",hh:"%d orë",d:"një ditë",dd:"%d ditë",M:"një muaj",MM:"%d muaj",y:"një vit",yy:"%d vite"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return t}))},c975:function(e,t,n){"use strict";var r=n("23e7"),i=n("4d64").indexOf,a=n("a640"),o=n("ae40"),s=[].indexOf,l=!!s&&1/[1].indexOf(1,-0)<0,u=a("indexOf"),c=o("indexOf",{ACCESSORS:!0,1:0});r({target:"Array",proto:!0,forced:l||!u||!c},{indexOf:function(e){return l?s.apply(this,arguments)||0:i(this,e,arguments.length>1?arguments[1]:void 0)}})},c9b2:function(e,t,n){},c9ff:function(e,t,n){},ca84:function(e,t,n){var r=n("5135"),i=n("fc6a"),a=n("4d64").indexOf,o=n("d012");e.exports=function(e,t){var n,s=i(e),l=0,u=[];for(n in s)!r(o,n)&&r(s,n)&&u.push(n);while(t.length>l)r(s,n=t[l++])&&(~a(u,n)||u.push(n));return u}},ca91:function(e,t,n){"use strict";var r=n("ebb5"),i=n("d58f").left,a=r.aTypedArray,o=r.exportTypedArrayMethod;o("reduce",(function(e){return i(a(this),e,arguments.length,arguments.length>1?arguments[1]:void 0)}))},caad:function(e,t,n){"use strict";var r=n("23e7"),i=n("4d64").includes,a=n("44d2"),o=n("ae40"),s=o("indexOf",{ACCESSORS:!0,1:0});r({target:"Array",proto:!0,forced:!s},{includes:function(e){return i(this,e,arguments.length>1?arguments[1]:void 0)}}),a("includes")},cc12:function(e,t,n){var r=n("da84"),i=n("861d"),a=r.document,o=i(a)&&i(a.createElement);e.exports=function(e){return o?a.createElement(e):{}}},cca6:function(e,t,n){var r=n("23e7"),i=n("60da");r({target:"Object",stat:!0,forced:Object.assign!==i},{assign:i})},cd26:function(e,t,n){"use strict";var r=n("ebb5"),i=r.aTypedArray,a=r.exportTypedArrayMethod,o=Math.floor;a("reverse",(function(){var e,t=this,n=i(t).length,r=o(n/2),a=0;while(a<r)e=t[a],t[a++]=t[--n],t[n]=e;return t}))},cdf9:function(e,t,n){var r=n("825a"),i=n("861d"),a=n("f069");e.exports=function(e,t){if(r(e),i(t)&&t.constructor===e)return t;var n=a.f(e),o=n.resolve;return o(t),n.promise}},ce4e:function(e,t,n){var r=n("da84"),i=n("9112");e.exports=function(e,t){try{i(r,e,t)}catch(n){r[e]=t}return t}},ce9e:function(e,t,n){"use strict";n("7758")},cf1e:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={words:{ss:["sekunda","sekunde","sekundi"],m:["jedan minut","jedne minute"],mm:["minut","minute","minuta"],h:["jedan sat","jednog sata"],hh:["sat","sata","sati"],dd:["dan","dana","dana"],MM:["mesec","meseca","meseci"],yy:["godina","godine","godina"]},correctGrammaticalCase:function(e,t){return 1===e?t[0]:e>=2&&e<=4?t[1]:t[2]},translate:function(e,n,r){var i=t.words[r];return 1===r.length?n?i[0]:i[1]:e+" "+t.correctGrammaticalCase(e,i)}},n=e.defineLocale("sr",{months:"januar_februar_mart_april_maj_jun_jul_avgust_septembar_oktobar_novembar_decembar".split("_"),monthsShort:"jan._feb._mar._apr._maj_jun_jul_avg._sep._okt._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"nedelja_ponedeljak_utorak_sreda_četvrtak_petak_subota".split("_"),weekdaysShort:"ned._pon._uto._sre._čet._pet._sub.".split("_"),weekdaysMin:"ne_po_ut_sr_če_pe_su".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"D. M. YYYY.",LL:"D. MMMM YYYY.",LLL:"D. MMMM YYYY. H:mm",LLLL:"dddd, D. MMMM YYYY. H:mm"},calendar:{sameDay:"[danas u] LT",nextDay:"[sutra u] LT",nextWeek:function(){switch(this.day()){case 0:return"[u] [nedelju] [u] LT";case 3:return"[u] [sredu] [u] LT";case 6:return"[u] [subotu] [u] LT";case 1:case 2:case 4:case 5:return"[u] dddd [u] LT"}},lastDay:"[juče u] LT",lastWeek:function(){var e=["[prošle] [nedelje] [u] LT","[prošlog] [ponedeljka] [u] LT","[prošlog] [utorka] [u] LT","[prošle] [srede] [u] LT","[prošlog] [četvrtka] [u] LT","[prošlog] [petka] [u] LT","[prošle] [subote] [u] LT"];return e[this.day()]},sameElse:"L"},relativeTime:{future:"za %s",past:"pre %s",s:"nekoliko sekundi",ss:t.translate,m:t.translate,mm:t.translate,h:t.translate,hh:t.translate,d:"dan",dd:t.translate,M:"mesec",MM:t.translate,y:"godinu",yy:t.translate},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}});return n}))},cf51:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("tzl",{months:"Januar_Fevraglh_Març_Avrïu_Mai_Gün_Julia_Guscht_Setemvar_Listopäts_Noemvar_Zecemvar".split("_"),monthsShort:"Jan_Fev_Mar_Avr_Mai_Gün_Jul_Gus_Set_Lis_Noe_Zec".split("_"),weekdays:"Súladi_Lúneçi_Maitzi_Márcuri_Xhúadi_Viénerçi_Sáturi".split("_"),weekdaysShort:"Súl_Lún_Mai_Már_Xhú_Vié_Sát".split("_"),weekdaysMin:"Sú_Lú_Ma_Má_Xh_Vi_Sá".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD.MM.YYYY",LL:"D. MMMM [dallas] YYYY",LLL:"D. MMMM [dallas] YYYY HH.mm",LLLL:"dddd, [li] D. MMMM [dallas] YYYY HH.mm"},meridiemParse:/d\'o|d\'a/i,isPM:function(e){return"d'o"===e.toLowerCase()},meridiem:function(e,t,n){return e>11?n?"d'o":"D'O":n?"d'a":"D'A"},calendar:{sameDay:"[oxhi à] LT",nextDay:"[demà à] LT",nextWeek:"dddd [à] LT",lastDay:"[ieiri à] LT",lastWeek:"[sür el] dddd [lasteu à] LT",sameElse:"L"},relativeTime:{future:"osprei %s",past:"ja%s",s:n,ss:n,m:n,mm:n,h:n,hh:n,d:n,dd:n,M:n,MM:n,y:n,yy:n},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});function n(e,t,n,r){var i={s:["viensas secunds","'iensas secunds"],ss:[e+" secunds",e+" secunds"],m:["'n míut","'iens míut"],mm:[e+" míuts",e+" míuts"],h:["'n þora","'iensa þora"],hh:[e+" þoras",e+" þoras"],d:["'n ziua","'iensa ziua"],dd:[e+" ziuas",e+" ziuas"],M:["'n mes","'iens mes"],MM:[e+" mesen",e+" mesen"],y:["'n ar","'iens ar"],yy:[e+" ars",e+" ars"]};return r||t?i[n][0]:i[n][1]}return t}))},cf75:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t="pagh_wa’_cha’_wej_loS_vagh_jav_Soch_chorgh_Hut".split("_");function n(e){var t=e;return t=-1!==e.indexOf("jaj")?t.slice(0,-3)+"leS":-1!==e.indexOf("jar")?t.slice(0,-3)+"waQ":-1!==e.indexOf("DIS")?t.slice(0,-3)+"nem":t+" pIq",t}function r(e){var t=e;return t=-1!==e.indexOf("jaj")?t.slice(0,-3)+"Hu’":-1!==e.indexOf("jar")?t.slice(0,-3)+"wen":-1!==e.indexOf("DIS")?t.slice(0,-3)+"ben":t+" ret",t}function i(e,t,n,r){var i=a(e);switch(n){case"ss":return i+" lup";case"mm":return i+" tup";case"hh":return i+" rep";case"dd":return i+" jaj";case"MM":return i+" jar";case"yy":return i+" DIS"}}function a(e){var n=Math.floor(e%1e3/100),r=Math.floor(e%100/10),i=e%10,a="";return n>0&&(a+=t[n]+"vatlh"),r>0&&(a+=(""!==a?" ":"")+t[r]+"maH"),i>0&&(a+=(""!==a?" ":"")+t[i]),""===a?"pagh":a}var o=e.defineLocale("tlh",{months:"tera’ jar wa’_tera’ jar cha’_tera’ jar wej_tera’ jar loS_tera’ jar vagh_tera’ jar jav_tera’ jar Soch_tera’ jar chorgh_tera’ jar Hut_tera’ jar wa’maH_tera’ jar wa’maH wa’_tera’ jar wa’maH cha’".split("_"),monthsShort:"jar wa’_jar cha’_jar wej_jar loS_jar vagh_jar jav_jar Soch_jar chorgh_jar Hut_jar wa’maH_jar wa’maH wa’_jar wa’maH cha’".split("_"),monthsParseExact:!0,weekdays:"lojmItjaj_DaSjaj_povjaj_ghItlhjaj_loghjaj_buqjaj_ghInjaj".split("_"),weekdaysShort:"lojmItjaj_DaSjaj_povjaj_ghItlhjaj_loghjaj_buqjaj_ghInjaj".split("_"),weekdaysMin:"lojmItjaj_DaSjaj_povjaj_ghItlhjaj_loghjaj_buqjaj_ghInjaj".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[DaHjaj] LT",nextDay:"[wa’leS] LT",nextWeek:"LLL",lastDay:"[wa’Hu’] LT",lastWeek:"LLL",sameElse:"L"},relativeTime:{future:n,past:r,s:"puS lup",ss:i,m:"wa’ tup",mm:i,h:"wa’ rep",hh:i,d:"wa’ jaj",dd:i,M:"wa’ jar",MM:i,y:"wa’ DIS",yy:i},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return o}))},d012:function(e,t){e.exports={}},d039:function(e,t){e.exports=function(e){try{return!!e()}catch(t){return!0}}},d066:function(e,t,n){var r=n("428f"),i=n("da84"),a=function(e){return"function"==typeof e?e:void 0};e.exports=function(e,t){return arguments.length<2?a(r[e])||a(i[e]):r[e]&&r[e][t]||i[e]&&i[e][t]}},d139:function(e,t,n){"use strict";var r=n("ebb5"),i=n("b727").find,a=r.aTypedArray,o=r.exportTypedArrayMethod;o("find",(function(e){return i(a(this),e,arguments.length>1?arguments[1]:void 0)}))},d1e7:function(e,t,n){"use strict";var r={}.propertyIsEnumerable,i=Object.getOwnPropertyDescriptor,a=i&&!r.call({1:2},1);t.f=a?function(e){var t=i(this,e);return!!t&&t.enumerable}:r},d26a:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"༡",2:"༢",3:"༣",4:"༤",5:"༥",6:"༦",7:"༧",8:"༨",9:"༩",0:"༠"},n={"༡":"1","༢":"2","༣":"3","༤":"4","༥":"5","༦":"6","༧":"7","༨":"8","༩":"9","༠":"0"},r=e.defineLocale("bo",{months:"ཟླ་བ་དང་པོ_ཟླ་བ་གཉིས་པ_ཟླ་བ་གསུམ་པ_ཟླ་བ་བཞི་པ_ཟླ་བ་ལྔ་པ_ཟླ་བ་དྲུག་པ_ཟླ་བ་བདུན་པ_ཟླ་བ་བརྒྱད་པ_ཟླ་བ་དགུ་པ_ཟླ་བ་བཅུ་པ_ཟླ་བ་བཅུ་གཅིག་པ_ཟླ་བ་བཅུ་གཉིས་པ".split("_"),monthsShort:"ཟླ་1_ཟླ་2_ཟླ་3_ཟླ་4_ཟླ་5_ཟླ་6_ཟླ་7_ཟླ་8_ཟླ་9_ཟླ་10_ཟླ་11_ཟླ་12".split("_"),monthsShortRegex:/^(ཟླ་\d{1,2})/,monthsParseExact:!0,weekdays:"གཟའ་ཉི་མ་_གཟའ་ཟླ་བ་_གཟའ་མིག་དམར་_གཟའ་ལྷག་པ་_གཟའ་ཕུར་བུ_གཟའ་པ་སངས་_གཟའ་སྤེན་པ་".split("_"),weekdaysShort:"ཉི་མ་_ཟླ་བ་_མིག་དམར་_ལྷག་པ་_ཕུར་བུ_པ་སངས་_སྤེན་པ་".split("_"),weekdaysMin:"ཉི_ཟླ_མིག_ལྷག_ཕུར_སངས_སྤེན".split("_"),longDateFormat:{LT:"A h:mm",LTS:"A h:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm",LLLL:"dddd, D MMMM YYYY, A h:mm"},calendar:{sameDay:"[དི་རིང] LT",nextDay:"[སང་ཉིན] LT",nextWeek:"[བདུན་ཕྲག་རྗེས་མ], LT",lastDay:"[ཁ་སང] LT",lastWeek:"[བདུན་ཕྲག་མཐའ་མ] dddd, LT",sameElse:"L"},relativeTime:{future:"%s ལ་",past:"%s སྔན་ལ",s:"ལམ་སང",ss:"%d སྐར་ཆ།",m:"སྐར་མ་གཅིག",mm:"%d སྐར་མ",h:"ཆུ་ཚོད་གཅིག",hh:"%d ཆུ་ཚོད",d:"ཉིན་གཅིག",dd:"%d ཉིན་",M:"ཟླ་བ་གཅིག",MM:"%d ཟླ་བ",y:"ལོ་གཅིག",yy:"%d ལོ"},preparse:function(e){return e.replace(/[༡༢༣༤༥༦༧༨༩༠]/g,(function(e){return n[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},meridiemParse:/མཚན་མོ|ཞོགས་ཀས|ཉིན་གུང|དགོང་དག|མཚན་མོ/,meridiemHour:function(e,t){return 12===e&&(e=0),"མཚན་མོ"===t&&e>=4||"ཉིན་གུང"===t&&e<5||"དགོང་དག"===t?e+12:e},meridiem:function(e,t,n){return e<4?"མཚན་མོ":e<10?"ཞོགས་ཀས":e<17?"ཉིན་གུང":e<20?"དགོང་དག":"མཚན་མོ"},week:{dow:0,doy:6}});return r}))},d28b:function(e,t,n){var r=n("746f");r("iterator")},d2bb:function(e,t,n){var r=n("825a"),i=n("3bbe");e.exports=Object.setPrototypeOf||("__proto__"in{}?function(){var e,t=!1,n={};try{e=Object.getOwnPropertyDescriptor(Object.prototype,"__proto__").set,e.call(n,[]),t=n instanceof Array}catch(a){}return function(n,a){return r(n),i(a),t?e.call(n,a):n.__proto__=a,n}}():void 0)},d2d4:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("pt-br",{months:"janeiro_fevereiro_março_abril_maio_junho_julho_agosto_setembro_outubro_novembro_dezembro".split("_"),monthsShort:"jan_fev_mar_abr_mai_jun_jul_ago_set_out_nov_dez".split("_"),weekdays:"domingo_segunda-feira_terça-feira_quarta-feira_quinta-feira_sexta-feira_sábado".split("_"),weekdaysShort:"dom_seg_ter_qua_qui_sex_sáb".split("_"),weekdaysMin:"do_2ª_3ª_4ª_5ª_6ª_sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY [às] HH:mm",LLLL:"dddd, D [de] MMMM [de] YYYY [às] HH:mm"},calendar:{sameDay:"[Hoje às] LT",nextDay:"[Amanhã às] LT",nextWeek:"dddd [às] LT",lastDay:"[Ontem às] LT",lastWeek:function(){return 0===this.day()||6===this.day()?"[Último] dddd [às] LT":"[Última] dddd [às] LT"},sameElse:"L"},relativeTime:{future:"em %s",past:"há %s",s:"poucos segundos",ss:"%d segundos",m:"um minuto",mm:"%d minutos",h:"uma hora",hh:"%d horas",d:"um dia",dd:"%d dias",M:"um mês",MM:"%d meses",y:"um ano",yy:"%d anos"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",invalidDate:"Data inválida"});return t}))},d3b7:function(e,t,n){var r=n("00ee"),i=n("6eeb"),a=n("b041");r||i(Object.prototype,"toString",a,{unsafe:!0})},d44e:function(e,t,n){var r=n("9bf2").f,i=n("5135"),a=n("b622"),o=a("toStringTag");e.exports=function(e,t,n){e&&!i(e=n?e:e.prototype,o)&&r(e,o,{configurable:!0,value:t})}},d58f:function(e,t,n){var r=n("1c0b"),i=n("7b0b"),a=n("44ad"),o=n("50c4"),s=function(e){return function(t,n,s,l){r(n);var u=i(t),c=a(u),d=o(u.length),f=e?d-1:0,h=e?-1:1;if(s<2)while(1){if(f in c){l=c[f],f+=h;break}if(f+=h,e?f<0:d<=f)throw TypeError("Reduce of empty array with no initial value")}for(;e?f>=0:d>f;f+=h)f in c&&(l=n(l,c[f],f,u));return l}};e.exports={left:s(!1),right:s(!0)}},d5d6:function(e,t,n){"use strict";var r=n("ebb5"),i=n("b727").forEach,a=r.aTypedArray,o=r.exportTypedArrayMethod;o("forEach",(function(e){i(a(this),e,arguments.length>1?arguments[1]:void 0)}))},d69a:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("fil",{months:"Enero_Pebrero_Marso_Abril_Mayo_Hunyo_Hulyo_Agosto_Setyembre_Oktubre_Nobyembre_Disyembre".split("_"),monthsShort:"Ene_Peb_Mar_Abr_May_Hun_Hul_Ago_Set_Okt_Nob_Dis".split("_"),weekdays:"Linggo_Lunes_Martes_Miyerkules_Huwebes_Biyernes_Sabado".split("_"),weekdaysShort:"Lin_Lun_Mar_Miy_Huw_Biy_Sab".split("_"),weekdaysMin:"Li_Lu_Ma_Mi_Hu_Bi_Sab".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"MM/D/YYYY",LL:"MMMM D, YYYY",LLL:"MMMM D, YYYY HH:mm",LLLL:"dddd, MMMM DD, YYYY HH:mm"},calendar:{sameDay:"LT [ngayong araw]",nextDay:"[Bukas ng] LT",nextWeek:"LT [sa susunod na] dddd",lastDay:"LT [kahapon]",lastWeek:"LT [noong nakaraang] dddd",sameElse:"L"},relativeTime:{future:"sa loob ng %s",past:"%s ang nakalipas",s:"ilang segundo",ss:"%d segundo",m:"isang minuto",mm:"%d minuto",h:"isang oras",hh:"%d oras",d:"isang araw",dd:"%d araw",M:"isang buwan",MM:"%d buwan",y:"isang taon",yy:"%d taon"},dayOfMonthOrdinalParse:/\d{1,2}/,ordinal:function(e){return e},week:{dow:1,doy:4}});return t}))},d69e:function(e,t,n){},d6b6:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("hy-am",{months:{format:"հունվարի_փետրվարի_մարտի_ապրիլի_մայիսի_հունիսի_հուլիսի_օգոստոսի_սեպտեմբերի_հոկտեմբերի_նոյեմբերի_դեկտեմբերի".split("_"),standalone:"հունվար_փետրվար_մարտ_ապրիլ_մայիս_հունիս_հուլիս_օգոստոս_սեպտեմբեր_հոկտեմբեր_նոյեմբեր_դեկտեմբեր".split("_")},monthsShort:"հնվ_փտր_մրտ_ապր_մյս_հնս_հլս_օգս_սպտ_հկտ_նմբ_դկտ".split("_"),weekdays:"կիրակի_երկուշաբթի_երեքշաբթի_չորեքշաբթի_հինգշաբթի_ուրբաթ_շաբաթ".split("_"),weekdaysShort:"կրկ_երկ_երք_չրք_հնգ_ուրբ_շբթ".split("_"),weekdaysMin:"կրկ_երկ_երք_չրք_հնգ_ուրբ_շբթ".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY թ.",LLL:"D MMMM YYYY թ., HH:mm",LLLL:"dddd, D MMMM YYYY թ., HH:mm"},calendar:{sameDay:"[այսօր] LT",nextDay:"[վաղը] LT",lastDay:"[երեկ] LT",nextWeek:function(){return"dddd [օրը ժամը] LT"},lastWeek:function(){return"[անցած] dddd [օրը ժամը] LT"},sameElse:"L"},relativeTime:{future:"%s հետո",past:"%s առաջ",s:"մի քանի վայրկյան",ss:"%d վայրկյան",m:"րոպե",mm:"%d րոպե",h:"ժամ",hh:"%d ժամ",d:"օր",dd:"%d օր",M:"ամիս",MM:"%d ամիս",y:"տարի",yy:"%d տարի"},meridiemParse:/գիշերվա|առավոտվա|ցերեկվա|երեկոյան/,isPM:function(e){return/^(ցերեկվա|երեկոյան)$/.test(e)},meridiem:function(e){return e<4?"գիշերվա":e<12?"առավոտվա":e<17?"ցերեկվա":"երեկոյան"},dayOfMonthOrdinalParse:/\d{1,2}|\d{1,2}-(ին|րդ)/,ordinal:function(e,t){switch(t){case"DDD":case"w":case"W":case"DDDo":return 1===e?e+"-ին":e+"-րդ";default:return e}},week:{dow:1,doy:7}});return t}))},d716:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("ca",{months:{standalone:"gener_febrer_març_abril_maig_juny_juliol_agost_setembre_octubre_novembre_desembre".split("_"),format:"de gener_de febrer_de març_d'abril_de maig_de juny_de juliol_d'agost_de setembre_d'octubre_de novembre_de desembre".split("_"),isFormat:/D[oD]?(\s)+MMMM/},monthsShort:"gen._febr._març_abr._maig_juny_jul._ag._set._oct._nov._des.".split("_"),monthsParseExact:!0,weekdays:"diumenge_dilluns_dimarts_dimecres_dijous_divendres_dissabte".split("_"),weekdaysShort:"dg._dl._dt._dc._dj._dv._ds.".split("_"),weekdaysMin:"dg_dl_dt_dc_dj_dv_ds".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM [de] YYYY",ll:"D MMM YYYY",LLL:"D MMMM [de] YYYY [a les] H:mm",lll:"D MMM YYYY, H:mm",LLLL:"dddd D MMMM [de] YYYY [a les] H:mm",llll:"ddd D MMM YYYY, H:mm"},calendar:{sameDay:function(){return"[avui a "+(1!==this.hours()?"les":"la")+"] LT"},nextDay:function(){return"[demà a "+(1!==this.hours()?"les":"la")+"] LT"},nextWeek:function(){return"dddd [a "+(1!==this.hours()?"les":"la")+"] LT"},lastDay:function(){return"[ahir a "+(1!==this.hours()?"les":"la")+"] LT"},lastWeek:function(){return"[el] dddd [passat a "+(1!==this.hours()?"les":"la")+"] LT"},sameElse:"L"},relativeTime:{future:"d'aquí %s",past:"fa %s",s:"uns segons",ss:"%d segons",m:"un minut",mm:"%d minuts",h:"una hora",hh:"%d hores",d:"un dia",dd:"%d dies",M:"un mes",MM:"%d mesos",y:"un any",yy:"%d anys"},dayOfMonthOrdinalParse:/\d{1,2}(r|n|t|è|a)/,ordinal:function(e,t){var n=1===e?"r":2===e?"n":3===e?"r":4===e?"t":"è";return"w"!==t&&"W"!==t||(n="a"),e+n},week:{dow:1,doy:4}});return t}))},d784:function(e,t,n){"use strict";n("ac1f");var r=n("6eeb"),i=n("d039"),a=n("b622"),o=n("9263"),s=n("9112"),l=a("species"),u=!i((function(){var e=/./;return e.exec=function(){var e=[];return e.groups={a:"7"},e},"7"!=="".replace(e,"$<a>")})),c=function(){return"$0"==="a".replace(/./,"$0")}(),d=a("replace"),f=function(){return!!/./[d]&&""===/./[d]("a","$0")}(),h=!i((function(){var e=/(?:)/,t=e.exec;e.exec=function(){return t.apply(this,arguments)};var n="ab".split(e);return 2!==n.length||"a"!==n[0]||"b"!==n[1]}));e.exports=function(e,t,n,d){var p=a(e),m=!i((function(){var t={};return t[p]=function(){return 7},7!=""[e](t)})),_=m&&!i((function(){var t=!1,n=/a/;return"split"===e&&(n={},n.constructor={},n.constructor[l]=function(){return n},n.flags="",n[p]=/./[p]),n.exec=function(){return t=!0,null},n[p](""),!t}));if(!m||!_||"replace"===e&&(!u||!c||f)||"split"===e&&!h){var v=/./[p],g=n(p,""[e],(function(e,t,n,r,i){return t.exec===o?m&&!i?{done:!0,value:v.call(t,n,r)}:{done:!0,value:e.call(n,t,r)}:{done:!1}}),{REPLACE_KEEPS_$0:c,REGEXP_REPLACE_SUBSTITUTES_UNDEFINED_CAPTURE:f}),b=g[0],y=g[1];r(String.prototype,e,b),r(RegExp.prototype,p,2==t?function(e,t){return y.call(e,this,t)}:function(e){return y.call(e,this)})}d&&s(RegExp.prototype[p],"sham",!0)}},d7d9:function(e,t,n){"use strict";n("3546")},d81d:function(e,t,n){"use strict";var r=n("23e7"),i=n("b727").map,a=n("1dde"),o=n("ae40"),s=a("map"),l=o("map");r({target:"Array",proto:!0,forced:!s||!l},{map:function(e){return i(this,e,arguments.length>1?arguments[1]:void 0)}})},d88f:function(e,t,n){"use strict";n("fb14")},d8a8:function(e,t,n){},d8da:function(e,t,n){},d9f8:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("fr-ca",{months:"janvier_février_mars_avril_mai_juin_juillet_août_septembre_octobre_novembre_décembre".split("_"),monthsShort:"janv._févr._mars_avr._mai_juin_juil._août_sept._oct._nov._déc.".split("_"),monthsParseExact:!0,weekdays:"dimanche_lundi_mardi_mercredi_jeudi_vendredi_samedi".split("_"),weekdaysShort:"dim._lun._mar._mer._jeu._ven._sam.".split("_"),weekdaysMin:"di_lu_ma_me_je_ve_sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[Aujourd’hui à] LT",nextDay:"[Demain à] LT",nextWeek:"dddd [à] LT",lastDay:"[Hier à] LT",lastWeek:"dddd [dernier à] LT",sameElse:"L"},relativeTime:{future:"dans %s",past:"il y a %s",s:"quelques secondes",ss:"%d secondes",m:"une minute",mm:"%d minutes",h:"une heure",hh:"%d heures",d:"un jour",dd:"%d jours",M:"un mois",MM:"%d mois",y:"un an",yy:"%d ans"},dayOfMonthOrdinalParse:/\d{1,2}(er|e)/,ordinal:function(e,t){switch(t){default:case"M":case"Q":case"D":case"DDD":case"d":return e+(1===e?"er":"e");case"w":case"W":return e+(1===e?"re":"e")}}});return t}))},da84:function(e,t,n){(function(t){var n=function(e){return e&&e.Math==Math&&e};e.exports=n("object"==typeof globalThis&&globalThis)||n("object"==typeof window&&window)||n("object"==typeof self&&self)||n("object"==typeof t&&t)||function(){return this}()||Function("return this")()}).call(this,n("c8ba"))},db29:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t="jan._feb._mrt._apr._mei_jun._jul._aug._sep._okt._nov._dec.".split("_"),n="jan_feb_mrt_apr_mei_jun_jul_aug_sep_okt_nov_dec".split("_"),r=[/^jan/i,/^feb/i,/^maart|mrt.?$/i,/^apr/i,/^mei$/i,/^jun[i.]?$/i,/^jul[i.]?$/i,/^aug/i,/^sep/i,/^okt/i,/^nov/i,/^dec/i],i=/^(januari|februari|maart|april|mei|ju[nl]i|augustus|september|oktober|november|december|jan\.?|feb\.?|mrt\.?|apr\.?|ju[nl]\.?|aug\.?|sep\.?|okt\.?|nov\.?|dec\.?)/i,a=e.defineLocale("nl-be",{months:"januari_februari_maart_april_mei_juni_juli_augustus_september_oktober_november_december".split("_"),monthsShort:function(e,r){return e?/-MMM-/.test(r)?n[e.month()]:t[e.month()]:t},monthsRegex:i,monthsShortRegex:i,monthsStrictRegex:/^(januari|februari|maart|april|mei|ju[nl]i|augustus|september|oktober|november|december)/i,monthsShortStrictRegex:/^(jan\.?|feb\.?|mrt\.?|apr\.?|mei|ju[nl]\.?|aug\.?|sep\.?|okt\.?|nov\.?|dec\.?)/i,monthsParse:r,longMonthsParse:r,shortMonthsParse:r,weekdays:"zondag_maandag_dinsdag_woensdag_donderdag_vrijdag_zaterdag".split("_"),weekdaysShort:"zo._ma._di._wo._do._vr._za.".split("_"),weekdaysMin:"zo_ma_di_wo_do_vr_za".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[vandaag om] LT",nextDay:"[morgen om] LT",nextWeek:"dddd [om] LT",lastDay:"[gisteren om] LT",lastWeek:"[afgelopen] dddd [om] LT",sameElse:"L"},relativeTime:{future:"over %s",past:"%s geleden",s:"een paar seconden",ss:"%d seconden",m:"één minuut",mm:"%d minuten",h:"één uur",hh:"%d uur",d:"één dag",dd:"%d dagen",M:"één maand",MM:"%d maanden",y:"één jaar",yy:"%d jaar"},dayOfMonthOrdinalParse:/\d{1,2}(ste|de)/,ordinal:function(e){return e+(1===e||8===e||e>=20?"ste":"de")},week:{dow:1,doy:4}});return a}))},db59:function(e,t,n){},dbb4:function(e,t,n){var r=n("23e7"),i=n("83ab"),a=n("56ef"),o=n("fc6a"),s=n("06cf"),l=n("8418");r({target:"Object",stat:!0,sham:!i},{getOwnPropertyDescriptors:function(e){var t,n,r=o(e),i=s.f,u=a(r),c={},d=0;while(u.length>d)n=i(r,t=u[d++]),void 0!==n&&l(c,t,n);return c}})},dc4d:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"१",2:"२",3:"३",4:"४",5:"५",6:"६",7:"७",8:"८",9:"९",0:"०"},n={"१":"1","२":"2","३":"3","४":"4","५":"5","६":"6","७":"7","८":"8","९":"9","०":"0"},r=[/^जन/i,/^फ़र|फर/i,/^मार्च/i,/^अप्रै/i,/^मई/i,/^जून/i,/^जुल/i,/^अग/i,/^सितं|सित/i,/^अक्टू/i,/^नव|नवं/i,/^दिसं|दिस/i],i=[/^जन/i,/^फ़र/i,/^मार्च/i,/^अप्रै/i,/^मई/i,/^जून/i,/^जुल/i,/^अग/i,/^सित/i,/^अक्टू/i,/^नव/i,/^दिस/i],a=e.defineLocale("hi",{months:{format:"जनवरी_फ़रवरी_मार्च_अप्रैल_मई_जून_जुलाई_अगस्त_सितम्बर_अक्टूबर_नवम्बर_दिसम्बर".split("_"),standalone:"जनवरी_फरवरी_मार्च_अप्रैल_मई_जून_जुलाई_अगस्त_सितंबर_अक्टूबर_नवंबर_दिसंबर".split("_")},monthsShort:"जन._फ़र._मार्च_अप्रै._मई_जून_जुल._अग._सित._अक्टू._नव._दिस.".split("_"),weekdays:"रविवार_सोमवार_मंगलवार_बुधवार_गुरूवार_शुक्रवार_शनिवार".split("_"),weekdaysShort:"रवि_सोम_मंगल_बुध_गुरू_शुक्र_शनि".split("_"),weekdaysMin:"र_सो_मं_बु_गु_शु_श".split("_"),longDateFormat:{LT:"A h:mm बजे",LTS:"A h:mm:ss बजे",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm बजे",LLLL:"dddd, D MMMM YYYY, A h:mm बजे"},monthsParse:r,longMonthsParse:r,shortMonthsParse:i,monthsRegex:/^(जनवरी|जन\.?|फ़रवरी|फरवरी|फ़र\.?|मार्च?|अप्रैल|अप्रै\.?|मई?|जून?|जुलाई|जुल\.?|अगस्त|अग\.?|सितम्बर|सितंबर|सित\.?|अक्टूबर|अक्टू\.?|नवम्बर|नवंबर|नव\.?|दिसम्बर|दिसंबर|दिस\.?)/i,monthsShortRegex:/^(जनवरी|जन\.?|फ़रवरी|फरवरी|फ़र\.?|मार्च?|अप्रैल|अप्रै\.?|मई?|जून?|जुलाई|जुल\.?|अगस्त|अग\.?|सितम्बर|सितंबर|सित\.?|अक्टूबर|अक्टू\.?|नवम्बर|नवंबर|नव\.?|दिसम्बर|दिसंबर|दिस\.?)/i,monthsStrictRegex:/^(जनवरी?|फ़रवरी|फरवरी?|मार्च?|अप्रैल?|मई?|जून?|जुलाई?|अगस्त?|सितम्बर|सितंबर|सित?\.?|अक्टूबर|अक्टू\.?|नवम्बर|नवंबर?|दिसम्बर|दिसंबर?)/i,monthsShortStrictRegex:/^(जन\.?|फ़र\.?|मार्च?|अप्रै\.?|मई?|जून?|जुल\.?|अग\.?|सित\.?|अक्टू\.?|नव\.?|दिस\.?)/i,calendar:{sameDay:"[आज] LT",nextDay:"[कल] LT",nextWeek:"dddd, LT",lastDay:"[कल] LT",lastWeek:"[पिछले] dddd, LT",sameElse:"L"},relativeTime:{future:"%s में",past:"%s पहले",s:"कुछ ही क्षण",ss:"%d सेकंड",m:"एक मिनट",mm:"%d मिनट",h:"एक घंटा",hh:"%d घंटे",d:"एक दिन",dd:"%d दिन",M:"एक महीने",MM:"%d महीने",y:"एक वर्ष",yy:"%d वर्ष"},preparse:function(e){return e.replace(/[१२३४५६७८९०]/g,(function(e){return n[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},meridiemParse:/रात|सुबह|दोपहर|शाम/,meridiemHour:function(e,t){return 12===e&&(e=0),"रात"===t?e<4?e:e+12:"सुबह"===t?e:"दोपहर"===t?e>=10?e:e+12:"शाम"===t?e+12:void 0},meridiem:function(e,t,n){return e<4?"रात":e<10?"सुबह":e<17?"दोपहर":e<20?"शाम":"रात"},week:{dow:0,doy:6}});return a}))},dd32:function(e,t,n){},ddb0:function(e,t,n){var r=n("da84"),i=n("fdbc"),a=n("e260"),o=n("9112"),s=n("b622"),l=s("iterator"),u=s("toStringTag"),c=a.values;for(var d in i){var f=r[d],h=f&&f.prototype;if(h){if(h[l]!==c)try{o(h,l,c)}catch(m){h[l]=c}if(h[u]||o(h,u,d),i[d])for(var p in a)if(h[p]!==a[p])try{o(h,p,a[p])}catch(m){h[p]=a[p]}}}},df75:function(e,t,n){var r=n("ca84"),i=n("7839");e.exports=Object.keys||function(e){return r(e,i)}},e01a:function(e,t,n){"use strict";var r=n("23e7"),i=n("83ab"),a=n("da84"),o=n("5135"),s=n("861d"),l=n("9bf2").f,u=n("e893"),c=a.Symbol;if(i&&"function"==typeof c&&(!("description"in c.prototype)||void 0!==c().description)){var d={},f=function(){var e=arguments.length<1||void 0===arguments[0]?void 0:String(arguments[0]),t=this instanceof f?new c(e):void 0===e?c():c(e);return""===e&&(d[t]=!0),t};u(f,c);var h=f.prototype=c.prototype;h.constructor=f;var p=h.toString,m="Symbol(test)"==String(c("test")),_=/^Symbol\((.*)\)[^)]+$/;l(h,"description",{configurable:!0,get:function(){var e=s(this)?this.valueOf():this,t=p.call(e);if(o(d,e))return"";var n=m?t.slice(7,-1):t.replace(_,"$1");return""===n?void 0:n}}),r({global:!0,forced:!0},{Symbol:f})}},e0c5:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"૧",2:"૨",3:"૩",4:"૪",5:"૫",6:"૬",7:"૭",8:"૮",9:"૯",0:"૦"},n={"૧":"1","૨":"2","૩":"3","૪":"4","૫":"5","૬":"6","૭":"7","૮":"8","૯":"9","૦":"0"},r=e.defineLocale("gu",{months:"જાન્યુઆરી_ફેબ્રુઆરી_માર્ચ_એપ્રિલ_મે_જૂન_જુલાઈ_ઑગસ્ટ_સપ્ટેમ્બર_ઑક્ટ્બર_નવેમ્બર_ડિસેમ્બર".split("_"),monthsShort:"જાન્યુ._ફેબ્રુ._માર્ચ_એપ્રિ._મે_જૂન_જુલા._ઑગ._સપ્ટે._ઑક્ટ્._નવે._ડિસે.".split("_"),monthsParseExact:!0,weekdays:"રવિવાર_સોમવાર_મંગળવાર_બુધ્વાર_ગુરુવાર_શુક્રવાર_શનિવાર".split("_"),weekdaysShort:"રવિ_સોમ_મંગળ_બુધ્_ગુરુ_શુક્ર_શનિ".split("_"),weekdaysMin:"ર_સો_મં_બુ_ગુ_શુ_શ".split("_"),longDateFormat:{LT:"A h:mm વાગ્યે",LTS:"A h:mm:ss વાગ્યે",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm વાગ્યે",LLLL:"dddd, D MMMM YYYY, A h:mm વાગ્યે"},calendar:{sameDay:"[આજ] LT",nextDay:"[કાલે] LT",nextWeek:"dddd, LT",lastDay:"[ગઇકાલે] LT",lastWeek:"[પાછલા] dddd, LT",sameElse:"L"},relativeTime:{future:"%s મા",past:"%s પહેલા",s:"અમુક પળો",ss:"%d સેકંડ",m:"એક મિનિટ",mm:"%d મિનિટ",h:"એક કલાક",hh:"%d કલાક",d:"એક દિવસ",dd:"%d દિવસ",M:"એક મહિનો",MM:"%d મહિનો",y:"એક વર્ષ",yy:"%d વર્ષ"},preparse:function(e){return e.replace(/[૧૨૩૪૫૬૭૮૯૦]/g,(function(e){return n[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},meridiemParse:/રાત|બપોર|સવાર|સાંજ/,meridiemHour:function(e,t){return 12===e&&(e=0),"રાત"===t?e<4?e:e+12:"સવાર"===t?e:"બપોર"===t?e>=10?e:e+12:"સાંજ"===t?e+12:void 0},meridiem:function(e,t,n){return e<4?"રાત":e<10?"સવાર":e<17?"બપોર":e<20?"સાંજ":"રાત"},week:{dow:0,doy:6}});return r}))},e163:function(e,t,n){var r=n("5135"),i=n("7b0b"),a=n("f772"),o=n("e177"),s=a("IE_PROTO"),l=Object.prototype;e.exports=o?Object.getPrototypeOf:function(e){return e=i(e),r(e,s)?e[s]:"function"==typeof e.constructor&&e instanceof e.constructor?e.constructor.prototype:e instanceof Object?l:null}},e177:function(e,t,n){var r=n("d039");e.exports=!r((function(){function e(){}return e.prototype.constructor=null,Object.getPrototypeOf(new e)!==e.prototype}))},e1d3:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("en-ie",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(e){var t=e%10,n=1===~~(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th";return e+n},week:{dow:1,doy:4}});return t}))},e260:function(e,t,n){"use strict";var r=n("fc6a"),i=n("44d2"),a=n("3f8c"),o=n("69f3"),s=n("7dd0"),l="Array Iterator",u=o.set,c=o.getterFor(l);e.exports=s(Array,"Array",(function(e,t){u(this,{type:l,target:r(e),index:0,kind:t})}),(function(){var e=c(this),t=e.target,n=e.kind,r=e.index++;return!t||r>=t.length?(e.target=void 0,{value:void 0,done:!0}):"keys"==n?{value:r,done:!1}:"values"==n?{value:t[r],done:!1}:{value:[r,t[r]],done:!1}}),"values"),a.Arguments=a.Array,i("keys"),i("values"),i("entries")},e2cc:function(e,t,n){var r=n("6eeb");e.exports=function(e,t,n){for(var i in t)r(e,i,t[i],n);return e}},e37d:function(e,t,n){"use strict";(function(e){var r=n("f0bd"),i=n("252c");function a(e){return a="function"===typeof Symbol&&"symbol"===typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"===typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e},a(e)}function o(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function s(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}function l(e,t,n){return t&&s(e.prototype,t),n&&s(e,n),e}function u(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function c(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),n.push.apply(n,r)}return n}function d(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{};t%2?c(Object(n),!0).forEach((function(t){u(e,t,n[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):c(Object(n)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(n,t))}))}return e}var f=function(){};function h(e){return"string"===typeof e&&(e=e.split(" ")),e}function p(e,t){var n,r=h(t);n=e.className instanceof f?h(e.className.baseVal):h(e.className),r.forEach((function(e){-1===n.indexOf(e)&&n.push(e)})),e instanceof SVGElement?e.setAttribute("class",n.join(" ")):e.className=n.join(" ")}function m(e,t){var n,r=h(t);n=e.className instanceof f?h(e.className.baseVal):h(e.className),r.forEach((function(e){var t=n.indexOf(e);-1!==t&&n.splice(t,1)})),e instanceof SVGElement?e.setAttribute("class",n.join(" ")):e.className=n.join(" ")}"undefined"!==typeof window&&(f=window.SVGAnimatedString);var _=!1;if("undefined"!==typeof window){_=!1;try{var v=Object.defineProperty({},"passive",{get:function(){_=!0}});window.addEventListener("test",null,v)}catch(ns){}}function g(){this.__data__=[],this.size=0}var b=g;function y(e,t){return e===t||e!==e&&t!==t}var M=y;function k(e,t){var n=e.length;while(n--)if(M(e[n][0],t))return n;return-1}var w=k,L=Array.prototype,x=L.splice;function S(e){var t=this.__data__,n=w(t,e);if(n<0)return!1;var r=t.length-1;return n==r?t.pop():x.call(t,n,1),--this.size,!0}var T=S;function D(e){var t=this.__data__,n=w(t,e);return n<0?void 0:t[n][1]}var Y=D;function C(e){return w(this.__data__,e)>-1}var E=C;function A(e,t){var n=this.__data__,r=w(n,e);return r<0?(++this.size,n.push([e,t])):n[r][1]=t,this}var O=A;function P(e){var t=-1,n=null==e?0:e.length;this.clear();while(++t<n){var r=e[t];this.set(r[0],r[1])}}P.prototype.clear=b,P.prototype["delete"]=T,P.prototype.get=Y,P.prototype.has=E,P.prototype.set=O;var F=P;function I(){this.__data__=new F,this.size=0}var j=I;function H(e){var t=this.__data__,n=t["delete"](e);return this.size=t.size,n}var R=H;function N(e){return this.__data__.get(e)}var $=N;function z(e){return this.__data__.has(e)}var B=z,W="undefined"!==typeof globalThis?globalThis:"undefined"!==typeof window?window:"undefined"!==typeof e?e:"undefined"!==typeof self?self:{};function U(e,t){return t={exports:{}},e(t,t.exports),t.exports}var V="object"==typeof W&&W&&W.Object===Object&&W,q=V,G="object"==typeof self&&self&&self.Object===Object&&self,J=q||G||Function("return this")(),K=J,X=K.Symbol,Q=X,Z=Object.prototype,ee=Z.hasOwnProperty,te=Z.toString,ne=Q?Q.toStringTag:void 0;function re(e){var t=ee.call(e,ne),n=e[ne];try{e[ne]=void 0;var r=!0}catch(ns){}var i=te.call(e);return r&&(t?e[ne]=n:delete e[ne]),i}var ie=re,ae=Object.prototype,oe=ae.toString;function se(e){return oe.call(e)}var le=se,ue="[object Null]",ce="[object Undefined]",de=Q?Q.toStringTag:void 0;function fe(e){return null==e?void 0===e?ce:ue:de&&de in Object(e)?ie(e):le(e)}var he=fe;function pe(e){var t=typeof e;return null!=e&&("object"==t||"function"==t)}var me=pe,_e="[object AsyncFunction]",ve="[object Function]",ge="[object GeneratorFunction]",be="[object Proxy]";function ye(e){if(!me(e))return!1;var t=he(e);return t==ve||t==ge||t==_e||t==be}var Me=ye,ke=K["__core-js_shared__"],we=ke,Le=function(){var e=/[^.]+$/.exec(we&&we.keys&&we.keys.IE_PROTO||"");return e?"Symbol(src)_1."+e:""}();function xe(e){return!!Le&&Le in e}var Se=xe,Te=Function.prototype,De=Te.toString;function Ye(e){if(null!=e){try{return De.call(e)}catch(ns){}try{return e+""}catch(ns){}}return""}var Ce=Ye,Ee=/[\\^$.*+?()[\]{}|]/g,Ae=/^\[object .+?Constructor\]$/,Oe=Function.prototype,Pe=Object.prototype,Fe=Oe.toString,Ie=Pe.hasOwnProperty,je=RegExp("^"+Fe.call(Ie).replace(Ee,"\\$&").replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g,"$1.*?")+"$");function He(e){if(!me(e)||Se(e))return!1;var t=Me(e)?je:Ae;return t.test(Ce(e))}var Re=He;function Ne(e,t){return null==e?void 0:e[t]}var $e=Ne;function ze(e,t){var n=$e(e,t);return Re(n)?n:void 0}var Be=ze,We=Be(K,"Map"),Ue=We,Ve=Be(Object,"create"),qe=Ve;function Ge(){this.__data__=qe?qe(null):{},this.size=0}var Je=Ge;function Ke(e){var t=this.has(e)&&delete this.__data__[e];return this.size-=t?1:0,t}var Xe=Ke,Qe="__lodash_hash_undefined__",Ze=Object.prototype,et=Ze.hasOwnProperty;function tt(e){var t=this.__data__;if(qe){var n=t[e];return n===Qe?void 0:n}return et.call(t,e)?t[e]:void 0}var nt=tt,rt=Object.prototype,it=rt.hasOwnProperty;function at(e){var t=this.__data__;return qe?void 0!==t[e]:it.call(t,e)}var ot=at,st="__lodash_hash_undefined__";function lt(e,t){var n=this.__data__;return this.size+=this.has(e)?0:1,n[e]=qe&&void 0===t?st:t,this}var ut=lt;function ct(e){var t=-1,n=null==e?0:e.length;this.clear();while(++t<n){var r=e[t];this.set(r[0],r[1])}}ct.prototype.clear=Je,ct.prototype["delete"]=Xe,ct.prototype.get=nt,ct.prototype.has=ot,ct.prototype.set=ut;var dt=ct;function ft(){this.size=0,this.__data__={hash:new dt,map:new(Ue||F),string:new dt}}var ht=ft;function pt(e){var t=typeof e;return"string"==t||"number"==t||"symbol"==t||"boolean"==t?"__proto__"!==e:null===e}var mt=pt;function _t(e,t){var n=e.__data__;return mt(t)?n["string"==typeof t?"string":"hash"]:n.map}var vt=_t;function gt(e){var t=vt(this,e)["delete"](e);return this.size-=t?1:0,t}var bt=gt;function yt(e){return vt(this,e).get(e)}var Mt=yt;function kt(e){return vt(this,e).has(e)}var wt=kt;function Lt(e,t){var n=vt(this,e),r=n.size;return n.set(e,t),this.size+=n.size==r?0:1,this}var xt=Lt;function St(e){var t=-1,n=null==e?0:e.length;this.clear();while(++t<n){var r=e[t];this.set(r[0],r[1])}}St.prototype.clear=ht,St.prototype["delete"]=bt,St.prototype.get=Mt,St.prototype.has=wt,St.prototype.set=xt;var Tt=St,Dt=200;function Yt(e,t){var n=this.__data__;if(n instanceof F){var r=n.__data__;if(!Ue||r.length<Dt-1)return r.push([e,t]),this.size=++n.size,this;n=this.__data__=new Tt(r)}return n.set(e,t),this.size=n.size,this}var Ct=Yt;function Et(e){var t=this.__data__=new F(e);this.size=t.size}Et.prototype.clear=j,Et.prototype["delete"]=R,Et.prototype.get=$,Et.prototype.has=B,Et.prototype.set=Ct;var At=Et,Ot="__lodash_hash_undefined__";function Pt(e){return this.__data__.set(e,Ot),this}var Ft=Pt;function It(e){return this.__data__.has(e)}var jt=It;function Ht(e){var t=-1,n=null==e?0:e.length;this.__data__=new Tt;while(++t<n)this.add(e[t])}Ht.prototype.add=Ht.prototype.push=Ft,Ht.prototype.has=jt;var Rt=Ht;function Nt(e,t){var n=-1,r=null==e?0:e.length;while(++n<r)if(t(e[n],n,e))return!0;return!1}var $t=Nt;function zt(e,t){return e.has(t)}var Bt=zt,Wt=1,Ut=2;function Vt(e,t,n,r,i,a){var o=n&Wt,s=e.length,l=t.length;if(s!=l&&!(o&&l>s))return!1;var u=a.get(e);if(u&&a.get(t))return u==t;var c=-1,d=!0,f=n&Ut?new Rt:void 0;a.set(e,t),a.set(t,e);while(++c<s){var h=e[c],p=t[c];if(r)var m=o?r(p,h,c,t,e,a):r(h,p,c,e,t,a);if(void 0!==m){if(m)continue;d=!1;break}if(f){if(!$t(t,(function(e,t){if(!Bt(f,t)&&(h===e||i(h,e,n,r,a)))return f.push(t)}))){d=!1;break}}else if(h!==p&&!i(h,p,n,r,a)){d=!1;break}}return a["delete"](e),a["delete"](t),d}var qt=Vt,Gt=K.Uint8Array,Jt=Gt;function Kt(e){var t=-1,n=Array(e.size);return e.forEach((function(e,r){n[++t]=[r,e]})),n}var Xt=Kt;function Qt(e){var t=-1,n=Array(e.size);return e.forEach((function(e){n[++t]=e})),n}var Zt=Qt,en=1,tn=2,nn="[object Boolean]",rn="[object Date]",an="[object Error]",on="[object Map]",sn="[object Number]",ln="[object RegExp]",un="[object Set]",cn="[object String]",dn="[object Symbol]",fn="[object ArrayBuffer]",hn="[object DataView]",pn=Q?Q.prototype:void 0,mn=pn?pn.valueOf:void 0;function _n(e,t,n,r,i,a,o){switch(n){case hn:if(e.byteLength!=t.byteLength||e.byteOffset!=t.byteOffset)return!1;e=e.buffer,t=t.buffer;case fn:return!(e.byteLength!=t.byteLength||!a(new Jt(e),new Jt(t)));case nn:case rn:case sn:return M(+e,+t);case an:return e.name==t.name&&e.message==t.message;case ln:case cn:return e==t+"";case on:var s=Xt;case un:var l=r&en;if(s||(s=Zt),e.size!=t.size&&!l)return!1;var u=o.get(e);if(u)return u==t;r|=tn,o.set(e,t);var c=qt(s(e),s(t),r,i,a,o);return o["delete"](e),c;case dn:if(mn)return mn.call(e)==mn.call(t)}return!1}var vn=_n;function gn(e,t){var n=-1,r=t.length,i=e.length;while(++n<r)e[i+n]=t[n];return e}var bn=gn,yn=Array.isArray,Mn=yn;function kn(e,t,n){var r=t(e);return Mn(e)?r:bn(r,n(e))}var wn=kn;function Ln(e,t){var n=-1,r=null==e?0:e.length,i=0,a=[];while(++n<r){var o=e[n];t(o,n,e)&&(a[i++]=o)}return a}var xn=Ln;function Sn(){return[]}var Tn=Sn,Dn=Object.prototype,Yn=Dn.propertyIsEnumerable,Cn=Object.getOwnPropertySymbols,En=Cn?function(e){return null==e?[]:(e=Object(e),xn(Cn(e),(function(t){return Yn.call(e,t)})))}:Tn,An=En;function On(e,t){var n=-1,r=Array(e);while(++n<e)r[n]=t(n);return r}var Pn=On;function Fn(e){return null!=e&&"object"==typeof e}var In=Fn,jn="[object Arguments]";function Hn(e){return In(e)&&he(e)==jn}var Rn=Hn,Nn=Object.prototype,$n=Nn.hasOwnProperty,zn=Nn.propertyIsEnumerable,Bn=Rn(function(){return arguments}())?Rn:function(e){return In(e)&&$n.call(e,"callee")&&!zn.call(e,"callee")},Wn=Bn;function Un(){return!1}var Vn=Un,qn=U((function(e,t){var n=t&&!t.nodeType&&t,r=n&&e&&!e.nodeType&&e,i=r&&r.exports===n,a=i?K.Buffer:void 0,o=a?a.isBuffer:void 0,s=o||Vn;e.exports=s})),Gn=9007199254740991,Jn=/^(?:0|[1-9]\d*)$/;function Kn(e,t){var n=typeof e;return t=null==t?Gn:t,!!t&&("number"==n||"symbol"!=n&&Jn.test(e))&&e>-1&&e%1==0&&e<t}var Xn=Kn,Qn=9007199254740991;function Zn(e){return"number"==typeof e&&e>-1&&e%1==0&&e<=Qn}var er=Zn,tr="[object Arguments]",nr="[object Array]",rr="[object Boolean]",ir="[object Date]",ar="[object Error]",or="[object Function]",sr="[object Map]",lr="[object Number]",ur="[object Object]",cr="[object RegExp]",dr="[object Set]",fr="[object String]",hr="[object WeakMap]",pr="[object ArrayBuffer]",mr="[object DataView]",_r="[object Float32Array]",vr="[object Float64Array]",gr="[object Int8Array]",br="[object Int16Array]",yr="[object Int32Array]",Mr="[object Uint8Array]",kr="[object Uint8ClampedArray]",wr="[object Uint16Array]",Lr="[object Uint32Array]",xr={};function Sr(e){return In(e)&&er(e.length)&&!!xr[he(e)]}xr[_r]=xr[vr]=xr[gr]=xr[br]=xr[yr]=xr[Mr]=xr[kr]=xr[wr]=xr[Lr]=!0,xr[tr]=xr[nr]=xr[pr]=xr[rr]=xr[mr]=xr[ir]=xr[ar]=xr[or]=xr[sr]=xr[lr]=xr[ur]=xr[cr]=xr[dr]=xr[fr]=xr[hr]=!1;var Tr=Sr;function Dr(e){return function(t){return e(t)}}var Yr=Dr,Cr=U((function(e,t){var n=t&&!t.nodeType&&t,r=n&&e&&!e.nodeType&&e,i=r&&r.exports===n,a=i&&q.process,o=function(){try{var e=r&&r.require&&r.require("util").types;return e||a&&a.binding&&a.binding("util")}catch(ns){}}();e.exports=o})),Er=Cr&&Cr.isTypedArray,Ar=Er?Yr(Er):Tr,Or=Ar,Pr=Object.prototype,Fr=Pr.hasOwnProperty;function Ir(e,t){var n=Mn(e),r=!n&&Wn(e),i=!n&&!r&&qn(e),a=!n&&!r&&!i&&Or(e),o=n||r||i||a,s=o?Pn(e.length,String):[],l=s.length;for(var u in e)!t&&!Fr.call(e,u)||o&&("length"==u||i&&("offset"==u||"parent"==u)||a&&("buffer"==u||"byteLength"==u||"byteOffset"==u)||Xn(u,l))||s.push(u);return s}var jr=Ir,Hr=Object.prototype;function Rr(e){var t=e&&e.constructor,n="function"==typeof t&&t.prototype||Hr;return e===n}var Nr=Rr;function $r(e,t){return function(n){return e(t(n))}}var zr=$r,Br=zr(Object.keys,Object),Wr=Br,Ur=Object.prototype,Vr=Ur.hasOwnProperty;function qr(e){if(!Nr(e))return Wr(e);var t=[];for(var n in Object(e))Vr.call(e,n)&&"constructor"!=n&&t.push(n);return t}var Gr=qr;function Jr(e){return null!=e&&er(e.length)&&!Me(e)}var Kr=Jr;function Xr(e){return Kr(e)?jr(e):Gr(e)}var Qr=Xr;function Zr(e){return wn(e,Qr,An)}var ei=Zr,ti=1,ni=Object.prototype,ri=ni.hasOwnProperty;function ii(e,t,n,r,i,a){var o=n&ti,s=ei(e),l=s.length,u=ei(t),c=u.length;if(l!=c&&!o)return!1;var d=l;while(d--){var f=s[d];if(!(o?f in t:ri.call(t,f)))return!1}var h=a.get(e);if(h&&a.get(t))return h==t;var p=!0;a.set(e,t),a.set(t,e);var m=o;while(++d<l){f=s[d];var _=e[f],v=t[f];if(r)var g=o?r(v,_,f,t,e,a):r(_,v,f,e,t,a);if(!(void 0===g?_===v||i(_,v,n,r,a):g)){p=!1;break}m||(m="constructor"==f)}if(p&&!m){var b=e.constructor,y=t.constructor;b==y||!("constructor"in e)||!("constructor"in t)||"function"==typeof b&&b instanceof b&&"function"==typeof y&&y instanceof y||(p=!1)}return a["delete"](e),a["delete"](t),p}var ai=ii,oi=Be(K,"DataView"),si=oi,li=Be(K,"Promise"),ui=li,ci=Be(K,"Set"),di=ci,fi=Be(K,"WeakMap"),hi=fi,pi="[object Map]",mi="[object Object]",_i="[object Promise]",vi="[object Set]",gi="[object WeakMap]",bi="[object DataView]",yi=Ce(si),Mi=Ce(Ue),ki=Ce(ui),wi=Ce(di),Li=Ce(hi),xi=he;(si&&xi(new si(new ArrayBuffer(1)))!=bi||Ue&&xi(new Ue)!=pi||ui&&xi(ui.resolve())!=_i||di&&xi(new di)!=vi||hi&&xi(new hi)!=gi)&&(xi=function(e){var t=he(e),n=t==mi?e.constructor:void 0,r=n?Ce(n):"";if(r)switch(r){case yi:return bi;case Mi:return pi;case ki:return _i;case wi:return vi;case Li:return gi}return t});var Si=xi,Ti=1,Di="[object Arguments]",Yi="[object Array]",Ci="[object Object]",Ei=Object.prototype,Ai=Ei.hasOwnProperty;function Oi(e,t,n,r,i,a){var o=Mn(e),s=Mn(t),l=o?Yi:Si(e),u=s?Yi:Si(t);l=l==Di?Ci:l,u=u==Di?Ci:u;var c=l==Ci,d=u==Ci,f=l==u;if(f&&qn(e)){if(!qn(t))return!1;o=!0,c=!1}if(f&&!c)return a||(a=new At),o||Or(e)?qt(e,t,n,r,i,a):vn(e,t,l,n,r,i,a);if(!(n&Ti)){var h=c&&Ai.call(e,"__wrapped__"),p=d&&Ai.call(t,"__wrapped__");if(h||p){var m=h?e.value():e,_=p?t.value():t;return a||(a=new At),i(m,_,n,r,a)}}return!!f&&(a||(a=new At),ai(e,t,n,r,i,a))}var Pi=Oi;function Fi(e,t,n,r,i){return e===t||(null==e||null==t||!In(e)&&!In(t)?e!==e&&t!==t:Pi(e,t,n,r,Fi,i))}var Ii=Fi;function ji(e,t){return Ii(e,t)}var Hi=ji,Ri={container:!1,delay:0,html:!1,placement:"top",title:"",template:'<div class="tooltip" role="tooltip"><div class="tooltip-arrow"></div><div class="tooltip-inner"></div></div>',trigger:"hover focus",offset:0},Ni=[],$i=function(){function e(t,n){var r=this;o(this,e),u(this,"_events",[]),u(this,"_setTooltipNodeEvent",(function(e,t,n,i){var a=e.relatedreference||e.toElement||e.relatedTarget,o=function n(a){var o=a.relatedreference||a.toElement||a.relatedTarget;r._tooltipNode.removeEventListener(e.type,n),t.contains(o)||r._scheduleHide(t,i.delay,i,a)};return!!r._tooltipNode.contains(a)&&(r._tooltipNode.addEventListener(e.type,o),!0)})),n=d({},Ri,{},n),t.jquery&&(t=t[0]),this.show=this.show.bind(this),this.hide=this.hide.bind(this),this.reference=t,this.options=n,this._isOpen=!1,this._init()}return l(e,[{key:"show",value:function(){this._show(this.reference,this.options)}},{key:"hide",value:function(){this._hide()}},{key:"dispose",value:function(){this._dispose()}},{key:"toggle",value:function(){return this._isOpen?this.hide():this.show()}},{key:"setClasses",value:function(e){this._classes=e}},{key:"setContent",value:function(e){this.options.title=e,this._tooltipNode&&this._setContent(e,this.options)}},{key:"setOptions",value:function(e){var t=!1,n=e&&e.classes||Xi.options.defaultClass;Hi(this._classes,n)||(this.setClasses(n),t=!0),e=Ui(e);var r=!1,i=!1;for(var a in this.options.offset===e.offset&&this.options.placement===e.placement||(r=!0),(this.options.template!==e.template||this.options.trigger!==e.trigger||this.options.container!==e.container||t)&&(i=!0),e)this.options[a]=e[a];if(this._tooltipNode)if(i){var o=this._isOpen;this.dispose(),this._init(),o&&this.show()}else r&&this.popperInstance.update()}},{key:"_init",value:function(){var e="string"===typeof this.options.trigger?this.options.trigger.split(" "):[];this._isDisposed=!1,this._enableDocumentTouch=-1===e.indexOf("manual"),e=e.filter((function(e){return-1!==["click","hover","focus"].indexOf(e)})),this._setEventListeners(this.reference,e,this.options),this.$_originalTitle=this.reference.getAttribute("title"),this.reference.removeAttribute("title"),this.reference.setAttribute("data-original-title",this.$_originalTitle)}},{key:"_create",value:function(e,t){var n=window.document.createElement("div");n.innerHTML=t.trim();var r=n.childNodes[0];return r.id="tooltip_".concat(Math.random().toString(36).substr(2,10)),r.setAttribute("aria-hidden","true"),this.options.autoHide&&-1!==this.options.trigger.indexOf("hover")&&(r.addEventListener("mouseenter",this.hide),r.addEventListener("click",this.hide)),r}},{key:"_setContent",value:function(e,t){var n=this;this.asyncContent=!1,this._applyContent(e,t).then((function(){n.popperInstance.update()}))}},{key:"_applyContent",value:function(e,t){var n=this;return new Promise((function(r,i){var a=t.html,o=n._tooltipNode;if(o){var s=o.querySelector(n.options.innerSelector);if(1===e.nodeType){if(a){while(s.firstChild)s.removeChild(s.firstChild);s.appendChild(e)}}else{if("function"===typeof e){var l=e();return void(l&&"function"===typeof l.then?(n.asyncContent=!0,t.loadingClass&&p(o,t.loadingClass),t.loadingContent&&n._applyContent(t.loadingContent,t),l.then((function(e){return t.loadingClass&&m(o,t.loadingClass),n._applyContent(e,t)})).then(r).catch(i)):n._applyContent(l,t).then(r).catch(i))}a?s.innerHTML=e:s.innerText=e}r()}}))}},{key:"_show",value:function(e,t){if(t&&"string"===typeof t.container){var n=document.querySelector(t.container);if(!n)return}clearTimeout(this._disposeTimer),t=Object.assign({},t),delete t.offset;var r=!0;this._tooltipNode&&(p(this._tooltipNode,this._classes),r=!1);var i=this._ensureShown(e,t);return r&&this._tooltipNode&&p(this._tooltipNode,this._classes),p(e,["v-tooltip-open"]),i}},{key:"_ensureShown",value:function(e,t){var n=this;if(this._isOpen)return this;if(this._isOpen=!0,Ni.push(this),this._tooltipNode)return this._tooltipNode.style.display="",this._tooltipNode.setAttribute("aria-hidden","false"),this.popperInstance.enableEventListeners(),this.popperInstance.update(),this.asyncContent&&this._setContent(t.title,t),this;var i=e.getAttribute("title")||t.title;if(!i)return this;var a=this._create(e,t.template);this._tooltipNode=a,e.setAttribute("aria-describedby",a.id);var o=this._findContainer(t.container,e);this._append(a,o);var s=d({},t.popperOptions,{placement:t.placement});return s.modifiers=d({},s.modifiers,{arrow:{element:this.options.arrowSelector}}),t.boundariesElement&&(s.modifiers.preventOverflow={boundariesElement:t.boundariesElement}),this.popperInstance=new r["a"](e,a,s),this._setContent(i,t),requestAnimationFrame((function(){!n._isDisposed&&n.popperInstance?(n.popperInstance.update(),requestAnimationFrame((function(){n._isDisposed?n.dispose():n._isOpen&&a.setAttribute("aria-hidden","false")}))):n.dispose()})),this}},{key:"_noLongerOpen",value:function(){var e=Ni.indexOf(this);-1!==e&&Ni.splice(e,1)}},{key:"_hide",value:function(){var e=this;if(!this._isOpen)return this;this._isOpen=!1,this._noLongerOpen(),this._tooltipNode.style.display="none",this._tooltipNode.setAttribute("aria-hidden","true"),this.popperInstance.disableEventListeners(),clearTimeout(this._disposeTimer);var t=Xi.options.disposeTimeout;return null!==t&&(this._disposeTimer=setTimeout((function(){e._tooltipNode&&(e._tooltipNode.removeEventListener("mouseenter",e.hide),e._tooltipNode.removeEventListener("click",e.hide),e._removeTooltipNode())}),t)),m(this.reference,["v-tooltip-open"]),this}},{key:"_removeTooltipNode",value:function(){if(this._tooltipNode){var e=this._tooltipNode.parentNode;e&&(e.removeChild(this._tooltipNode),this.reference.removeAttribute("aria-describedby")),this._tooltipNode=null}}},{key:"_dispose",value:function(){var e=this;return this._isDisposed=!0,this.reference.removeAttribute("data-original-title"),this.$_originalTitle&&this.reference.setAttribute("title",this.$_originalTitle),this._events.forEach((function(t){var n=t.func,r=t.event;e.reference.removeEventListener(r,n)})),this._events=[],this._tooltipNode?(this._hide(),this._tooltipNode.removeEventListener("mouseenter",this.hide),this._tooltipNode.removeEventListener("click",this.hide),this.popperInstance.destroy(),this.popperInstance.options.removeOnDestroy||this._removeTooltipNode()):this._noLongerOpen(),this}},{key:"_findContainer",value:function(e,t){return"string"===typeof e?e=window.document.querySelector(e):!1===e&&(e=t.parentNode),e}},{key:"_append",value:function(e,t){t.appendChild(e)}},{key:"_setEventListeners",value:function(e,t,n){var r=this,i=[],a=[];t.forEach((function(e){switch(e){case"hover":i.push("mouseenter"),a.push("mouseleave"),r.options.hideOnTargetClick&&a.push("click");break;case"focus":i.push("focus"),a.push("blur"),r.options.hideOnTargetClick&&a.push("click");break;case"click":i.push("click"),a.push("click");break}})),i.forEach((function(t){var i=function(t){!0!==r._isOpen&&(t.usedByTooltip=!0,r._scheduleShow(e,n.delay,n,t))};r._events.push({event:t,func:i}),e.addEventListener(t,i)})),a.forEach((function(t){var i=function(t){!0!==t.usedByTooltip&&r._scheduleHide(e,n.delay,n,t)};r._events.push({event:t,func:i}),e.addEventListener(t,i)}))}},{key:"_onDocumentTouch",value:function(e){this._enableDocumentTouch&&this._scheduleHide(this.reference,this.options.delay,this.options,e)}},{key:"_scheduleShow",value:function(e,t,n){var r=this,i=t&&t.show||t||0;clearTimeout(this._scheduleTimer),this._scheduleTimer=window.setTimeout((function(){return r._show(e,n)}),i)}},{key:"_scheduleHide",value:function(e,t,n,r){var i=this,a=t&&t.hide||t||0;clearTimeout(this._scheduleTimer),this._scheduleTimer=window.setTimeout((function(){if(!1!==i._isOpen&&i._tooltipNode.ownerDocument.body.contains(i._tooltipNode)){if("mouseleave"===r.type){var a=i._setTooltipNodeEvent(r,e,t,n);if(a)return}i._hide(e,n)}}),a)}}]),e}();"undefined"!==typeof document&&document.addEventListener("touchstart",(function(e){for(var t=0;t<Ni.length;t++)Ni[t]._onDocumentTouch(e)}),!_||{passive:!0,capture:!0});var zi={enabled:!0},Bi=["top","top-start","top-end","right","right-start","right-end","bottom","bottom-start","bottom-end","left","left-start","left-end"],Wi={defaultPlacement:"top",defaultClass:"vue-tooltip-theme",defaultTargetClass:"has-tooltip",defaultHtml:!0,defaultTemplate:'<div class="tooltip" role="tooltip"><div class="tooltip-arrow"></div><div class="tooltip-inner"></div></div>',defaultArrowSelector:".tooltip-arrow, .tooltip__arrow",defaultInnerSelector:".tooltip-inner, .tooltip__inner",defaultDelay:0,defaultTrigger:"hover focus",defaultOffset:0,defaultContainer:"body",defaultBoundariesElement:void 0,defaultPopperOptions:{},defaultLoadingClass:"tooltip-loading",defaultLoadingContent:"...",autoHide:!0,defaultHideOnTargetClick:!0,disposeTimeout:5e3,popover:{defaultPlacement:"bottom",defaultClass:"vue-popover-theme",defaultBaseClass:"tooltip popover",defaultWrapperClass:"wrapper",defaultInnerClass:"tooltip-inner popover-inner",defaultArrowClass:"tooltip-arrow popover-arrow",defaultOpenClass:"open",defaultDelay:0,defaultTrigger:"click",defaultOffset:0,defaultContainer:"body",defaultBoundariesElement:void 0,defaultPopperOptions:{},defaultAutoHide:!0,defaultHandleResize:!0}};function Ui(e){var t={placement:"undefined"!==typeof e.placement?e.placement:Xi.options.defaultPlacement,delay:"undefined"!==typeof e.delay?e.delay:Xi.options.defaultDelay,html:"undefined"!==typeof e.html?e.html:Xi.options.defaultHtml,template:"undefined"!==typeof e.template?e.template:Xi.options.defaultTemplate,arrowSelector:"undefined"!==typeof e.arrowSelector?e.arrowSelector:Xi.options.defaultArrowSelector,innerSelector:"undefined"!==typeof e.innerSelector?e.innerSelector:Xi.options.defaultInnerSelector,trigger:"undefined"!==typeof e.trigger?e.trigger:Xi.options.defaultTrigger,offset:"undefined"!==typeof e.offset?e.offset:Xi.options.defaultOffset,container:"undefined"!==typeof e.container?e.container:Xi.options.defaultContainer,boundariesElement:"undefined"!==typeof e.boundariesElement?e.boundariesElement:Xi.options.defaultBoundariesElement,autoHide:"undefined"!==typeof e.autoHide?e.autoHide:Xi.options.autoHide,hideOnTargetClick:"undefined"!==typeof e.hideOnTargetClick?e.hideOnTargetClick:Xi.options.defaultHideOnTargetClick,loadingClass:"undefined"!==typeof e.loadingClass?e.loadingClass:Xi.options.defaultLoadingClass,loadingContent:"undefined"!==typeof e.loadingContent?e.loadingContent:Xi.options.defaultLoadingContent,popperOptions:d({},"undefined"!==typeof e.popperOptions?e.popperOptions:Xi.options.defaultPopperOptions)};if(t.offset){var n=a(t.offset),r=t.offset;("number"===n||"string"===n&&-1===r.indexOf(","))&&(r="0, ".concat(r)),t.popperOptions.modifiers||(t.popperOptions.modifiers={}),t.popperOptions.modifiers.offset={offset:r}}return t.trigger&&-1!==t.trigger.indexOf("click")&&(t.hideOnTargetClick=!1),t}function Vi(e,t){for(var n=e.placement,r=0;r<Bi.length;r++){var i=Bi[r];t[i]&&(n=i)}return n}function qi(e){var t=a(e);return"string"===t?e:!(!e||"object"!==t)&&e.content}function Gi(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},r=qi(t),i="undefined"!==typeof t.classes?t.classes:Xi.options.defaultClass,a=d({title:r},Ui(d({},t,{placement:Vi(t,n)}))),o=e._tooltip=new $i(e,a);o.setClasses(i),o._vueEl=e;var s="undefined"!==typeof t.targetClasses?t.targetClasses:Xi.options.defaultTargetClass;return e._tooltipTargetClasses=s,p(e,s),o}function Ji(e){e._tooltip&&(e._tooltip.dispose(),delete e._tooltip,delete e._tooltipOldShow),e._tooltipTargetClasses&&(m(e,e._tooltipTargetClasses),delete e._tooltipTargetClasses)}function Ki(e,t){var n,r=t.value,i=(t.oldValue,t.modifiers),a=qi(r);a&&zi.enabled?(e._tooltip?(n=e._tooltip,n.setContent(a),n.setOptions(d({},r,{placement:Vi(r,i)}))):n=Gi(e,r,i),"undefined"!==typeof r.show&&r.show!==e._tooltipOldShow&&(e._tooltipOldShow=r.show,r.show?n.show():n.hide())):Ji(e)}var Xi={options:Wi,bind:Ki,update:Ki,unbind:function(e){Ji(e)}};function Qi(e){e.addEventListener("click",ea),e.addEventListener("touchstart",ta,!!_&&{passive:!0})}function Zi(e){e.removeEventListener("click",ea),e.removeEventListener("touchstart",ta),e.removeEventListener("touchend",na),e.removeEventListener("touchcancel",ra)}function ea(e){var t=e.currentTarget;e.closePopover=!t.$_vclosepopover_touch,e.closeAllPopover=t.$_closePopoverModifiers&&!!t.$_closePopoverModifiers.all}function ta(e){if(1===e.changedTouches.length){var t=e.currentTarget;t.$_vclosepopover_touch=!0;var n=e.changedTouches[0];t.$_vclosepopover_touchPoint=n,t.addEventListener("touchend",na),t.addEventListener("touchcancel",ra)}}function na(e){var t=e.currentTarget;if(t.$_vclosepopover_touch=!1,1===e.changedTouches.length){var n=e.changedTouches[0],r=t.$_vclosepopover_touchPoint;e.closePopover=Math.abs(n.screenY-r.screenY)<20&&Math.abs(n.screenX-r.screenX)<20,e.closeAllPopover=t.$_closePopoverModifiers&&!!t.$_closePopoverModifiers.all}}function ra(e){var t=e.currentTarget;t.$_vclosepopover_touch=!1}var ia={bind:function(e,t){var n=t.value,r=t.modifiers;e.$_closePopoverModifiers=r,("undefined"===typeof n||n)&&Qi(e)},update:function(e,t){var n=t.value,r=t.oldValue,i=t.modifiers;e.$_closePopoverModifiers=i,n!==r&&("undefined"===typeof n||n?Qi(e):Zi(e))},unbind:function(e){Zi(e)}};function aa(e){var t=Xi.options.popover[e];return"undefined"===typeof t?Xi.options[e]:t}var oa=!1;"undefined"!==typeof window&&"undefined"!==typeof navigator&&(oa=/iPad|iPhone|iPod/.test(navigator.userAgent)&&!window.MSStream);var sa=[],la=function(){};"undefined"!==typeof window&&(la=window.Element);var ua={name:"VPopover",components:{ResizeObserver:i["a"]},props:{open:{type:Boolean,default:!1},disabled:{type:Boolean,default:!1},placement:{type:String,default:function(){return aa("defaultPlacement")}},delay:{type:[String,Number,Object],default:function(){return aa("defaultDelay")}},offset:{type:[String,Number],default:function(){return aa("defaultOffset")}},trigger:{type:String,default:function(){return aa("defaultTrigger")}},container:{type:[String,Object,la,Boolean],default:function(){return aa("defaultContainer")}},boundariesElement:{type:[String,la],default:function(){return aa("defaultBoundariesElement")}},popperOptions:{type:Object,default:function(){return aa("defaultPopperOptions")}},popoverClass:{type:[String,Array],default:function(){return aa("defaultClass")}},popoverBaseClass:{type:[String,Array],default:function(){return Xi.options.popover.defaultBaseClass}},popoverInnerClass:{type:[String,Array],default:function(){return Xi.options.popover.defaultInnerClass}},popoverWrapperClass:{type:[String,Array],default:function(){return Xi.options.popover.defaultWrapperClass}},popoverArrowClass:{type:[String,Array],default:function(){return Xi.options.popover.defaultArrowClass}},autoHide:{type:Boolean,default:function(){return Xi.options.popover.defaultAutoHide}},handleResize:{type:Boolean,default:function(){return Xi.options.popover.defaultHandleResize}},openGroup:{type:String,default:null},openClass:{type:[String,Array],default:function(){return Xi.options.popover.defaultOpenClass}}},data:function(){return{isOpen:!1,id:Math.random().toString(36).substr(2,10)}},computed:{cssClass:function(){return u({},this.openClass,this.isOpen)},popoverId:function(){return"popover_".concat(this.id)}},watch:{open:function(e){e?this.show():this.hide()},disabled:function(e,t){e!==t&&(e?this.hide():this.open&&this.show())},container:function(e){if(this.isOpen&&this.popperInstance){var t=this.$refs.popover,n=this.$refs.trigger,r=this.$_findContainer(this.container,n);if(!r)return void console.warn("No container for popover",this);r.appendChild(t),this.popperInstance.scheduleUpdate()}},trigger:function(e){this.$_removeEventListeners(),this.$_addEventListeners()},placement:function(e){var t=this;this.$_updatePopper((function(){t.popperInstance.options.placement=e}))},offset:"$_restartPopper",boundariesElement:"$_restartPopper",popperOptions:{handler:"$_restartPopper",deep:!0}},created:function(){this.$_isDisposed=!1,this.$_mounted=!1,this.$_events=[],this.$_preventOpen=!1},mounted:function(){var e=this.$refs.popover;e.parentNode&&e.parentNode.removeChild(e),this.$_init(),this.open&&this.show()},deactivated:function(){this.hide()},beforeDestroy:function(){this.dispose()},methods:{show:function(){var e=this,t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},n=t.event,r=(t.skipDelay,t.force),i=void 0!==r&&r;!i&&this.disabled||(this.$_scheduleShow(n),this.$emit("show")),this.$emit("update:open",!0),this.$_beingShowed=!0,requestAnimationFrame((function(){e.$_beingShowed=!1}))},hide:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},t=e.event;e.skipDelay;this.$_scheduleHide(t),this.$emit("hide"),this.$emit("update:open",!1)},dispose:function(){if(this.$_isDisposed=!0,this.$_removeEventListeners(),this.hide({skipDelay:!0}),this.popperInstance&&(this.popperInstance.destroy(),!this.popperInstance.options.removeOnDestroy)){var e=this.$refs.popover;e.parentNode&&e.parentNode.removeChild(e)}this.$_mounted=!1,this.popperInstance=null,this.isOpen=!1,this.$emit("dispose")},$_init:function(){-1===this.trigger.indexOf("manual")&&this.$_addEventListeners()},$_show:function(){var e=this,t=this.$refs.trigger,n=this.$refs.popover;if(clearTimeout(this.$_disposeTimer),!this.isOpen){if(this.popperInstance&&(this.isOpen=!0,this.popperInstance.enableEventListeners(),this.popperInstance.scheduleUpdate()),!this.$_mounted){var i=this.$_findContainer(this.container,t);if(!i)return void console.warn("No container for popover",this);i.appendChild(n),this.$_mounted=!0}if(!this.popperInstance){var a=d({},this.popperOptions,{placement:this.placement});if(a.modifiers=d({},a.modifiers,{arrow:d({},a.modifiers&&a.modifiers.arrow,{element:this.$refs.arrow})}),this.offset){var o=this.$_getOffset();a.modifiers.offset=d({},a.modifiers&&a.modifiers.offset,{offset:o})}this.boundariesElement&&(a.modifiers.preventOverflow=d({},a.modifiers&&a.modifiers.preventOverflow,{boundariesElement:this.boundariesElement})),this.popperInstance=new r["a"](t,n,a),requestAnimationFrame((function(){if(e.hidden)return e.hidden=!1,void e.$_hide();!e.$_isDisposed&&e.popperInstance?(e.popperInstance.scheduleUpdate(),requestAnimationFrame((function(){if(e.hidden)return e.hidden=!1,void e.$_hide();e.$_isDisposed?e.dispose():e.isOpen=!0}))):e.dispose()}))}var s=this.openGroup;if(s)for(var l,u=0;u<sa.length;u++)l=sa[u],l.openGroup!==s&&(l.hide(),l.$emit("close-group"));sa.push(this),this.$emit("apply-show")}},$_hide:function(){var e=this;if(this.isOpen){var t=sa.indexOf(this);-1!==t&&sa.splice(t,1),this.isOpen=!1,this.popperInstance&&this.popperInstance.disableEventListeners(),clearTimeout(this.$_disposeTimer);var n=Xi.options.popover.disposeTimeout||Xi.options.disposeTimeout;null!==n&&(this.$_disposeTimer=setTimeout((function(){var t=e.$refs.popover;t&&(t.parentNode&&t.parentNode.removeChild(t),e.$_mounted=!1)}),n)),this.$emit("apply-hide")}},$_findContainer:function(e,t){return"string"===typeof e?e=window.document.querySelector(e):!1===e&&(e=t.parentNode),e},$_getOffset:function(){var e=a(this.offset),t=this.offset;return("number"===e||"string"===e&&-1===t.indexOf(","))&&(t="0, ".concat(t)),t},$_addEventListeners:function(){var e=this,t=this.$refs.trigger,n=[],r=[],i="string"===typeof this.trigger?this.trigger.split(" ").filter((function(e){return-1!==["click","hover","focus"].indexOf(e)})):[];i.forEach((function(e){switch(e){case"hover":n.push("mouseenter"),r.push("mouseleave");break;case"focus":n.push("focus"),r.push("blur");break;case"click":n.push("click"),r.push("click");break}})),n.forEach((function(n){var r=function(t){e.isOpen||(t.usedByTooltip=!0,!e.$_preventOpen&&e.show({event:t}),e.hidden=!1)};e.$_events.push({event:n,func:r}),t.addEventListener(n,r)})),r.forEach((function(n){var r=function(t){t.usedByTooltip||(e.hide({event:t}),e.hidden=!0)};e.$_events.push({event:n,func:r}),t.addEventListener(n,r)}))},$_scheduleShow:function(){var e=arguments.length>1&&void 0!==arguments[1]&&arguments[1];if(clearTimeout(this.$_scheduleTimer),e)this.$_show();else{var t=parseInt(this.delay&&this.delay.show||this.delay||0);this.$_scheduleTimer=setTimeout(this.$_show.bind(this),t)}},$_scheduleHide:function(){var e=this,t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:null,n=arguments.length>1&&void 0!==arguments[1]&&arguments[1];if(clearTimeout(this.$_scheduleTimer),n)this.$_hide();else{var r=parseInt(this.delay&&this.delay.hide||this.delay||0);this.$_scheduleTimer=setTimeout((function(){if(e.isOpen){if(t&&"mouseleave"===t.type){var n=e.$_setTooltipNodeEvent(t);if(n)return}e.$_hide()}}),r)}},$_setTooltipNodeEvent:function(e){var t=this,n=this.$refs.trigger,r=this.$refs.popover,i=e.relatedreference||e.toElement||e.relatedTarget,a=function i(a){var o=a.relatedreference||a.toElement||a.relatedTarget;r.removeEventListener(e.type,i),n.contains(o)||t.hide({event:a})};return!!r.contains(i)&&(r.addEventListener(e.type,a),!0)},$_removeEventListeners:function(){var e=this.$refs.trigger;this.$_events.forEach((function(t){var n=t.func,r=t.event;e.removeEventListener(r,n)})),this.$_events=[]},$_updatePopper:function(e){this.popperInstance&&(e(),this.isOpen&&this.popperInstance.scheduleUpdate())},$_restartPopper:function(){if(this.popperInstance){var e=this.isOpen;this.dispose(),this.$_isDisposed=!1,this.$_init(),e&&this.show({skipDelay:!0,force:!0})}},$_handleGlobalClose:function(e){var t=this,n=arguments.length>1&&void 0!==arguments[1]&&arguments[1];this.$_beingShowed||(this.hide({event:e}),e.closePopover?this.$emit("close-directive"):this.$emit("auto-hide"),n&&(this.$_preventOpen=!0,setTimeout((function(){t.$_preventOpen=!1}),300)))},$_handleResize:function(){this.isOpen&&this.popperInstance&&(this.popperInstance.scheduleUpdate(),this.$emit("resize"))}}};function ca(e){fa(e)}function da(e){fa(e,!0)}function fa(e){for(var t=arguments.length>1&&void 0!==arguments[1]&&arguments[1],n=function(n){var r=sa[n];if(r.$refs.popover){var i=r.$refs.popover.contains(e.target);requestAnimationFrame((function(){(e.closeAllPopover||e.closePopover&&i||r.autoHide&&!i)&&r.$_handleGlobalClose(e,t)}))}},r=0;r<sa.length;r++)n(r)}function ha(e,t,n,r,i,a,o,s,l,u){"boolean"!==typeof o&&(l=s,s=o,o=!1);const c="function"===typeof n?n.options:n;let d;if(e&&e.render&&(c.render=e.render,c.staticRenderFns=e.staticRenderFns,c._compiled=!0,i&&(c.functional=!0)),r&&(c._scopeId=r),a?(d=function(e){e=e||this.$vnode&&this.$vnode.ssrContext||this.parent&&this.parent.$vnode&&this.parent.$vnode.ssrContext,e||"undefined"===typeof __VUE_SSR_CONTEXT__||(e=__VUE_SSR_CONTEXT__),t&&t.call(this,l(e)),e&&e._registeredComponents&&e._registeredComponents.add(a)},c._ssrRegister=d):t&&(d=o?function(e){t.call(this,u(e,this.$root.$options.shadowRoot))}:function(e){t.call(this,s(e))}),d)if(c.functional){const e=c.render;c.render=function(t,n){return d.call(n),e(t,n)}}else{const e=c.beforeCreate;c.beforeCreate=e?[].concat(e,d):[d]}return n}"undefined"!==typeof document&&"undefined"!==typeof window&&(oa?document.addEventListener("touchend",da,!_||{passive:!0,capture:!0}):window.addEventListener("click",ca,!0));var pa=ua,ma=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"v-popover",class:e.cssClass},[n("div",{ref:"trigger",staticClass:"trigger",staticStyle:{display:"inline-block"},attrs:{"aria-describedby":e.popoverId,tabindex:-1!==e.trigger.indexOf("focus")?0:void 0}},[e._t("default")],2),e._v(" "),n("div",{ref:"popover",class:[e.popoverBaseClass,e.popoverClass,e.cssClass],style:{visibility:e.isOpen?"visible":"hidden"},attrs:{id:e.popoverId,"aria-hidden":e.isOpen?"false":"true",tabindex:e.autoHide?0:void 0},on:{keyup:function(t){if(!t.type.indexOf("key")&&e._k(t.keyCode,"esc",27,t.key,["Esc","Escape"]))return null;e.autoHide&&e.hide()}}},[n("div",{class:e.popoverWrapperClass},[n("div",{ref:"inner",class:e.popoverInnerClass,staticStyle:{position:"relative"}},[n("div",[e._t("popover")],2),e._v(" "),e.handleResize?n("ResizeObserver",{on:{notify:e.$_handleResize}}):e._e()],1),e._v(" "),n("div",{ref:"arrow",class:e.popoverArrowClass})])])])},_a=[];ma._withStripped=!0;var va=void 0,ga=void 0,ba=void 0,ya=!1,Ma=ha({render:ma,staticRenderFns:_a},va,pa,ga,ya,ba,!1,void 0,void 0,void 0),ka=function(){try{var e=Be(Object,"defineProperty");return e({},"",{}),e}catch(ns){}}(),wa=ka;function La(e,t,n){"__proto__"==t&&wa?wa(e,t,{configurable:!0,enumerable:!0,value:n,writable:!0}):e[t]=n}var xa=La;function Sa(e,t,n){(void 0!==n&&!M(e[t],n)||void 0===n&&!(t in e))&&xa(e,t,n)}var Ta=Sa;function Da(e){return function(t,n,r){var i=-1,a=Object(t),o=r(t),s=o.length;while(s--){var l=o[e?s:++i];if(!1===n(a[l],l,a))break}return t}}var Ya=Da,Ca=Ya(),Ea=Ca,Aa=U((function(e,t){var n=t&&!t.nodeType&&t,r=n&&e&&!e.nodeType&&e,i=r&&r.exports===n,a=i?K.Buffer:void 0,o=a?a.allocUnsafe:void 0;function s(e,t){if(t)return e.slice();var n=e.length,r=o?o(n):new e.constructor(n);return e.copy(r),r}e.exports=s}));function Oa(e){var t=new e.constructor(e.byteLength);return new Jt(t).set(new Jt(e)),t}var Pa=Oa;function Fa(e,t){var n=t?Pa(e.buffer):e.buffer;return new e.constructor(n,e.byteOffset,e.length)}var Ia=Fa;function ja(e,t){var n=-1,r=e.length;t||(t=Array(r));while(++n<r)t[n]=e[n];return t}var Ha=ja,Ra=Object.create,Na=function(){function e(){}return function(t){if(!me(t))return{};if(Ra)return Ra(t);e.prototype=t;var n=new e;return e.prototype=void 0,n}}(),$a=Na,za=zr(Object.getPrototypeOf,Object),Ba=za;function Wa(e){return"function"!=typeof e.constructor||Nr(e)?{}:$a(Ba(e))}var Ua=Wa;function Va(e){return In(e)&&Kr(e)}var qa=Va,Ga="[object Object]",Ja=Function.prototype,Ka=Object.prototype,Xa=Ja.toString,Qa=Ka.hasOwnProperty,Za=Xa.call(Object);function eo(e){if(!In(e)||he(e)!=Ga)return!1;var t=Ba(e);if(null===t)return!0;var n=Qa.call(t,"constructor")&&t.constructor;return"function"==typeof n&&n instanceof n&&Xa.call(n)==Za}var to=eo;function no(e,t){if(("constructor"!==t||"function"!==typeof e[t])&&"__proto__"!=t)return e[t]}var ro=no,io=Object.prototype,ao=io.hasOwnProperty;function oo(e,t,n){var r=e[t];ao.call(e,t)&&M(r,n)&&(void 0!==n||t in e)||xa(e,t,n)}var so=oo;function lo(e,t,n,r){var i=!n;n||(n={});var a=-1,o=t.length;while(++a<o){var s=t[a],l=r?r(n[s],e[s],s,n,e):void 0;void 0===l&&(l=e[s]),i?xa(n,s,l):so(n,s,l)}return n}var uo=lo;function co(e){var t=[];if(null!=e)for(var n in Object(e))t.push(n);return t}var fo=co,ho=Object.prototype,po=ho.hasOwnProperty;function mo(e){if(!me(e))return fo(e);var t=Nr(e),n=[];for(var r in e)("constructor"!=r||!t&&po.call(e,r))&&n.push(r);return n}var _o=mo;function vo(e){return Kr(e)?jr(e,!0):_o(e)}var go=vo;function bo(e){return uo(e,go(e))}var yo=bo;function Mo(e,t,n,r,i,a,o){var s=ro(e,n),l=ro(t,n),u=o.get(l);if(u)Ta(e,n,u);else{var c=a?a(s,l,n+"",e,t,o):void 0,d=void 0===c;if(d){var f=Mn(l),h=!f&&qn(l),p=!f&&!h&&Or(l);c=l,f||h||p?Mn(s)?c=s:qa(s)?c=Ha(s):h?(d=!1,c=Aa(l,!0)):p?(d=!1,c=Ia(l,!0)):c=[]:to(l)||Wn(l)?(c=s,Wn(s)?c=yo(s):me(s)&&!Me(s)||(c=Ua(l))):d=!1}d&&(o.set(l,c),i(c,l,r,a,o),o["delete"](l)),Ta(e,n,c)}}var ko=Mo;function wo(e,t,n,r,i){e!==t&&Ea(t,(function(a,o){if(i||(i=new At),me(a))ko(e,t,o,n,wo,r,i);else{var s=r?r(ro(e,o),a,o+"",e,t,i):void 0;void 0===s&&(s=a),Ta(e,o,s)}}),go)}var Lo=wo;function xo(e){return e}var So=xo;function To(e,t,n){switch(n.length){case 0:return e.call(t);case 1:return e.call(t,n[0]);case 2:return e.call(t,n[0],n[1]);case 3:return e.call(t,n[0],n[1],n[2])}return e.apply(t,n)}var Do=To,Yo=Math.max;function Co(e,t,n){return t=Yo(void 0===t?e.length-1:t,0),function(){var r=arguments,i=-1,a=Yo(r.length-t,0),o=Array(a);while(++i<a)o[i]=r[t+i];i=-1;var s=Array(t+1);while(++i<t)s[i]=r[i];return s[t]=n(o),Do(e,this,s)}}var Eo=Co;function Ao(e){return function(){return e}}var Oo=Ao,Po=wa?function(e,t){return wa(e,"toString",{configurable:!0,enumerable:!1,value:Oo(t),writable:!0})}:So,Fo=Po,Io=800,jo=16,Ho=Date.now;function Ro(e){var t=0,n=0;return function(){var r=Ho(),i=jo-(r-n);if(n=r,i>0){if(++t>=Io)return arguments[0]}else t=0;return e.apply(void 0,arguments)}}var No=Ro,$o=No(Fo),zo=$o;function Bo(e,t){return zo(Eo(e,t,So),e+"")}var Wo=Bo;function Uo(e,t,n){if(!me(n))return!1;var r=typeof t;return!!("number"==r?Kr(n)&&Xn(t,n.length):"string"==r&&t in n)&&M(n[t],e)}var Vo=Uo;function qo(e){return Wo((function(t,n){var r=-1,i=n.length,a=i>1?n[i-1]:void 0,o=i>2?n[2]:void 0;a=e.length>3&&"function"==typeof a?(i--,a):void 0,o&&Vo(n[0],n[1],o)&&(a=i<3?void 0:a,i=1),t=Object(t);while(++r<i){var s=n[r];s&&e(t,s,r,a)}return t}))}var Go=qo,Jo=Go((function(e,t,n){Lo(e,t,n)})),Ko=Jo;function Xo(e,t){void 0===t&&(t={});var n=t.insertAt;if(e&&"undefined"!==typeof document){var r=document.head||document.getElementsByTagName("head")[0],i=document.createElement("style");i.type="text/css","top"===n&&r.firstChild?r.insertBefore(i,r.firstChild):r.appendChild(i),i.styleSheet?i.styleSheet.cssText=e:i.appendChild(document.createTextNode(e))}}var Qo=".resize-observer[data-v-b329ee4c]{position:absolute;top:0;left:0;z-index:-1;width:100%;height:100%;border:none;background-color:transparent;pointer-events:none;display:block;overflow:hidden;opacity:0}.resize-observer[data-v-b329ee4c] object{display:block;position:absolute;top:0;left:0;height:100%;width:100%;overflow:hidden;pointer-events:none;z-index:-1}";function Zo(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{};if(!Zo.installed){Zo.installed=!0;var n={};Ko(n,Wi,t),es.options=n,Xi.options=n,e.directive("tooltip",Xi),e.directive("close-popover",ia),e.component("v-popover",Ma)}}Xo(Qo);var es={install:Zo,get enabled(){return zi.enabled},set enabled(e){zi.enabled=e}},ts=null;"undefined"!==typeof window?ts=window.Vue:"undefined"!==typeof e&&(ts=e.Vue),ts&&ts.use(es),t["a"]=es}).call(this,n("c8ba"))},e3db:function(e,t){var n={}.toString;e.exports=Array.isArray||function(e){return"[object Array]"==n.call(e)}},e439:function(e,t,n){var r=n("23e7"),i=n("d039"),a=n("fc6a"),o=n("06cf").f,s=n("83ab"),l=i((function(){o(1)})),u=!s||l;r({target:"Object",stat:!0,forced:u,sham:!s},{getOwnPropertyDescriptor:function(e,t){return o(a(e),t)}})},e4d7:function(e,t,n){"use strict";n("c516")},e538:function(e,t,n){var r=n("b622");t.f=r},e58c:function(e,t,n){"use strict";var r=n("fc6a"),i=n("a691"),a=n("50c4"),o=n("a640"),s=n("ae40"),l=Math.min,u=[].lastIndexOf,c=!!u&&1/[1].lastIndexOf(1,-0)<0,d=o("lastIndexOf"),f=s("indexOf",{ACCESSORS:!0,1:0}),h=c||!d||!f;e.exports=h?function(e){if(c)return u.apply(this,arguments)||0;var t=r(this),n=a(t.length),o=n-1;for(arguments.length>1&&(o=l(o,i(arguments[1]))),o<0&&(o=n+o);o>=0;o--)if(o in t&&t[o]===e)return o||0;return-1}:u},e59d:function(e,t,n){},e667:function(e,t){e.exports=function(e){try{return{error:!1,value:e()}}catch(t){return{error:!0,value:t}}}},e6cf:function(e,t,n){"use strict";var r,i,a,o,s=n("23e7"),l=n("c430"),u=n("da84"),c=n("d066"),d=n("fea9"),f=n("6eeb"),h=n("e2cc"),p=n("d44e"),m=n("2626"),_=n("861d"),v=n("1c0b"),g=n("19aa"),b=n("8925"),y=n("2266"),M=n("1c7e"),k=n("4840"),w=n("2cf4").set,L=n("b575"),x=n("cdf9"),S=n("44de"),T=n("f069"),D=n("e667"),Y=n("69f3"),C=n("94ca"),E=n("b622"),A=n("605d"),O=n("2d00"),P=E("species"),F="Promise",I=Y.get,j=Y.set,H=Y.getterFor(F),R=d,N=u.TypeError,$=u.document,z=u.process,B=c("fetch"),W=T.f,U=W,V=!!($&&$.createEvent&&u.dispatchEvent),q="function"==typeof PromiseRejectionEvent,G="unhandledrejection",J="rejectionhandled",K=0,X=1,Q=2,Z=1,ee=2,te=C(F,(function(){var e=b(R)!==String(R);if(!e){if(66===O)return!0;if(!A&&!q)return!0}if(l&&!R.prototype["finally"])return!0;if(O>=51&&/native code/.test(R))return!1;var t=R.resolve(1),n=function(e){e((function(){}),(function(){}))},r=t.constructor={};return r[P]=n,!(t.then((function(){}))instanceof n)})),ne=te||!M((function(e){R.all(e)["catch"]((function(){}))})),re=function(e){var t;return!(!_(e)||"function"!=typeof(t=e.then))&&t},ie=function(e,t){if(!e.notified){e.notified=!0;var n=e.reactions;L((function(){var r=e.value,i=e.state==X,a=0;while(n.length>a){var o,s,l,u=n[a++],c=i?u.ok:u.fail,d=u.resolve,f=u.reject,h=u.domain;try{c?(i||(e.rejection===ee&&le(e),e.rejection=Z),!0===c?o=r:(h&&h.enter(),o=c(r),h&&(h.exit(),l=!0)),o===u.promise?f(N("Promise-chain cycle")):(s=re(o))?s.call(o,d,f):d(o)):f(r)}catch(p){h&&!l&&h.exit(),f(p)}}e.reactions=[],e.notified=!1,t&&!e.rejection&&oe(e)}))}},ae=function(e,t,n){var r,i;V?(r=$.createEvent("Event"),r.promise=t,r.reason=n,r.initEvent(e,!1,!0),u.dispatchEvent(r)):r={promise:t,reason:n},!q&&(i=u["on"+e])?i(r):e===G&&S("Unhandled promise rejection",n)},oe=function(e){w.call(u,(function(){var t,n=e.facade,r=e.value,i=se(e);if(i&&(t=D((function(){A?z.emit("unhandledRejection",r,n):ae(G,n,r)})),e.rejection=A||se(e)?ee:Z,t.error))throw t.value}))},se=function(e){return e.rejection!==Z&&!e.parent},le=function(e){w.call(u,(function(){var t=e.facade;A?z.emit("rejectionHandled",t):ae(J,t,e.value)}))},ue=function(e,t,n){return function(r){e(t,r,n)}},ce=function(e,t,n){e.done||(e.done=!0,n&&(e=n),e.value=t,e.state=Q,ie(e,!0))},de=function(e,t,n){if(!e.done){e.done=!0,n&&(e=n);try{if(e.facade===t)throw N("Promise can't be resolved itself");var r=re(t);r?L((function(){var n={done:!1};try{r.call(t,ue(de,n,e),ue(ce,n,e))}catch(i){ce(n,i,e)}})):(e.value=t,e.state=X,ie(e,!1))}catch(i){ce({done:!1},i,e)}}};te&&(R=function(e){g(this,R,F),v(e),r.call(this);var t=I(this);try{e(ue(de,t),ue(ce,t))}catch(n){ce(t,n)}},r=function(e){j(this,{type:F,done:!1,notified:!1,parent:!1,reactions:[],rejection:!1,state:K,value:void 0})},r.prototype=h(R.prototype,{then:function(e,t){var n=H(this),r=W(k(this,R));return r.ok="function"!=typeof e||e,r.fail="function"==typeof t&&t,r.domain=A?z.domain:void 0,n.parent=!0,n.reactions.push(r),n.state!=K&&ie(n,!1),r.promise},catch:function(e){return this.then(void 0,e)}}),i=function(){var e=new r,t=I(e);this.promise=e,this.resolve=ue(de,t),this.reject=ue(ce,t)},T.f=W=function(e){return e===R||e===a?new i(e):U(e)},l||"function"!=typeof d||(o=d.prototype.then,f(d.prototype,"then",(function(e,t){var n=this;return new R((function(e,t){o.call(n,e,t)})).then(e,t)}),{unsafe:!0}),"function"==typeof B&&s({global:!0,enumerable:!0,forced:!0},{fetch:function(e){return x(R,B.apply(u,arguments))}}))),s({global:!0,wrap:!0,forced:te},{Promise:R}),p(R,F,!1,!0),m(F),a=c(F),s({target:F,stat:!0,forced:te},{reject:function(e){var t=W(this);return t.reject.call(void 0,e),t.promise}}),s({target:F,stat:!0,forced:l||te},{resolve:function(e){return x(l&&this===a?R:this,e)}}),s({target:F,stat:!0,forced:ne},{all:function(e){var t=this,n=W(t),r=n.resolve,i=n.reject,a=D((function(){var n=v(t.resolve),a=[],o=0,s=1;y(e,(function(e){var l=o++,u=!1;a.push(void 0),s++,n.call(t,e).then((function(e){u||(u=!0,a[l]=e,--s||r(a))}),i)})),--s||r(a)}));return a.error&&i(a.value),n.promise},race:function(e){var t=this,n=W(t),r=n.reject,i=D((function(){var i=v(t.resolve);y(e,(function(e){i.call(t,e).then(n.resolve,r)}))}));return i.error&&r(i.value),n.promise}})},e6d9:function(e,t,n){"use strict";n("9a70")},e6f9:function(e,t){function n(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,r=new Array(t);n<t;n++)r[n]=e[n];return r}e.exports=n},e81d:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"១",2:"២",3:"៣",4:"៤",5:"៥",6:"៦",7:"៧",8:"៨",9:"៩",0:"០"},n={"១":"1","២":"2","៣":"3","៤":"4","៥":"5","៦":"6","៧":"7","៨":"8","៩":"9","០":"0"},r=e.defineLocale("km",{months:"មករា_កុម្ភៈ_មីនា_មេសា_ឧសភា_មិថុនា_កក្កដា_សីហា_កញ្ញា_តុលា_វិច្ឆិកា_ធ្នូ".split("_"),monthsShort:"មករា_កុម្ភៈ_មីនា_មេសា_ឧសភា_មិថុនា_កក្កដា_សីហា_កញ្ញា_តុលា_វិច្ឆិកា_ធ្នូ".split("_"),weekdays:"អាទិត្យ_ច័ន្ទ_អង្គារ_ពុធ_ព្រហស្បតិ៍_សុក្រ_សៅរ៍".split("_"),weekdaysShort:"អា_ច_អ_ព_ព្រ_សុ_ស".split("_"),weekdaysMin:"អា_ច_អ_ព_ព្រ_សុ_ស".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},meridiemParse:/ព្រឹក|ល្ងាច/,isPM:function(e){return"ល្ងាច"===e},meridiem:function(e,t,n){return e<12?"ព្រឹក":"ល្ងាច"},calendar:{sameDay:"[ថ្ងៃនេះ ម៉ោង] LT",nextDay:"[ស្អែក ម៉ោង] LT",nextWeek:"dddd [ម៉ោង] LT",lastDay:"[ម្សិលមិញ ម៉ោង] LT",lastWeek:"dddd [សប្តាហ៍មុន] [ម៉ោង] LT",sameElse:"L"},relativeTime:{future:"%sទៀត",past:"%sមុន",s:"ប៉ុន្មានវិនាទី",ss:"%d វិនាទី",m:"មួយនាទី",mm:"%d នាទី",h:"មួយម៉ោង",hh:"%d ម៉ោង",d:"មួយថ្ងៃ",dd:"%d ថ្ងៃ",M:"មួយខែ",MM:"%d ខែ",y:"មួយឆ្នាំ",yy:"%d ឆ្នាំ"},dayOfMonthOrdinalParse:/ទី\d{1,2}/,ordinal:"ទី%d",preparse:function(e){return e.replace(/[១២៣៤៥៦៧៨៩០]/g,(function(e){return n[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},week:{dow:1,doy:4}});return r}))},e82e:function(e,t,n){"use strict";n("b2c4")},e893:function(e,t,n){var r=n("5135"),i=n("56ef"),a=n("06cf"),o=n("9bf2");e.exports=function(e,t){for(var n=i(t),s=o.f,l=a.f,u=0;u<n.length;u++){var c=n[u];r(e,c)||s(e,c,l(t,c))}}},e8b5:function(e,t,n){var r=n("c6b6");e.exports=Array.isArray||function(e){return"Array"==r(e)}},e91f:function(e,t,n){"use strict";var r=n("ebb5"),i=n("4d64").indexOf,a=r.aTypedArray,o=r.exportTypedArrayMethod;o("indexOf",(function(e){return i(a(this),e,arguments.length>1?arguments[1]:void 0)}))},e95a:function(e,t,n){var r=n("b622"),i=n("3f8c"),a=r("iterator"),o=Array.prototype;e.exports=function(e){return void 0!==e&&(i.Array===e||o[a]===e)}},ebb5:function(e,t,n){"use strict";var r,i=n("a981"),a=n("83ab"),o=n("da84"),s=n("861d"),l=n("5135"),u=n("f5df"),c=n("9112"),d=n("6eeb"),f=n("9bf2").f,h=n("e163"),p=n("d2bb"),m=n("b622"),_=n("90e3"),v=o.Int8Array,g=v&&v.prototype,b=o.Uint8ClampedArray,y=b&&b.prototype,M=v&&h(v),k=g&&h(g),w=Object.prototype,L=w.isPrototypeOf,x=m("toStringTag"),S=_("TYPED_ARRAY_TAG"),T=i&&!!p&&"Opera"!==u(o.opera),D=!1,Y={Int8Array:1,Uint8Array:1,Uint8ClampedArray:1,Int16Array:2,Uint16Array:2,Int32Array:4,Uint32Array:4,Float32Array:4,Float64Array:8},C=function(e){var t=u(e);return"DataView"===t||l(Y,t)},E=function(e){return s(e)&&l(Y,u(e))},A=function(e){if(E(e))return e;throw TypeError("Target is not a typed array")},O=function(e){if(p){if(L.call(M,e))return e}else for(var t in Y)if(l(Y,r)){var n=o[t];if(n&&(e===n||L.call(n,e)))return e}throw TypeError("Target is not a typed array constructor")},P=function(e,t,n){if(a){if(n)for(var r in Y){var i=o[r];i&&l(i.prototype,e)&&delete i.prototype[e]}k[e]&&!n||d(k,e,n?t:T&&g[e]||t)}},F=function(e,t,n){var r,i;if(a){if(p){if(n)for(r in Y)i=o[r],i&&l(i,e)&&delete i[e];if(M[e]&&!n)return;try{return d(M,e,n?t:T&&v[e]||t)}catch(s){}}for(r in Y)i=o[r],!i||i[e]&&!n||d(i,e,t)}};for(r in Y)o[r]||(T=!1);if((!T||"function"!=typeof M||M===Function.prototype)&&(M=function(){throw TypeError("Incorrect invocation")},T))for(r in Y)o[r]&&p(o[r],M);if((!T||!k||k===w)&&(k=M.prototype,T))for(r in Y)o[r]&&p(o[r].prototype,k);if(T&&h(y)!==k&&p(y,k),a&&!l(k,x))for(r in D=!0,f(k,x,{get:function(){return s(this)?this[S]:void 0}}),Y)o[r]&&c(o[r],S,r);e.exports={NATIVE_ARRAY_BUFFER_VIEWS:T,TYPED_ARRAY_TAG:D&&S,aTypedArray:A,aTypedArrayConstructor:O,exportTypedArrayMethod:P,exportTypedArrayStaticMethod:F,isView:C,isTypedArray:E,TypedArray:M,TypedArrayPrototype:k}},ebe4:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("ms",{months:"Januari_Februari_Mac_April_Mei_Jun_Julai_Ogos_September_Oktober_November_Disember".split("_"),monthsShort:"Jan_Feb_Mac_Apr_Mei_Jun_Jul_Ogs_Sep_Okt_Nov_Dis".split("_"),weekdays:"Ahad_Isnin_Selasa_Rabu_Khamis_Jumaat_Sabtu".split("_"),weekdaysShort:"Ahd_Isn_Sel_Rab_Kha_Jum_Sab".split("_"),weekdaysMin:"Ah_Is_Sl_Rb_Km_Jm_Sb".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [pukul] HH.mm",LLLL:"dddd, D MMMM YYYY [pukul] HH.mm"},meridiemParse:/pagi|tengahari|petang|malam/,meridiemHour:function(e,t){return 12===e&&(e=0),"pagi"===t?e:"tengahari"===t?e>=11?e:e+12:"petang"===t||"malam"===t?e+12:void 0},meridiem:function(e,t,n){return e<11?"pagi":e<15?"tengahari":e<19?"petang":"malam"},calendar:{sameDay:"[Hari ini pukul] LT",nextDay:"[Esok pukul] LT",nextWeek:"dddd [pukul] LT",lastDay:"[Kelmarin pukul] LT",lastWeek:"dddd [lepas pukul] LT",sameElse:"L"},relativeTime:{future:"dalam %s",past:"%s yang lepas",s:"beberapa saat",ss:"%d saat",m:"seminit",mm:"%d minit",h:"sejam",hh:"%d jam",d:"sehari",dd:"%d hari",M:"sebulan",MM:"%d bulan",y:"setahun",yy:"%d tahun"},week:{dow:1,doy:7}});return t}))},ec18:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
function t(e,t,n,r){var i={s:["mõne sekundi","mõni sekund","paar sekundit"],ss:[e+"sekundi",e+"sekundit"],m:["ühe minuti","üks minut"],mm:[e+" minuti",e+" minutit"],h:["ühe tunni","tund aega","üks tund"],hh:[e+" tunni",e+" tundi"],d:["ühe päeva","üks päev"],M:["kuu aja","kuu aega","üks kuu"],MM:[e+" kuu",e+" kuud"],y:["ühe aasta","aasta","üks aasta"],yy:[e+" aasta",e+" aastat"]};return t?i[n][2]?i[n][2]:i[n][1]:r?i[n][0]:i[n][1]}var n=e.defineLocale("et",{months:"jaanuar_veebruar_märts_aprill_mai_juuni_juuli_august_september_oktoober_november_detsember".split("_"),monthsShort:"jaan_veebr_märts_apr_mai_juuni_juuli_aug_sept_okt_nov_dets".split("_"),weekdays:"pühapäev_esmaspäev_teisipäev_kolmapäev_neljapäev_reede_laupäev".split("_"),weekdaysShort:"P_E_T_K_N_R_L".split("_"),weekdaysMin:"P_E_T_K_N_R_L".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd, D. MMMM YYYY H:mm"},calendar:{sameDay:"[Täna,] LT",nextDay:"[Homme,] LT",nextWeek:"[Järgmine] dddd LT",lastDay:"[Eile,] LT",lastWeek:"[Eelmine] dddd LT",sameElse:"L"},relativeTime:{future:"%s pärast",past:"%s tagasi",s:t,ss:t,m:t,mm:t,h:t,hh:t,d:t,dd:"%d päeva",M:t,MM:t,y:t,yy:t},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return n}))},ec2e:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("en-in",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(e){var t=e%10,n=1===~~(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th";return e+n},week:{dow:0,doy:6}});return t}))},ec70:function(e,t,n){"use strict";n("c9b2")},eda5:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("si",{months:"ජනවාරි_පෙබරවාරි_මාර්තු_අප්‍රේල්_මැයි_ජූනි_ජූලි_අගෝස්තු_සැප්තැම්බර්_ඔක්තෝබර්_නොවැම්බර්_දෙසැම්බර්".split("_"),monthsShort:"ජන_පෙබ_මාර්_අප්_මැයි_ජූනි_ජූලි_අගෝ_සැප්_ඔක්_නොවැ_දෙසැ".split("_"),weekdays:"ඉරිදා_සඳුදා_අඟහරුවාදා_බදාදා_බ්‍රහස්පතින්දා_සිකුරාදා_සෙනසුරාදා".split("_"),weekdaysShort:"ඉරි_සඳු_අඟ_බදා_බ්‍රහ_සිකු_සෙන".split("_"),weekdaysMin:"ඉ_ස_අ_බ_බ්‍ර_සි_සෙ".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"a h:mm",LTS:"a h:mm:ss",L:"YYYY/MM/DD",LL:"YYYY MMMM D",LLL:"YYYY MMMM D, a h:mm",LLLL:"YYYY MMMM D [වැනි] dddd, a h:mm:ss"},calendar:{sameDay:"[අද] LT[ට]",nextDay:"[හෙට] LT[ට]",nextWeek:"dddd LT[ට]",lastDay:"[ඊයේ] LT[ට]",lastWeek:"[පසුගිය] dddd LT[ට]",sameElse:"L"},relativeTime:{future:"%sකින්",past:"%sකට පෙර",s:"තත්පර කිහිපය",ss:"තත්පර %d",m:"මිනිත්තුව",mm:"මිනිත්තු %d",h:"පැය",hh:"පැය %d",d:"දිනය",dd:"දින %d",M:"මාසය",MM:"මාස %d",y:"වසර",yy:"වසර %d"},dayOfMonthOrdinalParse:/\d{1,2} වැනි/,ordinal:function(e){return e+" වැනි"},meridiemParse:/පෙර වරු|පස් වරු|පෙ.ව|ප.ව./,isPM:function(e){return"ප.ව."===e||"පස් වරු"===e},meridiem:function(e,t,n){return e>11?n?"ප.ව.":"පස් වරු":n?"පෙ.ව.":"පෙර වරු"}});return t}))},edc4:function(e,t,n){"use strict";n("3eb3")},ee83:function(e,t,n){},eedf:function(e,t,n){},ef5d:function(e,t){var n=Object.prototype.toString;function r(e){return"function"===typeof e.constructor?e.constructor.name:null}function i(e){return Array.isArray?Array.isArray(e):e instanceof Array}function a(e){return e instanceof Error||"string"===typeof e.message&&e.constructor&&"number"===typeof e.constructor.stackTraceLimit}function o(e){return e instanceof Date||"function"===typeof e.toDateString&&"function"===typeof e.getDate&&"function"===typeof e.setDate}function s(e){return e instanceof RegExp||"string"===typeof e.flags&&"boolean"===typeof e.ignoreCase&&"boolean"===typeof e.multiline&&"boolean"===typeof e.global}function l(e,t){return"GeneratorFunction"===r(e)}function u(e){return"function"===typeof e.throw&&"function"===typeof e.return&&"function"===typeof e.next}function c(e){try{if("number"===typeof e.length&&"function"===typeof e.callee)return!0}catch(t){if(-1!==t.message.indexOf("callee"))return!0}return!1}function d(e){return!(!e.constructor||"function"!==typeof e.constructor.isBuffer)&&e.constructor.isBuffer(e)}e.exports=function(e){if(void 0===e)return"undefined";if(null===e)return"null";var t=typeof e;if("boolean"===t)return"boolean";if("string"===t)return"string";if("number"===t)return"number";if("symbol"===t)return"symbol";if("function"===t)return l(e)?"generatorfunction":"function";if(i(e))return"array";if(d(e))return"buffer";if(c(e))return"arguments";if(o(e))return"date";if(a(e))return"error";if(s(e))return"regexp";switch(r(e)){case"Symbol":return"symbol";case"Promise":return"promise";case"WeakMap":return"weakmap";case"WeakSet":return"weakset";case"Map":return"map";case"Set":return"set";case"Int8Array":return"int8array";case"Uint8Array":return"uint8array";case"Uint8ClampedArray":return"uint8clampedarray";case"Int16Array":return"int16array";case"Uint16Array":return"uint16array";case"Int32Array":return"int32array";case"Uint32Array":return"uint32array";case"Float32Array":return"float32array";case"Float64Array":return"float64array"}if(u(e))return"generator";switch(t=n.call(e),t){case"[object Object]":return"object";case"[object Map Iterator]":return"mapiterator";case"[object Set Iterator]":return"setiterator";case"[object String Iterator]":return"stringiterator";case"[object Array Iterator]":return"arrayiterator"}return t.slice(8,-1).toLowerCase().replace(/\s/g,"")}},f009:function(e,t,n){"use strict";n("49c8")},f069:function(e,t,n){"use strict";var r=n("1c0b"),i=function(e){var t,n;this.promise=new e((function(e,r){if(void 0!==t||void 0!==n)throw TypeError("Bad Promise constructor");t=e,n=r})),this.resolve=r(t),this.reject=r(n)};e.exports.f=function(e){return new i(e)}},f0bd:function(e,t,n){"use strict";(function(e){
/**!
 * @fileOverview Kickass library to create and place poppers near their reference elements.
 * @version 1.16.1
 * @license
 * Copyright (c) 2016 Federico Zivolo and contributors
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
var n="undefined"!==typeof window&&"undefined"!==typeof document&&"undefined"!==typeof navigator,r=function(){for(var e=["Edge","Trident","Firefox"],t=0;t<e.length;t+=1)if(n&&navigator.userAgent.indexOf(e[t])>=0)return 1;return 0}();function i(e){var t=!1;return function(){t||(t=!0,window.Promise.resolve().then((function(){t=!1,e()})))}}function a(e){var t=!1;return function(){t||(t=!0,setTimeout((function(){t=!1,e()}),r))}}var o=n&&window.Promise,s=o?i:a;function l(e){var t={};return e&&"[object Function]"===t.toString.call(e)}function u(e,t){if(1!==e.nodeType)return[];var n=e.ownerDocument.defaultView,r=n.getComputedStyle(e,null);return t?r[t]:r}function c(e){return"HTML"===e.nodeName?e:e.parentNode||e.host}function d(e){if(!e)return document.body;switch(e.nodeName){case"HTML":case"BODY":return e.ownerDocument.body;case"#document":return e.body}var t=u(e),n=t.overflow,r=t.overflowX,i=t.overflowY;return/(auto|scroll|overlay)/.test(n+i+r)?e:d(c(e))}function f(e){return e&&e.referenceNode?e.referenceNode:e}var h=n&&!(!window.MSInputMethodContext||!document.documentMode),p=n&&/MSIE 10/.test(navigator.userAgent);function m(e){return 11===e?h:10===e?p:h||p}function _(e){if(!e)return document.documentElement;var t=m(10)?document.body:null,n=e.offsetParent||null;while(n===t&&e.nextElementSibling)n=(e=e.nextElementSibling).offsetParent;var r=n&&n.nodeName;return r&&"BODY"!==r&&"HTML"!==r?-1!==["TH","TD","TABLE"].indexOf(n.nodeName)&&"static"===u(n,"position")?_(n):n:e?e.ownerDocument.documentElement:document.documentElement}function v(e){var t=e.nodeName;return"BODY"!==t&&("HTML"===t||_(e.firstElementChild)===e)}function g(e){return null!==e.parentNode?g(e.parentNode):e}function b(e,t){if(!e||!e.nodeType||!t||!t.nodeType)return document.documentElement;var n=e.compareDocumentPosition(t)&Node.DOCUMENT_POSITION_FOLLOWING,r=n?e:t,i=n?t:e,a=document.createRange();a.setStart(r,0),a.setEnd(i,0);var o=a.commonAncestorContainer;if(e!==o&&t!==o||r.contains(i))return v(o)?o:_(o);var s=g(e);return s.host?b(s.host,t):b(e,g(t).host)}function y(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:"top",n="top"===t?"scrollTop":"scrollLeft",r=e.nodeName;if("BODY"===r||"HTML"===r){var i=e.ownerDocument.documentElement,a=e.ownerDocument.scrollingElement||i;return a[n]}return e[n]}function M(e,t){var n=arguments.length>2&&void 0!==arguments[2]&&arguments[2],r=y(t,"top"),i=y(t,"left"),a=n?-1:1;return e.top+=r*a,e.bottom+=r*a,e.left+=i*a,e.right+=i*a,e}function k(e,t){var n="x"===t?"Left":"Top",r="Left"===n?"Right":"Bottom";return parseFloat(e["border"+n+"Width"])+parseFloat(e["border"+r+"Width"])}function w(e,t,n,r){return Math.max(t["offset"+e],t["scroll"+e],n["client"+e],n["offset"+e],n["scroll"+e],m(10)?parseInt(n["offset"+e])+parseInt(r["margin"+("Height"===e?"Top":"Left")])+parseInt(r["margin"+("Height"===e?"Bottom":"Right")]):0)}function L(e){var t=e.body,n=e.documentElement,r=m(10)&&getComputedStyle(n);return{height:w("Height",t,n,r),width:w("Width",t,n,r)}}var x=function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")},S=function(){function e(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}return function(t,n,r){return n&&e(t.prototype,n),r&&e(t,r),t}}(),T=function(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e},D=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e};function Y(e){return D({},e,{right:e.left+e.width,bottom:e.top+e.height})}function C(e){var t={};try{if(m(10)){t=e.getBoundingClientRect();var n=y(e,"top"),r=y(e,"left");t.top+=n,t.left+=r,t.bottom+=n,t.right+=r}else t=e.getBoundingClientRect()}catch(f){}var i={left:t.left,top:t.top,width:t.right-t.left,height:t.bottom-t.top},a="HTML"===e.nodeName?L(e.ownerDocument):{},o=a.width||e.clientWidth||i.width,s=a.height||e.clientHeight||i.height,l=e.offsetWidth-o,c=e.offsetHeight-s;if(l||c){var d=u(e);l-=k(d,"x"),c-=k(d,"y"),i.width-=l,i.height-=c}return Y(i)}function E(e,t){var n=arguments.length>2&&void 0!==arguments[2]&&arguments[2],r=m(10),i="HTML"===t.nodeName,a=C(e),o=C(t),s=d(e),l=u(t),c=parseFloat(l.borderTopWidth),f=parseFloat(l.borderLeftWidth);n&&i&&(o.top=Math.max(o.top,0),o.left=Math.max(o.left,0));var h=Y({top:a.top-o.top-c,left:a.left-o.left-f,width:a.width,height:a.height});if(h.marginTop=0,h.marginLeft=0,!r&&i){var p=parseFloat(l.marginTop),_=parseFloat(l.marginLeft);h.top-=c-p,h.bottom-=c-p,h.left-=f-_,h.right-=f-_,h.marginTop=p,h.marginLeft=_}return(r&&!n?t.contains(s):t===s&&"BODY"!==s.nodeName)&&(h=M(h,t)),h}function A(e){var t=arguments.length>1&&void 0!==arguments[1]&&arguments[1],n=e.ownerDocument.documentElement,r=E(e,n),i=Math.max(n.clientWidth,window.innerWidth||0),a=Math.max(n.clientHeight,window.innerHeight||0),o=t?0:y(n),s=t?0:y(n,"left"),l={top:o-r.top+r.marginTop,left:s-r.left+r.marginLeft,width:i,height:a};return Y(l)}function O(e){var t=e.nodeName;if("BODY"===t||"HTML"===t)return!1;if("fixed"===u(e,"position"))return!0;var n=c(e);return!!n&&O(n)}function P(e){if(!e||!e.parentElement||m())return document.documentElement;var t=e.parentElement;while(t&&"none"===u(t,"transform"))t=t.parentElement;return t||document.documentElement}function F(e,t,n,r){var i=arguments.length>4&&void 0!==arguments[4]&&arguments[4],a={top:0,left:0},o=i?P(e):b(e,f(t));if("viewport"===r)a=A(o,i);else{var s=void 0;"scrollParent"===r?(s=d(c(t)),"BODY"===s.nodeName&&(s=e.ownerDocument.documentElement)):s="window"===r?e.ownerDocument.documentElement:r;var l=E(s,o,i);if("HTML"!==s.nodeName||O(o))a=l;else{var u=L(e.ownerDocument),h=u.height,p=u.width;a.top+=l.top-l.marginTop,a.bottom=h+l.top,a.left+=l.left-l.marginLeft,a.right=p+l.left}}n=n||0;var m="number"===typeof n;return a.left+=m?n:n.left||0,a.top+=m?n:n.top||0,a.right-=m?n:n.right||0,a.bottom-=m?n:n.bottom||0,a}function I(e){var t=e.width,n=e.height;return t*n}function j(e,t,n,r,i){var a=arguments.length>5&&void 0!==arguments[5]?arguments[5]:0;if(-1===e.indexOf("auto"))return e;var o=F(n,r,a,i),s={top:{width:o.width,height:t.top-o.top},right:{width:o.right-t.right,height:o.height},bottom:{width:o.width,height:o.bottom-t.bottom},left:{width:t.left-o.left,height:o.height}},l=Object.keys(s).map((function(e){return D({key:e},s[e],{area:I(s[e])})})).sort((function(e,t){return t.area-e.area})),u=l.filter((function(e){var t=e.width,r=e.height;return t>=n.clientWidth&&r>=n.clientHeight})),c=u.length>0?u[0].key:l[0].key,d=e.split("-")[1];return c+(d?"-"+d:"")}function H(e,t,n){var r=arguments.length>3&&void 0!==arguments[3]?arguments[3]:null,i=r?P(t):b(t,f(n));return E(n,i,r)}function R(e){var t=e.ownerDocument.defaultView,n=t.getComputedStyle(e),r=parseFloat(n.marginTop||0)+parseFloat(n.marginBottom||0),i=parseFloat(n.marginLeft||0)+parseFloat(n.marginRight||0),a={width:e.offsetWidth+i,height:e.offsetHeight+r};return a}function N(e){var t={left:"right",right:"left",bottom:"top",top:"bottom"};return e.replace(/left|right|bottom|top/g,(function(e){return t[e]}))}function $(e,t,n){n=n.split("-")[0];var r=R(e),i={width:r.width,height:r.height},a=-1!==["right","left"].indexOf(n),o=a?"top":"left",s=a?"left":"top",l=a?"height":"width",u=a?"width":"height";return i[o]=t[o]+t[l]/2-r[l]/2,i[s]=n===s?t[s]-r[u]:t[N(s)],i}function z(e,t){return Array.prototype.find?e.find(t):e.filter(t)[0]}function B(e,t,n){if(Array.prototype.findIndex)return e.findIndex((function(e){return e[t]===n}));var r=z(e,(function(e){return e[t]===n}));return e.indexOf(r)}function W(e,t,n){var r=void 0===n?e:e.slice(0,B(e,"name",n));return r.forEach((function(e){e["function"]&&console.warn("`modifier.function` is deprecated, use `modifier.fn`!");var n=e["function"]||e.fn;e.enabled&&l(n)&&(t.offsets.popper=Y(t.offsets.popper),t.offsets.reference=Y(t.offsets.reference),t=n(t,e))})),t}function U(){if(!this.state.isDestroyed){var e={instance:this,styles:{},arrowStyles:{},attributes:{},flipped:!1,offsets:{}};e.offsets.reference=H(this.state,this.popper,this.reference,this.options.positionFixed),e.placement=j(this.options.placement,e.offsets.reference,this.popper,this.reference,this.options.modifiers.flip.boundariesElement,this.options.modifiers.flip.padding),e.originalPlacement=e.placement,e.positionFixed=this.options.positionFixed,e.offsets.popper=$(this.popper,e.offsets.reference,e.placement),e.offsets.popper.position=this.options.positionFixed?"fixed":"absolute",e=W(this.modifiers,e),this.state.isCreated?this.options.onUpdate(e):(this.state.isCreated=!0,this.options.onCreate(e))}}function V(e,t){return e.some((function(e){var n=e.name,r=e.enabled;return r&&n===t}))}function q(e){for(var t=[!1,"ms","Webkit","Moz","O"],n=e.charAt(0).toUpperCase()+e.slice(1),r=0;r<t.length;r++){var i=t[r],a=i?""+i+n:e;if("undefined"!==typeof document.body.style[a])return a}return null}function G(){return this.state.isDestroyed=!0,V(this.modifiers,"applyStyle")&&(this.popper.removeAttribute("x-placement"),this.popper.style.position="",this.popper.style.top="",this.popper.style.left="",this.popper.style.right="",this.popper.style.bottom="",this.popper.style.willChange="",this.popper.style[q("transform")]=""),this.disableEventListeners(),this.options.removeOnDestroy&&this.popper.parentNode.removeChild(this.popper),this}function J(e){var t=e.ownerDocument;return t?t.defaultView:window}function K(e,t,n,r){var i="BODY"===e.nodeName,a=i?e.ownerDocument.defaultView:e;a.addEventListener(t,n,{passive:!0}),i||K(d(a.parentNode),t,n,r),r.push(a)}function X(e,t,n,r){n.updateBound=r,J(e).addEventListener("resize",n.updateBound,{passive:!0});var i=d(e);return K(i,"scroll",n.updateBound,n.scrollParents),n.scrollElement=i,n.eventsEnabled=!0,n}function Q(){this.state.eventsEnabled||(this.state=X(this.reference,this.options,this.state,this.scheduleUpdate))}function Z(e,t){return J(e).removeEventListener("resize",t.updateBound),t.scrollParents.forEach((function(e){e.removeEventListener("scroll",t.updateBound)})),t.updateBound=null,t.scrollParents=[],t.scrollElement=null,t.eventsEnabled=!1,t}function ee(){this.state.eventsEnabled&&(cancelAnimationFrame(this.scheduleUpdate),this.state=Z(this.reference,this.state))}function te(e){return""!==e&&!isNaN(parseFloat(e))&&isFinite(e)}function ne(e,t){Object.keys(t).forEach((function(n){var r="";-1!==["width","height","top","right","bottom","left"].indexOf(n)&&te(t[n])&&(r="px"),e.style[n]=t[n]+r}))}function re(e,t){Object.keys(t).forEach((function(n){var r=t[n];!1!==r?e.setAttribute(n,t[n]):e.removeAttribute(n)}))}function ie(e){return ne(e.instance.popper,e.styles),re(e.instance.popper,e.attributes),e.arrowElement&&Object.keys(e.arrowStyles).length&&ne(e.arrowElement,e.arrowStyles),e}function ae(e,t,n,r,i){var a=H(i,t,e,n.positionFixed),o=j(n.placement,a,t,e,n.modifiers.flip.boundariesElement,n.modifiers.flip.padding);return t.setAttribute("x-placement",o),ne(t,{position:n.positionFixed?"fixed":"absolute"}),n}function oe(e,t){var n=e.offsets,r=n.popper,i=n.reference,a=Math.round,o=Math.floor,s=function(e){return e},l=a(i.width),u=a(r.width),c=-1!==["left","right"].indexOf(e.placement),d=-1!==e.placement.indexOf("-"),f=l%2===u%2,h=l%2===1&&u%2===1,p=t?c||d||f?a:o:s,m=t?a:s;return{left:p(h&&!d&&t?r.left-1:r.left),top:m(r.top),bottom:m(r.bottom),right:p(r.right)}}var se=n&&/Firefox/i.test(navigator.userAgent);function le(e,t){var n=t.x,r=t.y,i=e.offsets.popper,a=z(e.instance.modifiers,(function(e){return"applyStyle"===e.name})).gpuAcceleration;void 0!==a&&console.warn("WARNING: `gpuAcceleration` option moved to `computeStyle` modifier and will not be supported in future versions of Popper.js!");var o=void 0!==a?a:t.gpuAcceleration,s=_(e.instance.popper),l=C(s),u={position:i.position},c=oe(e,window.devicePixelRatio<2||!se),d="bottom"===n?"top":"bottom",f="right"===r?"left":"right",h=q("transform"),p=void 0,m=void 0;if(m="bottom"===d?"HTML"===s.nodeName?-s.clientHeight+c.bottom:-l.height+c.bottom:c.top,p="right"===f?"HTML"===s.nodeName?-s.clientWidth+c.right:-l.width+c.right:c.left,o&&h)u[h]="translate3d("+p+"px, "+m+"px, 0)",u[d]=0,u[f]=0,u.willChange="transform";else{var v="bottom"===d?-1:1,g="right"===f?-1:1;u[d]=m*v,u[f]=p*g,u.willChange=d+", "+f}var b={"x-placement":e.placement};return e.attributes=D({},b,e.attributes),e.styles=D({},u,e.styles),e.arrowStyles=D({},e.offsets.arrow,e.arrowStyles),e}function ue(e,t,n){var r=z(e,(function(e){var n=e.name;return n===t})),i=!!r&&e.some((function(e){return e.name===n&&e.enabled&&e.order<r.order}));if(!i){var a="`"+t+"`",o="`"+n+"`";console.warn(o+" modifier is required by "+a+" modifier in order to work, be sure to include it before "+a+"!")}return i}function ce(e,t){var n;if(!ue(e.instance.modifiers,"arrow","keepTogether"))return e;var r=t.element;if("string"===typeof r){if(r=e.instance.popper.querySelector(r),!r)return e}else if(!e.instance.popper.contains(r))return console.warn("WARNING: `arrow.element` must be child of its popper element!"),e;var i=e.placement.split("-")[0],a=e.offsets,o=a.popper,s=a.reference,l=-1!==["left","right"].indexOf(i),c=l?"height":"width",d=l?"Top":"Left",f=d.toLowerCase(),h=l?"left":"top",p=l?"bottom":"right",m=R(r)[c];s[p]-m<o[f]&&(e.offsets.popper[f]-=o[f]-(s[p]-m)),s[f]+m>o[p]&&(e.offsets.popper[f]+=s[f]+m-o[p]),e.offsets.popper=Y(e.offsets.popper);var _=s[f]+s[c]/2-m/2,v=u(e.instance.popper),g=parseFloat(v["margin"+d]),b=parseFloat(v["border"+d+"Width"]),y=_-e.offsets.popper[f]-g-b;return y=Math.max(Math.min(o[c]-m,y),0),e.arrowElement=r,e.offsets.arrow=(n={},T(n,f,Math.round(y)),T(n,h,""),n),e}function de(e){return"end"===e?"start":"start"===e?"end":e}var fe=["auto-start","auto","auto-end","top-start","top","top-end","right-start","right","right-end","bottom-end","bottom","bottom-start","left-end","left","left-start"],he=fe.slice(3);function pe(e){var t=arguments.length>1&&void 0!==arguments[1]&&arguments[1],n=he.indexOf(e),r=he.slice(n+1).concat(he.slice(0,n));return t?r.reverse():r}var me={FLIP:"flip",CLOCKWISE:"clockwise",COUNTERCLOCKWISE:"counterclockwise"};function _e(e,t){if(V(e.instance.modifiers,"inner"))return e;if(e.flipped&&e.placement===e.originalPlacement)return e;var n=F(e.instance.popper,e.instance.reference,t.padding,t.boundariesElement,e.positionFixed),r=e.placement.split("-")[0],i=N(r),a=e.placement.split("-")[1]||"",o=[];switch(t.behavior){case me.FLIP:o=[r,i];break;case me.CLOCKWISE:o=pe(r);break;case me.COUNTERCLOCKWISE:o=pe(r,!0);break;default:o=t.behavior}return o.forEach((function(s,l){if(r!==s||o.length===l+1)return e;r=e.placement.split("-")[0],i=N(r);var u=e.offsets.popper,c=e.offsets.reference,d=Math.floor,f="left"===r&&d(u.right)>d(c.left)||"right"===r&&d(u.left)<d(c.right)||"top"===r&&d(u.bottom)>d(c.top)||"bottom"===r&&d(u.top)<d(c.bottom),h=d(u.left)<d(n.left),p=d(u.right)>d(n.right),m=d(u.top)<d(n.top),_=d(u.bottom)>d(n.bottom),v="left"===r&&h||"right"===r&&p||"top"===r&&m||"bottom"===r&&_,g=-1!==["top","bottom"].indexOf(r),b=!!t.flipVariations&&(g&&"start"===a&&h||g&&"end"===a&&p||!g&&"start"===a&&m||!g&&"end"===a&&_),y=!!t.flipVariationsByContent&&(g&&"start"===a&&p||g&&"end"===a&&h||!g&&"start"===a&&_||!g&&"end"===a&&m),M=b||y;(f||v||M)&&(e.flipped=!0,(f||v)&&(r=o[l+1]),M&&(a=de(a)),e.placement=r+(a?"-"+a:""),e.offsets.popper=D({},e.offsets.popper,$(e.instance.popper,e.offsets.reference,e.placement)),e=W(e.instance.modifiers,e,"flip"))})),e}function ve(e){var t=e.offsets,n=t.popper,r=t.reference,i=e.placement.split("-")[0],a=Math.floor,o=-1!==["top","bottom"].indexOf(i),s=o?"right":"bottom",l=o?"left":"top",u=o?"width":"height";return n[s]<a(r[l])&&(e.offsets.popper[l]=a(r[l])-n[u]),n[l]>a(r[s])&&(e.offsets.popper[l]=a(r[s])),e}function ge(e,t,n,r){var i=e.match(/((?:\-|\+)?\d*\.?\d*)(.*)/),a=+i[1],o=i[2];if(!a)return e;if(0===o.indexOf("%")){var s=void 0;switch(o){case"%p":s=n;break;case"%":case"%r":default:s=r}var l=Y(s);return l[t]/100*a}if("vh"===o||"vw"===o){var u=void 0;return u="vh"===o?Math.max(document.documentElement.clientHeight,window.innerHeight||0):Math.max(document.documentElement.clientWidth,window.innerWidth||0),u/100*a}return a}function be(e,t,n,r){var i=[0,0],a=-1!==["right","left"].indexOf(r),o=e.split(/(\+|\-)/).map((function(e){return e.trim()})),s=o.indexOf(z(o,(function(e){return-1!==e.search(/,|\s/)})));o[s]&&-1===o[s].indexOf(",")&&console.warn("Offsets separated by white space(s) are deprecated, use a comma (,) instead.");var l=/\s*,\s*|\s+/,u=-1!==s?[o.slice(0,s).concat([o[s].split(l)[0]]),[o[s].split(l)[1]].concat(o.slice(s+1))]:[o];return u=u.map((function(e,r){var i=(1===r?!a:a)?"height":"width",o=!1;return e.reduce((function(e,t){return""===e[e.length-1]&&-1!==["+","-"].indexOf(t)?(e[e.length-1]=t,o=!0,e):o?(e[e.length-1]+=t,o=!1,e):e.concat(t)}),[]).map((function(e){return ge(e,i,t,n)}))})),u.forEach((function(e,t){e.forEach((function(n,r){te(n)&&(i[t]+=n*("-"===e[r-1]?-1:1))}))})),i}function ye(e,t){var n=t.offset,r=e.placement,i=e.offsets,a=i.popper,o=i.reference,s=r.split("-")[0],l=void 0;return l=te(+n)?[+n,0]:be(n,a,o,s),"left"===s?(a.top+=l[0],a.left-=l[1]):"right"===s?(a.top+=l[0],a.left+=l[1]):"top"===s?(a.left+=l[0],a.top-=l[1]):"bottom"===s&&(a.left+=l[0],a.top+=l[1]),e.popper=a,e}function Me(e,t){var n=t.boundariesElement||_(e.instance.popper);e.instance.reference===n&&(n=_(n));var r=q("transform"),i=e.instance.popper.style,a=i.top,o=i.left,s=i[r];i.top="",i.left="",i[r]="";var l=F(e.instance.popper,e.instance.reference,t.padding,n,e.positionFixed);i.top=a,i.left=o,i[r]=s,t.boundaries=l;var u=t.priority,c=e.offsets.popper,d={primary:function(e){var n=c[e];return c[e]<l[e]&&!t.escapeWithReference&&(n=Math.max(c[e],l[e])),T({},e,n)},secondary:function(e){var n="right"===e?"left":"top",r=c[n];return c[e]>l[e]&&!t.escapeWithReference&&(r=Math.min(c[n],l[e]-("right"===e?c.width:c.height))),T({},n,r)}};return u.forEach((function(e){var t=-1!==["left","top"].indexOf(e)?"primary":"secondary";c=D({},c,d[t](e))})),e.offsets.popper=c,e}function ke(e){var t=e.placement,n=t.split("-")[0],r=t.split("-")[1];if(r){var i=e.offsets,a=i.reference,o=i.popper,s=-1!==["bottom","top"].indexOf(n),l=s?"left":"top",u=s?"width":"height",c={start:T({},l,a[l]),end:T({},l,a[l]+a[u]-o[u])};e.offsets.popper=D({},o,c[r])}return e}function we(e){if(!ue(e.instance.modifiers,"hide","preventOverflow"))return e;var t=e.offsets.reference,n=z(e.instance.modifiers,(function(e){return"preventOverflow"===e.name})).boundaries;if(t.bottom<n.top||t.left>n.right||t.top>n.bottom||t.right<n.left){if(!0===e.hide)return e;e.hide=!0,e.attributes["x-out-of-boundaries"]=""}else{if(!1===e.hide)return e;e.hide=!1,e.attributes["x-out-of-boundaries"]=!1}return e}function Le(e){var t=e.placement,n=t.split("-")[0],r=e.offsets,i=r.popper,a=r.reference,o=-1!==["left","right"].indexOf(n),s=-1===["top","left"].indexOf(n);return i[o?"left":"top"]=a[n]-(s?i[o?"width":"height"]:0),e.placement=N(t),e.offsets.popper=Y(i),e}var xe={shift:{order:100,enabled:!0,fn:ke},offset:{order:200,enabled:!0,fn:ye,offset:0},preventOverflow:{order:300,enabled:!0,fn:Me,priority:["left","right","top","bottom"],padding:5,boundariesElement:"scrollParent"},keepTogether:{order:400,enabled:!0,fn:ve},arrow:{order:500,enabled:!0,fn:ce,element:"[x-arrow]"},flip:{order:600,enabled:!0,fn:_e,behavior:"flip",padding:5,boundariesElement:"viewport",flipVariations:!1,flipVariationsByContent:!1},inner:{order:700,enabled:!1,fn:Le},hide:{order:800,enabled:!0,fn:we},computeStyle:{order:850,enabled:!0,fn:le,gpuAcceleration:!0,x:"bottom",y:"right"},applyStyle:{order:900,enabled:!0,fn:ie,onLoad:ae,gpuAcceleration:void 0}},Se={placement:"bottom",positionFixed:!1,eventsEnabled:!0,removeOnDestroy:!1,onCreate:function(){},onUpdate:function(){},modifiers:xe},Te=function(){function e(t,n){var r=this,i=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};x(this,e),this.scheduleUpdate=function(){return requestAnimationFrame(r.update)},this.update=s(this.update.bind(this)),this.options=D({},e.Defaults,i),this.state={isDestroyed:!1,isCreated:!1,scrollParents:[]},this.reference=t&&t.jquery?t[0]:t,this.popper=n&&n.jquery?n[0]:n,this.options.modifiers={},Object.keys(D({},e.Defaults.modifiers,i.modifiers)).forEach((function(t){r.options.modifiers[t]=D({},e.Defaults.modifiers[t]||{},i.modifiers?i.modifiers[t]:{})})),this.modifiers=Object.keys(this.options.modifiers).map((function(e){return D({name:e},r.options.modifiers[e])})).sort((function(e,t){return e.order-t.order})),this.modifiers.forEach((function(e){e.enabled&&l(e.onLoad)&&e.onLoad(r.reference,r.popper,r.options,e,r.state)})),this.update();var a=this.options.eventsEnabled;a&&this.enableEventListeners(),this.state.eventsEnabled=a}return S(e,[{key:"update",value:function(){return U.call(this)}},{key:"destroy",value:function(){return G.call(this)}},{key:"enableEventListeners",value:function(){return Q.call(this)}},{key:"disableEventListeners",value:function(){return ee.call(this)}}]),e}();Te.Utils=("undefined"!==typeof window?window:e).PopperUtils,Te.placements=fe,Te.Defaults=Se,t["a"]=Te}).call(this,n("c8ba"))},f13c:function(e,t,n){
/*!
  * vue-scrollto v2.20.0
  * (c) 2019 Randjelovic Igor
  * @license MIT
  */
(function(t,n){e.exports=n()})(0,(function(){"use strict";function e(t){return e="function"===typeof Symbol&&"symbol"===typeof Symbol.iterator?function(e){return typeof e}:function(e){return e&&"function"===typeof Symbol&&e.constructor===Symbol&&e!==Symbol.prototype?"symbol":typeof e},e(t)}function t(){return t=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},t.apply(this,arguments)}var n=4,r=.001,i=1e-7,a=10,o=11,s=1/(o-1),l="function"===typeof Float32Array;function u(e,t){return 1-3*t+3*e}function c(e,t){return 3*t-6*e}function d(e){return 3*e}function f(e,t,n){return((u(t,n)*e+c(t,n))*e+d(t))*e}function h(e,t,n){return 3*u(t,n)*e*e+2*c(t,n)*e+d(t)}function p(e,t,n,r,o){var s,l,u=0;do{l=t+(n-t)/2,s=f(l,r,o)-e,s>0?n=l:t=l}while(Math.abs(s)>i&&++u<a);return l}function m(e,t,r,i){for(var a=0;a<n;++a){var o=h(t,r,i);if(0===o)return t;var s=f(t,r,i)-e;t-=s/o}return t}function _(e){return e}var v=function(e,t,n,i){if(!(0<=e&&e<=1&&0<=n&&n<=1))throw new Error("bezier x values must be in [0, 1] range");if(e===t&&n===i)return _;for(var a=l?new Float32Array(o):new Array(o),u=0;u<o;++u)a[u]=f(u*s,e,n);function c(t){for(var i=0,l=1,u=o-1;l!==u&&a[l]<=t;++l)i+=s;--l;var c=(t-a[l])/(a[l+1]-a[l]),d=i+c*s,f=h(d,e,n);return f>=r?m(t,d,e,n):0===f?d:p(t,i,i+s,e,n)}return function(e){return 0===e?0:1===e?1:f(c(e),t,i)}},g={ease:[.25,.1,.25,1],linear:[0,0,1,1],"ease-in":[.42,0,1,1],"ease-out":[0,0,.58,1],"ease-in-out":[.42,0,.58,1]},b=!1;try{var y=Object.defineProperty({},"passive",{get:function(){b=!0}});window.addEventListener("test",null,y)}catch(F){}var M={$:function(e){return"string"!==typeof e?e:document.querySelector(e)},on:function(e,t,n){var r=arguments.length>3&&void 0!==arguments[3]?arguments[3]:{passive:!1};t instanceof Array||(t=[t]);for(var i=0;i<t.length;i++)e.addEventListener(t[i],n,!!b&&r)},off:function(e,t,n){t instanceof Array||(t=[t]);for(var r=0;r<t.length;r++)e.removeEventListener(t[r],n)},cumulativeOffset:function(e){var t=0,n=0;do{t+=e.offsetTop||0,n+=e.offsetLeft||0,e=e.offsetParent}while(e);return{top:t,left:n}}},k=["mousedown","wheel","DOMMouseScroll","mousewheel","keyup","touchmove"],w={container:"body",duration:500,lazy:!0,easing:"ease",offset:0,force:!0,cancelable:!0,onStart:!1,onDone:!1,onCancel:!1,x:!1,y:!0};function L(e){w=t({},w,e)}var x=function(){var t,n,r,i,a,o,s,l,u,c,d,f,h,p,m,_,b,y,L,x,S,T,D,Y,C,E,A,O=function(e){l&&(D=e,x=!0)};function P(e){var t=e.scrollTop;return"body"===e.tagName.toLowerCase()&&(t=t||document.documentElement.scrollTop),t}function F(e){var t=e.scrollLeft;return"body"===e.tagName.toLowerCase()&&(t=t||document.documentElement.scrollLeft),t}function I(){S=M.cumulativeOffset(n),T=M.cumulativeOffset(t),f&&(m=T.left-S.left+o,y=m-p),h&&(b=T.top-S.top+o,L=b-_)}function j(e){if(x)return H();C||(C=e),a||I(),E=e-C,A=Math.min(E/r,1),A=Y(A),R(n,_+L*A,p+y*A),E<r?window.requestAnimationFrame(j):H()}function H(){x||R(n,b,m),C=!1,M.off(n,k,O),x&&d&&d(D,t),!x&&c&&c(t)}function R(e,t,n){h&&(e.scrollTop=t),f&&(e.scrollLeft=n),"body"===e.tagName.toLowerCase()&&(h&&(document.documentElement.scrollTop=t),f&&(document.documentElement.scrollLeft=n))}function N(m,S){var T=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{};if("object"===e(S)?T=S:"number"===typeof S&&(T.duration=S),t=M.$(m),!t)return console.warn("[vue-scrollto warn]: Trying to scroll to an element that is not on the page: "+m);if(n=M.$(T.container||w.container),r=T.hasOwnProperty("duration")?T.duration:w.duration,a=T.hasOwnProperty("lazy")?T.lazy:w.lazy,i=T.easing||w.easing,o=T.hasOwnProperty("offset")?T.offset:w.offset,s=T.hasOwnProperty("force")?!1!==T.force:w.force,l=T.hasOwnProperty("cancelable")?!1!==T.cancelable:w.cancelable,u=T.onStart||w.onStart,c=T.onDone||w.onDone,d=T.onCancel||w.onCancel,f=void 0===T.x?w.x:T.x,h=void 0===T.y?w.y:T.y,"function"===typeof o&&(o=o(t,n)),p=F(n),_=P(n),I(),x=!1,!s){var C="body"===n.tagName.toLowerCase()?document.documentElement.clientHeight||window.innerHeight:n.offsetHeight,E=_,A=E+C,H=b-o,R=H+t.offsetHeight;if(H>=E&&R<=A)return void(c&&c(t))}if(u&&u(t),L||y)return"string"===typeof i&&(i=g[i]||g["ease"]),Y=v.apply(v,i),M.on(n,k,O,{passive:!0}),window.requestAnimationFrame(j),function(){D=null,x=!0};c&&c(t)}return N},S=x(),T=[];function D(e){for(var t=0;t<T.length;++t)if(T[t].el===e)return T.splice(t,1),!0;return!1}function Y(e){for(var t=0;t<T.length;++t)if(T[t].el===e)return T[t]}function C(e){var t=Y(e);return t||(T.push(t={el:e,binding:{}}),t)}function E(e){var t=C(this).binding;if(t.value){if(e.preventDefault(),"string"===typeof t.value)return S(t.value);S(t.value.el||t.value.element,t.value)}}var A={bind:function(e,t){C(e).binding=t,M.on(e,"click",E)},unbind:function(e){D(e),M.off(e,"click",E)},update:function(e,t){C(e).binding=t}},O={bind:A.bind,unbind:A.unbind,update:A.update,beforeMount:A.bind,unmounted:A.unbind,updated:A.update,scrollTo:S,bindings:T},P=function(e,t){t&&L(t),e.directive("scroll-to",O);var n=e.config.globalProperties||e.prototype;n.$scrollTo=O.scrollTo};return"undefined"!==typeof window&&window.Vue&&(window.VueScrollTo=O,window.VueScrollTo.setDefaults=L,window.VueScrollTo.scroller=x,window.Vue.use&&window.Vue.use(P)),O.install=P,O}))},f167:function(e,t,n){"use strict";n("8300")},f174:function(e,t,n){"use strict";n("a718")},f183:function(e,t,n){var r=n("d012"),i=n("861d"),a=n("5135"),o=n("9bf2").f,s=n("90e3"),l=n("bb2f"),u=s("meta"),c=0,d=Object.isExtensible||function(){return!0},f=function(e){o(e,u,{value:{objectID:"O"+ ++c,weakData:{}}})},h=function(e,t){if(!i(e))return"symbol"==typeof e?e:("string"==typeof e?"S":"P")+e;if(!a(e,u)){if(!d(e))return"F";if(!t)return"E";f(e)}return e[u].objectID},p=function(e,t){if(!a(e,u)){if(!d(e))return!0;if(!t)return!1;f(e)}return e[u].weakData},m=function(e){return l&&_.REQUIRED&&d(e)&&!a(e,u)&&f(e),e},_=e.exports={REQUIRED:!1,fastKey:h,getWeakData:p,onFreeze:m};r[u]=!0},f24f:function(e,t,n){},f260:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("pt",{months:"janeiro_fevereiro_março_abril_maio_junho_julho_agosto_setembro_outubro_novembro_dezembro".split("_"),monthsShort:"jan_fev_mar_abr_mai_jun_jul_ago_set_out_nov_dez".split("_"),weekdays:"Domingo_Segunda-feira_Terça-feira_Quarta-feira_Quinta-feira_Sexta-feira_Sábado".split("_"),weekdaysShort:"Dom_Seg_Ter_Qua_Qui_Sex_Sáb".split("_"),weekdaysMin:"Do_2ª_3ª_4ª_5ª_6ª_Sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY HH:mm",LLLL:"dddd, D [de] MMMM [de] YYYY HH:mm"},calendar:{sameDay:"[Hoje às] LT",nextDay:"[Amanhã às] LT",nextWeek:"dddd [às] LT",lastDay:"[Ontem às] LT",lastWeek:function(){return 0===this.day()||6===this.day()?"[Último] dddd [às] LT":"[Última] dddd [às] LT"},sameElse:"L"},relativeTime:{future:"em %s",past:"há %s",s:"segundos",ss:"%d segundos",m:"um minuto",mm:"%d minutos",h:"uma hora",hh:"%d horas",d:"um dia",dd:"%d dias",w:"uma semana",ww:"%d semanas",M:"um mês",MM:"%d meses",y:"um ano",yy:"%d anos"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%dº",week:{dow:1,doy:4}});return t}))},f3ff:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t={1:"੧",2:"੨",3:"੩",4:"੪",5:"੫",6:"੬",7:"੭",8:"੮",9:"੯",0:"੦"},n={"੧":"1","੨":"2","੩":"3","੪":"4","੫":"5","੬":"6","੭":"7","੮":"8","੯":"9","੦":"0"},r=e.defineLocale("pa-in",{months:"ਜਨਵਰੀ_ਫ਼ਰਵਰੀ_ਮਾਰਚ_ਅਪ੍ਰੈਲ_ਮਈ_ਜੂਨ_ਜੁਲਾਈ_ਅਗਸਤ_ਸਤੰਬਰ_ਅਕਤੂਬਰ_ਨਵੰਬਰ_ਦਸੰਬਰ".split("_"),monthsShort:"ਜਨਵਰੀ_ਫ਼ਰਵਰੀ_ਮਾਰਚ_ਅਪ੍ਰੈਲ_ਮਈ_ਜੂਨ_ਜੁਲਾਈ_ਅਗਸਤ_ਸਤੰਬਰ_ਅਕਤੂਬਰ_ਨਵੰਬਰ_ਦਸੰਬਰ".split("_"),weekdays:"ਐਤਵਾਰ_ਸੋਮਵਾਰ_ਮੰਗਲਵਾਰ_ਬੁਧਵਾਰ_ਵੀਰਵਾਰ_ਸ਼ੁੱਕਰਵਾਰ_ਸ਼ਨੀਚਰਵਾਰ".split("_"),weekdaysShort:"ਐਤ_ਸੋਮ_ਮੰਗਲ_ਬੁਧ_ਵੀਰ_ਸ਼ੁਕਰ_ਸ਼ਨੀ".split("_"),weekdaysMin:"ਐਤ_ਸੋਮ_ਮੰਗਲ_ਬੁਧ_ਵੀਰ_ਸ਼ੁਕਰ_ਸ਼ਨੀ".split("_"),longDateFormat:{LT:"A h:mm ਵਜੇ",LTS:"A h:mm:ss ਵਜੇ",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm ਵਜੇ",LLLL:"dddd, D MMMM YYYY, A h:mm ਵਜੇ"},calendar:{sameDay:"[ਅਜ] LT",nextDay:"[ਕਲ] LT",nextWeek:"[ਅਗਲਾ] dddd, LT",lastDay:"[ਕਲ] LT",lastWeek:"[ਪਿਛਲੇ] dddd, LT",sameElse:"L"},relativeTime:{future:"%s ਵਿੱਚ",past:"%s ਪਿਛਲੇ",s:"ਕੁਝ ਸਕਿੰਟ",ss:"%d ਸਕਿੰਟ",m:"ਇਕ ਮਿੰਟ",mm:"%d ਮਿੰਟ",h:"ਇੱਕ ਘੰਟਾ",hh:"%d ਘੰਟੇ",d:"ਇੱਕ ਦਿਨ",dd:"%d ਦਿਨ",M:"ਇੱਕ ਮਹੀਨਾ",MM:"%d ਮਹੀਨੇ",y:"ਇੱਕ ਸਾਲ",yy:"%d ਸਾਲ"},preparse:function(e){return e.replace(/[੧੨੩੪੫੬੭੮੯੦]/g,(function(e){return n[e]}))},postformat:function(e){return e.replace(/\d/g,(function(e){return t[e]}))},meridiemParse:/ਰਾਤ|ਸਵੇਰ|ਦੁਪਹਿਰ|ਸ਼ਾਮ/,meridiemHour:function(e,t){return 12===e&&(e=0),"ਰਾਤ"===t?e<4?e:e+12:"ਸਵੇਰ"===t?e:"ਦੁਪਹਿਰ"===t?e>=10?e:e+12:"ਸ਼ਾਮ"===t?e+12:void 0},meridiem:function(e,t,n){return e<4?"ਰਾਤ":e<10?"ਸਵੇਰ":e<17?"ਦੁਪਹਿਰ":e<20?"ਸ਼ਾਮ":"ਰਾਤ"},week:{dow:0,doy:6}});return r}))},f4f7:function(e,t,n){"use strict";n("85d9")},f5df:function(e,t,n){var r=n("00ee"),i=n("c6b6"),a=n("b622"),o=a("toStringTag"),s="Arguments"==i(function(){return arguments}()),l=function(e,t){try{return e[t]}catch(n){}};e.exports=r?i:function(e){var t,n,r;return void 0===e?"Undefined":null===e?"Null":"string"==typeof(n=l(t=Object(e),o))?n:s?i(t):"Object"==(r=i(t))&&"function"==typeof t.callee?"Arguments":r}},f6b4:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=["Am Faoilleach","An Gearran","Am Màrt","An Giblean","An Cèitean","An t-Ògmhios","An t-Iuchar","An Lùnastal","An t-Sultain","An Dàmhair","An t-Samhain","An Dùbhlachd"],n=["Faoi","Gear","Màrt","Gibl","Cèit","Ògmh","Iuch","Lùn","Sult","Dàmh","Samh","Dùbh"],r=["Didòmhnaich","Diluain","Dimàirt","Diciadain","Diardaoin","Dihaoine","Disathairne"],i=["Did","Dil","Dim","Dic","Dia","Dih","Dis"],a=["Dò","Lu","Mà","Ci","Ar","Ha","Sa"],o=e.defineLocale("gd",{months:t,monthsShort:n,monthsParseExact:!0,weekdays:r,weekdaysShort:i,weekdaysMin:a,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[An-diugh aig] LT",nextDay:"[A-màireach aig] LT",nextWeek:"dddd [aig] LT",lastDay:"[An-dè aig] LT",lastWeek:"dddd [seo chaidh] [aig] LT",sameElse:"L"},relativeTime:{future:"ann an %s",past:"bho chionn %s",s:"beagan diogan",ss:"%d diogan",m:"mionaid",mm:"%d mionaidean",h:"uair",hh:"%d uairean",d:"latha",dd:"%d latha",M:"mìos",MM:"%d mìosan",y:"bliadhna",yy:"%d bliadhna"},dayOfMonthOrdinalParse:/\d{1,2}(d|na|mh)/,ordinal:function(e){var t=1===e?"d":e%10===2?"na":"mh";return e+t},week:{dow:1,doy:4}});return o}))},f772:function(e,t,n){var r=n("5692"),i=n("90e3"),a=r("keys");e.exports=function(e){return a[e]||(a[e]=i(e))}},f8cd:function(e,t,n){var r=n("a691");e.exports=function(e){var t=r(e);if(t<0)throw RangeError("The argument can't be less than 0");return t}},facd:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t="jan._feb._mrt._apr._mei_jun._jul._aug._sep._okt._nov._dec.".split("_"),n="jan_feb_mrt_apr_mei_jun_jul_aug_sep_okt_nov_dec".split("_"),r=[/^jan/i,/^feb/i,/^maart|mrt.?$/i,/^apr/i,/^mei$/i,/^jun[i.]?$/i,/^jul[i.]?$/i,/^aug/i,/^sep/i,/^okt/i,/^nov/i,/^dec/i],i=/^(januari|februari|maart|april|mei|ju[nl]i|augustus|september|oktober|november|december|jan\.?|feb\.?|mrt\.?|apr\.?|ju[nl]\.?|aug\.?|sep\.?|okt\.?|nov\.?|dec\.?)/i,a=e.defineLocale("nl",{months:"januari_februari_maart_april_mei_juni_juli_augustus_september_oktober_november_december".split("_"),monthsShort:function(e,r){return e?/-MMM-/.test(r)?n[e.month()]:t[e.month()]:t},monthsRegex:i,monthsShortRegex:i,monthsStrictRegex:/^(januari|februari|maart|april|mei|ju[nl]i|augustus|september|oktober|november|december)/i,monthsShortStrictRegex:/^(jan\.?|feb\.?|mrt\.?|apr\.?|mei|ju[nl]\.?|aug\.?|sep\.?|okt\.?|nov\.?|dec\.?)/i,monthsParse:r,longMonthsParse:r,shortMonthsParse:r,weekdays:"zondag_maandag_dinsdag_woensdag_donderdag_vrijdag_zaterdag".split("_"),weekdaysShort:"zo._ma._di._wo._do._vr._za.".split("_"),weekdaysMin:"zo_ma_di_wo_do_vr_za".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD-MM-YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[vandaag om] LT",nextDay:"[morgen om] LT",nextWeek:"dddd [om] LT",lastDay:"[gisteren om] LT",lastWeek:"[afgelopen] dddd [om] LT",sameElse:"L"},relativeTime:{future:"over %s",past:"%s geleden",s:"een paar seconden",ss:"%d seconden",m:"één minuut",mm:"%d minuten",h:"één uur",hh:"%d uur",d:"één dag",dd:"%d dagen",w:"één week",ww:"%d weken",M:"één maand",MM:"%d maanden",y:"één jaar",yy:"%d jaar"},dayOfMonthOrdinalParse:/\d{1,2}(ste|de)/,ordinal:function(e){return e+(1===e||8===e||e>=20?"ste":"de")},week:{dow:1,doy:4}});return a}))},faf4:function(e,t,n){function r(e,t){if("undefined"!==typeof Symbol&&Symbol.iterator in Object(e)){var n=[],r=!0,i=!1,a=void 0;try{for(var o,s=e[Symbol.iterator]();!(r=(o=s.next()).done);r=!0)if(n.push(o.value),t&&n.length===t)break}catch(l){i=!0,a=l}finally{try{r||null==s["return"]||s["return"]()}finally{if(i)throw a}}return n}}n("a4d3"),n("e01a"),n("d28b"),n("d3b7"),n("3ca3"),n("ddb0"),e.exports=r},fb13:function(e,t,n){"use strict";n("4946")},fb14:function(e,t,n){},fb48:function(e,t,n){"use strict";
/*!
 * is-plain-object <https://github.com/jonschlinkert/is-plain-object>
 *
 * Copyright (c) 2014-2017, Jon Schlinkert.
 * Released under the MIT License.
 */var r=n("a832");function i(e){return!0===r(e)&&"[object Object]"===Object.prototype.toString.call(e)}e.exports=function(e){var t,n;return!1!==i(e)&&(t=e.constructor,"function"===typeof t&&(n=t.prototype,!1!==i(n)&&!1!==n.hasOwnProperty("isPrototypeOf")))}},fb6a:function(e,t,n){"use strict";var r=n("23e7"),i=n("861d"),a=n("e8b5"),o=n("23cb"),s=n("50c4"),l=n("fc6a"),u=n("8418"),c=n("b622"),d=n("1dde"),f=n("ae40"),h=d("slice"),p=f("slice",{ACCESSORS:!0,0:0,1:2}),m=c("species"),_=[].slice,v=Math.max;r({target:"Array",proto:!0,forced:!h||!p},{slice:function(e,t){var n,r,c,d=l(this),f=s(d.length),h=o(e,f),p=o(void 0===t?f:t,f);if(a(d)&&(n=d.constructor,"function"!=typeof n||n!==Array&&!a(n.prototype)?i(n)&&(n=n[m],null===n&&(n=void 0)):n=void 0,n===Array||void 0===n))return _.call(d,h,p);for(r=new(void 0===n?Array:n)(v(p-h,0)),c=0;h<p;h++,c++)h in d&&u(r,c,d[h]);return r.length=c,r}})},fc6a:function(e,t,n){var r=n("44ad"),i=n("1d80");e.exports=function(e){return r(i(e))}},fd7e:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("x-pseudo",{months:"J~áñúá~rý_F~ébrú~árý_~Márc~h_Áp~ríl_~Máý_~Júñé~_Júl~ý_Áú~gúst~_Sép~témb~ér_Ó~ctób~ér_Ñ~óvém~bér_~Décé~mbér".split("_"),monthsShort:"J~áñ_~Féb_~Már_~Ápr_~Máý_~Júñ_~Júl_~Áúg_~Sép_~Óct_~Ñóv_~Déc".split("_"),monthsParseExact:!0,weekdays:"S~úñdá~ý_Mó~ñdáý~_Túé~sdáý~_Wéd~ñésd~áý_T~húrs~dáý_~Fríd~áý_S~átúr~dáý".split("_"),weekdaysShort:"S~úñ_~Móñ_~Túé_~Wéd_~Thú_~Frí_~Sát".split("_"),weekdaysMin:"S~ú_Mó~_Tú_~Wé_T~h_Fr~_Sá".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[T~ódá~ý át] LT",nextDay:"[T~ómó~rró~w át] LT",nextWeek:"dddd [át] LT",lastDay:"[Ý~ést~érdá~ý át] LT",lastWeek:"[L~ást] dddd [át] LT",sameElse:"L"},relativeTime:{future:"í~ñ %s",past:"%s á~gó",s:"á ~féw ~sécó~ñds",ss:"%d s~écóñ~ds",m:"á ~míñ~úté",mm:"%d m~íñú~tés",h:"á~ñ hó~úr",hh:"%d h~óúrs",d:"á ~dáý",dd:"%d d~áýs",M:"á ~móñ~th",MM:"%d m~óñt~hs",y:"á ~ýéár",yy:"%d ý~éárs"},dayOfMonthOrdinalParse:/\d{1,2}(th|st|nd|rd)/,ordinal:function(e){var t=e%10,n=1===~~(e%100/10)?"th":1===t?"st":2===t?"nd":3===t?"rd":"th";return e+n},week:{dow:1,doy:4}});return t}))},fd86:function(e,t,n){"use strict";n("9182")},fdbc:function(e,t){e.exports={CSSRuleList:0,CSSStyleDeclaration:0,CSSValueList:0,ClientRectList:0,DOMRectList:0,DOMStringList:0,DOMTokenList:1,DataTransferItemList:0,FileList:0,HTMLAllCollection:0,HTMLCollection:0,HTMLFormElement:0,HTMLSelectElement:0,MediaList:0,MimeTypeArray:0,NamedNodeMap:0,NodeList:1,PaintRequestList:0,Plugin:0,PluginArray:0,SVGLengthList:0,SVGNumberList:0,SVGPathSegList:0,SVGPointList:0,SVGStringList:0,SVGTransformList:0,SourceBufferList:0,StyleSheetList:0,TextTrackCueList:0,TextTrackList:0,TouchList:0}},fdbf:function(e,t,n){var r=n("4930");e.exports=r&&!Symbol.sham&&"symbol"==typeof Symbol.iterator},fea9:function(e,t,n){var r=n("da84");e.exports=r.Promise},ff77:function(e,t,n){"use strict";n("741d")},ffff:function(e,t,n){(function(e,t){t(n("c1df"))})(0,(function(e){"use strict";
//! moment.js locale configuration
var t=e.defineLocale("se",{months:"ođđajagemánnu_guovvamánnu_njukčamánnu_cuoŋománnu_miessemánnu_geassemánnu_suoidnemánnu_borgemánnu_čakčamánnu_golggotmánnu_skábmamánnu_juovlamánnu".split("_"),monthsShort:"ođđj_guov_njuk_cuo_mies_geas_suoi_borg_čakč_golg_skáb_juov".split("_"),weekdays:"sotnabeaivi_vuossárga_maŋŋebárga_gaskavahkku_duorastat_bearjadat_lávvardat".split("_"),weekdaysShort:"sotn_vuos_maŋ_gask_duor_bear_láv".split("_"),weekdaysMin:"s_v_m_g_d_b_L".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"MMMM D. [b.] YYYY",LLL:"MMMM D. [b.] YYYY [ti.] HH:mm",LLLL:"dddd, MMMM D. [b.] YYYY [ti.] HH:mm"},calendar:{sameDay:"[otne ti] LT",nextDay:"[ihttin ti] LT",nextWeek:"dddd [ti] LT",lastDay:"[ikte ti] LT",lastWeek:"[ovddit] dddd [ti] LT",sameElse:"L"},relativeTime:{future:"%s geažes",past:"maŋit %s",s:"moadde sekunddat",ss:"%d sekunddat",m:"okta minuhta",mm:"%d minuhtat",h:"okta diimmu",hh:"%d diimmut",d:"okta beaivi",dd:"%d beaivvit",M:"okta mánnu",MM:"%d mánut",y:"okta jahki",yy:"%d jagit"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return t}))}});